@@ -1,43 +1,39 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGActivityGen.h
 /// @author  Piotr Woznica
 /// @author  Daniel Krajzewicz
 /// @author  Walter Bamberger
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGActivityGen.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Main class that handles City, Activities and Trips
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2014 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef AGACTIVITYGEN_H
-#define AGACTIVITYGEN_H
+#pragma once
+#include <config.h>
 
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#include <iostream>
-#include <list>
-#include <string>
-#include <router/RONet.h>
 #include "city/AGCity.h"
-#include "activities/AGTrip.h"
-#include "activities/AGActivities.h"
-#include "city/AGStreet.h"
-#include "city/AGPosition.h"
+
+
+// ===========================================================================
+// class declarations
+// ===========================================================================
+class OutputDevice;
+class RONet;
+class AGTrip;
 
 
 // ===========================================================================
@@ -105,7 +101,7 @@ protected:
      * @returns whether the trip is compatible with the time boundaries or not.
      * for this begin, end and duration of the simulation must be defined
      */
-    bool timeTripValidation(AGTrip);
+    bool timeTripValidation(const AGTrip& trip) const;
     /**
      * @brief generate the output file (trips or routes) using a trip list
      *
@@ -117,14 +113,9 @@ protected:
      *
      * @param[in] trip on which a random (normally distributed) variation will be tried
      */
-    void varDepTime(AGTrip& trip);
+    void varDepTime(AGTrip& trip) const;
 
 private:
     /// @brief invalidated assignment operator
     AGActivityGen& operator=(const AGActivityGen&);
 };
-
-#endif
-
-/****************************************************************************/
-
