@@ -1,4 +1,18 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGActivityGenHandler.cpp
 /// @author  Piotr Woznica
 /// @author  Daniel Krajzewicz
@@ -6,33 +20,10 @@
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    July 2010
-/// @version $Id: AGActivityGenHandler.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // The handler for parsing the statistics file.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "AGActivityGenHandler.h"
 #include <iostream>
@@ -136,13 +127,14 @@ AGActivityGenHandler::parseGeneralCityInfo(const SUMOSAXAttributes& attrs) {
         bool ok;
         myCity.statData.inhabitants = attrs.getInt(AGEN_ATTR_INHABITANTS);
         myCity.statData.households = attrs.getInt(AGEN_ATTR_HOUSEHOLDS);
-        myCity.statData.limitAgeChildren = attrs.getOpt<int>(AGEN_ATTR_CHILDREN, 0, ok, 18);
-        myCity.statData.limitAgeRetirement = attrs.getOpt<int>(AGEN_ATTR_RETIREMENT, 0, ok, 63);
-        myCity.statData.carRate = attrs.getOpt<double>(AGEN_ATTR_CARS, 0, ok, 0.58);
-        myCity.statData.unemployement = attrs.getOpt<double>(AGEN_ATTR_UNEMPLOYEMENT, 0, ok, 0.06);
-        myCity.statData.maxFootDistance = attrs.getOpt<double>(AGEN_ATTR_MAX_FOOT_DIST, 0, ok, 300.0);
-        myCity.statData.incomingTraffic = attrs.getOpt<int>(AGEN_ATTR_IN_TRAFFIC, 0, ok, 0);
-        myCity.statData.outgoingTraffic = attrs.getOpt<int>(AGEN_ATTR_OUT_TRAFFIC, 0, ok, 0);
+        myCity.statData.limitAgeChildren = attrs.getOpt<int>(AGEN_ATTR_CHILDREN, nullptr, ok, 18);
+        myCity.statData.limitAgeRetirement = attrs.getOpt<int>(AGEN_ATTR_RETIREMENT, nullptr, ok, 63);
+        myCity.statData.carRate = attrs.getOpt<double>(AGEN_ATTR_CARS, nullptr, ok, 0.58);
+        myCity.statData.unemployement = attrs.getOpt<double>(AGEN_ATTR_UNEMPLOYEMENT, nullptr, ok, 0.06);
+        myCity.statData.laborDemand = attrs.getOpt<double>(AGEN_ATTR_LABORDEMAND, nullptr, ok, 1.05);
+        myCity.statData.maxFootDistance = attrs.getOpt<double>(AGEN_ATTR_MAX_FOOT_DIST, nullptr, ok, 300.0);
+        myCity.statData.incomingTraffic = attrs.getOpt<int>(AGEN_ATTR_IN_TRAFFIC, nullptr, ok, 0);
+        myCity.statData.outgoingTraffic = attrs.getOpt<int>(AGEN_ATTR_OUT_TRAFFIC, nullptr, ok, 0);
     } catch (const std::exception& e) {
         WRITE_ERROR("Error while parsing the element " +
                     SUMOXMLDefinitions::Tags.getString(AGEN_TAG_GENERAL) + ": " +
@@ -155,11 +147,11 @@ void
 AGActivityGenHandler::parseParameters(const SUMOSAXAttributes& attrs) {
     try {
         bool ok;
-        myCity.statData.carPreference = attrs.getOpt<double>(AGEN_ATTR_CARPREF, 0, ok, 0.0);
-        myCity.statData.speedTimePerKm = attrs.getOpt<double>(AGEN_ATTR_CITYSPEED, 0, ok, 360.0);
-        myCity.statData.freeTimeActivityRate = attrs.getOpt<double>(AGEN_ATTR_FREETIMERATE, 0, ok, 0.15);
-        myCity.statData.uniformRandomTrafficRate = attrs.getOpt<double>(AGEN_ATTR_UNI_RAND_TRAFFIC, 0, ok, 0.0);
-        myCity.statData.departureVariation = attrs.getOpt<double>(AGEN_ATTR_DEP_VARIATION, 0, ok, 0.0);
+        myCity.statData.carPreference = attrs.getOpt<double>(AGEN_ATTR_CARPREF, nullptr, ok, 0.0);
+        myCity.statData.speedTimePerKm = attrs.getOpt<double>(AGEN_ATTR_CITYSPEED, nullptr, ok, 360.0);
+        myCity.statData.freeTimeActivityRate = attrs.getOpt<double>(AGEN_ATTR_FREETIMERATE, nullptr, ok, 0.15);
+        myCity.statData.uniformRandomTrafficRate = attrs.getOpt<double>(AGEN_ATTR_UNI_RAND_TRAFFIC, nullptr, ok, 0.0);
+        myCity.statData.departureVariation = attrs.getOpt<double>(AGEN_ATTR_DEP_VARIATION, nullptr, ok, 0.0);
     } catch (const std::exception& e) {
         WRITE_ERROR("Error while parsing the element " +
                     SUMOXMLDefinitions::Tags.getString(AGEN_TAG_PARAM) + ": " +
@@ -182,7 +174,7 @@ AGActivityGenHandler::parseStreets(const SUMOSAXAttributes& attrs) {
         }
         std::string eid = attrs.getString(SUMO_ATTR_EDGE);
         AGStreet* street = dynamic_cast<AGStreet*>(net->getEdge(eid));
-        if (street == 0) {
+        if (street == nullptr) {
             WRITE_ERROR("Edge '" + eid + "' is not known.");
             return;
         }
@@ -333,6 +325,9 @@ AGActivityGenHandler::parseStation(const SUMOSAXAttributes& attrs) {
         if (!ok) {
             throw ProcessError();
         }
+        if (myCity.statData.busStations.count(refID) == 0) {
+            throw ProcessError("Unknown bus station " + toString(refID));
+        }
         if (!isRevStation) {
             currentBusLine->locateStation(myCity.statData.busStations.find(refID)->second);
         } else {
@@ -375,11 +370,32 @@ AGActivityGenHandler::parsePopulation() {
 void
 AGActivityGenHandler::parseBracket(const SUMOSAXAttributes& attrs) {
     try {
-//TODO beginAge needs to be evaluated
-//        int beginAge = attrs.getInt(AGEN_ATTR_BEGINAGE); //included in the bracket
+
+        int beginAge = attrs.getInt(AGEN_ATTR_BEGINAGE); //included in the bracket
         int endAge = attrs.getInt(AGEN_ATTR_ENDAGE); //NOT included in the bracket
-        if (myCurrentObject == "population") {
-            myCity.statData.population[endAge] = attrs.getInt(AGEN_ATTR_PEOPLENBR);
+        bool overlapping = false;
+        // evaluate age
+        if (beginAge < endAge) {
+            for (auto it = myCity.statData.ageSpan.begin(); it != myCity.statData.ageSpan.end(); ++it) {
+                if (!(beginAge >= it->second || endAge <= it->first)) {
+                    overlapping = true;
+                }
+            }
+            if (!overlapping) {
+                myCity.statData.ageSpan[beginAge] = endAge;
+
+                if (myCurrentObject == "population") {
+                    myCity.statData.population[endAge] = attrs.getInt(AGEN_ATTR_PEOPLENBR);
+                }
+            } else {
+                WRITE_ERROR("Error while parsing the element " +
+                            SUMOXMLDefinitions::Tags.getString(AGEN_TAG_BRACKET) + ": begin and end age of the population is overlapping");
+                throw ProcessError();
+            }
+        } else {
+            WRITE_ERROR("Error while parsing the element " +
+                        SUMOXMLDefinitions::Tags.getString(AGEN_TAG_BRACKET) + ": begin and end age of the population are not properly set");
+            throw ProcessError();
         }
 
     } catch (const std::exception& e) {
@@ -390,5 +406,5 @@ AGActivityGenHandler::parseBracket(const SUMOSAXAttributes& attrs) {
     }
 }
 
-/****************************************************************************/
 
+/****************************************************************************/
