@@ -1,36 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGActivityTripWriter.cpp
 /// @author  Piotr Woznica
 /// @author  Daniel Krajzewicz
 /// @author  Walter Bamberger
 /// @date    July 2010
-/// @version $Id: AGActivityTripWriter.cpp 22608 2017-01-17 06:28:54Z behrisch $
 ///
 // Class for writing Trip objects in a SUMO-route file.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <utils/common/RGBColor.h>
 #include <utils/iodevices/OutputDevice.h>
@@ -66,7 +57,7 @@ AGActivityTripWriter::addTrip(const AGTrip& trip) {
     myTripOutput.openTag(SUMO_TAG_TRIP)
     .writeAttr(SUMO_ATTR_ID, trip.getVehicleName())
     .writeAttr(SUMO_ATTR_TYPE, trip.getType())
-    .writeAttr(SUMO_ATTR_DEPART, time)
+    .writeAttr(SUMO_ATTR_DEPART, time2string(TIME2STEPS(time)))
     .writeAttr(SUMO_ATTR_DEPARTPOS, trip.getDep().getPosition())
     .writeAttr(SUMO_ATTR_ARRIVALPOS, trip.getArr().getPosition())
     .writeAttr(SUMO_ATTR_ARRIVALSPEED, 0.)
