@@ -1,45 +1,35 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGActivityTripWriter.h
 /// @author  Piotr Woznica
 /// @author  Daniel Krajzewicz
 /// @author  Walter Bamberger
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGActivityTripWriter.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Class for writing Trip objects in a SUMO-route file.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2014 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef AGACTIVITYTRIPWRITER_H
-#define AGACTIVITYTRIPWRITER_H
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include "activities/AGTrip.h"
-#include <iostream>
-#include <map>
-#include <string>
-#include <utils/iodevices/OutputDevice.h>
+// ===========================================================================
+// class declarations
+// ===========================================================================
+class OutputDevice;
+class AGTrip;
 
 
 // ===========================================================================
@@ -47,19 +37,12 @@
 // ===========================================================================
 class AGActivityTripWriter {
 public:
-    AGActivityTripWriter(OutputDevice& file) :
-        routes(file) {
-        initialize();
-    }
+    AGActivityTripWriter(OutputDevice& file);
 
-    void initialize();
-    void addTrip(AGTrip trip);
+    void addTrip(const AGTrip& trip);
 
 private:
-    OutputDevice& routes;
-    std::map<std::string, std::string> colors;
-
-    void vtypes();
+    OutputDevice& myTripOutput;
 
 private:
     /// @brief Invalidated copy constructor.
@@ -69,8 +52,3 @@ private:
     AGActivityTripWriter& operator=(const AGActivityTripWriter&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
