@@ -1,37 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGFrame.cpp
 /// @author  Walter Bamberger
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mo, 13 Sept 2010
-/// @version $Id: AGFrame.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Configuration of the options of ActivityGen
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #ifdef HAVE_VERSION_H
 #include <version.h>
@@ -63,8 +54,6 @@ void AGFrame::fillOptions() {
     oc.addOptionSubTopic("Output");
     oc.addOptionSubTopic("Processing");
     oc.addOptionSubTopic("Time");
-    SystemFrame::addReportOptions(oc);
-    RandHelper::insertRandOptions();
 
     // Insert options
     oc.doRegister("net-file", 'n', new Option_FileName());
@@ -74,6 +63,10 @@ void AGFrame::fillOptions() {
     oc.doRegister("stat-file", 's', new Option_FileName());
     oc.addDescription("stat-file", "Input", "Loads the SUMO-statistics FILE");
 
+    // need to do this here to be able to check for network and route input options
+    SystemFrame::addReportOptions(oc);
+    RandHelper::insertRandOptions();
+
     oc.doRegister("output-file", 'o', new Option_FileName());
     oc.addSynonyme("output-file", "output", true);
     oc.addDescription("output-file", "Output", "Write generated trips to FILE");
@@ -84,13 +77,13 @@ void AGFrame::fillOptions() {
 
     // TODO: What time options are consistent with other parts of SUMO and
     // useful for the user?
-    oc.doRegister("begin", 'b', new Option_Integer());
+    oc.doRegister("begin", 'b', new Option_Integer(0));
     oc.addDescription("begin", "Time", "Sets the time of beginning of the simulation during the first day (in seconds)");
 
-    oc.doRegister("end", 'e', new Option_Integer());
+    oc.doRegister("end", 'e', new Option_Integer(0));
     oc.addDescription("end", "Time", "Sets the time of ending of the simulation during the last day (in seconds)");
 
-    oc.doRegister("duration-d", new Option_Integer());
+    oc.doRegister("duration-d", new Option_Integer(1));
     oc.addDescription("duration-d", "Time", "Sets the duration of the simulation in days");
 }
 
