@@ -1,42 +1,34 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGFreeTime.cpp
 /// @author  Piotr Woznica
 /// @author  Daniel Krajzewicz
 /// @author  Walter Bamberger
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGFreeTime.cpp 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Generates trips related to after-work activities
 // like visiting the family or party.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2014 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
-#include <math.h>
+#include <cmath>
 #include <utils/common/RandHelper.h>
 #include <utils/common/StdDefs.h>
+#include <activitygen/city/AGCity.h>
 #include <activitygen/city/AGTime.h>
 #include "AGFreeTime.h"
 
@@ -48,12 +40,12 @@ const int AGFreeTime::DAY = 1;
 const int AGFreeTime::EVENING = 2;
 const int AGFreeTime::NIGHT = 4;
 
-const int AGFreeTime::TB_DAY = (new AGTime(0, 8, 0))->getTime();
-const int AGFreeTime::TE_DAY = (new AGTime(0, 18, 0))->getTime();
-const int AGFreeTime::TB_EVENING = (new AGTime(0, 19, 0))->getTime();
-const int AGFreeTime::TE_EVENING = (new AGTime(0, 23, 59))->getTime();
-const int AGFreeTime::TB_NIGHT = (new AGTime(0, 23, 0))->getTime();
-const int AGFreeTime::TE_NIGHT = (new AGTime(1, 5, 0))->getTime();
+const int AGFreeTime::TB_DAY = AGTime(0, 8, 0).getTime();
+const int AGFreeTime::TE_DAY = AGTime(0, 18, 0).getTime();
+const int AGFreeTime::TB_EVENING = AGTime(0, 19, 0).getTime();
+const int AGFreeTime::TE_EVENING = AGTime(0, 23, 59).getTime();
+const int AGFreeTime::TB_NIGHT = AGTime(0, 23, 0).getTime();
+const int AGFreeTime::TE_NIGHT = AGTime(1, 5, 0).getTime();
 
 
 // ===========================================================================
@@ -76,8 +68,8 @@ AGFreeTime::decideTypeOfTrip() {
         if (num_poss == 0) {
             return 0;
         }
-        SUMOReal alea = RandHelper::rand(); //(float)(rand() % 1000) / 1000.0;
-        int decision = (int)floor(alea * (SUMOReal)num_poss);
+        double alea = RandHelper::rand(); //(float)(rand() % 1000) / 1000.0;
+        int decision = (int)floor(alea * (double)num_poss);
 
         if (possibleType & DAY) {
             if (decision == 0) {
@@ -123,7 +115,7 @@ AGFreeTime::possibleTypeOfTrip() {
             val += DAY;
         }
 
-        if (tReady < (*(new AGTime(0, 22, 0))).getTime()) {
+        if (tReady < AGTime(0, 22, 0).getTime()) {
             val += EVENING;
         }
     }
@@ -268,4 +260,5 @@ AGFreeTime::whenBeginActivityNextDay(int day) {
     return timeBack.getTime();
 }
 
+
 /****************************************************************************/
