@@ -1,4 +1,18 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGTrip.cpp
 /// @author  Piotr Woznica
 /// @author  Daniel Krajzewicz
@@ -6,33 +20,10 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGTrip.cpp 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Class containing all information of a given trip (car, bus)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2014 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "AGTrip.h"
 
@@ -86,8 +77,8 @@ AGTrip::addLayOverWithoutDestination(AGTrip& trip) {
     }
 }
 
-std::list<AGPosition>*
-AGTrip::getPassed() {
+const std::list<AGPosition>*
+AGTrip::getPassed() const {
     return &myPassBy;
 }
 
@@ -117,8 +108,8 @@ AGTrip::getTime() const {
 }
 
 int
-AGTrip::getTimeTrip(SUMOReal secPerKm) const {
-    SUMOReal dist = 0;
+AGTrip::getTimeTrip(double secPerKm) const {
+    double dist = 0;
     std::list<AGPosition> positions;
     positions.push_back(myFrom);
     std::list<AGPosition>::const_iterator it;
@@ -136,12 +127,12 @@ AGTrip::getTimeTrip(SUMOReal secPerKm) const {
 }
 
 int
-AGTrip::getArrTime(SUMOReal secPerKm) const {
+AGTrip::getArrTime(double secPerKm) const {
     return myDepTime + getTimeTrip(secPerKm);
 }
 
 int
-AGTrip::getRideBackArrTime(SUMOReal secPerKm) const {
+AGTrip::getRideBackArrTime(double secPerKm) const {
     return getArrTime(secPerKm) + (int)(secPerKm * myTo.distanceTo(myFrom) / 1000.0);
 }
 
@@ -151,7 +142,7 @@ AGTrip::setDepTime(int time) {
 }
 
 int
-AGTrip::estimateDepTime(int arrTime, SUMOReal secPerKm) const {
+AGTrip::estimateDepTime(int arrTime, double secPerKm) const {
     return arrTime - getTimeTrip(secPerKm);
 }
 
@@ -167,12 +158,12 @@ AGTrip::setVehicleName(std::string name) {
 
 void
 AGTrip::setArr(AGPosition arrival) {
-    myTo = *new AGPosition(arrival.getStreet(), arrival.getPosition());
+    myTo = AGPosition(arrival.getStreet(), arrival.getPosition());
 }
 
 void
 AGTrip::setDep(AGPosition departure) {
-    myFrom = *new AGPosition(departure.getStreet(), departure.getPosition());
+    myFrom = AGPosition(departure.getStreet(), departure.getPosition());
 }
 
 bool
@@ -190,4 +181,5 @@ AGTrip::setDay(int d) {
     myDay = d;
 }
 
+
 /****************************************************************************/
