@@ -1,37 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGAdult.cpp
 /// @author  Piotr Woznica
 /// @author  Walter Bamberger
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGAdult.cpp 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Person in working age: can be linked to a work position.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2017 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "AGAdult.h"
 #include "AGWorkPosition.h"
@@ -51,14 +42,14 @@ AGAdult::randomFreeWorkPosition(std::vector<AGWorkPosition>* wps) {
         }
     }
     if (freePos.empty()) {
-        return 0;
+        return nullptr;
     }
     return RandHelper::getRandomFrom(freePos);
 }
 
 
 AGAdult::AGAdult(int age)
-    : AGPerson(age), work(0) {}
+    : AGPerson(age), work(nullptr) {}
 
 
 void
@@ -74,13 +65,13 @@ AGAdult::tryToWork(double rate, std::vector<AGWorkPosition>* wps) {
         // This avoids that the current one is the same as the new one.
         AGWorkPosition* newWork = randomFreeWorkPosition(wps);
 
-        if (work != 0) {
+        if (work != nullptr) {
             work->let();
         }
         work = newWork;
         work->take(this);
     } else {
-        if (work != 0) {
+        if (work != nullptr) {
             // Also sets work = 0 with the call back lostWorkPosition
             work->let();
         }
@@ -90,19 +81,19 @@ AGAdult::tryToWork(double rate, std::vector<AGWorkPosition>* wps) {
 
 bool
 AGAdult::isWorking() const {
-    return (work != 0);
+    return (work != nullptr);
 }
 
 
 void
 AGAdult::lostWorkPosition() {
-    work = 0;
+    work = nullptr;
 }
 
 
 void
 AGAdult::resignFromWorkPosition() {
-    if (work != 0) {
+    if (work != nullptr) {
         work->let();
     }
 }
@@ -110,10 +101,11 @@ AGAdult::resignFromWorkPosition() {
 
 const AGWorkPosition&
 AGAdult::getWorkPosition() const {
-    if (work != 0) {
+    if (work != nullptr) {
         return *work;
     }
     throw std::runtime_error("AGAdult::getWorkPosition: Adult is unemployed.");
 }
 
+
 /****************************************************************************/
