@@ -1,39 +1,29 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGAdult.h
 /// @author  Piotr Woznica
 /// @author  Walter Bamberger
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGAdult.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Person in working age: can be linked to a work position.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2014 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef AGADULT_H
-#define AGADULT_H
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <stdexcept>
@@ -85,7 +75,7 @@ public:
      * @param[in]: employmentRate (1 - unemploymentRate)
      * @param[in]: wps the list of work positions (open or not) in the city
      */
-    void tryToWork(SUMOReal employmentRate, std::vector<AGWorkPosition>* wps);
+    void tryToWork(double employmentRate, std::vector<AGWorkPosition>* wps);
 
     /** @brief Called when the adult has lost her job.
      *
@@ -126,7 +116,3 @@ private:
      */
     static AGWorkPosition* randomFreeWorkPosition(std::vector<AGWorkPosition>* wps);
 };
-
-#endif /* AGADULT_H */
-
-/****************************************************************************/
