@@ -1,37 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGBus.h
 /// @author  Piotr Woznica
 /// @author  Daniel Krajzewicz
 /// @author  Walter Bamberger
 /// @date    July 2010
-/// @version $Id: AGBus.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // A bus driving in the city
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef AGBUS_H
-#define AGBUS_H
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <string>
@@ -43,20 +34,16 @@
 class AGBus {
 public:
     AGBus(std::string name, int depTime) :
-        name(name),
-        departureTime(depTime) {};
+        myName(name),
+        myDepartureTime(depTime) {};
     AGBus(int depTime) :
-        departureTime(depTime) {};
+        myDepartureTime(depTime) {};
     void setName(std::string name);
     int getDeparture();
     std::string getName();
     void print() const;
 
 private:
-    std::string name;
-    int departureTime;
+    std::string myName;
+    int myDepartureTime;
 };
-
-#endif
-
-/****************************************************************************/
