@@ -1,4 +1,18 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGChild.cpp
 /// @author  Piotr Woznica
 /// @author  Daniel Krajzewicz
@@ -6,33 +20,10 @@
 /// @author  Walter Bamberger
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGChild.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Person in age to go to school: linked to a school object
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2016 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <vector>
@@ -46,25 +37,25 @@
 // ===========================================================================
 void
 AGChild::print() const {
-    std::cout << "- Child: Age=" << age << " School=" << school << std::endl;
+    std::cout << "- Child: Age=" << age << " School=" << mySchool << std::endl;
 }
 
 bool
 AGChild::setSchool(AGSchool* school) {
-    if (school == NULL) {
+    if (school == nullptr) {
         return false;
     }
     bool enoughPlace = school->addNewChild();
     if (enoughPlace) {
-        this->school = school;
+        mySchool = school;
     }
     return enoughPlace;
 }
 
 bool
 AGChild::allocateASchool(std::list<AGSchool>* schools, AGPosition housePos) {
-    SUMOReal minDist = std::numeric_limits<SUMOReal>::infinity();
-    AGSchool* sch = NULL;
+    double minDist = std::numeric_limits<double>::infinity();
+    AGSchool* sch = nullptr;
     if (schools->size() == 0) {
         return false;
     }
@@ -81,32 +72,33 @@ AGChild::allocateASchool(std::list<AGSchool>* schools, AGPosition housePos) {
 
 bool
 AGChild::leaveSchool() {
-    if (school != NULL)
-        if (!school->removeChild()) {
+    if (mySchool != nullptr)
+        if (!mySchool->removeChild()) {
             return false;
         }
-    school = NULL;
+    mySchool = nullptr;
     return true;
 }
 
 bool
 AGChild::haveASchool() const {
-    return (school != NULL);
+    return (mySchool != nullptr);
 }
 
 AGPosition
 AGChild::getSchoolLocation() const {
-    return school->getPosition();
+    return mySchool->getPosition();
 }
 
 int
 AGChild::getSchoolClosing() const {
-    return school->getClosingHour();
+    return mySchool->getClosingHour();
 }
 
 int
 AGChild::getSchoolOpening() const {
-    return school->getOpeningHour();
+    return mySchool->getOpeningHour();
 }
 
+
 /****************************************************************************/
