@@ -1,42 +1,36 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGHousehold.cpp
 /// @author  Piotr Woznica
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    July 2010
-/// @version $Id: AGHousehold.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A household contains the people and cars of the city: roughly represents
 // families with their address, cars, adults and possibly children
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <utils/common/RandHelper.h>
-#include "AGHousehold.h"
 #include "AGCar.h"
+#include "AGChild.h"
+#include "AGCity.h"
+#include "AGDataAndStatistics.h"
+#include "AGHousehold.h"
 
 
 // ===========================================================================
@@ -69,7 +63,7 @@ AGHousehold::generatePeople(int numAdults, int numChilds, bool firstRetired) {
 }
 
 void
-AGHousehold::generateCars(SUMOReal rate) {
+AGHousehold::generateCars(double rate) {
     int peopleInNeed = static_cast<int>(myAdults.size()) - static_cast<int>(myCars.size());
     while (peopleInNeed > 0) {
         if (RandHelper::rand() < rate) {
@@ -90,14 +84,14 @@ AGHousehold::getCarNbr() {
     return static_cast<int>(myCars.size());
 }
 
-unsigned int
+int
 AGHousehold::getPeopleNbr() {
-    return static_cast<unsigned int>(myAdults.size() + myChildren.size());
+    return static_cast<int>(myAdults.size() + myChildren.size());
 }
 
-unsigned int
+int
 AGHousehold::getAdultNbr() {
-    return static_cast<unsigned int>(myAdults.size());
+    return static_cast<int>(myAdults.size());
 }
 
 const std::list<AGAdult>&
@@ -117,7 +111,7 @@ AGHousehold::getCars() const {
 
 bool
 AGHousehold::isCloseFromPubTransport(std::list<AGPosition>* pubTransport) {
-    SUMOReal distToPT = myLocation.minDistanceTo(*pubTransport);
+    double distToPT = myLocation.minDistanceTo(*pubTransport);
     if (distToPT > myCity->statData.maxFootDistance) {
         return false;
     }
@@ -126,7 +120,7 @@ AGHousehold::isCloseFromPubTransport(std::list<AGPosition>* pubTransport) {
 
 bool
 AGHousehold::isCloseFromPubTransport(std::map<int, AGPosition>* pubTransport) {
-    SUMOReal distToPT = myLocation.minDistanceTo(*pubTransport);
+    double distToPT = myLocation.minDistanceTo(*pubTransport);
     if (distToPT > myCity->statData.maxFootDistance) {
         return false;
     }
@@ -204,4 +198,5 @@ AGHousehold::retiredHouseholders() {
     return (myAdults.front().getAge() >= myCity->statData.limitAgeRetirement);
 }
 
+
 /****************************************************************************/
