@@ -1,37 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGPosition.cpp
 /// @author  Piotr Woznica
 /// @author  Walter Bamberger
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGPosition.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // References a street of the city and defines a position in this street
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "AGPosition.h"
 #include "AGStreet.h"
@@ -44,7 +35,7 @@
 // ===========================================================================
 // method definitions
 // ===========================================================================
-AGPosition::AGPosition(const AGStreet& str, SUMOReal pos) :
+AGPosition::AGPosition(const AGStreet& str, double pos) :
     street(&str), position(pos), pos2d(compute2dPosition()) {
 }
 
@@ -66,16 +57,16 @@ AGPosition::operator==(const AGPosition& pos) const {
 }
 
 
-SUMOReal
+double
 AGPosition::distanceTo(const AGPosition& otherPos) const {
     return pos2d.distanceTo(otherPos.pos2d);
 }
 
 
-SUMOReal
+double
 AGPosition::minDistanceTo(const std::list<AGPosition>& positions) const {
-    SUMOReal minDist = std::numeric_limits<SUMOReal>::infinity();
-    SUMOReal tempDist;
+    double minDist = std::numeric_limits<double>::infinity();
+    double tempDist;
     std::list<AGPosition>::const_iterator itt;
 
     for (itt = positions.begin(); itt != positions.end(); ++itt) {
@@ -88,10 +79,10 @@ AGPosition::minDistanceTo(const std::list<AGPosition>& positions) const {
 }
 
 
-SUMOReal
+double
 AGPosition::minDistanceTo(const std::map<int, AGPosition>& positions) const {
-    SUMOReal minDist = std::numeric_limits<SUMOReal>::infinity();
-    SUMOReal tempDist;
+    double minDist = std::numeric_limits<double>::infinity();
+    double tempDist;
     std::map<int, AGPosition>::const_iterator itt;
 
     for (itt = positions.begin(); itt != positions.end(); ++itt) {
@@ -110,13 +101,13 @@ AGPosition::getStreet() const {
 }
 
 
-SUMOReal
+double
 AGPosition::getPosition() const {
     return position;
 }
 
 
-SUMOReal
+double
 AGPosition::randomPositionInStreet(const AGStreet& s) {
     return RandHelper::rand(0.0, s.getLength());
 }
@@ -136,4 +127,5 @@ AGPosition::compute2dPosition() const {
     return position2d;
 }
 
+
 /****************************************************************************/
