@@ -1,43 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// activitygen module
+// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    AGPosition.h
 /// @author  Piotr Woznica
 /// @author  Walter Bamberger
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGPosition.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // References a street of the city and defines a position in this street
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-// activitygen module
-// Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef AGPOSITION_H
-#define AGPOSITION_H
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include "../../utils/geom/Position.h"
 #include <list>
 #include <map>
+#include <utils/geom/Position.h>
 
 
 // ===========================================================================
@@ -71,7 +61,7 @@ public:
      * param[in] str the street on which the AGPosition is located
      * param[in] pos the distance from the from node of the street
      */
-    AGPosition(const AGStreet& str, SUMOReal pos);
+    AGPosition(const AGStreet& str, double pos);
     /** @brief Constructs an AGPosition at a random point on a street.
      *
      * This constructor determines the distance from the from node with
@@ -94,7 +84,7 @@ public:
      *
      * @return the relative position
      */
-    SUMOReal getPosition() const;
+    double getPosition() const;
 
     /** @brief Tests whether two positions are at the same place.
      *
@@ -111,7 +101,7 @@ public:
      * @param[in] the other position the distance in computed to
      * @return the distance
      */
-    SUMOReal distanceTo(const AGPosition& otherPos) const;
+    double distanceTo(const AGPosition& otherPos) const;
 
     /** @brief Computes the distance to the closest position in a list.
      *
@@ -121,7 +111,7 @@ public:
      * @param[in] positions the list of positions the distances are computed to
      * @return the minimal distance
      */
-    SUMOReal minDistanceTo(const std::list<AGPosition>& positions) const;
+    double minDistanceTo(const std::list<AGPosition>& positions) const;
 
     /** @brief Computes the distance to the closest position in a map.
      *
@@ -131,7 +121,7 @@ public:
      * @param[in] positions the map of positions the distances are computed to
      * @return the minimal distance
      */
-    SUMOReal minDistanceTo(const std::map<int, AGPosition>& positions) const;
+    double minDistanceTo(const std::map<int, AGPosition>& positions) const;
 
     /** @brief Prints out a summary of the properties of this class
      * on standard output.
@@ -140,14 +130,14 @@ public:
 
 private:
     const AGStreet* street;
-    SUMOReal position;
+    double position;
     Position pos2d;
 
     /** @brief Determines a random relative position on a street.
      *
      * @return the random relative position
      */
-    static SUMOReal randomPositionInStreet(const AGStreet& street);
+    static double randomPositionInStreet(const AGStreet& street);
 
     /** Creates a Position object to the street and position attribute of
      * this class.
@@ -158,7 +148,3 @@ private:
      */
     Position compute2dPosition() const;
 };
-
-#endif /* AGPOSITION_H */
-
-/****************************************************************************/
