@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2006-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RODFDetector.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Eric Nicolay
@@ -8,31 +20,10 @@
 /// @author  Laura Bieker
 /// @author  Melanie Knocke
 /// @date    Thu, 16.03.2006
-/// @version $Id: RODFDetector.cpp 24095 2017-04-27 13:45:19Z namdre $
 ///
 // Class representing a detector within the DFROUTER
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2006-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cassert>
 #include "RODFDetector.h"
@@ -48,7 +39,7 @@
 #include <utils/vehicle/SUMOVTypeParameter.h>
 #include <utils/distribution/RandomDistributor.h>
 #include <utils/common/StdDefs.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/geom/GeomHelper.h>
 #include "RODFNet.h"
 #include <utils/iodevices/OutputDevice.h>
@@ -61,13 +52,13 @@
 // ===========================================================================
 RODFDetector::RODFDetector(const std::string& id, const std::string& laneID,
                            double pos, const RODFDetectorType type)
-    : Named(id), myLaneID(laneID), myPosition(pos), myType(type), myRoutes(0) {}
+    : Named(id), myLaneID(laneID), myPosition(pos), myType(type), myRoutes(nullptr) {}
 
 
 RODFDetector::RODFDetector(const std::string& id, const RODFDetector& f)
     : Named(id), myLaneID(f.myLaneID), myPosition(f.myPosition),
-      myType(f.myType), myRoutes(0) {
-    if (f.myRoutes != 0) {
+      myType(f.myType), myRoutes(nullptr) {
+    if (f.myRoutes != nullptr) {
         myRoutes = new RODFRouteCont(*(f.myRoutes));
     }
 }
@@ -99,7 +90,7 @@ void
 RODFDetector::computeSplitProbabilities(const RODFNet* net, const RODFDetectorCon& detectors,
                                         const RODFDetectorFlows& flows,
                                         SUMOTime startTime, SUMOTime endTime, SUMOTime stepOffset) {
-    if (myRoutes == 0) {
+    if (myRoutes == nullptr) {
         return;
     }
     // compute edges to determine split probabilities
@@ -175,7 +166,7 @@ RODFDetector::buildDestinationDistribution(const RODFDetectorCon& detectors,
         SUMOTime startTime, SUMOTime endTime, SUMOTime stepOffset,
         const RODFNet& net,
         std::map<SUMOTime, RandomDistributor<int>* >& into) const {
-    if (myRoutes == 0) {
+    if (myRoutes == nullptr) {
         if (myType != DISCARDED_DETECTOR && myType != BETWEEN_DETECTOR) {
             WRITE_ERROR("Missing routes for detector '" + myID + "'.");
         }
@@ -203,7 +194,7 @@ RODFDetector::buildDestinationDistribution(const RODFDetectorCon& detectors,
                     continue;
                 }
                 const std::map<RODFEdge*, double>& tprobs = probs[(int)((time - startTime) / stepOffset)];
-                RODFEdge* splitEdge = 0;
+                RODFEdge* splitEdge = nullptr;
                 for (std::map<RODFEdge*, double>::const_iterator k = tprobs.begin(); k != tprobs.end(); ++k) {
                     if (find(j, (*ri).edges2Pass.end(), (*k).first) != (*ri).edges2Pass.end()) {
                         prob *= (*k).second;
@@ -211,8 +202,8 @@ RODFDetector::buildDestinationDistribution(const RODFDetectorCon& detectors,
                         break;
                     }
                 }
-                if (splitEdge != 0) {
-                    j = find(j, (*ri).edges2Pass.end(), splitEdge);
+                if (splitEdge != nullptr) {
+                    j = std::find(j, (*ri).edges2Pass.end(), splitEdge);
                 } else {
                     ++j;
                 }
@@ -264,7 +255,7 @@ RODFDetector::addRoutes(RODFRouteCont* routes) {
 
 void
 RODFDetector::addRoute(RODFRouteDesc& nrd) {
-    if (myRoutes == 0) {
+    if (myRoutes == nullptr) {
         myRoutes = new RODFRouteCont();
     }
     myRoutes->addRouteDesc(nrd);
@@ -273,7 +264,7 @@ RODFDetector::addRoute(RODFRouteDesc& nrd) {
 
 bool
 RODFDetector::hasRoutes() const {
-    return myRoutes != 0 && myRoutes->get().size() != 0;
+    return myRoutes != nullptr && myRoutes->get().size() != 0;
 }
 
 
@@ -293,7 +284,7 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
         out.writeXMLHeader("additional", "additional_file.xsd");
     }
     // routes
-    if (myRoutes != 0 && myRoutes->get().size() != 0) {
+    if (myRoutes != nullptr && myRoutes->get().size() != 0) {
         const std::vector<RODFRouteDesc>& routes = myRoutes->get();
         out.openTag(SUMO_TAG_ROUTE_DISTRIBUTION).writeAttr(SUMO_ATTR_ID, myID);
         bool isEmptyDist = true;
@@ -303,11 +294,10 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
             }
         }
         for (std::vector<RODFRouteDesc>::const_iterator i = routes.begin(); i != routes.end(); ++i) {
-            if ((*i).overallProb > 0 || includeUnusedRoutes) {
-                out.openTag(SUMO_TAG_ROUTE).writeAttr(SUMO_ATTR_REFID, (*i).routename).writeAttr(SUMO_ATTR_PROB, (*i).overallProb).closeTag();
-            }
             if (isEmptyDist) {
-                out.openTag(SUMO_TAG_ROUTE).writeAttr(SUMO_ATTR_REFID, (*i).routename).writeAttr(SUMO_ATTR_PROB, double(1)).closeTag();
+                out.openTag(SUMO_TAG_ROUTE).writeAttr(SUMO_ATTR_REFID, (*i).routename).writeAttr(SUMO_ATTR_PROB, 1.0).closeTag();
+            } else if ((*i).overallProb > 0 || includeUnusedRoutes) {
+                out.openTag(SUMO_TAG_ROUTE).writeAttr(SUMO_ATTR_REFID, (*i).routename).writeAttr(SUMO_ATTR_PROB, (*i).overallProb).closeTag();
             }
         }
         out.closeTag(); // routeDistribution
@@ -316,6 +306,7 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
         return false;
     }
     // insertions
+    int vehicleIndex = 0;
     if (insertionsOnly || flows.knows(myID)) {
         // get the flows for this detector
         const std::vector<FlowDef>& mflows = flows.getFlowDefs(myID);
@@ -328,14 +319,36 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
             // get flows at end
             RandomDistributor<int>* destDist = dists.find(time) != dists.end() ? dists.find(time)->second : 0;
             // go through the cars
-            int carNo = (int)((srcFD.qPKW + srcFD.qLKW) * scale);
-            for (int car = 0; car < carNo; ++car) {
+            const int numCars = (int)((srcFD.qPKW + srcFD.qLKW) * scale);
+
+
+            std::vector<SUMOTime> departures;
+            if (oc.getBool("randomize-flows")) {
+                for (int i = 0; i < numCars; ++i) {
+                    departures.push_back(time + RandHelper::rand(stepOffset));
+                }
+                std::sort(departures.begin(), departures.end());
+            } else {
+                for (int i = 0; i < numCars; ++i) {
+                    departures.push_back(time + (SUMOTime)(stepOffset * i / (double)numCars));
+                }
+            }
+
+            for (int car = 0; car < numCars; ++car) {
                 // get the vehicle parameter
                 double v = -1;
                 std::string vtype;
-                int destIndex = destDist != 0 && destDist->getOverallProb() > 0 ? (int) destDist->get() : -1;
+                int destIndex = -1;
+                if (destDist != nullptr) {
+                    if (destDist->getOverallProb() > 0) {
+                        destIndex = destDist->get();
+                    } else if (myRoutes->get().size() > 0) {
+                        // equal probabilities. see writeEmitterDefinition()
+                        destIndex = RandHelper::rand((int)myRoutes->get().size());
+                    }
+                }
                 if (srcFD.isLKW >= 1) {
-                    srcFD.isLKW = srcFD.isLKW - (double) 1.;
+                    srcFD.isLKW = srcFD.isLKW - 1.;
                     v = srcFD.vLKW;
                     vtype = "LKW";
                 } else {
@@ -346,18 +359,14 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
                 if (v <= 0 || v > 250) {
                     v = defaultSpeed;
                 } else {
-                    v = (double)(v / 3.6);
+                    v /= 3.6;
                 }
                 // compute the departure time
-                SUMOTime ctime = (SUMOTime)(time + ((double) stepOffset * (double) car / (double) carNo));
+                const SUMOTime ctime = departures[car];
 
                 // write
                 out.openTag(SUMO_TAG_VEHICLE);
-                if (getType() == SOURCE_DETECTOR) {
-                    out.writeAttr(SUMO_ATTR_ID, "emitter_" + myID + "_" + toString(ctime));
-                } else {
-                    out.writeAttr(SUMO_ATTR_ID, "calibrator_" + myID + "_" + toString(ctime));
-                }
+                out.writeAttr(SUMO_ATTR_ID, myID + "." + toString(vehicleIndex));
                 if (oc.getBool("vtype")) {
                     out.writeAttr(SUMO_ATTR_TYPE, vtype);
                 }
@@ -365,7 +374,7 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
                 if (oc.isSet("departlane")) {
                     out.writeNonEmptyAttr(SUMO_ATTR_DEPARTLANE, oc.getString("departlane"));
                 } else {
-                    out.writeAttr(SUMO_ATTR_DEPARTLANE, TplConvert::_2int(myLaneID.substr(myLaneID.rfind("_") + 1).c_str()));
+                    out.writeAttr(SUMO_ATTR_DEPARTLANE, StringUtils::toInt(myLaneID.substr(myLaneID.rfind("_") + 1)));
                 }
                 if (oc.isSet("departpos")) {
                     std::string posDesc = oc.getString("departpos");
@@ -373,11 +382,11 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
                         double position = myPosition;
                         if (posDesc.length() > 8) {
                             if (posDesc[8] == '+') {
-                                position += TplConvert::_2double(posDesc.substr(9).c_str());
+                                position += StringUtils::toDouble(posDesc.substr(9));
                             } else if (posDesc[8] == '-') {
-                                position -= TplConvert::_2double(posDesc.substr(9).c_str());
+                                position -= StringUtils::toDouble(posDesc.substr(9));
                             } else {
-                                throw NumberFormatException();
+                                throw NumberFormatException("");
                             }
                         }
                         out.writeAttr(SUMO_ATTR_DEPARTPOS, position);
@@ -408,6 +417,7 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
                 }
                 out.closeTag();
                 srcFD.isLKW += srcFD.fLKW;
+                vehicleIndex++;
             }
         }
     }
@@ -421,7 +431,7 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
 bool
 RODFDetector::writeRoutes(std::vector<std::string>& saved,
                           OutputDevice& out) {
-    if (myRoutes != 0) {
+    if (myRoutes != nullptr) {
         return myRoutes->save(saved, "", out);
     }
     return false;
@@ -638,17 +648,17 @@ RODFDetectorCon::writeEmitters(const std::string& file,
         // write separate types
         SUMOVTypeParameter pkwType = SUMOVTypeParameter("PKW", SVC_PASSENGER);
         setSpeedFactorAndDev(pkwType, net.getMaxSpeedFactorPKW(), net.getAvgSpeedFactorPKW(), speedDev, forceDev);
-        pkwType.setParameter |= VTYPEPARS_VEHICLECLASS_SET;
+        pkwType.parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
         pkwType.write(vTypeOut);
         SUMOVTypeParameter lkwType = SUMOVTypeParameter("LKW", SVC_TRUCK);
         setSpeedFactorAndDev(lkwType, net.getMaxSpeedFactorLKW(), net.getAvgSpeedFactorLKW(), speedDev, forceDev);
-        lkwType.setParameter |= VTYPEPARS_VEHICLECLASS_SET;
+        lkwType.parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
         lkwType.write(vTypeOut);
     } else {
         // patch default type
         SUMOVTypeParameter type = SUMOVTypeParameter(DEFAULT_VTYPE_ID, SVC_PASSENGER);
         setSpeedFactorAndDev(type, MAX2(net.getMaxSpeedFactorPKW(), net.getMaxSpeedFactorLKW()), net.getAvgSpeedFactorPKW(), speedDev, forceDev);
-        if (type.setParameter != 0) {
+        if (type.parametersSet != 0) {
             type.write(vTypeOut);
         }
     }
@@ -699,13 +709,15 @@ RODFDetectorCon::setSpeedFactorAndDev(SUMOVTypeParameter& type, double maxFactor
         // systematically low speeds can easily be caused by traffic
         // conditions. Whereas elevated speeds probably reflect speeding
         type.speedFactor.getParameter()[0] = avgFactor;
-        type.setParameter |= VTYPEPARS_SPEEDFACTOR_SET;
+        type.parametersSet |= VTYPEPARS_SPEEDFACTOR_SET;
     }
     if (forceDev || (maxFactor > 1 && maxFactor > type.speedFactor.getParameter()[0] + NUMERICAL_EPS)) {
         // setting a non-zero speed deviation causes the simulation to recompute
         // individual speedFactors to match departSpeed (MSEdge::insertVehicle())
         type.speedFactor.getParameter()[1] = dev;
-        type.setParameter |= VTYPEPARS_SPEEDFACTOR_SET;
+        type.parametersSet |= VTYPEPARS_SPEEDFACTOR_SET;
+    } else {
+        type.speedFactor.getParameter()[1] = -1; // do not write speedDev, only simple speedFactor
     }
 }
 
@@ -747,7 +759,7 @@ RODFDetectorCon::getAggFlowFor(const ROEdge* edge, SUMOTime time, SUMOTime perio
                                const RODFDetectorFlows&) const {
     UNUSED_PARAMETER(period);
     UNUSED_PARAMETER(time);
-    if (edge == 0) {
+    if (edge == nullptr) {
         return 0;
     }
 //    double stepOffset = 60; // !!!
@@ -800,7 +812,7 @@ RODFDetectorCon::writeSpeedTrigger(const RODFNet* const net,
         if (det->getType() == SINK_DETECTOR && flows.knows(det->getID())) {
             std::string filename = FileHelpers::getFilePath(file) + "vss_" + det->getID() + ".def.xml";
             out.openTag(SUMO_TAG_VSS).writeAttr(SUMO_ATTR_ID, StringUtils::escapeXML(det->getID())).writeAttr(SUMO_ATTR_LANES, det->getLaneID()).writeAttr(SUMO_ATTR_FILE, filename).closeTag();
-            double defaultSpeed = net != 0 ? net->getEdge(det->getEdgeID())->getSpeedLimit() : (double) 200.;
+            double defaultSpeed = net != nullptr ? net->getEdge(det->getEdgeID())->getSpeedLimit() : (double) 200.;
             det->writeSingleSpeedTrigger(filename, flows, startTime, endTime, stepOffset, defaultSpeed);
         }
     }
@@ -863,7 +875,7 @@ RODFDetectorCon::removeDetector(const std::string& id) {
     myDetectorMap.erase(ri1);
     //
     std::vector<RODFDetector*>::iterator ri2 =
-        find(myDetectors.begin(), myDetectors.end(), oldDet);
+        std::find(myDetectors.begin(), myDetectors.end(), oldDet);
     myDetectors.erase(ri2);
     //
     bool found = false;
@@ -941,7 +953,7 @@ RODFDetectorCon::getAnyDetectorForEdge(const RODFEdge* const edge) const {
 void
 RODFDetectorCon::clearDists(std::map<SUMOTime, RandomDistributor<int>* >& dists) const {
     for (std::map<SUMOTime, RandomDistributor<int>* >::iterator i = dists.begin(); i != dists.end(); ++i) {
-        delete(*i).second;
+        delete (*i).second;
     }
 }
 
