@@ -1,44 +1,31 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2006-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RODFDetectorFlow.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Eric Nicolay
 /// @author  Michael Behrisch
 /// @author  Melanie Knocke
 /// @date    Thu, 16.03.2006
-/// @version $Id: RODFDetectorFlow.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Storage for flows within the DFROUTER
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2006-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <cassert>
 #include "RODFDetectorFlow.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -55,7 +42,7 @@ RODFDetectorFlows::~RODFDetectorFlows() {}
 void
 RODFDetectorFlows::addFlow(const std::string& id, SUMOTime t, const FlowDef& fd) {
     if (myFastAccessFlows.find(id) == myFastAccessFlows.end()) {
-        size_t noItems = (size_t)((myEndTime - myBeginTime) / myStepOffset);
+        int noItems = (int)((myEndTime - myBeginTime) / myStepOffset);
         myFastAccessFlows[id] = std::vector<FlowDef>(noItems);
         std::vector<FlowDef>& cflows = myFastAccessFlows[id];
         // initialise
@@ -90,8 +77,6 @@ RODFDetectorFlows::addFlow(const std::string& id, SUMOTime t, const FlowDef& fd)
 }
 
 
-
-
 void
 RODFDetectorFlows::setFlows(const std::string& detector_id,
                             std::vector<FlowDef>& flows) {
@@ -127,9 +112,9 @@ RODFDetectorFlows::getFlowDefs(const std::string& id) const {
 }
 
 
-SUMOReal
+double
 RODFDetectorFlows::getFlowSumSecure(const std::string& id) const {
-    SUMOReal ret = 0;
+    double ret = 0;
     if (knows(id)) {
         const std::vector<FlowDef>& flows = getFlowDefs(id);
         for (std::vector<FlowDef>::const_iterator i = flows.begin(); i != flows.end(); ++i) {
@@ -141,13 +126,13 @@ RODFDetectorFlows::getFlowSumSecure(const std::string& id) const {
 }
 
 
-SUMOReal
+double
 RODFDetectorFlows::getMaxDetectorFlow() const {
     if (myMaxDetectorFlow < 0) {
-        SUMOReal max = 0;
+        double max = 0;
         std::map<std::string, std::vector<FlowDef> >::const_iterator j;
         for (j = myFastAccessFlows.begin(); j != myFastAccessFlows.end(); ++j) {
-            SUMOReal curr = 0;
+            double curr = 0;
             const std::vector<FlowDef>& flows = (*j).second;
             for (std::vector<FlowDef>::const_iterator i = flows.begin(); i != flows.end(); ++i) {
                 curr += (*i).qPKW;
@@ -171,7 +156,7 @@ RODFDetectorFlows::mesoJoin(const std::string& nid,
             continue;
         }
         std::vector<FlowDef>& flows = myFastAccessFlows[*i];
-        size_t index = 0;
+        int index = 0;
         for (SUMOTime t = myBeginTime; t != myEndTime; t += myStepOffset) {
             addFlow(nid, t, flows[index++]); // !!!
         }
@@ -185,8 +170,8 @@ RODFDetectorFlows::printAbsolute() const {
     for (std::map<std::string, std::vector<FlowDef> >::const_iterator i = myFastAccessFlows.begin(); i != myFastAccessFlows.end(); ++i) {
         std::cout << (*i).first << ":";
         const std::vector<FlowDef>& flows = (*i).second;
-        SUMOReal qPKW = 0;
-        SUMOReal qLKW = 0;
+        double qPKW = 0;
+        double qLKW = 0;
         for (std::vector<FlowDef>::const_iterator j = flows.begin(); j != flows.end(); ++j) {
             qPKW += (*j).qPKW;
             qLKW += (*j).qLKW;
@@ -195,5 +180,5 @@ RODFDetectorFlows::printAbsolute() const {
     }
 }
 
-/****************************************************************************/
 
+/****************************************************************************/
