@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RODFDetectorFlow.h
 /// @author  Daniel Krajzewicz
 /// @author  Eric Nicolay
 /// @author  Michael Behrisch
 /// @date    Thu, 16.03.2006
-/// @version $Id: RODFDetectorFlow.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // missing_desc
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef RODFDetectorFlow_h
-#define RODFDetectorFlow_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <utils/common/SUMOTime.h>
 #include <map>
@@ -47,19 +37,19 @@
  */
 struct FlowDef {
     // Number of passenger vehicles that passed within the described time
-    SUMOReal qPKW;
+    double qPKW;
     // Number of heavy duty vehicles that passed within the described time
-    SUMOReal qLKW;
+    double qLKW;
     // Mean velocity of passenger vehicles within the described time
-    SUMOReal vPKW;
+    double vPKW;
     // Mean velocity of heavy duty vehicles within the described time
-    SUMOReal vLKW;
+    double vLKW;
     // begin time (in s)
 //    int time;
     // probability for having a heavy duty vehicle(qKFZ!=0 ? (qLKW / qKFZ) : 0;)
-    SUMOReal fLKW;
+    double fLKW;
     // initialise with 0
-    mutable SUMOReal isLKW;
+    mutable double isLKW;
     //
     bool firstSet;
 };
@@ -84,8 +74,8 @@ public:
 
     const std::vector<FlowDef>& getFlowDefs(const std::string& id) const;
     bool knows(const std::string& det_id) const;
-    SUMOReal getFlowSumSecure(const std::string& id) const;
-    SUMOReal getMaxDetectorFlow() const;
+    double getFlowSumSecure(const std::string& id) const;
+    double getMaxDetectorFlow() const;
     void printAbsolute() const;
 
     void mesoJoin(const std::string& nid, const std::vector<std::string>& oldids);
@@ -93,7 +83,7 @@ public:
 protected:
     std::map<std::string, std::vector<FlowDef> > myFastAccessFlows;
     SUMOTime myBeginTime, myEndTime, myStepOffset;
-    mutable SUMOReal myMaxDetectorFlow;
+    mutable double myMaxDetectorFlow;
 
 private:
     /// @brief Invalidated copy constructor
@@ -103,9 +93,3 @@ private:
     RODFDetectorFlows& operator=(const RODFDetectorFlows& src);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
