@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2006-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RODFFrame.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Eric Nicolay
@@ -6,31 +18,10 @@
 /// @author  Michael Behrisch
 /// @author  Melanie Knocke
 /// @date    Thu, 16.03.2006
-/// @version $Id: RODFFrame.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Sets and checks options for df-routing
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2006-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <fstream>
@@ -62,8 +53,6 @@ RODFFrame::fillOptions() {
     oc.addOptionSubTopic("Processing");
     oc.addOptionSubTopic("Defaults");
     oc.addOptionSubTopic("Time");
-    SystemFrame::addReportOptions(oc); // fill this subtopic, too
-
 
     // register the options
     // register input-options
@@ -85,13 +74,15 @@ RODFFrame::fillOptions() {
     oc.addSynonyme("measure-files", "detector-flow-files", true);
     oc.addDescription("measure-files", "Input", "Loads detector flows from FILE(s)");
 
+    // need to do this here to be able to check for network and route input options
+    SystemFrame::addReportOptions(oc);
 
     // register output options
     oc.doRegister("routes-output", 'o', new Option_FileName());
     oc.addDescription("routes-output", "Output", "Saves computed routes to FILE");
 
     oc.doRegister("routes-for-all", new Option_Bool(false));
-    oc.addDescription("routes-for-all", "Output", "Forces DFROUTER to compute routes for in-between detectors");
+    oc.addDescription("routes-for-all", "Output", "Forces dfrouter to compute routes for in-between detectors");
 
     oc.doRegister("detector-output", new Option_FileName());
     oc.addSynonyme("detector-output", "detectors-output", true);
@@ -106,7 +97,7 @@ RODFFrame::fillOptions() {
     oc.doRegister("vtype", new Option_Bool(false));
     oc.addDescription("vtype", "Output", "Add vehicle types to the emitters file (PKW, LKW)");
 
-    oc.doRegister("vtype-output", new Option_FileName(""));
+    oc.doRegister("vtype-output", new Option_FileName(StringVector({ "" })));
     oc.addDescription("vtype-output", "Output", "Write generated vehicle types into separate FILE instead of including them into the emitters-output");
 
     oc.doRegister("emitters-poi-output", new Option_FileName()); // !!! describe
@@ -161,9 +152,12 @@ RODFFrame::fillOptions() {
     oc.doRegister("emissions-only", new Option_Bool(false));
     oc.addDescription("emissions-only", "Processing", "Writes only emission times");
 
-    oc.doRegister("disallowed-edges", new Option_String(""));
+    oc.doRegister("disallowed-edges", new Option_StringVector(StringVector({ "" })));
     oc.addDescription("disallowed-edges", "Processing", "Do not route on these edges");
 
+    oc.doRegister("vclass", new Option_String("ignoring"));
+    oc.addDescription("vclass", "Processing", "Only route on edges allowing the given vclass");
+
     oc.doRegister("keep-turnarounds", new Option_Bool(false));
     oc.addDescription("keep-turnarounds", "Processing", "Allow turnarounds as route continuations");
 
@@ -172,6 +166,9 @@ RODFFrame::fillOptions() {
     oc.addSynonyme("min-route-length", "min-distance", true);
     oc.addDescription("min-route-length", "Processing", "Minimum distance in meters between start and end node of every route");
 
+    oc.doRegister("randomize-flows", new Option_Bool(false));
+    oc.addDescription("randomize-flows", "Processing", "generate random departure times for emitted vehicles");
+
     // flow reading
     oc.doRegister("time-factor", new Option_String("60", "TIME"));
     oc.addDescription("time-factor", "Processing", "Multiply flow times with TIME to get seconds");
@@ -203,12 +200,6 @@ RODFFrame::fillOptions() {
     oc.doRegister("respect-concurrent-inflows", new Option_Bool(false));
     oc.addDescription("respect-concurrent-inflows", "Processing", "Try to determine further inflows to an inbetween detector when computing split probabilities");
 
-    /* disabled, see ticket #521
-    oc.doRegister("join-lanes", new Option_Bool(false));
-    oc.addSynonyme("join-lanes", "mesosim", true);
-    oc.addDescription("join-lanes", "Processing", "Joins detectors lying on same height");
-    */
-
     //
     oc.doRegister("scale", new Option_Float(1.));
     oc.addDescription("scale", "Processing", "Scale factor for flows");
@@ -264,6 +255,4 @@ RODFFrame::checkOptions() {
 }
 
 
-
 /****************************************************************************/
-
