@@ -1,33 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RODFNet.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Eric Nicolay
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Thu, 16.03.2006
-/// @version $Id: RODFNet.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A DFROUTER-network
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cassert>
 #include <iostream>
@@ -46,18 +39,19 @@
 #include <utils/common/UtilExceptions.h>
 #include <utils/geom/GeomHelper.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
-RODFNet::RODFNet(bool amInHighwayMode)
-    : RONet(), myAmInHighwayMode(amInHighwayMode),
-      mySourceNumber(0), mySinkNumber(0), myInBetweenNumber(0), myInvalidNumber(0) {
+RODFNet::RODFNet(bool amInHighwayMode) :
+    RONet(), myAmInHighwayMode(amInHighwayMode),
+    mySourceNumber(0), mySinkNumber(0), myInBetweenNumber(0), myInvalidNumber(0),
+    myMaxSpeedFactorPKW(1),
+    myMaxSpeedFactorLKW(1),
+    myAvgSpeedFactorPKW(1),
+    myAvgSpeedFactorLKW(1) {
     myDisallowedEdges = OptionsCont::getOptions().getStringVector("disallowed-edges");
+    myAllowedVClass = getVehicleClassID(OptionsCont::getOptions().getString("vclass"));
     myKeepTurnarounds = OptionsCont::getOptions().getBool("keep-turnarounds");
 }
 
@@ -66,16 +60,25 @@ RODFNet::~RODFNet() {
 }
 
 
+bool
+RODFNet::isAllowed(const ROEdge* const edge) const {
+    return (!edge->isInternal() && !edge->isWalkingArea() && !edge->isCrossing() &&
+            (edge->getPermissions() & myAllowedVClass) == myAllowedVClass &&
+            find(myDisallowedEdges.begin(), myDisallowedEdges.end(), edge->getID()) == myDisallowedEdges.end());
+
+}
+
+
 void
 RODFNet::buildApproachList() {
-    const std::map<std::string, ROEdge*>& edges = getEdgeMap();
-    for (std::map<std::string, ROEdge*>::const_iterator rit = edges.begin(); rit != edges.end(); ++rit) {
-        ROEdge* ce = (*rit).second;
-        const ROEdgeVector& successors = ce->getSuccessors();
-        for (ROEdgeVector::const_iterator it = successors.begin(); it != successors.end(); ++it) {
-            ROEdge* help = *it;
-            if (find(myDisallowedEdges.begin(), myDisallowedEdges.end(), help->getID()) != myDisallowedEdges.end()) {
-                // edges in sinks will not be used
+    for (const auto& rit : getEdgeMap()) {
+        ROEdge* const ce = rit.second;
+        if (!isAllowed(ce)) {
+            continue;
+        }
+        for (ROEdge* const help : ce->getSuccessors()) {
+            if (!isAllowed(help)) {
+                // blocked edges will not be used
                 continue;
             }
             if (!myKeepTurnarounds && help->getToJunction() == ce->getFromJunction()) {
@@ -83,14 +86,8 @@ RODFNet::buildApproachList() {
                 continue;
             }
             // add the connection help->ce to myApproachingEdges
-            if (myApproachingEdges.find(help) == myApproachingEdges.end()) {
-                myApproachingEdges[help] = ROEdgeVector();
-            }
             myApproachingEdges[help].push_back(ce);
             // add the connection ce->help to myApproachingEdges
-            if (myApproachedEdges.find(ce) == myApproachedEdges.end()) {
-                myApproachedEdges[ce] = ROEdgeVector();
-            }
             myApproachedEdges[ce].push_back(help);
         }
     }
@@ -197,7 +194,7 @@ RODFNet::computeRoutesFor(ROEdge* edge, RODFRouteDesc& base, int /*no*/,
     std::map<ROEdge*, ROEdgeVector > dets2Follow;
     dets2Follow[edge] = ROEdgeVector();
     base.passedNo = 0;
-    SUMOReal minDist = OptionsCont::getOptions().getFloat("min-route-length");
+    double minDist = OptionsCont::getOptions().getFloat("min-route-length");
     toSolve.push(base);
     while (!toSolve.empty()) {
         RODFRouteDesc current = toSolve.top();
@@ -224,7 +221,7 @@ RODFNet::computeRoutesFor(ROEdge* edge, RODFRouteDesc& base, int /*no*/,
         if (!hasApproached(last)) {
             // ok, no further connections to follow
             current.factor = 1.;
-            SUMOReal cdist = current.edges2Pass[0]->getFromJunction()->getPosition().distanceTo(current.edges2Pass.back()->getToJunction()->getPosition());
+            double cdist = current.edges2Pass[0]->getFromJunction()->getPosition().distanceTo(current.edges2Pass.back()->getToJunction()->getPosition());
             if (minDist < cdist) {
                 into.addRouteDesc(current);
             }
@@ -248,7 +245,7 @@ RODFNet::computeRoutesFor(ROEdge* edge, RODFRouteDesc& base, int /*no*/,
 ///!!!                        //toDiscard.push_back(current);
                     }
                     current.factor = 1.;
-                    SUMOReal cdist = current.edges2Pass[0]->getFromJunction()->getPosition().distanceTo(current.edges2Pass.back()->getToJunction()->getPosition());
+                    double cdist = current.edges2Pass[0]->getFromJunction()->getPosition().distanceTo(current.edges2Pass.back()->getToJunction()->getPosition());
                     if (minDist < cdist) {
                         into.addRouteDesc(current);
                     }
@@ -266,7 +263,7 @@ RODFNet::computeRoutesFor(ROEdge* edge, RODFRouteDesc& base, int /*no*/,
         // check for highway off-ramps
         if (myAmInHighwayMode) {
             // if it's beside the highway...
-            if (last->getSpeed() < 19.4 && last != getDetectorEdge(det)) {
+            if (last->getSpeedLimit() < 19.4 && last != getDetectorEdge(det)) {
                 // ... and has more than one following edge
                 if (myApproachedEdges.find(last)->second.size() > 1) {
                     // -> let's add this edge and the following, but not any further
@@ -278,13 +275,13 @@ RODFNet::computeRoutesFor(ROEdge* edge, RODFRouteDesc& base, int /*no*/,
         // check for missing end connections
         if (!addNextNoFurther) {
             // ... if this one would be processed, but already too many edge
-            //  without a detector occured
+            //  without a detector occurred
             if (current.passedNo > maxFollowingLength) {
                 // mark not to process any further
                 WRITE_WARNING("Could not close route for '" + det.getID() + "'");
                 unfoundEnds.push_back(current);
                 current.factor = 1.;
-                SUMOReal cdist = current.edges2Pass[0]->getFromJunction()->getPosition().distanceTo(current.edges2Pass.back()->getToJunction()->getPosition());
+                double cdist = current.edges2Pass[0]->getFromJunction()->getPosition().distanceTo(current.edges2Pass.back()->getToJunction()->getPosition());
                 if (minDist < cdist) {
                     into.addRouteDesc(current);
                 }
@@ -294,22 +291,22 @@ RODFNet::computeRoutesFor(ROEdge* edge, RODFRouteDesc& base, int /*no*/,
         // ... else: loop over the next edges
         const ROEdgeVector& appr  = myApproachedEdges.find(last)->second;
         bool hadOne = false;
-        for (size_t i = 0; i < appr.size(); i++) {
+        for (int i = 0; i < (int)appr.size(); i++) {
             if (find(current.edges2Pass.begin(), current.edges2Pass.end(), appr[i]) != current.edges2Pass.end()) {
                 // do not append an edge twice (do not build loops)
                 continue;
             }
             RODFRouteDesc t(current);
-            t.duration_2 += (appr[i]->getLength() / appr[i]->getSpeed()); //!!!
+            t.duration_2 += (appr[i]->getLength() / appr[i]->getSpeedLimit()); //!!!
             t.distance += appr[i]->getLength();
             t.edges2Pass.push_back(appr[i]);
             if (!addNextNoFurther) {
-                t.passedNo = t.passedNo + 1;
+                t.passedNo++;
                 toSolve.push(t);
             } else {
                 if (!hadOne) {
-                    t.factor = (SUMOReal) 1. / (SUMOReal) appr.size();
-                    SUMOReal cdist = current.edges2Pass[0]->getFromJunction()->getPosition().distanceTo(current.edges2Pass.back()->getToJunction()->getPosition());
+                    t.factor = (double) 1. / (double) appr.size();
+                    double cdist = current.edges2Pass[0]->getFromJunction()->getPosition().distanceTo(current.edges2Pass.back()->getToJunction()->getPosition());
                     if (minDist < cdist) {
                         into.addRouteDesc(t);
                     }
@@ -328,7 +325,7 @@ RODFNet::computeRoutesFor(ROEdge* edge, RODFRouteDesc& base, int /*no*/,
             } else {
                 bool ok = into.removeRouteDesc(*i);
                 assert(ok);
-                UNUSED_PARAMETER(ok); // ony used for assertion
+                UNUSED_PARAMETER(ok); // only used for assertion
             }
         }
     } else {
@@ -362,9 +359,9 @@ RODFNet::buildRoutes(RODFDetectorCon& detcont, bool keepUnfoundEnds, bool includ
         doneEdges[e] = routes;
         RODFRouteDesc rd;
         rd.edges2Pass.push_back(e);
-        rd.duration_2 = (e->getLength() / e->getSpeed()); //!!!;
-        rd.endDetectorEdge = 0;
-        rd.lastDetectorEdge = 0;
+        rd.duration_2 = (e->getLength() / e->getSpeedLimit()); //!!!;
+        rd.endDetectorEdge = nullptr;
+        rd.lastDetectorEdge = nullptr;
         rd.distance = e->getLength();
         rd.distance2Last = 0;
         rd.duration2Last = 0;
@@ -384,22 +381,20 @@ RODFNet::buildRoutes(RODFDetectorCon& detcont, bool keepUnfoundEnds, bool includ
             const std::vector<RODFRouteDesc>& r = routes->get();
             for (std::vector<RODFRouteDesc>::const_iterator j = r.begin(); j != r.end(); ++j) {
                 const RODFRouteDesc& mrd = *j;
-                SUMOReal duration = mrd.duration_2;
-                SUMOReal distance = mrd.distance;
+                double duration = mrd.duration_2;
+                double distance = mrd.distance;
                 // go through each route's edges
                 ROEdgeVector::const_iterator routeend = mrd.edges2Pass.end();
                 for (ROEdgeVector::const_iterator k = mrd.edges2Pass.begin(); k != routeend; ++k) {
                     // check whether any detectors lies on the current edge
                     if (myDetectorsOnEdges.find(*k) == myDetectorsOnEdges.end()) {
-                        duration -= (*k)->getLength() / (*k)->getSpeed();
+                        duration -= (*k)->getLength() / (*k)->getSpeedLimit();
                         distance -= (*k)->getLength();
                         continue;
                     }
-                    // get the detectors
-                    const std::vector<std::string>& dets = myDetectorsOnEdges.find(*k)->second;
                     // go through the detectors
-                    for (std::vector<std::string>::const_iterator l = dets.begin(); l != dets.end(); ++l) {
-                        const RODFDetector& m = detcont.getDetector(*l);
+                    for (const std::string& l : myDetectorsOnEdges.find(*k)->second) {
+                        const RODFDetector& m = detcont.getDetector(l);
                         if (m.getType() == BETWEEN_DETECTOR) {
                             RODFRouteDesc nrd;
                             copy(k, routeend, back_inserter(nrd.edges2Pass));
@@ -414,7 +409,7 @@ RODFNet::buildRoutes(RODFDetectorCon& detcont, bool keepUnfoundEnds, bool includ
                             ((RODFDetector&) m).addRoute(nrd);
                         }
                     }
-                    duration -= (*k)->getLength() / (*k)->getSpeed();
+                    duration -= (*k)->getLength() / (*k)->getSpeedLimit();
                     distance -= (*k)->getLength();
                 }
             }
@@ -517,54 +512,50 @@ RODFNet::revalidateFlows(const RODFDetector* detector,
     std::vector<FlowDef> mflows;
     int index = 0;
     for (SUMOTime t = startTime; t < endTime; t += stepOffset, index++) {
+        // collect incoming
         FlowDef inFlow;
         inFlow.qLKW = 0;
         inFlow.qPKW = 0;
         inFlow.vLKW = 0;
         inFlow.vPKW = 0;
-        // collect incoming
-        {
-            // !! time difference is missing
-            for (ROEdgeVector::iterator i = previous.begin(); i != previous.end(); ++i) {
-                const std::vector<FlowDef>& flows = static_cast<const RODFEdge*>(*i)->getFlows();
-                if (flows.size() != 0) {
-                    const FlowDef& srcFD = flows[index];
-                    inFlow.qLKW += srcFD.qLKW;
-                    inFlow.qPKW += srcFD.qPKW;
-                    inFlow.vLKW += srcFD.vLKW;
-                    inFlow.vPKW += srcFD.vPKW;
-                }
+        // !! time difference is missing
+        for (const ROEdge* const e : previous) {
+            const std::vector<FlowDef>& eflows = static_cast<const RODFEdge*>(e)->getFlows();
+            if (eflows.size() != 0) {
+                const FlowDef& srcFD = eflows[index];
+                inFlow.qLKW += srcFD.qLKW;
+                inFlow.qPKW += srcFD.qPKW;
+                inFlow.vLKW += srcFD.vLKW;
+                inFlow.vPKW += srcFD.vPKW;
             }
         }
-        inFlow.vLKW /= (SUMOReal) previous.size();
-        inFlow.vPKW /= (SUMOReal) previous.size();
+        inFlow.vLKW /= (double) previous.size();
+        inFlow.vPKW /= (double) previous.size();
         // collect outgoing
         FlowDef outFlow;
         outFlow.qLKW = 0;
         outFlow.qPKW = 0;
         outFlow.vLKW = 0;
         outFlow.vPKW = 0;
-        {
-            // !! time difference is missing
-            for (ROEdgeVector::iterator i = latter.begin(); i != latter.end(); ++i) {
-                const std::vector<FlowDef>& flows = static_cast<const RODFEdge*>(*i)->getFlows();
-                if (flows.size() != 0) {
-                    const FlowDef& srcFD = flows[index];
-                    outFlow.qLKW += srcFD.qLKW;
-                    outFlow.qPKW += srcFD.qPKW;
-                    outFlow.vLKW += srcFD.vLKW;
-                    outFlow.vPKW += srcFD.vPKW;
-                }
+        // !! time difference is missing
+        for (const ROEdge* const e : latter) {
+            const std::vector<FlowDef>& eflows = static_cast<const RODFEdge*>(e)->getFlows();
+            if (eflows.size() != 0) {
+                const FlowDef& srcFD = eflows[index];
+                outFlow.qLKW += srcFD.qLKW;
+                outFlow.qPKW += srcFD.qPKW;
+                outFlow.vLKW += srcFD.vLKW;
+                outFlow.vPKW += srcFD.vPKW;
             }
         }
-        outFlow.vLKW /= (SUMOReal) latter.size();
-        outFlow.vPKW /= (SUMOReal) latter.size();
+        outFlow.vLKW /= (double) latter.size();
+        outFlow.vPKW /= (double) latter.size();
         //
         FlowDef mFlow;
         mFlow.qLKW = inFlow.qLKW - outFlow.qLKW;
         mFlow.qPKW = inFlow.qPKW - outFlow.qPKW;
-        mFlow.vLKW = (inFlow.vLKW + outFlow.vLKW) / (SUMOReal) 2.;
-        mFlow.vPKW = (inFlow.vPKW + outFlow.vPKW) / (SUMOReal) 2.;
+        mFlow.vLKW = (inFlow.vLKW + outFlow.vLKW) / (double) 2.;
+        mFlow.vPKW = (inFlow.vPKW + outFlow.vPKW) / (double) 2.;
         mflows.push_back(mFlow);
     }
     static_cast<RODFEdge*>(getDetectorEdge(*detector))->setFlows(mflows);
@@ -632,7 +623,7 @@ RODFNet::getDetectorEdge(const RODFDetector& det) const {
     std::string edgeName = det.getLaneID();
     edgeName = edgeName.substr(0, edgeName.rfind('_'));
     ROEdge* ret = getEdge(edgeName);
-    if (ret == 0) {
+    if (ret == nullptr) {
         throw ProcessError("Edge '" + edgeName + "' used by detector '" + det.getID() + "' is not known.");
     }
     return ret;
@@ -672,7 +663,7 @@ RODFNet::getDetectorList(ROEdge* edge) const {
 }
 
 
-SUMOReal
+double
 RODFNet::getAbsPos(const RODFDetector& det) const {
     if (det.getPos() >= 0) {
         return det.getPos();
@@ -739,7 +730,7 @@ RODFNet::isSource(const RODFDetector& det, ROEdge* edge,
     if (edge != getDetectorEdge(det)) {
         // ok, we are at one of the edges in front
         if (myAmInHighwayMode) {
-            if (edge->getSpeed() >= 19.4) {
+            if (edge->getSpeedLimit() >= 19.4) {
                 if (hasDetector(edge)) {
                     // we are still on the highway and there is another detector
                     return false;
@@ -747,17 +738,17 @@ RODFNet::isSource(const RODFDetector& det, ROEdge* edge,
                 // the next is a hack for the A100 scenario...
                 //  We have to look into further edges herein edges
                 const ROEdgeVector& appr = myApproachingEdges.find(edge)->second;
-                size_t noOk = 0;
-                size_t noFalse = 0;
-                size_t noSkipped = 0;
-                for (size_t i = 0; i < appr.size(); i++) {
+                int noOk = 0;
+                int noFalse = 0;
+                int noSkipped = 0;
+                for (int i = 0; i < (int)appr.size(); i++) {
                     if (!hasDetector(appr[i])) {
                         noOk++;
                     } else {
                         noFalse++;
                     }
                 }
-                if ((noFalse + noSkipped) == appr.size()) {
+                if (noFalse + noSkipped == (int)appr.size()) {
                     return false;
                 }
             }
@@ -765,7 +756,7 @@ RODFNet::isSource(const RODFDetector& det, ROEdge* edge,
     }
 
     if (myAmInHighwayMode) {
-        if (edge->getSpeed() < 19.4 && edge != getDetectorEdge(det)) {
+        if (edge->getSpeedLimit() < 19.4 && edge != getDetectorEdge(det)) {
             // we have left the highway already
             //  -> the detector will be a highway source
             if (!hasDetector(edge)) {
@@ -781,27 +772,26 @@ RODFNet::isSource(const RODFDetector& det, ROEdge* edge,
 
     // let's check the edges in front
     const ROEdgeVector& appr = myApproachingEdges.find(edge)->second;
-    size_t noOk = 0;
-    size_t noFalse = 0;
-    size_t noSkipped = 0;
+    int numOk = 0;
+    int numFalse = 0;
+    int numSkipped = 0;
     seen.push_back(edge);
-    for (size_t i = 0; i < appr.size(); i++) {
+    for (int i = 0; i < (int)appr.size(); i++) {
         bool had = std::find(seen.begin(), seen.end(), appr[i]) != seen.end();
         if (!had) {
             if (isSource(det, appr[i], seen, detectors, strict)) {
-                noOk++;
+                numOk++;
             } else {
-                noFalse++;
+                numFalse++;
             }
         } else {
-            noSkipped++;
+            numSkipped++;
         }
     }
-    if (!strict) {
-        return (noFalse + noSkipped) != appr.size();
-    } else {
-        return (noOk + noSkipped) == appr.size();
+    if (strict) {
+        return numOk + numSkipped == (int)appr.size();
     }
+    return numFalse + numSkipped != (int)appr.size();
 }
 
 
@@ -839,7 +829,7 @@ RODFNet::isDestination(const RODFDetector& det, ROEdge* edge, ROEdgeVector& seen
     if (edge != getDetectorEdge(det)) {
         // ok, we are at one of the edges coming behind
         if (myAmInHighwayMode) {
-            if (edge->getSpeed() >= 19.4) {
+            if (edge->getSpeedLimit() >= 19.4) {
                 if (hasDetector(edge)) {
                     // we are still on the highway and there is another detector
                     return false;
@@ -849,7 +839,7 @@ RODFNet::isDestination(const RODFDetector& det, ROEdge* edge, ROEdgeVector& seen
     }
 
     if (myAmInHighwayMode) {
-        if (edge->getSpeed() < 19.4 && edge != getDetectorEdge(det)) {
+        if (edge->getSpeedLimit() < 19.4 && edge != getDetectorEdge(det)) {
             if (hasDetector(edge)) {
                 return true;
             }
@@ -867,9 +857,9 @@ RODFNet::isDestination(const RODFDetector& det, ROEdge* edge, ROEdgeVector& seen
     }
     const ROEdgeVector& appr  = myApproachedEdges.find(edge)->second;
     bool isall = true;
-    size_t no = 0;
+    int no = 0;
     seen.push_back(edge);
-    for (size_t i = 0; i < appr.size() && isall; i++) {
+    for (int i = 0; i < (int)appr.size() && isall; i++) {
         bool had = std::find(seen.begin(), seen.end(), appr[i]) != seen.end();
         if (!had) {
             if (!isDestination(det, appr[i], seen, detectors)) {
@@ -905,7 +895,7 @@ RODFNet::isFalseSource(const RODFDetector& det, ROEdge* edge, ROEdgeVector& seen
                 }
             }
         } else {
-            if (myAmInHighwayMode && edge->getSpeed() < 19.) {
+            if (myAmInHighwayMode && edge->getSpeedLimit() < 19.) {
                 return false;
             }
         }
@@ -917,7 +907,7 @@ RODFNet::isFalseSource(const RODFDetector& det, ROEdge* edge, ROEdgeVector& seen
 
     const ROEdgeVector& appr  = myApproachedEdges.find(edge)->second;
     bool isall = false;
-    for (size_t i = 0; i < appr.size() && !isall; i++) {
+    for (int i = 0; i < (int)appr.size() && !isall; i++) {
         //printf("checking %s->\n", appr[i].c_str());
         bool had = std::find(seen.begin(), seen.end(), appr[i]) != seen.end();
         if (!had) {
@@ -936,18 +926,25 @@ RODFNet::buildEdgeFlowMap(const RODFDetectorFlows& flows,
                           SUMOTime startTime, SUMOTime endTime,
                           SUMOTime stepOffset) {
     std::map<ROEdge*, std::vector<std::string>, idComp>::iterator i;
+    double speedFactorSumPKW = 0;
+    double speedFactorSumLKW = 0;
+    double speedFactorCountPKW = 0;
+    double speedFactorCountLKW = 0;
     for (i = myDetectorsOnEdges.begin(); i != myDetectorsOnEdges.end(); ++i) {
         ROEdge* into = (*i).first;
+        const double maxSpeedPKW = into->getVClassMaxSpeed(SVC_PASSENGER);
+        const double maxSpeedLKW = into->getVClassMaxSpeed(SVC_TRUCK);
+
         const std::vector<std::string>& dets = (*i).second;
-        std::map<SUMOReal, std::vector<std::string> > cliques;
-        std::vector<std::string>* maxClique = 0;
+        std::map<double, std::vector<std::string> > cliques;
+        std::vector<std::string>* maxClique = nullptr;
         for (std::vector<std::string>::const_iterator j = dets.begin(); j != dets.end(); ++j) {
             if (!flows.knows(*j)) {
                 continue;
             }
             const RODFDetector& det = detectors.getDetector(*j);
             bool found = false;
-            for (std::map<SUMOReal, std::vector<std::string> >::iterator k = cliques.begin(); !found && k != cliques.end(); ++k) {
+            for (std::map<double, std::vector<std::string> >::iterator k = cliques.begin(); !found && k != cliques.end(); ++k) {
                 if (fabs((*k).first - det.getPos()) < 1) {
                     (*k).second.push_back(*j);
                     if ((*k).second.size() > maxClique->size()) {
@@ -961,7 +958,7 @@ RODFNet::buildEdgeFlowMap(const RODFDetectorFlows& flows,
                 maxClique = &cliques[det.getPos()];
             }
         }
-        if (maxClique == 0) {
+        if (maxClique == nullptr) {
             continue;
         }
         std::vector<FlowDef> mflows; // !!! reserve
@@ -984,22 +981,40 @@ RODFNet::buildEdgeFlowMap(const RODFDetectorFlows& flows,
                 FlowDef& fd = mflows[index];
                 fd.qPKW += srcFD.qPKW;
                 fd.qLKW += srcFD.qLKW;
-                fd.vLKW += (srcFD.vLKW / (SUMOReal) maxClique->size());
-                fd.vPKW += (srcFD.vPKW / (SUMOReal) maxClique->size());
-                fd.fLKW += (srcFD.fLKW / (SUMOReal) maxClique->size());
-                fd.isLKW += (srcFD.isLKW / (SUMOReal) maxClique->size());
-                if (!didWarn && srcFD.vPKW > 0 && srcFD.vPKW < 255 && srcFD.vPKW / 3.6 > into->getSpeed()) {
-                    WRITE_MESSAGE("Detected PKW speed higher than allowed speed at '" + (*l) + "' on edge '" + into->getID() + "'.");
+                fd.vLKW += srcFD.vLKW / (double) maxClique->size();
+                fd.vPKW += srcFD.vPKW / (double) maxClique->size();
+                fd.fLKW += srcFD.fLKW / (double) maxClique->size();
+                fd.isLKW += srcFD.isLKW / (double) maxClique->size();
+                const double speedFactorPKW = srcFD.vPKW / 3.6 / maxSpeedPKW;
+                const double speedFactorLKW = srcFD.vLKW / 3.6 / maxSpeedLKW;
+                myMaxSpeedFactorPKW = MAX2(myMaxSpeedFactorPKW, speedFactorPKW);
+                myMaxSpeedFactorLKW = MAX2(myMaxSpeedFactorLKW, speedFactorLKW);
+                speedFactorCountPKW += srcFD.qPKW;
+                speedFactorCountLKW += srcFD.qLKW;
+                speedFactorSumPKW += srcFD.qPKW * speedFactorPKW;
+                speedFactorSumLKW += srcFD.qLKW * speedFactorLKW;
+                if (!didWarn && srcFD.vPKW > 0 && srcFD.vPKW < 255 && srcFD.vPKW / 3.6 > into->getSpeedLimit()) {
+                    WRITE_MESSAGE("Detected PKW speed (" + toString(srcFD.vPKW / 3.6, 3) + ") higher than allowed speed (" + toString(into->getSpeedLimit(), 3) + ") at '" + (*l) + "' on edge '" + into->getID() + "'.");
                     didWarn = true;
                 }
-                if (!didWarn && srcFD.vLKW > 0 && srcFD.vLKW < 255 && srcFD.vLKW / 3.6 > into->getSpeed()) {
-                    WRITE_MESSAGE("Detected LKW speed higher than allowed speed at '" + (*l) + "' on edge '" + into->getID() + "'.");
+                if (!didWarn && srcFD.vLKW > 0 && srcFD.vLKW < 255 && srcFD.vLKW / 3.6 > into->getSpeedLimit()) {
+                    WRITE_MESSAGE("Detected LKW speed (" + toString(srcFD.vLKW / 3.6, 3) + ") higher than allowed speed (" + toString(into->getSpeedLimit(), 3) + ") at '" + (*l) + "' on edge '" + into->getID() + "'.");
                     didWarn = true;
                 }
             }
         }
         static_cast<RODFEdge*>(into)->setFlows(mflows);
     }
+    // @note: this assumes that the speedFactors are independent of location and time
+    if (speedFactorCountPKW > 0) {
+        myAvgSpeedFactorPKW = speedFactorSumPKW / speedFactorCountPKW;
+        WRITE_MESSAGE("Average speedFactor for PKW is " + toString(myAvgSpeedFactorPKW) + " maximum speedFactor is " + toString(myMaxSpeedFactorPKW) + ".");
+    }
+    if (speedFactorCountLKW > 0) {
+        myAvgSpeedFactorLKW = speedFactorSumLKW / speedFactorCountLKW;
+        WRITE_MESSAGE("Average speedFactor for LKW is " + toString(myAvgSpeedFactorLKW) + " maximum speedFactor is " + toString(myMaxSpeedFactorLKW) + ".");
+    }
+
 }
 
 
@@ -1056,12 +1071,12 @@ RODFNet::mesoJoin(RODFDetectorCon& detectors, RODFDetectorFlows& flows) {
     std::map<ROEdge*, std::vector<std::string>, idComp>::iterator i;
     for (i = myDetectorsOnEdges.begin(); i != myDetectorsOnEdges.end(); ++i) {
         const std::vector<std::string>& dets = (*i).second;
-        std::map<SUMOReal, std::vector<std::string> > cliques;
+        std::map<double, std::vector<std::string> > cliques;
         // compute detector cliques
         for (std::vector<std::string>::const_iterator j = dets.begin(); j != dets.end(); ++j) {
             const RODFDetector& det = detectors.getDetector(*j);
             bool found = false;
-            for (std::map<SUMOReal, std::vector<std::string> >::iterator k = cliques.begin(); !found && k != cliques.end(); ++k) {
+            for (std::map<double, std::vector<std::string> >::iterator k = cliques.begin(); !found && k != cliques.end(); ++k) {
                 if (fabs((*k).first - det.getPos()) < 10.) {
                     (*k).second.push_back(*j);
                     found = true;
@@ -1073,7 +1088,7 @@ RODFNet::mesoJoin(RODFDetectorCon& detectors, RODFDetectorFlows& flows) {
             }
         }
         // join detector cliques
-        for (std::map<SUMOReal, std::vector<std::string> >::iterator m = cliques.begin(); m != cliques.end(); ++m) {
+        for (std::map<double, std::vector<std::string> >::iterator m = cliques.begin(); m != cliques.end(); ++m) {
             std::vector<std::string> clique = (*m).second;
             // do not join if only one
             if (clique.size() == 1) {
@@ -1095,6 +1110,4 @@ RODFNet::mesoJoin(RODFDetectorCon& detectors, RODFDetectorFlows& flows) {
 }
 
 
-
 /****************************************************************************/
-
