@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    dfrouter_main.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Eric Nicolay
@@ -7,55 +19,35 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Thu, 16.03.2006
-/// @version $Id: dfrouter_main.cpp 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Main for the DFROUTER
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #ifdef HAVE_VERSION_H
 #include <version.h>
 #endif
 
-#include <xercesc/sax/SAXException.hpp>
-#include <xercesc/sax/SAXParseException.hpp>
-#include <utils/common/TplConvert.h>
 #include <iostream>
 #include <string>
 #include <limits.h>
 #include <ctime>
-#include <router/ROLoader.h>
-#include <router/RONet.h>
-#include "RODFEdgeBuilder.h"
-#include <router/ROFrame.h>
+#include <xercesc/sax/SAXException.hpp>
+#include <xercesc/sax/SAXParseException.hpp>
+#include <utils/common/StringUtils.h>
 #include <utils/common/MsgHandler.h>
-#include <utils/options/Option.h>
-#include <utils/options/OptionsCont.h>
-#include <utils/options/OptionsIO.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/SystemFrame.h>
 #include <utils/common/ToString.h>
+#include <utils/common/FileHelpers.h>
+#include <utils/options/Option.h>
+#include <utils/options/OptionsCont.h>
+#include <utils/options/OptionsIO.h>
+#include <utils/iodevices/OutputDevice.h>
 #include <utils/xml/XMLSubSys.h>
+#include <router/ROLoader.h>
+#include <router/RONet.h>
+#include "RODFEdgeBuilder.h"
 #include "RODFFrame.h"
 #include "RODFNet.h"
 #include "RODFEdge.h"
@@ -64,12 +56,6 @@
 #include "RODFRouteCont.h"
 #include "RODFDetectorFlow.h"
 #include "RODFDetFlowLoader.h"
-#include <utils/common/FileHelpers.h>
-#include <utils/iodevices/OutputDevice.h>
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
 
 
 // ===========================================================================
@@ -133,7 +119,7 @@ startComputation(RODFNet* optNet, RODFDetectorFlows& flows, RODFDetectorCon& det
     }
 
     // if a network was loaded... (mode1)
-    if (optNet != 0) {
+    if (optNet != nullptr) {
         if (oc.getBool("remove-empty-detectors")) {
             PROGRESS_BEGIN_MESSAGE("Removing empty detectors");
             optNet->removeEmptyDetectors(detectors, flows);
@@ -264,21 +250,23 @@ main(int argc, char** argv) {
     OptionsCont& oc = OptionsCont::getOptions();
     // give some application descriptions
     oc.setApplicationDescription("Builds vehicle routes for SUMO using detector values.");
-    oc.setApplicationName("dfrouter", "SUMO dfrouter Version " + getBuildName(VERSION_STRING));
+    oc.setApplicationName("dfrouter", "Eclipse SUMO dfrouter Version " VERSION_STRING);
     int ret = 0;
-    RODFNet* net = 0;
-    RODFDetectorCon* detectors = 0;
-    RODFDetectorFlows* flows = 0;
+    RODFNet* net = nullptr;
+    RODFDetectorCon* detectors = nullptr;
+    RODFDetectorFlows* flows = nullptr;
     try {
         // initialise the application system (messaging, xml, options)
         XMLSubSys::init();
         RODFFrame::fillOptions();
-        OptionsIO::getOptions(true, argc, argv);
+        OptionsIO::setArgs(argc, argv);
+        OptionsIO::getOptions();
         if (oc.processMetaOptions(argc < 2)) {
             SystemFrame::close();
             return 0;
         }
-        XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"));
+        SystemFrame::checkOptions();
+        XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"), "never");
         MsgHandler::initOutputOptions();
         if (!RODFFrame::checkOptions()) {
             throw ProcessError();
@@ -328,6 +316,4 @@ main(int argc, char** argv) {
 }
 
 
-
 /****************************************************************************/
-
