@@ -1,3 +1,27 @@
+/****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2016-2021 German Aerospace Center (DLR) and others.
+// PHEMlight module
+// Copyright (C) 2016-2017 Technische Universitaet Graz, https://www.tugraz.at/
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
+/// @file    CEP.cpp
+/// @author  Martin Dippold
+/// @author  Michael Behrisch
+/// @date    July 2016
+///
+//
+/****************************************************************************/
+
+
 #include "CEP.h"
 #include "Constants.h"
 #include "Helpers.h"
@@ -6,6 +30,7 @@
 namespace PHEMlightdll {
 
     CEP::CEP(bool heavyVehicle, double vehicleMass, double vehicleLoading, double vehicleMassRot, double crossArea, double cWValue, double f0, double f1, double f2, double f3, double f4, double axleRatio, std::vector<double>& transmissionGearRatios, double auxPower, double ratedPower, double engineIdlingSpeed, double engineRatedSpeed, double effictiveWheelDiameter, double pNormV0, double pNormP0, double pNormV1, double pNormP1, const std::string& vehicelFuelType, std::vector<std::vector<double> >& matrixFC, std::vector<std::string>& headerLinePollutants, std::vector<std::vector<double> >& matrixPollutants, std::vector<std::vector<double> >& matrixSpeedRotational, std::vector<std::vector<double> >& normedDragTable, double idlingFC, std::vector<double>& idlingPollutants) {
+        (void)transmissionGearRatios; // just to make the compiler happy about the unused parameter
         InitializeInstanceFields();
         _resistanceF0 = f0;
         _resistanceF1 = f1;
@@ -113,7 +138,7 @@ namespace PHEMlightdll {
         int headerCount = (int)headerLinePollutants.size();
         for (int i = 0; i < (int)matrixPollutants.size(); i++) {
             for (int j = 0; j < (int)matrixPollutants[i].size(); j++) {
-                if (matrixPollutants[i].size() != headerCount + 1) {
+                if ((int)matrixPollutants[i].size() != headerCount + 1) {
                     return;
                 }
 
@@ -173,21 +198,21 @@ namespace PHEMlightdll {
     }
 
     double CEP::CalcPower(double speed, double acc, double gradient) {
-        //Declaration
+        // Declaration
         double power = 0;
         double rotFactor = GetRotationalCoeffecient(speed);
         double powerAux = (_auxPower * _ratedPower);
 
-        //Calculate the power
+        // Calculate the power
         power += (_massVehicle + _vehicleLoading) * Constants::GRAVITY_CONST * (_resistanceF0 + _resistanceF1 * speed + _resistanceF4 * std::pow(speed, 4)) * speed;
         power += (_crossSectionalArea * _cWValue * Constants::AIR_DENSITY_CONST / 2) * std::pow(speed, 3);
         power += (_massVehicle * rotFactor + _vehicleMassRot + _vehicleLoading) * acc * speed;
         power += (_massVehicle + _vehicleLoading) * Constants::GRAVITY_CONST * gradient * 0.01 * speed;
         power /= 1000;
-        power /= Constants::_DRIVE_TRAIN_EFFICIENCY;
+        power /= Constants::getDRIVE_TRAIN_EFFICIENCY();
         power += powerAux;
 
-        //Return result
+        // Return result
         return power;
     }
 
@@ -285,6 +310,11 @@ namespace PHEMlightdll {
                 fCBr = 0.693;
                 fCHC = 0.803;
         }
+//ORIGINAL LINE: case Constants.strLPG:
+        else if (_fuelType == Constants::strLPG) {
+                fCBr = 0.825;
+                fCHC = 0.825;
+        }
         else {
                 VehicleClass->setErrMsg(std::string("The propolsion type is not known! (") + _fuelType + std::string(")"));
                 return 0;
@@ -387,6 +417,26 @@ namespace PHEMlightdll {
         return e1 + (px - p1) / (p2 - p1) * (e2 - e1);
     }
 
+    double CEP::GetMaxAccel(double speed, double gradient) {
+        double rotFactor = GetRotationalCoeffecient(speed);
+        double pMaxForAcc = GetPMaxNorm(speed) * _ratedPower - CalcPower(speed, 0, gradient);
+
+        return (pMaxForAcc * 1000) / ((_massVehicle * rotFactor + _vehicleMassRot + _vehicleLoading) * speed);
+    }
+
+    double CEP::GetPMaxNorm(double speed) {
+        // Linear function between v0 and v1, constant elsewhere
+        if (speed <= _pNormV0) {
+            return _pNormP0;
+        }
+        else if (speed >= _pNormV1) {
+            return _pNormP1;
+        }
+        else {
+            return Interpolate(speed, _pNormV0, _pNormV1, _pNormP0, _pNormP1);
+        }
+    }
+
     void CEP::InitializeInstanceFields() {
         _heavyVehicle = false;
         _normalizingType = static_cast<NormalizingType>(0);
