@@ -1,51 +1,41 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIApplicationWindow.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Christian Roessel
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIApplicationWindow.h 21018 2016-06-21 11:53:01Z behrisch $
 ///
 // The main window of the SUMO-gui.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIApplicationWindow_h
-#define GUIApplicationWindow_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
 #include <iostream>
-#include <fx.h>
-#include <utils/foxtools/MFXEventQue.h>
+#include <utils/foxtools/fxheader.h>
+#include <utils/foxtools/FXSynchQue.h>
 #include <utils/foxtools/FXThreadEvent.h>
 #include <utils/foxtools/MFXInterThreadEventClient.h>
-#include <utils/foxtools/FXRealSpinDial.h>
 #include <utils/foxtools/FXLCDLabel.h>
+#include <utils/foxtools/FXRecentNetworks.h>
 #include <utils/gui/windows/GUIMainWindow.h>
 #include <utils/common/ValueRetriever.h>
 #include <utils/common/ValueSource.h>
-#include <utils/common/RandomDistributor.h>
+#include <utils/distribution/RandomDistributor.h>
 #include "GUISUMOViewParent.h"
 
 
@@ -105,10 +95,11 @@ public:
 
     void setStatusBarText(const std::string& text);
 
-    void addRecentFile(const FX::FXString& f, const bool isNet);
+    void addRecentFile(const FX::FXString& f);
 
     FXGLCanvas* getBuildGLCanvas() const;
     SUMOTime getCurrentSimTime() const;
+    double getTrackerInterval() const;
 
     FXCursor* getDefaultCursor();
 
@@ -118,12 +109,11 @@ public:
     /// @name Inter-thread event handling
     /// @{
 
-    virtual void eventOccured();
+    virtual void eventOccurred();
     void handleEvent_SimulationLoaded(GUIEvent* e);
     void handleEvent_SimulationStep(GUIEvent* e);
     void handleEvent_Message(GUIEvent* e);
     void handleEvent_SimulationEnded(GUIEvent* e);
-    void handleEvent_Screenshot(GUIEvent* e);
     /// @}
 
 
@@ -131,6 +121,9 @@ public:
     /// @name FOX-callbacks
     /// @{
 
+    /// @brief Called on menu File->New Window
+    long onCmdNewWindow(FXObject*, FXSelector, void*);
+
     /// @brief Called on menu File->Open Configuration
     long onCmdOpenConfiguration(FXObject*, FXSelector, void*);
 
@@ -140,12 +133,18 @@ public:
     /// @brief Called on menu File->Load Shapes
     long onCmdOpenShapes(FXObject*, FXSelector, void*);
 
+    /// @brief Called on menu File->Load EdgeData
+    long onCmdOpenEdgeData(FXObject*, FXSelector, void*);
+
     /// @brief Called on reload
     long onCmdReload(FXObject*, FXSelector, void*);
 
     /// @brief Called on opening a recent file
     long onCmdOpenRecent(FXObject*, FXSelector, void*);
 
+    /// @brief Called on menu File->Close
+    long onCmdSaveConfig(FXObject*, FXSelector, void*);
+
     /// @brief Called on menu File->Close
     long onCmdClose(FXObject*, FXSelector, void*);
 
@@ -160,6 +159,12 @@ public:
     /// @brief Called on menu Edit->Edit Breakpoints
     long onCmdEditBreakpoints(FXObject*, FXSelector, void*);
 
+    /// @brief Called on menu Edit->Visualization
+    long onCmdEditViewScheme(FXObject*, FXSelector, void*);
+
+    /// @brief Called on menu Edit->Viewport
+    long onCmdEditViewport(FXObject*, FXSelector, void*);
+
     /// @brief called if the user selects help->Documentation
     long onCmdHelp(FXObject* sender, FXSelector sel, void* ptr);
 
@@ -178,9 +183,18 @@ public:
     /// @brief Toggle listing of internal structures
     long onCmdListInternal(FXObject*, FXSelector, void*);
 
+    /// @brief Toggle listing of parking vehicles
+    long onCmdListParking(FXObject*, FXSelector, void*);
+
+    /// @brief Toggle listing of teleporting vehicles
+    long onCmdListTeleporting(FXObject*, FXSelector, void*);
+
     /// @brief Shows the about dialog
     long onCmdAbout(FXObject*, FXSelector, void*);
 
+    /// @brief Shows the Hall of Fame dialog
+    long onCmdHallOfFame(FXObject*, FXSelector, void*);
+
     /// @brief Called on "play"
     long onCmdStart(FXObject*, FXSelector, void*);
 
@@ -190,12 +204,18 @@ public:
     /// @brief Called on "step"
     long onCmdStep(FXObject*, FXSelector, void*);
 
+    /// @brief Called on "save state"
+    long onCmdSaveState(FXObject*, FXSelector, void*);
+
     /// @brief Called on "time toggle"
     long onCmdTimeToggle(FXObject*, FXSelector, void*);
 
     /// @brief Called on "delay toggle"
     long onCmdDelayToggle(FXObject*, FXSelector, void*);
 
+    /// @brief Called on "demand scale"
+    long onCmdDemandScale(FXObject*, FXSelector, void*);
+
     /// @brief Called if a new view shall be opened (2D view)
     long onCmdNewView(FXObject*, FXSelector, void*);
 
@@ -228,6 +248,9 @@ public:
     /// @brief Determines whether some buttons which require an active simulation may be shown
     long onUpdNeedsSimulation(FXObject*, FXSelector, void*);
 
+    /// @brief Determines whether traci is active
+    long onUpdTraCIStatus(FXObject*, FXSelector, void*);
+
     /// @brief Called if the message window shall be cleared
     long onCmdClearMsgWindow(FXObject*, FXSelector, void*);
 
@@ -255,27 +278,34 @@ public:
     /** @brief Returns the simulation delay
      * @return delay in milliseconds
      */
-    virtual SUMOReal getDelay() const {
-        return mySimDelayTarget->getValue();
+    virtual double getDelay() const {
+        return mySimDelay;
     }
 
     /** @brief Sets the delay of the parent application
+     * @param delay the new delay in milliseconds
      */
-    virtual void setDelay(SUMOReal delay) {
-        mySimDelayTarget->setValue(delay);
+    virtual void setDelay(double delay) {
+        mySimDelay = delay;
     }
 
+    /** @brief Sets the breakpoints of the parent application
+     */
+    virtual void setBreakpoints(const std::vector<SUMOTime>& breakpoints);
+
     /** @brief Sends an event from the application thread to the GUI and waits until it is handled
      * @param event the event to send
      */
     virtual void sendBlockingEvent(GUIEvent* event);
 
+    const std::vector<SUMOTime> retrieveBreakpoints() const;
+
 protected:
     virtual void addToWindowsMenu(FXMenuPane*) { }
 
 private:
     /** starts to load a simulation */
-    void loadConfigOrNet(const std::string& file, bool isNet);
+    void loadConfigOrNet(const std::string& file);
 
     /** this method closes all windows and deletes the current simulation */
     void closeAllWindows();
@@ -288,10 +318,10 @@ private:
 
     /// @brief handles additional game-related events
     void checkGamingEvents();
+    void checkGamingEventsDRT();
 
 protected:
-    /// FOX needs this for static members
-    GUIApplicationWindow() { }
+    FOX_CONSTRUCTOR(GUIApplicationWindow)
 
 protected:
     /// Builds the menu bar
@@ -301,76 +331,83 @@ protected:
     virtual void buildToolBars();
 
 protected:
-    /** the name of the simulation */
+    /// @brief  the name of the simulation
     std::string myName;
 
-    /** the thread that loads simulations */
+    /// @brief  the thread that loads simulations
     GUILoadThread* myLoadThread;
 
-    /** the thread that runs simulations */
+    /// @brief  the thread that runs simulations
     GUIRunThread* myRunThread;
 
-    /** the information whether the simulation was started before */
-    bool myWasStarted;
+    /// @brief  the information whether the simulation was started before
+    bool myWasStarted = false;
 
-    /// The current view number
-    size_t myViewNumber;
+    /// @brief The current view number
+    int myViewNumber;
 
-    /// information whether the gui is currently loading and the load-options shall be greyed out
+    /// @brief information whether the gui is currently loading and the load-options shall be greyed out
     bool myAmLoading;
 
-    /// the submenus
-    FXMenuPane* myFileMenu, *myEditMenu, *mySelectByPermissions, *mySettingsMenu,
-                *myLocatorMenu, *myControlMenu,
-                *myWindowsMenu, *myHelpMenu;
+    /// @brief the submenus
+    FXMenuPane* myFileMenu = nullptr, *myEditMenu = nullptr, *mySelectByPermissions = nullptr, *mySettingsMenu = nullptr,
+                *myLocatorMenu, *myControlMenu = nullptr,
+                                 *myWindowsMenu, *myHelpMenu = nullptr;
 
-    /// Buttons showing and running values and triggering statistic windows
+    /// @brief the menu cascades
+    FXMenuCascade* mySelectLanesMenuCascade = nullptr;
+
+    /// @brief Buttons showing and running values and triggering statistic windows
     std::vector<FXButton*> myStatButtons;
 
-    /// A window to display messages, warnings and error in
-    GUIMessageWindow* myMessageWindow;
+    /// @brief A window to display messages, warnings and error in
+    GUIMessageWindow* myMessageWindow = nullptr;
 
-    /// The splitter that divides the main window into vies and the log window
-    FXSplitter* myMainSplitter;
+    /// @brief The splitter that divides the main window into views and the log window
+    FXSplitter* myMainSplitter = nullptr;
 
-    /// for some menu detaching fun
-    FXToolBarShell* myToolBarDrag1, *myToolBarDrag2, *myToolBarDrag3,
-                    *myToolBarDrag4, *myToolBarDrag5, *myMenuBarDrag;
+    /// @brief for some menu detaching fun
+    FXToolBarShell* myToolBarDrag1 = nullptr, *myToolBarDrag2 = nullptr, *myToolBarDrag3 = nullptr,
+                    *myToolBarDrag4 = nullptr, *myToolBarDrag5 = nullptr, *myMenuBarDrag = nullptr,
+                     *myToolBarDrag8 = nullptr;
 
-    ///
-    FXRealSpinDial* mySimDelayTarget;
+    /// @brief the simulation delay in milliseconds
+    double mySimDelay = 0.;
+    FXDataTarget* mySimDelayTarget = nullptr;
+    FXRealSpinner* mySimDelaySpinner = nullptr;
+    FXSlider* mySimDelaySlider = nullptr;
 
-    /// The alternate simulation delay for toggling
-    SUMOTime myAlternateSimDelay;
+    /// @brief the demand scale
+    FXRealSpinner* myDemandScaleSpinner = nullptr;
 
-    /// List of got requests
-    MFXEventQue<GUIEvent*> myEvents;
+    /// @brief The alternate simulation delay in milliseconds for toggling
+    double myAlternateSimDelay;
 
-    /// The menu used for the MDI-windows
-    FXMDIMenu* myMDIMenu;
+    /// @brief List of got requests
+    FXSynchQue<GUIEvent*> myEvents;
 
-    /// The application menu bar
-    FXMenuBar* myMenuBar;
+    /// @brief The menu used for the MDI-windows
+    FXMDIMenu* myMDIMenu = nullptr;
 
-    /// The application tool bar
-    FXToolBar* myToolBar1, *myToolBar2, *myToolBar3, *myToolBar4, *myToolBar5;
+    /// @brief The application menu bar
+    FXMenuBar* myMenuBar = nullptr;
 
-    /// the simulation step display
-    FXEX::FXLCDLabel* myLCDLabel;
+    /// @brief The application tool bar
+    FXToolBar* myToolBar1 = nullptr, *myToolBar2 = nullptr, *myToolBar3 = nullptr, *myToolBar4 = nullptr, *myToolBar5 = nullptr, *myToolBar8 = nullptr;
 
-    /// io-event with the load-thread
+    /// @brief the simulation step display
+    FXEX::FXLCDLabel* myLCDLabel = nullptr;
+
+    /// @brief io-event with the load-thread
     FXEX::FXThreadEvent myLoadThreadEvent;
 
-    /// io-event with the run-thread
+    /// @brief io-event with the run-thread
     FXEX::FXThreadEvent myRunThreadEvent;
 
-    /// List of recent config files
-    FXRecentFiles myRecentConfigs;
+    /// @brief List of recent networks and configs
+    FXRecentNetworks myRecentNetworksAndConfigs;
 
-    /// List of recent nets
-    FXRecentFiles myRecentNets;
-
-    /// Input file pattern
+    /// @brief Input file pattern
     std::string myConfigPattern;
 
     bool hadDependentBuild;
@@ -378,9 +415,6 @@ protected:
     /// @brief whether to show time as hour:minute:second
     bool myShowTimeAsHMS;
 
-    /// @brief whether to show the window in full screen mode
-    bool myAmFullScreen;
-
     /// @brief whether the simulation end was already announced
     bool myHaveNotifiedAboutSimEnd;
 
@@ -390,29 +424,32 @@ protected:
     /// @brief the semaphore when waiting for event completion
     FXCondition myEventCondition;
 
+    /// @brief menu checkbox to activate game mode
+    FXMenuCheck* myGamingModeCheckbox;
+
     /// @name game related things
     /// {
     RandomDistributor<std::string> myJamSounds;
     RandomDistributor<std::string> myCollisionSounds;
     /// @brief waiting time after which vehicles trigger jam sounds
-    SUMOReal myJamSoundTime;
+    double myJamSoundTime;
     /// @brief A random number generator used to choose a gaming sound
-    static MTRand myGamingRNG;
-    unsigned int myPreviousCollisionNumber;
+    static std::mt19937 myGamingRNG;
+    int myPreviousCollisionNumber;
+    /// @brief current game mode
+    bool myTLSGame;
 
-    /// performance indicators
-    FXEX::FXLCDLabel* myWaitingTimeLabel;
-    FXEX::FXLCDLabel* myTimeLossLabel;
+    /// @brief performance indicators
+    FXEX::FXLCDLabel* myWaitingTimeLabel = nullptr;
+    FXEX::FXLCDLabel* myTimeLossLabel = nullptr;
+    FXEX::FXLCDLabel* myTotalDistanceLabel = nullptr;
+    FXEX::FXLCDLabel* myEmergencyVehicleLabel = nullptr;
     SUMOTime myWaitingTime;
     SUMOTime myTimeLoss;
-    FXToolBar* myToolBar6, *myToolBar7;
-    FXToolBarShell* myToolBarDrag6, *myToolBarDrag7;
+    SUMOTime myEmergencyVehicleCount;
+    double myTotalDistance;
+    FXToolBar* myToolBar6 = nullptr, *myToolBar7 = nullptr, *myToolBar9 = nullptr, *myToolBar10 = nullptr;
+    FXToolBarShell* myToolBarDrag6 = nullptr, *myToolBarDrag7 = nullptr, *myToolBarDrag9 = nullptr, *myToolBarDrag10 = nullptr;
     ////}
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
