@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIRunThread.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIRunThread.cpp 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // The thread that runs the simulation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cassert>
 #include <string>
@@ -44,33 +35,28 @@
 #include "GUIGlobals.h"
 #include <microsim/MSVehicleControl.h>
 #include <utils/options/OptionsCont.h>
+#include <utils/options/OptionsIO.h>
 #include <utils/common/SysUtils.h>
 #include <utils/common/MsgRetrievingFunction.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/iodevices/OutputDevice.h>
-
-#ifndef NO_TRACI
 #include <traci-server/TraCIServer.h>
-#endif
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
+#include <libsumo/Simulation.h>
 
 
 // ===========================================================================
 // member method definitions
 // ===========================================================================
 GUIRunThread::GUIRunThread(FXApp* app, MFXInterThreadEventClient* parent,
-                           FXRealSpinDial& simDelay, MFXEventQue<GUIEvent*>& eq,
+                           double& simDelay, FXSynchQue<GUIEvent*>& eq,
                            FXEX::FXThreadEvent& ev)
     : FXSingleEventThread(app, parent),
-      myNet(0), myHalting(true), myQuit(false), mySimulationInProgress(false), myOk(true),
+      myNet(nullptr), myHalting(true), myQuit(false), mySimulationInProgress(false), myOk(true), myHaveSignaledEnd(false),
       mySimDelay(simDelay), myEventQue(eq), myEventThrow(ev) {
-    myErrorRetriever = new MsgRetrievingFunction<GUIRunThread>(this, &GUIRunThread::retrieveMessage, MsgHandler::MT_ERROR);
-    myMessageRetriever = new MsgRetrievingFunction<GUIRunThread>(this, &GUIRunThread::retrieveMessage, MsgHandler::MT_MESSAGE);
-    myWarningRetriever = new MsgRetrievingFunction<GUIRunThread>(this, &GUIRunThread::retrieveMessage, MsgHandler::MT_WARNING);
+    myErrorRetriever = new MsgRetrievingFunction<GUIRunThread>(this, &GUIRunThread::retrieveMessage, MsgHandler::MsgType::MT_ERROR);
+    myMessageRetriever = new MsgRetrievingFunction<GUIRunThread>(this, &GUIRunThread::retrieveMessage, MsgHandler::MsgType::MT_MESSAGE);
+    myWarningRetriever = new MsgRetrievingFunction<GUIRunThread>(this, &GUIRunThread::retrieveMessage, MsgHandler::MsgType::MT_WARNING);
 }
 
 
@@ -82,7 +68,7 @@ GUIRunThread::~GUIRunThread() {
     delete myMessageRetriever;
     delete myWarningRetriever;
     // wait for the thread
-    while (mySimulationInProgress || myNet != 0);
+    while (mySimulationInProgress || myNet != nullptr);
 }
 
 
@@ -90,9 +76,11 @@ bool
 GUIRunThread::init(GUINet* net, SUMOTime start, SUMOTime end) {
     assert(net != 0);
     // assign new values
+    myOk = true;
     myNet = net;
     mySimStartTime = start;
     mySimEndTime = end;
+    myHaveSignaledEnd = false;
     // register message callbacks
     MsgHandler::getErrorInstance()->addRetriever(myErrorRetriever);
     MsgHandler::getMessageInstance()->addRetriever(myMessageRetriever);
@@ -114,6 +102,7 @@ GUIRunThread::init(GUINet* net, SUMOTime start, SUMOTime end) {
         mySimulationInProgress = false;
 #ifndef _DEBUG
     } catch (...) {
+        MsgHandler::getErrorInstance()->inform("Quitting (on error).", false);
         myHalting = true;
         myOk = false;
         mySimulationInProgress = false;
@@ -131,30 +120,27 @@ GUIRunThread::run() {
     // perform an endless loop
     while (!myQuit) {
         // if the simulation shall be perfomed, do it
-        if (!myHalting && myNet != 0 && myOk) {
-            if (getNet().logSimulationDuration()) {
-                beg = SysUtils::getCurrentMillis();
-                if (end != -1) {
-                    getNet().setIdleDuration((int)(beg - end));
-                }
+        if (!myHalting && myNet != nullptr && myOk) {
+            beg = SysUtils::getCurrentMillis();
+            if (end != -1) {
+                getNet().setIdleDuration((int)(beg - end));
             }
             // check whether we shall stop at this step
             myBreakpointLock.lock();
-            const bool haltAfter = find(myBreakpoints.begin(), myBreakpoints.end(), myNet->getCurrentTimeStep()) != myBreakpoints.end();
+            const bool haltAfter = std::find(myBreakpoints.begin(), myBreakpoints.end(), myNet->getCurrentTimeStep()) != myBreakpoints.end();
             myBreakpointLock.unlock();
             // do the step
             makeStep();
+            waitForSnapshots(myNet->getCurrentTimeStep() - DELTA_T);
             // stop if wished
             if (haltAfter) {
                 stop();
             }
-            // wait if wanted
-            long wait = (long) mySimDelay.getValue();
-            if (getNet().logSimulationDuration()) {
-                end = SysUtils::getCurrentMillis();
-                getNet().setSimDuration((int)(end - beg));
-                wait -= (end - beg);
-            }
+            // wait if wanted (delay is per simulated second)
+            long wait = (long)(mySimDelay * TS);
+            end = SysUtils::getCurrentMillis();
+            getNet().setSimDuration((int)(end - beg));
+            wait -= (end - beg);
             if (wait > 0) {
                 sleep(wait);
             }
@@ -171,7 +157,7 @@ GUIRunThread::run() {
 
 void
 GUIRunThread::makeStep() {
-    GUIEvent* e = 0;
+    GUIEvent* e = nullptr;
     // simulation is being perfomed
     mySimulationInProgress = true;
     // execute a single step
@@ -183,32 +169,30 @@ GUIRunThread::makeStep() {
 
         // inform parent that a step has been performed
         e = new GUIEvent_SimulationStep();
-        myEventQue.add(e);
+        myEventQue.push_back(e);
         myEventThrow.signal();
 
-        e = 0;
-        MSNet::SimulationState state = myNet->simulationState(mySimEndTime);
-#ifndef NO_TRACI
-        if (state != MSNet::SIMSTATE_RUNNING) {
-            if (OptionsCont::getOptions().getInt("remote-port") != 0 && !TraCIServer::wasClosed()) {
-                state = MSNet::SIMSTATE_RUNNING;
-            }
-        }
-#endif
+        e = nullptr;
+        MSNet::SimulationState state = myNet->adaptToState(myNet->simulationState(mySimEndTime));
         switch (state) {
+            case MSNet::SIMSTATE_LOADING:
             case MSNet::SIMSTATE_END_STEP_REACHED:
             case MSNet::SIMSTATE_NO_FURTHER_VEHICLES:
             case MSNet::SIMSTATE_CONNECTION_CLOSED:
-            case MSNet::SIMSTATE_TOO_MANY_VEHICLES:
-                WRITE_MESSAGE("Simulation ended at time: " + time2string(myNet->getCurrentTimeStep()));
-                WRITE_MESSAGE("Reason: " + MSNet::getStateMessage(state));
-                e = new GUIEvent_SimulationEnded(state, myNet->getCurrentTimeStep() - DELTA_T);
+            case MSNet::SIMSTATE_TOO_MANY_TELEPORTS:
+                if (!myHaveSignaledEnd || state != MSNet::SIMSTATE_END_STEP_REACHED) {
+                    e = new GUIEvent_SimulationEnded(state, myNet->getCurrentTimeStep() - DELTA_T);
+                    // ensure that files are closed (deleteSim is called a bit later by the gui thread)
+                    // MSNet destructor may trigger MsgHandler (via routing device cleanup). Closing output devices here is not safe
+                    // OutputDevice::closeAll();
+                    myHaveSignaledEnd = true;
+                }
                 break;
             default:
                 break;
         }
-        if (e != 0) {
-            myEventQue.add(e);
+        if (e != nullptr) {
+            myEventQue.push_back(e);
             myEventThrow.signal();
             myHalting = true;
         }
@@ -227,16 +211,17 @@ GUIRunThread::makeStep() {
         mySimulationLock.unlock();
         mySimulationInProgress = false;
         e = new GUIEvent_SimulationEnded(MSNet::SIMSTATE_ERROR_IN_SIM, myNet->getCurrentTimeStep());
-        myEventQue.add(e);
+        myEventQue.push_back(e);
         myEventThrow.signal();
         myHalting = true;
         myOk = false;
 #ifndef _DEBUG
     } catch (...) {
+        MsgHandler::getErrorInstance()->inform("Quitting (on error).", false);
         mySimulationLock.unlock();
         mySimulationInProgress = false;
         e = new GUIEvent_SimulationEnded(MSNet::SIMSTATE_ERROR_IN_SIM, myNet->getCurrentTimeStep());
-        myEventQue.add(e);
+        myEventQue.push_back(e);
         myEventThrow.signal();
         myHalting = true;
         myOk = false;
@@ -276,7 +261,7 @@ GUIRunThread::stop() {
 
 bool
 GUIRunThread::simulationAvailable() const {
-    return myNet != 0;
+    return myNet != nullptr;
 }
 
 
@@ -289,13 +274,13 @@ GUIRunThread::deleteSim() {
     MsgHandler::getMessageInstance()->removeRetriever(myMessageRetriever);
     //
     mySimulationLock.lock();
-    if (myNet != 0) {
-        myNet->closeSimulation(mySimStartTime);
+    if (myNet != nullptr) {
+        myNet->closeSimulation(mySimStartTime, MSNet::getStateMessage(myNet->simulationState(mySimEndTime)));
     }
     while (mySimulationInProgress);
     delete myNet;
     GUIGlObjectStorage::gIDStorage.clear();
-    myNet = 0;
+    myNet = nullptr;
     OutputDevice::closeAll();
     mySimulationLock.unlock();
     MsgHandler::cleanupOnEnd();
@@ -318,29 +303,38 @@ GUIRunThread::prepareDestruction() {
 void
 GUIRunThread::retrieveMessage(const MsgHandler::MsgType type, const std::string& msg) {
     GUIEvent* e = new GUIEvent_Message(type, msg);
-    myEventQue.add(e);
+    myEventQue.push_back(e);
     myEventThrow.signal();
 }
 
 
 bool
 GUIRunThread::simulationIsStartable() const {
-    return myNet != 0 && myHalting;
+    return myNet != nullptr && myHalting;
 }
 
 
 bool
 GUIRunThread::simulationIsStopable() const {
-    return myNet != 0 && (!myHalting);
+    return myNet != nullptr && (!myHalting);
 }
 
 
 bool
 GUIRunThread::simulationIsStepable() const {
-    return myNet != 0 && myHalting;
+    return myNet != nullptr && myHalting;
 }
 
 
+void
+GUIRunThread::waitForSnapshots(const SUMOTime snapshotTime) {
+    GUIMainWindow* const mw = GUIMainWindow::getInstance();
+    if (mw != nullptr) {
+        for (GUIGlChildWindow* const window : mw->getViews()) {
+            window->getView()->waitForSnapshots(snapshotTime);
+        }
+    }
+}
+
 
 /****************************************************************************/
-
