@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUITLLogicPhasesTrackerWindow.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Oct/Nov 2003
-/// @version $Id: GUITLLogicPhasesTrackerWindow.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // A window displaying the phase diagram of a tl-logic
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <vector>
 #include <iostream>
@@ -42,13 +33,11 @@
 #include <guisim/GUITrafficLightLogicWrapper.h>
 #include <utils/gui/windows/GUIAppEnum.h>
 #include <utils/gui/images/GUIIconSubSys.h>
-#include <foreign/polyfonts/polyfonts.h>
+#include <utils/gui/settings/GUIVisualizationSettings.h>
+#include <utils/gui/div/GUIDesigns.h>
+#include <foreign/fontstash/fontstash.h>
 #include <utils/gui/globjects/GLIncludes.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // member method definitions
@@ -73,7 +62,7 @@ FXIMPLEMENT(GUITLLogicPhasesTrackerWindow::GUITLLogicPhasesTrackerPanel, FXGLCan
 GUITLLogicPhasesTrackerWindow::GUITLLogicPhasesTrackerPanel::GUITLLogicPhasesTrackerPanel(
     FXComposite* c, GUIMainWindow& app,
     GUITLLogicPhasesTrackerWindow& parent)
-    : FXGLCanvas(c, app.getGLVisual(), app.getBuildGLCanvas(), (FXObject*) 0, (FXSelector) 0, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y/*, 0, 0, 300, 200*/),
+    : FXGLCanvas(c, app.getGLVisual(), app.getBuildGLCanvas(), (FXObject*) nullptr, (FXSelector) 0, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y/*, 0, 0, 300, 200*/),
       myParent(&parent) {}
 
 
@@ -155,26 +144,26 @@ GUITLLogicPhasesTrackerWindow::GUITLLogicPhasesTrackerWindow(
     GUIMainWindow& app,
     MSTrafficLightLogic& logic, GUITrafficLightLogicWrapper& wrapper,
     ValueSource<std::pair<SUMOTime, MSPhaseDefinition> >* src)
-    : FXMainWindow(app.getApp(), "TLS-Tracker", NULL, NULL, DECOR_ALL,
+    : FXMainWindow(app.getApp(), "TLS-Tracker", nullptr, nullptr, DECOR_ALL,
                    20, 20, 300, 200),
       myApplication(&app), myTLLogic(&logic), myAmInTrackingMode(true) {
     // build the toolbar
-    myToolBarDrag = new FXToolBarShell(this, FRAME_NORMAL);
+    myToolBarDrag = new FXToolBarShell(this, GUIDesignToolBar);
     myToolBar = new FXToolBar(this, myToolBarDrag, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | FRAME_RAISED);
-    new FXToolBarGrip(myToolBar, myToolBar, FXToolBar::ID_TOOLBARGRIP, TOOLBARGRIP_DOUBLE);
+    new FXToolBarGrip(myToolBar, myToolBar, FXToolBar::ID_TOOLBARGRIP, GUIDesignToolBarGrip);
     // interval manipulation
-    myBeginOffset = new FXRealSpinDial(myToolBar, 10, this, MID_SIMSTEP, LAYOUT_TOP | FRAME_SUNKEN | FRAME_THICK);
-    myBeginOffset->setFormatString("%.0f");
-    myBeginOffset->setIncrements(1, 10, 100);
+    myBeginOffset = new FXRealSpinner(myToolBar, 10, this, MID_SIMSTEP, LAYOUT_TOP | FRAME_SUNKEN | FRAME_THICK);
+    //myBeginOffset->setFormatString("%.0f");
+    //myBeginOffset->setIncrements(1, 10, 100);
+    myBeginOffset->setIncrement(10);
     myBeginOffset->setRange(60, 3600);
     myBeginOffset->setValue(240);
-    new FXLabel(myToolBar, "(s)", 0, LAYOUT_CENTER_Y);
+    new FXLabel(myToolBar, "(s)", nullptr, LAYOUT_CENTER_Y);
     //
     myConnector = new GLObjectValuePassConnector<std::pair<SUMOTime, MSPhaseDefinition> >(wrapper, src, this);
-    FXint height = (FXint)(myTLLogic->getLinks().size() * 20 + 30 + 8 + 30);
-    app.addChild(this, true);
-    for (size_t i = 0; i < myTLLogic->getLinks().size(); ++i) {
-        myLinkNames.push_back(toString<size_t>(i));
+    app.addChild(this);
+    for (int i = 0; i < (int)myTLLogic->getLinks().size(); ++i) {
+        myLinkNames.push_back(toString<int>(i));
     }
     FXVerticalFrame* glcanvasFrame =
         new FXVerticalFrame(this,
@@ -183,8 +172,9 @@ GUITLLogicPhasesTrackerWindow::GUITLLogicPhasesTrackerWindow(
     myPanel = new
     GUITLLogicPhasesTrackerPanel(glcanvasFrame, *myApplication, *this);
     setTitle((logic.getID() + " - " + logic.getProgramID() + " - tracker").c_str());
-    setIcon(GUIIconSubSys::getIcon(ICON_APP_TLSTRACKER));
-    setHeight(height);
+    setIcon(GUIIconSubSys::getIcon(GUIIcon::APP_TLSTRACKER));
+    setHeight((FXint)(myTLLogic->getLinks().size() * 20 + 30 + 8 + 30));
+    setWidth(700);
 }
 
 
@@ -192,16 +182,15 @@ GUITLLogicPhasesTrackerWindow::GUITLLogicPhasesTrackerWindow(
     GUIMainWindow& app,
     MSTrafficLightLogic& logic, GUITrafficLightLogicWrapper& /*wrapper*/,
     const MSSimpleTrafficLightLogic::Phases& /*phases*/)
-    : FXMainWindow(app.getApp(), "TLS-Tracker", NULL, NULL, DECOR_ALL,
+    : FXMainWindow(app.getApp(), "TLS-Tracker", nullptr, nullptr, DECOR_ALL,
                    20, 20, 300, 200),
       myApplication(&app), myTLLogic(&logic), myAmInTrackingMode(false),
-      myToolBarDrag(0), myBeginOffset(0) {
-    myConnector = 0;
-    FXint height = (FXint)(myTLLogic->getLinks().size() * 20 + 30 + 8);
+      myToolBarDrag(nullptr), myBeginOffset(nullptr) {
+    myConnector = nullptr;
     setTitle("TLS-Tracker");
-    app.addChild(this, true);
-    for (size_t i = 0; i < myTLLogic->getLinks().size(); ++i) {
-        myLinkNames.push_back(toString<size_t>(i));
+    app.addChild(this);
+    for (int i = 0; i < (int)myTLLogic->getLinks().size(); ++i) {
+        myLinkNames.push_back(toString<int>(i));
     }
     FXVerticalFrame* glcanvasFrame =
         new FXVerticalFrame(this,
@@ -210,8 +199,9 @@ GUITLLogicPhasesTrackerWindow::GUITLLogicPhasesTrackerWindow(
     myPanel = new
     GUITLLogicPhasesTrackerPanel(glcanvasFrame, *myApplication, *this);
     setTitle((logic.getID() + " - " + logic.getProgramID() + " - tracker").c_str());
-    setIcon(GUIIconSubSys::getIcon(ICON_APP_TLSTRACKER));
-    setHeight(height);
+    setIcon(GUIIconSubSys::getIcon(GUIIcon::APP_TLSTRACKER));
+    setHeight((FXint)(myTLLogic->getLinks().size() * 20 + 30 + 8));
+    setWidth(700);
 }
 
 
@@ -229,7 +219,7 @@ GUITLLogicPhasesTrackerWindow::~GUITLLogicPhasesTrackerWindow() {
 void
 GUITLLogicPhasesTrackerWindow::create() {
     FXMainWindow::create();
-    if (myToolBarDrag != 0) {
+    if (myToolBarDrag != nullptr) {
         myToolBarDrag->create();
     }
 }
@@ -240,13 +230,17 @@ GUITLLogicPhasesTrackerWindow::drawValues(GUITLLogicPhasesTrackerPanel& caller)
     // compute what shall be shown (what is visible)
     myFirstPhase2Show = 0;
     myFirstPhaseOffset = 0;
-    size_t leftOffset = 0;
+    SUMOTime leftOffset = 0;
     myFirstTime2Show = 0;
     if (!myAmInTrackingMode) {
         myPhases.clear();
         myDurations.clear();
         // insert phases
-        const MSSimpleTrafficLightLogic::Phases& phases = static_cast<MSSimpleTrafficLightLogic*>(myTLLogic)->getPhases();
+        MSSimpleTrafficLightLogic* simpleTLLogic = dynamic_cast<MSSimpleTrafficLightLogic*>(myTLLogic);
+        if (simpleTLLogic == nullptr) {
+            return;
+        }
+        const MSSimpleTrafficLightLogic::Phases& phases = simpleTLLogic->getPhases();
         MSSimpleTrafficLightLogic::Phases::const_iterator j;
         myLastTime = 0;
         myBeginTime = 0;
@@ -256,7 +250,7 @@ GUITLLogicPhasesTrackerWindow::drawValues(GUITLLogicPhasesTrackerPanel& caller)
             myLastTime += (*j)->duration;
         }
         if (myLastTime <= myBeginTime) {
-            WRITE_ERROR("Overflow in time computation occured.");
+            WRITE_ERROR("Overflow in time computation occurred.");
             return;
         }
     } else {
@@ -266,7 +260,7 @@ GUITLLogicPhasesTrackerWindow::drawValues(GUITLLogicPhasesTrackerPanel& caller)
         // check whether no phases are known at all
         if (myDurations.size() != 0) {
             SUMOTime durs = 0;
-            size_t phaseOffset = myDurations.size() - 1;
+            int phaseOffset = (int)myDurations.size() - 1;
             DurationsVector::reverse_iterator i = myDurations.rbegin();
             while (i != myDurations.rend()) {
                 if (durs + (*i) > beginOffset) {
@@ -297,32 +291,30 @@ GUITLLogicPhasesTrackerWindow::drawValues(GUITLLogicPhasesTrackerPanel& caller)
     // draw the horizontal lines dividing the signal groups
     glColor3d(1, 1, 1);
     // compute some values needed more than once
-    const SUMOReal height = (SUMOReal) caller.getHeight();
-    const SUMOReal width = (SUMOReal) caller.getWidth();
-    pfSetScaleXY((SUMOReal)(.08 * 300. / width), (SUMOReal)(.08 * 300. / height));
-    const SUMOReal h4 = ((SUMOReal) 4 / height);
-    const SUMOReal h9 = ((SUMOReal) 9 / height);
-    const SUMOReal h10 = ((SUMOReal) 10 / height);
-    const SUMOReal h11 = ((SUMOReal) 11 / height);
-    const SUMOReal h16 = ((SUMOReal) 16 / height);
-    const SUMOReal h20 = ((SUMOReal) 20 / height);
+    const double panelHeight = (double) caller.getHeight();
+    const double panelWidth = (double) caller.getWidth();
+    const double barWidth = MAX2(1.0, panelWidth - 31);
+    const double fontHeight = 0.08 * 300. / panelHeight;
+    const double fontWidth = 0.08 * 300. / panelWidth;
+    const double h9 = 9. / panelHeight;
+    const double h10 = 10. / panelHeight;
+    const double h11 = 11. / panelHeight;
+    const double h16 = 16. / panelHeight;
+    const double h20 = 20. / panelHeight;
     // draw the link names and the lines dividing them
-    SUMOReal h = (SUMOReal)(1.0 - h10);
-    SUMOReal h2 = 12;
-    for (size_t i = 0; i < myTLLogic->getLinks().size() + 1; ++i) {
+    double h = 1. - h10;
+    double h2 = 12.;
+    for (int i = 0; i < (int)myTLLogic->getLinks().size() + 1; ++i) {
         // draw the bar
         glBegin(GL_LINES);
         glVertex2d(0, h);
-        glVertex2d((SUMOReal)(30. / width), h);
+        glVertex2d((double)(30. / panelWidth), h);
         glEnd();
         // draw the name
-        if (i < myTLLogic->getLinks().size()) {
-            glRotated(180, 1, 0, 0);
-            pfSetPosition(0, 0);
-            glTranslated(0.0, -h + h20 - h4, 0);
-            pfDrawString(myLinkNames[i].c_str());
-            glTranslated(-0.0, h - h20 + h4, 0);
-            glRotated(-180, 1, 0, 0);
+        if (i < (int)myTLLogic->getLinks().size()) {
+            glTranslated(0, h - h20, 0);
+            GLHelper::drawText(myLinkNames[i], Position(0, 0), 1, fontHeight, RGBColor::WHITE, 0, FONS_ALIGN_LEFT | FONS_ALIGN_BOTTOM, fontWidth);
+            glTranslated(0, -h + h20, 0);
             h2 += 20;
         }
         h -= h20;
@@ -333,11 +325,11 @@ GUITLLogicPhasesTrackerWindow::drawValues(GUITLLogicPhasesTrackerPanel& caller)
     glEnd();
 
     // draw the names closure (vertical line)
-    h += (SUMOReal) 20 / height;
+    h += 20. / panelHeight;
     glColor3d(1, 1, 1);
     glBegin(GL_LINES);
-    glVertex2d((SUMOReal) 30 / width, 1.0);
-    glVertex2d((SUMOReal) 30 / width, h);
+    glVertex2d(30. / panelWidth, 1.);
+    glVertex2d(30. / panelWidth, h);
     glEnd();
 
 
@@ -345,32 +337,33 @@ GUITLLogicPhasesTrackerWindow::drawValues(GUITLLogicPhasesTrackerPanel& caller)
     // disable value addition while drawing
     myLock.lock();
     // determine the initial offset
-    SUMOReal x = ((SUMOReal) 31. / width);
-    SUMOReal ta = (SUMOReal) leftOffset / width;
-    ta *= (SUMOReal)(((width - 31.0) / ((SUMOReal)(myLastTime - myBeginTime))));
+    double x = 31. / panelWidth;
+    double ta = (double) leftOffset / panelWidth;
+    ta *= barWidth / ((double)(myLastTime - myBeginTime));
     x += ta;
 
     // and the initial phase information
     PhasesVector::iterator pi = myPhases.begin() + myFirstPhase2Show;
-    DurationsVector::iterator pd = myDurations.begin() + myFirstPhase2Show;
-    size_t fpo = myFirstPhaseOffset;
+
+    SUMOTime fpo = myFirstPhaseOffset;
 
     // start drawing
-    for (size_t i = 30; pd != myDurations.end();) {
+    for (DurationsVector::iterator pd = myDurations.begin() + myFirstPhase2Show; pd != myDurations.end(); ++pd) {
+        SUMOTime i = 30;
         // the first phase may be drawn incompletely
-        size_t duration = *pd - fpo;
-        // compute the heigh and the width of the phase
-        h = (SUMOReal)(1.0 - h10);
-        SUMOReal a = (SUMOReal) duration / width;
-        a *= (SUMOReal)(((width - 31.0) / ((SUMOReal)(myLastTime - myBeginTime))));
-        SUMOReal x2 = x + a;
+        SUMOTime duration = *pd - fpo;
+        // compute the height and the width of the phase
+        h = 1. - h10;
+        double a = (double) duration / panelWidth;
+        a *= barWidth / ((double)(myLastTime - myBeginTime));
+        const double x2 = x + a;
 
         // go through the links
         for (int j = 0; j < (int) myTLLogic->getLinks().size(); ++j) {
             // determine the current link's color
             LinkState state = pi->getSignalState(j);
             // draw the bar (red is drawn as a line)
-            GLHelper::setColor(getLinkColor(state));
+            GLHelper::setColor(GUIVisualizationSettings::getLinkColor(state));
             switch (state) {
                 case LINKSTATE_TL_RED:
                 case LINKSTATE_TL_REDYELLOW:
@@ -386,8 +379,8 @@ GUITLLogicPhasesTrackerWindow::drawValues(GUITLLogicPhasesTrackerPanel& caller)
                     // draw a thick block
                     glBegin(GL_QUADS);
                     glVertex2d(x, h - h16);
-                    glVertex2d(x, h - h4);
-                    glVertex2d(x2, h - h4);
+                    glVertex2d(x, h);
+                    glVertex2d(x2, h);
                     glVertex2d(x2, h - h16);
                     glEnd();
                     break;
@@ -398,7 +391,6 @@ GUITLLogicPhasesTrackerWindow::drawValues(GUITLLogicPhasesTrackerPanel& caller)
         // proceed to next phase
         i += duration;
         ++pi;
-        ++pd;
         x = x2;
         // all further phases are drawn in full
         fpo = 0;
@@ -410,38 +402,37 @@ GUITLLogicPhasesTrackerWindow::drawValues(GUITLLogicPhasesTrackerPanel& caller)
     if (myPhases.size() != 0) {
         SUMOTime tickDist = TIME2STEPS(10);
         // patch distances - hack
-        SUMOReal t = myBeginOffset != 0 ? (SUMOReal) myBeginOffset->getValue() : STEPS2TIME(myLastTime - myBeginTime);
-        while (t > (width - 31.) / 4.) {
+        double t = myBeginOffset != nullptr ? myBeginOffset->getValue() : STEPS2TIME(myLastTime - myBeginTime);
+        while (t > barWidth / 4.) {
             tickDist += TIME2STEPS(10);
-            t -= (SUMOReal)((width - 31.) / 4.);
+            t -= barWidth / 4.;
         }
         // draw time information
-        //h = (SUMOReal)(myTLLogic->getLinks().size() * 20 + 12);
-        SUMOReal glh = (SUMOReal)(1.0 - myTLLogic->getLinks().size() * h20 - h10);
+        //h = (double)(myTLLogic->getLinks().size() * 20 + 12);
+        double glh = (double)(1.0 - myTLLogic->getLinks().size() * h20 - h10);
         // current begin time
-        pfSetScaleXY((SUMOReal)(.05 * 300. / width), (SUMOReal)(.05 * 300. / height));
         // time ticks
         SUMOTime currTime = myFirstTime2Show;
         int pos = 31;// + /*!!!currTime*/ - myFirstTime2Show;
-        SUMOReal glpos = (SUMOReal) pos / width;
-        while (pos < width + 50) {
-            const std::string timeStr = time2string(currTime);
-            const SUMOReal w = pfdkGetStringWidth(timeStr.c_str());
-            glRotated(180, 1, 0, 0);
-            pfSetPosition(0, 0);
-            glTranslated(glpos - w / 2., -glh + h20 - h4, 0);
-            pfDrawString(timeStr.c_str());
-            glTranslated(-glpos + w / 2., glh - h20 + h4, 0);
-            glRotated(-180, 1, 0, 0);
+        double glpos = (double) pos / panelWidth;
+        const double ticSize = 4. / panelHeight;
+        while (pos < panelWidth + 50.) {
+            const std::string timeStr = (gHumanReadableTime
+                                         ? time2string(currTime % 3600000).substr(3) // only write mn:ss
+                                         : toString((int)STEPS2TIME(currTime)));
+            const double w = 50. / panelWidth;
+            glTranslated(glpos - w / 2., glh - h20, 0);
+            GLHelper::drawText(timeStr, Position(0, 0), 1, fontHeight, RGBColor::WHITE, 0, FONS_ALIGN_LEFT | FONS_ALIGN_MIDDLE, fontWidth);
+            glTranslated(-glpos + w / 2., -glh + h20, 0);
 
             glBegin(GL_LINES);
             glVertex2d(glpos, glh);
-            glVertex2d(glpos, glh - h4);
+            glVertex2d(glpos, glh - ticSize);
             glEnd();
 
-            const SUMOReal a = STEPS2TIME(tickDist) * (width - 31.0) / STEPS2TIME(myLastTime - myBeginTime);
+            const double a = STEPS2TIME(tickDist) * barWidth / STEPS2TIME(myLastTime - myBeginTime);
             pos += (int) a;
-            glpos += a / width;
+            glpos += a / panelWidth;
             currTime += tickDist;
         }
     }
@@ -471,24 +462,21 @@ GUITLLogicPhasesTrackerWindow::addValue(std::pair<SUMOTime, MSPhaseDefinition> d
 
 
 long
-GUITLLogicPhasesTrackerWindow::onConfigure(FXObject* sender,
-        FXSelector sel, void* data) {
-    myPanel->onConfigure(sender, sel, data);
-    return FXMainWindow::onConfigure(sender, sel, data);
+GUITLLogicPhasesTrackerWindow::onConfigure(FXObject* sender, FXSelector sel, void* ptr) {
+    myPanel->onConfigure(sender, sel, ptr);
+    return FXMainWindow::onConfigure(sender, sel, ptr);
 }
 
 
 long
-GUITLLogicPhasesTrackerWindow::onPaint(FXObject* sender,
-                                       FXSelector sel, void* data) {
-    myPanel->onPaint(sender, sel, data);
-    return FXMainWindow::onPaint(sender, sel, data);
+GUITLLogicPhasesTrackerWindow::onPaint(FXObject* sender, FXSelector sel, void* ptr) {
+    myPanel->onPaint(sender, sel, ptr);
+    return FXMainWindow::onPaint(sender, sel, ptr);
 }
 
 
 long
-GUITLLogicPhasesTrackerWindow::onSimStep(FXObject*,
-        FXSelector, void*) {
+GUITLLogicPhasesTrackerWindow::onSimStep(FXObject*, FXSelector, void*) {
     update();
     return 1;
 }
@@ -501,4 +489,3 @@ GUITLLogicPhasesTrackerWindow::setBeginTime(SUMOTime time) {
 
 
 /****************************************************************************/
-
