@@ -1,44 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIViewTraffic.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Andreas Gaubatz
 /// @date    Sept 2002
-/// @version $Id: GUIViewTraffic.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A view on the simulation; this view is a microscopic one
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIViewTraffic_h
-#define GUIViewTraffic_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/geom/Boundary.h>
 #include <utils/geom/Position.h>
 #include <utils/common/RGBColor.h>
 #include <utils/geom/PositionVector.h>
-#include <utils/shapes/Polygon.h>
 #include "GUISUMOViewParent.h"
 #include <utils/gui/windows/GUISUMOAbstractView.h>
 
@@ -63,15 +52,15 @@ class MSRoute;
 class GUIViewTraffic : public GUISUMOAbstractView {
     FXDECLARE(GUIViewTraffic)
 public:
-    /// constructor
+    /// @brief constructor
     GUIViewTraffic(FXComposite* p, GUIMainWindow& app,
                    GUISUMOViewParent* parent, GUINet& net, FXGLVisual* glVis,
                    FXGLCanvas* share);
-    /// destructor
+    /// @brief destructor
     virtual ~GUIViewTraffic();
 
-    /// builds the view toolbars
-    virtual void buildViewToolBars(GUIGlChildWindow&);
+    /// @brief builds the view toolbars
+    virtual void buildViewToolBars(GUIGlChildWindow*);
 
 
     /** @brief Starts vehicle tracking
@@ -88,12 +77,29 @@ public:
     /** @brief Returns the id of the tracked vehicle (-1 if none)
      * @return The glID of the vehicle to track
      */
-    int getTrackedID() const;
+    GUIGlID getTrackedID() const;
 
     bool setColorScheme(const std::string& name);
 
+    /// @brief recalibrate color scheme according to the current value range
+    void buildColorRainbow(const GUIVisualizationSettings& s, GUIColorScheme& scheme, int active, GUIGlObjectType objectType,
+                           bool hide = false, double hideThreshold = 0);
+
+    /// @brief return list of loaded edgeData attributes
+    std::vector<std::string> getEdgeDataAttrs() const;
+
+    /// @brief return list of available edge parameters
+    std::vector<std::string> getEdgeLaneParamKeys(bool edgeKeys) const;
+
+    /// @brief return list of available vehicle parameters
+    std::vector<std::string> getVehicleParamKeys(bool vTypeKeys) const;
+
+    /// @brief return list of available POI parameters
+    std::vector<std::string> getPOIParamKeys() const;
+
     /// @brief handle mouse click in gaming mode
     void onGamingClick(Position pos);
+    void onGamingRightClick(Position pos);
 
     /// @brief get the current simulation time
     SUMOTime getCurrentTimeStep() const;
@@ -103,6 +109,11 @@ public:
     long onCmdCloseEdge(FXObject*, FXSelector, void*);
     long onCmdAddRerouter(FXObject*, FXSelector, void*);
 
+    /// @brief highlight edges according to reachability
+    long onCmdShowReachability(FXObject*, FXSelector, void*);
+
+    long onDoubleClicked(FXObject*, FXSelector, void*);
+
     /** @brief Adds a frame to a video snapshot which will be initialized if neccessary
      */
     void saveFrame(const std::string& destFile, FXColor* buf);
@@ -115,13 +126,19 @@ public:
      */
     void checkSnapshots();
 
+    /// @brief retrieve breakpoints from the current runThread
+    const std::vector<SUMOTime> retrieveBreakpoints() const;
+
 protected:
     int doPaintGL(int mode, const Boundary& bound);
 
     GUILane* getLaneUnderCursor();
 
 private:
-    int myTrackedID;
+    GUIGlID myTrackedID;
+
+    /// @brief whether game mode was set to 'tls'
+    bool myTLSGame;
 
 #ifdef HAVE_FFMPEG
     GUIVideoEncoder* myCurrentVideo;
@@ -131,9 +148,3 @@ protected:
     GUIViewTraffic() { }
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
