@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIDialog_AboutSUMO.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Fri, 29.04.2005
-/// @version $Id: GUIDialog_AboutSUMO.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // The application's "About" - dialog
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIDialog_AboutSUMO_h
-#define GUIDialog_AboutSUMO_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
 
 
 // ===========================================================================
@@ -44,35 +34,17 @@
 class GUIDialog_AboutSUMO : public FXDialogBox {
 public:
     /** @brief Constructor
-     *
      * @param[in] parent The parent window
-     * @param[in] name The title to show
-     * @param[in] x x-position of the window to use
-     * @param[in] y y-position of the window to use
      */
-    GUIDialog_AboutSUMO(FXWindow* parent,  const char* name, int x, int y);
-
+    GUIDialog_AboutSUMO(FXWindow* parent);
 
     /// @brief Destructor
     ~GUIDialog_AboutSUMO();
 
-
-    /** @brief Creates the widget (and the icons)
-     */
+    /// @brief Creates the widget
     void create();
 
-
 private:
-    /// @brief Icons for the widget
-    FXIcon* myDLRIcon;
-
     /// @brief Font for the widget
     FXFont* myHeadlineFont;
-
 };
-
-
-#endif
-
-/****************************************************************************/
-
