@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIDialog_Breakpoints.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Thu, 17 Jun 2004
-/// @version $Id: GUIDialog_Breakpoints.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Editor for simulation breakpoints
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -41,7 +32,7 @@
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <utils/foxtools/MFXUtils.h>
 #include <utils/common/ToString.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/gui/windows/GUISUMOAbstractView.h>
 #include <utils/gui/settings/GUISettingsHandler.h>
 #include <utils/common/FileHelpers.h>
@@ -57,6 +48,7 @@
 // ===========================================================================
 // FOX callback mapping
 // ===========================================================================
+
 FXDEFMAP(GUIDialog_Breakpoints) GUIDialog_BreakpointsMap[] = {
     FXMAPFUNC(SEL_COMMAND,  MID_CHOOSEN_LOAD,  GUIDialog_Breakpoints::onCmdLoad),
     FXMAPFUNC(SEL_COMMAND,  MID_CHOOSEN_SAVE,  GUIDialog_Breakpoints::onCmdSave),
@@ -68,17 +60,18 @@ FXDEFMAP(GUIDialog_Breakpoints) GUIDialog_BreakpointsMap[] = {
 
 FXIMPLEMENT(GUIDialog_Breakpoints, FXMainWindow, GUIDialog_BreakpointsMap, ARRAYNUMBER(GUIDialog_BreakpointsMap))
 
-
 // ===========================================================================
 // method definitions
 // ===========================================================================
-GUIDialog_Breakpoints::GUIDialog_Breakpoints(GUIMainWindow* parent, std::vector<SUMOTime>& breakpoints, FXMutex& breakpointLock) :
-    FXMainWindow(parent->getApp(), "Breakpoints Editor", NULL, NULL, DECOR_ALL, 20, 20, 170, 300),
-    myParent(parent), myBreakpoints(&breakpoints), myBreakpointLock(&breakpointLock) {
-    FXHorizontalFrame* hbox = new FXHorizontalFrame(this, GUIDesignAuxiliarFrame);
 
+GUIDialog_Breakpoints::GUIDialog_Breakpoints(GUIMainWindow* parent, std::vector<SUMOTime>& breakpoints, FXMutex& breakpointLock) :
+    FXMainWindow(parent->getApp(), "Breakpoints Editor", GUIIconSubSys::getIcon(GUIIcon::APP_BREAKPOINTS), nullptr, GUIDesignChooserDialog),
+    myParent(parent), myBreakpoints(&breakpoints), myBreakpointLock(&breakpointLock) {
+    // build main Frame
+    FXHorizontalFrame* hbox = new FXHorizontalFrame(this, GUIDesignAuxiliarFrame);
     // build the table
-    myTable = new FXTable(hbox, this, MID_TABLE, GUIDesignTable);
+    FXVerticalFrame* layoutLeft = new FXVerticalFrame(hbox, GUIDesignChooserLayoutLeft);
+    myTable = new FXTable(layoutLeft, this, MID_TABLE, GUIDesignBreakpointTable);
     myTable->setVisibleRows(20);
     myTable->setVisibleColumns(1);
     myTable->setTableSize(20, 1);
@@ -88,27 +81,25 @@ GUIDialog_Breakpoints::GUIDialog_Breakpoints(GUIMainWindow* parent, std::vector<
     rebuildList();
     myBreakpointLock->unlock();
     // build the layout
-    FXVerticalFrame* layout = new FXVerticalFrame(hbox, LAYOUT_TOP, 0, 0, 0, 0, 4, 4, 4, 4);
-
+    FXVerticalFrame* layoutRight = new FXVerticalFrame(hbox, GUIDesignChooserLayoutRight);
     // create buttons ('&' in the label creates a hot key)
     // "Load"
-    new FXButton(layout, "&Load\t\t", 0, this, MID_CHOOSEN_LOAD, GUIDesignButtonBreakpoint);
+    new FXButton(layoutRight, "&Load\t\t", GUIIconSubSys::getIcon(GUIIcon::OPEN_CONFIG), this, MID_CHOOSEN_LOAD, GUIDesignChooserButtons);
     // "Save"
-    new FXButton(layout, "&Save\t\t", 0, this, MID_CHOOSEN_SAVE, GUIDesignButtonBreakpoint);
-    new FXHorizontalSeparator(layout, GUIDesignHorizontalSeparator);
+    new FXButton(layoutRight, "&Save\t\t", GUIIconSubSys::getIcon(GUIIcon::SAVE), this, MID_CHOOSEN_SAVE, GUIDesignChooserButtons);
+    new FXHorizontalSeparator(layoutRight, GUIDesignHorizontalSeparator);
     // "Clear List"
-    new FXButton(layout, "Clea&r\t\t", 0, this, MID_CHOOSEN_CLEAR, GUIDesignButtonBreakpoint);
-    new FXHorizontalSeparator(layout, GUIDesignHorizontalSeparator);
+    new FXButton(layoutRight, "Clea&r\t\t", GUIIconSubSys::getIcon(GUIIcon::CLEANJUNCTIONS), this, MID_CHOOSEN_CLEAR, GUIDesignChooserButtons);
+    new FXHorizontalSeparator(layoutRight, GUIDesignHorizontalSeparator);
     // "Close"
-    new FXButton(layout, "&Close\t\t", 0, this, MID_CANCEL, GUIDesignButtonBreakpoint);
-
-    //
-    setIcon(GUIIconSubSys::getIcon(ICON_APP_BREAKPOINTS));
+    new FXButton(layoutRight, "&Close\t\t", GUIIconSubSys::getIcon(GUIIcon::NO), this, MID_CANCEL, GUIDesignChooserButtons);
+    // add this dialog as child of GUIMainWindow parent
     myParent->addChild(this);
 }
 
 
 GUIDialog_Breakpoints::~GUIDialog_Breakpoints() {
+    // remove this dialog as child of GUIMainWindow parent
     myParent->removeChild(this);
 }
 
@@ -128,7 +119,7 @@ GUIDialog_Breakpoints::rebuildList() {
     myTable->setTableSize((FXint)myBreakpoints->size() + 1, 1);
     myTable->setColumnText(0, "Time");
     FXHeader* header = myTable->getColumnHeader();
-    header->setHeight(getApp()->getNormalFont()->getFontHeight() + getApp()->getNormalFont()->getFontAscent());
+    header->setHeight(GUIDesignHeight);
     header->setItemJustify(0, JUSTIFY_CENTER_X);
     // insert into table
     for (int row = 0; row < (int)myBreakpoints->size(); row++) {
@@ -142,7 +133,7 @@ GUIDialog_Breakpoints::rebuildList() {
 long
 GUIDialog_Breakpoints::onCmdLoad(FXObject*, FXSelector, void*) {
     FXFileDialog opendialog(this, "Load Breakpoints");
-    opendialog.setIcon(GUIIconSubSys::getIcon(ICON_EMPTY));
+    opendialog.setIcon(GUIIconSubSys::getIcon(GUIIcon::EMPTY));
     opendialog.setSelectMode(SELECTFILE_ANY);
     opendialog.setPatternList("*.txt");
     if (gCurrentFolder.length() != 0) {
@@ -162,7 +153,7 @@ GUIDialog_Breakpoints::onCmdLoad(FXObject*, FXSelector, void*) {
 
 long
 GUIDialog_Breakpoints::onCmdSave(FXObject*, FXSelector, void*) {
-    FXString file = MFXUtils::getFilename2Write(this, "Save Breakpoints", ".txt", GUIIconSubSys::getIcon(ICON_EMPTY), gCurrentFolder);
+    FXString file = MFXUtils::getFilename2Write(this, "Save Breakpoints", ".txt", GUIIconSubSys::getIcon(GUIIcon::EMPTY), gCurrentFolder);
     if (file == "") {
         return 1;
     }
@@ -208,10 +199,10 @@ GUIDialog_Breakpoints::onCmdClose(FXObject*, FXSelector, void*) {
 
 
 long
-GUIDialog_Breakpoints::onCmdEditTable(FXObject*, FXSelector, void* data) {
+GUIDialog_Breakpoints::onCmdEditTable(FXObject*, FXSelector, void* ptr) {
     FXMutexLock lock(*myBreakpointLock);
-    const FXTablePos* const i = (FXTablePos*) data;
-    const std::string value = myTable->getItemText(i->row, i->col).text();
+    const FXTablePos* const i = (FXTablePos*) ptr;
+    const std::string value = StringUtils::prune(myTable->getItemText(i->row, i->col).text());
     // check whether the inserted value is empty
     const bool empty = value.find_first_not_of(" ") == std::string::npos;
     try {
@@ -228,12 +219,21 @@ GUIDialog_Breakpoints::onCmdEditTable(FXObject*, FXSelector, void* data) {
         }
     } catch (NumberFormatException&) {
         std::string msg = "The value must be a number, is:" + value;
-        FXMessageBox::error(this, MBOX_OK, "Number format error", "%s", msg.c_str());
+        FXMessageBox::error(this, MBOX_OK, "Time format error", "%s", msg.c_str());
+    } catch (ProcessError&) {
+        std::string msg = "The value must be a number or a string of the form hh:mm:ss, is:" + value;
+        FXMessageBox::error(this, MBOX_OK, "Time format error", "%s", msg.c_str());
     }
     rebuildList();
     return 1;
 }
 
 
-/****************************************************************************/
+void
+GUIDialog_Breakpoints::layout() {
+    FXMainWindow::layout();
+    myTable->setColumnWidth(0, myTable->getWidth() - 1);
+}
 
+
+/****************************************************************************/
