@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIEdgeControlBuilder.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIEdgeControlBuilder.h 17262 2014-11-04 20:41:44Z behrisch $
 ///
 // Derivation of NLEdgeControlBuilder which builds gui-edges
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIEdgeControlBuilder_h
-#define GUIEdgeControlBuilder_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -77,14 +67,17 @@ public:
      * @param[in] shape The shape of the lane
      * @param[in] width The width of the lane
      * @param[in] permissions Encoding of vehicle classes that may drive on this lane
+     * @param[in] index The index of this lane within its parent edge
      * @see SUMOVehicleClass
      * @see MSLane
      */
-    virtual MSLane* addLane(const std::string& id,
-                            SUMOReal maxSpeed, SUMOReal length,
-                            const PositionVector& shape, SUMOReal width,
-                            SVCPermissions permissions);
-
+    MSLane* addLane(const std::string& id,
+                    double maxSpeed, double length,
+                    const PositionVector& shape, double width,
+                    SVCPermissions permissions,
+                    SVCPermissions changeLeft, SVCPermissions changeRight,
+                    int index, bool isRampAccel,
+                    const std::string& type) override;
 
 
 
@@ -96,8 +89,8 @@ public:
      *
      * @param[in] id The id of the edge to build
      */
-    MSEdge* buildEdge(const std::string& id, const MSEdge::EdgeBasicFunction function,
-                      const std::string& streetName, const std::string& edgeType, const int priority);
+    MSEdge* buildEdge(const std::string& id, const SumoXMLEdgeFunc function,
+                      const std::string& streetName, const std::string& edgeType, const int priority, const double distance) override;
 
 
 private:
@@ -108,9 +101,3 @@ private:
     GUIEdgeControlBuilder& operator=(const GUIEdgeControlBuilder& s);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
