@@ -1,40 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUITriggerBuilder.h
 /// @author  Daniel Krajzewicz
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
+/// @author  Johannes Rummel
 /// @date    Mon, 26.04.2004
-/// @version $Id: GUITriggerBuilder.h 23648 2017-03-24 10:35:01Z laemmel $
 ///
 // Builds trigger objects for guisim
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUITriggerBuilder_h
-#define GUITriggerBuilder_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <netload/NLTriggerBuilder.h>
+#include <utils/common/RGBColor.h>
 
 
 // ===========================================================================
@@ -84,7 +76,7 @@ protected:
      */
     virtual MSLaneSpeedTrigger* buildLaneSpeedTrigger(MSNet& net,
             const std::string& id, const std::vector<MSLane*>& destLanes,
-            const std::string& file);
+            const std::string& file) override;
 
 
     /** @brief builds an rerouter
@@ -99,7 +91,9 @@ protected:
      */
     virtual MSTriggeredRerouter* buildRerouter(MSNet& net,
             const std::string& id, MSEdgeVector& edges,
-            double prob, const std::string& file, bool off);
+            double prob, const std::string& file, bool off,
+            SUMOTime timeThreshold,
+            const std::string& vTypes) override;
 
 
     /** @brief Builds a bus stop
@@ -116,7 +110,8 @@ protected:
      * @exception InvalidArgument If the stop can not be added to the net (is duplicate)
      */
     virtual void buildStoppingPlace(MSNet& net, std::string id, std::vector<std::string> lines, MSLane* lane,
-                                    double frompos, double topos, const SumoXMLTag element, std::string string);
+                                    double frompos, double topos, const SumoXMLTag element, std::string string,
+                                    int personCapacity, double parkingLength, RGBColor& color) override;
 
 
     /** @brief Builds a parking area
@@ -139,12 +134,13 @@ protected:
                                   const std::vector<std::string>& lines, MSLane* lane,
                                   double frompos, double topos,
                                   unsigned int capacity,
-                                  double width, double length, double angle);
+                                  double width, double length, double angle, const std::string& name,
+                                  bool onRoad) override;
 
 
     /** @brief Builds a charging station
      *
-     * Simply calls the ChrgSTN constructor.
+     * Simply calls the GUIChargingStation constructor.
      *
      * @param[in] net The net the charging station belongs to
      * @param[in] id The id of the charging station
@@ -158,30 +154,46 @@ protected:
      * @exception InvalidArgument If the charging station can not be added to the net (is duplicate)
      */
     virtual void buildChargingStation(MSNet& net, const std::string& id, MSLane* lane,
-                                      double frompos, double topos, double chargingPower, double efficiency,
-                                      bool chargeInTransit, int chargeDelay);
+                                      double frompos, double topos, const std::string& name,
+                                      double chargingPower, double efficiency,
+                                      bool chargeInTransit, double chargeDelay) override;
 
-    /** @brief builds a microscopic calibrator
-     *
-     * Simply calls the MSCalibrator constructor.
-     *
-     * @param[in] net The net the calibrator belongs to
-     * @param[in] id The id of the calibrator
-     * @param[in] edge The edge the calibrator is placed at
-     * @param[in] pos The position on the edge the calibrator lies at
-     * @param[in] file The file to read the flows from
-     * @todo Is the position correct/needed
-     */
-    virtual MSCalibrator* buildCalibrator(MSNet& net,
-                                          const std::string& id, MSEdge* edge, double pos,
-                                          const std::string& file, const std::string& outfile,
-                                          const SUMOTime freq, const MSRouteProbe* probe);
+    /** @brief Builds an overhead wire segment
+    *
+    * Simply calls the GUIOverheadWire constructor and adds the result to the network.
+    *
+    * @param[in] net The net the overhead wire segment belongs to
+    * @param[in] id The id of the overhead wire segment
+    * @param[in] lane The lane the overhead wire segment is placed on
+    * @param[in] frompos Begin position of the overhead wire segment on the lane
+    * @param[in] topos End position of the overhead wire segment  on the lane
+    * @param[in] voltageSource If the segment is the place of the connection of a traction substation
+    * @exception InvalidArgument If the overhead wire segment can not be added to the net (is duplicate according to the id)
+    */
+    virtual void buildOverheadWireSegment(MSNet& net, const std::string& id, MSLane* lane,
+                                          double frompos, double topos, bool voltageSource) override;
+
+    /** @brief Builds an overhead wire clamp
+    *
+    * Simply calls the GUIOverheadWireClamp constructor and adds the result to the network.
+    *
+    * @param[in] net The net the overhead wire clamp belongs to
+    * @param[in] id The id of the overhead wire clamp
+    * @param[in] lane_start The lane, where is the overhead wire segment placed, to the start of which the overhead wire clamp is connected
+    * @param[in] lane_end The lane, where is the overhead wire segment placed, to the end of which the overhead wire clamp is connected
+    */
+    virtual void buildOverheadWireClamp(MSNet& net, const std::string& id, MSLane* lane_start, MSLane* lane_end) override;
     /// @}
 
+
+    /** @brief End a parking area
+     * (it must be added to the SUMORTree after all parking spaces are loaded
+     */
+    virtual void endParkingArea() override;
+
+    /** @brief End a stopping place
+     *
+     * @exception InvalidArgument If the current stopping place is 0
+     */
+    virtual void endStoppingPlace() override;
 };
-
-
-#endif
-
-/****************************************************************************/
-
