@@ -1,32 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUICalibrator.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 26.04.2004
-/// @version $Id: GUICalibrator.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Changes flow and speed on a set of lanes (gui version)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/common/MsgHandler.h>
@@ -40,7 +33,6 @@
 #include <microsim/MSEdge.h>
 #include <guisim/GUINet.h>
 #include <guisim/GUIEdge.h>
-#include "GUICalibrator.h"
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
 #include <utils/gui/windows/GUIAppEnum.h>
 #include <gui/GUIGlobals.h>
@@ -48,14 +40,11 @@
 #include <gui/GUIApplicationWindow.h>
 #include <microsim/logging/FunctionBinding.h>
 #include <utils/gui/div/GUIGlobalSelection.h>
-#include <foreign/polyfonts/polyfonts.h>
 #include <utils/gui/images/GUIIconSubSys.h>
 #include <guisim/GUICalibrator.h>
 #include <utils/gui/globjects/GLIncludes.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
+#include "GUICalibrator.h"
 
 
 // ===========================================================================
@@ -102,7 +91,7 @@ GUICalibrator::GUIManip_Calibrator::GUIManip_Calibrator(
     GUIManipulator(app, name, 0, 0),
     myParent(&app),
     myChosenValue(0),
-    myChosenTarget(myChosenValue, NULL, MID_OPTION),
+    myChosenTarget(myChosenValue, nullptr, MID_OPTION),
     //mySpeed(o.getDefaultSpeed()),
     mySpeed(0),
     mySpeedTarget(mySpeed),
@@ -160,15 +149,16 @@ GUICalibrator::GUIManip_Calibrator::GUIManip_Calibrator(
                           ICON_BEFORE_TEXT | LAYOUT_SIDE_TOP | LAYOUT_CENTER_Y,
                           0, 0, 0, 0,   2, 2, 0, 0);
         myUserDefinedSpeed =
-            new FXRealSpinDial(gf12, 10, this, MID_USER_DEF,
-                               LAYOUT_TOP | FRAME_SUNKEN | FRAME_THICK);
-        myUserDefinedSpeed->setFormatString("%.0f km/h");
-        myUserDefinedSpeed->setIncrements(1, 10, 10);
+            new FXRealSpinner(gf12, 10, this, MID_USER_DEF,
+                              LAYOUT_TOP | FRAME_SUNKEN | FRAME_THICK);
+        //myUserDefinedSpeed->setFormatString("%.0f km/h");
+        //myUserDefinedSpeed->setIncrements(1, 10, 10);
+        myUserDefinedSpeed->setIncrement(10);
         myUserDefinedSpeed->setRange(0, 300);
         myUserDefinedSpeed->setValue(0);
         //static_cast<GUICalibrator*>(myObject)->getDefaultSpeed() * 3.6);
     }
-    new FXButton(f1, "Close", NULL, this, MID_CLOSE,
+    new FXButton(f1, "Close", nullptr, this, MID_CLOSE,
                  BUTTON_INITIAL | BUTTON_DEFAULT | FRAME_RAISED | FRAME_THICK | LAYOUT_TOP | LAYOUT_LEFT | LAYOUT_CENTER_X, 0, 0, 0, 0, 30, 30, 4, 4);
     //static_cast<GUICalibrator*>(myObject)->setOverriding(true);
 }
@@ -186,7 +176,7 @@ GUICalibrator::GUIManip_Calibrator::onCmdClose(FXObject*, FXSelector, void*) {
 
 long
 GUICalibrator::GUIManip_Calibrator::onCmdUserDef(FXObject*, FXSelector, void*) {
-    //mySpeed = (SUMOReal)(myUserDefinedSpeed->getValue() / 3.6);
+    //mySpeed = (double)(myUserDefinedSpeed->getValue() / 3.6);
     //static_cast<GUICalibrator*>(myObject)->setOverridingValue(mySpeed);
     //myParent->updateChildren();
     return 1;
@@ -205,7 +195,7 @@ GUICalibrator::GUIManip_Calibrator::onUpdUserDef(FXObject* sender, FXSelector, v
 
 long
 GUICalibrator::GUIManip_Calibrator::onCmdPreDef(FXObject*, FXSelector, void*) {
-    //mySpeed = (SUMOReal)(SUMOReal)((myPredefinedValues->getCurrentItem() * 20 + 20) / 3.6);
+    //mySpeed = (double)(double)((myPredefinedValues->getCurrentItem() * 20 + 20) / 3.6);
     //static_cast<GUICalibrator*>(myObject)->setOverridingValue(mySpeed);
     //myParent->updateChildren();
     return 1;
@@ -227,16 +217,16 @@ GUICalibrator::GUIManip_Calibrator::onCmdChangeOption(FXObject*, FXSelector, voi
     //static_cast<GUICalibrator*>(myObject)->setOverriding(true);
     //switch (myChosenValue) {
     //    case 0:
-    //        mySpeed = (SUMOReal) static_cast<GUICalibrator*>(myObject)->getDefaultSpeed();
+    //        mySpeed = (double) static_cast<GUICalibrator*>(myObject)->getDefaultSpeed();
     //        break;
     //    case 1:
-    //        mySpeed = (SUMOReal) static_cast<GUICalibrator*>(myObject)->getLoadedSpeed();
+    //        mySpeed = (double) static_cast<GUICalibrator*>(myObject)->getLoadedSpeed();
     //        break;
     //    case 2:
-    //        mySpeed = (SUMOReal)((myPredefinedValues->getCurrentItem() * 20 + 20) / 3.6);
+    //        mySpeed = (double)((myPredefinedValues->getCurrentItem() * 20 + 20) / 3.6);
     //        break;
     //    case 3:
-    //        mySpeed = (SUMOReal)(myUserDefinedSpeed->getValue() / 3.6);
+    //        mySpeed = (double)(myUserDefinedSpeed->getValue() / 3.6);
     //        break;
     //    default:
     //        // hmmm, should not happen
@@ -278,23 +268,20 @@ GUICalibrator::GUICalibratorPopupMenu::onCmdOpenManip(FXObject*,
 /* -------------------------------------------------------------------------
  * GUICalibrator - methods
  * ----------------------------------------------------------------------- */
-GUICalibrator::GUICalibrator(const std::string& id,
-                             MSEdge* edge, SUMOReal pos,
-                             const std::string& aXMLFilename,
-                             const std::string& outputFilename,
-                             const SUMOTime freq,
-                             const MSRouteProbe* probe) :
-    MSCalibrator(id, edge, pos, aXMLFilename, outputFilename, freq, edge->getLength(), probe),
-    GUIGlObject_AbstractAdd("calibrator", GLO_TRIGGER, id),
+GUICalibrator::GUICalibrator(MSCalibrator* calibrator) :
+    GUIGlObject_AbstractAdd(GLO_CALIBRATOR, calibrator->getID()),
+    myCalibrator(calibrator),
     myShowAsKMH(true) {
-    const std::vector<MSLane*>& destLanes = edge->getLanes();
-    myFGPositions.reserve(destLanes.size());
-    myFGRotations.reserve(destLanes.size());
+    const std::vector<MSLane*>& destLanes = calibrator->myEdge->getLanes();
+    const MSLane* lane = calibrator->myLane;
+    const double pos = calibrator->myPos;
     for (std::vector<MSLane*>::const_iterator i = destLanes.begin(); i != destLanes.end(); ++i) {
-        const PositionVector& v = (*i)->getShape();
-        myFGPositions.push_back(v.positionAtOffset(pos));
-        myBoundary.add(v.positionAtOffset(pos));
-        myFGRotations.push_back(-v.rotationDegreeAtOffset(pos));
+        if (lane == nullptr || (*i) == lane) {
+            const PositionVector& v = (*i)->getShape();
+            myFGPositions.push_back(v.positionAtOffset(pos));
+            myBoundary.add(v.positionAtOffset(pos));
+            myFGRotations.push_back(-v.rotationDegreeAtOffset(pos));
+        }
     }
 }
 
@@ -321,23 +308,26 @@ GUIParameterTableWindow*
 GUICalibrator::getParameterWindow(GUIMainWindow& app,
                                   GUISUMOAbstractView&) {
     GUIParameterTableWindow* ret;
-    if (isActive()) {
-        ret = new GUIParameterTableWindow(app, *this, 10);
+    auto myCurrentStateInterval = myCalibrator->myCurrentStateInterval;
+    if (myCalibrator->isActive()) {
+        ret = new GUIParameterTableWindow(app, *this);
         // add items
         ret->mkItem("interval start", false, STEPS2TIME(myCurrentStateInterval->begin));
         ret->mkItem("interval end", false, STEPS2TIME(myCurrentStateInterval->end));
         ret->mkItem("aspired flow [veh/h]", false, myCurrentStateInterval->q);
         ret->mkItem("aspired speed", false, myCurrentStateInterval->v);
-        ret->mkItem("default speed", false, myDefaultSpeed);
-        ret->mkItem("required vehicles", true, new FunctionBinding<GUICalibrator, int>(this, &GUICalibrator::totalWished));
-        ret->mkItem("passed vehicles", true, new FunctionBinding<GUICalibrator, int>(this, &GUICalibrator::passed));
-        ret->mkItem("inserted vehicles", true, new FunctionBinding<GUICalibrator, int>(this, &GUICalibrator::inserted));
-        ret->mkItem("removed vehicles", true, new FunctionBinding<GUICalibrator, int>(this, &GUICalibrator::removed));
-        ret->mkItem("cleared in jam", true, new FunctionBinding<GUICalibrator, int>(this, &GUICalibrator::clearedInJam));
+        ret->mkItem("current flow [veh/h]", true, new FunctionBinding<MSCalibrator, double>(myCalibrator, &MSCalibrator::currentFlow));
+        ret->mkItem("current speed", true, new FunctionBinding<MSCalibrator, double>(myCalibrator, &MSCalibrator::currentSpeed));
+        ret->mkItem("default speed", false, myCalibrator->myDefaultSpeed);
+        ret->mkItem("required vehicles", true, new FunctionBinding<MSCalibrator, int>(myCalibrator, &MSCalibrator::totalWished));
+        ret->mkItem("passed vehicles", true, new FunctionBinding<MSCalibrator, int>(myCalibrator, &MSCalibrator::passed));
+        ret->mkItem("inserted vehicles", true, new FunctionBinding<MSCalibrator, int>(myCalibrator, &MSCalibrator::inserted));
+        ret->mkItem("removed vehicles", true, new FunctionBinding<MSCalibrator, int>(myCalibrator, &MSCalibrator::removed));
+        ret->mkItem("cleared in jam", true, new FunctionBinding<MSCalibrator, int>(myCalibrator, &MSCalibrator::clearedInJam));
     } else {
-        ret = new GUIParameterTableWindow(app, *this, 1);
+        ret = new GUIParameterTableWindow(app, *this);
         const std::string nextStart =
-            (myCurrentStateInterval != myIntervals.end() ?
+            (myCurrentStateInterval != myCalibrator->myIntervals.end() ?
              time2string(myCurrentStateInterval->begin) :
              "simulation end");
         ret->mkItem("inactive until", false, nextStart);
@@ -353,7 +343,8 @@ GUICalibrator::drawGL(const GUIVisualizationSettings& s) const {
     glPushName(getGlID());
     std::string flow = "-";
     std::string speed = "-";
-    if (isActive()) {
+    if (myCalibrator->isActive()) {
+        auto myCurrentStateInterval = myCalibrator->myCurrentStateInterval;
         if (myCurrentStateInterval->v >= 0) {
             speed = toString(myCurrentStateInterval->v) + "m/s";
         }
@@ -361,10 +352,10 @@ GUICalibrator::drawGL(const GUIVisualizationSettings& s) const {
             flow = toString((int)myCurrentStateInterval->q) + "v/h";
         }
     }
-    const SUMOReal exaggeration = s.addSize.getExaggeration(s);
-    for (size_t i = 0; i < myFGPositions.size(); ++i) {
+    const double exaggeration = s.addSize.getExaggeration(s, this);
+    for (int i = 0; i < (int)myFGPositions.size(); ++i) {
         const Position& pos = myFGPositions[i];
-        SUMOReal rot = myFGRotations[i];
+        double rot = myFGRotations[i];
         glPushMatrix();
         glTranslated(pos.x(), pos.y(), getType());
         glRotated(rot, 0, 0, 1);
@@ -386,29 +377,9 @@ GUICalibrator::drawGL(const GUIVisualizationSettings& s) const {
         // draw text
         if (s.scale * exaggeration >= 1.) {
             glTranslated(0, 0, .1);
-            glColor3d(0, 0, 0);
-            pfSetPosition(0, 0);
-            pfSetScale(3.f);
-            SUMOReal w = pfdkGetStringWidth("C");
-            glRotated(180, 0, 1, 0);
-            glTranslated(-w / 2., 2, 0);
-            pfDrawString("C");
-            glTranslated(w / 2., -2, 0);
-
-
-            pfSetPosition(0, 0);
-            pfSetScale(.7f);
-            w = pfdkGetStringWidth(flow.c_str());
-            glTranslated(-w / 2., 4, 0);
-            pfDrawString(flow.c_str());
-            glTranslated(w / 2., -4, 0);
-
-            pfSetPosition(0, 0);
-            pfSetScale(.7f);
-            w = pfdkGetStringWidth(speed.c_str());
-            glTranslated(-w / 2., 5, 0);
-            pfDrawString(speed.c_str());
-            glTranslated(-w / 2., -5, 0);
+            GLHelper::drawText("C", Position(0, 2), 0.1, 3, RGBColor::BLACK, 180);
+            GLHelper::drawText(flow, Position(0, 4), 0.1, 0.7, RGBColor::BLACK, 180);
+            GLHelper::drawText(speed, Position(0, 5), 0.1, 0.7, RGBColor::BLACK, 180);
         }
         glPopMatrix();
     }
@@ -436,6 +407,4 @@ GUICalibrator::openManipulator(GUIMainWindow& app,
 }
 
 
-
 /****************************************************************************/
-
