@@ -1,43 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUICalibrator.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 26.04.2004
-/// @version $Id: GUICalibrator.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Changes flow and speed on a set of lanes (gui version)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUICalibrator_h
-#define GUICalibrator_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
 #include <microsim/trigger/MSCalibrator.h>
 #include <utils/gui/globjects/GUIGlObject_AbstractAdd.h>
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
-#include <utils/foxtools/FXRealSpinDial.h>
 #include <gui/GUIManipulator.h>
 
 
@@ -50,21 +39,12 @@
  *
  * This is the gui-version of the MSCalibrator-object
  */
-class GUICalibrator : public MSCalibrator, public GUIGlObject_AbstractAdd {
+class GUICalibrator : public GUIGlObject_AbstractAdd {
 public:
     /** @brief Constructor
-     * @param[in] idStorage The gl-id storage for giving this object an gl-id
-     * @param[in] id The id of the lane speed trigger
-     * @param[in] destLanes List of lanes affected by this speed trigger
-     * @param[in] file Name of the file to read the speeds to set from
+     * @param[in] calibrator MSCalibrator or METriggeredCalibrator to be wrapped
      */
-    GUICalibrator(const std::string& id,
-                  MSEdge* edge, double pos,
-                  const std::string& aXMLFilename,
-                  const std::string& outputFilename,
-                  const SUMOTime freq,
-                  const MSRouteProbe* probe);
-
+    GUICalibrator(MSCalibrator* calibrator);
 
     /** destructor */
     ~GUICalibrator();
@@ -171,7 +151,7 @@ public:
 
         FXDataTarget mySpeedTarget;
 
-        FXRealSpinDial* myUserDefinedSpeed;
+        FXRealSpinner* myUserDefinedSpeed;
 
         FXComboBox* myPredefinedValues;
 
@@ -189,7 +169,9 @@ private:
     /// Definition of a rotation container
     typedef std::vector<double> RotCont;
 
-private:
+    /// @brief the calibrator being wrapped
+    MSCalibrator* myCalibrator;
+
     /// The positions in full-geometry mode
     PosCont myFGPositions;
 
@@ -203,9 +185,3 @@ private:
     bool myShowAsKMH;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
