@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIChargingStation.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -6,31 +18,10 @@
 /// @author  Tamas Kurczveil
 /// @author  Pablo Alvarez Lopez
 /// @date    20-12-13
-/// @version $Id: GUIChargingStation.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A lane area vehicles can halt at (gui-version)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/common/MsgHandler.h>
@@ -52,7 +43,7 @@
 #include <gui/GUIApplicationWindow.h>
 #include <microsim/logging/FunctionBinding.h>
 #include <utils/gui/div/GUIGlobalSelection.h>
-#include <foreign/polyfonts/polyfonts.h>
+#include <foreign/fontstash/fontstash.h>
 #include <utils/geom/GeomHelper.h>
 #include <utils/gui/globjects/GLIncludes.h>
 
@@ -60,11 +51,15 @@
 // ===========================================================================
 // method definitions
 // ===========================================================================
-GUIChargingStation::GUIChargingStation(const std::string& id, MSLane& lane, double frompos, double topos,  double chargingPower, double efficiency, bool chargeInTransit, int chargeDelay) :
-    MSChargingStation(id, lane, frompos, topos, chargingPower, efficiency, chargeInTransit, chargeDelay),
-    GUIGlObject_AbstractAdd("chargingStation", GLO_TRIGGER, id) {
+GUIChargingStation::GUIChargingStation(const std::string& id, MSLane& lane, double frompos, double topos,
+                                       const std::string& name,
+                                       double chargingPower, double efficiency, bool chargeInTransit, double chargeDelay) :
+    MSChargingStation(id, lane, frompos, topos, name, chargingPower, efficiency, chargeInTransit, chargeDelay),
+    GUIGlObject_AbstractAdd(GLO_CHARGING_STATION, id) {
     myFGShape = lane.getShape();
-    myFGShape = myFGShape.getSubpart(frompos, topos);
+    myFGShape = myFGShape.getSubpart(
+                    lane.interpolateLanePosToGeometryPos(frompos),
+                    lane.interpolateLanePosToGeometryPos(topos));
     myFGShapeRotations.reserve(myFGShape.size() - 1);
     myFGShapeLengths.reserve(myFGShape.size() - 1);
     int e = (int) myFGShape.size() - 1;
@@ -72,7 +67,7 @@ GUIChargingStation::GUIChargingStation(const std::string& id, MSLane& lane, doub
         const Position& f = myFGShape[i];
         const Position& s = myFGShape[i + 1];
         myFGShapeLengths.push_back(f.distanceTo(s));
-        myFGShapeRotations.push_back((double) atan2((s.x() - f.x()), (f.y() - s.y())) * (double) 180.0 / (double) PI);
+        myFGShapeRotations.push_back((double) atan2((s.x() - f.x()), (f.y() - s.y())) * (double) 180.0 / (double) M_PI);
     }
     PositionVector tmp = myFGShape;
     tmp.move2side(1.5);
@@ -92,11 +87,14 @@ GUIChargingStation::~GUIChargingStation() {
 GUIParameterTableWindow*
 GUIChargingStation::getParameterWindow(GUIMainWindow& app, GUISUMOAbstractView&) {
     // Create table items
-    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this, 6);
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this);
 
     // add items
+    ret->mkItem("name", false, getMyName());
     ret->mkItem("begin position [m]", false, myBegPos);
     ret->mkItem("end position [m]", false, myEndPos);
+    ret->mkItem("stopped vehicles[#]", true, new FunctionBinding<GUIChargingStation, int>(this, &MSStoppingPlace::getStoppedVehicleNumber));
+    ret->mkItem("last free pos[m]", true, new FunctionBinding<GUIChargingStation, double>(this, &MSStoppingPlace::getLastFreePos));
     ret->mkItem("charging power [W]", false, myChargingPower);
     ret->mkItem("charging myEfficiency []", false, myEfficiency);
     ret->mkItem("charge in transit [true/false]", false, myChargeInTransit);
@@ -133,55 +131,30 @@ GUIChargingStation::drawGL(const GUIVisualizationSettings& s) const {
     // Draw Charging Station
     glPushName(getGlID());
     glPushMatrix();
-    RGBColor blue(114, 210, 252, 255);
-    RGBColor green(76, 170, 50, 255);
-    RGBColor yellow(255, 235, 0, 255);
-    RGBColor yellowCharge(255, 180, 0, 255);
 
     // draw the area depending if the vehicle is charging
     glTranslated(0, 0, getType());
 
+    // set color depending if charging station is charging
     if (myChargingVehicle == true) {
-        GLHelper::setColor(yellowCharge);
+        GLHelper::setColor(s.stoppingPlaceSettings.chargingStationColorCharge);
     } else {
-        GLHelper::setColor(blue);
+        GLHelper::setColor(s.stoppingPlaceSettings.chargingStationColor);
     }
-    const double exaggeration = s.addSize.getExaggeration(s);
+    const double exaggeration = s.addSize.getExaggeration(s, this);
     GLHelper::drawBoxLines(myFGShape, myFGShapeRotations, myFGShapeLengths, exaggeration);
 
     // draw details unless zoomed out to far
-    if (s.scale * exaggeration >= 10) {
+    if (s.drawDetail(s.detailSettings.stoppingPlaceDetails, exaggeration)) {
 
         // push charging power matrix
         glPushMatrix();
-
-        // Traslate End positionof signal
-        glTranslated(myFGSignPos.x(), myFGSignPos.y(), 0);
-
-        // Rotate 180 (Eje X -> Mirror)
-        glRotated(180, 1, 0, 0);
-
-        // Rotate again using myBlockIconRotation
-        glRotated(myFGSignRot, 0, 0, 1);
-
-        // Set poligon mode
-        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
-
-        // set polyfront position on 0,0
-        pfSetPosition(0, 0);
-
-        // Set polyfront scale to 1
-        pfSetScale(1.f);
-
-        // traslate matrix
-        glTranslated(1.2, 0, 0);
-
         // draw charging power
-        pfDrawString((toString(myChargingPower) + " W").c_str());
-
+        GLHelper::drawText((toString(myChargingPower) + " W").c_str(), myFGSignPos + Position(1.2, 0), .1, 1.f, s.stoppingPlaceSettings.chargingStationColor, myFGSignRot, FONS_ALIGN_LEFT);
         // pop charging power matrix
         glPopMatrix();
 
+        glPushMatrix();
         // draw the sign
         glTranslated(myFGSignPos.x(), myFGSignPos.y(), 0);
         int noPoints = 9;
@@ -193,19 +166,29 @@ GUIChargingStation::drawGL(const GUIVisualizationSettings& s) const {
         GLHelper::drawFilledCircle((double) 1.1, noPoints);
         glTranslated(0, 0, .1);
 
-        GLHelper::setColor(yellow);
+        GLHelper::setColor(s.stoppingPlaceSettings.chargingStationColorSign);
         GLHelper::drawFilledCircle((double) 0.9, noPoints);
 
-        if (s.scale * exaggeration >= 4.5) {
-            GLHelper::drawText("C", Position(), .1, 1.6, blue, myFGSignRot);
+        if (s.drawDetail(s.detailSettings.stoppingPlaceText, exaggeration)) {
+            GLHelper::drawText("C", Position(), .1, 1.6, s.stoppingPlaceSettings.chargingStationColor, myFGSignRot);
         }
 
         glTranslated(5, 0, 0);
+        glPopMatrix();
 
     }
+    if (s.addFullName.show && getMyName() != "") {
+        GLHelper::drawTextSettings(s.addFullName, getMyName(), myFGSignPos, s.scale, s.getTextAngle(myFGSignRot), GLO_MAX - getType());
+    }
     glPopMatrix();
     glPopName();
-    drawName(getCenteringBoundary().getCenter(), s.scale, s.addName);
+    drawName(getCenteringBoundary().getCenter(), s.scale, s.addName, s.angle);
 }
 
+const std::string
+GUIChargingStation::getOptionalName() const {
+    return myName;
+}
+
+
 /****************************************************************************/
