@@ -1,48 +1,36 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIContainer.cpp
 /// @author  Melanie Weber
 /// @author  Andreas Kendziorra
 /// @date    Wed, 01.08.2014
-/// @version $Id: GUIContainer.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // A MSContainer extended by some values for usage within the gui
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cmath>
 #include <vector>
 #include <string>
-#include <foreign/polyfonts/polyfonts.h>
-#include <microsim/MSContainer.h>
-#include <microsim/MSCModel_NonInteracting.h>
 #include <microsim/logging/CastingFunctionBinding.h>
 #include <microsim/logging/FunctionBinding.h>
+#include <microsim/MSLane.h>
 #include <microsim/MSVehicleControl.h>
 #include <microsim/lcmodels/MSAbstractLaneChangeModel.h>
 #include <microsim/devices/MSDevice_Vehroutes.h>
 #include <utils/common/StringUtils.h>
 #include <utils/vehicle/SUMOVehicleParameter.h>
-#include <utils/common/AbstractMutex.h>
 #include <utils/geom/GeomHelper.h>
 #include <utils/gui/images/GUITexturesHelper.h>
 #include <utils/gui/windows/GUISUMOAbstractView.h>
@@ -54,37 +42,26 @@
 #include <utils/geom/PositionVector.h>
 #include <gui/GUIApplicationWindow.h>
 #include <gui/GUIGlobals.h>
+#include <utils/gui/globjects/GLIncludes.h>
+#include <utils/gui/div/GUIDesigns.h>
+
 #include "GUIContainer.h"
 #include "GUINet.h"
 #include "GUIEdge.h"
-#include <utils/gui/globjects/GLIncludes.h>
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
 
 //#define GUIContainer_DEBUG_DRAW_WALKING_AREA_SHAPE
 
 // ===========================================================================
 // FOX callback mapping
 // ===========================================================================
-/*
 FXDEFMAP(GUIContainer::GUIContainerPopupMenu) GUIContainerPopupMenuMap[] = {
-    FXMAPFUNC(SEL_COMMAND, MID_SHOW_ALLROUTES, GUIContainer::GUIContainerPopupMenu::onCmdShowAllRoutes),
-    FXMAPFUNC(SEL_COMMAND, MID_HIDE_ALLROUTES, GUIContainer::GUIContainerPopupMenu::onCmdHideAllRoutes),
-    FXMAPFUNC(SEL_COMMAND, MID_SHOW_CURRENTROUTE, GUIContainer::GUIContainerPopupMenu::onCmdShowCurrentRoute),
-    FXMAPFUNC(SEL_COMMAND, MID_HIDE_CURRENTROUTE, GUIContainer::GUIContainerPopupMenu::onCmdHideCurrentRoute),
-    FXMAPFUNC(SEL_COMMAND, MID_SHOW_BEST_LANES, GUIContainer::GUIContainerPopupMenu::onCmdShowBestLanes),
-    FXMAPFUNC(SEL_COMMAND, MID_HIDE_BEST_LANES, GUIContainer::GUIContainerPopupMenu::onCmdHideBestLanes),
     FXMAPFUNC(SEL_COMMAND, MID_START_TRACK, GUIContainer::GUIContainerPopupMenu::onCmdStartTrack),
-    FXMAPFUNC(SEL_COMMAND, MID_STOP_TRACK, GUIContainer::GUIContainerPopupMenu::onCmdStopTrack),
-    FXMAPFUNC(SEL_COMMAND, MID_SHOW_LFLINKITEMS, GUIContainer::GUIContainerPopupMenu::onCmdShowLFLinkItems),
-    FXMAPFUNC(SEL_COMMAND, MID_HIDE_LFLINKITEMS, GUIContainer::GUIContainerPopupMenu::onCmdHideLFLinkItems),
+    FXMAPFUNC(SEL_COMMAND, MID_STOP_TRACK,  GUIContainer::GUIContainerPopupMenu::onCmdStopTrack),
+    FXMAPFUNC(SEL_COMMAND, MID_SHOWPLAN,    GUIContainer::GUIContainerPopupMenu::onCmdShowPlan),
 };
 
 // Object implementation
 FXIMPLEMENT(GUIContainer::GUIContainerPopupMenu, GUIGLObjectPopupMenu, GUIContainerPopupMenuMap, ARRAYNUMBER(GUIContainerPopupMenuMap))
-*/
 
 #define WATER_WAY_OFFSET 6.0
 
@@ -95,21 +72,56 @@ FXIMPLEMENT(GUIContainer::GUIContainerPopupMenu, GUIGLObjectPopupMenu, GUIContai
  * GUIContainer::GUIContainerPopupMenu - methods
  * ----------------------------------------------------------------------- */
 GUIContainer::GUIContainerPopupMenu::GUIContainerPopupMenu(
-    GUIMainWindow& app, GUISUMOAbstractView& parent,
-    GUIGlObject& o, std::map<GUISUMOAbstractView*, int>& additionalVisualizations)
-    : GUIGLObjectPopupMenu(app, parent, o), myVehiclesAdditionalVisualizations(additionalVisualizations) {
+    GUIMainWindow& app, GUISUMOAbstractView& parent, GUIGlObject& o)
+    : GUIGLObjectPopupMenu(app, parent, o) {
 }
 
 
 GUIContainer::GUIContainerPopupMenu::~GUIContainerPopupMenu() {}
 
 
+long
+GUIContainer::GUIContainerPopupMenu::onCmdShowPlan(FXObject*, FXSelector, void*) {
+    GUIContainer* p = dynamic_cast<GUIContainer*>(myObject);
+    if (p == nullptr) {
+        return 1;
+    }
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(*myApplication, *p);
+    // add items
+    for (int stage = 1; stage < p->getNumStages(); stage++) {
+        ret->mkItem(toString(stage).c_str(), false, p->getStageSummary(stage));
+    }
+    // close building (use an object that is not Parameterised as argument)
+    Parameterised dummy;
+    ret->closeBuilding(&dummy);
+    return 1;
+}
+
+
+long
+GUIContainer::GUIContainerPopupMenu::onCmdStartTrack(FXObject*, FXSelector, void*) {
+    assert(myObject->getType() == GLO_PERSON);
+    if (myParent->getTrackedID() != static_cast<GUIContainer*>(myObject)->getGlID()) {
+        myParent->startTrack(static_cast<GUIContainer*>(myObject)->getGlID());
+    }
+    return 1;
+}
+
+long
+GUIContainer::GUIContainerPopupMenu::onCmdStopTrack(FXObject*, FXSelector, void*) {
+    assert(myObject->getType() == GLO_PERSON);
+    myParent->stopTrack();
+    return 1;
+}
+
+
+
 
 /* -------------------------------------------------------------------------
  * GUIContainer - methods
  * ----------------------------------------------------------------------- */
-GUIContainer::GUIContainer(const SUMOVehicleParameter* pars, const MSVehicleType* vtype, MSTransportable::MSTransportablePlan* plan) :
-    MSContainer(pars, vtype, plan),
+GUIContainer::GUIContainer(const SUMOVehicleParameter* pars, MSVehicleType* vtype, MSTransportable::MSTransportablePlan* plan) :
+    MSTransportable(pars, vtype, plan, false),
     GUIGlObject(GLO_CONTAINER, pars->id) {
 }
 
@@ -121,13 +133,23 @@ GUIContainer::~GUIContainer() {
 GUIGLObjectPopupMenu*
 GUIContainer::getPopUpMenu(GUIMainWindow& app,
                            GUISUMOAbstractView& parent) {
-    GUIGLObjectPopupMenu* ret = new GUIContainerPopupMenu(app, parent, *this, myAdditionalVisualizations);
+    GUIGLObjectPopupMenu* ret = new GUIContainerPopupMenu(app, parent, *this);
     buildPopupHeader(ret, app);
     buildCenterPopupEntry(ret);
     buildNameCopyPopupEntry(ret);
     buildSelectionPopupEntry(ret);
+    new FXMenuSeparator(ret);
+    if (parent.getTrackedID() != getGlID()) {
+        GUIDesigns::buildFXMenuCommand(ret, "Start Tracking", nullptr, ret, MID_START_TRACK);
+    } else {
+        GUIDesigns::buildFXMenuCommand(ret, "Stop Tracking", nullptr, ret, MID_STOP_TRACK);
+    }
     //
+
     buildShowParamsPopupEntry(ret);
+    buildShowTypeParamsPopupEntry(ret);
+    GUIDesigns::buildFXMenuCommand(ret, "Show Plan", GUIIconSubSys::getIcon(GUIIcon::APP_TABLE), ret, MID_SHOWPLAN);
+    new FXMenuSeparator(ret);
     buildPositionCopyEntry(ret, false);
     return ret;
 }
@@ -136,20 +158,41 @@ GUIContainer::getPopUpMenu(GUIMainWindow& app,
 GUIParameterTableWindow*
 GUIContainer::getParameterWindow(GUIMainWindow& app,
                                  GUISUMOAbstractView&) {
-    GUIParameterTableWindow* ret =
-        new GUIParameterTableWindow(app, *this, 8);
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this);
     // add items
-    //ret->mkItem("type [NAME]", false, myType->getID());
     ret->mkItem("stage", false, getCurrentStageDescription());
+    // there is always the "start" stage which we do not count here because it is not strictly part of the plan
+    ret->mkItem("stage index", false, toString(getNumStages() - getNumRemainingStages()) + " of " + toString(getNumStages() - 1));
     ret->mkItem("start edge [id]", false, getFromEdge()->getID());
-    ret->mkItem("dest edge [id]", false, getDestination().getID());
+    ret->mkItem("dest edge [id]", false, getDestination()->getID());
+    ret->mkItem("arrivalPos [m]", false, toString(getCurrentStage()->getArrivalPos()));
     ret->mkItem("edge [id]", false, getEdge()->getID());
-    ret->mkItem("position [m]", true, new FunctionBinding<GUIContainer, SUMOReal>(this, &GUIContainer::getEdgePos));
-    ret->mkItem("speed [m/s]", true, new FunctionBinding<GUIContainer, SUMOReal>(this, &GUIContainer::getSpeed));
-    ret->mkItem("angle [degree]", true, new FunctionBinding<GUIContainer, SUMOReal>(this, &GUIContainer::getAngle));
-    ret->mkItem("waiting time [s]", true, new FunctionBinding<GUIContainer, SUMOReal>(this, &GUIContainer::getWaitingSeconds));
+    ret->mkItem("position [m]", true, new FunctionBinding<GUIContainer, double>(this, &GUIContainer::getEdgePos));
+    ret->mkItem("speed [m/s]", true, new FunctionBinding<GUIContainer, double>(this, &GUIContainer::getSpeed));
+    ret->mkItem("speed factor", false, getSpeedFactor());
+    ret->mkItem("angle [degree]", true, new FunctionBinding<GUIContainer, double>(this, &GUIContainer::getAngle));
+    ret->mkItem("waiting time [s]", true, new FunctionBinding<GUIContainer, double>(this, &GUIContainer::getWaitingSeconds));
+    ret->mkItem("desired depart [s]", false, time2string(getParameter().depart));
     // close building
-    ret->closeBuilding();
+    ret->closeBuilding(&getParameter());
+    return ret;
+}
+
+
+GUIParameterTableWindow*
+GUIContainer::getTypeParameterWindow(GUIMainWindow& app,
+                                     GUISUMOAbstractView&) {
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this);
+    // add items
+    ret->mkItem("Type Information:", false, "");
+    ret->mkItem("type [id]", false, myVType->getID());
+    ret->mkItem("length", false, myVType->getLength());
+    ret->mkItem("width", false, myVType->getWidth());
+    ret->mkItem("height", false, myVType->getHeight());
+    ret->mkItem("minGap", false, myVType->getMinGap());
+    ret->mkItem("maximum speed [m/s]", false, myVType->getMaxSpeed());
+    // close building
+    ret->closeBuilding(&(myVType->getParameter()));
     return ret;
 }
 
@@ -169,16 +212,17 @@ GUIContainer::drawGL(const GUIVisualizationSettings& s) const {
     glPushName(getGlID());
     glPushMatrix();
     Position p1 = getPosition();
-    if (getCurrentStageType() == DRIVING && !isWaiting4Vehicle()) {
-        p1 = myPositionInVehicle;
+    double angle = getAngle();
+    if (getCurrentStageType() == MSStageType::DRIVING && !isWaiting4Vehicle()) {
+        p1 = myPositionInVehicle.pos;
+        angle = myPositionInVehicle.angle;
     }
     glTranslated(p1.x(), p1.y(), getType());
-    glRotated(90, 0, 0, 1);
-    // XXX use container specific gui settings
+    glRotated(RAD2DEG(angle), 0, 0, 1);
     // set container color
     setColor(s);
     // scale
-    const SUMOReal upscale = s.containerSize.getExaggeration(s);
+    const double upscale = s.containerSize.getExaggeration(s, this);
     glScaled(upscale, upscale, 1);
     switch (s.containerQuality) {
         case 0:
@@ -193,7 +237,7 @@ GUIContainer::drawGL(const GUIVisualizationSettings& s) const {
     }
     glPopMatrix();
 
-    drawName(p1, s.scale, s.containerName);
+    drawName(p1, s.scale, s.containerName, s.angle);
     glPopName();
 }
 
@@ -214,7 +258,7 @@ GUIContainer::drawGLAdditional(GUISUMOAbstractView* const /* parent */, const GU
         if (getNumberReroutes() > 0) {
             const int noReroutePlus1 = getNumberReroutes() + 1;
             for (int i = noReroutePlus1 - 1; i >= 0; i--) {
-                SUMOReal darken = SUMOReal(0.4) / SUMOReal(noReroutePlus1) * SUMOReal(i);
+                double darken = double(0.4) / double(noReroutePlus1) * double(i);
                 drawRoute(s, i, darken);
             }
         } else {
@@ -227,14 +271,10 @@ GUIContainer::drawGLAdditional(GUISUMOAbstractView* const /* parent */, const GU
                 continue;
             }
             MSLink* link = (*i).myLink;
-    #ifdef HAVE_INTERNAL_LANES
             MSLane *via = link->getViaLane();
             if (via == 0) {
                 via = link->getLane();
             }
-    #else
-            MSLane *via = link->getLane();
-    #endif
             if (via != 0) {
                 Position p = via->getShape()[0];
                 if((*i).mySetRequest) {
@@ -260,13 +300,13 @@ void
 GUIContainer::setColor(const GUIVisualizationSettings& s) const {
     const GUIColorer& c = s.containerColorer;
     if (!setFunctionalColor(c.getActive())) {
-        GLHelper::setColor(c.getScheme().getColor(getColorValue(c.getActive())));
+        GLHelper::setColor(c.getScheme().getColor(getColorValue(s, c.getActive())));
     }
 }
 
 
 bool
-GUIContainer::setFunctionalColor(size_t activeScheme) const {
+GUIContainer::setFunctionalColor(int activeScheme) const {
     switch (activeScheme) {
         case 0: {
             if (getParameter().wasSet(VEHPARS_COLOR_SET)) {
@@ -293,27 +333,33 @@ GUIContainer::setFunctionalColor(size_t activeScheme) const {
             }
             return false;
         }
-        case 8: {
-            SUMOReal hue = GeomHelper::naviDegree(getAngle());
+        case 8: { // color by angle
+            double hue = GeomHelper::naviDegree(getAngle());
             GLHelper::setColor(RGBColor::fromHSV(hue, 1., 1.));
             return true;
         }
+        case 9: { // color randomly (by pointer)
+            const double hue = (double)((long long int)this % 360); // [0-360]
+            const double sat = (((long long int)this / 360) % 67) / 100.0 + 0.33; // [0.33-1]
+            GLHelper::setColor(RGBColor::fromHSV(hue, sat, 1.));
+            return true;
+        }
         default:
             return false;
     }
 }
 
 
-SUMOReal
-GUIContainer::getColorValue(size_t activeScheme) const {
+double
+GUIContainer::getColorValue(const GUIVisualizationSettings& /* s */, int activeScheme) const {
     switch (activeScheme) {
         case 4:
             return getSpeed();
         case 5:
             if (isWaiting4Vehicle()) {
-                return 3;
+                return 5;
             } else {
-                return (SUMOReal)getCurrentStageType();
+                return (double)getCurrentStageType();
             }
         case 6:
             return getWaitingSeconds();
@@ -324,50 +370,49 @@ GUIContainer::getColorValue(size_t activeScheme) const {
 }
 
 
-SUMOReal
+double
 GUIContainer::getEdgePos() const {
-    AbstractMutex::ScopedLocker locker(myLock);
-    return MSContainer::getEdgePos();
+    FXMutexLock locker(myLock);
+    return MSTransportable::getEdgePos();
 }
 
 
 Position
 GUIContainer::getPosition() const {
-    AbstractMutex::ScopedLocker locker(myLock);
-    if (getCurrentStageType() == WAITING && getEdge()->getPermissions() == SVC_SHIP) {
+    FXMutexLock locker(myLock);
+    if (getCurrentStageType() == MSStageType::WAITING && getEdge()->getPermissions() == SVC_SHIP) {
         MSLane* lane = getEdge()->getLanes().front();   //the most right lane of the water way
         PositionVector laneShape = lane->getShape();
         return laneShape.positionAtOffset2D(getEdgePos(), WATER_WAY_OFFSET);
     }
-    return MSContainer::getPosition();
+    return MSTransportable::getPosition();
 }
 
 
-SUMOReal
+double
 GUIContainer::getAngle() const {
-    AbstractMutex::ScopedLocker locker(myLock);
-    return MSContainer::getAngle();
+    FXMutexLock locker(myLock);
+    return MSTransportable::getAngle();
 }
 
 
-SUMOReal
+double
 GUIContainer::getWaitingSeconds() const {
-    AbstractMutex::ScopedLocker locker(myLock);
-    return MSContainer::getWaitingSeconds();
+    FXMutexLock locker(myLock);
+    return MSTransportable::getWaitingSeconds();
 }
 
 
-SUMOReal
+double
 GUIContainer::getSpeed() const {
-    AbstractMutex::ScopedLocker locker(myLock);
-    return MSContainer::getSpeed();
+    FXMutexLock locker(myLock);
+    return MSTransportable::getSpeed();
 }
 
 
 void
 GUIContainer::drawAction_drawAsPoly(const GUIVisualizationSettings& /* s */) const {
     // draw pedestrian shape
-    glRotated(RAD2DEG(getAngle() + PI / 2.), 0, 0, 1);
     glScaled(getVehicleType().getLength(), getVehicleType().getWidth(), 1);
     glBegin(GL_QUADS);
     glVertex2d(0, 0.5);
@@ -392,13 +437,13 @@ GUIContainer::drawAction_drawAsImage(const GUIVisualizationSettings& s) const {
     if (file != "") {
         // @todo invent an option for controlling whether images should be rotated or not
         //if (getVehicleType().getGuiShape() == SVS_CONTAINER) {
-        //    glRotated(RAD2DEG(getAngle() + PI / 2.), 0, 0, 1);
+        //    glRotated(RAD2DEG(getAngle() + M_PI / 2.), 0, 0, 1);
         //}
         int textureID = GUITexturesHelper::getTextureID(file);
         if (textureID > 0) {
-            const SUMOReal exaggeration = s.personSize.getExaggeration(s);
-            const SUMOReal halfLength = getVehicleType().getLength() / 2.0 * exaggeration;
-            const SUMOReal halfWidth = getVehicleType().getWidth() / 2.0 * exaggeration;
+            const double exaggeration = s.personSize.getExaggeration(s, this);
+            const double halfLength = getVehicleType().getLength() / 2.0 * exaggeration;
+            const double halfWidth = getVehicleType().getWidth() / 2.0 * exaggeration;
             GUITexturesHelper::drawTexturedBox(textureID, -halfWidth, -halfLength, halfWidth, halfLength);
         }
     } else {
@@ -406,5 +451,6 @@ GUIContainer::drawAction_drawAsImage(const GUIVisualizationSettings& s) const {
         drawAction_drawAsPoly(s);
     }
 }
-/****************************************************************************/
 
+
+/****************************************************************************/
