@@ -1,46 +1,36 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIEdge.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: GUIEdge.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A road/street connecting two junctions (gui-version)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <vector>
 #include <cmath>
 #include <string>
 #include <algorithm>
-#include <foreign/polyfonts/polyfonts.h>
+#include <utils/foxtools/fxheader.h>
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
 #include <utils/gui/windows/GUIMainWindow.h>
 #include <utils/gui/windows/GUISUMOAbstractView.h>
 #include <utils/geom/GeomHelper.h>
-#include <utils/foxtools/MFXMutex.h>
 #include <utils/gui/div/GUIParameterTableWindow.h>
 #include <utils/gui/div/GLHelper.h>
 #include <utils/gui/div/GUIGlobalSelection.h>
@@ -53,6 +43,13 @@
 #include <microsim/MSGlobals.h>
 #include <microsim/logging/CastingFunctionBinding.h>
 #include <microsim/logging/FunctionBinding.h>
+#include <utils/gui/div/GUIDesigns.h>
+#include <mesogui/GUIMEVehicleControl.h>
+#include <mesogui/GUIMEVehicle.h>
+#include <mesosim/MESegment.h>
+#include <mesosim/MELoop.h>
+#include <mesosim/MEVehicle.h>
+
 #include "GUITriggeredRerouter.h"
 #include "GUIEdge.h"
 #include "GUIVehicle.h"
@@ -61,20 +58,12 @@
 #include "GUIPerson.h"
 #include "GUIContainer.h"
 
-#include <mesogui/GUIMEVehicleControl.h>
-#include <mesogui/GUIMEVehicle.h>
-#include <mesosim/MESegment.h>
-#include <mesosim/MELoop.h>
-#include <mesosim/MEVehicle.h>
 
-
-// ===========================================================================
-// included modules
-// ===========================================================================
 GUIEdge::GUIEdge(const std::string& id, int numericalID,
-                 const EdgeBasicFunction function,
-                 const std::string& streetName, const std::string& edgeType, int priority)
-    : MSEdge(id, numericalID, function, streetName, edgeType, priority),
+                 const SumoXMLEdgeFunc function,
+                 const std::string& streetName, const std::string& edgeType, int priority,
+                 double distance)
+    : MSEdge(id, numericalID, function, streetName, edgeType, priority, distance),
       GUIGlObject(GLO_EDGE, id) {}
 
 
@@ -85,6 +74,21 @@ GUIEdge::~GUIEdge() {
     }
 }
 
+void
+GUIEdge::closeBuilding() {
+    MSEdge::closeBuilding();
+    bool hasNormalSuccessors = false;
+    for (const MSEdge* out : getSuccessors()) {
+        if (!out->isTazConnector()) {
+            hasNormalSuccessors = true;
+            break;
+        }
+    }
+    myShowDeadEnd = (!isTazConnector() && !hasNormalSuccessors && getToJunction()->getOutgoing().size() > 0
+                     && (getPermissions() & ~SVC_PEDESTRIAN) != 0
+                     && (getToJunction()->getOutgoing().size() > 1 ||
+                         getToJunction()->getOutgoing().front()->getToJunction() != getFromJunction()));
+}
 
 MSLane&
 GUIEdge::getLane(int laneNo) {
@@ -100,7 +104,7 @@ GUIEdge::getIDs(bool includeInternal) {
     for (MSEdge::DictType::const_iterator i = MSEdge::myDict.begin(); i != MSEdge::myDict.end(); ++i) {
         const GUIEdge* edge = dynamic_cast<const GUIEdge*>(i->second);
         assert(edge);
-        if (edge->getPurpose() != EDGEFUNCTION_INTERNAL || includeInternal) {
+        if (includeInternal || edge->isNormal()) {
             ret.push_back(edge->getGlID());
         }
     }
@@ -113,7 +117,7 @@ GUIEdge::getTotalLength(bool includeInternal, bool eachLane) {
     double result = 0;
     for (MSEdge::DictType::const_iterator i = MSEdge::myDict.begin(); i != MSEdge::myDict.end(); ++i) {
         const MSEdge* edge = i->second;
-        if (edge->getPurpose() != EDGEFUNCTION_INTERNAL || includeInternal) {
+        if (includeInternal || !edge->isInternal()) {
             // @note needs to be change once lanes may have different length
             result += edge->getLength() * (eachLane ? edge->getLanes().size() : 1);
         }
@@ -125,7 +129,7 @@ GUIEdge::getTotalLength(bool includeInternal, bool eachLane) {
 Boundary
 GUIEdge::getBoundary() const {
     Boundary ret;
-    if (getPurpose() != MSEdge::EDGEFUNCTION_DISTRICT) {
+    if (!isTazConnector()) {
         for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
             ret.add((*i)->getShape().getBoxBoundary());
         }
@@ -150,19 +154,6 @@ GUIEdge::getBoundary() const {
 }
 
 
-void
-GUIEdge::fill(std::vector<GUIEdge*>& netsWrappers) {
-    int size = MSEdge::dictSize();
-    netsWrappers.reserve(size);
-    for (DictType::iterator i = myDict.begin(); i != myDict.end(); ++i) {
-        if (i->second->getPurpose() != MSEdge::EDGEFUNCTION_DISTRICT) {
-            netsWrappers.push_back(static_cast<GUIEdge*>((*i).second));
-        }
-    }
-}
-
-
-
 GUIGLObjectPopupMenu*
 GUIEdge::getPopUpMenu(GUIMainWindow& app, GUISUMOAbstractView& parent) {
     GUIGLObjectPopupMenu* ret = new GUIGLObjectPopupMenu(app, parent, *this);
@@ -172,9 +163,10 @@ GUIEdge::getPopUpMenu(GUIMainWindow& app, GUISUMOAbstractView& parent) {
     buildSelectionPopupEntry(ret);
     if (MSGlobals::gUseMesoSim) {
         buildShowParamsPopupEntry(ret);
+        buildShowTypeParamsPopupEntry(ret);
     }
-    const double pos = getLanes()[0]->getShape().nearest_offset_to_point2D(parent.getPositionInformation());
-    new FXMenuCommand(ret, ("pos: " + toString(pos)).c_str(), 0, 0, 0);
+    MESegment* segment = getSegmentAtPosition(parent.getPositionInformation());
+    GUIDesigns::buildFXMenuCommand(ret, "segment: " + toString(segment->getIndex()), nullptr, nullptr, 0);
     buildPositionCopyEntry(ret, false);
     return ret;
 }
@@ -183,8 +175,8 @@ GUIEdge::getPopUpMenu(GUIMainWindow& app, GUISUMOAbstractView& parent) {
 GUIParameterTableWindow*
 GUIEdge::getParameterWindow(GUIMainWindow& app,
                             GUISUMOAbstractView& parent) {
-    GUIParameterTableWindow* ret = 0;
-    ret = new GUIParameterTableWindow(app, *this, 18);
+    GUIParameterTableWindow* ret = nullptr;
+    ret = new GUIParameterTableWindow(app, *this);
     // add edge items
     ret->mkItem("length [m]", false, (*myLanes)[0]->getLength());
     ret->mkItem("allowed speed [m/s]", false, getAllowedSpeed());
@@ -192,8 +184,7 @@ GUIEdge::getParameterWindow(GUIMainWindow& app,
     ret->mkItem("mean vehicle speed [m/s]", true, new FunctionBinding<GUIEdge, double>(this, &GUIEdge::getMeanSpeed));
     ret->mkItem("flow [veh/h/lane]", true, new FunctionBinding<GUIEdge, double>(this, &GUIEdge::getFlow));
     ret->mkItem("routing speed [m/s]", true, new FunctionBinding<MSEdge, double>(this, &MSEdge::getRoutingSpeed));
-    ret->mkItem("#vehicles", true, new CastingFunctionBinding<GUIEdge, double, int>(this, &GUIEdge::getVehicleNo));
-    ret->mkItem("vehicle ids", false, getVehicleIDs());
+    ret->mkItem("#vehicles", true, new CastingFunctionBinding<GUIEdge, int, int>(this, &MSEdge::getVehicleNumber));
     // add segment items
     MESegment* segment = getSegmentAtPosition(parent.getPositionInformation());
     ret->mkItem("segment index", false, segment->getIndex());
@@ -204,15 +195,39 @@ GUIEdge::getParameterWindow(GUIMainWindow& app,
     ret->mkItem("segment brutto occupancy [%]", true, new FunctionBinding<MESegment, double>(segment, &MESegment::getRelativeOccupancy, 100));
     ret->mkItem("segment mean vehicle speed [m/s]", true, new FunctionBinding<MESegment, double>(segment, &MESegment::getMeanSpeed));
     ret->mkItem("segment flow [veh/h/lane]", true, new FunctionBinding<MESegment, double>(segment, &MESegment::getFlow));
-    ret->mkItem("segment #vehicles", true, new CastingFunctionBinding<MESegment, double, int>(segment, &MESegment::getCarNumber));
+    ret->mkItem("segment #vehicles", true, new CastingFunctionBinding<MESegment, int, int>(segment, &MESegment::getCarNumber));
     ret->mkItem("segment leader leave time", true, new FunctionBinding<MESegment, double>(segment, &MESegment::getEventTimeSeconds));
     ret->mkItem("segment headway [s]", true, new FunctionBinding<MESegment, double>(segment, &MESegment::getLastHeadwaySeconds));
+    ret->mkItem("segment entry blocktime [s]", true, new FunctionBinding<MESegment, double>(segment, &MESegment::getEntryBlockTimeSeconds));
 
     // close building
     ret->closeBuilding();
     return ret;
 }
 
+GUIParameterTableWindow*
+GUIEdge::getTypeParameterWindow(GUIMainWindow& app,
+                                GUISUMOAbstractView&) {
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this);
+    const MSNet::MesoEdgeType& edgeType = MSNet::getInstance()->getMesoType(getEdgeType());
+    // add items
+    ret->mkItem("Type Information:", false, "");
+    ret->mkItem("type [id]", false, getEdgeType());
+    ret->mkItem("tauff", false, STEPS2TIME(edgeType.tauff));
+    ret->mkItem("taufj", false, STEPS2TIME(edgeType.taufj));
+    ret->mkItem("taujf", false, STEPS2TIME(edgeType.taujf));
+    ret->mkItem("taujj", false, STEPS2TIME(edgeType.taujj));
+    ret->mkItem("jam threshold", false, edgeType.jamThreshold);
+    ret->mkItem("junction control", false, edgeType.junctionControl);
+    ret->mkItem("tls penalty", false, edgeType.tlsPenalty);
+    ret->mkItem("tls flow penalty", false, edgeType.tlsFlowPenalty);
+    ret->mkItem("minor penalty", false, STEPS2TIME(edgeType.minorPenalty));
+    ret->mkItem("overtaking", false, edgeType.overtaking);
+    // close building
+    ret->closeBuilding();
+    return ret;
+}
+
 
 Boundary
 GUIEdge::getCenteringBoundary() const {
@@ -223,46 +238,54 @@ GUIEdge::getCenteringBoundary() const {
     return b;
 }
 
+const std::string
+GUIEdge::getOptionalName() const {
+    return myStreetName;
+}
 
 void
 GUIEdge::drawGL(const GUIVisualizationSettings& s) const {
-    if (s.hideConnectors && myFunction == MSEdge::EDGEFUNCTION_CONNECTOR) {
+    if (s.hideConnectors && myFunction == SumoXMLEdgeFunc::CONNECTOR) {
         return;
     }
     glPushName(getGlID());
     // draw the lanes
+    if (MSGlobals::gUseMesoSim) {
+        setColor(s);
+    }
     for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
-        if (MSGlobals::gUseMesoSim) {
-            setColor(s);
-        }
-        GUILane* l = dynamic_cast<GUILane*>(*i);
-        if (l != 0) {
-            l->drawGL(s);
-        }
+        static_cast<GUILane*>(*i)->drawGL(s);
     }
     if (MSGlobals::gUseMesoSim) {
-        if (s.scale * s.vehicleSize.getExaggeration(s) > s.vehicleSize.minSize) {
+        if (s.scale * s.vehicleSize.getExaggeration(s, nullptr) > s.vehicleSize.minSize) {
             drawMesoVehicles(s);
         }
     }
     glPopName();
     // (optionally) draw the name and/or the street name
-    const bool drawEdgeName = s.edgeName.show && myFunction == EDGEFUNCTION_NORMAL;
-    const bool drawInternalEdgeName = s.internalEdgeName.show && myFunction == EDGEFUNCTION_INTERNAL;
-    const bool drawCwaEdgeName = s.cwaEdgeName.show && (myFunction == EDGEFUNCTION_CROSSING || myFunction == EDGEFUNCTION_WALKINGAREA);
+    const bool drawEdgeName = s.edgeName.show && myFunction == SumoXMLEdgeFunc::NORMAL;
+    const bool drawInternalEdgeName = s.internalEdgeName.show && myFunction == SumoXMLEdgeFunc::INTERNAL;
+    const bool drawCwaEdgeName = s.cwaEdgeName.show && (myFunction == SumoXMLEdgeFunc::CROSSING || myFunction == SumoXMLEdgeFunc::WALKINGAREA);
     const bool drawStreetName = s.streetName.show && myStreetName != "";
-    if (drawEdgeName || drawInternalEdgeName || drawCwaEdgeName || drawStreetName) {
+    const bool drawEdgeValue = s.edgeValue.show && (myFunction == SumoXMLEdgeFunc::NORMAL
+                               || (myFunction == SumoXMLEdgeFunc::INTERNAL && !s.drawJunctionShape)
+                               || ((myFunction == SumoXMLEdgeFunc::CROSSING || myFunction == SumoXMLEdgeFunc::WALKINGAREA) && s.drawCrossingsAndWalkingareas));
+    if (drawEdgeName || drawInternalEdgeName || drawCwaEdgeName || drawStreetName || drawEdgeValue) {
         GUILane* lane1 = dynamic_cast<GUILane*>((*myLanes)[0]);
         GUILane* lane2 = dynamic_cast<GUILane*>((*myLanes).back());
-        if (lane1 != 0 && lane2 != 0) {
+        if (lane1 != nullptr && lane2 != nullptr) {
+            const bool spreadSuperposed = s.spreadSuperposed && getBidiEdge() != nullptr && lane2->drawAsRailway(s);
             Position p = lane1->getShape().positionAtOffset(lane1->getShape().length() / (double) 2.);
             p.add(lane2->getShape().positionAtOffset(lane2->getShape().length() / (double) 2.));
             p.mul(.5);
-            double angle = lane1->getShape().rotationDegreeAtOffset(lane1->getShape().length() / (double) 2.);
-            angle += 90;
-            if (angle > 90 && angle < 270) {
-                angle -= 180;
+            if (spreadSuperposed) {
+                // move name to the right of the edge and towards its beginning
+                const double dist = 0.6 * s.edgeName.scaledSize(s.scale);
+                const double shiftA = lane1->getShape().rotationAtOffset(lane1->getShape().length() / (double) 2.) - DEG2RAD(135);
+                Position shift(dist * cos(shiftA), dist * sin(shiftA));
+                p.add(shift);
             }
+            double angle = s.getTextAngle(lane1->getShape().rotationDegreeAtOffset(lane1->getShape().length() / (double) 2.) + 90);
             if (drawEdgeName) {
                 drawName(p, s.scale, s.edgeName, angle);
             } else if (drawInternalEdgeName) {
@@ -271,28 +294,50 @@ GUIEdge::drawGL(const GUIVisualizationSettings& s) const {
                 drawName(p, s.scale, s.cwaEdgeName, angle);
             }
             if (drawStreetName) {
-                GLHelper::drawText(getStreetName(), p, GLO_MAX,
-                                   s.streetName.size / s.scale, s.streetName.color, angle);
+                GLHelper::drawTextSettings(s.streetName, getStreetName(), p, s.scale, angle);
+            }
+            if (drawEdgeValue) {
+                const int activeScheme = s.getLaneEdgeMode();
+                std::string value = "";
+                if (activeScheme == 31) {
+                    // edge param, could be non-numerical
+                    value = getParameter(s.edgeParam, "");
+                } else if (activeScheme == 32) {
+                    // lane param, could be non-numerical
+                    value = lane2->getParameter(s.laneParam, "");
+                } else {
+                    // use numerical value value of leftmost lane to hopefully avoid sidewalks, bikelanes etc
+                    const double doubleValue = (MSGlobals::gUseMesoSim
+                                                ? getColorValue(s, activeScheme)
+                                                : lane2->getColorValue(s, activeScheme));
+                    const RGBColor color = (MSGlobals::gUseMesoSim ? s.edgeColorer : s.laneColorer).getScheme().getColor(doubleValue);
+                    if (doubleValue != s.MISSING_DATA
+                            && color.alpha() != 0
+                            && (!s.edgeValueHideCheck || doubleValue > s.edgeValueHideThreshold)) {
+                        value = toString(doubleValue);
+                    }
+                }
+                if (value != "") {
+                    GLHelper::drawTextSettings(s.edgeValue, value, p, s.scale, angle);
+                }
             }
         }
     }
-    if (s.scale * s.personSize.getExaggeration(s) > s.personSize.minSize) {
-        myLock.lock();
+    if (s.scale * s.personSize.getExaggeration(s, nullptr) > s.personSize.minSize) {
+        FXMutexLock locker(myLock);
         for (std::set<MSTransportable*>::const_iterator i = myPersons.begin(); i != myPersons.end(); ++i) {
             GUIPerson* person = dynamic_cast<GUIPerson*>(*i);
             assert(person != 0);
             person->drawGL(s);
         }
-        myLock.unlock();
     }
-    if (s.scale * s.containerSize.getExaggeration(s) > s.containerSize.minSize) {
-        myLock.lock();
+    if (s.scale * s.containerSize.getExaggeration(s, nullptr) > s.containerSize.minSize) {
+        FXMutexLock locker(myLock);
         for (std::set<MSTransportable*>::const_iterator i = myContainers.begin(); i != myContainers.end(); ++i) {
             GUIContainer* container = dynamic_cast<GUIContainer*>(*i);
             assert(container != 0);
             container->drawGL(s);
         }
-        myLock.unlock();
     }
 }
 
@@ -300,22 +345,21 @@ GUIEdge::drawGL(const GUIVisualizationSettings& s) const {
 void
 GUIEdge::drawMesoVehicles(const GUIVisualizationSettings& s) const {
     GUIMEVehicleControl* vehicleControl = GUINet::getGUIInstance()->getGUIMEVehicleControl();
-    if (vehicleControl != 0) {
+    if (vehicleControl != nullptr) {
         // draw the meso vehicles
         vehicleControl->secureVehicles();
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         int laneIndex = 0;
-        MESegment::Queue queue;
         for (std::vector<MSLane*>::const_iterator msl = myLanes->begin(); msl != myLanes->end(); ++msl, ++laneIndex) {
             GUILane* l = static_cast<GUILane*>(*msl);
             // go through the vehicles
             double segmentOffset = 0; // offset at start of current segment
             for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
-                    segment != 0; segment = segment->getNextSegment()) {
+                    segment != nullptr; segment = segment->getNextSegment()) {
                 const double length = segment->getLength();
                 if (laneIndex < segment->numQueues()) {
                     // make a copy so we don't have to worry about synchronization
-                    queue = segment->getQueue(laneIndex);
+                    std::vector<MEVehicle*> queue = segment->getQueue(laneIndex);
                     const int queueSize = (int)queue.size();
                     double vehiclePosition = segmentOffset + length;
                     // draw vehicles beginning with the leader at the end of the segment
@@ -346,51 +390,6 @@ GUIEdge::drawMesoVehicles(const GUIVisualizationSettings& s) const {
 
 
 
-int
-GUIEdge::getVehicleNo() const {
-    int vehNo = 0;
-    for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this); segment != 0; segment = segment->getNextSegment()) {
-        vehNo += segment->getCarNumber();
-    }
-    return (int)vehNo;
-}
-
-
-std::string
-GUIEdge::getVehicleIDs() const {
-    std::string result = " ";
-    std::vector<const MEVehicle*> vehs;
-    for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this); segment != 0; segment = segment->getNextSegment()) {
-        std::vector<const MEVehicle*> segmentVehs = segment->getVehicles();
-        vehs.insert(vehs.end(), segmentVehs.begin(), segmentVehs.end());
-    }
-    for (std::vector<const MEVehicle*>::const_iterator it = vehs.begin(); it != vehs.end(); it++) {
-        result += (*it)->getID() + " ";
-    }
-    return result;
-}
-
-
-double
-GUIEdge::getFlow() const {
-    double flow = 0;
-    for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this); segment != 0; segment = segment->getNextSegment()) {
-        flow += (double) segment->getCarNumber() * segment->getMeanSpeed();
-    }
-    return 3600 * flow / (*myLanes)[0]->getLength();
-}
-
-
-double
-GUIEdge::getBruttoOccupancy() const {
-    double occ = 0;
-    for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this); segment != 0; segment = segment->getNextSegment()) {
-        occ += segment->getBruttoOccupancy();
-    }
-    return occ / (*myLanes)[0]->getLength() / (double)(myLanes->size());
-}
-
-
 double
 GUIEdge::getAllowedSpeed() const {
     return (*myLanes)[0]->getSpeedLimit();
@@ -407,24 +406,31 @@ void
 GUIEdge::setColor(const GUIVisualizationSettings& s) const {
     myMesoColor = RGBColor(0, 0, 0); // default background color when using multiColor
     const GUIColorer& c = s.edgeColorer;
-    if (!setFunctionalColor(c.getActive()) && !setMultiColor(c)) {
-        myMesoColor = c.getScheme().getColor(getColorValue(c.getActive()));
+    if (!setFunctionalColor(c) && !setMultiColor(c)) {
+        myMesoColor = c.getScheme().getColor(getColorValue(s, c.getActive()));
     }
 }
 
 
 bool
-GUIEdge::setFunctionalColor(int activeScheme) const {
+GUIEdge::setFunctionalColor(const GUIColorer& c) const {
+    const int activeScheme = c.getActive();
+    int activeMicroScheme = -1;
     switch (activeScheme) {
-        case 9: {
-            const PositionVector& shape = getLanes()[0]->getShape();
-            double hue = GeomHelper::naviDegree(shape.beginEndAngle()); // [0-360]
-            myMesoColor = RGBColor::fromHSV(hue, 1., 1.);
-            return true;
-        }
+        case 0:
+            activeMicroScheme = 0; // color uniform
+            break;
+        case 9:
+            activeMicroScheme = 18; // color by angle
+            break;
+        case 17:
+            activeMicroScheme = 30; // color by TAZ
+            break;
         default:
             return false;
     }
+    GUILane* guiLane = static_cast<GUILane*>(getLanes()[0]);
+    return guiLane->setFunctionalColor(c, myMesoColor, activeMicroScheme);
 }
 
 
@@ -435,38 +441,40 @@ GUIEdge::setMultiColor(const GUIColorer& c) const {
     switch (activeScheme) {
         case 10: // alternating segments
             for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
-                    segment != 0; segment = segment->getNextSegment()) {
+                    segment != nullptr; segment = segment->getNextSegment()) {
                 mySegmentColors.push_back(c.getScheme().getColor(segment->getIndex() % 2));
             }
             //std::cout << getID() << " scheme=" << c.getScheme().getName() << " schemeCols=" << c.getScheme().getColors().size() << " thresh=" << toString(c.getScheme().getThresholds()) << " segmentColors=" << mySegmentColors.size() << " [0]=" << mySegmentColors[0] << " [1]=" << mySegmentColors[1] <<  "\n";
             return true;
         case 11: // by segment jammed state
             for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
-                    segment != 0; segment = segment->getNextSegment()) {
-                mySegmentColors.push_back(c.getScheme().getColor(segment->free() ? 0 : 1));
+                    segment != nullptr; segment = segment->getNextSegment()) {
+                mySegmentColors.push_back(
+                    c.getScheme().getColor(segment->getRelativeOccupancy() > segment->getRelativeJamThreshold() ? 2 :
+                                           (segment->getRelativeOccupancy() * 2 < segment->getRelativeJamThreshold() ? 0 : 1)));
             }
             return true;
         case 12: // by segment occupancy
             for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
-                    segment != 0; segment = segment->getNextSegment()) {
+                    segment != nullptr; segment = segment->getNextSegment()) {
                 mySegmentColors.push_back(c.getScheme().getColor(segment->getRelativeOccupancy()));
             }
             return true;
         case 13: // by segment speed
             for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
-                    segment != 0; segment = segment->getNextSegment()) {
+                    segment != nullptr; segment = segment->getNextSegment()) {
                 mySegmentColors.push_back(c.getScheme().getColor(segment->getMeanSpeed()));
             }
             return true;
         case 14: // by segment flow
             for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
-                    segment != 0; segment = segment->getNextSegment()) {
+                    segment != nullptr; segment = segment->getNextSegment()) {
                 mySegmentColors.push_back(c.getScheme().getColor(3600 * segment->getCarNumber() * segment->getMeanSpeed() / segment->getLength()));
             }
             return true;
         case 15: // by segment relative speed
             for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
-                    segment != 0; segment = segment->getNextSegment()) {
+                    segment != nullptr; segment = segment->getNextSegment()) {
                 mySegmentColors.push_back(c.getScheme().getColor(segment->getMeanSpeed() / getAllowedSpeed()));
             }
             return true;
@@ -477,12 +485,12 @@ GUIEdge::setMultiColor(const GUIColorer& c) const {
 
 
 double
-GUIEdge::getColorValue(int activeScheme) const {
+GUIEdge::getColorValue(const GUIVisualizationSettings& /*s*/, int activeScheme) const {
     switch (activeScheme) {
         case 1:
             return gSelected.isSelected(getType(), getGlID());
         case 2:
-            return getPurpose();
+            return (double)getFunction();
         case 3:
             return getAllowedSpeed();
         case 4:
@@ -544,6 +552,9 @@ GUIEdge::closeTraffic(const GUILane* lane) {
         }
     }
     rebuildAllowedLanes();
+    for (MSEdge* const pred : getPredecessors()) {
+        pred->rebuildAllowedTargets();
+    }
 }
 
 
@@ -551,7 +562,7 @@ void
 GUIEdge::addRerouter() {
     MSEdgeVector edges;
     edges.push_back(this);
-    GUITriggeredRerouter* rr = new GUITriggeredRerouter(getID() + "_dynamic_rerouter", edges, 1, "", false,
+    GUITriggeredRerouter* rr = new GUITriggeredRerouter(getID() + "_dynamic_rerouter", edges, 1, "", false, 0, "",
             GUINet::getGUIInstance()->getVisualisationSpeedUp());
 
     MSTriggeredRerouter::RerouteInterval ri;
@@ -573,5 +584,11 @@ GUIEdge::addRerouter() {
     }
 }
 
-/****************************************************************************/
 
+bool
+GUIEdge::isSelected() const {
+    return gSelected.isSelected(GLO_EDGE, getGlID());
+}
+
+
+/****************************************************************************/
