@@ -1,44 +1,34 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIEdge.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIEdge.h 20496 2016-04-19 12:08:29Z namdre $
 ///
 // A road/street connecting two junctions (gui-version)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIEdge_h
-#define GUIEdge_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
+#include <utils/foxtools/fxheader.h>
 #include <microsim/MSEdge.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <utils/gui/settings/GUIPropertySchemeStorage.h>
-#include <utils/foxtools/MFXMutex.h>
 
 
 // ===========================================================================
@@ -65,13 +55,16 @@ public:
      * @see MSEdge
      */
     GUIEdge(const std::string& id, int numericalID,
-            const EdgeBasicFunction function,
-            const std::string& streetName, const std::string& edgeType, int priority);
+            const SumoXMLEdgeFunc function,
+            const std::string& streetName, const std::string& edgeType, int priority,
+            double distance);
 
 
     /// @brief Destructor.
     ~GUIEdge();
 
+    /// Has to be called after all edges were built and all connections were set
+    virtual void closeBuilding();
 
     /* @brief Returns the gl-ids of all known edges
      * @param[in] includeInternal Whether to include ids of internal edges
@@ -82,24 +75,21 @@ public:
      * @param[in] includeInternal Whether to include lengths of internal edges
      * @param[in] eachLane Whether to count each lane separately
      */
-    static SUMOReal getTotalLength(bool includeInternal, bool eachLane);
+    static double getTotalLength(bool includeInternal, bool eachLane);
 
     /// Returns the street's geometry
     Boundary getBoundary() const;
 
     /// returns the enumerated lane (!!! why not private with a friend?)
-    MSLane& getLane(size_t laneNo);
+    MSLane& getLane(int laneNo);
 
 
 
     /** returns the position on the line given by the coordinates where "prev"
         is the length of the line and "wanted" the distance from the begin
         !!! should be within another class */
-    static std::pair<SUMOReal, SUMOReal> getLaneOffsets(SUMOReal x1, SUMOReal y1,
-            SUMOReal x2, SUMOReal y2, SUMOReal prev, SUMOReal wanted);
-
-    static void fill(std::vector<GUIEdge*>& netsWrappers);
-
+    static std::pair<double, double> getLaneOffsets(double x1, double y1,
+            double x2, double y2, double prev, double wanted);
 
 
     /// @name inherited from GUIGlObject
@@ -126,6 +116,13 @@ public:
     virtual GUIParameterTableWindow* getParameterWindow(GUIMainWindow& app,
             GUISUMOAbstractView& parent);
 
+    /** @brief Returns an own type parameter window
+     *
+     * @param[in] app The application needed to build the parameter window
+     * @param[in] parent The parent window needed to build the parameter window
+     * @return The built parameter window
+     */
+    GUIParameterTableWindow* getTypeParameterWindow(GUIMainWindow& app, GUISUMOAbstractView& parent);
 
     /** @brief Returns the boundary to which the view shall be centered in order to show the object
      *
@@ -134,6 +131,8 @@ public:
      */
     Boundary getCenteringBoundary() const;
 
+    /// @brief Returns the street name
+    const std::string getOptionalName() const;
 
     /** @brief Draws the object
      * @param[in] s The settings for the current view (may influence drawing)
@@ -144,49 +143,44 @@ public:
 
 
     void addPerson(MSTransportable* p) const {
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         MSEdge::addPerson(p);
     }
 
     void removePerson(MSTransportable* p) const {
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         MSEdge::removePerson(p);
     }
 
 
     void addContainer(MSTransportable* c) const {
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         MSEdge::addContainer(c);
     }
 
     void removeContainer(MSTransportable* c) const {
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         MSEdge::removeContainer(c);
     }
 
-    unsigned int getVehicleNo() const;
-    std::string getVehicleIDs() const;
-    SUMOReal getBruttoOccupancy() const;
-    SUMOReal getAllowedSpeed() const;
-    /// @brief return flow based on meanSpead @note: may produced incorrect results when jammed
-    SUMOReal getFlow() const;
+    double getAllowedSpeed() const;
     /// @brief return meanSpead divided by allowedSpeed
-    SUMOReal getRelativeSpeed() const;
+    double getRelativeSpeed() const;
 
     /// @brief sets the color according to the currente settings
     void setColor(const GUIVisualizationSettings& s) const;
 
     /// @brief sets the color according to the current scheme index and some edge function
-    bool setFunctionalColor(size_t activeScheme) const;
+    bool setFunctionalColor(const GUIColorer& c) const;
 
     /// @brief sets multiple colors according to the current scheme index and edge function
     bool setMultiColor(const GUIColorer& c) const;
 
     /// @brief gets the color value according to the current scheme index
-    SUMOReal getColorValue(size_t activeScheme) const;
+    double getColorValue(const GUIVisualizationSettings& s, int activeScheme) const;
 
     /// @brief gets the scaling value according to the current scheme index
-    SUMOReal getScaleValue(size_t activeScheme) const;
+    double getScaleValue(int activeScheme) const;
 
     /// @brief returns the segment closest to the given position
     MESegment* getSegmentAtPosition(const Position& pos);
@@ -214,9 +208,24 @@ public:
         return mySegmentColors;
     }
 
+    RGBColor getMesoColor() const {
+        return myMesoColor;
+    }
+
+    bool showDeadEnd() const {
+        return myShowDeadEnd;
+    }
+
+    /// @brief whether this lane is selected in the GUI
+    bool isSelected() const;
+
     /// The color of the segments (cached)
     mutable std::vector<RGBColor> mySegmentColors;
 
+    /// @brief whether to highlight this edge as a dead-end edge
+    bool myShowDeadEnd;
+
+
 private:
     /// @brief invalidated copy constructor
     GUIEdge(const GUIEdge& s);
@@ -227,12 +236,8 @@ private:
 
 private:
     /// The mutex used to avoid concurrent updates of myPersons/ myContainers
-    mutable MFXMutex myLock;
+    mutable FXMutex myLock;
+
+    mutable RGBColor myMesoColor;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
