@@ -1,45 +1,30 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIEventControl.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 04 Feb 2008
-/// @version $Id: GUIEventControl.h 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // Stores time-dependant events and executes them at the proper time (guisim)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIEventControl_h
-#define GUIEventControl_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
+#include <utils/foxtools/fxheader.h>
 #include <microsim/MSEventControl.h>
 
 
-// ===========================================================================
-// class declarations
-// ===========================================================================
-class MFXMutex;
-
-
 // ===========================================================================
 // class definitions
 // ===========================================================================
@@ -86,7 +71,7 @@ public:
 
 private:
     /// @brief The lock used to prohibit parallel addition and processing of events
-    MFXMutex myLock;
+    FXMutex myLock;
 
 
 private:
@@ -98,9 +83,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
