@@ -1,39 +1,29 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIInductLoop.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Aug 2003
-/// @version $Id: GUIInductLoop.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // The gui-version of the MSInductLoop, together with the according
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIInductLoop_h
-#define GUIInductLoop_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <utils/foxtools/MFXMutex.h>
+#include <utils/foxtools/fxheader.h>
 #include <microsim/output/MSInductLoop.h>
 #include <utils/geom/Position.h>
 #include "GUIDetectorWrapper.h"
@@ -61,78 +51,30 @@ public:
      * @param[in] position Position of the detector within the lane
      * @param[in] vTypes which vehicle types are considered
      */
-    GUIInductLoop(const std::string& id, MSLane* const lane, double position, const std::string& vTypes);
+    GUIInductLoop(const std::string& id, MSLane* const lane, double position, const std::string& vTypes, bool show);
 
 
     /// @brief Destructor
     ~GUIInductLoop();
 
 
-    /** @brief Resets all generated values to allow computation of next interval
-     *
-     * Locks the internal mutex before calling MSInductLoop::reset()
-     * @see MSInductLoop::reset()
-     */
-    void reset();
-
-
     /** @brief Returns this detector's visualisation-wrapper
      * @return The wrapper representing the detector
      */
     virtual GUIDetectorWrapper* buildDetectorGUIRepresentation();
 
+    /// @brief sets special caller for myWrapper
+    void setSpecialColor(const RGBColor* color);
 
-    /** @brief Returns vehicle data for vehicles that have been on the detector starting at the given time
-     *
-     * This method uses a mutex to prevent parallel read/write access to the vehicle buffer
-     *
-     * @param[in] t The time from which vehicles shall be counted
-     * @param[in] leaveTime Whether entryTime or leaveTime shall be compared against t
-     *            (the latter gives a more complete picture but may include vehicles in multiple steps even if they did not stay on the detector)
-     * @return The list of vehicles
-     * @see MSInductLoop::collectVehiclesOnDet()
-     */
-    std::vector<VehicleData> collectVehiclesOnDet(SUMOTime t, bool leaveTime = false) const;
-
-
-protected:
-    /// @name Methods that add and remove vehicles from internal container
-    /// @{
-
-    /** @brief Introduces a vehicle to the detector's map myVehiclesOnDet.
-     *
-     * Locks the internal mutex before calling MSInductLoop::enterDetectorByMove()
-     * @see MSInductLoop::enterDetectorByMove()
-     * @param veh The entering vehicle.
-     * @param entryTimestep Timestep (not necessary integer) of entrance.
-     * @see MSInductLoop::enterDetectorByMove()
-     */
-    void enterDetectorByMove(SUMOVehicle& veh, double entryTimestep);
-
-
-    /** @brief Processes a vehicle that leaves the detector
-     *
-     * Locks the internal mutex before calling MSInductLoop::leaveDetectorByMove()
-     * @see MSInductLoop::leaveDetectorByMove()
-     * @param veh The leaving vehicle.
-     * @param leaveTimestep Timestep (not necessary integer) of leaving.
-     * @see MSInductLoop::leaveDetectorByMove()
-     */
-    void leaveDetectorByMove(SUMOVehicle& veh, double leaveTimestep);
-
-
-    /** @brief Removes a vehicle from the detector's map myVehiclesOnDet.
-     *
-     * Locks the internal mutex before calling MSInductLoop::leaveDetectorByLaneChange()
-     * @see MSInductLoop::leaveDetectorByLaneChange()
-     * @param veh The leaving vehicle.
-     * @param lastPos The last position of the leaving vehicle.
-     */
-    void leaveDetectorByLaneChange(SUMOVehicle& veh, double lastPos);
-    /// @}
-
-
+    /// @brief whether the induction loop shall be visible
+    bool isVisible() const {
+        return myShow;
+    }
 
+    /// @brief toggle visibility
+    void setVisible(bool show) {
+        myShow = show;
+    }
 
 public:
     /**
@@ -178,9 +120,10 @@ public:
         //@}
 
 
-        /// @brief Returns the detector itself
-        GUIInductLoop& getLoop();
-
+        /// @brief set (outline) color for extra visualiaztion
+        void setSpecialColor(const RGBColor* color) {
+            mySpecialColor = color;
+        }
 
     private:
         /// @brief The wrapped detector
@@ -198,6 +141,9 @@ public:
         /// @brief The position on the lane
         double myPosition;
 
+        /// @brief color for extra visualization
+        const RGBColor* mySpecialColor;
+
     private:
         /// @brief Invalidated copy constructor.
         MyWrapper(const MyWrapper&);
@@ -207,14 +153,12 @@ public:
 
     };
 
+private:
 
-    /// @brief Mutex preventing parallel read/write access to internal MSInductLoop state
-    mutable MFXMutex myLock;
+    /// @brief the glObject wrapper for this induction loop
+    MyWrapper* myWrapper;
+
+    /// @brief whether this induction loop shall be visible in the gui
+    bool myShow;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
