@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2003-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIInstantInductLoop.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Aug 2003
-/// @version $Id: GUIInstantInductLoop.cpp 21851 2016-10-31 12:20:12Z behrisch $
 ///
 // The gui-version of the MSInstantInductLoop
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2003-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <utils/geom/PositionVector.h>
@@ -41,10 +32,6 @@
 #include "GUIEdge.h"
 #include <utils/gui/globjects/GLIncludes.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -53,7 +40,7 @@
  * GUIInstantInductLoop-methods
  * ----------------------------------------------------------------------- */
 GUIInstantInductLoop::GUIInstantInductLoop(const std::string& id, OutputDevice& od,
-        MSLane* const lane, SUMOReal positionInMeters,
+        MSLane* const lane, double positionInMeters,
         const std::string& vTypes)
     : MSInstantInductLoop(id, od, lane, positionInMeters, vTypes) {}
 
@@ -66,16 +53,16 @@ GUIInstantInductLoop::buildDetectorGUIRepresentation() {
     return new MyWrapper(*this, myPosition);
 }
 
+// -------------------------------------------------------------------------
+// GUIInstantInductLoop::MyWrapper-methods
+// -------------------------------------------------------------------------
 
-/* -------------------------------------------------------------------------
- * GUIInstantInductLoop::MyWrapper-methods
- * ----------------------------------------------------------------------- */
-GUIInstantInductLoop::MyWrapper::MyWrapper(GUIInstantInductLoop& detector, SUMOReal pos)
-    : GUIDetectorWrapper("instant induct loop", detector.getID()),
-      myDetector(detector), myPosition(pos) {
+GUIInstantInductLoop::MyWrapper::MyWrapper(GUIInstantInductLoop& detector, double pos) :
+    GUIDetectorWrapper(GLO_E1DETECTOR_INSTANT, detector.getID()),
+    myDetector(detector), myPosition(pos) {
     myFGPosition = detector.getLane()->geometryPositionAtOffset(pos);
-    myBoundary.add(myFGPosition.x() + (SUMOReal) 5.5, myFGPosition.y() + (SUMOReal) 5.5);
-    myBoundary.add(myFGPosition.x() - (SUMOReal) 5.5, myFGPosition.y() - (SUMOReal) 5.5);
+    myBoundary.add(myFGPosition.x() + (double) 5.5, myFGPosition.y() + (double) 5.5);
+    myBoundary.add(myFGPosition.x() - (double) 5.5, myFGPosition.y() - (double) 5.5);
     myFGRotation = -detector.getLane()->getShape().rotationDegreeAtOffset(pos);
 }
 
@@ -95,7 +82,7 @@ GUIInstantInductLoop::MyWrapper::getCenteringBoundary() const {
 GUIParameterTableWindow*
 GUIInstantInductLoop::MyWrapper::getParameterWindow(GUIMainWindow& app,
         GUISUMOAbstractView& /*parent !!! recheck this - never needed?*/) {
-    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this, 7);
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this);
     // add items
     // parameter
     ret->mkItem("position [m]", false, myPosition);
@@ -110,9 +97,9 @@ GUIInstantInductLoop::MyWrapper::getParameterWindow(GUIMainWindow& app,
 void
 GUIInstantInductLoop::MyWrapper::drawGL(const GUIVisualizationSettings& s) const {
     glPushName(getGlID());
-    SUMOReal width = (SUMOReal) 2.0 * s.scale;
+    double width = (double) 2.0 * s.scale;
     glLineWidth(1.0);
-    const SUMOReal exaggeration = s.addSize.getExaggeration(s);
+    const double exaggeration = s.addSize.getExaggeration(s, this);
     // shape
     glColor3d(1, 0, 1);
     glPushMatrix();
@@ -166,6 +153,4 @@ GUIInstantInductLoop::MyWrapper::getLoop() {
 }
 
 
-
 /****************************************************************************/
-
