@@ -1,43 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUILane.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUILane.h 23564 2017-03-21 07:38:52Z namdre $
 ///
 // Representation of a lane in the micro simulation (gui-version)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUILane_h
-#define GUILane_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
 #include <string>
 #include <utility>
 #include <microsim/MSLane.h>
 #include <microsim/MSEdge.h>
-#include <utils/foxtools/MFXMutex.h>
 #include <utils/geom/Position.h>
 #include <utils/geom/PositionVector.h>
 #include <utils/gui/globjects/GUIGlObject.h>
@@ -85,7 +74,10 @@ public:
     GUILane(const std::string& id, double maxSpeed,
             double length, MSEdge* const edge, int numericalID,
             const PositionVector& shape, double width,
-            SVCPermissions permissions, int index, bool isRampAccel);
+            SVCPermissions permissions,
+            SVCPermissions changeLeft, SVCPermissions changeRight,
+            int index, bool isRampAccel,
+            const std::string& type);
 
 
     /// @brief Destructor
@@ -95,7 +87,7 @@ public:
      * @note Inherited from GUIGlObject
      * @return This object's parent id
      */
-    const std::string& getParentName() const {
+    std::string getParentName() const {
         return getEdge().getID();
     }
 
@@ -134,11 +126,15 @@ public:
 
     /** the same as in MSLane, but locks the access for the visualisation
         first; the access will be granted at the end of this method */
-    bool executeMovements(SUMOTime t, std::vector<MSLane*>& into);
+    void setJunctionApproaches(const SUMOTime t) const;
 
     /** the same as in MSLane, but locks the access for the visualisation
         first; the access will be granted at the end of this method */
-    bool integrateNewVehicle(SUMOTime t);
+    void executeMovements(const SUMOTime t);
+
+    /** the same as in MSLane, but locks the access for the visualisation
+        first; the access will be granted at the end of this method */
+    void integrateNewVehicles();
     ///@}
 
 
@@ -151,6 +147,8 @@ public:
         first; the access will be granted at the end of this method */
     MSVehicle* removeVehicle(MSVehicle* remVehicle, MSMoveReminder::Notification notification, bool notify);
 
+    /// @brief remove parking vehicle
+    void removeParking(MSVehicle* veh);
 
     /** @brief Sets the information about a vehicle lapping into this lane
      *
@@ -221,11 +219,14 @@ public:
     /// @brief bike lane markings on top of an intersection
     void drawBikeMarkings() const;
 
-    /// @brief draw crossties for railroads or pedestrian crossings
-    void drawCrossties(double length, double spacing, double halfWidth) const;
+    /// @brief bike lane markings on top of an intersection
+    void drawJunctionChangeProhibitions() const;
 
     /// @brief direction indicators for lanes
-    void drawDirectionIndicators() const;
+    void drawDirectionIndicators(double exaggeration, bool spreadSuperposed) const;
+
+    /// @brief draw intersection positions of foe internal lanes with this one
+    void debugDrawFoeIntersections() const;
 
     double getEdgeLaneNumber() const;
 
@@ -237,6 +238,14 @@ public:
      */
     double getLoadedEdgeWeight() const;
 
+    void setReachability(double value) {
+        myReachability = value;
+    }
+
+    double getReachability() const {
+        return myReachability;
+    }
+
 #ifdef HAVE_OSG
     void setGeometry(osg::Geometry* geom) {
         myGeom = geom;
@@ -253,6 +262,21 @@ public:
         return myAmClosed;
     }
 
+    /// @brief gets the color value according to the current scheme index
+    double getColorValue(const GUIVisualizationSettings& s, int activeScheme) const;
+
+    /// @brief whether this lane is selected in the GUI
+    bool isSelected() const;
+
+    /* @brief sets the color according to the current scheme index and some lane function
+     * @param[in] id override active scheme when calling from meso gui
+     */
+    bool setFunctionalColor(const GUIColorer& c, RGBColor& col, int activeScheme = -1) const;
+
+    /// @brief whether to draw this lane as a railway
+    bool drawAsRailway(const GUIVisualizationSettings& s) const;
+
+
 protected:
     /// moves myTmpVehicles int myVehicles after a lane change procedure
     void swapAfterLaneChange(SUMOTime t);
@@ -278,36 +302,34 @@ private:
     void drawLinkNo(const GUIVisualizationSettings& s) const;
     void drawTLSLinkNo(const GUIVisualizationSettings& s, const GUINet& net) const;
     void drawLinkRules(const GUIVisualizationSettings& s, const GUINet& net) const;
-    void drawLinkRule(const GUIVisualizationSettings& s, const GUINet& net, MSLink* link, const PositionVector& shape, double x1, double x2) const;
+    void drawLinkRule(const GUIVisualizationSettings& s, const GUINet& net, const MSLink* link, const PositionVector& shape, double x1, double x2) const;
     void drawArrows() const;
-    void drawLane2LaneConnections() const;
+    void drawLane2LaneConnections(double exaggeration) const;
 
 
     /// @brief add intermediate points at segment borders
     PositionVector splitAtSegments(const PositionVector& shape);
 
+    /// @brief get number of vehicles waiting for departure on this lane
+    double getPendingEmits() const;
+
 private:
-    /// @brief gets the color value according to the current scheme index
-    double getColorValue(int activeScheme) const;
 
     /// @brief gets the scaling value according to the current scheme index
     double getScaleValue(int activeScheme) const;
 
-    /// @brief sets the color according to the current scheme index and some lane function
-    bool setFunctionalColor(int activeScheme) const;
-
     /// @brief sets multiple colors according to the current scheme index and some lane function
-    bool setMultiColor(const GUIColorer& c) const;
+    bool setMultiColor(const GUIVisualizationSettings& s, const GUIColorer& c, RGBColor& col) const;
 
     /// @brief sets the color according to the currente settings
-    void setColor(const GUIVisualizationSettings& s) const;
-
-    /// @brief whether to draw this lane as a railway
-    bool drawAsRailway(const GUIVisualizationSettings& s) const;
+    RGBColor setColor(const GUIVisualizationSettings& s) const;
 
     /// @brief whether to draw this lane as a waterway
     bool drawAsWaterway(const GUIVisualizationSettings& s) const;
 
+    /// @brief whether this lane or its parent edge is selected in the GUI
+    bool isLaneOrEdgeSelected() const;
+
     /// The rotations of the shape parts
     std::vector<double> myShapeRotations;
 
@@ -326,6 +348,9 @@ private:
     /// @brief Quarter of lane width, for speed-up
     double myQuarterLaneWidth;
 
+    /// @brief the time distance from a particular edge
+    double myReachability;
+
 #ifdef HAVE_OSG
     osg::Geometry* myGeom;
 #endif
@@ -335,13 +360,10 @@ private:
 
 private:
     /// The mutex used to avoid concurrent updates of the vehicle buffer
-    mutable MFXMutex myLock;
+    mutable FXMutex myLock;
+
+    /// @brief special color to signify alternative coloring scheme
+    static const RGBColor MESO_USE_LANE_COLOR;
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
