@@ -1,43 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUILaneSpeedTrigger.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 26.04.2004
-/// @version $Id: GUILaneSpeedTrigger.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Changes the speed allowed on a set of lanes (gui version)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUILaneSpeedTrigger_h
-#define GUILaneSpeedTrigger_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
 #include <microsim/trigger/MSLaneSpeedTrigger.h>
 #include <utils/gui/globjects/GUIGlObject_AbstractAdd.h>
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
-#include <utils/foxtools/FXRealSpinDial.h>
 #include <gui/GUIManipulator.h>
 
 
@@ -166,11 +155,11 @@ public:
 
         FXDataTarget myChosenTarget;
 
-        SUMOReal mySpeed;
+        double mySpeed;
 
         FXDataTarget mySpeedTarget;
 
-        FXRealSpinDial* myUserDefinedSpeed;
+        FXRealSpinner* myUserDefinedSpeed;
 
         FXComboBox* myPredefinedValues;
 
@@ -186,7 +175,7 @@ private:
     typedef std::vector<Position> PosCont;
 
     /// Definition of a rotation container
-    typedef std::vector<SUMOReal> RotCont;
+    typedef std::vector<double> RotCont;
 
 private:
     /// The positions in full-geometry mode
@@ -202,15 +191,9 @@ private:
     bool myShowAsKMH;
 
     /// Storage for last value to avoid string recomputation
-    mutable SUMOReal myLastValue;
+    mutable double myLastValue;
 
     /// Storage for speed string to avoid recomputation
     mutable std::string myLastValueString;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
