@@ -1,33 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUINet.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: GUINet.cpp 23987 2017-04-20 09:22:50Z namdre $
 ///
 // A MSNet extended by some values for usage within the gui
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <utility>
 #include <set>
@@ -38,15 +31,18 @@
 #include <utils/gui/globjects/GUIPointOfInterest.h>
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
 #include <utils/gui/div/GUIParameterTableWindow.h>
+#include <utils/gui/div/GUIGlobalSelection.h>
 #include <utils/gui/globjects/GUIShapeContainer.h>
+#include <utils/xml/XMLSubSys.h>
 #include <utils/common/StringUtils.h>
 #include <utils/common/RGBColor.h>
 #include <utils/gui/div/GLObjectValuePassConnector.h>
 #include <microsim/MSNet.h>
+#include <microsim/MSEdgeWeightsStorage.h>
 #include <microsim/MSJunction.h>
 #include <microsim/output/MSDetectorControl.h>
 #include <microsim/MSEdge.h>
-#include <microsim/pedestrians/MSPModel.h>
+#include <microsim/transportables/MSPModel.h>
 #include <microsim/MSInsertionControl.h>
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSTLLogicControl.h>
@@ -56,6 +52,7 @@
 #include <guisim/GUITransportableControl.h>
 #include <guisim/GUILaneSpeedTrigger.h>
 #include <guisim/GUIDetectorWrapper.h>
+#include <guisim/GUICalibrator.h>
 #include <guisim/GUITrafficLightLogicWrapper.h>
 #include <guisim/GUIJunctionWrapper.h>
 #include <guisim/GUIVehicleControl.h>
@@ -69,17 +66,18 @@
 // definition of static variables used for visualisation of objects' values
 // ===========================================================================
 template std::vector< GLObjectValuePassConnector<double>* > GLObjectValuePassConnector<double>::myContainer;
-template MFXMutex GLObjectValuePassConnector<double>::myLock;
+template FXMutex GLObjectValuePassConnector<double>::myLock;
 
 template std::vector< GLObjectValuePassConnector<std::pair<int, class MSPhaseDefinition> >* > GLObjectValuePassConnector<std::pair<int, class MSPhaseDefinition> >::myContainer;
-template MFXMutex GLObjectValuePassConnector<std::pair<int, class MSPhaseDefinition> >::myLock;
+template FXMutex GLObjectValuePassConnector<std::pair<int, class MSPhaseDefinition> >::myLock;
 
 
 // ===========================================================================
 // member method definitions
 // ===========================================================================
 GUINet::GUINet(MSVehicleControl* vc, MSEventControl* beginOfTimestepEvents,
-               MSEventControl* endOfTimestepEvents, MSEventControl* insertionEvents) :
+               MSEventControl* endOfTimestepEvents,
+               MSEventControl* insertionEvents) :
     MSNet(vc, beginOfTimestepEvents, endOfTimestepEvents, insertionEvents, new GUIShapeContainer(myGrid)),
     GUIGlObject(GLO_NETWORK, ""),
     myLastSimDuration(0), /*myLastVisDuration(0),*/ myLastIdleDuration(0),
@@ -95,18 +93,25 @@ GUINet::~GUINet() {
     // delete allocated wrappers
     //  of junctions
     for (std::vector<GUIJunctionWrapper*>::iterator i1 = myJunctionWrapper.begin(); i1 != myJunctionWrapper.end(); i1++) {
-        delete(*i1);
+        delete (*i1);
     }
     //  of additional structures
     GUIGlObject_AbstractAdd::clearDictionary();
     //  of tl-logics
     for (Logics2WrapperMap::iterator i3 = myLogics2Wrapper.begin(); i3 != myLogics2Wrapper.end(); i3++) {
-        delete(*i3).second;
+        delete (*i3).second;
     }
     //  of detectors
-    for (std::vector<GUIDetectorWrapper*>::iterator i = myDetectorDict.begin(); i != myDetectorDict.end(); ++i) {
+    for (std::vector<GUIDetectorWrapper*>::iterator i = myDetectorWrapper.begin(); i != myDetectorWrapper.end(); ++i) {
         delete *i;
     }
+    //  of calibrators
+    for (GUICalibrator* cw : myCalibratorWrapper) {
+        delete cw;
+    }
+    for (auto& item : myLoadedEdgeData) {
+        delete item.second;
+    }
 }
 
 
@@ -118,8 +123,8 @@ GUINet::getBoundary() const {
 
 MSTransportableControl&
 GUINet::getPersonControl() {
-    if (myPersonControl == 0) {
-        myPersonControl = new GUITransportableControl();
+    if (myPersonControl == nullptr) {
+        myPersonControl = new GUITransportableControl(true);
     }
     return *myPersonControl;
 }
@@ -127,8 +132,8 @@ GUINet::getPersonControl() {
 
 MSTransportableControl&
 GUINet::getContainerControl() {
-    if (myContainerControl == 0) {
-        myContainerControl = new GUITransportableControl();
+    if (myContainerControl == nullptr) {
+        myContainerControl = new GUITransportableControl(false);
     }
     return *myContainerControl;
 }
@@ -138,8 +143,6 @@ void
 GUINet::initTLMap() {
     // get the list of loaded tl-logics
     const std::vector<MSTrafficLightLogic*>& logics = getTLSControl().getAllLogics();
-    // allocate storage for the wrappers
-    myTLLogicWrappers.reserve(logics.size());
     // go through the logics
     for (std::vector<MSTrafficLightLogic*>::const_iterator i = logics.begin(); i != logics.end(); ++i) {
         createTLWrapper(*i);
@@ -147,15 +150,15 @@ GUINet::initTLMap() {
 }
 
 
-GUIGlID
+void
 GUINet::createTLWrapper(MSTrafficLightLogic* tll) {
     if (myLogics2Wrapper.count(tll) > 0) {
-        return myLogics2Wrapper[tll]->getGlID();
+        return;
     }
     // get the links
     const MSTrafficLightLogic::LinkVectorVector& links = tll->getLinks();
     if (links.size() == 0) { // @legacy this should never happen in 0.13.0+ networks
-        return 0;
+        return;
     }
     // build the wrapper
     GUITrafficLightLogicWrapper* tllw =
@@ -170,7 +173,7 @@ GUINet::createTLWrapper(MSTrafficLightLogic* tll) {
     }
     myGrid.addAdditionalGLObject(tllw);
     myLogics2Wrapper[tll] = tllw;
-    return tllw->getGlID();
+    return;
 }
 
 
@@ -183,12 +186,12 @@ GUINet::getJunctionPosition(const std::string& name) const {
 
 bool
 GUINet::vehicleExists(const std::string& name) const {
-    return myVehicleControl->getVehicle(name) != 0;
+    return myVehicleControl->getVehicle(name) != nullptr;
 }
 
 
 int
-GUINet::getLinkTLID(MSLink* link) const {
+GUINet::getLinkTLID(const MSLink* const link) const {
     if (myLinks2Logic.count(link) == 0) {
         assert(false);
         return 0;
@@ -203,7 +206,7 @@ GUINet::getLinkTLID(MSLink* link) const {
 
 
 int
-GUINet::getLinkTLIndex(MSLink* link) const {
+GUINet::getLinkTLIndex(const MSLink* const link) const {
     Links2LogicMap::const_iterator i = myLinks2Logic.find(link);
     if (i == myLinks2Logic.end()) {
         return -1;
@@ -224,7 +227,7 @@ GUINet::guiSimulationStep() {
 
 void
 GUINet::simulationStep() {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     MSNet::simulationStep();
 }
 
@@ -233,7 +236,7 @@ std::vector<GUIGlID>
 GUINet::getJunctionIDs(bool includeInternal) const {
     std::vector<GUIGlID> ret;
     for (std::vector<GUIJunctionWrapper*>::const_iterator i = myJunctionWrapper.begin(); i != myJunctionWrapper.end(); ++i) {
-        if (!(*i)->isInner() || includeInternal) {
+        if (!(*i)->isInternal() || includeInternal) {
             ret.push_back((*i)->getGlID());
         }
     }
@@ -261,25 +264,44 @@ GUINet::initGUIStructures() {
     // initialise detector storage for gui
     const std::vector<SumoXMLTag> types = myDetectorControl->getAvailableTypes();
     for (std::vector<SumoXMLTag>::const_iterator i = types.begin(); i != types.end(); ++i) {
-        const std::map<std::string, MSDetectorFileOutput*>& dets = myDetectorControl->getTypedDetectors(*i).getMyMap();
-        for (std::map<std::string, MSDetectorFileOutput*>::const_iterator j = dets.begin(); j != dets.end(); ++j) {
-            GUIDetectorWrapper* wrapper = (*j).second->buildDetectorGUIRepresentation();
-            if (wrapper != 0) {
-                myDetectorDict.push_back(wrapper);
+        for (const auto& j : myDetectorControl->getTypedDetectors(*i)) {
+            GUIDetectorWrapper* wrapper = j.second->buildDetectorGUIRepresentation();
+            if (wrapper != nullptr) {
+                myDetectorWrapper.push_back(wrapper);
                 myGrid.addAdditionalGLObject(wrapper);
             }
         }
     }
+    // initialise calibrators
+    for (auto& item : MSCalibrator::getInstances()) {
+        GUICalibrator* wrapper = new GUICalibrator(item.second);
+        myCalibratorWrapper.push_back(wrapper);
+        myGrid.addAdditionalGLObject(wrapper);
+    }
     // initialise the tl-map
     initTLMap();
     // initialise edge storage for gui
-    GUIEdge::fill(myEdgeWrapper);
+    const MSEdgeVector& edges = MSEdge::getAllEdges();
+    myEdgeWrapper.reserve(edges.size());
+    for (MSEdgeVector::const_iterator i = edges.begin(); i != edges.end(); ++i) {
+        // VISIM connector edges shall be drawn (they have lanes)
+        if (!(*i)->isTazConnector() || (*i)->getLanes().size() > 0) {
+            myEdgeWrapper.push_back(static_cast<GUIEdge*>(*i));
+        }
+    }
     // initialise junction storage for gui
     int size = myJunctions->size();
     myJunctionWrapper.reserve(size);
-    const std::map<std::string, MSJunction*>& junctions = myJunctions->getMyMap();
-    for (std::map<std::string, MSJunction*>::const_iterator i = junctions.begin(); i != junctions.end(); ++i) {
-        myJunctionWrapper.push_back(new GUIJunctionWrapper(*(*i).second));
+    std::map<MSJunction*, std::string> junction2TLL;
+    for (const auto tls : getTLSControl().getAllLogics()) {
+        for (const auto& links : tls->getLinks()) {
+            for (const MSLink* l : links) {
+                junction2TLL[l->getJunction()] = l->getTLLogic()->getID();
+            }
+        }
+    }
+    for (const auto& i : *myJunctions) {
+        myJunctionWrapper.push_back(new GUIJunctionWrapper(*i.second, junction2TLL[i.second]));
     }
     // build the visualization tree
     for (std::vector<GUIEdge*>::iterator i = myEdgeWrapper.begin(); i != myEdgeWrapper.end(); ++i) {
@@ -337,7 +359,7 @@ GUINet::getRTFactor() const {
     if (myLastSimDuration == 0) {
         return -1;
     }
-    return (double) 1000. / (double) myLastSimDuration;
+    return (double)DELTA_T / (double)myLastSimDuration;
 }
 
 
@@ -410,9 +432,8 @@ GUINet::getPopUpMenu(GUIMainWindow& app,
 
 GUIParameterTableWindow*
 GUINet::getParameterWindow(GUIMainWindow& app,
-                           GUISUMOAbstractView&) {
-    GUIParameterTableWindow* ret =
-        new GUIParameterTableWindow(app, *this, 29);
+                           GUISUMOAbstractView& parent) {
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this);
     // add items
     ret->mkItem("loaded vehicles [#]", true,
                 new FunctionBinding<MSVehicleControl, int>(&getVehicleControl(), &MSVehicleControl::getLoadedVehicleNo));
@@ -423,12 +444,20 @@ GUINet::getParameterWindow(GUIMainWindow& app,
     ret->mkItem("running vehicles [#]", true,
                 new FunctionBinding<MSVehicleControl, int>(&getVehicleControl(), &MSVehicleControl::getRunningVehicleNo));
     ret->mkItem("arrived vehicles [#]", true,
-                new FunctionBinding<MSVehicleControl, int>(&getVehicleControl(), &MSVehicleControl::getEndedVehicleNo));
+                new FunctionBinding<MSVehicleControl, int>(&getVehicleControl(), &MSVehicleControl::getArrivedVehicleNo));
+    ret->mkItem("discarded vehicles [#]", true,
+                new FunctionBinding<MSVehicleControl, int>(&getVehicleControl(), &MSVehicleControl::getDiscardedVehicleNo));
     ret->mkItem("collisions [#]", true,
                 new FunctionBinding<MSVehicleControl, int>(&getVehicleControl(), &MSVehicleControl::getCollisionCount));
     ret->mkItem("teleports [#]", true,
                 new FunctionBinding<MSVehicleControl, int>(&getVehicleControl(), &MSVehicleControl::getTeleportCount));
-    if (myPersonControl != 0) {
+    ret->mkItem("halting [#]", true,
+                new FunctionBinding<MSVehicleControl, int>(&getVehicleControl(), &MSVehicleControl::getHaltingVehicleNo));
+    ret->mkItem("avg. speed [m/s]", true,
+                new FunctionBinding<MSVehicleControl, double>(&getVehicleControl(), &MSVehicleControl::getVehicleMeanSpeed));
+    ret->mkItem("avg. relative speed", true,
+                new FunctionBinding<MSVehicleControl, double>(&getVehicleControl(), &MSVehicleControl::getVehicleMeanSpeedRelative));
+    if (myPersonControl != nullptr) {
         ret->mkItem("loaded persons [#]", true,
                     new FunctionBinding<MSTransportableControl, int>(&getPersonControl(), &MSTransportableControl::getLoadedNumber));
         ret->mkItem("running persons [#]", true,
@@ -441,6 +470,7 @@ GUINet::getParameterWindow(GUIMainWindow& app,
 //    ret->mkItem("time step [s]", true, new FunctionBinding<GUINet, SUMOTime>(this, &GUINet::getCurrentTimeStep));
     if (logSimulationDuration()) {
         ret->mkItem("step duration [ms]", true, new FunctionBinding<GUINet, int>(this, &GUINet::getWholeDuration));
+        ret->mkItem("FPS", true, new FunctionBinding<GUISUMOAbstractView, double>(&parent, &GUISUMOAbstractView::getFPS));
         ret->mkItem("simulation duration [ms]", true, new FunctionBinding<GUINet, int>(this, &GUINet::getSimDuration));
         /*
         ret->mkItem("visualisation duration [ms]", true,
@@ -448,14 +478,14 @@ GUINet::getParameterWindow(GUIMainWindow& app,
                 &(getNet()), &GUINet::getVisDuration));
         */
         ret->mkItem("idle duration [ms]", true, new FunctionBinding<GUINet, int>(this, &GUINet::getIdleDuration));
-        ret->mkItem("duration factor []", true, new FunctionBinding<GUINet, double>(this, &GUINet::getRTFactor));
+        ret->mkItem("duration factor", true, new FunctionBinding<GUINet, double>(this, &GUINet::getRTFactor));
         /*
         ret->mkItem("mean duration factor []", true,
             new FuncBinding_IntParam<GUINet, double>(
                 &(getNet()), &GUINet::getMeanRTFactor), 1);
                 */
-        ret->mkItem("ups [#]", true, new FunctionBinding<GUINet, double>(this, &GUINet::getUPS));
-        ret->mkItem("mean ups [#]", true, new FunctionBinding<GUINet, double>(this, &GUINet::getMeanUPS));
+        ret->mkItem("updates per second", true, new FunctionBinding<GUINet, double>(this, &GUINet::getUPS));
+        ret->mkItem("avg. updates per second", true, new FunctionBinding<GUINet, double>(this, &GUINet::getMeanUPS));
         if (OptionsCont::getOptions().getBool("duration-log.statistics")) {
             ret->mkItem("avg. trip length [m]", true, new FunctionBinding<GUINet, double>(this, &GUINet::getAvgRouteLength));
             ret->mkItem("avg. trip duration [s]", true, new FunctionBinding<GUINet, double>(this, &GUINet::getAvgDuration));
@@ -463,9 +493,14 @@ GUINet::getParameterWindow(GUIMainWindow& app,
             ret->mkItem("avg. trip time loss [s]", true, new FunctionBinding<GUINet, double>(this, &GUINet::getAvgTimeLoss));
             ret->mkItem("avg. trip depart delay [s]", true, new FunctionBinding<GUINet, double>(this, &GUINet::getAvgDepartDelay));
             ret->mkItem("avg. trip speed [m/s]", true, new FunctionBinding<GUINet, double>(this, &GUINet::getAvgTripSpeed));
+            if (myPersonControl != nullptr) {
+                ret->mkItem("avg. walk length [m]", true, new FunctionBinding<GUINet, double>(this, &GUINet::getAvgWalkRouteLength));
+                ret->mkItem("avg. walk duration [s]", true, new FunctionBinding<GUINet, double>(this, &GUINet::getAvgWalkDuration));
+                ret->mkItem("avg. walk time loss [s]", true, new FunctionBinding<GUINet, double>(this, &GUINet::getAvgWalkTimeLoss));
+            }
         }
     }
-    ret->mkItem("nodes [#]", false, (int)myJunctions->size());
+    ret->mkItem("nodes [#]", false, (int)getJunctionIDs(false).size());
     ret->mkItem("edges [#]", false, (int)GUIEdge::getIDs(false).size());
     ret->mkItem("total edge length [km]", false, GUIEdge::getTotalLength(false, false) / 1000);
     ret->mkItem("total lane length [km]", false, GUIEdge::getTotalLength(false, true) / 1000);
@@ -490,7 +525,7 @@ GUINet::getCenteringBoundary() const {
 GUINet*
 GUINet::getGUIInstance() {
     GUINet* net = dynamic_cast<GUINet*>(MSNet::getInstance());
-    if (net != 0) {
+    if (net != nullptr) {
         return net;
     }
     throw ProcessError("A gui-network was not yet constructed.");
@@ -520,11 +555,124 @@ GUINet::getGUIMEVehicleControl() {
 }
 
 
+double
+GUINet::getEdgeData(const MSEdge* edge, const std::string& attr) {
+    auto it = myLoadedEdgeData.find(attr);
+    if (it != myLoadedEdgeData.end()) {
+        double value;
+        bool found = it->second->retrieveExistingEffort(edge, STEPS2TIME(getCurrentTimeStep()), value);
+        if (found) {
+            return value;
+        } else {
+            return GUIVisualizationSettings::MISSING_DATA;
+        }
+    } else {
+        return GUIVisualizationSettings::MISSING_DATA;
+    }
+}
+
+
+void
+GUINet::DiscoverAttributes::myStartElement(int element, const SUMOSAXAttributes& attrs) {
+    if (element == SUMO_TAG_EDGE || element == SUMO_TAG_LANE) {
+        std::vector<std::string> tmp = attrs.getAttributeNames();
+        edgeAttrs.insert(tmp.begin(), tmp.end());
+    } else if (element == SUMO_TAG_EDGEREL) {
+        for (const std::string& a : attrs.getAttributeNames()) {
+            if (a != "from" && a != "to") {
+                edgeAttrs.insert(a);
+            }
+        }
+    } else if (element == SUMO_TAG_INTERVAL) {
+        bool ok;
+        lastIntervalEnd = MAX2(lastIntervalEnd, attrs.getSUMOTimeReporting(SUMO_ATTR_END, nullptr, ok));
+    }
+}
+
+std::vector<std::string>
+GUINet::DiscoverAttributes::getEdgeAttrs() {
+    edgeAttrs.erase(toString(SUMO_ATTR_ID));
+    return std::vector<std::string>(edgeAttrs.begin(), edgeAttrs.end());
+}
+
+void
+GUINet::EdgeFloatTimeLineRetriever_GUI::addEdgeWeight(const std::string& id,
+        double value, double begTime, double endTime) const {
+    MSEdge* edge = MSEdge::dictionary(id);
+    if (edge != nullptr) {
+        myWeightStorage->addEffort(edge, begTime, endTime, value);
+    } else {
+        WRITE_ERROR("Trying to set the effort for the unknown edge '" + id + "'.");
+    }
+}
+
+void
+GUINet::EdgeFloatTimeLineRetriever_GUI::addEdgeRelWeight(const std::string& from, const std::string& to,
+        double val, double beg, double end) const {
+    MSEdge* fromEdge = MSEdge::dictionary(from);
+    MSEdge* toEdge = MSEdge::dictionary(to);
+    if (fromEdge != nullptr && toEdge != nullptr) {
+        for (auto item : fromEdge->getViaSuccessors()) {
+            if (item.first == toEdge) {
+                const MSEdge* edge = item.second;
+                while (edge != nullptr && edge->isInternal()) {
+                    myWeightStorage->addEffort(edge, beg, end, val);
+                    edge = edge->getViaSuccessors().front().second;
+                }
+            }
+        }
+    } else if (fromEdge == nullptr) {
+        WRITE_ERROR("Trying to set the effort for the unknown edge '" + from + "'.");
+    } else {
+        WRITE_ERROR("Trying to set the effort for the unknown edge '" + to + "'.");
+    }
+}
+
+bool
+GUINet::loadEdgeData(const std::string& file) {
+    // discover edge attributes
+    DiscoverAttributes discoveryHandler(file);
+    XMLSubSys::runParser(discoveryHandler, file);
+    std::vector<std::string> attrs = discoveryHandler.getEdgeAttrs();
+    WRITE_MESSAGE("Loading edgedata from '" + file
+                  + "' Found " + toString(attrs.size())
+                  + " attributes: " + toString(attrs));
+    myEdgeDataEndTime = MAX2(myEdgeDataEndTime, discoveryHandler.lastIntervalEnd);
+    // create a retriever for each attribute
+    std::vector<EdgeFloatTimeLineRetriever_GUI> retrieverDefsInternal;
+    retrieverDefsInternal.reserve(attrs.size());
+    std::vector<SAXWeightsHandler::ToRetrieveDefinition*> retrieverDefs;
+    for (const std::string& attr : attrs) {
+        MSEdgeWeightsStorage* ws = new MSEdgeWeightsStorage();
+        myLoadedEdgeData[attr] = ws;
+        retrieverDefsInternal.push_back(EdgeFloatTimeLineRetriever_GUI(ws));
+        retrieverDefs.push_back(new SAXWeightsHandler::ToRetrieveDefinition(attr, true, retrieverDefsInternal.back()));
+    }
+    SAXWeightsHandler handler(retrieverDefs, "");
+    return XMLSubSys::runParser(handler, file);
+}
+
+
+std::vector<std::string>
+GUINet::getEdgeDataAttrs() const {
+    std::vector<std::string> result;
+    for (const auto& item : myLoadedEdgeData) {
+        result.push_back(item.first);
+    }
+    return result;
+}
+
+bool
+GUINet::isSelected(const MSTrafficLightLogic* tll) const {
+    const auto it = myLogics2Wrapper.find(const_cast<MSTrafficLightLogic*>(tll));
+    return it != myLogics2Wrapper.end() && gSelected.isSelected(GLO_TLLOGIC, it->second->getGlID());
+}
+
 #ifdef HAVE_OSG
 void
 GUINet::updateColor(const GUIVisualizationSettings& s) {
     for (std::vector<GUIEdge*>::const_iterator i = myEdgeWrapper.begin(); i != myEdgeWrapper.end(); ++i) {
-        if ((*i)->getPurpose() != MSEdge::EDGEFUNCTION_INTERNAL) {
+        if (!(*i)->isInternal()) {
             const std::vector<MSLane*>& lanes = (*i)->getLanes();
             for (std::vector<MSLane*>::const_iterator j = lanes.begin(); j != lanes.end(); ++j) {
                 static_cast<GUILane*>(*j)->updateColor(s);
@@ -537,5 +685,5 @@ GUINet::updateColor(const GUIVisualizationSettings& s) {
 }
 #endif
 
-/****************************************************************************/
 
+/****************************************************************************/
