@@ -1,62 +1,39 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIPerson.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIPerson.cpp 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // A MSPerson extended by some values for usage within the gui
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
-#include <cmath>
-#include <vector>
-#include <string>
-#include <foreign/polyfonts/polyfonts.h>
-#include <microsim/pedestrians/MSPerson.h>
-#include <microsim/pedestrians/MSPModel_Striping.h>
-#include <microsim/logging/CastingFunctionBinding.h>
-#include <microsim/logging/FunctionBinding.h>
-#include <microsim/MSVehicleControl.h>
-#include <microsim/devices/MSDevice_Vehroutes.h>
-#include <utils/common/StringUtils.h>
-#include <utils/vehicle/SUMOVehicleParameter.h>
-#include <utils/common/AbstractMutex.h>
-#include <utils/geom/GeomHelper.h>
-#include <utils/gui/images/GUITexturesHelper.h>
-#include <utils/gui/windows/GUISUMOAbstractView.h>
-#include <utils/gui/windows/GUIAppEnum.h>
-#include <utils/gui/div/GUIParameterTableWindow.h>
-#include <utils/gui/div/GUIGlobalSelection.h>
-#include <utils/gui/div/GLHelper.h>
-#include <utils/gui/div/GLObjectValuePassConnector.h>
-#include <utils/gui/globjects/GLIncludes.h>
 #include <gui/GUIApplicationWindow.h>
-#include <gui/GUIGlobals.h>
+#include <microsim/MSStoppingPlace.h>
+#include <microsim/transportables/MSTransportableControl.h>
+#include <microsim/logging/FunctionBinding.h>
+#include <microsim/transportables/MSPModel_Striping.h>
+#include <utils/gui/div/GLHelper.h>
+#include <utils/gui/div/GUIGlobalSelection.h>
+#include <utils/gui/div/GUIParameterTableWindow.h>
+#include <utils/gui/globjects/GLIncludes.h>
+#include <utils/gui/div/GUIBasePersonHelper.h>
+#include <utils/gui/div/GUIDesigns.h>
+
 #include "GUILane.h"
-#include "GUINet.h"
-#include "GUIEdge.h"
 #include "GUIPerson.h"
 
 //#define GUIPerson_DEBUG_DRAW_WALKINGAREA_PATHS 1
@@ -64,36 +41,38 @@
 // ===========================================================================
 // FOX callback mapping
 // ===========================================================================
+
 FXDEFMAP(GUIPerson::GUIPersonPopupMenu) GUIPersonPopupMenuMap[] = {
     FXMAPFUNC(SEL_COMMAND, MID_SHOW_CURRENTROUTE,     GUIPerson::GUIPersonPopupMenu::onCmdShowCurrentRoute),
     FXMAPFUNC(SEL_COMMAND, MID_HIDE_CURRENTROUTE,     GUIPerson::GUIPersonPopupMenu::onCmdHideCurrentRoute),
     FXMAPFUNC(SEL_COMMAND, MID_SHOW_WALKINGAREA_PATH, GUIPerson::GUIPersonPopupMenu::onCmdShowWalkingareaPath),
     FXMAPFUNC(SEL_COMMAND, MID_HIDE_WALKINGAREA_PATH, GUIPerson::GUIPersonPopupMenu::onCmdHideWalkingareaPath),
+    FXMAPFUNC(SEL_COMMAND, MID_SHOWPLAN,              GUIPerson::GUIPersonPopupMenu::onCmdShowPlan),
     FXMAPFUNC(SEL_COMMAND, MID_START_TRACK,           GUIPerson::GUIPersonPopupMenu::onCmdStartTrack),
     FXMAPFUNC(SEL_COMMAND, MID_STOP_TRACK,            GUIPerson::GUIPersonPopupMenu::onCmdStopTrack),
+    FXMAPFUNC(SEL_COMMAND, MID_REMOVE_OBJECT,         GUIPerson::GUIPersonPopupMenu::onCmdRemoveObject),
 };
 
 // Object implementation
 FXIMPLEMENT(GUIPerson::GUIPersonPopupMenu, GUIGLObjectPopupMenu, GUIPersonPopupMenuMap, ARRAYNUMBER(GUIPersonPopupMenuMap))
 
-
-
 // ===========================================================================
 // method definitions
 // ===========================================================================
-/* -------------------------------------------------------------------------
- * GUIPerson::GUIPersonPopupMenu - methods
- * ----------------------------------------------------------------------- */
+
+// -------------------------------------------------------------------------
+// GUIPerson::GUIPersonPopupMenu - methods
+// -------------------------------------------------------------------------
+
 GUIPerson::GUIPersonPopupMenu::GUIPersonPopupMenu(
-    GUIMainWindow& app, GUISUMOAbstractView& parent,
-    GUIGlObject& o, std::map<GUISUMOAbstractView*, int>& additionalVisualizations) :
-    GUIGLObjectPopupMenu(app, parent, o),
-    myVehiclesAdditionalVisualizations(additionalVisualizations) {
+    GUIMainWindow& app, GUISUMOAbstractView& parent, GUIGlObject& o) :
+    GUIGLObjectPopupMenu(app, parent, o) {
 }
 
 
 GUIPerson::GUIPersonPopupMenu::~GUIPersonPopupMenu() {}
 
+
 long
 GUIPerson::GUIPersonPopupMenu::onCmdShowCurrentRoute(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -103,6 +82,7 @@ GUIPerson::GUIPersonPopupMenu::onCmdShowCurrentRoute(FXObject*, FXSelector, void
     return 1;
 }
 
+
 long
 GUIPerson::GUIPersonPopupMenu::onCmdHideCurrentRoute(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -111,7 +91,6 @@ GUIPerson::GUIPersonPopupMenu::onCmdHideCurrentRoute(FXObject*, FXSelector, void
 }
 
 
-
 long
 GUIPerson::GUIPersonPopupMenu::onCmdShowWalkingareaPath(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -121,6 +100,7 @@ GUIPerson::GUIPersonPopupMenu::onCmdShowWalkingareaPath(FXObject*, FXSelector, v
     return 1;
 }
 
+
 long
 GUIPerson::GUIPersonPopupMenu::onCmdHideWalkingareaPath(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -129,6 +109,24 @@ GUIPerson::GUIPersonPopupMenu::onCmdHideWalkingareaPath(FXObject*, FXSelector, v
 }
 
 
+long
+GUIPerson::GUIPersonPopupMenu::onCmdShowPlan(FXObject*, FXSelector, void*) {
+    GUIPerson* p = dynamic_cast<GUIPerson*>(myObject);
+    if (p == nullptr) {
+        return 1;
+    }
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(*myApplication, *p);
+    // add items
+    for (int stage = 1; stage < p->getNumStages(); stage++) {
+        ret->mkItem(toString(stage).c_str(), false, p->getStageSummary(stage));
+    }
+    // close building (use an object that is not Parameterised as argument)
+    Parameterised dummyParameterised;
+    ret->closeBuilding(&dummyParameterised);
+    return 1;
+}
+
+
 long
 GUIPerson::GUIPersonPopupMenu::onCmdStartTrack(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -138,6 +136,7 @@ GUIPerson::GUIPersonPopupMenu::onCmdStartTrack(FXObject*, FXSelector, void*) {
     return 1;
 }
 
+
 long
 GUIPerson::GUIPersonPopupMenu::onCmdStopTrack(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -146,17 +145,30 @@ GUIPerson::GUIPersonPopupMenu::onCmdStopTrack(FXObject*, FXSelector, void*) {
 }
 
 
-
-
-/* -------------------------------------------------------------------------
- * GUIPerson - methods
- * ----------------------------------------------------------------------- */
-GUIPerson::GUIPerson(const SUMOVehicleParameter* pars, const MSVehicleType* vtype, MSTransportable::MSTransportablePlan* plan) :
-    MSPerson(pars, vtype, plan),
-    GUIGlObject(GLO_PERSON, pars->id),
-    myPositionInVehicle(Position::INVALID) {
+long
+GUIPerson::GUIPersonPopupMenu::onCmdRemoveObject(FXObject*, FXSelector, void*) {
+    GUIPerson* person = static_cast<GUIPerson*>(myObject);
+    MSStage* stage = person->getCurrentStage();
+    stage->abort(person);
+    stage->getEdge()->removePerson(person);
+    if (stage->getDestinationStop() != nullptr) {
+        stage->getDestinationStop()->removeTransportable(person);
+    }
+    MSNet::getInstance()->getPersonControl().erase(person);
+    myParent->update();
+    return 1;
 }
 
+// -------------------------------------------------------------------------
+// GUIPerson - methods
+// -------------------------------------------------------------------------
+
+GUIPerson::GUIPerson(const SUMOVehicleParameter* pars, MSVehicleType* vtype, MSTransportable::MSTransportablePlan* plan, const double speedFactor) :
+    MSPerson(pars, vtype, plan, speedFactor),
+    GUIGlObject(GLO_PERSON, pars->id),
+    myLock(true)
+{ }
+
 
 GUIPerson::~GUIPerson() {
     myLock.lock();
@@ -171,33 +183,35 @@ GUIPerson::~GUIPerson() {
 
 
 GUIGLObjectPopupMenu*
-GUIPerson::getPopUpMenu(GUIMainWindow& app,
-                        GUISUMOAbstractView& parent) {
-    GUIGLObjectPopupMenu* ret = new GUIPersonPopupMenu(app, parent, *this, myAdditionalVisualizations);
+GUIPerson::getPopUpMenu(GUIMainWindow& app, GUISUMOAbstractView& parent) {
+    GUIGLObjectPopupMenu* ret = new GUIPersonPopupMenu(app, parent, *this);
     buildPopupHeader(ret, app);
     buildCenterPopupEntry(ret);
     buildNameCopyPopupEntry(ret);
     buildSelectionPopupEntry(ret);
     if (hasActiveAddVisualisation(&parent, VO_SHOW_ROUTE)) {
-        new FXMenuCommand(ret, "Hide Current Route", 0, ret, MID_HIDE_CURRENTROUTE);
+        GUIDesigns::buildFXMenuCommand(ret, "Hide Current Route", nullptr, ret, MID_HIDE_CURRENTROUTE);
     } else {
-        new FXMenuCommand(ret, "Show Current Route", 0, ret, MID_SHOW_CURRENTROUTE);
+        GUIDesigns::buildFXMenuCommand(ret, "Show Current Route", nullptr, ret, MID_SHOW_CURRENTROUTE);
     }
     if (hasActiveAddVisualisation(&parent, VO_SHOW_WALKINGAREA_PATH)) {
-        new FXMenuCommand(ret, "Hide Walkingarea Path", 0, ret, MID_HIDE_WALKINGAREA_PATH);
+        GUIDesigns::buildFXMenuCommand(ret, "Hide Walkingarea Path", nullptr, ret, MID_HIDE_WALKINGAREA_PATH);
     } else {
-        new FXMenuCommand(ret, "Show Walkingarea Path", 0, ret, MID_SHOW_WALKINGAREA_PATH);
+        GUIDesigns::buildFXMenuCommand(ret, "Show Walkingarea Path", nullptr, ret, MID_SHOW_WALKINGAREA_PATH);
     }
     new FXMenuSeparator(ret);
     if (parent.getTrackedID() != getGlID()) {
-        new FXMenuCommand(ret, "Start Tracking", 0, ret, MID_START_TRACK);
+        GUIDesigns::buildFXMenuCommand(ret, "Start Tracking", nullptr, ret, MID_START_TRACK);
     } else {
-        new FXMenuCommand(ret, "Stop Tracking", 0, ret, MID_STOP_TRACK);
+        GUIDesigns::buildFXMenuCommand(ret, "Stop Tracking", nullptr, ret, MID_STOP_TRACK);
     }
+    GUIDesigns::buildFXMenuCommand(ret, "Remove", nullptr, ret, MID_REMOVE_OBJECT);
     new FXMenuSeparator(ret);
     //
     buildShowParamsPopupEntry(ret);
     buildShowTypeParamsPopupEntry(ret);
+    GUIDesigns::buildFXMenuCommand(ret, "Show Plan", GUIIconSubSys::getIcon(GUIIcon::APP_TABLE), ret, MID_SHOWPLAN);
+    new FXMenuSeparator(ret);
     buildPositionCopyEntry(ret, false);
     return ret;
 }
@@ -206,21 +220,23 @@ GUIPerson::getPopUpMenu(GUIMainWindow& app,
 GUIParameterTableWindow*
 GUIPerson::getParameterWindow(GUIMainWindow& app,
                               GUISUMOAbstractView&) {
-    GUIParameterTableWindow* ret =
-        new GUIParameterTableWindow(app, *this, 16);
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this);
     // add items
-    ret->mkItem("stage", false, getCurrentStageDescription());
-    ret->mkItem("start edge [id]", false, getFromEdge()->getID());
-    ret->mkItem("dest edge [id]", false, getDestination().getID());
-    ret->mkItem("edge [id]", false, getEdge()->getID());
+    ret->mkItem("stage", true, new FunctionBindingString<GUIPerson>(this, &MSTransportable::getCurrentStageDescription));
+    // there is always the "start" stage which we do not count here because it is not strictly part of the plan
+    ret->mkItem("stage index", true, new FunctionBindingString<GUIPerson>(this, &GUIPerson::getStageIndexDescription));
+    ret->mkItem("start edge [id]", true, new FunctionBindingString<GUIPerson>(this, &GUIPerson::getFromEdgeID));
+    ret->mkItem("dest edge [id]", true, new FunctionBindingString<GUIPerson>(this, &GUIPerson::getDestinationEdgeID));
+    ret->mkItem("arrivalPos [m]", true, new FunctionBinding<GUIPerson, double>(this, &GUIPerson::getStageArrivalPos));
+    ret->mkItem("edge [id]", true, new FunctionBindingString<GUIPerson>(this, &GUIPerson::getEdgeID));
     ret->mkItem("position [m]", true, new FunctionBinding<GUIPerson, double>(this, &GUIPerson::getEdgePos));
     ret->mkItem("speed [m/s]", true, new FunctionBinding<GUIPerson, double>(this, &GUIPerson::getSpeed));
+    ret->mkItem("speed factor", false, getSpeedFactor());
     ret->mkItem("angle [degree]", true, new FunctionBinding<GUIPerson, double>(this, &GUIPerson::getNaviDegree));
     ret->mkItem("waiting time [s]", true, new FunctionBinding<GUIPerson, double>(this, &GUIPerson::getWaitingSeconds));
-
-    ret->mkItem("parameters [key:val]", false, toString(getParameter().getMap()));
+    ret->mkItem("desired depart [s]", false, time2string(getParameter().depart));
     // close building
-    ret->closeBuilding();
+    ret->closeBuilding(&getParameter());
     return ret;
 }
 
@@ -228,8 +244,7 @@ GUIPerson::getParameterWindow(GUIMainWindow& app,
 GUIParameterTableWindow*
 GUIPerson::getTypeParameterWindow(GUIMainWindow& app,
                                   GUISUMOAbstractView&) {
-    GUIParameterTableWindow* ret =
-        new GUIParameterTableWindow(app, *this, 9);
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this);
     // add items
     ret->mkItem("Type Information:", false, "");
     ret->mkItem("type [id]", false, myVType->getID());
@@ -238,20 +253,17 @@ GUIPerson::getTypeParameterWindow(GUIMainWindow& app,
     ret->mkItem("height", false, myVType->getHeight());
     ret->mkItem("minGap", false, myVType->getMinGap());
     ret->mkItem("maximum speed [m/s]", false, myVType->getMaxSpeed());
-
-    ret->mkItem("type parameters [key:val]", false, toString(myVType->getParameter().getMap()));
     // close building
-    ret->closeBuilding();
+    ret->closeBuilding(&(myVType->getParameter()));
     return ret;
 }
 
 
-
 Boundary
 GUIPerson::getCenteringBoundary() const {
     Boundary b;
     // ensure that the vehicle is drawn, otherwise myPositionInVehicle will not be updated
-    b.add(getPosition());
+    b.add(getGUIPosition());
     b.grow(MAX2(getVehicleType().getWidth(), getVehicleType().getLength()));
     return b;
 }
@@ -261,47 +273,53 @@ void
 GUIPerson::drawGL(const GUIVisualizationSettings& s) const {
     glPushName(getGlID());
     glPushMatrix();
-    Position p1 = getPosition();
-    if (getCurrentStageType() == DRIVING && !isWaiting4Vehicle()) {
-        p1 = myPositionInVehicle;
-    }
+    Position p1 = getGUIPosition();
+    double angle = getGUIAngle();
     glTranslated(p1.x(), p1.y(), getType());
-    glRotated(90, 0, 0, 1);
     // set person color
     setColor(s);
     // scale
-    const double upscale = s.personSize.getExaggeration(s, 80);
-    glScaled(upscale, upscale, 1);
+    const double exaggeration = s.personSize.getExaggeration(s, this, 80);
+    glScaled(exaggeration, exaggeration, 1);
     switch (s.personQuality) {
         case 0:
+            GUIBasePersonHelper::drawAction_drawAsTriangle(angle, getVehicleType().getLength(), getVehicleType().getWidth());
+            break;
         case 1:
-            drawAction_drawAsTriangle(s);
+            GUIBasePersonHelper::drawAction_drawAsCircle(getVehicleType().getLength(), getVehicleType().getWidth(), s.scale * exaggeration);
             break;
         case 2:
-            drawAction_drawAsPoly(s);
+            GUIBasePersonHelper::drawAction_drawAsPoly(angle, getVehicleType().getLength(), getVehicleType().getWidth());
             break;
         case 3:
         default:
-            drawAction_drawAsImage(s);
+            GUIBasePersonHelper::drawAction_drawAsImage(angle, getVehicleType().getLength(), getVehicleType().getWidth(),
+                    getVehicleType().getImgFile(), getVehicleType().getGuiShape(), exaggeration);
             break;
     }
     glPopMatrix();
 #ifdef GUIPerson_DEBUG_DRAW_WALKINGAREA_PATHS
     drawAction_drawWalkingareaPath(s);
 #endif
-    drawName(p1, s.scale, s.personName);
+    drawName(p1, s.scale, s.personName, s.angle);
+    if (s.personValue.show) {
+        Position p2 = p1 + Position(0, 0.6 * s.personName.scaledSize(s.scale));
+        const double value = getColorValue(s, s.personColorer.getActive());
+        GLHelper::drawTextSettings(s.personValue, toString(value), p2, s.scale, s.angle, GLO_MAX - getType());
+    }
     glPopName();
 }
 
+
 void
 GUIPerson::drawAction_drawWalkingareaPath(const GUIVisualizationSettings& s) const {
     MSPersonStage_Walking* stage = dynamic_cast<MSPersonStage_Walking*>(getCurrentStage());
-    if (stage != 0) {
+    if (stage != nullptr) {
         setColor(s);
-        MSPModel_Striping::PState* stripingState = dynamic_cast<MSPModel_Striping::PState*>(stage->getPedestrianState());
-        if (stripingState != 0) {
-            MSPModel_Striping::WalkingAreaPath* waPath = stripingState->myWalkingAreaPath;
-            if (waPath != 0) {
+        MSPModel_Striping::PState* stripingState = dynamic_cast<MSPModel_Striping::PState*>(stage->getState());
+        if (stripingState != nullptr) {
+            const MSPModel_Striping::WalkingAreaPath* waPath = stripingState->myWalkingAreaPath;
+            if (waPath != nullptr) {
                 glPushMatrix();
                 glTranslated(0, 0, getType());
                 GLHelper::drawBoxLines(waPath->shape, 0.05);
@@ -311,6 +329,16 @@ GUIPerson::drawAction_drawWalkingareaPath(const GUIVisualizationSettings& s) con
     }
 }
 
+bool
+GUIPerson::isJammed() const {
+    MSPersonStage_Walking* stage = dynamic_cast<MSPersonStage_Walking*>(getCurrentStage());
+    if (stage != nullptr) {
+        return stage->getState()->isJammed();
+    }
+    return false;
+}
+
+
 void
 GUIPerson::drawGLAdditional(GUISUMOAbstractView* const parent, const GUIVisualizationSettings& s) const {
     glPushName(getGlID());
@@ -320,14 +348,14 @@ GUIPerson::drawGLAdditional(GUISUMOAbstractView* const parent, const GUIVisualiz
         drawAction_drawWalkingareaPath(s);
     }
     if (hasActiveAddVisualisation(parent, VO_SHOW_ROUTE)) {
-        if (getCurrentStageType() == MOVING_WITHOUT_VEHICLE) {
+        if (getCurrentStageType() == MSStageType::WALKING) {
             setColor(s);
             RGBColor current = GLHelper::getColor();
             RGBColor darker = current.changedBrightness(-51);
             GLHelper::setColor(darker);
             MSPersonStage_Walking* stage = dynamic_cast<MSPersonStage_Walking*>(getCurrentStage());
             assert(stage != 0);
-            const double exaggeration = s.personSize.getExaggeration(s);
+            const double exaggeration = s.personSize.getExaggeration(s, this);
             const ConstMSEdgeVector& edges = stage->getRoute();
             for (ConstMSEdgeVector::const_iterator it = edges.begin(); it != edges.end(); ++it) {
                 GUILane* lane = static_cast<GUILane*>((*it)->getLanes()[0]);
@@ -340,13 +368,17 @@ GUIPerson::drawGLAdditional(GUISUMOAbstractView* const parent, const GUIVisualiz
 }
 
 
+void
+GUIPerson::setPositionInVehicle(const GUIBaseVehicle::Seat& pos) {
+    myPositionInVehicle = pos;
+}
 
 
 void
 GUIPerson::setColor(const GUIVisualizationSettings& s) const {
     const GUIColorer& c = s.personColorer;
     if (!setFunctionalColor(c.getActive())) {
-        GLHelper::setColor(c.getScheme().getColor(getColorValue(c.getActive())));
+        GLHelper::setColor(c.getScheme().getColor(getColorValue(s, c.getActive())));
     }
 }
 
@@ -379,14 +411,14 @@ GUIPerson::setFunctionalColor(int activeScheme) const {
             }
             return false;
         }
-        case 8: { // color by angle
+        case 9: { // color by angle
             double hue = GeomHelper::naviDegree(getAngle());
             GLHelper::setColor(RGBColor::fromHSV(hue, 1., 1.));
             return true;
         }
-        case 9: { // color randomly (by pointer)
-            const double hue = (long)this % 360; // [0-360]
-            const double sat = (((long)this / 360) % 67) / 100.0 + 0.33; // [0.33-1]
+        case 10: { // color randomly (by pointer)
+            const double hue = (double)((long long int)this % 360); // [0-360]
+            const double sat = (((long long int)this / 360) % 67) / 100.0 + 0.33; // [0.33-1]
             GLHelper::setColor(RGBColor::fromHSV(hue, sat, 1.));
             return true;
         }
@@ -397,19 +429,21 @@ GUIPerson::setFunctionalColor(int activeScheme) const {
 
 
 double
-GUIPerson::getColorValue(int activeScheme) const {
+GUIPerson::getColorValue(const GUIVisualizationSettings& /* s */, int activeScheme) const {
     switch (activeScheme) {
         case 4:
             return getSpeed();
         case 5:
             if (isWaiting4Vehicle()) {
-                return 3;
+                return 5;
             } else {
                 return (double)getCurrentStageType();
             }
         case 6:
             return getWaitingSeconds();
         case 7:
+            return isJammed() ? 1 : 0;
+        case 8:
             return gSelected.isSelected(GLO_PERSON, getGlID());
     }
     return 0;
@@ -418,109 +452,141 @@ GUIPerson::getColorValue(int activeScheme) const {
 
 double
 GUIPerson::getEdgePos() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return -1;
+    }
     return MSPerson::getEdgePos();
 }
 
 
 Position
 GUIPerson::getPosition() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return Position::INVALID;
+    }
     return MSPerson::getPosition();
 }
 
 
+Position
+GUIPerson::getGUIPosition() const {
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return Position::INVALID;
+    }
+    if (getCurrentStageType() == MSStageType::DRIVING && !isWaiting4Vehicle() && myPositionInVehicle.pos != Position::INVALID) {
+        return myPositionInVehicle.pos;
+    } else {
+        return MSPerson::getPosition();
+    }
+}
+
+
+double
+GUIPerson::getGUIAngle() const {
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return INVALID_DOUBLE;
+    }
+    if (getCurrentStageType() == MSStageType::DRIVING && !isWaiting4Vehicle() && myPositionInVehicle.pos != Position::INVALID) {
+        return myPositionInVehicle.angle;
+    } else {
+        return MSPerson::getAngle();
+    }
+}
+
+
 double
 GUIPerson::getNaviDegree() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return INVALID_DOUBLE;
+    }
     return GeomHelper::naviDegree(MSPerson::getAngle());
 }
 
 
 double
 GUIPerson::getWaitingSeconds() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return -1;
+    }
     return MSPerson::getWaitingSeconds();
 }
 
 
 double
 GUIPerson::getSpeed() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return -1;
+    }
     return MSPerson::getSpeed();
 }
 
 
-void
-GUIPerson::drawAction_drawAsTriangle(const GUIVisualizationSettings& /* s */) const {
-    // draw triangle pointing forward
-    glRotated(RAD2DEG(getAngle() + PI / 2.), 0, 0, 1);
-    glScaled(getVehicleType().getLength(), getVehicleType().getWidth(), 1);
-    glBegin(GL_TRIANGLES);
-    glVertex2d(0., 0.);
-    glVertex2d(1, -0.5);
-    glVertex2d(1, 0.5);
-    glEnd();
-    // draw a smaller triangle to indicate facing
-    GLHelper::setColor(GLHelper::getColor().changedBrightness(-64));
-    glTranslated(0, 0, .045);
-    glBegin(GL_TRIANGLES);
-    glVertex2d(0., 0.);
-    glVertex2d(0.5, -0.25);
-    glVertex2d(0.5, 0.25);
-    glEnd();
-    glTranslated(0, 0, -.045);
-}
-
-
-void
-GUIPerson::drawAction_drawAsPoly(const GUIVisualizationSettings& /* s */) const {
-    // draw pedestrian shape
-    glRotated(GeomHelper::naviDegree(getAngle()) - 180, 0, 0, -1);
-    glScaled(getVehicleType().getLength(), getVehicleType().getWidth(), 1);
-    RGBColor lighter = GLHelper::getColor().changedBrightness(51);
-    glTranslated(0, 0, .045);
-    // head
-    glScaled(1, 0.5, 1.);
-    GLHelper::drawFilledCircle(0.5);
-    // nose
-    glBegin(GL_TRIANGLES);
-    glVertex2d(0.0, -0.2);
-    glVertex2d(0.0, 0.2);
-    glVertex2d(-0.6, 0.0);
-    glEnd();
-    glTranslated(0, 0, -.045);
-    // body
-    glScaled(0.9, 2.0, 1);
-    glTranslated(0, 0, .04);
-    GLHelper::setColor(lighter);
-    GLHelper::drawFilledCircle(0.5);
-    glTranslated(0, 0, -.04);
-}
-
-
-void
-GUIPerson::drawAction_drawAsImage(const GUIVisualizationSettings& s) const {
-    const std::string& file = getVehicleType().getImgFile();
-    if (file != "") {
-        if (getVehicleType().getGuiShape() == SVS_PEDESTRIAN) {
-            glRotated(RAD2DEG(getAngle() + PI / 2.), 0, 0, 1);
-        }
-        int textureID = GUITexturesHelper::getTextureID(file);
-        if (textureID > 0) {
-            const double exaggeration = s.personSize.getExaggeration(s);
-            const double halfLength = getVehicleType().getLength() / 2.0 * exaggeration;
-            const double halfWidth = getVehicleType().getWidth() / 2.0 * exaggeration;
-            GUITexturesHelper::drawTexturedBox(textureID, -halfWidth, -halfLength, halfWidth, halfLength);
-        }
-    } else {
-        // fallback if no image is defined
-        drawAction_drawAsPoly(s);
+std::string
+GUIPerson::getStageIndexDescription() const {
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return "arrived";
     }
+    return toString(getNumStages() - getNumRemainingStages()) + " of " + toString(getNumStages() - 1);
 }
 
 
-// ------------ Additional visualisations
+std::string
+GUIPerson::getEdgeID() const {
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return "arrived";
+    }
+    return  getEdge()->getID();
+}
+
+
+std::string
+GUIPerson::getFromEdgeID() const {
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return "arrived";
+    }
+    return getFromEdge()->getID();
+}
+
+
+std::string
+GUIPerson::getDestinationEdgeID() const {
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return "arrived";
+    }
+    return getDestination()->getID();
+}
+
+
+double
+GUIPerson::getStageArrivalPos() const {
+    FXMutexLock locker(myLock);
+    if (hasArrived()) {
+        return INVALID_DOUBLE;
+    }
+    return getCurrentStage()->getArrivalPos();
+}
+
+bool
+GUIPerson::proceed(MSNet* net, SUMOTime time, const bool vehicleArrived) {
+    FXMutexLock locker(myLock);
+    return MSTransportable::proceed(net, time, vehicleArrived);
+}
+
+// -------------------------------------------------------------------------
+// GUIPerson - Additional Visualsation methods
+// -------------------------------------------------------------------------
+
 bool
 GUIPerson::hasActiveAddVisualisation(GUISUMOAbstractView* const parent, int which) const {
     return myAdditionalVisualizations.find(parent) != myAdditionalVisualizations.end() && (myAdditionalVisualizations.find(parent)->second & which) != 0;
@@ -542,5 +608,11 @@ GUIPerson::removeActiveAddVisualisation(GUISUMOAbstractView* const parent, int w
     myAdditionalVisualizations[parent] &= ~which;
     parent->removeAdditionalGLVisualisation(this);
 }
-/****************************************************************************/
 
+bool
+GUIPerson::isSelected() const {
+    return gSelected.isSelected(GLO_PERSON, getGlID());
+}
+
+
+/****************************************************************************/
