@@ -1,60 +1,53 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUITrafficLightLogicWrapper.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Oct/Nov 2003
-/// @version $Id: GUITrafficLightLogicWrapper.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A wrapper for tl-logics to allow their visualisation and interaction
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cassert>
+#include <utils/geom/GeomHelper.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <utils/gui/div/GLObjectValuePassConnector.h>
 #include <utils/gui/windows/GUIAppEnum.h>
 #include <utils/gui/images/GUIIconSubSys.h>
 #include <utils/gui/div/GLHelper.h>
+#include <utils/gui/div/GUIParameterTableWindow.h>
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
 #include <utils/gui/div/GUIGlobalSelection.h>
 #include <microsim/MSLane.h>
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSTLLogicControl.h>
+#include <microsim/traffic_lights/MSActuatedTrafficLightLogic.h>
+#include <microsim/traffic_lights/MSDelayBasedTrafficLightLogic.h>
+#include <microsim/traffic_lights/MSRailSignal.h>
 #include <microsim/logging/FunctionBinding.h>
 #include <microsim/logging/FuncBinding_StringParam.h>
 #include <gui/GUIApplicationWindow.h>
 #include <gui/GUITLLogicPhasesTrackerWindow.h>
 #include <gui/GUIGlobals.h>
+#include <utils/gui/globjects/GLIncludes.h>
+#include <utils/gui/div/GUIDesigns.h>
+
 #include "GUITrafficLightLogicWrapper.h"
 #include "GUINet.h"
-#include <utils/gui/globjects/GLIncludes.h>
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // FOX callback mapping
@@ -63,6 +56,7 @@ FXDEFMAP(GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu)
 GUITrafficLightLogicWrapperPopupMenuMap[] = {
     FXMAPFUNC(SEL_COMMAND,  MID_SHOWPHASES,             GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdShowPhases),
     FXMAPFUNC(SEL_COMMAND,  MID_TRACKPHASES,            GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdBegin2TrackPhases),
+    FXMAPFUNC(SEL_COMMAND,  MID_SHOW_DETECTORS,         GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdShowDetectors),
     FXMAPFUNC(SEL_COMMAND,  MID_SWITCH_OFF,             GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdSwitchTLS2Off),
     FXMAPFUNCS(SEL_COMMAND, MID_SWITCH, MID_SWITCH + 20, GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdSwitchTLSLogic),
 };
@@ -104,6 +98,21 @@ GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdShowPhas
     return 1;
 }
 
+long
+GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdShowDetectors(
+    FXObject*, FXSelector, void*) {
+    assert(myObject->getType() == GLO_TLLOGIC);
+    GUITrafficLightLogicWrapper* w = static_cast<GUITrafficLightLogicWrapper*>(myObject);
+    MSActuatedTrafficLightLogic* act = dynamic_cast<MSActuatedTrafficLightLogic*>(&w->getTLLogic());
+    if (act == nullptr) {
+        MSDelayBasedTrafficLightLogic* db = dynamic_cast<MSDelayBasedTrafficLightLogic*>(&w->getTLLogic());
+        assert(db != 0);
+        db->setShowDetectors(!db->showDetectors());
+    } else {
+        act->setShowDetectors(!act->showDetectors());
+    }
+    return 1;
+}
 
 long
 GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdSwitchTLS2Off(
@@ -148,24 +157,37 @@ GUITrafficLightLogicWrapper::getPopUpMenu(GUIMainWindow& app,
     std::vector<MSTrafficLightLogic*> logics = vars.getAllLogics();
     if (logics.size() > 1) {
         std::vector<MSTrafficLightLogic*>::const_iterator i;
-        size_t index = 0;
+        int index = 0;
         for (i = logics.begin(); i != logics.end(); ++i, ++index) {
             if (!vars.isActive(*i)) {
-                new FXMenuCommand(ret, ("Switch to '" + (*i)->getProgramID() + "'").c_str(),
-                                  GUIIconSubSys::getIcon(ICON_FLAG_MINUS), ret, (FXSelector)(MID_SWITCH + index));
+                GUIDesigns::buildFXMenuCommand(ret, ("Switch to '" + (*i)->getProgramID() + "'").c_str(),
+                                               GUIIconSubSys::getIcon(GUIIcon::FLAG_MINUS), ret, (FXSelector)(MID_SWITCH + index));
             }
         }
         new FXMenuSeparator(ret);
     }
-    new FXMenuCommand(ret, "Switch off", GUIIconSubSys::getIcon(ICON_FLAG_MINUS), ret, MID_SWITCH_OFF);
-    new FXMenuCommand(ret, "Track Phases", 0, ret, MID_TRACKPHASES);
-    new FXMenuCommand(ret, "Show Phases", 0, ret, MID_SHOWPHASES);
+    GUIDesigns::buildFXMenuCommand(ret, "Switch off", GUIIconSubSys::getIcon(GUIIcon::FLAG_MINUS), ret, MID_SWITCH_OFF);
+    GUIDesigns::buildFXMenuCommand(ret, "Track Phases", nullptr, ret, MID_TRACKPHASES);
+    GUIDesigns::buildFXMenuCommand(ret, "Show Phases", nullptr, ret, MID_SHOWPHASES);
+    MSActuatedTrafficLightLogic* act = dynamic_cast<MSActuatedTrafficLightLogic*>(&myTLLogic);
+    if (act != nullptr) {
+        GUIDesigns::buildFXMenuCommand(ret, act->showDetectors() ? "Hide Detectors" : "Show Detectors", nullptr, ret, MID_SHOW_DETECTORS);
+    }
+    MSDelayBasedTrafficLightLogic* db = dynamic_cast<MSDelayBasedTrafficLightLogic*>(&myTLLogic);
+    if (db != nullptr) {
+        GUIDesigns::buildFXMenuCommand(ret, db->showDetectors() ? "Hide Detectors" : "Show Detectors", nullptr, ret, MID_SHOW_DETECTORS);
+    }
     new FXMenuSeparator(ret);
-    MSTrafficLightLogic* tll = myTLLogicControl.getActive(myTLLogic.getID());
+    MSTrafficLightLogic* tll = getActiveTLLogic();
     buildNameCopyPopupEntry(ret);
     buildSelectionPopupEntry(ret);
-    new FXMenuCommand(ret, ("phase: " + toString(tll->getCurrentPhaseIndex())).c_str(), 0, 0, 0);
+    GUIDesigns::buildFXMenuCommand(ret, "phase: " + toString(tll->getCurrentPhaseIndex()), nullptr, nullptr, 0);
+    const std::string& name =  tll->getCurrentPhaseDef().getName();
+    if (name != "") {
+        GUIDesigns::buildFXMenuCommand(ret, "phase name: " + name, nullptr, nullptr, 0);
+    }
     new FXMenuSeparator(ret);
+    buildShowParamsPopupEntry(ret, false);
     buildPositionCopyEntry(ret, false);
     return ret;
 }
@@ -194,9 +216,28 @@ GUITrafficLightLogicWrapper::showPhases() {
 
 
 GUIParameterTableWindow*
-GUITrafficLightLogicWrapper::getParameterWindow(GUIMainWindow&,
+GUITrafficLightLogicWrapper::getParameterWindow(GUIMainWindow& app,
         GUISUMOAbstractView&) {
-    return 0;
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this);
+    ret->mkItem("tlLogic [id]", false, myTLLogic.getID());
+    ret->mkItem("type", false, toString(myTLLogic.getLogicType()));
+    ret->mkItem("program", false, myTLLogic.getProgramID());
+    ret->mkItem("phase", true, new FunctionBinding<GUITrafficLightLogicWrapper, int>(this, &GUITrafficLightLogicWrapper::getCurrentPhase));
+    ret->mkItem("phase name", true, new FunctionBindingString<GUITrafficLightLogicWrapper>(this, &GUITrafficLightLogicWrapper::getCurrentPhaseName));
+    ret->mkItem("duration", true, new FunctionBinding<GUITrafficLightLogicWrapper, int>(this, &GUITrafficLightLogicWrapper::getCurrentDuration));
+    ret->mkItem("minDur", true, new FunctionBinding<GUITrafficLightLogicWrapper, int>(this, &GUITrafficLightLogicWrapper::getCurrentMinDur));
+    ret->mkItem("maxDur", true, new FunctionBinding<GUITrafficLightLogicWrapper, int>(this, &GUITrafficLightLogicWrapper::getCurrentMaxDur));
+    ret->mkItem("running duration", true, new FunctionBinding<GUITrafficLightLogicWrapper, int>(this, &GUITrafficLightLogicWrapper::getRunningDuration));
+    MSRailSignal* rs = dynamic_cast<MSRailSignal*>(&myTLLogic);
+    if (rs != nullptr) {
+        ret->mkItem("blocking", true, new FunctionBindingString<MSRailSignal>(rs, &MSRailSignal::getBlockingVehicleIDs));
+        ret->mkItem("rival", true, new FunctionBindingString<MSRailSignal>(rs, &MSRailSignal::getRivalVehicleIDs));
+        ret->mkItem("priority", true, new FunctionBindingString<MSRailSignal>(rs, &MSRailSignal::getPriorityVehicleIDs));
+        ret->mkItem("constraint", true, new FunctionBindingString<MSRailSignal>(rs, &MSRailSignal::getConstraintInfo));
+    }
+    // close building
+    ret->closeBuilding(&myTLLogic);
+    return ret;
 }
 
 
@@ -214,13 +255,17 @@ GUITrafficLightLogicWrapper::getCenteringBoundary() const {
     return ret;
 }
 
+const std::string
+GUITrafficLightLogicWrapper::getOptionalName() const {
+    return myTLLogic.getParameter("name", "");
+}
+
 
 void
 GUITrafficLightLogicWrapper::switchTLSLogic(int to) {
     if (to == -1) {
         myTLLogicControl.switchTo(myTLLogic.getID(), "off");
-        MSTrafficLightLogic* tll = myTLLogicControl.getActive(myTLLogic.getID());
-        GUINet::getGUIInstance()->createTLWrapper(tll);
+        GUINet::getGUIInstance()->createTLWrapper(getActiveTLLogic());
     } else {
         const MSTLLogicControl::TLSLogicVariants& vars = myTLLogicControl.get(myTLLogic.getID());
         std::vector<MSTrafficLightLogic*> logics = vars.getAllLogics();
@@ -247,10 +292,10 @@ GUITrafficLightLogicWrapper::drawGL(const GUIVisualizationSettings& s) const {
             const MSTrafficLightLogic::Phases& phases = myTLLogic.getPhases();
             int curPhaseIdx = myTLLogic.getCurrentPhaseIndex();
             int phaseIdx = (curPhaseIdx + 1) % phases.size();
-            std::vector<unsigned int> nextGreen;
+            std::vector<int> nextGreen;
             while (phaseIdx != curPhaseIdx) {
                 const std::string& state = phases[phaseIdx]->getState();
-                for (unsigned int linkIdx = 0; linkIdx < state.size(); linkIdx++) {
+                for (int linkIdx = 0; linkIdx < (int)state.size(); linkIdx++) {
                     if ((LinkState)state[linkIdx] == LINKSTATE_TL_GREEN_MINOR ||
                             (LinkState)state[linkIdx] == LINKSTATE_TL_GREEN_MAJOR) {
                         nextGreen.push_back(linkIdx);
@@ -262,14 +307,19 @@ GUITrafficLightLogicWrapper::drawGL(const GUIVisualizationSettings& s) const {
                 phaseIdx = (phaseIdx + 1) % phases.size();
             }
             // highlight nextGreen links
-            for (std::vector<unsigned int>::iterator it_idx = nextGreen.begin(); it_idx != nextGreen.end(); it_idx++) {
+            for (std::vector<int>::iterator it_idx = nextGreen.begin(); it_idx != nextGreen.end(); it_idx++) {
                 const MSTrafficLightLogic::LaneVector& lanes = myTLLogic.getLanesAt(*it_idx);
                 for (MSTrafficLightLogic::LaneVector::const_iterator it_lane = lanes.begin(); it_lane != lanes.end(); it_lane++) {
                     glPushMatrix();
-                    glColor3d(0, 1, 0);
+                    // split circle in red and yellow
                     Position pos = (*it_lane)->getShape().back();
                     glTranslated(pos.x(), pos.y(), GLO_MAX);
-                    GLHelper::drawFilledCircle((*it_lane)->getWidth() / 2.);
+                    double rot = RAD2DEG((*it_lane)->getShape().angleAt2D((int)(*it_lane)->getShape().size() - 2)) - 90;
+                    glRotated(rot, 0, 0, 1);
+                    GLHelper::setColor(s.getLinkColor(LINKSTATE_TL_RED));
+                    GLHelper::drawFilledCircle((*it_lane)->getWidth() / 2., 8, -90, 90);
+                    GLHelper::setColor(s.getLinkColor(LINKSTATE_TL_YELLOW_MAJOR));
+                    GLHelper::drawFilledCircle((*it_lane)->getWidth() / 2., 8, 90, 270);
                     glPopMatrix();
                 }
             }
@@ -277,6 +327,40 @@ GUITrafficLightLogicWrapper::drawGL(const GUIVisualizationSettings& s) const {
     }
 }
 
+MSTrafficLightLogic*
+GUITrafficLightLogicWrapper::getActiveTLLogic() const {
+    return myTLLogicControl.getActive(myTLLogic.getID());
+}
+
+int
+GUITrafficLightLogicWrapper::getCurrentPhase() const {
+    return getActiveTLLogic()->getCurrentPhaseIndex();
+}
+
+std::string
+GUITrafficLightLogicWrapper::getCurrentPhaseName() const {
+    return getActiveTLLogic()->getCurrentPhaseDef().getName();
+}
+
+int
+GUITrafficLightLogicWrapper::getCurrentDuration() const {
+    return (int)STEPS2TIME(getActiveTLLogic()->getCurrentPhaseDef().duration);
+}
+
+int
+GUITrafficLightLogicWrapper::getCurrentMinDur() const {
+    return (int)STEPS2TIME(getActiveTLLogic()->getCurrentPhaseDef().minDuration);
+}
+
+int
+GUITrafficLightLogicWrapper::getCurrentMaxDur() const {
+    return (int)STEPS2TIME(getActiveTLLogic()->getCurrentPhaseDef().maxDuration);
+}
+
+int
+GUITrafficLightLogicWrapper::getRunningDuration() const {
+    return (int)(SIMTIME - STEPS2TIME(getActiveTLLogic()->getCurrentPhaseDef().myLastSwitch));
+}
+
 
 /****************************************************************************/
-
