@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIVehicle.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIVehicle.h 20907 2016-06-07 14:06:01Z namdre $
 ///
 // A MSVehicle extended by some values for usage within the gui
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIVehicle_h
-#define GUIVehicle_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
@@ -68,7 +58,7 @@ public:
      * @exception ProcessError If a value is wrong
      */
     GUIVehicle(SUMOVehicleParameter* pars, const MSRoute* route,
-               const MSVehicleType* type, const SUMOReal speedFactor);
+               MSVehicleType* type, const double speedFactor);
 
 
     /// @brief destructor
@@ -78,7 +68,7 @@ public:
      *
      * @note implementation of abstract method does not work otherwise
      */
-    Position getPosition(const SUMOReal offset = 0) const {
+    Position getPosition(const double offset = 0) const {
         return MSVehicle::getPosition(offset);
     }
 
@@ -86,18 +76,17 @@ public:
      *
      * @note implementation of abstract method does not work otherwise
      */
-    SUMOReal getAngle() const {
+    double getAngle() const {
         return MSVehicle::getAngle();
     }
 
     /** @brief Draws the route
      * @param[in] r The route to draw
      */
-    void drawRouteHelper(const MSRoute& r, SUMOReal exaggeration) const;
+    void drawRouteHelper(const GUIVisualizationSettings& s, const MSRoute& r, bool future, bool noLoop, const RGBColor& col) const;
 
-    void drawAction_drawVehicleBlinker(SUMOReal length) const;
-    void drawAction_drawVehicleBrakeLight(SUMOReal length, bool onlyOne = 1) const;
-    void drawAction_drawPersonsAndContainers(const GUIVisualizationSettings& s) const;
+    void drawAction_drawVehicleBlinker(double length) const;
+    void drawAction_drawVehicleBrakeLight(double length, bool onlyOne = false) const;
     void drawAction_drawLinkItems(const GUIVisualizationSettings& s) const;
     void drawAction_drawVehicleBlueLight() const;
 
@@ -105,7 +94,7 @@ public:
      * @see MSVehicle::myLastLaneChangeOffset
      * @return The time since the last lane change in seconds
      */
-    SUMOReal getLastLaneChangeOffset() const;
+    double getLastLaneChangeOffset() const;
 
 
     /** @brief Draws the vehicle's best lanes
@@ -117,7 +106,7 @@ public:
     void selectBlockingFoes() const;
 
     /// @brief gets the color value according to the current scheme index
-    SUMOReal getColorValue(size_t activeScheme) const;
+    double getColorValue(const GUIVisualizationSettings& s, int activeScheme) const;
 
     /** @brief Returns an own parameter window
      *
@@ -136,21 +125,53 @@ public:
      */
     GUIParameterTableWindow* getTypeParameterWindow(GUIMainWindow& app, GUISUMOAbstractView& parent);
 
+    /// @brief whether this vehicle is selected in the GUI
+    bool isSelected() const;
+
+    /// @brief return right vehicle side on current edge (without argument)
+    double getRightSideOnEdge2() const {
+        return getRightSideOnEdge();
+    }
+    /// @brief return left vehicle side on current edge
+    double getLeftSideOnEdge() const {
+        return getRightSideOnEdge() + getVehicleType().getWidth();
+    }
+
+    /// @brief return the righmost sublane on the edge occupied by the vehicle
+    int getRightSublaneOnEdge() const;
+    int getLeftSublaneOnEdge() const;
+
+    /// @brief return the lanechange state
+    std::string getLCStateRight() const;
+    std::string getLCStateLeft() const;
+    std::string getLCStateCenter() const;
+
+    /// @brief return vehicle lane id
+    std::string getLaneID() const;
+    std::string getBackLaneID() const;
+    std::string getShadowLaneID() const;
+    std::string getTargetLaneID() const;
+
+    /// @brief return the lane-change maneuver distance
+    double getManeuverDist() const;
+    /// @brief return the speed mode as bit string
+    std::string getSpeedMode() const;
+    /// @brief return the lane change mode as bit string
+    std::string getLaneChangeMode() const;
+
+    /// @brief handle route to accomodate to given stop
+    void rerouteDRTStop(MSStoppingPlace* busStop);
+
 protected:
     /// @brief register vehicle for drawing while outside the network
-    void drawOutsideNetwork(bool /*add*/) const;
+    void drawOutsideNetwork(bool add);
 
 private:
 
     /* @brief draw train with individual carriages. The number of carriages is
      * determined from defaultLength of carriages and vehicle length
      * passengerSeats are computed beginning at firstPassengerCarriage */
-    void drawAction_drawRailCarriages(const GUIVisualizationSettings& s, SUMOReal defaultLength, SUMOReal carriageGap,
-                                      int firstPassengerCarriage, bool asImage) const;
-    /// @}
-
-    /// @brief draws the given guiShape if it has distinct carriages/modules and returns true if so
-    bool drawAction_drawCarriageClass(const GUIVisualizationSettings& s, SUMOVehicleShape guiShape, bool asImage) const;
+    void drawAction_drawCarriageClass(const GUIVisualizationSettings& s, bool asImage) const;
 
     /* @brief return the previous lane in this vehicles route including internal lanes
      * @param[in] current The lane of which the predecessor should be returned
@@ -158,19 +179,7 @@ private:
      */
     MSLane* getPreviousLane(MSLane* current, int& furtherIndex) const;
 
-    /// @brief return the number of passengers
-    int getNumPassengers() const;
-
-    /// @brief add seats to mySeatPositions and update requiredSeats
-    void computeSeats(const Position& front, const Position& back, int& requiredSeats) const;
-
     /// @brief retrieve information about the current stop state
     std::string getStopInfo() const;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
