@@ -1,48 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2003-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIVehicleControl.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Wed, 10. Dec 2003
-/// @version $Id: GUIVehicleControl.h 17262 2014-11-04 20:41:44Z behrisch $
 ///
 // The class responsible for building and deletion of vehicles (gui-version)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2003-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIVehicleControl_h
-#define GUIVehicleControl_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
+#include <utils/foxtools/fxheader.h>
 #include <microsim/MSVehicleControl.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 
 
-// ===========================================================================
-// class declarations
-// ===========================================================================
-class MFXMutex;
-
-
 // ===========================================================================
 // class definitions
 // ===========================================================================
@@ -82,7 +67,7 @@ public:
      * @see MSVehicleControl::buildVehicle
      */
     SUMOVehicle* buildVehicle(SUMOVehicleParameter* defs,
-                              const MSRoute* route, const MSVehicleType* type,
+                              const MSRoute* route, MSVehicleType* type,
                               const bool ignoreStopErrors, const bool fromRouteFile = true);
     /// @}
 
@@ -110,11 +95,21 @@ public:
 
     /** @brief Returns the list of all known vehicles by gl-id
      * @param[fill] into The list to fill with vehicle ids
+     * @param[listParking] Whether parking vehicles shall be listed as well
+     * @param[listTeleporting] Whether teleporting vehicles shall be listed as well
      * @todo Well, what about concurrent modifications?
      */
-    void insertVehicleIDs(std::vector<GUIGlID>& into);
+    void insertVehicleIDs(std::vector<GUIGlID>& into, bool listParking, bool listTeleporting);
 
 
+    /** @brief Returns the number of halting vehicles
+     * @return The number of halting vehicles
+     */
+    virtual int getHaltingVehicleNo() const;
+
+    /// @brief get current absolute and relative mean vehicle speed in the network
+    virtual std::pair<double, double> getVehicleMeanSpeeds() const;
+
     /// @brief lock access to vehicle removal/additions for thread synchronization
     void secureVehicles();
 
@@ -124,7 +119,7 @@ public:
 
 private:
     /// The mutex used to avoid concurrent updates of the vehicle buffer
-    mutable MFXMutex myLock;
+    mutable FXMutex myLock;
 
 
 private:
@@ -136,9 +131,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
