@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    ROJTREdge.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Yun-Pang Floetteroed
 /// @date    Tue, 20 Jan 2004
-/// @version $Id: ROJTREdge.h 21182 2016-07-18 06:46:01Z behrisch $
 ///
 // An edge the jtr-router may route through
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef ROJTREdge_h
-#define ROJTREdge_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -81,7 +71,7 @@ public:
      * @param[in] s The following edge
      * @see ROEdge::addFollower
      */
-    void addSuccessor(ROEdge* s, std::string dir = "");
+    void addSuccessor(ROEdge* s, ROEdge* via = nullptr, std::string dir = "");
 
 
     /** @brief adds the information about the percentage of using a certain follower
@@ -92,7 +82,7 @@ public:
      * @param[in] probability The probability to use the given follower
      */
     void addFollowerProbability(ROJTREdge* follower,
-                                SUMOReal begTime, SUMOReal endTime, SUMOReal probability);
+                                double begTime, double endTime, double probability);
 
 
     /** @brief Returns the next edge to use
@@ -101,25 +91,37 @@ public:
      * @param[in] avoid The set of edges to avoid
      * @return The chosen edge
      */
-    ROJTREdge* chooseNext(const ROVehicle* const veh, SUMOReal time, const std::set<const ROEdge*>& avoid) const;
+    ROJTREdge* chooseNext(const ROVehicle* const veh, double time, const std::set<const ROEdge*>& avoid) const;
 
 
     /** @brief Sets the turning definition defaults
      * @param[in] def The turning percentage defaults
      */
-    void setTurnDefaults(const std::vector<SUMOReal>& defs);
+    void setTurnDefaults(const std::vector<double>& defs);
 
+    /// @brief register source flow on this edge
+    int getSourceFlow() const {
+        return mySourceFlows;
+    }
+
+    /// @brief register flow on this edge
+    void changeSourceFlow(int value) {
+        mySourceFlows += value;
+    }
 
 private:
     /// @brief Definition of a map that stores the probabilities of using a certain follower over time
-    typedef std::map<ROJTREdge*, ValueTimeLine<SUMOReal>*, Named::ComparatorIdLess> FollowerUsageCont;
+    typedef std::map<ROJTREdge*, ValueTimeLine<double>*, ComparatorIdLess> FollowerUsageCont;
 
     /// @brief Storage for the probabilities of using a certain follower over time
     FollowerUsageCont myFollowingDefs;
 
     /// @brief The defaults for turnings
-    std::vector<SUMOReal> myParsedTurnings;
+    std::vector<double> myParsedTurnings;
 
+    /// @brief the flows departing from this edge in the given time
+    //ValueTimeLine<int> mySourceFlows;
+    int mySourceFlows;
 
 private:
     /// @brief invalidated copy constructor
@@ -130,9 +132,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
