@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    ROJTRTurnDefLoader.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Jan 2004
-/// @version $Id: ROJTRTurnDefLoader.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Loader for the of turning percentages and source/sink definitions
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef ROJTRTurnDefLoader_h
-#define ROJTRTurnDefLoader_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <set>
 #include <string>
@@ -116,22 +106,28 @@ private:
      */
     void addToEdge(const SUMOSAXAttributes& attrs);
 
+    /** @brief Parses the probability to use a certain incoming-outgoing edge relation
+     * @param[in] attrs The SAX-attributes to parse the destination edge and the probability to use it from
+     */
+    void addEdgeRel(const SUMOSAXAttributes& attrs);
 
 private:
     /// @brief The network to set the information into
     RONet& myNet;
 
     /// @brief The begin and the end of the current interval
-    SUMOReal myIntervalBegin, myIntervalEnd;
+    double myIntervalBegin, myIntervalEnd;
 
     /// @brief The current incoming edge the turning probabilities are set into
     ROJTREdge* myEdge;
 
+    /// @brief whether all sources are sinks
+    bool mySourcesAreSinks;
+
+    /// @brief whether upstream flows should be discounted from source flows
+    bool myDiscountSources;
+
+    /// @brief whether the warning for the deprecated format has been issued
+    bool myHaveWarnedAboutDeprecatedFormat;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
