@@ -1,38 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    ROMAAssignments.h
 /// @author  Yun-Pang Floetteroed
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    Feb 2013
-/// @version $Id: ROMAAssignments.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Assignment methods
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef ROMAAssignments_h
-#define ROMAAssignments_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <utils/vehicle/SUMOAbstractRouter.h>
+#include <utils/router/SUMOAbstractRouter.h>
 #include <utils/common/SUMOTime.h>
 
 // ===========================================================================
@@ -59,7 +49,8 @@ class ROMAAssignments {
 public:
     /// Constructor
     ROMAAssignments(const SUMOTime begin, const SUMOTime end, const bool additiveTraffic,
-                    const double adaptionFactor, RONet& net, ODMatrix& matrix, SUMOAbstractRouter<ROEdge, ROVehicle>& router);
+                    const double adaptionFactor, const int maxAlternatives,
+                    RONet& net, ODMatrix& matrix, SUMOAbstractRouter<ROEdge, ROVehicle>& router);
 
     /// Destructor
     ~ROMAAssignments();
@@ -95,7 +86,7 @@ public:
      * @param[in] v The (default) vehicle that is routed
      * @param[in] t The time for which the effort shall be returned
      * @return The effort (time to pass in this case) for an edge
-     * @see DijkstraRouterTT_ByProxi
+     * @see DijkstraRouter_ByProxi
      */
     static double getPenalizedEffort(const ROEdge* const e, const ROVehicle* const v, double t);
 
@@ -108,7 +99,7 @@ public:
      * @param[in] v The (default) vehicle that is routed
      * @param[in] t The time for which the effort shall be returned
      * @return The effort (time to pass in this case) for an edge
-     * @see DijkstraRouterTT_ByProxi
+     * @see DijkstraRouter_ByProxi
      */
     static double getPenalizedTT(const ROEdge* const e, const ROVehicle* const v, double t);
 
@@ -121,13 +112,15 @@ public:
      * @param[in] v The (default) vehicle that is routed
      * @param[in] t The time for which the effort shall be returned
      * @return The effort (time to pass in this case) for an edge
-     * @see DijkstraRouterTT_ByProxi
+     * @see DijkstraRouter_ByProxi
      */
     static double getTravelTime(const ROEdge* const e, const ROVehicle* const v, double t);
 
 private:
     /// @brief add a route and check for duplicates
-    bool addRoute(ConstROEdgeVector& edges, std::vector<RORoute*>& paths, std::string routeId, double prob);
+    bool addRoute(const ConstROEdgeVector& edges, std::vector<RORoute*>& paths, std::string routeId, double prob);
+
+    const ConstROEdgeVector computePath(ODCell* cell, const SUMOTime time = 0, const double probability = 0., SUMOAbstractRouter<ROEdge, ROVehicle>* router = nullptr);
 
     /// @brief get the k shortest paths
     void getKPaths(const int kPaths, const double penalty);
@@ -137,6 +130,7 @@ private:
     const SUMOTime myEnd;
     const bool myAdditiveTraffic;
     const double myAdaptionFactor;
+    const int myMaxAlternatives;
     RONet& myNet;
     ODMatrix& myMatrix;
     SUMOAbstractRouter<ROEdge, ROVehicle>& myRouter;
@@ -168,4 +162,3 @@ private:
 
 };
 
-#endif
