@@ -1,48 +1,40 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    marouter_main.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    Thu, 06 Jun 2002
-/// @version $Id: marouter_main.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Main for MAROUTER
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #ifdef HAVE_VERSION_H
 #include <version.h>
 #endif
 
-#include <xercesc/sax/SAXException.hpp>
-#include <xercesc/sax/SAXParseException.hpp>
-#include <utils/common/TplConvert.h>
 #include <iostream>
 #include <string>
 #include <limits.h>
 #include <ctime>
 #include <vector>
+#include <xercesc/sax/SAXException.hpp>
+#include <xercesc/sax/SAXParseException.hpp>
+#include <utils/common/FileHelpers.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/SystemFrame.h>
@@ -50,15 +42,15 @@
 #include <utils/common/ToString.h>
 #include <utils/distribution/Distribution_Points.h>
 #include <utils/iodevices/OutputDevice.h>
+#include <utils/iodevices/OutputDevice_String.h>
 #include <utils/options/Option.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/options/OptionsIO.h>
-#include <utils/vehicle/RouteCostCalculator.h>
-#include <utils/vehicle/DijkstraRouterTT.h>
-#include <utils/vehicle/DijkstraRouterEffort.h>
-#include <utils/vehicle/AStarRouter.h>
-#include <utils/vehicle/CHRouter.h>
-#include <utils/vehicle/CHRouterWrapper.h>
+#include <utils/router/RouteCostCalculator.h>
+#include <utils/router/DijkstraRouter.h>
+#include <utils/router/AStarRouter.h>
+#include <utils/router/CHRouter.h>
+#include <utils/router/CHRouterWrapper.h>
 #include <utils/xml/XMLSubSys.h>
 #include <od/ODCell.h>
 #include <od/ODDistrict.h>
@@ -77,13 +69,20 @@
 #include "ROMARouteHandler.h"
 #include "ROMAEdge.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
+// ===========================================================================
+// method declaration
+// ===========================================================================
+
+void initNet(RONet& net, ROLoader& loader, OptionsCont& oc);
+double getTravelTime(const ROEdge* const edge, const ROVehicle* const veh, double time);
+void computeAllPairs(RONet& net, OptionsCont& oc);
+void writeInterval(OutputDevice& dev, const SUMOTime begin, const SUMOTime end, const RONet& net, const ROVehicle* const veh);
+void writeInterval(OutputDevice& dev, const SUMOTime begin, const SUMOTime end, const RONet& net, const ROVehicle* const veh);
+void computeRoutes(RONet& net, OptionsCont& oc, ODMatrix& matrix);
 
 
 // ===========================================================================
-// functions
+// Method implementation
 // ===========================================================================
 /* -------------------------------------------------------------------------
  * data processing methods
@@ -103,7 +102,7 @@ initNet(RONet& net, ROLoader& loader, OptionsCont& oc) {
     /* const SUMOTime begin = string2time(oc.getString("begin"));
     const SUMOTime end = string2time(oc.getString("end"));
     for (std::map<std::string, ROEdge*>::const_iterator i = net.getEdgeMap().begin(); i != net.getEdgeMap().end(); ++i) {
-        (*i).second->addTravelTime(STEPS2TIME(begin), STEPS2TIME(end), (*i).second->getLength() / (*i).second->getSpeed());
+        (*i).second->addTravelTime(STEPS2TIME(begin), STEPS2TIME(end), (*i).second->getLength() / (*i).second->getSpeedLimit());
     }*/
     // load the weights when wished/available
     if (oc.isSet("weight-files")) {
@@ -114,9 +113,10 @@ initNet(RONet& net, ROLoader& loader, OptionsCont& oc) {
     }
 }
 
-SUMOReal
-getTravelTime(const ROEdge* const edge, const ROVehicle* const /* veh */, SUMOReal /* time */) {
-    return edge->getLength() / edge->getSpeed();
+
+double
+getTravelTime(const ROEdge* const edge, const ROVehicle* const /* veh */, double /* time */) {
+    return edge->getLength() / edge->getSpeedLimit();
 }
 
 
@@ -125,19 +125,31 @@ getTravelTime(const ROEdge* const edge, const ROVehicle* const /* veh */, SUMORe
  */
 void
 computeAllPairs(RONet& net, OptionsCont& oc) {
-    std::ofstream outFile(oc.getString("all-pairs-output").c_str(), std::ios::binary);
+    OutputDevice::createDeviceByOption("all-pairs-output");
+    OutputDevice& outFile = OutputDevice::getDeviceByOption("all-pairs-output");
     // build the router
-    typedef DijkstraRouterTT<ROEdge, ROVehicle, noProhibitions<ROEdge, ROVehicle> > Dijkstra;
+    typedef DijkstraRouter<ROEdge, ROVehicle> Dijkstra;
     Dijkstra router(ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &getTravelTime);
     ConstROEdgeVector into;
     const int numInternalEdges = net.getInternalEdgeNumber();
-    const int numTotalEdges = (int)net.getEdgeNo();
+    const int numTotalEdges = (int)net.getEdgeNumber();
     for (int i = numInternalEdges; i < numTotalEdges; i++) {
         const Dijkstra::EdgeInfo& ei = router.getEdgeInfo(i);
-        if (ei.edge->getFunc() != ROEdge::ET_INTERNAL) {
-            router.compute(ei.edge, 0, 0, 0, into);
+        if (!ei.edge->isInternal()) {
+            router.compute(ei.edge, nullptr, nullptr, 0, into);
+            double fromEffort = router.getEffort(ei.edge, nullptr, 0);
             for (int j = numInternalEdges; j < numTotalEdges; j++) {
-                FileHelpers::writeFloat(outFile, router.getEdgeInfo(j).traveltime);
+                double heuTT = router.getEdgeInfo(j).effort - fromEffort;
+                outFile << heuTT;
+                /*
+                if (heuTT >
+                        ei.edge->getDistanceTo(router.getEdgeInfo(j).edge)
+                        && router.getEdgeInfo(j).traveltime != std::numeric_limits<double>::max()
+                        ) {
+                    std::cout << " heuristic failure: from=" << ei.edge->getID() << " to=" << router.getEdgeInfo(j).edge->getID()
+                    << " fromEffort=" << fromEffort << " heuTT=" << heuTT << " airDist=" << ei.edge->getDistanceTo(router.getEdgeInfo(j).edge) << "\n";
+                }
+                */
             }
         }
     }
@@ -152,10 +164,10 @@ writeInterval(OutputDevice& dev, const SUMOTime begin, const SUMOTime end, const
     dev.openTag(SUMO_TAG_INTERVAL).writeAttr(SUMO_ATTR_BEGIN, time2string(begin)).writeAttr(SUMO_ATTR_END, time2string(end));
     for (std::map<std::string, ROEdge*>::const_iterator i = net.getEdgeMap().begin(); i != net.getEdgeMap().end(); ++i) {
         ROMAEdge* edge = static_cast<ROMAEdge*>(i->second);
-        if (edge->getFunc() == ROEdge::ET_NORMAL) {
+        if (edge->getFunction() == SumoXMLEdgeFunc::NORMAL) {
             dev.openTag(SUMO_TAG_EDGE).writeAttr(SUMO_ATTR_ID, edge->getID());
-            const SUMOReal traveltime = edge->getTravelTime(veh, STEPS2TIME(begin));
-            const SUMOReal flow = edge->getFlow(STEPS2TIME(begin));
+            const double traveltime = edge->getTravelTime(veh, STEPS2TIME(begin));
+            const double flow = edge->getFlow(STEPS2TIME(begin));
             dev.writeAttr("traveltime", traveltime);
             dev.writeAttr("speed", edge->getLength() / traveltime);
             dev.writeAttr("entered", flow);
@@ -173,73 +185,48 @@ writeInterval(OutputDevice& dev, const SUMOTime begin, const SUMOTime end, const
 void
 computeRoutes(RONet& net, OptionsCont& oc, ODMatrix& matrix) {
     // build the router
-    SUMOAbstractRouter<ROEdge, ROVehicle>* router = 0;
+    SUMOAbstractRouter<ROEdge, ROVehicle>* router = nullptr;
     const std::string measure = oc.getString("weight-attribute");
     const std::string routingAlgorithm = oc.getString("routing-algorithm");
-    const SUMOTime begin = string2time(oc.getString("begin"));
-    const SUMOTime end = string2time(oc.getString("end"));
-    if (measure == "traveltime") {
+    const double priorityFactor = oc.getFloat("weights.priority-factor");
+    SUMOTime begin = string2time(oc.getString("begin"));
+    SUMOTime end = string2time(oc.getString("end"));
+    if (oc.isDefault("begin") && matrix.getBegin() >= 0) {
+        begin = matrix.getBegin();
+    }
+    if (oc.isDefault("end") && matrix.getEnd() >= 0) {
+        end = matrix.getEnd();
+    }
+    DijkstraRouter<ROEdge, ROVehicle>::Operation ttOp = oc.getInt("paths") > 1 ? &ROMAAssignments::getPenalizedTT : &ROEdge::getTravelTimeStatic;
+    if (measure == "traveltime" && priorityFactor == 0) {
         if (routingAlgorithm == "dijkstra") {
-            if (net.hasPermissions()) {
-                if (oc.getInt("paths") > 1) {
-                    router = new DijkstraRouterTT<ROEdge, ROVehicle, prohibited_withPermissions<ROEdge, ROVehicle> >(
-                        ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROMAAssignments::getPenalizedTT);
-                } else {
-                    router = new DijkstraRouterTT<ROEdge, ROVehicle, prohibited_withPermissions<ROEdge, ROVehicle> >(
-                        ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROEdge::getTravelTimeStatic);
-                }
-            } else {
-                if (oc.getInt("paths") > 1) {
-                    router = new DijkstraRouterTT<ROEdge, ROVehicle, noProhibitions<ROEdge, ROVehicle> >(
-                        ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROMAAssignments::getPenalizedTT);
-                } else {
-                    router = new DijkstraRouterTT<ROEdge, ROVehicle, noProhibitions<ROEdge, ROVehicle> >(
-                        ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROEdge::getTravelTimeStatic);
-                }
-            }
+            router = new DijkstraRouter<ROEdge, ROVehicle>(ROEdge::getAllEdges(), oc.getBool("ignore-errors"), ttOp, nullptr, false, nullptr, net.hasPermissions());
         } else if (routingAlgorithm == "astar") {
-            if (net.hasPermissions()) {
-                if (oc.getInt("paths") > 1) {
-                    router = new AStarRouter<ROEdge, ROVehicle, prohibited_withPermissions<ROEdge, ROVehicle> >(
-                        ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROMAAssignments::getPenalizedTT);
-                } else {
-                    router = new AStarRouter<ROEdge, ROVehicle, prohibited_withPermissions<ROEdge, ROVehicle> >(
-                        ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROEdge::getTravelTimeStatic);
-                }
-            } else {
-                if (oc.getInt("paths") > 1) {
-                    router = new AStarRouter<ROEdge, ROVehicle, noProhibitions<ROEdge, ROVehicle> >(
-                        ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROMAAssignments::getPenalizedTT);
-                } else {
-                    router = new AStarRouter<ROEdge, ROVehicle, noProhibitions<ROEdge, ROVehicle> >(
-                        ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROEdge::getTravelTimeStatic);
-                }
-            }
+            router = new AStarRouter<ROEdge, ROVehicle>(ROEdge::getAllEdges(), oc.getBool("ignore-errors"), ttOp, nullptr, net.hasPermissions());
         } else if (routingAlgorithm == "CH") {
             const SUMOTime weightPeriod = (oc.isSet("weight-files") ?
                                            string2time(oc.getString("weight-period")) :
-                                           std::numeric_limits<int>::max());
-            if (net.hasPermissions()) {
-                router = new CHRouter<ROEdge, ROVehicle, prohibited_withPermissions<ROEdge, ROVehicle> >(
-                    ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROEdge::getTravelTimeStatic, SVC_IGNORING, weightPeriod, true);
-            } else {
-                router = new CHRouter<ROEdge, ROVehicle, noProhibitions<ROEdge, ROVehicle> >(
-                    ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROEdge::getTravelTimeStatic, SVC_IGNORING, weightPeriod, false);
-            }
+                                           SUMOTime_MAX);
+            router = new CHRouter<ROEdge, ROVehicle>(ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROEdge::getTravelTimeStatic, SVC_IGNORING, weightPeriod, net.hasPermissions(), false);
         } else if (routingAlgorithm == "CHWrapper") {
             const SUMOTime weightPeriod = (oc.isSet("weight-files") ?
                                            string2time(oc.getString("weight-period")) :
-                                           std::numeric_limits<int>::max());
-
-            router = new CHRouterWrapper<ROEdge, ROVehicle, prohibited_withPermissions<ROEdge, ROVehicle> >(
-                ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROEdge::getTravelTimeStatic, begin, weightPeriod);
+                                           SUMOTime_MAX);
+            router = new CHRouterWrapper<ROEdge, ROVehicle>(
+                ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROEdge::getTravelTimeStatic,
+                begin, end, weightPeriod, oc.getInt("routing-threads"));
         } else {
             throw ProcessError("Unknown routing Algorithm '" + routingAlgorithm + "'!");
         }
-
     } else {
-        DijkstraRouterEffort<ROEdge, ROVehicle, prohibited_withPermissions<ROEdge, ROVehicle> >::Operation op;
-        if (measure == "CO") {
+        DijkstraRouter<ROEdge, ROVehicle>::Operation op;
+        if (measure == "traveltime") {
+            if (ROEdge::initPriorityFactor(priorityFactor)) {
+                op = &ROEdge::getTravelTimeStaticPriorityFactor;
+            } else {
+                op = &ROEdge::getTravelTimeStatic;
+            }
+        } else if (measure == "CO") {
             op = &ROEdge::getEmissionEffort<PollutantsInterface::CO>;
         } else if (measure == "CO2") {
             op = &ROEdge::getEmissionEffort<PollutantsInterface::CO2>;
@@ -256,45 +243,38 @@ computeRoutes(RONet& net, OptionsCont& oc, ODMatrix& matrix) {
         } else if (measure == "noise") {
             op = &ROEdge::getNoiseEffort;
         } else {
-            throw ProcessError("Unknown measure (weight attribute '" + measure + "')!");
+            op = &ROEdge::getStoredEffort;
         }
-        if (net.hasPermissions()) {
-            if (oc.getInt("paths") > 1) {
-                router = new DijkstraRouterEffort<ROEdge, ROVehicle, prohibited_withPermissions<ROEdge, ROVehicle> >(
-                    ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROMAAssignments::getPenalizedEffort, &ROMAAssignments::getTravelTime);
-            } else {
-                router = new DijkstraRouterEffort<ROEdge, ROVehicle, prohibited_withPermissions<ROEdge, ROVehicle> >(
-                    ROEdge::getAllEdges(), oc.getBool("ignore-errors"), op, &ROEdge::getTravelTimeStatic);
-            }
-        } else {
-            if (oc.getInt("paths") > 1) {
-                router = new DijkstraRouterEffort<ROEdge, ROVehicle, noProhibitions<ROEdge, ROVehicle> >(
-                    ROEdge::getAllEdges(), oc.getBool("ignore-errors"), &ROMAAssignments::getPenalizedEffort, &ROMAAssignments::getTravelTime);
-            } else {
-                router = new DijkstraRouterEffort<ROEdge, ROVehicle, noProhibitions<ROEdge, ROVehicle> >(
-                    ROEdge::getAllEdges(), oc.getBool("ignore-errors"), op, &ROEdge::getTravelTimeStatic);
-            }
+        if (measure != "traveltime" && !net.hasLoadedEffort()) {
+            WRITE_WARNING("No weight data was loaded for attribute '" + measure + "'.");
         }
+        router = new DijkstraRouter<ROEdge, ROVehicle>(ROEdge::getAllEdges(), oc.getBool("ignore-errors"), op, ttOp, false, nullptr, net.hasPermissions());
     }
     try {
+        const RORouterProvider provider(router, nullptr, nullptr, nullptr);
         // prepare the output
-        net.openOutput(oc.isSet("output-file") ? oc.getString("output-file") : "", "", "");
+        net.openOutput(oc);
         // process route definitions
         if (oc.isSet("timeline")) {
             matrix.applyCurve(matrix.parseTimeLine(oc.getStringVector("timeline"), oc.getBool("timeline.day-in-hours")));
         }
         matrix.sortByBeginTime();
-        ROVehicle defaultVehicle(SUMOVehicleParameter(), 0, net.getVehicleTypeSecure(DEFAULT_VTYPE_ID), &net);
-        ROMAAssignments a(begin, end, oc.getBool("additive-traffic"), oc.getFloat("weight-adaption"), net, matrix, *router);
+        ROVehicle defaultVehicle(SUMOVehicleParameter(), nullptr, net.getVehicleTypeSecure(DEFAULT_VTYPE_ID), &net);
+        ROMAAssignments a(begin, end, oc.getBool("additive-traffic"), oc.getFloat("weight-adaption"), oc.getInt("max-alternatives"), net, matrix, *router);
         a.resetFlows();
 #ifdef HAVE_FOX
-        const RORouterProvider provider(router, 0, 0);
+        // this is just to init the CHRouter with the default vehicle
+        router->reset(&defaultVehicle);
         const int maxNumThreads = oc.getInt("routing-threads");
         while ((int)net.getThreadPool().size() < maxNumThreads) {
             new RONet::WorkerThread(net.getThreadPool(), provider);
         }
 #endif
-        const std::string assignMethod = oc.getString("assignment-method");
+        std::string assignMethod = oc.getString("assignment-method");
+        if (assignMethod == "UE") {
+            WRITE_WARNING("Deterministic user equilibrium ('UE') is not implemented yet, using stochastic method ('SUE').");
+            assignMethod = "SUE";
+        }
         if (assignMethod == "incremental") {
             a.incremental(oc.getInt("max-iterations"), oc.getBool("verbose"));
         } else if (assignMethod == "SUE") {
@@ -304,38 +284,75 @@ computeRoutes(RONet& net, OptionsCont& oc, ODMatrix& matrix) {
         // update path costs and output
         bool haveOutput = false;
         OutputDevice* dev = net.getRouteOutput();
-        if (dev != 0) {
+        if (dev != nullptr) {
+            std::vector<std::string> tazParamKeys;
+            if (oc.isSet("taz-param")) {
+                tazParamKeys = oc.getStringVector("taz-param");
+            }
+            std::map<SUMOTime, std::string> sortedOut;
+            SUMOTime lastEnd = -1;
             int num = 0;
-            for (std::vector<ODCell*>::const_iterator i = matrix.getCells().begin(); i != matrix.getCells().end(); ++i) {
-                const ODCell* const c = *i;
-                if (c->departures.empty()) {
-                    dev->openTag(SUMO_TAG_FLOW).writeAttr(SUMO_ATTR_ID, oc.getString("prefix") + toString(num++));
-                    dev->writeAttr(SUMO_ATTR_BEGIN, time2string(c->begin)).writeAttr(SUMO_ATTR_END, time2string(c->end));
-                    dev->writeAttr(SUMO_ATTR_NUMBER, int(c->vehicleNumber));
-                    matrix.writeDefaultAttrs(*dev, oc.getBool("ignore-vehicle-type"), c);
-                    dev->openTag(SUMO_TAG_ROUTE_DISTRIBUTION);
-                    for (std::vector<RORoute*>::const_iterator j = c->pathsVector.begin(); j != c->pathsVector.end(); ++j) {
-                        (*j)->setCosts(router->recomputeCosts((*j)->getEdgeVector(), &defaultVehicle, string2time(oc.getString("begin"))));
-                        (*j)->writeXMLDefinition(*dev, 0, true, false);
+            for (const ODCell* const c : matrix.getCells()) {
+                if (c->begin >= end || c->end <= begin ||
+                        c->pathsVector.empty() || c->pathsVector.front()->getEdgeVector().empty()) {
+                    continue;
+                }
+                if (lastEnd >= 0 && lastEnd <= c->begin) {
+                    for (std::map<SUMOTime, std::string>::const_iterator desc = sortedOut.begin(); desc != sortedOut.end(); ++desc) {
+                        dev->writePreformattedTag(desc->second);
                     }
-                    dev->closeTag();
-                    dev->closeTag();
+                    sortedOut.clear();
+                }
+                if (c->departures.empty()) {
+                    const SUMOTime b = MAX2(begin, c->begin);
+                    const SUMOTime e = MIN2(end, c->end);
+                    const int numVehs = int(c->vehicleNumber * (e - b) / (c->end - c->begin));
+                    OutputDevice_String od(1);
+                    od.openTag(SUMO_TAG_FLOW).writeAttr(SUMO_ATTR_ID, oc.getString("prefix") + toString(num++));
+                    od.writeAttr(SUMO_ATTR_BEGIN, time2string(b)).writeAttr(SUMO_ATTR_END, time2string(e));
+                    od.writeAttr(SUMO_ATTR_NUMBER, numVehs);
+                    matrix.writeDefaultAttrs(od, oc.getBool("ignore-vehicle-type"), c);
+                    od.openTag(SUMO_TAG_ROUTE_DISTRIBUTION);
+                    for (RORoute* const r : c->pathsVector) {
+                        r->setCosts(router->recomputeCosts(r->getEdgeVector(), &defaultVehicle, begin));
+                        r->writeXMLDefinition(od, nullptr, true, false);
+                    }
+                    od.closeTag();
+                    od.closeTag();
+                    sortedOut[c->begin] += od.getString();
                 } else {
                     for (std::map<SUMOTime, std::vector<std::string> >::const_iterator deps = c->departures.begin(); deps != c->departures.end(); ++deps) {
+                        if (deps->first >= end || deps->first < begin) {
+                            continue;
+                        }
                         const std::string routeDistId = c->origin + "_" + c->destination + "_" + time2string(c->begin) + "_" + time2string(c->end);
-                        for (std::vector<std::string>::const_iterator id = deps->second.begin(); id != deps->second.end(); ++id) {
-                            dev->openTag(SUMO_TAG_VEHICLE).writeAttr(SUMO_ATTR_ID, *id).writeAttr(SUMO_ATTR_DEPART, time2string(deps->first));
-                            matrix.writeDefaultAttrs(*dev, oc.getBool("ignore-vehicle-type"), c);
-                            dev->openTag(SUMO_TAG_ROUTE_DISTRIBUTION);
-                            for (std::vector<RORoute*>::const_iterator j = c->pathsVector.begin(); j != c->pathsVector.end(); ++j) {
-                                (*j)->setCosts(router->recomputeCosts((*j)->getEdgeVector(), &defaultVehicle, string2time(oc.getString("begin"))));
-                                (*j)->writeXMLDefinition(*dev, 0, true, false);
+                        for (const std::string& id : deps->second) {
+                            OutputDevice_String od(1);
+                            od.openTag(SUMO_TAG_VEHICLE).writeAttr(SUMO_ATTR_ID, id).writeAttr(SUMO_ATTR_DEPART, time2string(deps->first));
+                            matrix.writeDefaultAttrs(od, oc.getBool("ignore-vehicle-type"), c);
+                            od.openTag(SUMO_TAG_ROUTE_DISTRIBUTION);
+                            for (RORoute* const r : c->pathsVector) {
+                                r->setCosts(router->recomputeCosts(r->getEdgeVector(), &defaultVehicle, begin));
+                                r->writeXMLDefinition(od, nullptr, true, false);
                             }
-                            dev->closeTag();
-                            dev->closeTag();
+                            od.closeTag();
+                            if (!tazParamKeys.empty()) {
+                                od.openTag(SUMO_TAG_PARAM).writeAttr(SUMO_ATTR_KEY, tazParamKeys[0]).writeAttr(SUMO_ATTR_VALUE, c->origin).closeTag();
+                                if (tazParamKeys.size() > 1) {
+                                    od.openTag(SUMO_TAG_PARAM).writeAttr(SUMO_ATTR_KEY, tazParamKeys[1]).writeAttr(SUMO_ATTR_VALUE, c->destination).closeTag();
+                                }
+                            }
+                            od.closeTag();
+                            sortedOut[deps->first] += od.getString();
                         }
                     }
                 }
+                if (c->end > lastEnd) {
+                    lastEnd = c->end;
+                }
+            }
+            for (std::map<SUMOTime, std::string>::const_iterator desc = sortedOut.begin(); desc != sortedOut.end(); ++desc) {
+                dev->writePreformattedTag(desc->second);
             }
             haveOutput = true;
         }
@@ -374,10 +391,10 @@ computeRoutes(RONet& net, OptionsCont& oc, ODMatrix& matrix) {
 int
 main(int argc, char** argv) {
     OptionsCont& oc = OptionsCont::getOptions();
-    oc.setApplicationDescription("Import O/D-matrices for macroscopic traffic assignment");
-    oc.setApplicationName("marouter", "SUMO marouter Version " VERSION_STRING);
+    oc.setApplicationDescription("Import O/D-matrices for macroscopic traffic assignment to generate SUMO routes");
+    oc.setApplicationName("marouter", "Eclipse SUMO marouter Version " VERSION_STRING);
     int ret = 0;
-    RONet* net = 0;
+    RONet* net = nullptr;
     try {
         XMLSubSys::init();
         ROMAFrame::fillOptions();
@@ -387,7 +404,8 @@ main(int argc, char** argv) {
             SystemFrame::close();
             return 0;
         }
-        XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"));
+        SystemFrame::checkOptions();
+        XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"), oc.getString("xml-validation.routes"));
         MsgHandler::initOutputOptions();
         if (!ROMAFrame::checkOptions()) {
             throw ProcessError();
@@ -409,7 +427,7 @@ main(int argc, char** argv) {
             }
         }
         if (net->getDistricts().empty()) {
-            throw ProcessError("No districts loaded.");
+            WRITE_WARNING("No districts loaded, will use edge ids!");
         }
         // load districts
         ODDistrictCont districts;
@@ -419,14 +437,14 @@ main(int argc, char** argv) {
         matrix.loadMatrix(oc);
         ROMARouteHandler handler(matrix);
         matrix.loadRoutes(oc, handler);
-        if (matrix.getNoLoaded() == 0) {
-            throw ProcessError("No vehicles loaded.");
+        if (matrix.getNumLoaded() == matrix.getNumDiscarded()) {
+            throw ProcessError("No valid vehicles loaded.");
         }
         if (MsgHandler::getErrorInstance()->wasInformed() && !oc.getBool("ignore-errors")) {
             throw ProcessError("Loading failed.");
         }
         MsgHandler::getErrorInstance()->clear();
-        WRITE_MESSAGE(toString(matrix.getNoLoaded()) + " vehicles loaded.");
+        WRITE_MESSAGE(toString(matrix.getNumLoaded() - matrix.getNumDiscarded()) + " valid vehicles loaded (total seen: " + toString(matrix.getNumLoaded()) + ").");
 
         // build routes and parse the incremental rates if the incremental method is choosen.
         try {
@@ -435,7 +453,7 @@ main(int argc, char** argv) {
             WRITE_ERROR(toString(e.getLineNumber()));
             ret = 1;
         } catch (XERCES_CPP_NAMESPACE::SAXException& e) {
-            WRITE_ERROR(TplConvert::_2str(e.getMessage()));
+            WRITE_ERROR(StringUtils::transcode(e.getMessage()));
             ret = 1;
         }
         if (MsgHandler::getErrorInstance()->wasInformed() || ret != 0) {
@@ -458,6 +476,4 @@ main(int argc, char** argv) {
 }
 
 
-
 /****************************************************************************/
-
