@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIMEVehicle.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIMEVehicle.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // A MSVehicle extended by some values for usage within the gui
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIMEVehicle_h
-#define GUIMEVehicle_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <set>
@@ -44,6 +34,7 @@
 // class declarations
 // ===========================================================================
 class GUISUMOAbstractView;
+class GUIVisualizationSettings;
 
 
 // ===========================================================================
@@ -68,7 +59,7 @@ public:
      * @exception ProcessError If a value is wrong
      */
     GUIMEVehicle(SUMOVehicleParameter* pars, const MSRoute* route,
-                 const MSVehicleType* type, const double speedFactor);
+                 MSVehicleType* type, const double speedFactor);
 
 
     /// @brief destructor
@@ -83,6 +74,13 @@ public:
         return MEVehicle::getPosition(offset);
     }
 
+    /** @brief Returns the boundary to which the view shall be centered in order to show the object
+     *
+     * @return The boundary the object is within
+     * @see GUIGlObject::getCenteringBoundary
+     */
+    virtual Boundary getCenteringBoundary() const;
+
     /** @brief Return current angle
     *
     * @note implementation of abstract method does not work otherwise
@@ -92,10 +90,10 @@ public:
     }
 
     /// @brief gets the color value according to the current scheme index
-    double getColorValue(int activeScheme) const;
+    double getColorValue(const GUIVisualizationSettings& s, int activeScheme) const;
 
-    /// @brief draws the given guiShape if it has distinc carriages/modules and eturns true if so
-    bool drawAction_drawCarriageClass(const GUIVisualizationSettings& s, SUMOVehicleShape guiShape, bool asImage) const;
+    /// @brief draws the given guiShape with distinct carriages/modules
+    void drawAction_drawCarriageClass(const GUIVisualizationSettings& s, bool asImage) const;
 
     /** @brief Returns the time since the last lane change in seconds
      * @see MSVehicle::myLastLaneChangeOffset
@@ -106,11 +104,15 @@ public:
     /** @brief Draws the route
      * @param[in] r The route to draw
      */
-    void drawRouteHelper(const MSRoute& r, double exaggeration) const;
+    void drawRouteHelper(const GUIVisualizationSettings& s, const MSRoute& r, bool future, bool noLoop, const RGBColor& col) const;
 
     /// @brief retrieve information about the current stop state
     std::string getStopInfo() const;
 
+    std::string getEdgeID() const;
+
+    int getSegmentIndex() const;
+
     /// @brief adds the blocking foes to the current selection
     void selectBlockingFoes() const;
 
@@ -132,9 +134,3 @@ public:
     GUIParameterTableWindow* getTypeParameterWindow(GUIMainWindow& app, GUISUMOAbstractView& parent);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
