@@ -1,37 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MELoop.h
 /// @author  Daniel Krajzewicz
 /// @date    Tue, May 2005
-/// @version $Id: MELoop.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // The main mesocopic simulation loop
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MELoop_h
-#define MELoop_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <map>
+#include <utils/common/SUMOTime.h>
 
 
 // ===========================================================================
@@ -42,7 +33,6 @@ class MEVehicle;
 class MSEdge;
 class MSLink;
 class MSVehicleControl;
-class BinaryInputDevice;
 class OptionsCont;
 
 
@@ -81,6 +71,12 @@ public:
      */
     void removeLeaderCar(MEVehicle* v);
 
+    /** @brief remove the given car and clean up the relevant data structures */
+    void vaporizeCar(MEVehicle* v, MSMoveReminder::Notification reason);
+
+    /// @brief whether the given edge is entering a roundabout
+    static bool isEnteringRoundabout(const MSEdge& e);
+
     /** @brief Compute number of segments per edge (best value stay close to the configured segment length) */
     static int numSegmentsFor(const double length, const double slength);
 
@@ -90,6 +86,12 @@ public:
      */
     void buildSegmentsFor(const MSEdge& e, const OptionsCont& oc);
 
+    /** @brief Update segments after loading meso edge type parameters from
+     * additional file
+     * @param[in] e the edge to update
+     */
+    void updateSegementsForEdge(const MSEdge& e);
+
     /** @brief Get the segment for a given edge at a given position
      *
      * @param[in] e the edge to get the segment for
@@ -102,15 +104,11 @@ public:
      * this handles combinations of the following cases:
      * (ending / continuing route) and (leaving segment / finishing teleport)
      */
-    bool changeSegment(MEVehicle* veh, SUMOTime leaveTime, MESegment* const toSegment, const bool ignoreLink = false);
-
-    /** @brief registers vehicle with the given link
-     *
-     * @param[in] veh the car to register
-     * @param[in] link the link on which the car shall register its approach
-     */
-    static void setApproaching(MEVehicle* veh, MSLink* link);
+    SUMOTime changeSegment(MEVehicle* veh, SUMOTime leaveTime, MESegment* const toSegment,
+                           MSMoveReminder::Notification reason, const bool ignoreLink = false) const;
 
+    /** @brief Remove all vehicles before quick-loading state */
+    void clearState();
 
 private:
     /** @brief Check whether the vehicle may move
@@ -162,9 +160,3 @@ private:
     /// @brief Invalidated assignment operator.
     MELoop& operator=(const MELoop&);
 };
-
-
-#endif
-
-/****************************************************************************/
-
