@@ -1,34 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MESegment.h
 /// @author  Daniel Krajzewicz
 /// @date    Tue, May 2005
-/// @version $Id: MESegment.h 21206 2016-07-20 08:08:35Z behrisch $
 ///
 // A single mesoscopic segment (cell)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MESegment_h
-#define MESegment_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <utils/common/Named.h>
@@ -41,9 +31,9 @@
 class MSEdge;
 class MSLink;
 class MSMoveReminder;
+class MSDetectorFileOutput;
 class MSVehicleControl;
 class MEVehicle;
-class BinaryInputDevice;
 class OutputDevice;
 
 
@@ -55,6 +45,67 @@ class OutputDevice;
  * @brief A single mesoscopic segment (cell)
  */
 class MESegment : public Named {
+private:
+    class Queue {
+    public:
+        Queue(const SVCPermissions permissions) : myPermissions(permissions) {}
+        inline int size() const {
+            return (int)myVehicles.size();
+        }
+        inline const std::vector<MEVehicle*>& getVehicles() const {
+            return myVehicles;
+        }
+        MEVehicle* remove(MEVehicle* v);
+        inline std::vector<MEVehicle*>& getModifiableVehicles() {
+            return myVehicles;
+        }
+        inline double getOccupancy() const {
+            return myOccupancy;
+        }
+        inline void setOccupancy(const double occ) {
+            myOccupancy = occ;
+        }
+        inline bool allows(SUMOVehicleClass vclass) const {
+            return (myPermissions & vclass) == vclass;
+        }
+
+        /// @brief return the next time at which a vehicle may enter this queue
+        inline SUMOTime getEntryBlockTime() const {
+            return myEntryBlockTime;
+        }
+
+        /// @brief set the next time at which a vehicle may enter this queue
+        inline void setEntryBlockTime(SUMOTime entryBlockTime) {
+            myEntryBlockTime = entryBlockTime;
+        }
+
+        inline SUMOTime getBlockTime() const {
+            return myBlockTime;
+        }
+        inline void setBlockTime(SUMOTime t) {
+            myBlockTime = t;
+        }
+
+    private:
+        /// The vClass permissions for this queue
+        const SVCPermissions myPermissions;
+
+        std::vector<MEVehicle*> myVehicles;
+
+        /// @brief The occupied space (in m) in the queue
+        double myOccupancy = 0.;
+
+        /// @brief The block time for vehicles who wish to enter this queue
+        SUMOTime myEntryBlockTime = SUMOTime_MIN;
+
+        /// @brief The block time
+        SUMOTime myBlockTime = -1;
+
+    private:
+        /// @brief Invalidated assignment operator.
+        Queue& operator=(const Queue&) = delete;
+    };
+
 public:
     /** @brief constructor
      * @param[in] id The id of this segment (currently: "<EDGEID>:<SEGMENTNO>")
@@ -70,22 +121,18 @@ public:
      * @param[in] jamThresh percentage of occupied space before the segment is jammed
      * @param[in] multiQueue whether to install multiple queues on this segment
      * @param[in] junctionControl whether junction control is enabled on this segment
-     * @param[in] the quotient of geometrical length / given length
      * @todo recheck the id; using a ':' as divider is not really nice
      */
     MESegment(const std::string& id,
               const MSEdge& parent, MESegment* next,
-              SUMOReal length, SUMOReal speed,
-              int idx,
-              SUMOTime tauff, SUMOTime taufj,
-              SUMOTime taujf, SUMOTime taujj,
-              SUMOReal jamThresh,
-              bool multiQueue, bool junctionControl,
-              SUMOReal lengthGeometryFactor);
+              const double length, const double speed,
+              const int idx,
+              const bool multiQueue,
+              const MSNet::MesoEdgeType& edgeTyp);
 
+    /// @brief set model parameters (may be updated from additional file after network loading is complete)
+    void initSegment(const MSNet::MesoEdgeType& edgeType, const MSEdge& parent);
 
-    typedef std::vector<MEVehicle*> Queue;
-    typedef std::vector<Queue> Queues;
     /// @name Measure collection
     /// @{
 
@@ -112,10 +159,11 @@ public:
      *
      * @param[in] veh The vehicle to check space for
      * @param[in] entryTime The time at which the vehicle wants to enter
+     * @param[out] qIdx The index of the queue the vehicle should choose
      * @param[in] init whether the check is done at insertion time
-     * @return true if the vehicle may be added to this segment, false otherwise
+     * @return the earliest time a vehicle may be added to this segment
      */
-    bool hasSpaceFor(const MEVehicle* veh, SUMOTime entryTime, bool init = false) const;
+    SUMOTime hasSpaceFor(const MEVehicle* const veh, const SUMOTime entryTime, int& qIdx, const bool init = false) const;
 
     /** @brief Inserts (emits) vehicle into the segment
      *
@@ -129,18 +177,20 @@ public:
      *
      * @return the total number of cars on the segment
      */
-    int getCarNumber() const;
+    inline int getCarNumber() const {
+        return myNumVehicles;
+    }
 
     /// @brief return the number of queues
     inline int numQueues() const {
-        return (int)myCarQues.size();
+        return (int)myQueues.size();
     }
     /** @brief Returns the cars in the queue with the given index for visualization
      * @return the Queue (XXX not thread-safe!)
      */
-    inline const Queue& getQueue(int index) const {
-        assert(index < (int)myCarQues.size());
-        return myCarQues[index];
+    inline const std::vector<MEVehicle*>& getQueue(int index) const {
+        assert(index < (int)myQueues.size());
+        return myQueues[index].getVehicles();
     }
 
     /** @brief Returns the running index of the segment in the edge (0 is the most upstream).
@@ -163,7 +213,7 @@ public:
      *
      * @return the length of the segment
      */
-    inline SUMOReal getLength() const {
+    inline double getLength() const {
         return myLength;
     }
 
@@ -171,23 +221,26 @@ public:
      *
      * @return the occupany of the segment in meters
      */
-    inline SUMOReal getBruttoOccupancy() const {
-        return myOccupancy;
+    inline double getBruttoOccupancy() const {
+        double occ = 0.;
+        for (const Queue& q : myQueues) {
+            occ += q.getOccupancy();
+        }
+        return occ;
     }
 
-
     /** @brief Returns the relative occupany of the segment (percentage of road used))
      * @return the occupany of the segment in percent
      */
-    inline SUMOReal getRelativeOccupancy() const {
-        return myOccupancy / myCapacity;
+    inline double getRelativeOccupancy() const {
+        return getBruttoOccupancy() / myCapacity;
     }
 
     /** @brief Returns the relative occupany of the segment (percentage of road used))
      * at which the segment is considered jammed
      * @return the jam treshold of the segment in percent
      */
-    inline SUMOReal getRelativeJamThreshold() const {
+    inline double getRelativeJamThreshold() const {
         return myJamThreshold / myCapacity;
     }
 
@@ -201,10 +254,10 @@ public:
      * initializsation of vehicles onto the segment.
      * @return the average speed on the segment
      */
-    SUMOReal getMeanSpeed(bool useCache) const;
+    double getMeanSpeed(bool useCache) const;
 
     /// @brief wrapper to satisfy the FunctionBinding signature
-    inline SUMOReal getMeanSpeed() const {
+    inline double getMeanSpeed() const {
         return getMeanSpeed(true);
     }
 
@@ -215,10 +268,10 @@ public:
      *
      * @param[in] v The vehicle to remove
      * @param[in] leaveTime The time at which the vehicle is leaving the que
-     * @param[in] next The next segment for this vehicle
+     * @param[in] reason The reason for removing to send to reminders
      * @return The next first vehicle to add to the net's que
      */
-    MEVehicle* removeCar(MEVehicle* v, SUMOTime leaveTime, MESegment* next);
+    MEVehicle* removeCar(MEVehicle* v, SUMOTime leaveTime, const MSMoveReminder::Notification reason);
 
     /** @brief Returns the link the given car will use when passing the next junction
      *
@@ -247,7 +300,7 @@ public:
      * @param[in] time the leave time
      * @todo Isn't always time == veh->getEventTime?
      */
-    void send(MEVehicle* veh, MESegment* next, SUMOTime time);
+    void send(MEVehicle* veh, MESegment* const next, const int nextQIdx, SUMOTime time, const MSMoveReminder::Notification reason);
 
     /** @brief Adds the vehicle to the segment, adapting its parameters
      *
@@ -256,7 +309,7 @@ public:
      * @param[in] isDepart whether the vehicle just departed
      * @todo Isn't always time == veh->getEventTime?
      */
-    void receive(MEVehicle* veh, SUMOTime time, bool isDepart = false, bool afterTeleport = false);
+    void receive(MEVehicle* veh, const int qIdx, SUMOTime time, const bool isDepart = false, const bool isTeleport = false, const bool newEdge = false);
 
 
     /** @brief tries to remove any car from this segment
@@ -264,7 +317,7 @@ public:
      * @param[in] currentTime the current time
      * @return Whether vaporization was successful
      * @note: cars removed via this method do NOT count as arrivals */
-    bool vaporizeAnyCar(SUMOTime currentTime);
+    bool vaporizeAnyCar(SUMOTime currentTime, const MSDetectorFileOutput* filter);
 
     /** @brief Returns the edge this segment belongs to
      * @return the edge this segment belongs to
@@ -278,7 +331,7 @@ public:
      * all vehicles in it. Also set/recompute myJamThreshold
      * @param[in] jamThresh follows the semantic of option meso-jam-threshold
      */
-    void setSpeed(SUMOReal newSpeed, SUMOTime currentTime, SUMOReal jamThresh = DO_NOT_PATCH_JAM_THRESHOLD);
+    void setSpeed(double newSpeed, SUMOTime currentTime, double jamThresh = DO_NOT_PATCH_JAM_THRESHOLD);
 
     /** @brief Returns the (planned) time at which the next vehicle leaves this segment
      * @return The time the vehicle thinks it leaves
@@ -286,10 +339,27 @@ public:
     SUMOTime getEventTime() const;
 
     /// @brief Like getEventTime but returns seconds (for visualization)
-    inline SUMOReal getEventTimeSeconds() const {
+    inline double getEventTimeSeconds() const {
         return STEPS2TIME(getEventTime());
     }
 
+    /// @brief get the last headway time in seconds
+    inline double getLastHeadwaySeconds() const {
+        return STEPS2TIME(myLastHeadway);
+    }
+
+    /// @brief get the earliest entry time in seconds
+    inline double getEntryBlockTimeSeconds() const {
+        SUMOTime t = SUMOTime_MAX;
+        for (const Queue& q : myQueues) {
+            t = MIN2(t, q.getEntryBlockTime());
+        }
+        return STEPS2TIME(t);
+    }
+
+    /// @brief Get the waiting time for vehicles in all queues
+    double getWaitingSeconds() const;
+
     /// @name State saving/loading
     /// @{
 
@@ -302,7 +372,10 @@ public:
      * @param[in, filled] out The (possibly binary) device to write the state into
      * @todo What about throwing an IOError?
      */
-    void saveState(OutputDevice& out);
+    void saveState(OutputDevice& out) const;
+
+    /** @brief Remove all vehicles before quick-loading state */
+    void clearState();
 
     /** @brief Loads the state of this segment with the given parameters
      *
@@ -319,7 +392,7 @@ public:
      * @todo What about throwing an IOError?
      * @todo What about throwing an error if something else fails (a vehicle can not be referenced)?
      */
-    void loadState(std::vector<std::string>& vehIDs, MSVehicleControl& vc, const SUMOTime blockTime, const int queIdx);
+    void loadState(const std::vector<std::string>& vehIDs, MSVehicleControl& vc, const SUMOTime blockTime, const int queIdx);
     /// @}
 
 
@@ -327,44 +400,31 @@ public:
      */
     std::vector<const MEVehicle*> getVehicles() const;
 
-
     /** @brief returns flow based on headway
      * @note: returns magic number 10000 when headway cannot be computed
      */
-    SUMOReal getFlow() const;
-
+    double getFlow() const;
 
     /// @brief whether the given segment is 0 or encodes vaporization
     static inline bool isInvalid(const MESegment* segment) {
-        return segment == 0 || segment == &myVaporizationTarget;
+        return segment == nullptr || segment == &myVaporizationTarget;
     }
 
     /// @brief return a time after earliestEntry at which a vehicle may be inserted at full speed
     SUMOTime getNextInsertionTime(SUMOTime earliestEntry) const;
 
-    /** @brief return whether this segment is considered free as opposed to jammed
-     */
-    inline bool free() const {
-        return (myOccupancy <= myJamThreshold) || myTLSPenalty;
-    }
-
     /// @brief return the remaining physical space on this segment
-    inline int remainingVehicleCapacity(const SUMOReal vehLength) const {
-        if (myOccupancy == 0. && myCapacity < vehLength) {
-            // even small segments can hold at least one vehicle
-            return 1;
+    inline int remainingVehicleCapacity(const double vehLength) const {
+        int cap = 0;
+        for (const Queue& q : myQueues) {
+            if (q.getOccupancy() == 0. && myQueueCapacity < vehLength) {
+                // even small segments can hold at least one vehicle
+                cap += 1;
+            } else {
+                cap += (int)((myQueueCapacity - q.getOccupancy()) / vehLength);
+            }
         }
-        return (int)((myCapacity - myOccupancy) / vehLength);
-    }
-
-    /// @brief return the next time at which a vehicle my enter this segment
-    inline SUMOTime getEntryBlockTime() const {
-        return myEntryBlockTime;
-    }
-
-    /// @brief set the next time at which a vehicle my enter this segment
-    inline void setEntryBlockTime(SUMOTime entryBlockTime) {
-        myEntryBlockTime = entryBlockTime;
+        return cap;
     }
 
     /// @brief return the minimum headway-time with which vehicles may enter or leave this segment
@@ -372,55 +432,49 @@ public:
         return myTau_ff;
     }
 
-    static const SUMOReal DO_NOT_PATCH_JAM_THRESHOLD;
+    static const double DO_NOT_PATCH_JAM_THRESHOLD;
 
     /// @brief add this lanes MoveReminders to the given vehicle
     void addReminders(MEVehicle* veh) const;
 
-    /// @brief return precomputed geometrical length / myLength
-    inline SUMOReal getLengthGeometryFactor() const {
-        return myLengthGeometryFactor;
-    }
-
-    /** @brief Returns the penalty time for passing a tls-controlled link (if using gMesoTLSPenalty > 0)
+    /** @brief Returns the penalty time for passing a link (if using gMesoTLSPenalty > 0 or gMesoMinorPenalty > 0)
      * @param[in] veh The vehicle in question
      * @return The time penalty
      */
-    SUMOTime getTLSPenalty(const MEVehicle* veh) const;
+    SUMOTime getLinkPenalty(const MEVehicle* veh) const;
 
 private:
-    /** @brief Updates data of all detectors for a leaving vehicle
-     *
-     * @param[in] v The vehicle to update values for
-     * @param[in] currentTime The leave time of the vehicle
-     * @param[in] next The next segment on this vehicles route
-     */
-    void updateDetectorsOnLeave(MEVehicle* v, SUMOTime currentTime, MESegment* next);
-
     bool overtake();
 
-    SUMOTime getTimeHeadway(bool predecessorIsFree, SUMOReal leaderLength);
-
-    void setSpeedForQueue(SUMOReal newSpeed, SUMOTime currentTime,
+    void setSpeedForQueue(double newSpeed, SUMOTime currentTime,
                           SUMOTime blockTime, const std::vector<MEVehicle*>& vehs);
 
     /** @brief compute the new arrival time when switching speed
      */
-    SUMOTime newArrival(const MEVehicle* const v, SUMOReal newSpeed, SUMOTime currentTime);
+    SUMOTime newArrival(const MEVehicle* const v, double newSpeed, SUMOTime currentTime);
 
+    /// @brief whether a leader in any queue is blocked
+    bool hasBlockedLeader() const;
 
     /** @brief compute a value for myJamThreshold
      * if jamThresh is negative, compute a value which allows free flow at mySpeed
      * interpret jamThresh as the relative occupation at which jam starts
      */
-    void recomputeJamThreshold(SUMOReal jamThresh);
+    void recomputeJamThreshold(double jamThresh);
 
     /// @brief compute jam threshold for the given speed and jam-threshold option
-    SUMOReal jamThresholdForSpeed(SUMOReal speed, SUMOReal jamThresh) const;
+    double jamThresholdForSpeed(double speed, double jamThresh) const;
 
     /// @brief whether the given link may be passed because the option meso-junction-control.limited is set
     bool limitedControlOverride(const MSLink* link) const;
 
+    /// @brief convert net time gap (leader back to follower front) to gross time gap (leader front to follower front)
+    inline SUMOTime tauWithVehLength(SUMOTime tau, double lengthWithGap) const {
+        return tau + (SUMOTime)(lengthWithGap * myTau_length);
+    }
+
+    SUMOTime getTauJJ(int nextQueueSize, double nextQueueCapacity, double nextJamThreshold) const;
+
 private:
     /// @brief The microsim edge this segment belongs to
     const MSEdge& myEdge;
@@ -429,57 +483,57 @@ private:
     MESegment* myNextSegment;
 
     /// @brief The segment's length
-    const SUMOReal myLength;
+    const double myLength;
 
     /// @brief Running number of the segment in the edge
     const int myIndex;
 
+    /// @name Model constants that may be reset once via additional file
+    /// @{
+
     /// @brief The time headway parameters, see the Eissfeldt thesis
-    const SUMOTime myTau_ff, myTau_fj, myTau_jf, myTau_jj;
-    /// @brief Headway paramter for computing gross time headyway from net time heawdway, length and edge speed
-    SUMOReal myTau_length;
-
-    /// @brief slope and axis offset for the jam-jam headway function
-    SUMOReal myA, myB;
-
-    /// @brief The capacity of the segment in number of cars, used only in time headway calculation
-    /// This parameter has only an effect if tau_jf != tau_jj, which is not(!) the case per default
-    const SUMOReal myHeadwayCapacity;
-
-    /// @brief The number of lanes * the length
-    const SUMOReal myCapacity;
-
-    /// @brief The occupied space (in m) on the segment
-    SUMOReal myOccupancy;
-
-    /// @brief Whether junction control is enabled
-    const bool myJunctionControl;
+    SUMOTime myTau_ff, myTau_fj, myTau_jf, myTau_jj;
 
     /// @brief Whether tls penalty is enabled
-    const bool myTLSPenalty;
+    bool myTLSPenalty;
+
+    /// @brief penalty for minor links
+    bool myCheckMinorPenalty; // for legacy compatibility (#7802, 7804)
+    SUMOTime myMinorPenalty;
+
+    /// @brief Whether junction control is enabled
+    bool myJunctionControl;
+
+    /// @brief Whether overtaking is permitted on this segment
+    bool myOvertaking;
+    /// @}
+
+    /// @brief Headway parameter for computing gross time headyway from net time headway, length and edge speed
+    double myTau_length;
+
+    /// @brief The number of lanes represented by the queue * the length of the lane
+    const double myCapacity;
+
+    /// @brief The number of lanes represented by the queue * the length of the lane
+    double myQueueCapacity;
 
     /// @brief The space (in m) which needs to be occupied before the segment is considered jammed
-    SUMOReal myJamThreshold;
+    double myJamThreshold;
 
     /// @brief The data collection for all kinds of detectors
     std::vector<MSMoveReminder*> myDetectorData;
 
     /// @brief The car queues. Vehicles are inserted in the front and removed in the back
-    Queues myCarQues;
+    std::vector<Queue> myQueues;
 
-    /// @brief The follower edge to que index mapping for multi queue segments
-    std::map<const MSEdge*, std::vector<int> > myFollowerMap;
+    /// @brief The cached value for the number of vehicles
+    int myNumVehicles;
 
-    /// @brief The block times
-    std::vector<SUMOTime> myBlockTimes;
+    /// @brief The follower edge to allowed que index mapping for multi queue segments
+    std::map<const MSEdge*, int> myFollowerMap;
 
-    /* @brief The block time for vehicles who wish to enter this segment.
-     * @note since we do not know which queue will be used there is only one
-     * value for all queues */
-    SUMOTime myEntryBlockTime;
-
-    // precomputed factor between geometrical length and given length
-    const SUMOReal myLengthGeometryFactor;
+    /// @brief the last headway
+    SUMOTime myLastHeadway;
 
     /* @brief segment for signifying vaporization. This segment has invalid
      * data and should only be used as a unique pointer */
@@ -487,7 +541,7 @@ private:
     static MESegment myVaporizationTarget;
 
     /// @brief the mean speed on this segment. Updated at event time or on demand
-    mutable SUMOReal myMeanSpeed;
+    mutable double myMeanSpeed;
 
     /// @brief the time at which myMeanSpeed was last updated
     mutable SUMOTime myLastMeanSpeedUpdate;
@@ -502,8 +556,3 @@ private:
     /// @brief constructor for dummy segment
     MESegment(const std::string& id);
 };
-
-
-#endif
-
-/****************************************************************************/
