@@ -1,32 +1,23 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    METriggeredCalibrator.cpp
 /// @author  Daniel Krajzewicz
 /// @date    Tue, May 2005
-/// @version $Id: METriggeredCalibrator.cpp 24113 2017-04-28 07:07:07Z behrisch $
 ///
 // Calibrates the flow on a segment to a specified one
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <algorithm>
@@ -35,15 +26,16 @@
 #include <microsim/MSNet.h>
 #include <microsim/MSEdge.h>
 #include <microsim/MSEventControl.h>
+#include <microsim/MSVehicleControl.h>
 #include <microsim/output/MSRouteProbe.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include <utils/common/ToString.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/StringTokenizer.h>
 #include <utils/xml/XMLSubSys.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/options/OptionsCont.h>
-#include <utils/xml/SUMOVehicleParserHelper.h>
+#include <utils/vehicle/SUMOVehicleParserHelper.h>
 #include <utils/distribution/RandomDistributor.h>
 #include <utils/vehicle/SUMOVehicleParameter.h>
 #include "MELoop.h"
@@ -60,16 +52,25 @@ METriggeredCalibrator::METriggeredCalibrator(const std::string& id,
         const std::string& aXMLFilename,
         const std::string& outputFilename,
         const SUMOTime freq, const double length,
-        const MSRouteProbe* probe)
-    : MSCalibrator(id, edge, pos, aXMLFilename, outputFilename, freq, length, probe, false),
-      mySegment(MSGlobals::gMesoNet->getSegmentForEdge(*edge, pos)) {
+        const MSRouteProbe* probe,
+        const double invalidJamThreshold,
+        const std::string& vTypes) :
+    MSCalibrator(id, edge, (MSLane*)nullptr, pos, aXMLFilename, outputFilename, freq, length, probe, invalidJamThreshold, vTypes, false),
+    mySegment(MSGlobals::gMesoNet->getSegmentForEdge(*edge, pos)) {
     myEdgeMeanData.setDescription("meandata_calibrator_" + getID());
     mySegment->addDetector(&myEdgeMeanData);
 }
 
 
 METriggeredCalibrator::~METriggeredCalibrator() {
-    mySegment->removeDetector(&myEdgeMeanData);
+    if (myCurrentStateInterval != myIntervals.end()) {
+        // need to do it here and not in MSCalibrator because otherwise meandata is gone
+        intervalEnd();
+        // but avoid to call it again in MSCalibrator
+        myCurrentStateInterval = myIntervals.end();
+    }
+    // TODO this is just commented out to work around https://github.com/eclipse/sumo/issues/7861
+    //mySegment->removeDetector(&myEdgeMeanData);
 }
 
 
@@ -94,14 +95,16 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
     // check whether an adaptation value exists
     if (isCurrentStateActive(currentTime)) {
         // all happens in isCurrentStateActive()
+        myAmActive = true;
     } else {
+        myAmActive = false;
         myEdgeMeanData.reset(); // discard collected values
         if (!mySpeedIsDefault) {
             // if not, reset adaptation values
             mySegment->getEdge().setMaxSpeed(myDefaultSpeed);
             MESegment* first = MSGlobals::gMesoNet->getSegmentForEdge(mySegment->getEdge());
             const double jamThresh = OptionsCont::getOptions().getFloat("meso-jam-threshold");
-            while (first != 0) {
+            while (first != nullptr) {
                 first->setSpeed(myDefaultSpeed, currentTime, jamThresh);
                 first = first->getNextSegment();
             }
@@ -113,11 +116,13 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
         }
         return myFrequency;
     }
+    const bool calibrateFlow = myCurrentStateInterval->q >= 0;
+    const bool calibrateSpeed = myCurrentStateInterval->v >= 0;
     // we are active
-    if (!myDidSpeedAdaption && myCurrentStateInterval->v >= 0 && myCurrentStateInterval->v != mySegment->getEdge().getSpeedLimit()) {
+    if (!myDidSpeedAdaption && calibrateSpeed && myCurrentStateInterval->v != mySegment->getEdge().getSpeedLimit()) {
         mySegment->getEdge().setMaxSpeed(myCurrentStateInterval->v);
         MESegment* first = MSGlobals::gMesoNet->getSegmentForEdge(mySegment->getEdge());
-        while (first != 0) {
+        while (first != nullptr) {
             first->setSpeed(myCurrentStateInterval->v, currentTime, -1);
             first = first->getNextSegment();
         }
@@ -126,24 +131,24 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
     }
     // clear invalid jams
     bool hadInvalidJam = false;
-    while (invalidJam()) {
+    while ((calibrateFlow || calibrateSpeed) && invalidJam()) {
         hadInvalidJam = true;
         if (!myHaveWarnedAboutClearingJam) {
-            WRITE_WARNING("Clearing jam at calibrator '" + myID + "' at time " + time2string(currentTime));
+            WRITE_WARNING("Clearing jam at calibrator '" + getID() + "' at time " + time2string(currentTime));
         }
         // remove one vehicle currently on the segment
-        if (mySegment->vaporizeAnyCar(currentTime)) {
+        if (mySegment->vaporizeAnyCar(currentTime, this)) {
             myClearedInJam++;
         } else {
             if (!myHaveWarnedAboutClearingJam) {
                 // this frequenly happens for very short edges
-                WRITE_WARNING("Could not clear jam at calibrator '" + myID + "' at time " + time2string(currentTime));
+                WRITE_WARNING("Could not clear jam at calibrator '" + getID() + "' at time " + time2string(currentTime));
             }
             break;
         }
         myHaveWarnedAboutClearingJam = true;
     }
-    if (myCurrentStateInterval->q >= 0) {
+    if (calibrateFlow) {
         // flow calibration starts here ...
         // compute the number of vehicles that should have passed the calibrator within the time
         // rom begin of the interval
@@ -163,16 +168,16 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
             //std::cout << "time:" << STEPS2TIME(currentTime) << " w:" << wishedNum << " s:" << insertionSlack << " before:" << adaptedNum;
             while (wishedNum > adaptedNum + insertionSlack && remainingVehicleCapacity() > maximumInflow()) {
                 SUMOVehicleParameter* pars = myCurrentStateInterval->vehicleParameter;
-                const MSRoute* route = myProbe != 0 ? myProbe->getRoute() : 0;
-                if (route == 0) {
+                const MSRoute* route = myProbe != nullptr ? myProbe->sampleRoute() : nullptr;
+                if (route == nullptr) {
                     route = MSRoute::dictionary(pars->routeid);
                 }
-                if (route == 0) {
-                    WRITE_WARNING("No valid routes in calibrator '" + myID + "'.");
+                if (route == nullptr) {
+                    WRITE_WARNING("No valid routes in calibrator '" + getID() + "'.");
                     break;
                 }
                 if (!route->contains(myEdge)) {
-                    WRITE_WARNING("Route '" + route->getID() + "' in calibrator '" + myID + "' does not contain edge '" + myEdge->getID() + "'.");
+                    WRITE_WARNING("Route '" + route->getID() + "' in calibrator '" + getID() + "' does not contain edge '" + myEdge->getID() + "'.");
                     break;
                 }
                 MSVehicleType* vtype = MSNet::getInstance()->getVehicleControl().getVType(pars->vtypeid);
@@ -180,11 +185,22 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
                 // build the vehicle
                 const SUMOTime depart = mySegment->getNextInsertionTime(currentTime);
                 SUMOVehicleParameter* newPars = new SUMOVehicleParameter(*pars);
-                newPars->id = myID + "." + toString(depart) + "." + toString(myInserted);
+                newPars->id = getNewVehicleID();
                 newPars->depart = depart;
                 newPars->routeid = route->getID();
-                MEVehicle* vehicle = static_cast<MEVehicle*>(MSNet::getInstance()->getVehicleControl().buildVehicle(
-                                         newPars, route, vtype, false, false));
+                MEVehicle* vehicle;
+                try {
+                    vehicle = static_cast<MEVehicle*>(MSNet::getInstance()->getVehicleControl().buildVehicle(
+                                                          newPars, route, vtype, false, false));
+                } catch (const ProcessError& e) {
+                    if (!MSGlobals::gCheckRoutes) {
+                        WRITE_WARNING(e.what());
+                        vehicle = nullptr;
+                        break;
+                    } else {
+                        throw e;
+                    }
+                }
                 vehicle->setSegment(mySegment); // needed or vehicle will not be registered (XXX why?)
                 vehicle->setEventTime(currentTime); // XXX superfluous?
                 // move vehicle forward when the route does not begin at the calibrator's edge
@@ -208,7 +224,7 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
         //std::cout << " after:" << adaptedNum << "\n";
         // we only remove vehicles once we really have to
         while (totalWishedNum < adaptedNum) {
-            if (!mySegment->vaporizeAnyCar(currentTime)) {
+            if (!mySegment->vaporizeAnyCar(currentTime, this)) {
                 // @bug: short edges may be jumped in a single step, giving us no chance to remove a vehicle
                 break;
             }
@@ -217,9 +233,12 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
         }
     }
     if (myCurrentStateInterval->end <= currentTime + myFrequency) {
-        writeXMLOutput();
+        intervalEnd();
+    }
+    //assert(!invalidJam());
+    if (invalidJam()) {
+        WRITE_WARNING("DEBUG: Could not clear jam at calibrator '" + getID() + "' at time " + time2string(currentTime));
     }
-    assert(!invalidJam());
     return myFrequency;
 }
 
@@ -230,7 +249,7 @@ METriggeredCalibrator::invalidJam() const {
         return false;
     }
     // maxSpeed reflects the calibration target
-    const bool toSlow = mySegment->getMeanSpeed() < 0.8 * mySegment->getEdge().getSpeedLimit();
+    const bool toSlow = mySegment->getMeanSpeed() < myInvalidJamThreshold * mySegment->getEdge().getSpeedLimit();
     return toSlow && remainingVehicleCapacity() < maximumInflow();
 }
 
@@ -250,4 +269,3 @@ METriggeredCalibrator::reset() {
 
 
 /****************************************************************************/
-
