@@ -1,34 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MEVehicle.h
 /// @author  Daniel Krajzewicz
 /// @date    Tue, May 2005
-/// @version $Id: MEVehicle.h 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // A vehicle from the mesoscopic point of view
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MEVehicle_h
-#define MEVehicle_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <cassert>
@@ -59,7 +49,7 @@ public:
      * @exception ProcessError If a value is wrong
      */
     MEVehicle(SUMOVehicleParameter* pars, const MSRoute* route,
-              const MSVehicleType* type, const double speedFactor);
+              MSVehicleType* type, const double speedFactor);
 
 
     /** @brief Get the vehicle's position along the lane
@@ -80,7 +70,7 @@ public:
     double getAngle() const;
 
 
-    /** @brief Returns the slope of the road at vehicle's position
+    /** @brief Returns the slope of the road at vehicle's position in degrees
      * @return The slope
      */
     double getSlope() const;
@@ -88,8 +78,8 @@ public:
     /** @brief Returns the lane the vehicle is on
     * @return The vehicle's current lane
     */
-    MSLane* getLane() const {
-        return 0;
+    const MSLane* getLane() const {
+        return nullptr;
     }
 
     /** @brief Return current position (x/y, cartesian)
@@ -125,6 +115,19 @@ public:
      */
     double getConservativeSpeed(SUMOTime& earliestArrival) const;
 
+    /// @name insertion/removal
+    //@{
+
+    /** @brief Called when the vehicle is removed from the network.
+     *
+     * Moves along work reminders and
+     *  informs all devices about quitting. Calls "leaveLane" then.
+     *
+     * @param[in] reason why the vehicle leaves (reached its destination, parking, teleport)
+     */
+    void onRemovalFromNet(const MSMoveReminder::Notification reason);
+    //@}
+
 
     /** @brief Update when the vehicle enters a new edge in the move step.
      * @return Whether the vehicle's route has ended (due to vaporization, or because the destination was reached)
@@ -141,41 +144,46 @@ public:
      */
     bool isOnRoad() const;
 
-    /** @brief Returns whether the vehicle is parking
-     * @return whether the vehicle is parking
+    /** @brief Returns whether the vehicle is trying to re-enter the net
+     * @return true if the vehicle is trying to enter the net (eg after parking)
      */
-    bool isParking() const;
+    virtual bool isIdling() const;
 
-    /** @brief Adds a stop
+
+    /** @brief registers vehicle with the given link
      *
-     * The stop is put into the sorted list.
-     * @param[in] stop The stop to add
-     * @return Whether the stop could be added
+     * @param[in] link the link on which the car shall register its approach
      */
-    bool addStop(const SUMOVehicleParameter::Stop& stopPar, std::string& errorMsg, SUMOTime untilOffset = 0, bool collision = false);
+    void setApproaching(MSLink* link);
 
+    /// @brief Returns the remaining stop duration for a stopped vehicle or 0
+    SUMOTime remainingStopDuration() const {
+        return 0;
+    }
 
-    /** @brief Returns whether the vehicle is at a stop
-     * @return Whether it has stopped
-     */
-    bool isStopped() const;
+    ///@brief ends the current stop and performs loading/unloading
+    void processStop();
 
+    /** @brief Returns whether the vehicle stops at the given stopping place */
+    bool stopsAt(MSStoppingPlace* /*stop*/) const {
+        return false;
+    }
 
-    /** @brief Returns whether the vehicle is on a triggered stop
-     * @return whether the vehicle is on a triggered stop
-     */
-    bool isStoppedTriggered() const;
+    bool stopsAtEdge(const MSEdge* /*edge*/) const {
+        return false;
+    }
 
-    /** @brief Returns how long to stop at the given segment
-     * @param[in] seg The segment in question
+    /** @brief Returns until when to stop at the current segment and sets the information that the stop has been reached
+     * @param[in] time the current time
      * @return stop time for the segment
      */
-    SUMOTime getStoptime(const MESegment* const seg) const;
+    SUMOTime checkStop(SUMOTime time);
 
 
-    /** @brief Returns the list of still pending stop edges
-     */
-    const ConstMSEdgeVector getStopEdges() const;
+    /// @brief get distance for coming to a stop (used for rerouting checks)
+    double getBrakeGap() const {
+        return 0;
+    }
 
     /** @brief replace the current parking area stop with a new stop with merge duration
      */
@@ -273,6 +281,16 @@ public:
         return MAX2(SUMOTime(0), myEventTime - myBlockTime);
     }
 
+    inline SUMOTime getTimeLoss() const {
+        // slow-downs while driving are not modelled
+        return getWaitingTime();
+    }
+
+    /// @brief Returns the duration for which the vehicle was blocked
+    inline SUMOTime getAccumulatedWaitingTime() const {
+        return getWaitingTime();
+    }
+
 
     /** @brief Returns the number of seconds waited (speed was lesser than 0.1m/s)
      *
@@ -303,9 +321,11 @@ public:
     /// @brief Returns the delay that is accrued due to option --meso-tls-penalty or --meso-minor-penalty
     double getCurrentLinkPenaltySeconds() const;
 
+    /// @brief Returns the delay that is accrued due to option --meso-tls-penalty or --meso-minor-penalty
+    double getCurrentStoppingTimeSeconds() const;
 
     /// Replaces the current route by the given one
-    bool replaceRoute(const MSRoute* route, bool onInit = false, int offset = 0, bool addStops = true);
+    bool replaceRoute(const MSRoute* route,  const std::string& info, bool onInit = false, int offset = 0, bool addRouteStops = true, bool removeStops = true);
 
     /** @brief Returns whether the vehicle is allowed to pass the next junction
      * @return true iff the vehicle may drive over the next junction
@@ -321,6 +341,19 @@ public:
     void updateDetectors(SUMOTime currentTime, const bool isLeave,
                          const MSMoveReminder::Notification reason = MSMoveReminder::NOTIFICATION_JUNCTION);
 
+    /** @brief Returns the velocity/lane influencer
+     *
+     * If no influencer was existing before, one is built, first
+     * @return Reference to this vehicle's speed influencer
+     */
+    BaseInfluencer& getBaseInfluencer();
+
+    const BaseInfluencer* getBaseInfluencer() const;
+
+    bool hasInfluencer() const {
+        return myInfluencer != nullptr;
+    }
+
     /// @name state io
     //@{
 
@@ -349,11 +382,7 @@ protected:
     /// @brief The time at which the vehicle was blocked on its current segment
     SUMOTime myBlockTime;
 
-    /// @brief where to stop
-    std::map<const MESegment* const, SUMOTime> myStops;
+    /// @brief An instance of a velocity/lane influencing instance; built in "getInfluencer"
+    BaseInfluencer* myInfluencer;
 
 };
-
-#endif
-
-/****************************************************************************/
