@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSEdge.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
@@ -6,62 +18,46 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSEdge.h 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // A road/street connecting two junctions
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSEdge_h
-#define MSEdge_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <map>
 #include <string>
 #include <iostream>
+#ifdef HAVE_FOX
+#include <utils/foxtools/fxheader.h>
+#endif
 #include <utils/common/Named.h>
 #include <utils/common/Parameterised.h>
 #include <utils/common/SUMOTime.h>
-#include <utils/vehicle/SUMOVehicle.h>
 #include <utils/common/SUMOVehicleClass.h>
-#include <utils/common/ValueTimeLine.h>
-#include <utils/common/UtilExceptions.h>
+#include <utils/geom/Boundary.h>
+#include <utils/router/ReversedEdge.h>
+#include <utils/router/RailEdge.h>
+#include <utils/vehicle/SUMOVehicle.h>
+#include <utils/vehicle/SUMOTrafficObject.h>
 #include "MSNet.h"
-#include "MSVehicleType.h"
 
 
 // ===========================================================================
 // class declarations
 // ===========================================================================
-class MSLaneChanger;
+class Boundary;
 class OutputDevice;
 class SUMOVehicle;
 class SUMOVehicleParameter;
 class MSVehicle;
 class MSLane;
+class MSLaneChanger;
 class MSPerson;
 class MSJunction;
 class MSEdge;
-class MSContainer;
+class MSTransportable;
+
 
 // ===========================================================================
 // class definitions
@@ -76,42 +72,20 @@ class MSContainer;
 
 typedef std::vector<MSEdge*> MSEdgeVector;
 typedef std::vector<const MSEdge*> ConstMSEdgeVector;
+typedef std::vector<std::pair<const MSEdge*, const MSEdge*> > MSConstEdgePairVector;
 
 class MSEdge : public Named, public Parameterised {
-public:
-    /**
-     * @enum EdgeBasicFunction
-     * @brief Defines possible edge types
-     *
-     * For different purposes, it is necessary to know whether the edge
-     *  is a normal street or something special.
-     */
-    enum EdgeBasicFunction {
-        /// @brief The purpose of the edge is not known
-        EDGEFUNCTION_UNKNOWN = -1,
-        /// @brief The edge is a normal street
-        EDGEFUNCTION_NORMAL = 0,
-        /// @brief The edge is a macroscopic connector (source/sink)
-        EDGEFUNCTION_CONNECTOR = 1,
-        /// @brief The edge is an internal edge
-        EDGEFUNCTION_INTERNAL = 2,
-        /// @brief The edge is a district edge
-        EDGEFUNCTION_DISTRICT = 3,
-        /// @brief The edge is a pedestrian crossing (a special type of internal edge)
-        EDGEFUNCTION_CROSSING = 4,
-        /// @brief The edge is a pedestrian walking area (a special type of internal edge)
-        EDGEFUNCTION_WALKINGAREA = 5
-    };
+private:
+    /** @brief "Map" from vehicle class to allowed lanes */
+    typedef std::vector<std::pair<SVCPermissions, std::shared_ptr<const std::vector<MSLane*> > > > AllowedLanesCont;
 
-
-    /** @brief Suceeding edges (keys) and allowed lanes to reach these edges (values). */
-    typedef std::map< const MSEdge*, std::vector<MSLane*>* > AllowedLanesCont;
-
-    /** @brief Map from vehicle types to lanes that may be used to reach one of the next edges */
-    typedef std::map< SUMOVehicleClass, AllowedLanesCont > ClassedAllowedLanesCont;
+    /** @brief Succeeding edges (keys) and allowed lanes to reach these edges (values). */
+    typedef std::map<const MSEdge*, AllowedLanesCont> AllowedLanesByTarget;
 
 
 public:
+    friend class MSLaneChangerSublane; // needs access to myLaneChanger
+
     /** @brief Constructor.
      *
      * After calling this constructor, the edge is not yet initialised
@@ -123,8 +97,9 @@ public:
      * @param[in] function A basic type of the edge
      * @param[in] streetName The street name for that edge
      */
-    MSEdge(const std::string& id, int numericalID, const EdgeBasicFunction function,
-           const std::string& streetName, const std::string& edgeType, int priority);
+    MSEdge(const std::string& id, int numericalID, const SumoXMLEdgeFunc function,
+           const std::string& streetName, const std::string& edgeType, int priority,
+           double distance);
 
 
     /// @brief Destructor.
@@ -145,7 +120,7 @@ public:
 
 
     /// @todo Has to be called after all edges were built and all connections were set...; Still, is not very nice
-    void closeBuilding();
+    virtual void closeBuilding();
 
     /// Has to be called after all sucessors and predecessors have been set (after closeBuilding())
     void buildLaneChanger();
@@ -153,7 +128,7 @@ public:
     /* @brief returns whether initizliaing a lane change is permitted on this edge
      * @note Has to be called after all sucessors and predecessors have been set (after closeBuilding())
      */
-    bool allowsLaneChanging();
+    bool allowsLaneChanging() const;
 
     /// @name Access to the edge's lanes
     /// @{
@@ -180,19 +155,37 @@ public:
      *
      * @param[in] lane The base lane
      * @param[in] offset The offset of the result lane
+     * @param[in] includeOpposte Whether an opposite direction lane may be returned
      * @todo This method searches for the given in the container; probably, this could be done faster
      */
-    MSLane* parallelLane(const MSLane* const lane, int offset) const;
+    MSLane* parallelLane(const MSLane* const lane, int offset, bool includeOpposite = true) const;
 
 
     /** @brief Returns this edge's lanes
      *
      * @return This edge's lanes
      */
-    const std::vector<MSLane*>& getLanes() const {
+    inline const std::vector<MSLane*>& getLanes() const {
         return *myLanes;
     }
 
+    /// @brief return total number of vehicles on this edges lanes or segments
+    int getVehicleNumber() const;
+
+    /// @brief return vehicles on this edges lanes or segments
+    std::vector<const SUMOVehicle*> getVehicles() const;
+
+    double getBruttoOccupancy() const;
+
+    /// @brief return flow based on meanSpead @note: may produced incorrect results when jammed
+    double getFlow() const;
+
+    /// @brief return accumated waiting time for all vehicles on this edges lanes or segments
+    double getWaitingSeconds() const;
+
+    /// @brief return mean occupancy on this edges lanes or segments
+    double getOccupancy() const;
+
     /** @brief Returns this edge's persons set.
      *  @brief Avoids the creation of new vector as in getSortedPersons
      *
@@ -217,24 +210,30 @@ public:
 
     /** @brief Get the allowed lanes to reach the destination-edge.
      *
-     * If there is no such edge, get 0. Then you are on the wrong edge.
+     * If there is no such edge, return nullptr. Then you are on the wrong edge.
      *
      * @param[in] destination The edge to reach
      * @param[in] vclass The vehicle class for which this information shall be returned
-     * @return The lanes that may be used to reach the given edge, 0 if no such lanes exist
+     * @return The lanes that may be used to reach the given edge, nullptr if no such lanes exist
      */
     const std::vector<MSLane*>* allowedLanes(const MSEdge& destination,
             SUMOVehicleClass vclass = SVC_IGNORING) const;
 
 
+
     /** @brief Get the allowed lanes for the given vehicle class.
      *
-     * If there is no such edge, get 0. Then you are on the wrong edge.
+     * If there is no such edge, return nullptr. Then you are on the wrong edge.
      *
      * @param[in] vclass The vehicle class for which this information shall be returned
      * @return The lanes that may be used by the given vclass
      */
     const std::vector<MSLane*>* allowedLanes(SUMOVehicleClass vclass = SVC_IGNORING) const;
+
+    inline bool isConnectedTo(const MSEdge& destination, SUMOVehicleClass vclass) const {
+        const std::vector<MSLane*>* const lanes = allowedLanes(destination, vclass);
+        return lanes != nullptr && !lanes->empty();
+    }
     /// @}
 
 
@@ -242,31 +241,53 @@ public:
     /// @name Access to other edge attributes
     /// @{
 
-    /** @brief Returns the edge type (EdgeBasicFunction)
-     * @return This edge's EdgeBasicFunction
-     * @see EdgeBasicFunction
+    /** @brief Returns the edge type (SumoXMLEdgeFunc)
+     * @return This edge's SumoXMLEdgeFunc
+     * @see SumoXMLEdgeFunc
      */
-    EdgeBasicFunction getPurpose() const {
+    inline SumoXMLEdgeFunc getFunction() const {
         return myFunction;
     }
 
+    /// @brief return whether this edge is an internal edge
+    inline bool isNormal() const {
+        return myFunction == SumoXMLEdgeFunc::NORMAL;
+    }
+
     /// @brief return whether this edge is an internal edge
     inline bool isInternal() const {
-        return myFunction == EDGEFUNCTION_INTERNAL;
+        return myFunction == SumoXMLEdgeFunc::INTERNAL;
     }
 
     /// @brief return whether this edge is a pedestrian crossing
     inline bool isCrossing() const {
-        return myFunction == EDGEFUNCTION_CROSSING;
+        return myFunction == SumoXMLEdgeFunc::CROSSING;
+    }
+
+
+    /// @brief check and register the opposite superposable edge if any
+    void checkAndRegisterBiDirEdge(const std::string& bidiID = "");
+
+    /// @brief return opposite superposable/congruent edge, if it exist and 0 else
+    inline const MSEdge* getBidiEdge() const {
+        return myBidiEdge;
     }
 
     /// @brief return whether this edge is walking area
     inline bool isWalkingArea() const {
-        return myFunction == EDGEFUNCTION_WALKINGAREA;
+        return myFunction == SumoXMLEdgeFunc::WALKINGAREA;
     }
 
-    inline bool isTaz() const {
-        return myFunction == EDGEFUNCTION_DISTRICT;
+    inline bool isTazConnector() const {
+        return myFunction == SumoXMLEdgeFunc::CONNECTOR;
+    }
+
+    void setOtherTazConnector(const MSEdge* edge) {
+        myOtherTazConnector = edge;
+    }
+
+    const MSEdge* getOtherTazConnector() const {
+        return myOtherTazConnector;
     }
 
     /** @brief Returns the numerical id of the edge
@@ -289,11 +310,20 @@ public:
         return myEdgeType;
     }
 
+    // @brief try to infer edge type for internal edges
+    void inferEdgeType();
+
     /** @brief Returns the priority of the edge
      */
     int getPriority() const {
         return myPriority;
     }
+
+    /** @brief Returns the kilometrage/mileage at the start of the edge
+     */
+    double getDistance() const {
+        return myDistance;
+    }
     /// @}
 
     /**@brief Sets the crossed edge ids for a crossing edge
@@ -320,11 +350,7 @@ public:
      * This is mainly used by the taz (district) parsing
      * @param[in] edge The edge to add
      */
-    void addSuccessor(MSEdge* edge) {
-        mySuccessors.push_back(edge);
-        edge->myPredecessors.push_back(this);
-    }
-
+    void addSuccessor(MSEdge* edge, const MSEdge* via = nullptr);
 
     /** @brief Returns the number of edges that may be reached from this edge
      * @return The number of following edges
@@ -334,18 +360,17 @@ public:
     }
 
 
-    /** @brief Returns the following edges
-     */
-    const MSEdgeVector& getSuccessors() const {
-        return mySuccessors;
-    }
-
-
     /** @brief Returns the following edges, restricted by vClass
      * @param[in] vClass The vClass for which to restrict the successors
      * @return The eligible following edges
      */
-    const MSEdgeVector& getSuccessors(SUMOVehicleClass vClass) const;
+    const MSEdgeVector& getSuccessors(SUMOVehicleClass vClass = SVC_IGNORING) const;
+
+    /** @brief Returns the following edges with internal vias, restricted by vClass
+     * @param[in] vClass The vClass for which to restrict the successors
+     * @return The eligible following edges
+     */
+    const MSConstEdgePairVector& getViaSuccessors(SUMOVehicleClass vClass = SVC_IGNORING) const;
 
 
     /** @brief Returns the number of edges this edge is connected to
@@ -374,10 +399,7 @@ public:
     }
 
 
-    void setJunctions(MSJunction* from, MSJunction* to) {
-        myFromJunction = from;
-        myToJunction = to;
-    }
+    void setJunctions(MSJunction* from, MSJunction* to);
     /// @}
 
 
@@ -431,12 +453,12 @@ public:
 
     /// @brief returns the minimum travel time for the given vehicle
     inline double getMinimumTravelTime(const SUMOVehicle* const veh) const {
-        if (myFunction == EDGEFUNCTION_DISTRICT) {
+        if (myFunction == SumoXMLEdgeFunc::CONNECTOR) {
             return 0;
         } else if (veh != 0) {
-            return getLength() / getVehicleMaxSpeed(veh);
+            return getLength() / getVehicleMaxSpeed(veh) + myTimePenalty;
         } else {
-            return getLength() / getSpeedLimit();
+            return myEmptyTraveltime;
         }
     }
 
@@ -452,6 +474,8 @@ public:
         return MSNet::getInstance()->getTravelTime(edge, veh, time);
     }
 
+    static double getTravelTimeAggregated(const MSEdge* const edge, const SUMOVehicle* const veh, double time);
+
     /** @brief Returns the averaged speed used by the routing device
      */
     double getRoutingSpeed() const;
@@ -481,6 +505,8 @@ public:
      */
     bool insertVehicle(SUMOVehicle& v, SUMOTime time, const bool checkOnly = false, const bool forceCheck = false) const;
 
+    /// @brief check whether the given departSpeed is valid for this edge
+    bool validateDepartSpeed(SUMOVehicle& v) const;
 
     /** @brief Finds the emptiest lane allowing the vehicle class
      *
@@ -535,7 +561,7 @@ public:
 
 
     /** @brief Performs lane changing on this edge */
-    virtual void changeLanes(SUMOTime t);
+    void changeLanes(SUMOTime t) const;
 
 
     /// @todo extension: inner junctions are not filled
@@ -545,15 +571,31 @@ public:
     /// @brief returns the length of all internal edges on the junction until reaching the non-internal edge followerAfterInternal.
     double getInternalFollowingLengthTo(const MSEdge* followerAfterInternal) const;
 
+    /// @brief if this edge is an internal edge, return its first normal predecessor, otherwise the edge itself
+    const MSEdge* getNormalBefore() const;
+
+    /// @brief if this edge is an internal edge, return its first normal successor, otherwise the edge itself
+    const MSEdge* getNormalSuccessor() const;
+
     /// @brief Returns whether the vehicle (class) is not allowed on the edge
     inline bool prohibits(const SUMOVehicle* const vehicle) const {
-        if (vehicle == 0) {
+        if (vehicle == nullptr) {
             return false;
         }
-        const SUMOVehicleClass svc = vehicle->getVehicleType().getVehicleClass();
+        const SUMOVehicleClass svc = vehicle->getVClass();
         return (myCombinedPermissions & svc) != svc;
     }
 
+    /** @brief Returns whether this edge has restriction parameters forbidding the given vehicle to pass it
+     * The restriction mechanism is not implemented yet for the microsim, so it always returns false.
+     * @param[in] vehicle The vehicle for which the information has to be returned
+     * @return Whether the vehicle must not enter this edge
+     */
+    inline bool restricts(const SUMOVehicle* const /* vehicle */) const {
+        return false;
+    }
+
+    /// @brief Returns the combined permissions of all lanes of this edge
     inline SVCPermissions getPermissions() const {
         return myCombinedPermissions;
     }
@@ -572,12 +614,19 @@ public:
 
     void rebuildAllowedLanes();
 
+    void rebuildAllowedTargets(const bool updateVehicles = true);
+
 
     /** @brief optimistic air distance heuristic for use in routing
      * @param[in] other The edge to which the distance shall be returned
+     * @param[in] doBoundaryEstimate whether the distance should be estimated by looking at the distance of the bounding boxes
      * @return The distance to the other edge
      */
-    double getDistanceTo(const MSEdge* other) const;
+    double getDistanceTo(const MSEdge* other, const bool doBoundaryEstimate = false) const;
+
+
+    /// @brief return the coordinates of the center of the given stop
+    static const Position getStopPosition(const SUMOVehicleParameter::Stop& stop);
 
 
     /** @brief return the length of the edge
@@ -607,18 +656,12 @@ public:
      * @caution Only the first lane is considered
      * @return The maximum velocity on this edge for the given vehicle
      */
-    double getVehicleMaxSpeed(const SUMOVehicle* const veh) const;
+    double getVehicleMaxSpeed(const SUMOTrafficObject* const veh) const;
 
-    virtual void addPerson(MSTransportable* p) const {
-        myPersons.insert(p);
-    }
 
-    virtual void removePerson(MSTransportable* p) const {
-        std::set<MSTransportable*>::iterator i = myPersons.find(p);
-        if (i != myPersons.end()) {
-            myPersons.erase(i);
-        }
-    }
+    virtual void addPerson(MSTransportable* p) const;
+
+    virtual void removePerson(MSTransportable* p) const;
 
     /// @brief Add a container to myContainers
     virtual void addContainer(MSTransportable* container) const {
@@ -645,25 +688,61 @@ public:
         myAmDelayed = true;
     }
 
+    // return whether there have been vehicles on this edge at least once
+    inline bool isDelayed() const {
+        return myAmDelayed || myBidiEdge == nullptr || myBidiEdge->myAmDelayed;
+    }
+
     bool hasLaneChanger() const {
         return myLaneChanger != 0;
     }
 
     /// @brief whether this edge allows changing to the opposite direction edge
-    bool canChangeToOpposite();
+    bool canChangeToOpposite() const;
+
+    /// @brief Returns the opposite direction edge if on exists else a nullptr
+    const MSEdge* getOppositeEdge() const;
 
     /// @brief get the mean speed
     double getMeanSpeed() const;
 
+    /// @brief get the mean speed of all bicycles on this edge
+    double getMeanSpeedBike() const;
+
     /// @brief whether any lane has a minor link
     bool hasMinorLink() const;
 
+    /// @brief return whether this edge is at the fringe of the network
+    bool isFringe() const {
+        return myAmFringe;
+    }
+
+    /// @brief whether this lane is selected in the GUI
+    virtual bool isSelected() const {
+        return false;
+    }
+
     /// @brief grant exclusive access to the mesoscopic state
     virtual void lock() const {}
 
     /// @brief release exclusive access to the mesoscopic state
     virtual void unlock() const {};
 
+    /// @brief Adds a vehicle to the list of waiting vehicles
+    void addWaiting(SUMOVehicle* vehicle) const;
+
+    /// @brief Removes a vehicle from the list of waiting vehicles
+    void removeWaiting(const SUMOVehicle* vehicle) const;
+
+    /* @brief returns a vehicle that is waiting for a for a person or a container at this edge at the given position
+     * @param[in] transportable The person or container that wants to ride
+     * @param[in] position The vehicle shall be positioned in the interval [position - t, position + t], where t is some tolerance
+     */
+    SUMOVehicle* getWaitingVehicle(MSTransportable* transportable, const double position) const;
+
+    /// @brief update meso segment parameters
+    void updateMesoType();
+
     /** @brief Inserts edge into the static dictionary
         Returns true if the key id isn't already in the dictionary. Otherwise
         returns false. */
@@ -712,6 +791,20 @@ public:
     /// @}
 
 
+    ReversedEdge<MSEdge, SUMOVehicle>* getReversedRoutingEdge() const {
+        if (myReversedRoutingEdge == nullptr) {
+            myReversedRoutingEdge = new ReversedEdge<MSEdge, SUMOVehicle>(this);
+        }
+        return myReversedRoutingEdge;
+    }
+
+    RailEdge<MSEdge, SUMOVehicle>* getRailwayRoutingEdge() const {
+        if (myRailwayRoutingEdge == nullptr) {
+            myRailwayRoutingEdge = new RailEdge<MSEdge, SUMOVehicle>(this);
+        }
+        return myRailwayRoutingEdge;
+    }
+
 protected:
     /** @class by_id_sorter
      * @brief Sorts edges by their ids
@@ -723,7 +816,7 @@ protected:
 
         /// @brief comparing operator
         int operator()(const MSEdge* const e1, const MSEdge* const e2) const {
-            return e1->getID() < e2->getID();
+            return e1->getNumericalID() < e2->getNumericalID();
         }
 
     };
@@ -743,22 +836,6 @@ protected:
     };
 
 
-    /** @brief Get the allowed lanes to reach the destination-edge.
-     *
-     * If there is no such edge, get 0. Then you are on the wrong edge.
-     *
-     * @param[in] destination The edge to reach
-     * @param[in] vclass The vehicle class for which this information shall be returned
-     * @return The lanes that may be used to reach the given edge, 0 if no such lanes exist
-     */
-    const std::vector<MSLane*>* allowedLanes(const MSEdge* destination,
-            SUMOVehicleClass vclass = SVC_IGNORING) const;
-
-
-    /// @brief lookup in map and return 0 if not found
-    const std::vector<MSLane*>* getAllowedLanesWithDefault(const AllowedLanesCont& c, const MSEdge* dest) const;
-
-
     /// @brief return upper bound for the depart position on this edge
     double getDepartPosBound(const MSVehicle& veh, bool upper = true) const;
 
@@ -767,13 +844,13 @@ protected:
     const int myNumericalID;
 
     /// @brief Container for the edge's lane; should be sorted: (right-hand-traffic) the more left the lane, the higher the container-index
-    const std::vector<MSLane*>* myLanes;
+    std::shared_ptr<const std::vector<MSLane*> > myLanes;
 
     /// @brief This member will do the lane-change
     MSLaneChanger* myLaneChanger;
 
     /// @brief the purpose of the edge
-    const EdgeBasicFunction myFunction;
+    const SumoXMLEdgeFunc myFunction;
 
     /// @brief Vaporizer counter
     int myVaporizationRequests;
@@ -792,6 +869,8 @@ protected:
     /// @brief The succeeding edges
     MSEdgeVector mySuccessors;
 
+    MSConstEdgePairVector myViaSuccessors;
+
     /// @brief The preceeding edges
     MSEdgeVector myPredecessors;
 
@@ -808,19 +887,21 @@ protected:
     /// @name Storages for allowed lanes (depending on vehicle classes)
     /// @{
 
-    /// @brief Associative container from destination-edge to allowed-lanes.
+    /// @brief Associative container from vehicle class to allowed-lanes.
     AllowedLanesCont myAllowed;
 
-    /// @brief From vehicle class to lanes allowed to be used by it
-    // @note: this map is filled on demand
-    mutable ClassedAllowedLanesCont myClassedAllowed;
+    /// @brief From target edge to lanes allowed to be used to reach it
+    AllowedLanesByTarget myAllowedTargets;
 
     /// @brief The intersection of lane permissions for this edge
-    SVCPermissions myMinimumPermissions;
+    SVCPermissions myMinimumPermissions = SVCAll;
     /// @brief The union of lane permissions for this edge
-    SVCPermissions myCombinedPermissions;
+    SVCPermissions myCombinedPermissions = 0;
     /// @}
 
+    /// @brief the other taz-connector if this edge isTazConnector, otherwise nullptr
+    const MSEdge* myOtherTazConnector;
+
     /// @brief the real-world name of this edge (need not be unique)
     std::string myStreetName;
 
@@ -830,6 +911,9 @@ protected:
     /// @brief the priority of the edge (used during network creation)
     const int myPriority;
 
+    /// @brief the kilometrage/mileage at the start of the edge
+    const double myDistance;
+
     /// Edge width [m]
     double myWidth;
 
@@ -839,12 +923,18 @@ protected:
     /// @brief the traveltime on the empty edge (cached value for speedup)
     double myEmptyTraveltime;
 
+    /// @brief flat penalty when computing traveltime
+    double myTimePenalty;
+
     /// @brief whether this edge had a vehicle with less than max speed on it
     mutable bool myAmDelayed;
 
     /// @brief whether this edge belongs to a roundabout
     bool myAmRoundabout;
 
+    /// @brief whether this edge is at the network fringe
+    bool myAmFringe;
+
     /// @brief the right side for each sublane on this edge
     std::vector<double> mySublaneSides;
 
@@ -869,17 +959,39 @@ protected:
     /// @brief The successors available for a given vClass
     mutable std::map<SUMOVehicleClass, MSEdgeVector> myClassesSuccessorMap;
 
+    /// @brief The successors available for a given vClass
+    mutable std::map<SUMOVehicleClass, MSConstEdgePairVector> myClassesViaSuccessorMap;
+
+    /// @brief The bounding rectangle of end nodes incoming or outgoing edges for taz connectors or of my own start and end node for normal edges
+    Boundary myBoundary;
+
+    /// @brief List of waiting vehicles
+    mutable std::vector<SUMOVehicle*> myWaiting;
+
+#ifdef HAVE_FOX
+    /// @brief Mutex for accessing waiting vehicles
+    mutable FXMutex myWaitingMutex;
+
+    /// @brief Mutex for accessing successor edges
+    mutable FXMutex mySuccessorMutex;
+#endif
+
 private:
+
+    /// @brief the oppositing superposable edge
+    const MSEdge* myBidiEdge;
+
+    /// @brief a reversed version for backward routing
+    mutable ReversedEdge<MSEdge, SUMOVehicle>* myReversedRoutingEdge = nullptr;
+    mutable RailEdge<MSEdge, SUMOVehicle>* myRailwayRoutingEdge = nullptr;
+
     /// @brief Invalidated copy constructor.
     MSEdge(const MSEdge&);
 
     /// @brief assignment operator.
-    MSEdge& operator=(const MSEdge&);
+    MSEdge& operator=(const MSEdge&) = delete;
 
+    bool isSuperposable(const MSEdge* other);
+
+    void addToAllowed(const SVCPermissions permissions, std::shared_ptr<const std::vector<MSLane*> > allowedLanes, AllowedLanesCont& laneCont) const;
 };
-
-
-#endif
-
-/****************************************************************************/
-
