@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSEdgeControl.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
@@ -7,33 +19,11 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 09 Apr 2001
-/// @version $Id: MSEdgeControl.h 22998 2017-02-17 08:46:10Z namdre $
 ///
 // Stores edges and lanes, performs moving of vehicle
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSEdgeControl_h
-#define MSEdgeControl_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <map>
@@ -41,8 +31,24 @@
 #include <iostream>
 #include <list>
 #include <set>
+#include <queue>
 #include <utils/common/SUMOTime.h>
 #include <utils/common/Named.h>
+#include <utils/common/StopWatch.h>
+#include <utils/router/SUMOAbstractRouter.h>
+#include <utils/router/RouterProvider.h>
+#include <utils/vehicle/SUMOVehicle.h>
+
+#include <utils/foxtools/FXSynchQue.h>
+#include <utils/foxtools/FXSynchSet.h>
+//#define THREAD_POOL
+#ifdef THREAD_POOL
+#include <utils/threadpool/WorkStealingThreadPool.h>
+#else
+#ifdef HAVE_FOX
+#include <utils/foxtools/FXWorkerThread.h>
+#endif
+#endif
 
 
 // ===========================================================================
@@ -50,6 +56,7 @@
 // ===========================================================================
 class MSEdge;
 class MSLane;
+class MSJunction;
 class OutputDevice;
 
 typedef std::vector<MSEdge*> MSEdgeVector;
@@ -74,6 +81,8 @@ typedef std::vector<MSEdge*> MSEdgeVector;
 class MSEdgeControl {
 
 public:
+    typedef RouterProvider<MSEdge, MSLane, MSJunction, SUMOVehicle> MSRouterProvider;
+
     /** @brief Constructor
      *
      * Builds LaneUsage information for each lane and assigns them to lanes.
@@ -109,6 +118,13 @@ public:
      */
     void planMovements(SUMOTime t);
 
+    /** @brief Register junction approaches for all vehicles after velocities
+     * have been planned. This is a prerequisite for executeMovements
+     *
+     * This method goes through all active lanes calling their "setJunctionApproaches" method.
+     */
+    void setJunctionApproaches(SUMOTime t);
+
 
     /** @brief Executes planned vehicle movements with regards to right-of-way
      *
@@ -121,9 +137,12 @@ public:
      *
      * @see MSLane::executeMovements
      * @see MSLane::integrateNewVehicle
-     * @todo When moving to parallel processing, the usage of myWithVehicles2Integrate would get insecure!!
      */
     void executeMovements(SUMOTime t);
+
+    void needsVehicleIntegration(MSLane* const l) {
+        myWithVehicles2Integrate.push_back(l);
+    }
     /// @}
 
 
@@ -135,7 +154,7 @@ public:
      *
      * @see MSEdge::changeLanes
      */
-    void changeLanes(SUMOTime t);
+    void changeLanes(const SUMOTime t);
 
 
     /** @brief Detect collisions
@@ -161,13 +180,6 @@ public:
     }
 
 
-    /** @brief Returns the list of names of all known edges
-     *
-     * @return a vector of names of all known edges
-     */
-    std::vector<std::string> getEdgeNames() const;
-
-
     /** @brief Informs the control that the given lane got active
      *
      * @param[in] l The activated lane
@@ -175,9 +187,22 @@ public:
      */
     void gotActive(MSLane* l);
 
+    /// @brief trigger collision checking for inactive lane
+    void checkCollisionForInactive(MSLane* l);
+
     /// @brief apply additional restrictions
     void setAdditionalRestrictions();
 
+    /// @brief update meso edge type parameters
+    void setMesoTypes();
+
+#ifndef THREAD_POOL
+#ifdef HAVE_FOX
+    FXWorkerThread::Pool& getThreadPool() {
+        return myThreadPool;
+    }
+#endif
+#endif
 
 public:
     /**
@@ -194,16 +219,41 @@ public:
     struct LaneUsage {
         /// @brief The described lane
         MSLane* lane;
-        /// @brief The lane left to the described lane (==lastNeigh if none)
-        std::vector<MSLane*>::const_iterator firstNeigh;
-        /// @brief The end of this lane's edge's lane container
-        std::vector<MSLane*>::const_iterator lastNeigh;
         /// @brief Information whether this lane is active
         bool amActive;
         /// @brief Information whether this lane belongs to a multi-lane edge
         bool haveNeighbors;
     };
 
+#ifdef HAVE_FOX
+    /**
+     * @class WorkerThread
+     * @brief the thread which provides the router instance as context
+     */
+    class WorkerThread : public FXWorkerThread {
+    public:
+        WorkerThread(FXWorkerThread::Pool& pool)
+            : FXWorkerThread(pool), myRouterProvider(nullptr) {}
+
+        bool setRouterProvider(MSRouterProvider* routerProvider) {
+            if (myRouterProvider == nullptr) {
+                myRouterProvider = routerProvider;
+                return true;
+            }
+            return false;
+        }
+        SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouter(SUMOVehicleClass svc) const {
+            return myRouterProvider->getVehicleRouter(svc);
+        }
+        virtual ~WorkerThread() {
+            stop();
+            delete myRouterProvider;
+        }
+    private:
+        MSRouterProvider* myRouterProvider;
+    };
+#endif
+
 private:
     /// @brief Loaded edges
     MSEdgeVector myEdges;
@@ -218,14 +268,29 @@ private:
     std::list<MSLane*> myActiveLanes;
 
     /// @brief A storage for lanes which shall be integrated because vehicles have moved onto them
-    std::vector<MSLane*> myWithVehicles2Integrate;
+    FXSynchQue<MSLane*, std::vector<MSLane*> > myWithVehicles2Integrate;
 
     /// @brief Lanes which changed the state without informing the control
-    std::set<MSLane*, Named::ComparatorIdLess> myChangedStateLanes;
+    std::set<MSLane*, ComparatorNumericalIdLess> myChangedStateLanes;
 
     /// @brief The list of active (not empty) lanes
     std::vector<SUMOTime> myLastLaneChange;
 
+    /// @brief Additional lanes for which collision checking must be performed
+    FXSynchSet<MSLane*, std::set<MSLane*, ComparatorNumericalIdLess> > myInactiveCheckCollisions;
+
+    double myMinLengthGeometryFactor;
+
+#ifdef THREAD_POOL
+    WorkStealingThreadPool<> myThreadPool;
+#else
+#ifdef HAVE_FOX
+    FXWorkerThread::Pool myThreadPool;
+#endif
+#endif
+
+    std::vector<StopWatch<std::chrono::nanoseconds> > myStopWatch;
+
 private:
     /// @brief Copy constructor.
     MSEdgeControl(const MSEdgeControl&);
@@ -234,9 +299,3 @@ private:
     MSEdgeControl& operator=(const MSEdgeControl&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
