@@ -1,37 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSEdgeWeightsStorage.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    02.11.2009
-/// @version $Id: MSEdgeWeightsStorage.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // A storage for edge travel times and efforts
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSEdgeWeightsStorage_h
-#define MSEdgeWeightsStorage_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <utils/common/SUMOTime.h>
 #include <utils/common/ValueTimeLine.h>
 
 
@@ -65,7 +54,7 @@ public:
      * @param[in] value The value if the requested edge/time is described
      * @return Whether the requested edge/time is described
      */
-    bool retrieveExistingTravelTime(const MSEdge* const e, const SUMOReal t, SUMOReal& value) const;
+    bool retrieveExistingTravelTime(const MSEdge* const e, const double t, double& value) const;
 
 
     /** @brief Returns an effort for an edge and time if stored
@@ -74,7 +63,7 @@ public:
      * @param[in] value The value if the requested edge/time is described
      * @return Whether the requested edge/time is described
      */
-    bool retrieveExistingEffort(const MSEdge* const e, const SUMOReal t, SUMOReal& value) const;
+    bool retrieveExistingEffort(const MSEdge* const e, const double t, double& value) const;
 
 
     /** @brief Adds a travel time information for an edge and a time span
@@ -83,7 +72,7 @@ public:
      * @param[in] end The end of the described time span
      * @param[in] value The travel time value for this edge and time span
      */
-    void addTravelTime(const MSEdge* const e, SUMOReal begin, SUMOReal end, SUMOReal value);
+    void addTravelTime(const MSEdge* const e, double begin, double end, double value);
 
 
     /** @brief Adds an effort information for an edge and a time span
@@ -92,7 +81,7 @@ public:
      * @param[in] end The end of the described time span
      * @param[in] value Theeffort value for this edge and time span
      */
-    void addEffort(const MSEdge* const e, SUMOReal begin, SUMOReal end, SUMOReal value);
+    void addEffort(const MSEdge* const e, double begin, double end, double value);
 
 
     /** @brief Removes the travel time information for an edge
@@ -123,10 +112,10 @@ public:
 
 private:
     /// @brief A map of edge->time->travel time
-    std::map<const MSEdge*, ValueTimeLine<SUMOReal> > myTravelTimes;
+    std::map<const MSEdge*, ValueTimeLine<double> > myTravelTimes;
 
     /// @brief A map of edge->time->effort
-    std::map<const MSEdge*, ValueTimeLine<SUMOReal> > myEfforts;
+    std::map<const MSEdge*, ValueTimeLine<double> > myEfforts;
 
 
 private:
@@ -138,9 +127,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
