@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSEventControl.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Matthias Heppner
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSEventControl.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Stores time-dependant events and executes them at the proper time
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSEventControl_h
-#define MSEventControl_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <utility>
 #include <queue>
@@ -60,18 +50,6 @@ public:
     typedef std::pair< Command*, SUMOTime > Event;
 
 
-    /**
-     * @enum AdaptType
-     * @brief Defines what to do if the insertion time lies before the current simulation time
-     */
-    enum AdaptType {
-        /// @brief Patch the time in a way that it is at least as high as the simulation begin time
-        ADAPT_AFTER_EXECUTION = 1,
-        /// @brief Do nothing
-        NO_CHANGE = 2
-    };
-
-
 public:
     /// @brief Default constructor.
     MSEventControl();
@@ -82,21 +60,12 @@ public:
 
 
     /** @brief Adds an Event.
-     *
-     * If the given execution time step lies before the current and ADAPT_AFTER_EXECUTION
-     *  is passed for adaptation type, the execution time step will be set to the
-     *  current time step.
-     *
-     * Returns the time the event will be executed, really.
      *
      * @param[in] operation The event to add
-     * @param[in] execTimeStep The time the event shall be executed at
-     * @param[in] type The adaptation type
+     * @param[in] execTimeStep The time the event shall be executed at (-1 means at sim start)
      * @see Command
-     * @see AdaptType
      */
-    virtual SUMOTime addEvent(Command* operation, SUMOTime execTimeStep,
-                              AdaptType type);
+    virtual void addEvent(Command* operation, SUMOTime execTimeStep = -1);
 
 
     /** @brief Executes time-dependant commands
@@ -125,42 +94,22 @@ public:
      */
     bool isEmpty();
 
+    /** @brief Remove all events before quick-loading state */
+    void clearState(SUMOTime currentTime, SUMOTime newTime);
 
-    /** @brief Set the current Time.
-     *
-     * This method is only for Unit Testing.
-     * Set the current TimeStep used in addEvent.
-     * Normally the time is set automatically from an instance of MSNet.
-     */
-    void setCurrentTimeStep(SUMOTime time);
-
+    /** @brief get the next scheduled event time for the given command, -2 if it is not scheduled */
+    SUMOTime getEventTime(Command* cmd) const;
 
 protected:
-    /** @brief Sort-criterion for events.
-     *
-     * Sorts events by their execution time
-     */
-    class EventSortCrit {
-    public:
-        /// @brief compares two events
-        bool operator()(const Event& e1, const Event& e2) const {
-            return e1.second > e2.second;
-        }
-    };
+    /// @brief compares two events
+    static bool eventCompare(const Event& e1, const Event& e2) {
+        return e1.second > e2.second;
+    }
 
 
 private:
-    /// @brief Container for time-dependant events, e.g. traffic-light-change.
-    typedef std::priority_queue< Event, std::vector< Event >, EventSortCrit > EventCont;
-
-    /// The Current TimeStep
-    SUMOTime currentTimeStep;
-
     /// @brief Event-container, holds executable events.
-    EventCont myEvents;
-
-    /// get the Current TimeStep used in addEvent.
-    SUMOTime getCurrentTimeStep();
+    std::vector<Event> myEvents;
 
 
 private:
@@ -172,9 +121,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
