@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2003-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSGlobals.h
 /// @author  Daniel Krajzewicz
 /// @author  Christian Roessel
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    late summer 2003
-/// @version $Id: MSGlobals.h 23379 2017-03-14 13:02:11Z namdre $
 ///
 // Some static variables for faster access
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2003-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSGlobals_h
-#define MSGlobals_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <map>
 #include <utils/common/SUMOTime.h>
@@ -69,6 +59,9 @@ public:
     /** The time to detect grid locks on highways */
     static SUMOTime gTimeToGridlockHighways;
 
+    /** The time to wait for teleport on disconected routes */
+    static SUMOTime gTimeToTeleportDisconnected;
+
     /** Information how long a vehicle must wait for impatience to grow from 0 to 1 */
     static SUMOTime gTimeToImpatience;
 
@@ -100,25 +93,54 @@ public:
     /** Information whether limited junction control shall be used */
     static bool gMesoLimitedJunctionControl;
 
-    /** Information whether overtaking is enabled in the mesoscopic simulation */
-    static bool gMesoOvertaking;
-
-    /** scaling factor for macroscopic time penalty when passing tls controlled intersection */
-    static double gMesoTLSPenalty;
-
-    /** penalty time for passing a minor link */
-    static SUMOTime gMesoMinorPenalty;
-
     /// mesoscopic simulation infrastructure
     static MELoop* gMesoNet;
 
     /// length of memory for waiting times (in millisecs)
     static SUMOTime gWaitingTimeMemory;
 
+    /// default value for the interval between two action points for MSVehicle (defaults to DELTA_T)
+    static SUMOTime gActionStepLength;
+
+    // whether Kirchhoff's laws are used for solving overhead wire circuit
+    static bool gOverheadWireSolver;
+
+    /// encoding of the string-option default.emergencydecel
+    static double gDefaultEmergencyDecel;
+
+    /// whether the simulation should replay previous stop times
+    static bool gUseStopEnded;
+
+    /// whether unit tests are being run
+    static bool gUnitTests;
+
+    /// whether the simulationLoop is in the lane changing phase
+    static bool gComputeLC;
+
+    /// how many threads to use for simulation
+    static int gNumSimThreads;
+
+    /// how many threads to use
+    static int gNumThreads;
+
+    /// treshold for warning about strong deceleration
+    static double gEmergencyDecelWarningThreshold;
+
+    /// time penalty for passing a minor link when routing
+    static double gMinorPenalty;
+
+    /// whether parking simulation includes manoeuver time and any associated lane blocking
+    static bool gModelParkingManoeuver;
+
+    /// whether sublane simulation is enabled (sublane model or continuous lanechanging)
+    static bool gSublane;
+
+    /// @brief The tolerance to apply when matching waiting persons and vehicles
+    static double gStopTolerance;
+
+    /// @brief The minimum deceleration at a yellow traffic light (only overruled by emergencyDecel)
+    static double gTLSYellowMinDecel;
+
+    /// @brief Whether lefthand-drive is being simulated
+    static bool gLefthand;
 };
-
-
-#endif
-
-/****************************************************************************/
-
