@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSInternalJunction.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Wed, 12 Dez 2001
-/// @version $Id: MSInternalJunction.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // junction.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSInternalJunction_h
-#define MSInternalJunction_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <utils/common/StdDefs.h>
 #include "MSLogicJunction.h"
@@ -56,7 +46,6 @@ class MSLink;
  * The rules for the right-of-way themselves are stored within the associated
  * "MSJunctionLogic" - structure.
  */
-#ifdef HAVE_INTERNAL_LANES
 class MSInternalJunction : public MSLogicJunction {
 public:
     /** @brief Constructor
@@ -88,6 +77,8 @@ public:
 
 private:
 
+    bool indirectBicycleTurn(const MSLane* specialLane, const MSLink* thisLink, const MSLane* foeFirstPart, const MSLink* foeLink) const;
+
     std::vector<MSLink*> myInternalLinkFoes;
     std::vector<MSLane*> myInternalLaneFoes;
 
@@ -98,10 +89,3 @@ private:
     MSInternalJunction& operator=(const MSInternalJunction&);
 
 };
-
-
-#endif
-#endif
-
-/****************************************************************************/
-
