@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSJunctionLogic.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Sascha Krieg
 /// @date    Wed, 12 Dez 2001
-/// @version $Id: MSJunctionLogic.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // kinds of logic-implementations.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSJunctionLogic_h
-#define MSJunctionLogic_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <utils/common/StdDefs.h>
 #include "MSLogicJunction.h"
@@ -49,26 +39,26 @@ public:
     virtual ~MSJunctionLogic();
 
     /// Returns the logic's number of links.
-    unsigned int nLinks();
+    int nLinks();
 
     /// @brief Returns the response for the given link
-    virtual const MSLogicJunction::LinkBits& getResponseFor(unsigned int linkIndex) const {
+    virtual const MSLogicJunction::LinkBits& getResponseFor(int linkIndex) const {
         UNUSED_PARAMETER(linkIndex);
         return myDummyFoes;
     }
 
     /// @brief Returns the foes for the given link
-    virtual const MSLogicJunction::LinkBits& getFoesFor(unsigned int linkIndex) const {
+    virtual const MSLogicJunction::LinkBits& getFoesFor(int linkIndex) const {
         UNUSED_PARAMETER(linkIndex);
         return myDummyFoes;
     }
-    virtual bool getIsCont(unsigned int linkIndex) const {
+    virtual bool getIsCont(int linkIndex) const {
         UNUSED_PARAMETER(linkIndex);
         return false;
     }
 
 
-    unsigned int getLogicSize() const {
+    int getLogicSize() const {
         return myNLinks;
     }
 
@@ -81,10 +71,10 @@ public:
 
 protected:
     /// Constructor.
-    MSJunctionLogic(unsigned int nLinks);
+    MSJunctionLogic(int nLinks);
 
     /// The logic's number of links.
-    unsigned int myNLinks;
+    int myNLinks;
 
     /// @brief A dummy foe container
     static MSLogicJunction::LinkBits myDummyFoes;
@@ -100,9 +90,3 @@ private:
     MSJunctionLogic& operator=(const MSJunctionLogic&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
