@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSLaneChanger.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Fri, 01 Feb 2002
-/// @version $Id: MSLaneChanger.h 20687 2016-05-10 11:27:00Z behrisch $
 ///
 // Performs lane changing of vehicles
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSLaneChanger_h
-#define MSLaneChanger_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include "MSLane.h"
 #include "MSEdge.h"
@@ -55,7 +45,7 @@
 class MSLaneChanger {
 public:
     /// Constructor
-    MSLaneChanger(const std::vector<MSLane*>* lanes, bool allowChanging, bool allowSwap);
+    MSLaneChanger(const std::vector<MSLane*>* lanes, bool allowChanging);
 
     /// Destructor.
     virtual ~MSLaneChanger();
@@ -72,26 +62,37 @@ public:
 
         ChangeElem(MSLane* _lane);
 
-        /// the vehicle in front of the current vehicle
+        /// @brief Register that vehicle belongs to Changer Item to after LC decisions
+        void registerHop(MSVehicle* vehicle);
+
+        ///@brief the leader vehicle for the current change candidate
         MSVehicle*                lead;
-        /// the lane the vehicle is on
+        ///@brief the lane corresponding to this ChangeElem (the current change candidate is on this lane)
         MSLane*                   lane;
-        /// last vehicle that changed into this lane
+        ///@brief last vehicle that changed into this lane
         MSVehicle*                hoppedVeh;
-        /// the vehicle that really wants to change to this lane
+        /// @brief the next vehicle downstream of the ego vehicle that is blocked from changing to this lane
         MSVehicle*                lastBlocked;
-        /// the first vehicle on this edge that wants to change to this lane
+        /// @brief the farthest downstream vehicle on this edge that is blocked from changing to this lane
         MSVehicle*                firstBlocked;
 
-        SUMOReal dens;
+        double dens;
+
+        /// @brief whether changing is possible to either direction
+        bool mayChangeRight;
+        bool mayChangeLeft;
+
+        /// relative indices of internal lanes with the same origin lane (siblings)
+        /// only used for changes on internal edges
+        std::vector<int>          siblings;
 
         /// @name Members which are used only by MSLaneChangerSublane
         /// @{
-        // the vehicles in from of the current vehicle
+        // the vehicles in front of the current vehicle (only on the current edge, continously updated during change() )
         MSLeaderInfo ahead;
 
-
-        void addLink(MSLink* link);
+        // the vehicles in front of the current vehicle (including those on the next edge, contiously update during change() ))
+        MSLeaderDistanceInfo aheadNext;
         ///@}
 
     };
@@ -107,9 +108,15 @@ public:
     /// the iterator moving over the ChangeElems
     typedef Changer::const_iterator ConstChangerIt;
 
+    /// @brief return changer (only to be used by MSLaneChangerSublane from another instance)
+    Changer& getChanger() {
+        return myChanger;
+    }
+
+
 protected:
     /// Initialize the changer before looping over all vehicles.
-    void initChanger();
+    virtual void initChanger();
 
     /** @brief Check if there is a single change-candidate in the changer.
         Returns true if there is one. */
@@ -141,16 +148,26 @@ protected:
 
 
     /** try changing to the opposite direction edge. */
-    virtual bool changeOpposite(std::pair<MSVehicle*, SUMOReal> leader);
+    bool changeOpposite(MSVehicle* vehicle, std::pair<MSVehicle*, double> leader);
+    std::pair<MSVehicle* const, double> getOncomingVehicle(const MSLane* opposite, std::pair<MSVehicle*, double> neighOncoming, double searchDist, double& vMax);
 
     /** Update changer for vehicles that did not change */
     void registerUnchanged(MSVehicle* vehicle);
 
+    /// @brief Take into account traci LC-commands.
+    /// @note This is currently only used within non-actionsteps.
+    void checkTraCICommands(MSVehicle* vehicle);
+
+    /// @brief Execute TraCI LC-commands.
+    /// @note This is currently only used within non-actionsteps for the non-sublane model.
+    /// @return whether lane was changed
+    bool applyTraCICommands(MSVehicle* vehicle);
+
     /** After the possible change, update the changer. */
     virtual void updateChanger(bool vehHasChanged);
 
     /** During lane-change a temporary vehicle container is filled within
-        the lanes (bad pratice to modify foreign members, I know). Swap
+        the lanes (bad practice to modify foreign members, I know). Swap
         this container with the real one. */
     void updateLanes(SUMOTime t);
 
@@ -162,7 +179,7 @@ protected:
      * and possible */
     int checkChangeWithinEdge(
         int laneOffset,
-        const std::pair<MSVehicle* const, SUMOReal>& leader,
+        const std::pair<MSVehicle* const, double>& leader,
         const std::vector<MSVehicle::LaneQ>& preb) const;
 
     /* @brief check whether lane changing in the given direction is desirable
@@ -170,26 +187,40 @@ protected:
     int checkChange(
         int laneOffset,
         const MSLane* targetLane,
-        const std::pair<MSVehicle* const, SUMOReal>& leader,
-        const std::pair<MSVehicle* const, SUMOReal>& neighLead,
-        const std::pair<MSVehicle* const, SUMOReal>& neighFollow,
+        const std::pair<MSVehicle* const, double>& leader,
+        const std::pair<MSVehicle* const, double>& follower,
+        const std::pair<MSVehicle* const, double>& neighLead,
+        const std::pair<MSVehicle* const, double>& neighFollow,
         const std::vector<MSVehicle::LaneQ>& preb) const;
 
-    ///  @brief start the lane change maneuver (and finish it instantly if gLaneChangeDuration == 0)
-    void startChange(MSVehicle* vehicle, ChangerIt& from, int direction);
+    /* @brief call lanechange model to check the merits of an opposite-direction
+     * change and update state accordingly */
+    virtual bool checkChangeOpposite(
+        MSVehicle* vehicle,
+        int laneOffset,
+        MSLane* targetLane,
+        const std::pair<MSVehicle* const, double>& leader,
+        const std::pair<MSVehicle* const, double>& neighLead,
+        const std::pair<MSVehicle* const, double>& neighFollow,
+        const std::vector<MSVehicle::LaneQ>& preb);
 
-    ///  @brief continue a lane change maneuver and return whether the midpoint was passed in this step (used if gLaneChangeDuration > 0)
+
+    /*  @brief start the lane change maneuver (and finish it instantly if gLaneChangeDuration == 0)
+     *  @return False when aborting the change due to being remote controlled*/
+    bool startChange(MSVehicle* vehicle, ChangerIt& from, int direction);
+
+    ///  @brief continue a lane change maneuver and return whether the vehicle has completely moved onto the new lane (used if gLaneChangeDuration > 0)
     bool continueChange(MSVehicle* vehicle, ChangerIt& from);
 
-    std::pair<MSVehicle* const, SUMOReal> getRealFollower(const ChangerIt& target) const;
+    std::pair<MSVehicle* const, double> getRealFollower(const ChangerIt& target) const;
 
-    std::pair<MSVehicle* const, SUMOReal> getRealLeader(const ChangerIt& target) const;
+    std::pair<MSVehicle* const, double> getRealLeader(const ChangerIt& target) const;
 
     /// @brief whether changing to the lane in the given direction should be considered
     bool mayChange(int direction) const;
 
     /// @brief return the closer follower of ego
-    static MSVehicle* getCloserFollower(const SUMOReal maxPos, MSVehicle* follow1, MSVehicle* follow2);
+    static MSVehicle* getCloserFollower(const double maxPos, MSVehicle* follow1, MSVehicle* follow2);
 
     /** @brief Compute the time and space required for overtaking the given leader
      * @param[in] vehicle The vehicle that wants to overtake
@@ -198,7 +229,13 @@ protected:
      * @param[out] timeToOvertake The time for overtaking
      * @param[out] spaceToOvertake The space for overtaking
      */
-    static void computeOvertakingTime(const MSVehicle* vehicle, const MSVehicle* leader, SUMOReal gap, SUMOReal& timeToOvertake, SUMOReal& spaceToOvertake);
+    static void computeOvertakingTime(const MSVehicle* vehicle, double vMax, const MSVehicle* leader, double gap, double& timeToOvertake, double& spaceToOvertake);
+
+    // @brief return leader vehicle that is to be overtaken
+    static std::pair<MSVehicle*, double> getColumnleader(MSVehicle* vehicle, std::pair<MSVehicle*, double> leader, double maxLookAhead = std::numeric_limits<double>::max());
+
+    /// @brief return the next lane in conts beyond lane or nullptr
+    static MSLane* getLaneAfter(const MSLane* lane, const std::vector<MSLane*>& conts);
 
 protected:
     /// Container for ChangeElemements, one for every lane in the edge.
@@ -209,12 +246,9 @@ protected:
         once in the change-process. */
     ChangerIt myCandi;
 
-    /// @brief Whether blocking vehicles may be swapped
-    bool myAllowsSwap;
-
     /* @brief Whether vehicles may start to change lanes on this edge
      * (finishing a change in progress is always permitted) */
-    bool myAllowsChanging;
+    const bool myAllowsChanging;
 
     /// @brief whether this edge allows changing to the opposite direction edge
     const bool myChangeToOpposite;
@@ -229,9 +263,3 @@ private:
     /// Assignment operator.
     MSLaneChanger& operator=(const MSLaneChanger&);
 };
-
-
-#endif
-
-/****************************************************************************/
-
