@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSLogicJunction.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Wed, 12 Dez 2001
-/// @version $Id: MSLogicJunction.h 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // with one ore more logics.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSLogicJunction_h
-#define MSLogicJunction_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include "MSJunction.h"
 #include <utils/common/SUMOTime.h>
@@ -70,9 +60,7 @@ public:
 
     /** @brief Returns all internal lanes on the junction
      */
-    virtual const std::vector<MSLane*>& getInternalLanes() const {
-        return myInternalLanes;
-    }
+    const std::vector<MSLane*> getInternalLanes() const;
 
 protected:
     /** @brief Constructor
@@ -87,6 +75,7 @@ protected:
                     SumoXMLNodeType type,
                     const Position& position,
                     const PositionVector& shape,
+                    const std::string& name,
                     std::vector<MSLane*> incoming,
                     std::vector<MSLane*> internal
                    );
@@ -106,9 +95,3 @@ private:
     MSLogicJunction& operator=(const MSLogicJunction&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
