@@ -1,33 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2008-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSMoveReminder.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    2008-10-27
-/// @version $Id: MSMoveReminder.cpp 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Something on a lane to be noticed about vehicle movement
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2008-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include "MSLane.h"
@@ -39,8 +31,12 @@
 // ===========================================================================
 MSMoveReminder::MSMoveReminder(const std::string& description, MSLane* const lane, const bool doAdd) :
     myLane(lane),
-    myDescription(description) {
-    if (myLane != 0 && doAdd) {
+    myDescription(description)
+#ifdef HAVE_FOX
+    , myNotificationMutex(true)
+#endif
+{
+    if (myLane != nullptr && doAdd) {
         // add reminder to lane
         myLane->addMoveReminder(this);
     }
@@ -48,7 +44,7 @@ MSMoveReminder::MSMoveReminder(const std::string& description, MSLane* const lan
 
 
 void
-MSMoveReminder::updateDetector(SUMOVehicle& veh, double entryPos, double leavePos,
+MSMoveReminder::updateDetector(SUMOTrafficObject& veh, double entryPos, double leavePos,
                                SUMOTime entryTime, SUMOTime currentTime, SUMOTime leaveTime,
                                bool cleanUp) {
     // each vehicle is tracked linearly across its segment. For each vehicle,
@@ -57,7 +53,7 @@ MSMoveReminder::updateDetector(SUMOVehicle& veh, double entryPos, double leavePo
     if (entryTime > currentTime) {
         return; // calibrator may insert vehicles a tiny bit into the future; ignore those
     }
-    std::map<SUMOVehicle*, std::pair<SUMOTime, double> >::iterator j = myLastVehicleUpdateValues.find(&veh);
+    auto j = myLastVehicleUpdateValues.find(&veh);
     if (j != myLastVehicleUpdateValues.end()) {
         // the vehicle already has reported its values before; use these
         // however, if this was called from prepareDetectorForWriting the time
@@ -69,13 +65,12 @@ MSMoveReminder::updateDetector(SUMOVehicle& veh, double entryPos, double leavePo
         }
     }
     assert(entryTime <= currentTime);
-    if ((entryTime < leaveTime) && (entryPos < leavePos)) {
+    if ((entryTime < leaveTime) && (entryPos <= leavePos)) {
         const double timeOnLane = STEPS2TIME(currentTime - entryTime);
         const double speed = (leavePos - entryPos) / STEPS2TIME(leaveTime - entryTime);
         myLastVehicleUpdateValues[&veh] = std::pair<SUMOTime, double>(currentTime, entryPos + speed * timeOnLane);
         assert(timeOnLane >= 0);
-        assert(speed >= 0);
-        notifyMoveInternal(veh, timeOnLane, timeOnLane, speed, speed, speed * timeOnLane, speed * timeOnLane);
+        notifyMoveInternal(veh, timeOnLane, timeOnLane, speed, speed, speed * timeOnLane, speed * timeOnLane, 0.);
     } else {
         // it would be natrual to
         // assert(entryTime == leaveTime);
@@ -91,8 +86,9 @@ MSMoveReminder::updateDetector(SUMOVehicle& veh, double entryPos, double leavePo
 
 
 void
-MSMoveReminder::removeFromVehicleUpdateValues(SUMOVehicle& veh) {
+MSMoveReminder::removeFromVehicleUpdateValues(SUMOTrafficObject& veh) {
     myLastVehicleUpdateValues.erase(&veh);
 }
-/****************************************************************************/
 
+
+/****************************************************************************/
