@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSNet.h
 /// @author  Christian Roessel
 /// @author  Jakob Erdmann
@@ -7,34 +19,13 @@
 /// @author  Eric Nicolay
 /// @author  Clemens Honomichl
 /// @author  Michael Behrisch
+/// @author  Leonhard Luecken
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSNet.h 21198 2016-07-19 11:34:56Z namdre $
 ///
-// The simulated network and simulation perfomer
+// The simulated network and simulation performer
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSNet_h
-#define MSNet_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <typeinfo>
 #include <vector>
@@ -44,44 +35,51 @@
 #include <iostream>
 #include <cmath>
 #include <iomanip>
-#include "MSVehicleControl.h"
-#include "MSEventControl.h"
-#include <utils/geom/Boundary.h>
-#include <utils/geom/Position.h>
+#include <memory>
 #include <utils/common/SUMOTime.h>
-#include <microsim/trigger/MSChargingStation.h>
-#include <microsim/MSStoppingPlace.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/NamedObjectCont.h>
-#include <utils/vehicle/SUMOAbstractRouter.h>
-#include <utils/vehicle/DijkstraRouterTT.h>
-#include <utils/vehicle/DijkstraRouterEffort.h>
-#include <utils/vehicle/AStarRouter.h>
 #include <utils/common/NamedRTree.h>
-#include <utils/vehicle/PedestrianRouter.h>
+#include <utils/router/SUMOAbstractRouter.h>
+#include "MSJunction.h"
+
+#ifdef HAVE_FOX
+#include <utils/foxtools/FXConditionalLock.h>
+#endif
 
 
 // ===========================================================================
 // class declarations
 // ===========================================================================
 class MSEdge;
-class MSJunction;
 class MSEdgeControl;
+class MSEventControl;
+class MSVehicleControl;
 class MSJunctionControl;
 class MSInsertionControl;
 class SUMORouteLoaderControl;
 class MSTransportableControl;
+class MSTransportable;
 class MSVehicle;
 class MSRoute;
 class MSLane;
 class MSTLLogicControl;
+class MSTrafficLightLogic;
 class MSDetectorControl;
 class ShapeContainer;
-class BinaryInputDevice;
+class MSDynamicShapeUpdater;
+class PolygonDynamics;
 class MSEdgeWeightsStorage;
 class SUMOVehicle;
+class SUMOTrafficObject;
+class MSTractionSubstation;
+class MSStoppingPlace;
+template<class E, class L, class N, class V>
+class IntermodalRouter;
+template<class E, class L, class N, class V>
+class PedestrianRouter;
+class OptionsCont;
 
-typedef std::vector<MSEdge*> MSEdgeVector;
 
 // ===========================================================================
 // class definitions
@@ -90,12 +88,14 @@ typedef std::vector<MSEdge*> MSEdgeVector;
  * @class MSNet
  * @brief The simulated network and simulation perfomer
  */
-class MSNet {
+class MSNet : public Parameterised {
 public:
     /** @enum SimulationState
      * @brief Possible states of a simulation - running or stopped with different reasons
      */
     enum SimulationState {
+        /// @brief The simulation is loading
+        SIMSTATE_LOADING,
         /// @brief The simulation is running
         SIMSTATE_RUNNING,
         /// @brief The final simulation step has been performed
@@ -104,16 +104,44 @@ public:
         SIMSTATE_NO_FURTHER_VEHICLES,
         /// @brief The connection to a client was closed by the client
         SIMSTATE_CONNECTION_CLOSED,
-        /// @brief An error occured during the simulation step
+        /// @brief An error occurred during the simulation step
         SIMSTATE_ERROR_IN_SIM,
-        /// @brief The simulation contains too many vehicles (@deprecated)
-        SIMSTATE_TOO_MANY_VEHICLES
+        /// @brief An external interrupt occured
+        SIMSTATE_INTERRUPTED,
+        /// @brief The simulation had too many teleports
+        SIMSTATE_TOO_MANY_TELEPORTS
     };
 
-    //typedef PedestrianRouterDijkstra<MSEdge, MSLane> MSPedestrianRouterDijkstra;
-    typedef PedestrianRouterDijkstra<MSEdge, MSLane, MSJunction, MSVehicle> MSPedestrianRouterDijkstra;
+    typedef PedestrianRouter<MSEdge, MSLane, MSJunction, MSVehicle> MSPedestrianRouter;
+    typedef IntermodalRouter<MSEdge, MSLane, MSJunction, SUMOVehicle> MSIntermodalRouter;
 
+    /// @brief edge type specific meso parameters
+    struct MesoEdgeType {
+        SUMOTime tauff;
+        SUMOTime taufj;
+        SUMOTime taujf;
+        SUMOTime taujj;
+        double jamThreshold;
+        bool junctionControl;
+        double tlsPenalty;
+        double tlsFlowPenalty;
+        SUMOTime minorPenalty;
+        bool overtaking;
+    };
 
+    /// @brief collision tracking
+    struct Collision {
+        std::string victim;
+        std::string colliderType;
+        std::string victimType;
+        double colliderSpeed;
+        double victimSpeed;
+        std::string type;
+        const MSLane* lane;
+        double pos;
+    };
+
+    typedef std::map<std::string, std::vector<Collision> > CollisionMap;
 
 public:
     /** @brief Returns the pointer to the unique instance of MSNet (singleton).
@@ -122,6 +150,27 @@ public:
      */
     static MSNet* getInstance();
 
+    /**
+     * @brief Returns whether this is a GUI Net
+     */
+    virtual bool isGUINet() const {
+        return false;
+    }
+
+    /// @brief Place for static initializations of simulation components (called after successful net build)
+    static void initStatic();
+
+    /// @brief Place for static initializations of simulation components (called after successful net build)
+    static void cleanupStatic();
+
+
+    /** @brief Returns whether the network was already constructed
+    * @return whether the network was already constructed
+    */
+    static bool hasInstance() {
+        return myInstance != nullptr;
+    }
+
 
     /** @brief Constructor
      *
@@ -138,7 +187,8 @@ public:
      * @see closeBuilding
      */
     MSNet(MSVehicleControl* vc, MSEventControl* beginOfTimestepEvents,
-          MSEventControl* endOfTimestepEvents, MSEventControl* insertionEvents,
+          MSEventControl* endOfTimestepEvents,
+          MSEventControl* insertionEvents,
           ShapeContainer* shapeCont = 0);
 
 
@@ -157,15 +207,16 @@ public:
      * @param[in] stateDumpTimes List of time steps at which state shall be written
      * @param[in] stateDumpFiles Filenames for states
      * @param[in] hasInternalLinks Whether the network actually contains internal links
-     * @param[in] lefthand Whether the network was built for left-hand traffic
+     * @param[in] junctionHigherSpeeds Whether the network was built with higher junction speeds
      * @param[in] version The network version
      * @todo Try to move all this to the constructor?
      */
     void closeBuilding(const OptionsCont& oc, MSEdgeControl* edges, MSJunctionControl* junctions,
                        SUMORouteLoaderControl* routeLoaders, MSTLLogicControl* tlc,
                        std::vector<SUMOTime> stateDumpTimes, std::vector<std::string> stateDumpFiles,
-                       bool hasInternalLinks, bool hasNeighs, bool lefthand,
-                       SUMOReal version);
+                       bool hasInternalLinks,
+                       bool junctionHigherSpeeds,
+                       double version);
 
 
     /** @brief Returns whether the network has specific vehicle class permissions
@@ -187,7 +238,7 @@ public:
      * @param[in] svc The vehicle class the restriction refers to
      * @param[in] speed The restricted speed
      */
-    void addRestriction(const std::string& id, const SUMOVehicleClass svc, const SUMOReal speed);
+    void addRestriction(const std::string& id, const SUMOVehicleClass svc, const double speed);
 
 
     /** @brief Returns the restrictions for an edge type
@@ -195,8 +246,18 @@ public:
      * @param[in] id The id of the type
      * @return The mapping of vehicle classes to maximum speeds
      */
-    const std::map<SUMOVehicleClass, SUMOReal>* getRestrictions(const std::string& id) const;
+    const std::map<SUMOVehicleClass, double>* getRestrictions(const std::string& id) const;
 
+    /** @brief Adds edge type specific meso parameters
+     * @param[in] id The id of the type
+     * @param[in] edgeType The parameter object
+     */
+    void addMesoType(const std::string& typeID, const MesoEdgeType& edgeType);
+
+    /** @brief Returns edge type specific meso parameters
+     * if no type specific parameters have been loaded, default values are returned
+     */
+    const MesoEdgeType& getMesoType(const std::string& typeID);
 
     /** @brief Clears all dictionaries
      * @todo Try to move all this to the destructor
@@ -211,7 +272,7 @@ public:
      * @todo Recheck return value
      * @todo What exceptions may occure?
      */
-    int simulate(SUMOTime start, SUMOTime stop);
+    SimulationState simulate(SUMOTime start, SUMOTime stop);
 
 
     /** @brief Performs a single simulation step
@@ -224,17 +285,28 @@ public:
     void loadRoutes();
 
 
+    /** @brief Writes performance output and running vehicle stats
+     *
+     * @param[in] start The step the simulation was started with
+     */
+    const std::string generateStatistics(SUMOTime start);
+
+    /// @brief write collision output to (xml) file
+    void writeCollisions() const;
+
+    /// @brief write statistic output to (xml) file
+    void writeStatistics() const;
+
     /** @brief Closes the simulation (all files, connections, etc.)
      *
      * Writes also performance output
      *
      * @param[in] start The step the simulation was started with
-     * @todo What exceptions may occure?
      */
-    void closeSimulation(SUMOTime start);
+    void closeSimulation(SUMOTime start, const std::string& reason = "");
 
 
-    /** @brief Called after a simulation step, this method returns the current simulation state
+    /** @brief This method returns the current simulation state. It should not modify status.
      * @param[in] stopTime The time the simulation shall stop at
      * @return The current simulation state
      * @see SimulationState
@@ -242,6 +314,14 @@ public:
     SimulationState simulationState(SUMOTime stopTime) const;
 
 
+    /** @brief Called after a simulation step, this method adapts the current simulation state if necessary
+     * @param[in] state The current simulation state
+     * @return The new simulation state
+     * @see SimulationState
+     */
+    SimulationState adaptToState(const SimulationState state) const;
+
+
     /** @brief Returns the message to show if a certain state occurs
      * @return Readable description of the state
      */
@@ -264,6 +344,11 @@ public:
     }
 
 
+    /** @brief Resets events when quick-loading state
+     * @param step The new simulation step
+     */
+    void clearState(const SUMOTime step);
+
     /** @brief Write netstate, summary and detector output
      * @todo Which exceptions may occur?
      */
@@ -322,7 +407,7 @@ public:
     /** @brief Returns whether persons are simulated
      */
     bool hasPersons() const {
-        return myPersonControl != 0;
+        return myPersonControl != nullptr;
     }
 
     /** @brief Returns the container control
@@ -338,7 +423,7 @@ public:
     /** @brief Returns whether containers are simulated
     */
     bool hasContainers() const {
-        return myContainerControl != 0;
+        return myContainerControl != nullptr;
     }
 
 
@@ -431,6 +516,18 @@ public:
         return *myShapeContainer;
     }
 
+    /** @brief Returns the dynamic shapes updater
+     * @see PolygonDynamics
+     */
+    MSDynamicShapeUpdater* getDynamicShapeUpdater() {
+        return myDynamicShapeUpdater.get();
+    }
+
+    /** @brief Creates and returns a dynamic shapes updater
+     * @see PolygonDynamics
+     */
+    MSDynamicShapeUpdater* makeDynamicShapeUpdater();
+
     /** @brief Returns the net's internal edge travel times/efforts container
      *
      * If the net does not have such a container, it is built.
@@ -439,122 +536,102 @@ public:
     MSEdgeWeightsStorage& getWeightsStorage();
     /// @}
 
-    /// @name Insertion and retrieval of bus stops
+    /// @name Insertion and retrieval of stopping places
     /// @{
 
-    /** @brief Adds a bus stop
+    /** @brief Adds a stopping place
      *
-     * If another bus stop with the same id exists, false is returned.
-     *  Otherwise, the bus stop is added to the internal bus stop
-     *  container "myBusStopDict".
+     * If another stop with the same id and category exists, false is returned.
+     *  Otherwise, the stop is added to the internal stopping place container.
      *
-     * This control gets responsible for deletion of the added bus stop.
+     * This control gets responsible for deletion of the added stop.
      *
-     * @param[in] busStop The bus stop to add
-     * @return Whether the bus stop could be added
+     * @param[in] stop The stop to add
+     * @return Whether the stop could be added
      */
-    bool addBusStop(MSStoppingPlace* busStop);
+    bool addStoppingPlace(const SumoXMLTag category, MSStoppingPlace* stop);
 
 
-    /** @brief Returns the named bus stop
-     * @param[in] id The id of the bus stop to return.
-     * @return The named bus stop, or 0 if no such stop exists
+    /** @brief Adds a traction substation
+     *
+     * If another traction substation with the same id and category exists, false is returned.
+     *  Otherwise, the traction substation is added to the internal substations container.
+     *
+     * @param[in] substation The traction substation to add
+     * @return Whether the stop could be added
      */
-    MSStoppingPlace* getBusStop(const std::string& id) const;
+    bool addTractionSubstation(MSTractionSubstation* substation);
 
 
-    /** @brief Returns the bus stop close to the given position
-     * @param[in] lane the lane of the bus stop to return.
-     * @param[in] pos the position of the bus stop to return.
-     * @return The bus stop id on the location, or "" if no such stop exists
+    /** @brief Returns the named stopping place of the given category
+     * @param[in] id The id of the stop to return.
+     * @param[in] category The type of stop
+     * @return The named stop, or 0 if no such stop exists
      */
-    std::string getBusStopID(const MSLane* lane, const SUMOReal pos) const;
+    MSStoppingPlace* getStoppingPlace(const std::string& id, const SumoXMLTag category) const;
+
+    /** @brief Returns the stop of the given category close to the given position
+     * @param[in] lane the lane of the stop to return.
+     * @param[in] pos the position of the stop to return.
+     * @param[in] category The type of stop
+     * @return The stop id on the location, or "" if no such stop exists
+     */
+    std::string getStoppingPlaceID(const MSLane* lane, const double pos, const SumoXMLTag category) const;
     /// @}
 
+    const NamedObjectCont<MSStoppingPlace*>& getStoppingPlaces(SumoXMLTag category) const;
 
-    /// @name Insertion and retrieval of container stops
-    /// @{
+    /// @brief write charging station output
+    void writeChargingStationOutput() const;
 
-    /** @brief Adds a container stop
-     *
-     * If another container stop with the same id exists, false is returned.
-     *  Otherwise, the container stop is added to the internal container stop
-     *  container "myContainerStopDict".
-     *
-     * This control gets responsible for deletion of the added container stop.
-     *
-     * @param[in] containerStop The container stop to add
-     * @return Whether the container stop could be added
-     */
-    bool addContainerStop(MSStoppingPlace* containerStop);
+    /// @brief write rail signal block output
+    void writeRailSignalBlocks() const;
 
-    /** @brief Returns the named container stop
-     * @param[in] id The id of the container stop to return.
-     * @return The named container stop, or 0 if no such stop exists
-     */
-    MSStoppingPlace* getContainerStop(const std::string& id) const;
+    /// @brief creates a wrapper for the given logic (see GUINet)
+    virtual void createTLWrapper(MSTrafficLightLogic*) {};
 
-    /** @brief Returns the container stop close to the given position
-     * @param[in] lane the lane of the container stop to return.
-     * @param[in] pos the position of the container stop to return.
-     * @return The container stop id on the location, or "" if no such stop exists
-     */
-    std::string getContainerStopID(const MSLane* lane, const SUMOReal pos) const;
-    /// @}
+    /// @brief write the output generated by an overhead wire segment
+    void writeOverheadWireSegmentOutput() const;
 
-    /** @brief Adds a chargingg station
-     *
-     * If another charging station with the same id exists, false is returned.
-     *  Otherwise, the charging station is added to the internal bus stop
-     *  container "myChargingStationDict".
-     *
-     * This control gets responsible for deletion of the added charging station.
-     *
-     * @param[in] chargingStation The charging station add
-     * @return Whether the charging station could be added
-     */
-    bool addChargingStation(MSChargingStation* chargingStation);
-
-    /** @brief Returns the named charging station
-     * @param[in] id The id of the charging station to return.
-     * @return The named charging station, or 0 if no such stop exists
-     */
-    MSChargingStation* getChargingStation(const std::string& id) const;
-
-    /** @brief Returns the charging station close to the given position
-     * @param[in] lane the lane of the charging station to return.
-     * @param[in] pos the position of the bus stop to return.
-     * @return The charging station id on the location, or "" if no such stop exists
-     */
-    std::string getChargingStationID(const MSLane* lane, const SUMOReal pos) const;
-    /// @}
+    /// @brief write electrical substation output
+    void writeSubstationOutput() const;
 
+    /// @brief return wheter the given logic (or rather it's wrapper) is selected in the GUI
+    virtual bool isSelected(const MSTrafficLightLogic*) const {
+        return false;
+    }
 
     /// @name Notification about vehicle state changes
     /// @{
 
     /// @brief Definition of a vehicle state
-    enum VehicleState {
+    enum class VehicleState {
         /// @brief The vehicle was built, but has not yet departed
-        VEHICLE_STATE_BUILT,
+        BUILT,
         /// @brief The vehicle has departed (was inserted into the network)
-        VEHICLE_STATE_DEPARTED,
+        DEPARTED,
         /// @brief The vehicle started to teleport
-        VEHICLE_STATE_STARTING_TELEPORT,
+        STARTING_TELEPORT,
         /// @brief The vehicle ended being teleported
-        VEHICLE_STATE_ENDING_TELEPORT,
+        ENDING_TELEPORT,
         /// @brief The vehicle arrived at his destination (is deleted)
-        VEHICLE_STATE_ARRIVED,
+        ARRIVED,
         /// @brief The vehicle got a new route
-        VEHICLE_STATE_NEWROUTE,
+        NEWROUTE,
         /// @brief The vehicles starts to park
-        VEHICLE_STATE_STARTING_PARKING,
+        STARTING_PARKING,
         /// @brief The vehicle ends to park
-        VEHICLE_STATE_ENDING_PARKING,
+        ENDING_PARKING,
         /// @brief The vehicles starts to stop
-        VEHICLE_STATE_STARTING_STOP,
+        STARTING_STOP,
         /// @brief The vehicle ends to stop
-        VEHICLE_STATE_ENDING_STOP
+        ENDING_STOP,
+        /// @brief The vehicle is involved in a collision
+        COLLISION,
+        /// @brief The vehicle had to brake harder than permitted
+        EMERGENCYSTOP,
+        /// @brief Vehicle maneuvering either entering or exiting a parking space
+        MANEUVERING
     };
 
 
@@ -572,8 +649,9 @@ public:
         /** @brief Called if a vehicle changes its state
          * @param[in] vehicle The vehicle which changed its state
          * @param[in] to The state the vehicle has changed to
+         * @param[in] info Additional information on the state change
          */
-        virtual void vehicleStateChanged(const SUMOVehicle* const vehicle, VehicleState to) = 0;
+        virtual void vehicleStateChanged(const SUMOVehicle* const vehicle, VehicleState to, const std::string& info = "") = 0;
 
     };
 
@@ -593,21 +671,88 @@ public:
     /** @brief Informs all added listeners about a vehicle's state change
      * @param[in] vehicle The vehicle which changed its state
      * @param[in] to The state the vehicle has changed to
+     * @param[in] info Information regarding the replacement
      * @see VehicleStateListener:vehicleStateChanged
      */
-    void informVehicleStateListener(const SUMOVehicle* const vehicle, VehicleState to);
+    void informVehicleStateListener(const SUMOVehicle* const vehicle, VehicleState to, const std::string& info = "");
     /// @}
 
 
+    /// @name Notification about transportable state changes
+    /// @{
+
+    /// @brief Definition of a transportable state
+    enum class TransportableState {
+        /// @brief The transportable person has departed (was inserted into the network)
+        PERSON_DEPARTED,
+        /// @brief The transportable person arrived at his destination (is deleted)
+        PERSON_ARRIVED,
+        /// @brief The transportable container has departed (was inserted into the network)
+        CONTAINER_DEPARTED,
+        /// @brief The transportable container arrived at his destination (is deleted)
+        CONTAINER_ARRIVED
+    };
+
+
+    /** @class TransportableStateListener
+     * @brief Interface for objects listening to transportable state changes
+     */
+    class TransportableStateListener {
+    public:
+        /// @brief Constructor
+        TransportableStateListener() { }
+
+        /// @brief Destructor
+        virtual ~TransportableStateListener() { }
+
+        /** @brief Called if a transportable changes its state
+         * @param[in] transportable The transportable which changed its state
+         * @param[in] to The state the transportable has changed to
+         * @param[in] info Additional information on the state change
+         */
+        virtual void transportableStateChanged(const MSTransportable* const transportable, TransportableState to, const std::string& info = "") = 0;
+
+    };
+
+
+    /** @brief Adds a transportable states listener
+     * @param[in] listener The listener to add
+     */
+    void addTransportableStateListener(TransportableStateListener* listener);
+
+
+    /** @brief Removes a transportable states listener
+     * @param[in] listener The listener to remove
+     */
+    void removeTransportableStateListener(TransportableStateListener* listener);
+
+
+    /** @brief Informs all added listeners about a transportable's state change
+     * @param[in] transportable The transportable which changed its state
+     * @param[in] to The state the transportable has changed to
+     * @param[in] info Information regarding the replacement
+     * @see TransportableStateListener:TransportableStateChanged
+     */
+    void informTransportableStateListener(const MSTransportable* const transportable, TransportableState to, const std::string& info = "");
+    /// @}
+
+
+    /// @brief register collision and return whether it was the first one involving these vehicles
+    bool registerCollision(const SUMOTrafficObject* collider, const SUMOTrafficObject* victim, const std::string& collisionType, const MSLane* lane, double pos);
+
+    const CollisionMap& getCollisions() const {
+        return myCollisions;
+    }
+
 
     /** @brief Returns the travel time to pass an edge
      * @param[in] e The edge for which the travel time to be passed shall be returned
      * @param[in] v The vehicle that is rerouted
      * @param[in] t The time for which the travel time shall be returned [s]
      * @return The travel time for an edge
-     * @see DijkstraRouterTT_ByProxi
+     * @see DijkstraRouter_ByProxi
      */
-    static SUMOReal getTravelTime(const MSEdge* const e, const SUMOVehicle* const v, SUMOReal t);
+    static double getTravelTime(const MSEdge* const e, const SUMOVehicle* const v, double t);
 
 
     /** @brief Returns the effort to pass an edge
@@ -615,34 +760,32 @@ public:
      * @param[in] v The vehicle that is rerouted
      * @param[in] t The time for which the effort shall be returned [s]
      * @return The effort (abstract) for an edge
-     * @see DijkstraRouterTT_ByProxi
+     * @see DijkstraRouter_ByProxi
      */
-    static SUMOReal getEffort(const MSEdge* const e, const SUMOVehicle* const v, SUMOReal t);
+    static double getEffort(const MSEdge* const e, const SUMOVehicle* const v, double t);
 
 
     /* @brief get the router, initialize on first use
      * @param[in] prohibited The vector of forbidden edges (optional)
      */
-    SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouterTT(
-        const MSEdgeVector& prohibited = MSEdgeVector()) const;
-    SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouterEffort(
-        const MSEdgeVector& prohibited = MSEdgeVector()) const;
-    MSPedestrianRouterDijkstra& getPedestrianRouter(const MSEdgeVector& prohibited = MSEdgeVector()) const;
+    SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouterTT(const int rngIndex,
+            const MSEdgeVector& prohibited = MSEdgeVector()) const;
+    SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouterEffort(const int rngIndex,
+            const MSEdgeVector& prohibited = MSEdgeVector()) const;
+    MSPedestrianRouter& getPedestrianRouter(const int rngIndex, const MSEdgeVector& prohibited = MSEdgeVector()) const;
+    MSIntermodalRouter& getIntermodalRouter(const int rngIndex, const int routingMode = 0, const MSEdgeVector& prohibited = MSEdgeVector()) const;
 
+    static void adaptIntermodalRouter(MSIntermodalRouter& router);
 
-    /** @brief Returns an RTree that contains lane IDs
-     * @return An Rtree containing lane IDs
-     */
-    const NamedRTree& getLanesRTree() const;
 
     /// @brief return whether the network contains internal links
     bool hasInternalLinks() const {
         return myHasInternalLinks;
     }
 
-    /// @brief return whether the network contains explicit neighbor lanes
-    bool hasNeighs() const {
-        return myHasNeighs;
+    /// @brief return whether the network was built with higher junction speeds
+    bool hasJunctionHigherSpeeds() const {
+        return myJunctionHigherSpeeds;
     }
 
     /// @brief return whether the network contains elevation data
@@ -650,20 +793,48 @@ public:
         return myHasElevation;
     }
 
-    /// @brief return whether the network was built for lefthand traffic
-    bool lefthand() const {
-        return myLefthand;
+    /// @brief return whether the network contains walkingareas and crossings
+    bool hasPedestrianNetwork() const {
+        return myHasPedestrianNetwork;
+
+    }
+    /// @brief return whether the network contains bidirectional rail edges
+    bool hasBidiEdges() const {
+        return myHasBidiEdges;
     }
 
     /// @brief return the network version
-    SUMOReal version() const {
+    double getNetworkVersion() const {
         return myVersion;
     }
 
+    /// @brief return whether a warning regarding the given object shall be issued
+    bool warnOnce(const std::string& typeAndID);
+
+    void interrupt() {
+        myAmInterrupted = true;
+    }
+
+    bool isInterrupted() const {
+        return myAmInterrupted;
+    }
+
+    /// @brief find electrical substation by its id
+    MSTractionSubstation* findTractionSubstation(const std::string& substationId);
+
+    /// @brief return whether given electrical substation exists in the network
+    bool existTractionSubstation(const std::string& substationId);
+
 protected:
     /// @brief check all lanes for elevation data
     bool checkElevation();
 
+    /// @brief check all lanes for type walkingArea
+    bool checkWalkingarea();
+
+    /// @brief check wether bidirectional edges occur in the network
+    bool checkBidiEdges();
+
 protected:
     /// @brief Unique instance of MSNet
     static MSNet* myInstance;
@@ -674,6 +845,12 @@ protected:
     /// @brief Current time step.
     SUMOTime myStep;
 
+    /// @brief Maximum number of teleports.
+    int myMaxTeleports;
+
+    /// @brief whether an interrupt occured
+    bool myAmInterrupted;
+
 
 
     /// @name Substructures
@@ -717,15 +894,21 @@ protected:
 
     /// @brief Information whether the number of the simulation step shall be logged
     bool myLogStepNumber;
+    /// @brief Period between successive step-log outputs
+    int myLogStepPeriod;
 
-    /// @brief The last simulation step begin, end and duration
-    long mySimStepBegin, mySimStepEnd, mySimStepDuration;
+    /// @brief The last simulation step duration
+    long myTraCIStepDuration = 0, mySimStepDuration = 0;
 
     /// @brief The overall simulation duration
     long mySimBeginMillis;
 
+    /// @brief The overall time spent waiting for traci operations including
+    long myTraCIMillis;
+
     /// @brief The overall number of vehicle movements
     long long int myVehiclesMoved;
+    long long int myPersonsMoved;
     //}
 
 
@@ -750,50 +933,79 @@ protected:
     bool myHavePermissions;
 
     /// @brief The vehicle class specific speed restrictions
-    std::map<std::string, std::map<SUMOVehicleClass, SUMOReal> > myRestrictions;
+    std::map<std::string, std::map<SUMOVehicleClass, double> > myRestrictions;
+
+    /// @brief The edge type specific meso parameters
+    std::map<std::string, MesoEdgeType> myMesoEdgeTypes;
 
     /// @brief Whether the network contains internal links/lanes/edges
     bool myHasInternalLinks;
 
-    /// @brief Whether the network contains explicit neighbor lanes
-    bool myHasNeighs;
+    /// @brief Whether the network was built with higher speed on junctions
+    bool myJunctionHigherSpeeds;
 
     /// @brief Whether the network contains elevation data
     bool myHasElevation;
 
+    /// @brief Whether the network contains pedestrian network elements
+    bool myHasPedestrianNetwork;
+
+    /// @brief Whether the network contains bidirectional rail edges
+    bool myHasBidiEdges;
+
     /// @brief Whether the network was built for left-hand traffic
     bool myLefthand;
 
     /// @brief the network version
-    SUMOReal myVersion;
+    double myVersion;
 
-    /// @brief Dictionary of bus stops
-    NamedObjectCont<MSStoppingPlace*> myBusStopDict;
+    /// @brief end of loaded edgeData
+    SUMOTime myEdgeDataEndTime;
 
-    /// @brief Dictionary of container stops
-    NamedObjectCont<MSStoppingPlace*> myContainerStopDict;
+    /// @brief Dictionary of bus / container stops
+    std::map<SumoXMLTag, NamedObjectCont<MSStoppingPlace*> > myStoppingPlaces;
 
-    /// @brief Dictionary of charging Stations
-    NamedObjectCont<MSChargingStation*> myChargingStationDict;
+    /// @brief Dictionary of traction substations
+    std::vector<MSTractionSubstation*> myTractionSubstations;
 
     /// @brief Container for vehicle state listener
     std::vector<VehicleStateListener*> myVehicleStateListeners;
 
+    /// @brief Container for transportable state listener
+    std::vector<TransportableStateListener*> myTransportableStateListeners;
+
+    /// @brief collisions in the current time step
+    CollisionMap myCollisions;
+
+#ifdef HAVE_FOX
+    /// @brief to avoid concurrent access to the state update function
+    FXMutex myVehicleStateListenerMutex;
+
+    /// @brief to avoid concurrent access to the state update function
+    FXMutex myTransportableStateListenerMutex;
+#endif
+
+    /// @brief container to record warnings that shall only be issued once
+    std::map<std::string, bool> myWarnedOnce;
 
     /* @brief The router instance for routing by trigger and by traci
      * @note MSDevice_Routing has its own instance since it uses a different weight function
      * @note we provide one member for every switchable router type
-     * because the class structure makes it inconvenient to use a superclass*/
-    mutable bool myRouterTTInitialized;
-    mutable DijkstraRouterTT<MSEdge, SUMOVehicle, prohibited_withPermissions<MSEdge, SUMOVehicle> >* myRouterTTDijkstra;
-    mutable AStarRouter<MSEdge, SUMOVehicle, prohibited_withPermissions<MSEdge, SUMOVehicle> >* myRouterTTAStar;
-    mutable DijkstraRouterEffort<MSEdge, SUMOVehicle, prohibited_withPermissions<MSEdge, SUMOVehicle> >* myRouterEffort;
-    mutable MSPedestrianRouterDijkstra* myPedestrianRouter;
-
+     * because the class structure makes it inconvenient to use a superclass
+     */
+    mutable std::map<int, SUMOAbstractRouter<MSEdge, SUMOVehicle>*> myRouterTT;
+    mutable std::map<int, SUMOAbstractRouter<MSEdge, SUMOVehicle>*> myRouterEffort;
+    mutable std::map<int, MSPedestrianRouter*> myPedestrianRouter;
+    mutable std::map<int, MSIntermodalRouter*> myIntermodalRouter;
 
     /// @brief An RTree structure holding lane IDs
     mutable std::pair<bool, NamedRTree> myLanesRTree;
 
+    /// @brief Updater for dynamic shapes that are tracking traffic objects
+    ///        (ensures removal of shape dynamics when the objects are removed)
+    /// @see utils/shapes/PolygonDynamics
+    std::unique_ptr<MSDynamicShapeUpdater> myDynamicShapeUpdater;
+
 
     /// @brief string constants for simstep stages
     static const std::string STAGE_EVENTS;
@@ -810,9 +1022,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
