@@ -1,45 +1,37 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSRightOfWayJunction.cpp
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Wed, 12 Dez 2001
-/// @version $Id: MSRightOfWayJunction.cpp 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // junction.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
-#include "MSRightOfWayJunction.h"
-#include "MSLane.h"
-#include "MSEdge.h"
-#include "MSJunctionLogic.h"
-#include "MSGlobals.h"
 #include <algorithm>
 #include <cassert>
 #include <cmath>
 #include <utils/common/RandHelper.h>
+#include "MSEdge.h"
+#include "MSJunctionLogic.h"
+#include "MSGlobals.h"
+#include "MSLane.h"
+#include "MSLink.h"
+#include "MSRightOfWayJunction.h"
 
 
 // ===========================================================================
@@ -49,9 +41,10 @@ MSRightOfWayJunction::MSRightOfWayJunction(const std::string& id,
         SumoXMLNodeType type,
         const Position& position,
         const PositionVector& shape,
+        const std::string& name,
         std::vector<MSLane*> incoming,
         std::vector<MSLane*> internal,
-        MSJunctionLogic* logic) : MSLogicJunction(id, type, position, shape, incoming, internal),
+        MSJunctionLogic* logic) : MSLogicJunction(id, type, position, shape, name, incoming, internal),
     myLogic(logic) {}
 
 
@@ -64,30 +57,33 @@ void
 MSRightOfWayJunction::postloadInit() {
     // inform links where they have to report approaching vehicles to
     int requestPos = 0;
-    std::vector<MSLane*>::iterator i;
     // going through the incoming lanes...
     int maxNo = 0;
-    std::vector<std::pair<MSLane*, MSLink*> > sortedLinks;
-    for (i = myIncomingLanes.begin(); i != myIncomingLanes.end(); ++i) {
-        const MSLinkCont& links = (*i)->getLinkCont();
+    std::vector<std::pair<const MSLane*, MSLink*> > sortedLinks;
+    for (MSLane* const lane : myIncomingLanes) {
         // ... set information for every link
-        for (MSLinkCont::const_iterator j = links.begin(); j != links.end(); j++) {
-            if ((*j)->getLane()->getEdge().isWalkingArea() ||
-                    ((*i)->getEdge().isWalkingArea() && !(*j)->getLane()->getEdge().isCrossing())) {
+        for (MSLink* const link : lane->getLinkCont()) {
+            if (link->getLane()->getEdge().isWalkingArea() ||
+                    (lane->getEdge().isWalkingArea() && !link->getLane()->getEdge().isCrossing())) {
                 continue;
             }
-            sortedLinks.push_back(std::make_pair(*i, *j));
+            sortedLinks.emplace_back(lane, link);
             ++maxNo;
         }
     }
 
     const bool hasFoes = myLogic->hasFoes();
-    for (i = myIncomingLanes.begin(); i != myIncomingLanes.end(); ++i) {
-        const MSLinkCont& links = (*i)->getLinkCont();
+    for (const MSLane* const lane : myIncomingLanes) {
         // ... set information for every link
-        for (MSLinkCont::const_iterator j = links.begin(); j != links.end(); j++) {
-            if ((*j)->getLane()->getEdge().isWalkingArea() ||
-                    ((*i)->getEdge().isWalkingArea() && !(*j)->getLane()->getEdge().isCrossing())) {
+        const MSLane* walkingAreaFoe = nullptr;
+        for (MSLink* const link : lane->getLinkCont()) {
+            if (link->getLane()->getEdge().isWalkingArea()) {
+                if (lane->getPermissions() != SVC_PEDESTRIAN) {
+                    // vehicular lane connects to a walkingarea
+                    walkingAreaFoe = link->getLane();
+                }
+                continue;
+            } else if ((lane->getEdge().isWalkingArea() && !link->getLane()->getEdge().isCrossing())) {
                 continue;
             }
             if (myLogic->getLogicSize() <= requestPos) {
@@ -96,17 +92,16 @@ MSRightOfWayJunction::postloadInit() {
             const MSLogicJunction::LinkBits& linkResponse = myLogic->getResponseFor(requestPos); // SUMO_ATTR_RESPONSE
             const MSLogicJunction::LinkBits& linkFoes = myLogic->getFoesFor(requestPos); // SUMO_ATTR_FOES
             bool cont = myLogic->getIsCont(requestPos);
-            myLinkFoeLinks[*j] = std::vector<MSLink*>();
             for (int c = 0; c < maxNo; ++c) {
                 if (linkResponse.test(c)) {
                     MSLink* foe = sortedLinks[c].second;
-                    myLinkFoeLinks[*j].push_back(foe);
-                    if (MSGlobals::gUsingInternalLanes && foe->getViaLane() != 0) {
+                    myLinkFoeLinks[link].push_back(foe);
+                    if (MSGlobals::gUsingInternalLanes && foe->getViaLane() != nullptr) {
                         assert(foe->getViaLane()->getLinkCont().size() == 1);
                         MSLink* foeExitLink = foe->getViaLane()->getLinkCont()[0];
                         // add foe links after an internal junction
-                        if (foeExitLink->getViaLane() != 0) {
-                            myLinkFoeLinks[*j].push_back(foeExitLink);
+                        if (foeExitLink->getViaLane() != nullptr) {
+                            myLinkFoeLinks[link].push_back(foeExitLink);
                         }
                     }
                 }
@@ -117,56 +112,73 @@ MSRightOfWayJunction::postloadInit() {
                     MSLink* foe = sortedLinks[c].second;
                     foes.push_back(foe);
                     MSLane* l = foe->getViaLane();
-                    if (l == 0) {
+                    if (l == nullptr) {
                         continue;
                     }
                     // add foe links after an internal junction
-                    const MSLinkCont& lc = l->getLinkCont();
-                    for (MSLinkCont::const_iterator q = lc.begin(); q != lc.end(); ++q) {
-                        if ((*q)->getViaLane() != 0) {
-                            foes.push_back(*q);
+                    for (MSLink* const vLink : l->getLinkCont()) {
+                        if (vLink->getViaLane() != nullptr) {
+                            foes.push_back(vLink);
                         }
                     }
                 }
             }
 
-            myLinkFoeInternalLanes[*j] = std::vector<MSLane*>();
             if (MSGlobals::gUsingInternalLanes && myInternalLanes.size() > 0) {
                 int li = 0;
                 for (int c = 0; c < (int)sortedLinks.size(); ++c) {
-                    if (sortedLinks[c].second->getLane() == 0) { // dead end
+                    if (sortedLinks[c].second->getLane() == nullptr) { // dead end
                         continue;
                     }
                     if (linkFoes.test(c)) {
-                        myLinkFoeInternalLanes[*j].push_back(myInternalLanes[li]);
+                        myLinkFoeInternalLanes[link].push_back(myInternalLanes[li]);
                         if (linkResponse.test(c)) {
                             const std::vector<MSLane::IncomingLaneInfo>& l = myInternalLanes[li]->getIncomingLanes();
-                            if (l.size() == 1 && l[0].lane->getEdge().getPurpose() == MSEdge::EDGEFUNCTION_INTERNAL) {
-                                myLinkFoeInternalLanes[*j].push_back(l[0].lane);
+                            if (l.size() == 1 && l[0].lane->getEdge().isInternal()) {
+                                myLinkFoeInternalLanes[link].push_back(l[0].lane);
                             }
                         }
                     }
                     ++li;
                 }
             }
-            (*j)->setRequestInformation((int)requestPos, hasFoes, cont, myLinkFoeLinks[*j], myLinkFoeInternalLanes[*j]);
+            link->setRequestInformation((int)requestPos, hasFoes, cont, myLinkFoeLinks[link], myLinkFoeInternalLanes[link]);
             // the exit link for a link before an internal junction is handled in MSInternalJunction
             // so we need to skip if cont=true
-            if (MSGlobals::gUsingInternalLanes && (*j)->getViaLane() != 0 && !cont) {
-                assert((*j)->getViaLane()->getLinkCont().size() == 1);
-                MSLink* exitLink = (*j)->getViaLane()->getLinkCont()[0];
+            if (MSGlobals::gUsingInternalLanes && link->getViaLane() != nullptr && !cont) {
+                assert(link->getViaLane()->getLinkCont().size() == 1);
+                MSLink* exitLink = link->getViaLane()->getLinkCont()[0];
                 exitLink->setRequestInformation((int)requestPos, false, false, std::vector<MSLink*>(),
-                                                myLinkFoeInternalLanes[*j], (*j)->getViaLane());
+                                                myLinkFoeInternalLanes[link], link->getViaLane());
+                for (const auto& ili : exitLink->getLane()->getIncomingLanes()) {
+                    if (ili.lane->getEdge().isWalkingArea()) {
+                        exitLink->addWalkingAreaFoeExit(ili.lane);
+                        break;
+                    }
+                }
             }
-            for (std::vector<MSLink*>::const_iterator k = foes.begin(); k != foes.end(); ++k) {
-                (*j)->addBlockedLink(*k);
-                (*k)->addBlockedLink(*j);
+            // the exit link for a crossing is needed for the pedestrian model
+            if (MSGlobals::gUsingInternalLanes && link->getLane()->getEdge().isCrossing()) {
+                MSLink* exitLink = link->getLane()->getLinkCont()[0];
+                exitLink->setRequestInformation((int)requestPos, false, false, std::vector<MSLink*>(),
+                                                myLinkFoeInternalLanes[link], link->getLane());
+            }
+            for (MSLink* const foe : foes) {
+                link->addBlockedLink(foe);
+                foe->addBlockedLink(link);
             }
             requestPos++;
         }
+        if (walkingAreaFoe != nullptr && lane->getLinkCont().size() > 1) {
+            for (const MSLink* const link : lane->getLinkCont()) {
+                if (!link->getLane()->getEdge().isWalkingArea()) {
+                    MSLink* exitLink = link->getViaLane()->getLinkCont()[0];
+                    exitLink->addWalkingAreaFoe(walkingAreaFoe);
+                }
+            }
+        }
     }
 }
 
 
 /****************************************************************************/
-
