@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSRightOfWayJunction.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Wed, 12 Dez 2001
-/// @version $Id: MSRightOfWayJunction.h 23509 2017-03-17 12:27:35Z luecken $
 ///
 // A junction with right-of-way - rules
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSRightOfWayJunction_h
-#define MSRightOfWayJunction_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include "MSLogicJunction.h"
 #include <bitset>
@@ -68,6 +58,7 @@ public:
      */
     MSRightOfWayJunction(const std::string& id, SumoXMLNodeType type, const Position& position,
                          const PositionVector& shape,
+                         const std::string& name,
                          std::vector<MSLane*> incoming,
                          std::vector<MSLane*> internal,
                          MSJunctionLogic* logic);
@@ -107,9 +98,3 @@ private:
     MSRightOfWayJunction& operator=(const MSRightOfWayJunction&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
