@@ -1,51 +1,37 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSRoute.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Friedemann Wesner
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: MSRoute.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A vehicle route
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cassert>
 #include <algorithm>
 #include <limits>
 #include <utils/common/FileHelpers.h>
 #include <utils/common/RGBColor.h>
-#include <utils/iodevices/BinaryInputDevice.h>
 #include <utils/iodevices/OutputDevice.h>
 #include "MSEdge.h"
 #include "MSLane.h"
 #include "MSRoute.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static member variables
@@ -63,10 +49,15 @@ FXMutex MSRoute::myDictMutex(true);
 MSRoute::MSRoute(const std::string& id,
                  const ConstMSEdgeVector& edges,
                  const bool isPermanent, const RGBColor* const c,
-                 const std::vector<SUMOVehicleParameter::Stop>& stops)
-    : Named(id), myEdges(edges), myAmPermanent(isPermanent),
-      myReferenceCounter(isPermanent ? 1 : 0),
-      myColor(c), myStops(stops) {}
+                 const std::vector<SUMOVehicleParameter::Stop>& stops) :
+    Named(id), myEdges(edges), myAmPermanent(isPermanent),
+    myReferenceCounter(isPermanent ? 1 : 0),
+    myColor(c),
+    myPeriod(0),
+    myCosts(-1),
+    mySavings(0),
+    myReroute(false),
+    myStops(stops) {}
 
 
 MSRoute::~MSRoute() {
@@ -86,9 +77,9 @@ MSRoute::end() const {
 }
 
 
-unsigned
+int
 MSRoute::size() const {
-    return (unsigned) myEdges.size();
+    return (int)myEdges.size();
 }
 
 
@@ -145,7 +136,7 @@ MSRoute::dictionary(const std::string& id, RandomDistributor<const MSRoute*>* co
 
 
 const MSRoute*
-MSRoute::dictionary(const std::string& id, MTRand* rng) {
+MSRoute::dictionary(const std::string& id, std::mt19937* rng) {
 #ifdef HAVE_FOX
     FXMutexLock f(myDictMutex);
 #endif
@@ -153,7 +144,7 @@ MSRoute::dictionary(const std::string& id, MTRand* rng) {
     if (it == myDict.end()) {
         RouteDistDict::iterator it2 = myDistDict.find(id);
         if (it2 == myDistDict.end() || it2->second.first->getOverallProb() == 0) {
-            return 0;
+            return nullptr;
         }
         return it2->second.first->get(rng);
     }
@@ -161,6 +152,15 @@ MSRoute::dictionary(const std::string& id, MTRand* rng) {
 }
 
 
+bool
+MSRoute::hasRoute(const std::string& id) {
+#ifdef HAVE_FOX
+    FXMutexLock f(myDictMutex);
+#endif
+    return myDict.find(id) != myDict.end();
+}
+
+
 RandomDistributor<const MSRoute*>*
 MSRoute::distDictionary(const std::string& id) {
 #ifdef HAVE_FOX
@@ -168,7 +168,7 @@ MSRoute::distDictionary(const std::string& id) {
 #endif
     RouteDistDict::iterator it2 = myDistDict.find(id);
     if (it2 == myDistDict.end()) {
-        return 0;
+        return nullptr;
     }
     return it2->second.first;
 }
@@ -226,7 +226,7 @@ int
 MSRoute::writeEdgeIDs(OutputDevice& os, const MSEdge* const from, const MSEdge* const upTo) const {
     int numWritten = 0;
     ConstMSEdgeVector::const_iterator i = myEdges.begin();
-    if (from != 0) {
+    if (from != nullptr) {
         i = std::find(myEdges.begin(), myEdges.end(), from);
     }
     for (; i != myEdges.end(); ++i) {
@@ -256,7 +256,7 @@ MSRoute::containsAnyOf(const MSEdgeVector& edgelist) const {
 
 
 const MSEdge*
-MSRoute::operator[](unsigned index) const {
+MSRoute::operator[](int index) const {
     return myEdges[index];
 }
 
@@ -271,23 +271,70 @@ MSRoute::dict_saveState(OutputDevice& out) {
         out.writeAttr(SUMO_ATTR_STATE, (*it).second->myAmPermanent);
         out.writeAttr(SUMO_ATTR_EDGES, (*it).second->myEdges).closeTag();
     }
-    for (RouteDistDict::iterator it = myDistDict.begin(); it != myDistDict.end(); ++it) {
-        out.openTag(SUMO_TAG_ROUTE_DISTRIBUTION).writeAttr(SUMO_ATTR_ID, (*it).first);
-        out.writeAttr(SUMO_ATTR_STATE, (*it).second.second);
-        out.writeAttr(SUMO_ATTR_ROUTES, (*it).second.first->getVals());
-        out.writeAttr(SUMO_ATTR_PROBS, (*it).second.first->getProbs());
-        out.closeTag();
+    for (const auto& item : myDistDict) {
+        if (item.second.first->getVals().size() > 0) {
+            out.openTag(SUMO_TAG_ROUTE_DISTRIBUTION).writeAttr(SUMO_ATTR_ID, item.first);
+            out.writeAttr(SUMO_ATTR_STATE, item.second.second);
+            out.writeAttr(SUMO_ATTR_ROUTES, item.second.first->getVals());
+            out.writeAttr(SUMO_ATTR_PROBS, item.second.first->getProbs());
+            out.closeTag();
+        }
     }
 }
 
+void
+MSRoute::dict_clearState() {
+#ifdef HAVE_FOX
+    FXMutexLock f(myDictMutex);
+#endif
+    for (auto item : myDict) {
+        delete item.second;
+    }
+    myDistDict.clear();
+    myDict.clear();
+}
 
-SUMOReal
-MSRoute::getDistanceBetween(SUMOReal fromPos, SUMOReal toPos,
-                            const MSEdge* fromEdge, const MSEdge* toEdge, bool includeInternal) const {
-    ConstMSEdgeVector::const_iterator it = std::find(myEdges.begin(), myEdges.end(), fromEdge);
+
+double
+MSRoute::getDistanceBetween(double fromPos, double toPos,
+                            const MSEdge* fromEdge, const MSEdge* toEdge, bool includeInternal, int routePosition) const {
+    //std::cout << SIMTIME << " getDistanceBetween from=" << fromEdge->getID() << " to=" << toEdge->getID() << " fromPos=" << fromPos << " toPos=" << toPos << " includeInternal=" << includeInternal << "\n";
+    if (routePosition < 0 || routePosition >= (int)myEdges.size()) {
+        throw ProcessError("Invalid routePosition " + toString(routePosition) + " for route with " + toString(myEdges.size()) + " edges");
+    }
+    if (fromEdge->isInternal() && toEdge->isInternal() && fromEdge->getToJunction() == toEdge->getToJunction()) {
+        // internal edges within the same junction
+        if (fromEdge == toEdge) {
+            if (fromPos <= toPos) {
+                return toPos - fromPos;
+            }
+        } else if (fromEdge->getSuccessors().front() == toEdge) {
+            return fromEdge->getLength() - fromPos + toPos;
+        }
+    }
+    if (fromEdge->isInternal()) {
+        if (fromEdge == myEdges.front()) {
+            const MSEdge* succ = fromEdge->getSuccessors().front();
+            assert(succ != 0);
+            //std::cout << "  recurse fromSucc=" << succ->getID() << "\n";
+            return (fromEdge->getLength() - fromPos) + getDistanceBetween(0, toPos, succ, toEdge, includeInternal);
+        } else {
+            const MSEdge* pred = fromEdge->getPredecessors().front();
+            assert(pred != 0);
+            //std::cout << "  recurse fromPred=" << pred->getID() << "\n";
+            return getDistanceBetween(pred->getLength(), toPos, pred, toEdge, includeInternal, routePosition) - fromPos;
+        }
+    }
+    if (toEdge->isInternal()) {
+        const MSEdge* pred = toEdge->getPredecessors().front();
+        assert(pred != 0);
+        //std::cout << "  recurse toPred=" << pred->getID() << "\n";
+        return toPos + getDistanceBetween(fromPos, pred->getLength(), fromEdge, pred, includeInternal, routePosition);
+    }
+    ConstMSEdgeVector::const_iterator it = std::find(myEdges.begin() + routePosition, myEdges.end(), fromEdge);
     if (it == myEdges.end() || std::find(it, myEdges.end(), toEdge) == myEdges.end()) {
         // start or destination not contained in route
-        return std::numeric_limits<SUMOReal>::max();
+        return std::numeric_limits<double>::max();
     }
     ConstMSEdgeVector::const_iterator it2 = std::find(it + 1, myEdges.end(), toEdge);
 
@@ -296,18 +343,18 @@ MSRoute::getDistanceBetween(SUMOReal fromPos, SUMOReal toPos,
             return toPos - fromPos;
         } else if (it2 == myEdges.end()) {
             // we don't visit the edge again
-            return std::numeric_limits<SUMOReal>::max();
+            return std::numeric_limits<double>::max();
         }
     }
     return getDistanceBetween(fromPos, toPos, it, it2, includeInternal);
 }
 
 
-SUMOReal
-MSRoute::getDistanceBetween(SUMOReal fromPos, SUMOReal toPos,
+double
+MSRoute::getDistanceBetween(double fromPos, double toPos,
                             const MSRouteIterator& fromEdge, const MSRouteIterator& toEdge, bool includeInternal) const {
     bool isFirstIteration = true;
-    SUMOReal distance = -fromPos;
+    double distance = -fromPos;
     MSRouteIterator it = fromEdge;
     if (fromEdge == toEdge) {
         // destination position is on start edge
@@ -315,11 +362,11 @@ MSRoute::getDistanceBetween(SUMOReal fromPos, SUMOReal toPos,
             return toPos - fromPos;
         } else {
             // we cannot go backwards. Something is wrong here
-            return std::numeric_limits<SUMOReal>::max();
+            return std::numeric_limits<double>::max();
         }
     } else if (fromEdge > toEdge) {
         // we don't visit the edge again
-        return std::numeric_limits<SUMOReal>::max();
+        return std::numeric_limits<double>::max();
     }
     for (; it != end(); ++it) {
         if (it == toEdge && !isFirstIteration) {
@@ -327,17 +374,9 @@ MSRoute::getDistanceBetween(SUMOReal fromPos, SUMOReal toPos,
             break;
         } else {
             distance += (*it)->getLength();
-#ifdef HAVE_INTERNAL_LANES
-            if (includeInternal) {
-                // add length of internal lanes to the result
-                const MSEdge* internal = (*it)->getInternalFollowingEdge(*(it + 1));
-                if (internal != 0) {
-                    distance += internal->getLength();
-                }
+            if (includeInternal && (it + 1) != end()) {
+                distance += (*it)->getInternalFollowingLengthTo(*(it + 1));
             }
-#else
-            UNUSED_PARAMETER(includeInternal);
-#endif
         }
         isFirstIteration = false;
     }
@@ -347,7 +386,7 @@ MSRoute::getDistanceBetween(SUMOReal fromPos, SUMOReal toPos,
 
 const RGBColor&
 MSRoute::getColor() const {
-    if (myColor == 0) {
+    if (myColor == nullptr) {
         return RGBColor::DEFAULT_COLOR;
     }
     return *myColor;
@@ -361,4 +400,3 @@ MSRoute::getStops() const {
 
 
 /****************************************************************************/
-
