@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSRoute.h
 /// @author  Daniel Krajzewicz
 /// @author  Friedemann Wesner
@@ -6,33 +18,11 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: MSRoute.h 23341 2017-03-13 10:11:12Z behrisch $
 ///
 // A vehicle route
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSRoute_h
-#define MSRoute_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -44,7 +34,7 @@
 #include <utils/vehicle/SUMOVehicleParameter.h>
 #include <utils/common/Parameterised.h>
 #ifdef HAVE_FOX
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
 #include <FXThread.h>
 #endif
 
@@ -53,7 +43,6 @@
 // class declarations
 // ===========================================================================
 class MSEdge;
-class BinaryInputDevice;
 class OutputDevice;
 
 
@@ -115,7 +104,7 @@ public:
 
     const MSEdge* operator[](int index) const;
 
-    /// @name State I/O (mesosim only)
+    /// @name State I/O
     /// @{
 
     /** @brief Saves all known routes into the given stream
@@ -123,6 +112,9 @@ public:
      * @param[in] os The stream to write the routes into (binary)
      */
     static void dict_saveState(OutputDevice& out);
+
+    /** @brief Decrement  all route references before quick-loading state */
+    static void dict_clearState();
     /// @}
 
     const ConstMSEdgeVector& getEdges() const {
@@ -139,9 +131,10 @@ public:
      * @param[in] fromEdge edge at wich computation begins
      * @param[in] toEdge   edge at which distance computation shall stop
      * @param[in] includeInternal Whether the lengths of internal edges shall be counted
+     * @param[in] routePosition Optional offset when searching for the fromEdge within the route
      * @return             distance between the position fromPos on fromEdge and toPos on toEdge
      */
-    double getDistanceBetween(double fromPos, double toPos, const MSEdge* fromEdge, const MSEdge* toEdge, bool includeInternal = true) const;
+    double getDistanceBetween(double fromPos, double toPos, const MSEdge* fromEdge, const MSEdge* toEdge, bool includeInternal = true, int routePosition = 0) const;
 
     /** @brief Compute the distance between 2 given edges on this route, including the length of internal lanes.
      * This has the same semantics as above but uses iterators instead of edge
@@ -156,9 +149,14 @@ public:
      */
     double getDistanceBetween(double fromPos, double toPos, const MSRouteIterator& fromEdge, const MSRouteIterator& toEdge, bool includeInternal = true) const;
 
-    /// Returns the color
+    /// @brief Returns the color
     const RGBColor& getColor() const;
 
+    /// @brief returns the period
+    SUMOTime getPeriod() const {
+        return myPeriod;
+    }
+
     /** @brief Returns the costs of the route
      *
      * @return The route's costs (normally the time needed to pass it)
@@ -167,6 +165,19 @@ public:
         return myCosts;
     }
 
+    /** @brief Returns the estimated savings due to using this route (compare to the route before rerouting)
+     *
+     * @return The route's estimated savings (the difference in costs of this route to the previous one)
+     */
+    double getSavings() const {
+        return mySavings;
+    }
+
+    /// @brief sets the period
+    void setPeriod(SUMOTime period) {
+        myPeriod = period;
+    }
+
     /** @brief Sets the costs of the route
      *
      * @param[in] costs The new route costs
@@ -174,6 +185,21 @@ public:
     void setCosts(double costs) {
         myCosts = costs;
     }
+    /** @brief Sets the savings of the route
+     *
+     * @param[in] costs The new route costs
+     */
+    void setSavings(double savings) {
+        mySavings = savings;
+    }
+
+    bool mustReroute() const {
+        return myReroute;
+    }
+
+    void setReroute(bool reroute = true) {
+        myReroute = reroute;
+    }
 
     /// Returns the stops
     const std::vector<SUMOVehicleParameter::Stop>& getStops() const;
@@ -210,7 +236,10 @@ public:
      * @param[in] id    the id of the route or the distribution
      * @return          the route (sample)
      */
-    static const MSRoute* dictionary(const std::string& id, MTRand* rng = 0);
+    static const MSRoute* dictionary(const std::string& id, std::mt19937* rng = 0);
+
+    /// @brief returns whether a route with the given id exists
+    static bool hasRoute(const std::string& id);
 
     /** @brief Returns the named route distribution.
      *
@@ -242,9 +271,18 @@ private:
     /// The color
     const RGBColor* const myColor;
 
+    /// The period when repeating this route
+    SUMOTime myPeriod;
+
     /// @brief The assigned or calculated costs
     double myCosts;
 
+    /// @brief The estimated savings when rerouting
+    double mySavings;
+
+    /// @brief Whether this route is incomplete and requires rerouting
+    bool myReroute;
+
     /// @brief List of the stops on the parsed route
     std::vector<SUMOVehicleParameter::Stop> myStops;
 
@@ -270,9 +308,3 @@ private:
     MSRoute& operator=(const MSRoute& s);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
