@@ -1,47 +1,41 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2005-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSStoppingPlace.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
+/// @author  Johannes Rummel
 /// @date    Mon, 13.12.2005
-/// @version $Id: MSStoppingPlace.h 23648 2017-03-24 10:35:01Z laemmel $
 ///
 // A lane area vehicles can halt at
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2005-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSStoppingPlace_h
-#define MSStoppingPlace_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <algorithm>
 #include <map>
 #include <string>
 #include <utils/common/Named.h>
+#include <utils/common/Parameterised.h>
+#include <utils/common/RGBColor.h>
 
 
 // ===========================================================================
 // class declarations
 // ===========================================================================
 class MSLane;
+class MSEdge;
 class SUMOVehicle;
 class MSTransportable;
 class Position;
@@ -62,7 +56,7 @@ class Position;
  * Please note that using the last free space disallows vehicles to enter a
  *  free space in between other vehicles.
  */
-class MSStoppingPlace : public Named {
+class MSStoppingPlace : public Named, public Parameterised {
 public:
     /** @brief Constructor
      *
@@ -74,8 +68,14 @@ public:
      * @param[in] endPos End position of the stop on the lane
      */
     MSStoppingPlace(const std::string& id,
+                    SumoXMLTag element,
                     const std::vector<std::string>& lines, MSLane& lane,
-                    double begPos, double endPos, const std::string name = "");
+                    double begPos, double endPos,
+                    const std::string name = "",
+                    int capacity = 0,
+                    double parkingLength = 0,
+                    const RGBColor& color = RGBColor::WHITE);
+
 
 
     /// @brief Destructor
@@ -114,7 +114,7 @@ public:
      * @param[in] what The end halting position of the vehicle
      * @see computeLastFreePos
      */
-    void enter(SUMOVehicle* what, double beg, double end);
+    void enter(SUMOVehicle* veh, bool parking);
 
 
     /** @brief Called if a vehicle leaves this stop
@@ -135,12 +135,20 @@ public:
      */
     double getLastFreePos(const SUMOVehicle& forVehicle) const;
 
+    /// @brief return whether the given vehicle fits at the given position
+    bool fits(double pos, const SUMOVehicle& veh) const;
 
     /** @brief Returns the next free waiting place for pedestrians / containers
      *
      * @return The next free waiting place for pedestrians / containers
      */
-    Position getWaitPosition() const;
+    virtual Position getWaitPosition(MSTransportable* person) const;
+
+    /** @brief Returns the lane position corresponding to getWaitPosition()
+     *
+     * @return The waiting position along the stop lane
+     */
+    double getWaitingPositionOnLane(MSTransportable* t) const;
 
 
     /** @brief For vehicles at the stop this gives the the actual stopping
@@ -155,19 +163,72 @@ public:
         return (int)myWaitingTransportables.size();
     }
 
-    /// @brief adds a transportable to this stop
-    void addTransportable(MSTransportable* p);
+    /** @brief Returns the tranportables waiting on this stop
+     */
+    std::vector<const MSTransportable*> getTransportables() const;
 
-    /// @brief Removes a transportable from this stop
-    void removeTransportable(MSTransportable* p);
-
-    /// @brief adds an access point to this stop
-    virtual void addAccess(MSLane* lane, const double pos) {
-        myAccessPos.insert(std::make_pair(lane, pos));
+    /** @brief Returns the number of stopped vehicles waiting on this stop
+    */
+    int getStoppedVehicleNumber() const {
+        return (int)myEndPositions.size();
     }
 
-    /// @brief checks whether this stop provides access to the given edge
-    bool hasAccess(const MSEdge* edge) const;
+    double getLastFreePos() const {
+        return myLastFreePos;
+    }
+
+    /// @brief whether there is still capacity for more transportables
+    bool hasSpaceForTransportable() const;
+
+    /// @brief adds a transportable to this stop
+    bool addTransportable(const MSTransportable* p);
+
+    /// @brief Removes a transportable from this stop
+    void removeTransportable(const MSTransportable* p);
+
+    /// @brief adds an access point to this stop
+    virtual bool addAccess(MSLane* lane, const double pos, const double length);
+
+    /// @brief lanes and positions connected to this stop
+    const std::vector<std::tuple<MSLane*, double, double> >& getAllAccessPos() const {
+        return myAccessPos;
+    }
+
+    /// @brief the position on the given edge which is connected to this stop, -1 on failure
+    double getAccessPos(const MSEdge* edge) const;
+
+    /// @brief the distance from the access on the given edge to the stop, -1 on failure
+    double getAccessDistance(const MSEdge* edge) const;
+
+    const std::string& getMyName() const;
+
+    /// @brief return the type of this stopping place
+    SumoXMLTag getElement() const {
+        return myElement;
+    }
+
+    const RGBColor& getColor() const;
+
+    static int getTransportablesAbreast(double length, SumoXMLTag element);
+
+    /// @brief get list of vehicles waiting at this stop
+    std::vector<const SUMOVehicle*> getStoppedVehicles() const;
+
+    /// @brief get number of persons waiting at this stop
+    inline int getNumWaitingPersons() const {
+        return (int)myWaitingTransportables.size();
+    }
+
+    /// @brief get number of persons that can wait at this stop
+    inline int getWaitingCapacity() const {
+        return myTransportableCapacity;
+    }
+
+    /// @brief get IDs of persons waiting at this stop
+    void getWaitingPersonIDs(std::vector<std::string>& into) const;
+
+    /** @brief Remove all vehicles before quick-loading state */
+    void clearState();
 
 protected:
     /** @brief Computes the last free position on this stop
@@ -178,13 +239,17 @@ protected:
      */
     void computeLastFreePos();
 
+    int getTransportablesAbreast() const;
 
 protected:
+    /// @brief the type of stopping place
+    const SumoXMLTag myElement;
+
     /// @brief The list of lines that are assigned to this stop
     std::vector<std::string> myLines;
 
     /// @brief A map from objects (vehicles) to the areas they acquire after entering the stop
-    std::map<const SUMOVehicle*, std::pair<double, double> > myEndPositions;
+    std::map<const SUMOVehicle*, std::pair<double, double>, ComparatorNumericalIdLess> myEndPositions;
 
     /// @brief The lane this bus stop is located at
     const MSLane& myLane;
@@ -198,20 +263,29 @@ protected:
     /// @brief The last free position at this stop (variable)
     double myLastFreePos;
 
-    /// @brief The next free position for persons / containers
-    double myWaitingPos;
-
     /// @brief The name of the stopping place
     const std::string myName;
-public:
-    const std::string& getMyName() const;
+
+    /// @brief The number of transportables that can wait here
+    const int myTransportableCapacity;
+
+    /// @brief the scaled space capacity for parking vehicles
+    const double myParkingFactor;
+
+    /// @brief The color of the stopping place
+    const RGBColor myColor;
+
+    /// @brief row depth of waiting transportables
+    const double myTransportableDepth;
+
 protected:
 
-    /// @brief Persons waiting at this stop
-    std::vector<MSTransportable*> myWaitingTransportables;
+    /// @brief Persons waiting at this stop (mapped to waiting position)
+    std::map<const MSTransportable*, int> myWaitingTransportables;
+    std::set<int> myWaitingSpots;
 
     /// @brief lanes and positions connected to this stop
-    std::multimap<MSLane*, double> myAccessPos;
+    std::vector<std::tuple<MSLane*, double, double> > myAccessPos;
 
 private:
     /// @brief Invalidated copy constructor.
@@ -222,9 +296,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
