@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSVehicleContainer.cpp
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSVehicleContainer.cpp 21182 2016-07-18 06:46:01Z behrisch $
 ///
 // vehicles sorted by their departures
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <algorithm>
 #include <cassert>
@@ -36,10 +27,6 @@
 #include "MSVehicle.h"
 #include "MSVehicleContainer.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -90,7 +77,9 @@ MSVehicleContainer::add(SUMOVehicle* veh) {
         find_if(array.begin() + 1, array.begin() + currentSize + 1, DepartFinder(veh->getParameter().depart));
     if (currentSize == 0 || i == array.begin() + currentSize + 1) {
         // a new heap-item is necessary
-        VehicleDepartureVector newElem(veh->getParameter().depart, VehicleVector());
+        const SUMOTime delay = veh->getParameter().depart % DELTA_T;
+        const SUMOTime depart = veh->getParameter().depart + (delay == 0 ? 0 : DELTA_T - delay);
+        VehicleDepartureVector newElem(depart, VehicleVector());
         newElem.second.push_back(veh);
         addReplacing(newElem);
     } else {
@@ -145,17 +134,17 @@ MSVehicleContainer::addReplacing(const VehicleDepartureVector& x) {
     // Percolate up
     int hole = ++currentSize;
     for (; hole > 1 && (x.first < array[ hole / 2 ].first); hole /= 2) {
-        assert(array.size() > (int) hole);
-        array[ hole ] = array[ hole / 2 ];
+        assert((int)array.size() > hole);
+        array[hole] = array[ hole / 2 ];
     }
-    assert(array.size() > (int) hole);
-    array[ hole ] = x;
+    assert((int)array.size() > hole);
+    array[hole] = x;
 }
 
 
 bool
 MSVehicleContainer::anyWaitingBefore(SUMOTime time) const {
-    return !isEmpty() && topTime() < time;
+    return !isEmpty() && topTime() <= time;
 }
 
 
@@ -208,23 +197,23 @@ MSVehicleContainer::isFull() const {
 void
 MSVehicleContainer::percolateDown(int hole) {
     int child;
-    assert(array.size() > (int)hole);
+    assert((int)array.size() > hole);
     VehicleDepartureVector tmp = array[ hole ];
 
     for (; hole * 2 <= currentSize; hole = child) {
         child = hole * 2;
-        if (child != currentSize && (array[ child + 1 ].first < array[ child ].first)) {
+        if (child != currentSize && (array[child + 1].first < array[child].first)) {
             child++;
         }
         if ((array[ child ].first < tmp.first)) {
-            assert(array.size() > (int) hole);
-            array[ hole ] = array[ child ];
+            assert((int)array.size() > hole);
+            array[hole] = array[child];
         } else {
             break;
         }
     }
-    assert(array.size() > (int) hole);
-    array[ hole ] = tmp;
+    assert((int)array.size() > hole);
+    array[hole] = tmp;
 }
 
 
@@ -258,7 +247,13 @@ std::ostream& operator << (std::ostream& strm, MSVehicleContainer& cont) {
     return strm;
 }
 
-
+void
+MSVehicleContainer::clearState() {
+    for (VehicleDepartureVector& vdv : array) {
+        vdv.first = 0;
+        vdv.second.clear();
+    }
+    currentSize = 0;
+}
 
 /****************************************************************************/
-
