@@ -1,44 +1,39 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSVehicleControl.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 10. Dec 2003
-/// @version $Id: MSVehicleControl.h 20593 2016-04-29 06:10:53Z namdre $
 ///
 // The class responsible for building and deletion of vehicles
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSVehicleControl_h
-#define MSVehicleControl_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <math.h>
+#include <cmath>
 #include <string>
 #include <map>
 #include <set>
-#include <utils/common/RandomDistributor.h>
+#ifdef HAVE_FOX
+#include <utils/foxtools/fxheader.h>
+#include <utils/foxtools/FXSynchQue.h>
+#endif
+#include <utils/distribution/RandomDistributor.h>
 #include <utils/common/SUMOTime.h>
 #include <utils/common/SUMOVehicleClass.h>
+#include "MSNet.h"
 
 
 // ===========================================================================
@@ -46,6 +41,7 @@
 // ===========================================================================
 class SUMOVehicle;
 class SUMOVehicleParameter;
+class MSBaseVehicle;
 class MSVehicle;
 class MSRoute;
 class MSVehicleType;
@@ -84,6 +80,8 @@ public:
     /// @brief Destructor
     virtual ~MSVehicleControl();
 
+    /// @brief create default types
+    void initDefaultTypes();
 
     /// @name Vehicle creation
     /// @{
@@ -101,7 +99,7 @@ public:
      * @return The built vehicle (MSVehicle instance)
      */
     virtual SUMOVehicle* buildVehicle(SUMOVehicleParameter* defs, const MSRoute* route,
-                                      const MSVehicleType* type,
+                                      MSVehicleType* type,
                                       const bool ignoreStopErrors, const bool fromRouteFile = true);
     /// @}
 
@@ -113,8 +111,9 @@ public:
     /** @brief Tries to insert the vehicle into the internal vehicle container
      *
      * Checks whether another vehicle with the same id exists; returns false
-     *  if so. Otherwise, the vehicle is added to "myVehicleDict" and
-     *  true is returned.
+     *  if so. Otherwise, the vehicle is added to "myVehicleDict".
+     *  It also checks whether the vehicle has a "triggered" departure
+     *  and registers it accordingly.
      *
      * The vehicle control gets responsible for vehicle deletion.
      *
@@ -144,6 +143,11 @@ public:
      */
     virtual void deleteVehicle(SUMOVehicle* v, bool discard = false);
 
+    void fixVehicleCounts() {
+        myLoadedVehNo++;
+        myEndedVehNo++;
+        myDiscarded++;
+    }
 
     /** @brief Removes a vehicle after it has ended
      *
@@ -156,7 +160,21 @@ public:
      *
      * @param[in] veh The vehicle to remove
      */
-    void scheduleVehicleRemoval(SUMOVehicle* veh);
+    void scheduleVehicleRemoval(SUMOVehicle* veh, bool checkDuplicate = false);
+
+
+    /** @brief Removes a vehicle after it has ended
+     *
+     * Writes output to tripinfos and vehroutes if wished; decrements
+     *  the number of running vehicles and increments the number of ended
+     *  vehicles. Then deletes the vehicle using "deleteVehicle".
+     *
+     * This method should be called for each vehicle that was inserted
+     *  into the network and quits its ride.
+     *
+     * @param[in] veh The vehicle to remove
+     */
+    void removePending();
 
 
     /** @brief Returns the begin of the internal vehicle map
@@ -198,23 +216,51 @@ public:
     /** @brief Returns the number of build vehicles
      * @return The number of loaded (build) vehicles
      */
-    unsigned int getLoadedVehicleNo() const {
+    int getLoadedVehicleNo() const {
         return myLoadedVehNo;
     }
 
 
+    /** @brief Returns the number of halting vehicles
+     * @return The number of halting vehicles
+     */
+    virtual int getHaltingVehicleNo() const;
+
+    /// @brief get current absolute and relative mean vehicle speed in the network
+    virtual std::pair<double, double> getVehicleMeanSpeeds() const;
+    double getVehicleMeanSpeed() const {
+        return getVehicleMeanSpeeds().first;
+    }
+    double getVehicleMeanSpeedRelative() const {
+        return getVehicleMeanSpeeds().second;
+    }
+
     /** @brief Returns the number of removed vehicles
      * @return The number of vehicles that have left the simulation
      */
-    unsigned int getEndedVehicleNo() const {
+    int getEndedVehicleNo() const {
         return myEndedVehNo;
     }
 
+    /** @brief Returns the number of arrived vehicles
+     * @return The number of vehicles that have arrived at their destination
+     */
+    int getArrivedVehicleNo() const {
+        return myEndedVehNo - myDiscarded;
+    }
+
+    /** @brief Returns the number of discarded vehicles
+     * @return The number of vehicles that could not be inserted and were permantently discarded
+     */
+    int getDiscardedVehicleNo() const {
+        return myDiscarded;
+    }
+
 
     /** @brief Returns the number of build and inserted, but not yet deleted vehicles
      * @return The number simulated vehicles (including those in teleporter)
      */
-    unsigned int getRunningVehicleNo() const {
+    int getRunningVehicleNo() const {
         return myRunningVehNo;
     }
 
@@ -222,7 +268,7 @@ public:
     /** @brief Returns the number of inserted vehicles
      * @return The number of vehicles that have entered the simulation so far
      */
-    unsigned int getDepartedVehicleNo() const {
+    int getDepartedVehicleNo() const {
         return myRunningVehNo + myEndedVehNo - myDiscarded;
     }
 
@@ -231,9 +277,10 @@ public:
      * considering that "frac" of all vehicles shall be emitted overall
      * if a negative fraction is given the demand scaling factor is used
      * (--scale)
+     * if a negative loaded number is is given, myLoadedVehNo is used
      * @return the number of vehicles to create (something between 0 and ceil(frac))
      */
-    unsigned int getQuota(SUMOReal frac = -1) const;
+    int getQuota(double frac = -1, int loaded = -1) const;
 
 
     /** @brief Returns the number of build vehicles that have not been removed or
@@ -241,43 +288,47 @@ public:
      * @return Number of active vehicles
      */
     int getActiveVehicleCount() const {
-        return myLoadedVehNo - (myWaitingForPerson + myWaitingForContainer + myEndedVehNo);
+        return myLoadedVehNo - (myWaitingForTransportable + myEndedVehNo);
     }
 
 
     /// @brief return the number of collisions
-    unsigned int getCollisionCount() const {
+    int getCollisionCount() const {
         return myCollisions;
     }
 
     /// @brief return the number of teleports due to jamming
-    unsigned int getTeleportsJam() const {
+    int getTeleportsJam() const {
         return myTeleportsJam;
     }
 
     /// @brief return the number of teleports due to vehicles stuck on a minor road
-    unsigned int getTeleportsYield() const {
+    int getTeleportsYield() const {
         return myTeleportsYield;
     }
 
     /// @brief return the number of teleports due to vehicles stuck on the wrong lane
-    unsigned int getTeleportsWrongLane() const {
+    int getTeleportsWrongLane() const {
         return myTeleportsWrongLane;
     }
 
     /// @brief return the number of teleports (including collisions)
-    unsigned int getTeleportCount() const;
+    int getTeleportCount() const;
 
     /// @brief return the number of emergency stops
-    unsigned int getEmergencyStops() const {
+    int getEmergencyStops() const {
         return myEmergencyStops;
     }
 
+    /// @brief return the number of vehicles that are currently stopped
+    int getStoppedVehiclesCount() const {
+        return myStoppedVehicles;
+    }
 
     /** @brief Returns the total departure delay
      * @return Sum of steps vehicles had to wait until being inserted
      */
-    SUMOReal getTotalDepartureDelay() const {
+    double getTotalDepartureDelay() const {
         return myTotalDepartureDelay;
     }
 
@@ -285,7 +336,7 @@ public:
     /** @brief Returns the total travel time
      * @return Sum of travel times of arrived vehicles
      */
-    SUMOReal getTotalTravelTime() const {
+    double getTotalTravelTime() const {
         return myTotalTravelTime;
     }
     /// @}
@@ -309,6 +360,12 @@ public:
      */
     bool addVType(MSVehicleType* vehType);
 
+    /*
+     * @param[in] vehType The vehicle type to remove
+     * @return Whether the vehicle type could be removed
+     */
+    void removeVType(const MSVehicleType* vehType);
+
 
     /** @brief Adds a vehicle type distribution
      *
@@ -326,6 +383,16 @@ public:
     bool addVTypeDistribution(const std::string& id, RandomDistributor<MSVehicleType*>* vehTypeDistribution);
 
 
+    /** @brief Asks for existence of a vehicle type
+     *
+     * If vehicle type or distribution with the id exists, true is returned, false otherwise.
+     *
+     * @param[in] id The id of the type or distribution
+     * @return Whether the vehicle type or distribution exists
+     */
+    bool hasVType(const std::string& id) const;
+
+
     /** @brief Asks for a vehicle type distribution
      *
      * If vehicle type distribution with the id exists, true is returned, false otherwise.
@@ -338,53 +405,37 @@ public:
 
     /** @brief Returns the named vehicle type or a sample from the named distribution
      * @param[in] id The id of the vehicle type to return. If left out, the default type is returned.
-     * @return The named vehicle type, or 0 if no such type exists
+     * @return The named vehicle type, or nullptr if no such type exists
      */
-    MSVehicleType* getVType(const std::string& id = DEFAULT_VTYPE_ID, MTRand* rng = 0);
+    MSVehicleType* getVType(const std::string& id = DEFAULT_VTYPE_ID, std::mt19937* rng = nullptr, bool readOnly = false);
 
 
     /** @brief Inserts ids of all known vehicle types and vehicle type distributions to the given vector
      * @param[in] into The vector to fill with ids
      */
     void insertVTypeIDs(std::vector<std::string>& into) const;
+
+
+    /** @brief Return the distribution IDs the vehicle type is a member of
+    * @param[in] vehType The vehicle type to look for membership in distributions
+    */
+    const std::set<std::string> getVTypeDistributionMembership(const std::string& id) const;
+
+    /// @brief return the vehicle type distribution with the given id
+    const RandomDistributor<MSVehicleType*>* getVTypeDistribution(const std::string& typeDistID) const;
+
     /// @}
 
-    /// @brief Adds a vehicle to the list of waiting vehiclse to a given edge
-    void addWaiting(const MSEdge* const edge, SUMOVehicle* vehicle);
-
-    /// @brief Removes a vehicle from the list of waiting vehicles to a given edge
-    void removeWaiting(const MSEdge* const edge, SUMOVehicle* vehicle);
-
-    /* @brief returns a vehicle of the given lines that is waiting for a for a person or a container at this edge at the given positions
-     * @param[in] edge The edge at which the vehicle is positioned.
-     * @param[in] lines The set of lines from which at least one must correspond to the line of the vehicle
-     * @param[in] position The vehicle shall be positioned in the interval [position - t, position + t], where t is some tolerance
-     * @param[in] ridingID The id of the person or container that wants to ride
+    /** @brief increases the count of vehicles waiting for a transport to allow recognition of person / container related deadlocks
      */
-    SUMOVehicle* getWaitingVehicle(const MSEdge* const edge, const std::set<std::string>& lines, const SUMOReal position, const std::string ridingID);
-
-    /** @brief increases the count of vehicles waiting for a person to allow recogniztion of person related deadlocks
-     */
-    void registerOneWaitingForPerson() {
-        myWaitingForPerson++;
+    void registerOneWaiting() {
+        myWaitingForTransportable++;
     }
 
-    /** @brief decreases the count of vehicles waiting for a person to allow recogniztion of person related deadlocks
+    /** @brief decreases the count of vehicles waiting for a transport to allow recognition of person / container related deadlocks
      */
-    void unregisterOneWaitingForPerson() {
-        myWaitingForPerson--;
-    }
-
-    /** @brief increases the count of vehicles waiting for a container to allow recogniztion of container related deadlocks
-     */
-    void registerOneWaitingForContainer() {
-        myWaitingForContainer++;
-    }
-
-    /** @brief decreases the count of vehicles waiting for a container to allow recogniztion of container related deadlocks
-     */
-    void unregisterOneWaitingForContainer() {
-        myWaitingForContainer--;
+    void unregisterOneWaiting() {
+        myWaitingForTransportable--;
     }
 
     /// @brief registers one collision-related teleport
@@ -412,21 +463,36 @@ public:
         myEmergencyStops++;
     }
 
+    /// @brief register emergency stop
+    void registerStopStarted() {
+        myStoppedVehicles++;
+    }
+
+    /// @brief register emergency stop
+    void registerStopEnded() {
+        myStoppedVehicles--;
+    }
+
     /// @name State I/O
     /// @{
 
     /** @brief Sets the current state variables as loaded from the stream
      */
-    void setState(int runningVehNo, int loadedVehNo, int endedVehNo, SUMOReal totalDepartureDelay, SUMOReal totalTravelTime);
+    void setState(int runningVehNo, int loadedVehNo, int endedVehNo, double totalDepartureDelay, double totalTravelTime);
 
     /** @brief Saves the current state into the given stream
      */
     void saveState(OutputDevice& out);
+
+    /** @brief Remove all vehicles before quick-loading state */
+    void clearState();
     /// @}
 
-    /// @brief avoid counting a vehicle twice if it was loaded from state and route input
-    void discountStateLoaded() {
-        myLoadedVehNo--;
+    /// @brief discount vehicles that were removed during state loading
+    void discountStateRemoved(int n) {
+        myRunningVehNo -= n;
+        myDiscarded += n;
+        myEndedVehNo += n;
     }
 
 
@@ -434,19 +500,23 @@ public:
      */
     void abortWaiting();
 
-    /// @brief compute (optional) random offset to the departure time
-    SUMOTime computeRandomDepartOffset() const;
-
     /// @brief return the maximum speed factor for all vehicles that ever entered the network
-    SUMOReal getMaxSpeedFactor() const {
+    double getMaxSpeedFactor() const {
         return myMaxSpeedFactor;
     }
 
     /// @brief return the minimum deceleration capability for all vehicles that ever entered the network
-    SUMOReal getMinDeceleration() const {
+    double getMinDeceleration() const {
         return myMinDeceleration;
     }
 
+    void adaptIntermodalRouter(MSNet::MSIntermodalRouter& router) const;
+
+    /// @brief sets the demand scaling factor
+    void setScale(double scale) {
+        myScale = scale;
+    }
+
 private:
     /** @brief Checks whether the vehicle type (distribution) may be added
      *
@@ -456,37 +526,45 @@ private:
      */
     bool checkVType(const std::string& id);
 
+    /// @brief whether the given vehicle is scheduled for removal
+    bool isPendingRemoval(SUMOVehicle* veh);
+
 protected:
-    /// @name Vehicle statistics (always accessable)
+    void initVehicle(MSBaseVehicle* built, const bool ignoreStopErrors);
+
+private:
+    /// @name Vehicle statistics (always accessible)
     /// @{
 
     /// @brief The number of build vehicles
-    unsigned int myLoadedVehNo;
+    int myLoadedVehNo;
 
     /// @brief The number of vehicles within the network (build and inserted but not removed)
-    unsigned int myRunningVehNo;
+    int myRunningVehNo;
 
     /// @brief The number of removed vehicles
-    unsigned int myEndedVehNo;
+    int myEndedVehNo;
 
     /// @brief The number of vehicles which were discarded while loading
-    unsigned int myDiscarded;
+    int myDiscarded;
 
     /// @brief The number of collisions
-    unsigned int myCollisions;
+    int myCollisions;
 
     /// @brief The number of teleports due to jam
-    unsigned int myTeleportsJam;
+    int myTeleportsJam;
 
     /// @brief The number of teleports due to vehicles stuck on a minor road
-    unsigned int myTeleportsYield;
+    int myTeleportsYield;
 
     /// @brief The number of teleports due to vehicles stuck on the wrong lane
-    unsigned int myTeleportsWrongLane;
+    int myTeleportsWrongLane;
 
     /// @brief The number of emergency stops
-    unsigned int myEmergencyStops;
+    int myEmergencyStops;
 
+    /// @brief The number of stopped vehicles
+    int myStoppedVehicles;
     /// @}
 
 
@@ -494,13 +572,14 @@ protected:
     /// @{
 
     /// @brief The aggregated time vehicles had to wait for departure (in seconds)
-    SUMOReal myTotalDepartureDelay;
+    double myTotalDepartureDelay;
 
     /// @brief The aggregated time vehicles needed to aacomplish their route (in seconds)
-    SUMOReal myTotalTravelTime;
+    double myTotalTravelTime;
     /// @}
 
 
+protected:
     /// @name Vehicle container
     /// @{
 
@@ -511,6 +590,7 @@ protected:
     /// @}
 
 
+private:
     /// @name Vehicle type container
     /// @{
 
@@ -524,32 +604,45 @@ protected:
     /// @brief A distribution of vehicle types (probability->vehicle type)
     VTypeDistDictType myVTypeDistDict;
 
-    /// @brief Whether no vehicle type was loaded
+    /// @brief Inverse lookup from vehicle type to distributions it is a member of
+    std::map<std::string, std::set<std::string>> myVTypeToDist;
+
+    /// @brief Whether the default vehicle type was already used or can still be replaced
     bool myDefaultVTypeMayBeDeleted;
 
-    /// @brief Whether no pedestrian type was loaded
+    /// @brief Whether the default pedestrian type was already used or can still be replaced
     bool myDefaultPedTypeMayBeDeleted;
 
-    /// the lists of waiting vehicles to a given edge
-    std::map<const MSEdge* const, std::vector<SUMOVehicle*> > myWaiting;
+    /// @brief Whether the default container type was already used or can still be replaced
+    bool myDefaultContainerTypeMayBeDeleted;
 
-    /// the number of vehicles wainting for persons contained in myWaiting which can only continue by being triggered
-    unsigned int myWaitingForPerson;
+    /// @brief Whether the default bicycle type was already used or can still be replaced
+    bool myDefaultBikeTypeMayBeDeleted;
 
-    /// the number of vehicles wainting for containers contained in myWaiting which can only continue by being triggered
-    unsigned int myWaitingForContainer;
+    /// @brief Whether the default taxi type was already used or can still be replaced
+    bool myDefaultTaxiTypeMayBeDeleted;
+
+    /// the number of vehicles wainting for persons or containers contained in myWaiting which can only continue by being triggered
+    int myWaitingForTransportable;
 
     /// @brief The scaling factor (especially for inc-dua)
-    SUMOReal myScale;
-
-    /// @brief The maximum random offset to be added to vehicles departure times (non-negative)
-    SUMOTime myMaxRandomDepartOffset;
+    double myScale;
 
     /// @brief The maximum speed factor for all vehicles in the network
-    SUMOReal myMaxSpeedFactor;
+    double myMaxSpeedFactor;
 
     /// @brief The minimum deceleration capability for all vehicles in the network
-    SUMOReal myMinDeceleration;
+    double myMinDeceleration;
+
+    /// @brief List of vehicles which belong to public transport
+    std::vector<SUMOVehicle*> myPTVehicles;
+
+    /// @brief List of vehicles which are going to be removed
+#ifdef HAVE_FOX
+    FXSynchQue<SUMOVehicle*, std::vector<SUMOVehicle*> > myPendingRemovals;
+#else
+    std::vector<SUMOVehicle*> myPendingRemovals;
+#endif
 
 private:
     /// @brief invalidated copy constructor
@@ -560,9 +653,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
