@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSVehicleType.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSVehicleType.h 21182 2016-07-18 06:46:01Z behrisch $
 ///
 // The car-following model and parameter
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSVehicleType_h
-#define MSVehicleType_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <cassert>
 #include <map>
@@ -43,14 +33,12 @@
 #include <utils/common/RandHelper.h>
 #include <utils/vehicle/SUMOVTypeParameter.h>
 #include <utils/common/RGBColor.h>
-#include <foreign/mersenne/MersenneTwister.h>
 
 
 // ===========================================================================
 // class declarations
 // ===========================================================================
 class MSLane;
-class BinaryInputDevice;
 class MSCFModel;
 class SUMOVTypeParameter;
 
@@ -89,7 +77,7 @@ public:
      * @return Whether the given parameter was set
      */
     bool wasSet(int what) const {
-        return (myParameter.setParameter & what) != 0;
+        return (myParameter.parametersSet & what) != 0;
     }
 
 
@@ -115,7 +103,7 @@ public:
     /** @brief Get vehicle's length [m]
      * @return The length vehicles of this type have in m
      */
-    SUMOReal getLength() const {
+    double getLength() const {
         return myParameter.length;
     }
 
@@ -123,7 +111,7 @@ public:
     /** @brief Get vehicle's length including the minimum gap [m]
      * @return The length vehicles of this type have (including the minimum gap in m
      */
-    SUMOReal getLengthWithGap() const {
+    double getLengthWithGap() const {
         return myParameter.length + myParameter.minGap;
     }
 
@@ -131,14 +119,14 @@ public:
     /** @brief Get the free space in front of vehicles of this class
      * @return The place before the vehicle
      */
-    SUMOReal getMinGap() const {
+    double getMinGap() const {
         return myParameter.minGap;
     }
 
     /** @brief Get the minimum lateral gap that vehicles of this type maintain
      * @return The place before the vehicle
      */
-    SUMOReal getMinGapLat() const {
+    double getMinGapLat() const {
         return myParameter.minGapLat;
     }
 
@@ -167,7 +155,7 @@ public:
     /** @brief Get vehicle's maximum speed [m/s].
      * @return The maximum speed (in m/s) of vehicles of this class
      */
-    SUMOReal getMaxSpeed() const {
+    double getMaxSpeed() const {
         return myParameter.maxSpeed;
     }
 
@@ -175,13 +163,13 @@ public:
     /** @brief Computes and returns the speed deviation
      * @return A new, random speed deviation
      */
-    SUMOReal computeChosenSpeedDeviation(MTRand* rng, const SUMOReal minDevFactor = 0.2) const;
+    double computeChosenSpeedDeviation(std::mt19937* rng, const double minDev = -1.) const;
 
 
     /** @brief Get the default probability of this vehicle type
      * @return The probability to use this type
      */
-    SUMOReal getDefaultProbability() const {
+    double getDefaultProbability() const {
         return myParameter.defaultProbability;
     }
 
@@ -215,23 +203,31 @@ public:
     /** @brief Returns this type's speed factor
      * @return The speed factor of this type
      */
-    SUMOReal getSpeedFactor() const {
+    const Distribution_Parameterized& getSpeedFactor() const {
         return myParameter.speedFactor;
     }
 
 
-    /** @brief Returns this type's speed deviation
-     * @return The speed deviation of this type
+    /** @brief Returns this type's default action step length
+     * @return The default action step length of this type (in ms.)
      */
-    SUMOReal getSpeedDeviation() const {
-        return myParameter.speedDev;
+    SUMOTime getActionStepLength() const {
+        return myParameter.actionStepLength;
+    }
+
+
+    /** @brief Returns this type's default action step length in seconds
+     * @return The default action step length of this type (in s.)
+     */
+    double getActionStepLengthSecs() const {
+        return myCachedActionStepLengthSecs;
     }
 
 
     /** @brief Returns this type's impatience
      * @return The impatience of this type
      */
-    SUMOReal getImpatience() const {
+    double getImpatience() const {
         return myParameter.impatience;
     }
     /// @}
@@ -244,14 +240,14 @@ public:
     /** @brief Get the width which vehicles of this class shall have when being drawn
      * @return The width of this type's vehicles
      */
-    SUMOReal getWidth() const {
+    double getWidth() const {
         return myParameter.width;
     }
 
     /** @brief Get the height which vehicles of this class shall have when being drawn
      * @return The height of this type's vehicles
      */
-    SUMOReal getHeight() const {
+    double getHeight() const {
         return myParameter.height;
     }
 
@@ -311,7 +307,7 @@ public:
     /** @brief Get vehicle's maximum lateral speed [m/s].
      * @return The maximum lateral speed (in m/s) of vehicles of this class
      */
-    SUMOReal getMaxSpeedLat() const {
+    double getMaxSpeedLat() const {
         return myParameter.maxSpeedLat;
     }
 
@@ -327,6 +323,36 @@ public:
     /// @name Setter methods
     /// @{
 
+    /** @brief Set a new value for this type's acceleration.
+     * @param[in] accel The new acceleration of this type
+     */
+    void setAccel(double accel);
+
+    /** @brief Set a new value for this type's deceleration.
+     * @param[in] decel The new deceleration of this type
+     */
+    void setDecel(double decel);
+
+    /** @brief Set a new value for this type's emergency deceleration.
+     * @param[in] emergencyDecel The new emergency deceleration of this type
+     */
+    void setEmergencyDecel(double emergencyDecel);
+
+    /** @brief Set a new value for this type's apparent deceleration.
+     * @param[in] apparentDecel The new apparent deceleration of this type
+     */
+    void setApparentDecel(double apparentDecel);
+
+    /** @brief Set a new value for this type's imperfection.
+     * @param[in] imperfection The new imperfection of this type
+     */
+    void setImperfection(double imperfection);
+
+    /** @brief Set a new value for this type's headway.
+     * @param[in] tau The new headway of this type
+     */
+    void setTau(double tau);
+
     /** @brief Set a new value for this type's length
      *
      * If the given value<0 then the one from the original type will
@@ -334,7 +360,17 @@ public:
      *
      * @param[in] length The new length of this type
      */
-    void setLength(const SUMOReal& length);
+    void setLength(const double& length);
+
+
+    /** @brief Set a new value for this type's height
+     *
+     * If the given value<0 then the one from the original type will
+     *  be used.
+     *
+     * @param[in] height The new height of this type
+     */
+    void setHeight(const double& height);
 
 
     /** @brief Set a new value for this type's minimum gap
@@ -344,9 +380,18 @@ public:
      *
      * @param[in] minGap The new minimum gap of this type
      */
-    void setMinGap(const SUMOReal& minGap);
+    void setMinGap(const double& minGap);
 
 
+    /** @brief Set a new value for this type's minimum lataral gap
+     *
+     * If the given value<0 then the one from the original type will
+     *  be used.
+     *
+     * @param[in] minGapLat The new minimum lateral gap of this type
+     */
+    void setMinGapLat(const double& minGapLat);
+
     /** @brief Set a new value for this type's maximum speed
      *
      * If the given value<0 then the one from the original type will
@@ -354,8 +399,16 @@ public:
      *
      * @param[in] maxSpeed The new maximum speed of this type
      */
-    void setMaxSpeed(const SUMOReal& maxSpeed);
+    void setMaxSpeed(const double& maxSpeed);
 
+    /** @brief Set a new value for this type's maximum lateral speed
+     *
+     * If the given value<0 then the one from the original type will
+     *  be used.
+     *
+     * @param[in] maxSpeedLat The new maximum lateral speed of this type
+     */
+    void setMaxSpeedLat(const double& maxSpeedLat);
 
     /** @brief Set a new value for this type's vehicle class
      * @param[in] vclass The new vehicle class of this type
@@ -370,7 +423,7 @@ public:
      *
      * @param[in] prob The new default probability of this type
      */
-    void setDefaultProbability(const SUMOReal& prob);
+    void setDefaultProbability(const double& prob);
 
 
     /** @brief Set a new value for this type's speed factor
@@ -380,7 +433,7 @@ public:
      *
      * @param[in] factor The new speed factor of this type
      */
-    void setSpeedFactor(const SUMOReal& factor);
+    void setSpeedFactor(const double& factor);
 
 
     /** @brief Set a new value for this type's speed deviation
@@ -390,7 +443,23 @@ public:
      *
      * @param[in] dev The new speed deviation of this type
      */
-    void setSpeedDeviation(const SUMOReal& dev);
+    void setSpeedDeviation(const double& dev);
+
+
+    /** @brief Set a new value for this type's action step length
+     *
+     * @param[in] actionStepLength The new action step length of this type (in ms.)
+     * @param[in] resetActionOffset If True (default), the next action point is
+     *            scheduled immediately. if If resetActionOffset == False, the interval
+     *            between the last and the next action point is updated to match the given
+     *            value for all vehicles of this type, or if the latter is smaller than the
+     *            time since the last action point, the next action follows immediately.
+     *
+     * @note: Singular vtypes do not update the state of the corresponding vehicle, because
+     *        the global lookup would be too expensive. The caller is responsible to
+     *        perform the actionOffsetReset operation at caller context, where the vehicle is known.
+     */
+    void setActionStepLength(const SUMOTime actionStepLength, bool resetActionOffset);
 
 
     /** @brief Set a new value for this type's emission class
@@ -412,7 +481,7 @@ public:
      *
      * @param[in] width The new width of this type
      */
-    void setWidth(const SUMOReal& width);
+    void setWidth(const double& width);
 
 
     /** @brief Set a new value for this type's shape
@@ -420,16 +489,19 @@ public:
      */
     void setShape(SUMOVehicleShape shape);
 
-
     /** @brief Set a new value for this type's impatience
      * @param[in] impatience The new impatience of this type
      */
-    void setImpatience(const SUMOReal impatience);
+    void setImpatience(const double impatience);
+
+    /** @brief Set vehicle's preferred lateral alignment
+     */
+    void setPreferredLateralAlignment(LateralAlignment latAlignment);
     /// @}
 
 
 
-    /// @name Static methods for building vehicle types
+    /// @name methods for building vehicle types
     /// @{
 
     /** @brief Builds the microsim vehicle type described by the given parameter
@@ -439,21 +511,46 @@ public:
      */
     static MSVehicleType* build(SUMOVTypeParameter& from);
 
+    /// @brief   Accessor function for parameter equivalent returning entry time for a specific manoeuver angle
+    SUMOTime getEntryManoeuvreTime(const int angle) const;
+    /// @brief   Accessor function for parameter equivalent returning exit time for a specific manoeuver angle
+    SUMOTime getExitManoeuvreTime(const int angle) const;
 
-    /** @brief Duplicates the microsim vehicle type giving it a the given id
+
+    /** @brief Duplicates the microsim vehicle type giving the newly created type the given id,
+     *         marking it as vehicle specific
      * @param[in] id The new id of the type
-     * @param[in] from The vehicle type
+     * @return The built vehicle type
+     * @note This method is used in case that a vType is meant to be used only for a specific vehicle
+     *       The created vType will be removed with the vehicle or if it is assigned a new type.
+     */
+    MSVehicleType* buildSingularType(const std::string& id) const;
+
+
+    /** @brief Duplicates the microsim vehicle type giving the newly created type the given id.
+     *
+     * @param[in] id The new id of the type
+     * @param[in] persistent If true the created vType will be persistent and can be used by several vehicles,
+     *            otherwise it may be removed before simulation end, @see buildSingularType()
      * @return The built vehicle type
      */
-    static MSVehicleType* build(const std::string& id, const MSVehicleType* from);
+    MSVehicleType* duplicateType(const std::string& id, bool persistent) const;
     /// @}
 
 
     /** @brief Returns whether this type belongs to a single vehicle only (was modified)
-     * @return Whether this vehicle type is based on a differen one, and belongs to one vehicle only
+     * @return Whether this vehicle type is based on a different one, and belongs to one vehicle only
      */
-    bool amVehicleSpecific() const {
-        return myOriginalType != 0;
+    bool isVehicleSpecific() const {
+        return myOriginalType != nullptr;
+    }
+
+
+    /** @brief Returns the id of the original vehicle type if this is a vehicle specific type, the id otherwise
+     * @return the original vehicle type id
+     */
+    const std::string& getOriginalID() const {
+        return myOriginalType != nullptr ? myOriginalType->getID() : getID();
     }
 
 
@@ -461,11 +558,28 @@ public:
         return myParameter;
     }
 
+    /** @brief Checks whether vehicle type parameters may be problematic
+     *         (Currently, only the value for the action step length is
+     *         compared with the value for the desired headway time.)
+     */
+    void check();
+
+protected:
+    /// @brief init Rail Visualization Parameters
+    void initRailVisualizationParameters();
 
 private:
     /// @brief the parameter container
     SUMOVTypeParameter myParameter;
 
+    /// @brief the vtypes actionsStepLength in seconds (cached because needed very often)
+    double myCachedActionStepLengthSecs;
+
+    /// @brief Indicator whether the user was already warned once about an action step length
+    ///        larger than the desired time headway.
+    bool myWarnedActionStepLengthTauOnce;
+    bool myWarnedActionStepLengthBallisticOnce;
+
     /// @brief the running index
     const int myIndex;
 
@@ -481,15 +595,8 @@ private:
 
 private:
     /// @brief Invalidated copy constructor
-    MSVehicleType(const MSVehicleType&);
+    MSVehicleType(const MSVehicleType&) = delete;
 
     /// @brief Invalidated assignment operator
-    MSVehicleType& operator=(const MSVehicleType&);
-
+    MSVehicleType& operator=(const MSVehicleType&) = delete;
 };
-
-
-#endif
-
-/****************************************************************************/
-
