@@ -1,41 +1,31 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    Command_SaveTLCoupledLaneDet.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    15 Feb 2004
-/// @version $Id: Command_SaveTLCoupledLaneDet.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Writes e2 state of a link for the time the link has yellow/red
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "Command_SaveTLCoupledLaneDet.h"
 #include <microsim/MSNet.h>
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
 #include <microsim/MSEventControl.h>
 #include <microsim/output/MSDetectorFileOutput.h>
-#include <microsim/MSLinkCont.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/iodevices/OutputDevice.h>
@@ -45,7 +35,7 @@
 // method definitions
 // ===========================================================================
 Command_SaveTLCoupledLaneDet::Command_SaveTLCoupledLaneDet(MSTLLogicControl::TLSLogicVariants& tlls,
-        MSDetectorFileOutput* dtf, SUMOTime begin, OutputDevice& device, MSLink* link)
+        MSDetectorFileOutput* dtf, SUMOTime begin, OutputDevice& device, const MSLink* link)
     : Command_SaveTLCoupledDet(tlls, dtf, begin, device),
       myLink(link), myLastState(LINKSTATE_TL_RED),
       myHadOne(false) {
@@ -77,6 +67,4 @@ Command_SaveTLCoupledLaneDet::execute() {
 }
 
 
-
 /****************************************************************************/
-
