@@ -1,31 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    Command_SaveTLSSwitchStates.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    08.05.2007
-/// @version $Id: Command_SaveTLSSwitchStates.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Writes the switch times of a tls into a file when the tls switches
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "Command_SaveTLSSwitchStates.h"
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
@@ -35,10 +28,6 @@
 #include <utils/common/MsgHandler.h>
 #include <utils/iodevices/OutputDevice.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -46,8 +35,8 @@
 Command_SaveTLSSwitchStates::Command_SaveTLSSwitchStates(const MSTLLogicControl::TLSLogicVariants& logics,
         OutputDevice& od)
     : myOutputDevice(od), myLogics(logics) {
-    MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(this, 0, MSEventControl::ADAPT_AFTER_EXECUTION);
-    myOutputDevice.writeXMLHeader("tls-switch-states");
+    MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(this);
+    myOutputDevice.writeXMLHeader("tlsStates", "tlsstates_file.xsd");
 }
 
 
@@ -59,7 +48,7 @@ SUMOTime
 Command_SaveTLSSwitchStates::execute(SUMOTime currentTime) {
     const std::string& state = myLogics.getActive()->getCurrentPhaseDef().getState();
     if (state != myPreviousState || myLogics.getActive()->getProgramID() != myPreviousProgramID) {
-        myOutputDevice << "    <tlsstate time=\"" << time2string(currentTime)
+        myOutputDevice << "    <tlsState time=\"" << time2string(currentTime)
                        << "\" id=\"" << myLogics.getActive()->getID()
                        << "\" programID=\"" << myLogics.getActive()->getProgramID()
                        << "\" phase=\"" << myLogics.getActive()->getCurrentPhaseIndex()
@@ -71,6 +60,4 @@ Command_SaveTLSSwitchStates::execute(SUMOTime currentTime) {
 }
 
 
-
 /****************************************************************************/
-
