@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    Command_SaveTLSSwitchStates.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    08.05.2007
-/// @version $Id: Command_SaveTLSSwitchStates.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Writes the switch times of a tls into a file when the tls switches
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef Command_SaveTLSSwitchStates_h
-#define Command_SaveTLSSwitchStates_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -71,7 +61,7 @@ public:
     /// @name Derived from Command
     /// @{
 
-    /** @brief Writes the state of the tls if a change occured
+    /** @brief Writes the state of the tls if a change occurred
      *
      * If the state or the active program has changed, the state is written
      *  to the output device.
@@ -107,9 +97,3 @@ private:
     Command_SaveTLSSwitchStates& operator=(const Command_SaveTLSSwitchStates&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
