@@ -1,32 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    Command_SaveTLSSwitches.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    06 Jul 2006
-/// @version $Id: Command_SaveTLSSwitches.cpp 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Writes information about the green durations of a tls
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "Command_SaveTLSSwitches.h"
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
@@ -38,10 +31,6 @@
 #include <utils/common/MsgHandler.h>
 #include <utils/iodevices/OutputDevice.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -49,8 +38,8 @@
 Command_SaveTLSSwitches::Command_SaveTLSSwitches(const MSTLLogicControl::TLSLogicVariants& logics,
         OutputDevice& od)
     : myOutputDevice(od), myLogics(logics) {
-    MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(this, 0, MSEventControl::ADAPT_AFTER_EXECUTION);
-    myOutputDevice.writeXMLHeader("tls-switches");
+    MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(this);
+    myOutputDevice.writeXMLHeader("tlsSwitches", "tlsswitches_file.xsd");
 }
 
 
@@ -63,7 +52,7 @@ Command_SaveTLSSwitches::execute(SUMOTime currentTime) {
     MSTrafficLightLogic* light = myLogics.getActive();
     const MSTrafficLightLogic::LinkVectorVector& links = light->getLinks();
     const std::string& state = light->getCurrentPhaseDef().getState();
-    for (unsigned int i = 0; i < (unsigned int) links.size(); i++) {
+    for (int i = 0; i < (int) links.size(); i++) {
         if (state[i] == LINKSTATE_TL_GREEN_MAJOR || state[i] == LINKSTATE_TL_GREEN_MINOR) {
             if (myPreviousLinkStates.find(i) == myPreviousLinkStates.end()) {
                 // was not saved before
@@ -96,6 +85,4 @@ Command_SaveTLSSwitches::execute(SUMOTime currentTime) {
 }
 
 
-
 /****************************************************************************/
-
