@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSCFModel_Daniel1.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Tue, 05 Jun 2012
-/// @version $Id: MSCFModel_Daniel1.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // The original Krauss (1998) car-following model and parameter
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSCFModel_Daniel1_h
-#define	MSCFModel_Daniel1_h
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include "MSCFModel.h"
 #include <utils/xml/SUMOXMLDefinitions.h>
@@ -44,13 +35,9 @@
 class MSCFModel_Daniel1 : public MSCFModel {
 public:
     /** @brief Constructor
-     * @param[in] accel The maximum acceleration
-     * @param[in] decel The maximum deceleration
-     * @param[in] dawdle The driver imperfection
-     * @param[in] tau The driver's reaction time
+     *  @param[in] vtype the type for which this model is built and also the parameter object to configure this model
      */
-    MSCFModel_Daniel1(const MSVehicleType* vtype, SUMOReal accel, SUMOReal decel, SUMOReal dawdle, SUMOReal headwayTime,
-                      SUMOReal tmp1, SUMOReal tmp2, SUMOReal tmp3, SUMOReal tmp4, SUMOReal tmp5);
+    MSCFModel_Daniel1(const MSVehicleType* vtype);
 
 
     /// @brief Destructor
@@ -65,7 +52,7 @@ public:
      * @param[in] vPos The possible velocity
      * @return The velocity after applying interactions with stops and lane change model influences
      */
-    SUMOReal moveHelper(MSVehicle* const veh, SUMOReal vPos) const;
+    double finalizeSpeed(MSVehicle* const veh, double vPos) const;
 
 
     /** @brief Computes the vehicle's safe speed (no dawdling)
@@ -76,7 +63,7 @@ public:
      * @return EGO's safe speed
      * @see MSCFModel::ffeV
      */
-    virtual SUMOReal followSpeed(const MSVehicle* const veh, SUMOReal speed, SUMOReal gap2pred, SUMOReal predSpeed, SUMOReal predMaxDecel) const;
+    virtual double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed, double predMaxDecel, const MSVehicle* const pred = 0) const;
 
 
     /** @brief Computes the vehicle's safe speed for approaching a non-moving obstacle (no dawdling)
@@ -86,7 +73,7 @@ public:
      * @see MSCFModel::ffeS
      * @todo generic Interface, models can call for the values they need
      */
-    virtual SUMOReal stopSpeed(const MSVehicle* const veh, const SUMOReal speed, SUMOReal gap2pred) const;
+    virtual double stopSpeed(const MSVehicle* const veh, const double speed, double gap2pred, double decel) const;
 
 
     /** @brief Returns the model's name
@@ -101,7 +88,7 @@ public:
     /** @brief Get the driver's imperfection
      * @return The imperfection of drivers of this class
      */
-    SUMOReal getImperfection() const {
+    double getImperfection() const {
         return myDawdle;
     }
     /// @}
@@ -113,7 +100,7 @@ public:
     /** @brief Sets a new value for maximum deceleration [m/s^2]
      * @param[in] accel The new deceleration in m/s^2
      */
-    void setMaxDecel(SUMOReal decel) {
+    void setMaxDecel(double decel) {
         myDecel = decel;
         myTauDecel = myDecel * myHeadwayTime;
     }
@@ -122,7 +109,7 @@ public:
     /** @brief Sets a new value for driver imperfection
      * @param[in] accel The new driver imperfection
      */
-    void setImperfection(SUMOReal imperfection) {
+    void setImperfection(double imperfection) {
         myDawdle = imperfection;
     }
 
@@ -130,7 +117,7 @@ public:
     /** @brief Sets a new value for driver reaction time [s]
      * @param[in] headwayTime The new driver reaction time (in s)
      */
-    void setHeadwayTime(SUMOReal headwayTime) {
+    void setHeadwayTime(double headwayTime) {
         myHeadwayTime = headwayTime;
         myTauDecel = myDecel * headwayTime;
     }
@@ -149,26 +136,25 @@ private:
      * @param[in] predSpeed The LEADER's speed
      * @return the safe velocity
      */
-    virtual SUMOReal _vsafe(SUMOReal gap, SUMOReal predSpeed) const;
+    virtual double _vsafe(double gap, double predSpeed) const;
 
 
     /** @brief Applies driver imperfection (dawdling / sigma)
      * @param[in] speed The speed with no dawdling
      * @return The speed after dawdling
      */
-    virtual SUMOReal dawdle(SUMOReal speed) const;
+    virtual double dawdle(double speed, std::mt19937* rng) const;
 
 protected:
     /// @brief The vehicle's dawdle-parameter. 0 for no dawdling, 1 for max.
-    SUMOReal myDawdle;
+    double myDawdle;
 
     /// @brief The precomputed value for myDecel*myTau
-    SUMOReal myTauDecel;
+    double myTauDecel;
 
     /// @brief temporary (testing) parameter
-    SUMOReal myTmp1, myTmp2, myTmp3, myTmp4, myTmp5;
+    double myTmp1, myTmp2, myTmp3, myTmp4, myTmp5;
 
 };
 
-#endif	/* MSCFModel_Daniel1_H */
 
