@@ -1,36 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSCFModel_KraussOrig1.h
 /// @author  Tobias Mayer
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 28 Jul 2009
-/// @version $Id: MSCFModel_KraussOrig1.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // The original Krauss (1998) car-following model and parameter
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSCFModel_KraussOrig1_h
-#define	MSCFModel_KraussOrig1_h
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include "MSCFModel.h"
 #include <utils/xml/SUMOXMLDefinitions.h>
@@ -46,12 +37,9 @@
 class MSCFModel_KraussOrig1 : public MSCFModel {
 public:
     /** @brief Constructor
-     * @param[in] accel The maximum acceleration
-     * @param[in] decel The maximum deceleration
-     * @param[in] dawdle The driver imperfection
-     * @param[in] tau The driver's reaction time
+     *  @param[in] vtype the type for which this model is built and also the parameter object to configure this model
      */
-    MSCFModel_KraussOrig1(const MSVehicleType* vtype, SUMOReal accel, SUMOReal decel, SUMOReal dawdle, SUMOReal headwayTime);
+    MSCFModel_KraussOrig1(const MSVehicleType* vtype);
 
 
     /// @brief Destructor
@@ -60,14 +48,8 @@ public:
 
     /// @name Implementations of the MSCFModel interface
     /// @{
-
-    /** @brief Applies interaction with stops and lane changing model influences
-     * @param[in] veh The ego vehicle
-     * @param[in] vPos The possible velocity
-     * @return The velocity after applying interactions with stops and lane change model influences
-     */
-    SUMOReal moveHelper(MSVehicle* const veh, SUMOReal vPos) const;
-
+    /// @brief apply dawdling
+    double patchSpeedBeforeLC(const MSVehicle* veh, double vMin, double vMax) const;
 
     /** @brief Computes the vehicle's safe speed (no dawdling)
      * @param[in] veh The vehicle (EGO)
@@ -77,7 +59,7 @@ public:
      * @return EGO's safe speed
      * @see MSCFModel::ffeV
      */
-    SUMOReal followSpeed(const MSVehicle* const veh, SUMOReal speed, SUMOReal gap2pred, SUMOReal predSpeed, SUMOReal predMaxDecel) const;
+    double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed, double predMaxDecel, const MSVehicle* const pred = 0) const;
 
 
     /** @brief Computes the vehicle's safe speed for approaching a non-moving obstacle (no dawdling)
@@ -87,22 +69,7 @@ public:
      * @see MSCFModel::ffeS
      * @todo generic Interface, models can call for the values they need
      */
-    virtual SUMOReal stopSpeed(const MSVehicle* const veh, const SUMOReal speed, SUMOReal gap2pred) const;
-
-
-    /** @brief Computes the vehicle's safe speed (no dawdling)
-     * This method is used during the insertion stage. Whereas the method
-     * followSpeed returns the desired speed which may be lower than the safe
-     * speed, this method only considers safety constraints
-     *
-     * Returns the velocity of the vehicle in dependence to the vehicle's and its leader's values and the distance between them.
-     * @param[in] veh The vehicle (EGO)
-     * @param[in] speed The vehicle's speed
-     * @param[in] gap2pred The (netto) distance to the LEADER
-     * @param[in] predSpeed The speed of LEADER
-     * @return EGO's safe speed
-     */
-    SUMOReal insertionFollowSpeed(const MSVehicle* const veh, SUMOReal speed, SUMOReal gap2pred, SUMOReal predSpeed, SUMOReal predMaxDecel) const;
+    virtual double stopSpeed(const MSVehicle* const veh, const double speed, double gap2pred, double decel) const;
 
 
     /** @brief Returns the model's name
@@ -117,7 +84,7 @@ public:
     /** @brief Get the driver's imperfection
      * @return The imperfection of drivers of this class
      */
-    SUMOReal getImperfection() const {
+    double getImperfection() const {
         return myDawdle;
     }
     /// @}
@@ -129,7 +96,7 @@ public:
     /** @brief Sets a new value for maximum deceleration [m/s^2]
      * @param[in] accel The new deceleration in m/s^2
      */
-    void setMaxDecel(SUMOReal decel) {
+    void setMaxDecel(double decel) {
         myDecel = decel;
         myTauDecel = myDecel * myHeadwayTime;
     }
@@ -138,15 +105,15 @@ public:
     /** @brief Sets a new value for driver imperfection
      * @param[in] accel The new driver imperfection
      */
-    void setImperfection(SUMOReal imperfection) {
+    void setImperfection(double imperfection) {
         myDawdle = imperfection;
     }
 
 
-    /** @brief Sets a new value for driver reaction time [s]
-     * @param[in] headwayTime The new driver reaction time (in s)
+    /** @brief Sets a new value for desired headway [s]
+     * @param[in] headwayTime The new desired headway (in s)
      */
-    void setHeadwayTime(SUMOReal headwayTime) {
+    void setHeadwayTime(double headwayTime) {
         myHeadwayTime = headwayTime;
         myTauDecel = myDecel * headwayTime;
     }
@@ -159,29 +126,28 @@ public:
      */
     virtual MSCFModel* duplicate(const MSVehicleType* vtype) const;
 
-private:
+protected:
     /** @brief Returns the "safe" velocity
      * @param[in] gap2pred The (netto) distance to the LEADER
      * @param[in] predSpeed The LEADER's speed
      * @param[in] predMaxDecel The LEADER's maximum deceleration
      * @return the safe velocity
      */
-    virtual SUMOReal vsafe(SUMOReal gap, SUMOReal predSpeed, SUMOReal predMaxDecel) const;
+    virtual double vsafe(double gap, double predSpeed, double predMaxDecel) const;
 
 
     /** @brief Applies driver imperfection (dawdling / sigma)
      * @param[in] speed The speed with no dawdling
      * @return The speed after dawdling
      */
-    virtual SUMOReal dawdle(SUMOReal speed) const;
+    virtual double dawdle(double speed, std::mt19937* rng) const;
 
 protected:
     /// @brief The vehicle's dawdle-parameter. 0 for no dawdling, 1 for max.
-    SUMOReal myDawdle;
+    double myDawdle;
 
     /// @brief The precomputed value for myDecel*myTau
-    SUMOReal myTauDecel;
+    double myTauDecel;
 };
 
-#endif	/* MSCFModel_KraussOrig1_H */
 
