@@ -1,36 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSCFModel_KraussPS.h
 /// @author  Tobias Mayer
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 28 Jul 2009
-/// @version $Id: MSCFModel_KraussPS.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Krauss car-following model, changing accel and speed by slope
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSCFModel_KraussPS_h
-#define MSCFModel_KraussPS_h
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include "MSCFModel_Krauss.h"
 #include <utils/xml/SUMOXMLDefinitions.h>
@@ -47,12 +38,9 @@
 class MSCFModel_KraussPS : public MSCFModel_Krauss {
 public:
     /** @brief Constructor
-     * @param[in] accel The maximum acceleration
-     * @param[in] decel The maximum deceleration
-     * @param[in] dawdle The driver imperfection
-     * @param[in] headwayTime The driver's reaction time
+     *  @param[in] vtype the type for which this model is built and also the parameter object to configure this model
      */
-    MSCFModel_KraussPS(const MSVehicleType* vtype, SUMOReal accel, SUMOReal decel, SUMOReal dawdle, SUMOReal headwayTime);
+    MSCFModel_KraussPS(const MSVehicleType* vtype);
 
 
     /// @brief Destructor
@@ -74,7 +62,7 @@ public:
      * @param[in] speed The vehicle itself, for obtaining other values
      * @return The maximum possible speed for the next step
      */
-    SUMOReal maxNextSpeed(SUMOReal speed, const MSVehicle* const veh) const;
+    double maxNextSpeed(double speed, const MSVehicle* const veh) const;
 
 
     /** @brief Returns the model's name
@@ -96,5 +84,4 @@ public:
 
 };
 
-#endif /* MSCFModel_KraussPS_H */
 
