@@ -1,24 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSCFModel_Rail.h
 /// @author  Gregor L\"ammel
 /// @date    Tue, 08 Feb 2017
-/// @version $Id: MSCFModel_Rail.h 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // <description missing>
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef SUMO_MSCFMODEL_RAIL_H
-#define SUMO_MSCFMODEL_RAIL_H
+#pragma once
+#include <config.h>
 
 
 #include "MSCFModel.h"
@@ -28,18 +28,18 @@
 class MSCFModel_Rail : public MSCFModel {
 
 public:
+    /** @brief Constructor
+     *  @param[in] vtype the type for which this model is built and also the parameter object to configure this model
+     */
+    MSCFModel_Rail(const MSVehicleType* vtype);
 
     double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed,
-                       double predMaxDecel) const;
+                       double predMaxDecel, const MSVehicle* const pred = 0) const;
 
     virtual int getModelID() const;
 
     virtual MSCFModel* duplicate(const MSVehicleType* vtype) const;
 
-    /** @brief Constructor
-     * @param[in] trainType The train type
-     */
-    MSCFModel_Rail(const MSVehicleType* vtype, std::string trainType);
 
 
     virtual ~MSCFModel_Rail();
@@ -48,11 +48,13 @@ public:
 
     virtual double minNextSpeed(double speed, const MSVehicle* const veh) const;
 
+    virtual double minNextSpeedEmergency(double speed, const MSVehicle* const veh = 0) const;
+
     double getSpeedAfterMaxDecel(double v) const;
 
     MSCFModel::VehicleVariables* createVehicleVariables() const;
 
-    double moveHelper(MSVehicle* const veh, double vPos) const;
+    double finalizeSpeed(MSVehicle* const veh, double vPos) const;
 
     double freeSpeed(const MSVehicle* const veh, double speed, double seen, double maxSpeed,
                      const bool onInsertion) const;
@@ -78,7 +80,7 @@ private:
 
 
 public:
-    double stopSpeed(const MSVehicle* const veh, const double speed, double gap) const;
+    double stopSpeed(const MSVehicle* const veh, const double speed, double gap, double decel) const;
 
 //    class VehicleVariables : public MSCFModel::VehicleVariables {
 //
@@ -733,4 +735,3 @@ private:
 };
 
 
-#endif //SUMO_MSCFMODEL_RAIL_H
