@@ -1,35 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSCFModel_SmartSK.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Peter Wagner
 /// @date    Tue, 05 Jun 2012
-/// @version $Id: MSCFModel_SmartSK.h 23699 2017-03-29 07:16:27Z namdre $
 ///
 // A smarter SK
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSCFModel_SmartSK_h
-#define MSCFModel_SmartSK_h
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include "MSCFModel.h"
 #include <utils/xml/SUMOXMLDefinitions.h>
@@ -45,14 +36,9 @@
 class MSCFModel_SmartSK : public MSCFModel {
 public:
     /** @brief Constructor
-     * @param[in] accel The maximum acceleration
-     * @param[in] decel The maximum deceleration
-     * @param[in] emergencyDecel The maximum emergency deceleration
-     * @param[in] dawdle The driver imperfection
-     * @param[in] tau The driver's reaction time
+     *  @param[in] vtype the type for which this model is built and also the parameter object to configure this model
      */
-    MSCFModel_SmartSK(const MSVehicleType* vtype, double accel, double decel, double emergencyDecel, double dawdle, double headwayTime,
-                      double tmp1, double tmp2, double tmp3, double tmp4, double tmp5);
+    MSCFModel_SmartSK(const MSVehicleType* vtype);
 
 
     /// @brief Destructor
@@ -67,7 +53,7 @@ public:
      * @param[in] vPos The possible velocity
      * @return The velocity after applying interactions with stops and lane change model influences
      */
-    double moveHelper(MSVehicle* const veh, double vPos) const;
+    double finalizeSpeed(MSVehicle* const veh, double vPos) const;
 
 
     /** @brief Computes the vehicle's safe speed (no dawdling)
@@ -78,7 +64,7 @@ public:
      * @return EGO's safe speed
      * @see MSCFModel::ffeV
      */
-    virtual double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed, double predMaxDecel) const;
+    virtual double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed, double predMaxDecel, const MSVehicle* const pred = 0) const;
 
 
     /** @brief Computes the vehicle's safe speed for approaching a non-moving obstacle (no dawdling)
@@ -88,7 +74,7 @@ public:
      * @see MSCFModel::ffeS
      * @todo generic Interface, models can call for the values they need
      */
-    virtual double stopSpeed(const MSVehicle* const veh, const double speed, double gap2pred) const;
+    virtual double stopSpeed(const MSVehicle* const veh, const double speed, double gap2pred, double decel) const;
 
 
     /** @brief Returns the model's name
@@ -129,8 +115,8 @@ public:
     }
 
 
-    /** @brief Sets a new value for driver reaction time [s]
-     * @param[in] headwayTime The new driver reaction time (in s)
+    /** @brief Sets a new value for desired headway [s]
+     * @param[in] headwayTime The new desired headway (in s)
      */
     void setHeadwayTime(double headwayTime) {
         myHeadwayTime = headwayTime;
@@ -138,6 +124,8 @@ public:
     }
     /// @}
 
+    /// @brief apply dawdling
+    double patchSpeedBeforeLC(const MSVehicle* veh, double vMin, double vMax) const;
 
     /** @brief Duplicates the car-following model
      * @param[in] vtype The vehicle type this model belongs to (1:1)
@@ -158,13 +146,13 @@ private:
      * @param[in] speed The speed with no dawdling
      * @return The speed after dawdling
      */
-    virtual double dawdle(double speed) const;
+    virtual double dawdle(double speed, std::mt19937* rng) const;
 
     virtual void updateMyHeadway(const MSVehicle* const veh) const {
         // this is the point were the preferred headway changes slowly:
         SSKVehicleVariables* vars = (SSKVehicleVariables*)veh->getCarFollowVariables();
         double tTau = vars->myHeadway;
-        tTau = tTau + (myHeadwayTime - tTau) * myTmp2 + myTmp3 * tTau * RandHelper::rand(double(-1.0), double(1.0));
+        tTau = tTau + (myHeadwayTime - tTau) * myTmp2 + myTmp3 * tTau * RandHelper::rand(double(-1.0), double(1.0), veh->getRNG());
         if (tTau < TS) { // this ensures the SK safety condition
             tTau = TS;
         }
@@ -204,5 +192,4 @@ protected:
 
 };
 
-#endif /* MSCFModel_SmartSK_H */
 
