@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSCFModel_Wiedemann.h
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    June 2011
-/// @version $Id: MSCFModel_Wiedemann.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The psycho-physical model of Wiedemann
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSCFModel_Wiedemann_H
-#define MSCFModel_Wiedemann_H
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include "MSCFModel.h"
 #include <microsim/MSLane.h>
@@ -41,25 +32,17 @@
 // class definitions
 // ===========================================================================
 /** @class MSCFModel_Wiedemann
- * @brief The Intellignet Driver Model (IDM) car-following model
+ * @brief The Wiedemann Model car-following model
  * @see MSCFModel
  */
+// XXX: which Wiedemann is this? There are several versions... Below it is stated that it is modified it with Krauss vsafe... (Leo)
 class MSCFModel_Wiedemann : public MSCFModel {
 public:
 
     /** @brief Constructor
-     *
-     * @param[in] security The security parameter in [0,1] (dimensionless)
-     * @param[in] estimation The estimation capability parameter in [0,1] (dimensionless)
-     * @param[in] accel The maximum acceleration
-     * @param[in] decel The maximum deceleration
-     *
-     * @note other parameters of the wiedemann model:
-     * - speed: included in MSVehicleType
+     *  @param[in] vtype the type for which this model is built and also the parameter object to configure this model
      */
-    MSCFModel_Wiedemann(const MSVehicleType* vtype,
-                        SUMOReal accel, SUMOReal decel,
-                        SUMOReal security, SUMOReal estimation);
+    MSCFModel_Wiedemann(const MSVehicleType* vtype);
 
 
     /// @brief Destructor
@@ -74,7 +57,7 @@ public:
      * @param[in] vPos The possible velocity
      * @return The velocity after applying interactions with stops and lane change model influences
      */
-    SUMOReal moveHelper(MSVehicle* const veh, SUMOReal vPos) const;
+    double finalizeSpeed(MSVehicle* const veh, double vPos) const;
 
 
     /** @brief Computes the vehicle's safe speed (no dawdling)
@@ -85,7 +68,7 @@ public:
      * @return EGO's safe speed
      * @see MSCFModel::ffeV
      */
-    SUMOReal followSpeed(const MSVehicle* const veh, SUMOReal speed, SUMOReal gap2pred, SUMOReal predSpeed, SUMOReal predMaxDecel) const;
+    double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed, double predMaxDecel, const MSVehicle* const pred = 0) const;
 
 
     /** @brief Computes the vehicle's safe speed for approaching a non-moving obstacle (no dawdling)
@@ -95,7 +78,7 @@ public:
      * @see MSCFModel::ffeS
      * @todo generic Interface, models can call for the values they need
      */
-    SUMOReal stopSpeed(const MSVehicle* const veh, const SUMOReal speed, SUMOReal gap) const;
+    double stopSpeed(const MSVehicle* const veh, const double speed, double gap, double decel) const;
 
 
     /** @brief Returns the maximum gap at which an interaction between both vehicles occurs
@@ -107,8 +90,16 @@ public:
      * @todo evaluate signature
      * @see MSCFModel::interactionGap
      */
-    SUMOReal interactionGap(const MSVehicle* const , SUMOReal vL) const;
+    double interactionGap(const MSVehicle* const, double vL) const;
 
+    /** @brief Returns the minimum gap to reserve if the leader is braking at maximum (>=0)
+     * @param[in] veh The vehicle itself, for obtaining other values
+     * @param[in] pred The leader vehicle, for obtaining other values
+     * @param[in] speed EGO's speed
+     * @param[in] leaderSpeed LEADER's speed
+     * @param[in] leaderMaxDecel LEADER's max. deceleration rate
+     */
+    double getSecureGap(const MSVehicle* const veh, const MSVehicle* const pred, const double speed, const double leaderSpeed, const double leaderMaxDecel) const;
 
     /** @brief Returns the model's name
      * @return The model's name
@@ -137,7 +128,7 @@ private:
     public:
         VehicleVariables() : accelSign(1) {}
         /// @brief state variable for remembering the drift direction
-        SUMOReal accelSign;
+        double accelSign;
     };
 
 
@@ -145,14 +136,14 @@ private:
     /* @brief the main enty point for the speed computation
      * @param[in] gap The netto gap (front bumper of ego to back bumper of leader)
      */
-    SUMOReal _v(const MSVehicle* veh, SUMOReal predSpeed, SUMOReal gap) const;
+    double _v(const MSVehicle* veh, double predSpeed, double gap) const;
 
     /// @name acceleration based on the 'driving regime'
     /// @{
-    SUMOReal fullspeed(SUMOReal v, SUMOReal vpref, SUMOReal dx, SUMOReal bx) const; // also 'WUNSCH'
-    SUMOReal following(SUMOReal sign) const; // also 'FOLGEN'
-    SUMOReal approaching(SUMOReal dv, SUMOReal dx, SUMOReal bx) const;  // also 'BREMSBX'
-    SUMOReal emergency(SUMOReal dv, SUMOReal dx) const; // also 'BREMSAX'
+    double fullspeed(double v, double vpref, double dx, double bx) const; // also 'WUNSCH'
+    double following(double sign) const; // also 'FOLGEN'
+    double approaching(double dv, double dx, double abx) const;  // also 'BREMSBX'
+    double emergency(double dv, double dx) const; // also 'BREMSAX'
     /// @}
 
 private:
@@ -160,31 +151,26 @@ private:
     /// @{
 
     /// @brief The driver's security parameter // also 'ZF1'
-    const SUMOReal mySecurity;
+    const double mySecurity;
 
     /// @brief The driver's estimation parameter // also 'ZF2'
-    const SUMOReal myEstimation;
+    const double myEstimation;
 
-    /// @brief front-bumper to front-bumper distance
-    const SUMOReal myAX;
+    /// @brief the minimum front-bumper to front-bumper distance when standing
+    const double myAX;
 
     /// @brief perception threshold modifier
-    const SUMOReal myCX;
+    const double myCX;
 
-    /// @brief The vehicle's minimum acceleration [m/s^2]
-    const SUMOReal myMinAccel;
+    /// @brief The vehicle's minimum acceleration [m/s^2] // also b_null
+    const double myMinAccel;
 
     /// @brief free-flow distance in m
-    static const SUMOReal D_MAX;
+    static const double D_MAX;
     /// @}
 
-    /// @brief vsafe from krauss since Wiedemann is deficient at approaching
-    // standing obstacles (see MSCFModel_Krauss::_vsafe)
-    SUMOReal krauss_vsafe(SUMOReal gap, SUMOReal predSpeed) const;
-
 private:
     /// @brief Invalidated assignment operator
     MSCFModel_Wiedemann& operator=(const MSCFModel_Wiedemann& s);
 };
 
-#endif /* MSCFModel_Wiedemann_H */
