@@ -1,33 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSDevice_BTreceiver.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    14.08.2013
-/// @version $Id: MSDevice_BTreceiver.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
-// A BT sender
+// A BT Receiver
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <utils/common/MsgHandler.h>
 #include <utils/options/OptionsCont.h>
@@ -39,22 +31,19 @@
 #include <microsim/MSLane.h>
 #include <microsim/MSEdge.h>
 #include <microsim/MSVehicle.h>
+#include <microsim/MSEventControl.h>
 #include "MSDevice_Tripinfo.h"
 #include "MSDevice_BTreceiver.h"
 #include "MSDevice_BTsender.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static members
 // ===========================================================================
 bool MSDevice_BTreceiver::myWasInitialised = false;
-SUMOReal MSDevice_BTreceiver::myRange = -1.;
-SUMOReal MSDevice_BTreceiver::myOffTime = -1.;
-MTRand MSDevice_BTreceiver::sRecognitionRNG;
+double MSDevice_BTreceiver::myRange = -1.;
+double MSDevice_BTreceiver::myOffTime = -1.;
+std::mt19937 MSDevice_BTreceiver::sRecognitionRNG;
 std::map<std::string, MSDevice_BTreceiver::VehicleInformation*> MSDevice_BTreceiver::sVehicles;
 
 
@@ -82,9 +71,9 @@ MSDevice_BTreceiver::insertOptions(OptionsCont& oc) {
 
 
 void
-MSDevice_BTreceiver::buildVehicleDevices(SUMOVehicle& v, std::vector<MSDevice*>& into) {
+MSDevice_BTreceiver::buildVehicleDevices(SUMOVehicle& v, std::vector<MSVehicleDevice*>& into) {
     OptionsCont& oc = OptionsCont::getOptions();
-    if (equippedByDefaultAssignmentOptions(oc, "btreceiver", v)) {
+    if (equippedByDefaultAssignmentOptions(oc, "btreceiver", v, false)) {
         MSDevice_BTreceiver* device = new MSDevice_BTreceiver(v, "btreceiver_" + v.getID());
         into.push_back(device);
         if (!myWasInitialised) {
@@ -102,7 +91,7 @@ MSDevice_BTreceiver::buildVehicleDevices(SUMOVehicle& v, std::vector<MSDevice*>&
 // MSDevice_BTreceiver::BTreceiverUpdate-methods
 // ---------------------------------------------------------------------------
 MSDevice_BTreceiver::BTreceiverUpdate::BTreceiverUpdate() {
-    MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(this, 0, MSEventControl::ADAPT_AFTER_EXECUTION);
+    MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(this);
 }
 
 
@@ -143,17 +132,17 @@ MSDevice_BTreceiver::BTreceiverUpdate::execute(SUMOTime /*currentTime*/) {
         b.grow(vi->range);
         const float cmin[2] = {(float) b.xmin(), (float) b.ymin()};
         const float cmax[2] = {(float) b.xmax(), (float) b.ymax()};
-        std::set<std::string> surroundingVehicles;
+        std::set<const Named*> surroundingVehicles;
         Named::StoringVisitor sv(surroundingVehicles);
         rt.Search(cmin, cmax, sv);
 
         // loop over surrounding vehicles, check visibility status
-        for (std::set<std::string>::const_iterator j = surroundingVehicles.begin(); j != surroundingVehicles.end(); ++j) {
-            if ((*i).first == *j) {
+        for (const Named* vehicle : surroundingVehicles) {
+            if ((*i).first == vehicle->getID()) {
                 // seeing oneself? skip
                 continue;
             }
-            updateVisibility(*vi, *MSDevice_BTsender::sVehicles.find(*j)->second);
+            updateVisibility(*vi, *MSDevice_BTsender::sVehicles.find(vehicle->getID())->second);
         }
 
         if (vi->haveArrived) {
@@ -205,7 +194,7 @@ MSDevice_BTreceiver::BTreceiverUpdate::updateVisibility(MSDevice_BTreceiver::Veh
     const Position receiverDelta = receiverData.position - oldReceiverPosition;
     const Position translatedSender = senderData.position - receiverDelta;
     // find crossing points
-    std::vector<SUMOReal> intersections;
+    std::vector<double> intersections;
     GeomHelper::findLineCircleIntersections(oldReceiverPosition, receiver.range, oldSenderPosition, translatedSender, intersections);
     switch (intersections.size()) {
         case 0:
@@ -262,7 +251,7 @@ MSDevice_BTreceiver::BTreceiverUpdate::updateVisibility(MSDevice_BTreceiver::Veh
 
 
 void
-MSDevice_BTreceiver::BTreceiverUpdate::enterRange(SUMOReal atOffset, const MSDevice_BTsender::VehicleState& receiverState,
+MSDevice_BTreceiver::BTreceiverUpdate::enterRange(double atOffset, const MSDevice_BTsender::VehicleState& receiverState,
         const std::string& senderID, const MSDevice_BTsender::VehicleState& senderState,
         std::map<std::string, SeenDevice*>& currentlySeen) {
     MeetingPoint mp(SIMTIME + atOffset, receiverState, senderState);
@@ -275,7 +264,7 @@ MSDevice_BTreceiver::BTreceiverUpdate::enterRange(SUMOReal atOffset, const MSDev
 void
 MSDevice_BTreceiver::BTreceiverUpdate::leaveRange(VehicleInformation& receiverInfo, const MSDevice_BTsender::VehicleState& receiverState,
         MSDevice_BTsender::VehicleInformation& senderInfo, const MSDevice_BTsender::VehicleState& senderState,
-        SUMOReal tOffset) {
+        double tOffset) {
     std::map<std::string, SeenDevice*>::iterator i = receiverInfo.currentlySeen.find(senderInfo.getID());
     // check whether the other was recognized
     addRecognitionPoint(SIMTIME + tOffset, receiverState, senderState, i->second);
@@ -292,24 +281,24 @@ MSDevice_BTreceiver::BTreceiverUpdate::leaveRange(VehicleInformation& receiverIn
 }
 
 
-SUMOReal
+double
 MSDevice_BTreceiver::inquiryDelaySlots(const int backoffLimit) {
-    const int phaseOffset = sRecognitionRNG.randInt(2047);
-    const bool interlaced = sRecognitionRNG.rand() < 0.7;
-    const SUMOReal delaySlots = sRecognitionRNG.rand() * 15;
-    const int backoff = sRecognitionRNG.randInt(backoffLimit);
+    const int phaseOffset = RandHelper::rand(2047, &sRecognitionRNG);
+    const bool interlaced = RandHelper::rand(&sRecognitionRNG) < 0.7;
+    const double delaySlots = RandHelper::rand(&sRecognitionRNG) * 15;
+    const int backoff = RandHelper::rand(backoffLimit, &sRecognitionRNG);
     if (interlaced) {
-        return sRecognitionRNG.rand() * 31 + backoff;
+        return RandHelper::rand(&sRecognitionRNG) * 31 + backoff;
     }
-    if (sRecognitionRNG.randInt(31) < 16) {
+    if (RandHelper::rand(31, &sRecognitionRNG) < 16) {
         // correct train for f0
         return delaySlots + backoff;
     }
-    if (sRecognitionRNG.randInt(30) < 16) {
+    if (RandHelper::rand(30, &sRecognitionRNG) < 16) {
         // correct train for f1
         return 2048 - phaseOffset + delaySlots + backoff;
     }
-    if (sRecognitionRNG.randInt(29) < 16) {
+    if (RandHelper::rand(29, &sRecognitionRNG) < 16) {
         // f2 is in train A but has overlap with both trains
         if (2 * 2048 - phaseOffset + backoff < 4096) {
             return 2 * 2048 - phaseOffset + delaySlots + backoff;
@@ -322,7 +311,7 @@ MSDevice_BTreceiver::inquiryDelaySlots(const int backoffLimit) {
 
 
 void
-MSDevice_BTreceiver::BTreceiverUpdate::addRecognitionPoint(const SUMOReal tEnd, const MSDevice_BTsender::VehicleState& receiverState,
+MSDevice_BTreceiver::BTreceiverUpdate::addRecognitionPoint(const double tEnd, const MSDevice_BTsender::VehicleState& receiverState,
         const MSDevice_BTsender::VehicleState& senderState,
         SeenDevice* senderDevice) const {
     if (senderDevice->nextView == -1.) {
@@ -384,7 +373,7 @@ MSDevice_BTreceiver::BTreceiverUpdate::writeOutput(const std::string& id, const
 // MSDevice_BTreceiver-methods
 // ---------------------------------------------------------------------------
 MSDevice_BTreceiver::MSDevice_BTreceiver(SUMOVehicle& holder, const std::string& id)
-    : MSDevice(holder, id) {
+    : MSVehicleDevice(holder, id) {
 }
 
 
@@ -393,7 +382,7 @@ MSDevice_BTreceiver::~MSDevice_BTreceiver() {
 
 
 bool
-MSDevice_BTreceiver::notifyEnter(SUMOVehicle& veh, Notification reason) {
+MSDevice_BTreceiver::notifyEnter(SUMOTrafficObject& veh, Notification reason, const MSLane* /* enteredLane */) {
     if (reason == MSMoveReminder::NOTIFICATION_DEPARTED && sVehicles.find(veh.getID()) == sVehicles.end()) {
         sVehicles[veh.getID()] = new VehicleInformation(veh.getID(), myRange);
         sVehicles[veh.getID()]->route.push_back(veh.getEdge());
@@ -404,26 +393,28 @@ MSDevice_BTreceiver::notifyEnter(SUMOVehicle& veh, Notification reason) {
     if (reason == MSMoveReminder::NOTIFICATION_TELEPORT || reason == MSMoveReminder::NOTIFICATION_JUNCTION) {
         sVehicles[veh.getID()]->route.push_back(veh.getEdge());
     }
-    const MSVehicle& v = static_cast<MSVehicle&>(veh);
-    sVehicles[veh.getID()]->updates.push_back(MSDevice_BTsender::VehicleState(veh.getSpeed(), veh.getPosition(), v.getLane()->getID(), veh.getPositionOnLane(), v.getRoutePosition()));
+    const std::string location = MSGlobals::gUseMesoSim ? veh.getEdge()->getID() : static_cast<MSVehicle&>(veh).getLane()->getID();
+    const MSBaseVehicle& v = static_cast<MSBaseVehicle&>(veh);
+    sVehicles[veh.getID()]->updates.push_back(MSDevice_BTsender::VehicleState(veh.getSpeed(), veh.getPosition(), location, veh.getPositionOnLane(), v.getRoutePosition()));
     return true;
 }
 
 
 bool
-MSDevice_BTreceiver::notifyMove(SUMOVehicle& veh, SUMOReal /* oldPos */, SUMOReal newPos, SUMOReal newSpeed) {
+MSDevice_BTreceiver::notifyMove(SUMOTrafficObject& veh, double /* oldPos */, double newPos, double newSpeed) {
     if (sVehicles.find(veh.getID()) == sVehicles.end()) {
         WRITE_WARNING("btreceiver: Can not update position of vehicle '" + veh.getID() + "' which is not on the road.");
         return true;
     }
-    const MSVehicle& v = static_cast<MSVehicle&>(veh);
-    sVehicles[veh.getID()]->updates.push_back(MSDevice_BTsender::VehicleState(newSpeed, veh.getPosition(), v.getLane()->getID(), newPos, v.getRoutePosition()));
+    const std::string location = MSGlobals::gUseMesoSim ? veh.getEdge()->getID() : static_cast<MSVehicle&>(veh).getLane()->getID();
+    const MSBaseVehicle& v = static_cast<MSBaseVehicle&>(veh);
+    sVehicles[veh.getID()]->updates.push_back(MSDevice_BTsender::VehicleState(newSpeed, veh.getPosition(), location, newPos, v.getRoutePosition()));
     return true;
 }
 
 
 bool
-MSDevice_BTreceiver::notifyLeave(SUMOVehicle& veh, SUMOReal /* lastPos */, Notification reason) {
+MSDevice_BTreceiver::notifyLeave(SUMOTrafficObject& veh, double /* lastPos */, Notification reason, const MSLane* /* enteredLane */) {
     if (reason < MSMoveReminder::NOTIFICATION_TELEPORT) {
         return true;
     }
@@ -431,8 +422,9 @@ MSDevice_BTreceiver::notifyLeave(SUMOVehicle& veh, SUMOReal /* lastPos */, Notif
         WRITE_WARNING("btreceiver: Can not update position of vehicle '" + veh.getID() + "' which is not on the road.");
         return true;
     }
-    const MSVehicle& v = static_cast<MSVehicle&>(veh);
-    sVehicles[veh.getID()]->updates.push_back(MSDevice_BTsender::VehicleState(veh.getSpeed(), veh.getPosition(), v.getLane()->getID(), veh.getPositionOnLane(), v.getRoutePosition()));
+    const std::string location = MSGlobals::gUseMesoSim ? veh.getEdge()->getID() : static_cast<MSVehicle&>(veh).getLane()->getID();
+    const MSBaseVehicle& v = static_cast<MSBaseVehicle&>(veh);
+    sVehicles[veh.getID()]->updates.push_back(MSDevice_BTsender::VehicleState(veh.getSpeed(), veh.getPosition(), location, veh.getPositionOnLane(), v.getRoutePosition()));
     if (reason == MSMoveReminder::NOTIFICATION_TELEPORT) {
         sVehicles[veh.getID()]->amOnNet = false;
     }
@@ -444,8 +436,4 @@ MSDevice_BTreceiver::notifyLeave(SUMOVehicle& veh, SUMOReal /* lastPos */, Notif
 }
 
 
-
-
-
 /****************************************************************************/
-
