@@ -1,39 +1,29 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSDevice_BTreceiver.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    14.08.2013
-/// @version $Id: MSDevice_BTreceiver.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // A BT receiver
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSDevice_BTreceiver_h
-#define MSDevice_BTreceiver_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include "MSDevice.h"
+#include <random>
+#include "MSVehicleDevice.h"
 #include "MSDevice_BTsender.h"
-#include <microsim/MSNet.h>
 #include <utils/common/SUMOTime.h>
 #include <utils/common/Command.h>
 #include <utils/common/RandHelper.h>
@@ -54,7 +44,7 @@ class SUMOVehicle;
  *
  * @see MSDevice
  */
-class MSDevice_BTreceiver : public MSDevice {
+class MSDevice_BTreceiver : public MSVehicleDevice {
 public:
     /** @brief Inserts MSDevice_BTreceiver-options
      * @param[filled] oc The options container to add the options to
@@ -72,17 +62,20 @@ public:
      * @param[in] v The vehicle for which a device may be built
      * @param[filled] into The vector to store the built device in
      */
-    static void buildVehicleDevices(SUMOVehicle& v, std::vector<MSDevice*>& into);
+    static void buildVehicleDevices(SUMOVehicle& v, std::vector<MSVehicleDevice*>& into);
 
 
     /** @brief Returns the configured range
      * @return the device range
      */
-    static SUMOReal getRange() {
+    static double getRange() {
         return myRange;
     }
 
 
+    static std::mt19937* getRNG() {
+        return &sRecognitionRNG;
+    }
 
 public:
     /// @brief Destructor.
@@ -101,7 +94,7 @@ public:
      * @see MSMoveReminder::notifyEnter
      * @see MSMoveReminder::Notification
      */
-    bool notifyEnter(SUMOVehicle& veh, Notification reason);
+    bool notifyEnter(SUMOTrafficObject& veh, Notification reason, const MSLane* enteredLane = 0);
 
 
     /** @brief Checks whether the reminder still has to be notified about the vehicle moves
@@ -117,7 +110,7 @@ public:
      *
      * @return True if vehicle hasn't passed the reminder completely.
      */
-    bool notifyMove(SUMOVehicle& veh, SUMOReal oldPos, SUMOReal newPos, SUMOReal newSpeed);
+    bool notifyMove(SUMOTrafficObject& veh, double oldPos, double newPos, double newSpeed);
 
 
     /** @brief Moves (the known) vehicle from running to arrived vehicles' list
@@ -126,13 +119,16 @@ public:
      * @param[in] lastPos Position on the lane when leaving.
      * @param[in] isArrival whether the vehicle arrived at its destination
      * @param[in] isLaneChange whether the vehicle changed from the lane
-     * @see leaveDetectorByLaneChange
      * @see MSMoveReminder
      * @see MSMoveReminder::notifyLeave
      */
-    bool notifyLeave(SUMOVehicle& veh, SUMOReal lastPos, Notification reason);
-    //@}
+    bool notifyLeave(SUMOTrafficObject& veh, double lastPos, Notification reason, const MSLane* enteredLane = 0);
+    ///@}
 
+    /// @brief return the name for this type of device
+    const std::string deviceName() const {
+        return "btreceiver";
+    }
 
 
     /** @class MeetingPoint
@@ -142,44 +138,27 @@ public:
     public:
         /** @brief Constructor
          * @param[in] _t The time of the meeting
-         * @param[in] _observerPos The position the observer had at the time
-         * @param[in] _observerSpeed The speed the observer had at the time
-         * @param[in] _observerLaneID The lane the observer was at
-         * @param[in] _observerLanePos The position at the lane of the observer
-         * @param[in] _seenPos The position the seen vehicle had at the time
-         * @param[in] _seenSpeed The speed the vehicle had at the time
-         * @param[in] _seenLaneID The lane the vehicle was at
-         * @param[in] _seenLanePos The position at the lane of the vehicle
+         * @param[in] _observerState The position, speed, lane etc. the observer had at the time
+         * @param[in] _seenState The position, speed, lane etc. the seen vehicle had at the time
          */
-        MeetingPoint(SUMOReal _t, const Position& _observerPos, SUMOReal _observerSpeed,
-                     const std::string& _observerLaneID, SUMOReal _observerLanePos,
-                     const Position& _seenPos, SUMOReal _seenSpeed,
-                     const std::string& _seenLaneID, SUMOReal _seenLanePos)
-            : t(_t), observerPos(_observerPos), observerSpeed(_observerSpeed), observerLaneID(_observerLaneID), observerLanePos(_observerLanePos),
-              seenPos(_seenPos), seenSpeed(_seenSpeed), seenLaneID(_seenLaneID), seenLanePos(_seenLanePos) {}
+        MeetingPoint(double _t, const MSDevice_BTsender::VehicleState& _observerState,
+                     const MSDevice_BTsender::VehicleState& _seenState)
+            : t(_t), observerState(_observerState), seenState(_seenState) {}
 
         /// @brief Destructor
         ~MeetingPoint() {}
 
     public:
         /// @brief The time of the meeting
-        SUMOReal t;
-        /// @brief The position the observer had at the time
-        Position observerPos;
-        /// @brief The speed the observer had at the time
-        SUMOReal observerSpeed;
-        /// @brief The lane the observer was at
-        std::string observerLaneID;
-        /// @brief The position at the lane of the observer
-        SUMOReal observerLanePos;
-        /// @brief The position the seen vehicle had at the time
-        Position seenPos;
-        /// @brief The speed the vehicle had at the time
-        SUMOReal seenSpeed;
-        /// @brief The lane the vehicle was at
-        std::string seenLaneID;
-        /// @brief The position at the lane of the vehicle
-        SUMOReal seenLanePos;
+        const double t;
+        /// @brief The state the observer had at the time
+        const MSDevice_BTsender::VehicleState observerState;
+        /// @brief The state the seen vehicle had at the time
+        const MSDevice_BTsender::VehicleState seenState;
+
+    private:
+        /// @brief Invalidated assignment operator.
+        MeetingPoint& operator=(const MeetingPoint&) = delete;
 
     };
 
@@ -194,10 +173,11 @@ public:
          * @param[in] meetingBegin_ Description of the meeting's begin
          */
         SeenDevice(const MeetingPoint& meetingBegin_)
-            : meetingBegin(meetingBegin_), meetingEnd(meetingBegin_), lastView(meetingBegin_.t) {}
+            : meetingBegin(meetingBegin_), meetingEnd(0), lastView(meetingBegin_.t), nextView(-1.) {}
 
         /// @brief Destructor
         ~SeenDevice() {
+            delete meetingEnd;
             for (std::vector<MeetingPoint*>::iterator i = recognitionPoints.begin(); i != recognitionPoints.end(); ++i) {
                 delete *i;
             }
@@ -207,13 +187,23 @@ public:
 
     public:
         /// @brief Description of the meeting's begin
-        MeetingPoint meetingBegin;
+        const MeetingPoint meetingBegin;
         /// @brief Description of the meeting's end
-        MeetingPoint meetingEnd;
+        MeetingPoint* meetingEnd;
         /// @brief Last recognition point
-        SUMOReal lastView;
+        double lastView;
+        /// @brief Next possible recognition point
+        double nextView;
         /// @brief List of recognition points
         std::vector<MeetingPoint*> recognitionPoints;
+        /// @brief string of travelled receiver edges
+        std::string receiverRoute;
+        /// @brief string of travelled sender edges
+        std::string senderRoute;
+
+    private:
+        /// @brief Invalidated assignment operator.
+        SeenDevice& operator=(const SeenDevice&);
 
     };
 
@@ -242,10 +232,10 @@ private:
     static bool myWasInitialised;
 
     /// @brief The range of the device
-    static SUMOReal myRange;
+    static double myRange;
 
     /// @brief The offtime of the device
-    static SUMOReal myOffTime;
+    static double myOffTime;
 
 
     /** @class VehicleInformation
@@ -257,13 +247,25 @@ private:
          * @param[in] id The id of the vehicle
          * @param[in] range Recognition range of the vehicle
          */
-        VehicleInformation(const std::string& id, const SUMOReal _range) : MSDevice_BTsender::VehicleInformation(id), range(_range) {}
+        VehicleInformation(const std::string& id, const double _range) : MSDevice_BTsender::VehicleInformation(id), range(_range) {}
 
         /// @brief Destructor
-        ~VehicleInformation() {}
+        ~VehicleInformation() {
+            std::map<std::string, SeenDevice*>::iterator i;
+            for (i = currentlySeen.begin(); i != currentlySeen.end(); i++) {
+                delete i->second;
+            }
+            std::map<std::string, std::vector<SeenDevice*> >::iterator j;
+            std::vector<SeenDevice*>::iterator k;
+            for (j = seen.begin(); j != seen.end(); j++) {
+                for (k = j->second.begin(); k != j->second.end(); k++) {
+                    delete *k;
+                }
+            }
+        }
 
         /// @brief Recognition range of the vehicle
-        const SUMOReal range;
+        const double range;
 
         /// @brief The map of devices seen by the vehicle at removal time
         std::map<std::string, SeenDevice*> currentlySeen;
@@ -309,60 +311,39 @@ private:
 
         /** @brief Informs the receiver about a sender entering it's radius
          * @param[in] atOffset The time offset to the current time step
-         * @param[in] thisPos The receiver's position at the time
-         * @param[in] thisSpeed The receiver's speed at the time
-         * @param[in] thisLaneID The lane the observer was at
-         * @param[in] thisLanePos The position at the lane of the observer
-         * @param[in] otherID The ID of the entering sender
-         * @param[in] otherPos The position of the entering sender
-         * @param[in] otherSpeed The speed of the entering sender
-         * @param[in] otherLaneID The lane the sender was at
-         * @param[in] otherLanePos The position at the lane of the sender
+         * @param[in] receiverState The position, speed, lane etc. the observer had at the time
+         * @param[in] senderID The ID of the entering sender
+         * @param[in] senderState The position, speed, lane etc. the seen vehicle had at the time
          * @param[in] currentlySeen The container storing episodes
          */
-        void enterRange(SUMOReal atOffset, const Position& thisPos, SUMOReal thisSpeed, const std::string& thisLaneID, SUMOReal thisLanePos,
-                        const std::string& otherID, const Position& otherPos, SUMOReal otherSpeed, const std::string& otherLaneID, SUMOReal otherLanePos,
+        void enterRange(double atOffset, const MSDevice_BTsender::VehicleState& receiverState,
+                        const std::string& senderID, const MSDevice_BTsender::VehicleState& senderState,
                         std::map<std::string, SeenDevice*>& currentlySeen);
 
 
         /** @brief Removes the sender from the currently seen devices to past episodes
-         * @param[in] currentlySeen The currently seen devices
-         * @param[in] seen The lists of episodes to add this one to
-         * @param[in] thisPos The receiver's position at the time
-         * @param[in] thisSpeed The receiver's speed at the time
-         * @param[in] thisLaneID The lane the observer was at
-         * @param[in] thisLanePos The position at the lane of the observer
-         * @param[in] otherID The ID of the entering sender
-         * @param[in] otherPos The position of the entering sender
-         * @param[in] otherSpeed The speed of the entering sender
-         * @param[in] otherLaneID The lane the sender was at
-         * @param[in] otherLanePos The position at the lane of the sender
+         * @param[in] receiverInfo The static information of the observer (id, route, etc.)
+         * @param[in] receiverState The position, speed, lane etc. the observer had at the time
+         * @param[in] senderInfo The static information of the seen vehicle (id, route, etc.)
+         * @param[in] senderState The position, speed, lane etc. the seen vehicle had at the time
          * @param[in] tOffset The time offset to the current time step
-         * @param[in] remove Whether the sender shall be removed from this device's myCurrentlySeen
          */
-        void leaveRange(std::map<std::string, SeenDevice*>& currentlySeen, std::map<std::string, std::vector<SeenDevice*> >& seen,
-                        const Position& thisPos, SUMOReal thisSpeed, const std::string& thisLaneID, SUMOReal thisLanePos,
-                        const std::string& otherID, const Position& otherPos, SUMOReal otherSpeed, const std::string& otherLaneID, SUMOReal otherLanePos,
-                        SUMOReal tOffset);
+        void leaveRange(VehicleInformation& receiverInfo, const MSDevice_BTsender::VehicleState& receiverState,
+                        MSDevice_BTsender::VehicleInformation& senderInfo, const MSDevice_BTsender::VehicleState& senderState,
+                        double tOffset);
 
 
 
 
         /** @brief Adds a point of recognition
          * @param[in] tEnd The time of the recognition
-         * @param[in] thisPos The receiver's position at the time
-         * @param[in] thisSpeed The receiver's speed at the time
-         * @param[in] thisLaneID The lane the observer was at
-         * @param[in] thisLanePos The position at the lane of the observer
-         * @param[in] otherPos The position of the entering sender
-         * @param[in] otherSpeed The speed of the entering sender
-         * @param[in] otherLaneID The lane the sender was at
-         * @param[in] otherLanePos The position at the lane of the sender
-         * @param[in] otherDevice The device of the entering sender
+         * @param[in] receiverState The position, speed, lane etc. the observer had at the time
+         * @param[in] senderState The position, speed, lane etc. the seen vehicle had at the time
+         * @param[in] senderDevice The device of the entering sender
          */
-        void addRecognitionPoint(const SUMOReal tEnd, const Position& thisPos, const SUMOReal thisSpeed, const std::string& thisLaneID, const SUMOReal thisLanePos,
-                                 const Position& otherPos, const SUMOReal otherSpeed, const std::string& otherLaneID, const SUMOReal otherLanePos,
-                                 SeenDevice* otherDevice) const;
+        void addRecognitionPoint(const double tEnd, const MSDevice_BTsender::VehicleState& receiverState,
+                                 const MSDevice_BTsender::VehicleState& senderState,
+                                 SeenDevice* senderDevice) const;
 
 
         /** @brief Writes the output
@@ -379,8 +360,10 @@ private:
     };
 
 
+    static double inquiryDelaySlots(const int backoffLimit);
+
     /// @brief A random number generator used to determine whether the opposite was recognized
-    static MTRand sRecognitionRNG;
+    static std::mt19937 sRecognitionRNG;
 
     /// @brief The list of arrived receivers
     static std::map<std::string, VehicleInformation*> sVehicles;
@@ -396,8 +379,3 @@ private:
 
 
 };
-
-#endif
-
-/****************************************************************************/
-
