@@ -1,34 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2009-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSDevice_Vehroutes.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Fri, 30.01.2009
-/// @version $Id: MSDevice_Vehroutes.cpp 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // A device which collects info on the vehicle trip
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2009-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <microsim/MSGlobals.h>
 #include <microsim/MSNet.h>
@@ -37,6 +29,7 @@
 #include <microsim/MSRoute.h>
 #include <microsim/MSVehicle.h>
 #include <microsim/MSVehicleType.h>
+#include <microsim/transportables/MSTransportableControl.h>
 #include <utils/vehicle/SUMOVehicle.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/iodevices/OutputDevice_String.h>
@@ -50,9 +43,13 @@
 bool MSDevice_Vehroutes::mySaveExits = false;
 bool MSDevice_Vehroutes::myLastRouteOnly = false;
 bool MSDevice_Vehroutes::myDUAStyle = false;
+bool MSDevice_Vehroutes::myWriteCosts = false;
 bool MSDevice_Vehroutes::mySorted = false;
 bool MSDevice_Vehroutes::myIntendedDepart = false;
 bool MSDevice_Vehroutes::myRouteLength = false;
+bool MSDevice_Vehroutes::mySkipPTLines = false;
+bool MSDevice_Vehroutes::myIncludeIncomplete = false;
+bool MSDevice_Vehroutes::myWriteStopPriorEdges = false;
 MSDevice_Vehroutes::StateListener MSDevice_Vehroutes::myStateListener;
 std::map<const SUMOTime, int> MSDevice_Vehroutes::myDepartureCounts;
 std::map<const SUMOTime, std::map<const std::string, std::string> > MSDevice_Vehroutes::myRouteInfos;
@@ -66,25 +63,41 @@ std::map<const SUMOTime, std::map<const std::string, std::string> > MSDevice_Veh
 // ---------------------------------------------------------------------------
 void
 MSDevice_Vehroutes::init() {
-    if (OptionsCont::getOptions().isSet("vehroute-output")) {
+    const OptionsCont& oc = OptionsCont::getOptions();
+    if (oc.isSet("vehroute-output")) {
         OutputDevice::createDeviceByOption("vehroute-output", "routes", "routes_file.xsd");
-        mySaveExits = OptionsCont::getOptions().getBool("vehroute-output.exit-times");
-        myLastRouteOnly = OptionsCont::getOptions().getBool("vehroute-output.last-route");
-        myDUAStyle = OptionsCont::getOptions().getBool("vehroute-output.dua");
-        mySorted = myDUAStyle || OptionsCont::getOptions().getBool("vehroute-output.sorted");
-        myIntendedDepart = OptionsCont::getOptions().getBool("vehroute-output.intended-depart");
-        myRouteLength = OptionsCont::getOptions().getBool("vehroute-output.route-length");
+        mySaveExits = oc.getBool("vehroute-output.exit-times");
+        myLastRouteOnly = oc.getBool("vehroute-output.last-route");
+        myDUAStyle = oc.getBool("vehroute-output.dua");
+        myWriteCosts = oc.getBool("vehroute-output.cost");
+        mySorted = myDUAStyle || oc.getBool("vehroute-output.sorted");
+        myIntendedDepart = oc.getBool("vehroute-output.intended-depart");
+        myRouteLength = oc.getBool("vehroute-output.route-length");
+        mySkipPTLines = oc.getBool("vehroute-output.skip-ptlines");
+        myIncludeIncomplete = oc.getBool("vehroute-output.incomplete");
+        myWriteStopPriorEdges = oc.getBool("vehroute-output.stop-edges");
         MSNet::getInstance()->addVehicleStateListener(&myStateListener);
     }
 }
 
+void
+MSDevice_Vehroutes::insertOptions(OptionsCont& oc) {
+    oc.addOptionSubTopic("Vehroutes Device");
+    insertDefaultAssignmentOptions("vehroute", "Vehroutes Device", oc);
+}
+
+
 
 MSDevice_Vehroutes*
-MSDevice_Vehroutes::buildVehicleDevices(SUMOVehicle& v, std::vector<MSDevice*>& into, int maxRoutes) {
-    if (maxRoutes < INT_MAX) {
+MSDevice_Vehroutes::buildVehicleDevices(SUMOVehicle& v, std::vector<MSVehicleDevice*>& into, int maxRoutes) {
+    if (maxRoutes < std::numeric_limits<int>::max()) {
         return new MSDevice_Vehroutes(v, "vehroute_" + v.getID(), maxRoutes);
     }
-    if (OptionsCont::getOptions().isSet("vehroute-output")) {
+    if (mySkipPTLines && v.getParameter().line != "") {
+        return nullptr;
+    }
+    OptionsCont& oc = OptionsCont::getOptions();
+    if (equippedByDefaultAssignmentOptions(oc, "vehroute", v, oc.isSet("vehroute-output"))) {
         if (myLastRouteOnly) {
             maxRoutes = 0;
         }
@@ -92,7 +105,7 @@ MSDevice_Vehroutes::buildVehicleDevices(SUMOVehicle& v, std::vector<MSDevice*>&
         into.push_back(myStateListener.myDevices[&v]);
         return myStateListener.myDevices[&v];
     }
-    return 0;
+    return nullptr;
 }
 
 
@@ -100,9 +113,12 @@ MSDevice_Vehroutes::buildVehicleDevices(SUMOVehicle& v, std::vector<MSDevice*>&
 // MSDevice_Vehroutes::StateListener-methods
 // ---------------------------------------------------------------------------
 void
-MSDevice_Vehroutes::StateListener::vehicleStateChanged(const SUMOVehicle* const vehicle, MSNet::VehicleState to) {
-    if (to == MSNet::VEHICLE_STATE_NEWROUTE) {
-        myDevices[vehicle]->addRoute();
+MSDevice_Vehroutes::StateListener::vehicleStateChanged(const SUMOVehicle* const vehicle, MSNet::VehicleState to, const std::string& info) {
+    if (to == MSNet::VehicleState::NEWROUTE) {
+        const auto& deviceEntry = myDevices.find(vehicle);
+        if (deviceEntry != myDevices.end()) {
+            deviceEntry->second->addRoute(info);
+        }
     }
 }
 
@@ -111,21 +127,22 @@ MSDevice_Vehroutes::StateListener::vehicleStateChanged(const SUMOVehicle* const
 // MSDevice_Vehroutes-methods
 // ---------------------------------------------------------------------------
 MSDevice_Vehroutes::MSDevice_Vehroutes(SUMOVehicle& holder, const std::string& id, int maxRoutes) :
-    MSDevice(holder, id),
+    MSVehicleDevice(holder, id),
     myCurrentRoute(&holder.getRoute()),
     myMaxRoutes(maxRoutes),
-    myLastSavedAt(0),
+    myLastSavedAt(nullptr),
     myDepartLane(-1),
     myDepartPos(-1),
     myDepartSpeed(-1),
-    myDepartPosLat(0) {
+    myDepartPosLat(0),
+    myStopOut(2) {
     myCurrentRoute->addReference();
 }
 
 
 MSDevice_Vehroutes::~MSDevice_Vehroutes() {
-    for (std::vector<RouteReplaceInfo>::iterator i = myReplacedRoutes.begin(); i != myReplacedRoutes.end(); ++i) {
-        (*i).route->release();
+    for (const RouteReplaceInfo& rri : myReplacedRoutes) {
+        rri.route->release();
     }
     myCurrentRoute->release();
     myStateListener.myDevices.erase(&myHolder);
@@ -133,25 +150,29 @@ MSDevice_Vehroutes::~MSDevice_Vehroutes() {
 
 
 bool
-MSDevice_Vehroutes::notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason, const MSLane* /* enteredLane */) {
+MSDevice_Vehroutes::notifyEnter(SUMOTrafficObject& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane) {
     if (reason == MSMoveReminder::NOTIFICATION_DEPARTED) {
-        if (mySorted && myStateListener.myDevices[&veh] == this) {
+        if (mySorted && myStateListener.myDevices[static_cast<SUMOVehicle*>(&veh)] == this) {
             const SUMOTime departure = myIntendedDepart ? myHolder.getParameter().depart : MSNet::getInstance()->getCurrentTimeStep();
             myDepartureCounts[departure]++;
         }
         if (!MSGlobals::gUseMesoSim) {
-            myDepartLane = static_cast<MSVehicle&>(veh).getLane()->getIndex();
-            myDepartPosLat = static_cast<MSVehicle&>(veh).getLateralPositionOnLane();
+            const MSVehicle& vehicle = static_cast<MSVehicle&>(veh);
+            myDepartLane = vehicle.getLane()->getIndex();
+            myDepartPosLat = vehicle.getLateralPositionOnLane();
         }
         myDepartSpeed = veh.getSpeed();
         myDepartPos = veh.getPositionOnLane();
     }
-    return mySaveExits;
+    if (myWriteStopPriorEdges) {
+        myPriorEdges.push_back(&enteredLane->getEdge());
+    }
+    return mySaveExits || myWriteStopPriorEdges;
 }
 
 
 bool
-MSDevice_Vehroutes::notifyLeave(SUMOVehicle& veh, double /*lastPos*/, MSMoveReminder::Notification reason, const MSLane* /* enteredLane */) {
+MSDevice_Vehroutes::notifyLeave(SUMOTrafficObject& veh, double /*lastPos*/, MSMoveReminder::Notification reason, const MSLane* /* enteredLane */) {
     if (mySaveExits && reason != NOTIFICATION_LANE_CHANGE) {
         if (reason != NOTIFICATION_TELEPORT && myLastSavedAt == veh.getEdge()) { // need to check this for internal lanes
             myExits.back() = MSNet::getInstance()->getCurrentTimeStep();
@@ -160,42 +181,76 @@ MSDevice_Vehroutes::notifyLeave(SUMOVehicle& veh, double /*lastPos*/, MSMoveRemi
             myLastSavedAt = veh.getEdge();
         }
     }
-    return mySaveExits;
+    return mySaveExits || myWriteStopPriorEdges;
+}
+
+
+void
+MSDevice_Vehroutes::stopEnded(const SUMOVehicleParameter::Stop& stop) {
+    const bool closeLater = myWriteStopPriorEdges || mySaveExits;
+    stop.write(myStopOut, !closeLater);
+    if (myWriteStopPriorEdges) {
+        myStopOut.writeAttr("priorEdges", myPriorEdges);
+        myPriorEdges.clear();
+    }
+    if (mySaveExits) {
+        myStopOut.writeAttr(SUMO_ATTR_STARTED, time2string(stop.started));
+        myStopOut.writeAttr(SUMO_ATTR_ENDED, stop.ended < 0 ? "-1" : time2string(stop.ended));
+    }
+    if (closeLater) {
+        myStopOut.closeTag();
+    }
 }
 
 
 void
 MSDevice_Vehroutes::writeXMLRoute(OutputDevice& os, int index) const {
-    if (index == 0 && myReplacedRoutes[index].route->size() == 2 &&
-            myReplacedRoutes[index].route->getEdges().front()->getPurpose() == MSEdge::EDGEFUNCTION_DISTRICT &&
-            myReplacedRoutes[index].route->getEdges().back()->getPurpose() == MSEdge::EDGEFUNCTION_DISTRICT) {
+    if (index == 0 && !myIncludeIncomplete && myReplacedRoutes[index].route->size() == 2 &&
+            myReplacedRoutes[index].route->getEdges().front()->isTazConnector() &&
+            myReplacedRoutes[index].route->getEdges().back()->isTazConnector()) {
         return;
     }
     // check if a previous route shall be written
     os.openTag(SUMO_TAG_ROUTE);
     if (index >= 0) {
         assert((int)myReplacedRoutes.size() > index);
-        // write edge on which the vehicle was when the route was valid
-        os << " replacedOnEdge=\"";
-        if (myReplacedRoutes[index].edge) {
-            os << myReplacedRoutes[index].edge->getID();
+        if (myDUAStyle || myWriteCosts) {
+            os.writeAttr(SUMO_ATTR_COST, myReplacedRoutes[index].route->getCosts());
         }
+        if (myWriteCosts) {
+            os.writeAttr(SUMO_ATTR_SAVINGS, myReplacedRoutes[index].route->getSavings());
+        }
+        // write edge on which the vehicle was when the route was valid
+        os.writeAttr("replacedOnEdge", (myReplacedRoutes[index].edge ?
+                                        myReplacedRoutes[index].edge->getID() : ""));
+        // write the reason for replacement
+        os.writeAttr("reason", myReplacedRoutes[index].info);
+
         // write the time at which the route was replaced
-        os << "\" replacedAtTime=\"" << time2string(myReplacedRoutes[index].time) << "\" probability=\"0\" edges=\"";
+        os.writeAttr("replacedAtTime", time2string(myReplacedRoutes[index].time));
+        os.writeAttr(SUMO_ATTR_PROB, "0");
+        os << " edges=\"";
         // get the route
         int i = index;
-        while (i > 0 && myReplacedRoutes[i - 1].edge) {
+        while (i > 0 && myReplacedRoutes[i - 1].edge != nullptr && !myIncludeIncomplete) {
             i--;
         }
-        const MSEdge* lastEdge = 0;
+        const MSEdge* lastEdge = nullptr;
         for (; i < index; ++i) {
             myReplacedRoutes[i].route->writeEdgeIDs(os, lastEdge, myReplacedRoutes[i].edge);
             lastEdge = myReplacedRoutes[i].edge;
         }
         myReplacedRoutes[index].route->writeEdgeIDs(os, lastEdge);
+        os << "\"";
     } else {
+        if (myDUAStyle || myWriteCosts) {
+            os.writeAttr(SUMO_ATTR_COST, myHolder.getRoute().getCosts());
+        }
+        if (myWriteCosts) {
+            os.writeAttr(SUMO_ATTR_SAVINGS, myHolder.getRoute().getSavings());
+        }
         os << " edges=\"";
-        const MSEdge* lastEdge = 0;
+        const MSEdge* lastEdge = nullptr;
         int numWritten = 0;
         if (myHolder.getNumberReroutes() > 0) {
             assert((int)myReplacedRoutes.size() <= myHolder.getNumberReroutes());
@@ -208,32 +263,26 @@ MSDevice_Vehroutes::writeXMLRoute(OutputDevice& os, int index) const {
                 lastEdge = myReplacedRoutes[i].edge;
             }
         }
-        const MSEdge* upTo = 0;
+        numWritten += myCurrentRoute->writeEdgeIDs(os, lastEdge, nullptr);
+        os << "\"";
+
         if (mySaveExits) {
-            int remainingWithExitTime = (int)myExits.size() - numWritten;
-            assert(remainingWithExitTime >= 0);
-            assert(remainingWithExitTime <= (int)myCurrentRoute->size());
-            if (remainingWithExitTime < (int)myCurrentRoute->size()) {
-                upTo = *(myCurrentRoute->begin() + remainingWithExitTime);
-            }
-        }
-        myCurrentRoute->writeEdgeIDs(os, lastEdge, upTo);
-        if (mySaveExits) {
-            os << "\" exitTimes=\"";
-            for (std::vector<SUMOTime>::const_iterator it = myExits.begin(); it != myExits.end(); ++it) {
-                if (it != myExits.begin()) {
-                    os << " ";
-                }
-                os << time2string(*it);
+            std::vector<std::string> exits;
+            for (SUMOTime t : myExits) {
+                exits.push_back(time2string(t));
             }
+            assert(numWritten >= (int)myExits.size());
+            std::vector<std::string> missing(numWritten - (int)myExits.size(), "-1");
+            exits.insert(exits.end(), missing.begin(), missing.end());
+            os.writeAttr(SUMO_ATTR_EXITTIMES, exits);
         }
     }
-    (os << "\"").closeTag();
+    os.closeTag();
 }
 
 
 void
-MSDevice_Vehroutes::generateOutput() const {
+MSDevice_Vehroutes::generateOutput(OutputDevice* /*tripinfoOut*/) const {
     writeOutput(true);
 }
 
@@ -241,25 +290,25 @@ MSDevice_Vehroutes::generateOutput() const {
 void
 MSDevice_Vehroutes::writeOutput(const bool hasArrived) const {
     OutputDevice& routeOut = OutputDevice::getDeviceByOption("vehroute-output");
-    OutputDevice_String od(routeOut.isBinary(), 1);
+    OutputDevice_String od(1);
     SUMOVehicleParameter tmp = myHolder.getParameter();
     tmp.depart = myIntendedDepart ? myHolder.getParameter().depart : myHolder.getDeparture();
     if (!MSGlobals::gUseMesoSim) {
         if (tmp.wasSet(VEHPARS_DEPARTLANE_SET)) {
-            tmp.departLaneProcedure = DEPART_LANE_GIVEN;
+            tmp.departLaneProcedure = DepartLaneDefinition::GIVEN;
             tmp.departLane = myDepartLane;
         }
         if (tmp.wasSet(VEHPARS_DEPARTPOSLAT_SET)) {
-            tmp.departPosLatProcedure = DEPART_POSLAT_GIVEN;
+            tmp.departPosLatProcedure = DepartPosLatDefinition::GIVEN;
             tmp.departPosLat = myDepartPosLat;
         }
     }
     if (tmp.wasSet(VEHPARS_DEPARTPOS_SET)) {
-        tmp.departPosProcedure = DEPART_POS_GIVEN;
+        tmp.departPosProcedure = DepartPosDefinition::GIVEN;
         tmp.departPos = myDepartPos;
     }
     if (tmp.wasSet(VEHPARS_DEPARTSPEED_SET)) {
-        tmp.departSpeedProcedure = DEPART_SPEED_GIVEN;
+        tmp.departSpeedProcedure = DepartSpeedDefinition::GIVEN;
         tmp.departSpeed = myDepartSpeed;
     }
     const std::string typeID = myHolder.getVehicleType().getID() != DEFAULT_VTYPE_ID ? myHolder.getVehicleType().getID() : "";
@@ -275,7 +324,7 @@ MSDevice_Vehroutes::writeOutput(const bool hasArrived) const {
     }
     if (myDUAStyle) {
         const RandomDistributor<const MSRoute*>* const routeDist = MSRoute::distDictionary("!" + myHolder.getID());
-        if (routeDist != 0) {
+        if (routeDist != nullptr) {
             const std::vector<const MSRoute*>& routes = routeDist->getVals();
             unsigned index = 0;
             while (index < routes.size() && routes[index] != myCurrentRoute) {
@@ -285,7 +334,11 @@ MSDevice_Vehroutes::writeOutput(const bool hasArrived) const {
             const std::vector<double>& probs = routeDist->getProbs();
             for (int i = 0; i < (int)routes.size(); ++i) {
                 od.setPrecision();
-                od.openTag(SUMO_TAG_ROUTE).writeAttr(SUMO_ATTR_COST, routes[i]->getCosts());
+                od.openTag(SUMO_TAG_ROUTE);
+                od.writeAttr(SUMO_ATTR_COST, routes[i]->getCosts());
+                if (myWriteCosts) {
+                    od.writeAttr(SUMO_ATTR_SAVINGS, routes[i]->getSavings());
+                }
                 od.setPrecision(8);
                 od.writeAttr(SUMO_ATTR_PROB, probs[i]);
                 od.setPrecision();
@@ -298,23 +351,19 @@ MSDevice_Vehroutes::writeOutput(const bool hasArrived) const {
             writeXMLRoute(od);
         }
     } else {
-        if (myReplacedRoutes.size() > 0) {
+        const int routesToSkip = myHolder.getParameter().wasSet(VEHPARS_FORCE_REROUTE) && !myIncludeIncomplete ? 1 : 0;
+        if ((int)myReplacedRoutes.size() > routesToSkip) {
             od.openTag(SUMO_TAG_ROUTE_DISTRIBUTION);
-            for (int i = 0; i < (int)myReplacedRoutes.size(); ++i) {
+            for (int i = routesToSkip; i < (int)myReplacedRoutes.size(); ++i) {
                 writeXMLRoute(od, i);
             }
-        }
-        writeXMLRoute(od);
-        if (myReplacedRoutes.size() > 0) {
+            writeXMLRoute(od);
             od.closeTag();
+        } else {
+            writeXMLRoute(od);
         }
     }
-    for (std::vector<SUMOVehicleParameter::Stop>::const_iterator i = myHolder.getParameter().stops.begin(); i != myHolder.getParameter().stops.end(); ++i) {
-        i->write(od);
-    }
-    for (std::vector<SUMOVehicleParameter::Stop>::const_iterator i = myHolder.getRoute().getStops().begin(); i != myHolder.getRoute().getStops().end(); ++i) {
-        i->write(od);
-    }
+    od << myStopOut.getString();
     myHolder.getParameter().writeParams(od);
     od.closeTag();
     od.lf();
@@ -342,18 +391,18 @@ MSDevice_Vehroutes::getRoute(int index) const {
     if (index < (int)myReplacedRoutes.size()) {
         return myReplacedRoutes[index].route;
     } else {
-        return 0;
+        return nullptr;
     }
 }
 
 
 void
-MSDevice_Vehroutes::addRoute() {
+MSDevice_Vehroutes::addRoute(const std::string& info) {
     if (myMaxRoutes > 0) {
         if (myHolder.hasDeparted()) {
-            myReplacedRoutes.push_back(RouteReplaceInfo(myHolder.getEdge(), MSNet::getInstance()->getCurrentTimeStep(), myCurrentRoute));
+            myReplacedRoutes.push_back(RouteReplaceInfo(myHolder.getEdge(), MSNet::getInstance()->getCurrentTimeStep(), myCurrentRoute, info));
         } else {
-            myReplacedRoutes.push_back(RouteReplaceInfo(0, MSNet::getInstance()->getCurrentTimeStep(), myCurrentRoute));
+            myReplacedRoutes.push_back(RouteReplaceInfo(nullptr, MSNet::getInstance()->getCurrentTimeStep(), myCurrentRoute, info));
         }
         if ((int)myReplacedRoutes.size() > myMaxRoutes) {
             myReplacedRoutes.front().route->release();
@@ -369,10 +418,17 @@ MSDevice_Vehroutes::addRoute() {
 
 void
 MSDevice_Vehroutes::generateOutputForUnfinished() {
-    for (std::map<const SUMOVehicle*, MSDevice_Vehroutes*, Named::NamedLikeComparatorIdLess<SUMOVehicle> >::const_iterator it = myStateListener.myDevices.begin();
-            it != myStateListener.myDevices.end(); ++it) {
-        if (it->first->hasDeparted()) {
-            it->second->writeOutput(false);
+    for (const auto& it : myStateListener.myDevices) {
+        if (it.first->hasDeparted()) {
+            it.second->writeOutput(false);
+        }
+    }
+    // unfinished persons
+    MSNet* net = MSNet::getInstance();
+    if (net->hasPersons()) {
+        MSTransportableControl& pc = net->getPersonControl();
+        while (pc.loadedBegin() != pc.loadedEnd()) {
+            pc.erase(pc.loadedBegin()->second);
         }
     }
 }
@@ -391,11 +447,16 @@ MSDevice_Vehroutes::saveState(OutputDevice& out) const {
     internals.push_back(toString(myDepartPos));
     internals.push_back(toString(myReplacedRoutes.size()));
     for (int i = 0; i < (int)myReplacedRoutes.size(); ++i) {
-        internals.push_back(myReplacedRoutes[i].edge->getID());
+        const std::string replacedOnEdge = myReplacedRoutes[i].edge == nullptr ? "!NULL" : myReplacedRoutes[i].edge->getID();
+        internals.push_back(replacedOnEdge);
         internals.push_back(toString(myReplacedRoutes[i].time));
         internals.push_back(myReplacedRoutes[i].route->getID());
+        internals.push_back(myReplacedRoutes[i].info);
     }
     out.writeAttr(SUMO_ATTR_STATE, toString(internals));
+    if (mySaveExits) {
+        out.writeAttr(SUMO_ATTR_EXITTIMES, myExits);
+    }
     out.closeTag();
 }
 
@@ -415,10 +476,19 @@ MSDevice_Vehroutes::loadState(const SUMOSAXAttributes& attrs) {
         std::string edgeID;
         SUMOTime time;
         std::string routeID;
+        std::string info;
         bis >> edgeID;
         bis >> time;
         bis >> routeID;
-        myReplacedRoutes.push_back(RouteReplaceInfo(MSEdge::dictionary(edgeID), time, MSRoute::dictionary(routeID)));
+        bis >> info;
+        const MSRoute* route = MSRoute::dictionary(routeID);
+        route->addReference();
+        myReplacedRoutes.push_back(RouteReplaceInfo(MSEdge::dictionary(edgeID), time, route, info));
+    }
+    if (mySaveExits && attrs.hasAttribute(SUMO_ATTR_EXITTIMES)) {
+        for (const std::string t : attrs.getStringVector(SUMO_ATTR_EXITTIMES)) {
+            myExits.push_back(StringUtils::toLong(t));
+        }
     }
 }
 
