@@ -1,40 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2009-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSDevice_Vehroutes.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Fri, 30.01.2009
-/// @version $Id: MSDevice_Vehroutes.h 23989 2017-04-20 13:02:53Z namdre $
 ///
 // A device which collects info on the vehicle trip
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2009-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSDevice_Vehroutes_h
-#define MSDevice_Vehroutes_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include "MSDevice.h"
+#include "MSVehicleDevice.h"
 #include <microsim/MSNet.h>
+#include <microsim/MSVehicle.h>
 #include <utils/common/SUMOTime.h>
+#include <utils/iodevices/OutputDevice_String.h>
 
 
 // ===========================================================================
@@ -55,12 +47,16 @@ class MSRoute;
  *
  * @see MSDevice
  */
-class MSDevice_Vehroutes : public MSDevice {
+class MSDevice_Vehroutes : public MSVehicleDevice {
 public:
     /** @brief Static intialization
      */
     static void init();
 
+    /** @brief Inserts MSDevice_FCD-options
+     * @param[filled] oc The options container to add the options to
+     */
+    static void insertOptions(OptionsCont& oc);
 
     /** @brief Build devices for the given vehicle, if needed
      *
@@ -72,7 +68,7 @@ public:
      * @param[in] v The vehicle for which a device may be built
      * @param[filled] into The vector to store the built device in
      */
-    static MSDevice_Vehroutes* buildVehicleDevices(SUMOVehicle& v, std::vector<MSDevice*>& into, int maxRoutes = INT_MAX);
+    static MSDevice_Vehroutes* buildVehicleDevices(SUMOVehicle& v, std::vector<MSVehicleDevice*>& into, int maxRoutes = std::numeric_limits<int>::max());
 
 
     /// @brief generate vehroute output for vehicles which are still in the network
@@ -96,7 +92,7 @@ public:
      * @see MSMoveReminder::notifyEnter
      * @see MSMoveReminder::Notification
      */
-    bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+    bool notifyEnter(SUMOTrafficObject& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
 
     /** @brief Saves exit times if needed
@@ -110,7 +106,7 @@ public:
      *
      * @return True, if exit times are to be collected.
      */
-    bool notifyLeave(SUMOVehicle& veh, double lastPos, Notification reason, const MSLane* enteredLane = 0);
+    bool notifyLeave(SUMOTrafficObject& veh, double lastPos, Notification reason, const MSLane* enteredLane = 0);
     /// @}
 
     /// @brief return the name for this type of device
@@ -118,12 +114,13 @@ public:
         return "vehroute";
     }
 
+    void stopEnded(const SUMOVehicleParameter::Stop& stop);
 
     /** @brief Called on writing vehroutes output
      *
      * @exception IOError not yet implemented
      */
-    void generateOutput() const;
+    void generateOutput(OutputDevice* tripinfoOut) const;
 
 
     /** @brief Called on route retrieval
@@ -174,7 +171,7 @@ private:
 
     /** @brief Called on route change
      */
-    void addRoute();
+    void addRoute(const std::string& info);
 
 
 
@@ -188,6 +185,9 @@ private:
     /// @brief A shortcut for the Option "vehroute-output.dua"
     static bool myDUAStyle;
 
+    /// @brief A shortcut for the Option "vehroute-output.costs"
+    static bool myWriteCosts;
+
     /// @brief A shortcut for the Option "vehroute-output.sorted"
     static bool mySorted;
 
@@ -197,6 +197,14 @@ private:
     /// @brief A shortcut for the Option "vehroute-output.route-length"
     static bool myRouteLength;
 
+    /// @brief A shortcut for the Option "vehroute-output.skip-ptlines"
+    static bool mySkipPTLines;
+
+    /// @brief A shortcut for the Option "vehroute-output.incomplete"
+    static bool myIncludeIncomplete;
+
+    /// @brief A shortcut for the Option "vehroute-output.stop-edges"
+    static bool myWriteStopPriorEdges;
 
     /** @class StateListener
      * @brief A class that is notified about reroutings
@@ -210,10 +218,10 @@ private:
          * @param[in] vehicle The vehicle which changed its state
          * @param[in] to The state the vehicle has changed to
          */
-        void vehicleStateChanged(const SUMOVehicle* const vehicle, MSNet::VehicleState to);
+        void vehicleStateChanged(const SUMOVehicle* const vehicle, MSNet::VehicleState to, const std::string& info = "");
 
         /// @brief A map for internal notification
-        std::map<const SUMOVehicle*, MSDevice_Vehroutes*, SUMOVehicle::ComparatorIdLess> myDevices;
+        std::map<const SUMOVehicle*, MSDevice_Vehroutes*, ComparatorNumericalIdLess> myDevices;
 
     };
 
@@ -244,8 +252,8 @@ private:
          * @param[in] time_ The time the route was replaced
          * @param[in] route_ The prior route
          */
-        RouteReplaceInfo(const MSEdge* const edge_, const SUMOTime time_, const MSRoute* const route_)
-            : edge(edge_), time(time_), route(route_) {}
+        RouteReplaceInfo(const MSEdge* const edge_, const SUMOTime time_, const MSRoute* const route_, const std::string& info_)
+            : edge(edge_), time(time_), route(route_), info(info_) {}
 
         /// @brief Destructor
         ~RouteReplaceInfo() { }
@@ -259,6 +267,9 @@ private:
         /// @brief The prior route
         const MSRoute* route;
 
+        /// @brief Information regarding rerouting
+        std::string info;
+
     };
 
     /// @brief The currently used route
@@ -288,6 +299,11 @@ private:
     /// @brief The lateral depart position
     double myDepartPosLat;
 
+    /// @brief the edges that were passed before the current stop
+    std::vector<const MSEdge*> myPriorEdges;
+
+    OutputDevice_String myStopOut;
+
 private:
     /// @brief Invalidated copy constructor.
     MSDevice_Vehroutes(const MSDevice_Vehroutes&);
@@ -297,9 +313,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
