@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2005-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSLCM_DK2008.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Friedemann Wesner
@@ -6,45 +18,19 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Tue, 29.05.2005
-/// @version $Id: MSLCM_DK2008.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A lane change model developed by D. Krajzewicz between 2004 and 2010
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2005-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <utils/common/RandHelper.h>
 #include <microsim/MSEdge.h>
 #include <microsim/MSLane.h>
+#include <microsim/MSLink.h>
 #include <microsim/MSNet.h>
 #include "MSLCM_DK2008.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
-//#define DEBUG_VEHICLE_GUI_SELECTION 1
-
 // ===========================================================================
 // variable definitions
 // ===========================================================================
@@ -64,21 +50,22 @@
 // member method definitions
 // ===========================================================================
 MSLCM_DK2008::MSLCM_DK2008(MSVehicle& v)
-    : MSAbstractLaneChangeModel(v),
+    : MSAbstractLaneChangeModel(v, LCM_DK2008),
       myChangeProbability(0),
       myLeadingBlockerLength(0), myLeftSpace(0) {}
 
 MSLCM_DK2008::~MSLCM_DK2008() {
-    changed(0);
+    changed();
 }
 
 int
 MSLCM_DK2008::wantsChange(
     int laneOffset,
     MSAbstractLaneChangeModel::MSLCMessager& msgPass, int blocked,
-    const std::pair<MSVehicle*, SUMOReal>& leader,
-    const std::pair<MSVehicle*, SUMOReal>& neighLead,
-    const std::pair<MSVehicle*, SUMOReal>& neighFollow,
+    const std::pair<MSVehicle*, double>& leader,
+    const std::pair<MSVehicle*, double>& /*follower*/,
+    const std::pair<MSVehicle*, double>& neighLead,
+    const std::pair<MSVehicle*, double>& neighFollow,
     const MSLane& neighLane,
     const std::vector<MSVehicle::LaneQ>& preb,
     MSVehicle** lastBlocked,
@@ -94,28 +81,23 @@ MSLCM_DK2008::wantsChange(
 int
 MSLCM_DK2008::wantsChangeToRight(MSAbstractLaneChangeModel::MSLCMessager& msgPass,
                                  int blocked,
-                                 const std::pair<MSVehicle*, SUMOReal>& leader,
-                                 const std::pair<MSVehicle*, SUMOReal>& neighLead,
-                                 const std::pair<MSVehicle*, SUMOReal>& neighFollow,
+                                 const std::pair<MSVehicle*, double>& leader,
+                                 const std::pair<MSVehicle*, double>& neighLead,
+                                 const std::pair<MSVehicle*, double>& neighFollow,
                                  const MSLane& neighLane,
                                  const std::vector<MSVehicle::LaneQ>& preb,
                                  MSVehicle** lastBlocked,
                                  MSVehicle** firstBlocked) {
     UNUSED_PARAMETER(firstBlocked);
-#ifdef DEBUG_VEHICLE_GUI_SELECTION
-    if (gDebugSelectedVehicle == myVehicle.getID()) {
-        int bla = 0;
-    }
-#endif
     MSVehicle::LaneQ curr, best;
     int bestLaneOffset = 0;
-    SUMOReal currentDist = 0;
-    SUMOReal neighDist = 0;
-    SUMOReal neighExtDist = 0;
-    SUMOReal currExtDist = 0;
+    double currentDist = 0;
+    double neighDist = 0;
+    double neighExtDist = 0;
+    double currExtDist = 0;
     int currIdx = 0;
-    MSLane* prebLane = myVehicle.getLane();
-    if (prebLane->getEdge().getPurpose() == MSEdge::EDGEFUNCTION_INTERNAL) {
+    const MSLane* prebLane = myVehicle.getLane();
+    if (prebLane->getEdge().isInternal()) {
         // internal edges are not kept inside the bestLanes structure
         prebLane = prebLane->getLinkCont()[0]->getLane();
     }
@@ -153,8 +135,8 @@ MSLCM_DK2008::wantsChangeToRight(MSAbstractLaneChangeModel::MSLCMessager& msgPas
 
     // process information about the last blocked vehicle
     //  if this vehicle is blocking someone in front, we maybe decelerate to let him in
-    if ((*lastBlocked) != 0) {
-        SUMOReal gap = (*lastBlocked)->getPositionOnLane() - (*lastBlocked)->getVehicleType().getLength() - myVehicle.getPositionOnLane() - myVehicle.getVehicleType().getMinGap();
+    if ((*lastBlocked) != nullptr) {
+        double gap = (*lastBlocked)->getPositionOnLane() - (*lastBlocked)->getVehicleType().getLength() - myVehicle.getPositionOnLane() - myVehicle.getVehicleType().getMinGap();
         if (gap > 0.1) {
             if (myVehicle.getSpeed() < ACCEL2SPEED(myVehicle.getCarFollowModel().getMaxDecel())) {
                 if ((*lastBlocked)->getSpeed() < SUMO_const_haltingSpeed) {
@@ -162,8 +144,8 @@ MSLCM_DK2008::wantsChangeToRight(MSAbstractLaneChangeModel::MSLCMessager& msgPas
                 } else {
                     ret |= LCA_AMBACKBLOCKER;
                 }
-                myVSafes.push_back(myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), (SUMOReal)(gap - 0.1), (*lastBlocked)->getSpeed(), (*lastBlocked)->getCarFollowModel().getMaxDecel()));
-                (*lastBlocked) = 0;
+                myVSafes.push_back(myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), (double)(gap - 0.1), (*lastBlocked)->getSpeed(), (*lastBlocked)->getCarFollowModel().getMaxDecel()));
+                (*lastBlocked) = nullptr;
             }
             return ret;
         }
@@ -178,19 +160,19 @@ MSLCM_DK2008::wantsChangeToRight(MSAbstractLaneChangeModel::MSLCMessager& msgPas
     //  definitely try to get to the desired lane
     //
     // this rule forces our vehicle to change the lane if a lane changing is necessary soon
-    SUMOReal rv = myVehicle.getSpeed() > LOOK_FORWARD_SPEED_DIVIDER
-                  ? myVehicle.getSpeed() * (SUMOReal) LOOK_FORWARD_FAR
-                  : myVehicle.getSpeed() * (SUMOReal) LOOK_FORWARD_NEAR;
-    rv += myVehicle.getVehicleType().getLengthWithGap() * (SUMOReal) 2.;
+    double rv = myVehicle.getSpeed() > LOOK_FORWARD_SPEED_DIVIDER
+                ? myVehicle.getSpeed() * (double) LOOK_FORWARD_FAR
+                : myVehicle.getSpeed() * (double) LOOK_FORWARD_NEAR;
+    rv += myVehicle.getVehicleType().getLengthWithGap() * (double) 2.;
 
-    SUMOReal tdist = currentDist - myVehicle.getPositionOnLane() - best.occupation * (SUMOReal) JAM_FACTOR2;
+    double tdist = currentDist - myVehicle.getPositionOnLane() - best.occupation * (double) JAM_FACTOR2;
 
     // assert(best.length > curr.length);
     // XXX if (curr.length != best.length) && ...
-    if (fabs(best.length - curr.length) > MIN2((SUMOReal) .1, best.lane->getLength()) && bestLaneOffset < 0 && currentDistDisallows(tdist/*currentDist*/, bestLaneOffset, rv)) {
+    if (fabs(best.length - curr.length) > MIN2((double) .1, best.lane->getLength()) && bestLaneOffset < 0 && currentDistDisallows(tdist/*currentDist*/, bestLaneOffset, rv)) {
         informBlocker(msgPass, blocked, LCA_MRIGHT, neighLead, neighFollow);
         if (neighLead.second > 0 && neighLead.second > leader.second) {
-            myVSafes.push_back(myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), neighLead.second, neighLead.first->getSpeed(), neighLead.first->getCarFollowModel().getMaxDecel()) - (SUMOReal) 0.5);
+            myVSafes.push_back(myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), neighLead.second, neighLead.first->getSpeed(), neighLead.first->getCarFollowModel().getMaxDecel()) - (double) 0.5);
         }
 
         // letting vehicles merge in at the end of the lane in case of counter-lane change, step#1, right
@@ -213,8 +195,8 @@ MSLCM_DK2008::wantsChangeToRight(MSAbstractLaneChangeModel::MSLCMessager& msgPas
     // this rule prevents the vehicle from moving in opposite direction of the best lane
     //  unless the way till the end where the vehicle has to be on the best lane
     //  is long enough
-    SUMOReal maxJam = MAX2(preb[currIdx - 1].occupation, preb[currIdx].occupation);
-    SUMOReal neighLeftPlace = MAX2((SUMOReal) 0, neighDist - myVehicle.getPositionOnLane() - maxJam);
+    double maxJam = MAX2(preb[currIdx - 1].occupation, preb[currIdx].occupation);
+    double neighLeftPlace = MAX2((double) 0, neighDist - myVehicle.getPositionOnLane() - maxJam);
     if (bestLaneOffset >= 0 && (currentDistDisallows(neighLeftPlace, bestLaneOffset + 2, rv))) {
         // ...we will not change the lane if not
         return ret | LCA_STAY | LCA_STRATEGIC;
@@ -257,11 +239,11 @@ MSLCM_DK2008::wantsChangeToRight(MSAbstractLaneChangeModel::MSLCMessager& msgPas
     // --------
 
     // -------- higher speed
-    if ((congested(neighLead.first) && neighLead.second < 20) || predInteraction(leader.first)) { //!!!
+    if ((congested(neighLead.first) && neighLead.second < 20) || predInteraction(leader)) { //!!!
         return ret;
     }
-    SUMOReal thisLaneVSafe = myVehicle.getLane()->getVehicleMaxSpeed(&myVehicle);
-    SUMOReal neighLaneVSafe = neighLane.getVehicleMaxSpeed(&myVehicle);
+    double thisLaneVSafe = myVehicle.getLane()->getVehicleMaxSpeed(&myVehicle);
+    double neighLaneVSafe = neighLane.getVehicleMaxSpeed(&myVehicle);
     if (neighLead.first == 0) {
         neighLaneVSafe = MIN2(neighLaneVSafe, myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), neighDist, 0, 0));
     } else {
@@ -288,13 +270,13 @@ MSLCM_DK2008::wantsChangeToRight(MSAbstractLaneChangeModel::MSLCMessager& msgPas
     }
 
     // let's recheck the "Rechtsfahrgebot"
-    SUMOReal vmax = MIN2(myVehicle.getLane()->getVehicleMaxSpeed(&myVehicle), myVehicle.getVehicleType().getMaxSpeed());
-    vmax -= (SUMOReal)(5. / 2.6);
+    double vmax = MIN2(myVehicle.getLane()->getVehicleMaxSpeed(&myVehicle), myVehicle.getVehicleType().getMaxSpeed());
+    vmax -= (double)(5. / 2.6);
     if (neighLaneVSafe >= vmax) {
         myChangeProbability -= TS * ((neighLaneVSafe - vmax) / (vmax));
     }
 
-    if (myChangeProbability < -2 && neighDist / MAX2((SUMOReal) .1, myVehicle.getSpeed()) > 20.) { //./MAX2((SUMOReal) .1, myVehicle.getSpeed())) { // -.1
+    if (myChangeProbability < -2 && neighDist / MAX2((double) .1, myVehicle.getSpeed()) > 20.) { //./MAX2((double) .1, myVehicle.getSpeed())) { // -.1
         return ret | LCA_RIGHT | LCA_SPEEDGAIN;
     }
     // --------
@@ -306,28 +288,23 @@ MSLCM_DK2008::wantsChangeToRight(MSAbstractLaneChangeModel::MSLCMessager& msgPas
 int
 MSLCM_DK2008::wantsChangeToLeft(MSAbstractLaneChangeModel::MSLCMessager& msgPass,
                                 int blocked,
-                                const std::pair<MSVehicle*, SUMOReal>& leader,
-                                const std::pair<MSVehicle*, SUMOReal>& neighLead,
-                                const std::pair<MSVehicle*, SUMOReal>& neighFollow,
+                                const std::pair<MSVehicle*, double>& leader,
+                                const std::pair<MSVehicle*, double>& neighLead,
+                                const std::pair<MSVehicle*, double>& neighFollow,
                                 const MSLane& neighLane,
                                 const std::vector<MSVehicle::LaneQ>& preb,
                                 MSVehicle** lastBlocked,
                                 MSVehicle** firstBlocked) {
     UNUSED_PARAMETER(firstBlocked);
-#ifdef DEBUG_VEHICLE_GUI_SELECTION
-    if (gDebugSelectedVehicle == myVehicle.getID()) {
-        int bla = 0;
-    }
-#endif
     MSVehicle::LaneQ curr, best;
     int bestLaneOffset = 0;
-    SUMOReal currentDist = 0;
-    SUMOReal neighDist = 0;
-    SUMOReal neighExtDist = 0;
-    SUMOReal currExtDist = 0;
+    double currentDist = 0;
+    double neighDist = 0;
+    double neighExtDist = 0;
+    double currExtDist = 0;
     int currIdx = 0;
-    MSLane* prebLane = myVehicle.getLane();
-    if (prebLane->getEdge().getPurpose() == MSEdge::EDGEFUNCTION_INTERNAL) {
+    const MSLane* prebLane = myVehicle.getLane();
+    if (prebLane->getEdge().isInternal()) {
         // internal edges are not kept inside the bestLanes structure
         prebLane = prebLane->getLinkCont()[0]->getLane();
     }
@@ -365,8 +342,8 @@ MSLCM_DK2008::wantsChangeToLeft(MSAbstractLaneChangeModel::MSLCMessager& msgPass
 
     // process information about the last blocked vehicle
     //  if this vehicle is blocking someone in front, we maybe decelerate to let him in
-    if ((*lastBlocked) != 0) {
-        SUMOReal gap = (*lastBlocked)->getPositionOnLane() - (*lastBlocked)->getVehicleType().getLength() - myVehicle.getPositionOnLane() - myVehicle.getVehicleType().getMinGap();
+    if ((*lastBlocked) != nullptr) {
+        double gap = (*lastBlocked)->getPositionOnLane() - (*lastBlocked)->getVehicleType().getLength() - myVehicle.getPositionOnLane() - myVehicle.getVehicleType().getMinGap();
         if (gap > 0.1) {
             if (myVehicle.getSpeed() < ACCEL2SPEED(myVehicle.getCarFollowModel().getMaxDecel())) {
                 if ((*lastBlocked)->getSpeed() < SUMO_const_haltingSpeed) {
@@ -374,8 +351,8 @@ MSLCM_DK2008::wantsChangeToLeft(MSAbstractLaneChangeModel::MSLCMessager& msgPass
                 } else {
                     ret |= LCA_AMBACKBLOCKER;
                 }
-                myVSafes.push_back(myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), (SUMOReal)(gap - 0.1), (*lastBlocked)->getSpeed(), (*lastBlocked)->getCarFollowModel().getMaxDecel()));
-                (*lastBlocked) = 0;
+                myVSafes.push_back(myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), (double)(gap - 0.1), (*lastBlocked)->getSpeed(), (*lastBlocked)->getCarFollowModel().getMaxDecel()));
+                (*lastBlocked) = nullptr;
             }
             return ret;
         }
@@ -390,19 +367,19 @@ MSLCM_DK2008::wantsChangeToLeft(MSAbstractLaneChangeModel::MSLCMessager& msgPass
     //  definitely try to get to the desired lane
     //
     // this rule forces our vehicle to change the lane if a lane changing is necessary soon
-    SUMOReal lv = myVehicle.getSpeed() > LOOK_FORWARD_SPEED_DIVIDER
-                  ? myVehicle.getSpeed() * (SUMOReal) LOOK_FORWARD_FAR
-                  : myVehicle.getSpeed() * (SUMOReal) LOOK_FORWARD_NEAR;
-    lv += myVehicle.getVehicleType().getLengthWithGap() * (SUMOReal) 2.;
+    double lv = myVehicle.getSpeed() > LOOK_FORWARD_SPEED_DIVIDER
+                ? myVehicle.getSpeed() * (double) LOOK_FORWARD_FAR
+                : myVehicle.getSpeed() * (double) LOOK_FORWARD_NEAR;
+    lv += myVehicle.getVehicleType().getLengthWithGap() * (double) 2.;
 
 
-    SUMOReal tdist = currentDist - myVehicle.getPositionOnLane() - best.occupation * (SUMOReal) JAM_FACTOR2;
-    if (fabs(best.length - curr.length) > MIN2((SUMOReal) .1, best.lane->getLength()) && bestLaneOffset > 0
+    double tdist = currentDist - myVehicle.getPositionOnLane() - best.occupation * (double) JAM_FACTOR2;
+    if (fabs(best.length - curr.length) > MIN2((double) .1, best.lane->getLength()) && bestLaneOffset > 0
             &&
             currentDistDisallows(tdist/*currentDist*/, bestLaneOffset, lv)) {
         informBlocker(msgPass, blocked, LCA_MLEFT, neighLead, neighFollow);
         if (neighLead.second > 0 && neighLead.second > leader.second) {
-            myVSafes.push_back(myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), neighLead.second, neighLead.first->getSpeed(), neighLead.first->getCarFollowModel().getMaxDecel()) - (SUMOReal) 0.5);
+            myVSafes.push_back(myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), neighLead.second, neighLead.first->getSpeed(), neighLead.first->getCarFollowModel().getMaxDecel()) - (double) 0.5);
         }
 
         // letting vehicles merge in at the end of the lane in case of counter-lane change, step#1, left
@@ -425,8 +402,8 @@ MSLCM_DK2008::wantsChangeToLeft(MSAbstractLaneChangeModel::MSLCMessager& msgPass
     // this rule prevents the vehicle from moving in opposite direction of the best lane
     //  unless the way till the end where the vehicle has to be on the best lane
     //  is long enough
-    SUMOReal maxJam = MAX2(preb[currIdx + 1].occupation, preb[currIdx].occupation);
-    SUMOReal neighLeftPlace = MAX2((SUMOReal) 0, neighDist - myVehicle.getPositionOnLane() - maxJam);
+    double maxJam = MAX2(preb[currIdx + 1].occupation, preb[currIdx].occupation);
+    double neighLeftPlace = MAX2((double) 0, neighDist - myVehicle.getPositionOnLane() - maxJam);
     if (bestLaneOffset <= 0 && (currentDistDisallows(neighLeftPlace, bestLaneOffset - 2, lv))) {
         // ...we will not change the lane if not
         return ret | LCA_STAY | LCA_STRATEGIC;
@@ -476,11 +453,11 @@ MSLCM_DK2008::wantsChangeToLeft(MSAbstractLaneChangeModel::MSLCMessager& msgPass
     }
 
     // -------- higher speed
-    if ((congested(neighLead.first) && neighLead.second < 20) || predInteraction(leader.first)) { //!!!
+    if ((congested(neighLead.first) && neighLead.second < 20) || predInteraction(leader)) { //!!!
         return ret;
     }
-    SUMOReal neighLaneVSafe = neighLane.getVehicleMaxSpeed(&myVehicle);
-    SUMOReal thisLaneVSafe = myVehicle.getLane()->getVehicleMaxSpeed(&myVehicle);
+    double neighLaneVSafe = neighLane.getVehicleMaxSpeed(&myVehicle);
+    double thisLaneVSafe = myVehicle.getLane()->getVehicleMaxSpeed(&myVehicle);
     if (neighLead.first == 0) {
         neighLaneVSafe = MIN2(neighLaneVSafe, myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), neighDist, 0, 0)); // !!! warum nicht die Folgesgeschw.?
     } else {
@@ -504,7 +481,7 @@ MSLCM_DK2008::wantsChangeToLeft(MSAbstractLaneChangeModel::MSLCMessager& msgPass
         // right lane is better
         myChangeProbability += TS * ((neighLaneVSafe - thisLaneVSafe) / (myVehicle.getLane()->getVehicleMaxSpeed(&myVehicle))); // !!! Fahrzeuggeschw.!
     }
-    if (myChangeProbability > .2 && neighDist / MAX2((SUMOReal) .1, myVehicle.getSpeed()) > 20.) { // .1
+    if (myChangeProbability > .2 && neighDist / MAX2((double) .1, myVehicle.getSpeed()) > 20.) { // .1
         return ret | LCA_LEFT | LCA_SPEEDGAIN | LCA_URGENT;
     }
     // --------
@@ -513,23 +490,18 @@ MSLCM_DK2008::wantsChangeToLeft(MSAbstractLaneChangeModel::MSLCMessager& msgPass
 }
 
 
-SUMOReal
-MSLCM_DK2008::patchSpeed(const SUMOReal min, const SUMOReal wanted, const SUMOReal max, const MSCFModel& cfModel) {
-#ifdef DEBUG_VEHICLE_GUI_SELECTION
-    if (gDebugSelectedVehicle == myVehicle.getID()) {
-        int bla = 0;
-    }
-#endif
+double
+MSLCM_DK2008::patchSpeed(const double min, const double wanted, const double max, const MSCFModel& cfModel) {
     int state = myOwnState;
 
     // letting vehicles merge in at the end of the lane in case of counter-lane change, step#2
-    SUMOReal MAGIC_offset = 1.;
+    double MAGIC_offset = 1.;
     //   if we want to change and have a blocking leader and there is enough room for him in front of us
     if (myLeadingBlockerLength != 0) {
-        SUMOReal space = myLeftSpace - myLeadingBlockerLength - MAGIC_offset - myVehicle.getVehicleType().getMinGap();
+        double space = myLeftSpace - myLeadingBlockerLength - MAGIC_offset - myVehicle.getVehicleType().getMinGap();
         if (space > 0) {
             // compute speed for decelerating towards a place which allows the blocking leader to merge in in front
-            SUMOReal safe = cfModel.stopSpeed(&myVehicle, myVehicle.getSpeed(), space);
+            double safe = cfModel.stopSpeed(&myVehicle, myVehicle.getSpeed(), space);
             // if we are approaching this place
             if (safe < wanted) {
                 // return this speed as the speed to use
@@ -539,16 +511,16 @@ MSLCM_DK2008::patchSpeed(const SUMOReal min, const SUMOReal wanted, const SUMORe
     }
 
     // just to make sure to be notified about lane chaning end
-    if (myVehicle.getLane()->getEdge().getLanes().size() == 1 || myVehicle.getLane()->getEdge().getPurpose() == MSEdge::EDGEFUNCTION_INTERNAL) {
+    if (myVehicle.getLane()->getEdge().getLanes().size() == 1 || myVehicle.getLane()->getEdge().isInternal()) {
         // remove chaning information if on a road with a single lane
-        changed(0);
+        changed();
         return wanted;
     }
 
-    SUMOReal nVSafe = wanted;
+    double nVSafe = wanted;
     bool gotOne = false;
-    for (std::vector<SUMOReal>::const_iterator i = myVSafes.begin(); i != myVSafes.end(); ++i) {
-        SUMOReal v = (*i);
+    for (std::vector<double>::const_iterator i = myVSafes.begin(); i != myVSafes.end(); ++i) {
+        double v = (*i);
         if (v >= min && v <= max) {
             nVSafe = MIN2(v, nVSafe);
             gotOne = true;
@@ -565,12 +537,12 @@ MSLCM_DK2008::patchSpeed(const SUMOReal min, const SUMOReal wanted, const SUMORe
         if ((state & LCA_BLOCKED) != 0) {
             if ((state & LCA_BLOCKED_BY_LEADER) != 0) {
                 // if interacting with leader and not too slow
-                return (min + wanted) / (SUMOReal) 2.0;
+                return (min + wanted) / (double) 2.0;
             }
             if ((state & LCA_BLOCKED_BY_FOLLOWER) != 0) {
-                return (max + wanted) / (SUMOReal) 2.0;
+                return (max + wanted) / (double) 2.0;
             }
-            return (min + wanted) / (SUMOReal) 2.0;
+            return (min + wanted) / (double) 2.0;
         }
     }
 
@@ -581,7 +553,7 @@ MSLCM_DK2008::patchSpeed(const SUMOReal min, const SUMOReal wanted, const SUMORe
         if (fabs(max - myVehicle.getCarFollowModel().maxNextSpeed(myVehicle.getSpeed(), &myVehicle)) < 0.001 && min == 0) { // !!! was standing
             return 0;
         }
-        return (min + wanted) / (SUMOReal) 2.0;
+        return (min + wanted) / (double) 2.0;
     }
     if ((state & LCA_AMBACKBLOCKER) != 0) {
         if (max <= myVehicle.getCarFollowModel().maxNextSpeed(myVehicle.getSpeed(), &myVehicle) && min == 0) { // !!! was standing
@@ -594,13 +566,13 @@ MSLCM_DK2008::patchSpeed(const SUMOReal min, const SUMOReal wanted, const SUMORe
     // accelerate if being a blocking leader or blocking follower not able to brake
     //  (and does not have to change lanes)
     if ((state & LCA_AMBLOCKINGLEADER) != 0) {
-        return (max + wanted) / (SUMOReal) 2.0;
+        return (max + wanted) / (double) 2.0;
     }
     if ((state & LCA_AMBLOCKINGFOLLOWER_DONTBRAKE) != 0) {
         if (max <= myVehicle.getCarFollowModel().maxNextSpeed(myVehicle.getSpeed(), &myVehicle) && min == 0) { // !!! was standing
             return wanted;
         }
-        return (min + wanted) / (SUMOReal) 2.0;
+        return (min + wanted) / (double) 2.0;
     }
     return wanted;
 }
@@ -617,14 +589,13 @@ MSLCM_DK2008::inform(void* info, MSVehicle* /*sender*/) {
 
 
 void
-MSLCM_DK2008::changed(int dir) {
+MSLCM_DK2008::changed() {
     myOwnState = 0;
     myChangeProbability = 0;
     myLeadingBlockerLength = 0;
     myLeftSpace = 0;
     myVSafes.clear();
     myDontBrake = false;
-    initLastLaneChangeOffset(dir);
 }
 
 
@@ -632,20 +603,21 @@ void
 MSLCM_DK2008::informBlocker(MSAbstractLaneChangeModel::MSLCMessager& msgPass,
                             int& blocked,
                             int dir,
-                            const std::pair<MSVehicle*, SUMOReal>& neighLead,
-                            const std::pair<MSVehicle*, SUMOReal>& neighFollow) {
+                            const std::pair<MSVehicle*, double>& neighLead,
+                            const std::pair<MSVehicle*, double>& neighFollow) {
     if ((blocked & LCA_BLOCKED_BY_FOLLOWER) != 0) {
         assert(neighFollow.first != 0);
         MSVehicle* nv = neighFollow.first;
-        SUMOReal decelGap =
+        double decelGap =
             neighFollow.second
-            + SPEED2DIST(myVehicle.getSpeed()) * (SUMOReal) 2.0
-            - MAX2(nv->getSpeed() - (SUMOReal) ACCEL2DIST(nv->getCarFollowModel().getMaxDecel()) * (SUMOReal) 2.0, (SUMOReal) 0);
-        if (neighFollow.second > 0 && decelGap > 0 && decelGap >= nv->getCarFollowModel().getSecureGap(nv->getSpeed(), myVehicle.getSpeed(), myVehicle.getCarFollowModel().getMaxDecel())) {
-            SUMOReal vsafe = myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), neighFollow.second, neighFollow.first->getSpeed(), neighFollow.first->getCarFollowModel().getMaxDecel());
+            + SPEED2DIST(myVehicle.getSpeed()) * (double) 2.0
+            - MAX2(nv->getSpeed() - (double) ACCEL2DIST(nv->getCarFollowModel().getMaxDecel()) * (double) 2.0, (double) 0);
+        if (neighFollow.second > 0 && decelGap > 0 && decelGap >= nv->getCarFollowModel().getSecureGap(nv, &myVehicle,
+                nv->getSpeed(), myVehicle.getSpeed(), myVehicle.getCarFollowModel().getMaxDecel())) {
+            double vsafe = myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), neighFollow.second, neighFollow.first->getSpeed(), neighFollow.first->getCarFollowModel().getMaxDecel());
             msgPass.informNeighFollower(new Info(vsafe, dir | LCA_AMBLOCKINGFOLLOWER), &myVehicle);
         } else {
-            SUMOReal vsafe = neighFollow.second <= 0 ? 0 : myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), neighFollow.second, neighFollow.first->getSpeed(), neighFollow.first->getCarFollowModel().getMaxDecel());
+            double vsafe = neighFollow.second <= 0 ? 0 : myCarFollowModel.followSpeed(&myVehicle, myVehicle.getSpeed(), neighFollow.second, neighFollow.first->getSpeed(), neighFollow.first->getCarFollowModel().getMaxDecel());
             msgPass.informNeighFollower(new Info(vsafe, dir | LCA_AMBLOCKINGFOLLOWER_DONTBRAKE), &myVehicle);
         }
     }
@@ -659,6 +631,7 @@ MSLCM_DK2008::informBlocker(MSAbstractLaneChangeModel::MSLCMessager& msgPass,
 
 void
 MSLCM_DK2008::prepareStep() {
+    MSAbstractLaneChangeModel::prepareStep();
     myOwnState = 0;
     myLeadingBlockerLength = 0;
     myLeftSpace = 0;
@@ -670,4 +643,3 @@ MSLCM_DK2008::prepareStep() {
 
 
 /****************************************************************************/
-
