@@ -1,38 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    FunctionBinding.h
 /// @author  Daniel Krajzewicz
 /// @author  Christian Roessel
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Fri, 29.04.2005
-/// @version $Id: FunctionBinding.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 //  Function type template
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef FunctionBinding_h
-#define FunctionBinding_h
-
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <utils/common/ValueSource.h>
 #include "CastingFunctionBinding.h"
@@ -86,8 +75,41 @@ private:
 
 };
 
+template<class T>
+class FunctionBindingString : public ValueSource<std::string> {
+public:
+    /// Type of the function to execute.
+    typedef std::string(T::* Operation)() const;
 
-#endif
+    FunctionBindingString(T* const source, Operation operation) :
+        mySource(source),
+        myOperation(operation)
+    {}
 
-/****************************************************************************/
+    /// Destructor.
+    ~FunctionBindingString() {}
 
+    std::string getValue() const {
+        return (mySource->*myOperation)();
+    }
+
+    ValueSource<std::string>* copy() const {
+        return new FunctionBindingString<T>(mySource, myOperation);
+    }
+
+    ValueSource<double>* makedoubleReturningCopy() const {
+        return nullptr;
+    }
+
+private:
+    /// The object the action is directed to.
+    T* mySource;
+
+    /// The object's operation to perform.
+    Operation myOperation;
+
+private:
+    /// @brief invalidated assignment operator
+    FunctionBindingString<T>& operator=(const FunctionBindingString<T>&);
+
+};
