@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSCrossSection.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Sascha Krieg
 /// @date    Tue Nov 25 15:23:28 2003
-/// @version $Id: MSCrossSection.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A simple description of a position on a lane (crossing of a lane)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSCrossSection_h
-#define MSCrossSection_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 
@@ -55,7 +45,7 @@ public:
      * @param[in] lane The lane to cross
      * @param[in] pos The position at the lane
      */
-    MSCrossSection(MSLane* lane, SUMOReal pos) : myLane(lane) , myPosition(pos) {}
+    MSCrossSection(MSLane* lane, double pos) : myLane(lane), myPosition(pos) {}
 
 
 public:
@@ -63,7 +53,7 @@ public:
     MSLane* myLane;
 
     /// @brief The position at the lane
-    SUMOReal myPosition;
+    double myPosition;
 
 };
 
@@ -71,9 +61,3 @@ public:
 typedef std::vector< MSCrossSection > CrossSectionVector;
 typedef CrossSectionVector::iterator CrossSectionVectorIt;
 typedef CrossSectionVector::const_iterator CrossSectionVectorConstIt;
-
-
-#endif
-
-/****************************************************************************/
-
