@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSDetectorControl.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -7,31 +19,10 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    2005-09-15
-/// @version $Id: MSDetectorControl.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Detectors container; responsible for string and output generation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include "MSDetectorControl.h"
@@ -40,14 +31,6 @@
 #include <utils/options/Option.h>
 #include <utils/common/MsgHandler.h>
 
-#ifdef HAVE_INTERNAL
-#include <mesosim/MEInductLoop.h>
-#endif
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // member method definitions
@@ -57,12 +40,15 @@ MSDetectorControl::MSDetectorControl() {
 
 
 MSDetectorControl::~MSDetectorControl() {
-    for (std::map<SumoXMLTag, NamedObjectCont<MSDetectorFileOutput*> >::iterator i = myDetectors.begin(); i != myDetectors.end(); ++i) {
+    for (auto i = myDetectors.begin(); i != myDetectors.end(); ++i) {
         (*i).second.clear();
     }
-    for (std::vector<MSMeanData*>::const_iterator i = myMeanData.begin(); i != myMeanData.end(); ++i) {
-        delete *i;
+    for (auto item : myMeanData) {
+        for (MSMeanData* md : item.second) {
+            delete md;
+        }
     }
+    myMeanData.clear();
 }
 
 
@@ -76,11 +62,11 @@ MSDetectorControl::close(SUMOTime step) {
 
 
 void
-MSDetectorControl::add(SumoXMLTag type, MSDetectorFileOutput* d, const std::string& device, int splInterval, SUMOTime begin) {
+MSDetectorControl::add(SumoXMLTag type, MSDetectorFileOutput* d, const std::string& device, SUMOTime interval, SUMOTime begin) {
     if (!myDetectors[type].add(d->getID(), d)) {
         throw ProcessError(toString(type) + " detector '" + d->getID() + "' could not be build (declared twice?).");
     }
-    addDetectorAndInterval(d, &OutputDevice::getDevice(device), splInterval, begin);
+    addDetectorAndInterval(d, &OutputDevice::getDevice(device), interval, begin);
 }
 
 
@@ -95,12 +81,12 @@ MSDetectorControl::add(SumoXMLTag type, MSDetectorFileOutput* d) {
 
 
 void
-MSDetectorControl::add(MSMeanData* mn, const std::string& device,
+MSDetectorControl::add(MSMeanData* md, const std::string& device,
                        SUMOTime frequency, SUMOTime begin) {
-    myMeanData.push_back(mn);
-    addDetectorAndInterval(mn, &OutputDevice::getDevice(device), frequency, begin);
-    if (begin == string2time(OptionsCont::getOptions().getString("begin"))) {
-        mn->init();
+    myMeanData[md->getID()].push_back(md);
+    addDetectorAndInterval(md, &OutputDevice::getDevice(device), frequency, begin);
+    if (begin <= string2time(OptionsCont::getOptions().getString("begin"))) {
+        md->init();
     }
 }
 
@@ -126,14 +112,15 @@ MSDetectorControl::getTypedDetectors(SumoXMLTag type) const {
 
 void
 MSDetectorControl::updateDetectors(const SUMOTime step) {
-    for (std::map<SumoXMLTag, NamedObjectCont<MSDetectorFileOutput*> >::const_iterator i = myDetectors.begin(); i != myDetectors.end(); ++i) {
-        const std::map<std::string, MSDetectorFileOutput*>& dets = getTypedDetectors((*i).first).getMyMap();
-        for (std::map<std::string, MSDetectorFileOutput*>::const_iterator j = dets.begin(); j != dets.end(); ++j) {
-            (*j).second->detectorUpdate(step);
+    for (const auto& i : myDetectors) {
+        for (const auto& j : getTypedDetectors(i.first)) {
+            j.second->detectorUpdate(step);
         }
     }
-    for (std::vector<MSMeanData*>::const_iterator i = myMeanData.begin(); i != myMeanData.end(); ++i) {
-        (*i)->detectorUpdate(step);
+    for (auto item : myMeanData) {
+        for (MSMeanData* md : item.second) {
+            md->detectorUpdate(step);
+        }
     }
 }
 
@@ -161,8 +148,9 @@ MSDetectorControl::addDetectorAndInterval(MSDetectorFileOutput* det,
         OutputDevice* device,
         SUMOTime interval,
         SUMOTime begin) {
+    const SUMOTime simBegin = string2time(OptionsCont::getOptions().getString("begin"));
     if (begin == -1) {
-        begin = string2time(OptionsCont::getOptions().getString("begin"));
+        begin = simBegin;
     }
     IntervalsKey key = std::make_pair(interval, begin);
     Intervals::iterator it = myIntervals.find(key);
@@ -171,10 +159,17 @@ MSDetectorControl::addDetectorAndInterval(MSDetectorFileOutput* det,
         DetectorFileVec detAndFileVec;
         detAndFileVec.push_back(std::make_pair(det, device));
         myIntervals.insert(std::make_pair(key, detAndFileVec));
-        myLastCalls[key] = begin;
+        SUMOTime lastCall = begin;
+        if (begin < simBegin) {
+            SUMOTime divRest = (simBegin - begin) % interval;
+            lastCall = simBegin - divRest;
+        }
+        myLastCalls[key] = lastCall;
     } else {
         DetectorFileVec& detAndFileVec = it->second;
-        if (find_if(detAndFileVec.begin(), detAndFileVec.end(), bind2nd(detectorEquals(), det)) == detAndFileVec.end()) {
+        if (find_if(detAndFileVec.begin(), detAndFileVec.end(), [&](const DetectorFilePair & pair) {
+        return pair.first == det;
+    }) == detAndFileVec.end()) {
             detAndFileVec.push_back(std::make_pair(det, device));
         } else {
             // detector already in container. Don't add several times
@@ -185,7 +180,13 @@ MSDetectorControl::addDetectorAndInterval(MSDetectorFileOutput* det,
     det->writeXMLDetectorProlog(*device);
 }
 
-
+void
+MSDetectorControl::clearState() {
+    for (const auto& i : myDetectors) {
+        for (const auto& j : getTypedDetectors(i.first)) {
+            j.second->clearState();
+        }
+    }
+}
 
 /****************************************************************************/
-
