@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSDetectorFileOutput.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    2004-11-23
-/// @version $Id: MSDetectorFileOutput.h 22608 2017-01-17 06:28:54Z behrisch $
 ///
 // Base of value-generating classes (detectors)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSDetectorFileOutput_h
-#define MSDetectorFileOutput_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <set>
@@ -40,8 +30,10 @@
 #include <utils/common/SUMOTime.h>
 #include <utils/common/StringTokenizer.h>
 #include <utils/iodevices/OutputDevice.h>
-#include <utils/vehicle/SUMOVehicle.h>
+#include <utils/vehicle/SUMOTrafficObject.h>
 #include <microsim/MSVehicleType.h>
+#include <microsim/MSVehicleControl.h>
+#include <microsim/MSNet.h>
 
 
 // ===========================================================================
@@ -69,16 +61,17 @@ enum DetectorUsage {
 class MSDetectorFileOutput : public Named {
 public:
     /// @brief Constructor
-    MSDetectorFileOutput(const std::string& id, const std::string& vTypes)
-        : Named(id) {
+    MSDetectorFileOutput(const std::string& id, const std::string& vTypes, const int detectPersons = false) :
+        Named(id),
+        myDetectPersons(detectPersons) {
         const std::vector<std::string> vt = StringTokenizer(vTypes).getVector();
         myVehicleTypes.insert(vt.begin(), vt.end());
     }
 
     /// @brief Constructor
-    MSDetectorFileOutput(const std::string& id, const std::set<std::string>& vTypes)
-        : Named(id), myVehicleTypes(vTypes) {
-    }
+    MSDetectorFileOutput(const std::string& id, const std::set<std::string>& vTypes, const int detectPersons = false)
+        : Named(id), myVehicleTypes(vTypes), myDetectPersons(detectPersons)
+    { }
 
 
     /// @brief (virtual) destructor
@@ -143,8 +136,20 @@ public:
     * @param[in] veh the vehicle of which the type is checked.
     * @return whether it should be measured
     */
-    bool vehicleApplies(const SUMOVehicle& veh) const {
-        return myVehicleTypes.empty() || myVehicleTypes.count(veh.getVehicleType().getID()) > 0;
+    bool vehicleApplies(const SUMOTrafficObject& veh) const {
+        if (veh.isVehicle() == detectPersons()) {
+            return false;
+        } else if (myVehicleTypes.empty() || myVehicleTypes.count(veh.getVehicleType().getOriginalID()) > 0) {
+            return true;
+        } else {
+            std::set<std::string> vTypeDists = MSNet::getInstance()->getVehicleControl().getVTypeDistributionMembership(veh.getVehicleType().getOriginalID());
+            for (auto vTypeDist : vTypeDists) {
+                if (myVehicleTypes.count(vTypeDist) > 0) {
+                    return true;
+                }
+            }
+            return false;
+        }
     }
 
 
@@ -156,11 +161,24 @@ public:
         return !myVehicleTypes.empty();
     }
 
+    const std::set<std::string>& getVehicleTypes() const {
+        return myVehicleTypes;
+    }
+
+    inline bool detectPersons() const {
+        return myDetectPersons != 0;
+    }
+
+    /** @brief Remove all vehicles before quick-loading state */
+    virtual void clearState() {};
 
 protected:
     /// @brief The vehicle types to look for (empty means all)
     std::set<std::string> myVehicleTypes;
 
+    /// @brief Whether pedestrians shall be detected instead of vehicles
+    const int myDetectPersons;
+
 private:
     /// @brief Invalidated copy constructor.
     MSDetectorFileOutput(const MSDetectorFileOutput&);
@@ -170,9 +188,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
