@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2003-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSE3Collector.h
 /// @author  Christian Roessel
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Tue Dec 02 2003 22:17 CET
-/// @version $Id: MSE3Collector.h 21201 2016-07-19 11:57:22Z behrisch $
 ///
 // A detector of vehicles passing an area between entry/exit points
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2003-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSE3Collector_h
-#define MSE3Collector_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -41,12 +31,15 @@
 #include <utils/common/Named.h>
 #include <microsim/output/MSCrossSection.h>
 #include <utils/common/UtilExceptions.h>
+#ifdef HAVE_FOX
+#include <utils/foxtools/fxheader.h>
+#endif
 
 
 // ===========================================================================
 // class declarations
 // ===========================================================================
-class SUMOVehicle;
+class SUMOTrafficObject;
 class OutputDevice;
 
 
@@ -62,7 +55,7 @@ class OutputDevice;
  *  out-cross-section. Vehicles passing the out-cross-section without having
  *  passed the in-cross-section are not detected.
  */
-class MSE3Collector : public MSDetectorFileOutput {
+class MSE3Collector : public MSDetectorFileOutput, public Parameterised {
 public:
     /**
      * @class MSE3EntryReminder
@@ -80,6 +73,17 @@ public:
 
         /// @name Methods inherited from MSMoveReminder.
         /// @{
+        /** @brief Checks whether the reminder is activated by a vehicle entering the lane
+         *
+         * Lane change means in this case that the vehicle changes to the lane
+         *  the reminder is placed at.
+         *
+         * @param[in] veh The entering vehicle.
+         * @param[in] reason how the vehicle enters the lane
+         * @return True if vehicle enters the reminder.
+         * @see Notification
+         */
+        bool notifyEnter(SUMOTrafficObject& veh, Notification reason, const MSLane* enteredLane);
 
         /** @brief Checks whether the vehicle enters
          *
@@ -96,7 +100,7 @@ public:
          * @see MSMoveReminder::notifyMove
          * @see MSE3Collector::enter
          */
-        bool notifyMove(SUMOVehicle& veh, SUMOReal , SUMOReal newPos, SUMOReal);
+        bool notifyMove(SUMOTrafficObject& veh, double, double newPos, double);
 
 
         /** @brief Processes state changes of a vehicle
@@ -109,7 +113,7 @@ public:
         * @param[in] reason The reason for the state change
         * @see MSMoveReminder::notifyLeave
         */
-        bool notifyLeave(SUMOVehicle& veh, SUMOReal lastPos, MSMoveReminder::Notification reason);
+        bool notifyLeave(SUMOTrafficObject& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
         /// @}
 
 
@@ -118,7 +122,7 @@ public:
         MSE3Collector& myCollector;
 
         /// @brief The position on the lane
-        SUMOReal myPosition;
+        double myPosition;
 
     private:
         /// @brief Invalidated copy constructor.
@@ -147,6 +151,17 @@ public:
 
         /// @name methods from MSMoveReminder
         //@{
+        /** @brief Checks whether the reminder is activated by a vehicle entering the lane
+         *
+         * Lane change means in this case that the vehicle changes to the lane
+         *  the reminder is placed at.
+         *
+         * @param[in] veh The entering vehicle.
+         * @param[in] reason how the vehicle enters the lane
+         * @return True if vehicle enters the reminder.
+         * @see Notification
+         */
+        bool notifyEnter(SUMOTrafficObject& veh, Notification reason, const MSLane* enteredLane);
 
         /** @brief Checks whether the vehicle leaves
          *
@@ -163,7 +178,7 @@ public:
          * @see MSMoveReminder::notifyMove
          * @see MSE3Collector::leave
          */
-        bool notifyMove(SUMOVehicle& veh, SUMOReal oldPos, SUMOReal newPos, SUMOReal);
+        bool notifyMove(SUMOTrafficObject& veh, double oldPos, double newPos, double);
 
         /** @brief Processes state changes of a vehicle
         *
@@ -174,7 +189,7 @@ public:
         * @param[in] reason The reason for the state change
         * @see MSMoveReminder::notifyLeave
         */
-        bool notifyLeave(SUMOVehicle& veh, SUMOReal lastPos, MSMoveReminder::Notification reason);
+        bool notifyLeave(SUMOTrafficObject& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
         //@}
 
 
@@ -183,7 +198,7 @@ public:
         MSE3Collector& myCollector;
 
         /// @brief The position on the lane
-        SUMOReal myPosition;
+        double myPosition;
 
     private:
         /// @brief Invalidated copy constructor.
@@ -207,8 +222,9 @@ public:
      */
     MSE3Collector(const std::string& id,
                   const CrossSectionVector& entries, const CrossSectionVector& exits,
-                  SUMOReal haltingSpeedThreshold,
-                  SUMOTime haltingTimeThreshold);
+                  double haltingSpeedThreshold,
+                  SUMOTime haltingTimeThreshold,
+                  const std::string& vTypes, bool openEntry);
 
 
     /// @brief Destructor
@@ -228,18 +244,26 @@ public:
      *  @param[in] entryTimestep The time in seconds the vehicle entered the area
      *  @param[in] fractionTimeOnDet The interpolated time in seconds the vehicle already spent on the detector
      */
-    void enter(const SUMOVehicle& veh, const SUMOReal entryTimestep, const SUMOReal fractionTimeOnDet);
+    void enter(const SUMOTrafficObject& veh, const double entryTimestep, const double fractionTimeOnDet, MSE3EntryReminder* entryReminder);
 
 
-    /** @brief Called if a vehicle passes a leave-cross-section.
-     *
-     * Removes vehicle from internal containers.
-     *
-     *  @param[in] veh The vehicle that left the area
-     *  @param[in] leaveTimestep The time in seconds the vehicle left the area
-     *  @param[in] fractionTimeOnDet The interpolated time in seconds the vehicle still spent on the detector
-     */
-    void leave(const SUMOVehicle& veh, const SUMOReal leaveTimestep, const SUMOReal fractionTimeOnDet);
+    /** @brief Called if a vehicle front passes a leave-cross-section.
+    *
+    *  @param[in] veh The vehicle that left the area
+    *  @param[in] leaveTimestep The time in seconds the vehicle started crossing the line
+    */
+    void leaveFront(const SUMOTrafficObject& veh, const double leaveTimestep);
+
+
+    /** @brief Called if a vehicle back passes a leave-cross-section.
+    *
+    * Removes vehicle from internal containers.
+    *
+    *  @param[in] veh The vehicle that left the area
+    *  @param[in] leaveTimestep The time in seconds the vehicle left the area
+    *  @param[in] fractionTimeOnDet The interpolated time in seconds the vehicle still spent on the detector
+    */
+    void leave(const SUMOTrafficObject& veh, const double leaveTimestep, const double fractionTimeOnDet);
 
 
     /// @name Methods returning current values
@@ -251,7 +275,7 @@ public:
      *
      * @return The mean speed [m/s] of all vehicles within the area, -1 if there is none
      */
-    SUMOReal getCurrentMeanSpeed() const;
+    double getCurrentMeanSpeed() const;
 
 
     /** @brief Returns the number of current haltings within the area
@@ -314,6 +338,8 @@ public:
      */
     void detectorUpdate(const SUMOTime step);
 
+    /** @brief Remove all vehicles before quick-loading state */
+    virtual void clearState();
 
 protected:
     /// @brief The detector's entries
@@ -333,7 +359,7 @@ protected:
     SUMOTime myHaltingTimeThreshold;
 
     /// @brief Speed-threshold to determine if a vehicle is halting.
-    SUMOReal myHaltingSpeedThreshold;
+    double myHaltingSpeedThreshold;
 
     /**
      * @struct E3Values
@@ -345,35 +371,47 @@ protected:
      */
     struct E3Values {
         /// @brief The vehicle's entry time
-        SUMOReal entryTime;
-        /// @brief The vehicle's leaving time
-        SUMOReal leaveTime;
+        double entryTime;
+        /// @brief The time the vehicle's front was crossing the leave line
+        double frontLeaveTime;
+        /// @brief The time the vehicle's back was crossing the leave line
+        double backLeaveTime;
         /// @brief The sum of registered speeds the vehicle has/had inside the area
-        SUMOReal speedSum;
+        double speedSum;
         /// @brief The sum of haltings the vehicle has/had within the area
         int haltings;
         /// @brief Begin time of last halt begin
-        SUMOReal haltingBegin;
+        SUMOTime haltingBegin;
         /// @brief The sum of registered speeds the vehicle has/had inside the area during the current interval
-        SUMOReal intervalSpeedSum;
+        double intervalSpeedSum;
         /// @brief The sum of haltings the vehicle has/had within the area during the current interval
         int intervalHaltings;
+        /// @brief The timeLoss of the vehicle when entering. Updated to the actual time loss within the area when leaving
+        SUMOTime timeLoss;
+        /// @brief The timeLoss of the vehicle when entering. Updated to the current timeLoss at interval write
+        SUMOTime intervalTimeLoss;
         /// @brief An internal information whether the update step was performed
         bool hadUpdate;
+        /// @brief the reminder on which the vehicle entered the detector
+        MSE3EntryReminder* entryReminder;
     };
 
     /// @brief Container for vehicles that have entered the area
-    std::map<const SUMOVehicle*, E3Values> myEnteredContainer;
+    std::map<const SUMOTrafficObject*, E3Values> myEnteredContainer;
 
     /// @brief Container for vehicles that have left the area
-    std::map<const SUMOVehicle*, E3Values> myLeftContainer;
+    std::vector<E3Values> myLeftContainer;
 
+#ifdef HAVE_FOX
+    /// @brief the mutex for access to the containers
+    FXMutex myContainerMutex;
+#endif
 
     /// @name Storages for current values
     /// @{
 
     /// @brief The current mean speed of known vehicles (inside)
-    SUMOReal myCurrentMeanSpeed;
+    double myCurrentMeanSpeed;
 
     /// @brief The current number of haltings (inside)
     int myCurrentHaltingsNumber;
@@ -383,6 +421,8 @@ protected:
     /// @brief Information when the last reset has been done
     SUMOTime myLastResetTime;
 
+    /// @brief whether this dector is declared as having incomplete entry detectors
+    const bool myOpenEntry;
 
 private:
     /// @brief Invalidated copy constructor.
@@ -393,9 +433,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
