@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2011-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSInstantInductLoop.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    2011-09.08
-/// @version $Id: MSInstantInductLoop.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // An instantaneous induction loop
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2011-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "MSInstantInductLoop.h"
 #include <cassert>
@@ -66,7 +57,7 @@ MSInstantInductLoop::~MSInstantInductLoop() {
 
 
 bool
-MSInstantInductLoop::notifyMove(SUMOVehicle& veh, double oldPos,
+MSInstantInductLoop::notifyMove(SUMOTrafficObject& veh, double oldPos,
                                 double newPos, double newSpeed) {
     if (!vehicleApplies(veh)) {
         return false;
@@ -75,6 +66,9 @@ MSInstantInductLoop::notifyMove(SUMOVehicle& veh, double oldPos,
         // detector not reached yet
         return true;
     }
+#ifdef HAVE_FOX
+    FXConditionalLock lock(myNotificationMutex, MSGlobals::gNumSimThreads > 1);
+#endif
 
     const double oldSpeed = veh.getPreviousSpeed();
     double enterSpeed = MSGlobals::gSemiImplicitEulerUpdate ? newSpeed : oldSpeed; // NOTE: For the euler update, the vehicle is assumed to travel at constant speed for the whole time step
@@ -93,7 +87,7 @@ MSInstantInductLoop::notifyMove(SUMOVehicle& veh, double oldPos,
     const double newBackPos = newPos - veh.getVehicleType().getLength();
     const double oldBackPos = oldPos - veh.getVehicleType().getLength();
     if (newBackPos > myPosition) {
-        std::map<SUMOVehicle*, double>::iterator i = myEntryTimes.find(&veh);
+        std::map<SUMOTrafficObject*, double>::iterator i = myEntryTimes.find(&veh);
         if (i != myEntryTimes.end()) {
             // vehicle passed the detector
             const double timeBeforeLeave = MSCFModel::passingTime(oldBackPos, myPosition, newBackPos, oldSpeed, newSpeed);
@@ -111,13 +105,13 @@ MSInstantInductLoop::notifyMove(SUMOVehicle& veh, double oldPos,
 
 
 void
-MSInstantInductLoop::write(const char* state, double t, SUMOVehicle& veh, double speed, const char* add, double addValue) {
+MSInstantInductLoop::write(const char* state, double t, SUMOTrafficObject& veh, double speed, const char* add, double addValue) {
     myOutputDevice.openTag("instantOut").writeAttr(
         "id", getID()).writeAttr("time", toString(t)).writeAttr("state", state).writeAttr(
             "vehID", veh.getID()).writeAttr("speed", toString(speed)).writeAttr(
                 "length", toString(veh.getVehicleType().getLength())).writeAttr(
                     "type", veh.getVehicleType().getID());
-    if (add != 0) {
+    if (add != nullptr) {
         myOutputDevice.writeAttr(add, toString(addValue));
     }
     myOutputDevice.closeTag();
@@ -125,13 +119,13 @@ MSInstantInductLoop::write(const char* state, double t, SUMOVehicle& veh, double
 
 
 bool
-MSInstantInductLoop::notifyLeave(SUMOVehicle& veh, double /* lastPos */, MSMoveReminder::Notification reason, const MSLane* /* enteredLane */) {
+MSInstantInductLoop::notifyLeave(SUMOTrafficObject& veh, double /* lastPos */, MSMoveReminder::Notification reason, const MSLane* /* enteredLane */) {
     if (reason == MSMoveReminder::NOTIFICATION_JUNCTION) {
         // vehicle might have jumped over detector at the end of the lane. we need
         // one more notifyMove to register it
         return true;
     }
-    std::map<SUMOVehicle*, double>::iterator i = myEntryTimes.find(&veh);
+    std::map<SUMOTrafficObject*, double>::iterator i = myEntryTimes.find(&veh);
     if (i != myEntryTimes.end()) {
         write("leave", SIMTIME, veh, veh.getSpeed());
         myEntryTimes.erase(i);
