@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSMeanData.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Tue, 17.11.2009
-/// @version $Id: MSMeanData.h 23188 2017-03-02 11:59:04Z luecken $
 ///
 // Data collector for edges/lanes
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSMeanData_h
-#define MSMeanData_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <set>
@@ -46,7 +36,7 @@
 class OutputDevice;
 class MSEdge;
 class MSLane;
-class SUMOVehicle;
+class SUMOTrafficObject;
 
 typedef std::vector<MSEdge*> MSEdgeVector;
 
@@ -101,7 +91,7 @@ public:
          * @see MSMoveReminder::notifyEnter
          * @see MSMoveReminder::Notification
          */
-        virtual bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+        virtual bool notifyEnter(SUMOTrafficObject& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
 
         /** @brief Checks whether the reminder still has to be notified about the vehicle moves
@@ -117,7 +107,7 @@ public:
          *
          * @return True if vehicle hasn't passed the reminder completely.
          */
-        bool notifyMove(SUMOVehicle& veh, double oldPos,
+        bool notifyMove(SUMOTrafficObject& veh, double oldPos,
                         double newPos, double newSpeed);
 
 
@@ -129,7 +119,7 @@ public:
          * @see MSMoveReminder
          * @see MSMoveReminder::notifyLeave
          */
-        virtual bool notifyLeave(SUMOVehicle& veh, double lastPos,
+        virtual bool notifyLeave(SUMOTrafficObject& veh, double lastPos,
                                  MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
 
@@ -151,7 +141,7 @@ public:
          * @param[in] numLanes The total number of lanes for which the data was collected
          * @exception IOError If an error on writing occurs (!!! not yet implemented)
          */
-        virtual void write(OutputDevice& dev, const SUMOTime period,
+        virtual void write(OutputDevice& dev, long long int attributeMask, const SUMOTime period,
                            const double numLanes, const double defaultTravelTime,
                            const int numVehicles = -1) const = 0;
 
@@ -215,7 +205,7 @@ public:
         /** @brief Internal notification about the vehicle moves
          *  @see MSMoveReminder::notifyMoveInternal().
          */
-        void notifyMoveInternal(const SUMOVehicle& veh, const double frontOnLane, const double timeOnLane, const double meanSpeedFrontOnLane, const double meanSpeedVehicleOnLane, const double travelledDistanceFrontOnLane, const double travelledDistanceVehicleOnLane);
+        void notifyMoveInternal(const SUMOTrafficObject& veh, const double frontOnLane, const double timeOnLane, const double meanSpeedFrontOnLane, const double meanSpeedVehicleOnLane, const double travelledDistanceFrontOnLane, const double travelledDistanceVehicleOnLane, const double meanLengthOnLane);
 
 
         /** @brief Called if the vehicle leaves the reminder's lane
@@ -227,7 +217,7 @@ public:
          * @see MSMoveReminder
          * @see MSMoveReminder::notifyLeave
          */
-        bool notifyLeave(SUMOVehicle& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+        bool notifyLeave(SUMOTrafficObject& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
 
         /** @brief Computes current values and adds them to their sums
@@ -241,7 +231,7 @@ public:
          * @see MSMoveReminder::notifyEnter
          * @return Always true
          */
-        bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+        bool notifyEnter(SUMOTrafficObject& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
         //@}
 
         bool isEmpty() const;
@@ -253,7 +243,7 @@ public:
          * @param[in] numLanes The total number of lanes for which the data was collected
          * @exception IOError If an error on writing occurs (!!! not yet implemented)
          */
-        void write(OutputDevice& dev, const SUMOTime period,
+        void write(OutputDevice& dev, long long int attributeMask, const SUMOTime period,
                    const double numLanes, const double defaultTravelTime,
                    const int numVehicles = -1) const;
 
@@ -286,7 +276,7 @@ public:
         };
 
         /// @brief The map of vehicles to data entries
-        std::map<const SUMOVehicle*, TrackerEntry*> myTrackedData;
+        std::map<const SUMOTrafficObject*, TrackerEntry*> myTrackedData;
 
         /// @brief The currently active meandata "intervals"
         std::list<TrackerEntry*> myCurrentData;
@@ -304,6 +294,7 @@ public:
      * @param[in] withEmpty Information whether empty lanes/edges shall be written
      * @param[in] withInternal Information whether internal lanes/edges shall be written
      * @param[in] trackVehicles Information whether vehicles shall be tracked
+     * @param[in] detectPersons Whether pedestrians shall be detected instead of vehicles
      * @param[in] maxTravelTime the maximum travel time to use when calculating per vehicle output
      * @param[in] defaultEffort the value to use when calculating defaults
      * @param[in] minSamples the minimum number of sample seconds before the values are valid
@@ -313,9 +304,11 @@ public:
                const SUMOTime dumpBegin, const SUMOTime dumpEnd,
                const bool useLanes, const bool withEmpty,
                const bool printDefaults, const bool withInternal,
-               const bool trackVehicles, const double minSamples,
+               const bool trackVehicles, const int detectPersons,
+               const double minSamples,
                const double maxTravelTime,
-               const std::string& vTypes);
+               const std::string& vTypes,
+               const std::string& writeAttributes);
 
 
     /// @brief Destructor
@@ -365,6 +358,10 @@ public:
         return myMaxTravelTime;
     }
 
+    bool isEdgeData() const {
+        return myAmEdgeBased;
+    }
+
 
 protected:
     /** @brief Create an instance of MeanDataValues
@@ -438,12 +435,17 @@ protected:
     const bool myDumpEmpty;
 
 private:
+    static long long int initWrittenAttributes(const std::string writeAttributes, const std::string& id);
+
     /// @brief Information whether the output shall be edge-based (not lane-based)
     const bool myAmEdgeBased;
 
     /// @brief The first and the last time step to write information (-1 indicates always)
     const SUMOTime myDumpBegin, myDumpEnd;
 
+    /// @brief time at which init was called();
+    SUMOTime myInitTime;
+
     /// @brief The corresponding first edges
     MSEdgeVector myEdges;
 
@@ -456,6 +458,9 @@ private:
     /// @brief Whether vehicles are tracked
     const bool myTrackVehicles;
 
+    /// @brief bit mask for checking attributes to be written
+    const long long int myWrittenAttributes;
+
     /// @brief The intervals for which output still has to be generated (only in the tracking case)
     std::list< std::pair<SUMOTime, SUMOTime> > myPendingIntervals;
 
@@ -467,9 +472,3 @@ private:
     MSMeanData& operator=(const MSMeanData&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
