@@ -1,33 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSMeanData_Harmonoise.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 10.05.2004
-/// @version $Id: MSMeanData_Harmonoise.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Redirector for mean data output (net->edgecontrol)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <microsim/MSLane.h>
 #include <microsim/MSVehicle.h>
@@ -39,10 +30,6 @@
 #include <utils/emissions/HelpersHarmonoise.h>
 #include <limits>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -50,9 +37,9 @@
 // ---------------------------------------------------------------------------
 // MSMeanData_Harmonoise::MSLaneMeanDataValues - methods
 // ---------------------------------------------------------------------------
-MSMeanData_Harmonoise::MSLaneMeanDataValues::MSLaneMeanDataValues(MSLane* const lane, const SUMOReal length, const bool doAdd,
-        const std::set<std::string>* const vTypes, const MSMeanData_Harmonoise* parent)
-    : MSMeanData::MeanDataValues(lane, length, doAdd, vTypes),
+MSMeanData_Harmonoise::MSLaneMeanDataValues::MSLaneMeanDataValues(MSLane* const lane, const double length, const bool doAdd,
+        const MSMeanData_Harmonoise* parent)
+    : MSMeanData::MeanDataValues(lane, length, doAdd, parent),
       currentTimeN(0), meanNTemp(0), myParent(parent) {}
 
 
@@ -73,47 +60,43 @@ void
 MSMeanData_Harmonoise::MSLaneMeanDataValues::addTo(MSMeanData::MeanDataValues& val) const {
     MSLaneMeanDataValues& v = (MSLaneMeanDataValues&) val;
     v.sampleSeconds += sampleSeconds;
-    v.meanNTemp += (SUMOReal) pow(10., HelpersHarmonoise::sum(meanNTemp) / 10.);
+    v.meanNTemp += meanNTemp;
     v.travelledDistance += travelledDistance;
 }
 
 
 void
 MSMeanData_Harmonoise::MSLaneMeanDataValues::update() {
-    meanNTemp += (SUMOReal) pow(10., HelpersHarmonoise::sum(currentTimeN) / 10.);
+    meanNTemp += currentTimeN;
     currentTimeN = 0;
 }
 
 
 void
-MSMeanData_Harmonoise::MSLaneMeanDataValues::notifyMoveInternal(SUMOVehicle& veh, SUMOReal timeOnLane, SUMOReal speed) {
-    const SUMOReal sn = HelpersHarmonoise::computeNoise(veh.getVehicleType().getEmissionClass(),
-                        (double) speed, veh.getAcceleration());
-    currentTimeN += (SUMOReal) pow(10., (sn / 10.));
+MSMeanData_Harmonoise::MSLaneMeanDataValues::notifyMoveInternal(const SUMOTrafficObject& veh, const double /* frontOnLane */, const double timeOnLane, const double /*meanSpeedFrontOnLane*/, const double meanSpeedVehicleOnLane, const double /*travelledDistanceFrontOnLane*/, const double travelledDistanceVehicleOnLane, const double /* meanLengthOnLane */) {
+    const double sn = HelpersHarmonoise::computeNoise(veh.getVehicleType().getEmissionClass(),
+                      // XXX: recheck, which value to use here for the speed. (Leo) Refs. #2579
+                      (double) meanSpeedVehicleOnLane, veh.getAcceleration());
+    currentTimeN += (double) pow(10., (sn / 10.));
     sampleSeconds += timeOnLane;
-    travelledDistance += speed * timeOnLane;
-}
-
-
-bool
-MSMeanData_Harmonoise::MSLaneMeanDataValues::notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification /*reason*/) {
-    return vehicleApplies(veh);
+    travelledDistance += travelledDistanceVehicleOnLane;
 }
 
 
 void
-MSMeanData_Harmonoise::MSLaneMeanDataValues::write(OutputDevice& dev, const SUMOTime period,
-        const SUMOReal /*numLanes*/, const SUMOReal defaultTravelTime, const int /*numVehicles*/) const {
-    dev.writeAttr("noise", (meanNTemp != 0 ? (SUMOReal)(10. * log10(meanNTemp * TS / STEPS2TIME(period))) : (SUMOReal) 0.));
+MSMeanData_Harmonoise::MSLaneMeanDataValues::write(OutputDevice& dev, long long int attributeMask, const SUMOTime period,
+        const double /*numLanes*/, const double defaultTravelTime, const int /*numVehicles*/) const {
+    const double noise = meanNTemp != 0 ? (double)(10. * log10(meanNTemp * TS / STEPS2TIME(period))) : (double) 0.;
+    dev.writeOptionalAttr(SUMO_ATTR_NOISE, noise, attributeMask);
     if (sampleSeconds > myParent->myMinSamples) {
-        SUMOReal traveltime = myParent->myMaxTravelTime;
+        double traveltime = myParent->myMaxTravelTime;
         if (travelledDistance > 0.f) {
             traveltime = MIN2(traveltime, myLaneLength * sampleSeconds / travelledDistance);
         }
-        dev.writeAttr("traveltime", traveltime);
+        dev.writeOptionalAttr(SUMO_ATTR_TRAVELTIME, traveltime, attributeMask);
     } else if (defaultTravelTime >= 0.) {
         // @todo default value for noise
-        dev.writeAttr("traveltime", defaultTravelTime);
+        dev.writeOptionalAttr(SUMO_ATTR_TRAVELTIME, defaultTravelTime, attributeMask);
     }
     dev.closeTag();
 }
@@ -128,10 +111,11 @@ MSMeanData_Harmonoise::MSMeanData_Harmonoise(const std::string& id,
         const bool useLanes, const bool withEmpty,
         const bool printDefaults, const bool withInternal,
         const bool trackVehicles,
-        const SUMOReal maxTravelTime, const SUMOReal minSamples,
-        const std::set<std::string> vTypes)
-    : MSMeanData(id, dumpBegin, dumpEnd, useLanes, withEmpty, printDefaults,
-                 withInternal, trackVehicles, maxTravelTime, minSamples, vTypes) {
+        const double maxTravelTime, const double minSamples,
+        const std::string& vTypes,
+        const std::string& writeAttributes) :
+    MSMeanData(id, dumpBegin, dumpEnd, useLanes, withEmpty, printDefaults,
+               withInternal, trackVehicles, 0, maxTravelTime, minSamples, vTypes, writeAttributes) {
 }
 
 
@@ -139,8 +123,8 @@ MSMeanData_Harmonoise::~MSMeanData_Harmonoise() {}
 
 
 MSMeanData::MeanDataValues*
-MSMeanData_Harmonoise::createValues(MSLane* const lane, const SUMOReal length, const bool doAdd) const {
-    return new MSLaneMeanDataValues(lane, length, doAdd, &myVehicleTypes, this);
+MSMeanData_Harmonoise::createValues(MSLane* const lane, const double length, const bool doAdd) const {
+    return new MSLaneMeanDataValues(lane, length, doAdd, this);
 }
 
 
@@ -157,4 +141,3 @@ MSMeanData_Harmonoise::detectorUpdate(const SUMOTime step) {
 
 
 /****************************************************************************/
-
