@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSMeanData_Harmonoise.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 10.05.2004
-/// @version $Id: MSMeanData_Harmonoise.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Noise data collector for edges/lanes
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSMeanData_Harmonoise_h
-#define MSMeanData_Harmonoise_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <limits>
@@ -69,9 +59,8 @@ public:
     class MSLaneMeanDataValues : public MSMeanData::MeanDataValues {
     public:
         /** @brief Constructor */
-        MSLaneMeanDataValues(MSLane* const lane, const SUMOReal length, const bool doAdd,
-                             const std::set<std::string>* const vTypes = 0,
-                             const MSMeanData_Harmonoise* parent = 0);
+        MSLaneMeanDataValues(MSLane* const lane, const double length, const bool doAdd,
+                             const MSMeanData_Harmonoise* parent);
 
         /** @brief Destructor */
         virtual ~MSLaneMeanDataValues();
@@ -86,25 +75,6 @@ public:
         void addTo(MSMeanData::MeanDataValues& val) const;
 
 
-        /// @name Methods inherited from MSMoveReminder.
-        /// @{
-
-        /** @brief Computes current emission values and adds them to their sums
-         *
-         * The fraction of time the vehicle is on the lane is computed and
-         *  used as a weight for the vehicle's current emission values
-         *  which are computed using the current velocity and acceleration.
-         *
-         * @param[in] veh The entering vehicle.
-         * @param[in] reason how the vehicle enters the lane
-         * @return Always true
-         * @see MSMoveReminder::notifyEnter
-         * @see MSMoveReminder::Notification
-         */
-        bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason);
-        //@}
-
-
         /** @brief Computes the noise in the last time step
          *
          * The sum of noises collected so far (in the last seen step)
@@ -120,34 +90,26 @@ public:
          * @param[in] length The length of the object for which the data was collected
          * @exception IOError If an error on writing occurs (!!! not yet implemented)
          */
-        void write(OutputDevice& dev, const SUMOTime period,
-                   const SUMOReal numLanes, const SUMOReal defaultTravelTime,
+        void write(OutputDevice& dev, long long int attributeMask, const SUMOTime period,
+                   const double numLanes, const double defaultTravelTime,
                    const int numVehicles = -1) const;
 
 
     protected:
         /** @brief Internal notification about the vehicle moves
-         *
-         * Indicator if the reminders is still active for the passed
-         * vehicle/parameters. If false, the vehicle will erase this reminder
-         * from it's reminder-container.
-         *
-         * @param[in] veh Vehicle that asks this reminder.
-         * @param[in] timeOnLane time the vehicle spent on the lane.
-         * @param[in] speed Moving speed.
+         *  @see MSMoveReminder::notifyMoveInternal()
          */
-        void notifyMoveInternal(SUMOVehicle& veh, SUMOReal timeOnLane,
-                                SUMOReal speed);
+        void notifyMoveInternal(const SUMOTrafficObject& veh, const double /* frontOnLane */, const double timeOnLane, const double meanSpeedFrontOnLane, const double meanSpeedVehicleOnLane, const double travelledDistanceFrontOnLane, const double travelledDistanceVehicleOnLane, const double /* meanLengthOnLane */);
 
     private:
         /// @name Collected values
         /// @{
 
         /// @brief Sum of produced noise at this time step(pow(10, (<NOISE>/10.)))
-        SUMOReal currentTimeN;
+        double currentTimeN;
 
         /// @brief Sum of produced noise over time (pow(10, (<NOISE>/10.)))
-        SUMOReal meanNTemp;
+        double meanNTemp;
         //@}
 
 
@@ -176,8 +138,9 @@ public:
                           const bool useLanes, const bool withEmpty,
                           const bool printDefaults, const bool withInternal,
                           const bool trackVehicles,
-                          const SUMOReal minSamples, const SUMOReal maxTravelTime,
-                          const std::set<std::string> vTypes);
+                          const double minSamples, const double maxTravelTime,
+                          const std::string& vTypes,
+                          const std::string& writeAttributes);
 
 
     /// @brief Destructor
@@ -194,7 +157,7 @@ protected:
      * @param[in] lane The lane to create for
      * @param[in] doAdd whether to add the values as reminder to the lane
      */
-    MSMeanData::MeanDataValues* createValues(MSLane* const lane, const SUMOReal length, const bool doAdd) const;
+    MSMeanData::MeanDataValues* createValues(MSLane* const lane, const double length, const bool doAdd) const;
 
 private:
     /// @brief Invalidated copy constructor.
@@ -204,9 +167,3 @@ private:
     MSMeanData_Harmonoise& operator=(const MSMeanData_Harmonoise&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
