@@ -1,50 +1,39 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSVTypeProbe.cpp
 /// @author  Tino Morenz
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 24.10.2007
-/// @version $Id: MSVTypeProbe.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Writes positions of vehicles that have a certain (named) type
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/common/WrappingCommand.h>
 #include <microsim/MSNet.h>
 #include <microsim/MSVehicle.h>
+#include <microsim/MSEventControl.h>
+#include <microsim/MSVehicleControl.h>
 #include <microsim/MSLane.h>
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/geom/GeoConvHelper.h>
 
 #include "MSVTypeProbe.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -53,8 +42,8 @@ MSVTypeProbe::MSVTypeProbe(const std::string& id,
                            const std::string& vType,
                            OutputDevice& od, SUMOTime frequency)
     : Named(id), myVType(vType), myOutputDevice(od), myFrequency(frequency) {
-    MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(this, 0, MSEventControl::ADAPT_AFTER_EXECUTION);
-    myOutputDevice.writeXMLHeader("vehicle-type-probes");
+    MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(this);
+    myOutputDevice.writeXMLHeader("vehicle-type-probes", "vtypeprobe_file.xsd");
 }
 
 
@@ -64,7 +53,10 @@ MSVTypeProbe::~MSVTypeProbe() {
 
 SUMOTime
 MSVTypeProbe::execute(SUMOTime currentTime) {
-    myOutputDevice.openTag("timestep") << " time=\"" << time2string(currentTime) << "\" id=\"" << getID() << "\" vType=\"" << myVType << "\"";
+    myOutputDevice.openTag(SUMO_TAG_TIMESTEP);
+    myOutputDevice.writeAttr(SUMO_ATTR_TIME, time2string(currentTime));
+    myOutputDevice.writeAttr(SUMO_ATTR_ID, getID());
+    myOutputDevice.writeAttr("vType", myVType);
     MSVehicleControl& vc = MSNet::getInstance()->getVehicleControl();
     for (MSVehicleControl::constVehIt it = vc.loadedVehBegin(); it != vc.loadedVehEnd(); ++it) {
         const SUMOVehicle* veh = it->second;
@@ -72,19 +64,25 @@ MSVTypeProbe::execute(SUMOTime currentTime) {
         if (myVType == "" || myVType == veh->getVehicleType().getID()) {
             if (veh->isOnRoad()) {
                 Position pos = veh->getPosition();
-                myOutputDevice.openTag("vehicle").writeAttr("id", veh->getID());
-                if (microVeh != 0) {
-                    myOutputDevice.writeAttr("lane", microVeh->getLane()->getID());
+                myOutputDevice.openTag(SUMO_TAG_VEHICLE);
+                myOutputDevice.writeAttr(SUMO_ATTR_ID, veh->getID());
+                if (microVeh != nullptr) {
+                    myOutputDevice.writeAttr(SUMO_ATTR_LANE, microVeh->getLane()->getID());
+                }
+                myOutputDevice.writeAttr(SUMO_ATTR_POSITION, veh->getPositionOnLane());
+                myOutputDevice.writeAttr(SUMO_ATTR_X, pos.x());
+                myOutputDevice.writeAttr(SUMO_ATTR_Y, pos.y());
+                if (MSNet::getInstance()->hasElevation()) {
+                    myOutputDevice.writeAttr(SUMO_ATTR_Z, pos.z());
                 }
-                myOutputDevice.writeAttr("pos", veh->getPositionOnLane());
-                myOutputDevice.writeAttr("x", pos.x()).writeAttr("y", pos.y());
                 if (GeoConvHelper::getFinal().usingGeoProjection()) {
                     GeoConvHelper::getFinal().cartesian2geo(pos);
-                    myOutputDevice.setPrecision(GEO_OUTPUT_ACCURACY);
-                    myOutputDevice.writeAttr("lat", pos.y()).writeAttr("lon", pos.x());
+                    myOutputDevice.setPrecision(gPrecisionGeo);
+                    myOutputDevice.writeAttr(SUMO_ATTR_LAT, pos.y());
+                    myOutputDevice.writeAttr(SUMO_ATTR_LON, pos.x());
                     myOutputDevice.setPrecision();
                 }
-                myOutputDevice.writeAttr("speed", veh->getSpeed());
+                myOutputDevice.writeAttr(SUMO_ATTR_SPEED, veh->getSpeed());
                 myOutputDevice.closeTag();
             }
         }
