@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSXMLRawOut.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -6,33 +18,11 @@
 /// @author  Bjoern Hendriks
 /// @author  Michael Behrisch
 /// @date    Mon, 10.05.2004
-/// @version $Id: MSXMLRawOut.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Realises dumping the complete network state
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSXMLRawOut_h
-#define MSXMLRawOut_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <utils/common/SUMOTime.h>
 
@@ -69,10 +59,11 @@ public:
      * @param[in] of The output device to use
      * @param[in] ec The EdgeControl which holds the edges to write
      * @param[in] timestep The current time step
+     * @param[in] precision The output precision
      * @exception IOError If an error on writing occurs (!!! not yet implemented)
      */
     static void write(OutputDevice& of, const MSEdgeControl& ec,
-                      SUMOTime timestep);
+                      SUMOTime timestep, int precision);
 
 
     /** @brief Writes the dump of the given vehicle into the given device
@@ -109,6 +100,8 @@ private:
      */
     static void writeLane(OutputDevice& of, const MSLane& lane);
 
+    /// @brief write transportable
+    static void writeTransportable(OutputDevice& of, const MSTransportable* p, SumoXMLTag tag);
 
 private:
     /// @brief Invalidated copy constructor.
@@ -119,9 +112,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
