@@ -1,36 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSDeterministicHiLevelTrafficLightLogic.h
 /// @author  Riccardo Belletti
 /// @date    Mar 2014
-/// @version $Id: MSDeterministicHiLevelTrafficLightLogic.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // The class for deterministic high level traffic light logic
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSDeterministicHiLevelTrafficLightLogic_h
-#define MSDeterministicHiLevelTrafficLightLogic_h
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#define SWARM_DEBUG
-#include <utils/common/SwarmDebug.h>
 #include "MSSOTLHiLevelTrafficLightLogic.h"
 #include "MSSOTLPhasePolicy.h"
 #include "MSSOTLPlatoonPolicy.h"
@@ -48,14 +37,14 @@ public:
      * @brief Constructor without sensors passed
      * @param[in] tlcontrol The tls control responsible for this tls
      * @param[in] id This traffic light id
-     * @param[in] subid This tls' sub-id (program id)
+     * @param[in] programID This tls' sub-id (program id)
      * @param[in] phases Definitions of the phases
      * @param[in] step The initial phase index
      * @param[in] delay The time to wait before the first switch
      * @param[in] parameters Parameters defined for the tll
      */
     MSDeterministicHiLevelTrafficLightLogic(MSTLLogicControl& tlcontrol, const std::string& id,
-                                            const std::string& subid, const Phases& phases, int step,
+                                            const std::string& programID, const Phases& phases, int step,
                                             SUMOTime delay,
                                             const std::map<std::string, std::string>& parameters);
 
@@ -67,7 +56,7 @@ public:
      * @param[in] nb The detector builder
      * @exception ProcessError If something fails on initialisation
      */
-    void init(NLDetectorBuilder& nb) throw(ProcessError);
+    void init(NLDetectorBuilder& nb);
 
     /** @brief Returns the type of the logic as a string
     * @return The type of the logic
@@ -125,6 +114,3 @@ protected:
 
 
 };
-
-#endif
-/****************************************************************************/
