@@ -1,32 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSOffTrafficLightLogic.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    08.05.2007
-/// @version $Id: MSOffTrafficLightLogic.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A traffic lights logic which represents a tls in an off-mode
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <utility>
 #include <vector>
@@ -36,17 +29,13 @@
 #include "MSTrafficLightLogic.h"
 #include "MSOffTrafficLightLogic.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // member method definitions
 // ===========================================================================
 MSOffTrafficLightLogic::MSOffTrafficLightLogic(MSTLLogicControl& tlcontrol,
         const std::string& id) :
-    MSTrafficLightLogic(tlcontrol, id, "off", 0, std::map<std::string, std::string>()) {
+    MSTrafficLightLogic(tlcontrol, id, "off", TrafficLightType::OFF, 0, std::map<std::string, std::string>()) {
     myDefaultCycleTime = TIME2STEPS(120);
 }
 
@@ -74,17 +63,31 @@ MSOffTrafficLightLogic::adaptLinkInformationFrom(const MSTrafficLightLogic& logi
 
 void
 MSOffTrafficLightLogic::rebuildPhase() {
-    size_t no = getLinks().size();
+    int no = (int)getLinks().size();
     std::string state;
-    for (unsigned int i = 0; i < no; ++i) {
-        // !!! no brake mask!
-        state += 'o';
+    for (int i = 0; i < no; ++i) {
+        bool foundMajor = false;
+        bool foundMinor = false;
+        for (const MSLink* l : myLinks[i]) {
+            /// @note. all links for the same index should have the same
+            if (l->getOffState() == 'o') {
+                foundMinor = true;
+            } else if (l->getOffState() == 'O') {
+                foundMajor = true;
+            } else {
+                WRITE_WARNING("Invalid 'off'-state for link " + toString(l->getIndex()) + " at junction '" + l->getJunction()->getID() + "'");
+            }
+        }
+        if (foundMajor && foundMinor) {
+            WRITE_WARNING("Inconsistent 'off'-states for linkIndex " + toString(i) + " at tlLogic '" + getID() + "'");
+        }
+        state += foundMinor ? 'o' : 'O';
     }
     for (MSTrafficLightLogic::Phases::const_iterator i = myPhaseDefinition.begin(); i != myPhaseDefinition.end(); ++i) {
         delete *i;
     }
     myPhaseDefinition.clear();
-    myPhaseDefinition.push_back(new MSPhaseDefinition(TIME2STEPS(120), state));
+    myPhaseDefinition.push_back(new MSPhaseDefinition(TIME2STEPS(120), state, -1));
 }
 
 
@@ -128,17 +131,15 @@ MSOffTrafficLightLogic::getPhaseIndexAtTime(SUMOTime) const {
 
 
 SUMOTime
-MSOffTrafficLightLogic::getOffsetFromIndex(unsigned int) const {
+MSOffTrafficLightLogic::getOffsetFromIndex(int) const {
     return 0;
 }
 
 
-unsigned int
+int
 MSOffTrafficLightLogic::getIndexFromOffset(SUMOTime) const {
     return 0;
 }
 
 
-
 /****************************************************************************/
-
