@@ -1,24 +1,25 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2010-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 /****************************************************************************/
 /// @file    MSSOTLDefinitions.h
 /// @author  Gianfilippo Slager
 /// @date    Mar 2010
-/// @version $Id$
 ///
 // The repository for definitions about SOTL and Swarm-based logics
 /****************************************************************************/
 
-#ifndef MSSOTLDefinitions_h
-#define MSSOTLDefinitions_h
+#pragma once
+#include <config.h>
 
 ///For MSSOTLSensors
 //SENSOR_START in meters, counting from the traffic light and moving backward with respect to traffic direction
@@ -77,6 +78,3 @@ typedef std::set<std::string> MSLaneID_set;
 
 typedef std::map<MSLane*, bool> LaneCheckMap;
 typedef std::vector<std::string> LaneIdVector;
-
-#endif
-/****************************************************************************/
