@@ -1,31 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSSOTLE2Sensors.cpp
 /// @author  Gianfilippo Slager
 /// @author  Alessio Bonfietti
 /// @author  Federico Caselli
 /// @date    2010-02-25
-/// @version $Id: MSSOTLE2Sensors.cpp 20113 2016-03-01 13:22:42Z martintaraz $
 ///
 // The class for SOTL sensors of "E2" type
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright 2001-2009 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
 
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <microsim/output/MSDetectorControl.h>
 #include <microsim/MSEdge.h>
 #include <microsim/MSVehicleType.h>
 #include "MSSOTLE2Sensors.h"
 
+#define INVALID_POSITION std::numeric_limits<double>::max()
+
 MSSOTLE2Sensors::MSSOTLE2Sensors(std::string tlLogicID,
                                  const MSTrafficLightLogic::Phases* phases) :
     MSSOTLSensors(tlLogicID, phases) {
@@ -43,9 +45,9 @@ void MSSOTLE2Sensors::buildSensors(
 
 void MSSOTLE2Sensors::buildSensors(
     MSTrafficLightLogic::LaneVectorVector controlledLanes,
-    NLDetectorBuilder& nb, SUMOReal sensorLength) {
+    NLDetectorBuilder& nb, double sensorLength) {
     //for each lane build an appropriate sensor on it
-    MSLane* currentLane = NULL;
+    MSLane* currentLane = nullptr;
 
     //input and ouput lanes
     for (MSTrafficLightLogic::LaneVectorVector::const_iterator laneVector =
@@ -66,7 +68,7 @@ void MSSOTLE2Sensors::buildCountSensors(
     MSTrafficLightLogic::LaneVectorVector controlledLanes,
     NLDetectorBuilder& nb) {
     //for each lane build an appropriate sensor on it
-    MSLane* currentLane = NULL;
+    MSLane* currentLane = nullptr;
     //input and ouput lanes
     for (MSTrafficLightLogic::LaneVectorVector::const_iterator laneVector =
                 controlledLanes.begin(); laneVector != controlledLanes.end();
@@ -83,7 +85,7 @@ void MSSOTLE2Sensors::buildCountOutSensors(
     MSTrafficLightLogic::LaneVectorVector controlledLanes,
     NLDetectorBuilder& nb) {
     //for each lane build an appropriate sensor on it
-    MSLane* currentLane = NULL;
+    MSLane* currentLane = nullptr;
 
     //input and ouput lanes
     for (MSTrafficLightLogic::LaneVectorVector::const_iterator laneVector =
@@ -104,9 +106,9 @@ void MSSOTLE2Sensors::buildOutSensors(
 }
 void MSSOTLE2Sensors::buildOutSensors(
     MSTrafficLightLogic::LaneVectorVector controlledLanes,
-    NLDetectorBuilder& nb, SUMOReal sensorLength) {
+    NLDetectorBuilder& nb, double sensorLength) {
     //for each lane build an appropriate sensor on it
-    MSLane* currentLane = NULL;
+    MSLane* currentLane = nullptr;
 
     //input and ouput lanes
     for (MSTrafficLightLogic::LaneVectorVector::const_iterator laneVector =
@@ -121,9 +123,9 @@ void MSSOTLE2Sensors::buildOutSensors(
 }
 
 void MSSOTLE2Sensors::buildCountSensorForLane(MSLane* lane, NLDetectorBuilder& nb) {
-    SUMOReal sensorPos;
-    SUMOReal lensorLength;
-    MSE2Collector* newSensor = NULL;
+    double sensorPos;
+    double lensorLength;
+    MSE2Collector* newSensor = nullptr;
     //Check not to have more than a sensor for lane
     if (m_sensorMap.find(lane->getID()) == m_sensorMap.end()) {
 
@@ -131,29 +133,28 @@ void MSSOTLE2Sensors::buildCountSensorForLane(MSLane* lane, NLDetectorBuilder& n
         sensorPos = COUNT_SENSOR_START <= lane->getLength() ? COUNT_SENSOR_START : 0;
 
         //Original:
-        SUMOReal sensorLength = INPUT_COUNT_SENSOR_LENGTH;
+        double sensorLength = INPUT_COUNT_SENSOR_LENGTH;
 
         //Check and trim if the lane is not long enough for the specified sensor length
         lensorLength = sensorLength <= (lane->getLength() - sensorPos) ? sensorLength : (lane->getLength() - sensorPos);
 
         //TODO check this lengths
-        DBG(
-            std::ostringstream phero_str;
-            phero_str << " lane " << lane->getID() << " sensorPos= " << sensorPos
-            << " ,SENSOR_START  " << SENSOR_START << "; lane->getLength = "
-            << lane->getLength() << " ,lensorLength= " << lensorLength
-            << " ,SENSOR_LENGTH= " << INPUT_SENSOR_LENGTH;
-            WRITE_MESSAGE(
-                "MSSOTLE2Sensors::buildSensorForLane::" + phero_str.str());
-        )
+//        DBG(
+//            std::ostringstream phero_str;
+//            phero_str << " lane " << lane->getID() << " sensorPos= " << sensorPos
+//            << " ,SENSOR_START  " << SENSOR_START << "; lane->getLength = "
+//            << lane->getLength() << " ,lensorLength= " << lensorLength
+//            << " ,SENSOR_LENGTH= " << INPUT_SENSOR_LENGTH;
+//            WRITE_MESSAGE(
+//                "MSSOTLE2Sensors::buildSensorForLane::" + phero_str.str());
+//        )
 
         //Create sensor for lane and insert it into the map<MSLane*, MSE2Collector*>
-        newSensor = nb.buildSingleLaneE2Det(
+        newSensor = nb.createE2Detector(
                         "SOTL_E2_lane:" + lane->getID() + "_tl:" + tlLogicID,
                         DU_TL_CONTROL, lane,
-                        (lane->getLength() - sensorPos - lensorLength), lensorLength,
-                        HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS);
-        //newSensor = nb.buildSingleLaneE2Det("SOTL_E2_lane:"+lane->getID()+"_tl:"+tlLogicID, DU_TL_CONTROL, lane, (lane->getLength() - sensorPos- 5), lensorLength, HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS);
+                        (lane->getLength() - sensorPos - lensorLength), INVALID_POSITION, lensorLength,
+                        HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS, "");
 
         MSNet::getInstance()->getDetectorControl().add(
             SUMO_TAG_LANE_AREA_DETECTOR, newSensor);
@@ -164,41 +165,40 @@ void MSSOTLE2Sensors::buildCountSensorForLane(MSLane* lane, NLDetectorBuilder& n
 }
 
 void MSSOTLE2Sensors::buildCountSensorForOutLane(MSLane* lane, NLDetectorBuilder& nb) {
-    SUMOReal sensorPos;
-    SUMOReal lensorLength;
-    MSE2Collector* newSensor = NULL;
+    double sensorPos;
+    double lensorLength;
+    MSE2Collector* newSensor = nullptr;
     //Check not to have more than a sensor for lane
     if (m_sensorMap.find(lane->getID()) == m_sensorMap.end()) {
 
         //Original:
-        SUMOReal sensorLength = OUTPUT_COUNT_SENSOR_LENGTH;
+        double sensorLength = OUTPUT_COUNT_SENSOR_LENGTH;
         //Check and set zero if the lane is not long enough for the specified sensor start
         sensorPos = (lane->getLength() - sensorLength)
                     - (SENSOR_START <= lane->getLength() ? SENSOR_START : 0);
 
-        //Check and trim if the lane is not long enough for the specified sensor lenght
+        //Check and trim if the lane is not long enough for the specified sensor length
         lensorLength =
             sensorLength <= (lane->getLength() - sensorPos) ?
             sensorLength : (lane->getLength() - sensorPos);
 
         //TODO check this lengths
-        DBG(
-            std::ostringstream phero_str;
-            phero_str << " lane " << lane->getID() << " sensorPos= " << sensorPos
-            << " ,SENSOR_START  " << SENSOR_START << "; lane->getLength = "
-            << lane->getLength() << " ,lensorLength= " << lensorLength
-            << " ,SENSOR_LENGTH= " << INPUT_SENSOR_LENGTH;
-            WRITE_MESSAGE(
-                "MSSOTLE2Sensors::buildSensorForLane::" + phero_str.str());
-        )
+//        DBG(
+//            std::ostringstream phero_str;
+//            phero_str << " lane " << lane->getID() << " sensorPos= " << sensorPos
+//            << " ,SENSOR_START  " << SENSOR_START << "; lane->getLength = "
+//            << lane->getLength() << " ,lensorLength= " << lensorLength
+//            << " ,SENSOR_LENGTH= " << INPUT_SENSOR_LENGTH;
+//            WRITE_MESSAGE(
+//                "MSSOTLE2Sensors::buildSensorForLane::" + phero_str.str());
+//        )
 
         //Create sensor for lane and insert it into the map<MSLane*, MSE2Collector*>
-        newSensor = nb.buildSingleLaneE2Det(
+        newSensor = nb.createE2Detector(
                         "SOTL_E2_lane:" + lane->getID() + "_tl:" + tlLogicID,
                         DU_TL_CONTROL, lane,
-                        (lane->getLength() - sensorPos - lensorLength), lensorLength,
-                        HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS);
-        //newSensor = nb.buildSingleLaneE2Det("SOTL_E2_lane:"+lane->getID()+"_tl:"+tlLogicID, DU_TL_CONTROL, lane, (lane->getLength() - sensorPos- 5), lensorLength, HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS);
+                        (lane->getLength() - sensorPos - lensorLength), INVALID_POSITION, lensorLength,
+                        HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS, "");
 
         MSNet::getInstance()->getDetectorControl().add(
             SUMO_TAG_LANE_AREA_DETECTOR, newSensor);
@@ -215,10 +215,10 @@ int MSSOTLE2Sensors::getPassedVeh(std::string laneId, bool /* out */) {
         assert(0);
         return 0;
     } else {
-        unsigned int additional = 0;
+        int additional = 0;
         if (m_continueSensorOnLanes.find(laneId) != m_continueSensorOnLanes.end())
             for (std::vector<std::string>::iterator it = m_continueSensorOnLanes[laneId].begin(); it != m_continueSensorOnLanes[laneId].end(); ++ it) {
-                unsigned int tmp = 0;
+                int tmp = 0;
                 if (getVelueFromSensor(*it, &MSE2Collector::getPassedVeh, tmp)) {
                     additional += tmp;
                 }
@@ -239,26 +239,6 @@ MSSOTLE2Sensors::subtractPassedVeh(std::string laneId, int passed) {
 }
 
 /*******************************************************************************/
-/* @brief Builds an e2 detector that lies on only one lane
- *
- * @param[in] id The id the detector shall have
- * @param[in] usage Information how the detector is used within the simulation
- * @param[in] lane The lane the detector is placed at
- * @param[in] pos The position on the lane the detector is placed at
- * @param[in] length The length the detector has
- * @param[in] haltingTimeThreshold Detector parameter: the time a vehicle's speed must be below haltingSpeedThreshold to be assigned as jammed
- * @param[in] haltingSpeedThreshold Detector parameter: the speed a vehicle's speed must be below to be assigned as jammed
- * @param[in] jamDistThreshold Detector parameter: the distance between two vehicles in order to not count them to one jam
- * @todo Check whether this method is really needful
- */
-/*
- Using
- MSE2Collector* buildSingleLaneE2Det(const std::string& id,
- DetectorUsage usage, MSLane* lane, SUMOReal pos, SUMOReal length,
- SUMOTime haltingTimeThreshold, SUMOReal haltingSpeedThreshold,
- SUMOReal jamDistThreshold);
-
- */
 
 void MSSOTLE2Sensors::buildSensorForLane(MSLane* lane, NLDetectorBuilder& nb) {
     buildSensorForLane(lane, nb, INPUT_SENSOR_LENGTH);
@@ -266,39 +246,38 @@ void MSSOTLE2Sensors::buildSensorForLane(MSLane* lane, NLDetectorBuilder& nb) {
 
 
 
-void MSSOTLE2Sensors::buildSensorForLane(MSLane* lane, NLDetectorBuilder& nb, SUMOReal sensorLength) {
+void MSSOTLE2Sensors::buildSensorForLane(MSLane* lane, NLDetectorBuilder& nb, double sensorLength) {
     const MSEdge* edge = &lane->getEdge();
     if (edge->isInternal() || edge->isWalkingArea() || edge->isCrossing()) {
         return;
     }
-    SUMOReal sensorPos;
-    SUMOReal lensorLength;
-    MSE2Collector* newSensor = NULL;
+    double sensorPos;
+    double lensorLength;
+    MSE2Collector* newSensor = nullptr;
     //Check not to have more than a sensor for lane
     if (m_sensorMap.find(lane->getID()) == m_sensorMap.end()) {
 
         //Check and set zero if the lane is not long enough for the specified sensor start
         sensorPos = SENSOR_START <= lane->getLength() ? SENSOR_START : 0;
-        //Check and trim if the lane is not long enough for the specified sensor lenght
+        //Check and trim if the lane is not long enough for the specified sensor length
         lensorLength = sensorLength <= (lane->getLength() - sensorPos) ? sensorLength : (lane->getLength() - sensorPos);
         //TODO check this lengths
-        DBG(
-            std::ostringstream phero_str;
-            phero_str << " lane " << lane->getID() << " sensorPos= " << sensorPos
-            << " ,SENSOR_START  " << SENSOR_START << "; lane->getLength = "
-            << lane->getLength() << " ,lensorLength= " << lensorLength
-            << " ,SENSOR_LENGTH= " << INPUT_SENSOR_LENGTH;
-            WRITE_MESSAGE(
-                "MSSOTLE2Sensors::buildSensorForLane::" + phero_str.str());
-        )
+//        DBG(
+//            std::ostringstream phero_str;
+//            phero_str << " lane " << lane->getID() << " sensorPos= " << sensorPos
+//            << " ,SENSOR_START  " << SENSOR_START << "; lane->getLength = "
+//            << lane->getLength() << " ,lensorLength= " << lensorLength
+//            << " ,SENSOR_LENGTH= " << INPUT_SENSOR_LENGTH;
+//            WRITE_MESSAGE(
+//                "MSSOTLE2Sensors::buildSensorForLane::" + phero_str.str());
+//        )
 
         //Create sensor for lane and insert it into the map<MSLane*, MSE2Collector*>
-        newSensor = nb.buildSingleLaneE2Det(
+        newSensor = nb.createE2Detector(
                         "SOTL_E2_lane:" + lane->getID() + "_tl:" + tlLogicID,
                         DU_TL_CONTROL, lane,
-                        (lane->getLength() - sensorPos - lensorLength), lensorLength,
-                        HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS);
-//newSensor = nb.buildSingleLaneE2Det("SOTL_E2_lane:"+lane->getID()+"_tl:"+tlLogicID, DU_TL_CONTROL, lane, (lane->getLength() - sensorPos- 5), lensorLength, HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS);
+                        (lane->getLength() - sensorPos - lensorLength), INVALID_POSITION, lensorLength,
+                        HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS, "");
 
         MSNet::getInstance()->getDetectorControl().add(SUMO_TAG_LANE_AREA_DETECTOR, newSensor);
 
@@ -309,25 +288,25 @@ void MSSOTLE2Sensors::buildSensorForLane(MSLane* lane, NLDetectorBuilder& nb, SU
             std::ostringstream oss;
             oss << "Sensor on lane " << lane->getID() << " is long " << lensorLength << ", while it should be " << sensorLength << ". Continuing it on the other lanes if possible";
             WRITE_MESSAGE(oss.str())
-            for (std::vector<MSLane::IncomingLaneInfo>::const_iterator it = lane->getIncomingLanes().begin(); it != lane->getIncomingLanes().end(); ++it) {
-                const MSEdge* edge = &it->lane->getEdge();
-                if (!edge->isInternal() && !edge->isWalkingArea() && !edge->isCrossing()) {
-                    buildContinueSensior(lane, nb, sensorLength, it->lane, lensorLength);
+            for (const MSLane::IncomingLaneInfo& it : lane->getIncomingLanes()) {
+                const MSEdge& inEdge = it.lane->getEdge();
+                if (!inEdge.isInternal() && !inEdge.isWalkingArea() && !inEdge.isCrossing()) {
+                    buildContinueSensior(lane, nb, sensorLength, it.lane, lensorLength);
                 }
             }
         }
     }
 }
 
-void MSSOTLE2Sensors::buildContinueSensior(MSLane* lane, NLDetectorBuilder& nb, SUMOReal sensorLength, MSLane* continueOnLane, SUMOReal usedLength) {
-    SUMOReal availableLength = sensorLength - usedLength;
+void MSSOTLE2Sensors::buildContinueSensior(MSLane* lane, NLDetectorBuilder& nb, double sensorLength, MSLane* continueOnLane, double usedLength) {
+    double availableLength = sensorLength - usedLength;
     if (m_sensorMap.find(continueOnLane->getID()) == m_sensorMap.end()) {
-        SUMOReal length = availableLength <= continueOnLane->getLength() ? availableLength : continueOnLane->getLength();
-        MSE2Collector* newSensor = nb.buildSingleLaneE2Det(
+        double length = availableLength <= continueOnLane->getLength() ? availableLength : continueOnLane->getLength();
+        MSE2Collector* newSensor = nb.createE2Detector(
                                        "SOTL_E2_lane:" + continueOnLane->getID() + "_tl:" + tlLogicID,
                                        DU_TL_CONTROL, continueOnLane,
-                                       (continueOnLane->getLength() - length), length,
-                                       HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS);
+                                       (continueOnLane->getLength() - length), INVALID_POSITION, length,
+                                       HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS, "");
         MSNet::getInstance()->getDetectorControl().add(SUMO_TAG_LANE_AREA_DETECTOR, newSensor);
         m_sensorMap.insert(MSLaneID_MSE2Collector(continueOnLane->getID(), newSensor));
         m_continueSensorOnLanes[lane->getID()].push_back(continueOnLane->getID());
@@ -352,42 +331,41 @@ void MSSOTLE2Sensors::buildSensorForOutLane(MSLane* lane,
 }
 
 void MSSOTLE2Sensors::buildSensorForOutLane(MSLane* lane,
-        NLDetectorBuilder& nb, SUMOReal sensorLength) {
-    SUMOReal sensorPos;
-    SUMOReal lensorLength;
-    MSE2Collector* newSensor = NULL;
+        NLDetectorBuilder& nb, double sensorLength) {
+    double sensorPos;
+    double lensorLength;
+    MSE2Collector* newSensor = nullptr;
     //Check not to have more than a sensor for lane
     if (m_sensorMap.find(lane->getID()) == m_sensorMap.end()) {
 
         //Original:
-        //SUMOReal sensorLength = OUTPUT_SENSOR_LENGTH;
+        //double sensorLength = OUTPUT_SENSOR_LENGTH;
         //Check and set zero if the lane is not long enough for the specified sensor start
         sensorPos = (lane->getLength() - sensorLength)
                     - (SENSOR_START <= lane->getLength() ? SENSOR_START : 0);
 
-        //Check and trim if the lane is not long enough for the specified sensor lenght
+        //Check and trim if the lane is not long enough for the specified sensor length
         lensorLength =
             sensorLength <= (lane->getLength() - sensorPos) ?
             sensorLength : (lane->getLength() - sensorPos);
 
         //TODO check this lengths
-        DBG(
-            std::ostringstream phero_str;
-            phero_str << " lane " << lane->getID() << " sensorPos= " << sensorPos
-            << " ,SENSOR_START  " << SENSOR_START << "; lane->getLength = "
-            << lane->getLength() << " ,lensorLength= " << lensorLength
-            << " ,SENSOR_LENGTH= " << INPUT_SENSOR_LENGTH;
-            WRITE_MESSAGE(
-                "MSSOTLE2Sensors::buildSensorForLane::" + phero_str.str());
-        )
+//        DBG(
+//            std::ostringstream phero_str;
+//            phero_str << " lane " << lane->getID() << " sensorPos= " << sensorPos
+//            << " ,SENSOR_START  " << SENSOR_START << "; lane->getLength = "
+//            << lane->getLength() << " ,lensorLength= " << lensorLength
+//            << " ,SENSOR_LENGTH= " << INPUT_SENSOR_LENGTH;
+//            WRITE_MESSAGE(
+//                "MSSOTLE2Sensors::buildSensorForLane::" + phero_str.str());
+//        )
 
         //Create sensor for lane and insert it into the map<MSLane*, MSE2Collector*>
-        newSensor = nb.buildSingleLaneE2Det(
+        newSensor = nb.createE2Detector(
                         "SOTL_E2_lane:" + lane->getID() + "_tl:" + tlLogicID,
                         DU_TL_CONTROL, lane,
-                        (lane->getLength() - sensorPos - lensorLength), lensorLength,
-                        HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS);
-        //newSensor = nb.buildSingleLaneE2Det("SOTL_E2_lane:"+lane->getID()+"_tl:"+tlLogicID, DU_TL_CONTROL, lane, (lane->getLength() - sensorPos- 5), lensorLength, HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS);
+                        (lane->getLength() - sensorPos - lensorLength), INVALID_POSITION, lensorLength,
+                        HALTING_TIME_THRS, HALTING_SPEED_THRS, DIST_THRS, "");
 
         MSNet::getInstance()->getDetectorControl().add(
             SUMO_TAG_LANE_AREA_DETECTOR, newSensor);
@@ -402,31 +380,33 @@ int MSSOTLE2Sensors::countVehicles(MSLane* lane) {
 }
 
 /*
- * Estimate queue lenght according to the distance of the last vehicles
+ * Estimate queue length according to the distance of the last vehicles
  */
 
-SUMOReal MSSOTLE2Sensors::getEstimateQueueLength(std::string laneId) {
+double MSSOTLE2Sensors::getEstimateQueueLength(std::string laneId) {
     MSLaneID_MSE2CollectorMap::const_iterator sensorsIterator = m_sensorMap.find(laneId);
     if (sensorsIterator == m_sensorMap.end()) {
         assert(0);
         return 0;
     } else {
-        SUMOReal estQL = sensorsIterator->second->getEstimateQueueLength();
+        double estQL = sensorsIterator->second->getEstimateQueueLength();
         if (estQL == -1) {
             return 0;
         } else
-            DBG(
-                std::ostringstream str;
-                str << "MSSOTLE2Sensors::getEstimateQueueLength lane " << sensorsIterator->second->getLane()->getID()
-                << " laneLength " << sensorsIterator->second->getLane()->getLength() << " estimateQueueLength " <<  estQL;
-                WRITE_MESSAGE(str.str());
-            )
+//            DBG(
+//                std::ostringstream str;
+//                str << "MSSOTLE2Sensors::getEstimateQueueLength lane " << sensorsIterator->second->getLane()->getID()
+//                << " laneLength " << sensorsIterator->second->getLane()->getLength() << " estimateQueueLength " <<  estQL;
+//                WRITE_MESSAGE(str.str());
+//            )
+        {
             return estQL;
+        }
     }
 }
 
 /*
- * Estimate queue lenght according to the distance of the last vehicles that exceed a threshold
+ * Estimate queue length according to the distance of the last vehicles that exceed a threshold
  */
 
 int MSSOTLE2Sensors::estimateVehicles(std::string laneId) {
@@ -451,7 +431,7 @@ int MSSOTLE2Sensors::countVehicles(std::string laneId) {
         assert(0);
         return 0;
     }
-    unsigned int additional = 0;
+    int additional = 0;
     if (m_continueSensorOnLanes.find(laneId) != m_continueSensorOnLanes.end()) {
         for (std::vector<std::string>::iterator it = m_continueSensorOnLanes[laneId].begin(); it != m_continueSensorOnLanes[laneId].end(); ++ it) {
             if (m_sensorMap.find(*it) != m_sensorMap.end()) {
@@ -462,7 +442,7 @@ int MSSOTLE2Sensors::countVehicles(std::string laneId) {
     return count(sensorsIterator->second) + additional;
 }
 
-SUMOReal MSSOTLE2Sensors::getMaxSpeed(std::string laneId) {
+double MSSOTLE2Sensors::getMaxSpeed(std::string laneId) {
     MSLaneID_MaxSpeedMap::const_iterator sensorsIteratorIn = m_maxSpeedMap.find(laneId);
     if (sensorsIteratorIn == m_maxSpeedMap.end()) {
         assert(0);
@@ -472,40 +452,40 @@ SUMOReal MSSOTLE2Sensors::getMaxSpeed(std::string laneId) {
     return sensorsIteratorIn->second;
 }
 
-SUMOReal MSSOTLE2Sensors::meanVehiclesSpeed(MSLane* lane) {
+double MSSOTLE2Sensors::meanVehiclesSpeed(MSLane* lane) {
     return meanVehiclesSpeed(lane->getID());
 }
 
-SUMOReal MSSOTLE2Sensors::meanVehiclesSpeed(std::string laneId) {
+double MSSOTLE2Sensors::meanVehiclesSpeed(std::string laneId) {
     MSLaneID_MSE2CollectorMap::const_iterator sensorsIteratorOut = m_sensorMap.find(laneId);
     if (sensorsIteratorOut == m_sensorMap.end()) {
         assert(0);
         WRITE_ERROR("MSSOTLE2Sensors::meanVehiclesSpeed:: No lane found " + laneId);
         return 0;
     }
-    SUMOReal meanSpeedAcc = 0;
-    unsigned int totalCarNumer = 0;
+    double meanSpeedAcc = 0;
+    int totalCarNumer = 0;
     if (m_continueSensorOnLanes.find(laneId) != m_continueSensorOnLanes.end())
         for (std::vector<std::string>::iterator it = m_continueSensorOnLanes[laneId].begin(); it != m_continueSensorOnLanes[laneId].end(); ++ it) {
-            unsigned int number = 0;
-            SUMOReal mean = -1;
+            int number = 0;
+            double mean = -1;
             if (!getVelueFromSensor(*it, &MSE2Collector::getCurrentVehicleNumber, number)) {
                 continue;
             }
             totalCarNumer += number;
             getVelueFromSensor(*it, &MSE2Collector::getCurrentMeanSpeed, mean);
-            meanSpeedAcc += mean * (SUMOReal) number;
+            meanSpeedAcc += mean * (double) number;
         }
-    unsigned int number = sensorsIteratorOut->second->getCurrentVehicleNumber();
+    int number = sensorsIteratorOut->second->getCurrentVehicleNumber();
     totalCarNumer += number;
-    SUMOReal mean = sensorsIteratorOut->second->getCurrentMeanSpeed();
-    meanSpeedAcc += mean * (SUMOReal) number;
-    return totalCarNumer == 0 ? -1 : meanSpeedAcc / (SUMOReal) totalCarNumer;
+    double mean = sensorsIteratorOut->second->getCurrentMeanSpeed();
+    meanSpeedAcc += mean * (double) number;
+    return totalCarNumer == 0 ? -1 : meanSpeedAcc / (double) totalCarNumer;
 }
 
 std::string trim(std::string& str) {
-    size_t first = str.find_first_not_of(' ');
-    size_t last = str.find_last_not_of(' ');
+    int first = (int)str.find_first_not_of(' ');
+    int last = (int)str.find_last_not_of(' ');
     return str.substr(first, (last - first + 1));
 }
 
@@ -530,7 +510,7 @@ void MSSOTLE2Sensors::setVehicleWeigths(const std::string& weightString) {
         split(*typesIt, '=', typeWeight);
         if (typeWeight.size() == 2) {
             std::string type = trim(typeWeight[0]);
-            unsigned int value = TplConvert::_2int(typeWeight[1].c_str());
+            int value = StringUtils::toInt(typeWeight[1]);
             logstr << type << "=" << value << " ";
             m_typeWeightMap[type] = value;
         }
@@ -543,22 +523,24 @@ int MSSOTLE2Sensors::count(MSE2Collector* sensor) {
     if (m_typeWeightMap.size() == 0) {
         return totCars;
     }
-    unsigned int number = 0;
-    const std::list<SUMOVehicle*> vehicles = sensor->getCurrentVehicles();
+    int number = 0;
+    const std::vector<MSE2Collector::VehicleInfo*> vehicles = sensor->getCurrentVehicles();
     std::ostringstream logstr;
     logstr << "[MSSOTLE2Sensors::count]";
-    for (std::list<SUMOVehicle*>::const_iterator vit = vehicles.begin(); vit != vehicles.end(); ++vit) {
-        const std::string vtype = vit.operator * ()->getVehicleType().getID();
-        if (m_typeWeightMap.find(vtype) != m_typeWeightMap.end()) {
-            number += m_typeWeightMap[vtype];
-            DBG(logstr << " Added " << m_typeWeightMap[vtype] << " for vtype " << vtype;)
-        } else {
-            ++number;
+    for (std::vector<MSE2Collector::VehicleInfo*>::const_iterator vit = vehicles.begin(); vit != vehicles.end(); ++vit) {
+        if ((*vit)->onDetector) {
+            const std::string vtype = (*vit)->type;
+            if (m_typeWeightMap.find(vtype) != m_typeWeightMap.end()) {
+                number += m_typeWeightMap[vtype];
+//                DBG(logstr << " Added " << m_typeWeightMap[vtype] << " for vtype " << vtype;)
+            } else {
+                ++number;
+            }
         }
     }
-    DBG(if (totCars != number) {
-    logstr << ". Real number " << totCars << "; weighted " << number;
-    WRITE_MESSAGE(logstr.str());
-    })
+//    DBG(if (totCars != number) {
+//    logstr << ". Real number " << totCars << "; weighted " << number;
+//    WRITE_MESSAGE(logstr.str());
+//    })
     return number;
 }
