@@ -1,38 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSSOTLPolicyBasedTrafficLightLogic.h
 /// @author  Alessio Bonfietti
 /// @author  Riccardo Belletti
 /// @date    2014-03-20
-/// @version $Id: MSSOTLPolicyBasedTrafficLightLogic.h 22627 2017-01-18 12:56:36Z behrisch $
 ///
 // The class for SOTL Policy-based logics
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
 
-#ifndef MSSOTLPOLICYBASEDTRAFFICLIGHTLOGIC_H_
-#define MSSOTLPOLICYBASEDTRAFFICLIGHTLOGIC_H_
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-//#define SWARM_DEBUG
-#include <utils/common/SwarmDebug.h>
 #include "MSSOTLPolicy.h"
 #include "MSSOTLTrafficLightLogic.h"
 
@@ -51,23 +40,25 @@ public:
      * @brief Constructor without sensors passed
      * @param[in] tlcontrol The tls control responsible for this tls
      * @param[in] id This traffic light id
-     * @param[in] subid This tls' sub-id (program id)
+     * @param[in] programID This tls' sub-id (program id)
+     * @param[in] logicType This tls' type (static, actuated etc.)
      * @param[in] phases Definitions of the phases
      * @param[in] step The initial phase index
      * @param[in] delay The time to wait before the first switch
      * @param[in] parameters Parameters defined for the tll
      */
     MSSOTLPolicyBasedTrafficLightLogic(MSTLLogicControl& tlcontrol,
-                                       const std::string& id, const std::string& subid,
+                                       const std::string& id, const std::string& programID, const TrafficLightType logicType,
                                        const Phases& phases, int step, SUMOTime delay,
                                        const std::map<std::string, std::string>& parameters,
-                                       MSSOTLPolicy* policy) throw();
+                                       MSSOTLPolicy* policy);
 
     /**
      * @brief Constructor with sensors passed
      * @param[in] tlcontrol The tls control responsible for this tls
      * @param[in] id This tls' id
-     * @param[in] subid This tls' sub-id (program id)
+     * @param[in] programID This tls' sub-id (program id)
+     * @param[in] logicType This tls' type (static, actuated etc.)
      * @param[in] phases Definitions of the phases
      * @param[in] step The initial phase index
      * @param[in] delay The time to wait before the first switch
@@ -75,10 +66,10 @@ public:
      * @param[in] sensors The already defined sensor logic
      */
     MSSOTLPolicyBasedTrafficLightLogic(MSTLLogicControl& tlcontrol,
-                                       const std::string& id, const std::string& subid,
+                                       const std::string& id, const std::string& programID, const TrafficLightType logicType,
                                        const Phases& phases, int step, SUMOTime delay,
                                        const std::map<std::string, std::string>& parameters,
-                                       MSSOTLPolicy* policy, MSSOTLSensors* sensors) throw();
+                                       MSSOTLPolicy* policy, MSSOTLSensors* sensors);
 
     ~MSSOTLPolicyBasedTrafficLightLogic();
 
@@ -101,7 +92,7 @@ protected:
     /*
      * @brief Contains the logic to decide the phase change
      */
-    bool canRelease() throw();
+    bool canRelease();
 
     /*
      * This member has to contain the switching logic for SOTL policies
@@ -112,4 +103,3 @@ private:
     MSSOTLPolicy* myPolicy;
 };
 
-#endif /* MSSOTLPOLICYBASEDTRAFFICLIGHTLOGIC_H_ */
