@@ -1,44 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSSOTLTrafficLightLogic.h
 /// @author  Gianfilippo Slager
 /// @author	 Anna Chiara Bellini
 /// @author  Federico Caselli
 /// @date    Apr 2013
-/// @version $Id: MSSOTLTrafficLightLogic.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // The base abstract class for SOTL logics
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSSOTLTrafficLightLogic_h
-#define MSSOTLTrafficLightLogic_h
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
-
-#include <utils/common/SwarmDebug.h>
 
+#include <utils/common/RandHelper.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/SUMOTime.h>
 #include "MSSOTLDefinitions.h"
 #include "MSPhasedTrafficLightLogic.h"
 #include "MSSOTLE2Sensors.h"
-#include <utils/common/RandHelper.h>
-#include <utils/common/TplConvert.h>
 
 
 // ===========================================================================
@@ -71,14 +60,15 @@ public:
      * @brief Constructor without sensors passed
      * @param[in] tlcontrol The tls control responsible for this tls
      * @param[in] id This traffic light id
-     * @param[in] subid This tls' sub-id (program id)
+     * @param[in] programID This tls' sub-id (program id)
+     * @param[in] logicType This tls' type (static, actuated etc.)
      * @param[in] phases Definitions of the phases
      * @param[in] step The initial phase index
      * @param[in] delay The time to wait before the first switch
      * @param[in] parameters Parameters defined for the tll
      */
     MSSOTLTrafficLightLogic(MSTLLogicControl& tlcontrol, const std::string& id,
-                            const std::string& subid, const Phases& phases, int step,
+                            const std::string& programID, const TrafficLightType logicType, const Phases& phases, int step,
                             SUMOTime delay,
                             const std::map<std::string, std::string>& parameters);
 
@@ -86,7 +76,8 @@ public:
      * @brief Constructor with sensors passed
      * @param[in] tlcontrol The tls control responsible for this tls
      * @param[in] id This tls' id
-     * @param[in] subid This tls' sub-id (program id)
+     * @param[in] programID This tls' sub-id (program id)
+     * @param[in] logicType This tls' type (static, actuated etc.)
      * @param[in] phases Definitions of the phases
      * @param[in] step The initial phase index
      * @param[in] delay The time to wait before the first switch
@@ -94,7 +85,7 @@ public:
      * @param[in] sensors The already defined sensor logic
      */
     MSSOTLTrafficLightLogic(MSTLLogicControl& tlcontrol, const std::string& id,
-                            const std::string& subid, const Phases& phases, int step,
+                            const std::string& programID, const TrafficLightType logicType, const Phases& phases, int step,
                             SUMOTime delay,
                             const std::map<std::string, std::string>& parameters,
                             MSSOTLSensors* sensors);
@@ -108,7 +99,7 @@ public:
      * @param[in] nb The detector builder
      * @exception ProcessError If something fails on initialisation
      */
-    void init(NLDetectorBuilder& nb) throw(ProcessError);
+    void init(NLDetectorBuilder& nb);
 
     /*
      * This member implements the base operations for all SOTL logics.
@@ -155,19 +146,19 @@ protected:
     bool isPushButtonPressed();
 
     int getThreshold() {
-        return TplConvert::_2int(getParameter("THRESHOLD", "10").c_str());
+        return StringUtils::toInt(getParameter("THRESHOLD", "10"));
     }
 
     double getSpeedThreshold() {
-        return TplConvert::_2double(getParameter("THRESHOLDSPEED", "2").c_str());
+        return StringUtils::toDouble(getParameter("THRESHOLDSPEED", "2"));
     }
 
     double getInputSensorsLength() {
-        return TplConvert::_2double(getParameter("INSENSORSLENGTH", "100").c_str());
+        return StringUtils::toDouble(getParameter("INSENSORSLENGTH", "100"));
     }
 
     double getOutputSensorsLength() {
-        return TplConvert::_2double(getParameter("OUTSENSORSLENGTH", "80").c_str());
+        return StringUtils::toDouble(getParameter("OUTSENSORSLENGTH", "80"));
     }
 
     /*
@@ -278,7 +269,7 @@ private:
      * 2-> queue length
      */
     int getMode() {
-        return TplConvert::_2int(getParameter("MODE", "0").c_str());
+        return StringUtils::toInt(getParameter("MODE", "0"));
     }
     /*
      * Decay threshold that should be used in case of penetration rate != 100%
@@ -286,14 +277,11 @@ private:
      * 1-> active
      */
     bool isDecayThresholdActivated() {
-        return TplConvert::_2bool(getParameter("DECAY_THRESHOLD", "0").c_str());
+        return StringUtils::toBool(getParameter("DECAY_THRESHOLD", "0"));
     }
 
     double getDecayConstant() {
-        return TplConvert::_2double(getParameter("DECAY_CONSTANT", "-0.001").c_str());
+        return StringUtils::toDouble(getParameter("DECAY_CONSTANT", "-0.001"));
     }
 
 };
-
-#endif
-/****************************************************************************/
