@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSTLLogicControl.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -8,29 +20,10 @@
 /// @author  Michael Behrisch
 /// @author  Sascha Krieg
 /// @date    Sept 2002
-/// @version $Id: MSTLLogicControl.cpp 20997 2016-06-20 07:00:52Z namdre $
 ///
 // A class that stores and controls tls and switching of their programs
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <vector>
 #include <algorithm>
@@ -40,16 +33,14 @@
 #include "MSSimpleTrafficLightLogic.h"
 #include "MSTLLogicControl.h"
 #include "MSOffTrafficLightLogic.h"
+#include "MSRailSignalConstraint.h"
 #include <microsim/MSEventControl.h>
 #include <microsim/MSNet.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include <utils/common/MsgHandler.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
+#define TRACI_PROGRAM "online"
 
 // ===========================================================================
 // method definitions
@@ -57,18 +48,19 @@
 /* -------------------------------------------------------------------------
  * MSTLLogicControl::TLSLogicVariants - methods
  * ----------------------------------------------------------------------- */
-MSTLLogicControl::TLSLogicVariants::TLSLogicVariants()
-    : myCurrentProgram(0) {
+MSTLLogicControl::TLSLogicVariants::TLSLogicVariants() :
+    myCurrentProgram(nullptr),
+    myDefaultProgram(nullptr) {
 }
 
 
 MSTLLogicControl::TLSLogicVariants::~TLSLogicVariants() {
     std::map<std::string, MSTrafficLightLogic*>::const_iterator j;
-    for (std::map<std::string, MSTrafficLightLogic*>::iterator j = myVariants.begin(); j != myVariants.end(); ++j) {
-        delete(*j).second;
+    for (const auto& var : myVariants) {
+        delete var.second;
     }
-    for (std::vector<OnSwitchAction*>::iterator i = mySwitchActions.begin(); i != mySwitchActions.end(); ++i) {
-        delete *i;
+    for (OnSwitchAction* osa : mySwitchActions) {
+        delete osa;
     }
 }
 
@@ -78,10 +70,10 @@ MSTLLogicControl::TLSLogicVariants::checkOriginalTLS() const {
     bool hadErrors = false;
     for (std::map<std::string, MSTrafficLightLogic*>::const_iterator j = myVariants.begin(); j != myVariants.end(); ++j) {
         const MSTrafficLightLogic::Phases& phases = (*j).second->getPhases();
-        unsigned int linkNo = (unsigned int)(*j).second->getLinks().size();
+        int linkNo = (int)(*j).second->getLinks().size();
         bool hadProgramErrors = false;
         for (MSTrafficLightLogic::Phases::const_iterator i = phases.begin(); i != phases.end(); ++i) {
-            if ((*i)->getState().length() < linkNo) {
+            if ((int)(*i)->getState().length() < linkNo) {
                 hadProgramErrors = true;
             }
         }
@@ -100,6 +92,11 @@ MSTLLogicControl::TLSLogicVariants::saveInitialStates() {
 }
 
 
+void
+MSTLLogicControl::TLSLogicVariants::saveState(OutputDevice& /*out*/) {
+}
+
+
 bool
 MSTLLogicControl::TLSLogicVariants::addLogic(const std::string& programID,
         MSTrafficLightLogic* logic, bool netWasLoaded, bool isNewDefault) {
@@ -109,7 +106,7 @@ MSTLLogicControl::TLSLogicVariants::addLogic(const std::string& programID,
     // assert the links are set
     if (netWasLoaded) {
         // this one has not yet its links set
-        if (myCurrentProgram == 0) {
+        if (myCurrentProgram == nullptr) {
             throw ProcessError("No initial signal plan loaded for tls '" + logic->getID() + "'.");
         }
         logic->adaptLinkInformationFrom(*myCurrentProgram);
@@ -119,7 +116,14 @@ MSTLLogicControl::TLSLogicVariants::addLogic(const std::string& programID,
     }
     // add to the list of active
     if (myVariants.size() == 0 || isNewDefault) {
+        if (myCurrentProgram != nullptr) {
+            myCurrentProgram->deactivateProgram();
+        }
         myCurrentProgram = logic;
+        myCurrentProgram->activateProgram();
+        if (myVariants.size() == 0) {
+            myDefaultProgram = logic;
+        }
     }
     // add to the list of logic
     myVariants[programID] = logic;
@@ -134,7 +138,7 @@ MSTLLogicControl::TLSLogicVariants::addLogic(const std::string& programID,
 MSTrafficLightLogic*
 MSTLLogicControl::TLSLogicVariants::getLogic(const std::string& programID) const {
     if (myVariants.find(programID) == myVariants.end()) {
-        return 0;
+        return nullptr;
     }
     return myVariants.find(programID)->second;
 }
@@ -163,19 +167,20 @@ void
 MSTLLogicControl::TLSLogicVariants::setStateInstantiatingOnline(MSTLLogicControl& tlc,
         const std::string& state) {
     // build only once...
-    MSTrafficLightLogic* logic = getLogic("online");
-    if (logic == 0) {
-        MSPhaseDefinition* phase = new MSPhaseDefinition(DELTA_T, state);
+    MSTrafficLightLogic* logic = getLogic(TRACI_PROGRAM);
+    if (logic == nullptr) {
+        MSPhaseDefinition* phase = new MSPhaseDefinition(DELTA_T, state, -1);
         std::vector<MSPhaseDefinition*> phases;
         phases.push_back(phase);
-        logic = new MSSimpleTrafficLightLogic(tlc, myCurrentProgram->getID(), "online", phases, 0,
+        logic = new MSSimpleTrafficLightLogic(tlc, myCurrentProgram->getID(), TRACI_PROGRAM, TrafficLightType::STATIC, phases, 0,
                                               MSNet::getInstance()->getCurrentTimeStep() + DELTA_T,
                                               std::map<std::string, std::string>());
-        addLogic("online", logic, true, true);
+        addLogic(TRACI_PROGRAM, logic, true, true);
+        MSNet::getInstance()->createTLWrapper(logic);
     } else {
-        MSPhaseDefinition nphase(DELTA_T, state);
+        MSPhaseDefinition nphase(DELTA_T, state, -1);
         *(dynamic_cast<MSSimpleTrafficLightLogic*>(logic)->getPhases()[0]) = nphase;
-        switchTo(tlc, "online");
+        switchTo(tlc, TRACI_PROGRAM);
     }
 }
 
@@ -208,11 +213,18 @@ MSTLLogicControl::TLSLogicVariants::getActive() const {
     return myCurrentProgram;
 }
 
+MSTrafficLightLogic*
+MSTLLogicControl::TLSLogicVariants::getDefault() const {
+    return myDefaultProgram;
+}
+
 
 void
 MSTLLogicControl::TLSLogicVariants::switchTo(MSTLLogicControl& tlc, const std::string& programID) {
     // set the found wished sub-program as this tls' current one
+    myCurrentProgram->deactivateProgram();
     myCurrentProgram = getLogicInstantiatingOff(tlc, programID);
+    myCurrentProgram->activateProgram();
     myCurrentProgram->setTrafficLightSignals(MSNet::getInstance()->getCurrentTimeStep());
     executeOnSwitchActions();
 }
@@ -233,6 +245,12 @@ MSTLLogicControl::TLSLogicVariants::addLink(MSLink* link, MSLane* lane, int pos)
     }
 }
 
+void
+MSTLLogicControl::TLSLogicVariants::ignoreLinkIndex(int pos) {
+    for (std::map<std::string, MSTrafficLightLogic*>::iterator i = myVariants.begin(); i != myVariants.end(); ++i) {
+        (*i).second->ignoreLinkIndex(pos);
+    }
+}
 
 
 /* -------------------------------------------------------------------------
@@ -241,28 +259,41 @@ MSTLLogicControl::TLSLogicVariants::addLink(MSLink* link, MSLane* lane, int pos)
 /* -------------------------------------------------------------------------
  * method definitions for WAUTSwitchProcedure
  * ----------------------------------------------------------------------- */
-unsigned int
-MSTLLogicControl::WAUTSwitchProcedure::getGSPValue(const MSTrafficLightLogic& logic) const {
-    std::string val = logic.getParameter("GSP", "");
-    if (val.length() == 0) {
-        return 0;
+bool
+MSTLLogicControl::WAUTSwitchProcedure::trySwitch(SUMOTime step) {
+    // switch to the next programm if the GSP is reached
+    if (isPosAtGSP(step, *myFrom)) {
+        // adapt program's state
+        if (mySwitchSynchron) {
+            adaptLogic(step);
+        } else {
+            switchToPos(step, *myTo, getGSPTime(*myTo));
+        }
+        // switch to destination program
+        return true;
     }
-    return TplConvert::_2int(val.c_str());
+    // do not switch, yet
+    return false;
+}
+
+
+SUMOTime
+MSTLLogicControl::WAUTSwitchProcedure::getGSPTime(const MSTrafficLightLogic& logic) const {
+    return string2time(logic.getParameter("GSP", "0"));
 }
 
 
 bool
 MSTLLogicControl::WAUTSwitchProcedure::isPosAtGSP(SUMOTime currentTime, const MSTrafficLightLogic& logic) {
-    SUMOTime gspTime = TIME2STEPS(getGSPValue(logic)) % logic.getDefaultCycleTime();
-    SUMOTime programTime = logic.getOffsetFromIndex(logic.getCurrentPhaseIndex())
-                           + (logic.getCurrentPhaseDef().duration - (logic.getNextSwitchTime() - currentTime));
+    const SUMOTime gspTime = getGSPTime(logic) % logic.getDefaultCycleTime();
+    const SUMOTime programTime = logic.getOffsetFromIndex(logic.getCurrentPhaseIndex()) + logic.getSpentDuration(currentTime);
     return gspTime == programTime;
 }
 
 
 SUMOTime
 MSTLLogicControl::WAUTSwitchProcedure::getDiffToStartOfPhase(MSTrafficLightLogic& logic, SUMOTime toTime) {
-    unsigned int stepOfMyPos = logic.getIndexFromOffset(toTime);
+    int stepOfMyPos = logic.getIndexFromOffset(toTime);
     SUMOTime startOfPhase = logic.getOffsetFromIndex(stepOfMyPos);
     assert(toTime >= startOfPhase);
     return toTime - startOfPhase;
@@ -271,7 +302,7 @@ MSTLLogicControl::WAUTSwitchProcedure::getDiffToStartOfPhase(MSTrafficLightLogic
 
 void
 MSTLLogicControl::WAUTSwitchProcedure::switchToPos(SUMOTime simStep, MSTrafficLightLogic& logic, SUMOTime toTime) {
-    unsigned int stepTo = logic.getIndexFromOffset(toTime);
+    int stepTo = logic.getIndexFromOffset(toTime);
     SUMOTime diff = getDiffToStartOfPhase(logic, toTime);
     const MSPhaseDefinition& phase = logic.getPhase(stepTo);
     SUMOTime leftDuration = phase.duration - diff;
@@ -311,44 +342,16 @@ MSTLLogicControl::WAUTSwitchProcedure_GSP::WAUTSwitchProcedure_GSP(
 MSTLLogicControl::WAUTSwitchProcedure_GSP::~WAUTSwitchProcedure_GSP() {}
 
 
-bool
-MSTLLogicControl::WAUTSwitchProcedure_GSP::trySwitch(SUMOTime step) {
-    // switch to the next programm if the GSP is reached
-    if (isPosAtGSP(step, *myFrom)) {
-        // adapt program's state
-        if (mySwitchSynchron) {
-            adaptLogic(step);
-        } else {
-            switchToPos(step, *myTo, TIME2STEPS(getGSPValue(*myTo)));
-        }
-        // switch to destination program
-        return true;
-    }
-    // do not switch, yet
-    return false;
-}
-
-
 void
 MSTLLogicControl::WAUTSwitchProcedure_GSP::adaptLogic(SUMOTime step) {
-    SUMOTime gspTo = TIME2STEPS(getGSPValue(*myTo));
-    unsigned int stepTo = myTo->getIndexFromOffset(gspTo);
-    SUMOTime cycleTimeTo = myTo->getDefaultCycleTime();
-    if (gspTo == cycleTimeTo) {
-        gspTo = 0;
+    const SUMOTime gspTo = getGSPTime(*myTo) % myTo->getDefaultCycleTime();
+    const SUMOTime currentPosTo = myTo->getOffsetFromIndex(myTo->getCurrentPhaseIndex()) + myTo->getSpentDuration(step);
+    SUMOTime deltaToStretch = gspTo - currentPosTo;
+    if (deltaToStretch < 0) {
+        deltaToStretch += myTo->getDefaultCycleTime();
     }
-
-    SUMOTime currentPosTo = myTo->getOffsetFromIndex(myTo->getCurrentPhaseIndex());
-    currentPosTo += (myTo->getCurrentPhaseDef().duration - (myTo->getNextSwitchTime() - step));
-    SUMOTime diff = getDiffToStartOfPhase(*myTo, gspTo);
-
-    SUMOTime deltaToStretch = 0;
-    if (gspTo >= currentPosTo) {
-        deltaToStretch = (gspTo - currentPosTo);
-    } else {
-        deltaToStretch = (cycleTimeTo - currentPosTo + gspTo);
-    }
-    const SUMOTime newdur = myTo->getPhase(stepTo).duration - diff + deltaToStretch;
+    const int stepTo = myTo->getIndexFromOffset(gspTo);
+    const SUMOTime newdur = myTo->getPhase(stepTo).duration - getDiffToStartOfPhase(*myTo, gspTo) + deltaToStretch;
     myTo->changeStepAndDuration(myControl, step, stepTo, newdur);
 }
 
@@ -360,33 +363,25 @@ MSTLLogicControl::WAUTSwitchProcedure_GSP::adaptLogic(SUMOTime step) {
 MSTLLogicControl::WAUTSwitchProcedure_Stretch::WAUTSwitchProcedure_Stretch(
     MSTLLogicControl& control, WAUT& waut,
     MSTrafficLightLogic* from, MSTrafficLightLogic* to, bool synchron)
-    : MSTLLogicControl::WAUTSwitchProcedure(control, waut, from, to, synchron) {}
+    : MSTLLogicControl::WAUTSwitchProcedure(control, waut, from, to, synchron) {
+    int idx = 1;
+    while (myTo->knowsParameter("B" + toString(idx) + ".begin")) {
+        StretchRange def;
+        def.begin = string2time(myTo->getParameter("B" + toString(idx) + ".begin"));
+        def.end = string2time(myTo->getParameter("B" + toString(idx) + ".end"));
+        def.fac = StringUtils::toDouble(myTo->getParameter("B" + toString(idx) + ".factor"));
+        myStretchRanges.emplace_back(def);
+    }
+
+}
 
 
 MSTLLogicControl::WAUTSwitchProcedure_Stretch::~WAUTSwitchProcedure_Stretch() {}
 
 
-bool
-MSTLLogicControl::WAUTSwitchProcedure_Stretch::trySwitch(SUMOTime step) {
-    // switch to the next programm if the GSP is reached
-    if (isPosAtGSP(step, *myFrom)) {
-        // adapt program's state
-        if (mySwitchSynchron) {
-            adaptLogic(step);
-        } else {
-            switchToPos(step, *myTo, TIME2STEPS(getGSPValue(*myTo)));
-        }
-        // switch to destination program
-        return true;
-    }
-    // do not switch, yet
-    return false;
-}
-
-
 void
 MSTLLogicControl::WAUTSwitchProcedure_Stretch::adaptLogic(SUMOTime step) {
-    SUMOTime gspTo = TIME2STEPS(getGSPValue(*myTo));
+    SUMOTime gspTo = getGSPTime(*myTo);
     SUMOTime cycleTime = myTo->getDefaultCycleTime();
     // the position, where the logic has to be after synchronisation
     SUMOTime posAfterSyn = myTo->getPhaseIndexAtTime(step);
@@ -399,13 +394,11 @@ MSTLLogicControl::WAUTSwitchProcedure_Stretch::adaptLogic(SUMOTime step) {
     }
     // test, wheter cutting of the Signalplan is possible
     SUMOTime deltaPossible = 0;
-    int areasNo = getStretchAreaNo(myTo);
-    for (int i = 0; i < areasNo; i++) {
-        StretchBereichDef def = getStretchBereichDef(myTo, i + 1);
+    for (const StretchRange& def : myStretchRanges) {
         assert(def.end >= def.begin);
-        deltaPossible += TIME2STEPS(def.end - def.begin);
+        deltaPossible += def.end - def.begin;
     }
-    int stretchUmlaufAnz = (int) TplConvert::_2SUMOReal(myTo->getParameter("StretchUmlaufAnz", "").c_str());
+    int stretchUmlaufAnz = (int) StringUtils::toDouble(myTo->getParameter("StretchUmlaufAnz", ""));
     deltaPossible = stretchUmlaufAnz * deltaPossible;
     if ((deltaPossible > deltaToCut) && (deltaToCut < (cycleTime / 2))) {
         cutLogic(step, gspTo, deltaToCut);
@@ -418,20 +411,16 @@ MSTLLogicControl::WAUTSwitchProcedure_Stretch::adaptLogic(SUMOTime step) {
 
 void
 MSTLLogicControl::WAUTSwitchProcedure_Stretch::cutLogic(SUMOTime step, SUMOTime startPos, SUMOTime allCutTime) {
-    unsigned int actStep = myTo->getIndexFromOffset(startPos);
+    int actStep = myTo->getIndexFromOffset(startPos);
     // switches to startPos and cuts this phase, if there is a "Bereich"
-    int areasNo = getStretchAreaNo(myTo);
     SUMOTime toCut = 0;
-    for (int i = 0; i < areasNo; i++) {
-        StretchBereichDef def = getStretchBereichDef(myTo, i + 1);
-        const SUMOTime begin = TIME2STEPS(def.begin);
-        const SUMOTime end = TIME2STEPS(def.end);
-        size_t stepOfBegin = myTo->getIndexFromOffset(begin);
+    for (const StretchRange& def : myStretchRanges) {
+        int stepOfBegin = myTo->getIndexFromOffset(def.begin);
         if (stepOfBegin == actStep) {
-            if (begin < startPos) {
-                toCut = end - startPos;
+            if (def.begin < startPos) {
+                toCut = def.end - startPos;
             } else {
-                toCut = end - begin;
+                toCut = def.end - def.begin;
             }
             toCut = MIN2(allCutTime, toCut);
             allCutTime = allCutTime - toCut;
@@ -448,12 +437,9 @@ MSTLLogicControl::WAUTSwitchProcedure_Stretch::cutLogic(SUMOTime step, SUMOTime
             SUMOTime beginOfPhase = myTo->getOffsetFromIndex(i);
             SUMOTime durOfPhase = myTo->getPhase(i).duration;
             SUMOTime endOfPhase = beginOfPhase + durOfPhase;
-            for (int i = 0; i < areasNo; i++) {
-                StretchBereichDef def = getStretchBereichDef(myTo, i + 1);
-                SUMOTime begin = TIME2STEPS(def.begin);
-                SUMOTime end = TIME2STEPS(def.end);
-                if ((beginOfPhase <= begin) && (endOfPhase >= end)) {
-                    SUMOTime maxCutOfPhase = MIN2(end - begin, allCutTime);
+            for (const StretchRange& def : myStretchRanges) {
+                if ((beginOfPhase <= def.begin) && (endOfPhase >= def.end)) {
+                    SUMOTime maxCutOfPhase = MIN2(def.end - def.begin, allCutTime);
                     allCutTime = allCutTime - maxCutOfPhase;
                     durOfPhase = durOfPhase - maxCutOfPhase;
                 }
@@ -466,29 +452,24 @@ MSTLLogicControl::WAUTSwitchProcedure_Stretch::cutLogic(SUMOTime step, SUMOTime
 
 void
 MSTLLogicControl::WAUTSwitchProcedure_Stretch::stretchLogic(SUMOTime step, SUMOTime startPos, SUMOTime allStretchTime) {
-    unsigned int currStep = myTo->getIndexFromOffset(startPos);
+    int currStep = myTo->getIndexFromOffset(startPos);
     SUMOTime durOfPhase = myTo->getPhase(currStep).duration;
     SUMOTime remainingStretchTime = allStretchTime;
     SUMOTime StretchTimeOfPhase = 0;
-    unsigned int stretchUmlaufAnz = (unsigned int) TplConvert::_2SUMOReal(myTo->getParameter("StretchUmlaufAnz", "").c_str());
-    SUMOReal facSum = 0;
-    int areasNo = getStretchAreaNo(myTo);
-    for (int x = 0; x < areasNo; x++) {
-        StretchBereichDef def = getStretchBereichDef(myTo, x + 1);
+    int stretchUmlaufAnz = (int) StringUtils::toDouble(myTo->getParameter("StretchUmlaufAnz", ""));
+    double facSum = 0;
+    for (const StretchRange& def : myStretchRanges) {
         facSum += def.fac;
     }
     facSum *= stretchUmlaufAnz;
 
     //switch to startPos and stretch this phase, if there is a end of "bereich" between startpos and end of phase
     SUMOTime diffToStart = getDiffToStartOfPhase(*myTo, startPos);
-    for (int x = 0; x < areasNo; x++) {
-        StretchBereichDef def = getStretchBereichDef(myTo, x + 1);
-        SUMOTime end = TIME2STEPS(def.end);
+    for (const StretchRange& def : myStretchRanges) {
         SUMOTime endOfPhase = (startPos + durOfPhase - diffToStart);
-        if (end <= endOfPhase && end >= startPos) {
-            SUMOReal fac = def.fac;
-            SUMOReal actualfac = fac / facSum;
-            facSum = facSum - fac;
+        if (def.end <= endOfPhase && def.end >= startPos) {
+            double actualfac = def.fac / facSum;
+            facSum = facSum - def.fac;
             StretchTimeOfPhase = TIME2STEPS(int(STEPS2TIME(remainingStretchTime) * actualfac + 0.5));
             remainingStretchTime = allStretchTime - StretchTimeOfPhase;
         }
@@ -503,18 +484,15 @@ MSTLLogicControl::WAUTSwitchProcedure_Stretch::stretchLogic(SUMOTime step, SUMOT
     currStep = (currStep + 1) % (int)myTo->getPhases().size();
     // stretch all other phases, if there is a "bereich"
     while (remainingStretchTime > 0) {
-        for (unsigned int i = currStep; i < myTo->getPhases().size() && remainingStretchTime > 0; i++) {
+        for (int i = currStep; i < (int)myTo->getPhases().size() && remainingStretchTime > 0; i++) {
             durOfPhase = myTo->getPhase(i).duration;
             SUMOTime beginOfPhase = myTo->getOffsetFromIndex(i);
             SUMOTime endOfPhase = beginOfPhase + durOfPhase;
-            for (int j = 0; j < areasNo && remainingStretchTime > 0; j++) {
-                StretchBereichDef def = getStretchBereichDef(myTo, j + 1);
-                SUMOTime end = TIME2STEPS(def.end);
-                SUMOReal fac = def.fac;
-                if ((beginOfPhase <= end) && (endOfPhase >= end)) {
-                    SUMOReal actualfac = fac / facSum;
+            for (const StretchRange& def : myStretchRanges) {
+                if ((beginOfPhase <= def.end) && (endOfPhase >= def.end)) {
+                    double actualfac = def.fac / facSum;
                     StretchTimeOfPhase = TIME2STEPS(int(STEPS2TIME(remainingStretchTime) * actualfac + 0.5));
-                    facSum -= fac;
+                    facSum -= def.fac;
                     durOfPhase += StretchTimeOfPhase;
                     remainingStretchTime -= StretchTimeOfPhase;
                 }
@@ -525,26 +503,6 @@ MSTLLogicControl::WAUTSwitchProcedure_Stretch::stretchLogic(SUMOTime step, SUMOT
     }
 }
 
-int
-MSTLLogicControl::WAUTSwitchProcedure_Stretch::getStretchAreaNo(MSTrafficLightLogic* from) const {
-    int no = 0;
-    while (from->getParameter("B" + toString(no + 1) + ".begin", "") != "") {
-        no++;
-    }
-    return no;
-}
-
-
-MSTLLogicControl::WAUTSwitchProcedure_Stretch::StretchBereichDef
-MSTLLogicControl::WAUTSwitchProcedure_Stretch::getStretchBereichDef(MSTrafficLightLogic* from, int index) const {
-    StretchBereichDef def;
-    def.begin = TplConvert::_2SUMOReal(from->getParameter("B" + toString(index) + ".begin", "").c_str());
-    def.end = TplConvert::_2SUMOReal(from->getParameter("B" + toString(index) + ".end", "").c_str());
-    def.fac = TplConvert::_2SUMOReal(from->getParameter("B" + toString(index) + ".factor", "").c_str());
-    return def;
-}
-
-
 
 /* -------------------------------------------------------------------------
  * method definitions for MSTLLogicControl
@@ -556,11 +514,11 @@ MSTLLogicControl::MSTLLogicControl()
 MSTLLogicControl::~MSTLLogicControl() {
     // delete tls
     for (std::map<std::string, TLSLogicVariants*>::const_iterator i = myLogics.begin(); i != myLogics.end(); ++i) {
-        delete(*i).second;
+        delete (*i).second;
     }
     // delete WAUTs
     for (std::map<std::string, WAUT*>::const_iterator i = myWAUTs.begin(); i != myWAUTs.end(); ++i) {
-        delete(*i).second;
+        delete (*i).second;
     }
 }
 
@@ -598,7 +556,7 @@ MSTrafficLightLogic*
 MSTLLogicControl::get(const std::string& id, const std::string& programID) const {
     std::map<std::string, TLSLogicVariants*>::const_iterator i = myLogics.find(id);
     if (i == myLogics.end()) {
-        return 0;
+        return nullptr;
     }
     return (*i).second->getLogic(programID);
 }
@@ -662,7 +620,7 @@ MSTrafficLightLogic*
 MSTLLogicControl::getActive(const std::string& id) const {
     std::map<std::string, TLSLogicVariants*>::const_iterator i = myLogics.find(id);
     if (i == myLogics.end()) {
-        return 0;
+        return nullptr;
     }
     return (*i).second->getActive();
 }
@@ -682,7 +640,7 @@ MSTLLogicControl::switchTo(const std::string& id, const std::string& programID)
 
 void
 MSTLLogicControl::addWAUT(SUMOTime refTime, const std::string& id,
-                          const std::string& startProg) {
+                          const std::string& startProg, SUMOTime period) {
     // check whether the waut was already defined
     if (myWAUTs.find(id) != myWAUTs.end()) {
         // report an error if so
@@ -692,6 +650,7 @@ MSTLLogicControl::addWAUT(SUMOTime refTime, const std::string& id,
     w->id = id;
     w->refTime = refTime;
     w->startProg = startProg;
+    w->period = period;
     myWAUTs[id] = w;
 }
 
@@ -705,9 +664,13 @@ MSTLLogicControl::addWAUTSwitch(const std::string& wautid,
         throw InvalidArgument("Waut '" + wautid + "' was not yet defined.");
     }
     // build and save the waut switch definition
+    WAUT* waut = myWAUTs[wautid];
     WAUTSwitch s;
     s.to = to;
-    s.when = (myWAUTs[wautid]->refTime + when) % 86400000;
+    s.when = (waut->refTime + when);
+    if (waut->period > 0) {
+        s.when = s.when % waut->period;
+    }
     myWAUTs[wautid]->switches.push_back(s);
 }
 
@@ -772,8 +735,8 @@ MSTLLogicControl::closeWAUT(const std::string& wautid) {
     if (first != w->switches.end()) {
         std::vector<WAUTSwitch>::const_iterator mbegin = w->switches.begin();
         MSNet::getInstance()->getBeginOfTimestepEvents()->addEvent(
-            new SwitchInitCommand(*this, wautid, (unsigned int)distance(mbegin, first)),
-            (*first).when, MSEventControl::NO_CHANGE);
+            new SwitchInitCommand(*this, wautid, (int)distance(mbegin, first)),
+            (*first).when);
     }
     /*
     // set the current program to all junctions
@@ -787,14 +750,15 @@ MSTLLogicControl::closeWAUT(const std::string& wautid) {
 SUMOTime
 MSTLLogicControl::initWautSwitch(MSTLLogicControl::SwitchInitCommand& cmd) {
     const std::string& wautid = cmd.getWAUTID();
-    unsigned int& index = cmd.getIndex();
-    WAUTSwitch s = myWAUTs[wautid]->switches[index];
+    int& index = cmd.getIndex();
+    WAUT* waut = myWAUTs[wautid];
+    WAUTSwitch s = waut->switches[index];
     for (std::vector<WAUTJunction>::iterator i = myWAUTs[wautid]->junctions.begin(); i != myWAUTs[wautid]->junctions.end(); ++i) {
         // get the current program and the one to instantiate
         TLSLogicVariants* vars = myLogics.find((*i).junction)->second;
         MSTrafficLightLogic* from = vars->getActive();
         MSTrafficLightLogic* to = vars->getLogicInstantiatingOff(*this, s.to);
-        WAUTSwitchProcedure* proc = 0;
+        WAUTSwitchProcedure* proc = nullptr;
         if ((*i).procedure == "GSP") {
             proc = new WAUTSwitchProcedure_GSP(*this, *myWAUTs[wautid], from, to, (*i).synchron);
         } else if ((*i).procedure == "Stretch") {
@@ -812,8 +776,15 @@ MSTLLogicControl::initWautSwitch(MSTLLogicControl::SwitchInitCommand& cmd) {
         myCurrentlySwitched.push_back(p);
     }
     index++;
-    if (index == static_cast<unsigned int>(myWAUTs[wautid]->switches.size())) {
-        return 0;
+    if (index == (int)waut->switches.size()) {
+        if (waut->period <= 0) {
+            return 0;
+        } else {
+            index = 0; // start over
+            for (WAUTSwitch& ws : waut->switches) {
+                ws.when += waut->period;
+            }
+        }
     }
     return myWAUTs[wautid]->switches[index].when - MSNet::getInstance()->getCurrentTimeStep();
 }
@@ -825,7 +796,10 @@ MSTLLogicControl::check2Switch(SUMOTime step) {
         const WAUTSwitchProcess& proc = *i;
         if (proc.proc->trySwitch(step)) {
             delete proc.proc;
-            switchTo((*i).to->getID(), (*i).to->getProgramID());
+            // do not switch away from TraCI control
+            if (getActive(proc.to->getID())->getProgramID() != TRACI_PROGRAM) {
+                switchTo(proc.to->getID(), proc.to->getProgramID());
+            }
             i = myCurrentlySwitched.erase(i);
         } else {
             ++i;
@@ -843,10 +817,25 @@ MSTLLogicControl::getPhaseDef(const std::string& tlid) const {
 
 void
 MSTLLogicControl::switchOffAll() {
-    for (std::map<std::string, TLSLogicVariants*>::const_iterator i = myLogics.begin(); i != myLogics.end(); ++i) {
-        (*i).second->addLogic("off",  new MSOffTrafficLightLogic(*this, (*i).first), true, true);
+    for (const auto& logic : myLogics) {
+        logic.second->addLogic("off",  new MSOffTrafficLightLogic(*this, logic.first), true, true);
     }
 }
 
-/****************************************************************************/
 
+void
+MSTLLogicControl::saveState(OutputDevice& out) {
+    MSRailSignalConstraint::saveState(out); // always saves vehicle tracker states
+    for (const auto& logic : myLogics) {
+        logic.second->saveState(out);
+    }
+}
+
+
+void
+MSTLLogicControl::clearState() {
+    MSRailSignalConstraint::clearState();
+}
+
+
+/****************************************************************************/
