@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSLaneSpeedTrigger.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -6,31 +18,10 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: MSLaneSpeedTrigger.cpp 24010 2017-04-21 17:00:09Z behrisch $
 ///
 // Changes the speed allowed on a set of lanes
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/common/MsgHandler.h>
@@ -38,7 +29,7 @@
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/xml/XMLSubSys.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <microsim/MSEventControl.h>
 #include <microsim/MSLane.h>
 #include <microsim/MSNet.h>
@@ -50,6 +41,11 @@
 #include <mesosim/MESegment.h>
 
 
+// ===========================================================================
+// static members
+// ===========================================================================
+std::map<std::string, MSLaneSpeedTrigger*> MSLaneSpeedTrigger::myInstances;
+
 // ===========================================================================
 // method definitions
 // ===========================================================================
@@ -64,6 +60,7 @@ MSLaneSpeedTrigger::MSLaneSpeedTrigger(const std::string& id,
     myAmOverriding(false),
     mySpeedOverrideValue(destLanes[0]->getSpeedLimit()),
     myDidInit(false) {
+    myInstances[id] = this;
     if (file != "") {
         if (!XMLSubSys::runParser(*this, file)) {
             throw ProcessError();
@@ -115,7 +112,7 @@ MSLaneSpeedTrigger::processCommand(bool move2next, SUMOTime currentTime) {
         if (myDestLanes.size() > 0 && myDestLanes.front()->getSpeedLimit() != speed) {
             myDestLanes.front()->getEdge().setMaxSpeed(speed);
             MESegment* first = MSGlobals::gMesoNet->getSegmentForEdge(myDestLanes.front()->getEdge());
-            while (first != 0) {
+            while (first != nullptr) {
                 first->setSpeed(speed, currentTime, -1);
                 first = first->getNextSegment();
             }
@@ -230,4 +227,3 @@ MSLaneSpeedTrigger::getCurrentSpeed() const {
 
 
 /****************************************************************************/
-
