@@ -1,36 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSTriggeredRerouter.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
+/// @author  Mirco Sturari
 /// @date    Mon, 25 July 2005
-/// @version $Id: MSTriggeredRerouter.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Reroutes vehicles passing an edge
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSTriggeredRerouter_h
-#define MSTriggeredRerouter_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -38,7 +29,7 @@
 #include <microsim/MSMoveReminder.h>
 #include "MSTrigger.h"
 #include <utils/xml/SUMOSAXHandler.h>
-#include <utils/common/RandomDistributor.h>
+#include <utils/distribution/RandomDistributor.h>
 
 
 // ===========================================================================
@@ -48,6 +39,7 @@ class MSNet;
 class MSLane;
 class MSRoute;
 class SUMOVehicle;
+class MSParkingArea;
 
 
 // ===========================================================================
@@ -80,32 +72,41 @@ public:
      */
     MSTriggeredRerouter(const std::string& id,
                         const MSEdgeVector& edges,
-                        SUMOReal prob, const std::string& file, bool off);
+                        double prob, const std::string& file, bool off,
+                        SUMOTime timeThreshold,
+                        const std::string& vTypes);
 
 
     /** @brief Destructor */
     virtual ~MSTriggeredRerouter();
 
+    typedef std::pair<MSParkingArea*, bool> ParkingAreaVisible;
 
     /**
      * @struct RerouteInterval
      * Describes the rerouting definitions valid for an interval
      */
     struct RerouteInterval {
+        /// unique ID for this interval
+        long long id;
         /// The begin time these definitions are valid
         SUMOTime begin;
         /// The end time these definitions are valid
         SUMOTime end;
         /// The list of closed edges
         MSEdgeVector closed;
+        /// The list of closed lanes
+        std::vector<MSLane*> closedLanes;
+        /// The list of edges that are affect by closed lanes
+        MSEdgeVector closedLanesAffected;
         /// The distributions of new destinations to use
         RandomDistributor<MSEdge*> edgeProbs;
         /// The distributions of new routes to use
         RandomDistributor<const MSRoute*> routeProbs;
         /// The permissions to use
         SVCPermissions permissions;
-        /// The old permissions for all lanes
-        std::vector<SVCPermissions> prevPermissions;
+        /// The distributions of new parking areas to use as destinations
+        RandomDistributor<ParkingAreaVisible> parkProbs;
     };
 
     /** @brief Tries to reroute the vehicle
@@ -122,7 +123,7 @@ public:
      * @see MSMoveReminder::notifyEnter
      * @see MSMoveReminder::Notification
      */
-    bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason);
+    bool notifyEnter(SUMOTrafficObject& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
     /// @name Methods called on vehicle movement / state change, overwriting MSDevice
     /// @{
@@ -135,7 +136,7 @@ public:
      * @param[in] newSpeed Moving speed.
      * @return True (always).
      */
-    bool notifyMove(SUMOVehicle& veh, SUMOReal oldPos, SUMOReal newPos, SUMOReal newSpeed);
+    bool notifyMove(SUMOTrafficObject& veh, double oldPos, double newPos, double newSpeed);
 
     /** @brief Removes the reminder
      *
@@ -145,7 +146,7 @@ public:
      * @param[in] isLaneChange whether the vehicle changed from the lane
      * @return false if the vehicle left th edge
      */
-    bool notifyLeave(SUMOVehicle& veh, SUMOReal lastPos, MSMoveReminder::Notification reason);
+    bool notifyLeave(SUMOTrafficObject& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
     /// Returns the rerouting definition valid for the given time and vehicle, 0 if none
     const RerouteInterval* getCurrentReroute(SUMOTime time, SUMOVehicle& veh) const;
@@ -160,16 +161,27 @@ public:
     void setUserMode(bool val);
 
     /// Sets the probability with which a vehicle is rerouted given by the user
-    void setUserUsageProbability(SUMOReal prob);
+    void setUserUsageProbability(double prob);
 
     /// Returns whether the user is setting the rerouting probability
     bool inUserMode() const;
 
     /// Returns the rerouting probability
-    SUMOReal getProbability() const;
+    double getProbability() const;
 
     /// Returns the rerouting probability given by the user
-    SUMOReal getUserProbability() const;
+    double getUserProbability() const;
+
+    double getWeight(SUMOVehicle& veh, const std::string param, const double defaultWeight) const;
+
+    MSParkingArea* rerouteParkingArea(const MSTriggeredRerouter::RerouteInterval* rerouteDef,
+                                      SUMOVehicle& veh, bool& newDestination, ConstMSEdgeVector& newRoute) const;
+
+    /// @brief return all rerouter instances
+    static const std::map<std::string, MSTriggeredRerouter*>& getInstances() {
+        return myInstances;
+    }
+
 
 protected:
     /// @name inherited from GenericSAXHandler
@@ -195,16 +207,30 @@ protected:
     virtual void myEndElement(int element);
     //@}
 
+    /** @brief Checks whether the detector measures vehicles of the given type.
+    *
+    * @param[in] veh the vehicle of which the type is checked.
+    * @return whether it should be measured
+    */
+    bool vehicleApplies(const SUMOVehicle& veh) const;
+
+
 protected:
     /// List of rerouting definition intervals
     std::vector<RerouteInterval> myIntervals;
 
     /// The probability and the user-given probability
-    SUMOReal myProbability, myUserProbability;
+    double myProbability, myUserProbability;
 
     /// Information whether the current rerouting probability is the user-given
     bool myAmInUserMode;
 
+    // @brief waiting time threshold for activation
+    SUMOTime myTimeThreshold;
+
+    /// @brief The vehicle types to look for (empty means all)
+    std::set<std::string> myVehicleTypes;
+
     /// @name members used during loading
     //@{
 
@@ -212,9 +238,13 @@ protected:
     SUMOTime myCurrentIntervalBegin, myCurrentIntervalEnd;
     /// List of closed edges
     MSEdgeVector myCurrentClosed;
+    /// List of closed lanes
+    std::vector<MSLane*> myCurrentClosedLanes;
     /// List of permissions for closed edges
     SVCPermissions myCurrentPermissions;
     /// new destinations with probabilities
+    RandomDistributor<ParkingAreaVisible> myCurrentParkProb;
+    /// new destinations with probabilities
     RandomDistributor<MSEdge*> myCurrentEdgeProb;
     /// new routes with probabilities
     RandomDistributor<const MSRoute*> myCurrentRouteProb;
@@ -225,6 +255,8 @@ protected:
     static MSEdge mySpecialDest_keepDestination;
     static MSEdge mySpecialDest_terminateRoute;
 
+    static std::map<std::string, MSTriggeredRerouter*> myInstances;
+
 private:
     /// @brief Invalidated copy constructor.
     MSTriggeredRerouter(const MSTriggeredRerouter&);
@@ -234,9 +266,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
