@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBAlgorithms_Ramps.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    29. March 2012
-/// @version $Id: NBAlgorithms_Ramps.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Algorithms for highway on-/off-ramps computation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBAlgorithms_Ramps_h
-#define NBAlgorithms_Ramps_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 
@@ -59,7 +49,7 @@ public:
      * @param[in, changed] nb The network builder which contains the current network representation
      * @param[in] oc The options container
      */
-    static void computeRamps(NBNetBuilder& nb, OptionsCont& oc);
+    static void computeRamps(NBNetBuilder& nb, OptionsCont& oc, bool mayAddOrRemove);
 
     /// @brief suffix for newly generated on-ramp edges
     static const std::string ADDED_ON_RAMP_EDGE;
@@ -70,10 +60,11 @@ private:
      * @param[in] minHighwaySpeed The minimum speed limit a highway must have for being a highway
      * @param[in] maxRampSpeed The maximum speed limit a ramp must have for being a ramp
      * @param[in] noramps Edges that shall not be treated as ramps
+     * @param[in] minWeaveLength The minimum length for weaving areas
      * @return Whether the node is assumed to be an on-ramp begin
      */
     static bool mayNeedOnRamp(NBNode* cur, double minHighwaySpeed, double maxRampSpeed,
-                              const std::set<std::string>& noramps);
+                              const std::set<std::string>& noramps, double minWeaveLength);
 
 
     /** @brief Determines whether the given node may be an off-ramp end
@@ -96,7 +87,7 @@ private:
      * @param[in] dontSplit Whether no edges shall be split
      * @param[in, filled] incremented The list of edges which lane number was already incremented
      */
-    static void buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDistrictCont& dc, double rampLength, bool dontSplit);
+    static void buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDistrictCont& dc, double rampLength, bool dontSplit, bool addLanes);
 
 
     /** @brief Builds an off-ramp ending at the given node
@@ -108,7 +99,8 @@ private:
      * @param[in] dontSplit Whether no edges shall be split
      * @param[in, filled] incremented The list of edges which lane number was already incremented
      */
-    static void buildOffRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDistrictCont& dc, double rampLength, bool dontSplit);
+    static void buildOffRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDistrictCont& dc, double rampLength, bool dontSplit, bool addLanes,
+                             const std::set<NBNode*, ComparatorIdLess>& potOnRamps);
 
 
     static void getOnRampEdges(NBNode* n, NBEdge** potHighway, NBEdge** potRamp, NBEdge** other);
@@ -141,10 +133,7 @@ private:
 
     /// @brief whether the edge has a mode that does not indicate a ramp edge
     static bool hasWrongMode(NBEdge* edge);
+
+    /// @brief shift ramp geometry to merge smoothly with the motorway
+    static void patchRampGeometry(NBEdge* potRamp, NBEdge* first, NBEdge* potHighway, bool onRamp);
 };
-
-
-#endif
-
-/****************************************************************************/
-
