@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBConnection.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NBConnection.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // The class holds a description of a connection between two edges
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <sstream>
 #include <iostream>
@@ -42,7 +33,7 @@
 // static members
 // ===========================================================================
 const int NBConnection::InvalidTlIndex = -1;
-const NBConnection NBConnection::InvalidConnection("invalidFrom", 0, "invalidTo", 0);
+const NBConnection NBConnection::InvalidConnection("invalidFrom", nullptr, "invalidTo", nullptr);
 
 // ===========================================================================
 // method definitions
@@ -51,7 +42,8 @@ NBConnection::NBConnection(NBEdge* from, NBEdge* to) :
     myFrom(from), myTo(to),
     myFromID(from->getID()), myToID(to->getID()),
     myFromLane(-1), myToLane(-1),
-    myTlIndex(InvalidTlIndex) {
+    myTlIndex(InvalidTlIndex),
+    myTlIndex2(InvalidTlIndex) {
 }
 
 
@@ -60,21 +52,23 @@ NBConnection::NBConnection(const std::string& fromID, NBEdge* from,
     myFrom(from), myTo(to),
     myFromID(fromID), myToID(toID),
     myFromLane(-1), myToLane(-1),
-    myTlIndex(InvalidTlIndex) {
+    myTlIndex(InvalidTlIndex),
+    myTlIndex2(InvalidTlIndex) {
 }
 
 
 NBConnection::NBConnection(NBEdge* from, int fromLane,
-                           NBEdge* to, int toLane, int tlIndex) :
+                           NBEdge* to, int toLane, int tlIndex, int tlIndex2) :
     myFrom(from), myTo(to),
     myFromLane(fromLane), myToLane(toLane),
-    myTlIndex(tlIndex) {
+    myTlIndex(tlIndex),
+    myTlIndex2(tlIndex2) {
     /* @todo what should we assert here?
     assert(myFromLane<0||from->getNumLanes()>(int) myFromLane);
     assert(myToLane<0||to->getNumLanes()>(int) myToLane);
     */
-    myFromID = from->getID();
-    myToID = to != 0 ? to->getID() : "";
+    myFromID = from != nullptr ? from->getID() : "";
+    myToID = to != nullptr ? to->getID() : "";
 }
 
 
@@ -85,7 +79,8 @@ NBConnection::NBConnection(const NBConnection& c) :
     myFrom(c.myFrom), myTo(c.myTo),
     myFromID(c.myFromID), myToID(c.myToID),
     myFromLane(c.myFromLane), myToLane(c.myToLane),
-    myTlIndex(c.myTlIndex) {
+    myTlIndex(c.myTlIndex),
+    myTlIndex2(c.myTlIndex2) {
 }
 
 
@@ -105,7 +100,7 @@ bool
 NBConnection::replaceFrom(NBEdge* which, NBEdge* by) {
     if (myFrom == which) {
         myFrom = by;
-        if (myFrom != 0) {
+        if (myFrom != nullptr) {
             myFromID = myFrom->getID();
         } else {
             myFromID = "invalidFrom";
@@ -121,7 +116,7 @@ NBConnection::replaceFrom(NBEdge* which, int whichLane,
                           NBEdge* by, int byLane) {
     if (myFrom == which && (myFromLane == whichLane || myFromLane < 0 || whichLane < 0)) {
         myFrom = by;
-        if (myFrom != 0) {
+        if (myFrom != nullptr) {
             myFromID = myFrom->getID();
         } else {
             myFromID = "invalidFrom";
@@ -139,7 +134,7 @@ bool
 NBConnection::replaceTo(NBEdge* which, NBEdge* by) {
     if (myTo == which) {
         myTo = by;
-        if (myTo != 0) {
+        if (myTo != nullptr) {
             myToID = myTo->getID();
         } else {
             myToID = "invalidTo";
@@ -155,7 +150,7 @@ NBConnection::replaceTo(NBEdge* which, int whichLane,
                         NBEdge* by, int byLane) {
     if (myTo == which && (myToLane == whichLane || myFromLane < 0 || whichLane < 0)) {
         myTo = by;
-        if (myTo != 0) {
+        if (myTo != nullptr) {
             myToID = myTo->getID();
         } else {
             myToID = "invalidTo";
@@ -189,7 +184,8 @@ NBConnection::operator==(const NBConnection& c) const {
     return (myFrom    == c.myFrom     && myTo    == c.myTo &&
             myFromID  == c.myFromID   && myToID  == c.myToID &&
             myFromLane == c.myFromLane && myToLane == c.myToLane &&
-            myTlIndex == c.myTlIndex);
+            myTlIndex == c.myTlIndex &&
+            myTlIndex2 == c.myTlIndex2);
 }
 
 
@@ -197,7 +193,7 @@ bool
 NBConnection::check(const NBEdgeCont& ec) {
     myFrom = checkFrom(ec);
     myTo = checkTo(ec);
-    return myFrom != 0 && myTo != 0;
+    return myFrom != nullptr && myTo != nullptr;
 }
 
 
@@ -246,10 +242,10 @@ NBConnection::getToLane() const {
 
 
 void
-NBConnection::shiftLaneIndex(NBEdge* edge, int offset) {
-    if (myFrom == edge) {
+NBConnection::shiftLaneIndex(NBEdge* edge, int offset, int threshold) {
+    if (myFrom == edge && myFromLane > threshold) {
         myFromLane += offset;
-    } else if (myTo == edge) {
+    } else if (myTo == edge && myToLane > threshold) {
         myToLane += offset;
     }
 }
@@ -268,6 +264,4 @@ operator<<(std::ostream& os, const NBConnection& c) {
 }
 
 
-
 /****************************************************************************/
-
