@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBContHelper.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: NBContHelper.h 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // Some methods for traversing lists of edges
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBContHelper_h
-#define NBContHelper_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <iostream>
@@ -84,45 +74,17 @@ public:
     class relative_outgoing_edge_sorter {
     public:
         /// constructor
-        explicit relative_outgoing_edge_sorter(NBEdge* e) : myEdge(e) {}
-
-    public:
-        /// comparing operation
-        int operator()(NBEdge* e1, NBEdge* e2) const;
-
-    private:
-        /// the edge to compute the relative angle of
-        NBEdge* myEdge;
-    };
-
-
-    /**
-     * straightness_sorter
-     * Class to sort edges according to how straight they are in relation to the
-     * reference edge at the given node
-     */
-    class straightness_sorter {
-    public:
+        explicit relative_outgoing_edge_sorter(NBEdge* e) : myAngle(e->getEndAngle()) {}
         /// constructor
-        explicit straightness_sorter(const NBNode* n, const NBEdge* e):
-            myRefIncoming(e->getToNode() == n) {
-            if (myRefIncoming) {
-                myReferencePos = e->getLaneShape(0).back();
-                myReferenceAngle = e->getShapeEndAngle();
-            } else {
-                myReferencePos = e->getLaneShape(0).front();
-                myReferenceAngle = e->getShapeStartAngle();
-            }
-        }
+        explicit relative_outgoing_edge_sorter(double angle) : myAngle(angle) {}
 
     public:
         /// comparing operation
-        int operator()(NBEdge* e1, NBEdge* e2) const;
+        bool operator()(const NBEdge* e1, const NBEdge* e2) const;
 
     private:
-        bool myRefIncoming;
-        Position myReferencePos;
-        double myReferenceAngle;
+        /// @brief the reference angle to compare edges agains
+        double myAngle;
     };
 
 
@@ -136,15 +98,17 @@ public:
     class relative_incoming_edge_sorter {
     public:
         /// constructor
-        explicit relative_incoming_edge_sorter(NBEdge* e) : myEdge(e) {}
+        explicit relative_incoming_edge_sorter(NBEdge* e) : myAngle(e->getStartAngle()) {}
+        /// constructor
+        explicit relative_incoming_edge_sorter(double angle) : myAngle(angle) {}
 
     public:
         /// comparing operation
-        int operator()(NBEdge* e1, NBEdge* e2) const;
+        bool operator()(const NBEdge* e1, const NBEdge* e2) const;
 
     private:
-        /// the edge to compute the relative angle of
-        NBEdge* myEdge;
+        /// @brief the reference angle to compare edges agains
+        double myAngle;
     };
 
 
@@ -181,8 +145,10 @@ public:
          * @param[in] e The edge to which the sorting relates
          * @param[in] n The node to consider
          */
-        explicit edge_opposite_direction_sorter(const NBEdge* const e, const NBNode* const n)
-            : myNode(n) {
+        explicit edge_opposite_direction_sorter(const NBEdge* const e, const NBNode* const n, bool regardPriority) :
+            myNode(n),
+            myEdge(e),
+            myRegardPriority(regardPriority) {
             myAngle = getEdgeAngleAt(e, n);
         }
 
@@ -192,7 +158,11 @@ public:
          * @return Which edge is more opposite to the related one
          */
         int operator()(NBEdge* e1, NBEdge* e2) const {
-            return getDiff(e1) > getDiff(e2);
+            if (!myRegardPriority || e1->getPriority() == e2->getPriority() || e1 == myEdge || e2 == myEdge) {
+                return getDiff(e1) > getDiff(e2);
+            } else {
+                return e1->getPriority() > e2->getPriority();
+            }
         }
 
     protected:
@@ -219,12 +189,19 @@ public:
         }
 
     private:
-        /// @brief The angle of the related edge at the given node
-        double myAngle;
 
         /// @brief The related node
         const NBNode* const myNode;
 
+        /// @brief the reference edge
+        const NBEdge* const myEdge;
+
+        /// @brief The angle of the related edge at the given node
+        double myAngle;
+
+        /// @brief Whether edge priority may override closer angles
+        bool myRegardPriority;
+
     private:
         /// @brief Invalidated assignment operator
         edge_opposite_direction_sorter& operator=(const edge_opposite_direction_sorter& s);
@@ -237,23 +214,46 @@ public:
      * edge_similar_direction_sorter
      * Class to sort edges by their angle in relation to the given edge
      * The resulting list should have the edge in the most similar direction
-     * to the given edge as her first entry
+     * to the given edge as its first entry
      */
     class edge_similar_direction_sorter {
     public:
         /// constructor
-        explicit edge_similar_direction_sorter(const NBEdge* const e)
-            : myAngle(e->getTotalAngle()) {}
+        explicit edge_similar_direction_sorter(const NBEdge* const e, bool outgoing = true) :
+            myCompareOutgoing(outgoing),
+            myAngle(outgoing ? e->getShapeEndAngle() : e->getShapeStartAngle())
+        {}
 
         /// comparing operation
-        int operator()(NBEdge* e1, NBEdge* e2) const {
-            double d1 = GeomHelper::getMinAngleDiff(e1->getTotalAngle(), myAngle);
-            double d2 = GeomHelper::getMinAngleDiff(e2->getTotalAngle(), myAngle);
-            return d1 < d2;
+        int operator()(const NBEdge* e1, const NBEdge* e2) const {
+            const double d1 = angleDiff(myCompareOutgoing ? e1->getShapeStartAngle() : e1->getShapeEndAngle(), myAngle);
+            const double d2 = angleDiff(myCompareOutgoing ? e2->getShapeStartAngle() : e2->getShapeEndAngle(), myAngle);
+            if (fabs(fabs(d1) - fabs(d2)) < NUMERICAL_EPS) {
+                if (fabs(d1 - d2) > NUMERICAL_EPS) {
+                    return d1 < d2;
+                } else {
+                    return e1->getNumericalID() < e2->getNumericalID();
+                }
+            }
+            return fabs(d1) < fabs(d2);
         }
 
+    private:
+        double angleDiff(const double angle1, const double angle2) const {
+            double d = angle2 - angle1;
+            while (d >= 180.) {
+                d -= 360.;
+            }
+            while (d < -180.) {
+                d += 360.;
+            }
+            return d;
+        }
+
+
     private:
         /// the angle to find the edge with the opposite direction
+        bool myCompareOutgoing;
         double myAngle;
     };
 
@@ -404,7 +404,7 @@ public:
 
     public:
         /// comparing operation
-        int operator()(const NBEdge* e1, const NBEdge* e2) const;
+        bool operator()(const NBEdge* e1, const NBEdge* e2) const;
 
     private:
         /// the edge to compute the relative angle of
@@ -412,9 +412,3 @@ public:
     };
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
