@@ -1,33 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBDistrictCont.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBDistrictCont.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A container for districts
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <iostream>
@@ -46,7 +37,7 @@ NBDistrictCont::NBDistrictCont() {}
 
 NBDistrictCont::~NBDistrictCont() {
     for (DistrictCont::iterator i = myDistricts.begin(); i != myDistricts.end(); i++) {
-        delete((*i).second);
+        delete ((*i).second);
     }
     myDistricts.clear();
 }
@@ -67,7 +58,7 @@ NBDistrict*
 NBDistrictCont::retrieve(const std::string& id) const {
     DistrictCont::const_iterator i = myDistricts.find(id);
     if (i == myDistricts.end()) {
-        return 0;
+        return nullptr;
     }
     return (*i).second;
 }
@@ -83,7 +74,7 @@ bool
 NBDistrictCont::addSource(const std::string& dist, NBEdge* const source,
                           double weight) {
     NBDistrict* o = retrieve(dist);
-    if (o == 0) {
+    if (o == nullptr) {
         return false;
     }
     return o->addSource(source, weight);
@@ -94,7 +85,7 @@ bool
 NBDistrictCont::addSink(const std::string& dist, NBEdge* const destination,
                         double weight) {
     NBDistrict* o = retrieve(dist);
-    if (o == 0) {
+    if (o == nullptr) {
         return false;
     }
     return o->addSink(destination, weight);
@@ -109,6 +100,4 @@ NBDistrictCont::removeFromSinksAndSources(NBEdge* const e) {
 }
 
 
-
 /****************************************************************************/
-
