@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBEdgeCont.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBEdgeCont.h 20640 2016-05-04 10:17:39Z namdre $
 ///
 // Storage for edges, including some functionality operating on multiple edges
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBEdgeCont_h
-#define NBEdgeCont_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <map>
 #include <iostream>
@@ -41,19 +31,22 @@
 #include <utils/common/SUMOVehicleClass.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/geom/PositionVector.h>
+#include <utils/common/NamedRTree.h>
 
 
 // ===========================================================================
 // class declarations
 // ===========================================================================
+class OptionsCont;
+class OutputDevice;
 class NBNodeCont;
 class NBTypeCont;
 class NBEdge;
 class NBNode;
-class OptionsCont;
 class NBDistrictCont;
 class NBTrafficLightLogicCont;
-class OutputDevice;
+class NBPTStopCont;
+class NBPTLineCont;
 
 
 // ===========================================================================
@@ -160,7 +153,7 @@ public:
      * @return The searched edge
      * @todo Recheck usage
      */
-    NBEdge* retrievePossiblySplit(const std::string& id, SUMOReal pos) const;
+    NBEdge* retrievePossiblySplit(const std::string& id, double pos) const;
 
 
     /** @brief Removes the given edge from the container (deleting it)
@@ -205,9 +198,37 @@ public:
     /// @name explicit edge manipulation methods
     /// @{
 
+    /** @struct Split
+     * @brief A structure which describes changes of lane number or speed along the road
+     */
+    struct Split {
+        /// @brief The lanes after this change
+        std::vector<int> lanes;
+        /// @brief The position of this change
+        double pos = INVALID_DOUBLE;
+        /// @brief The speed after this change
+        double speed = INVALID_DOUBLE;
+        /// @brief The new node that is created for this split
+        NBNode* node = nullptr;
+        /// @brief The id for the edge before the split
+        std::string idBefore;
+        /// @brief The id for the edge after the split
+        std::string idAfter;
+        /// @brief the default node id
+        std::string nameID;
+        /// @brief lateral offset to edge geometry
+        double offset = 0.;
+        /// @brief direction in which to apply the offset (used by netgenerate for lefthand networks)
+        int offsetFactor = 1;
+    };
+
+    void processSplits(NBEdge* e, std::vector<Split> splits,
+                       NBNodeCont& nc, NBDistrictCont& dc, NBTrafficLightLogicCont& tlc);
+
+
     /** @brief Splits the edge at the position nearest to the given node
      *
-     * Uses "splitAt(NBDistrictCont &, NBEdge *, NBNode *, const std::string &, const std::string &, unsigned int , unsigned int)"
+     * Uses "splitAt(NBDistrictCont &, NBEdge *, NBNode *, const std::string &, const std::string &, int , int)"
      *  to perform the split; the edge names are built by appending "[0]" and "[1]",
      *  respectively. Both edges will have the same number of lanes.
      *
@@ -216,7 +237,7 @@ public:
      * @param[in] node The node to split the edge at
      * @return Whether the edge could be split
      * @exception ProcessError If connections between the edges can not be built
-     * @see NBEdge::splitAt(NBDistrictCont &, NBEdge *, NBNode *, const std::string &, const std::string &, unsigned int , unsigned int)
+     * @see NBEdge::splitAt(NBDistrictCont &, NBEdge *, NBNode *, const std::string &, const std::string &, int , int)
      */
     bool splitAt(NBDistrictCont& dc, NBEdge* edge, NBNode* node);
 
@@ -227,7 +248,7 @@ public:
      *  edge to the given node. If this position is too near to the edges begin/end,
      *  false is returned.
      *
-     * Otherwise, "splitAt(NBDistrictCont &, NBEdge *, SUMOReal, NBNode *, const std::string &, const std::string &, unsigned int , unsigned int)"
+     * Otherwise, "splitAt(NBDistrictCont &, NBEdge *, double, NBNode *, const std::string &, const std::string &, int , int)"
      *  is used to perform the split.
      *
      * @param[in] dc The district container, in order to remove/add the edge from/to sources/sinks
@@ -242,12 +263,12 @@ public:
      *            (By default all added/removed lanes are assumed to be on the right when computing connections)
      * @return Whether the edge could be split
      * @exception ProcessError If connections between the edges can not be built
-     * @see NBEdge::splitAt(NBDistrictCont &, NBEdge *, SUMOReal, NBNode *, const std::string &, const std::string &, unsigned int , unsigned int)
+     * @see NBEdge::splitAt(NBDistrictCont &, NBEdge *, double, NBNode *, const std::string &, const std::string &, int , int)
      */
     bool splitAt(NBDistrictCont& dc, NBEdge* edge, NBNode* node,
                  const std::string& firstEdgeName, const std::string& secondEdgeName,
-                 unsigned int noLanesFirstEdge, unsigned int noLanesSecondEdge,
-                 const SUMOReal speed = -1., const int changedLeft = 0);
+                 int noLanesFirstEdge, int noLanesSecondEdge,
+                 const double speed = -1., const int changedLeft = 0);
 
 
     /** @brief Splits the edge at the position nearest to the given node using the given modifications
@@ -265,10 +286,10 @@ public:
      * @return Whether the edge could be split
      * @exception ProcessError If connections between the edges can not be built
      */
-    bool splitAt(NBDistrictCont& dc, NBEdge* edge, SUMOReal edgepos, NBNode* node,
+    bool splitAt(NBDistrictCont& dc, NBEdge* edge, double edgepos, NBNode* node,
                  const std::string& firstEdgeName, const std::string& secondEdgeName,
-                 unsigned int noLanesFirstEdge, unsigned int noLanesSecondEdge,
-                 const SUMOReal speed = -1., const int changedLeft = 0);
+                 int noLanesFirstEdge, int noLanesSecondEdge,
+                 const double speed = -1., const int changedLeft = 0);
     /// @}
 
 
@@ -279,8 +300,8 @@ public:
     /** @brief Returns the number of edges
      * @return The number of edges stored in this container
      */
-    unsigned int size() const {
-        return (unsigned int) myEdges.size();
+    int size() const {
+        return (int) myEdges.size();
     }
 
 
@@ -294,7 +315,7 @@ public:
     /** @brief Returns the number of edge splits
      * @return How often an edge was split
      */
-    unsigned int getNoEdgeSplits() const {
+    int getNoEdgeSplits() const {
         return myEdgesSplit;
     }
     /// @}
@@ -319,14 +340,14 @@ public:
      * @param[in] nc The node container needed to build (geometry) nodes
      * @see NBEdge::splitGeometry
      */
-    void splitGeometry(NBNodeCont& nc);
+    void splitGeometry(NBDistrictCont& dc, NBNodeCont& nc);
 
 
     /** @brief
      * @param[in] nc The node container needed to build (geometry) nodes
      * @see NBEdge::reduceGeometry
      */
-    void reduceGeometries(const SUMOReal minDist);
+    void reduceGeometries(const double minDist);
 
 
     /** @brief
@@ -335,7 +356,7 @@ public:
      * @param[in] fix Whether to prune geometry points to avoid sharp turns at start and end
      * @see NBEdge::checkGeometry
      */
-    void checkGeometries(const SUMOReal maxAngle, const SUMOReal minRadius, bool fix);
+    void checkGeometries(const double maxAngle, const double minRadius, bool fix, bool fixRailways, bool silent = false);
     /// @}
 
 
@@ -389,10 +410,13 @@ public:
      * Calls "NBEdge::appendTurnaround" for all edges within the container.
      *
      * @param[in] noTLSControlled Whether the turnaround shall not be connected if the edge is controlled by a tls
-     * @todo Recheck whether a visitor-pattern should be used herefor
+     * @param[in] noFringe Whether the turnaround shall not be connected if the junction is at the (outer) fringe
+     * @param[in] onlyDeadends Whether the turnaround shall only be built at deadends
+     * @param[in] onlyTurnlane Whether the turnaround shall only be built when there is an exclusive (left) turn lane
+     * @param[in] noGeometryLike Whether the turnaround shall be built at geometry-like nodes
      * @see NBEdge::appendTurnaround
      */
-    void appendTurnarounds(bool noTLSControlled);
+    void appendTurnarounds(bool noTLSControlled, bool noFringe, bool onlyDeadends, bool onlyTurnlane, bool noGeometryLike);
 
 
     /** @brief Appends turnarounds to all edges stored in the container
@@ -403,6 +427,10 @@ public:
      */
     void appendTurnarounds(const std::set<std::string>& ids, bool noTLSControlled);
 
+    /** @brief Appends turnarounds to all bidiRail edges with stops
+     */
+    void appendRailwayTurnarounds(const NBPTStopCont& sc);
+
 
     /** @brief Computes the shapes of all edges stored in the container
      *
@@ -412,7 +440,7 @@ public:
      * @todo Recheck usage
      * @see NBEdge::computeEdgeShape
      */
-    void computeEdgeShapes();
+    void computeEdgeShapes(double smoothElevationThreshold = -1);
 
 
     /** @brief Computes the shapes of all lanes of all edges stored in the container
@@ -452,9 +480,14 @@ public:
      * @todo Recheck usage; check whether this is really needed and whether it works at all
      */
     void recheckLaneSpread();
+
     /// @}
 
+    /// @brief Returns the edge with negated id if it exists
+    NBEdge* getOppositeByID(const std::string& edgeID) const;
 
+    /// @brief Returns the edge with id if it exists
+    NBEdge* getByID(const std::string& edgeID) const;
 
     /** @brief Determines which edges belong to roundabouts and increases their priority
      * @param[out] marked Edges which belong to a roundabout are stored here
@@ -498,8 +531,22 @@ public:
      * @param[in] mayDefinitelyPass Whether the connection may be passed without braking
      * @param[in] keepClear Whether the connection must check to keep the junction clear
      * @param[in] contPos Custom position for internal junction
+     * @param[in] visibility Custom foe visiblity connection
+     * @param[in] speed Custom speed
+     * @param[in] customShape Custom shape
+     * @param[in] warnOnly Whether a failure to set this connection should only result in a warning
      */
-    void addPostProcessConnection(const std::string& from, int fromLane, const std::string& to, int toLane, bool mayDefinitelyPass, bool keepClear, SUMOReal contPos);
+    void addPostProcessConnection(const std::string& from, int fromLane, const std::string& to, int toLane, bool mayDefinitelyPass,
+                                  KeepClear keepClear, double contPos, double visibility,
+                                  double speed, double length,
+                                  const PositionVector& customShape,
+                                  bool uncontrolled,
+                                  bool warnOnly,
+                                  SVCPermissions permissions = SVC_UNSPECIFIED,
+                                  SVCPermissions changeLeft = SVC_UNSPECIFIED,
+                                  SVCPermissions changeRight = SVC_UNSPECIFIED);
+
+    bool hasPostProcessConnection(const std::string& from, const std::string& to = "");
 
 
     /** @brief Try to set any stored connections
@@ -511,7 +558,7 @@ public:
     void generateStreetSigns();
 
     /// @brief add sidwalks to edges within the given limits or permissions and return the number of edges affected
-    int guessSidewalks(SUMOReal width, SUMOReal minSpeed, SUMOReal maxSpeed, bool fromPermissions);
+    int guessSpecialLanes(SUMOVehicleClass svc, double width, double minSpeed, double maxSpeed, bool fromPermissions, const std::string& excludeOpt);
 
 
     /** @brief Returns the determined roundabouts
@@ -522,16 +569,30 @@ public:
     /// @brief add user specified roundabout
     void addRoundabout(const EdgeSet& roundabout);
 
+    /// @brief remove roundabout that contains the given node
+    void removeRoundabout(const NBNode* node);
+    /// @brief remove edges from all stored roundabouts
+    void removeRoundaboutEdges(const EdgeSet& toRemove);
+
     /// @brief mark edge priorities and prohibit turn-arounds for all roundabout edges
     void markRoundabouts();
 
+    /// @brief fix roundabout information after splitting an edge
+    void patchRoundabouts(NBEdge* orig, NBEdge* part1, NBEdge* part2, std::set<EdgeSet>& roundabouts);
+
     /// @brief Returns true if this edge matches one of the removal criteria
     bool ignoreFilterMatch(NBEdge* edge);
 
-    /// @brief ensure that all edge ids are integers
-    int mapToNumericalIDs();
+    /// @brief remap node IDs accoring to options --numerical-ids and --reserved-ids
+    int remapIDs(bool numericaIDs, bool reservedIDs, const std::string& prefix, NBPTStopCont& sc);
+
+    /// @brief check whether edges overlap
+    void checkOverlap(double threshold, double zThreshold) const;
+
+    /// @brief check whether edges are to steep
+    void checkGrade(double threshold) const;
+
 
-private:
     /** @brief Returns the edges which have been built by splitting the edge of the given id
      *
      * @param[in] id The id of the original edge
@@ -540,9 +601,27 @@ private:
      */
     EdgeVector getGeneratedFrom(const std::string& id) const;
 
+    /// @brief join adjacent lanes with the given permissions
+    int joinLanes(SVCPermissions perms);
 
+    /// @brief join tram edges into adjacent lanes
+    int joinTramEdges(NBDistrictCont& dc, NBPTStopCont& sc, NBPTLineCont& lc, double maxDist);
+
+    /// @brief return all edges
+    EdgeVector getAllEdges() const;
+    RouterEdgeVector getAllRouterEdges() const;
+
+    /// @brief ensure that all edges have valid nodes
+    bool checkConsistency(const NBNodeCont& nc);
+
+    /// @brief modify all restrictions on lane changing for edges and connections
+    void updateAllChangeRestrictions(SVCPermissions ignoring);
+
+private:
     /// @brief compute the form factor for a loop of edges
-    static SUMOReal formFactor(const EdgeVector& loopEdges);
+    static double formFactor(const EdgeVector& loopEdges);
+
+    void removeRoundaboutEdges(const EdgeSet& toRemove, std::set<EdgeSet>& roundabouts);
 
 private:
     /// @brief The network builder; used to obtain type information
@@ -560,9 +639,26 @@ private:
          * @param[in] toLane The number of the lane the connection ends at
          * @param[in] mayDefinitelyPass Whether the connection may be passed without braking
          */
-        PostProcessConnection(const std::string& from_, int fromLane_, const std::string& to_, int toLane_, bool mayDefinitelyPass_, bool keepClear_, SUMOReal contPos_) :
-            from(from_), fromLane(fromLane_), to(to_), toLane(toLane_), mayDefinitelyPass(mayDefinitelyPass_), keepClear(keepClear_), contPos(contPos_) {
-        }
+        PostProcessConnection(const std::string& from_, int fromLane_, const std::string& to_, int toLane_,
+                              bool mayDefinitelyPass_, KeepClear keepClear_, double contPos_, double visibility_, double speed_,
+                              double length_,
+                              const PositionVector& customShape_,
+                              bool uncontrolled_,
+                              bool warnOnly_,
+                              SVCPermissions permissions_,
+                              SVCPermissions changeLeft_,
+                              SVCPermissions changeRight_) :
+            from(from_), fromLane(fromLane_), to(to_), toLane(toLane_), mayDefinitelyPass(mayDefinitelyPass_), keepClear(keepClear_), contPos(contPos_),
+            visibility(visibility_),
+            speed(speed_),
+            customLength(length_),
+            customShape(customShape_),
+            uncontrolled(uncontrolled_),
+            permissions(permissions_),
+            changeLeft(changeLeft_),
+            changeRight(changeRight_),
+            warnOnly(warnOnly_)
+        {}
         /// @brief The id of the edge the connection starts at
         std::string from;
         /// @brief The number of the lane the connection starts at
@@ -574,13 +670,31 @@ private:
         /// @brief Whether the connection may be passed without braking
         bool mayDefinitelyPass;
         /// @brief Whether the connection may be passed without braking
-        bool keepClear;
+        KeepClear keepClear;
         /// @brief custom position for internal junction on this connection
-        SUMOReal contPos;
+        double contPos;
+        /// @brief custom foe visiblity for connection
+        double visibility;
+        /// @brief custom speed for connection
+        double speed;
+        /// @brief custom length for connection
+        double customLength;
+        /// @brief custom shape for connection
+        PositionVector customShape;
+        /// @brief whether this connection shall not be controlled by a traffic light
+        bool uncontrolled;
+        /// @brief custom permissions for connection
+        SVCPermissions permissions;
+        /// @brief custom lane changing permissions for connection
+        SVCPermissions changeLeft;
+        /// @brief custom lane changing permissions for connection
+        SVCPermissions changeRight;
+        /// @brief whether a failure to set this connection is a warning or an error
+        bool warnOnly;
     };
 
     /// @brief The list of connections to recheck
-    std::vector<PostProcessConnection> myConnections;
+    std::map<std::string, std::vector<PostProcessConnection> > myConnections;
 
 
     /// @brief The type of the dictionary where an edge may be found by its id
@@ -596,13 +710,13 @@ private:
     std::set<std::string> myIgnoredEdges;
 
     /// @brief the number of splits of edges during the building
-    unsigned int myEdgesSplit;
+    int myEdgesSplit;
 
     /// @name Settings for accepting/dismissing edges
     /// @{
 
     /// @brief The minimum speed an edge may have in order to be kept (default: -1)
-    SUMOReal myEdgesMinSpeed;
+    double myEdgesMinSpeed;
 
     /// @brief Whether edges shall be joined first, then removed
     bool myRemoveEdgesAfterJoining;
@@ -626,10 +740,10 @@ private:
     std::set<std::string> myTypes2Remove;
 
     /// @brief Boundary within which an edge must be located in order to be kept
-    PositionVector myPrunningBoundary;
+    PositionVector myPruningBoundary;
 
     /// @brief whether a geo transform has been applied to the pruning boundary
-    bool myNeedGeoTransformedPrunningBoundary;
+    bool myNeedGeoTransformedPruningBoundary;
     /// @}
 
     /// @brief Edges marked as belonging to a roundabout by the user (each EdgeVector is a roundabout)
@@ -637,6 +751,20 @@ private:
     /// @brief Edges marked as belonging to a roundabout after guessing
     std::set<EdgeSet> myGuessedRoundabouts;
 
+    /** @class split_sorter
+     * @brief Sorts splits by their position (increasing)
+     */
+    class split_sorter {
+    public:
+        /// @brief Constructor
+        explicit split_sorter() { }
+
+        /// @brief Comparing operator
+        int operator()(const Split& e1, const Split& e2) const {
+            return e1.pos < e2.pos;
+        }
+    };
+
 
 private:
     /// @brief invalidated copy constructor
@@ -645,11 +773,4 @@ private:
     /// @brief invalidated assignment operator
     NBEdgeCont& operator=(const NBEdgeCont& s);
 
-
 };
-
-
-#endif
-
-/****************************************************************************/
-
