@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2011-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBHeightMapper.cpp
 /// @author  Jakob Erdmann
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    Sept 2011
-/// @version $Id: NBHeightMapper.cpp 20801 2016-05-28 05:31:30Z behrisch $
 ///
 // Set z-values for all network positions based on data from a height map
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/common/MsgHandler.h>
@@ -41,27 +32,29 @@
 #include <utils/common/RGBColor.h>
 
 #ifdef HAVE_GDAL
+#if __GNUC__ > 3
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wpedantic"
+#endif
 #include <ogrsf_frmts.h>
 #include <ogr_api.h>
 #include <gdal_priv.h>
+#if __GNUC__ > 3
+#pragma GCC diagnostic pop
+#endif
 #endif
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
 
 // ===========================================================================
 // static members
 // ===========================================================================
-NBHeightMapper NBHeightMapper::Singleton;
+NBHeightMapper NBHeightMapper::myInstance;
+
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
-
-
 NBHeightMapper::NBHeightMapper():
-    myRTree(&Triangle::addSelf), myRaster(0) {
+    myRTree(&Triangle::addSelf) {
 }
 
 
@@ -72,29 +65,43 @@ NBHeightMapper::~NBHeightMapper() {
 
 const NBHeightMapper&
 NBHeightMapper::get() {
-    return Singleton;
+    return myInstance;
 }
 
 
 bool
 NBHeightMapper::ready() const {
-    return myRaster != 0 || myTriangles.size() > 0;
+    return myRasters.size() > 0 || myTriangles.size() > 0;
 }
 
 
-SUMOReal
+double
 NBHeightMapper::getZ(const Position& geo) const {
     if (!ready()) {
         WRITE_WARNING("Cannot supply height since no height data was loaded");
         return 0;
     }
-    if (myRaster != 0) {
-        SUMOReal result = -1e6;
-        if (myBoundary.around(geo)) {
-            const int xSize = int((myBoundary.xmax() - myBoundary.xmin()) / mySizeOfPixel.x() + .5);
-            const int iX = int((geo.x() - myBoundary.xmin()) / mySizeOfPixel.x() + .5);
-            const int iY = int((geo.y() - myBoundary.ymax()) / mySizeOfPixel.y() + .5);
-            result = myRaster[iY * xSize + iX];
+    for (auto& item : myRasters) {
+        const Boundary& boundary = item.boundary;
+        int16_t* raster = item.raster;
+        double result = -1e6;
+        if (boundary.around(geo)) {
+            const int xSize = item.xSize;
+            const double normX = (geo.x() - boundary.xmin()) / mySizeOfPixel.x();
+            const double normY = (geo.y() - boundary.ymax()) / mySizeOfPixel.y();
+            PositionVector corners;
+            corners.push_back(Position(floor(normX) + 0.5, floor(normY) + 0.5, raster[(int)normY * xSize + (int)normX]));
+            if (normX - floor(normX) > 0.5) {
+                corners.push_back(Position(floor(normX) + 1.5, floor(normY) + 0.5, raster[(int)normY * xSize + (int)normX + 1]));
+            } else {
+                corners.push_back(Position(floor(normX) - 0.5, floor(normY) + 0.5, raster[(int)normY * xSize + (int)normX - 1]));
+            }
+            if (normY - floor(normY) > 0.5 && ((int)normY + 1) < item.ySize) {
+                corners.push_back(Position(floor(normX) + 0.5, floor(normY) + 1.5, raster[((int)normY + 1) * xSize + (int)normX]));
+            } else {
+                corners.push_back(Position(floor(normX) + 0.5, floor(normY) - 0.5, raster[((int)normY - 1) * xSize + (int)normX]));
+            }
+            result = Triangle(corners).getZ(Position(normX, normY));
         }
         if (result > -1e5 && result < 1e5) {
             return result;
@@ -119,7 +126,7 @@ NBHeightMapper::getZ(const Position& geo) const {
             return triangle->getZ(geo);
         }
     }
-    WRITE_WARNING("Could not get height data for coordinate " + toString(geo));
+    WRITE_WARNINGF("Could not get height data for coordinate %", toString(geo));
     return 0;
 }
 
@@ -142,10 +149,10 @@ NBHeightMapper::loadIfSet(OptionsCont& oc) {
         std::vector<std::string> files = oc.getStringVector("heightmap.geotiff");
         for (std::vector<std::string>::const_iterator file = files.begin(); file != files.end(); ++file) {
             PROGRESS_BEGIN_MESSAGE("Parsing from GeoTIFF '" + *file + "'");
-            int numFeatures = Singleton.loadTiff(*file);
+            int numFeatures = myInstance.loadTiff(*file);
             MsgHandler::getMessageInstance()->endProcessMsg(
                 " done (parsed " + toString(numFeatures) +
-                " features, Boundary: " + toString(Singleton.getBoundary()) + ").");
+                " features, Boundary: " + toString(myInstance.getBoundary()) + ").");
         }
     }
     if (oc.isSet("heightmap.shapefiles")) {
@@ -153,10 +160,10 @@ NBHeightMapper::loadIfSet(OptionsCont& oc) {
         std::vector<std::string> files = oc.getStringVector("heightmap.shapefiles");
         for (std::vector<std::string>::const_iterator file = files.begin(); file != files.end(); ++file) {
             PROGRESS_BEGIN_MESSAGE("Parsing from shape-file '" + *file + "'");
-            int numFeatures = Singleton.loadShapeFile(*file);
+            int numFeatures = myInstance.loadShapeFile(*file);
             MsgHandler::getMessageInstance()->endProcessMsg(
                 " done (parsed " + toString(numFeatures) +
-                " features, Boundary: " + toString(Singleton.getBoundary()) + ").");
+                " features, Boundary: " + toString(myInstance.getBoundary()) + ").");
         }
     }
 }
@@ -170,9 +177,9 @@ NBHeightMapper::loadShapeFile(const std::string& file) {
     OGRDataSource* ds = OGRSFDriverRegistrar::Open(file.c_str(), FALSE);
 #else
     GDALAllRegister();
-    GDALDataset* ds = (GDALDataset*) GDALOpen(file.c_str(), GA_ReadOnly);
+    GDALDataset* ds = (GDALDataset*)GDALOpenEx(file.c_str(), GDAL_OF_VECTOR | GA_ReadOnly, nullptr, nullptr, nullptr);
 #endif
-    if (ds == NULL) {
+    if (ds == nullptr) {
         throw ProcessError("Could not open shape file '" + file + "'.");
     }
 
@@ -186,35 +193,38 @@ NBHeightMapper::loadShapeFile(const std::string& file) {
     OGRSpatialReference sr_dest;
     sr_dest.SetWellKnownGeogCS("WGS84");
     OGRCoordinateTransformation* toWGS84 = OGRCreateCoordinateTransformation(sr_src, &sr_dest);
-    if (toWGS84 == 0) {
+    if (toWGS84 == nullptr) {
         WRITE_WARNING("Could not create geocoordinates converter; check whether proj.4 is installed.");
     }
 
     int numFeatures = 0;
     OGRFeature* feature;
     layer->ResetReading();
-    while ((feature = layer->GetNextFeature()) != NULL) {
+    while ((feature = layer->GetNextFeature()) != nullptr) {
         OGRGeometry* geom = feature->GetGeometryRef();
         assert(geom != 0);
 
-        // @todo gracefull handling of shapefiles with unexpected contents or any error handling for that matter
-        assert(std::string(geom->getGeometryName()) == std::string("POLYGON"));
-        // try transform to wgs84
-        geom->transform(toWGS84);
-        OGRLinearRing* cgeom = ((OGRPolygon*) geom)->getExteriorRing();
-        // assume TIN with with 4 points and point0 == point3
-        assert(cgeom->getNumPoints() == 4);
-        PositionVector corners;
-        for (int j = 0; j < 3; j++) {
-            Position pos((SUMOReal) cgeom->getX(j), (SUMOReal) cgeom->getY(j), (SUMOReal) cgeom->getZ(j));
-            corners.push_back(pos);
-            myBoundary.add(pos);
+        OGRwkbGeometryType gtype = geom->getGeometryType();
+        if (gtype == wkbPolygon) {
+            assert(std::string(geom->getGeometryName()) == std::string("POLYGON"));
+            // try transform to wgs84
+            geom->transform(toWGS84);
+            OGRLinearRing* cgeom = ((OGRPolygon*) geom)->getExteriorRing();
+            // assume TIN with with 4 points and point0 == point3
+            assert(cgeom->getNumPoints() == 4);
+            PositionVector corners;
+            for (int j = 0; j < 3; j++) {
+                Position pos((double) cgeom->getX(j), (double) cgeom->getY(j), (double) cgeom->getZ(j));
+                corners.push_back(pos);
+                myBoundary.add(pos);
+            }
+            addTriangle(corners);
+            numFeatures++;
+        } else {
+            WRITE_WARNINGF("Ignored heightmap feature type %", geom->getGeometryName());
         }
-        addTriangle(corners);
-        numFeatures++;
 
         /*
-        OGRwkbGeometryType gtype = geom->getGeometryType();
         switch (gtype) {
             case wkbPolygon: {
                 break;
@@ -240,7 +250,7 @@ NBHeightMapper::loadShapeFile(const std::string& file) {
                 break;
             }
             default:
-                WRITE_WARNING("Unsupported shape type occured");
+                WRITE_WARNING("Unsupported shape type occurred");
             break;
         }
         */
@@ -251,10 +261,11 @@ NBHeightMapper::loadShapeFile(const std::string& file) {
 #else
     GDALClose(ds);
 #endif
-    OCTDestroyCoordinateTransformation(toWGS84);
+    OCTDestroyCoordinateTransformation(reinterpret_cast<OGRCoordinateTransformationH>(toWGS84));
     OGRCleanupAll();
     return numFeatures;
 #else
+    UNUSED_PARAMETER(file);
     WRITE_ERROR("Cannot load shape file since SUMO was compiled without GDAL support.");
     return 0;
 #endif
@@ -270,6 +281,7 @@ NBHeightMapper::loadTiff(const std::string& file) {
         WRITE_ERROR("Cannot load GeoTIFF file.");
         return 0;
     }
+    Boundary boundary;
     const int xSize = poDataset->GetRasterXSize();
     const int ySize = poDataset->GetRasterYSize();
     double adfGeoTransform[6];
@@ -278,36 +290,54 @@ NBHeightMapper::loadTiff(const std::string& file) {
         mySizeOfPixel.set(adfGeoTransform[1], adfGeoTransform[5]);
         const double horizontalSize = xSize * mySizeOfPixel.x();
         const double verticalSize = ySize * mySizeOfPixel.y();
-        myBoundary.add(topLeft);
-        myBoundary.add(topLeft.x() + horizontalSize, topLeft.y() + verticalSize);
+        boundary.add(topLeft);
+        boundary.add(topLeft.x() + horizontalSize, topLeft.y() + verticalSize);
     } else {
         WRITE_ERROR("Could not parse geo information from " + file + ".");
         return 0;
     }
     const int picSize = xSize * ySize;
-    myRaster = (int16_t*)CPLMalloc(sizeof(int16_t) * picSize);
+    int16_t* raster = (int16_t*)CPLMalloc(sizeof(int16_t) * picSize);
+    bool ok = true;
     for (int i = 1; i <= poDataset->GetRasterCount(); i++) {
         GDALRasterBand* poBand = poDataset->GetRasterBand(i);
         if (poBand->GetColorInterpretation() != GCI_GrayIndex) {
             WRITE_ERROR("Unknown color band in " + file + ".");
             clearData();
-            break;
-        }
-        if (poBand->GetRasterDataType() != GDT_Int16) {
-            WRITE_ERROR("Unknown data type in " + file + ".");
-            clearData();
+            ok = false;
             break;
         }
         assert(xSize == poBand->GetXSize() && ySize == poBand->GetYSize());
-        if (poBand->RasterIO(GF_Read, 0, 0, xSize, ySize, myRaster, xSize, ySize, GDT_Int16, 0, 0) == CE_Failure) {
+        if (poBand->RasterIO(GF_Read, 0, 0, xSize, ySize, raster, xSize, ySize, GDT_Int16, 0, 0) == CE_Failure) {
             WRITE_ERROR("Failure in reading " + file + ".");
             clearData();
+            ok = false;
             break;
         }
     }
+    double min = std::numeric_limits<double>::max();
+    double max = -std::numeric_limits<double>::max();
+    for (int i = 0; i < picSize; i++) {
+        min = MIN2(min, (double)raster[i]);
+        max = MAX2(max, (double)raster[i]);
+    }
     GDALClose(poDataset);
-    return picSize;
+    if (ok) {
+        WRITE_MESSAGE("Read geotiff heightmap with size " + toString(xSize) + "," + toString(ySize)
+                      + " for geo boundary [" + toString(boundary)
+                      + "] with elevation range [" + toString(min) + "," + toString(max) + "].");
+        RasterData rasterData;
+        rasterData.raster = raster;
+        rasterData.boundary = boundary;
+        rasterData.xSize = xSize;
+        rasterData.ySize = ySize;
+        myRasters.push_back(rasterData);
+        return picSize;
+    } else {
+        return 0;
+    }
 #else
+    UNUSED_PARAMETER(file);
     WRITE_ERROR("Cannot load GeoTIFF file since SUMO was compiled without GDAL support.");
     return 0;
 #endif
@@ -321,10 +351,10 @@ NBHeightMapper::clearData() {
     }
     myTriangles.clear();
 #ifdef HAVE_GDAL
-    if (myRaster != 0) {
-        CPLFree(myRaster);
-        myRaster = 0;
+    for (auto& item : myRasters) {
+        CPLFree(item.raster);
     }
+    myRasters.clear();
 #endif
     myBoundary.reset();
 }
@@ -352,7 +382,7 @@ NBHeightMapper::Triangle::contains(const Position& pos) const {
 }
 
 
-SUMOReal
+double
 NBHeightMapper::Triangle::getZ(const Position& geo) const {
     // en.wikipedia.org/wiki/Line-plane_intersection
     Position p0 = myCorners.front();
@@ -373,4 +403,3 @@ NBHeightMapper::Triangle::normalVector() const {
 
 
 /****************************************************************************/
-
