@@ -1,38 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2011-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBHeightMapper.h
 /// @author  Jakob Erdmann
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    Sept 2011
-/// @version $Id: NBHeightMapper.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Set z-values for all network positions based on data from a height map
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBHeightMapper_h
-#define NBHeightMapper_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#ifdef _MSC_VER
+#ifdef WIN32
 typedef __int16 int16_t;
 #else
 #include <stdint.h>
@@ -67,7 +57,7 @@ class NBHeightMapper {
     friend class NBHeightMapperTest;
 
 public:
-    /** @brief loads heigh map data if any loading options are set
+    /** @brief loads height map data if any loading options are set
      *
      * @param[in] oc The options container to get further options from
      * @exception ProcessError if something fails
@@ -82,11 +72,11 @@ public:
 
     /// @brief returns the convex boundary of all known triangles
     const Boundary& getBoundary() {
-        return Singleton.myBoundary;
+        return myInstance.myBoundary;
     }
 
     /// @brief returns height for the given geo coordinate (WGS84)
-    SUMOReal getZ(const Position& geo) const;
+    double getZ(const Position& geo) const;
 
     class QueryResult;
     /* @brief content class for the rtree. Since we wish to be able to use the
@@ -107,7 +97,7 @@ public:
         bool contains(const Position& pos) const;
 
         /// @brief returns the projection of the give geoCoordinate (WGS84) onto triangle plane
-        SUMOReal getZ(const Position& geo) const;
+        double getZ(const Position& geo) const;
 
         /// @brief returns the normal vector for this triangles plane
         Position normalVector() const;
@@ -132,16 +122,24 @@ public:
     };
 
 private:
+
+    struct RasterData {
+        int16_t* raster;
+        Boundary boundary;
+        int xSize;
+        int ySize;
+    };
+
     /// @brief the singleton instance
-    static NBHeightMapper Singleton;
+    static NBHeightMapper myInstance;
 
     Triangles myTriangles;
 
     /// @brief The RTree for spatial queries
     TRIANGLE_RTREE_QUAL myRTree;
 
-    /// @brief raster height information in m
-    int16_t* myRaster;
+    /// @brief raster height information in m for all loaded files
+    std::vector<RasterData> myRasters;
 
     /// @brief dimensions of one pixel in raster data
     Position mySizeOfPixel;
@@ -202,9 +200,3 @@ inline TRIANGLE_RTREE_QUAL::Rect TRIANGLE_RTREE_QUAL::CombineRect(Rect* a_rectA,
     newRect.m_max[1] = rtree_max(a_rectA->m_max[1], a_rectB->m_max[1]);
     return newRect;
 }
-
-
-#endif
-
-/****************************************************************************/
-
