@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2011-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBLoadedSUMOTLDef.h
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mar 2011
-/// @version $Id: NBLoadedSUMOTLDef.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A complete traffic light logic loaded from a sumo-net. (opted to reimplement
 // since NBLoadedTLDef is quite vissim specific)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBLoadedSUMOTLDef_h
-#define NBLoadedSUMOTLDef_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
@@ -64,12 +54,17 @@ public:
      * @param[in] def The definition to copy
      * @param[in] logic The computed logic of the given def
      */
-    NBLoadedSUMOTLDef(NBTrafficLightDefinition* def, NBTrafficLightLogic* logic);
+    NBLoadedSUMOTLDef(const NBTrafficLightDefinition& def, const NBTrafficLightLogic& logic);
 
 
     /// @brief Destructor
     ~NBLoadedSUMOTLDef();
 
+    /** @brief Sets the programID
+     * @param[in] programID The new ID of the program (subID)
+     */
+    void setProgramID(const std::string& programID);
+
     /** @brief Informs edges about being controlled by a tls
      */
     void setTLControllingInformation() const;
@@ -90,22 +85,30 @@ public:
      * @param[in] byLane This edge's lane to insert instead
      */
     void replaceRemoved(NBEdge* removed, int removedLane,
-                        NBEdge* by, int byLane);
+                        NBEdge* by, int byLane, bool incoming);
 
-    /// @brief patches signal plans by modifying lane indices
-    void shiftTLConnectionLaneIndex(NBEdge* edge, int offset);
+    /** @brief patches signal plans by modifying lane indices
+     * with the given offset, only indices with a value above threshold are modified
+     */
+    void shiftTLConnectionLaneIndex(NBEdge* edge, int offset, int threshold = -1);
 
     /** @brief Adds a phase to the logic
      * the new phase is inserted at the end of the list of already added phases
-     * @param[in] duration The duration of the phase to add in SECONDS!
+     * @param[in] duration The duration of the phase to add
      * @param[in] state The state definition of a tls phase
+     * @param[in] minDur The minimum duration of the phase to add
+     * @param[in] maxDur The maximum duration of the phase to add
      */
-    void addPhase(SUMOTime duration, const std::string& state);
+    void addPhase(SUMOTime duration, const std::string& state, SUMOTime minDur, SUMOTime maxDur, const std::vector<int>& next, const std::string& name);
 
+    /// @brief mark phases as load
+    void phasesLoaded() {
+        myPhasesLoaded = true;
+    }
 
     /** @brief Adds a connection and immediately informs the edges
      */
-    void addConnection(NBEdge* from, NBEdge* to, int fromLane, int toLane, int linkIndex);
+    void addConnection(NBEdge* from, NBEdge* to, int fromLane, int toLane, int linkIndex, int linkIndex2, bool reconstruct = true);
 
 
     /** @brief removes the given connection from the traffic light
@@ -114,6 +117,9 @@ public:
      */
     void removeConnection(const NBConnection& conn, bool reconstruct = true);
 
+    /// @brief register changes that necessitate recomputation
+    void registerModifications(bool addedConnections, bool removedConnections);
+
     /** @brief Returns the internal logic
      */
     NBTrafficLightLogic* getLogic() {
@@ -133,6 +139,30 @@ public:
     /// @brief whether the given index must yield to the foeIndex while turing right on a red light
     bool rightOnRedConflict(int index, int foeIndex) const;
 
+    /* @brief shortens phase states to remove states that are not referenced by
+     * any controlled link and returns whether states were shortened
+    */
+    bool cleanupStates();
+
+    /// @brief whether this definition uses signal group (multiple connections with the same link index)
+    bool usingSignalGroups() const;
+
+    /// @brief join nodes and states from the given logic (append red state)
+    void joinLogic(NBTrafficLightDefinition* def);
+
+    /// @brief heuristically add minDur and maxDur when switching from tlType fixed to actuated
+    void guessMinMaxDuration();
+
+    /// @brief let connections with the same state use the same link index
+    void groupSignals();
+
+    /// @brief let all connections use a distinct link index
+    void ungroupSignals();
+
+    /// @brief copy the assignment of link indices to connections from the given definition and rebuilt the states to match
+    // Note: Issues a warning when the grouping of def is incompatible with the current states
+    void copyIndices(NBTrafficLightDefinition* def);
+
 protected:
     /** @brief Collects the links participating in this traffic light
      *    (only if not previously loaded)
@@ -147,27 +177,47 @@ protected:
      * @param[in] brakingTime Duration a vehicle needs for braking in front of the tls in seconds
      * @return The computed logic
      */
-    NBTrafficLightLogic* myCompute(unsigned int brakingTimeSeconds);
+    NBTrafficLightLogic* myCompute(int brakingTimeSeconds);
 
     bool amInvalid() const;
 
     /* initialize myNeedsContRelation and set myNeedsContRelationReady to true */
     void initNeedsContRelation() const;
 
+    /// @brief return the highest known tls link index used by any controlled connection or crossing
+    int getMaxIndex();
+
+    ///@brief Returns the maximum index controlled by this traffic light
+    int getMaxValidIndex();
+
+    /// @brief get all states for the given link index
+    std::string getStates(int index);
+
+    /// @brief return whether the given link index is used by any connectons
+    bool isUsed(int index);
+
+    /// @brief replace the given link index in all connections
+    void replaceIndex(int oldIndex, int newIndex);
+
+    /// brief retrieve all edges with connections that use the given traffic light index
+    std::set<const NBEdge*> getEdgesUsingIndex(int index) const;
+
 private:
 
     /** @brief phases are added directly to myTLLogic which is then returned in myCompute() */
     NBTrafficLightLogic* myTLLogic;
 
-    /// @brief The original nodes for which the loaded logic is valid
-    std::set<NBNode*> myOriginalNodes;
-
     /// @brief repair the plan if controlled nodes received pedestrian crossings
     void patchIfCrossingsAdded();
 
     /// @brief set of edges with shifted lane indices (to avoid shifting twice)
     std::set<NBEdge*> myShifted;
 
+    /// @brief whether the logic must be reconstructed
+    bool myReconstructAddedConnections;
+    bool myReconstructRemovedConnections;
+    bool myPhasesLoaded;
+
     /** @brief Collects the edges for each tlIndex
      * @param[out] fromEdges The from-edge for each controlled tlIndex
      * @param[out] toEdges The to-edge for each controlled tlIndex
@@ -175,6 +225,12 @@ private:
      */
     void collectEdgeVectors(EdgeVector& fromEdges, EdgeVector& toEdges, std::vector<int>& fromLanes) const;
 
+    /// @brief adapt to removal or addition of connections
+    void reconstructLogic();
+
+    /// @brief return whether all tls link indices are valid
+    bool hasValidIndices() const;
+
 private:
     /// @brief class for identifying connections
     class connection_equal {
@@ -195,9 +251,3 @@ private:
     };
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
