@@ -1,34 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBLoadedTLDef.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Tue, 29.05.2005
-/// @version $Id: NBLoadedTLDef.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A loaded (complete) traffic light logic
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <vector>
 #include <set>
@@ -76,38 +68,36 @@ NBLoadedTLDef::SignalGroup::setYellowTimes(SUMOTime tRedYellow, SUMOTime tYellow
 
 void
 NBLoadedTLDef::SignalGroup::sortPhases() {
-    sort(myPhases.begin(), myPhases.end(), phase_by_time_sorter());
+    std::sort(myPhases.begin(), myPhases.end(), [](const PhaseDef & p1, const PhaseDef & p2) {
+        return p1.myTime < p2.myTime;
+    });
 }
 
 
 void
-NBLoadedTLDef::SignalGroup::patchTYellow(int tyellow, bool forced) {
+NBLoadedTLDef::SignalGroup::patchTYellow(SUMOTime tyellow, bool forced) {
     if (myTYellow < 0) {
         // was not set before (was not loaded)
         myTYellow = tyellow;
     } else if (forced && myTYellow < tyellow) {
-        WRITE_WARNING("TYellow of signal group '" + getID() + "' was less than the computed one; patched (was:" + toString(myTYellow) + ", is:" + toString(tyellow) + ")");
+        WRITE_WARNING("TYellow of signal group '" + getID() + "' was less than the computed one; patched (was:" + toString(myTYellow) + ", is:" + time2string(tyellow) + ")");
         myTYellow = tyellow;
     }
 }
 
 
-std::vector<double>
+std::vector<SUMOTime>
 NBLoadedTLDef::SignalGroup::getTimes(SUMOTime cycleDuration) const {
     // within the phase container, we should have the green and red phases add their times
-    std::vector<double> ret; // !!! time vector
-    for (std::vector<PhaseDef>::const_iterator i = myPhases.begin(); i != myPhases.end(); i++) {
-        ret.push_back((double)(*i).myTime);
+    std::vector<SUMOTime> ret;
+    for (const PhaseDef& p : myPhases) {
+        ret.push_back(p.myTime);
     }
     // further, we possibly should set the yellow phases
     if (myTYellow > 0) {
-        for (std::vector<PhaseDef>::const_iterator i = myPhases.begin(); i != myPhases.end(); i++) {
-            if ((*i).myColor == TLCOLOR_RED) {
-                SUMOTime time = (SUMOTime)(*i).myTime + myTYellow;
-                if (time > cycleDuration) {
-                    time = time - cycleDuration;
-                }
-                ret.push_back((double) time);
+        for (const PhaseDef& p : myPhases) {
+            if (p.myColor == TLCOLOR_RED) {
+                ret.push_back((p.myTime + myTYellow) % cycleDuration);
             }
         }
     }
@@ -142,18 +132,6 @@ NBLoadedTLDef::SignalGroup::hasYellow(SUMOTime time) const {
 }
 
 
-bool
-NBLoadedTLDef::SignalGroup::containsConnection(NBEdge* from, NBEdge* to) const {
-    for (NBConnectionVector::const_iterator i = myConnections.begin(); i != myConnections.end(); i++) {
-        if ((*i).getFrom() == from && (*i).getTo() == to) {
-            return true;
-        }
-    }
-    return false;
-
-}
-
-
 const NBConnection&
 NBLoadedTLDef::SignalGroup::getConnection(int pos) const {
     assert(pos < (int)myConnections.size());
@@ -282,7 +260,7 @@ NBLoadedTLDef::NBLoadedTLDef(const NBEdgeCont& ec, const std::string& id, SUMOTi
 
 NBLoadedTLDef::~NBLoadedTLDef() {
     for (SignalGroupCont::iterator i = mySignalGroups.begin(); i != mySignalGroups.end(); ++i) {
-        delete(*i).second;
+        delete (*i).second;
     }
 }
 
@@ -290,48 +268,33 @@ NBLoadedTLDef::~NBLoadedTLDef() {
 NBTrafficLightLogic*
 NBLoadedTLDef::myCompute(int brakingTimeSeconds) {
     MsgHandler::getWarningInstance()->clear(); // !!!
-    NBLoadedTLDef::SignalGroupCont::const_iterator i;
-    // compute the switching times
-    std::set<double> tmpSwitchTimes;
-    for (i = mySignalGroups.begin(); i != mySignalGroups.end(); i++) {
-        NBLoadedTLDef::SignalGroup* group = (*i).second;
+    // compute the switching times and count the signals
+    std::set<SUMOTime> switchTimes;
+    int numSignals = 0;
+    for (const auto& i : mySignalGroups) {
+        NBLoadedTLDef::SignalGroup* const group = i.second;
         // needed later
         group->sortPhases();
         // patch the yellow time for this group
-        group->patchTYellow(brakingTimeSeconds, OptionsCont::getOptions().getBool("tls.yellow.patch-small"));
+        group->patchTYellow(TIME2STEPS(brakingTimeSeconds), OptionsCont::getOptions().getBool("tls.yellow.patch-small"));
         // copy the now valid times into the container
         //  both the given red and green phases are added and also the
         //  yellow times
-        std::vector<double> gtimes = group->getTimes(myCycleDuration);
-        for (std::vector<double>::const_iterator k = gtimes.begin(); k != gtimes.end(); k++) {
-            tmpSwitchTimes.insert(*k);
-        }
+        const std::vector<SUMOTime> gtimes = group->getTimes(myCycleDuration);
+        switchTimes.insert(gtimes.begin(), gtimes.end());
+        numSignals += group->getLinkNo();
     }
-    std::vector<double> switchTimes;
-    copy(tmpSwitchTimes.begin(), tmpSwitchTimes.end(), back_inserter(switchTimes));
-    sort(switchTimes.begin(), switchTimes.end());
 
-    // count the signals
-    int noSignals = 0;
-    for (i = mySignalGroups.begin(); i != mySignalGroups.end(); i++) {
-        noSignals += (*i).second->getLinkNo();
-    }
     // build the phases
-    NBTrafficLightLogic* logic = new NBTrafficLightLogic(getID(), getProgramID(), noSignals, myOffset, myType);
-    for (std::vector<double>::iterator l = switchTimes.begin(); l != switchTimes.end(); l++) {
-        // compute the duration of the current phase
-        int duration;
-        if (l != switchTimes.end() - 1) {
-            // get from the difference to the next switching time
-            duration = (int)((*(l + 1)) - (*l));
-        } else {
-            // get from the differenc to the first switching time
-            duration = (int)(myCycleDuration - (*l) + * (switchTimes.begin()));
+    NBTrafficLightLogic* logic = new NBTrafficLightLogic(getID(), getProgramID(), numSignals, myOffset, myType);
+    SUMOTime prev = -1;
+    for (const SUMOTime l : switchTimes) {
+        if (prev != -1) {
+            logic->addStep(l - prev, buildPhaseState(prev));
         }
-        // no information about yellow times will be generated
-        assert((*l) >= 0);
-        logic->addStep(TIME2STEPS(duration), buildPhaseState((int)(*l)));
+        prev = l;
     }
+    logic->addStep(myCycleDuration + (*switchTimes.begin()) - prev, buildPhaseState(prev));
     // check whether any warnings were printed
     if (MsgHandler::getWarningInstance()->wasInformed()) {
         WRITE_WARNING("During computation of traffic light '" + getID() + "'.");
@@ -347,7 +310,7 @@ NBLoadedTLDef::myCompute(int brakingTimeSeconds) {
         for (NBConnectionVector::const_iterator it1 = myControlledLinks.begin(); it1 != myControlledLinks.end(); it1++) {
             const NBConnection& c1 = *it1;
             const int i1 = c1.getTLIndex();
-            if (i1 == NBConnection::InvalidTlIndex || state[i1] != 'g' || c1.getFrom() == 0 || c1.getTo() == 0) {
+            if (i1 == NBConnection::InvalidTlIndex || state[i1] != 'g' || c1.getFrom() == nullptr || c1.getTo() == nullptr) {
                 continue;
             }
             for (NBConnectionVector::const_iterator it2 = myControlledLinks.begin(); it2 != myControlledLinks.end(); it2++) {
@@ -356,7 +319,7 @@ NBLoadedTLDef::myCompute(int brakingTimeSeconds) {
                 if (i2 != NBConnection::InvalidTlIndex
                         && i2 != i1
                         && (state[i2] == 'G' || state[i2] == 'g')
-                        && c2.getFrom() != 0 && c2.getTo() != 0) {
+                        && c2.getFrom() != nullptr && c2.getTo() != nullptr) {
                     const bool rightTurnConflict = NBNode::rightTurnConflict(
                                                        c1.getFrom(), c1.getTo(), c1.getFromLane(), c2.getFrom(), c2.getTo(), c2.getFromLane());
                     if (forbids(c2.getFrom(), c2.getTo(), c1.getFrom(), c1.getTo(), true, controlledWithin) || rightTurnConflict) {
@@ -385,7 +348,7 @@ NBLoadedTLDef::setTLControllingInformation() const {
 
 
 std::string
-NBLoadedTLDef::buildPhaseState(int time) const {
+NBLoadedTLDef::buildPhaseState(const SUMOTime time) const {
     int pos = 0;
     std::string state;
     // set the green and yellow information first;
@@ -478,6 +441,15 @@ NBLoadedTLDef::mustBrake(const NBConnection& possProhibited,
 }
 
 
+void
+NBLoadedTLDef::setParticipantsInformation() {
+    // assign participating nodes to the request
+    collectNodes();
+    // collect the information about participating edges and links
+    collectEdges();
+    collectLinks();
+}
+
 void
 NBLoadedTLDef::collectNodes() {
     myControlledNodes.clear();
@@ -507,7 +479,7 @@ NBLoadedTLDef::collectLinks() {
             std::vector<NBEdge::Connection> elv = incoming->getConnectionsFromLane(j);
             for (std::vector<NBEdge::Connection>::iterator k = elv.begin(); k != elv.end(); k++) {
                 NBEdge::Connection el = *k;
-                if (el.toEdge != 0) {
+                if (el.toEdge != nullptr) {
                     myControlledLinks.push_back(NBConnection(incoming, j, el.toEdge, el.toLane));
                 }
             }
@@ -546,17 +518,6 @@ NBLoadedTLDef::collectLinks() {
 }
 
 
-NBLoadedTLDef::SignalGroup*
-NBLoadedTLDef::findGroup(NBEdge* from, NBEdge* to) const {
-    for (SignalGroupCont::const_iterator i = mySignalGroups.begin(); i != mySignalGroups.end(); i++) {
-        if ((*i).second->containsConnection(from, to)) {
-            return (*i).second;
-        }
-    }
-    return 0;
-}
-
-
 bool
 NBLoadedTLDef::addToSignalGroup(const std::string& groupid,
                                 const NBConnection& connection) {
@@ -565,12 +526,12 @@ NBLoadedTLDef::addToSignalGroup(const std::string& groupid,
     }
     mySignalGroups[groupid]->addConnection(connection);
     NBNode* n1 = connection.getFrom()->getToNode();
-    if (n1 != 0) {
+    if (n1 != nullptr) {
         addNode(n1);
         n1->addTrafficLight(this);
     }
     NBNode* n2 = connection.getTo()->getFromNode();
-    if (n2 != 0) {
+    if (n2 != nullptr) {
         addNode(n2);
         n2->addTrafficLight(this);
     }
@@ -612,7 +573,7 @@ NBLoadedTLDef::setSignalYellowTimes(const std::string& groupid,
 
 
 void
-NBLoadedTLDef::setCycleDuration(int cycleDur) {
+NBLoadedTLDef::setCycleDuration(SUMOTime cycleDur) {
     myCycleDuration = cycleDur;
 }
 
@@ -635,10 +596,10 @@ NBLoadedTLDef::remapRemoved(NBEdge* removed,
 
 void
 NBLoadedTLDef::replaceRemoved(NBEdge* removed, int removedLane,
-                              NBEdge* by, int byLane) {
+                              NBEdge* by, int byLane, bool incoming) {
     for (SignalGroupCont::const_iterator i = mySignalGroups.begin(); i != mySignalGroups.end(); i++) {
         SignalGroup* group = (*i).second;
-        if (group->containsIncoming(removed) || group->containsOutgoing(removed)) {
+        if ((incoming && group->containsIncoming(removed)) || (!incoming && group->containsOutgoing(removed))) {
             group->remap(removed, removedLane, by, byLane);
         }
     }
@@ -653,5 +614,16 @@ NBLoadedTLDef::initNeedsContRelation() const {
 }
 
 
-/****************************************************************************/
+int
+NBLoadedTLDef::getMaxIndex() {
+    setParticipantsInformation();
+    NBTrafficLightLogic* logic = compute(OptionsCont::getOptions());
+    if (logic != nullptr) {
+        return logic->getNumLinks() - 1;
+    } else {
+        return -1;
+    }
+}
 
+
+/****************************************************************************/
