@@ -1,49 +1,43 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBNetBuilder.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
+/// @author  Gregor Laemmel
 /// @date    Fri, 29.04.2005
-/// @version $Id: NBNetBuilder.h 18639 2015-08-17 13:59:19Z namdre $
 ///
 // Instance responsible for building networks
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBNetBuilder_h
-#define NBNetBuilder_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <iostream>
 #include <vector>
 #include <set>
+#include <utils/shapes/ShapeContainer.h>
 #include "NBEdgeCont.h"
 #include "NBTypeCont.h"
 #include "NBNodeCont.h"
 #include "NBNode.h"
+#include "NBParking.h"
 #include "NBTrafficLightLogicCont.h"
 #include "NBDistrictCont.h"
-#include "NBJoinedEdgesMap.h"
+#include "NBPTStopCont.h"
+#include "NBPTLineCont.h"
 #include <utils/common/UtilExceptions.h>
 
 
@@ -120,7 +114,6 @@ public:
     /// @brief Destructor
     ~NBNetBuilder();
 
-
     /** @brief Initialises the storage by applying given options
      *
      * Options, mainly steering the acceptance of edges, are parsed
@@ -131,74 +124,79 @@ public:
      */
     void applyOptions(OptionsCont& oc);
 
-
     /** @brief Performs the network building steps
      *
      * @param[in] oc Container that contains options for building
      * @param[in] explicitTurnarounds List of edge ids for which turn-arounds should be added (used by NETEDIT)
-     * @param[in] removeElements whether processing steps which cause nodes and edges to be removed shall be triggered
+     * @param[in] mayAddOrRemove whether processing steps which cause nodes and edges to be added or removed shall be triggered (used by netedit)
      * @exception ProcessError (recheck)
      */
-    void compute(OptionsCont& oc,
-                 const std::set<std::string>& explicitTurnarounds = std::set<std::string>(),
-                 bool removeElements = true);
-
+    void compute(OptionsCont& oc, const std::set<std::string>& explicitTurnarounds = std::set<std::string>(), bool mayAddOrRemove = true);
 
+    /** @brief Updates the shape for a single Node
+     *
+     * @param[in] oc Container that contains options for building
+     * @param[in] explicitTurnarounds List of edge ids for which turn-arounds should be added (used by NETEDIT)
+     * @param[in] mayAddOrRemove whether processing steps which cause nodes and edges to be added or removed shall be triggered (used by netedit)
+     * @exception ProcessError (recheck)
+     */
+    //void computeSingleNode(NBNode* node, OptionsCont& oc, const std::set<std::string>& explicitTurnarounds = std::set<std::string>(), bool mayAddOrRemove = true);
 
     /// @name Retrieval of subcontainers
     /// @{
-
-    /** @brief Returns the edge container
-     * @return The edge container (reference)
-     */
+    /// @brief Returns a reference to edge container
     NBEdgeCont& getEdgeCont() {
         return myEdgeCont;
     }
 
-
-    /** @brief Returns the node container
-     * @return The node container (reference)
-     */
+    /// @brief Returns a reference to the node container
     NBNodeCont& getNodeCont() {
         return myNodeCont;
     }
 
-
-    /** @brief Returns the type container
-     * @return The type container (reference)
-     */
+    /// @brief Returns a reference to the type container
     NBTypeCont& getTypeCont() {
         return myTypeCont;
     }
 
-
-    /** @brief Returns the traffic light logics container
-     * @return The traffic light logics container (reference)
-     */
+    /// @brief Returns a reference to the traffic light logics container
     NBTrafficLightLogicCont& getTLLogicCont() {
         return myTLLCont;
     }
 
-
-    /** @brief Returns the districts container
-     * @return The districts container (reference)
-     */
+    /// @brief Returns a reference the districts container
     NBDistrictCont& getDistrictCont() {
         return myDistrictCont;
     }
 
-    /** @brief Returns the joined edges map
-     * @return The map of joined edges
-     */
-    const NBJoinedEdgesMap& getJoinedEdgesMap() const {
-        return myJoinedEdges;
+
+    /// @brief Returns a reference to the pt stop container
+    NBPTStopCont& getPTStopCont() {
+        return myPTStopCont;
+    }
+
+    /// @brief Returns a reference to the pt line container
+    NBPTLineCont& getPTLineCont() {
+        return myPTLineCont;
     }
     /// @}
 
+    NBParkingCont& getParkingCont() {
+        return myParkingCont;
+    }
 
-    /// @brief notify about style of loaded network
-    void haveLoadedNetworkWithoutInternalEdges() {
-        myHaveLoadedNetworkWithoutInternalEdges = true;
+    ShapeContainer& getShapeCont() {
+        return myShapeCont;
+    }
+
+    /// @brief notify about style of loaded network (Without Crossings)
+    bool haveNetworkCrossings() {
+        return myNetworkHaveCrossings;
+    }
+
+    /// @brief enable crossing in networks
+    void setHaveNetworkCrossings(bool value) {
+        myNetworkHaveCrossings = value;
     }
 
     /**
@@ -209,9 +207,15 @@ public:
      * @param[in] from_srs The spatial reference system of the input coordinate
      * @notde These methods are located outside of GeoConvHelper to avoid linker-dependencies on GDAL for libgeom
      */
-    static bool transformCoordinates(Position& from, bool includeInBoundary = true, GeoConvHelper* from_srs = 0);
+    static bool transformCoordinate(Position& from, bool includeInBoundary = true, GeoConvHelper* from_srs = 0);
     static bool transformCoordinates(PositionVector& from, bool includeInBoundary = true, GeoConvHelper* from_srs = 0);
 
+    /// @brief insertion geometry points to ensure maximum segment length between points
+    static int addGeometrySegments(PositionVector& from, const PositionVector& cartesian, const double maxLength);
+
+    /// @brief whether netbuilding takes place in the context of NETEDIT
+    static bool runningNetedit();
+
 
 protected:
     /**
@@ -221,12 +225,12 @@ protected:
     class by_id_sorter {
     public:
         /// @brief constructor
-        explicit by_id_sorter() { }
+        explicit by_id_sorter() {}
 
+        /// @brief selection operator
         int operator()(const NBNode* n1, const NBNode* n2) const {
             return n1->getID() < n2->getID();
         }
-
     };
 
 protected:
@@ -245,15 +249,23 @@ protected:
     /// @brief The used container for districts
     NBDistrictCont myDistrictCont;
 
-    /// @brief Map of joined edges
-    NBJoinedEdgesMap myJoinedEdges;
+    /// @brief The used container for pt stops
+    NBPTStopCont myPTStopCont;
 
-    /// @brief whether a .net.xml without internal edges was loaded
-    bool myHaveLoadedNetworkWithoutInternalEdges;
+    /// @brief The used container for pt stops
+    NBPTLineCont myPTLineCont;
+
+    NBParkingCont myParkingCont;
+
+    /// @brief container for loaded polygon data
+    ShapeContainer myShapeCont;
+
+    /// @brief flag to indicate that network has crossings
+    bool myNetworkHaveCrossings;
 
 private:
     /// @brief shift network so its lower left corner is at 0,0
-    void moveToOrigin(GeoConvHelper& geoConvHelper);
+    void moveToOrigin(GeoConvHelper& geoConvHelper, bool lefthand);
 
     /// @brief mirror the network along the X-axis
     void mirrorX();
@@ -264,11 +276,4 @@ private:
 
     /// @brief invalidated assignment operator
     NBNetBuilder& operator=(const NBNetBuilder& s);
-
 };
-
-
-#endif
-
-/****************************************************************************/
-
