@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBNodeShapeComputer.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    2004-01-12
-/// @version $Id: NBNodeShapeComputer.h 18169 2015-03-27 13:40:16Z behrisch $
 ///
 // This class computes shapes of junctions
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBNodeShapeComputer_h
-#define NBNodeShapeComputer_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <utils/geom/PositionVector.h>
 
@@ -58,7 +48,12 @@ public:
     ~NBNodeShapeComputer();
 
     /// Computes the shape of the assigned junction
-    PositionVector compute(bool leftHand);
+    PositionVector compute();
+
+    /// @brief get computed radius for node
+    double getRadius() const {
+        return myRadius;
+    }
 
 private:
     typedef std::map<NBEdge*, PositionVector> GeomsMap;
@@ -83,6 +78,10 @@ private:
      */
     PositionVector computeNodeShapeSmall();
 
+    /// @brief compute clockwise/counter-clockwise edge boundaries
+    void computeEdgeBoundaries(const EdgeVector& edges,
+                               GeomsMap& geomsCCW,
+                               GeomsMap& geomsCW);
 
     /** @brief Joins edges and computes ccw/cw boundaries
      *
@@ -93,23 +92,19 @@ private:
      *  all edges within the value-vector which direction at the node differs
      *  less than 1 from the key-edge's direction.
      */
-    void joinSameDirectionEdges(std::map<NBEdge*, std::set<NBEdge*> >& same,
-                                GeomsMap& geomsCCW,
-                                GeomsMap& geomsCW);
+    void joinSameDirectionEdges(const EdgeVector& edges, std::map<NBEdge*, std::set<NBEdge*> >& same);
 
-    /** @brief Joins edges and computes ccw/cw boundaries
+    /** @brief Joins edges
      *
      * This methods joins edges which are in marked as being "same" in the means
      *  as given by joinSameDirectionEdges. The result (list of so-to-say "directions"
-     *  is returned; additionally, the boundaries of these directions are stored in
-     *  ccwBoundary/cwBoundary.
+     *  is returned;
      */
     EdgeVector computeUniqueDirectionList(
+        const EdgeVector& all,
         std::map<NBEdge*, std::set<NBEdge*> >& same,
         GeomsMap& geomsCCW,
-        GeomsMap& geomsCW,
-        std::map<NBEdge*, NBEdge*>& ccwBoundary,
-        std::map<NBEdge*, NBEdge*>& cwBoundary);
+        GeomsMap& geomsCW);
 
     /** @brief Compute smoothed corner shape
      * @param[in] begShape
@@ -137,27 +132,36 @@ private:
                               GeomsMap& geomsCCW,
                               EdgeVector::const_iterator& cwi,
                               EdgeVector::const_iterator& ccwi,
-                              SUMOReal& cad,
-                              SUMOReal& ccad);
+                              double& cad,
+                              double& ccad);
 
     /// @return whether trying to intersect these edges would probably fail
-    bool badIntersection(const NBEdge* e1, const NBEdge* e2, 
-            const PositionVector& e1cw, const PositionVector& e2ccw, SUMOReal distance);
+    bool badIntersection(const NBEdge* e1, const NBEdge* e2, double distance);
 
     /// @brief return the intersection point closest to the given offset
-    SUMOReal closestIntersection(const PositionVector& geom1, const PositionVector& geom2, SUMOReal offset);
+    double closestIntersection(const PositionVector& geom1, const PositionVector& geom2, double offset);
+
+    /// @brief whether the given edges (along with those in the same direction) requires a large turning radius
+    bool needsLargeTurn(NBEdge* e1, NBEdge* e2,
+                        std::map<NBEdge*, std::set<NBEdge*> >& same) const;
+
+    /// @brief determine the default radius appropriate for the current junction
+    double getDefaultRadius(const OptionsCont& oc);
+
+    /// @brief compute with of rightmost lanes that exlude the given permissions
+    static double getExtraWidth(const NBEdge* e, SVCPermissions exclude);
 
 private:
     /// The node to compute the geometry for
     const NBNode& myNode;
 
+    /// @brief the computed node radius
+    double myRadius;
+
+    static const SVCPermissions SVC_LARGE_TURN;
+
 private:
     /// @brief Invalidated assignment operator
     NBNodeShapeComputer& operator=(const NBNodeShapeComputer& s);
 
 };
-
-#endif
-
-/****************************************************************************/
-
