@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBOwnTLDef.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @date    Tue, 29.05.2005
-/// @version $Id: NBOwnTLDef.h 21182 2016-07-18 06:46:01Z behrisch $
 ///
 // A traffic light logics which must be computed (only nodes/edges are given)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBOwnTLDef_h
-#define NBOwnTLDef_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <set>
@@ -87,12 +77,6 @@ public:
     ~NBOwnTLDef();
 
 
-    /** @brief Builds the list of participating nodes/edges/links
-     * @see NBTrafficLightDefinition::setParticipantsInformation
-     */
-    void setParticipantsInformation();
-
-
     /// @name Public methods from NBTrafficLightDefinition-interface
     /// @{
 
@@ -122,15 +106,16 @@ public:
     /// @brief add an additional pedestrian phase if there are crossings that did not get green yet
     static void addPedestrianScramble(NBTrafficLightLogic* logic, int noLinksAll,
                                       SUMOTime greenTime, SUMOTime yellowTime,
-                                      const std::vector<NBNode::Crossing>& crossings, const EdgeVector& fromEdges, const EdgeVector& toEdges);
+                                      const std::vector<NBNode::Crossing*>& crossings, const EdgeVector& fromEdges, const EdgeVector& toEdges);
 
     /// @brief add 1 or 2 phases depending on the presence of pedestrian crossings
     static std::string addPedestrianPhases(NBTrafficLightLogic* logic, SUMOTime greenTime,
-                                           std::string state, const std::vector<NBNode::Crossing>& crossings, const EdgeVector& fromEdges, const EdgeVector& toEdges);
+                                           SUMOTime minDur, SUMOTime maxDur,
+                                           std::string state, const std::vector<NBNode::Crossing*>& crossings, const EdgeVector& fromEdges, const EdgeVector& toEdges);
 
     /// @brief compute phase state in regard to pedestrian crossings
     static std::string patchStateForCrossings(const std::string& state,
-            const std::vector<NBNode::Crossing>& crossings, const EdgeVector& fromEdges, const EdgeVector& toEdges);
+            const std::vector<NBNode::Crossing*>& crossings, const EdgeVector& fromEdges, const EdgeVector& toEdges);
 
     /** @brief helper function for myCompute
      * @param[in] brakingTime Duration a vehicle needs for braking in front of the tls
@@ -142,6 +127,21 @@ public:
     /* initialize myNeedsContRelation and set myNeedsContRelationReady to true */
     void initNeedsContRelation() const;
 
+    /* build optional all-red phase */
+    void buildAllRedState(SUMOTime allRedTime, NBTrafficLightLogic* logic, const std::string& state);
+
+    ///@brief Returns the maximum index controlled by this traffic light
+    int getMaxIndex();
+
+    /// @brief sets the layout for the generated signal plan
+    void setLayout(TrafficLightLayout layout) {
+        myLayout = layout;
+    }
+
+    TrafficLightLayout getLayout() const {
+        return myLayout;
+    }
+
 protected:
     /// @name Protected methods from NBTrafficLightDefinition-interface
     /// @{
@@ -154,12 +154,6 @@ protected:
     NBTrafficLightLogic* myCompute(int brakingTimeSeconds);
 
 
-    /** @brief Collects the nodes participating in this traffic light
-     * @see NBTrafficLightDefinition::collectNodes
-     */
-    void collectNodes();
-
-
     /** @brief Collects the links participating in this traffic light
      * @exception ProcessError If a link could not be found
      * @see NBTrafficLightDefinition::collectLinks
@@ -175,17 +169,21 @@ protected:
      * @see NBTrafficLightDefinition::replaceRemoved
      */
     void replaceRemoved(NBEdge* removed, int removedLane,
-                        NBEdge* by, int byLane);
+                        NBEdge* by, int byLane, bool incoming);
     /// @}
 
 
 protected:
+
+    /// @brief test whether a joined tls with layout 'opposites' would be built without dedicated left-turn phase
+    bool corridorLike() const;
+
     /** @brief Returns the weight of a stream given its direction
      * @param[in] dir The direction of the stream
      * @return This stream's weight
      * @todo There are several magic numbers; describe
      */
-    SUMOReal getDirectionalWeight(LinkDirection dir);
+    double getDirectionalWeight(LinkDirection dir);
 
 
     /** @brief Returns this edge's priority at the node it ends at
@@ -200,7 +198,7 @@ protected:
      * @param[in] e2 The second edge
      * @todo There are several magic numbers; describe
      */
-    SUMOReal computeUnblockedWeightedStreamNumber(const NBEdge* const e1, const NBEdge* const e2);
+    double computeUnblockedWeightedStreamNumber(const NBEdge* const e1, const NBEdge* const e2);
 
 
     /** @brief Returns the combination of two edges from the given which has most unblocked streams
@@ -221,14 +219,31 @@ protected:
 
 
     /// @brief compute whether the given connection is crossed by pedestrians
-    static bool hasCrossing(const NBEdge* from, const NBEdge* to, const std::vector<NBNode::Crossing>& crossings);
+    static bool hasCrossing(const NBEdge* from, const NBEdge* to, const std::vector<NBNode::Crossing*>& crossings);
 
     /// @brief get edges that have connections
     static EdgeVector getConnectedOuterEdges(const EdgeVector& incoming);
 
 
-    /// @brief allow connections that follow on of the chosen edges
-    std::string allowFollowersOfChosen(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges);
+    /// @brief allow connections that are compatible with the chosen edges
+    std::string allowCompatible(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges,
+                                const std::vector<int>& fromLanes, const std::vector<int>& toLanes);
+
+    std::string allowSingleEdge(std::string state, const EdgeVector& fromEdges);
+
+    std::string allowFollowers(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges);
+
+    std::string allowPredecessors(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges,
+                                  const std::vector<int>& fromLanes, const std::vector<int>& toLanes);
+
+    std::string allowUnrelated(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges,
+                               const std::vector<bool>& isTurnaround,
+                               const std::vector<NBNode::Crossing*>& crossings);
+
+    std::string allowByVClass(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges, SVCPermissions perm);
+
+    /// @brief whether the given index is forbidden by a green link in the current state
+    bool forbidden(const std::string& state, int index, const EdgeVector& fromEdges, const EdgeVector& toEdges);
 
     /** @brief change 'G' to 'g' for conflicting connections
      * @param[in] state
@@ -239,13 +254,40 @@ protected:
      * @param[in] hadGreenMajor
      * @param[out] haveForbiddenLeftMover
      * @param[out] rightTurnConflicts
+     * @param[out] mergeConflicts
      * @return The corrected state
      */
     std::string correctConflicting(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges,
                                    const std::vector<bool>& isTurnaround,
                                    const std::vector<int>& fromLanes,
+                                   const std::vector<int>& toLanes,
                                    const std::vector<bool>& hadGreenMajor,
-                                   bool& haveForbiddenLeftMover, std::vector<bool>& rightTurnConflicts);
+                                   bool& haveForbiddenLeftMover,
+                                   std::vector<bool>& rightTurnConflicts,
+                                   std::vector<bool>& mergeConflicts);
+
+    /// @brief prevent green and red from the same lane
+    std::string correctMixed(std::string state, const EdgeVector& fromEdges,
+                             const std::vector<int>& fromLanes,
+                             bool& buildMixedGreenPhase, std::vector<bool>& mixedGreen);
+
+    /// @brief fix states in regard to custom crossing indices
+    void checkCustomCrossingIndices(NBTrafficLightLogic* logic) const;
+
+    /// @brief avoid yellow signal between successive green (major) phases
+    void fixSuperfluousYellow(NBTrafficLightLogic* logic) const;
+
+    /// @brief switch of signal for links that are always green
+    void deactivateAlwaysGreen(NBTrafficLightLogic* logic) const;
+
+    /// @brief switch of signal for links that are inside a joined tls
+    void deactivateInsideEdges(NBTrafficLightLogic* logic, const EdgeVector& fromEdges) const;
+
+    /// @brief compute time to clear all vehicles from within an alternateOneWay layout
+    SUMOTime computeEscapeTime(const std::string& state, const EdgeVector& fromEdges, const EdgeVector& toEdges) const;
+
+    /// @brief check whether there is a straight connection from this edge
+    bool hasStraightConnection(const NBEdge* fromEdge);
 
     /** @class edge_by_incoming_priority_sorter
      * @brief Sorts edges by their priority within the node they end at
@@ -269,10 +311,7 @@ private:
     /// @brief Whether left-mover should not have an additional phase
     bool myHaveSinglePhase;
 
+    /// @brief the layout for generated signal plans
+    TrafficLightLayout myLayout;
+
 };
-
-
-#endif
-
-/****************************************************************************/
-
