@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBRequest.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBRequest.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // This class computes the logic of a junction
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBRequest_h
-#define NBRequest_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -77,15 +67,14 @@ public:
               const EdgeVector& outgoing,
               const NBConnectionProhibits& loadedProhibits);
 
-    /** destructor */
+    /// @brief destructor
     ~NBRequest();
 
-    /** builds the bitset-representation of the logic */
+    /// @brief builds the bitset-representation of the logic
     void buildBitfieldLogic();
 
-    /** @brief returns the number of the junction's lanes and the number
-        of the junction's links in respect. @note: only connected lanes are counted */
-    std::pair<unsigned int, unsigned int> getSizes() const;
+    /// @brief @brief returns the number of the junction's lanes and the number of the junction's links in respect. @note: only connected lanes are counted
+    std::pair<int, int> getSizes() const;
 
     /** @brief Returns the information whether "prohibited" flow must let "prohibitor" flow pass
      * @param[in] possProhibitorFrom The maybe prohibiting connection's begin
@@ -139,34 +128,51 @@ public:
                  const NBEdge* const possProhibitedFrom, const NBEdge* const possProhibitedTo,
                  bool regardNonSignalisedLowerPriority) const;
 
-    /** writes the XML-representation of the logic as a bitset-logic
-        XML representation */
-    void writeLogic(std::string key, OutputDevice& into, const bool checkLaneFoes) const;
+    /// @brief writes the XML-representation of the logic as a bitset-logic XML representation
+    void computeLogic(const bool checkLaneFoes);
 
-    /// prints the request
+    void writeLogic(OutputDevice& into) const;
+
+    const std::string& getFoes(int linkIndex) const;
+    const std::string& getResponse(int linkIndex) const;
+
+    /// @brief prints the request
     friend std::ostream& operator<<(std::ostream& os, const NBRequest& r);
 
-    /// reports warnings if any occured
+    /// @brief reports warnings if any occurred
     static void reportWarnings();
 
-    /// @brief whether multple connections from the same edge target the same lane
-    static bool mergeConflict(const NBEdge* from, const NBEdge::Connection& con,
-                              const NBEdge* prohibitorFrom,  const NBEdge::Connection& prohibitorCon, bool foes);
+    /// @brief whether the given connections crosses the foe connection from the same lane and must yield
+    bool rightTurnConflict(const NBEdge* from, const NBEdge::Connection& con,
+                           const NBEdge* prohibitorFrom, const NBEdge::Connection& prohibitorCon) const;
 
+    /// @brief whether multple connections from the same edge target the same lane
+    bool mergeConflict(const NBEdge* from, const NBEdge::Connection& con,
+                       const NBEdge* prohibitorFrom, const NBEdge::Connection& prohibitorCon, bool foes) const;
+
+    /// @brief whether opposite left turns intersect
+    bool oppositeLeftTurnConflict(const NBEdge* from, const NBEdge::Connection& con,
+                                  const NBEdge* prohibitorFrom,  const NBEdge::Connection& prohibitorCon, bool foes) const;
+
+
+    /// @brief whether there are conflicting streams of traffic at this node
+    bool hasConflict() const;
+
+    /// @brief whether there are conflicting streams of traffic for the given link index
+    bool hasConflictAtLink(int linkIndex) const;
 
 private:
     /** sets the information that the edge from1->to1 blocks the edge
         from2->to2 (is higher priorised than this) */
     void setBlocking(NBEdge* from1, NBEdge* to1, NBEdge* from2, NBEdge* to2);
 
-    /** @brief writes the response of a certain lane
+    /** @brief computes the response of a certain lane
         Returns the next link index within the junction */
-    int writeLaneResponse(OutputDevice& od, NBEdge* from, int lane,
-                          int pos, const bool checkLaneFoes) const;
+    int computeLaneResponse(NBEdge* from, int lane, int pos, const bool checkLaneFoes);
 
-    /** @brief writes the response of a certain crossing
+    /** @brief computes the response of a certain crossing
         Returns the next link index within the junction */
-    int writeCrossingResponse(OutputDevice& od, const NBNode::Crossing& crossing, int pos) const;
+    int computeCrossingResponse(const NBNode::Crossing& crossing, int pos);
 
     /** @brief Writes the response of a certain link
      *
@@ -187,8 +193,7 @@ private:
      * @return the response string
      * @exception IOError not yet implemented
      */
-    std::string getResponseString(int tlIndex, const NBEdge* const from, const NBEdge* const to,
-                                  int fromLane, int toLane, bool mayDefinitelyPass, const bool checkLaneFoes) const;
+    std::string getResponseString(const NBEdge* const from, const NBEdge::Connection& c, const bool checkLaneFoes) const;
 
 
     /** writes which participating links are foes to the given */
@@ -207,76 +212,71 @@ private:
     int getIndex(const NBEdge* const from, const NBEdge* const to) const;
 
 
-    /** returns the distance between the incoming (from) and the outgoing (to)
-        edge clockwise in edges */
-    size_t distanceCounterClockwise(NBEdge* from, NBEdge* to);
+    /// @brief returns the distance between the incoming (from) and the outgoing (to) edge clockwise in edges
+    int distanceCounterClockwise(NBEdge* from, NBEdge* to);
 
-    /** computes the relationships between links outgoing right of the given
-        link */
+    /// @brief computes the relationships between links outgoing right of the given link */
     void computeRightOutgoingLinkCrossings(NBEdge* from, NBEdge* to);
 
-    /** computes the relationships between links outgoing left of the given
-        link */
+    /// @brief computes the relationships between links outgoing left of the given link
     void computeLeftOutgoingLinkCrossings(NBEdge* from, NBEdge* to);
 
 
     void resetSignalised();
 
-
     /// @brief reset foes it the number of lanes matches (or exceeds) the number of incoming connections for an edge
     void resetCooperating();
 
+    /// @brief whether the given connections must be checked for lane conflicts due to the vClasses involved
+    bool checkLaneFoesByClass(const NBEdge::Connection& con,
+                              const NBEdge* prohibitorFrom,  const NBEdge::Connection& prohibitorCon) const;
+
+    /// @brief whether the given connections must be checked for lane conflicts due to disjunct target lanes
+    bool checkLaneFoesByCooperation(const NBEdge* from, const NBEdge::Connection& con,
+                                    const NBEdge* prohibitorFrom,  const NBEdge::Connection& prohibitorCon) const;
+
     /** @brief return whether the given laneToLane connections prohibit each other
      * under the assumption that the edge2edge connections are in conflict
      */
     bool laneConflict(const NBEdge* from, const NBEdge* to, int toLane, const NBEdge* prohibitorFrom, const NBEdge* prohibitorTo, int prohibitorToLane) const;
 
-    /// @brief return to total number of edge-to-edge connections of this request-logic
-    inline size_t numLinks() const;
 
+    /// @brief return to total number of edge-to-edge connections of this request-logic
+    inline int numLinks() const;
 
 private:
-    /// the node the request is assigned to
+    /// @brief the node the request is assigned to
     NBNode* myJunction;
 
-    /** all (icoming and outgoing) of the junctions edges */
+    /// @brief all (icoming and outgoing) of the junctions edges
     const EdgeVector& myAll;
 
-    /** edges incoming to the junction */
+    /// @brief edges incoming to the junction
     const EdgeVector& myIncoming;
 
-    /** edges outgoing from the junction */
+    /// @brief edges outgoing from the junction
     const EdgeVector& myOutgoing;
 
-    /** edges outgoing from the junction */
-    const std::vector<NBNode::Crossing>& myCrossings;
-
-    /** definition of a container to store boolean informations about a link
-        into */
+    /// @brief definition of a container to store boolean informations about a link into
     typedef std::vector<bool> LinkInfoCont;
 
-    /** definition of a container for link(edge->edge) X link(edge->edge)
-        combinations (size = |myIncoming|*|myOutgoing|) */
+    /// @brief definition of a container for link(edge->edge) X link(edge->edge) combinations (size = |myIncoming|*|myOutgoing|)
     typedef std::vector<LinkInfoCont> CombinationsCont;
 
-    /** a container for approached lanes of a certain edge */
-    typedef std::map<NBEdge*, LaneVector> OccupiedLanes;
-
-    /** the link X link blockings */
+    /// @brief the link X link blockings
     CombinationsCont  myForbids;
 
-    /** the link X link is done-checks */
+    /// @brief the link X link is done-checks
     CombinationsCont  myDone;
 
-private:
-    static size_t myGoodBuilds, myNotBuild;
+    /// @brief precomputed right-of-way matrices for each lane-to-lane link
+    std::vector<std::string> myFoes;
+    std::vector<std::string> myResponse;
+    std::vector<bool> myHaveVia;
 
 private:
+    static int myGoodBuilds, myNotBuild;
+
     /// @brief Invalidated assignment operator
-    NBRequest& operator=(const NBRequest& s);
+    NBRequest& operator=(const NBRequest& s) = delete;
 };
-
-#endif
-
-/****************************************************************************/
-
