@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBTrafficLightDefinition.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NBTrafficLightDefinition.cpp 23654 2017-03-24 12:21:55Z namdre $
 ///
 // The base class for traffic light logic definitions
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
@@ -44,11 +35,15 @@
 #include "NBOwnTLDef.h"
 #include "NBContHelper.h"
 
+//#define DEBUG_RIGHT_OF_WAY
+#define DEBUGCOND true
+
 // ===========================================================================
 // static members
 // ===========================================================================
 const std::string NBTrafficLightDefinition::DefaultProgramID = "0";
 const std::string NBTrafficLightDefinition::DummyID = "dummy";
+const SUMOTime NBTrafficLightDefinition::UNSPECIFIED_DURATION(-1);
 
 // ===========================================================================
 // method definitions
@@ -74,8 +69,8 @@ NBTrafficLightDefinition::NBTrafficLightDefinition(const std::string& id,
         i++;
     }
     std::sort(myControlledNodes.begin(), myControlledNodes.end(), NBNode::nodes_by_id_sorter());
-    for (std::vector<NBNode*>::const_iterator i = junctions.begin(); i != junctions.end(); i++) {
-        (*i)->addTrafficLight(this);
+    for (NBNode* const node : junctions) {
+        node->addTrafficLight(this);
     }
 }
 
@@ -112,11 +107,11 @@ NBTrafficLightDefinition::compute(OptionsCont& oc) {
     if (amInvalid()) {
         // make a copy of myControlledNodes because it will be modified;
         std::vector<NBNode*> nodes = myControlledNodes;
-        for (std::vector<NBNode*>::iterator it = nodes.begin(); it != nodes.end(); it++) {
-            (*it)->removeTrafficLight(this);
+        for (auto it : nodes) {
+            it->removeTrafficLight(this);
         }
         WRITE_WARNING("The traffic light '" + getID() + "' does not control any links; it will not be build.");
-        return 0;
+        return nullptr;
     }
     // compute the time needed to brake
     int brakingTime = computeBrakingTime(oc.getFloat("tls.yellow.min-decel"));
@@ -126,7 +121,7 @@ NBTrafficLightDefinition::compute(OptionsCont& oc) {
         brakingTime = oc.getInt("tls.yellow.time");
     }
     NBTrafficLightLogic* ret = myCompute(brakingTime);
-    ret->addParameter(getMap());
+    ret->updateParameters(getParametersMap());
     return ret;
 }
 
@@ -139,8 +134,21 @@ NBTrafficLightDefinition::amInvalid() const {
 
 int
 NBTrafficLightDefinition::computeBrakingTime(double minDecel) const {
-    double vmax = NBContHelper::maxSpeed(myIncomingEdges);
-    return (int)(vmax / minDecel);
+    if (myIncomingEdges.size() == 0) {
+        // don't crash
+        return 3;
+    }
+    const double vmax = NBContHelper::maxSpeed(myIncomingEdges);
+    if (vmax < 71 / 3.6) {
+        // up to 50kmh: 3 seconds , 60km/h: 4, 70kmh: 5
+        // @note: these are German regulations, other countries may differ
+        return 3 + (int)MAX2(0.0, (floor((vmax - 50 / 3.6) * 0.37)));
+    } else {
+        // above 70km/h we use a function that grows according to the "natural"
+        // formula (vmax / 2 * minDecel) but continues smoothly where the german
+        // rules leave of
+        return (int)(1.8 + vmax / 2 / minDecel);
+    }
 }
 
 
@@ -187,10 +195,10 @@ NBTrafficLightDefinition::collectEdges() {
     EdgeVector outer;
     // check which of the edges are completely within the junction
     //  add them to the list of edges lying within the node
-    for (EdgeVector::iterator j = myIncomingEdges.begin(); j != myIncomingEdges.end(); ++j) {
-        NBEdge* edge = *j;
+    for (NBEdge* edge : myIncomingEdges) {
+        edge->setInsideTLS(false); // reset
         // an edge lies within the logic if it is outgoing as well as incoming
-        EdgeVector::iterator k = find(myOutgoing.begin(), myOutgoing.end(), edge);
+        EdgeVector::iterator k = std::find(myOutgoing.begin(), myOutgoing.end(), edge);
         if (k != myOutgoing.end()) {
             myEdgesWithin.push_back(edge);
         } else  {
@@ -208,7 +216,7 @@ NBTrafficLightDefinition::collectEdges() {
         // edges that are marked as 'inner' will not get their own phase when
         // computing traffic light logics (unless they cannot be reached from the outside at all)
         if (reachable.count(edge) == 1) {
-            edge->setIsInnerEdge();
+            edge->setInsideTLS(true);
             // legacy behavior
             if (uncontrolledWithin && myControlledInnerEdges.count(edge->getID()) == 0) {
                 myIncomingEdges.erase(find(myIncomingEdges.begin(), myIncomingEdges.end(), edge));
@@ -266,7 +274,7 @@ NBTrafficLightDefinition::forbids(const NBEdge* const possProhibitorFrom,
                                   const NBEdge* const possProhibitedTo,
                                   bool regardNonSignalisedLowerPriority,
                                   bool sameNodeOnly) const {
-    if (possProhibitorFrom == 0 || possProhibitorTo == 0 || possProhibitedFrom == 0 || possProhibitedTo == 0) {
+    if (possProhibitorFrom == nullptr || possProhibitorTo == nullptr || possProhibitedFrom == nullptr || possProhibitedTo == nullptr) {
         return false;
     }
     // retrieve both nodes
@@ -279,8 +287,18 @@ NBTrafficLightDefinition::forbids(const NBEdge* const possProhibitorFrom,
     NBNode* outnode = *outgoing;
     EdgeVector::const_iterator i;
 
+#ifdef DEBUG_RIGHT_OF_WAY
+    if (DEBUGCOND) {
+        std::cout << "foribds tls=" << getID() << " from=" << possProhibitedFrom->getID() << " to=" << possProhibitedTo->getID() << " foeFrom=" << possProhibitorFrom->getID() << " foeTo=" << possProhibitorTo->getID() << " rnslp=" << regardNonSignalisedLowerPriority << " sameNodeOnly=" << sameNodeOnly;
+    }
+#endif
     if (incnode != outnode) {
         if (sameNodeOnly) {
+#ifdef DEBUG_RIGHT_OF_WAY
+            if (DEBUGCOND) {
+                std::cout << "   differentNodes: allows (no check)\n";
+            }
+#endif
             return false;
         }
         // the links are located at different nodes
@@ -297,7 +315,7 @@ NBTrafficLightDefinition::forbids(const NBEdge* const possProhibitorFrom,
             if (incnode != outnode2) {
                 continue;
             }
-            if (incnode->getDirection(possProhibitedTo, *i) != LINKDIR_STRAIGHT) {
+            if (incnode->getDirection(possProhibitedTo, *i) != LinkDirection::STRAIGHT) {
                 continue;
             }
             bool ret1 = incnode->foes(possProhibitorFrom, possProhibitorTo,
@@ -307,6 +325,11 @@ NBTrafficLightDefinition::forbids(const NBEdge* const possProhibitorFrom,
                                          regardNonSignalisedLowerPriority);
             bool ret = ret1 || ret2;
             if (ret) {
+#ifdef DEBUG_RIGHT_OF_WAY
+                if (DEBUGCOND) {
+                    std::cout << "   differentNodes: forbids\n";
+                }
+#endif
                 return true;
             }
         }
@@ -324,7 +347,7 @@ NBTrafficLightDefinition::forbids(const NBEdge* const possProhibitorFrom,
             if (incnode2 != outnode) {
                 continue;
             }
-            if (incnode2->getDirection(possProhibitorTo, *i) != LINKDIR_STRAIGHT) {
+            if (incnode2->getDirection(possProhibitorTo, *i) != LinkDirection::STRAIGHT) {
                 continue;
             }
             bool ret1 = incnode2->foes(possProhibitorTo, *i,
@@ -334,23 +357,39 @@ NBTrafficLightDefinition::forbids(const NBEdge* const possProhibitorFrom,
                                           regardNonSignalisedLowerPriority);
             bool ret = ret1 || ret2;
             if (ret) {
+#ifdef DEBUG_RIGHT_OF_WAY
+                if (DEBUGCOND) {
+                    std::cout << "   differentNodes: forbids (2)\n";
+                }
+#endif
                 return true;
             }
         }
+#ifdef DEBUG_RIGHT_OF_WAY
+        if (DEBUGCOND) {
+            std::cout << "   differentNodes: allows\n";
+        }
+#endif
         return false;
     }
     // both links are located at the same node
     //  check using this node's information
-    return incnode->forbids(possProhibitorFrom, possProhibitorTo,
-                            possProhibitedFrom, possProhibitedTo,
-                            regardNonSignalisedLowerPriority);
+    const bool result = incnode->forbids(possProhibitorFrom, possProhibitorTo,
+                                         possProhibitedFrom, possProhibitedTo,
+                                         regardNonSignalisedLowerPriority);
+#ifdef DEBUG_RIGHT_OF_WAY
+    if (DEBUGCOND) {
+        std::cout << "   sameNodes: " << (result ? "forbids" : "allows") << "\n";
+    }
+#endif
+    return result;
 }
 
 
 bool
 NBTrafficLightDefinition::foes(const NBEdge* const from1, const NBEdge* const to1,
                                const NBEdge* const from2, const NBEdge* const to2) const {
-    if (to1 == 0 || to2 == 0) {
+    if (to1 == nullptr || to2 == nullptr) {
         return false;
     }
     // retrieve both nodes (it is possible that a connection
@@ -409,8 +448,7 @@ NBTrafficLightDefinition::getIncomingEdges() const {
 
 
 void
-NBTrafficLightDefinition::collectAllLinks() {
-    myControlledLinks.clear();
+NBTrafficLightDefinition::collectAllLinks(NBConnectionVector& into) {
     int tlIndex = 0;
     // build the list of links which are controled by the traffic light
     for (EdgeVector::iterator i = myIncomingEdges.begin(); i != myIncomingEdges.end(); i++) {
@@ -421,20 +459,26 @@ NBTrafficLightDefinition::collectAllLinks() {
             for (std::vector<NBEdge::Connection>::iterator k = connected.begin(); k != connected.end(); k++) {
                 const NBEdge::Connection& el = *k;
                 if (incoming->mayBeTLSControlled(el.fromLane, el.toEdge, el.toLane)) {
-                    if (el.toEdge != 0 && el.toLane >= (int) el.toEdge->getNumLanes()) {
+                    if (el.toEdge != nullptr && el.toLane >= (int) el.toEdge->getNumLanes()) {
                         throw ProcessError("Connection '" + incoming->getID() + "_" + toString(j) + "->" + el.toEdge->getID() + "_" + toString(el.toLane) + "' yields in a not existing lane.");
                     }
-                    if (incoming->getToNode()->getType() != NODETYPE_RAIL_CROSSING || !isRailway(incoming->getPermissions())) {
-                        myControlledLinks.push_back(NBConnection(incoming, el.fromLane, el.toEdge, el.toLane, tlIndex++));
+                    if (incoming->getToNode()->getType() == SumoXMLNodeType::RAIL_CROSSING
+                            && isRailway(incoming->getPermissions())) {
+                        // railways stay uncontrolled at rail crossing but they
+                        // must be registered in MSRailCrossing
+                        into.push_back(NBConnection(incoming, el.fromLane, el.toEdge, el.toLane, -1));
+                    } else if (incoming->getToNode()->getType() == SumoXMLNodeType::RAIL_SIGNAL
+                               && incoming->getToNode()->getDirection(incoming, el.toEdge) == LinkDirection::TURN) {
+                        // turnarounds stay uncontrolled at rail signal
                     } else {
-                        myControlledLinks.push_back(NBConnection(incoming, el.fromLane, el.toEdge, el.toLane, -1));
+                        into.push_back(NBConnection(incoming, el.fromLane, el.toEdge, el.toLane, tlIndex++));
                     }
                 }
             }
         }
     }
-    if (myControlledLinks.size() > 0 && tlIndex == 0) {
-        WRITE_WARNING("The rail crossing '" + getID() + "' does not have any roads.");
+    if (into.size() > 0 && tlIndex == 0) {
+        WRITE_WARNINGF("The rail crossing '%' does not have any roads.", getID());
     }
 }
 
@@ -453,7 +497,7 @@ NBTrafficLightDefinition::needsCont(const NBEdge* fromE, const NBEdge* toE, cons
 void
 NBTrafficLightDefinition::initNeedsContRelation() const {
     if (!amInvalid()) {
-        NBOwnTLDef dummy(DummyID, myControlledNodes, 0, TLTYPE_STATIC);
+        NBOwnTLDef dummy(DummyID, myControlledNodes, 0, TrafficLightType::STATIC);
         dummy.initNeedsContRelation();
         myNeedsContRelation = dummy.myNeedsContRelation;
         for (std::vector<NBNode*>::const_iterator i = myControlledNodes.begin(); i != myControlledNodes.end(); i++) {
@@ -467,7 +511,7 @@ NBTrafficLightDefinition::initNeedsContRelation() const {
 bool
 NBTrafficLightDefinition::rightOnRedConflict(int index, int foeIndex) const {
     if (!myRightOnRedConflictsReady) {
-        NBOwnTLDef dummy(DummyID, myControlledNodes, 0, TLTYPE_STATIC);
+        NBOwnTLDef dummy(DummyID, myControlledNodes, 0, TrafficLightType::STATIC);
         dummy.setParticipantsInformation();
         NBTrafficLightLogic* tllDummy = dummy.computeLogicAndConts(0, true);
         delete tllDummy;
@@ -484,5 +528,10 @@ NBTrafficLightDefinition::rightOnRedConflict(int index, int foeIndex) const {
     return std::find(myRightOnRedConflicts.begin(), myRightOnRedConflicts.end(), std::make_pair(index, foeIndex)) != myRightOnRedConflicts.end();
 }
 
-/****************************************************************************/
+std::string
+NBTrafficLightDefinition::getDescription() const {
+    return getID() + ':' + getProgramID() + '@' + toString(this);
+}
 
+
+/****************************************************************************/
