@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBTrafficLightDefinition.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NBTrafficLightDefinition.h 21182 2016-07-18 06:46:01Z behrisch $
 ///
 // The base class for traffic light logic definitions
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBTrafficLightDefinition_h
-#define NBTrafficLightDefinition_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
@@ -75,6 +65,7 @@ class NBTrafficLightLogic;
 class NBTrafficLightDefinition : public Named, public Parameterised {
 public:
 
+    static const SUMOTime UNSPECIFIED_DURATION;
     static const std::string DefaultProgramID;
 
     /**
@@ -164,6 +155,14 @@ public:
      */
     virtual void removeNode(NBNode* node);
 
+    /** @brief removes the given connection from the traffic light
+     * if recontruct=true, reconstructs the logic and informs the edges for immediate use in NETEDIT
+     * @note: tlIndex is not necessarily unique. we need the whole connection data here
+     */
+    virtual void removeConnection(const NBConnection& conn, bool reconstruct = true) {
+        UNUSED_PARAMETER(conn);
+        UNUSED_PARAMETER(reconstruct);
+    }
 
     /** @brief Returns the list of controlled nodes
      * @return Controlled nodes
@@ -272,14 +271,16 @@ public:
      * @param[in] removedLane The lane of this edge to replace
      * @param[in] by The edge to insert instead
      * @param[in] byLane This edge's lane to insert instead
+     * @param[in] incoming Whether the removed edge is incoming or outgoing
      */
     virtual void replaceRemoved(NBEdge* removed, int removedLane,
-                                NBEdge* by, int byLane) = 0;
+                                NBEdge* by, int byLane, bool incoming) = 0;
 
     /// @brief patches (loaded) signal plans by modifying lane indices
-    virtual void shiftTLConnectionLaneIndex(NBEdge* edge, int offset) {
+    virtual void shiftTLConnectionLaneIndex(NBEdge* edge, int offset, int threshold = -1) {
         UNUSED_PARAMETER(edge);
         UNUSED_PARAMETER(offset);
+        UNUSED_PARAMETER(threshold);
     }
 
     /** @brief Returns the list of incoming edges (must be build first)
@@ -294,6 +295,12 @@ public:
     }
 
 
+    /// @brief returns the controlled links (non const version)
+    NBConnectionVector& getControlledLinks() {
+        return myControlledLinks;
+    }
+
+
     /** @brief Returns the ProgramID
      * @return The ID of the program (subID)
      */
@@ -305,10 +312,13 @@ public:
     /** @brief Sets the programID
      * @param[in] programID The new ID of the program (subID)
      */
-    void setProgramID(const std::string& programID) {
+    virtual void setProgramID(const std::string& programID) {
         mySubID = programID;
     }
 
+    virtual TrafficLightLayout getLayout() const {
+        return TrafficLightLayout::DEFAULT;
+    }
 
     /** @brief Returns the offset
      * @return Offset
@@ -323,12 +333,17 @@ public:
         return myType;
     }
 
+    /// @brief set the algorithm type (static etc..)
+    virtual void setType(TrafficLightType type) {
+        myType = type;
+    }
+
     /* @brief computes whether the given stream may have green minor while the
      * other stream has green major in the same phase
      */
     bool needsCont(const NBEdge* fromE, const NBEdge* toE, const NBEdge* otherFromE, const NBEdge* otherToE) const;
 
-    /// @brief whether the given index must yield to the foeIndex while turing right on a red light
+    /// @brief whether the given index must yield to the foeIndex while turning right on a red light
     virtual bool rightOnRedConflict(int index, int foeIndex) const;
 
     /* initialize myNeedsContRelation and set myNeedsContRelationReady to true
@@ -336,6 +351,30 @@ public:
      * subclasses instantiate a private instance of NBOwnTLDef to answer this query */
     virtual void initNeedsContRelation() const;
 
+    ///@brief Returns the maximum index controlled by this traffic light and assigned to a connection
+    virtual int getMaxIndex() = 0;
+
+    ///@brief Returns the maximum index controlled by this traffic light
+    virtual int getMaxValidIndex() {
+        return getMaxIndex();
+    }
+
+    /** @brief Computes the time vehicles may need to brake
+     *
+     * This time depends on the maximum speed allowed on incoming junctions.
+     * It is computed as max_speed_allowed / minimum_vehicle_decleration
+     */
+    int computeBrakingTime(double minDecel) const;
+
+    /// @brief whether this definition uses signal group (multiple connections with the same link index)
+    virtual bool usingSignalGroups() const {
+        return false;
+    };
+
+    /// @brief get ID and programID together (for convenient debugging)
+    std::string getDescription() const;
+
+
 protected:
     /// @brief id for temporary definitions
     static const std::string DummyID;
@@ -358,19 +397,11 @@ protected:
     virtual void collectEdges();
 
 
-    /** @brief Computes the time vehicles may need to brake
-     *
-     * This time depends on the maximum speed allowed on incoming junctions.
-     * It is computed as max_speed_allowed / minimum_vehicle_decleration
-     */
-    int computeBrakingTime(SUMOReal minDecel) const;
-
-
     // @return whether this traffic light is invalid and should be computed
     virtual bool amInvalid() const;
 
     /// @brief helper method for use in NBOwnTLDef and NBLoadedSUMOTLDef
-    void collectAllLinks();
+    void collectAllLinks(NBConnectionVector& into);
 
 protected:
     /// @brief The container with participating nodes
@@ -441,9 +472,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
