@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBTrafficLightLogic.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NBTrafficLightLogic.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A SUMO-compliant built logic for a traffic light
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBTrafficLightLogic_h
-#define NBTrafficLightLogic_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
@@ -66,16 +56,29 @@ public:
     public:
         /// @brief The duration of the phase in s
         SUMOTime duration;
+        SUMOTime minDur;
+        SUMOTime maxDur;
 
         /// @brief The state definition
         std::string state;
 
+        /// @brief next phase indices or empty list
+        std::vector<int> next;
+        /// @brief option phase name
+        std::string name;
+
         /** @brief Constructor
          * @param[in] durationArg The duration of the phase
          * @param[in] stateArg Signals per link
          */
-        PhaseDefinition(SUMOTime durationArg, const std::string& stateArg)
-            : duration(durationArg), state(stateArg) { }
+        PhaseDefinition(SUMOTime durationArg, const std::string& stateArg, SUMOTime minDurArg, SUMOTime maxDurArg, const std::vector<int>& nextArg, const std::string& nameArg) :
+            duration(durationArg),
+            minDur(minDurArg),
+            maxDur(maxDurArg),
+            state(stateArg),
+            next(nextArg),
+            name(nameArg)
+        { }
 
         /// @brief Destructor
         ~PhaseDefinition() { }
@@ -85,7 +88,12 @@ public:
          * @return Whether this and the given phases are same
          */
         bool operator!=(const PhaseDefinition& pd) const {
-            return pd.duration != duration || pd.state != state;
+            return (pd.duration != duration
+                    || pd.minDur != minDur
+                    || pd.maxDur != maxDur
+                    || pd.state != state
+                    || pd.next != next
+                    || pd.name != name);
         }
 
     };
@@ -98,8 +106,8 @@ public:
      * @param[in] offset The offset of the program (delay)
      * @param[in] type The algorithm type for the computed traffic light
      */
-    NBTrafficLightLogic(const std::string& id, const std::string& subid, unsigned int noLinks,
-                        SUMOTime offset = 0, TrafficLightType type = TLTYPE_STATIC);
+    NBTrafficLightLogic(const std::string& id, const std::string& subid, int noLinks,
+                        SUMOTime offset = 0, TrafficLightType type = TrafficLightType::STATIC);
 
 
     /** @brief Copy Constructor
@@ -116,11 +124,18 @@ public:
      *
      * @param[in] duration The duration of the phase to add
      * @param[in] state The state definition of a tls phase
+     * @param[in] minDur The minimum duration of the phase to add
+     * @param[in] maxDur The maximum duration of the phase to add
+     * @param[in] name The name of the phase
+     * @param[in] next The index of the next phase
      * @param[in] index The index of the new phase (-1 means append to end)
      * @note: the length of the state has to match the number of links
      *        and the length given in previous calls to addStep (throws ProcessError)
      */
-    void addStep(SUMOTime duration, const std::string& state, int index = -1);
+    void addStep(SUMOTime duration, const std::string& state,
+                 const std::vector<int>& next = std::vector<int>(), const std::string& name = "", int index = -1);
+    void addStep(SUMOTime duration, const std::string& state, SUMOTime minDur, SUMOTime maxDur,
+                 const std::vector<int>& next = std::vector<int>(), const std::string& name = "", int index = -1);
 
 
     /** @brief Modifies the state for an existing phase (used by NETEDIT)
@@ -128,18 +143,30 @@ public:
      * @param[in] tlIndex The index at which to modify the state
      * @param[in] linkState The new link state for the given index
      */
-    void setPhaseState(unsigned int phaseIndex, unsigned int tlIndex, LinkState linkState);
+    void setPhaseState(int phaseIndex, int tlIndex, LinkState linkState);
 
     /** @brief Modifies the duration for an existing phase (used by NETEDIT)
      * @param[in] phaseIndex The index of the phase to modify
      * @param[in] duration The new duration for this phase
      */
-    void setPhaseDuration(unsigned int phaseIndex, SUMOTime duration);
+    void setPhaseDuration(int phaseIndex, SUMOTime duration);
+    void setPhaseMinDuration(int phaseIndex, SUMOTime duration);
+    void setPhaseMaxDuration(int phaseIndex, SUMOTime duration);
+    void setPhaseNext(int phaseIndex, const std::vector<int>& next);
+    void setPhaseName(int phaseIndex, const std::string& name);
 
     /* @brief deletes the phase at the given index
      * @note thhrows InvalidArgument on out-of range index
     */
-    void deletePhase(unsigned int index);
+    void deletePhase(int index);
+
+    /* @brief changes state size either by cutting of at the end or by adding
+     * new states at the end
+    */
+    void setStateLength(int numLinks, LinkState fill = LINKSTATE_TL_RED);
+
+    /// @brief remove the index from all phase states
+    void deleteStateIndex(int index);
 
     /* @brief deletes all phases and reset the expect number of links
     */
@@ -149,7 +176,7 @@ public:
      *
      * Joins equal steps.
      */
-    void closeBuilding();
+    void closeBuilding(bool checkVarDurations = true);
 
 
     /** @brief Returns the duration of the complete cycle
@@ -192,7 +219,7 @@ public:
 
     /** @brief Returns the number of participating links
      */
-    unsigned int getNumLinks() {
+    int getNumLinks() {
         return myNumLinks;
     }
 
@@ -206,12 +233,19 @@ public:
         myType = type;
     }
 
+    /** @brief Sets the programID
+     * @param[in] programID The new ID of the program (subID)
+     */
+    void setProgramID(const std::string& programID) {
+        mySubID = programID;
+    }
+
 private:
     /// @brief The number of participating links
-    unsigned int myNumLinks;
+    int myNumLinks;
 
     /// @brief The tls program's subid
-    const std::string mySubID;
+    std::string mySubID;
 
     /// @brief The tls program's offset
     SUMOTime myOffset;
@@ -225,18 +259,8 @@ private:
     /// @brief The algorithm type for the traffic light
     TrafficLightType myType;
 
-    /// @brief all allowed characters for phase state (see SUMOXMLDefinitions)
-    static const char allowedStatesInitializer[];
-    static const std::string ALLOWED_STATES;
-
 private:
     /// @brief Invalidated assignment operator
-    NBTrafficLightLogic& operator=(const NBTrafficLightLogic& s);
+    NBTrafficLightLogic& operator=(const NBTrafficLightLogic& s) = delete;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
