@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBTrafficLightLogicCont.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NBTrafficLightLogicCont.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A container for traffic light definitions and built programs
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 #include <map>
 #include <string>
 #include <algorithm>
@@ -40,13 +31,10 @@
 #include "NBTrafficLightLogic.h"
 #include "NBTrafficLightLogicCont.h"
 #include "NBOwnTLDef.h"
+#include "NBLoadedSUMOTLDef.h"
 #include "NBEdgeCont.h"
 #include "NBNodeCont.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static members
@@ -76,7 +64,7 @@ NBTrafficLightLogicCont::applyOptions(OptionsCont& oc) {
     //  them to a quarter of the duration
     if (oc.isSet("tls.quarter-offset")) {
         std::vector<std::string> ids = oc.getStringVector("tls.quarter-offset");
-        myHalfOffsetTLS.insert(ids.begin(), ids.end());
+        myQuarterOffsetTLS.insert(ids.begin(), ids.end());
     }
 }
 
@@ -148,7 +136,7 @@ NBTrafficLightLogicCont::extract(NBTrafficLightDefinition* definition) {
 }
 
 
-std::pair<unsigned int, unsigned int>
+std::pair<int, int>
 NBTrafficLightLogicCont::computeLogics(OptionsCont& oc) {
     // clean previous logics
     Logics logics = getComputed();
@@ -157,38 +145,72 @@ NBTrafficLightLogicCont::computeLogics(OptionsCont& oc) {
     }
     myComputed.clear();
 
-    unsigned int numPrograms = 0;
-    Definitions definitions = getDefinitions();
-    for (Definitions::iterator it = definitions.begin(); it != definitions.end(); it++) {
-        if (computeSingleLogic(oc, *it)) {
+    if (oc.getBool("tls.group-signals")) {
+        // replace NBOwnTLDef tld with NBLoadedSUMOTLDef
+        for (NBTrafficLightDefinition* def : getDefinitions()) {
+            NBLoadedSUMOTLDef* lDef = dynamic_cast<NBLoadedSUMOTLDef*>(def);
+            if (lDef == nullptr) {
+                NBTrafficLightLogic* logic = def->compute(oc);
+                if (logic != nullptr) {
+                    lDef = new NBLoadedSUMOTLDef(*def, *logic);
+                    lDef->setParticipantsInformation();
+                    for (NBNode* node : lDef->getNodes()) {
+                        node->removeTrafficLight(def);
+                        node->addTrafficLight(lDef);
+                    }
+                    removeProgram(def->getID(), def->getProgramID());
+                    insert(lDef);
+                }
+            }
+            if (lDef != nullptr) {
+                lDef->groupSignals();
+            }
+        }
+    } else if (oc.getBool("tls.ungroup-signals")) {
+        for (NBTrafficLightDefinition* def : getDefinitions()) {
+            NBLoadedSUMOTLDef* lDef = dynamic_cast<NBLoadedSUMOTLDef*>(def);
+            // NBOwnTLDef are always ungrouped
+            if (lDef != nullptr) {
+                if (lDef->usingSignalGroups()) {
+                    lDef->ungroupSignals();
+                }
+            }
+        }
+    }
+    int numPrograms = 0;
+    for (NBTrafficLightDefinition* def : getDefinitions()) {
+        if (computeSingleLogic(oc, def)) {
             numPrograms++;
         }
     }
-    return std::pair<unsigned int, unsigned int>((unsigned int)myComputed.size(), numPrograms);
+    return std::pair<int, int>((int)myComputed.size(), numPrograms);
 }
 
 
 bool
 NBTrafficLightLogicCont::computeSingleLogic(OptionsCont& oc, NBTrafficLightDefinition* def) {
+    if (def->getNodes().size() == 0) {
+        return false;
+    }
     const std::string& id = def->getID();
     const std::string& programID = def->getProgramID();
     // build program
     NBTrafficLightLogic* built = def->compute(oc);
-    if (built == 0) {
+    if (built == nullptr) {
         WRITE_WARNING("Could not build program '" + programID + "' for traffic light '" + id + "'");
         return false;
     }
     // compute offset
     SUMOTime T = built->getDuration();
     if (myHalfOffsetTLS.count(id)) {
-        built->setOffset((SUMOTime)(T / 2.));
+        built->setOffset(TIME2STEPS(floor(STEPS2TIME(T / 2.))));
     }
     if (myQuarterOffsetTLS.count(id)) {
-        built->setOffset((SUMOTime)(T / 4.));
+        built->setOffset(TIME2STEPS(floor(STEPS2TIME(T / 4.))));
     }
     // and insert the result after computation
     // make sure we don't leak memory if computeSingleLogic is called externally
-    if (myComputed[id][programID] != 0) {
+    if (myComputed[id][programID] != nullptr) {
         delete myComputed[id][programID];
     }
     myComputed[id][programID] = built;
@@ -227,10 +249,10 @@ NBTrafficLightLogicCont::remapRemoved(NBEdge* removed, const EdgeVector& incomin
 
 void
 NBTrafficLightLogicCont::replaceRemoved(NBEdge* removed, int removedLane,
-                                        NBEdge* by, int byLane) {
+                                        NBEdge* by, int byLane, bool incoming) {
     Definitions definitions = getDefinitions();
     for (Definitions::iterator it = definitions.begin(); it != definitions.end(); it++) {
-        (*it)->replaceRemoved(removed, removedLane, by, byLane);
+        (*it)->replaceRemoved(removed, removedLane, by, byLane, incoming);
     }
 }
 
@@ -245,7 +267,7 @@ NBTrafficLightLogicCont::getDefinition(const std::string& id, const std::string&
             return i2->second;
         }
     }
-    return 0;
+    return nullptr;
 }
 
 const NBTrafficLightLogicCont::Program2Def&
@@ -269,7 +291,7 @@ NBTrafficLightLogicCont::getLogic(const std::string& id, const std::string& prog
             return i2->second;
         }
     }
-    return 0;
+    return nullptr;
 }
 
 
@@ -289,16 +311,41 @@ NBTrafficLightLogicCont::setTLControllingInformation(const NBEdgeCont& ec, const
     // handle rail signals which are not instantiated as normal definitions
     for (std::map<std::string, NBNode*>::const_iterator it = nc.begin(); it != nc.end(); it ++) {
         NBNode* n = it->second;
-        if (n->getType() == NODETYPE_RAIL_SIGNAL || n->getType() == NODETYPE_RAIL_CROSSING) {
-            NBOwnTLDef dummy(n->getID(), n, 0, TLTYPE_STATIC);
+        if (n->getType() == SumoXMLNodeType::RAIL_SIGNAL || n->getType() == SumoXMLNodeType::RAIL_CROSSING) {
+            NBOwnTLDef dummy(n->getID(), n, 0, TrafficLightType::STATIC);
             dummy.setParticipantsInformation();
             dummy.setTLControllingInformation();
+            n->setCrossingTLIndices(dummy.getID(), (int)dummy.getControlledLinks().size());
             n->removeTrafficLight(&dummy);
         }
     }
 }
 
 
+void
+NBTrafficLightLogicCont::setOpenDriveSignalParameters() {
+    Definitions definitions = getDefinitions();
+    for (NBTrafficLightDefinition* def : getDefinitions()) {
+        std::map<NBEdge*, std::string> defaultSignalIDs;
+        for (const NBConnection& con : def->getControlledLinks()) {
+            const NBEdge::Connection& c = con.getFrom()->getConnection(con.getFromLane(), con.getTo(), con.getToLane());
+            if (c.knowsParameter("signalID")) {
+                defaultSignalIDs[con.getFrom()] = c.getParameter("signalID");
+                def->setParameter("linkSignalID:" + toString(con.getTLIndex()), c.getParameter("signalID"));
+            }
+        }
+        // oftentimes, signals are placed on connecting road but are meant to apply to all connections from the incoming edge
+        for (const NBConnection& con : def->getControlledLinks()) {
+            const NBEdge::Connection& c = con.getFrom()->getConnection(con.getFromLane(), con.getTo(), con.getToLane());
+            if (!c.knowsParameter("signalID") && defaultSignalIDs.count(con.getFrom()) != 0) {
+                WRITE_WARNINGF("Guessing signalID for link index % at traffic light '%'.", con.getTLIndex(), def->getID());
+                def->setParameter("linkSignalID:" + toString(con.getTLIndex()), defaultSignalIDs[con.getFrom()]);
+            }
+        }
+    }
+}
+
+
 NBTrafficLightLogicCont::Logics
 NBTrafficLightLogicCont::getComputed() const {
     Logics result;
@@ -326,4 +373,3 @@ NBTrafficLightLogicCont::getDefinitions() const {
 
 
 /****************************************************************************/
-
