@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NBTrafficLightLogicCont.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NBTrafficLightLogicCont.h 22608 2017-01-17 06:28:54Z behrisch $
 ///
 // A container for traffic light definitions and built programs
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NBTrafficLightLogicCont_h
-#define NBTrafficLightLogicCont_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <map>
 #include <string>
@@ -174,7 +164,7 @@ public:
      * @todo Recheck usage
      */
     void replaceRemoved(NBEdge* removed, int removedLane,
-                        NBEdge* by, int byLane);
+                        NBEdge* by, int byLane, bool incoming);
 
 
     /** @brief Returns the named definition
@@ -214,6 +204,12 @@ public:
      */
     void setTLControllingInformation(const NBEdgeCont& ec, const NBNodeCont& nc);
 
+    /// @brief set OpenDRIVE signal reference parameters after all link indices are known
+    void setOpenDriveSignalParameters();
+
+    /// @brief Returns a list of all definitions (convenience for easier iteration)
+    typedef std::vector<NBTrafficLightDefinition*> Definitions;
+    Definitions getDefinitions() const;
 
 private:
     /// @brief Definition of internal the container types
@@ -222,7 +218,6 @@ private:
     typedef std::map<std::string, NBTrafficLightDefinition*> Program2Def;
     typedef std::map<std::string, Program2Def> Id2Defs;
     typedef std::vector<NBTrafficLightLogic*> Logics;
-    typedef std::vector<NBTrafficLightDefinition*> Definitions;
 
     /// @brief The container for previously computed tl-logics
     Id2Logics myComputed;
@@ -242,8 +237,6 @@ private:
     static const Program2Def EmptyDefinitions;
 
 private:
-    /// @brief Returns a list of all definitions (convenience for easier iteration)
-    Definitions getDefinitions() const;
 
     /** @brief Destroys all stored definitions and logics
      */
@@ -251,9 +244,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
