@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    netconvert_main.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: netconvert_main.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Main for NETCONVERT
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #ifdef HAVE_VERSION_H
 #include <version.h>
@@ -40,7 +31,6 @@
 #include <netimport/NILoader.h>
 #include <netbuild/NBFrame.h>
 #include <netbuild/NBNetBuilder.h>
-#include <netbuild/NBDistribution.h>
 #include <netwrite/NWFrame.h>
 #include <utils/options/OptionsIO.h>
 #include <utils/options/OptionsCont.h>
@@ -48,14 +38,11 @@
 #include <utils/common/RandHelper.h>
 #include <utils/common/SystemFrame.h>
 #include <utils/common/MsgHandler.h>
+#include <utils/distribution/DistributionCont.h>
 #include <utils/xml/XMLSubSys.h>
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/geom/GeoConvHelper.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -72,13 +59,17 @@ fillOptions() {
     oc.addOptionSubTopic("Input");
     oc.addOptionSubTopic("Output");
     GeoConvHelper::addProjectionOptions(oc);
+    oc.addOptionSubTopic("Processing");
+    oc.addOptionSubTopic("Building Defaults");
     oc.addOptionSubTopic("TLS Building");
     oc.addOptionSubTopic("Ramp Guessing");
     oc.addOptionSubTopic("Edge Removal");
     oc.addOptionSubTopic("Unregulated Nodes");
-    oc.addOptionSubTopic("Processing");
-    oc.addOptionSubTopic("Building Defaults");
-    SystemFrame::addReportOptions(oc); // this subtopic is filled here, too
+    oc.addOptionSubTopic("Junctions");
+    oc.addOptionSubTopic("Pedestrian");
+    oc.addOptionSubTopic("Bicycle");
+    oc.addOptionSubTopic("Railway");
+    oc.addOptionSubTopic("Formats");
 
     NIFrame::fillOptions();
     NBFrame::fillOptions(false);
@@ -92,6 +83,7 @@ checkOptions() {
     bool ok = NIFrame::checkOptions();
     ok &= NBFrame::checkOptions();
     ok &= NWFrame::checkOptions();
+    ok &= SystemFrame::checkOptions();
     return ok;
 }
 
@@ -103,18 +95,22 @@ int
 main(int argc, char** argv) {
     OptionsCont& oc = OptionsCont::getOptions();
     // give some application descriptions
-    oc.setApplicationDescription("Road network importer / builder for the road traffic simulation SUMO.");
-    oc.setApplicationName("netconvert", "SUMO netconvert Version " + getBuildName(VERSION_STRING));
+    oc.setApplicationDescription("Network importer / builder for the microscopic, multi-modal traffic simulation SUMO.");
+    oc.setApplicationName("netconvert", "Eclipse SUMO netconvert Version " VERSION_STRING);
     int ret = 0;
     try {
         XMLSubSys::init();
         fillOptions();
-        OptionsIO::getOptions(true, argc, argv);
+        OptionsIO::setArgs(argc, argv);
+        OptionsIO::getOptions();
         if (oc.processMetaOptions(argc < 2)) {
             SystemFrame::close();
             return 0;
         }
-        XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"));
+        XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"), "never");
+        if (oc.isDefault("aggregate-warnings")) {
+            oc.set("aggregate-warnings", "5");
+        }
         MsgHandler::initOutputOptions();
         if (!checkOptions()) {
             throw ProcessError();
@@ -129,16 +125,23 @@ main(int argc, char** argv) {
         // load data
         NILoader nl(nb);
         nl.load(oc);
-        if (oc.getBool("ignore-errors")) {
-            MsgHandler::getErrorInstance()->clear();
-        }
-        // check whether any errors occured
+        // flush aggregated errors and optionally ignore them
+        MsgHandler::getErrorInstance()->clear(oc.getBool("ignore-errors"));
+        // check whether any errors occurred
         if (MsgHandler::getErrorInstance()->wasInformed()) {
             throw ProcessError();
         }
         nb.compute(oc);
+        // check whether any errors occurred
+        if (MsgHandler::getErrorInstance()->wasInformed()) {
+            throw ProcessError();
+        }
+        // report
+        nb.getNodeCont().printBuiltNodesStatistics();
         NWFrame::writeNetwork(oc, nb);
     } catch (const ProcessError& e) {
+        MsgHandler::getWarningInstance()->clear(false);
+        MsgHandler::getErrorInstance()->clear(false);
         if (std::string(e.what()) != std::string("Process Error") && std::string(e.what()) != std::string("")) {
             WRITE_ERROR(e.what());
         }
@@ -146,17 +149,21 @@ main(int argc, char** argv) {
         ret = 1;
 #ifndef _DEBUG
     } catch (const std::exception& e) {
+        MsgHandler::getWarningInstance()->clear(false);
+        MsgHandler::getErrorInstance()->clear(false);
         if (std::string(e.what()) != std::string("")) {
             WRITE_ERROR(e.what());
         }
         MsgHandler::getErrorInstance()->inform("Quitting (on error).", false);
         ret = 1;
     } catch (...) {
+        MsgHandler::getWarningInstance()->clear(false);
+        MsgHandler::getErrorInstance()->clear(false);
         MsgHandler::getErrorInstance()->inform("Quitting (on unknown error).", false);
         ret = 1;
 #endif
     }
-    NBDistribution::clear();
+    DistributionCont::clear();
     SystemFrame::close();
     // report about ending
     if (ret == 0) {
@@ -166,6 +173,4 @@ main(int argc, char** argv) {
 }
 
 
-
 /****************************************************************************/
-
