@@ -1,47 +1,35 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GNELoadThread.h
 /// @author  Jakob Erdmann
 /// @date    Feb 2011
-/// @version $Id: GNELoadThread.h 20975 2016-06-15 13:02:40Z palcraft $
 ///
 // The thread that performs the loading of a Netedit-net (adapted from
 // GUILoadThread)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GNELoadThread_h
-#define GNELoadThread_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <utils/common/SUMOTime.h>
-#include <utils/foxtools/FXSingleEventThread.h>
-#include <utils/foxtools/FXThreadEvent.h>
-#include <utils/foxtools/MFXEventQue.h>
 #include <utils/common/MsgHandler.h>
+#include <utils/foxtools/FXSingleEventThread.h>
+#include <utils/foxtools/FXSynchQue.h>
+#include <utils/foxtools/MFXInterThreadEventClient.h>
 
 
 // ===========================================================================
 // class declarations
 // ===========================================================================
-class MFXInterThreadEventClient;
 class GNENet;
 class GUIEvent;
 
@@ -55,7 +43,7 @@ class GUIEvent;
 class GNELoadThread : public FXSingleEventThread {
 public:
     /// @brief constructor
-    GNELoadThread(FXApp* app, MFXInterThreadEventClient* mw, MFXEventQue<GUIEvent*>& eq,
+    GNELoadThread(FXApp* app, MFXInterThreadEventClient* mw, FXSynchQue<GUIEvent*>& eq,
                   FXEX::FXThreadEvent& ev);
 
     /// @brief destructor
@@ -67,9 +55,9 @@ public:
     /**@brief begins the loading of a netconvert configuration or a a network
      * @param[in] file The network or configuration-file to be loaded
      * @param[in] isNet whether file is a network file
-     * @param[in] optionsReady whether options are already initialized
+     * @param[in] useStartupOptions whether the initial startup options shall be used
      */
-    void loadConfigOrNet(const std::string& file, bool isNet, bool optionsReady = false, bool newNet = false);
+    void loadConfigOrNet(const std::string& file, bool isNet, bool useStartupOptions, bool newNet = false);
 
     /// @brief Retrieves messages from the loading module
     void retrieveMessage(const MsgHandler::MsgType type, const std::string& msg);
@@ -95,14 +83,14 @@ protected:
     /// @brief the parent window to inform about the loading
     MFXInterThreadEventClient* myParent;
 
-    /// @brief the path to load the simulation from
+    /// @brief the path to load the network from
     std::string myFile;
 
     /// @brief @brief The instances of message retriever encapsulations Needed to be deleted from the handler later on
-    OutputDevice* myErrorRetriever, *myMessageRetriever, *myWarningRetriever;
+    OutputDevice* myErrorRetriever, *myMessageRetriever, *myWarningRetriever, *myDebugRetriever, *myGLDebugRetriever;
 
     /// @brief event Queue
-    MFXEventQue<GUIEvent*>& myEventQue;
+    FXSynchQue<GUIEvent*>& myEventQue;
 
     /// @brief event throw
     FXEX::FXThreadEvent& myEventThrow;
@@ -110,14 +98,6 @@ protected:
     /// @brief Information whether only the network shall be loaded
     bool myLoadNet;
 
-    /// @brief if true, options will not be read from myFile
-    bool myOptionsReady;
-
     /// @brief if true, a new net is created
     bool myNewNet;
 };
-
-
-#endif
-
-/****************************************************************************/
