@@ -1,8 +1,19 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GNEUndoList.h
 /// @author  Jakob Erdmann
 /// @date    Mar 2011
-/// @version $Id: GNEUndoList.h 21131 2016-07-08 07:59:22Z behrisch $
 ///
 // FXUndoList is pretty dandy but some features are missing:
 //   - we cannot find out wether we have currently begun an undo-group and
@@ -12,32 +23,12 @@
 // GNEUndoList inherits from FXUndoList and patches some methods. these are
 // prefixed with p_
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GNEUndoList_h
-#define GNEUndoList_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <stack>
-#include <fx.h>
+#include <string>
+#include <utils/foxtools/fxheader.h>
 
 
 // ===========================================================================
@@ -81,6 +72,9 @@ public:
     /// @brief reverts and discards ALL active command groups
     void p_abort();
 
+    /// @brief reverts last command group
+    void p_abortLastCommandGroup();
+
     /// @brief undo the last command group
     void undo();
 
@@ -90,6 +84,9 @@ public:
     /// @brief special method, avoid empty changes, always execute
     void p_add(GNEChange_Attribute* cmd);
 
+    /// @brief get size of current CommandGroup
+    int currentCommandGroupSize() const;
+
     /// @name FOX-callbacks
     /// @{
     /// @brief event after Undo
@@ -123,15 +120,9 @@ private:
         const std::string myDescription;
     };
 
-    // the stack of currently active command groups
+    // @brief the stack of currently active command groups
     std::stack<CommandGroup*> myCommandGroups;
 
-    // the parent application for this undolist
-    GNEApplicationWindow* const myParent;
+    // @brief the parent GNEApplicationWindow for this undolist
+    GNEApplicationWindow* const myGNEApplicationWindowParent;
 };
-
-
-#endif
-
-/****************************************************************************/
-
