@@ -1,56 +1,36 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    netedit_main.cpp
 /// @author  Jakob Erdmann
 /// @date    Feb 2011
-/// @version $Id: netedit_main.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Main for NETEDIT (adapted from guisim_main)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #ifdef HAVE_VERSION_H
 #include <version.h>
 #endif
 
-#include <ctime>
 #include <signal.h>
-#include <iostream>
-#include <utils/options/Option.h>
+#include <utils/common/SystemFrame.h>
+#include <utils/foxtools/MsgHandlerSynchronized.h>
+#include <utils/gui/settings/GUICompleteSchemeStorage.h>
+#include <utils/gui/windows/GUIAppEnum.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/options/OptionsIO.h>
-#include <utils/common/UtilExceptions.h>
-#include <utils/common/FileHelpers.h>
-#include <utils/common/MsgHandler.h>
-#include <utils/common/SystemFrame.h>
-#include <utils/common/RandHelper.h>
 #include <utils/xml/XMLSubSys.h>
-#include <utils/gui/windows/GUIAppEnum.h>
-#include <utils/gui/images/GUITexturesHelper.h>
-#include <utils/gui/settings/GUICompleteSchemeStorage.h>
-#include <utils/geom/GeoConvHelper.h>
-#include <netimport/NIFrame.h>
-#include <netbuild/NBFrame.h>
-#include <netwrite/NWFrame.h>
 
 #include "GNEApplicationWindow.h"
 #include "GNELoadThread.h"
@@ -62,13 +42,12 @@
 int
 main(int argc, char** argv) {
     // make the output aware of threading
-    MFXMutex lock;
-    MsgHandler::assignLock(&lock);
+    MsgHandler::setFactory(&MsgHandlerSynchronized::create);
     // get the options
     OptionsCont& oc = OptionsCont::getOptions();
     // give some application descriptions
     oc.setApplicationDescription("Graphical editor for SUMO networks.");
-    oc.setApplicationName("netedit", "Netedit Version " VERSION_STRING);
+    oc.setApplicationName("netedit", "Eclipse SUMO netedit Version " VERSION_STRING);
     int ret = 0;
 #ifndef _DEBUG
     try {
@@ -85,7 +64,7 @@ main(int argc, char** argv) {
             return 0;
         }
         // Make application
-        FXApp application("Netedit", "DLR");
+        FXApp application("SUMO netedit", "netedit");
         // Open display
         application.init(argc, argv);
         int minor, major;
@@ -95,10 +74,10 @@ main(int argc, char** argv) {
         // build the main window
         GNEApplicationWindow* window =
         new GNEApplicationWindow(&application, "*.netc.cfg,*.netccfg");
-        gSchemeStorage.init(&application);
+        gSchemeStorage.init(&application, true);
         window->dependentBuild();
         // Create app
-        application.addSignal(SIGINT, window, MID_QUIT);
+        application.addSignal(SIGINT, window, MID_HOTKEY_CTRL_Q_CLOSE);
         application.create();
         // Load configuration given on command line
         if (argc > 1) {
@@ -107,6 +86,8 @@ main(int argc, char** argv) {
             // load options
             window->loadOptionOnStartup();
         }
+        // focus window at startup
+        window->setFocus();
         // Run
         ret = application.run();
 #ifndef _DEBUG
@@ -126,6 +107,4 @@ main(int argc, char** argv) {
 }
 
 
-
 /****************************************************************************/
-
