@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2011-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NGFrame.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    06.05.2011
-/// @version $Id: NGFrame.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Sets and checks options for netgen
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include "NGFrame.h"
@@ -39,10 +30,6 @@
 #include <utils/common/SystemFrame.h>
 #include <utils/common/ToString.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -50,6 +37,26 @@
 void
 NGFrame::fillOptions() {
     OptionsCont& oc = OptionsCont::getOptions();
+    oc.doRegister("type-files", 't', new Option_FileName());
+    oc.addDescription("type-files", "Input", "Read edge-type defs from FILE");
+
+    oc.doRegister("alphanumerical-ids", new Option_Bool(true));
+    oc.addDescription("alphanumerical-ids", "Output", "The Ids of generated nodes use an alphanumerical code for easier readability when possible");
+
+    // register processing options
+    oc.doRegister("turn-lanes", new Option_Integer(0));
+    oc.addDescription("turn-lanes", "Processing", "Generate INT left-turn lanes");
+    oc.doRegister("turn-lanes.length", new Option_Float(20));
+    oc.addDescription("turn-lanes.length", "Processing", "Set the length of generated turning lanes to FLOAT");
+
+    oc.doRegister("perturb-x", new Option_String("0"));
+    oc.addDescription("perturb-x", "Processing", "Apply random spatial perturbation in x direction according the the given distribution");
+    oc.doRegister("perturb-y", new Option_String("0"));
+    oc.addDescription("perturb-y", "Processing", "Apply random spatial perturbation in y direction according the the given distribution");
+    oc.doRegister("perturb-z", new Option_String("0"));
+    oc.addDescription("perturb-z", "Processing", "Apply random spatial perturbation in z direction according the the given distribution");
+
+
     //  register grid-net options
     oc.doRegister("grid", 'g', new Option_Bool(false));
     oc.addSynonyme("grid", "grid-net", true);
@@ -89,7 +96,6 @@ NGFrame::fillOptions() {
     oc.addSynonyme("grid.attach-length", "attach-length", true);
     oc.addDescription("grid.attach-length", "Grid Network", "The length of streets attached at the boundary; 0 means no streets are attached");
 
-
     //  register spider-net options
     oc.doRegister("spider", 's', new Option_Bool(false));
     oc.addSynonyme("spider", "spider-net", true);
@@ -141,17 +147,17 @@ NGFrame::fillOptions() {
     oc.addSynonyme("rand.min-distance", "min-dist");
     oc.addDescription("rand.min-distance", "Random Network", "The minimum distance for each edge");
 
-    oc.doRegister("rand.min-angle", new Option_Float(DEG2RAD(45.0)));
+    oc.doRegister("rand.min-angle", new Option_Float(45.0));
     oc.addSynonyme("rand.min-angle", "rand-min-anglee", true);
     oc.addSynonyme("rand.min-angle", "min-angle");
-    oc.addDescription("rand.min-angle", "Random Network", "The minimum angle for each pair of (bidirectional) roads");
+    oc.addDescription("rand.min-angle", "Random Network", "The minimum angle for each pair of (bidirectional) roads in DEGREES");
 
     oc.doRegister("rand.num-tries", new Option_Integer(50));
     oc.addSynonyme("rand.num-tries", "rand-num-tries", true);
     oc.addSynonyme("rand.num-tries", "num-tries");
     oc.addDescription("rand.num-tries", "Random Network", "The number of tries for creating each node");
 
-    oc.doRegister("rand.connectivity", new Option_Float((SUMOReal) 0.95));
+    oc.doRegister("rand.connectivity", new Option_Float((double) 0.95));
     oc.addSynonyme("rand.connectivity", "rand-connectivity", true);
     oc.addSynonyme("rand.connectivity", "connectivity");
     oc.addDescription("rand.connectivity", "Random Network", "Probability for roads to continue at each node");
@@ -185,6 +191,15 @@ NGFrame::fillOptions() {
     oc.addSynonyme("rand.neighbor-dist6", "rand-neighbor-dist6", true);
     oc.addSynonyme("rand.neighbor-dist6", "dist6");
     oc.addDescription("rand.neighbor-dist6", "Random Network", "Probability for a node having exactly 6 neighbors");
+
+    oc.doRegister("rand.random-lanenumber", new Option_Bool(false));
+    oc.addDescription("rand.random-lanenumber", "Random Network", "Draw lane numbers randomly from [1,default.lanenumber]");
+
+    oc.doRegister("rand.random-priority", new Option_Bool(false));
+    oc.addDescription("rand.random-priority", "Random Network", "Draw edge priority randomly from [1,default.priority]");
+
+    oc.doRegister("rand.grid", new Option_Bool(false));
+    oc.addDescription("rand.grid", "Random Network", "Place nodes on a regular grid with spacing rand.min-distance");
 }
 
 
@@ -214,29 +229,29 @@ NGFrame::checkOptions() {
     // check whether the junction type to use is properly set
     if (oc.isSet("default-junction-type")) {
         std::string type = oc.getString("default-junction-type");
-        if (type != toString(NODETYPE_TRAFFIC_LIGHT) &&
-                type != toString(NODETYPE_TRAFFIC_LIGHT_NOJUNCTION) &&
-                type != toString(NODETYPE_TRAFFIC_LIGHT_RIGHT_ON_RED) &&
-                type != toString(NODETYPE_PRIORITY) &&
-                type != toString(NODETYPE_PRIORITY_STOP) &&
-                type != toString(NODETYPE_ALLWAY_STOP) &&
-                type != toString(NODETYPE_ZIPPER) &&
-                type != toString(NODETYPE_NOJUNCTION) &&
-                type != toString(NODETYPE_RAIL_SIGNAL) &&
-                type != toString(NODETYPE_RAIL_CROSSING) &&
-                type != toString(NODETYPE_RIGHT_BEFORE_LEFT)) {
+        if (type != toString(SumoXMLNodeType::TRAFFIC_LIGHT) &&
+                type != toString(SumoXMLNodeType::TRAFFIC_LIGHT_NOJUNCTION) &&
+                type != toString(SumoXMLNodeType::TRAFFIC_LIGHT_RIGHT_ON_RED) &&
+                type != toString(SumoXMLNodeType::PRIORITY) &&
+                type != toString(SumoXMLNodeType::PRIORITY_STOP) &&
+                type != toString(SumoXMLNodeType::ALLWAY_STOP) &&
+                type != toString(SumoXMLNodeType::ZIPPER) &&
+                type != toString(SumoXMLNodeType::NOJUNCTION) &&
+                type != toString(SumoXMLNodeType::RAIL_SIGNAL) &&
+                type != toString(SumoXMLNodeType::RAIL_CROSSING) &&
+                type != toString(SumoXMLNodeType::RIGHT_BEFORE_LEFT)) {
             WRITE_ERROR("Only the following junction types are known: " +
-                        toString(NODETYPE_TRAFFIC_LIGHT) + ", " +
-                        toString(NODETYPE_TRAFFIC_LIGHT_NOJUNCTION) + ", " +
-                        toString(NODETYPE_TRAFFIC_LIGHT_RIGHT_ON_RED) + ", " +
-                        toString(NODETYPE_PRIORITY) + ", " +
-                        toString(NODETYPE_PRIORITY_STOP) + ", " +
-                        toString(NODETYPE_ALLWAY_STOP) + ", " +
-                        toString(NODETYPE_ZIPPER) + ", " +
-                        toString(NODETYPE_NOJUNCTION) + ", " +
-                        toString(NODETYPE_RAIL_SIGNAL) + ", " +
-                        toString(NODETYPE_RAIL_CROSSING) + ", " +
-                        toString(NODETYPE_RIGHT_BEFORE_LEFT));
+                        toString(SumoXMLNodeType::TRAFFIC_LIGHT) + ", " +
+                        toString(SumoXMLNodeType::TRAFFIC_LIGHT_NOJUNCTION) + ", " +
+                        toString(SumoXMLNodeType::TRAFFIC_LIGHT_RIGHT_ON_RED) + ", " +
+                        toString(SumoXMLNodeType::PRIORITY) + ", " +
+                        toString(SumoXMLNodeType::PRIORITY_STOP) + ", " +
+                        toString(SumoXMLNodeType::ALLWAY_STOP) + ", " +
+                        toString(SumoXMLNodeType::ZIPPER) + ", " +
+                        toString(SumoXMLNodeType::NOJUNCTION) + ", " +
+                        toString(SumoXMLNodeType::RAIL_SIGNAL) + ", " +
+                        toString(SumoXMLNodeType::RAIL_CROSSING) + ", " +
+                        toString(SumoXMLNodeType::RIGHT_BEFORE_LEFT));
             ok = false;
         }
     }
@@ -245,4 +260,3 @@ NGFrame::checkOptions() {
 
 
 /****************************************************************************/
-
