@@ -1,37 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NGNet.h
 /// @author  Markus Hartinger
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mar, 2003
-/// @version $Id: NGNet.h 20507 2016-04-20 11:40:48Z namdre $
 ///
 // The class storing the generated network
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NGNet_h
-#define NGNet_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
+#include <utils/distribution/Distribution_Parameterized.h>
 #include "NGEdge.h"
 #include "NGNode.h"
 
@@ -91,7 +82,7 @@ public:
      * @param[in] phi The angle the position is located at
      * @return The x-position at the described circle at angle phi
      */
-    SUMOReal radialToX(SUMOReal radius, SUMOReal phi);
+    double radialToX(double radius, double phi);
 
 
     /** @brief Returns the y-position resulting from the given radius and angle
@@ -100,7 +91,7 @@ public:
      * @param[in] phi The angle the position is located at
      * @return The y-position at the described circle at angle phi
      */
-    SUMOReal radialToY(SUMOReal radius, SUMOReal phi);
+    double radialToY(double radius, double phi);
 
 
     /** @brief Creates a grid network
@@ -117,10 +108,9 @@ public:
      * @param[in] spaceX The space between nodes in x-direction
      * @param[in] spaceY The space between nodes in y-direction
      * @param[in] attachLength The length of streets attached at the border
-     * @param[in] alphaIDs Whether to use alphanumerial node ids
      * @see NGNet::connect
      */
-    void createChequerBoard(int numX, int numY, SUMOReal spaceX, SUMOReal spaceY, SUMOReal attachLength, bool alphaIDs);
+    void createChequerBoard(int numX, int numY, double spaceX, double spaceY, double attachLength);
 
 
     /** @brief Creates a spider network
@@ -140,7 +130,7 @@ public:
      * @see NGNet::connect
      * @todo consolidate the name of the center node
      */
-    void createSpiderWeb(int numRadDiv, int numCircles, SUMOReal spaceRad, bool hasCenter);
+    void createSpiderWeb(int numRadDiv, int numCircles, double spaceRad, bool hasCenter);
 
 
     /** @brief Converts the stored network into its netbuilder-representation
@@ -185,7 +175,7 @@ public:
      *
      * @return The number of stored nodes
      */
-    size_t nodeNo() const;
+    int nodeNo() const;
 
 
 private:
@@ -199,11 +189,19 @@ private:
      */
     void connect(NGNode* node1, NGNode* node2);
 
+    /// @brief return a letter code for the given integer index
+    std::string alphabeticalCode(int i, int iMax);
+
+    /// @brief get distribution from option
+    static Distribution_Parameterized getDistribution(const std::string& option);
 
 private:
     /// @brief The last ID given to node or link
     int myLastID;
 
+    /// @brief Whether to use alphanumericalIDs
+    const bool myAlphaIDs;
+
     /// @brief The builder used to build NB*-structures
     NBNetBuilder& myNetBuilder;
 
@@ -221,9 +219,3 @@ private:
     NGNet& operator=(const NGNet&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
