@@ -1,35 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NGNode.cpp
 /// @author  Markus Hartinger
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mar, 2003
-/// @version $Id: NGNode.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // A netgen-representation of a node
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <algorithm>
 #include <netbuild/NBNode.h>
@@ -46,56 +37,52 @@
 #include <utils/options/Option.h>
 #include "NGNode.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
 NGNode::NGNode(const std::string& id)
-    : Named(id), xID(-1), yID(-1), myAmCenter(false) {}
+    : Named(id), xID(-1), yID(-1), myAmCenter(false), myAmFringe(false) {}
 
 
 NGNode::NGNode(const std::string& id, int xIDa, int yIDa)
-    : Named(id), xID(xIDa), yID(yIDa), myAmCenter(false) {}
+    : Named(id), xID(xIDa), yID(yIDa), myAmCenter(false), myAmFringe(false) {}
 
 
 NGNode::NGNode(const std::string& id, int xIDa, int yIDa, bool amCenter)
-    : Named(id), xID(xIDa), yID(yIDa), myAmCenter(amCenter) {}
+    : Named(id), xID(xIDa), yID(yIDa), myAmCenter(amCenter), myAmFringe(false) {}
 
 
 NGNode::~NGNode() {
     NGEdgeList::iterator li;
     while (LinkList.size() != 0) {
         li = LinkList.begin();
-        delete(*li);
+        delete (*li);
     }
 }
 
 
 NBNode*
-NGNode::buildNBNode(NBNetBuilder& nb) const {
-    Position pos(myPosition);
+NGNode::buildNBNode(NBNetBuilder& nb, const Position& perturb) const {
+    Position pos(myPosition + perturb);
     GeoConvHelper::getProcessing().x2cartesian(pos);
     // the center will have no logic!
     if (myAmCenter) {
-        return new NBNode(myID, pos, NODETYPE_NOJUNCTION);
+        return new NBNode(myID, pos, SumoXMLNodeType::NOJUNCTION);
     }
-    NBNode* node = 0;
+    NBNode* node = nullptr;
     std::string typeS = OptionsCont::getOptions().isSet("default-junction-type") ?
                         OptionsCont::getOptions().getString("default-junction-type") : "";
 
     if (SUMOXMLDefinitions::NodeTypes.hasString(typeS)) {
-        SumoXMLNodeType type = SUMOXMLDefinitions::NodeTypes.get(typeS);
+        const SumoXMLNodeType type = SUMOXMLDefinitions::NodeTypes.get(typeS);
         node = new NBNode(myID, pos, type);
 
         // check whether it is a traffic light junction
-        if (type == NODETYPE_TRAFFIC_LIGHT || type == NODETYPE_TRAFFIC_LIGHT_NOJUNCTION) {
-            TrafficLightType type = SUMOXMLDefinitions::TrafficLightTypes.get(
-                                        OptionsCont::getOptions().getString("tls.default-type"));
-            NBTrafficLightDefinition* tlDef = new NBOwnTLDef(myID, node, 0, type);
+        if (NBNode::isTrafficLight(type)) {
+            const TrafficLightType tlType = SUMOXMLDefinitions::TrafficLightTypes.get(
+                                                OptionsCont::getOptions().getString("tls.default-type"));
+            NBTrafficLightDefinition* tlDef = new NBOwnTLDef(myID, node, 0, tlType);
             if (!nb.getTLLogicCont().insert(tlDef)) {
                 // actually, nothing should fail here
                 delete tlDef;
@@ -103,8 +90,11 @@ NGNode::buildNBNode(NBNetBuilder& nb) const {
             }
         }
     } else {
-        // otherwise netbuild may guess NODETYPE_TRAFFIC_LIGHT without actually building one
-        node = new NBNode(myID, pos, NODETYPE_PRIORITY);
+        // otherwise netbuild may guess SumoXMLNodeType::TRAFFIC_LIGHT without actually building one
+        node = new NBNode(myID, pos, SumoXMLNodeType::PRIORITY);
+    }
+    if (myAmFringe) {
+        node->setFringeType(FringeType::OUTER);
     }
 
     return node;
@@ -135,4 +125,3 @@ NGNode::connected(NGNode* node) const {
 
 
 /****************************************************************************/
-
