@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NGNode.h
 /// @author  Markus Hartinger
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mar, 2003
-/// @version $Id: NGNode.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A netgen-representation of a node
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NGNode_h
-#define NGNode_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <list>
 #include <utils/common/Named.h>
@@ -100,7 +90,7 @@ public:
      *
      * @return The maximum neighbour number of the node
      */
-    SUMOReal getMaxNeighbours() {
+    int getMaxNeighbours() {
         return myMaxNeighbours;
     }
 
@@ -109,7 +99,7 @@ public:
      *
      * @param[in] value The new maximum neighbour number of the node
      */
-    void setMaxNeighbours(SUMOReal value) {
+    void setMaxNeighbours(int value) {
         myMaxNeighbours = value;
     }
 
@@ -118,7 +108,7 @@ public:
      *
      * @param[in] value The new x-position of this node
      */
-    void setX(SUMOReal x) {
+    void setX(double x) {
         myPosition.set(x, myPosition.y());
     }
 
@@ -127,16 +117,20 @@ public:
      *
      * @param[in] value The new y-position of this node
      */
-    void setY(SUMOReal y) {
+    void setY(double y) {
         myPosition.set(myPosition.x(), y);
     }
 
+    /// @brief mark node as fringe
+    void setFringe() {
+        myAmFringe = true;
+    }
 
     /** @brief Builds and returns this node's netbuild-representation
      *
      * The position of the node is transformed to cartesian using GeoConvHelper::x2cartesian,
      *  first. If this node is the center node of a spider net, a node of the type
-     *  NBNode::NODETYPE_NOJUNCTION is returned.
+     *  NBNode::SumoXMLNodeType::NOJUNCTION is returned.
      * Otherwise, a plain node is built and it is checked whether the options
      *  indicate building one of the tls node-types. In this case, a logic is built and
      *  stored. A ProcessError is thrown if this fails (should never happen, in fact).
@@ -147,7 +141,7 @@ public:
      * @todo There is no interaction with explicit node setting options? Where is this done?
      * @todo Check whether throwing an exception is really necessary, here
      */
-    NBNode* buildNBNode(NBNetBuilder& nb) const;
+    NBNode* buildNBNode(NBNetBuilder& nb, const Position& perturb) const;
 
 
     /** @brief Adds the given link to the internal list
@@ -201,11 +195,14 @@ private:
     Position myPosition;
 
     /// @brief The maximum number of neighbours
-    SUMOReal myMaxNeighbours;
+    int myMaxNeighbours;
 
     /// @brief Information whether this is the center of a cpider-net
     bool myAmCenter;
 
+    /// @brief Information whether this is the center of a cpider-net
+    bool myAmFringe;
+
 };
 
 /**
@@ -213,10 +210,3 @@ private:
  * @brief A list of nodes (node pointers)
  */
 typedef std::list<NGNode*> NGNodeList;
-
-
-
-#endif
-
-/****************************************************************************/
-
