@@ -1,76 +1,35 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NGRandomNetBuilder.h
 /// @author  Markus Hartinger
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mar, 2003
-/// @version $Id: NGRandomNetBuilder.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Additional structures for building random nets
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NGRandomNetBuilder_h
-#define NGRandomNetBuilder_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include "NGNet.h"
 #include <map>
+#include <utils/distribution/RandomDistributor.h>
+#include "NGNet.h"
 
 
 // ===========================================================================
 // class definitions
 // ===========================================================================
-/**
- * @class TNeighbourDistribution
- * @todo Check whether this can be replaced by RandomDistributor
- */
-class TNeighbourDistribution {
-public:
-    /**
-     * @brief adds a neighbour item to list
-     *
-     * If NumNeighbours is already existing, the old ratio is overwritten
-     *
-     * @param[in] numNeighbours The number of neighbors this item shall describe
-     * @param[in] ratio The probability of this number of neighbors
-     */
-    void add(int numNeighbours, SUMOReal ratio);
-
-
-    /**
-     * @brief Get random number of neighbours
-     *
-     * @return A random number of neighbors
-     */
-    int num();
-
-
-private:
-    /// @brief A map from neighbor number to their probabilities
-    std::map<int, SUMOReal> myNeighbours;
-
-};
-
-
 /**
  * @class NGRandomNetBuilder
  * @brief A class that builds random network using an algorithm by Markus Hartinger.
@@ -89,8 +48,8 @@ public:
      * @param[in] numTries ?
      * @todo check meanings of connectivity/numTries
      */
-    NGRandomNetBuilder(NGNet& net, SUMOReal minAngle, SUMOReal minDistance, SUMOReal maxDistance, SUMOReal connectivity,
-                       int numTries, const TNeighbourDistribution& neighborDist);
+    NGRandomNetBuilder(NGNet& net, double minAngle, double minDistance, double maxDistance, double connectivity,
+                       int numTries, const RandomDistributor<int>& neighborDist);
 
 
 
@@ -99,7 +58,7 @@ public:
      * @param[in] numNodes Number of iterations (node insertions) to perform
      * @todo Describe the algorithm
      */
-    void createNet(int numNodes);
+    void createNet(int numNodes, bool gridMode);
 
 
 private:
@@ -139,7 +98,7 @@ private:
      * @param[in] baseNode ?
      * @todo Describe better
      */
-    bool createNewNode(NGNode* baseNode);
+    bool createNewNode(NGNode* baseNode, bool gridMode);
 
 
     /** @brief finds possible connections between Node and OuterNodes complying with restrictions
@@ -168,16 +127,16 @@ private:
     //@{
 
     /// @brief Minimum angle allowed between two links
-    SUMOReal myMinLinkAngle;
+    double myMinLinkAngle;
 
     /// @brief Minimum distance allowed between two nodes
-    SUMOReal myMinDistance;
+    double myMinDistance;
 
     /// @brief Maximum distance allowed between two nodes
-    SUMOReal myMaxDistance;
+    double myMaxDistance;
 
     /// @brief Probability of connecting to a existing node if possible
-    SUMOReal myConnectivity;
+    double myConnectivity;
     //@}
 
 
@@ -187,8 +146,8 @@ private:
     /// @brief Number of nodes to be created
     int myNumNodes;
 
-    /// @brief The distrubtion of number of neighbours
-    TNeighbourDistribution myNeighbourDistribution;
+    /// @brief The distribution of number of neighbours
+    RandomDistributor<int> myNeighbourDistribution;
 
 private:
     /// @brief Invalidated copy constructor.
@@ -198,9 +157,3 @@ private:
     NGRandomNetBuilder& operator=(const NGRandomNetBuilder&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
