@@ -1,35 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIFrame.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Gregor Laemmel
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NIFrame.cpp 23561 2017-03-20 15:03:10Z laemmel $
 ///
 // Sets and checks options for netimport
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <iostream>
@@ -53,11 +44,12 @@
 // method definitions
 // ===========================================================================
 void
-NIFrame::fillOptions() {
+NIFrame::fillOptions(bool forNetedit) {
     OptionsCont& oc = OptionsCont::getOptions();
     // register input formats
     oc.doRegister("sumo-net-file", 's', new Option_FileName());
     oc.addSynonyme("sumo-net-file", "sumo-net", true);
+    oc.addSynonyme("sumo-net-file", "net-file");
     oc.addDescription("sumo-net-file", "Input", "Read SUMO-net from FILE");
     oc.addXMLDefault("sumo-net-file", "net");
 
@@ -88,6 +80,16 @@ NIFrame::fillOptions() {
     oc.addSynonyme("type-files", "types");
     oc.addDescription("type-files", "Input", "Read XML-type defs from FILE");
 
+    if (!forNetedit) {
+        // would cause confusion because netedit loads stops and shapes using option --additional-files
+        oc.doRegister("ptstop-files", new Option_FileName());
+        oc.addDescription("ptstop-files", "Input", "Reads public transport stops from FILE");
+        oc.doRegister("ptline-files", new Option_FileName());
+        oc.addDescription("ptline-files", "Input", "Reads public transport lines from FILE");
+        oc.doRegister("polygon-files", new Option_FileName());
+        oc.addDescription("polygon-files", "Input", "Reads polygons from FILE for embedding in network where applicable");
+    }
+
     oc.doRegister("shapefile-prefix", new Option_FileName());
     oc.addSynonyme("shapefile-prefix", "shapefile");
     oc.addSynonyme("shapefile-prefix", "arcview", true);
@@ -134,23 +136,22 @@ NIFrame::fillOptions() {
     oc.doRegister("heightmap.geotiff", new Option_FileName());
     oc.addDescription("heightmap.geotiff", "Input", "Read heightmap from GeoTIFF");
 
+    // need to do this here to be able to check for network and route input options
+    SystemFrame::addReportOptions(oc);
+
     // register basic processing options
     oc.doRegister("ignore-errors", new Option_Bool(false));
     oc.addSynonyme("ignore-errors", "dismiss-loading-errors", true);
-    oc.addDescription("ignore-errors", "Processing", "Continue on broken input");
+    oc.addDescription("ignore-errors", "Report", "Continue on broken input");
 
     oc.doRegister("ignore-errors.connections", new Option_Bool(false));
-    oc.addDescription("ignore-errors.connections", "Processing", "Continue on invalid connections");
+    oc.addDescription("ignore-errors.connections", "Report", "Continue on invalid connections");
 
     oc.doRegister("show-errors.connections-first-try", new Option_Bool(false));
-    oc.addDescription("show-errors.connections-first-try", "Processing", "Show errors in connections at parsing");
+    oc.addDescription("show-errors.connections-first-try", "Report", "Show errors in connections at parsing");
 
     oc.doRegister("ignore-errors.edge-type", new Option_Bool(false));
-    oc.addDescription("ignore-errors.edge-type", "Processing", "Continue on unknown edge types");
-
-    oc.doRegister("lanes-from-capacity.norm", new Option_Float((double) 1800));
-    oc.addSynonyme("lanes-from-capacity.norm", "capacity-norm");
-    oc.addDescription("lanes-from-capacity.norm", "Processing", "The factor for flow to no. lanes conversion");
+    oc.addDescription("ignore-errors.edge-type", "Report", "Continue on unknown edge types");
 
     oc.doRegister("speed-in-kmh", new Option_Bool(false));
     oc.addDescription("speed-in-kmh", "Processing", "vmax is parsed as given in km/h (some)");
@@ -158,130 +159,195 @@ NIFrame::fillOptions() {
     oc.doRegister("construction-date", new Option_String());
     oc.addDescription("construction-date", "Processing", "Use YYYY-MM-DD date to determine the readiness of features under construction");
 
+    oc.doRegister("flatten", new Option_Bool(false));
+    oc.addDescription("flatten", "Processing", "Remove all z-data");
+
+    oc.doRegister("discard-params", new Option_StringVector());
+    oc.addDescription("discard-params", "Formats", "Remove the list of keys from all params");
+
+    oc.doRegister("ignore-change-restrictions", new Option_StringVector(StringVector({"authority"})));
+    oc.addDescription("ignore-change-restrictions", "Formats", "List vehicle classes that may ignore lane changing restrictions ('all' discards all restrictions)");
 
     // register xml options
     oc.doRegister("plain.extend-edge-shape", new Option_Bool(false));
     oc.addSynonyme("plain.extend-edge-shape", "xml.keep-shape", true);
     oc.addDescription("plain.extend-edge-shape", "Processing", "If edge shapes do not end at the node positions, extend them");
 
+    // register osm options
+    oc.doRegister("osm.skip-duplicates-check", new Option_Bool(false));
+    oc.addDescription("osm.skip-duplicates-check", "Formats", "Skips the check for duplicate nodes and edges");
+
+    oc.doRegister("osm.elevation", new Option_Bool(false));
+    oc.addDescription("osm.elevation", "Formats", "Imports elevation data");
+
+    oc.doRegister("osm.layer-elevation", new Option_Float(0));
+    oc.addDescription("osm.layer-elevation", "Formats", "Reconstruct (relative) elevation based on layer data. Each layer is raised by FLOAT m");
+
+    oc.doRegister("osm.layer-elevation.max-grade", new Option_Float(10));
+    oc.addDescription("osm.layer-elevation.max-grade", "Formats", "Maximum grade threshold in % at 50km/h when reconstrucing elevation based on layer data. The value is scaled according to road speed.");
+
+    oc.doRegister("osm.oneway-spread-right", new Option_Bool(false));
+    oc.addDescription("osm.oneway-spread-right", "Formats", "Whether one-way roads should be spread to the side instead of centered");
+
+    oc.doRegister("osm.lane-access", new Option_Bool(false));
+    oc.addDescription("osm.lane-access", "Formats", "Import lane-specific access restrictions");
+
+    oc.doRegister("osm.stop-output.length", new Option_Float(25));
+    oc.addDescription("osm.stop-output.length", "Formats", "The default length of a public transport stop in FLOAT m");
+    oc.doRegister("osm.stop-output.length.bus", new Option_Float(15));
+    oc.addDescription("osm.stop-output.length.bus", "Formats", "The default length of a bus stop in FLOAT m");
+    oc.doRegister("osm.stop-output.length.tram", new Option_Float(25));
+    oc.addDescription("osm.stop-output.length.tram", "Formats", "The default length of a tram stop in FLOAT m");
+    oc.doRegister("osm.stop-output.length.train", new Option_Float(200));
+    oc.addDescription("osm.stop-output.length.train", "Formats", "The default length of a train stop in FLOAT m");
+
+    oc.doRegister("osm.all-attributes", new Option_Bool(false));
+    oc.addDescription("osm.all-attributes", "Formats", "Whether additional attributes shall be imported");
+
+    oc.doRegister("osm.extra-attributes", new Option_StringVector(StringVector({"bridge", "tunnel", "layer", "postal_code"})));
+    oc.addDescription("osm.extra-attributes", "Formats", "List of additional attributes that shall be imported from OSM via osm.all-attributes (set 'all' to import all)");
+
 
     // register matsim options
     oc.doRegister("matsim.keep-length", new Option_Bool(false));
-    oc.addDescription("matsim.keep-length", "Processing", "The edge lengths given in the MATSIM-file will be kept");
+    oc.addDescription("matsim.keep-length", "Formats", "The edge lengths given in the MATSIM-file will be kept");
 
     oc.doRegister("matsim.lanes-from-capacity", new Option_Bool(false));
-    oc.addDescription("matsim.lanes-from-capacity", "Processing", "The lane number will be computed from the capacity");
+    oc.addDescription("matsim.lanes-from-capacity", "Formats", "The lane number will be computed from the capacity");
 
 
     // register shapefile options
     oc.doRegister("shapefile.street-id", new Option_String());
     oc.addSynonyme("shapefile.street-id", "arcview.street-id", true);
-    oc.addDescription("shapefile.street-id", "Processing", "Read edge ids from column STR");
+    oc.addDescription("shapefile.street-id", "Formats", "Read edge ids from column STR");
 
     oc.doRegister("shapefile.from-id", new Option_String());
     oc.addSynonyme("shapefile.from-id", "arcview.from-id", true);
-    oc.addDescription("shapefile.from-id", "Processing", "Read from-node ids from column STR");
+    oc.addDescription("shapefile.from-id", "Formats", "Read from-node ids from column STR");
 
     oc.doRegister("shapefile.to-id", new Option_String());
     oc.addSynonyme("shapefile.to-id", "arcview.to-id", true);
-    oc.addDescription("shapefile.to-id", "Processing", "Read to-node ids from column STR");
+    oc.addDescription("shapefile.to-id", "Formats", "Read to-node ids from column STR");
 
     oc.doRegister("shapefile.type-id", new Option_String());
     oc.addSynonyme("shapefile.type-id", "arcview.type-id", true);
-    oc.addDescription("shapefile.type-id", "Processing", "Read type ids from column STR");
+    oc.addDescription("shapefile.type-id", "Formats", "Read type ids from column STR");
+
+    oc.doRegister("shapefile.laneNumber", new Option_String());
+    oc.addDescription("shapefile.laneNumber", "Formats", "Read lane number from column STR");
+
+    oc.doRegister("shapefile.speed", new Option_String());
+    oc.addDescription("shapefile.speed", "Formats", "Read speed from column STR");
+
+    oc.doRegister("shapefile.name", new Option_String());
+    oc.addDescription("shapefile.name", "Formats", "Read (non-unique) name from column STR");
+
+    oc.doRegister("shapefile.node-join-dist", new Option_Float(0));
+    oc.addDescription("shapefile.node-join-dist", "Formats", "Distance threshold for determining whether distinct shapes are connected (used when from-id and to-id are not available)");
+
+    oc.doRegister("shapefile.add-params", new Option_StringVector());
+    oc.addDescription("shapefile.add-params", "Formats", "Add the list of field names as edge params");
 
     oc.doRegister("shapefile.use-defaults-on-failure", new Option_Bool(false));
     oc.addSynonyme("shapefile.use-defaults-on-failure", "arcview.use-defaults-on-failure", true);
-    oc.addDescription("shapefile.use-defaults-on-failure", "Processing", "Uses edge type defaults on problems");
+    oc.addDescription("shapefile.use-defaults-on-failure", "Formats", "Uses edge type defaults on problems");
 
     oc.doRegister("shapefile.all-bidirectional", new Option_Bool(false));
     oc.addSynonyme("shapefile.all-bidirectional", "shapefile.all-bidi");
     oc.addSynonyme("shapefile.all-bidirectional", "arcview.all-bidi", true);
-    oc.addDescription("shapefile.all-bidirectional", "Processing", "Insert edges in both directions");
+    oc.addDescription("shapefile.all-bidirectional", "Formats", "Insert edges in both directions");
 
     oc.doRegister("shapefile.guess-projection", new Option_Bool(false));
     oc.addSynonyme("shapefile.guess-projection", "arcview.guess-projection", true);
-    oc.addDescription("shapefile.guess-projection", "Processing", "Guess the proper projection");
+    oc.addDescription("shapefile.guess-projection", "Formats", "Guess the proper projection");
+
+    oc.doRegister("shapefile.traditional-axis-mapping", new Option_Bool(false));
+    oc.addDescription("shapefile.traditional-axis-mapping", "Formats", "Use traditional axis order (lon, lat)");
+
+
+    // register dlr-navteq options
+    oc.doRegister("dlr-navteq.tolerant-permissions", new Option_Bool(false));
+    oc.addDescription("dlr-navteq.tolerant-permissions", "Formats", "Allow more vehicle classes by default");
+
+    oc.doRegister("dlr-navteq.keep-length", new Option_Bool(false));
+    oc.addDescription("dlr-navteq.keep-length", "Formats", "The edge lengths given in the DLR Navteq-file will be kept");
 
 
     // register vissim options
     oc.doRegister("vissim.join-distance", new Option_Float(5.0f));
     oc.addSynonyme("vissim.join-distance", "vissim.offset", true);
-    oc.addDescription("vissim.join-distance", "Processing", "Structure join offset");
+    oc.addDescription("vissim.join-distance", "Formats", "Structure join offset");
 
     oc.doRegister("vissim.default-speed", new Option_Float(50.0f));
-    oc.addDescription("vissim.default-speed", "Processing", "Use FLOAT as default speed");
+    oc.addDescription("vissim.default-speed", "Formats", "Use FLOAT as default speed");
 
     oc.doRegister("vissim.speed-norm", new Option_Float(1.0f));
-    oc.addDescription("vissim.speed-norm", "Processing", "Factor for edge velocity");
+    oc.addDescription("vissim.speed-norm", "Formats", "Factor for edge velocity");
 
     oc.doRegister("vissim.report-unset-speeds", new Option_Bool(false));
-    oc.addDescription("vissim.report-unset-speeds", "Processing", "Writes lanes without an explicit speed set");
+    oc.addDescription("vissim.report-unset-speeds", "Formats", "Writes lanes without an explicit speed set");
 
 
     // register visum options
+    oc.doRegister("visum.language-file", new Option_FileName());
+    oc.addDescription("visum.language-file", "Formats", "Load language mappings from FILE");
+
     oc.doRegister("visum.use-type-priority", new Option_Bool(false));
-    oc.addDescription("visum.use-type-priority", "Processing", "Uses priorities from types");
+    oc.addDescription("visum.use-type-priority", "Formats", "Uses priorities from types");
 
     oc.doRegister("visum.use-type-laneno", new Option_Bool(false));
-    oc.addDescription("visum.use-type-laneno", "Processing", "Uses lane numbers from types");
+    oc.addDescription("visum.use-type-laneno", "Formats", "Uses lane numbers from types");
 
     oc.doRegister("visum.use-type-speed", new Option_Bool(false));
-    oc.addDescription("visum.use-type-speed", "Processing", "Uses speeds from types");
+    oc.addDescription("visum.use-type-speed", "Formats", "Uses speeds from types");
 
     oc.doRegister("visum.connector-speeds", new Option_Float(100.));
-    oc.addDescription("visum.connector-speeds", "Processing", "Sets connector speed");
+    oc.addDescription("visum.connector-speeds", "Formats", "Sets connector speed");
 
     oc.doRegister("visum.connectors-lane-number", new Option_Integer(3));
     oc.addSynonyme("visum.connectors-lane-number", "visum.connector-laneno", true);
-    oc.addDescription("visum.connectors-lane-number", "Processing", "Sets connector lane number");
+    oc.addDescription("visum.connectors-lane-number", "Formats", "Sets connector lane number");
 
-    oc.doRegister("visum.no-connectors", new Option_Bool(false));
-    oc.addDescription("visum.no-connectors", "Processing", "Excludes connectors");
+    oc.doRegister("visum.no-connectors", new Option_Bool(true));
+    oc.addDescription("visum.no-connectors", "Formats", "Excludes connectors");
 
     oc.doRegister("visum.recompute-lane-number", new Option_Bool(false));
     oc.addSynonyme("visum.recompute-lane-number", "visum.recompute-laneno", true);
-    oc.addDescription("visum.recompute-lane-number", "Processing", "Computes the number of lanes from the edges' capacities");
+    oc.addDescription("visum.recompute-lane-number", "Formats", "Computes the number of lanes from the edges' capacities");
 
     oc.doRegister("visum.verbose-warnings", new Option_Bool(false));
-    oc.addDescription("visum.verbose-warnings", "Processing", "Prints all warnings, some of which are due to VISUM misbehaviour");
+    oc.addDescription("visum.verbose-warnings", "Formats", "Prints all warnings, some of which are due to VISUM misbehaviour");
 
+    oc.doRegister("visum.lanes-from-capacity.norm", new Option_Float(1800.));
+    oc.addSynonyme("visum.lanes-from-capacity.norm", "capacity-norm", true);
+    oc.addSynonyme("visum.lanes-from-capacity.norm", "lanes-from-capacity.norm");
+    oc.addDescription("visum.lanes-from-capacity.norm", "Formats", "The factor for flow to no. lanes conversion");
 
-    // register osm options
-    oc.doRegister("osm.skip-duplicates-check", new Option_Bool(false));
-    oc.addDescription("osm.skip-duplicates-check", "Processing", "Skips the check for duplicate nodes and edges");
-
-    oc.doRegister("osm.elevation", new Option_Bool(false));
-    oc.addDescription("osm.elevation", "Processing", "Imports elevation data");
-
-    oc.doRegister("osm.layer-elevation", new Option_Float(0));
-    oc.addDescription("osm.layer-elevation", "Processing", "Reconstruct (relative) elevation based on layer data. Each layer is raised by FLOAT m");
-
-    oc.doRegister("osm.layer-elevation.max-grade", new Option_Float(10));
-    oc.addDescription("osm.layer-elevation.max-grade", "Processing", "Maximum grade threshold in % at 50km/h when reconstrucing elevation based on layer data. The value is scaled according to road speed.");
-
-    oc.doRegister("osm.oneway-spread-right", new Option_Bool(false));
-    oc.addDescription("osm.oneway-spread-right", "Processing", "Whether one-way roads should be spread to the side instead of centered");
-
-    oc.doRegister("osm.stop-output.length", new Option_Float(100));
-    oc.addDescription("osm.stop-output.length", "Processing", "The default length of a bus/train stop in FLOAT m");
 
     // register opendrive options
     oc.doRegister("opendrive.import-all-lanes", new Option_Bool(false));
-    oc.addDescription("opendrive.import-all-lanes", "Processing", "Imports all lane types");
+    oc.addDescription("opendrive.import-all-lanes", "Formats", "Imports all lane types");
     oc.doRegister("opendrive.ignore-widths", new Option_Bool(false));
-    oc.addDescription("opendrive.ignore-widths", "Processing", "Whether lane widths shall be ignored.");
+    oc.addDescription("opendrive.ignore-widths", "Formats", "Whether lane widths shall be ignored.");
     oc.doRegister("opendrive.curve-resolution", new Option_Float(2.0));
-    oc.addDescription("opendrive.curve-resolution", "Processing", "The geometry resolution in m when importing curved geometries as line segments.");
-    oc.doRegister("opendrive.advance-stopline", new Option_Float(12.0));
-    oc.addDescription("opendrive.advance-stopline", "Processing", "Allow stop lines to be built beyond the start of the junction if the geometries allow so");
+    oc.addDescription("opendrive.curve-resolution", "Formats", "The geometry resolution in m when importing curved geometries as line segments.");
+    oc.doRegister("opendrive.advance-stopline", new Option_Float(0.0));
+    oc.addDescription("opendrive.advance-stopline", "Formats", "Allow stop lines to be built beyond the start of the junction if the geometries allow so");
+    oc.doRegister("opendrive.min-width", new Option_Float(1.8));
+    oc.addDescription("opendrive.min-width", "Formats", "The minimum lane width for determining start or end of variable-width lanes");
+    oc.doRegister("opendrive.internal-shapes", new Option_Bool(false));
+    oc.addDescription("opendrive.internal-shapes", "Formats", "Import internal lane shapes");
 
     // register some additional options
     oc.doRegister("tls.discard-loaded", new Option_Bool(false));
-    oc.addDescription("tls.discard-loaded", "TLS Building", "Does not instatiate traffic lights loaded from other formats than XML");
+    oc.addDescription("tls.discard-loaded", "TLS Building", "Does not instatiate traffic lights loaded from other formats than plain-XML");
 
     oc.doRegister("tls.discard-simple", new Option_Bool(false));
-    oc.addDescription("tls.discard-simple", "TLS Building", "Does not instatiate traffic lights at geometry-like nodes loaded from other formats than XML");
+    oc.addDescription("tls.discard-simple", "TLS Building", "Does not instatiate traffic lights at geometry-like nodes loaded from other formats than plain-XML");
+
+    // register railway options
+    oc.doRegister("railway.signals.discard", new Option_Bool(false));
+    oc.addDescription("railway.signals.discard", "Railway", "Discard all railway signal information loaded from other formats than plain-xml");
 }
 
 
@@ -291,7 +357,7 @@ NIFrame::checkOptions() {
     bool ok = oc.checkDependingSuboptions("shapefile", "shapefile.");
     ok &= oc.checkDependingSuboptions("visum-file", "visum.");
     ok &= oc.checkDependingSuboptions("vissim-file", "vissim.");
-#ifdef HAVE_PROJ
+#ifdef PROJ_API_FILE
     int numProjections = oc.getBool("simple-projection") + oc.getBool("proj.utm") + oc.getBool("proj.dhdn") + (oc.getString("proj").length() > 1);
     if ((oc.isSet("osm-files") || oc.isSet("dlr-navteq-prefix") || oc.isSet("shapefile-prefix")) && numProjections == 0) {
         if (oc.isDefault("proj")) {
@@ -299,7 +365,7 @@ NIFrame::checkOptions() {
         }
     }
     if (oc.isSet("dlr-navteq-prefix") && oc.isDefault("proj.scale")) {
-        oc.set("proj.scale", toString(NIImporter_DlrNavteq::GEO_SCALE));
+        oc.set("proj.scale", NIImporter_DlrNavteq::GEO_SCALE);
     }
 #else
     if ((oc.isSet("osm-files") || oc.isSet("dlr-navteq-prefix") || oc.isSet("shapefile-prefix")) && !oc.getBool("simple-projection")) {
@@ -316,10 +382,18 @@ NIFrame::checkOptions() {
             // changed default since we wish to preserve the network as far as possible
             oc.set("offset.disable-normalization", "true");
         }
+        if (oc.isWriteable("geometry.max-grade.fix")) {
+            // changed default since we wish to preserve the network as far as possible
+            oc.set("geometry.max-grade.fix", "false");
+        }
+        if (oc.isWriteable("geometry.min-radius.fix.railways")) {
+            // changed default since we wish to preserve the network as far as possible
+            oc.set("geometry.min-radius.fix.railways", "false");
+        }
     }
     if (!oc.isSet("type-files")) {
         const char* sumoPath = std::getenv("SUMO_HOME");
-        if (sumoPath == 0) {
+        if (sumoPath == nullptr) {
             WRITE_WARNING("Environment variable SUMO_HOME is not set, using built in type maps.");
         } else {
             const std::string path = sumoPath + std::string("/data/typemap/");
@@ -340,11 +414,16 @@ NIFrame::checkOptions() {
             // a better interpretation of imported geometries
             oc.set("rectangular-lane-cut", "true");
         }
+        if (oc.isDefault("geometry.max-grade.fix")) {
+            // a better interpretation of imported geometries
+            oc.set("geometry.max-grade.fix", "false");
+        }
+    }
+    if (!oc.isDefault("osm.extra-attributes") && oc.isDefault("osm.all-attributes")) {
+        oc.set("osm.all-attributes", "true");
     }
     return ok;
 }
 
 
-
 /****************************************************************************/
-
