@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIImporter_ArcView.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: NIImporter_ArcView.h 21182 2016-07-18 06:46:01Z behrisch $
 ///
 // Importer for networks stored in ArcView-shape format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIImporter_ArcView_h
-#define NIImporter_ArcView_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 
@@ -100,7 +90,7 @@ private:
      * @param[in] f The entry to read the speed from
      * @param[in] edgeid The id of the edge for error output
      */
-    SUMOReal getSpeed(OGRFeature& f, const std::string& edgeid);
+    double getSpeed(OGRFeature& f, const std::string& edgeid);
 
 
     /** @brief Parses the number of lanes of the edge currently processed
@@ -109,7 +99,7 @@ private:
      * @param[in] speed The edge's speed used to help determinig the edge's lane number
      */
     int getLaneNo(OGRFeature& f,
-                           const std::string& edgeid, SUMOReal speed);
+                  const std::string& edgeid, double speed);
 
     /** @brief Parses the priority of the edge currently processed
      * @param[in] f The entry to read the priority from
@@ -121,7 +111,7 @@ private:
     /** @brief Checks whether the lane spread shall be changed
      *
      * If for the given edge an edge into the vice direction is already
-     *  stored, both edges' lane spread functions are set to LANESPREAD_RIGHT.
+     *  stored, both edges' lane spread functions are set to LaneSpreadFunction::RIGHT.
      *
      * @param[in] e The edge to check
      */
@@ -142,6 +132,13 @@ private:
      * @param[out] into The read value/missing field is stored here
      */
     bool getStringEntry(OGRFeature* poFeature, const std::string& optionName, const char* defaultName, bool prune, std::string& into);
+
+    /// @brief return all fields support by the given feature
+    std::vector<std::string> getFieldNames(OGRFeature* poFeature) const;
+
+    /// @brief add list of parameters to edge
+    void addParams(NBEdge* edge, OGRFeature* poFeature, const std::vector<std::string>& params) const;
+
 #endif
 
 private:
@@ -166,7 +163,8 @@ private:
     /// @brief Whether the speed is given in km/h
     bool mySpeedInKMH;
 
-    /// @brief A running number to assure unique node ids
+    /// @brief A running number to assure unique ids (as fallback)
+    int myRunningEdgeID;
     int myRunningNodeID;
 
 
@@ -178,9 +176,3 @@ private:
     NIImporter_ArcView& operator=(const NIImporter_ArcView&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
