@@ -1,33 +1,25 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 /****************************************************************************/
 /// @file    NIImporter_DlrNavteq.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 14.04.2008
-/// @version $Id$
 ///
 // Importer for networks stored in Elmar's format
 /****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <sstream>
@@ -36,7 +28,7 @@
 #include <utils/common/StringTokenizer.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/UtilExceptions.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include <utils/common/StringUtils.h>
 #include <utils/options/OptionsCont.h>
@@ -60,6 +52,7 @@
 const std::string NIImporter_DlrNavteq::GEO_SCALE("1e-5");
 const int NIImporter_DlrNavteq::EdgesHandler::MISSING_COLUMN = std::numeric_limits<int>::max();
 const std::string NIImporter_DlrNavteq::UNDEFINED("-1");
+bool NIImporter_DlrNavteq::keepLength = false;
 
 // ===========================================================================
 // method definitions
@@ -75,6 +68,7 @@ NIImporter_DlrNavteq::loadNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
     }
     time_t csTime;
     time(&csTime);
+    keepLength = oc.getBool("dlr-navteq.keep-length");
     // parse file(s)
     LineReader lr;
     // load nodes
@@ -93,7 +87,7 @@ NIImporter_DlrNavteq::loadNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
     if (oc.getBool("output.street-names")) {
         file = oc.getString("dlr-navteq-prefix") + "_names.txt";
         if (lr.setFile(file)) {
-            PROGRESS_BEGIN_MESSAGE("Loading Street Names");
+            PROGRESS_BEGIN_MESSAGE("Loading street names");
             NamesHandler handler4(file, streetNames);
             lr.readAll(handler4);
             PROGRESS_DONE_MESSAGE();
@@ -153,6 +147,7 @@ NIImporter_DlrNavteq::loadNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
         handler5.printSummary();
         PROGRESS_DONE_MESSAGE();
     }
+
 }
 
 double
@@ -166,7 +161,7 @@ NIImporter_DlrNavteq::readVersion(const std::string& line, const std::string& fi
     const int vStart = (int)(lowerCase.find(marker) + marker.size());
     const int vEnd = (int)line.find(" ", vStart);
     try {
-        const double version = TplConvert::_2double(line.substr(vStart, vEnd - vStart).c_str());
+        const double version = StringUtils::toDouble(line.substr(vStart, vEnd - vStart));
         if (version < 0) {
             throw ProcessError("Invalid version number '" + toString(version) + "' in file '" + file + "'.");
         }
@@ -305,21 +300,21 @@ NIImporter_DlrNavteq::EdgesHandler::report(const std::string& result) {
     // form of way (for priority and permissions)
     int form_of_way;
     try {
-        form_of_way = TplConvert::_2int(getColumn(st, FORM_OF_WAY).c_str());
+        form_of_way = StringUtils::toInt(getColumn(st, FORM_OF_WAY));
     } catch (NumberFormatException&) {
         throw ProcessError("Non-numerical value for form_of_way of link '" + id + "'.");
     }
     // brunnel type (bridge/tunnel/ferry (for permissions)
     int brunnel_type;
     try {
-        brunnel_type = TplConvert::_2int(getColumn(st, BRUNNEL_TYPE).c_str());
+        brunnel_type = StringUtils::toInt(getColumn(st, BRUNNEL_TYPE));
     } catch (NumberFormatException&) {
         throw ProcessError("Non-numerical value for brunnel_type of link '" + id + "'.");
     }
     // priority based on street_type / frc
     int priority;
     try {
-        priority = -TplConvert::_2int(getColumn(st, FUNCTIONAL_ROAD_CLASS).c_str());
+        priority = -StringUtils::toInt(getColumn(st, FUNCTIONAL_ROAD_CLASS));
         // lower priority using form_of_way
         if (form_of_way == 11) {
             priority -= 1; // frontage road, very often with lowered curb
@@ -338,17 +333,17 @@ NIImporter_DlrNavteq::EdgesHandler::report(const std::string& result) {
     const std::string toID = getColumn(st, NODE_ID_TO);
     NBNode* from = myNodeCont.retrieve(fromID);
     NBNode* to = myNodeCont.retrieve(toID);
-    if (from == 0) {
+    if (from == nullptr) {
         throw ProcessError("The from-node '" + fromID + "' of link '" + id + "' could not be found");
     }
-    if (to == 0) {
+    if (to == nullptr) {
         throw ProcessError("The to-node '" + toID + "' of link '" + id + "' could not be found");
     }
     // speed
     double speed;
     try {
-        speed = TplConvert::_2int(getColumn(st, SPEED_RESTRICTION, "-1").c_str()) / 3.6;
-    } catch (NumberFormatException) {
+        speed = StringUtils::toInt(getColumn(st, SPEED_RESTRICTION, "-1")) / 3.6;
+    } catch (NumberFormatException&) {
         throw ProcessError("Non-numerical value for the SPEED_RESTRICTION of link '" + id + "'.");
     }
     if (speed < 0) {
@@ -359,19 +354,21 @@ NIImporter_DlrNavteq::EdgesHandler::report(const std::string& result) {
     int numLanes;
     try {
         // EXTENDED_NUMBER_OF_LANES is prefered but may not be defined
-        numLanes = TplConvert::_2int(getColumn(st, EXTENDED_NUMBER_OF_LANES, "-1").c_str());
+        numLanes = StringUtils::toInt(getColumn(st, EXTENDED_NUMBER_OF_LANES, "-1"));
         if (numLanes == -1) {
             numLanes = NINavTeqHelper::getLaneNumber(id, getColumn(st, NUMBER_OF_LANES), speed);
         }
     } catch (NumberFormatException&) {
         throw ProcessError("Non-numerical value for the number of lanes of link '" + id + "'.");
     }
+
+    const std::string navTeqTypeId = getColumn(st, VEHICLE_TYPE) + "_" + getColumn(st, FORM_OF_WAY);
     // build the edge
-    NBEdge* e = 0;
+    NBEdge* e = nullptr;
     const std::string interID = getColumn(st, BETWEEN_NODE_ID);
     if (interID == "-1") {
-        e = new NBEdge(id, from, to, "", speed, numLanes, priority,
-                       NBEdge::UNSPECIFIED_WIDTH, NBEdge::UNSPECIFIED_OFFSET, streetName);
+        e = new NBEdge(id, from, to, myTypeCont.knows(navTeqTypeId) ? navTeqTypeId : "", speed, numLanes, priority,
+                       NBEdge::UNSPECIFIED_WIDTH, NBEdge::UNSPECIFIED_OFFSET, LaneSpreadFunction::RIGHT, streetName);
     } else {
         PositionVector geoms = myGeoms[interID];
         if (getColumn(st, CONNECTION, "0") == "1") {
@@ -380,24 +377,27 @@ NIImporter_DlrNavteq::EdgesHandler::report(const std::string& result) {
         geoms.insert(geoms.begin(), from->getPosition());
         geoms.push_back(to->getPosition());
         const std::string origID = OptionsCont::getOptions().getBool("output.original-names") ? id : "";
-        e = new NBEdge(id, from, to, "", speed, numLanes, priority,
-                       NBEdge::UNSPECIFIED_WIDTH, NBEdge::UNSPECIFIED_OFFSET, geoms, streetName, origID, LANESPREAD_CENTER);
+        e = new NBEdge(id, from, to, myTypeCont.knows(navTeqTypeId) ? navTeqTypeId : "", speed, numLanes, priority,
+                       NBEdge::UNSPECIFIED_WIDTH, NBEdge::UNSPECIFIED_OFFSET, geoms, LaneSpreadFunction::CENTER, streetName, origID);
+    }
+
+    //Import length or edges from input file
+    if (keepLength) {
+        const double length = StringUtils::toDouble(getColumn(st, LENGTH));
+        e->setLoadedLength(length);
     }
 
     // NavTeq imports can be done with a typemap (if supplied), if not, the old defaults are used
-    const std::string navTeqTypeId = getColumn(st, VEHICLE_TYPE) + "_" + getColumn(st, FORM_OF_WAY);
-
     if (myTypeCont.knows(navTeqTypeId)) {
-        e->setPermissions(myTypeCont.getPermissions(navTeqTypeId));
+        e->setPermissions(myTypeCont.getEdgeTypePermissions(navTeqTypeId));
     } else {
         // add vehicle type information to the edge
+        const SVCPermissions allPermissions = myTypeCont.getEdgeTypePermissions("");
+        const SVCPermissions defaultPermissions = OptionsCont::getOptions().getBool("dlr-navteq.tolerant-permissions") ? allPermissions : 0;
         if (myVersion < 6.0) {
-            NINavTeqHelper::addVehicleClasses(*e, getColumn(st, VEHICLE_TYPE));
+            NINavTeqHelper::addVehicleClasses(*e, getColumn(st, VEHICLE_TYPE), allPermissions, defaultPermissions);
         } else {
-            NINavTeqHelper::addVehicleClassesV6(*e, getColumn(st, VEHICLE_TYPE));
-        }
-        if (e->getPermissions() == SVCAll) {
-            e->setPermissions(myTypeCont.getPermissions(""));
+            NINavTeqHelper::addVehicleClassesV6(*e, getColumn(st, VEHICLE_TYPE), allPermissions, defaultPermissions);
         }
         // permission modifications based on form_of_way
         if (form_of_way == 14) { // pedestrian area (fussgaengerzone)
@@ -491,12 +491,12 @@ NIImporter_DlrNavteq::TrafficlightsHandler::report(const std::string& result) {
     StringTokenizer st(result, StringTokenizer::WHITECHARS);
     const std::string edgeID = st.get(5);
     NBEdge* edge = myEdgeCont.retrieve(edgeID);
-    if (edge == 0) {
-        WRITE_WARNING("The traffic light edge '" + edgeID + "' could not be found");
+    if (edge == nullptr) {
+        WRITE_WARNINGF("The traffic light edge '%' could not be found.", edgeID);
     } else {
         NBNode* node = edge->getToNode();
-        if (node->getType() != NODETYPE_TRAFFIC_LIGHT) {
-            node->reinit(node->getPosition(), NODETYPE_TRAFFIC_LIGHT);
+        if (node->getType() != SumoXMLNodeType::TRAFFIC_LIGHT) {
+            node->reinit(node->getPosition(), SumoXMLNodeType::TRAFFIC_LIGHT);
             // @note. There may be additional information somewhere in the GDF files about traffic light type ...
             TrafficLightType type = SUMOXMLDefinitions::TrafficLightTypes.get(OptionsCont::getOptions().getString("tls.default-type"));
             // @note actually we could use the navteq node ID here
@@ -600,7 +600,7 @@ NIImporter_DlrNavteq::TimeRestrictionsHandler::report(const std::string& result)
             //std::cout << " duration=" << duration << " tEnd=" << tEnd << " translation=" << asctime(localtime(&tEnd)) << "\n";
             if (myConstructionTime < tEnd) {
                 NBEdge* edge = myEdgeCont.retrieve(id);
-                if (edge != 0) {
+                if (edge != nullptr) {
                     myRemovedEdges++;
                     myEdgeCont.extract(myDistrictCont, edge, true);
                 }
@@ -687,9 +687,9 @@ NIImporter_DlrNavteq::readDate(const std::string& yyyymmdd) {
             && yyyymmdd[4] == '-'
             && yyyymmdd[7] == '-') {
         try {
-            timeinfo.tm_year = TplConvert::_str2int(yyyymmdd.substr(0, 4)) - 1900;
-            timeinfo.tm_mon = TplConvert::_str2int(yyyymmdd.substr(5, 2)) - 1;
-            timeinfo.tm_mday = TplConvert::_str2int(yyyymmdd.substr(8, 2));
+            timeinfo.tm_year = StringUtils::toInt(yyyymmdd.substr(0, 4)) - 1900;
+            timeinfo.tm_mon = StringUtils::toInt(yyyymmdd.substr(5, 2)) - 1;
+            timeinfo.tm_mday = StringUtils::toInt(yyyymmdd.substr(8, 2));
             return mktime(&timeinfo);
         } catch (...) {
         }
@@ -739,7 +739,7 @@ NIImporter_DlrNavteq::ProhibitionHandler::report(const std::string& result) {
         const std::string throughTraffic = st.next();
         const std::string vehicleType = st.next();
         if (validityPeriod != UNDEFINED) {
-            WRITE_WARNING("Ignoring temporary prohibited manoeuvre (" + validityPeriod + ")");
+            WRITE_WARNINGF("Ignoring temporary prohibited manoeuvre (%).", validityPeriod);
             return true;
         }
     }
@@ -747,13 +747,13 @@ NIImporter_DlrNavteq::ProhibitionHandler::report(const std::string& result) {
     const std::string endEdge = st.get(st.size() - 1);
 
     NBEdge* from = myEdgeCont.retrieve(startEdge);
-    if (from == 0) {
-        WRITE_WARNING("Ignoring prohibition from unknown start edge '" + startEdge + "'");
+    if (from == nullptr) {
+        WRITE_WARNINGF("Ignoring prohibition from unknown start edge '%'.", startEdge);
         return true;
     }
     NBEdge* to = myEdgeCont.retrieve(endEdge);
-    if (to == 0) {
-        WRITE_WARNING("Ignoring prohibition from unknown end edge '" + endEdge + "'");
+    if (to == nullptr) {
+        WRITE_WARNINGF("Ignoring prohibition from unknown end edge '%'.", endEdge);
         return true;
     }
     from->removeFromConnections(to, -1, -1, true);
@@ -792,40 +792,41 @@ NIImporter_DlrNavteq::ConnectedLanesHandler::report(const std::string& result) {
     const std::string endEdge = st.get(st.size() - 1);
 
     NBEdge* from = myEdgeCont.retrieve(startEdge);
-    if (from == 0) {
-        WRITE_WARNING("Ignoring prohibition from unknown start edge '" + startEdge + "'");
+    if (from == nullptr) {
+        WRITE_WARNINGF("Ignoring prohibition from unknown start edge '%'.", startEdge);
         return true;
     }
     NBEdge* to = myEdgeCont.retrieve(endEdge);
-    if (to == 0) {
-        WRITE_WARNING("Ignoring prohibition from unknown end edge '" + endEdge + "'");
+    if (to == nullptr) {
+        WRITE_WARNINGF("Ignoring prohibition from unknown end edge '%'.", endEdge);
         return true;
     }
-    int fromLane = TplConvert::_2int(fromLaneS.c_str()) - 1; // one based
+    int fromLane = StringUtils::toInt(fromLaneS) - 1; // one based
     if (fromLane < 0 || fromLane >= from->getNumLanes()) {
-        WRITE_WARNING("Ignoring invalid lane index '" + fromLaneS + "' in connection from edge '" + startEdge + "' with " + toString(from->getNumLanes()) + " lanes");
+        WRITE_WARNINGF("Ignoring invalid lane index '%' in connection from edge '%' with % lanes.", fromLaneS, startEdge, from->getNumLanes());
         return true;
     }
-    int toLane = TplConvert::_2int(toLaneS.c_str()) - 1; // one based
+    int toLane = StringUtils::toInt(toLaneS) - 1; // one based
     if (toLane < 0 || toLane >= to->getNumLanes()) {
-        WRITE_WARNING("Ignoring invalid lane index '" + toLaneS + "' in connection to edge '" + endEdge + "' with " + toString(to->getNumLanes()) + " lanes");
+        WRITE_WARNINGF("Ignoring invalid lane index '%' in connection to edge '%' with % lanes", toLaneS, endEdge, to->getNumLanes());
         return true;
     }
-    if (!from->addLane2LaneConnection(fromLane, to, toLane, NBEdge::L2L_USER, true)) {
+    if (!from->addLane2LaneConnection(fromLane, to, toLane, NBEdge::Lane2LaneInfoType::USER, true)) {
         if (OptionsCont::getOptions().getBool("show-errors.connections-first-try")) {
-            WRITE_WARNING("Could not set loaded connection from '" + from->getLaneID(fromLane) + "' to '" + to->getLaneID(toLane) + "'.");
+            WRITE_WARNINGF("Could not set loaded connection from '%' to '%'.", from->getLaneID(fromLane), to->getLaneID(toLane));
         }
         // set as to be re-applied after network processing
         // if this connection runs across a node cluster it may not be possible to set this
         const bool warnOnly = st.size() > 7;
-        myEdgeCont.addPostProcessConnection(from->getID(), fromLane, to->getID(), toLane, false, true,
+        myEdgeCont.addPostProcessConnection(from->getID(), fromLane, to->getID(), toLane, false, KEEPCLEAR_UNSPECIFIED,
                                             NBEdge::UNSPECIFIED_CONTPOS, NBEdge::UNSPECIFIED_VISIBILITY_DISTANCE,
-                                            NBEdge::UNSPECIFIED_SPEED, PositionVector::EMPTY, warnOnly);
+                                            NBEdge::UNSPECIFIED_SPEED, NBEdge::UNSPECIFIED_LOADED_LENGTH, PositionVector::EMPTY, false, warnOnly);
     }
     // ensure that connections for other lanes are guessed if not specified
-    from->declareConnectionsAsLoaded(NBEdge::INIT);
+    from->declareConnectionsAsLoaded(NBEdge::EdgeBuildingStep::INIT);
     from->getLaneStruct(fromLane).connectionsDone = true;
     return true;
 }
 
+
 /****************************************************************************/
