@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2008-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIImporter_DlrNavteq.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 14.04.2008
-/// @version $Id: NIImporter_DlrNavteq.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Importer for networks stored in Elmar's format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2008-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIImporter_DlrNavteq_h
-#define NIImporter_DlrNavteq_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -75,7 +65,10 @@ public:
     static void loadNetwork(const OptionsCont& oc, NBNetBuilder& nb);
 
     /// @brief scaling factor for geo coordinates (DLRNavteq format uses this to increase floating point precisions)
-    static const int GEO_SCALE;
+    static const std::string GEO_SCALE;
+
+    /// @brief decides whether the edge length of the input format should be used
+    static bool keepLength;
 
     /// @brief magic value for undefined stuff
     static const std::string UNDEFINED;
@@ -522,9 +515,3 @@ protected:
     static time_t readDate(const std::string& yyyymmdd);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
