@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIImporter_MATSim.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    Tue, 26.04.2011
-/// @version $Id: NIImporter_MATSim.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Importer for networks stored in MATSim format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIImporter_MATSim_h
-#define NIImporter_MATSim_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -137,7 +127,7 @@ private:
          * @param[in] lanesFromCapacity Whether the lane number shall be computed from the capacity
          * @param[in] capacity2Lanes The converter from flow to lanes
          */
-        EdgesHandler(const NBNodeCont& nc, NBEdgeCont& toFill,
+        EdgesHandler(NBNodeCont& nc, NBEdgeCont& toFill,
                      bool keepEdgeLengths, bool lanesFromCapacity,
                      NBCapacity2Lanes capacity2Lanes);
 
@@ -160,16 +150,18 @@ private:
         void myStartElement(int element, const SUMOSAXAttributes& attrs);
         //@}
 
+    private:
+        void insertEdge(const std::string& id, NBNode* fromNode, NBNode* toNode, double freeSpeed, int numLanes, double capacity, double length);
 
     private:
         /// @brief The previously parsed nodes
-        const NBNodeCont& myNodeCont;
+        NBNodeCont& myNodeCont;
 
         /// @brief The edge container to fill
         NBEdgeCont& myEdgeCont;
 
         /// @brief The capacity norming
-        SUMOReal myCapacityNorm;
+        double myCapacityNorm;
 
         /// @brief Whether the loaded lengths shal be used
         bool myKeepEdgeLengths;
@@ -236,9 +228,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
