@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIImporter_OpenDrive.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 14.04.2008
-/// @version $Id: NIImporter_OpenDrive.h 20687 2016-05-10 11:27:00Z behrisch $
 ///
 // Importer for networks stored in openDrive format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIImporter_OpenDrive_h
-#define NIImporter_OpenDrive_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -98,18 +88,25 @@ protected:
         OPENDRIVE_TAG_SPIRAL,
         OPENDRIVE_TAG_ARC,
         OPENDRIVE_TAG_POLY3,
+        OPENDRIVE_TAG_PARAMPOLY3,
         OPENDRIVE_TAG_LANESECTION,
+        OPENDRIVE_TAG_LANEOFFSET,
         OPENDRIVE_TAG_LEFT,
         OPENDRIVE_TAG_CENTER,
         OPENDRIVE_TAG_RIGHT,
         OPENDRIVE_TAG_LANE,
         OPENDRIVE_TAG_SIGNAL,
+        OPENDRIVE_TAG_SIGNALREFERENCE,
+        OPENDRIVE_TAG_VALIDITY,
         OPENDRIVE_TAG_JUNCTION,
         OPENDRIVE_TAG_CONNECTION,
         OPENDRIVE_TAG_LANELINK,
         OPENDRIVE_TAG_WIDTH,
         OPENDRIVE_TAG_SPEED,
-        OPENDRIVE_TAG_ELEVATION
+        OPENDRIVE_TAG_ELEVATION,
+        OPENDRIVE_TAG_GEOREFERENCE,
+        OPENDRIVE_TAG_OBJECT,
+        OPENDRIVE_TAG_REPEAT
     };
 
 
@@ -124,11 +121,19 @@ protected:
         OPENDRIVE_ATTR_REVMINOR,
         OPENDRIVE_ATTR_ID,
         OPENDRIVE_ATTR_LENGTH,
+        OPENDRIVE_ATTR_WIDTH,
+        OPENDRIVE_ATTR_RADIUS,
+        OPENDRIVE_ATTR_DISTANCE,
+        OPENDRIVE_ATTR_TSTART,
+        OPENDRIVE_ATTR_TEND,
+        OPENDRIVE_ATTR_WIDTHSTART,
+        OPENDRIVE_ATTR_WIDTHEND,
         OPENDRIVE_ATTR_JUNCTION,
         OPENDRIVE_ATTR_ELEMENTTYPE,
         OPENDRIVE_ATTR_ELEMENTID,
         OPENDRIVE_ATTR_CONTACTPOINT,
         OPENDRIVE_ATTR_S,
+        OPENDRIVE_ATTR_T,
         OPENDRIVE_ATTR_X,
         OPENDRIVE_ATTR_Y,
         OPENDRIVE_ATTR_HDG,
@@ -139,6 +144,15 @@ protected:
         OPENDRIVE_ATTR_B,
         OPENDRIVE_ATTR_C,
         OPENDRIVE_ATTR_D,
+        OPENDRIVE_ATTR_AU,
+        OPENDRIVE_ATTR_BU,
+        OPENDRIVE_ATTR_CU,
+        OPENDRIVE_ATTR_DU,
+        OPENDRIVE_ATTR_AV,
+        OPENDRIVE_ATTR_BV,
+        OPENDRIVE_ATTR_CV,
+        OPENDRIVE_ATTR_DV,
+        OPENDRIVE_ATTR_PRANGE,
         OPENDRIVE_ATTR_TYPE,
         OPENDRIVE_ATTR_LEVEL,
         OPENDRIVE_ATTR_ORIENTATION,
@@ -147,9 +161,12 @@ protected:
         OPENDRIVE_ATTR_CONNECTINGROAD,
         OPENDRIVE_ATTR_FROM,
         OPENDRIVE_ATTR_TO,
+        OPENDRIVE_ATTR_FROMLANE,
+        OPENDRIVE_ATTR_TOLANE,
         OPENDRIVE_ATTR_MAX,
         OPENDRIVE_ATTR_SOFFSET,
-        OPENDRIVE_ATTR_NAME
+        OPENDRIVE_ATTR_NAME,
+        OPENDRIVE_ATTR_UNIT    // xodr v1.4
     };
 
 
@@ -185,7 +202,8 @@ protected:
         OPENDRIVE_GT_LINE,
         OPENDRIVE_GT_SPIRAL,
         OPENDRIVE_GT_ARC,
-        OPENDRIVE_GT_POLY3
+        OPENDRIVE_GT_POLY3,
+        OPENDRIVE_GT_PARAMPOLY3 // rev 1.4
     };
 
 
@@ -222,24 +240,24 @@ protected:
          * @param[in] yArg y-position at this part's begin
          * @param[in] hdgArg heading at this part's begin
          */
-        OpenDriveGeometry(SUMOReal lengthArg, SUMOReal sArg, SUMOReal xArg, SUMOReal yArg, SUMOReal hdgArg)
+        OpenDriveGeometry(double lengthArg, double sArg, double xArg, double yArg, double hdgArg)
             : length(lengthArg), s(sArg), x(xArg), y(yArg), hdg(hdgArg),
               type(OPENDRIVE_GT_UNKNOWN) { }
 
-        SUMOReal length;
-        SUMOReal s;
-        SUMOReal x;
-        SUMOReal y;
-        SUMOReal hdg;
+        double length;
+        double s;
+        double x;
+        double y;
+        double hdg;
         GeometryType type;
-        std::vector<SUMOReal> params;
+        std::vector<double> params;
     };
 
     /**
      * @struct OpenDriveElevation
      * @brief Coefficients of an elevation profile (3rd degree polynomial)
      */
-    struct OpenDriveElevation {
+    struct Poly3 {
         /** @brief Constructor
          * @param[in] s The start offset
          * @param[in] a constant
@@ -247,16 +265,26 @@ protected:
          * @param[in] c second order
          * @param[in] d third order
          */
-        OpenDriveElevation(SUMOReal _s, SUMOReal _a, SUMOReal _b, SUMOReal _c, SUMOReal _d) :
+        Poly3(double _s, double _a, double _b, double _c, double _d) :
             s(_s), a(_a), b(_b), c(_c), d(_d) {}
 
-        SUMOReal s;
-        SUMOReal a;
-        SUMOReal b;
-        SUMOReal c;
-        SUMOReal d;
+        double computeAt(double pos) const {
+            const double ds = pos - s;
+            return a + b * ds + c * ds * ds + d * ds * ds * ds;
+        }
+
+        double s;
+        double a;
+        double b;
+        double c;
+        double d;
     };
 
+    /// LaneOffset has the same fields as Elevation
+    typedef Poly3 OpenDriveElevation;
+    typedef Poly3 OpenDriveLaneOffset;
+    typedef Poly3 OpenDriveWidth;
+
 
     /**
      * @struct OpenDriveLane
@@ -268,18 +296,19 @@ protected:
          * @param[in] levelArg The level
          * @param[in] typeArg type of the lane
          */
-        OpenDriveLane(int idArg, const std::string& levelArg, const std::string& typeArg)
-            : id(idArg), level(levelArg), type(typeArg), successor(UNSET_CONNECTION), predecessor(UNSET_CONNECTION),
-              speed(0), width(0) { }
+        OpenDriveLane(int idArg, const std::string& levelArg, const std::string& typeArg) :
+            id(idArg), level(levelArg), type(typeArg), successor(UNSET_CONNECTION), predecessor(UNSET_CONNECTION),
+            speed(0), width(NBEdge::UNSPECIFIED_WIDTH) { }
 
         int id; //!< The lane's id
         std::string level; //!< The lane's level (not used)
         std::string type; //!< The lane's type
         int successor; //!< The lane's successor lane
         int predecessor; //!< The lane's predecessor lane
-        std::vector<std::pair<SUMOReal, SUMOReal> > speeds; //!< List of positions/speeds of speed changes
-        SUMOReal speed; //!< The lane's speed (set in post-processing)
-        SUMOReal width; //!< The lane's width; @todo: this is the maximum width only
+        std::vector<std::pair<double, double> > speeds; //!< List of positions/speeds of speed changes
+        double speed; //!< The lane's speed (set in post-processing)
+        double width; //The lane's maximum width
+        std::vector<OpenDriveWidth> widthData;
     };
 
 
@@ -291,7 +320,7 @@ protected:
         /** @brief Constructor
          * @param[in] sArg The offset from the start, counted from the begin
          */
-        OpenDriveLaneSection(SUMOReal sArg);
+        OpenDriveLaneSection(double sArg);
 
 
         /** @brief Build the mapping from OpenDrive to SUMO lanes
@@ -312,10 +341,14 @@ protected:
 
 
         bool buildSpeedChanges(const NBTypeCont& tc, std::vector<OpenDriveLaneSection>& newSections);
-        OpenDriveLaneSection buildLaneSection(SUMOReal startPos);
+        OpenDriveLaneSection buildLaneSection(double startPos);
 
         /// @brief The starting offset of this lane section
-        SUMOReal s;
+        double s;
+        /// @brief The original starting offset of this lane section (differs from s if the section had to be split)
+        double sOrig;
+        /// @brief The length of this lane section
+        double length;
         /// @brief A mapping from OpenDrive to SUMO-index (the first is signed, the second unsigned)
         std::map<int, int> laneMap;
         /// @brief The lanes, sorted by their direction
@@ -323,7 +356,10 @@ protected:
         /// @brief The id (generic, without the optionally leading '-') of the edge generated for this section
         std::string sumoID;
         /// @brief The number of lanes on the right and on the left side, respectively
-        unsigned int rightLaneNumber, leftLaneNumber;
+        int rightLaneNumber, leftLaneNumber;
+        /// @brief the composite type built from all used lane types
+        std::string rightType;
+        std::string leftType;
     };
 
 
@@ -341,15 +377,25 @@ protected:
          * @param[in] dynamicArg Whether the signal is dynamic
          * @param[in] sArg The offset from the start, counted from the begin
          */
-        OpenDriveSignal(int idArg, const std::string typeArg, const std::string nameArg, int orientationArg, bool dynamicArg, SUMOReal sArg)
-            : id(idArg), type(typeArg), name(nameArg), orientation(orientationArg), dynamic(dynamicArg), s(sArg) { }
+        OpenDriveSignal(const std::string& idArg, const std::string typeArg, const std::string nameArg,
+                        int orientationArg, bool dynamicArg, double sArg) :
+            id(idArg), type(typeArg), name(nameArg),
+            orientation(orientationArg), dynamic(dynamicArg), s(sArg),
+            minLane(0), maxLane(0)
+        { }
 
-        int id;
+        /// dummy constructor for use in maps
+        OpenDriveSignal() {}
+
+        std::string id;
         std::string type;
         std::string name;
         int orientation;
         bool dynamic;
-        SUMOReal s;
+        double s;
+        /// @brief signal validity range
+        int minLane;
+        int maxLane;
     };
 
 
@@ -367,15 +413,44 @@ protected:
         bool all;
         std::string origID;
         int origLane;
+        PositionVector shape;
+
+        std::string getDescription() const {
+            return "Connection from=" + fromEdge + "_" + toString(fromLane)
+                   + " to=" + toEdge + "_" + toString(toLane)
+                   + " fromCP=" + (fromCP == OPENDRIVE_CP_START ? "start" : fromCP == OPENDRIVE_CP_END ? "end" : "unknown")
+                   + " toCP=" + (toCP == OPENDRIVE_CP_START ? "start" : toCP == OPENDRIVE_CP_END ? "end" : "unknown")
+                   + " all=" + toString(all);
+            //+ " origID=" + origID + " origLane=" + toString(origLane);
+        }
     };
 
+    /**
+     * @struct Object
+     * @brief A road object (e.g. parkingSpace)
+     */
+    struct OpenDriveObject {
+        std::string type;
+        std::string name;
+        std::string id;
+        double s;
+        double t;
+        double zOffset;
+        double length;
+        double width;
+        double height;
+        double radius;
+        double hdg;
+        double pitch;
+        double roll;
+    };
 
     /**
      * @struct OpenDriveEdge
      * @brief Representation of an openDrive "link"
      */
     struct OpenDriveEdge {
-        OpenDriveEdge(const std::string& idArg, const std::string& streetNameArg, const std::string& junctionArg, SUMOReal lengthArg) :
+        OpenDriveEdge(const std::string& idArg, const std::string& streetNameArg, const std::string& junctionArg, double lengthArg) :
             id(idArg),
             streetName(streetNameArg),
             junction(junctionArg),
@@ -402,16 +477,19 @@ protected:
         /// @brief The id of the junction the edge belongs to
         std::string junction;
         /// @brief The length of the edge
-        SUMOReal length;
+        double length;
         std::vector<OpenDriveLink> links;
         std::vector<OpenDriveGeometry> geometries;
         std::vector<OpenDriveElevation> elevations;
+        std::vector<OpenDriveLaneOffset> offsets;
         NBNode* from;
         NBNode* to;
         PositionVector geom;
+        std::vector<double> laneOffsets;
         std::vector<OpenDriveLaneSection> laneSections;
         std::vector<OpenDriveSignal> signals;
         std::set<Connection> connections;
+        std::vector<OpenDriveObject> objects;
         bool isInner;
     };
 
@@ -432,10 +510,10 @@ protected:
     class same_position_finder {
     public:
         /** @brief constructor */
-        explicit same_position_finder(SUMOReal pos) : myPosition(pos) { }
+        explicit same_position_finder(double pos) : myPosition(pos) { }
 
         /** @brief the comparing function */
-        bool operator()(const std::pair<SUMOReal, SUMOReal>& ps) {
+        bool operator()(const std::pair<double, double>& ps) {
             return ps.first == myPosition;
         }
 
@@ -443,7 +521,7 @@ protected:
         same_position_finder& operator=(const same_position_finder&); // just to avoid a compiler warning
     private:
         /// @brief The position to search for
-        SUMOReal myPosition;
+        double myPosition;
 
     };
 
@@ -475,6 +553,16 @@ protected:
      */
     void myStartElement(int element, const SUMOSAXAttributes& attrs);
 
+    /**
+     * @brief Callback method for characters to implement by derived classes
+     *
+     * Called by "endElement" (see there).
+     * @param[in] element The opened element, given as a int
+     * @param[in] chars The complete embedded character string
+     * @exceptions ProcessError These method may throw a ProcessError if something fails
+     */
+    void myCharacters(int element, const std::string& chars);
+
 
     /** @brief Called when a closing tag occurs
      *
@@ -485,14 +573,17 @@ protected:
     void myEndElement(int element);
     //@}
 
-
+    std::map<std::string, OpenDriveSignal>& getSignals() {
+        return mySignals;
+    }
 
 private:
     void addLink(LinkType lt, const std::string& elementType, const std::string& elementID,
                  const std::string& contactPoint);
-    void addGeometryShape(GeometryType type, const std::vector<SUMOReal>& vals);
+    void addGeometryShape(GeometryType type, const std::vector<double>& vals);
     static void setEdgeLinks2(OpenDriveEdge& e, const std::map<std::string, OpenDriveEdge*>& edges);
-    static void buildConnectionsToOuter(const Connection& c, const std::map<std::string, OpenDriveEdge*>& innerEdges, std::vector<Connection>& into);
+    static void buildConnectionsToOuter(const Connection& c, const std::map<std::string, OpenDriveEdge*>& innerEdges, std::vector<Connection>& into, std::set<Connection>& seen);
+    static bool laneSectionsConnected(OpenDriveEdge* edge, int in, int out);
     friend bool operator<(const Connection& c1, const Connection& c2);
     static std::string revertID(const std::string& id);
     const NBTypeCont& myTypeContainer;
@@ -506,9 +597,12 @@ private:
     std::string myCurrentConnectingRoad;
     ContactPoint myCurrentContactPoint;
     bool myConnectionWasEmpty;
+    std::map<std::string, OpenDriveSignal> mySignals;
 
     static bool myImportAllTypes;
     static bool myImportWidths;
+    static double myMinWidth;
+    static bool myImportInternalShapes;
 
 
 protected:
@@ -528,14 +622,15 @@ protected:
     static NBNode* getOrBuildNode(const std::string& id, const Position& pos, NBNodeCont& nc);
 
 
-    static PositionVector geomFromLine(const OpenDriveEdge& e, const OpenDriveGeometry& g);
-    static PositionVector geomFromSpiral(const OpenDriveEdge& e, const OpenDriveGeometry& g);
-    static PositionVector geomFromArc(const OpenDriveEdge& e, const OpenDriveGeometry& g);
-    static PositionVector geomFromPoly(const OpenDriveEdge& e, const OpenDriveGeometry& g);
+    static PositionVector geomFromLine(const OpenDriveEdge& e, const OpenDriveGeometry& g, double resolution);
+    static PositionVector geomFromSpiral(const OpenDriveEdge& e, const OpenDriveGeometry& g, double resolution);
+    static PositionVector geomFromArc(const OpenDriveEdge& e, const OpenDriveGeometry& g, double resolution);
+    static PositionVector geomFromPoly(const OpenDriveEdge& e, const OpenDriveGeometry& g, double resolution);
+    static PositionVector geomFromParamPoly(const OpenDriveEdge& e, const OpenDriveGeometry& g, double resolution);
     static Position calculateStraightEndPoint(double hdg, double length, const Position& start);
-    static void calculateCurveCenter(SUMOReal* ad_x, SUMOReal* ad_y, SUMOReal ad_radius, SUMOReal ad_hdg);
-    static void calcPointOnCurve(SUMOReal* ad_x, SUMOReal* ad_y, SUMOReal ad_centerX, SUMOReal ad_centerY,
-                                 SUMOReal ad_r, SUMOReal ad_length);
+    static void calculateCurveCenter(double* ad_x, double* ad_y, double ad_radius, double ad_hdg);
+    static void calcPointOnCurve(double* ad_x, double* ad_y, double ad_centerX, double ad_centerY,
+                                 double ad_r, double ad_length);
 
 
     /** @brief Computes a polygon representation of each edge's geometry
@@ -543,6 +638,8 @@ protected:
      */
     static void computeShapes(std::map<std::string, OpenDriveEdge*>& edges);
 
+    static bool hasNonLinearElevation(OpenDriveEdge& e);
+
     /** @brief Rechecks lane sections of the given edges
      *
      *
@@ -551,10 +648,26 @@ protected:
     static void revisitLaneSections(const NBTypeCont& tc, std::map<std::string, OpenDriveEdge*>& edges);
 
     static void setNodeSecure(NBNodeCont& nc, OpenDriveEdge& e,
-                              const std::string& nodeID, NIImporter_OpenDrive::LinkType lt);
+                              const std::string& nodeID, NIImporter_OpenDrive::LinkType lt, std::vector<NodeSet>& joinedNodeIDs);
+
+    static NBTrafficLightDefinition* getTLSSecure(NBEdge* inEdge, NBNetBuilder& nb);
 
 
+    static std::pair<NBEdge*, NBEdge*> retrieveSignalEdges(NBNetBuilder& nb, const std::string& fromID, const std::string& toID, const std::string& junction);
 
+    static void splitMinWidths(OpenDriveEdge* e, const NBTypeCont& tc, double minDist);
+
+    static void findWidthSplit(const NBTypeCont& tc, std::vector<OpenDriveLane>& lanes,
+                               int section, double sectionStart, double sectionEnd,
+                               std::vector<double>& splitPositions);
+
+    static void sanitizeWidths(OpenDriveEdge* e);
+    static void sanitizeWidths(std::vector<OpenDriveLane>& lanes, double length);
+
+    static void setStraightConnections(std::vector<OpenDriveLane>& lanes);
+    static void recomputeWidths(OpenDriveLaneSection& sec, double start, double end, double sectionStart, double sectionEnd);
+    static void recomputeWidths(std::vector<OpenDriveLane>& lanes, double start, double end, double sectionStart, double sectionEnd);
+    static void setLaneAttributes(const OpenDriveEdge* e, NBEdge::Lane& sumoLane, const OpenDriveLane& odLane, bool saveOrigIDs, const NBTypeCont& tc);
 
     /// The names of openDrive-XML elements (for passing to GenericSAXHandler)
     static StringBijection<int>::Entry openDriveTags[];
@@ -565,9 +678,3 @@ protected:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
