@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIImporter_OpenStreetMap.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -6,39 +18,18 @@
 /// @author  Walter Bamberger
 /// @author  Gregor Laemmel
 /// @date    Mon, 14.04.2008
-/// @version $Id: NIImporter_OpenStreetMap.h 23648 2017-03-24 10:35:01Z laemmel $
 ///
 // Importer for networks stored in OpenStreetMap format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIImporter_OpenStreetMap_h
-#define NIImporter_OpenStreetMap_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
 #include <utils/xml/SUMOSAXHandler.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/Parameterised.h>
+#include <netbuild/NBPTPlatform.h>
 
 
 // ===========================================================================
@@ -77,13 +68,21 @@ public:
      */
     static void loadNetwork(const OptionsCont& oc, NBNetBuilder& nb);
 
-
 protected:
     /** @brief An internal representation of an OSM-node
      */
     struct NIOSMNode {
-        NIOSMNode(long long int _id, double _lon, double _lat) :
-            id(_id), lon(_lon), lat(_lat), ele(0), tlsControlled(false), railwayCrossing(false), ptStopPostion(false), ptStopLength(0), node(0) {}
+        NIOSMNode(long long int _id, double _lon, double _lat)
+            :
+            id(_id), lon(_lon), lat(_lat), ele(0),
+            tlsControlled(false),
+            railwayCrossing(false),
+            railwaySignal(false),
+            railwayBufferStop(false),
+            ptStopPosition(false), ptStopLength(0), name(""),
+            permissions(SVC_IGNORING),
+            positionMeters(std::numeric_limits<double>::max()),
+            node(0) { }
 
         /// @brief The node's id
         const long long int id;
@@ -97,21 +96,39 @@ protected:
         bool tlsControlled;
         /// @brief Whether this is a railway crossing
         bool railwayCrossing;
+        /// @brief Whether this is a railway (main) signal
+        bool railwaySignal;
+        /// @brief Whether this is a railway buffer stop
+        bool railwayBufferStop;
         /// @brief Whether this is a public transport stop position
-        bool ptStopPostion;
+        bool ptStopPosition;
         /// @brief The length of the pt stop
         double ptStopLength;
         /// @brief The name of the node
         std::string name;
+        /// @brief type of pt stop
+        SVCPermissions permissions;
+        /// @brief kilometrage/mileage
+        std::string position;
+        /// @brief position converted to m (using highest precision available)
+        double positionMeters;
         /// @brief the NBNode that was instantiated
         NBNode* node;
 
     private:
         /// invalidated assignment operator
-        NIOSMNode& operator=(const NIOSMNode& s);
+        NIOSMNode& operator=(const NIOSMNode& s) = delete;
+
 
     };
 
+public:
+    /// @brief translate osm transport designations into sumo vehicle class
+    static SUMOVehicleClass interpretTransportType(const std::string& type, NIOSMNode* toSet = nullptr);
+
+protected:
+
+
     /** @enum CycleWayType
      * @brief details on the kind of cycleway along this road
      */
@@ -123,47 +140,102 @@ protected:
         WAY_UNKNOWN = 4
     };
 
+    enum ParkingType {
+        PARKING_NONE = 0,
+        PARKING_LEFT = 1,
+        PARKING_RIGHT = 2,
+        PARKING_BOTH = WAY_FORWARD | WAY_BACKWARD,
+        PARKING_UNKNOWN = 4,
+        PARKING_FORBIDDEN = 8,
+        PARKING_PERPENDICULAR = 16,
+        PARKING_DIAGONAL = 32
+    };
+
+    enum ChangeType {
+        CHANGE_YES = 0,
+        CHANGE_NO_LEFT = 1,
+        CHANGE_NO_RIGHT = 2,
+        CHANGE_NO = 3
+    };
 
     /** @brief An internal definition of a loaded edge
      */
-    struct Edge : public Parameterised {
-
-        Edge(long long int _id) :
-            id(_id), myNoLanes(-1), myNoLanesForward(0), myMaxSpeed(MAXSPEED_UNGIVEN),
+    class Edge : public Parameterised {
+    public:
+        explicit Edge(long long int _id) :
+            id(_id), myNoLanes(-1), myNoLanesForward(0),
+            myMaxSpeed(MAXSPEED_UNGIVEN),
+            myMaxSpeedBackward(MAXSPEED_UNGIVEN),
+            myExtraAllowed(0),
+            myExtraDisallowed(0),
             myCyclewayType(WAY_UNKNOWN), // building of extra lane depends on bikelaneWidth of loaded typemap
             myBuswayType(WAY_NONE), // buslanes are always built when declared
+            mySidewalkType(WAY_UNKNOWN), // building of extra lanes depends on sidewalkWidth of loaded typemap
+            myRailDirection(WAY_UNKNOWN), // store direction(s) of railway usage
+            myParkingType(PARKING_NONE), // parking areas exported optionally
+            myChangeForward(CHANGE_YES),
+            myChangeBackward(CHANGE_YES),
             myLayer(0), // layer is non-zero only in conflict areas
-            myCurrentIsRoad(false) {}
+            myCurrentIsRoad(false),
+            myCurrentIsPlatform(false),
+            myCurrentIsElectrified(false)
+        { }
+
+        virtual ~Edge() {}
 
         /// @brief The edge's id
         const long long int id;
         /// @brief The edge's street name
         std::string streetName;
+        /// @brief The edge's track name
+        std::string ref;
         /// @brief number of lanes, or -1 if unknown
         int myNoLanes;
         /// @brief number of lanes in forward direction or 0 if unknown, negative if backwards lanes are meant
         int myNoLanesForward;
         /// @brief maximum speed in km/h, or MAXSPEED_UNGIVEN
         double myMaxSpeed;
+        /// @brief maximum speed in km/h, or MAXSPEED_UNGIVEN
+        double myMaxSpeedBackward;
+        /// @brief Extra permissions added from tags instead of highway type
+        SVCPermissions myExtraAllowed;
+        /// @brief Extra permissions prohibited from tags instead of highway type
+        SVCPermissions myExtraDisallowed;
         /// @brief The type, stored in "highway" key
         std::string myHighWayType;
         /// @brief Information whether this is an one-way road
-        std::string  myIsOneWay;
+        std::string myIsOneWay;
         /// @brief Information about the kind of cycleway along this road
         WayType myCyclewayType;
         /// @brief Information about the kind of busway along this road
         WayType myBuswayType;
+        /// @brief Information about the kind of sidwalk along this road
+        WayType mySidewalkType;
+        /// @brief Information about the direction(s) of railway usage
+        WayType myRailDirection;
+        /// @brief Information about road-side parking
+        int myParkingType;
+        /// @brief Information about change prohibitions (forward direction
+        int myChangeForward;
+        /// @brief Information about change prohibitions (backward direction
+        int myChangeBackward;
+        /// @brief (optional) information about the permitted vehicle classes on each lane
+        std::vector<SVCPermissions> myLaneUseForward;
+        std::vector<SVCPermissions> myLaneUseBackward;
         /// @brief Information about the relative z-ordering of ways
         int myLayer;
         /// @brief The list of nodes this edge is made of
         std::vector<long long int> myCurrentNodes;
         /// @brief Information whether this is a road
         bool myCurrentIsRoad;
+        /// @brief Information whether this is a pt platform
+        bool myCurrentIsPlatform;
+        /// @brief Information whether this is railway is electrified
+        bool myCurrentIsElectrified;
 
     private:
         /// invalidated assignment operator
-        Edge& operator=(const Edge& s);
-
+        Edge& operator=(const Edge& s) = delete;
     };
 
 
@@ -202,12 +274,22 @@ private:
     /** @brief the map from OSM way ids to edge objects */
     std::map<long long int, Edge*> myEdges;
 
+    /** @brief the map from OSM way ids to platform shapes */
+    std::map<long long int, Edge*> myPlatformShapes;
+
+    /** @brief the map from stop_area relations to member node */
+    std::map<long long int, std::set<long long int> > myStopAreas;
+
     /// @brief The compounds types that do not contain known types
     std::set<std::string> myUnusableTypes;
 
     /// @brief The compound types that have already been mapped to other known types
     std::map<std::string, std::string> myKnownCompoundTypes;
 
+    /// @brief import lane specifc access restrictions
+    bool myImportLaneAccess;
+
+
     /** @brief Builds an NBNode
      *
      * If a node with the given id is already known, nothing is done.
@@ -244,12 +326,25 @@ private:
     void reconstructLayerElevation(double layerElevation, NBNetBuilder& nb);
 
     /// @brief collect neighboring nodes with their road distance and maximum between-speed. Search does not continue beyond knownElevation-nodes
-    std::map<NBNode*, std::pair<double, double> > getNeighboringNodes(NBNode* node, double maxDist, const std::set<NBNode*>& knownElevation);
+    std::map<NBNode*, std::pair<double, double> >
+    getNeighboringNodes(NBNode* node, double maxDist, const std::set<NBNode*>& knownElevation);
+
+    /// @brief check whether the type is known or consists of known type compounds. return empty string otherwise
+    std::string usableType(const std::string& type, const std::string& id, NBTypeCont& tc);
+
+    /// @brief extend kilometrage data for all nodes along railway
+    void extendRailwayDistances(Edge* e, NBTypeCont& tc);
+
+    /// @brief read distance value from node and return value in m
+    static double interpretDistance(NIOSMNode* node);
 
 protected:
     static const double MAXSPEED_UNGIVEN;
     static const long long int INVALID_ID;
 
+    static void applyChangeProhibition(NBEdge* e, int changeProhibition);
+    void applyLaneUseInformation(NBEdge* e, const std::vector<SVCPermissions>& laneUse);
+
     /**
      * @class NodesHandler
      * @brief A class which extracts OSM-nodes from a parsed OSM-file
@@ -263,12 +358,16 @@ protected:
          * @param[in] options The options to use
          */
         NodesHandler(std::map<long long int, NIOSMNode*>& toFill, std::set<NIOSMNode*,
-                     CompareNodes>& uniqueNodes, const OptionsCont& cont);
+                     CompareNodes>& uniqueNodes,
+                     const OptionsCont& cont);
 
 
         /// @brief Destructor
-        ~NodesHandler();
+        ~NodesHandler() override;
 
+        int getDuplicateNodes() const {
+            return myDuplicateNodes;
+        }
 
     protected:
         /// @name inherited from GenericSAXHandler
@@ -281,7 +380,7 @@ protected:
          * @exception ProcessError If something fails
          * @see GenericSAXHandler::myStartElement
          */
-        void myStartElement(int element, const SUMOSAXAttributes& attrs);
+        void myStartElement(int element, const SUMOSAXAttributes& attrs) override;
 
 
         /** @brief Called when a closing tag occurs
@@ -290,7 +389,7 @@ protected:
          * @exception ProcessError If something fails
          * @see GenericSAXHandler::myEndElement
          */
-        void myEndElement(int element);
+        void myEndElement(int element) override;
         //@}
 
 
@@ -314,10 +413,12 @@ protected:
         /// @brief whether elevation data should be imported
         const bool myImportElevation;
 
+        /// @brief number of diplicate nodes
+        int myDuplicateNodes;
+
         /// @brief the options
         const OptionsCont& myOptionsCont;
 
-
     private:
         /** @brief invalidated copy constructor */
         NodesHandler(const NodesHandler& s);
@@ -328,7 +429,6 @@ protected:
     };
 
 
-
     /**
      * @class EdgesHandler
      * @brief A class which extracts OSM-edges from a parsed OSM-file
@@ -341,11 +441,11 @@ protected:
          * @param[in, out] toFill The edges container to fill with read edges
          */
         EdgesHandler(const std::map<long long int, NIOSMNode*>& osmNodes,
-                     std::map<long long int, Edge*>& toFill);
+                     std::map<long long int, Edge*>& toFill, std::map<long long int, Edge*>& platformShapes);
 
 
         /// @brief Destructor
-        ~EdgesHandler();
+        ~EdgesHandler() override;
 
 
     protected:
@@ -359,7 +459,7 @@ protected:
          * @exception ProcessError If something fails
          * @see GenericSAXHandler::myStartElement
          */
-        void myStartElement(int element, const SUMOSAXAttributes& attrs);
+        void myStartElement(int element, const SUMOSAXAttributes& attrs) override;
 
 
         /** @brief Called when a closing tag occurs
@@ -368,9 +468,14 @@ protected:
          * @exception ProcessError If something fails
          * @see GenericSAXHandler::myEndElement
          */
-        void myEndElement(int element);
+        void myEndElement(int element) override;
         //@}
 
+        double interpretSpeed(const std::string& key, std::string value);
+
+        int interpretChangeType(const std::string& value) const;
+
+        void interpretLaneUse(const std::string& value, SUMOVehicleClass svc, std::vector<SVCPermissions>& result) const;
 
     private:
         /// @brief The previously parsed nodes
@@ -379,6 +484,9 @@ protected:
         /// @brief A map of built edges
         std::map<long long int, Edge*>& myEdgeMap;
 
+        /// @brief A map of built edges
+        std::map<long long int, Edge*>& myPlatformShapesMap;
+
         /// @brief The currently built edge
         Edge* myCurrentEdge;
 
@@ -388,6 +496,12 @@ protected:
         /// @brief A map of non-numeric speed descriptions to their numeric values
         std::map<std::string, double> mySpeedMap;
 
+        /// @brief whether additional way attributes shall be added to the edge
+        bool myAllAttributes;
+        /// @brief extra attributes to import
+        std::set<std::string> myExtraAttributes;
+
+
     private:
         /** @brief invalidated copy constructor */
         EdgesHandler(const EdgesHandler& s);
@@ -410,12 +524,13 @@ protected:
          * @param[in] osmEdges The previously parse OSM-edges
          */
         RelationHandler(const std::map<long long int, NIOSMNode*>& osmNodes,
-                        const std::map<long long int, Edge*>& osmEdges);
+                        const std::map<long long int, Edge*>& osmEdges, NBPTStopCont* nbptStopCont,
+                        const std::map<long long int, Edge*>& platfromShapes, NBPTLineCont* nbptLineCont,
+                        const OptionsCont& oc);
 
 
         /// @brief Destructor
-        ~RelationHandler();
-
+        ~RelationHandler() override;
 
     protected:
         /// @name inherited from GenericSAXHandler
@@ -428,7 +543,7 @@ protected:
          * @exception ProcessError If something fails
          * @see GenericSAXHandler::myStartElement
          */
-        void myStartElement(int element, const SUMOSAXAttributes& attrs);
+        void myStartElement(int element, const SUMOSAXAttributes& attrs) override;
 
 
         /** @brief Called when a closing tag occurs
@@ -437,7 +552,7 @@ protected:
          * @exception ProcessError If something fails
          * @see GenericSAXHandler::myEndElement
          */
-        void myEndElement(int element);
+        void myEndElement(int element) override;
         //@}
 
 
@@ -448,6 +563,15 @@ protected:
         /// @brief The previously parsed edges
         const std::map<long long int, Edge*>& myOSMEdges;
 
+        /// @brief The previously parsed platform shapes
+        const std::map<long long int, Edge*>& myPlatformShapes;
+
+        /// @brief The previously filled pt stop container
+        NBPTStopCont* myNBPTStopCont;
+
+        /// @brief PT Line container to be filled
+        NBPTLineCont* myNBPTLineCont;
+
         /// @brief The currently parsed relation
         long long int myCurrentRelation;
 
@@ -468,16 +592,19 @@ protected:
         long long int myViaWay;
 
 
+        /// @brief the options cont
+        const OptionsCont& myOptionsCont;
+
         /** @enum RestrictionType
          * @brief whether the only allowed or the only forbidden connection is defined
          */
-        enum RestrictionType {
+        enum class RestrictionType {
             /// @brief The only valid connection is declared
-            RESTRICTION_ONLY,
+            ONLY,
             /// @brief The only invalid connection is declared
-            RESTRICTION_NO,
+            NO,
             /// @brief The relation tag was missing
-            RESTRICTION_UNKNOWN
+            UNKNOWN
         };
         RestrictionType myRestrictionType;
 
@@ -500,12 +627,41 @@ protected:
         /** @brief invalidated assignment operator */
         RelationHandler& operator=(const RelationHandler& s);
 
+        /// @brief bus stop references
+        std::vector<long long int> myStops;
+
+
+        struct NIIPTPlatform {
+            long long int ref;
+            bool isWay;
+        };
+
+        /// @brief bus stop platforms
+        std::vector<NIIPTPlatform> myPlatforms;
+
+        /// @brief ways in pt line references
+        std::vector<long long int> myWays;
+
+        /// @brief indicates whether current relation is a pt stop area
+        bool myIsStopArea;
+
+        /// @brief indicates whether current relation is a route
+        bool myIsRoute;
+
+        /// @brief indicates whether current relation is a pt route
+        std::string myPTRouteType;
+
+        /// @brief name of the relation
+        std::string myName;
+
+        /// @brief ref of the pt line
+        std::string myRef;
+
+        /// @brief service interval of the pt line in minutes
+        int myInterval;
+
+        /// @brief night service information of the pt line
+        std::string myNightService;
     };
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
