@@ -1,42 +1,34 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIImporter_SUMO.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 14.04.2008
-/// @version $Id: NIImporter_SUMO.h 21182 2016-07-18 06:46:01Z behrisch $
 ///
 // Importer for networks stored in SUMO format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIImporter_SUMO_h
-#define NIImporter_SUMO_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
 #include <utils/xml/SUMOSAXHandler.h>
 #include <utils/geom/GeoConvHelper.h>
+#include <utils/common/Parameterised.h>
 #include <netbuild/NBLoadedSUMOTLDef.h>
+#include "NIXMLTypesHandler.h"
 
 
 // ===========================================================================
@@ -89,12 +81,9 @@ protected:
      */
     NIImporter_SUMO(NBNetBuilder& nb);
 
-
     /// @brief Destructor
     ~NIImporter_SUMO();
 
-
-
     /// @name inherited from GenericSAXHandler
     //@{
 
@@ -140,6 +129,10 @@ private:
      */
     void addLane(const SUMOSAXAttributes& attrs);
 
+    /** @brief parses stop offsets for the current lane or edge
+     * @param[in] attrs The attributes to get the stop offset sepcifics from
+     */
+    void addStopOffsets(const SUMOSAXAttributes& attrs, bool& ok);
 
     /** @brief Parses a junction and saves it in the node control
      * @param[in] attrs The attributes to get the junction's values from
@@ -178,7 +171,8 @@ private:
      * @struct Connection
      * @brief A connection description.
      */
-    struct Connection {
+    class Connection final : public Parameterised {
+    public:
         /// @brief The id of the target edge
         std::string toEdgeID;
         /// @brief The index of the target lane
@@ -186,23 +180,43 @@ private:
         /// @brief The id of the traffic light that controls this connection
         std::string tlID;
         /// @brief The index of this connection within the controlling traffic light
-        int tlLinkNo;
+        int tlLinkIndex;
+        int tlLinkIndex2;
         /// @brief Information about being definitely free to drive (on-ramps)
         bool mayDefinitelyPass;
-        /// @brief Whether the junction must be kept clear coming from this connection
+        /* @brief Whether the junction must be kept clear coming from this connection
+         * @note: The enum NBEdge::KeepClear is not needed here because data
+         *        from a .net.xml is fully specified */
         bool keepClear;
         /// @brief custom position for internal junction on this connection
-        SUMOReal contPos;
+        double contPos;
+        /// @brief custom foe visibility for connection
+        double visibility;
+        /// @brief custom permissions for connection
+        SVCPermissions permissions;
+        /// @brief custom lane changing permissions for connection
+        SVCPermissions changeLeft;
+        /// @brief custom lane changing permissions for connection
+        SVCPermissions changeRight;
+        /// @brief custom speed for connection
+        double speed;
+        /// @brief custom length for connection
+        double customLength;
+        /// @brief custom shape connection
+        PositionVector customShape;
+        /// @brief if set to true, This connection will not be TLS-controlled despite its node being controlled.
+        bool uncontrolled;
     };
 
 
     /** @struct LaneAttrs
      * @brief Describes the values found in a lane's definition
      */
-    struct LaneAttrs {
+    class LaneAttrs final : public Parameterised {
+    public:
         /// @brief The maximum velocity allowed on this lane
-        SUMOReal maxSpeed;
-        /// @brief This lane's shape (needed to reconstruct edge shape for legacy networks)
+        double maxSpeed;
+        /// @brief This lane's shape (may be custom)
         PositionVector shape;
         /// @brief This lane's connections
         std::vector<Connection> connections;
@@ -210,19 +224,32 @@ private:
         std::string allow;
         /// @brief This lane's disallowed vehicle classes
         std::string disallow;
+        /// @brief This lane's vehicle classes allowed to change left
+        std::string changeLeft;
+        /// @brief This lane's vehicle classes allowed to change right
+        std::string changeRight;
         /// @brief The width of this lane
-        SUMOReal width;
+        double width;
         /// @brief This lane's offset from the intersection
-        SUMOReal endOffset;
+        double endOffset;
+        /// @brief This lane's vehicle specific stop offsets
+        std::map<SVCPermissions, double> stopOffsets;
+        /// @brief Whether this lane is an acceleration lane
+        bool accelRamp;
         /// @brief This lane's opposite lane
         std::string oppositeID;
+        /// @brief Whether this lane has a custom shape
+        bool customShape;
+        /// @brief the type of this lane
+        std::string type;
     };
 
 
     /** @struct EdgeAttrs
      * @brief Describes the values found in an edge's definition and this edge's lanes
      */
-    struct EdgeAttrs {
+    class EdgeAttrs final : public Parameterised {
+    public:
         /// @brief This edge's id
         std::string id;
         /// @brief This edge's street name
@@ -238,17 +265,21 @@ private:
         /// @brief This edges's shape
         PositionVector shape;
         /// @brief The length of the edge if set explicitly
-        SUMOReal length;
+        double length;
         /// @brief This edge's priority
         int priority;
         /// @brief The maximum velocity allowed on this edge (!!!)
-        SUMOReal maxSpeed;
+        double maxSpeed;
         /// @brief This edge's lanes
         std::vector<LaneAttrs*> lanes;
         /// @brief The built edge
         NBEdge* builtEdge;
         /// @brief The lane spread function
         LaneSpreadFunction lsf;
+        /// @brief This edge's vehicle specific stop offsets (used for lanes, that do not have a specified stopOffset)
+        std::map<SVCPermissions, double> stopOffsets;
+        /// @brief The position at the start of this edge (kilometrage/mileage)
+        double distance;
     };
 
 
@@ -266,10 +297,28 @@ private:
      * @brief Describes a pedestrian crossing
      */
     struct Crossing {
+        Crossing(const std::string& _edgeID) :
+            edgeID(_edgeID), customTLIndex(-1), customTLIndex2(-1) {}
+
         std::string edgeID;
         std::vector<std::string> crossingEdges;
-        SUMOReal width;
+        double width;
         bool priority;
+        PositionVector customShape;
+        int customTLIndex;
+        int customTLIndex2;
+    };
+
+    /** @struct WalkingAreaParsedCustomShape
+     * @brief Describes custom shape for a walking area during parsing
+     */
+    struct WalkingAreaParsedCustomShape {
+        PositionVector shape;
+        std::vector<std::string> fromEdges;
+        std::vector<std::string> toEdges;
+        std::vector<std::string> fromCrossed;
+        std::vector<std::string> toCrossed;
+        double width;
     };
 
     /** @struct JunctionAttrs
@@ -298,6 +347,9 @@ private:
     /// @brief The node container to fill
     NBTrafficLightLogicCont& myTLLCont;
 
+    /// @brief The handler for parsing edge types and restrictions
+    NIXMLTypesHandler myTypesHandler;
+
     /// @brief The currently parsed edge's definition (to add loaded lanes to)
     EdgeAttrs* myCurrentEdge;
 
@@ -316,47 +368,67 @@ private:
     /// @brief The pedestrian crossings found in the network
     std::map<std::string, std::vector<Crossing> > myPedestrianCrossings;
 
+    /// @brief Map from walkingArea edge IDs to custom shapes
+    std::map<std::string, WalkingAreaParsedCustomShape> myWACustomShapes;
+
+    /// @brief element to receive parameters
+    std::vector<Parameterised*> myLastParameterised;
+
+    /// @brief the loaded network version
+    double myNetworkVersion;
+
     /// @brief whether the loaded network contains internal lanes
     bool myHaveSeenInternalEdge;
 
     /// @brief whether the loaded network was built for lefthand traffic
     bool myAmLefthand;
 
+    /// @brief whether the the written network should have a different "handedness" (LHT/RHT) than the loaded network
+    bool myChangeLefthand;
+
     /// @brief the level of corner detail in the loaded network
     int myCornerDetail;
 
     /// @brief the level of geometry detail for internal lanes in the loaded network
     int myLinkDetail;
 
+    /// @brief whether all lanes of an edge should have the same stop line
+    bool myRectLaneCut;
+
+    /// @brief whether walkingareas must be built
+    bool myWalkingAreas;
+
+    /// @brief whether turning speed was limited in the network
+    double myLimitTurnSpeed;
+
+    /// @brief whether foe-relationships where checked at lane-level
+    bool myCheckLaneFoesAll;
+    bool myCheckLaneFoesRoundabout;
+    /// @brief whether some right-of-way checks at traffic light junctions should be disabled
+    bool myTlsIgnoreInternalJunctionJam;
+    /// @brief default spreadType defined in the network
+    std::string myDefaultSpreadType;
+    /// @brief overlap option for loaded network
+    bool myGeomAvoidOverlap;
+    /// @brief higherSpeed  option for loaded network
+    bool myJunctionsHigherSpeed;
+
     /// @brief loaded roundabout edges
     std::vector<std::vector<std::string> > myRoundabouts;
 
-    /// @brief customLaneShape (cannot be added to the NBNode when parsed since the node doesn't yet exist
-    std::map<std::string, NBNode::CustomShapeMap> myCustomShapeMaps;
-
     /// @brief list of node id with rail signals (no NBTrafficLightDefinition exists)
     std::set<std::string> myRailSignals;
 
+    /// @brief list of parameter keys to discard
+    std::set<std::string> myDiscardableParams;
+
+private:
     /** @brief Parses lane index from lane ID an retrieve lane from EdgeAttrs
      * @param[in] edge The EdgeAttrs* which should contain the lane
      * @param[in] lane_id The ID of the lane
      */
     LaneAttrs* getLaneAttrsFromID(EdgeAttrs* edge, std::string lane_id);
 
-    /** @brief parses edge-id and index from lane-id
-     * @param[in] lane_id The lane-id
-     * @param[out] edge_id ID of this lane's edge
-     * @param[out] index Index of this lane
-     */
-    static void interpretLaneID(const std::string& lane_id, std::string& edge_id, int& index);
-
-    /** @brief reconstructs the edge shape from the node positions and the given lane shapes
-     * since we do not know the original LaneSpreadFunction this is only an
-     * approximation
-     * @param[in] lanes The list of lane attributes
-     */
-    static PositionVector reconstructEdgeShape(const EdgeAttrs* edge, const Position& from, const Position& to);
-
     /// @brief read position from the given attributes, attribute errors to id
     static Position readPosition(const SUMOSAXAttributes& attrs, const std::string& id, bool& ok);
 
@@ -368,9 +440,3 @@ private:
      */
     void parseProhibitionConnection(const std::string& attr, std::string& from, std::string& to, bool& ok);
 };
-
-
-#endif
-
-/****************************************************************************/
-
