@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIImporter_VISUM.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Fri, 19 Jul 2002
-/// @version $Id: NIImporter_VISUM.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // A VISUM network importer
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIImporter_VISUM_h
-#define NIImporter_VISUM_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -101,7 +91,8 @@ protected:
      * @param[in] useVisumPrio Information whether the VISUM type's priority shall be used
      */
     NIImporter_VISUM(NBNetBuilder& nb, const std::string& file,
-                     NBCapacity2Lanes capacity2Lanes, bool useVisumPrio);
+                     NBCapacity2Lanes capacity2Lanes, bool useVisumPrio,
+                     const std::string& languageFile);
 
 
     /// @brief destructor
@@ -129,7 +120,7 @@ private:
      * @exception NumberFormatException If the float is not numeric
      * @exception UnknownElement If the named data field is not in the line
      */
-    SUMOReal getNamedFloat(const std::string& fieldName);
+    double getNamedFloat(const std::string& fieldName);
 
     /** @brief The same, but two different names for the field are allowed
      *
@@ -140,26 +131,26 @@ private:
      * @exception NumberFormatException If the float is not numeric
      * @exception UnknownElement If the named data field is not in the line
      */
-    SUMOReal getNamedFloat(const std::string& fieldName1, const std::string& fieldName2);
+    double getNamedFloat(const std::string& fieldName1, const std::string& fieldName2);
 
 
     /** @brief Returns the value from the named column as a float or the default value if an error occurs
      *
      * @param[in] fieldName Name of the column to extract the float from
      * @param[in] defaultValue The default to return in the case of an error
-     * @return The parsed real or the default value if an error while parsing occured
+     * @return The parsed real or the default value if an error while parsing occurred
      */
-    SUMOReal getNamedFloat(const std::string& fieldName, SUMOReal defaultValue);
+    double getNamedFloat(const std::string& fieldName, double defaultValue);
 
     /** @brief The same, but two different names for the field are allowed
      *
      * @param[in] fieldName1 Name of the first column to extract the float from
      * @param[in] fieldName2 Name of the second column to extract the efloat from
      * @param[in] defaultValue The default to return in the case of an error
-     * @return The parsed real or the default value if an error while parsing occured
+     * @return The parsed real or the default value if an error while parsing occurred
      */
-    SUMOReal getNamedFloat(const std::string& fieldName1, const std::string& fieldName2,
-                           SUMOReal defaultValue);
+    double getNamedFloat(const std::string& fieldName1, const std::string& fieldName2,
+                         double defaultValue);
 
 
     /** @brief Returns the value from the named column as a normalised string
@@ -186,17 +177,22 @@ private:
     std::string getNamedString(const std::string& fieldName1, const std::string& fieldName2);
 
 
-    /** @brief tries to get a SUMOReal which is possibly assigned to a certain modality
+    /** @brief tries to get a double which is possibly assigned to a certain modality
      *
-     * When the SUMOReal cannot be extracted using the given name, "(IV)" is
+     * When the double cannot be extracted using the given name, "(IV)" is
      * appended to the begin of the name. Note that this function does not
      * yet support public traffic.
      *
      * @param[in] name Name of the column to extract the real from
-     * @return The real stored under the named column, or if not found the one from "(IV)"+name, or if not found -1
+     * @return The real stored under the named column, or if not found the one from name + suffix, or if not found -1
      */
-    SUMOReal getWeightedFloat(const std::string& name);
+    double getWeightedFloat(const std::string& name, const std::string& suffix);
 
+    /// @brief as above but with two alternative names
+    double getWeightedFloat2(const std::string& name, const std::string& name2, const std::string& suffix);
+
+    /// @brief parse permissions
+    SVCPermissions getPermissions(const std::string& name, bool warn = false, SVCPermissions unknown = SVCAll);
 
     /** @brief tries to get a bool which is possibly assigned to a certain modality
      *
@@ -223,6 +219,7 @@ private:
      * @exception UnknownElement If the named data field is not in the line
      */
     NBNode* getNamedNode(const std::string& fieldName);
+    NBNode* getNamedNodeSecure(const std::string& fieldName, NBNode* fallback = 0);
 
     /** @brief The same, but two different names for the field are allowed
      *
@@ -332,7 +329,7 @@ private:
      * @param[in] id Name of the district
      * @param[in] dest Name of the according network node
      * @param[in] isSource Information whether this node will be used as a source
-     * @return The built node, zero if an error occured
+     * @return The built node, zero if an error occurred
      */
     NBNode* buildDistrictNode(const std::string& id, NBNode* dest, bool isSource);
 
@@ -415,6 +412,7 @@ private:
 
     /// @brief Parses ANBINDUNG
     void parse_Connectors();
+    void parse_Connectors_legacy();
 
     /// @brief Parses ABBIEGEBEZIEHUNG/ABBIEGER
     void parse_Turns();
@@ -449,6 +447,8 @@ private:
     /// @brief Parses FAHRSTREIFENABBIEGER
     void parse_LanesConnections();
 
+    /// @brief Parses HALTEPUNKT (public transport stop locations)
+    void parse_stopPoints();
 
     /** @brief Adds a parser into the sorted list of parsers to use
      *
@@ -457,6 +457,11 @@ private:
      */
     void addParser(const std::string& name, ParsingFunction function);
 
+private:
+
+    /// @brief whether the edge id ends with _nodeID
+    static bool isSplitEdge(NBEdge* edge, NBNode* node);
+
 
 private:
     /// @brief The network builder to fill with loaded values
@@ -502,25 +507,79 @@ private:
 
 
     /// @brief A map of point ids to positions
-    std::map<SUMOLong, Position> myPoints;
+    std::map<long long int, Position> myPoints;
 
     /// @brief A map of edge (not road, but "edge" in this case) ids to from/to-points
-    std::map<SUMOLong, std::pair<SUMOLong, SUMOLong> > myEdges;
+    std::map<long long int, std::pair<long long int, long long int> > myEdges;
 
     /// @brief A map from district shape definition name to the district
-    std::map<SUMOLong, NBDistrict*> myShapeDistrictMap;
+    std::map<long long int, NBDistrict*> myShapeDistrictMap;
 
     /// @brief A map from area parts to area ids
-    std::map<SUMOLong, std::vector<SUMOLong> > mySubPartsAreas;
+    std::map<long long int, std::vector<long long int> > mySubPartsAreas;
 
     /// @brief A temporary storage for district shapes as they are filled incrementally
     std::map<NBDistrict*, PositionVector> myDistrictShapes;
 
+protected:
+    /**
+     * @enum VISUM keys
+     * @brief Numbers representing VISUM keywords
+     */
+    enum VISUM_KEY {
+        VISUM_SYS,
+        VISUM_LINKTYPE,
+        VISUM_NODE,
+        VISUM_DISTRICT,
+        VISUM_POINT,
+        VISUM_LINK,
+        VISUM_V0,
+        VISUM_TYPES,
+        VISUM_RANK,
+        VISUM_CAPACITY,
+        VISUM_XCOORD,
+        VISUM_YCOORD,
+        VISUM_FROMNODE,
+        VISUM_TONODE,
+        VISUM_TYPE,
+        VISUM_TYP,
+        VISUM_ID,
+        VISUM_CODE,
+        VISUM_DISTRICT_CONNECTION,
+        VISUM_SOURCE_DISTRICT,
+        VISUM_FROMNODENO,
+        VISUM_DIRECTION,
+        VISUM_SURFACEID,
+        VISUM_FACEID,
+        VISUM_FROMPOINTID,
+        VISUM_TOPOINTID,
+        VISUM_EDGE,
+        VISUM_VIANODENO,
+        VISUM_NUMLANES,
+        VISUM_TURN,
+        VISUM_INDEX,
+        VISUM_LINKPOLY,
+        VISUM_SURFACEITEM,
+        VISUM_FACEITEM,
+        VISUM_EDGEID,
+        VISUM_ORIGIN,
+        VISUM_DESTINATION,
+        VISUM_STOPPOINT,
+        VISUM_NAME,
+        VISUM_LINKNO,
+        VISUM_RELPOS,
+        // polyconvert keys added to avoid warnings
+        VISUM_CATID,
+        VISUM_EDGEITEM,
+        VISUM_POICATEGORY,
+        VISUM_NO // must be the last one
+    };
 
+    /// Strings for the keywords
+    static StringBijection<VISUM_KEY>::Entry KEYS_DE[];
+
+    /// @brief link directions
+    static StringBijection<VISUM_KEY> KEYS;
+
+    void loadLanguage(const std::string& file);
 };
-
-
-#endif
-
-/****************************************************************************/
-
