@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NINavTeqHelper.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Jul 2006
-/// @version $Id: NINavTeqHelper.h 21182 2016-07-18 06:46:01Z behrisch $
 ///
 // Some parser methods shared around several formats containing NavTeq-Nets
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NINavTeqHelper_h
-#define NINavTeqHelper_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/common/StdDefs.h>
@@ -68,8 +58,8 @@ public:
      * @return The converted speed (in m/s)
      * @exception ProcessError If the given speed class definition is not a number or if it is not known
      */
-    static SUMOReal getSpeed(const std::string& id,
-                             const std::string& speedClassS);
+    static double getSpeed(const std::string& id,
+                           const std::string& speedClassS);
 
 
     /** @brief Returns the lane number evaluating the given Navteq-description
@@ -81,7 +71,7 @@ public:
      * @exception ProcessError If the given lane number definition is not a number or if it is not known
      */
     static int getLaneNumber(const std::string& id,
-                                      const std::string& laneNoS, SUMOReal speed);
+                             const std::string& laneNoS, double speed);
 
 
     /** @brief Adds vehicle classes parsing the given list of allowed vehicles
@@ -93,15 +83,12 @@ public:
      * @param[in] classS The string that contains the information whether a vehicle class is allowed
      * @see addVehicleClass
      */
-    static void addVehicleClasses(NBEdge& e, const std::string& classS);
+    static void addVehicleClasses(NBEdge& e, const std::string& classS, const SVCPermissions allPermissions, const SVCPermissions defaultPermissions);
 
     /// @brief same as addVehicleClasses but for version 6+
-    static void addVehicleClassesV6(NBEdge& e, const std::string& classS);
+    static void addVehicleClassesV6(NBEdge& e, const std::string& classS, const SVCPermissions allPermissions, const SVCPermissions defaultPermissions);
+
+private:
+    static bool addCommonVehicleClasses(NBEdge& e, const std::string& classS, const int offset);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
