@@ -1,34 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVisumTL.h
 /// @author  Daniel Krajzewicz
 /// @date    Wed, 07 May 2003
-/// @version $Id: NIVisumTL.h 22608 2017-01-17 06:28:54Z behrisch $
 ///
 // Intermediate class for storing visum traffic lights during their import
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIVisumTL_h
-#define NIVisumTL_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <map>
@@ -79,11 +69,15 @@ public:
 
     private:
         /// @brief Start time
-        SUMOTime myStartTime;
+        const SUMOTime myStartTime;
         /// @brief End time
-        SUMOTime myEndTime;
+        const SUMOTime myEndTime;
         /// @brief Yellow time
-        SUMOTime myYellowTime;
+        const SUMOTime myYellowTime;
+
+    private:
+        /// @brief Invalidated assignment operator
+        TimePeriod& operator=(const TimePeriod& s) = delete;
     };
 
 
@@ -185,7 +179,7 @@ private:
     /// @brief The all-red time (unused here)
     SUMOTime myIntermediateTime;
 
-    /// @brief Toogles the usage either of phases or of time periods in signal groups
+    /// @brief Toggles the usage either of phases or of time periods in signal groups
     bool myPhaseDefined;
 
     /// @brief Vector of nodes belonging to this traffic light
@@ -199,9 +193,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
