@@ -1,36 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIXMLEdgesHandler.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @author  Laura Bieker
+/// @author  Leonhard Luecken
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NIXMLEdgesHandler.cpp 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // Importer for network edges stored in XML
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <iostream>
@@ -46,7 +38,7 @@
 #include <netbuild/NBNetBuilder.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include <utils/common/MsgHandler.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/StringTokenizer.h>
 #include <utils/geom/GeomConvHelper.h>
 #include <utils/common/ToString.h>
@@ -64,18 +56,20 @@ NIXMLEdgesHandler::NIXMLEdgesHandler(NBNodeCont& nc,
                                      NBTypeCont& tc,
                                      NBDistrictCont& dc,
                                      NBTrafficLightLogicCont& tlc,
-                                     OptionsCont& options)
-    : SUMOSAXHandler("xml-edges - file"),
-      myOptions(options),
-      myNodeCont(nc),
-      myEdgeCont(ec),
-      myTypeCont(tc),
-      myDistrictCont(dc),
-      myTLLogicCont(tlc),
-      myCurrentEdge(0), myHaveReportedAboutOverwriting(false),
-      myHaveReportedAboutTypeOverride(false),
-      myHaveWarnedAboutDeprecatedLaneId(false),
-      myKeepEdgeShape(!options.getBool("plain.extend-edge-shape")) {
+                                     OptionsCont& options) :
+    SUMOSAXHandler("xml-edges - file"),
+    myOptions(options),
+    myNodeCont(nc),
+    myEdgeCont(ec),
+    myTypeCont(tc),
+    myDistrictCont(dc),
+    myTLLogicCont(tlc),
+    myCurrentEdge(nullptr),
+    myCurrentLaneIndex(-1),
+    myHaveReportedAboutOverwriting(false),
+    myHaveReportedAboutTypeOverride(false),
+    myHaveWarnedAboutDeprecatedLaneId(false),
+    myKeepEdgeShape(!options.getBool("plain.extend-edge-shape")) {
 }
 
 
@@ -98,12 +92,58 @@ NIXMLEdgesHandler::myStartElement(int element,
         case SUMO_TAG_SPLIT:
             addSplit(attrs);
             break;
-        case SUMO_TAG_DELETE:
+        case SUMO_TAG_DEL:
             deleteEdge(attrs);
             break;
         case SUMO_TAG_ROUNDABOUT:
             addRoundabout(attrs);
             break;
+        case SUMO_TAG_PARAM:
+            if (myLastParameterised.size() != 0 && myCurrentEdge != nullptr) {
+                bool ok = true;
+                const std::string key = attrs.get<std::string>(SUMO_ATTR_KEY, nullptr, ok);
+                // circumventing empty string test
+                const std::string val = attrs.hasAttribute(SUMO_ATTR_VALUE) ? attrs.getString(SUMO_ATTR_VALUE) : "";
+                myLastParameterised.back()->setParameter(key, val);
+            }
+            break;
+        case SUMO_TAG_STOPOFFSET: {
+            bool ok = true;
+            std::map<SVCPermissions, double> stopOffsets = parseStopOffsets(attrs, ok);
+            assert(stopOffsets.size() == 1);
+            if (!ok) {
+                std::stringstream ss;
+                ss << "(Error encountered at lane " << myCurrentLaneIndex << " of edge '" << myCurrentID << "' while parsing stopOffsets.)";
+                WRITE_ERROR(ss.str());
+            } else {
+                if (myCurrentEdge->getStopOffsets(myCurrentLaneIndex).size() != 0) {
+                    std::stringstream ss;
+                    ss << "Duplicate definition of stopOffset for ";
+                    if (myCurrentLaneIndex != -1) {
+                        ss << "lane " << myCurrentLaneIndex << " on ";
+                    }
+                    ss << "edge " << myCurrentEdge->getID() << ". Ignoring duplicate specification.";
+                    WRITE_WARNING(ss.str());
+                    return;
+                } else if (stopOffsets.begin()->second > myCurrentEdge->getLength() || stopOffsets.begin()->second < 0) {
+                    std::stringstream ss;
+                    ss << "Ignoring invalid stopOffset for ";
+                    if (myCurrentLaneIndex != -1) {
+                        ss << "lane " << myCurrentLaneIndex << " on ";
+                    }
+                    ss << "edge " << myCurrentEdge->getID();
+                    if (stopOffsets.begin()->second > myCurrentEdge->getLength()) {
+                        ss << " (offset larger than the edge length).";
+                    } else {
+                        ss << " (negative offset).";
+                    }
+                    WRITE_WARNING(ss.str());
+                } else {
+                    myCurrentEdge->setStopOffsets(myCurrentLaneIndex, stopOffsets);
+                }
+            }
+        }
+        break;
         default:
             break;
     }
@@ -115,34 +155,36 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
     myIsUpdate = false;
     bool ok = true;
     // initialise the edge
-    myCurrentEdge = 0;
+    myCurrentEdge = nullptr;
     mySplits.clear();
     // get the id, report an error if not given or empty...
-    myCurrentID = attrs.get<std::string>(SUMO_ATTR_ID, 0, ok);
+    myCurrentID = attrs.get<std::string>(SUMO_ATTR_ID, nullptr, ok);
     if (!ok) {
         return;
     }
     myCurrentEdge = myEdgeCont.retrieve(myCurrentID);
     // check deprecated (unused) attributes
     // use default values, first
-    myCurrentPriority = myTypeCont.getPriority("");
-    myCurrentLaneNo = myTypeCont.getNumLanes("");
+    myCurrentType = myOptions.getString("default.type");
+    myCurrentPriority = myTypeCont.getEdgeTypePriority(myCurrentType);
+    myCurrentLaneNo = myTypeCont.getEdgeTypeNumLanes(myCurrentType);
     myCurrentEndOffset = NBEdge::UNSPECIFIED_OFFSET;
-    if (myCurrentEdge != 0) {
+    if (myCurrentEdge != nullptr) {
         // update existing edge. only update lane-specific settings when explicitly requested
         myIsUpdate = true;
         myCurrentSpeed = NBEdge::UNSPECIFIED_SPEED;
         myPermissions = SVC_UNSPECIFIED;
         myCurrentWidth = NBEdge::UNSPECIFIED_WIDTH;
+        myCurrentType = myCurrentEdge->getTypeID();
+        myLanesSpread = SUMOXMLDefinitions::LaneSpreadFunctions.get(myOptions.getString("default.spreadtype"));
     } else {
         // this is a completely new edge. get the type specific defaults
-        myCurrentSpeed = myTypeCont.getSpeed("");
-        myPermissions = myTypeCont.getPermissions("");
-        myCurrentWidth = myTypeCont.getWidth("");
+        myCurrentSpeed = myTypeCont.getEdgeTypeSpeed(myCurrentType);
+        myPermissions = myTypeCont.getEdgeTypePermissions(myCurrentType);
+        myCurrentWidth = myTypeCont.getEdgeTypeWidth(myCurrentType);
+        myLanesSpread = myTypeCont.getEdgeTypeSpreadType(myCurrentType);
     }
-    myCurrentType = "";
     myShape = PositionVector();
-    myLanesSpread = LANESPREAD_RIGHT;
     myLength = NBEdge::UNSPECIFIED_LOADED_LENGTH;
     myCurrentStreetName = "";
     myReinitKeepEdgeShape = false;
@@ -155,16 +197,17 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
             return;
         }
         if (!myTypeCont.knows(myCurrentType) && !myOptions.getBool("ignore-errors.edge-type")) {
-            WRITE_ERROR("Type '" + myCurrentType + "' used by edge '" + myCurrentID + "' was not defined (ignore with option --ignore-errors.edge-type).");
+            WRITE_ERRORF("Type '%' used by edge '%' was not defined (ignore with option --ignore-errors.edge-type).", myCurrentType, myCurrentID);
             return;
         }
-        myCurrentSpeed = myTypeCont.getSpeed(myCurrentType);
-        myCurrentPriority = myTypeCont.getPriority(myCurrentType);
-        myCurrentLaneNo = myTypeCont.getNumLanes(myCurrentType);
-        myPermissions = myTypeCont.getPermissions(myCurrentType);
-        myCurrentWidth = myTypeCont.getWidth(myCurrentType);
-        mySidewalkWidth = myTypeCont.getSidewalkWidth(myCurrentType);
-        myBikeLaneWidth = myTypeCont.getBikeLaneWidth(myCurrentType);
+        myCurrentSpeed = myTypeCont.getEdgeTypeSpeed(myCurrentType);
+        myCurrentPriority = myTypeCont.getEdgeTypePriority(myCurrentType);
+        myCurrentLaneNo = myTypeCont.getEdgeTypeNumLanes(myCurrentType);
+        myPermissions = myTypeCont.getEdgeTypePermissions(myCurrentType);
+        myCurrentWidth = myTypeCont.getEdgeTypeWidth(myCurrentType);
+        myLanesSpread = myTypeCont.getEdgeTypeSpreadType(myCurrentType);
+        mySidewalkWidth = myTypeCont.getEdgeTypeSidewalkWidth(myCurrentType);
+        myBikeLaneWidth = myTypeCont.getEdgeTypeBikeLaneWidth(myCurrentType);
     }
     // use values from the edge to overwrite if existing, then
     if (myIsUpdate) {
@@ -180,7 +223,7 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
         }
         if (attrs.getOpt<bool>(SUMO_ATTR_REMOVE, myCurrentID.c_str(), ok, false)) {
             myEdgeCont.erase(myDistrictCont, myCurrentEdge);
-            myCurrentEdge = 0;
+            myCurrentEdge = nullptr;
             return;
         }
         myCurrentPriority = myCurrentEdge->getPriority();
@@ -237,6 +280,7 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
     // try to set the nodes
     if (!setNodes(attrs)) {
         // return if this failed
+        myCurrentEdge = nullptr;
         return;
     }
     // try to get the shape
@@ -251,10 +295,12 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
     myBikeLaneWidth = attrs.getOpt<double>(SUMO_ATTR_BIKELANEWIDTH, myCurrentID.c_str(), ok, myBikeLaneWidth);
     // insert the parsed edge into the edges map
     if (!ok) {
+        myCurrentEdge = nullptr;
         return;
     }
     // check whether a previously defined edge shall be overwritten
-    if (myCurrentEdge != 0) {
+    const bool applyLaneType = myCurrentEdge == nullptr;
+    if (myCurrentEdge != nullptr) {
         myCurrentEdge->reinit(myFromNode, myToNode, myCurrentType, myCurrentSpeed,
                               myCurrentLaneNo, myCurrentPriority, myShape,
                               myCurrentWidth, myCurrentEndOffset,
@@ -265,11 +311,11 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
         if (myShape.size() == 0) {
             myCurrentEdge = new NBEdge(myCurrentID, myFromNode, myToNode, myCurrentType, myCurrentSpeed,
                                        myCurrentLaneNo, myCurrentPriority, myCurrentWidth, myCurrentEndOffset,
-                                       myCurrentStreetName, myLanesSpread);
+                                       myLanesSpread, myCurrentStreetName);
         } else {
             myCurrentEdge = new NBEdge(myCurrentID, myFromNode, myToNode, myCurrentType, myCurrentSpeed,
                                        myCurrentLaneNo, myCurrentPriority, myCurrentWidth, myCurrentEndOffset,
-                                       myShape, myCurrentStreetName, "", myLanesSpread,
+                                       myShape, myLanesSpread, myCurrentStreetName, "",
                                        myKeepEdgeShape);
         }
     }
@@ -277,14 +323,40 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
     if (myPermissions != SVC_UNSPECIFIED) {
         myCurrentEdge->setPermissions(myPermissions);
     }
+    // apply laneType if given
+    if (applyLaneType && myCurrentType != "" && myTypeCont.knows(myCurrentType)) {
+        const NBTypeCont::EdgeTypeDefinition* eType = myTypeCont.getEdgeType(myCurrentType);
+        if (eType->needsLaneType()) {
+            int lane = 0;
+            for (const NBTypeCont::LaneTypeDefinition& laneType : eType->laneTypeDefinitions) {
+                if (lane >= myCurrentLaneNo) {
+                    break;
+                }
+                if (laneType.attrs.count(SUMO_ATTR_SPEED) > 0) {
+                    myCurrentEdge->setSpeed(lane, laneType.speed);
+                }
+                if (laneType.attrs.count(SUMO_ATTR_DISALLOW) > 0 || laneType.attrs.count(SUMO_ATTR_ALLOW) > 0) {
+                    myCurrentEdge->setPermissions(laneType.permissions, lane);
+                }
+                if (laneType.attrs.count(SUMO_ATTR_WIDTH) > 0) {
+                    myCurrentEdge->setLaneWidth(lane, laneType.width);
+                }
+                lane++;
+            }
+        }
+    }
+    // try to get the kilometrage/mileage
+    myCurrentEdge->setDistance(attrs.getOpt<double>(SUMO_ATTR_DISTANCE, myCurrentID.c_str(), ok, myCurrentEdge->getDistance()));
+
+    myLastParameterised.push_back(myCurrentEdge);
 }
 
 
 void
 NIXMLEdgesHandler::addLane(const SUMOSAXAttributes& attrs) {
-    if (myCurrentEdge == 0) {
+    if (myCurrentEdge == nullptr) {
         if (!OptionsCont::getOptions().isInStringVector("remove-edges.explicit", myCurrentID)) {
-            WRITE_ERROR("Additional lane information could not be set - the edge with id '" + myCurrentID + "' is not known.");
+            WRITE_ERRORF("Additional lane information could not be set - the edge with id '%s' is not known.", myCurrentID);
         }
         return;
     }
@@ -303,20 +375,26 @@ NIXMLEdgesHandler::addLane(const SUMOSAXAttributes& attrs) {
         return;
     }
     // check whether this lane exists
-    if (lane >= (int) myCurrentEdge->getNumLanes()) {
+    if (lane >= myCurrentEdge->getNumLanes()) {
         WRITE_ERROR("Lane index is larger than number of lanes (edge '" + myCurrentID + "').");
         return;
     }
+    myCurrentLaneIndex = lane;
     // set information about allowed / disallowed vehicle classes (if specified)
     if (attrs.hasAttribute(SUMO_ATTR_ALLOW) || attrs.hasAttribute(SUMO_ATTR_DISALLOW)) {
-        const std::string allowed = attrs.getOpt<std::string>(SUMO_ATTR_ALLOW, 0, ok, "");
-        const std::string disallowed = attrs.getOpt<std::string>(SUMO_ATTR_DISALLOW, 0, ok, "");
+        const std::string allowed = attrs.getOpt<std::string>(SUMO_ATTR_ALLOW, nullptr, ok, "");
+        const std::string disallowed = attrs.getOpt<std::string>(SUMO_ATTR_DISALLOW, nullptr, ok, "");
         myCurrentEdge->setPermissions(parseVehicleClasses(allowed, disallowed), lane);
     }
     if (attrs.hasAttribute(SUMO_ATTR_PREFER)) {
-        const std::string preferred  = attrs.get<std::string>(SUMO_ATTR_PREFER, 0, ok);
+        const std::string preferred  = attrs.get<std::string>(SUMO_ATTR_PREFER, nullptr, ok);
         myCurrentEdge->setPreferredVehicleClass(parseVehicleClasses(preferred), lane);
     }
+    if (attrs.hasAttribute(SUMO_ATTR_CHANGE_LEFT) || attrs.hasAttribute(SUMO_ATTR_CHANGE_RIGHT)) {
+        const std::string changeLeft = attrs.getOpt<std::string>(SUMO_ATTR_CHANGE_LEFT, nullptr, ok, "");
+        const std::string changeRight = attrs.getOpt<std::string>(SUMO_ATTR_CHANGE_RIGHT, nullptr, ok, "");
+        myCurrentEdge->setPermittedChanging(lane, parseVehicleClasses(changeLeft, ""), parseVehicleClasses(changeRight, ""));
+    }
     // try to get the width
     if (attrs.hasAttribute(SUMO_ATTR_WIDTH)) {
         myCurrentEdge->setLaneWidth(lane, attrs.get<double>(SUMO_ATTR_WIDTH, myCurrentID.c_str(), ok));
@@ -333,38 +411,61 @@ NIXMLEdgesHandler::addLane(const SUMOSAXAttributes& attrs) {
     if (attrs.hasAttribute(SUMO_ATTR_ACCELERATION)) {
         myCurrentEdge->setAcceleration(lane, attrs.get<bool>(SUMO_ATTR_ACCELERATION, myCurrentID.c_str(), ok));
     }
+    // check whether this lane has a custom shape
+    if (attrs.hasAttribute(SUMO_ATTR_SHAPE)) {
+        PositionVector shape = attrs.get<PositionVector>(SUMO_ATTR_SHAPE, myCurrentID.c_str(), ok);
+        if (!NBNetBuilder::transformCoordinates(shape)) {
+            const std::string laneID = myCurrentID + "_" + toString(lane);
+            WRITE_ERROR("Unable to project coordinates for lane '" + laneID + "'.");
+        }
+        if (shape.size() == 1) {
+            // lane shape of length 1 is not permitted
+            shape.push_front(myCurrentEdge->getFromNode()->getPosition());
+            shape.push_back(myCurrentEdge->getToNode()->getPosition());
+        }
+        shape.removeDoublePoints();
+        if (shape.size() < 2) {
+            // ignore lane shape for very short lanes
+            shape.clear();
+        }
+        myCurrentEdge->setLaneShape(lane, shape);
+    }
+    // set custom lane type
+    if (attrs.hasAttribute(SUMO_ATTR_TYPE)) {
+        myCurrentEdge->setLaneType(lane, attrs.get<std::string>(SUMO_ATTR_TYPE, myCurrentID.c_str(), ok));
+    }
+    myLastParameterised.push_back(&myCurrentEdge->getLaneStruct(lane));
 }
 
 
 void NIXMLEdgesHandler::addSplit(const SUMOSAXAttributes& attrs) {
-    if (myCurrentEdge == 0) {
+    if (myCurrentEdge == nullptr) {
         if (!OptionsCont::getOptions().isInStringVector("remove-edges.explicit", myCurrentID)) {
             WRITE_WARNING("Ignoring 'split' because it cannot be assigned to an edge");
         }
         return;
     }
     bool ok = true;
-    Split e;
-    e.pos = attrs.get<double>(SUMO_ATTR_POSITION, 0, ok);
+    NBEdgeCont::Split e;
+    e.pos = attrs.get<double>(SUMO_ATTR_POSITION, nullptr, ok);
     if (ok) {
         if (fabs(e.pos) > myCurrentEdge->getGeometry().length()) {
             WRITE_ERROR("Edge '" + myCurrentID + "' has a split at invalid position " + toString(e.pos) + ".");
             return;
         }
-        std::vector<Split>::iterator i = find_if(mySplits.begin(), mySplits.end(), split_by_pos_finder(e.pos));
+        std::vector<NBEdgeCont::Split>::iterator i = find_if(mySplits.begin(), mySplits.end(), split_by_pos_finder(e.pos));
         if (i != mySplits.end()) {
             WRITE_ERROR("Edge '" + myCurrentID + "' has already a split at position " + toString(e.pos) + ".");
             return;
         }
-        const std::string nameid = toString((int)e.pos);
+        // XXX rounding to int may duplicate the id of another split
+        e.nameID = myCurrentID + "." + toString((int)e.pos);
         if (e.pos < 0) {
             e.pos += myCurrentEdge->getGeometry().length();
         }
-        std::vector<std::string> lanes;
-        SUMOSAXAttributes::parseStringVector(attrs.getOpt<std::string>(SUMO_ATTR_LANES, 0, ok, ""), lanes);
-        for (std::vector<std::string>::iterator i = lanes.begin(); i != lanes.end(); ++i) {
+        for (const std::string& id : attrs.getOptStringVector(SUMO_ATTR_LANES, myCurrentID.c_str(), ok)) {
             try {
-                int lane = TplConvert::_2int((*i).c_str());
+                int lane = StringUtils::toInt(id);
                 e.lanes.push_back(lane);
             } catch (NumberFormatException&) {
                 WRITE_ERROR("Error on parsing a split (edge '" + myCurrentID + "').");
@@ -377,19 +478,28 @@ void NIXMLEdgesHandler::addSplit(const SUMOSAXAttributes& attrs) {
                 e.lanes.push_back(l);
             }
         }
-        e.speed = attrs.getOpt(SUMO_ATTR_SPEED, 0, ok, myCurrentEdge->getSpeed());
+        e.speed = attrs.getOpt(SUMO_ATTR_SPEED, nullptr, ok, myCurrentEdge->getSpeed());
         if (attrs.hasAttribute(SUMO_ATTR_SPEED) && myOptions.getBool("speed-in-kmh")) {
-            e.speed /= (double) 3.6;
+            e.speed /= 3.6;
         }
-        e.idBefore = attrs.getOpt(SUMO_ATTR_ID_BEFORE, 0, ok, std::string(""));
-        e.idAfter = attrs.getOpt(SUMO_ATTR_ID_AFTER, 0, ok, std::string(""));
+        e.idBefore = attrs.getOpt(SUMO_ATTR_ID_BEFORE, nullptr, ok, std::string(""));
+        e.idAfter = attrs.getOpt(SUMO_ATTR_ID_AFTER, nullptr, ok, std::string(""));
         if (!ok) {
             return;
         }
-        e.node = new NBNode(myCurrentID + "." + nameid,
-                            myCurrentEdge->getGeometry().positionAtOffset(e.pos));
+        const std::string nodeID = attrs.getOpt(SUMO_ATTR_ID, nullptr, ok, e.nameID);
+        if (nodeID == myCurrentEdge->getFromNode()->getID() || nodeID == myCurrentEdge->getToNode()->getID()) {
+            WRITE_ERROR("Invalid split node id for edge '" + myCurrentEdge->getID() + "' (from- and to-node are forbidden)");
+            return;
+        }
+        e.node = myNodeCont.retrieve(nodeID);
+        e.offsetFactor = OptionsCont::getOptions().getBool("lefthand") ? -1 : 1;
+        if (e.node == nullptr) {
+            e.node = new NBNode(nodeID, myCurrentEdge->getGeometry().positionAtOffset(e.pos));
+            myNodeCont.insert(e.node);
+        }
         NIXMLNodesHandler::processNodeType(attrs, e.node, e.node->getID(), e.node->getPosition(), false,
-                                           myNodeCont, myTLLogicCont);
+                                           myNodeCont, myEdgeCont, myTLLogicCont);
         mySplits.push_back(e);
     }
 }
@@ -400,54 +510,60 @@ NIXMLEdgesHandler::setNodes(const SUMOSAXAttributes& attrs) {
     // the names and the coordinates of the beginning and the end node
     // may be found, try
     bool ok = true;
-    std::string begNodeID = myIsUpdate ? myCurrentEdge->getFromNode()->getID() : "";
-    std::string endNodeID = myIsUpdate ? myCurrentEdge->getToNode()->getID() : "";
-    std::string oldBegID = begNodeID;
-    std::string oldEndID = endNodeID;
+    if (myIsUpdate) {
+        myFromNode = myCurrentEdge->getFromNode();
+        myToNode = myCurrentEdge->getToNode();
+    }
     if (attrs.hasAttribute(SUMO_ATTR_FROM)) {
-        begNodeID = attrs.get<std::string>(SUMO_ATTR_FROM, 0, ok);
+        const std::string begNodeID = attrs.get<std::string>(SUMO_ATTR_FROM, nullptr, ok);
+        if (begNodeID != "") {
+            myFromNode = myNodeCont.retrieve(begNodeID);
+            if (myFromNode == nullptr) {
+                WRITE_ERROR("Edge's '" + myCurrentID + "' from-node '" + begNodeID + "' is not known.");
+            }
+        }
     } else if (!myIsUpdate) {
         WRITE_ERROR("The from-node is not given for edge '" + myCurrentID + "'.");
         ok = false;
     }
     if (attrs.hasAttribute(SUMO_ATTR_TO)) {
-        endNodeID = attrs.get<std::string>(SUMO_ATTR_TO, 0, ok);
+        const std::string endNodeID = attrs.get<std::string>(SUMO_ATTR_TO, nullptr, ok);
+        if (endNodeID != "") {
+            myToNode = myNodeCont.retrieve(endNodeID);
+            if (myToNode == nullptr) {
+                WRITE_ERROR("Edge's '" + myCurrentID + "' to-node '" + endNodeID + "' is not known.");
+            }
+        }
     } else if (!myIsUpdate) {
         WRITE_ERROR("The to-node is not given for edge '" + myCurrentID + "'.");
         ok = false;
     }
-    if (!ok) {
-        return false;
-    }
-    myFromNode = myNodeCont.retrieve(begNodeID);
-    myToNode = myNodeCont.retrieve(endNodeID);
-    if (myFromNode == 0) {
-        WRITE_ERROR("Edge's '" + myCurrentID + "' from-node '" + begNodeID + "' is not known.");
-    }
-    if (myToNode == 0) {
-        WRITE_ERROR("Edge's '" + myCurrentID + "' to-node '" + endNodeID + "' is not known.");
-    }
-    if (myFromNode != 0 && myToNode != 0) {
-        if (myIsUpdate && (myFromNode->getID() != oldBegID || myToNode->getID() != oldEndID)) {
-            myShape = PositionVector();
-        }
-    }
-    return myFromNode != 0 && myToNode != 0;
+    return ok && myFromNode != nullptr && myToNode != nullptr;
 }
 
 
 PositionVector
 NIXMLEdgesHandler::tryGetShape(const SUMOSAXAttributes& attrs) {
-    if (!attrs.hasAttribute(SUMO_ATTR_SHAPE)) {
+    if (!attrs.hasAttribute(SUMO_ATTR_SHAPE) && myShape.size() > 0) {
         return myShape;
     }
     // try to build shape
     bool ok = true;
     if (!attrs.hasAttribute(SUMO_ATTR_SHAPE)) {
-        myReinitKeepEdgeShape = false;
-        return PositionVector();
+        const double maxSegmentLength = OptionsCont::getOptions().getFloat("geometry.max-segment-length");
+        if (maxSegmentLength > 0) {
+            PositionVector shape;
+            shape.push_back(myFromNode->getPosition());
+            shape.push_back(myToNode->getPosition());
+            // shape is already cartesian but we must use a copy because the original will be modified
+            NBNetBuilder::addGeometrySegments(shape, PositionVector(shape), maxSegmentLength);
+            return shape;
+        } else {
+            myReinitKeepEdgeShape = false;
+            return PositionVector();
+        }
     }
-    PositionVector shape = attrs.getOpt<PositionVector>(SUMO_ATTR_SHAPE, 0, ok, PositionVector());
+    PositionVector shape = attrs.getOpt<PositionVector>(SUMO_ATTR_SHAPE, nullptr, ok, PositionVector());
     if (!NBNetBuilder::transformCoordinates(shape)) {
         WRITE_ERROR("Unable to project coordinates for edge '" + myCurrentID + "'.");
     }
@@ -474,13 +590,13 @@ NIXMLEdgesHandler::tryGetLaneSpread(const SUMOSAXAttributes& attrs) {
 void
 NIXMLEdgesHandler::deleteEdge(const SUMOSAXAttributes& attrs) {
     bool ok = true;
-    myCurrentID = attrs.get<std::string>(SUMO_ATTR_ID, 0, ok);
+    myCurrentID = attrs.get<std::string>(SUMO_ATTR_ID, nullptr, ok);
     if (!ok) {
         return;
     }
     NBEdge* edge = myEdgeCont.retrieve(myCurrentID);
-    if (edge == 0) {
-        WRITE_WARNING("Ignoring tag '" + toString(SUMO_TAG_DELETE) + "' for unknown edge '" +
+    if (edge == nullptr) {
+        WRITE_WARNING("Ignoring tag '" + toString(SUMO_TAG_DEL) + "' for unknown edge '" +
                       myCurrentID + "'");
         return;
     }
@@ -488,19 +604,18 @@ NIXMLEdgesHandler::deleteEdge(const SUMOSAXAttributes& attrs) {
     if (lane < 0) {
         myEdgeCont.extract(myDistrictCont, edge, true);
     } else {
-        edge->removeFromConnections(0, lane, -1, false, true);
-        const EdgeVector incoming = edge->getIncomingEdges();
-        for (EdgeVector::const_iterator e = incoming.begin(); e != incoming.end(); ++e) {
-            (*e)->removeFromConnections(edge, -1, lane, false, true);
-        }
-        edge->deleteLane(lane, false);
+        edge->deleteLane(lane, false, true);
     }
 }
 
 
 void
 NIXMLEdgesHandler::myEndElement(int element) {
-    if (element == SUMO_TAG_EDGE && myCurrentEdge != 0) {
+    if (myCurrentEdge == nullptr) {
+        return;
+    }
+    if (element == SUMO_TAG_EDGE) {
+        myLastParameterised.pop_back();
         // add bike lane, wait until lanes are loaded to avoid building if it already exists
         if (myBikeLaneWidth != NBEdge::UNSPECIFIED_WIDTH) {
             myCurrentEdge->addBikeLane(myBikeLaneWidth);
@@ -509,10 +624,17 @@ NIXMLEdgesHandler::myEndElement(int element) {
         if (mySidewalkWidth != NBEdge::UNSPECIFIED_WIDTH) {
             myCurrentEdge->addSidewalk(mySidewalkWidth);
         }
+        // apply default stopOffsets of edge to all lanes without specified stopOffset.
+        std::map<SVCPermissions, double> stopOffsets = myCurrentEdge->getStopOffsets(-1);
+        if (stopOffsets.size() != 0) {
+            for (int i = 0; i < (int)myCurrentEdge->getLanes().size(); i++) {
+                myCurrentEdge->setStopOffsets(i, stopOffsets, false);
+            }
+        }
         if (!myIsUpdate) {
             try {
                 if (!myEdgeCont.insert(myCurrentEdge)) {
-                    WRITE_ERROR("Duplicate edge occured. ID='" + myCurrentID + "'");
+                    WRITE_ERROR("Duplicate edge occurred. ID='" + myCurrentID + "'");
                     delete myCurrentEdge;
                 }
             } catch (InvalidArgument& e) {
@@ -522,132 +644,11 @@ NIXMLEdgesHandler::myEndElement(int element) {
                 WRITE_ERROR("An important information is missing in edge '" + myCurrentID + "'.");
             }
         }
-        if (mySplits.size() != 0) {
-            std::vector<Split>::iterator i;
-            NBEdge* e = myCurrentEdge;
-            sort(mySplits.begin(), mySplits.end(), split_sorter());
-            int noLanesMax = e->getNumLanes();
-            // compute the node positions and sort the lanes
-            for (i = mySplits.begin(); i != mySplits.end(); ++i) {
-                sort((*i).lanes.begin(), (*i).lanes.end());
-                noLanesMax = MAX2(noLanesMax, (int)(*i).lanes.size());
-            }
-            // split the edge
-            std::vector<int> currLanes;
-            for (int l = 0; l < e->getNumLanes(); ++l) {
-                currLanes.push_back(l);
-            }
-            if (e->getNumLanes() != (int)mySplits.back().lanes.size()) {
-                // invalidate traffic light definitions loaded from a SUMO network
-                // XXX it would be preferable to reconstruct the phase definitions heuristically
-                e->getToNode()->invalidateTLS(myTLLogicCont);
-                // if the number of lanes changes the connections should be
-                // recomputed
-                e->invalidateConnections(true);
-            }
-
-            std::string firstID = "";
-            double seen = 0;
-            for (i = mySplits.begin(); i != mySplits.end(); ++i) {
-                const Split& exp = *i;
-                assert(exp.lanes.size() != 0);
-                if (exp.pos > 0 && e->getGeometry().length() + seen > exp.pos && exp.pos > seen) {
-                    if (myNodeCont.insert(exp.node)) {
-                        myNodeCont.markAsSplit(exp.node);
-                        //  split the edge
-                        std::string idBefore = exp.idBefore == "" ? e->getID() : exp.idBefore;
-                        std::string idAfter = exp.idAfter == "" ? exp.node->getID() : exp.idAfter;
-                        if (firstID == "") {
-                            firstID = idBefore;
-                        }
-                        myEdgeCont.splitAt(myDistrictCont, e, exp.pos - seen, exp.node,
-                                           idBefore, idAfter, e->getNumLanes(), (int) exp.lanes.size(), exp.speed);
-                        seen = exp.pos;
-                        std::vector<int> newLanes = exp.lanes;
-                        NBEdge* pe = myEdgeCont.retrieve(idBefore);
-                        NBEdge* ne = myEdgeCont.retrieve(idAfter);
-                        // reconnect lanes
-                        pe->invalidateConnections(true);
-                        //  new on right
-                        int rightMostP = currLanes[0];
-                        int rightMostN = newLanes[0];
-                        for (int l = 0; l < (int) rightMostP - (int) rightMostN; ++l) {
-                            pe->addLane2LaneConnection(0, ne, l, NBEdge::L2L_VALIDATED, true);
-                        }
-                        //  new on left
-                        int leftMostP = currLanes.back();
-                        int leftMostN = newLanes.back();
-                        for (int l = 0; l < (int) leftMostN - (int) leftMostP; ++l) {
-                            pe->addLane2LaneConnection(pe->getNumLanes() - 1, ne, leftMostN - l - rightMostN, NBEdge::L2L_VALIDATED, true);
-                        }
-                        //  all other connected
-                        for (int l = 0; l < noLanesMax; ++l) {
-                            if (find(currLanes.begin(), currLanes.end(), l) == currLanes.end()) {
-                                continue;
-                            }
-                            if (find(newLanes.begin(), newLanes.end(), l) == newLanes.end()) {
-                                continue;
-                            }
-                            pe->addLane2LaneConnection(l - rightMostP, ne, l - rightMostN, NBEdge::L2L_VALIDATED, true);
-                        }
-                        // move to next
-                        e = ne;
-                        currLanes = newLanes;
-                    } else {
-                        WRITE_WARNING("Error on parsing a split (edge '" + myCurrentID + "').");
-                    }
-                }  else if (exp.pos == 0) {
-                    const int laneCountDiff = e->getNumLanes() - (int)exp.lanes.size();
-                    if (laneCountDiff < 0) {
-                        e->incLaneNo(-laneCountDiff);
-                    } else {
-                        e->decLaneNo(laneCountDiff);
-                    }
-                    currLanes = exp.lanes;
-                    // invalidate traffic light definition loaded from a SUMO network
-                    // XXX it would be preferable to reconstruct the phase definitions heuristically
-                    e->getFromNode()->invalidateTLS(myTLLogicCont);
-                } else {
-                    WRITE_WARNING("Split at '" + toString(exp.pos) + "' lies beyond the edge's length (edge '" + myCurrentID + "').");
-                }
-            }
-            // patch lane offsets
-            e = myEdgeCont.retrieve(firstID);
-            if (mySplits.front().pos != 0) {
-                // add a dummy split at the beginning to ensure correct offset
-                Split start;
-                start.pos = 0;
-                for (int lane = 0; lane < (int)e->getNumLanes(); ++lane) {
-                    start.lanes.push_back(lane);
-                }
-                mySplits.insert(mySplits.begin(), start);
-            }
-            i = mySplits.begin();
-            for (; i != mySplits.end(); ++i) {
-                int maxLeft = (*i).lanes.back();
-                double offset = 0;
-                if (maxLeft < noLanesMax) {
-                    if (e->getLaneSpreadFunction() == LANESPREAD_RIGHT) {
-                        offset = SUMO_const_laneWidthAndOffset * (noLanesMax - 1 - maxLeft);
-                    } else {
-                        offset = SUMO_const_halfLaneAndOffset * (noLanesMax - 1 - maxLeft);
-                    }
-                }
-                int maxRight = (*i).lanes.front();
-                if (maxRight > 0 && e->getLaneSpreadFunction() == LANESPREAD_CENTER) {
-                    offset -= SUMO_const_halfLaneAndOffset * maxRight;
-                }
-                if (offset != 0) {
-                    PositionVector g = e->getGeometry();
-                    g.move2side(offset);
-                    e->setGeometry(g);
-                }
-                if (e->getToNode()->getOutgoingEdges().size() != 0) {
-                    e = e->getToNode()->getOutgoingEdges()[0];
-                }
-            }
-        }
-        myCurrentEdge = 0;
+        myEdgeCont.processSplits(myCurrentEdge, mySplits, myNodeCont, myDistrictCont, myTLLogicCont);
+        myCurrentEdge = nullptr;
+    } else if (element == SUMO_TAG_LANE && myCurrentLaneIndex != -1) {
+        myLastParameterised.pop_back();
+        myCurrentLaneIndex = -1;
     }
 }
 
@@ -659,7 +660,7 @@ NIXMLEdgesHandler::addRoundabout(const SUMOSAXAttributes& attrs) {
         EdgeSet roundabout;
         for (std::vector<std::string>::iterator it = edgeIDs.begin(); it != edgeIDs.end(); ++it) {
             NBEdge* edge = myEdgeCont.retrieve(*it);
-            if (edge == 0) {
+            if (edge == nullptr) {
                 if (!myEdgeCont.wasIgnored(*it)) {
                     WRITE_ERROR("Unknown edge '" + (*it) + "' in roundabout");
                 }
@@ -674,6 +675,4 @@ NIXMLEdgesHandler::addRoundabout(const SUMOSAXAttributes& attrs) {
 }
 
 
-
 /****************************************************************************/
-
