@@ -1,35 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIXMLNodesHandler.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NIXMLNodesHandler.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Importer for network nodes stored in XML
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <iostream>
@@ -40,7 +31,7 @@
 #include <utils/xml/SUMOSAXHandler.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include <utils/common/MsgHandler.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include <utils/common/StringTokenizer.h>
 #include <utils/options/OptionsCont.h>
@@ -56,14 +47,16 @@
 // ===========================================================================
 // method definitions
 // ===========================================================================
-NIXMLNodesHandler::NIXMLNodesHandler(NBNodeCont& nc,
+NIXMLNodesHandler::NIXMLNodesHandler(NBNodeCont& nc, NBEdgeCont& ec,
                                      NBTrafficLightLogicCont& tlc,
                                      OptionsCont& options) :
     SUMOSAXHandler("xml-nodes - file"),
     myOptions(options),
     myNodeCont(nc),
+    myEdgeCont(ec),
     myTLLogicCont(tlc),
-    myLocation(0) {
+    myLocation(nullptr),
+    myLastParameterised(nullptr) {
 }
 
 
@@ -88,9 +81,30 @@ NIXMLNodesHandler::myStartElement(int element,
         case SUMO_TAG_JOINEXCLUDE:
             addJoinExclusion(attrs);
             break;
-        case SUMO_TAG_DELETE:
+        case SUMO_TAG_DEL:
             deleteNode(attrs);
             break;
+        case SUMO_TAG_PARAM:
+            if (myLastParameterised != nullptr) {
+                bool ok = true;
+                const std::string key = attrs.get<std::string>(SUMO_ATTR_KEY, nullptr, ok);
+                // circumventing empty string test
+                const std::string val = attrs.hasAttribute(SUMO_ATTR_VALUE) ? attrs.getString(SUMO_ATTR_VALUE) : "";
+                myLastParameterised->setParameter(key, val);
+            }
+            break;
+        default:
+            break;
+    }
+}
+
+
+void
+NIXMLNodesHandler::myEndElement(int element) {
+    switch (element) {
+        case SUMO_TAG_NODE:
+            myLastParameterised = nullptr;
+            break;
         default:
             break;
     }
@@ -101,7 +115,7 @@ void
 NIXMLNodesHandler::addNode(const SUMOSAXAttributes& attrs) {
     bool ok = true;
     // get the id, report a warning if not given or empty...
-    myID = attrs.get<std::string>(SUMO_ATTR_ID, 0, ok);
+    myID = attrs.get<std::string>(SUMO_ATTR_ID, nullptr, ok);
     if (!ok) {
         return;
     }
@@ -110,7 +124,7 @@ NIXMLNodesHandler::addNode(const SUMOSAXAttributes& attrs) {
     bool xOk = false;
     bool yOk = false;
     bool needConversion = true;
-    if (node != 0) {
+    if (node != nullptr) {
         myPosition = node->getPosition();
         xOk = yOk = true;
         needConversion = false;
@@ -137,37 +151,34 @@ NIXMLNodesHandler::addNode(const SUMOSAXAttributes& attrs) {
     } else {
         WRITE_ERROR("Missing position (at node ID='" + myID + "').");
     }
-    bool updateEdgeGeometries = node != 0 && myPosition != node->getPosition();
-    // check whether the y-axis shall be flipped
-    if (myOptions.getBool("flip-y-axis")) {
-        myPosition.mul(1.0, -1.0);
-    }
-    processNodeType(attrs, node, myID, myPosition, updateEdgeGeometries, myNodeCont, myTLLogicCont);
+    bool updateEdgeGeometries = node != nullptr && myPosition != node->getPosition();
+    node = processNodeType(attrs, node, myID, myPosition, updateEdgeGeometries, myNodeCont, myEdgeCont, myTLLogicCont);
+    myLastParameterised = node;
 }
 
 
-void
+NBNode*
 NIXMLNodesHandler::processNodeType(const SUMOSAXAttributes& attrs, NBNode* node, const std::string& nodeID, const Position& position,
                                    bool updateEdgeGeometries,
-                                   NBNodeCont& nc, NBTrafficLightLogicCont& tlc) {
+                                   NBNodeCont& nc, NBEdgeCont& ec, NBTrafficLightLogicCont& tlc) {
     bool ok = true;
     // get the type
-    SumoXMLNodeType type = NODETYPE_UNKNOWN;
-    if (node != 0) {
+    SumoXMLNodeType type = SumoXMLNodeType::UNKNOWN;
+    if (node != nullptr) {
         type = node->getType();
     }
     std::string typeS = attrs.getOpt<std::string>(SUMO_ATTR_TYPE, nodeID.c_str(), ok, "");
     if (SUMOXMLDefinitions::NodeTypes.hasString(typeS)) {
         type = SUMOXMLDefinitions::NodeTypes.get(typeS);
-        if (type == NODETYPE_DEAD_END_DEPRECATED || type == NODETYPE_DEAD_END) {
+        if (type == SumoXMLNodeType::DEAD_END_DEPRECATED || type == SumoXMLNodeType::DEAD_END) {
             // dead end is a computed status. Reset this to unknown so it will
             // be corrected if additional connections are loaded
-            type = NODETYPE_UNKNOWN;
+            type = SumoXMLNodeType::UNKNOWN;
         }
     }
     std::set<NBTrafficLightDefinition*> oldTLS;
     // check whether a prior node shall be modified
-    if (node == 0) {
+    if (node == nullptr) {
         node = new NBNode(nodeID, position, type);
         if (!nc.insert(node)) {
             throw ProcessError("Could not insert node though checked this before (id='" + nodeID + "').");
@@ -175,6 +186,9 @@ NIXMLNodesHandler::processNodeType(const SUMOSAXAttributes& attrs, NBNode* node,
     } else {
         // patch information
         oldTLS = node->getControllingTLS();
+        if (node->getType() == SumoXMLNodeType::PRIORITY && type == SumoXMLNodeType::RIGHT_BEFORE_LEFT) {
+            ec.removeRoundabout(node);
+        }
         node->reinit(position, type, updateEdgeGeometries);
     }
     // process traffic light definition
@@ -192,6 +206,9 @@ NIXMLNodesHandler::processNodeType(const SUMOSAXAttributes& attrs, NBNode* node,
     PositionVector shape;
     if (attrs.hasAttribute(SUMO_ATTR_SHAPE)) {
         shape = attrs.getOpt<PositionVector>(SUMO_ATTR_SHAPE, nodeID.c_str(), ok, PositionVector());
+        if (!NBNetBuilder::transformCoordinates(shape)) {
+            WRITE_ERROR("Unable to project node shape at node '" + node->getID() + "'.");
+        }
         if (shape.size() > 2) {
             shape.closePolygon();
         }
@@ -205,6 +222,21 @@ NIXMLNodesHandler::processNodeType(const SUMOSAXAttributes& attrs, NBNode* node,
     if (attrs.hasAttribute(SUMO_ATTR_KEEP_CLEAR)) {
         node->setKeepClear(attrs.get<bool>(SUMO_ATTR_KEEP_CLEAR, nodeID.c_str(), ok));
     }
+
+    // set optional right-of-way hint
+    if (attrs.hasAttribute(SUMO_ATTR_RIGHT_OF_WAY)) {
+        node->setRightOfWay(attrs.getRightOfWay(ok));
+    }
+
+    // set optional fringe type
+    if (attrs.hasAttribute(SUMO_ATTR_FRINGE)) {
+        node->setFringeType(attrs.getFringeType(ok));
+    }
+    // set optional name
+    if (attrs.hasAttribute(SUMO_ATTR_NAME)) {
+        node->setName(attrs.get<std::string>(SUMO_ATTR_NAME, nodeID.c_str(), ok));
+    }
+    return node;
 }
 
 
@@ -212,13 +244,13 @@ void
 NIXMLNodesHandler::deleteNode(const SUMOSAXAttributes& attrs) {
     bool ok = true;
     // get the id, report a warning if not given or empty...
-    myID = attrs.get<std::string>(SUMO_ATTR_ID, 0, ok);
+    myID = attrs.get<std::string>(SUMO_ATTR_ID, nullptr, ok);
     if (!ok) {
         return;
     }
     NBNode* node = myNodeCont.retrieve(myID);
-    if (node == 0) {
-        WRITE_WARNING("Ignoring tag '" + toString(SUMO_TAG_DELETE) + "' for unknown node '" +
+    if (node == nullptr) {
+        WRITE_WARNING("Ignoring tag '" + toString(SUMO_TAG_DEL) + "' for unknown node '" +
                       myID + "'");
         return;
     } else {
@@ -230,10 +262,26 @@ NIXMLNodesHandler::deleteNode(const SUMOSAXAttributes& attrs) {
 void
 NIXMLNodesHandler::addJoinCluster(const SUMOSAXAttributes& attrs) {
     bool ok = true;
-    const std::string clusterString = attrs.get<std::string>(SUMO_ATTR_NODES, 0, ok);
-    const std::vector<std::string> ids = StringTokenizer(clusterString).getVector();
+    const std::string clusterString = attrs.get<std::string>(SUMO_ATTR_NODES, nullptr, ok);
+    std::vector<std::string> ids = StringTokenizer(clusterString).getVector();
+    std::sort(ids.begin(), ids.end());
+
+    myID = attrs.getOpt<std::string>(SUMO_ATTR_ID, nullptr, ok, "cluster_" + joinToString(ids, "_"));
+
+    Position pos = Position::INVALID;
+    if (attrs.hasAttribute(SUMO_ATTR_X)) {
+        pos.setx(attrs.get<double>(SUMO_ATTR_X, myID.c_str(), ok));
+    }
+    if (attrs.hasAttribute(SUMO_ATTR_Y)) {
+        pos.sety(attrs.get<double>(SUMO_ATTR_Y, myID.c_str(), ok));
+    }
+    if (attrs.hasAttribute(SUMO_ATTR_Z)) {
+        pos.setz(attrs.get<double>(SUMO_ATTR_Z, myID.c_str(), ok));
+    }
+
+    NBNode* node = processNodeType(attrs, nullptr, myID, pos, false, myNodeCont, myEdgeCont, myTLLogicCont);
     if (ok) {
-        myNodeCont.addCluster2Join(std::set<std::string>(ids.begin(), ids.end()));
+        myNodeCont.addCluster2Join(std::set<std::string>(ids.begin(), ids.end()), node);
     }
 }
 
@@ -242,7 +290,7 @@ void
 NIXMLNodesHandler::addJoinExclusion(const SUMOSAXAttributes& attrs) {
     bool ok = true;
     const std::vector<std::string> ids = StringTokenizer(
-            attrs.get<std::string>(SUMO_ATTR_NODES, 0, ok)).getVector();
+            attrs.get<std::string>(SUMO_ATTR_NODES, nullptr, ok)).getVector();
     if (ok) {
         myNodeCont.addJoinExclusion(ids);
     }
@@ -267,8 +315,8 @@ NIXMLNodesHandler::processTrafficLightDefinitions(const SUMOSAXAttributes& attrs
         oldTlID = oldDef->getID();
         oldTypeS = toString(oldDef->getType());
     }
-    std::string tlID = attrs.getOpt<std::string>(SUMO_ATTR_TLID, 0, ok, oldTlID);
-    std::string typeS = attrs.getOpt<std::string>(SUMO_ATTR_TLTYPE, 0, ok, oldTypeS);
+    std::string tlID = attrs.getOpt<std::string>(SUMO_ATTR_TLID, nullptr, ok, oldTlID);
+    std::string typeS = attrs.getOpt<std::string>(SUMO_ATTR_TLTYPE, nullptr, ok, oldTypeS);
     if (tlID != oldTlID || typeS != oldTypeS) {
         currentNode->removeTrafficLights();
     }
@@ -279,33 +327,47 @@ NIXMLNodesHandler::processTrafficLightDefinitions(const SUMOSAXAttributes& attrs
         WRITE_ERROR("Unknown traffic light type '" + typeS + "' for node '" + currentNode->getID() + "'.");
         return;
     }
+    TrafficLightLayout layout = TrafficLightLayout::DEFAULT;
+    if (attrs.hasAttribute(SUMO_ATTR_TLLAYOUT)) {
+        std::string layoutS = attrs.get<std::string>(SUMO_ATTR_TLLAYOUT, nullptr, ok);
+        if (SUMOXMLDefinitions::TrafficLightLayouts.hasString(layoutS)) {
+            layout = SUMOXMLDefinitions::TrafficLightLayouts.get(layoutS);
+        } else {
+            WRITE_ERROR("Unknown traffic light layout '" + typeS + "' for node '" + currentNode->getID() + "'.");
+            return;
+        }
+    }
     if (tlID != "" && tlc.getPrograms(tlID).size() > 0) {
         // we already have definitions for this tlID
-        const std::map<std::string, NBTrafficLightDefinition*>& programs = tlc.getPrograms(tlID);
-        std::map<std::string, NBTrafficLightDefinition*>::const_iterator it;
-        for (it = programs.begin(); it != programs.end(); it++) {
-            if (it->second->getType() != type) {
-                WRITE_ERROR("Mismatched traffic light type '" + typeS + "' for tl '" + tlID + "'.");
-                ok = false;
-            } else {
-                tlDefs.insert(it->second);
-                it->second->addNode(currentNode);
+        for (auto item : tlc.getPrograms(tlID)) {
+            NBTrafficLightDefinition* def = item.second;
+            tlDefs.insert(def);
+            def->addNode(currentNode);
+            if (def->getType() != type && attrs.hasAttribute(SUMO_ATTR_TLTYPE)) {
+                WRITE_WARNINGF("Changing traffic light type '%' to '%' for tl '%'.", toString(def->getType()), typeS, tlID);
+                def->setType(type);
+                if (type != TrafficLightType::STATIC && dynamic_cast<NBLoadedSUMOTLDef*>(def) != nullptr) {
+                    dynamic_cast<NBLoadedSUMOTLDef*>(def)->guessMinMaxDuration();
+                }
+            }
+            if (layout != TrafficLightLayout::DEFAULT && dynamic_cast<NBOwnTLDef*>(def) != nullptr) {
+                dynamic_cast<NBOwnTLDef*>(def)->setLayout(layout);
             }
         }
     } else {
         // we need to add a new defition
         tlID = (tlID == "" ? currentNode->getID() : tlID);
-        NBTrafficLightDefinition* tlDef = new NBOwnTLDef(tlID, currentNode, 0, type);
+        NBOwnTLDef* tlDef = new NBOwnTLDef(tlID, currentNode, 0, type);
         if (!tlc.insert(tlDef)) {
             // actually, nothing should fail here
             delete tlDef;
             throw ProcessError("Could not allocate tls '" + currentNode->getID() + "'.");
         }
+        tlDef->setLayout(layout);
         tlDefs.insert(tlDef);
     }
     // process inner edges which shall be controlled
-    std::vector<std::string> controlledInner;
-    SUMOSAXAttributes::parseStringVector(attrs.getOpt<std::string>(SUMO_ATTR_CONTROLLED_INNER, 0, ok, ""), controlledInner);
+    const std::vector<std::string>& controlledInner = attrs.getOptStringVector(SUMO_ATTR_CONTROLLED_INNER, nullptr, ok);
     if (controlledInner.size() != 0) {
         for (std::set<NBTrafficLightDefinition*>::iterator it = tlDefs.begin(); it != tlDefs.end(); it++) {
             (*it)->addControlledInnerEdges(controlledInner);
@@ -314,6 +376,4 @@ NIXMLNodesHandler::processTrafficLightDefinitions(const SUMOSAXAttributes& attrs
 }
 
 
-
 /****************************************************************************/
-
