@@ -1,34 +1,25 @@
 /****************************************************************************/
-/// @file    NIXMLTrafficLightsHandler.h
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2011-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
+/// @file    NIXMLTrafficLightsHandler.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    2011-10-05
-/// @version $Id: NIXMLTrafficLightsHandler.cpp 23879 2017-04-07 12:31:47Z namdre $
 ///
 // Importer for traffic lights stored in XML
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <iostream>
@@ -40,7 +31,7 @@
 #include <utils/xml/SUMOSAXHandler.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include <utils/common/ToString.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/options/OptionsCont.h>
@@ -58,13 +49,14 @@
 // method definitions
 // ===========================================================================
 NIXMLTrafficLightsHandler::NIXMLTrafficLightsHandler(
-    NBTrafficLightLogicCont& tlCont, NBEdgeCont& ec) :
+    NBTrafficLightLogicCont& tlCont, NBEdgeCont& ec, bool ignoreUnknown) :
     SUMOSAXHandler("xml-tllogics"),
     myTLLCont(tlCont),
     myEdgeCont(ec),
-    myCurrentTL(0),
-    myResetPhases(false) {
-}
+    myCurrentTL(nullptr),
+    myResetPhases(false),
+    myIgnoreUnknown(ignoreUnknown)
+{ }
 
 
 NIXMLTrafficLightsHandler::~NIXMLTrafficLightsHandler() {}
@@ -78,20 +70,30 @@ NIXMLTrafficLightsHandler::myStartElement(
             myCurrentTL = initTrafficLightLogic(attrs, myCurrentTL);
             break;
         case SUMO_TAG_PHASE:
-            if (myCurrentTL != 0) {
+            if (myCurrentTL != nullptr) {
                 if (myResetPhases) {
                     myCurrentTL->getLogic()->resetPhases();
                     myResetPhases = false;
                 }
                 NIImporter_SUMO::addPhase(attrs, myCurrentTL);
+                myCurrentTL->phasesLoaded();
             }
             break;
         case SUMO_TAG_CONNECTION:
             addTlConnection(attrs);
             break;
-        case SUMO_TAG_DELETE:
+        case SUMO_TAG_DEL:
             removeTlConnection(attrs);
             break;
+        case SUMO_TAG_PARAM:
+            if (myCurrentTL != nullptr) {
+                bool ok = true;
+                const std::string key = attrs.get<std::string>(SUMO_ATTR_KEY, nullptr, ok);
+                // circumventing empty string test
+                const std::string val = attrs.hasAttribute(SUMO_ATTR_VALUE) ? attrs.getString(SUMO_ATTR_VALUE) : "";
+                myCurrentTL->setParameter(key, val);
+            }
+            break;
         default:
             break;
     }
@@ -102,11 +104,7 @@ void
 NIXMLTrafficLightsHandler::myEndElement(int element) {
     switch (element) {
         case SUMO_TAG_TLLOGIC:
-            if (!myCurrentTL) {
-                WRITE_ERROR("Unmatched closing tag for tlLogic.");
-            } else {
-                myCurrentTL = 0;
-            }
+            myCurrentTL = nullptr;
             break;
         default:
             break;
@@ -118,20 +116,20 @@ NBLoadedSUMOTLDef*
 NIXMLTrafficLightsHandler::initTrafficLightLogic(const SUMOSAXAttributes& attrs, NBLoadedSUMOTLDef* currentTL) {
     if (currentTL) {
         WRITE_ERROR("Definition of tlLogic '" + currentTL->getID() + "' was not finished.");
-        return 0;
+        return nullptr;
     }
     bool ok = true;
-    std::string id = attrs.get<std::string>(SUMO_ATTR_ID, 0, ok);
-    std::string programID = attrs.getOpt<std::string>(SUMO_ATTR_PROGRAMID, id.c_str(), ok, "<unknown>");
+    std::string id = attrs.get<std::string>(SUMO_ATTR_ID, nullptr, ok);
+    std::string programID = attrs.getOpt<std::string>(SUMO_ATTR_PROGRAMID, id.c_str(), ok, "UNKNOWN_PROGRAM");
     SUMOTime offset = attrs.hasAttribute(SUMO_ATTR_OFFSET) ? TIME2STEPS(attrs.get<double>(SUMO_ATTR_OFFSET, id.c_str(), ok)) : 0;
-    std::string typeS = attrs.getOpt<std::string>(SUMO_ATTR_TYPE, 0, ok,
+    std::string typeS = attrs.getOpt<std::string>(SUMO_ATTR_TYPE, nullptr, ok,
                         OptionsCont::getOptions().getString("tls.default-type"));
     TrafficLightType type;
     if (SUMOXMLDefinitions::TrafficLightTypes.hasString(typeS)) {
         type = SUMOXMLDefinitions::TrafficLightTypes.get(typeS);
     } else {
         WRITE_ERROR("Unknown traffic light type '" + typeS + "' for tlLogic '" + id + "'.");
-        return 0;
+        return nullptr;
     }
     // there are three scenarios to consider
     // 1) the tll.xml is loaded to update traffic lights defined in a net.xml:
@@ -142,39 +140,44 @@ NIXMLTrafficLightsHandler::initTrafficLightLogic(const SUMOSAXAttributes& attrs,
     //   there should be a definition with the same id but different programID
     const std::map<std::string, NBTrafficLightDefinition*>& programs = myTLLCont.getPrograms(id);
     if (programs.size() == 0) {
-        WRITE_ERROR("Cannot load traffic light program for unknown id '" + id + "', programID '" + programID + "'.");
-        return 0;
+        if (!myIgnoreUnknown) {
+            WRITE_ERROR("Cannot load traffic light program for unknown id '" + id + "', programID '" + programID + "'.");
+        }
+        return nullptr;
     }
     const std::string existingProgram = programs.begin()->first; // arbitrary for our purpose
     NBLoadedSUMOTLDef* loadedDef = dynamic_cast<NBLoadedSUMOTLDef*>(myTLLCont.getDefinition(id, programID));
-    if (loadedDef == 0) {
+    if (loadedDef == nullptr) {
         NBLoadedSUMOTLDef* oldDef = dynamic_cast<NBLoadedSUMOTLDef*>(myTLLCont.getDefinition(id, existingProgram));
-        if (oldDef == 0) {
+        if (oldDef == nullptr) {
             // case 2
             NBTrafficLightDefinition* newDef = dynamic_cast<NBOwnTLDef*>(myTLLCont.getDefinition(
                                                    id, NBTrafficLightDefinition::DefaultProgramID));
-            if (newDef == 0) {
+            bool deleteDefault = false;
+            if (newDef == nullptr) {
                 // the default program may have already been replaced with a loaded program
                 newDef = dynamic_cast<NBLoadedSUMOTLDef*>(myTLLCont.getDefinition(
                              id, NBTrafficLightDefinition::DefaultProgramID));
-                if (newDef == 0) {
+                if (newDef == nullptr) {
                     WRITE_ERROR("Cannot load traffic light program for unknown id '" + id + "', programID '" + programID + "'.");
-                    return 0;
+                    return nullptr;
                 }
+            } else {
+                deleteDefault = true;
             }
-            assert(newDef != 0);
+            assert(newDef != nullptr);
             loadedDef = new NBLoadedSUMOTLDef(id, programID, offset, type);
             // copy nodes and controlled inner edges
-            std::vector<NBNode*> nodes = newDef->getNodes();
-            for (std::vector<NBNode*>::iterator it = nodes.begin(); it != nodes.end(); it++) {
-                loadedDef->addNode(*it);
+            for (NBNode* const n : newDef->getNodes()) {
+                loadedDef->addNode(n);
             }
             loadedDef->addControlledInnerEdges(newDef->getControlledInnerEdges());
-            if (programID == NBTrafficLightDefinition::DefaultProgramID) {
+            if (deleteDefault) {
+                // make a copy because the vector is modified in the loop
+                const std::vector<NBNode*> nodes = newDef->getNodes();
                 // replace default Program
-                std::vector<NBNode*> nodes = newDef->getNodes();
-                for (std::vector<NBNode*>::iterator it = nodes.begin(); it != nodes.end(); it++) {
-                    (*it)->removeTrafficLight(newDef);
+                for (NBNode* const n : nodes) {
+                    n->removeTrafficLight(newDef);
                 }
                 myTLLCont.removeProgram(id, NBTrafficLightDefinition::DefaultProgramID);
             }
@@ -182,26 +185,30 @@ NIXMLTrafficLightsHandler::initTrafficLightLogic(const SUMOSAXAttributes& attrs,
         } else {
             // case 3
             NBTrafficLightLogic* oldLogic = oldDef->getLogic();
-            NBTrafficLightLogic* newLogic = new NBTrafficLightLogic(id, programID,
-                    oldLogic->getNumLinks(), offset, type);
-            loadedDef = new NBLoadedSUMOTLDef(oldDef, newLogic);
+            NBTrafficLightLogic newLogic(id, programID, oldLogic->getNumLinks(), offset, type);
+            loadedDef = new NBLoadedSUMOTLDef(*oldDef, newLogic);
             // copy nodes
             std::vector<NBNode*> nodes = oldDef->getNodes();
             for (std::vector<NBNode*>::iterator it = nodes.begin(); it != nodes.end(); it++) {
                 loadedDef->addNode(*it);
             }
+            //std::cout << " case3 oldDef=" << oldDef->getDescription() << " loadedDef=" << loadedDef->getDescription() << "\n";
             myTLLCont.insert(loadedDef);
         }
     } else {
         // case 1
-        loadedDef->setOffset(offset);
-        loadedDef->setType(type);
+        if (attrs.hasAttribute(SUMO_ATTR_OFFSET)) {
+            loadedDef->setOffset(offset);
+        }
+        if (attrs.hasAttribute(SUMO_ATTR_TYPE)) {
+            loadedDef->setType(type);
+        }
     }
     if (ok) {
         myResetPhases = true;
         return loadedDef;
     } else {
-        return 0;
+        return nullptr;
     }
 }
 
@@ -232,16 +239,21 @@ NIXMLTrafficLightsHandler::addTlConnection(const SUMOSAXAttributes& attrs) {
     }
     NBEdge::Connection c = *con_it;
     // read other  attributes
-    std::string tlID = attrs.getOpt<std::string>(SUMO_ATTR_TLID, 0, ok, "");
+    std::string tlID = attrs.getOpt<std::string>(SUMO_ATTR_TLID, nullptr, ok, "");
     if (tlID == "") {
         // we are updating an existing tl-controlled connection
-        tlID = c.tlID;
+        tlID = (*(from->getToNode()->getControllingTLS().begin()))->getID();
         assert(tlID != "");
     }
-    int tlIndex = attrs.getOpt<int>(SUMO_ATTR_TLLINKINDEX, 0, ok, -1);
+    int tlIndex = attrs.getOpt<int>(SUMO_ATTR_TLLINKINDEX, nullptr, ok, -1);
     if (tlIndex == -1) {
         // we are updating an existing tl-controlled connection
-        tlIndex = c.tlLinkNo;
+        tlIndex = c.tlLinkIndex;
+    }
+    int tlIndex2 = attrs.getOpt<int>(SUMO_ATTR_TLLINKINDEX2, nullptr, ok, -1);
+    if (tlIndex2 == -1) {
+        // we are updating an existing tl-controlled connection or index2 is not used
+        tlIndex2 = c.tlLinkIndex2;
     }
 
     // register the connection with all definitions
@@ -251,7 +263,7 @@ NIXMLTrafficLightsHandler::addTlConnection(const SUMOSAXAttributes& attrs) {
         for (it = programs.begin(); it != programs.end(); it++) {
             NBLoadedSUMOTLDef* tlDef = dynamic_cast<NBLoadedSUMOTLDef*>(it->second);
             if (tlDef) {
-                tlDef->addConnection(from, c.toEdge, c.fromLane, c.toLane, tlIndex);
+                tlDef->addConnection(from, c.toEdge, c.fromLane, c.toLane, tlIndex, tlIndex2,  false);
             } else {
                 throw ProcessError("Corrupt traffic light definition '"
                                    + tlID + "' (program '" + it->first + "')");
@@ -259,7 +271,7 @@ NIXMLTrafficLightsHandler::addTlConnection(const SUMOSAXAttributes& attrs) {
         }
     } else {
         SumoXMLNodeType type = from->getToNode()->getType();
-        if (type != NODETYPE_RAIL_CROSSING && type != NODETYPE_RAIL_SIGNAL) {
+        if (type != SumoXMLNodeType::RAIL_CROSSING && type != SumoXMLNodeType::RAIL_SIGNAL) {
             WRITE_ERROR("The traffic light '" + tlID + "' is not known.");
         }
     }
@@ -269,22 +281,20 @@ NIXMLTrafficLightsHandler::addTlConnection(const SUMOSAXAttributes& attrs) {
 void
 NIXMLTrafficLightsHandler::removeTlConnection(const SUMOSAXAttributes& attrs) {
     bool ok = true;
-    std::string tlID = attrs.get<std::string>(SUMO_ATTR_TLID, 0, ok);
+    std::string tlID = attrs.get<std::string>(SUMO_ATTR_TLID, nullptr, ok);
     // does the traffic light still exist?
     const std::map<std::string, NBTrafficLightDefinition*>& programs = myTLLCont.getPrograms(tlID);
     if (programs.size() > 0) {
         // parse identifying attributes
         NBEdge* from = retrieveEdge(attrs, SUMO_ATTR_FROM, ok);
         NBEdge* to = retrieveEdge(attrs, SUMO_ATTR_TO, ok);
-        if (!ok) {
-            return;
+        int fromLane = -1;
+        int toLane = -1;
+        if (ok) {
+            fromLane = retrieveLaneIndex(attrs, SUMO_ATTR_FROM_LANE, from, ok, true);
+            toLane = retrieveLaneIndex(attrs, SUMO_ATTR_TO_LANE, to, ok, true);
         }
-        int fromLane = retrieveLaneIndex(attrs, SUMO_ATTR_FROM_LANE, from, ok);
-        int toLane = retrieveLaneIndex(attrs, SUMO_ATTR_TO_LANE, to, ok);
-        if (!ok) {
-            return;
-        }
-        int tlIndex = attrs.get<int>(SUMO_ATTR_TLLINKINDEX, 0, ok);
+        int tlIndex = attrs.get<int>(SUMO_ATTR_TLLINKINDEX, nullptr, ok);
 
         NBConnection conn(from, fromLane, to, toLane, tlIndex);
         // remove the connection from all definitions
@@ -305,9 +315,9 @@ NIXMLTrafficLightsHandler::removeTlConnection(const SUMOSAXAttributes& attrs) {
 NBEdge*
 NIXMLTrafficLightsHandler::retrieveEdge(
     const SUMOSAXAttributes& attrs, SumoXMLAttr attr, bool& ok) {
-    std::string edgeID = attrs.get<std::string>(attr, 0, ok);
+    std::string edgeID = attrs.get<std::string>(attr, nullptr, ok);
     NBEdge* edge = myEdgeCont.retrieve(edgeID, true);
-    if (edge == 0) {
+    if (edge == nullptr) {
         WRITE_ERROR("Unknown edge '" + edgeID + "' given in connection.");
         ok = false;
     }
@@ -317,10 +327,12 @@ NIXMLTrafficLightsHandler::retrieveEdge(
 
 int
 NIXMLTrafficLightsHandler::retrieveLaneIndex(
-    const SUMOSAXAttributes& attrs, SumoXMLAttr attr, NBEdge* edge, bool& ok) {
-    int laneIndex = attrs.get<int>(attr, 0, ok);
+    const SUMOSAXAttributes& attrs, SumoXMLAttr attr, NBEdge* edge, bool& ok, bool isDelete) {
+    int laneIndex = attrs.get<int>(attr, nullptr, ok);
     if (edge->getNumLanes() <= laneIndex) {
-        WRITE_ERROR("Invalid lane index '" + toString(laneIndex) + "' for edge '" + edge->getID() + "'.");
+        if (!isDelete) {
+            WRITE_ERROR("Invalid lane index '" + toString(laneIndex) + "' for edge '" + edge->getID() + "'.");
+        }
         ok = false;
     }
     return laneIndex;
@@ -328,4 +340,3 @@ NIXMLTrafficLightsHandler::retrieveLaneIndex(
 
 
 /****************************************************************************/
-
