@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIImporter_Vissim.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -6,37 +18,16 @@
 /// @author  Lukas Grohmann (AIT)
 /// @author  Gerald Richter (AIT)
 /// @date    Sept 2002
-/// @version $Id: NIImporter_Vissim.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // -------------------
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 
 #include <string>
 #include <fstream>
 #include <utils/common/StringUtils.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/options/OptionsCont.h>
 #include <netbuild/NBNetBuilder.h>
@@ -117,13 +108,9 @@
 #include <utils/common/FileHelpers.h>
 #include <utils/common/StringTokenizer.h>
 #include <utils/distribution/Distribution_Points.h>
+#include <utils/distribution/DistributionCont.h>
 
 #include <netbuild/NBEdgeCont.h> // !!! only for debugging purposes
-#include <netbuild/NBDistribution.h>
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
 
 
 // ===========================================================================
@@ -137,6 +124,8 @@ StringBijection<int>::Entry NIImporter_Vissim::vissimTags[] = {
     { "links",            NIImporter_Vissim::VISSIM_TAG_LINKS },
     { "points3D",         NIImporter_Vissim::VISSIM_TAG_POINTS3D },
     { "point3D",          NIImporter_Vissim::VISSIM_TAG_POINT3D },
+    { "linkPolyPoint",    NIImporter_Vissim::VISSIM_TAG_LINKPOLYPOINT },
+    { "linkPolyPts",      NIImporter_Vissim::VISSIM_TAG_LINKPOLYPTS },
     { "fromLinkEndPt",    NIImporter_Vissim::VISSIM_TAG_FROM },
     { "toLinkEndPt",      NIImporter_Vissim::VISSIM_TAG_TO },
     { "vehicleInput",     NIImporter_Vissim::VISSIM_TAG_VEHICLE_INPUT },
@@ -204,20 +193,7 @@ NIImporter_Vissim::loadNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
     if (!oc.isSet("vissim-file")) {
         return;
     }
-    NIImporter_Vissim loader(nb, oc.getString("vissim-file"));
-    // check if legacy format file or newer XML file
-    // file name extension check
-    if ((oc.getString("vissim-file").find(".inpx") != std::string::npos))
-        //TODO: check if the given position of .inpx is at the end
-    {
-        // load the XML vissim network
-        loader.loadXML(oc, nb);
-        loader.myInputIsLegacyFormat = false;
-    } else {
-        // load the legacy vissim network
-        loader.load(oc);
-        loader.myInputIsLegacyFormat = true;
-    }
+    NIImporter_Vissim(nb).load(oc);
 }
 
 
@@ -246,23 +222,23 @@ NIImporter_Vissim::NIVissimXMLHandler_Streckendefinition::myStartElement(int ele
     if (element == VISSIM_TAG_LINK) {
         //parse all links
         bool ok = true;
-        int id = attrs.get<int>(VISSIM_ATTR_NO, 0, ok);
+        int id = attrs.get<int>(VISSIM_ATTR_NO, nullptr, ok);
         myLastNodeID = id;
 
         // !!! assuming empty myElemData
-        myElemData["id"].push_back(attrs.get<std::string>(VISSIM_ATTR_NO, 0, ok));
+        myElemData["id"].push_back(attrs.get<std::string>(VISSIM_ATTR_NO, nullptr, ok));
         // error ignored if name is empty
-        myElemData["name"].push_back(attrs.get<std::string>(VISSIM_ATTR_NAME, 0, ok, false));
-        myElemData["type"].push_back(attrs.get<std::string>(VISSIM_ATTR_LINKBEHAVETYPE, 0, ok));
-        myElemData["zuschlag1"].push_back(attrs.get<std::string>(VISSIM_ATTR_ZUSCHLAG1, 0, ok));
-        myElemData["zuschlag2"].push_back(attrs.get<std::string>(VISSIM_ATTR_ZUSCHLAG2, 0, ok));
+        myElemData["name"].push_back(attrs.get<std::string>(VISSIM_ATTR_NAME, nullptr, ok, false));
+        myElemData["type"].push_back(attrs.get<std::string>(VISSIM_ATTR_LINKBEHAVETYPE, nullptr, ok));
+        myElemData["zuschlag1"].push_back(attrs.get<std::string>(VISSIM_ATTR_ZUSCHLAG1, nullptr, ok));
+        myElemData["zuschlag2"].push_back(attrs.get<std::string>(VISSIM_ATTR_ZUSCHLAG2, nullptr, ok));
     }
 
     if (element == VISSIM_TAG_LANE) {
         bool ok = true;
         // appends empty element if no width found
         // error ignored if name is empty
-        myElemData["width"].push_back(attrs.get<std::string>(VISSIM_ATTR_WIDTH, 0, ok, false));
+        myElemData["width"].push_back(attrs.get<std::string>(VISSIM_ATTR_WIDTH, nullptr, ok, false));
     }
 
     if (element == VISSIM_TAG_FROM) {
@@ -271,8 +247,8 @@ NIImporter_Vissim::NIVissimXMLHandler_Streckendefinition::myStartElement(int ele
         }
         bool ok = true;
         std::vector<std::string> from(StringTokenizer(attrs.get<std::string>(
-                                          VISSIM_ATTR_LANE, 0, ok), " ").getVector());
-        myElemData["from_pos"].push_back(attrs.get<std::string>(VISSIM_ATTR_POS, 0, ok));
+                                          VISSIM_ATTR_LANE, nullptr, ok), " ").getVector());
+        myElemData["from_pos"].push_back(attrs.get<std::string>(VISSIM_ATTR_POS, nullptr, ok));
         myElemData["from_id"].push_back(from[0]);
         myElemData["from_lane"].push_back(from[1]);
     }
@@ -280,22 +256,22 @@ NIImporter_Vissim::NIVissimXMLHandler_Streckendefinition::myStartElement(int ele
     if (element == VISSIM_TAG_TO) {
         bool ok = true;
         std::vector<std::string> to(StringTokenizer(attrs.get<std::string>(
-                                        VISSIM_ATTR_LANE, 0, ok), " ").getVector());
-        myElemData["to_pos"].push_back(attrs.get<std::string>(VISSIM_ATTR_POS, 0, ok));
+                                        VISSIM_ATTR_LANE, nullptr, ok), " ").getVector());
+        myElemData["to_pos"].push_back(attrs.get<std::string>(VISSIM_ATTR_POS, nullptr, ok));
         myElemData["to_id"].push_back(to[0]);
         myElemData["to_lane"].push_back(to[1]);
     }
 
-    if (element == VISSIM_TAG_POINT3D) {
+    if (element == VISSIM_TAG_POINT3D || element == VISSIM_TAG_LINKPOLYPOINT) {
         bool ok = true;
         // create a <sep> separated string of coordinate data
         std::string sep(" ");
 
-        std::string posS(attrs.get<std::string>(VISSIM_ATTR_X, 0, ok));
+        std::string posS(attrs.get<std::string>(VISSIM_ATTR_X, nullptr, ok));
         posS += sep;
-        posS.append(attrs.get<std::string>(VISSIM_ATTR_Y, 0, ok));
+        posS.append(attrs.get<std::string>(VISSIM_ATTR_Y, nullptr, ok));
         // allow for no Z
-        std::string z(attrs.get<std::string>(VISSIM_ATTR_ZOFFSET, 0, ok, false));
+        std::string z(attrs.get<std::string>(VISSIM_ATTR_ZOFFSET, nullptr, ok, false));
         if (z.length() > 0) {
             posS += sep;
             posS.append(z);
@@ -313,7 +289,7 @@ NIImporter_Vissim::NIVissimXMLHandler_Streckendefinition::myEndElement(int eleme
 
         NIVissimClosedLanesVector clv;          //FIXME -> clv einlesen
         std::vector<int> assignedVehicles;      //FIXME -> assignedVehicles einlesen
-        int id(TplConvert::_str2int(myElemData["id"].front()));
+        int id(StringUtils::toInt(myElemData["id"].front()));
 
         PositionVector geom;
         // convert all position coordinate strings to PositionVectors
@@ -321,32 +297,30 @@ NIImporter_Vissim::NIVissimXMLHandler_Streckendefinition::myEndElement(int eleme
             std::vector<std::string> sPos_v(StringTokenizer(
                                                 myElemData["pos"].front(), " ").getVector());
             myElemData["pos"].pop_front();
-            std::vector<SUMOReal> pos_v(3);
+            std::vector<double> pos_v(3);
 
             // doing a transform with explicit hint on function signature
             std::transform(sPos_v.begin(), sPos_v.end(), pos_v.begin(),
-                           TplConvert::_str2SUMOReal);
+                           StringUtils::toDouble);
             geom.push_back_noDoublePos(Position(pos_v[0], pos_v[1], pos_v[2]));
         }
         // FIXME: a length = 0 PosVec seems fatal -> segfault
-        SUMOReal length(geom.length());
+        double length(geom.length());
 
         if (isConnector == false) {
             // Add Edge
+            std::vector<double> laneWidths;
+            for (std::string& w : myElemData["width"]) {
+                laneWidths.push_back(StringUtils::toDouble(w));
+            }
             NIVissimEdge* edge = new NIVissimEdge(id,
                                                   myElemData["name"].front(),
                                                   myElemData["type"].front(),
-                                                  (int)myElemData["width"].size(),   // numLanes,
-                                                  TplConvert::_str2SUMOReal(myElemData["zuschlag1"].front()),
-                                                  TplConvert::_str2SUMOReal(myElemData["zuschlag2"].front()),
+                                                  laneWidths,
+                                                  StringUtils::toDouble(myElemData["zuschlag1"].front()),
+                                                  StringUtils::toDouble(myElemData["zuschlag2"].front()),
                                                   length, geom, clv);
             NIVissimEdge::dictionary(id, edge);
-            if (id == 85 || id == 91) {
-                std::cout << id << "\n";
-                std::cout << myElemData["width"].size() << "\n";
-                std::cout << length << "\n";
-                std::cout << geom << "\n";
-            }
         } else {
             int numLanes = (int)myElemData["width"].size();
             std::vector<int> laneVec(numLanes);
@@ -354,30 +328,30 @@ NIImporter_Vissim::NIVissimXMLHandler_Streckendefinition::myEndElement(int eleme
 
             //NOTE: there should be only 1 lane number in XML
             // subtraction of 1 as in readExtEdgePointDef()
-            laneVec[0] = TplConvert::_str2int(myElemData["from_lane"].front()) - 1;
+            laneVec[0] = StringUtils::toInt(myElemData["from_lane"].front()) - 1;
             // then count up, building lane number vector
             for (std::vector<int>::iterator each = ++laneVec.begin(); each != laneVec.end(); ++each) {
                 *each = *(each - 1) + 1;
             }
 
             NIVissimExtendedEdgePoint from_def(
-                TplConvert::_str2int(myElemData["from_id"].front()),
+                StringUtils::toInt(myElemData["from_id"].front()),
                 laneVec,
-                TplConvert::_str2SUMOReal(myElemData["from_pos"].front()),
+                StringUtils::toDouble(myElemData["from_pos"].front()),
                 assignedVehicles);
 
             //NOTE: there should be only 1 lane number in XML
             // subtraction of 1 as in readExtEdgePointDef()
-            laneVec[0] = TplConvert::_str2int(myElemData["to_lane"].front()) - 1;
+            laneVec[0] = StringUtils::toInt(myElemData["to_lane"].front()) - 1;
             // then count up, building lane number vector
             for (std::vector<int>::iterator each = ++laneVec.begin(); each != laneVec.end(); ++each) {
                 *each = *(each - 1) + 1;
             }
 
             NIVissimExtendedEdgePoint to_def(
-                TplConvert::_str2int(myElemData["to_id"].front()),
+                StringUtils::toInt(myElemData["to_id"].front()),
                 laneVec,
-                TplConvert::_str2SUMOReal(myElemData["to_pos"].front()),
+                StringUtils::toDouble(myElemData["to_pos"].front()),
                 assignedVehicles);
 
             NIVissimConnection* connector = new
@@ -390,8 +364,10 @@ NIImporter_Vissim::NIVissimXMLHandler_Streckendefinition::myEndElement(int eleme
         }
         // clear the element data
         myElemData.clear();
+        isConnector = false;
         //std::cout << "elemData len (clear):" << myElemData.size() << std::endl;
         //std::cout.flush();
+
     }
     --myHierarchyLevel;
 }
@@ -414,9 +390,9 @@ NIImporter_Vissim::NIVissimXMLHandler_Zuflussdefinition::myStartElement(int elem
     if (element == VISSIM_TAG_VEHICLE_INPUT) {
         //parse all flows
         bool ok = true;
-        std::string id = attrs.get<std::string>(VISSIM_ATTR_NO, 0, ok);
-        std::string edgeid = attrs.get<std::string>(VISSIM_ATTR_LINK, 0, ok);
-        std::string name = attrs.get<std::string>(VISSIM_ATTR_NAME, 0, ok, false);
+        std::string id = attrs.get<std::string>(VISSIM_ATTR_NO, nullptr, ok);
+        std::string edgeid = attrs.get<std::string>(VISSIM_ATTR_LINK, nullptr, ok);
+        std::string name = attrs.get<std::string>(VISSIM_ATTR_NAME, nullptr, ok, false);
 
         NIVissimSource::dictionary(id,
                                    name,
@@ -441,18 +417,18 @@ NIImporter_Vissim::NIVissimXMLHandler_Parkplatzdefinition::myStartElement(int el
     if (element == VISSIM_TAG_PARKINGLOT) {
         //parse all parkinglots
         bool ok = true;
-        int id = attrs.get<int>(VISSIM_ATTR_NO, 0, ok);
-        int edgeid = attrs.get<int>(VISSIM_ATTR_INTLINK, 0, ok);
-        std::string name = attrs.get<std::string>(VISSIM_ATTR_NAME, 0, ok, false);
-        SUMOReal position = attrs.get<SUMOReal>(VISSIM_ATTR_POS, 0, ok);
+        int id = attrs.get<int>(VISSIM_ATTR_NO, nullptr, ok);
+        int edgeid = attrs.get<int>(VISSIM_ATTR_INTLINK, nullptr, ok);
+        std::string name = attrs.get<std::string>(VISSIM_ATTR_NAME, nullptr, ok, false);
+        double position = attrs.get<double>(VISSIM_ATTR_POS, nullptr, ok);
         std::vector<std::pair<int, int> > assignedVehicles; // (vclass, vwunsch)
         //FIXME: vWunsch + Fahzeugklassen einlesen
         // There can be s
         std::vector<int> districts;
         //FIXME: Parkplatzdefinition für mehrere Zonen implementieren
-        std::vector<SUMOReal> percentages;
-        districts.push_back(attrs.get<int>(VISSIM_ATTR_DISTRICT, 0, ok));
-        percentages.push_back(attrs.get<SUMOReal>(VISSIM_ATTR_PERCENTAGE, 0, ok));
+        std::vector<double> percentages;
+        districts.push_back(attrs.get<int>(VISSIM_ATTR_DISTRICT, nullptr, ok));
+        percentages.push_back(attrs.get<double>(VISSIM_ATTR_PERCENTAGE, nullptr, ok));
 
         NIVissimDistrictConnection::dictionary(id,
                                                name,
@@ -485,17 +461,17 @@ NIImporter_Vissim::NIVissimXMLHandler_Fahrzeugklassendefinition::myStartElement(
 
     if (element == VISSIM_TAG_VEHICLE_CLASS) {
         bool ok = true;
-        myElemData["id"].push_back(attrs.get<std::string>(VISSIM_ATTR_NO, 0, ok));
-        myElemData["name"].push_back(attrs.get<std::string>(VISSIM_ATTR_NAME, 0, ok, false));
-        std::string colorStr(attrs.get<std::string>(VISSIM_ATTR_COLOR, 0, ok));
-        for (size_t pos = colorStr.size() - 2; pos > 0; pos -= 2) {
+        myElemData["id"].push_back(attrs.get<std::string>(VISSIM_ATTR_NO, nullptr, ok));
+        myElemData["name"].push_back(attrs.get<std::string>(VISSIM_ATTR_NAME, nullptr, ok, false));
+        std::string colorStr(attrs.get<std::string>(VISSIM_ATTR_COLOR, nullptr, ok));
+        for (int pos = (int)colorStr.size() - 2; pos > 0; pos -= 2) {
             colorStr.insert(pos, " ");
         }
         myElemData["color"].push_back(colorStr);
     }
     if (element == VISSIM_TAG_INTOBJECTREF) {
         bool ok = true;
-        myElemData["types"].push_back(attrs.get<std::string>(VISSIM_ATTR_KEY, 0, ok));
+        myElemData["types"].push_back(attrs.get<std::string>(VISSIM_ATTR_KEY, nullptr, ok));
 
 
     }
@@ -509,8 +485,7 @@ NIImporter_Vissim::NIVissimXMLHandler_Fahrzeugklassendefinition::myEndElement(in
         std::vector<std::string> sCol_v(StringTokenizer(
                                             myElemData["color"].front(), " ").getVector());
         std::vector<int> myColorVector(sCol_v.size());
-        std::transform(sCol_v.begin(), sCol_v.end(), myColorVector.begin(),
-                       (TplConvert::_strHex2int));
+        std::transform(sCol_v.begin(), sCol_v.end(), myColorVector.begin(), StringUtils::hexToInt);
 
         color = RGBColor((unsigned char)myColorVector[0],
                          (unsigned char)myColorVector[1],
@@ -518,11 +493,11 @@ NIImporter_Vissim::NIVissimXMLHandler_Fahrzeugklassendefinition::myEndElement(in
                          (unsigned char)myColorVector[3]);
         std::vector<int> types;
         while (!myElemData["types"].empty()) {
-            types.push_back(TplConvert::_str2int(myElemData["types"].front()));
+            types.push_back(StringUtils::toInt(myElemData["types"].front()));
             myElemData["types"].pop_front();
         }
 
-        NIVissimVehTypeClass::dictionary(TplConvert::_str2int(myElemData["id"].front()),
+        NIVissimVehTypeClass::dictionary(StringUtils::toInt(myElemData["id"].front()),
                                          myElemData["name"].front(),
                                          color,
                                          types);
@@ -550,15 +525,15 @@ NIImporter_Vissim::NIVissimXMLHandler_Geschwindigkeitsverteilungsdefinition::myS
     myHierarchyLevel++;
     if (element == VISSIM_TAG_SPEED_DIST) {
         bool ok = true;
-        myElemData["id"].push_back(attrs.get<std::string>(VISSIM_ATTR_NO, 0, ok));
+        myElemData["id"].push_back(attrs.get<std::string>(VISSIM_ATTR_NO, nullptr, ok));
     }
 
     if (element == VISSIM_TAG_DATAPOINT) {
         bool ok = true;
         std::string sep(" ");
-        std::string posS(attrs.get<std::string>(VISSIM_ATTR_X, 0, ok));
+        std::string posS(attrs.get<std::string>(VISSIM_ATTR_X, nullptr, ok));
         posS += sep;
-        posS.append(attrs.get<std::string>(VISSIM_ATTR_FX, 0, ok));
+        posS.append(attrs.get<std::string>(VISSIM_ATTR_FX, nullptr, ok));
         myElemData["points"].push_back(posS);
 
     }
@@ -568,22 +543,14 @@ NIImporter_Vissim::NIVissimXMLHandler_Geschwindigkeitsverteilungsdefinition::myS
 void
 NIImporter_Vissim::NIVissimXMLHandler_Geschwindigkeitsverteilungsdefinition::myEndElement(int element) {
     if (element == VISSIM_TAG_SPEED_DIST && myHierarchyLevel == 3) {
-        PositionVector points;
+        Distribution_Points* points = new Distribution_Points(myElemData["id"].front());
         while (!myElemData["points"].empty()) {
             std::vector<std::string> sPos_v(StringTokenizer(
                                                 myElemData["points"].front(), " ").getVector());
             myElemData["points"].pop_front();
-            std::vector<SUMOReal> pos_v(2);
-
-            // doing a transform with explicit hint on function signature
-            std::transform(sPos_v.begin(), sPos_v.end(), pos_v.begin(),
-                           TplConvert::_str2SUMOReal);
-            points.push_back_noDoublePos(Position(pos_v[0], pos_v[1]));
+            points->add(StringUtils::toDouble(sPos_v[0]), StringUtils::toDouble(sPos_v[1]));
         }
-        NBDistribution::dictionary("speed",
-                                   myElemData["id"].front(),
-                                   new Distribution_Points(myElemData["id"].front(),
-                                           points));
+        DistributionCont::dictionary("speed", myElemData["id"].front(), points);
         myElemData.clear();
     }
     --myHierarchyLevel;
@@ -608,7 +575,7 @@ NIImporter_Vissim::NIVissimXMLHandler_VWunschentscheidungsdefinition::myStartEle
     myHierarchyLevel++;
     if (element == VISSIM_TAG_SPEED_DECISION) {
         bool ok = true;
-        myElemData["name"].push_back(attrs.get<std::string>(VISSIM_ATTR_NAME, 0, ok, false));
+        myElemData["name"].push_back(attrs.get<std::string>(VISSIM_ATTR_NAME, nullptr, ok, false));
         //FIXME: 2 vWunsch in the xml file, but only 1 of them is set???
     }
 
@@ -639,14 +606,14 @@ NIImporter_Vissim::NIVissimXMLHandler_Routenentscheidungsdefinition::myStartElem
     myHierarchyLevel++;
     if (element == VISSIM_TAG_DECISION_STATIC) {
         bool ok = true;
-        myElemData["startLink"].push_back(attrs.get<std::string>(VISSIM_ATTR_LINK, 0, ok));
-        myElemData["startPos"].push_back(attrs.get<std::string>(VISSIM_ATTR_POS, 0, ok));
+        myElemData["startLink"].push_back(attrs.get<std::string>(VISSIM_ATTR_LINK, nullptr, ok));
+        myElemData["startPos"].push_back(attrs.get<std::string>(VISSIM_ATTR_POS, nullptr, ok));
     }
     if (element == VISSIM_TAG_ROUTE_STATIC) {
         bool ok = true;
-        myElemData["destLink"].push_back(attrs.get<std::string>(VISSIM_ATTR_DESTLINK, 0, ok));
-        myElemData["destPos"].push_back(attrs.get<std::string>(VISSIM_ATTR_DESTPOS, 0, ok));
-        myElemData["id"].push_back(attrs.get<std::string>(VISSIM_ATTR_NO, 0, ok));
+        myElemData["destLink"].push_back(attrs.get<std::string>(VISSIM_ATTR_DESTLINK, nullptr, ok));
+        myElemData["destPos"].push_back(attrs.get<std::string>(VISSIM_ATTR_DESTPOS, nullptr, ok));
+        myElemData["id"].push_back(attrs.get<std::string>(VISSIM_ATTR_NO, nullptr, ok));
     }
     if (element == VISSIM_TAG_INTOBJECTREF) {
         // bool ok = true;
@@ -675,12 +642,12 @@ NIImporter_Vissim::NIVissimXMLHandler_ConflictArea::myStartElement(int element,
     if (element == VISSIM_TAG_CA) {
         //parse all flows
         bool ok = true;
-        std::string status = attrs.get<std::string>(VISSIM_ATTR_STATUS, 0, ok);
+        std::string status = attrs.get<std::string>(VISSIM_ATTR_STATUS, nullptr, ok);
         //get only the conflict areas which were set in VISSIM
         if (status != "PASSIVE") {
-            NIVissimConflictArea::dictionary(attrs.get<int>(VISSIM_ATTR_NO, 0, ok),
-                                             attrs.get<std::string>(VISSIM_ATTR_LINK1, 0, ok),
-                                             attrs.get<std::string>(VISSIM_ATTR_LINK2, 0, ok),
+            NIVissimConflictArea::dictionary(attrs.get<int>(VISSIM_ATTR_NO, nullptr, ok),
+                                             attrs.get<std::string>(VISSIM_ATTR_LINK1, nullptr, ok),
+                                             attrs.get<std::string>(VISSIM_ATTR_LINK2, nullptr, ok),
                                              status);
         }
 
@@ -778,7 +745,7 @@ NIImporter_Vissim::VissimSingleTypeParser::overrideOptionalLabel(std::istream& f
 
 Position
 NIImporter_Vissim::VissimSingleTypeParser::getPosition(std::istream& from) {
-    SUMOReal x, y;
+    double x, y;
     from >> x; // type-checking is missing!
     from >> y; // type-checking is missing!
     return Position(x, y);
@@ -795,7 +762,7 @@ NIImporter_Vissim::VissimSingleTypeParser::parseAssignedVehicleTypes(
         return ret;
     }
     while (tmp != "DATAEND" && tmp != next) {
-        ret.push_back(TplConvert::_2int(tmp.c_str()));
+        ret.push_back(StringUtils::toInt(tmp));
         tmp = readEndSecure(from);
     }
     return ret;
@@ -814,11 +781,11 @@ NIImporter_Vissim::VissimSingleTypeParser::readExtEdgePointDef(
     while (tag != "bei") {
         tag = readEndSecure(from);
         if (tag != "bei") {
-            int lane = TplConvert::_2int(tag.c_str());
+            int lane = StringUtils::toInt(tag);
             lanes.push_back(lane - 1);
         }
     }
-    SUMOReal position;
+    double position;
     from >> position;
     std::vector<int> dummy;
     return NIVissimExtendedEdgePoint(edgeid, lanes, position, dummy);
@@ -868,9 +835,7 @@ NIImporter_Vissim::VissimSingleTypeParser::skipOverreading(std::istream& from,
 /* -------------------------------------------------------------------------
  * NIImporter_Vissim-methods
  * ----------------------------------------------------------------------- */
-NIImporter_Vissim::NIImporter_Vissim(NBNetBuilder& nb, const std::string& file)
-    : myNetBuilder(nb), myInputIsLegacyFormat(false) {
-    UNUSED_PARAMETER(file);
+NIImporter_Vissim::NIImporter_Vissim(NBNetBuilder& nb) : myNetBuilder(nb) {
     insertKnownElements();
     buildParsers();
     myColorMap["blau"] = RGBColor(77, 77, 255, 255);
@@ -908,95 +873,90 @@ NIImporter_Vissim::~NIImporter_Vissim() {
     NIVissimConnection::clearDict();
     NIVissimConflictArea::clearDict();
     for (ToParserMap::iterator i = myParsers.begin(); i != myParsers.end(); i++) {
-        delete(*i).second;
+        delete (*i).second;
     }
 }
 
 
 void
 NIImporter_Vissim::load(const OptionsCont& options) {
-    // load file contents
+    const std::string file = options.getString("vissim-file");
     // try to open the file
-    std::ifstream strm(options.getString("vissim-file").c_str());
+    std::ifstream strm(file.c_str());
     if (!strm.good()) {
-        WRITE_ERROR("The vissim-file '" + options.getString("vissim-file") + "' was not found.");
-        return;
-    }
-    if (!readContents(strm)) {
-        return;
-    }
-    postLoadBuild(options.getFloat("vissim.join-distance"));
-}
-
-void
-NIImporter_Vissim::loadXML(const OptionsCont& options, NBNetBuilder& /* nb */) {
-    // Parse file
-    std::string file = options.getString("vissim-file");
-    // Create NIVissimXMLHandlers
-    NIVissimXMLHandler_Streckendefinition XMLHandler_Streckendefinition(elementData);
-    NIVissimXMLHandler_Zuflussdefinition XMLHandler_Zuflussdefinition;
-    //NIVissimXMLHandler_Parkplatzdefinition XMLHandler_Parkplatzdefinition;
-    NIVissimXMLHandler_Fahrzeugklassendefinition XMLHandler_Fahrzeugklassendefinition(elementData);
-    NIVissimXMLHandler_Geschwindigkeitsverteilungsdefinition XMLHandler_Geschwindigkeitsverteilung(elementData);
-    NIVissimXMLHandler_ConflictArea XMLHandler_ConflictAreas;
-    if (!FileHelpers::isReadable(file)) {
         WRITE_ERROR("Could not open vissim-file '" + file + "'.");
         return;
     }
+    std::string token;
+    strm >> token;
+    if (StringUtils::endsWith(file, ".inpx") || StringUtils::endsWith(token, "<?xml") || StringUtils::endsWith(token, "<network")) {
+        // Create NIVissimXMLHandlers
+        NIVissimXMLHandler_Streckendefinition XMLHandler_Streckendefinition(elementData);
+        NIVissimXMLHandler_Zuflussdefinition XMLHandler_Zuflussdefinition;
+        //NIVissimXMLHandler_Parkplatzdefinition XMLHandler_Parkplatzdefinition;
+        NIVissimXMLHandler_Fahrzeugklassendefinition XMLHandler_Fahrzeugklassendefinition(elementData);
+        NIVissimXMLHandler_Geschwindigkeitsverteilungsdefinition XMLHandler_Geschwindigkeitsverteilung(elementData);
+        NIVissimXMLHandler_ConflictArea XMLHandler_ConflictAreas;
 
-    // Strecken + Verbinder
-    XMLHandler_Streckendefinition.setFileName(file);
-    PROGRESS_BEGIN_MESSAGE("Parsing strecken+verbinder from vissim-file '" + file + "'");
-    if (!XMLSubSys::runParser(XMLHandler_Streckendefinition, file)) {
+        // Strecken + Verbinder
+        XMLHandler_Streckendefinition.setFileName(file);
+        PROGRESS_BEGIN_MESSAGE("Parsing strecken+verbinder from vissim-file '" + file + "'");
+        if (!XMLSubSys::runParser(XMLHandler_Streckendefinition, file)) {
+            return;
+        }
+        PROGRESS_DONE_MESSAGE();
+
+        // Zuflüsse
+        XMLHandler_Zuflussdefinition.setFileName(file);
+        PROGRESS_BEGIN_MESSAGE("Parsing zuflüsse from vissim-file '" + file + "'");
+        if (!XMLSubSys::runParser(XMLHandler_Zuflussdefinition, file)) {
+            return;
+        }
+        PROGRESS_DONE_MESSAGE();
+
+        //Geschwindigkeitsverteilungen
+        XMLHandler_Geschwindigkeitsverteilung.setFileName(file);
+        PROGRESS_BEGIN_MESSAGE("Parsing parkplätze from vissim-file '" + file + "'");
+        if (!XMLSubSys::runParser(XMLHandler_Geschwindigkeitsverteilung, file)) {
+            return;
+        }
+        PROGRESS_DONE_MESSAGE();
+
+
+        //Fahrzeugklassen
+        XMLHandler_Fahrzeugklassendefinition.setFileName(file);
+        PROGRESS_BEGIN_MESSAGE("Parsing parkplätze from vissim-file '" + file + "'");
+        if (!XMLSubSys::runParser(XMLHandler_Fahrzeugklassendefinition, file)) {
+            return;
+        }
+        PROGRESS_DONE_MESSAGE();
+
+        //Parkplätze
+        /*XMLHandler_Parkplatzdefinition.setFileName(file);
+        PROGRESS_BEGIN_MESSAGE("Parsing parkplätze from vissim-file '" + file + "'");
+        if (!XMLSubSys::runParser(XMLHandler_Parkplatzdefinition, file)) {
         return;
+        }
+        PROGRESS_DONE_MESSAGE();*/
+
+
+        //Konfliktflächen
+        XMLHandler_ConflictAreas.setFileName(file);
+        PROGRESS_BEGIN_MESSAGE("Parsing conflict areas from vissim-file '" + file + "'");
+        if (!XMLSubSys::runParser(XMLHandler_ConflictAreas, file)) {
+            return;
+        }
+        PROGRESS_DONE_MESSAGE();
+    } else {
+        strm.seekg(strm.beg);
+        if (!readContents(strm)) {
+            return;
+        }
     }
-    PROGRESS_DONE_MESSAGE();
-
-    // Zuflüsse
-    XMLHandler_Zuflussdefinition.setFileName(file);
-    PROGRESS_BEGIN_MESSAGE("Parsing zuflüsse from vissim-file '" + file + "'");
-    if (!XMLSubSys::runParser(XMLHandler_Zuflussdefinition, file)) {
-        return;
-    }
-    PROGRESS_DONE_MESSAGE();
-
-    //Geschwindigkeitsverteilungen
-    XMLHandler_Geschwindigkeitsverteilung.setFileName(file);
-    PROGRESS_BEGIN_MESSAGE("Parsing parkplätze from vissim-file '" + file + "'");
-    if (!XMLSubSys::runParser(XMLHandler_Geschwindigkeitsverteilung, file)) {
-        return;
-    }
-    PROGRESS_DONE_MESSAGE();
-
-
-    //Fahrzeugklassen
-    XMLHandler_Fahrzeugklassendefinition.setFileName(file);
-    PROGRESS_BEGIN_MESSAGE("Parsing parkplätze from vissim-file '" + file + "'");
-    if (!XMLSubSys::runParser(XMLHandler_Fahrzeugklassendefinition, file)) {
-        return;
-    }
-    PROGRESS_DONE_MESSAGE();
-
-    //Parkplätze
-    /*XMLHandler_Parkplatzdefinition.setFileName(file);
-    PROGRESS_BEGIN_MESSAGE("Parsing parkplätze from vissim-file '" + file + "'");
-    if (!XMLSubSys::runParser(XMLHandler_Parkplatzdefinition, file)) {
-        return;
-    }
-    PROGRESS_DONE_MESSAGE();*/
-
-
-    //Konfliktflächen
-    XMLHandler_ConflictAreas.setFileName(file);
-    PROGRESS_BEGIN_MESSAGE("Parsing conflict areas from vissim-file '" + file + "'");
-    if (!XMLSubSys::runParser(XMLHandler_ConflictAreas, file)) {
-        return;
-    }
-    PROGRESS_DONE_MESSAGE();
-
     postLoadBuild(options.getFloat("vissim.join-distance"));
 }
 
+
 bool
 NIImporter_Vissim::admitContinue(const std::string& tag) {
     ToElemIDMap::const_iterator i = myKnownElements.find(tag);
@@ -1051,7 +1011,7 @@ NIImporter_Vissim::readContents(std::istream& strm) {
 
 
 void
-NIImporter_Vissim::postLoadBuild(SUMOReal offset) {
+NIImporter_Vissim::postLoadBuild(double offset) {
     // close the loading process
     NIVissimBoundedClusterObject::closeLoading();
     NIVissimConnection::dict_assignToEdges();
@@ -1302,6 +1262,4 @@ NIImporter_Vissim::buildParsers() {
 }
 
 
-
 /****************************************************************************/
-
