@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIImporter_Vissim.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Lukas Grohmann (AIT)
 /// @author  Gerald Richter (AIT)
 /// @date    Sept 2002
-/// @version $Id: NIImporter_Vissim.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Importer for networks stored in Vissim format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIImporter_Vissim_h
-#define NIImporter_Vissim_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -481,7 +471,7 @@ private:
 
 protected:
     /// constructor
-    NIImporter_Vissim(NBNetBuilder& nb, const std::string& file);
+    NIImporter_Vissim(NBNetBuilder& nb);
 
     /// destructor
     ~NIImporter_Vissim();
@@ -489,8 +479,6 @@ protected:
     /// loads the vissim file
     void load(const OptionsCont& options);
 
-    void loadXML(const OptionsCont& options, NBNetBuilder& nb);
-
     bool admitContinue(const std::string& tag);
 
 public:
@@ -503,7 +491,7 @@ public:
         virtual ~VissimSingleTypeParser();
 
         /** @brief Parses a single data type.
-            Returns whether no error occured */
+            Returns whether no error occurred */
         virtual bool parse(std::istream& from) = 0;
 
     protected:
@@ -587,8 +575,6 @@ private:
 
     NBNetBuilder& myNetBuilder;
 
-    bool myInputIsLegacyFormat;
-
 private:
     /// @brief Invalidated copy constructor.
     NIImporter_Vissim(const NIImporter_Vissim&);
@@ -612,6 +598,8 @@ private:
         VISSIM_TAG_LINKS,
         VISSIM_TAG_POINTS3D,
         VISSIM_TAG_POINT3D,
+        VISSIM_TAG_LINKPOLYPOINT,
+        VISSIM_TAG_LINKPOLYPTS,
         VISSIM_TAG_FROM,
         VISSIM_TAG_TO,
         VISSIM_TAG_VEHICLE_INPUT,
@@ -667,9 +655,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
