@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimConnection.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -6,31 +18,10 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: NIVissimConnection.cpp 21202 2016-07-19 13:40:35Z behrisch $
 ///
 // -------------------
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -52,10 +43,6 @@
 #include "NIVissimConnection.h"
 #include <utils/common/UtilExceptions.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static members
@@ -79,7 +66,7 @@ NIVissimConnection::NIVissimConnection(int id,
 
 NIVissimConnection::~NIVissimConnection() {
     for (NIVissimClosedLanesVector::iterator i = myClosedLanes.begin(); i != myClosedLanes.end(); i++) {
-        delete(*i);
+        delete (*i);
     }
     myClosedLanes.clear();
 }
@@ -101,7 +88,7 @@ NIVissimConnection*
 NIVissimConnection::dictionary(int id) {
     DictType::iterator i = myDict.find(id);
     if (i == myDict.end()) {
-        return 0;
+        return nullptr;
     }
     return (*i).second;
 }
@@ -176,13 +163,13 @@ NIVissimConnection::getToEdgeID() const {
 }
 
 
-SUMOReal
+double
 NIVissimConnection::getFromPosition() const {
     return myFromDef.getPosition();
 }
 
 
-SUMOReal
+double
 NIVissimConnection::getToPosition() const {
     return myToDef.getPosition();
 }
@@ -222,13 +209,13 @@ int
 NIVissimConnection::buildEdgeConnections(NBEdgeCont& ec) {
     int unsetConnections = 0;
     // try to determine the connected edges
-    NBEdge* fromEdge = 0;
-    NBEdge* toEdge = 0;
+    NBEdge* fromEdge = nullptr;
+    NBEdge* toEdge = nullptr;
     NIVissimEdge* vissimFrom = NIVissimEdge::dictionary(getFromEdgeID());
     if (vissimFrom->wasWithinAJunction()) {
         // this edge was not built, try to get one that approaches it
         vissimFrom = vissimFrom->getBestIncoming();
-        if (vissimFrom != 0) {
+        if (vissimFrom != nullptr) {
             fromEdge = ec.retrievePossiblySplit(toString(vissimFrom->getID()), toString(getFromEdgeID()), true);
         }
     } else {
@@ -238,7 +225,7 @@ NIVissimConnection::buildEdgeConnections(NBEdgeCont& ec) {
     NIVissimEdge* vissimTo = NIVissimEdge::dictionary(getToEdgeID());
     if (vissimTo->wasWithinAJunction()) {
         vissimTo = vissimTo->getBestOutgoing();
-        if (vissimTo != 0) {
+        if (vissimTo != nullptr) {
             toEdge = ec.retrievePossiblySplit(toString(vissimTo->getID()), toString(getToEdgeID()), true);
         }
     } else {
@@ -250,7 +237,7 @@ NIVissimConnection::buildEdgeConnections(NBEdgeCont& ec) {
     NBEdge *fromEdge = ec.retrievePossiblySplit(toString(getFromEdgeID()), toString(getToEdgeID()), true);
     NBEdge *toEdge = ec.retrievePossiblySplit(toString(getToEdgeID()), toString(getFromEdgeID()), false);
     */
-    if (fromEdge == 0 || toEdge == 0) {
+    if (fromEdge == nullptr || toEdge == nullptr) {
         WRITE_WARNING("Could not build connection between '" + toString(getFromEdgeID()) + "' and '" + toString(getToEdgeID()) + "'.");
         return 1; // !!! actually not 1
     }
@@ -264,7 +251,7 @@ NIVissimConnection::buildEdgeConnections(NBEdgeCont& ec) {
             if (fromEdge->getNumLanes() <= fromLanes[index]) {
                 WRITE_WARNING("Could not set connection between '" + fromEdge->getID() + "_" + toString(fromLanes[index]) + "' and '" + toEdge->getID() + "_" + toString(toLanes[index]) + "'.");
                 ++unsetConnections;
-            } else if (!fromEdge->addLane2LaneConnection(fromLanes[index], toEdge, toLanes[index], NBEdge::L2L_VALIDATED)) {
+            } else if (!fromEdge->addLane2LaneConnection(fromLanes[index], toEdge, toLanes[index], NBEdge::Lane2LaneInfoType::VALIDATED, true)) {
                 WRITE_WARNING("Could not set connection between '" + fromEdge->getID() + "_" + toString(fromLanes[index]) + "' and '" + toEdge->getID() + "_" + toString(toLanes[index]) + "'.");
                 ++unsetConnections;
             }
@@ -330,5 +317,3 @@ NIVissimConnection::getMaxID() {
 
 
 /****************************************************************************/
-
-
