@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimConnectionCluster.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NIVissimConnectionCluster.cpp 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // -------------------
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <algorithm>
 #include <iostream>
@@ -157,7 +148,7 @@ NIVissimConnectionCluster::NIVissimConnectionCluster(
         myIncomingEdges.push_back(c->getFromEdgeID());
         assert(find(edges.begin(), edges.end(), c->getFromEdgeID()) != edges.end()
                ||
-               find(edges.begin(), edges.end(), c->getToEdgeID()) != edges.end());
+               std::find(edges.begin(), edges.end(), c->getToEdgeID()) != edges.end());
     }
     VectorHelper<int>::removeDouble(myIncomingEdges);
     VectorHelper<int>::removeDouble(myOutgoingEdges);
@@ -395,13 +386,12 @@ NIVissimConnectionCluster::isWeakDistrictConnRealisation(NIVissimConnectionClust
     // connections must cross
     bool crosses = false;
     for (std::vector<int>::const_iterator j1 = myConnections.begin(); j1 != myConnections.end() && !crosses; j1++) {
-        NIVissimConnection* c1 = NIVissimConnection::dictionary(*j1);
-        const PositionVector& g1 = c1->getGeometry();
-        for (std::vector<int>::const_iterator j2 = c2->myConnections.begin(); j2 != c2->myConnections.end() && !crosses; j2++) {
-            NIVissimConnection* c2 = NIVissimConnection::dictionary(*j2);
-            const PositionVector& g2 = c2->getGeometry();
+        const PositionVector& g1 = NIVissimConnection::dictionary(*j1)->getGeometry();
+        for (const int j2 : c2->myConnections) {
+            const PositionVector& g2 = NIVissimConnection::dictionary(j2)->getGeometry();
             if (g1.intersects(g2)) {
                 crosses = true;
+                break;
             }
         }
     }
@@ -415,7 +405,7 @@ NIVissimConnectionCluster::isWeakDistrictConnRealisation(NIVissimConnectionClust
     // check whether the connection is bidirectional
     NIVissimEdge* oe = NIVissimEdge::dictionary(myOutgoingEdges[0]);
     NIVissimEdge* ie = NIVissimEdge::dictionary(c2->myIncomingEdges[0]);
-    if (oe == 0 || ie == 0) {
+    if (oe == nullptr || ie == nullptr) {
         return false;
     }
     return fabs(GeomHelper::angleDiff(oe->getGeometry().beginEndAngle(), ie->getGeometry().beginEndAngle())) < DEG2RAD(5);
@@ -554,7 +544,7 @@ NIVissimConnectionCluster::removeConnections(const NodeSubCluster& c) {
     for (NodeSubCluster::ConnectionCont::const_iterator i = c.myConnections.begin(); i != c.myConnections.end(); i++) {
         NIVissimConnection* conn = *i;
         int connid = conn->getID();
-        std::vector<int>::iterator j = find(myConnections.begin(), myConnections.end(), connid);
+        std::vector<int>::iterator j = std::find(myConnections.begin(), myConnections.end(), connid);
         if (j != myConnections.end()) {
             myConnections.erase(j);
         }
@@ -568,7 +558,7 @@ NIVissimConnectionCluster::recomputeBoundary() {
     myBoundary = Boundary();
     for (std::vector<int>::iterator i = myConnections.begin(); i != myConnections.end(); i++) {
         NIVissimConnection* c = NIVissimConnection::dictionary(*i);
-        if (c != 0) {
+        if (c != nullptr) {
             myBoundary.add(c->getFromGeomPosition());
             myBoundary.add(c->getToGeomPosition());
             if (c->getGeometry().size() != 0) {
@@ -653,7 +643,7 @@ NIVissimConnectionCluster::getPositionForEdge(int edgeid) const {
         //  !!! only the main geometry is regarded
         NIVissimNodeDef* node =
             NIVissimNodeDef::dictionary(myNodeCluster);
-        if (node != 0) {
+        if (node != nullptr) {
             double pos = node->getEdgePosition(edgeid);
             if (pos >= 0) {
                 return pos;
@@ -679,7 +669,7 @@ NIVissimConnectionCluster::getPositionForEdge(int edgeid) const {
     // !!!
     assert(myBoundary.xmin() <= myBoundary.xmax());
     NIVissimEdge* edge = NIVissimEdge::dictionary(edgeid);
-    std::vector<int>::const_iterator i = find(myEdges.begin(), myEdges.end(), edgeid);
+    std::vector<int>::const_iterator i = std::find(myEdges.begin(), myEdges.end(), edgeid);
     if (i == myEdges.end()) {
         // edge does not exist!?
         throw 1;
@@ -695,7 +685,7 @@ NIVissimConnectionCluster::getPositionForEdge(int edgeid) const {
 void
 NIVissimConnectionCluster::clearDict() {
     for (ContType::iterator i = myClusters.begin(); i != myClusters.end(); i++) {
-        delete(*i);
+        delete (*i);
     }
     myClusters.clear();
     myFirstFreeID = 100000;
@@ -737,7 +727,7 @@ NIVissimConnectionCluster::getIncomingContinuation(NIVissimEdge* e) const {
     }
     //
     if (edgeIsIncoming.size() == 0) {
-        return 0;
+        return nullptr;
     }
     // sort connected edges in same direction
     sort(edgeIsIncoming.begin(), edgeIsIncoming.end(),
@@ -781,7 +771,7 @@ NIVissimConnectionCluster::getOutgoingContinuation(NIVissimEdge* e) const {
     }
     //
     if (edgeIsOutgoing.size() == 0) {
-        return 0;
+        return nullptr;
     }
     // sort connected edges in same direction
     sort(edgeIsOutgoing.begin(), edgeIsOutgoing.end(),
@@ -790,6 +780,4 @@ NIVissimConnectionCluster::getOutgoingContinuation(NIVissimEdge* e) const {
 }
 
 
-
 /****************************************************************************/
-
