@@ -1,33 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimDisturbance.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NIVissimDisturbance.cpp 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // -------------------
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 
 #include <map>
@@ -48,10 +39,6 @@
 #include "NIVissimDisturbance.h"
 #include "NIVissimNodeParticipatingEdgeVector.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static member variables
@@ -105,7 +92,7 @@ NIVissimDisturbance*
 NIVissimDisturbance::dictionary(int id) {
     DictType::iterator i = myDict.find(id);
     if (i == myDict.end()) {
-        return 0;
+        return nullptr;
     }
     return (*i).second;
 }
@@ -126,10 +113,10 @@ void
 NIVissimDisturbance::computeBounding() {
     assert(myBoundary == 0);
     Boundary* bound = new Boundary();
-    if (NIVissimAbstractEdge::dictionary(myEdge.getEdgeID()) != 0) {
+    if (NIVissimAbstractEdge::dictionary(myEdge.getEdgeID()) != nullptr) {
         bound->add(myEdge.getGeomPosition());
     }
-    if (NIVissimAbstractEdge::dictionary(myDisturbance.getEdgeID()) != 0) {
+    if (NIVissimAbstractEdge::dictionary(myDisturbance.getEdgeID()) != nullptr) {
         bound->add(myDisturbance.getGeomPosition());
     }
     myBoundary = bound;
@@ -146,7 +133,7 @@ NIVissimDisturbance::addToNode(NBNode* node, NBDistrictCont& dc,
         NIVissimConnection::dictionary(myEdge.getEdgeID());
     NIVissimConnection* bc =
         NIVissimConnection::dictionary(myDisturbance.getEdgeID());
-    if (pc == 0 && bc == 0) {
+    if (pc == nullptr && bc == nullptr) {
         // This has not been tested completely, yet
         // Both competing abstract edges are normal edges
         // We have to find a crossing point, build a node here,
@@ -159,9 +146,9 @@ NIVissimDisturbance::addToNode(NBNode* node, NBDistrictCont& dc,
         std::string id2 = toString<int>(e2->getID()) + "x" + toString<int>(e1->getID());
         NBNode* node1 = nc.retrieve(id1);
         NBNode* node2 = nc.retrieve(id2);
-        NBNode* node = 0;
-        assert(node1 == 0 || node2 == 0);
-        if (node1 == 0 && node2 == 0) {
+        NBNode* splitNode = node1 == nullptr ? node2 : node1;
+        assert(node1 == nullptr || node2 == nullptr);
+        if (splitNode == nullptr) {
             refusedProhibits++;
             return false;
             /*            node = new NBNode(id1, pos.x(), pos.y(), "priority");
@@ -169,18 +156,16 @@ NIVissimDisturbance::addToNode(NBNode* node, NBDistrictCont& dc,
                              "nope, NIVissimDisturbance" << endl;
                             throw 1;
                         }*/
-        } else {
-            node = node1 == 0 ? node2 : node1;
         }
-        ec.splitAt(dc, ec.retrievePossiblySplit(toString<int>(e1->getID()), myEdge.getPosition()), node);
-        ec.splitAt(dc, ec.retrievePossiblySplit(toString<int>(e2->getID()), myDisturbance.getPosition()), node);
+        ec.splitAt(dc, ec.retrievePossiblySplit(toString<int>(e1->getID()), myEdge.getPosition()), splitNode);
+        ec.splitAt(dc, ec.retrievePossiblySplit(toString<int>(e2->getID()), myDisturbance.getPosition()), splitNode);
         // !!! in some cases, one of the edges is not being build because it's too short
         // !!! what to do in these cases?
         NBEdge* mayDriveFrom = ec.retrieve(toString<int>(e1->getID()) + "[0]");
         NBEdge* mayDriveTo = ec.retrieve(toString<int>(e1->getID()) + "[1]");
         NBEdge* mustStopFrom = ec.retrieve(toString<int>(e2->getID()) + "[0]");
         NBEdge* mustStopTo = ec.retrieve(toString<int>(e2->getID()) + "[1]");
-        if (mayDriveFrom != 0 && mayDriveTo != 0 && mustStopFrom != 0 && mustStopTo != 0) {
+        if (mayDriveFrom != nullptr && mayDriveTo != nullptr && mustStopFrom != nullptr && mustStopTo != nullptr) {
             node->addSortedLinkFoes(
                 NBConnection(mayDriveFrom, mayDriveTo),
                 NBConnection(mayDriveFrom, mayDriveTo));
@@ -190,14 +175,14 @@ NIVissimDisturbance::addToNode(NBNode* node, NBDistrictCont& dc,
             // !!! warning
         }
 //        }
-    } else if (pc != 0 && bc == 0) {
+    } else if (pc != nullptr && bc == nullptr) {
         // The prohibited abstract edge is a connection, the other
         //  is not;
         // The connection will be prohibitesd by all connections
         //  outgoing from the "real" edge
 
         NBEdge* e = ec.retrievePossiblySplit(toString<int>(myDisturbance.getEdgeID()), myDisturbance.getPosition());
-        if (e == 0) {
+        if (e == nullptr) {
             WRITE_WARNING("Could not prohibit '" + toString<int>(myEdge.getEdgeID()) + "' by '" + toString<int>(myDisturbance.getEdgeID()) + "'. Have not found disturbance.");
             refusedProhibits++;
             return false;
@@ -211,18 +196,15 @@ NIVissimDisturbance::addToNode(NBNode* node, NBDistrictCont& dc,
         // get the begin of the prohibited connection
         std::string id_pcoe = toString<int>(pc->getFromEdgeID());
         std::string id_pcie = toString<int>(pc->getToEdgeID());
-        NBEdge* pcoe = ec.retrievePossiblySplit(id_pcoe, id_pcie, true);
-        NBEdge* pcie = ec.retrievePossiblySplit(id_pcie, id_pcoe, false);
+        NBEdge* const pcoe = ec.retrievePossiblySplit(id_pcoe, id_pcie, true);
+        NBEdge* const pcie = ec.retrievePossiblySplit(id_pcie, id_pcoe, false);
         // check whether it's ending node is the node the prohibited
         //  edge end at
-        if (pcoe != 0 && pcie != 0 && pcoe->getToNode() == e->getToNode()) {
+        if (pcoe != nullptr && pcie != nullptr && pcoe->getToNode() == e->getToNode()) {
             // if so, simply prohibit the connections
-            NBNode* node = e->getToNode();
-            const EdgeVector& connected = e->getConnectedEdges();
-            for (EdgeVector::const_iterator i = connected.begin(); i != connected.end(); i++) {
-                node->addSortedLinkFoes(
-                    NBConnection(e, *i),
-                    NBConnection(pcoe, pcie));
+            NBNode* const toNode = e->getToNode();
+            for (NBEdge* const edge : e->getConnectedEdges()) {
+                toNode->addSortedLinkFoes(NBConnection(e, edge), NBConnection(pcoe, pcie));
             }
         } else {
             WRITE_WARNING("Would have to split edge '" + e->getID() + "' to build a prohibition");
@@ -244,14 +226,14 @@ NIVissimDisturbance::addToNode(NBNode* node, NBDistrictCont& dc,
             }
             */
         }
-    } else if (bc != 0 && pc == 0) {
+    } else if (bc != nullptr && pc == nullptr) {
         // The prohibiting abstract edge is a connection, the other
         //  is not;
         // We have to split the other one and add the prohibition
         //  description
 
         NBEdge* e = ec.retrievePossiblySplit(toString<int>(myEdge.getEdgeID()), myEdge.getPosition());
-        if (e == 0) {
+        if (e == nullptr) {
             WRITE_WARNING("Could not prohibit '" + toString<int>(myEdge.getEdgeID()) + "' - it was not built.");
             return false;
         }
@@ -266,18 +248,15 @@ NIVissimDisturbance::addToNode(NBNode* node, NBDistrictCont& dc,
         // get the begin of the prohibiting connection
         std::string id_bcoe = toString<int>(bc->getFromEdgeID());
         std::string id_bcie = toString<int>(bc->getToEdgeID());
-        NBEdge* bcoe = ec.retrievePossiblySplit(id_bcoe, id_bcie, true);
-        NBEdge* bcie = ec.retrievePossiblySplit(id_bcie, id_bcoe, false);
+        NBEdge* const bcoe = ec.retrievePossiblySplit(id_bcoe, id_bcie, true);
+        NBEdge* const bcie = ec.retrievePossiblySplit(id_bcie, id_bcoe, false);
         // check whether it's ending node is the node the prohibited
         //  edge end at
-        if (bcoe != 0 && bcie != 0 && bcoe->getToNode() == e->getToNode()) {
+        if (bcoe != nullptr && bcie != nullptr && bcoe->getToNode() == e->getToNode()) {
             // if so, simply prohibit the connections
-            NBNode* node = e->getToNode();
-            const EdgeVector& connected = e->getConnectedEdges();
-            for (EdgeVector::const_iterator i = connected.begin(); i != connected.end(); i++) {
-                node->addSortedLinkFoes(
-                    NBConnection(bcoe, bcie),
-                    NBConnection(e, *i));
+            NBNode* const toNode = e->getToNode();
+            for (NBEdge* const edge : e->getConnectedEdges()) {
+                toNode->addSortedLinkFoes(NBConnection(bcoe, bcie), NBConnection(e, edge));
             }
         } else {
             WRITE_WARNING("Would have to split edge '" + e->getID() + "' to build a prohibition");
@@ -314,7 +293,7 @@ NIVissimDisturbance::addToNode(NBNode* node, NBDistrictCont& dc,
 
 NBConnection
 NIVissimDisturbance::getConnection(NBNode* node, int aedgeid) {
-    if (NIVissimEdge::dictionary(myEdge.getEdgeID()) == 0) {
+    if (NIVissimEdge::dictionary(myEdge.getEdgeID()) == nullptr) {
         NIVissimConnection* c = NIVissimConnection::dictionary(aedgeid);
         NBEdge* from =
             node->getPossiblySplittedIncoming(toString<int>(c->getFromEdgeID()));
@@ -335,7 +314,7 @@ NIVissimDisturbance::getConnection(NBNode* node, int aedgeid) {
 void
 NIVissimDisturbance::clearDict() {
     for (DictType::iterator i = myDict.begin(); i != myDict.end(); i++) {
-        delete(*i).second;
+        delete (*i).second;
     }
     myDict.clear();
 }
@@ -358,11 +337,9 @@ NIVissimDisturbance::dict_SetDisturbances() {
 void
 NIVissimDisturbance::reportRefused() {
     if (refusedProhibits > 0) {
-        WRITE_WARNING("Could not build " + toString<size_t>(refusedProhibits) + " of " + toString<size_t>(myDict.size()) + " disturbances.");
+        WRITE_WARNING("Could not build " + toString<int>(refusedProhibits) + " of " + toString<int>((int)myDict.size()) + " disturbances.");
     }
 }
 
 
-
 /****************************************************************************/
-
