@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimEdge.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NIVissimEdge.cpp 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // A temporary storage for edges imported from Vissim
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <algorithm>
@@ -113,27 +104,30 @@ NIVissimEdge::connection_cluster_position_sorter::operator()(
 
 
 NIVissimEdge::NIVissimEdge(int id, const std::string& name,
-                           const std::string& type, int noLanes,
+                           const std::string& type,
+                           std::vector<double> laneWidths,
                            double zuschlag1, double zuschlag2,
                            double /*length*/, const PositionVector& geom,
-                           const NIVissimClosedLanesVector& clv)
-    : NIVissimAbstractEdge(id, geom),
-      myName(name), myType(type), myNoLanes(noLanes),
-      myZuschlag1(zuschlag1), myZuschlag2(zuschlag2),
-      myClosedLanes(clv), myAmWithinJunction(false) { //, mySpeed(-1)
-    assert(noLanes >= 0);
+                           const NIVissimClosedLanesVector& clv) :
+    NIVissimAbstractEdge(id, geom),
+    myName(name), myType(type), myNoLanes((int)laneWidths.size()),
+    myLaneWidths(laneWidths),
+    myZuschlag1(zuschlag1), myZuschlag2(zuschlag2),
+    myClosedLanes(clv),
+    myLaneSpeeds(myNoLanes, -1),
+    myAmWithinJunction(false)
+    //, mySpeed(-1)
+{
+    assert(myNoLanes >= 0);
     if (myMaxID < myID) {
         myMaxID = myID;
     }
-    for (int i = 0; i < noLanes; i++) {
-        myLaneSpeeds.push_back(-1);
-    }
 }
 
 
 NIVissimEdge::~NIVissimEdge() {
     for (NIVissimClosedLanesVector::iterator i = myClosedLanes.begin(); i != myClosedLanes.end(); i++) {
-        delete(*i);
+        delete (*i);
     }
     myClosedLanes.clear();
 }
@@ -145,8 +139,8 @@ NIVissimEdge::dictionary(int id, const std::string& name,
                          double zuschlag1, double zuschlag2, double length,
                          const PositionVector& geom,
                          const NIVissimClosedLanesVector& clv) {
-    NIVissimEdge* o = new NIVissimEdge(id, name, type, noLanes, zuschlag1,
-                                       zuschlag2, length, geom, clv);
+    NIVissimEdge* o = new NIVissimEdge(id, name, type, std::vector<double>(noLanes, NBEdge::UNSPECIFIED_WIDTH),
+                                       zuschlag1, zuschlag2, length, geom, clv);
     if (!dictionary(id, o)) {
         delete o;
         return false;
@@ -172,7 +166,7 @@ NIVissimEdge*
 NIVissimEdge::dictionary(int id) {
     DictType::iterator i = myDict.find(id);
     if (i == myDict.end()) {
-        return 0;
+        return nullptr;
     }
     return (*i).second;
 }
@@ -379,7 +373,7 @@ NIVissimEdge::setDistrictSpeed() {
         if (pos < getLength() - pos) {
             NIVissimDistrictConnection* d =
                 NIVissimDistrictConnection::dict_findForEdge(myID);
-            if (d != 0) {
+            if (d != nullptr) {
                 double speed = d->getMeanSpeed(/*dc*/);
                 if (speed == -1) {
                     return;
@@ -411,7 +405,7 @@ NIVissimEdge::getOutgoingConnected(int lane) const {
         const std::vector<int>& lanes = c->getFromLanes();
         if (find(lanes.begin(), lanes.end(), lane) != lanes.end()) {
             NIVissimEdge* e = NIVissimEdge::dictionary(c->getToEdgeID());
-            if (e != 0) {
+            if (e != nullptr) {
                 ret.push_back(c);
             }
         }
@@ -426,7 +420,7 @@ NIVissimEdge::buildNBEdge(NBDistrictCont& dc, NBNodeCont& nc, NBEdgeCont& ec,
     // build the edge
     std::pair<NIVissimConnectionCluster*, NBNode*> fromInf, toInf;
     NBNode* fromNode, *toNode;
-    fromNode = toNode = 0;
+    fromNode = toNode = nullptr;
     sort(myConnectionClusters.begin(), myConnectionClusters.end(), connection_cluster_position_sorter(myID));
     sort(myDistrictConnections.begin(), myDistrictConnections.end());
     ConnectionClusters tmpClusters = myConnectionClusters;
@@ -441,7 +435,7 @@ NIVissimEdge::buildNBEdge(NBDistrictCont& dc, NBNodeCont& nc, NBEdgeCont& ec,
         //if(tmpClusters.size()>0) {
         toInf = getToNode(nc, tmpClusters);
         toNode = toInf.second;
-        if (fromInf.first != 0 && toNode != 0 && fromInf.first->around(toNode->getPosition())) {
+        if (fromInf.first != 0 && toNode != nullptr && fromInf.first->around(toNode->getPosition())) {
             WRITE_WARNING("Will not build edge '" + toString(myID) + "'.");
             myAmWithinJunction = true;
             return;
@@ -462,18 +456,18 @@ NIVissimEdge::buildNBEdge(NBDistrictCont& dc, NBNodeCont& nc, NBEdgeCont& ec,
     }
 
     //
-    if (fromNode == 0) {
+    if (fromNode == nullptr) {
         fromInf.first = 0;
         Position pos = myGeom[0];
-        fromNode = new NBNode(toString<int>(myID) + "-SourceNode", pos, NODETYPE_NOJUNCTION);
+        fromNode = new NBNode(toString<int>(myID) + "-SourceNode", pos, SumoXMLNodeType::NOJUNCTION);
         if (!nc.insert(fromNode)) {
             throw ProcessError("Could not insert node '" + fromNode->getID() + "' to nodes container.");
         }
     }
-    if (toNode == 0) {
+    if (toNode == nullptr) {
         toInf.first = 0;
         Position pos = myGeom[-1];
-        toNode = new NBNode(toString<int>(myID) + "-DestinationNode", pos, NODETYPE_NOJUNCTION);
+        toNode = new NBNode(toString<int>(myID) + "-DestinationNode", pos, SumoXMLNodeType::NOJUNCTION);
         if (!nc.insert(toNode)) {
             throw ProcessError("Could not insert node '" + toNode->getID() + "' to nodes container.");
         }
@@ -500,8 +494,9 @@ NIVissimEdge::buildNBEdge(NBDistrictCont& dc, NBNodeCont& nc, NBEdgeCont& ec,
     NBEdge* buildEdge = new NBEdge(toString<int>(myID), fromNode, toNode, myType,
                                    avgSpeed / (double) 3.6, myNoLanes, -1,
                                    NBEdge::UNSPECIFIED_WIDTH, NBEdge::UNSPECIFIED_OFFSET,
-                                   myGeom, myName, "", LANESPREAD_CENTER, true);
+                                   myGeom, LaneSpreadFunction::CENTER, myName, "", true);
     for (int i = 0; i < myNoLanes; i++) {
+        buildEdge->setLaneWidth(i, myLaneWidths[i]);
         if ((int) myLaneSpeeds.size() <= i || myLaneSpeeds[i] == -1) {
             buildEdge->setSpeed(i, OptionsCont::getOptions().getFloat("vissim.default-speed") / (double) 3.6);
         } else {
@@ -527,7 +522,7 @@ double
 NIVissimEdge::getRealSpeed(int distNo) {
     std::string id = toString<int>(distNo);
     Distribution* dist = DistributionCont::dictionary("speed", id);
-    if (dist == 0) {
+    if (dist == nullptr) {
         WRITE_WARNING("The referenced speed distribution '" + id + "' is not known.");
         return -1;
     }
@@ -547,7 +542,7 @@ NIVissimEdge::recheckSpeedPatches()
     // check set speeds
     if(myPatchedSpeeds.size()!=0) {
         std::vector<double>::iterator i =
-            find(myPatchedSpeeds.begin(), myPatchedSpeeds.end(), -1);
+            std::find(myPatchedSpeeds.begin(), myPatchedSpeeds.end(), -1);
         if(myPatchedSpeeds.size()!=myNoLanes||i!=myPatchedSpeeds.end()) {
             cot << "Warning! Not all lanes are patched! (edge:" << myID << ")." << endl;
         }
@@ -594,22 +589,22 @@ NIVissimEdge::getFromNode(NBNodeCont& nc, ConnectionClusters& clusters) {
     if (myDistrictConnections.size() > 0) {
         double pos = *(myDistrictConnections.begin());
         if (pos < 10) {
-            NBNode* node = new NBNode(toString<int>(myID) + "-begin", beg, NODETYPE_NOJUNCTION);
+            NBNode* node = new NBNode(toString<int>(myID) + "-begin", beg, SumoXMLNodeType::NOJUNCTION);
             if (!nc.insert(node)) {
                 throw 1;
             }
             while (myDistrictConnections.size() > 0 && *(myDistrictConnections.begin()) < 10) {
                 myDistrictConnections.erase(myDistrictConnections.begin());
             }
-            return std::pair<NIVissimConnectionCluster*, NBNode*>(static_cast<NIVissimConnectionCluster*>(0), node);
+            return std::pair<NIVissimConnectionCluster*, NBNode*>(static_cast<NIVissimConnectionCluster*>(nullptr), node);
         }
     }
     // build a new node for the edge's begin otherwise
-    NBNode* node = new NBNode(toString<int>(myID) + "-begin", beg, NODETYPE_NOJUNCTION);
+    NBNode* node = new NBNode(toString<int>(myID) + "-begin", beg, SumoXMLNodeType::NOJUNCTION);
     if (!nc.insert(node)) {
         throw 1;
     }
-    return std::pair<NIVissimConnectionCluster*, NBNode*>(static_cast<NIVissimConnectionCluster*>(0), node);
+    return std::pair<NIVissimConnectionCluster*, NBNode*>(static_cast<NIVissimConnectionCluster*>(nullptr), node);
 }
 
 
@@ -630,23 +625,23 @@ NIVissimEdge::getToNode(NBNodeCont& nc, ConnectionClusters& clusters) {
     if (myDistrictConnections.size() > 0) {
         double pos = *(myDistrictConnections.end() - 1);
         if (pos > myGeom.length() - 10) {
-            NBNode* node = new NBNode(toString<int>(myID) + "-end", end, NODETYPE_NOJUNCTION);
+            NBNode* node = new NBNode(toString<int>(myID) + "-end", end, SumoXMLNodeType::NOJUNCTION);
             if (!nc.insert(node)) {
                 throw 1;
             }
             while (myDistrictConnections.size() > 0 && *(myDistrictConnections.end() - 1) < myGeom.length() - 10) {
                 myDistrictConnections.erase(myDistrictConnections.end() - 1);
             }
-            return std::pair<NIVissimConnectionCluster*, NBNode*>(static_cast<NIVissimConnectionCluster*>(0), node);
+            return std::pair<NIVissimConnectionCluster*, NBNode*>(static_cast<NIVissimConnectionCluster*>(nullptr), node);
         }
     }
 
     // build a new node for the edge's end otherwise
-    NBNode* node = new NBNode(toString<int>(myID) + "-end", end, NODETYPE_NOJUNCTION);
+    NBNode* node = new NBNode(toString<int>(myID) + "-end", end, SumoXMLNodeType::NOJUNCTION);
     if (!nc.insert(node)) {
         throw 1;
     }
-    return std::pair<NIVissimConnectionCluster*, NBNode*>(static_cast<NIVissimConnectionCluster*>(0), node);
+    return std::pair<NIVissimConnectionCluster*, NBNode*>(static_cast<NIVissimConnectionCluster*>(nullptr), node);
     /*
     if (clusters.size()>0) {
     NIVissimConnectionCluster *c = *(clusters.end()-1);
@@ -671,14 +666,14 @@ NIVissimEdge::remapOneOfNodes(NBNodeCont& nc,
 
         NBNode* newNode = new NBNode(nid,
                                      fromNode->getPosition(),
-                                     NODETYPE_NOJUNCTION);
+                                     SumoXMLNodeType::NOJUNCTION);
         nc.erase(fromNode);
         nc.insert(newNode);
         return std::pair<NBNode*, NBNode*>(newNode, toNode);
     } else {
         NBNode* newNode = new NBNode(nid,
                                      toNode->getPosition(),
-                                     NODETYPE_NOJUNCTION);
+                                     SumoXMLNodeType::NOJUNCTION);
         nc.erase(toNode);
         nc.insert(newNode);
         return std::pair<NBNode*, NBNode*>(fromNode, newNode);
@@ -694,16 +689,16 @@ NIVissimEdge::resolveSameNode(NBNodeCont& nc, double offset,
     //  use it if so
     NIVissimDistrictConnection* d =
         NIVissimDistrictConnection::dict_findForEdge(myID);
-    if (d != 0) {
+    if (d != nullptr) {
         Position pos = d->geomPosition();
         double position = d->getPosition();
         // the district is at the begin of the edge
         if (myGeom.length() - position > position) {
             std::string nid = "ParkingPlace" + toString<int>(d->getID());
             NBNode* node = nc.retrieve(nid);
-            if (node == 0) {
+            if (node == nullptr) {
                 node = new NBNode(nid,
-                                  pos, NODETYPE_NOJUNCTION);
+                                  pos, SumoXMLNodeType::NOJUNCTION);
                 if (!nc.insert(node)) {
                     throw 1;
                 }
@@ -714,8 +709,8 @@ NIVissimEdge::resolveSameNode(NBNodeCont& nc, double offset,
         else {
             std::string nid = "ParkingPlace" + toString<int>(d->getID());
             NBNode* node = nc.retrieve(nid);
-            if (node == 0) {
-                node = new NBNode(nid, pos, NODETYPE_NOJUNCTION);
+            if (node == nullptr) {
+                node = new NBNode(nid, pos, SumoXMLNodeType::NOJUNCTION);
                 if (!nc.insert(node)) {
                     throw 1;
                 }
@@ -736,7 +731,7 @@ NIVissimEdge::resolveSameNode(NBNodeCont& nc, double offset,
             NBNode* end = new NBNode(
                 toString<int>(myID) + "-End",
                 myGeom.back(),
-                NODETYPE_NOJUNCTION);
+                SumoXMLNodeType::NOJUNCTION);
             if (!nc.insert(end)) {
                 throw 1;
             }
@@ -748,7 +743,7 @@ NIVissimEdge::resolveSameNode(NBNodeCont& nc, double offset,
             NBNode* beg = new NBNode(
                 toString<int>(myID) + "-Begin",
                 myGeom.front(),
-                NODETYPE_NOJUNCTION);
+                SumoXMLNodeType::NOJUNCTION);
             if (!nc.insert(beg)) {
                 std::cout << "nope, NIVissimDisturbance" << std::endl;
                 throw 1;
@@ -796,11 +791,11 @@ void
 NIVissimEdge::mergedInto(NIVissimConnectionCluster* old,
                          NIVissimConnectionCluster* act) {
     ConnectionClusters::iterator i =
-        find(myConnectionClusters.begin(), myConnectionClusters.end(), old);
+        std::find(myConnectionClusters.begin(), myConnectionClusters.end(), old);
     if (i != myConnectionClusters.end()) {
         myConnectionClusters.erase(i);
     }
-    i = find(myConnectionClusters.begin(), myConnectionClusters.end(), act);
+    i = std::find(myConnectionClusters.begin(), myConnectionClusters.end(), act);
     if (i == myConnectionClusters.end()) {
         myConnectionClusters.push_back(act);
     }
@@ -811,7 +806,7 @@ NIVissimEdge::mergedInto(NIVissimConnectionCluster* old,
 void
 NIVissimEdge::removeFromConnectionCluster(NIVissimConnectionCluster* c) {
     ConnectionClusters::iterator i =
-        find(myConnectionClusters.begin(), myConnectionClusters.end(), c);
+        std::find(myConnectionClusters.begin(), myConnectionClusters.end(), c);
     assert(i != myConnectionClusters.end());
     myConnectionClusters.erase(i);
 }
@@ -820,7 +815,7 @@ NIVissimEdge::removeFromConnectionCluster(NIVissimConnectionCluster* c) {
 void
 NIVissimEdge::addToConnectionCluster(NIVissimConnectionCluster* c) {
     ConnectionClusters::iterator i =
-        find(myConnectionClusters.begin(), myConnectionClusters.end(), c);
+        std::find(myConnectionClusters.begin(), myConnectionClusters.end(), c);
     if (i == myConnectionClusters.end()) {
         myConnectionClusters.push_back(c);
     }
@@ -883,11 +878,11 @@ NIVissimEdge::dict_checkEdges2Join() {
             const PositionVector& g2 = e2->getGeometry();
             // get the connection description
             NIVissimConnection* c = e1->getConnectionTo(e2);
-            if (c == 0) {
+            if (c == nullptr) {
                 c = e2->getConnectionTo(e1);
             }
             // the edge must not be a direct contiuation of the other
-            if (c != 0) {
+            if (c != nullptr) {
                 if ((c->getFromEdgeID() == e1->getID() && fabs(c->getFromPosition() - e1->getGeometry().length()) < 5)
                         ||
                         (c->getFromEdgeID() == e2->getID() && fabs(c->getFromPosition() - e2->getGeometry().length()) < 5)) {
@@ -964,7 +959,7 @@ NIVissimEdge::getConnectionTo(NIVissimEdge* e) {
             return c;
         }
     }
-    return 0;
+    return nullptr;
 }
 
 
@@ -1004,7 +999,7 @@ NIVissimEdge::getBestIncoming() const {
     if (myIncomingConnections.size() != 0) {
         return NIVissimEdge::dictionary(NIVissimConnection::dictionary(myIncomingConnections.front())->getFromEdgeID());
     }
-    return 0;
+    return nullptr;
 }
 
 
@@ -1021,10 +1016,8 @@ NIVissimEdge::getBestOutgoing() const {
     if (myOutgoingConnections.size() != 0) {
         return NIVissimEdge::dictionary(NIVissimConnection::dictionary(myOutgoingConnections.front())->getToEdgeID());
     }
-    return 0;
+    return nullptr;
 }
 
 
-
 /****************************************************************************/
-
