@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimEdge.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    End of 2002
-/// @version $Id: NIVissimEdge.h 21182 2016-07-18 06:46:01Z behrisch $
 ///
 // A temporary storage for edges imported from Vissim
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIVissimEdge_h
-#define NIVissimEdge_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include "NIVissimConnectionCluster.h"
 #include <vector>
@@ -46,9 +36,8 @@
 // class declarations
 // ===========================================================================
 class NBNode;
-class NIVissimDistrictConnection;
-class NBDistribution;
 class NBDistrictCont;
+class NIVissimDistrictConnection;
 
 
 // ===========================================================================
@@ -63,8 +52,10 @@ class NIVissimEdge
 public:
     /// Constructor
     NIVissimEdge(int id, const std::string& name,
-                 const std::string& type, int noLanes, SUMOReal zuschlag1,
-                 SUMOReal zuschlag2, SUMOReal length,
+                 const std::string& type,
+                 std::vector<double> laneWidths,
+                 double zuschlag1,
+                 double zuschlag2, double length,
                  const PositionVector& geom,
                  const NIVissimClosedLanesVector& clv);
 
@@ -80,10 +71,6 @@ public:
     /// Adds a connection where this edge is the source
     void addOutgoingConnection(int id);
 
-    /** @brief Returns the node at the given position
-        As this may be ambigous, a second node not to return may be supplied */
-    NBNode* getNodeAt(const Position& p, NBNode* other = 0);
-
     /** Returns the begin position of the edge */
     Position getBegin2D() const;
 
@@ -91,9 +78,9 @@ public:
     Position getEnd2D() const;
 
     /// Returns the length of the node
-    SUMOReal getLength() const;
+    double getLength() const;
 
-    void checkDistrictConnectionExistanceAt(SUMOReal pos);
+    void checkDistrictConnectionExistanceAt(double pos);
 
     void mergedInto(NIVissimConnectionCluster* old,
                     NIVissimConnectionCluster* act);
@@ -124,8 +111,8 @@ public:
     /** @brief Adds the described item to the dictionary
         Builds the edge first */
     static bool dictionary(int id, const std::string& name,
-                           const std::string& type, int noLanes, SUMOReal zuschlag1,
-                           SUMOReal zuschlag2, SUMOReal length,
+                           const std::string& type, int noLanes, double zuschlag1,
+                           double zuschlag2, double length,
                            const PositionVector& geom,
                            const NIVissimClosedLanesVector& clv);
 
@@ -147,9 +134,9 @@ public:
 
     /// Builds NBEdges from the VissimEdges within the dictionary
     static void dict_buildNBEdges(NBDistrictCont& dc, NBNodeCont& nc,
-                                  NBEdgeCont& ec, SUMOReal offset);
+                                  NBEdgeCont& ec, double offset);
 
-    static void dict_propagateSpeeds(/* NBDistribution &dc */);
+    static void dict_propagateSpeeds();
 
     static void dict_checkEdges2Join();
 
@@ -179,7 +166,7 @@ private:
      * @exception ProcessError If one of the built nodes or edges could not be added to the according container
      */
     void buildNBEdge(NBDistrictCont& dc, NBNodeCont& nc,
-                     NBEdgeCont& ec, SUMOReal sameNodesOffset);
+                     NBEdgeCont& ec, double sameNodesOffset);
 
     /// Returns the origin node
     std::pair<NIVissimConnectionCluster*, NBNode*>
@@ -191,19 +178,19 @@ private:
 
     /// Tries to resolve the problem that the same node has been returned as origin and destination node
     std::pair<NBNode*, NBNode*> resolveSameNode(NBNodeCont& nc,
-            SUMOReal offset, NBNode* prevFrom, NBNode* prevTo);
+            double offset, NBNode* prevFrom, NBNode* prevTo);
 
-//    SUMOReal recheckSpeedPatches();
+//    double recheckSpeedPatches();
 
     std::vector<NIVissimConnection*> getOutgoingConnected(int lane) const;
 
-    void propagateSpeed(/* NBDistribution &dc */SUMOReal speed, std::vector<int> forLanes);
+    void propagateSpeed(double speed, std::vector<int> forLanes);
 
 
-    void setDistrictSpeed(/* NBDistribution &dc */);
-    SUMOReal getRealSpeed(/* NBDistribution &dc */int distNo);
-    void checkUnconnectedLaneSpeeds(/* NBDistribution &dc */);
-    void propagateOwn(/* NBDistribution &dc */);
+    void setDistrictSpeed();
+    double getRealSpeed(int distNo);
+    void checkUnconnectedLaneSpeeds();
+    void propagateOwn();
 
 
 
@@ -264,9 +251,10 @@ private:
 
     /// The number of lanes the edge has
     int myNoLanes;
+    std::vector<double> myLaneWidths;
 
     /// Additional load values for this edge
-    SUMOReal myZuschlag1, myZuschlag2;
+    double myZuschlag1, myZuschlag2;
 
     /// List of lanes closed on this edge
     NIVissimClosedLanesVector myClosedLanes;
@@ -280,11 +268,11 @@ private:
     /// List of connections outgoing from this edge
     std::vector<int> myOutgoingConnections;
 
-    std::vector<SUMOReal> myDistrictConnections;
+    std::vector<double> myDistrictConnections;
 
     std::vector<int> myPatchedSpeeds;
 
-    std::vector<SUMOReal> myLaneSpeeds;
+    std::vector<double> myLaneSpeeds;
 
     std::vector<NIVissimEdge*> myToTreatAsSame;
 
@@ -304,9 +292,3 @@ private:
     static std::vector<std::string> myLanesWithMissingSpeeds;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
