@@ -1,41 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimEdgePosMap.cpp
 /// @author  Daniel Krajzewicz
 /// @date    Sept 2002
-/// @version $Id: NIVissimEdgePosMap.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // -------------------
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 
 #include <map>
 #include "NIVissimEdgePosMap.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 NIVissimEdgePosMap::NIVissimEdgePosMap() {}
 
@@ -44,15 +31,15 @@ NIVissimEdgePosMap::~NIVissimEdgePosMap() {}
 
 
 void
-NIVissimEdgePosMap::add(int edgeid, SUMOReal pos) {
+NIVissimEdgePosMap::add(int edgeid, double pos) {
     add(edgeid, pos, pos);
 }
 
 
 void
-NIVissimEdgePosMap::add(int edgeid, SUMOReal from, SUMOReal to) {
+NIVissimEdgePosMap::add(int edgeid, double from, double to) {
     if (from > to) {
-        SUMOReal tmp = from;
+        double tmp = from;
         from = to;
         to = tmp;
     }
@@ -60,8 +47,8 @@ NIVissimEdgePosMap::add(int edgeid, SUMOReal from, SUMOReal to) {
     if (i == myCont.end()) {
         myCont[edgeid] = Range(from, to);
     } else {
-        SUMOReal pfrom = (*i).second.first;
-        SUMOReal pto = (*i).second.second;
+        double pfrom = (*i).second.first;
+        double pto = (*i).second.second;
         if (pfrom < from) {
             from = pfrom;
         }
@@ -81,6 +68,4 @@ NIVissimEdgePosMap::join(NIVissimEdgePosMap& with) {
 }
 
 
-
 /****************************************************************************/
-
