@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimNodeCluster.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NIVissimNodeCluster.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // -------------------
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 
 #include <map>
@@ -44,10 +35,6 @@
 #include "NIVissimConnection.h"
 #include "NIVissimNodeCluster.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static member variables
@@ -65,7 +52,7 @@ NIVissimNodeCluster::NIVissimNodeCluster(int id, int nodeid, int tlid,
         bool amEdgeSplitOnly)
     : myID(id), myNodeID(nodeid), myTLID(tlid),
       myConnectors(connectors), myDisturbances(disturbances),
-      myNBNode(0), myAmEdgeSplit(amEdgeSplitOnly) {}
+      myNBNode(nullptr), myAmEdgeSplit(amEdgeSplitOnly) {}
 
 
 NIVissimNodeCluster::~NIVissimNodeCluster() {}
@@ -105,16 +92,16 @@ NIVissimNodeCluster*
 NIVissimNodeCluster::dictionary(int id) {
     DictType::iterator i = myDict.find(id);
     if (i == myDict.end()) {
-        return 0;
+        return nullptr;
     }
     return (*i).second;
 }
 
 
 
-size_t
+int
 NIVissimNodeCluster::contSize() {
-    return myDict.size();
+    return (int)myDict.size();
 }
 
 
@@ -180,7 +167,7 @@ NIVissimNodeCluster::buildNBNode(NBNodeCont& nc) {
                     "actuated_traffic_light");
             }
         }*/
-    NBNode* node = new NBNode(getNodeName(), pos, NODETYPE_PRIORITY);
+    NBNode* node = new NBNode(getNodeName(), pos, SumoXMLNodeType::PRIORITY);
     if (!nc.insert(node)) {
         delete node;
         throw 1;
@@ -211,7 +198,7 @@ NIVissimNodeCluster::getFromNode(int edgeid) {
         NIVissimNodeCluster* c = (*i).second;
         for (std::vector<int>::iterator j = c->myConnectors.begin(); j != c->myConnectors.end(); j++) {
             NIVissimConnection* conn = NIVissimConnection::dictionary(*j);
-            if (conn != 0 && conn->getToEdgeID() == edgeid) {
+            if (conn != nullptr && conn->getToEdgeID() == edgeid) {
 //                return (*i).first;
                 if (ret != -1 && (*i).first != ret) {
 //                     "NIVissimNodeCluster:DoubleNode:" << ret << endl;
@@ -233,7 +220,7 @@ NIVissimNodeCluster::getToNode(int edgeid) {
         NIVissimNodeCluster* c = (*i).second;
         for (std::vector<int>::iterator j = c->myConnectors.begin(); j != c->myConnectors.end(); j++) {
             NIVissimConnection* conn = NIVissimConnection::dictionary(*j);
-            if (conn != 0 && conn->getFromEdgeID() == edgeid) {
+            if (conn != nullptr && conn->getFromEdgeID() == edgeid) {
 //                return (*i).first;
                 if (ret != -1 && ret != (*i).first) {
 //                  << "NIVissimNodeCluster: multiple to-nodes" << endl;
@@ -296,7 +283,7 @@ NIVissimNodeCluster::dict_addDisturbances(NBDistrictCont& dc,
 void
 NIVissimNodeCluster::clearDict() {
     for (DictType::iterator i = myDict.begin(); i != myDict.end(); i++) {
-        delete(*i).second;
+        delete (*i).second;
     }
     myDict.clear();
 }
@@ -308,6 +295,4 @@ NIVissimNodeCluster::setCurrentVirtID(int id) {
 }
 
 
-
 /****************************************************************************/
-
