@@ -1,46 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimNodeParticipatingEdge.cpp
 /// @author  Daniel Krajzewicz
 /// @date    Sept 2002
-/// @version $Id: NIVissimNodeParticipatingEdge.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // -------------------
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 
 #include "NIVissimEdge.h"
 #include "NIVissimConnection.h"
 #include "NIVissimNodeParticipatingEdge.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 
 NIVissimNodeParticipatingEdge::NIVissimNodeParticipatingEdge(
-    int edgeid, SUMOReal frompos, SUMOReal topos)
+    int edgeid, double frompos, double topos)
     : myEdgeID(edgeid), myFromPos(frompos), myToPos(topos) {}
 
 NIVissimNodeParticipatingEdge::~NIVissimNodeParticipatingEdge() {}
@@ -53,25 +40,23 @@ NIVissimNodeParticipatingEdge::getID() const {
 
 
 bool
-NIVissimNodeParticipatingEdge::positionLiesWithin(SUMOReal pos) const {
+NIVissimNodeParticipatingEdge::positionLiesWithin(double pos) const {
     return
         (myFromPos < myToPos && myFromPos >= pos && myToPos <= pos)
         ||
         (myFromPos > myToPos && myFromPos <= pos && myToPos >= pos);
 }
 
-SUMOReal
+double
 NIVissimNodeParticipatingEdge::getFromPos() const {
     return myFromPos;
 }
 
 
-SUMOReal
+double
 NIVissimNodeParticipatingEdge::getToPos() const {
     return myToPos;
 }
 
 
-
 /****************************************************************************/
-
