@@ -1,34 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimTrafficDescription.h
 /// @author  Daniel Krajzewicz
 /// @date    Sept 2002
-/// @version $Id: NIVissimTrafficDescription.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // -------------------
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NIVissimTrafficDescription_h
-#define NIVissimTrafficDescription_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 
 #include <string>
@@ -51,8 +41,8 @@ public:
     static bool dictionary(int id, NIVissimTrafficDescription* o);
     static NIVissimTrafficDescription* dictionary(int id);
     static void clearDict();
-    static SUMOReal meanSpeed(int id);
-    SUMOReal meanSpeed() const;
+    static double meanSpeed(int id);
+    double meanSpeed() const;
 private:
     std::string myName;
     NIVissimVehicleClassVector myVehicleTypes;
@@ -60,9 +50,3 @@ private:
     typedef std::map<int, NIVissimTrafficDescription*> DictType;
     static DictType myDict;
 };
-
-
-#endif
-
-/****************************************************************************/
-
