@@ -1,36 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimSingleTypeParser_Fahrzeugtypdefinition.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Wed, 18 Dec 2002
-/// @version $Id: NIVissimSingleTypeParser_Fahrzeugtypdefinition.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 //
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include "../NIImporter_Vissim.h"
 #include "../tempstructs/NIVissimVehicleType.h"
@@ -73,15 +64,15 @@ NIVissimSingleTypeParser_Fahrzeugtypdefinition::parse(std::istream& from) {
                 color = (*i).second;
             } else {
                 int r, g, b;
-                r = TplConvert::_2int(colorName.c_str());
+                r = StringUtils::toInt(colorName);
                 if (!(from >> g)) {
-                    throw NumberFormatException();
+                    throw NumberFormatException("");
                 }
                 if (!(from >> b)) {
-                    throw NumberFormatException();
+                    throw NumberFormatException("");
                 }
                 if (r < 0 || r > 255 || g < 0 || g > 255 || b < 0 || b > 255) {
-                    throw NumberFormatException();
+                    throw NumberFormatException("");
                 }
                 color = RGBColor((unsigned char)r, (unsigned char)g, (unsigned char)b, 255);
             }
@@ -112,6 +103,4 @@ NIVissimSingleTypeParser_Fahrzeugtypdefinition::parse(std::istream& from) {
 }
 
 
-
 /****************************************************************************/
-
