@@ -1,46 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimSingleTypeParser_Lichtsignalanlagendefinition.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 18 Dec 2002
-/// @version $Id: NIVissimSingleTypeParser_Lichtsignalanlagendefinition.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 //
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/MsgHandler.h>
 #include "../NIImporter_Vissim.h"
 #include "../tempstructs/NIVissimTL.h"
 #include "NIVissimSingleTypeParser_Lichtsignalanlagendefinition.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -94,7 +81,7 @@ NIVissimSingleTypeParser_Lichtsignalanlagendefinition::parse(std::istream& from)
     if (type == "extern") {
         return parseRestActuated(id, name, from, type);
     }
-    WRITE_ERROR("Unsupported LSA-Type '" + type + "' occured.");
+    WRITE_ERROR("Unsupported LSA-Type '" + type + "' occurred.");
     return false;
 }
 
@@ -106,11 +93,11 @@ NIVissimSingleTypeParser_Lichtsignalanlagendefinition::parseFixedTime(
     std::string tag;
     from >> tag;
     //
-    SUMOReal absdur;
+    double absdur;
     from >> absdur; // type-checking is missing!
     //
     tag = readEndSecure(from);
-    SUMOReal offset = 0;
+    double offset = 0;
     if (tag == "versatz") {
         from >> offset; // type-checking is missing!
     }
@@ -120,7 +107,7 @@ NIVissimSingleTypeParser_Lichtsignalanlagendefinition::parseFixedTime(
             type = "festzeit_fake";
         }
     }
-    return NIVissimTL::dictionary(id, type, name, (SUMOTime) absdur, (SUMOTime) offset);
+    return NIVissimTL::dictionary(id, type, name, TIME2STEPS(absdur), TIME2STEPS(offset));
 }
 
 
@@ -130,15 +117,15 @@ NIVissimSingleTypeParser_Lichtsignalanlagendefinition::parseVAS(
     std::string tag;
     from >> tag;
     //
-    SUMOReal absdur;
+    double absdur;
     from >> absdur; // type-checking is missing!
     //
     tag = readEndSecure(from);
-    SUMOReal offset = 0;
+    double offset = 0;
     if (tag == "versatz") {
         from >> offset; // type-checking is missing!
     }
-    return NIVissimTL::dictionary(id, "vas", name, (SUMOTime) absdur, (SUMOTime) offset);
+    return NIVissimTL::dictionary(id, "vas", name, TIME2STEPS(absdur), TIME2STEPS(offset));
 }
 
 
@@ -148,21 +135,19 @@ NIVissimSingleTypeParser_Lichtsignalanlagendefinition::parseRestActuated(
     std::string tag;
     from >> tag;
     //
-    SUMOReal absdur;
+    double absdur;
     from >> absdur; // type-checking is missing!
     //
     tag = readEndSecure(from);
-    SUMOReal offset = 0;
+    double offset = 0;
     if (tag == "versatz") {
         from >> offset; // type-checking is missing!
     }
     while (tag != "datei") {
         tag = myRead(from);
     }
-    return NIVissimTL::dictionary(id, type, name, (SUMOTime) absdur, (SUMOTime) offset);
+    return NIVissimTL::dictionary(id, type, name, TIME2STEPS(absdur), TIME2STEPS(offset));
 }
 
 
-
 /****************************************************************************/
-
