@@ -1,47 +1,34 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimSingleTypeParser_Querverkehrsstoerungsdefinition.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 18 Dec 2002
-/// @version $Id: NIVissimSingleTypeParser_Querverkehrsstoerungsdefinition.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 //
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/MsgHandler.h>
 #include "../NIImporter_Vissim.h"
 #include "../tempstructs/NIVissimExtendedEdgePoint.h"
 #include "../tempstructs/NIVissimDisturbance.h"
 #include "NIVissimSingleTypeParser_Querverkehrsstoerungsdefinition.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -87,14 +74,14 @@ NIVissimSingleTypeParser_Querverkehrsstoerungsdefinition::parsePositionDescribed
         from >> tag; // "Strecke"
         NIVissimExtendedEdgePoint by = parsePos(from);
         //
-        SUMOReal timegap;
+        double timegap;
         from >> timegap;
 
         from >> tag;
-        SUMOReal waygap;
+        double waygap;
         from >> waygap;
 
-        SUMOReal vmax = -1;
+        double vmax = -1;
         tag = readEndSecure(from);
         if (tag == "vmax") {
             from >> vmax;
@@ -131,14 +118,14 @@ NIVissimSingleTypeParser_Querverkehrsstoerungsdefinition::parseNumbered(std::ist
         from >> tag; // "Strecke"
         NIVissimExtendedEdgePoint by = parsePos(from);
         //
-        SUMOReal timegap;
+        double timegap;
         from >> timegap;
 
-        SUMOReal waygap;
+        double waygap;
         from >> tag;
         from >> waygap;
 
-        SUMOReal vmax = -1;
+        double vmax = -1;
         tag = readEndSecure(from);
         if (tag == "vmax") {
             from >> vmax;
@@ -166,10 +153,10 @@ NIVissimSingleTypeParser_Querverkehrsstoerungsdefinition::parsePos(std::istream&
     if (tag == "ALLE") {
         //lanes.push_back(1); // !!!
     } else {
-        lanes.push_back(TplConvert::_2int(tag.c_str()));
+        lanes.push_back(StringUtils::toInt(tag));
     }
     //
-    SUMOReal position;
+    double position;
     from >> tag;
     from >> position;
     // assigned vehicle types
@@ -183,7 +170,7 @@ NIVissimSingleTypeParser_Querverkehrsstoerungsdefinition::parsePos(std::istream&
                 from >> tag;
                 tag = "alle";
             } else if (tag != "zeitluecke" && tag != "durch" && tag != "DATAEND") {
-                int tmp = TplConvert::_2int(tag.c_str());
+                int tmp = StringUtils::toInt(tag);
                 types.push_back(tmp);
             }
         }
@@ -192,6 +179,4 @@ NIVissimSingleTypeParser_Querverkehrsstoerungsdefinition::parsePos(std::istream&
 }
 
 
-
 /****************************************************************************/
-
