@@ -1,38 +1,29 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimSingleTypeParser_Signalgruppendefinition.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 18 Dec 2002
-/// @version $Id: NIVissimSingleTypeParser_Signalgruppendefinition.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 //
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cassert>
 #include <iostream>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/VectorHelper.h>
@@ -40,10 +31,6 @@
 #include "../tempstructs/NIVissimTL.h"
 #include "NIVissimSingleTypeParser_Signalgruppendefinition.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -72,8 +59,8 @@ NIVissimSingleTypeParser_Signalgruppendefinition::parse(std::istream& from) {
     int lsaid;
     from >> lsaid;
     NIVissimTL* tl = NIVissimTL::dictionary(lsaid);
-    if (tl == 0) {
-        WRITE_ERROR("A traffic light group with an unknown traffic light occured.\n  Group-ID: " + toString<int>(id)
+    if (tl == nullptr) {
+        WRITE_ERROR("A traffic light group with an unknown traffic light occurred.\n  Group-ID: " + toString<int>(id)
                     + "\n  TrafficLight-ID: " + toString<int>(lsaid));
         return false;
     }
@@ -103,7 +90,7 @@ NIVissimSingleTypeParser_Signalgruppendefinition::parse(std::istream& from) {
     if (type == "pos") {
         return parsePOS(id, name, lsaid, from);
     }
-    WRITE_WARNING("Unsupported LSA-Type '" + type + "' occured.");
+    WRITE_WARNING("Unsupported LSA-Type '" + type + "' occurred.");
     return true;
 }
 
@@ -113,7 +100,7 @@ NIVissimSingleTypeParser_Signalgruppendefinition::parseFixedTime(
     int id, const std::string& name, int lsaid, std::istream& from) {
     //
     bool isGreenBegin;
-    std::vector<SUMOReal> times;
+    std::vector<SUMOTime> times;
     std::string tag = myRead(from);
     if (tag == "dauergruen") {
         isGreenBegin = true;
@@ -125,23 +112,23 @@ NIVissimSingleTypeParser_Signalgruppendefinition::parseFixedTime(
         // the first phase will be green
         isGreenBegin = true;
         while (tag == "rotende" || tag == "gruenanfang") {
-            SUMOReal point;
+            double point;
             from >> point; // type-checking is missing!
-            times.push_back(point);
+            times.push_back(TIME2STEPS(point));
             from >> tag;
             from >> point; // type-checking is missing!
-            times.push_back(point);
+            times.push_back(TIME2STEPS(point));
             tag = myRead(from);
         }
     }
     //
-    SUMOReal tredyellow, tyellow;
+    double tredyellow, tyellow;
     from >> tredyellow;
     from >> tag;
     from >> tyellow;
     NIVissimTL::NIVissimTLSignalGroup* group =
         new NIVissimTL::NIVissimTLSignalGroup(
-        id, name, isGreenBegin, times, (SUMOTime) tredyellow, (SUMOTime) tyellow);
+        id, name, isGreenBegin, times, TIME2STEPS(tredyellow), TIME2STEPS(tyellow));
     if (!NIVissimTL::NIVissimTLSignalGroup::dictionary(lsaid, id, group)) {
         throw 1; // !!!
     }
@@ -209,6 +196,4 @@ NIVissimSingleTypeParser_Signalgruppendefinition::parseExternFixedTime(
 }
 
 
-
 /****************************************************************************/
-
