@@ -1,37 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NIVissimSingleTypeParser_Verbindungsdefinition.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Wed, 18 Dec 2002
-/// @version $Id: NIVissimSingleTypeParser_Verbindungsdefinition.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 //
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <utils/geom/PositionVector.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include "../NIImporter_Vissim.h"
 #include "../tempstructs/NIVissimConnection.h"
 #include "NIVissimSingleTypeParser_Verbindungsdefinition.h"
@@ -70,12 +61,12 @@ NIVissimSingleTypeParser_Verbindungsdefinition::parse(std::istream& from) {
         if (y != "nach") {
             geom.push_back_noDoublePos(
                 Position(
-                    TplConvert::_2double(x.c_str()),
-                    TplConvert::_2double(y.c_str())
+                    StringUtils::toDouble(x),
+                    StringUtils::toDouble(y)
                 ));
             tag = myRead(from);
             try {
-                TplConvert::_2double(tag.c_str());
+                StringUtils::toDouble(tag);
                 tag = myRead(from);
             } catch (NumberFormatException&) {}
         } else {
@@ -122,7 +113,7 @@ NIVissimSingleTypeParser_Verbindungsdefinition::parse(std::istream& from) {
     if (tag == "fahrzeugklassen") {
         tag = readEndSecure(from);
         while (tag != "DATAEND" && tag != "sperrung" && tag != "auswertung") {
-            int classes = TplConvert::_2int(tag.c_str());
+            int classes = StringUtils::toInt(tag);
             assignedVehicles.push_back(classes);
             tag = readEndSecure(from, "auswertung");
         }
@@ -143,18 +134,18 @@ NIVissimSingleTypeParser_Verbindungsdefinition::parse(std::istream& from) {
                 int laneNo;
                 from >> laneNo; // unused and type-checking is missing!
                 // get the list of assigned car classes
-                std::vector<int> assignedVehicles;
+                std::vector<int> assigned;
                 tag = myRead(from);
                 if (tag == "fahrzeugklassen") {
                     tag = myRead(from);
                 }
                 while (tag != "DATAEND" && tag != "spur" && tag != "keinspurwechsel") {
-                    int classes = TplConvert::_2int(tag.c_str());
-                    assignedVehicles.push_back(classes);
+                    int classes = StringUtils::toInt(tag);
+                    assigned.push_back(classes);
                     tag = readEndSecure(from);
                 }
                 // build and add the definition
-                NIVissimClosedLaneDef* cld = new NIVissimClosedLaneDef(assignedVehicles);
+                NIVissimClosedLaneDef* cld = new NIVissimClosedLaneDef(assigned);
                 clv.push_back(cld);
             }
         } while (tag != "DATAEND");
@@ -170,6 +161,4 @@ NIVissimSingleTypeParser_Verbindungsdefinition::parse(std::istream& from) {
 }
 
 
-
 /****************************************************************************/
-
