@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NLBuilder.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: NLBuilder.cpp 24010 2017-04-21 17:00:09Z behrisch $
 ///
 // The main interface for loading a microsim
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <vector>
@@ -38,28 +29,33 @@
 #include <utils/common/MsgHandler.h>
 #include <utils/common/StringTokenizer.h>
 #include <utils/common/SystemFrame.h>
-#include <utils/iodevices/BinaryInputDevice.h>
 #include <utils/options/Option.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/options/OptionsIO.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/FileHelpers.h>
 #include <utils/common/SysUtils.h>
 #include <utils/common/ToString.h>
-#include <utils/xml/SUMORouteLoaderControl.h>
-#include <utils/xml/SUMORouteLoader.h>
+#include <utils/vehicle/SUMORouteLoaderControl.h>
+#include <utils/vehicle/SUMORouteLoader.h>
 #include <utils/xml/XMLSubSys.h>
+#ifdef HAVE_FOX
+#include <utils/foxtools/MsgHandlerSynchronized.h>
+#endif
 #include <mesosim/MEVehicleControl.h>
 #include <microsim/MSVehicleControl.h>
 #include <microsim/MSVehicleTransfer.h>
 #include <microsim/MSNet.h>
 #include <microsim/devices/MSDevice.h>
+#include <microsim/devices/MSDevice_ToC.h>
 #include <microsim/MSEdgeControl.h>
 #include <microsim/MSGlobals.h>
 #include <microsim/output/MSDetectorControl.h>
 #include <microsim/MSFrame.h>
 #include <microsim/MSEdgeWeightsStorage.h>
 #include <microsim/MSStateHandler.h>
+#include <microsim/MSDriverState.h>
+#include <traci-server/TraCIServer.h>
 
 #include "NLHandler.h"
 #include "NLEdgeControlBuilder.h"
@@ -68,9 +64,6 @@
 #include "NLTriggerBuilder.h"
 #include "NLBuilder.h"
 
-#ifndef NO_TRACI
-#include <traci-server/TraCIServer.h>
-#endif
 
 // ===========================================================================
 // method definitions
@@ -82,7 +75,7 @@ void
 NLBuilder::EdgeFloatTimeLineRetriever_EdgeEffort::addEdgeWeight(const std::string& id,
         double value, double begTime, double endTime) const {
     MSEdge* edge = MSEdge::dictionary(id);
-    if (edge != 0) {
+    if (edge != nullptr) {
         myNet.getWeightsStorage().addEffort(edge, begTime, endTime, value);
     } else {
         WRITE_ERROR("Trying to set the effort for the unknown edge '" + id + "'.");
@@ -97,7 +90,7 @@ void
 NLBuilder::EdgeFloatTimeLineRetriever_EdgeTravelTime::addEdgeWeight(const std::string& id,
         double value, double begTime, double endTime) const {
     MSEdge* edge = MSEdge::dictionary(id);
-    if (edge != 0) {
+    if (edge != nullptr) {
         myNet.getWeightsStorage().addTravelTime(edge, begTime, endTime, value);
     } else {
         WRITE_ERROR("Trying to set the travel time for the unknown edge '" + id + "'.");
@@ -128,18 +121,70 @@ NLBuilder::build() {
     if (!load("net-file", true)) {
         return false;
     }
-    // check whether the loaded net agrees with the simulation options
-    if (myOptions.getBool("no-internal-links") && myXMLHandler.haveSeenInternalEdge()) {
-        WRITE_WARNING("Network contains internal links but option --no-internal-links is set. Vehicles will 'jump' across junctions and thus underestimate route lengths and travel times.");
+    if (myXMLHandler.networkVersion() == 0.) {
+        throw ProcessError("Invalid network, no network version declared.");
     }
-    if (myOptions.getString("lanechange.duration") != "0" && myXMLHandler.haveSeenNeighs()) {
-        throw ProcessError("Network contains explicit neigh lanes which do not work together with option --lanechange.duration.");
+    // check whether the loaded net agrees with the simulation options
+    if ((myOptions.getBool("no-internal-links") || myOptions.getBool("mesosim")) && myXMLHandler.haveSeenInternalEdge() && myXMLHandler.haveSeenDefaultLength()) {
+        WRITE_WARNING("Network contains internal links which are ignored. Vehicles will 'jump' across junctions and thus underestimate route lengths and travel times.");
     }
     buildNet();
     // @note on loading order constraints:
     // - additional-files before route-files and state-files due to referencing
-    // - weight-files before additional-files since the latter might contain trips which depend on the weights
+    // - additional-files before weight-files since the latter might contain intermodal edge data and the intermodal net depends on the stops and public transport from the additionals
 
+    // load additional net elements (sources, detectors, ...)
+    if (myOptions.isSet("additional-files")) {
+        if (!load("additional-files")) {
+            return false;
+        }
+        // load shapes with separate handler
+        NLShapeHandler sh("", myNet.getShapeContainer());
+        if (!ShapeHandler::loadFiles(myOptions.getStringVector("additional-files"), sh)) {
+            return false;
+        }
+        if (myXMLHandler.haveSeenAdditionalSpeedRestrictions()) {
+            myNet.getEdgeControl().setAdditionalRestrictions();
+        }
+        if (MSGlobals::gUseMesoSim && myXMLHandler.haveSeenMesoEdgeType()) {
+            myNet.getEdgeControl().setMesoTypes();
+            for (MSTrafficLightLogic* tll : myNet.getTLSControl().getAllLogics()) {
+                tll->initMesoTLSPenalties();
+            }
+        }
+    }
+    if (myOptions.getBool("junction-taz")) {
+        // create a TAZ for every junction
+        const MSJunctionControl& junctions = myNet.getJunctionControl();
+        for (auto it = junctions.begin(); it != junctions.end(); it++) {
+            const std::string sinkID = it->first + "-sink";
+            const std::string sourceID = it->first + "-source";
+            if (MSEdge::dictionary(sinkID) == nullptr && MSEdge::dictionary(sourceID) == nullptr) {
+                // sink must be built and addd before source
+                MSEdge* sink = myEdgeBuilder.buildEdge(sinkID, SumoXMLEdgeFunc::CONNECTOR, "", "", -1, 0);
+                MSEdge* source = myEdgeBuilder.buildEdge(sourceID, SumoXMLEdgeFunc::CONNECTOR, "", "", -1, 0);
+                sink->setOtherTazConnector(source);
+                source->setOtherTazConnector(sink);
+                MSEdge::dictionary(sinkID, sink);
+                MSEdge::dictionary(sourceID, source);
+                sink->initialize(new std::vector<MSLane*>());
+                source->initialize(new std::vector<MSLane*>());
+                const MSJunction* junction = it->second;
+                for (const MSEdge* edge : junction->getIncoming()) {
+                    if (!edge->isInternal()) {
+                        const_cast<MSEdge*>(edge)->addSuccessor(sink);
+                    }
+                }
+                for (const MSEdge* edge : junction->getOutgoing()) {
+                    if (!edge->isInternal()) {
+                        source->addSuccessor(const_cast<MSEdge*>(edge));
+                    }
+                }
+            } else {
+                WRITE_WARNINGF("A TAZ with id '%' already exists. Not building junction TAZ.", it->first)
+            }
+        }
+    }
     // load weights if wished
     if (myOptions.isSet("weight-files")) {
         if (!myOptions.isUsableFileList("weight-files")) {
@@ -172,29 +217,17 @@ NLBuilder::build() {
             }
         }
     }
-    // load additional net elements (sources, detectors, ...)
-    if (myOptions.isSet("additional-files")) {
-        if (!load("additional-files")) {
-            return false;
-        }
-        // load shapes with separate handler
-        NLShapeHandler sh("", myNet.getShapeContainer());
-        if (!ShapeHandler::loadFiles(myOptions.getStringVector("additional-files"), sh)) {
-            return false;
-        }
-        if (myXMLHandler.haveSeenAdditionalSpeedRestrictions()) {
-            myNet.getEdgeControl().setAdditionalRestrictions();
-        }
-    }
     // load the previous state if wished
     if (myOptions.isSet("load-state")) {
-        long before = SysUtils::getCurrentMillis();
         const std::string& f = myOptions.getString("load-state");
-        PROGRESS_BEGIN_MESSAGE("Loading state from '" + f + "'");
+        long before = PROGRESS_BEGIN_TIME_MESSAGE("Loading state from '" + f + "'");
         MSStateHandler h(f, string2time(myOptions.getString("load-state.offset")));
         XMLSubSys::runParser(h, f);
         if (myOptions.isDefault("begin")) {
             myOptions.set("begin", time2string(h.getTime()));
+            if (TraCIServer::getInstance() != nullptr) {
+                TraCIServer::getInstance()->stateLoaded(h.getTime());
+            }
         }
         if (MsgHandler::getErrorInstance()->wasInformed()) {
             return false;
@@ -219,94 +252,104 @@ NLBuilder::build() {
 }
 
 
-int
-NLBuilder::loadAndRun() {
-    MSNet::SimulationState state = MSNet::SIMSTATE_LOADING;
-    while (state == MSNet::SIMSTATE_LOADING) {
-        OptionsCont& oc = OptionsCont::getOptions();
-        oc.clear();
-        MSFrame::fillOptions();
-        OptionsIO::getOptions();
-        if (oc.processMetaOptions(OptionsIO::getArgC() < 2)) {
-            SystemFrame::close();
-            return 0;
-        }
-        XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"));
-        if (!MSFrame::checkOptions()) {
-            throw ProcessError();
-        }
-        MsgHandler::initOutputOptions();
-        RandHelper::initRandGlobal();
-        RandHelper::initRandGlobal(MSRouteHandler::getParsingRNG());
-        RandHelper::initRandGlobal(MSDevice::getEquipmentRNG());
-        MSFrame::setMSGlobals(oc);
-        MSVehicleControl* vc = 0;
-        if (MSGlobals::gUseMesoSim) {
-            vc = new MEVehicleControl();
-        } else {
-            vc = new MSVehicleControl();
-        }
-        MSNet* net = new MSNet(vc, new MSEventControl(), new MSEventControl(), new MSEventControl());
-#ifndef NO_TRACI
-        // need to init TraCI-Server before loading routes to catch VEHICLE_STATE_BUILT
-        TraCIServer::openSocket(std::map<int, TraCIServer::CmdExecutor>());
-#endif
-
-        NLEdgeControlBuilder eb;
-        NLDetectorBuilder db(*net);
-        NLJunctionControlBuilder jb(*net, db);
-        NLTriggerBuilder tb;
-        NLHandler handler("", *net, db, tb, eb, jb);
-        tb.setHandler(&handler);
-        NLBuilder builder(oc, *net, eb, jb, db, handler);
-        if (builder.build()) {
-            state = net->simulate(string2time(oc.getString("begin")), string2time(oc.getString("end")));
-            delete net;
-        } else {
-            MsgHandler::getErrorInstance()->inform("Quitting (on error).", false);
-            delete net;
-            return 1;
-        }
+MSNet*
+NLBuilder::init(const bool isLibsumo) {
+    OptionsCont& oc = OptionsCont::getOptions();
+    oc.clear();
+    MSFrame::fillOptions();
+    OptionsIO::getOptions();
+    if (oc.processMetaOptions(OptionsIO::getArgC() < 2)) {
+        SystemFrame::close();
+        return nullptr;
     }
-    return 0;
+    SystemFrame::checkOptions();
+    XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"), oc.getString("xml-validation.routes"));
+    if (!MSFrame::checkOptions()) {
+        throw ProcessError();
+    }
+#ifdef HAVE_FOX
+    if (oc.getInt("threads") > 1) {
+        // make the output aware of threading
+        MsgHandler::setFactory(&MsgHandlerSynchronized::create);
+    }
+#endif
+    MsgHandler::initOutputOptions();
+    initRandomness();
+    MSFrame::setMSGlobals(oc);
+    MSVehicleControl* vc = nullptr;
+    if (MSGlobals::gUseMesoSim) {
+        vc = new MEVehicleControl();
+    } else {
+        vc = new MSVehicleControl();
+    }
+    MSNet* net = new MSNet(vc, new MSEventControl(), new MSEventControl(), new MSEventControl());
+    // need to init TraCI-Server before loading routes to catch VehicleState::BUILT
+    TraCIServer::openSocket(std::map<int, TraCIServer::CmdExecutor>());
+    if (isLibsumo) {
+        libsumo::Helper::registerVehicleStateListener();
+    }
+
+    NLEdgeControlBuilder eb;
+    NLDetectorBuilder db(*net);
+    NLJunctionControlBuilder jb(*net, db);
+    NLTriggerBuilder tb;
+    NLHandler handler("", *net, db, tb, eb, jb);
+    tb.setHandler(&handler);
+    NLBuilder builder(oc, *net, eb, jb, db, handler);
+    MsgHandler::getErrorInstance()->clear();
+    MsgHandler::getWarningInstance()->clear();
+    MsgHandler::getMessageInstance()->clear();
+    if (builder.build()) {
+        // preload the routes especially for TraCI
+        net->loadRoutes();
+        return net;
+    }
+    delete net;
+    throw ProcessError();
 }
 
+void
+NLBuilder::initRandomness() {
+    RandHelper::initRandGlobal();
+    RandHelper::initRandGlobal(MSRouteHandler::getParsingRNG());
+    RandHelper::initRandGlobal(MSDevice::getEquipmentRNG());
+    RandHelper::initRandGlobal(OUProcess::getRNG());
+    RandHelper::initRandGlobal(MSDevice_ToC::getResponseTimeRNG());
+    MSLane::initRNGs(OptionsCont::getOptions());
+}
 
 void
 NLBuilder::buildNet() {
-    MSEdgeControl* edges = 0;
-    MSJunctionControl* junctions = 0;
-    SUMORouteLoaderControl* routeLoaders = 0;
-    MSTLLogicControl* tlc = 0;
+    MSEdgeControl* edges = nullptr;
+    MSJunctionControl* junctions = nullptr;
+    SUMORouteLoaderControl* routeLoaders = nullptr;
+    MSTLLogicControl* tlc = nullptr;
+    std::vector<SUMOTime> stateDumpTimes;
+    std::vector<std::string> stateDumpFiles;
     try {
-        edges = myEdgeBuilder.build();
+        MSFrame::buildStreams(); // ensure streams are ready for output during building
+        edges = myEdgeBuilder.build(myXMLHandler.networkVersion());
         junctions = myJunctionBuilder.build();
+        junctions->postloadInitContainer();
         routeLoaders = buildRouteLoaderControl(myOptions);
         tlc = myJunctionBuilder.buildTLLogics();
-        MSFrame::buildStreams();
-        std::vector<SUMOTime> stateDumpTimes;
-        std::vector<std::string> stateDumpFiles;
-        const std::vector<int> times = myOptions.getIntVector("save-state.times");
-        for (std::vector<int>::const_iterator i = times.begin(); i != times.end(); ++i) {
-            stateDumpTimes.push_back(TIME2STEPS(*i));
+        for (std::string timeStr : myOptions.getStringVector("save-state.times")) {
+            stateDumpTimes.push_back(string2time(timeStr));
         }
         if (myOptions.isSet("save-state.files")) {
             stateDumpFiles = myOptions.getStringVector("save-state.files");
             if (stateDumpFiles.size() != stateDumpTimes.size()) {
-                WRITE_ERROR("Wrong number of state file names!");
+                throw ProcessError("Wrong number of state file names!");
             }
         } else {
             const std::string prefix = myOptions.getString("save-state.prefix");
             const std::string suffix = myOptions.getString("save-state.suffix");
             for (std::vector<SUMOTime>::iterator i = stateDumpTimes.begin(); i != stateDumpTimes.end(); ++i) {
-                stateDumpFiles.push_back(prefix + "_" + time2string(*i) + suffix);
+                std::string timeStamp = time2string(*i);
+                std::replace(timeStamp.begin(), timeStamp.end(), ':', '-');
+                stateDumpFiles.push_back(prefix + "_" + timeStamp + suffix);
             }
         }
-        myNet.closeBuilding(myOptions, edges, junctions, routeLoaders, tlc, stateDumpTimes, stateDumpFiles,
-                            myXMLHandler.haveSeenInternalEdge(),
-                            myXMLHandler.haveSeenNeighs(),
-                            myXMLHandler.lefthand(),
-                            myXMLHandler.networkVersion());
     } catch (IOError& e) {
         delete edges;
         delete junctions;
@@ -320,6 +363,11 @@ NLBuilder::buildNet() {
         delete tlc;
         throw;
     }
+    // if anthing goes wrong after this point, the net is responsible for cleaning up
+    myNet.closeBuilding(myOptions, edges, junctions, routeLoaders, tlc, stateDumpTimes, stateDumpFiles,
+                        myXMLHandler.haveSeenInternalEdge(),
+                        myXMLHandler.hasJunctionHigherSpeeds(),
+                        myXMLHandler.networkVersion());
 }
 
 
@@ -330,8 +378,7 @@ NLBuilder::load(const std::string& mmlWhat, const bool isNet) {
     }
     std::vector<std::string> files = myOptions.getStringVector(mmlWhat);
     for (std::vector<std::string>::const_iterator fileIt = files.begin(); fileIt != files.end(); ++fileIt) {
-        PROGRESS_BEGIN_MESSAGE("Loading " + mmlWhat + " from '" + *fileIt + "'");
-        long before = SysUtils::getCurrentMillis();
+        const long before = PROGRESS_BEGIN_TIME_MESSAGE("Loading " + mmlWhat + " from '" + *fileIt + "'");
         if (!XMLSubSys::runParser(myXMLHandler, *fileIt, isNet)) {
             WRITE_MESSAGE("Loading of " + mmlWhat + " failed.");
             return false;
