@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NLBuilder.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: NLBuilder.h 23645 2017-03-24 09:23:31Z behrisch $
 ///
 // The main interface for loading a microsim
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NLBuilder_h
-#define NLBuilder_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -101,9 +91,12 @@ public:
     virtual bool build();
 
     /**
-    * loads the net, additional routes and the detectors and starts the simulation
+    * loads the net, additional routes and the detectors
     */
-    static int loadAndRun();
+    static MSNet* init(const bool isLibsumo = false);
+
+    /// @brief initializes all RNGs
+    static void initRandomness();
 
 
 protected:
@@ -135,7 +128,7 @@ protected:
      *  "route-files" and builds loaders reading these files
      * @param[in] oc The options to read the list of route files to open from
      * @return The built route loader control
-     * @exception ProcessError If an error occured
+     * @exception ProcessError If an error occurred
      */
     SUMORouteLoaderControl* buildRouteLoaderControl(const OptionsCont& oc);
 
@@ -232,9 +225,3 @@ private:
     NLBuilder& operator=(const NLBuilder& s);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
