@@ -1,42 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NLDetectorBuilder.h
 /// @author  Daniel Krajzewicz
 /// @author  Clemens Honomichl
 /// @author  Christian Roessel
 /// @author  Michael Behrisch
 /// @date    Mon, 15 Apr 2002
-/// @version $Id: NLDetectorBuilder.h 21851 2016-10-31 12:20:12Z behrisch $
 ///
 // Builds detectors for microsim
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NLDetectorBuilder_h
-#define NLDetectorBuilder_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <microsim/output/MSCrossSection.h>
 #include <microsim/traffic_lights/MSTLLogicControl.h>
 #include <microsim/output/MSE2Collector.h>
+// #include <microsim/output/MSMultiLaneE2Collector.h>
 
 // ===========================================================================
 // class declarations
@@ -92,11 +83,11 @@ public:
      * @param[in] splInterval The aggregation time span the detector shall use
      * @param[in] device The output device the detector shall write into
      * @param[in] friendlyPos Whether the position information shall be used "friendly" (see user docs)
-     * @param[in] splitByType Whether vehicle types shall be disaggregated
+     * @param[in] vTypes which vehicle types are considered
      * @exception InvalidArgument If one of the values is invalid
      */
     void buildInductLoop(const std::string& id,
-                         const std::string& lane, SUMOReal pos, SUMOTime splInterval,
+                         const std::string& lane, double pos, SUMOTime splInterval,
                          const std::string& device, bool friendlyPos,
                          const std::string& vTypes);
 
@@ -117,105 +108,35 @@ public:
      * @exception InvalidArgument If one of the values is invalid
      */
     void buildInstantInductLoop(const std::string& id,
-                                const std::string& lane, SUMOReal pos,
+                                const std::string& lane, double pos,
                                 const std::string& device, bool friendlyPos,
                                 const std::string& vTypes);
 
 
-    /** @brief Builds an e2 detector with a fixed interval and adds it to the net
-     *
-     * Checks the given values, first. If one of the values is invalid
-     *  (lane is not known, sampling frequency<=0, position is larger
-     *  than lane's length, length is too large, the id is already in use),
-     *  an InvalidArgument is thrown.
-     *
-     * Otherwise the e2 detector is built, either by calling "buildMultiLaneE2Det"
-     *  if the detector shall continue on consecutive lanes, or by calling
-     *  "buildSingleLaneE2Det" if it is a one-lane detector.
-     *
-     * @param[in] id The id the detector shall have
-     * @param[in] lane The name of the lane the detector is placed at
-     * @param[in] pos The definition of the position on the lane the detector shall be placed at
-     * @param[in] length The definition of the length the detector shall have
-     * @param[in] cont Whether the detector shall continue on predeceeding lanes
-     * @param[in] splInterval The aggregation time span the detector shall use
-     * @param[in] device The output device the detector shall write into
-     * @param[in] haltingTimeThreshold Detector parameter: the time a vehicle's speed must be below haltingSpeedThreshold to be assigned as jammed
-     * @param[in] haltingSpeedThreshold Detector parameter: the speed a vehicle's speed must be below to be assigned as jammed
-     * @param[in] jamDistThreshold Detector parameter: the distance between two vehicles in order to not count them to one jam
-     * @param[in] friendlyPos Whether the position information shall be used "friendly" (see user docs)
-     * @exception InvalidArgument If one of the values is invalid
-     */
-    void buildE2Detector(const std::string& id, const std::string& lane, SUMOReal pos, SUMOReal length,
-                         bool cont, SUMOTime splInterval, const std::string& device, SUMOTime haltingTimeThreshold,
-                         SUMOReal haltingSpeedThreshold, SUMOReal jamDistThreshold,
-                         bool friendlyPos,
-                         const std::string& vTypes);
 
 
-    /** @brief Builds an e2 detector connected to a lsa
-     *
-     * Checks the given values, first. If one of the values is invalid
-     *  (lane is not known, position is larger than lane's length, length is too large,
-     *  the tls is not known, the id is already in use),
-     *  an InvalidArgument is thrown.
-     *
-     * Otherwise the e2 detector is built, either by calling "buildMultiLaneE2Det"
-     *  if the detector shall continue on consecutive lanes, or by calling
-     *  "buildSingleLaneE2Det" if it is a one-lane detector.
-     *
-     * @param[in] id The id the detector shall have
-     * @param[in] lane The name of the lane the detector is placed at
-     * @param[in] pos The definition of the position on the lane the detector shall be placed at
-     * @param[in] length The definition of the length the detector shall have
-     * @param[in] cont Whether the detector shall continue on predeceeding lanes
-     * @param[in] tlls The tls the detector is assigned to
-     * @param[in] device The output device the detector shall write into
-     * @param[in] haltingTimeThreshold Detector parameter: the time a vehicle's speed must be below haltingSpeedThreshold to be assigned as jammed
-     * @param[in] haltingSpeedThreshold Detector parameter: the speed a vehicle's speed must be below to be assigned as jammed
-     * @param[in] jamDistThreshold Detector parameter: the distance between two vehicles in order to not count them to one jam
-     * @param[in] friendlyPos Whether the position information shall be used "friendly" (see user docs)
-     * @exception InvalidArgument If one of the values is invalid
-     */
-    void buildE2Detector(const std::string& id, const std::string& lane, SUMOReal pos, SUMOReal length,
-                         bool cont, MSTLLogicControl::TLSLogicVariants& tlls,
-                         const std::string& device, SUMOTime haltingTimeThreshold,
-                         SUMOReal haltingSpeedThreshold, SUMOReal jamDistThreshold,
-                         bool friendlyPos,
-                         const std::string& vTypes);
+    /** @brief Builds a new E2 detector and adds it to the net's detector control. Also performs some
+    *          consistency checks for the detector positioning and applies "friendly positioning"
+    *
+    *   @param[in] tlls Traffic light logic associated to the detector
+    *   @param[in] toLane Lane associated to the detector (only for tlls != 0)
+    *   @param[in] friendlyPos Whether automatic adjustments of the detector position shall be applied in case of erroneous specification
+    *   @see For the other parameters see the MSE2Collector constructors
+    *
+    *   @todo Add parameter showDetector to indicate whether the detector should be visible in the GUI
+    *
+    */
+    void buildE2Detector(const std::string& id, MSLane* lane, double pos, double endPos, double length,
+                         const std::string& device, SUMOTime frequency,
+                         SUMOTime haltingTimeThreshold, double haltingSpeedThreshold, double jamDistThreshold,
+                         const std::string& vTypes, bool friendlyPos, bool showDetector,
+                         MSTLLogicControl::TLSLogicVariants* tlls = 0, MSLane* toLane = 0);
 
-
-    /** @brief Builds an e2 detector connected to a link's state
-     *
-     * Checks the given values, first. If one of the values is invalid
-     *  (lane is not known, position is larger than lane's length, length is too large,
-     *  the tls or the destination lane is not known, the id is already in use),
-     *  an InvalidArgument is thrown.
-     *
-     * Otherwise the e2 detector is built, either by calling "buildMultiLaneE2Det"
-     *  if the detector shall continue on consecutive lanes, or by calling
-     *  "buildSingleLaneE2Det" if it is a one-lane detector.
-     *
-     * @param[in] id The id the detector shall have
-     * @param[in] lane The name of the lane the detector is placed at
-     * @param[in] pos The definition of the position on the lane the detector shall be placed at
-     * @param[in] length The definition of the length the detector shall have
-     * @param[in] cont Whether the detector shall continue on predeceeding lanes
-     * @param[in] tlls The tls the detector is assigned to
-     * @param[in] tolane The name of the lane to which the link to which the detector to build shall be assigned to points
-     * @param[in] device The output device the detector shall write into
-     * @param[in] haltingTimeThreshold Detector parameter: the time a vehicle's speed must be below haltingSpeedThreshold to be assigned as jammed
-     * @param[in] haltingSpeedThreshold Detector parameter: the speed a vehicle's speed must be below to be assigned as jammed
-     * @param[in] jamDistThreshold Detector parameter: the distance between two vehicles in order to not count them to one jam
-     * @param[in] friendlyPos Whether the position information shall be used "friendly" (see user docs)
-     * @exception InvalidArgument If one of the values is invalid
-     */
-    void buildE2Detector(const std::string& id, const std::string& lane, SUMOReal pos, SUMOReal length,
-                         bool cont, MSTLLogicControl::TLSLogicVariants& tlls, const std::string& tolane,
-                         const std::string& device, SUMOTime haltingTimeThreshold,
-                         SUMOReal haltingSpeedThreshold, SUMOReal jamDistThreshold,
-                         bool friendlyPos,
-                         const std::string& vTypes);
+    void buildE2Detector(const std::string& id, std::vector<MSLane*> lanes, double pos, double endPos,
+                         const std::string& device, SUMOTime frequency,
+                         SUMOTime haltingTimeThreshold, double haltingSpeedThreshold, double jamDistThreshold,
+                         const std::string& vTypes, bool friendlyPos, bool showDetector,
+                         MSTLLogicControl::TLSLogicVariants* tlls = 0, MSLane* toLane = 0);
 
 
     /** @brief Stores temporary the initial information about an e3 detector to build
@@ -232,8 +153,8 @@ public:
      * @exception InvalidArgument If one of the values is invalid
      */
     void beginE3Detector(const std::string& id, const std::string& device, SUMOTime splInterval,
-                         SUMOReal haltingSpeedThreshold, SUMOTime haltingTimeThreshold,
-                         const std::string& vTypes);
+                         double haltingSpeedThreshold, SUMOTime haltingTimeThreshold,
+                         const std::string& vTypes, bool openEntry);
 
 
     /** @brief Builds an entry point of an e3 detector
@@ -248,7 +169,7 @@ public:
      * @param[in] friendlyPos Whether the position information shall be used "friendly" (see user docs)
      * @exception InvalidArgument If one of the values is invalid
      */
-    void addE3Entry(const std::string& lane, SUMOReal pos, bool friendlyPos);
+    void addE3Entry(const std::string& lane, double pos, bool friendlyPos);
 
 
     /** @brief Builds an exit point of an e3 detector
@@ -263,7 +184,7 @@ public:
      * @param[in] friendlyPos Whether the position information shall be used "friendly" (see user docs)
      * @exception InvalidArgument If one of the values is invalid
      */
-    void addE3Exit(const std::string& lane, SUMOReal pos, bool friendlyPos);
+    void addE3Exit(const std::string& lane, double pos, bool friendlyPos);
 
 
     /** @brief Builds of an e3 detector using collected values
@@ -340,11 +261,11 @@ public:
      * @param[in] id The id the detector shall have
      * @param[in] lane The lane the detector is placed at
      * @param[in] pos The position on the lane the detector is placed at
-     * @param[in] splitByType Whether additional information split by vehicle classes shall be generated
+     * @param[in] vTypes which vehicle types are considered
      * @param[in] show Whether to show the detector in the gui if available
      */
     virtual MSDetectorFileOutput* createInductLoop(const std::string& id,
-            MSLane* lane, SUMOReal pos,
+            MSLane* lane, double pos,
             const std::string& vTypes, bool show = true);
 
 
@@ -358,48 +279,25 @@ public:
      * @param[in] od The output device the loop shall use
      */
     virtual MSDetectorFileOutput* createInstantInductLoop(const std::string& id,
-            MSLane* lane, SUMOReal pos, const std::string& od,
+            MSLane* lane, double pos, const std::string& od,
             const std::string& vTypes);
 
-    /** @brief Creates an instance of an e2 detector using the given values
+
+    /** @brief Creates a MSE2Collector instance, overridden by GUIE2Collector::createE2Detector()
      *
      * Simply calls the MSE2Collector constructor
      *
-     * @param[in] id The id the detector shall have
-     * @param[in] lane The lane the detector is placed at
-     * @param[in] pos The position on the lane the detector is placed at
-     * @param[in] length The length the detector has
-     * @param[in] haltingTimeThreshold Detector parameter: the time a vehicle's speed must be below haltingSpeedThreshold to be assigned as jammed
-     * @param[in] haltingSpeedThreshold Detector parameter: the speed a vehicle's speed must be below to be assigned as jammed
-     * @param[in] jamDistThreshold Detector parameter: the distance between two vehicles in order to not count them to one jam
+     *  @see  MSE2Collector Constructor documentation
      */
-    virtual MSE2Collector* createSingleLaneE2Detector(const std::string& id,
-            DetectorUsage usage, MSLane* lane, SUMOReal pos, SUMOReal length,
-            SUMOTime haltingTimeThreshold,
-            SUMOReal haltingSpeedThreshold,
-            SUMOReal jamDistThreshold,
-            const std::string& vTypes);
-
-
-    /** @brief Creates an instance of an e2ol-detector using the given values
-     *
-     * Simply calls the MS_E2_ZS_CollectorOverLanes constructor. After this call,
-     *  the detector must be initialised.
-     *
-     * @param[in] id The id the detector shall have
-     * @param[in] lane The lane the detector is placed at
-     * @param[in] pos The position on the lane the detector is placed at
-     * @param[in] length The length the detector has
-     * @param[in] haltingTimeThreshold Detector parameter: the time a vehicle's speed must be below haltingSpeedThreshold to be assigned as jammed
-     * @param[in] haltingSpeedThreshold Detector parameter: the speed a vehicle's speed must be below to be assigned as jammed
-     * @param[in] jamDistThreshold Detector parameter: the distance between two vehicles in order to not count them to one jam
-     */
-    virtual MSDetectorFileOutput* createMultiLaneE2Detector(
-        const std::string& id, DetectorUsage usage, MSLane* lane, SUMOReal pos,
-        SUMOTime haltingTimeThreshold, SUMOReal haltingSpeedThreshold,
-        SUMOReal jamDistThreshold,
-        const std::string& vTypes);
+    virtual MSE2Collector* createE2Detector(const std::string& id,
+                                            DetectorUsage usage, MSLane* lane, double pos, double endPos, double length,
+                                            SUMOTime haltingTimeThreshold, double haltingSpeedThreshold, double jamDistThreshold,
+                                            const std::string& vTypes, bool showDetector = true);
 
+    virtual MSE2Collector* createE2Detector(const std::string& id,
+                                            DetectorUsage usage, std::vector<MSLane*> lanes, double pos, double endPos,
+                                            SUMOTime haltingTimeThreshold, double haltingSpeedThreshold, double jamDistThreshold,
+                                            const std::string& vTypes, bool showDetector = true);
 
     /** @brief Creates an instance of an e3 detector using the given values
      *
@@ -413,8 +311,8 @@ public:
      */
     virtual MSDetectorFileOutput* createE3Detector(const std::string& id,
             const CrossSectionVector& entries, const CrossSectionVector& exits,
-            SUMOReal haltingSpeedThreshold, SUMOTime haltingTimeThreshold,
-            const std::string& vTypes);
+            double haltingSpeedThreshold, SUMOTime haltingTimeThreshold,
+            const std::string& vTypes, bool openEntry);
 
 
     /** @brief Creates edge based mean data collector using the given specification
@@ -428,6 +326,7 @@ public:
      * @param[in] withEmpty Information whether empty lanes/edges shall be written
      * @param[in] withInternal Information whether internal lanes/edges shall be written
      * @param[in] trackVehicles Information whether information shall be collected per vehicle
+     * @param[in] detectPersons Whether pedestrians shall be detected instead of vehicles
      * @param[in] maxTravelTime the maximum travel time to output
      * @param[in] minSamples the minimum number of sample seconds before the values are valid
      * @param[in] haltSpeed the maximum speed to consider a vehicle waiting
@@ -437,9 +336,10 @@ public:
     void createEdgeLaneMeanData(const std::string& id, SUMOTime frequency,
                                 SUMOTime begin, SUMOTime end, const std::string& type,
                                 const bool useLanes, const bool withEmpty, const bool printDefaults,
-                                const bool withInternal, const bool trackVehicles,
-                                const SUMOReal maxTravelTime, const SUMOReal minSamples,
-                                const SUMOReal haltSpeed, const std::string& vTypes,
+                                const bool withInternal, const bool trackVehicles, const int detectPersons,
+                                const double maxTravelTime, const double minSamples,
+                                const double haltSpeed, const std::string& vTypes,
+                                const std::string& writeAttributes,
                                 const std::string& device);
     /// @}
 
@@ -458,32 +358,12 @@ public:
      * @todo Check whether this method is really needful
      */
     MSE2Collector* buildSingleLaneE2Det(const std::string& id,
-                                        DetectorUsage usage, MSLane* lane, SUMOReal pos, SUMOReal length,
-                                        SUMOTime haltingTimeThreshold, SUMOReal haltingSpeedThreshold,
-                                        SUMOReal jamDistThreshold,
+                                        DetectorUsage usage, MSLane* lane, double pos, double length,
+                                        SUMOTime haltingTimeThreshold, double haltingSpeedThreshold,
+                                        double jamDistThreshold,
                                         const std::string& vTypes);
 
 
-    /** @brief Builds an e2 detector that continues on preceeding lanes
-     *
-     * @param[in] id The id the detector shall have
-     * @param[in] usage Information how the detector is used within the simulation
-     * @param[in] lane The lane the detector is placed at
-     * @param[in] pos The position on the lane the detector is placed at
-     * @param[in] length The length the detector has
-     * @param[in] haltingTimeThreshold Detector parameter: the time a vehicle's speed must be below haltingSpeedThreshold to be assigned as jammed
-     * @param[in] haltingSpeedThreshold Detector parameter: the speed a vehicle's speed must be below to be assigned as jammed
-     * @param[in] jamDistThreshold Detector parameter: the distance between two vehicles in order to not count them to one jam
-     * @todo Check whether this method is really needful
-     */
-    MSDetectorFileOutput* buildMultiLaneE2Det(const std::string& id, DetectorUsage usage, MSLane* lane, SUMOReal pos, SUMOReal length,
-            SUMOTime haltingTimeThreshold, SUMOReal haltingSpeedThreshold,
-            SUMOReal jamDistThreshold,
-            const std::string& vTypes);
-
-
-
-
 protected:
     /**
      * @class E3DetectorDefinition
@@ -499,9 +379,9 @@ protected:
          * @param[in] splInterval The aggregation time span the detector shall use
          */
         E3DetectorDefinition(const std::string& id,
-                             const std::string& device, SUMOReal haltingSpeedThreshold,
+                             const std::string& device, double haltingSpeedThreshold,
                              SUMOTime haltingTimeThreshold, SUMOTime splInterval,
-                             const std::string& vTypes);
+                             const std::string& vTypes, bool openEntry);
 
         /// @brief Destructor
         ~E3DetectorDefinition();
@@ -511,7 +391,7 @@ protected:
         /// @brief The device the detector shall use
         const std::string myDevice;
         /// @brief The speed a vehicle's speed must be below to be assigned as jammed
-        SUMOReal myHaltingSpeedThreshold;
+        double myHaltingSpeedThreshold;
         /// @brief The time a vehicle's speed must be below haltingSpeedThreshold to be assigned as jammed
         SUMOTime myHaltingTimeThreshold;
         /// @brief List of detector's entries
@@ -522,6 +402,8 @@ protected:
         SUMOTime mySampleInterval;
         /// @brief The device the detector shall use
         const std::string myVehicleTypes;
+        /// @brief Whether the detector is declared as having incomplete entry detectors
+        bool myOpenEntry;
         //@}
 
     private:
@@ -551,33 +433,8 @@ protected:
      * @param[in] detid The id of the currently built detector (for error message generation)
      * @exception InvalidArgument If the defined position is invalid
      */
-    SUMOReal getPositionChecking(SUMOReal pos, MSLane* lane, bool friendlyPos,
-                                 const std::string& detid);
-
-
-    /** @brief Converts the length and the position information for an e2 detector
-     *
-     * @param[in] id The id of the currently built detector (for error message generation)
-     * @param[in] clane The lane the detector is placed at
-     * @param[in, out] pos The position definition to convert
-     * @param[in, out] length The length definition to convert
-     * @exception InvalidArgument If the defined position or the defined length is invalid
-     */
-    void convUncontE2PosLength(const std::string& id, MSLane* clane,
-                               SUMOReal& pos, SUMOReal& length, bool frinedly_pos);
-
-
-    /** @brief Converts the length and the position information for an e2ol-detector
-     *
-     * @param[in] id The id of the currently built detector (for error message generation)
-     * @param[in] clane The lane the detector is placed at
-     * @param[in, out] pos The position definition to convert
-     * @param[in, out] length The length definition to convert
-     * @exception InvalidArgument If the defined position or the defined length is invalid
-     */
-    void convContE2PosLength(const std::string& id, MSLane* clane,
-                             SUMOReal& pos, SUMOReal& length, bool frinedly_pos);
-
+    double getPositionChecking(double pos, MSLane* lane, bool friendlyPos,
+                               const std::string& detid);
 
 
     /// @name Value checking/adapting methods
@@ -592,7 +449,7 @@ protected:
     MSEdge* getEdgeChecking(const std::string& edgeID, SumoXMLTag type,
                             const std::string& detid);
 
-
+public:
     /** @brief Returns the named lane
      * @param[in] laneID The id of the lane
      * @param[in] type The type of the detector (for error message generation)
@@ -602,7 +459,7 @@ protected:
     MSLane* getLaneChecking(const std::string& laneID, SumoXMLTag type,
                             const std::string& detid);
 
-
+protected:
     /** @brief Checks whether the given frequency (sample interval) is valid
      * @param[in] splInterval The sample interval
      * @param[in] type The type of the detector (for error message generation)
@@ -632,9 +489,3 @@ private:
     NLDetectorBuilder& operator=(const NLDetectorBuilder&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
