@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NLHandler.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Clemens Honomichl
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NLHandler.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // The XML-Handler for network loading
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NLHandler_h
-#define NLHandler_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <utils/geom/Boundary.h>
 #include <utils/shapes/ShapeHandler.h>
@@ -48,6 +38,7 @@ class NLEdgeControlBuilder;
 class NLJunctionControlBuilder;
 class NLTriggerBuilder;
 class MSTrafficLightLogic;
+class MSRailSignal;
 
 
 // ===========================================================================
@@ -69,7 +60,11 @@ public:
     /// @brief Destructor
     virtual ~NLShapeHandler() {}
 
-    Position getLanePos(const std::string& poiID, const std::string& laneID, double lanePos);
+    Position getLanePos(const std::string& poiID, const std::string& laneID, double lanePos, double lanePosLat);
+
+    virtual bool addLanePosParams() {
+        return true;
+    }
 };
 
 
@@ -109,6 +104,14 @@ public:
         return myHaveSeenInternalEdge;
     }
 
+    bool hasJunctionHigherSpeeds() const {
+        return myHaveJunctionHigherSpeeds;
+    }
+
+    bool haveSeenDefaultLength() const {
+        return myHaveSeenDefaultLength;
+    }
+
     bool haveSeenNeighs() const {
         return myHaveSeenNeighs;
     }
@@ -117,14 +120,16 @@ public:
         return myHaveSeenAdditionalSpeedRestrictions;
     }
 
-    bool lefthand() const {
-        return myLefthand;
+    bool haveSeenMesoEdgeType() const {
+        return myHaveSeenMesoEdgeType;
     }
 
     double networkVersion() const {
         return myNetworkVersion;
     }
 
+    static void addPredecessorConstraint(int element, const SUMOSAXAttributes& attrs, MSRailSignal* rs);
+
 protected:
     /// @name inherited from GenericSAXHandler
     //@{
@@ -204,6 +209,11 @@ protected:
      */
     virtual void addEdgeLaneMeanData(const SUMOSAXAttributes& attrs, int objecttype);
 
+    /** @brief Loads edge type specific meso parameters
+     * @param[in] attrs The attributes that hold the parameters
+     */
+    virtual void addMesoEdgeType(const SUMOSAXAttributes& attrs);
+
     /// Closes the process of building an edge
     virtual void closeEdge();
 
@@ -220,12 +230,6 @@ private:
     /// adds a lane to the previously opened edge
     void addLane(const SUMOSAXAttributes& attrs);
 
-    /// adds a polygon
-    void addPOI(const SUMOSAXAttributes& attrs);
-
-    /// adds a polygon
-    void addPoly(const SUMOSAXAttributes& attrs);
-
     ///  adds a request item to the current junction logic
     void addRequest(const SUMOSAXAttributes& attrs);
 
@@ -292,7 +296,6 @@ private:
     /// @brief Parses the given character into an enumeration typed link state
     LinkState parseLinkState(const std::string& state);
 
-
 protected:
     /// @brief A builder for object actions
     NLDiscreteEventBuilder myActionBuilder;
@@ -317,7 +320,7 @@ protected:
     std::string myCurrentDistrictID;
 
     /// internal information whether a tls-logic is currently read
-    bool myAmInTLLogicMode;
+    bool myAmParsingTLLogicOrJunction;
 
     /// The id of the currently processed WAUT
     std::string myCurrentWAUTID;
@@ -333,21 +336,28 @@ protected:
 
     bool myCurrentIsBroken;
 
-    bool myHaveWarnedAboutDeprecatedLanes;
+    bool myHaveWarnedAboutInvalidTLType;
 
-    Parameterised* myLastParameterised;
+    Parameterised myLastEdgeParameters;
+    std::vector<Parameterised*> myLastParameterised;
 
     /// @brief whether the loaded network contains internal lanes
     bool myHaveSeenInternalEdge;
 
+    /// @brief Whether the network was built with higher speed on junctions
+    bool myHaveJunctionHigherSpeeds;
+
+    /// @brief whether the loaded network contains edges with default lengths
+    bool myHaveSeenDefaultLength;
+
     /// @brief whether the loaded network contains explicit neighbor lanes
     bool myHaveSeenNeighs;
 
     /// @brief whether additional files contained type-specific speed limits
     bool myHaveSeenAdditionalSpeedRestrictions;
 
-    /// @brief whether the loaded network was built for left hand traffic
-    bool myLefthand;
+    /// @brief whether edge type specific meso paramters were loaded
+    bool myHaveSeenMesoEdgeType;
 
     /// @brief the loaded network version
     double myNetworkVersion;
@@ -355,6 +365,9 @@ protected:
     /// @brief whether the location element was already loadee
     bool myNetIsLoaded;
 
+    /// @brief rail signal for which constraints are being loaded
+    MSRailSignal* myConstrainedSignal;
+
     /// @brief temporary data for building the junction graph after network parsing is finished
     typedef std::map<std::string, std::pair<std::string, std::string> > JunctionGraph;
     JunctionGraph myJunctionGraph;
@@ -367,11 +380,3 @@ private:
     NLHandler& operator=(const NLHandler& s);
 
 };
-
-
-
-
-#endif
-
-/****************************************************************************/
-
