@@ -1,51 +1,47 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NLJunctionControlBuilder.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: NLJunctionControlBuilder.cpp 21206 2016-07-20 08:08:35Z behrisch $
 ///
 // Builder of microsim-junctions and tls
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <map>
 #include <string>
 #include <vector>
 #include <list>
 #include <algorithm>
+#include <utils/xml/SUMOXMLDefinitions.h>
+#include <utils/common/UtilExceptions.h>
+#include <utils/common/ToString.h>
+#include <microsim/MSGlobals.h>
+#include <microsim/MSNet.h>
 #include <microsim/MSJunctionLogic.h>
 #include <microsim/MSNoLogicJunction.h>
 #include <microsim/MSRightOfWayJunction.h>
 #include <microsim/MSInternalJunction.h>
 #include <microsim/MSJunctionControl.h>
+#include <microsim/MSEventControl.h>
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSSimpleTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSRailSignal.h>
 #include <microsim/traffic_lights/MSRailCrossing.h>
-#include <microsim/MSEventControl.h>
 #include <microsim/traffic_lights/MSSOTLPolicyBasedTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSSOTLPlatoonPolicy.h>
 #include <microsim/traffic_lights/MSSOTLRequestPolicy.h>
@@ -54,22 +50,12 @@
 #include <microsim/traffic_lights/MSSwarmTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSDeterministicHiLevelTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSSOTLWaveTrafficLightLogic.h>
-#include <microsim/MSEventControl.h>
-#include <microsim/MSGlobals.h>
-#include <microsim/MSNet.h>
+#include <microsim/traffic_lights/MSDelayBasedTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSOffTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSTLLogicControl.h>
-#include <utils/xml/SUMOXMLDefinitions.h>
-#include <utils/common/UtilExceptions.h>
-#include <utils/common/ToString.h>
-#include <netbuild/NBNode.h>
 #include "NLBuilder.h"
 #include "NLJunctionControlBuilder.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static members
@@ -83,7 +69,7 @@ NLJunctionControlBuilder::NLJunctionControlBuilder(MSNet& net, NLDetectorBuilder
     myNet(net),
     myDetectorBuilder(db),
     myOffset(0),
-    myJunctions(0),
+    myJunctions(nullptr),
     myNetIsLoaded(false) {
     myLogicControl = new MSTLLogicControl();
     myJunctions = new MSJunctionControl();
@@ -100,89 +86,84 @@ void
 NLJunctionControlBuilder::openJunction(const std::string& id,
                                        const std::string& key,
                                        const SumoXMLNodeType type,
-                                       SUMOReal x, SUMOReal y,
+                                       const Position pos,
                                        const PositionVector& shape,
                                        const std::vector<MSLane*>& incomingLanes,
-                                       const std::vector<MSLane*>& internalLanes) {
-#ifdef HAVE_INTERNAL_LANES
+                                       const std::vector<MSLane*>& internalLanes,
+                                       const std::string& name) {
     myActiveInternalLanes = internalLanes;
-#else
-    UNUSED_PARAMETER(internalLanes);
-#endif
     myActiveIncomingLanes = incomingLanes;
     myActiveID = id;
     myActiveKey = key;
     myType = type;
-    myPosition.set(x, y);
+    myPosition.set(pos);
     myShape = shape;
+    myActiveName = name;
+    myAdditionalParameter.clear();
 }
 
 
 void
 NLJunctionControlBuilder::closeJunction(const std::string& basePath) {
-    if (myJunctions == 0) {
+    if (myJunctions == nullptr) {
         throw ProcessError("Information about the number of nodes was missing.");
     }
-    MSJunction* junction = 0;
+    MSJunction* junction = nullptr;
     switch (myType) {
-        case NODETYPE_NOJUNCTION:
-        case NODETYPE_DEAD_END:
-        case NODETYPE_DEAD_END_DEPRECATED:
-        case NODETYPE_DISTRICT:
-        case NODETYPE_TRAFFIC_LIGHT_NOJUNCTION:
+        case SumoXMLNodeType::NOJUNCTION:
+        case SumoXMLNodeType::DEAD_END:
+        case SumoXMLNodeType::DEAD_END_DEPRECATED:
+        case SumoXMLNodeType::DISTRICT:
+        case SumoXMLNodeType::TRAFFIC_LIGHT_NOJUNCTION:
             junction = buildNoLogicJunction();
             break;
-        case NODETYPE_TRAFFIC_LIGHT:
-        case NODETYPE_TRAFFIC_LIGHT_RIGHT_ON_RED:
-        case NODETYPE_RIGHT_BEFORE_LEFT:
-        case NODETYPE_PRIORITY:
-        case NODETYPE_PRIORITY_STOP:
-        case NODETYPE_ALLWAY_STOP:
-        case NODETYPE_ZIPPER:
+        case SumoXMLNodeType::TRAFFIC_LIGHT:
+        case SumoXMLNodeType::TRAFFIC_LIGHT_RIGHT_ON_RED:
+        case SumoXMLNodeType::RIGHT_BEFORE_LEFT:
+        case SumoXMLNodeType::PRIORITY:
+        case SumoXMLNodeType::PRIORITY_STOP:
+        case SumoXMLNodeType::ALLWAY_STOP:
+        case SumoXMLNodeType::ZIPPER:
             junction = buildLogicJunction();
             break;
-        case NODETYPE_INTERNAL:
-#ifdef HAVE_INTERNAL_LANES
+        case SumoXMLNodeType::INTERNAL:
             if (MSGlobals::gUsingInternalLanes) {
                 junction = buildInternalJunction();
             }
-#endif
             break;
-        case NODETYPE_RAIL_SIGNAL:
-        case NODETYPE_RAIL_CROSSING:
+        case SumoXMLNodeType::RAIL_SIGNAL:
+        case SumoXMLNodeType::RAIL_CROSSING:
             myOffset = 0;
             myActiveKey = myActiveID;
             myActiveProgram = "0";
-            myLogicType = TLTYPE_RAIL;
+            myLogicType = myType == SumoXMLNodeType::RAIL_SIGNAL ? TrafficLightType::RAIL_SIGNAL : TrafficLightType::RAIL_CROSSING;
             closeTrafficLightLogic(basePath);
             junction = buildLogicJunction();
             break;
         default:
             throw InvalidArgument("False junction logic type.");
     }
-    if (junction != 0) {
+    if (junction != nullptr) {
         if (!myJunctions->add(myActiveID, junction)) {
             throw InvalidArgument("Another junction with the id '" + myActiveID + "' exists.");
         }
     }
+    junction->updateParameters(myAdditionalParameter);
 }
 
 
 MSJunctionControl*
 NLJunctionControlBuilder::build() const {
     MSJunctionControl* js = myJunctions;
-    myJunctions = 0;
+    myJunctions = nullptr;
     return js;
 }
 
 
 MSJunction*
 NLJunctionControlBuilder::buildNoLogicJunction() {
-    return new MSNoLogicJunction(myActiveID, myType, myPosition, myShape, myActiveIncomingLanes
-#ifdef HAVE_INTERNAL_LANES
-                                 , myActiveInternalLanes
-#endif
-                                );
+    return new MSNoLogicJunction(myActiveID, myType, myPosition, myShape, myActiveName,
+                                 myActiveIncomingLanes, myActiveInternalLanes);
 }
 
 
@@ -190,22 +171,19 @@ MSJunction*
 NLJunctionControlBuilder::buildLogicJunction() {
     MSJunctionLogic* jtype = getJunctionLogicSecure();
     // build the junction
-    return new MSRightOfWayJunction(myActiveID, myType, myPosition, myShape, myActiveIncomingLanes,
-#ifdef HAVE_INTERNAL_LANES
+    return new MSRightOfWayJunction(myActiveID, myType, myPosition, myShape, myActiveName,
+                                    myActiveIncomingLanes,
                                     myActiveInternalLanes,
-#endif
                                     jtype);
 }
 
 
-#ifdef HAVE_INTERNAL_LANES
 MSJunction*
 NLJunctionControlBuilder::buildInternalJunction() {
     // build the junction
     return new MSInternalJunction(myActiveID, myType, myPosition, myShape, myActiveIncomingLanes,
                                   myActiveInternalLanes);
 }
-#endif
 
 
 MSJunctionLogic*
@@ -232,18 +210,18 @@ NLJunctionControlBuilder::closeTrafficLightLogic(const std::string& basePath) {
         }
         if (!getTLLogicControlToUse().add(myActiveKey, myActiveProgram,
                                           new MSOffTrafficLightLogic(getTLLogicControlToUse(), myActiveKey))) {
-            throw InvalidArgument("Another logic with id '" + myActiveKey + "' and subid '" + myActiveProgram + "' exists.");
+            throw InvalidArgument("Another logic with id '" + myActiveKey + "' and programID '" + myActiveProgram + "' exists.");
         }
         return;
     }
     SUMOTime firstEventOffset = 0;
     int step = 0;
-    MSTrafficLightLogic* existing = 0;
+    MSTrafficLightLogic* existing = nullptr;
     MSSimpleTrafficLightLogic::Phases::const_iterator i = myActivePhases.begin();
-    if (myLogicType != TLTYPE_RAIL) {
+    if (myLogicType != TrafficLightType::RAIL_SIGNAL && myLogicType != TrafficLightType::RAIL_CROSSING) {
         if (myAbsDuration == 0) {
             existing = getTLLogicControlToUse().get(myActiveKey, myActiveProgram);
-            if (existing == 0) {
+            if (existing == nullptr) {
                 throw InvalidArgument("TLS program '" + myActiveProgram + "' for TLS '" + myActiveKey + "' has a duration of 0.");
             } else {
                 // only modify the offset of an existing logic
@@ -266,9 +244,13 @@ NLJunctionControlBuilder::closeTrafficLightLogic(const std::string& basePath) {
             ++i;
         }
         firstEventOffset = (*i)->duration - offset + myNet.getCurrentTimeStep();
-        if (existing != 0) {
+        if (existing != nullptr) {
             existing->changeStepAndDuration(getTLLogicControlToUse(),
                                             myNet.getCurrentTimeStep(), step, (*i)->duration - offset);
+            // parameters that are used when initializing a logic will not take
+            // effect but parameters that are checked at runtime can be used
+            // here (i.e. device.glosa.range)
+            existing->updateParameters(myAdditionalParameter);
             return;
         }
     }
@@ -276,32 +258,32 @@ NLJunctionControlBuilder::closeTrafficLightLogic(const std::string& basePath) {
     if (myActiveProgram == "") {
         myActiveProgram = "default";
     }
-    MSTrafficLightLogic* tlLogic = 0;
+    MSTrafficLightLogic* tlLogic = nullptr;
     // build the tls-logic in dependance to its type
     switch (myLogicType) {
-        case TLTYPE_SWARM_BASED:
+        case TrafficLightType::SWARM_BASED:
             firstEventOffset = DELTA_T; //this is needed because swarm needs to update the pheromone on the lanes at every step
             tlLogic = new MSSwarmTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter);
             break;
-        case TLTYPE_HILVL_DETERMINISTIC:
+        case TrafficLightType::HILVL_DETERMINISTIC:
             tlLogic = new MSDeterministicHiLevelTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter);
             break;
-        case TLTYPE_SOTL_REQUEST:
-            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLRequestPolicy(myAdditionalParameter));
+        case TrafficLightType::SOTL_REQUEST:
+            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myLogicType, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLRequestPolicy(myAdditionalParameter));
             break;
-        case TLTYPE_SOTL_PLATOON:
-            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLPlatoonPolicy(myAdditionalParameter));
+        case TrafficLightType::SOTL_PLATOON:
+            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myLogicType, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLPlatoonPolicy(myAdditionalParameter));
             break;
-        case TLTYPE_SOTL_WAVE:
+        case TrafficLightType::SOTL_WAVE:
             tlLogic = new MSSOTLWaveTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter);
             break;
-        case TLTYPE_SOTL_PHASE:
-            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLPhasePolicy(myAdditionalParameter));
+        case TrafficLightType::SOTL_PHASE:
+            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myLogicType, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLPhasePolicy(myAdditionalParameter));
             break;
-        case TLTYPE_SOTL_MARCHING:
-            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLMarchingPolicy(myAdditionalParameter));
+        case TrafficLightType::SOTL_MARCHING:
+            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myLogicType, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLMarchingPolicy(myAdditionalParameter));
             break;
-        case TLTYPE_ACTUATED:
+        case TrafficLightType::ACTUATED:
             // @note it is unclear how to apply the given offset in the context
             // of variable-length phases
             tlLogic = new MSActuatedTrafficLightLogic(getTLLogicControlToUse(),
@@ -309,32 +291,36 @@ NLJunctionControlBuilder::closeTrafficLightLogic(const std::string& basePath) {
                     myActivePhases, step, (*i)->minDuration + myNet.getCurrentTimeStep(),
                     myAdditionalParameter, basePath);
             break;
-        case TLTYPE_STATIC:
-            tlLogic =
-                new MSSimpleTrafficLightLogic(getTLLogicControlToUse(),
-                                              myActiveKey, myActiveProgram,
-                                              myActivePhases, step, firstEventOffset,
-                                              myAdditionalParameter);
+        case TrafficLightType::DELAYBASED:
+            tlLogic = new MSDelayBasedTrafficLightLogic(getTLLogicControlToUse(),
+                    myActiveKey, myActiveProgram,
+                    myActivePhases, step, (*i)->minDuration + myNet.getCurrentTimeStep(),
+                    myAdditionalParameter, basePath);
             break;
-        case TLTYPE_RAIL:
-            if (myType == NODETYPE_RAIL_SIGNAL) {
-                tlLogic = new MSRailSignal(getTLLogicControlToUse(),
-                                           myActiveKey, myActiveProgram,
-                                           myAdditionalParameter);
-            } else if (myType == NODETYPE_RAIL_CROSSING) {
-                tlLogic = new MSRailCrossing(getTLLogicControlToUse(),
-                                             myActiveKey, myActiveProgram,
-                                             myAdditionalParameter);
-            } else {
-                throw ProcessError("Invalid node type '" + toString(myType)
-                                   + "' for traffic light type '" + toString(myLogicType) + "'");
-            }
+        case TrafficLightType::STATIC:
+            tlLogic = new MSSimpleTrafficLightLogic(getTLLogicControlToUse(),
+                                                    myActiveKey, myActiveProgram, TrafficLightType::STATIC,
+                                                    myActivePhases, step, firstEventOffset,
+                                                    myAdditionalParameter);
             break;
-        case TLTYPE_INVALID:
+        case TrafficLightType::RAIL_SIGNAL:
+            tlLogic = new MSRailSignal(getTLLogicControlToUse(),
+                                       myActiveKey, myActiveProgram, myNet.getCurrentTimeStep(),
+                                       myAdditionalParameter);
+            break;
+        case TrafficLightType::RAIL_CROSSING:
+            tlLogic = new MSRailCrossing(getTLLogicControlToUse(),
+                                         myActiveKey, myActiveProgram, myNet.getCurrentTimeStep(),
+                                         myAdditionalParameter);
+            break;
+        case TrafficLightType::OFF:
+            tlLogic = new MSOffTrafficLightLogic(getTLLogicControlToUse(), myActiveKey);
+            break;
+        case TrafficLightType::INVALID:
             throw ProcessError("Invalid traffic light type '" + toString(myLogicType) + "'");
     }
     myActivePhases.clear();
-    if (tlLogic != 0) {
+    if (tlLogic != nullptr) {
         if (getTLLogicControlToUse().add(myActiveKey, myActiveProgram, tlLogic)) {
             if (myNetIsLoaded) {
                 tlLogic->init(myDetectorBuilder);
@@ -342,7 +328,7 @@ NLJunctionControlBuilder::closeTrafficLightLogic(const std::string& basePath) {
                 myLogics2PostLoadInit.push_back(tlLogic);
             }
         } else {
-            WRITE_ERROR("Another logic with id '" + myActiveKey + "' and subid '" + myActiveProgram + "' exists.");
+            WRITE_ERROR("Another logic with id '" + myActiveKey + "' and programID '" + myActiveProgram + "' exists.");
             delete tlLogic;
         }
     }
@@ -419,27 +405,19 @@ NLJunctionControlBuilder::initTrafficLightLogic(const std::string& id, const std
 
 
 void
-NLJunctionControlBuilder::addPhase(SUMOTime duration, const std::string& state, SUMOTime minDuration, SUMOTime maxDuration, bool transient_notdecisional, bool commit) throw() {
+NLJunctionControlBuilder::addPhase(SUMOTime duration, const std::string& state, const std::vector<int>& nextPhases, SUMOTime minDuration, SUMOTime maxDuration, const std::string& name, bool transient_notdecisional, bool commit, MSPhaseDefinition::LaneIdVector* targetLanes) {
     // build and add the phase definition to the list
-    myActivePhases.push_back(new MSPhaseDefinition(duration, minDuration, maxDuration, state, transient_notdecisional, commit));
-    // add phase duration to the absolute duration
-    myAbsDuration += duration;
-}
-
-void
-NLJunctionControlBuilder::addPhase(SUMOTime duration, const std::string& state, SUMOTime minDuration, SUMOTime maxDuration, bool transient_notdecisional, bool commit, MSPhaseDefinition::LaneIdVector& targetLanes) throw() {
-    // build and add the phase definition to the list
-    myActivePhases.push_back(new MSPhaseDefinition(duration, minDuration, maxDuration, state, transient_notdecisional, commit, targetLanes));
+    myActivePhases.push_back(new MSPhaseDefinition(duration, state, minDuration, maxDuration, nextPhases, name, transient_notdecisional, commit, targetLanes));
     // add phase duration to the absolute duration
     myAbsDuration += duration;
 }
 
 
 void
-NLJunctionControlBuilder::addPhase(SUMOTime duration, const std::string& state,
-                                   SUMOTime minDuration, SUMOTime maxDuration) {
+NLJunctionControlBuilder::addPhase(SUMOTime duration, const std::string& state, const std::vector<int>& nextPhases,
+                                   SUMOTime minDuration, SUMOTime maxDuration, const std::string& name) {
     // build and add the phase definition to the list
-    myActivePhases.push_back(new MSPhaseDefinition(duration, minDuration, maxDuration, state));
+    myActivePhases.push_back(new MSPhaseDefinition(duration, state, minDuration, maxDuration, nextPhases, name));
     // add phase duration to the absolute duration
     myAbsDuration += duration;
 }
@@ -476,7 +454,7 @@ NLJunctionControlBuilder::buildTLLogics() {
         throw ProcessError("Traffic lights could not be built.");
     }
     MSTLLogicControl* ret = myLogicControl;
-    myLogicControl = 0;
+    myLogicControl = nullptr;
     return ret;
 }
 
@@ -490,7 +468,7 @@ NLJunctionControlBuilder::addParam(const std::string& key,
 
 MSTLLogicControl&
 NLJunctionControlBuilder::getTLLogicControlToUse() const {
-    if (myLogicControl != 0) {
+    if (myLogicControl != nullptr) {
         return *myLogicControl;
     }
     return myNet.getTLSControl();
@@ -511,9 +489,8 @@ NLJunctionControlBuilder::getActiveSubKey() const {
 
 void
 NLJunctionControlBuilder::postLoadInitialization() {
-    for (std::vector<MSTrafficLightLogic*>::const_iterator it = myLogics2PostLoadInit.begin();
-            it != myLogics2PostLoadInit.end(); ++it) {
-        (*it)->init(myDetectorBuilder);
+    for (MSTrafficLightLogic* const logic : myLogics2PostLoadInit) {
+        logic->init(myDetectorBuilder);
     }
     myNetIsLoaded = true;
 }
@@ -521,11 +498,12 @@ NLJunctionControlBuilder::postLoadInitialization() {
 
 MSJunction*
 NLJunctionControlBuilder::retrieve(const std::string id) {
-    if (myJunctions != 0) {
+    if (myJunctions != nullptr) {
         return myJunctions->get(id);
     } else {
-        return 0;
+        return nullptr;
     }
 }
 
+
 /****************************************************************************/
