@@ -1,4 +1,16 @@
-﻿/****************************************************************************/
+/****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NLTriggerBuilder.h
 /// @author  Daniel Krajzewicz
 /// @author  Tino Morenz
@@ -6,36 +18,15 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Thu, 17 Oct 2002
-/// @version $Id: NLTriggerBuilder.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Builds trigger objects for microsim
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NLTriggerBuilder_h
-#define NLTriggerBuilder_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
+#include <utils/common/RGBColor.h>
 
 
 // ===========================================================================
@@ -140,6 +131,49 @@ public:
     void parseAndBuildStoppingPlace(MSNet& net, const SUMOSAXAttributes& attrs, const SumoXMLTag element);
 
 
+    /** @brief Begin a parking area
+     *
+     * Simply calls the MSParkingArea constructor.
+     *
+     * @param[in] net The net the parking area belongs to
+     * @param[in] id The id of the parking area
+     * @param[in] lines Names of the lines that halt on this parking area
+     * @param[in] lane The lane the parking area is placed on
+     * @param[in] frompos Begin position of the parking area on the lane
+     * @param[in] topos End position of the parking area on the lane
+     * @param[in] capacity Capacity of the parking area
+     * @param[in] width Width of the default lot rectangle
+     * @param[in] length Length of the default lot rectangle
+     * @param[in] angle Angle of the default lot rectangle
+     * @exception InvalidArgument If the parking area can not be added to the net (is duplicate)
+     */
+    virtual void beginParkingArea(MSNet& net,
+                                  const std::string& id, const std::vector<std::string>& lines,
+                                  MSLane* lane, double frompos, double topos,
+                                  unsigned int capacity,
+                                  double width, double length, double angle, const std::string& name,
+                                  bool onRoad);
+
+
+    /** @brief Add a lot entry to current parking area
+     *
+     * Simply calls the addLotEntry method for current parking area.
+     *
+     * @param[in] x X position of the lot center
+     * @param[in] y Y position of the lot center
+     * @param[in] z Z position of the lot center
+     * @param[in] width Width of the lot rectangle
+     * @param[in] length Length of the lot rectangle
+     * @param[in] angle Angle of the lot rectangle
+     * @param[in] slope Slope of the lot rectangle
+     * @exception InvalidArgument If the current parking area is 0
+     */
+    void addLotEntry(double x, double y, double z,
+                     double width, double length,
+                     double angle, double slope);
+
+
+
     /** @brief Parses the values and adds an access point to the currently parsed stopping place
      *
      * @param[in] net The network the stop belongs to
@@ -149,6 +183,35 @@ public:
     void addAccess(MSNet& net, const SUMOSAXAttributes& attrs);
 
 
+    /** @brief Parses his values and builds a parking area
+     *
+     * @param[in] net The network the parking area belongs to
+     * @param[in] attrs SAX-attributes which define the trigger
+     * @exception InvalidArgument If a parameter (lane/position) is not valid
+     */
+    void parseAndBeginParkingArea(MSNet& net, const SUMOSAXAttributes& attrs);
+
+
+    /** @brief Parses his values and adds a lot entry to current parking area
+     *
+     * @param[in] attrs SAX-attributes which define the lot entry
+     */
+    void parseAndAddLotEntry(const SUMOSAXAttributes& attrs);
+
+
+    /** @brief End a parking area
+     *
+     * @exception InvalidArgument If the current parking area is 0
+     */
+    virtual void endParkingArea();
+
+    /** @brief End a stopping place
+     *
+     * @exception InvalidArgument If the current stopping place is 0
+     */
+    virtual void endStoppingPlace();
+
+
     /** @brief Parses his values and builds a charging station
      *
      * @param[in] net The network the charging station belongs to
@@ -157,6 +220,40 @@ public:
      */
     void parseAndBuildChargingStation(MSNet& net, const SUMOSAXAttributes& attrs);
 
+    /** @brief Parses its values and builds an overhead wire segment
+    *
+    * @param[in] net The network the overhead wire segment belongs to
+    * @param[in] attrs SAXattributes which define the trigger
+    * @exception InvalidArgument If a parameter (lane/position) is not valid
+    */
+    void parseAndBuildOverheadWireSegment(MSNet& net, const SUMOSAXAttributes& attrs);
+
+    /** @brief Parses its values and builds an overhead wire section
+    *
+    * @param[in] net The network the overhead wire segment belongs to
+    * @param[in] attrs SAXattributes which define the trigger
+    * @exception InvalidArgument If a substationId is not known, a segment is not known or is assigned to another overheadWireSection
+    */
+    void parseAndBuildOverheadWireSection(MSNet& net, const SUMOSAXAttributes& attrs);
+
+    /** @brief Parses its values and builds a traction substation
+    *
+    * @param[in] net The network the overhead wire segment belongs to
+    * @param[in] attrs SAXattributes which define the trigger
+    * @exception InvalidArgument  If the id is not given or is empty...
+    */
+    void parseAndBuildTractionSubstation(MSNet& net, const SUMOSAXAttributes& attrs);
+
+    /** @brief Parses its values and builds an overhead wire clamp
+    *
+    * An overhead wire clamp is a conductive connection of wires in the opposite direction over the road
+    *
+    * @param[in] net The network the overhead wire clamp belongs to
+    * @param[in] attrs SAXattributes which define the trigger
+    * @exception InvalidArgument If the traction substation is not found in the net or if the start or end segment belongs to a different traction substation
+    */
+    void parseAndBuildOverheadWireClamp(MSNet& net, const SUMOSAXAttributes& attrs);
+
     /** @brief Parses his values and builds a mesoscopic or microscopic calibrator
      *
      * @param[in] net The network the calibrator belongs to
@@ -169,6 +266,9 @@ public:
     //@}
 
 
+    MSStoppingPlace* getCurrentStop();
+
+
 protected:
     /// @name building methods
     ///
@@ -209,25 +309,67 @@ protected:
      * @param[in] element which kind of stop is to be built
      * @exception InvalidArgument If the stop can not be added to the net (is duplicate)
      */
-    virtual void buildStoppingPlace(MSNet& net, const std::string& id, const std::vector<std::string>& lines,
-                                    MSLane* lane, SUMOReal frompos, SUMOReal topos, const SumoXMLTag element);
+    virtual void buildStoppingPlace(MSNet& net, std::string id, std::vector<std::string> lines, MSLane* lane,
+                                    double frompos, double topos, const SumoXMLTag element, std::string string,
+                                    int personCapacity, double parkingLength, RGBColor& color);
 
-    /** @brief Builds a charging Station
+    /** @brief Builds a charging station
      *
-     * Simply calls the MSBusStop constructor.
+     * Simply calls the MSChargingStation constructor and adds the result to the network.
      *
-     * @param[in] net The net the charging Station belongs to
-     * @param[in] id The id of the charging Station
-     * @param[in] lines Names of the bus lines that halt on this charging Station
-     * @param[in] lane The lane the charging Station is placed on
-     * @param[in] frompos Begin position of the charging Station on the lane
-     * @param[in] topos End position of the charging Station on the lane
-     * @exception InvalidArgument If the charging Station can not be added to the net (is duplicate)
+     * @param[in] net The net the charging station belongs to
+     * @param[in] id The id of the charging station
+     * @param[in] lane The lane the charging station is placed on
+     * @param[in] frompos Begin position of the charging station on the lane
+     * @param[in] topos End position of the charging station on the lane
+     * @param[in] chargingPower energy charged in every timeStep
+     * @param[in] efficiency efficiency of the charge
+     * @param[in] chargeInTransit enable or disable charge in transit
+     * @param[in] chargeDelay delay in the charge
+     * @exception InvalidArgument If the charging station can not be added to the net (is duplicate)
      */
-    virtual void buildChargingStation(MSNet& net,
-                                      const std::string& id, const std::vector<std::string>& lines,
-                                      MSLane* lane, SUMOReal frompos, SUMOReal topos, SUMOReal chrgpower,
-                                      SUMOReal efficiency, SUMOReal chargeInTransit, SUMOReal ChargeDelay);
+    virtual void buildChargingStation(MSNet& net, const std::string& id, MSLane* lane, double frompos, double topos, const std::string& name,
+                                      double chargingPower, double efficiency, bool chargeInTransit, double chargeDelay);
+
+    /** @brief Builds an overhead wire segment
+    *
+    * Simply calls the MSOverheadWire constructor and adds the result to the network.
+    *
+    * @param[in] net The net the overhead wire segment belongs to
+    * @param[in] id The id of the overhead wire segment
+    * @param[in] lane The lane the overhead wire segment is placed on
+    * @param[in] frompos Begin position of the overhead wire segment on the lane
+    * @param[in] topos End position of the overhead wire segment  on the lane
+    * @param[in] voltageSource default voltage of overhead wire segment (unused) TODORICE
+    * @exception InvalidArgument If the overhead wire segment can not be added to the net (is duplicate according to the id)
+    */
+    virtual void buildOverheadWireSegment(MSNet& net, const std::string& id, MSLane* lane, double frompos, double topos, bool voltageSource);
+
+    /** @brief Builds an overhead wire inner segments
+    *
+    * Simply calls the buildOverheadWireSegment for inner edges connection, frontConnection and behindConnection if exists.
+    *
+    * @param[in] net The net the overhead wire inner segments belongs to
+    * @param[in] connection The inner lane that connects two regular lanes, or a regular lane with the inner "behindConnection" lane,
+                 or the inner "frontConnection" lane with a regular lane, or the inner "frontConnection" lane with the inner "behindConnection" lane
+    * @param[in] frontConnection The inner lane that connects a regular lane with the inner "connection" lane
+    * @param[in] behindConnection The inner lane that connects the inner "connection" lane with a regular lane
+    * @exception InvalidArgument If the over can not be added to the net (is duplicate according to the id)
+    */
+    void buildInnerOverheadWireSegments(MSNet& net, const MSLane* connection, const MSLane* frontConnection, const MSLane* behindConnection);
+
+    /** @brief Builds a traction substation
+    *
+    * Simply calls the MSTractionSubstaion cosnstructor and adds the substation to the list of substations in the net.
+    *
+    * @param[in] net The net the traction substation belongs to
+    * @param[in] id The id of the traction substation
+    * @param[in] voltage The voltage level of the voltage source representing the traction substation
+    * @exception InvalidArgument If the over can not be added to the net (is duplicate according to the id)
+    */
+    void buildTractionSubstation(MSNet& net, std::string id, double voltage);
+
+    virtual void buildOverheadWireClamp(MSNet& net, const std::string& id, MSLane* lane_start, MSLane* lane_end);
 
     /** @brief builds a microscopic calibrator
      *
@@ -241,9 +383,12 @@ protected:
      * @todo Is the position correct/needed
      */
     virtual MSCalibrator* buildCalibrator(MSNet& net,
-                                          const std::string& id, MSEdge* edge, SUMOReal pos,
+                                          const std::string& id, MSEdge* edge, MSLane* lane, double pos,
                                           const std::string& file, const std::string& outfile,
-                                          const SUMOTime freq, const MSRouteProbe* probe);
+                                          const SUMOTime freq,
+                                          const MSRouteProbe* probe,
+                                          const double invalidJamThreshold,
+                                          const std::string& vTypes);
 
     /** @brief builds a mesoscopic calibrator
      *
@@ -257,9 +402,12 @@ protected:
      * @todo Is the position correct/needed
      */
     virtual METriggeredCalibrator* buildMECalibrator(MSNet& net,
-            const std::string& id, const MSEdge* edge, SUMOReal pos,
+            const std::string& id, const MSEdge* edge, double pos,
             const std::string& file, const std::string& outfile,
-            const SUMOTime freq, MSRouteProbe* probe);
+            const SUMOTime freq,
+            MSRouteProbe* probe,
+            const double invalidJamThreshold,
+            const std::string& vTypes);
 
 
     /** @brief builds an rerouter
@@ -274,7 +422,9 @@ protected:
      */
     virtual MSTriggeredRerouter* buildRerouter(MSNet& net,
             const std::string& id, MSEdgeVector& edges,
-            SUMOReal prob, const std::string& file, bool off);
+            double prob, const std::string& file, bool off,
+            SUMOTime timeThreshold,
+            const std::string& vTypes);
     //@}
 
 
@@ -326,21 +476,18 @@ protected:
      * @return The position on the lane
      * @exception InvalidArgument If the position is beyond the lane
      */
-    SUMOReal getPosition(const SUMOSAXAttributes& attrs,
-                         MSLane* lane, const std::string& tt, const std::string& tid);
+    double getPosition(const SUMOSAXAttributes& attrs,
+                       MSLane* lane, const std::string& tt, const std::string& tid, MSEdge* edge = 0);
     /// @}
 
 
 protected:
     /// @brief The parent handler to set for subhandlers
     NLHandler* myHandler;
-
+    /// @brief definition of the currently parsed parking area
+    MSParkingArea* myParkingArea;
     /// @brief The currently parsed stop to add access points to
     MSStoppingPlace* myCurrentStop;
+
+    bool myHaveWarnedAboutEigen = false;
 };
-
-
-#endif
-
-/****************************************************************************/
-
