@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NWFrame.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NWFrame.cpp 23611 2017-03-22 11:45:08Z laemmel $
 ///
 // Sets and checks options for netwrite
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/options/Option.h>
@@ -49,8 +40,6 @@
 // ===========================================================================
 // static members
 // ===========================================================================
-const std::string NWFrame::MAJOR_VERSION = "0.27";
-
 
 // ===========================================================================
 // method definitions
@@ -73,12 +62,15 @@ NWFrame::fillOptions(bool forNetgen) {
     oc.addDescription("junctions.join-output", "Output",
                       "Writes information about joined junctions to FILE (can be loaded as additional node-file to reproduce joins");
 
-#ifdef HAVE_PROJ
+    oc.doRegister("prefix", new Option_String(""));
+    oc.addDescription("prefix", "Output", "Defines a prefix for edge and junction names");
+
+#ifdef PROJ_API_FILE
     if (!forNetgen) {
         oc.doRegister("proj.plain-geo", new Option_Bool(false));
         oc.addDescription("proj.plain-geo", "Projection", "Write geo coordinates in plain-xml");
     }
-#endif // HAVE_PROJ
+#endif // PROJ_API_FILE
 
     oc.doRegister("amitran-output", new Option_FileName());
     oc.addDescription("amitran-output", "Output", "The generated net will be written to FILE using Amitran format");
@@ -107,6 +99,20 @@ NWFrame::fillOptions(bool forNetgen) {
     if (!forNetgen) {
         oc.doRegister("ptstop-output", new Option_FileName());
         oc.addDescription("ptstop-output", "Output", "Writes public transport stops to FILE");
+        oc.doRegister("ptline-output", new Option_FileName());
+        oc.addDescription("ptline-output", "Output", "Writes public transport lines to FILE");
+        oc.doRegister("ptline-clean-up", new Option_Bool(false));
+        oc.addDescription("ptline-clean-up", "Output", "Clean-up pt stops that are not served by any line");
+
+        oc.doRegister("parking-output", new Option_FileName());
+        oc.addDescription("parking-output", "Output", "Writes parking areas to FILE");
+
+        oc.doRegister("railway.topology.output", new Option_FileName());
+        oc.addDescription("railway.topology.output", "Output", "Analyze topology of the railway network");
+
+        oc.doRegister("polygon-output", new Option_FileName());
+        oc.addSynonyme("polygon-output", "taz-output");
+        oc.addDescription("polygon-output", "Output", "Write shapes that are embedded in the network input and that are not supported by polyconvert (OpenDRIVE)");
     }
 
     // register opendrive options
@@ -137,6 +143,9 @@ NWFrame::checkOptions() {
         WRITE_ERROR("OpenDRIVE export needs internal links computation.");
         ok = false;
     }
+    if (oc.isSet("opendrive-output") && oc.isDefault("no-internal-links")) {
+        oc.set("no-internal-links", "false");
+    }
     if (oc.isSet("opendrive-output") && oc.isDefault("rectangular-lane-cut")) {
         oc.set("rectangular-lane-cut", "true");
     }
@@ -146,14 +155,24 @@ NWFrame::checkOptions() {
     if (oc.isSet("dlr-navteq-output") && oc.isDefault("numerical-ids")) {
         oc.set("numerical-ids", "true");
     }
+    if (oc.isSet("dlr-navteq-output") && oc.isDefault("osm.all-attributes")) {
+        oc.set("osm.all-attributes", "true");
+    }
+    if (oc.exists("ptline-output") && oc.isSet("ptline-output") && !oc.isSet("ptstop-output")) {
+        WRITE_ERROR("public transport lines output requires 'ptstop-output' to be set");
+        ok = false;
+    }
+    if (oc.exists("ptline-clean-up") && oc.getBool("ptline-clean-up") && !oc.isSet("ptline-output")) {
+        WRITE_WARNING("'ptline-clean-up' only works in conjunction with 'ptline-output'. Ignoring invalid option.");
+    }
+
     return ok;
 }
 
 
 void
 NWFrame::writeNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
-    long before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Writing network");
+    const long before = PROGRESS_BEGIN_TIME_MESSAGE("Writing network");
     NWWriter_SUMO::writeNetwork(oc, nb);
     NWWriter_Amitran::writeNetwork(oc, nb);
     NWWriter_MATSim::writeNetwork(oc, nb);
@@ -173,5 +192,5 @@ NWFrame::writePositionLong(const Position& pos, OutputDevice& dev) {
     }
 }
 
-/****************************************************************************/
 
+/****************************************************************************/
