@@ -1,32 +1,23 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2014-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NWWriter_Amitran.cpp
 /// @author  Michael Behrisch
 /// @date    13.03.2014
-/// @version $Id: NWWriter_Amitran.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Exporter writing networks using the Amitran format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2014-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <utils/common/MsgHandler.h>
 #include <netbuild/NBEdge.h>
@@ -85,40 +76,40 @@ NWWriter_Amitran::writeNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
             continue;
         }
         switch (i->second->getType()) {
-            case NODETYPE_TRAFFIC_LIGHT:
-            case NODETYPE_TRAFFIC_LIGHT_NOJUNCTION:
-            case NODETYPE_TRAFFIC_LIGHT_RIGHT_ON_RED:
+            case SumoXMLNodeType::TRAFFIC_LIGHT:
+            case SumoXMLNodeType::TRAFFIC_LIGHT_NOJUNCTION:
+            case SumoXMLNodeType::TRAFFIC_LIGHT_RIGHT_ON_RED:
                 device << "\" type=\"trafficLight";
                 break;
-            case NODETYPE_PRIORITY:
+            case SumoXMLNodeType::PRIORITY:
                 device << "\" type=\"priority";
                 break;
-            case NODETYPE_PRIORITY_STOP:
+            case SumoXMLNodeType::PRIORITY_STOP:
                 device << "\" type=\"priorityStop";
                 break;
-            case NODETYPE_RIGHT_BEFORE_LEFT:
+            case SumoXMLNodeType::RIGHT_BEFORE_LEFT:
                 device << "\" type=\"rightBeforeLeft";
                 break;
-            case NODETYPE_ALLWAY_STOP:
+            case SumoXMLNodeType::ALLWAY_STOP:
                 device << "\" type=\"allwayStop";
                 break;
-            case NODETYPE_ZIPPER:
+            case SumoXMLNodeType::ZIPPER:
                 device << "\" type=\"zipper";
                 break;
-            case NODETYPE_RAIL_SIGNAL:
+            case SumoXMLNodeType::RAIL_SIGNAL:
                 device << "\" type=\"railSignal";
                 break;
-            case NODETYPE_RAIL_CROSSING:
+            case SumoXMLNodeType::RAIL_CROSSING:
                 device << "\" type=\"railCrossing";
                 break;
-            case NODETYPE_DEAD_END:
-            case NODETYPE_DEAD_END_DEPRECATED:
+            case SumoXMLNodeType::DEAD_END:
+            case SumoXMLNodeType::DEAD_END_DEPRECATED:
                 device << "\" type=\"deadEnd";
                 break;
-            case NODETYPE_DISTRICT:
-            case NODETYPE_NOJUNCTION:
-            case NODETYPE_INTERNAL:
-            case NODETYPE_UNKNOWN:
+            case SumoXMLNodeType::DISTRICT:
+            case SumoXMLNodeType::NOJUNCTION:
+            case SumoXMLNodeType::INTERNAL:
+            case SumoXMLNodeType::UNKNOWN:
                 break;
         }
         device << "\"/>\n";
@@ -141,4 +132,3 @@ NWWriter_Amitran::writeNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
 
 
 /****************************************************************************/
-
