@@ -1,33 +1,24 @@
 /****************************************************************************/
-/// @file    NWWriter_DlrNavteq.h
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
+/// @file    NWWriter_DlrNavteq.cpp
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    26.10.2012
-/// @version $Id: NWWriter_DlrNavteq.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Exporter writing networks using DlrNavteq (Elmar) format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 #include <algorithm>
 #include <ctime>
 #include <cmath>
@@ -47,8 +38,6 @@
 #include "NWFrame.h"
 #include "NWWriter_DlrNavteq.h"
 
-#define OUTPUT_VERSION "6.5"
-
 
 // ---------------------------------------------------------------------------
 // static members
@@ -74,12 +63,7 @@ NWWriter_DlrNavteq::writeNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
 
 
 void NWWriter_DlrNavteq::writeHeader(OutputDevice& device, const OptionsCont& oc) {
-    time_t rawtime;
-    time(&rawtime);
-    char buffer [80];
-    strftime(buffer, 80, "on %c", localtime(&rawtime));
-    device << "# Generated " << buffer << " by " << oc.getFullName() << "\n";
-    device << "# Format matches Extraction version: V" << OUTPUT_VERSION << " \n";
+    device << "# Format matches Extraction version: V6.5 \n";
     std::stringstream tmp;
     oc.writeConfiguration(tmp, true, false, false);
     tmp.seekg(std::ios_base::beg);
@@ -145,7 +129,8 @@ NWWriter_DlrNavteq::writeNodesUnsplitted(const OptionsCont& oc, NBNodeCont& nc,
     std::set<std::string> reservedNodeIDs;
     const bool numericalIDs = oc.getBool("numerical-ids");
     if (oc.isSet("reserved-ids")) {
-        NBHelpers::loadPrefixedIDsFomFile(oc.getString("reserved-ids"), "node:", reservedNodeIDs);
+        NBHelpers::loadPrefixedIDsFomFile(oc.getString("reserved-ids"), "node:", reservedNodeIDs); // backward compatibility
+        NBHelpers::loadPrefixedIDsFomFile(oc.getString("reserved-ids"), "junction:", reservedNodeIDs); // selection format
     }
     if (numericalIDs) {
         avoid = nc.getAllNames();
@@ -161,15 +146,15 @@ NWWriter_DlrNavteq::writeNodesUnsplitted(const OptionsCont& oc, NBNodeCont& nc,
             // the import NIImporter_DlrNavteq checks for the presence of a
             // negated edge id to determine spread type. We may need to do some
             // shifting to make this consistent
-            const bool hasOppositeID = ec.getOppositeByID(e->getID()) != 0;
-            if (e->getLaneSpreadFunction() == LANESPREAD_RIGHT && !hasOppositeID) {
+            const bool hasOppositeID = ec.getOppositeByID(e->getID()) != nullptr;
+            if (e->getLaneSpreadFunction() == LaneSpreadFunction::RIGHT && !hasOppositeID) {
                 // need to write center-line geometry instead
                 try {
                     geom.move2side(e->getTotalWidth() / 2);
                 } catch (InvalidArgument& exception) {
                     WRITE_WARNING("Could not reconstruct shape for edge:'" + e->getID() + "' (" + exception.what() + ").");
                 }
-            } else if (e->getLaneSpreadFunction() == LANESPREAD_CENTER && hasOppositeID) {
+            } else if (e->getLaneSpreadFunction() == LaneSpreadFunction::CENTER && hasOppositeID) {
                 // need to write left-border geometry instead
                 try {
                     geom.move2side(-e->getTotalWidth() / 2);
@@ -180,7 +165,7 @@ NWWriter_DlrNavteq::writeNodesUnsplitted(const OptionsCont& oc, NBNodeCont& nc,
 
             std::string internalNodeID = e->getID();
             if (internalNodeID == UNDEFINED
-                    || (nc.retrieve(internalNodeID) != 0)
+                    || (nc.retrieve(internalNodeID) != nullptr)
                     || reservedNodeIDs.count(internalNodeID) > 0
                ) {
                 // need to invent a new name to avoid clashing with the id of a 'real' node or a reserved name
@@ -218,11 +203,21 @@ NWWriter_DlrNavteq::writeLinksUnsplitted(const OptionsCont& oc, NBEdgeCont& ec,
         const int kph = speedInKph(e->getSpeed());
         const std::string& betweenNodeID = (e->getGeometry().size() > 2) ? internalNodes[e] : UNDEFINED;
         std::string nameID = UNDEFINED;
+        std::string nameIDRegional = UNDEFINED;
         if (oc.getBool("output.street-names")) {
-            const std::string& name = i->second->getStreetName();
-            if (name != "" && nameIDs.count(name) == 0) {
-                nameID = toString(nameIDs.size());
-                nameIDs[name] = nameID;
+            const std::string& name = e->getStreetName();
+            if (name != "") {
+                if (nameIDs.count(name) == 0) {
+                    nameIDs[name] = toString(nameIDs.size());
+                }
+                nameID = nameIDs[name];
+            }
+            const std::string& name2 = e->getParameter("ref", "");
+            if (name2 != "") {
+                if (nameIDs.count(name2) == 0) {
+                    nameIDs[name2] = toString(nameIDs.size());
+                }
+                nameIDRegional = nameIDs[name2];
             }
         }
         device << e->getID() << "\t"
@@ -238,8 +233,8 @@ NWWriter_DlrNavteq::writeLinksUnsplitted(const OptionsCont& oc, NBEdgeCont& ec,
                << getNavteqLaneCode(e->getNumLanes()) << "\t"
                << getSpeedCategoryUpperBound(kph) << "\t"
                << kph << "\t"
-               << nameID << "\t" // NAME_ID1_REGIONAL XXX
-               << UNDEFINED << "\t" // NAME_ID2_LOCAL XXX
+               << nameIDRegional << "\t"
+               << nameID << "\t" // NAME_ID2_LOCAL
                << UNDEFINED << "\t" // housenumbers_right
                << UNDEFINED << "\t" // housenumbers_left
                << getSinglePostalCode(e->getParameter("postal_code", UNDEFINED), e->getID()) << "\t" // ZIP_CODE
@@ -415,7 +410,7 @@ int
 NWWriter_DlrNavteq::getFormOfWay(NBEdge* edge) {
     if (edge->getPermissions() == SVC_PEDESTRIAN) {
         return 15;
-    } else if (edge->getJunctionPriority(edge->getToNode()) == NBEdge::ROUNDABOUT) {
+    } else if (edge->getJunctionPriority(edge->getToNode()) == NBEdge::JunctionPriority::ROUNDABOUT) {
         return 4;
     } else if (edge->getTypeID() == "highway.service") {
         return 14;
@@ -558,5 +553,5 @@ NWWriter_DlrNavteq::writeConnectedLanes(const OptionsCont& oc, NBNodeCont& nc) {
     device.close();
 }
 
-/****************************************************************************/
 
+/****************************************************************************/
