@@ -1,37 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NWWriter_OpenDrive.h
 /// @author  Daniel Krajzewicz
 /// @date    Tue, 04.05.2011
-/// @version $Id: NWWriter_OpenDrive.h 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // Exporter writing networks using the openDRIVE format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NWWriter_OpenDrive_h
-#define NWWriter_OpenDrive_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <utils/common/StringBijection.h>
 #include <utils/common/SUMOVehicleClass.h>
+#include <netbuild/NBEdge.h>
 
 
 // ===========================================================================
@@ -43,6 +34,7 @@ class OptionsCont;
 class PositionVector;
 class OutputDevice;
 class OutputDevice_String;
+class ShapeContainer;
 
 
 // ===========================================================================
@@ -63,6 +55,25 @@ public:
     static void writeNetwork(const OptionsCont& oc, NBNetBuilder& nb);
 
 protected:
+    /// @brief write normal edge to device
+    static void writeNormalEdge(OutputDevice& device, const NBEdge* e,
+                                int edgeID, int fromNodeID, int toNodeID,
+                                const bool origNames,
+                                const double straightThresh,
+                                const ShapeContainer& shc);
+
+    /// @brief write internal edge to device, return next connectionID
+    static int writeInternalEdge(OutputDevice& device, OutputDevice& junctionDevice,
+                                 const NBEdge* inEdge, int nodeID,
+                                 int edgeID, int inEdgeID, int outEdgeID,
+                                 int connectionID,
+                                 const std::vector<NBEdge::Connection>& parallel,
+                                 const bool isOuterEdge,
+                                 const double straightThresh,
+                                 const std::string& centerMark);
+
+    static void addPedestrianConnection(const NBEdge* inEdge, const NBEdge* outEdge, std::vector<NBEdge::Connection>& parallel);
+
     /// @brief write geometry as sequence of lines (sumo style)
     static double writeGeomLines(const PositionVector& shape, OutputDevice& device, OutputDevice& elevationDevice, double offset = 0);
 
@@ -88,14 +99,12 @@ protected:
     static std::string getLaneType(SVCPermissions permissions);
 
     /// @brief get the left border of the given lane (the leftmost one by default)
-    static PositionVector getLeftLaneBorder(const NBEdge* edge, int laneIndex = -1);
+    static PositionVector getLeftLaneBorder(const NBEdge* edge, int laneIndex = -1, double widthOffset = 0);
+    static PositionVector getRightLaneBorder(const NBEdge* edge, int laneIndex = -1);
 
     /// @brief check if the lane geometries are compatible with OpenDRIVE assumptions (colinear stop line)
     static void checkLaneGeometries(const NBEdge* e);
+
+    /// @brief write road objects referenced as edge parameters
+    static void writeRoadObjects(OutputDevice& device, const NBEdge* e, const ShapeContainer& shc);
 };
-
-
-#endif
-
-/****************************************************************************/
-
