@@ -1,34 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NWWriter_XML.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
+/// @author  Leonhard Luecken
 /// @date    Tue, 11.05.2011
-/// @version $Id: NWWriter_XML.cpp 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // Exporter writing networks using XML (native input) format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 #include <algorithm>
 #include <utils/common/MsgHandler.h>
 #include <netbuild/NBEdge.h>
@@ -36,6 +28,8 @@
 #include <netbuild/NBNode.h>
 #include <netbuild/NBNodeCont.h>
 #include <netbuild/NBNetBuilder.h>
+#include <netbuild/NBPTLineCont.h>
+#include <netbuild/NBParking.h>
 #include <utils/common/ToString.h>
 #include <utils/common/StringUtils.h>
 #include <utils/options/OptionsCont.h>
@@ -73,6 +67,16 @@ NWWriter_XML::writeNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
     if (oc.exists("ptstop-output") && oc.isSet("ptstop-output")) {
         writePTStops(oc, nb.getPTStopCont());
     }
+    if (oc.exists("ptline-output") && oc.isSet("ptline-output")) {
+        writePTLines(oc, nb.getPTLineCont(), nb.getEdgeCont());
+    }
+
+    if (oc.exists("parking-output") && oc.isSet("parking-output")) {
+        writeParkingAreas(oc, nb.getParkingCont(), nb.getEdgeCont());
+    }
+    if (oc.exists("taz-output") && oc.isSet("taz-output")) {
+        writeDistricts(oc, nb.getDistrictCont());
+    }
 }
 
 
@@ -88,7 +92,7 @@ NWWriter_XML::writeNodes(const OptionsCont& oc, NBNodeCont& nc) {
 
     OutputDevice& device = OutputDevice::getDevice(oc.getString("plain-output-prefix") + ".nod.xml");
     std::map<SumoXMLAttr, std::string> attrs;
-    attrs[SUMO_ATTR_VERSION] = NWFrame::MAJOR_VERSION;
+    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION, 1);
     device.writeXMLHeader("nodes", "nodes_file.xsd", attrs);
 
     // write network offsets and projection to allow reconstruction of original coordinates
@@ -136,7 +140,7 @@ NWWriter_XML::writeNodes(const OptionsCont& oc, NBNodeCont& nc) {
             }
         }
         if (n->hasCustomShape()) {
-            device.writeAttr(SUMO_ATTR_SHAPE, n->getShape());
+            writeShape(device, gch, n->getShape(), SUMO_ATTR_SHAPE, useGeo, geoAccuracy);
         }
         if (n->getRadius() != NBNode::UNSPECIFIED_RADIUS) {
             device.writeAttr(SUMO_ATTR_RADIUS, n->getRadius());
@@ -144,6 +148,16 @@ NWWriter_XML::writeNodes(const OptionsCont& oc, NBNodeCont& nc) {
         if (n->getKeepClear() == false) {
             device.writeAttr<bool>(SUMO_ATTR_KEEP_CLEAR, n->getKeepClear());
         }
+        if (n->getRightOfWay() != RightOfWay::DEFAULT) {
+            device.writeAttr<std::string>(SUMO_ATTR_RIGHT_OF_WAY, toString(n->getRightOfWay()));
+        }
+        if (n->getFringeType() != FringeType::DEFAULT) {
+            device.writeAttr<std::string>(SUMO_ATTR_FRINGE, toString(n->getFringeType()));
+        }
+        if (n->getName() != "") {
+            device.writeAttr<std::string>(SUMO_ATTR_NAME, n->getName());
+        }
+        n->writeParams(device);
         device.closeTag();
     }
     device.close();
@@ -154,9 +168,9 @@ void
 NWWriter_XML::writeTypes(const OptionsCont& oc, NBTypeCont& tc) {
     OutputDevice& device = OutputDevice::getDevice(oc.getString("plain-output-prefix") + ".typ.xml");
     std::map<SumoXMLAttr, std::string> attrs;
-    attrs[SUMO_ATTR_VERSION] = NWFrame::MAJOR_VERSION;
+    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION, 1);
     device.writeXMLHeader("types", "types_file.xsd", attrs);
-    tc.writeTypes(device);
+    tc.writeEdgeTypes(device);
     device.close();
 }
 
@@ -168,12 +182,13 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
     const bool geoAccuracy = useGeo || gch.usingInverseGeoProjection();
 
     std::map<SumoXMLAttr, std::string> attrs;
-    attrs[SUMO_ATTR_VERSION] = NWFrame::MAJOR_VERSION;
+    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION, 1);
     OutputDevice& edevice = OutputDevice::getDevice(oc.getString("plain-output-prefix") + ".edg.xml");
     edevice.writeXMLHeader("edges", "edges_file.xsd", attrs);
     OutputDevice& cdevice = OutputDevice::getDevice(oc.getString("plain-output-prefix") + ".con.xml");
     cdevice.writeXMLHeader("connections", "connections_file.xsd", attrs);
     const bool writeNames = oc.getBool("output.street-names");
+    LaneSpreadFunction defaultSpread = SUMOXMLDefinitions::LaneSpreadFunctions.get(oc.getString("default.spreadtype"));
     for (std::map<std::string, NBEdge*>::const_iterator i = ec.begin(); i != ec.end(); ++i) {
         // write the edge itself to the edges-files
         NBEdge* e = (*i).second;
@@ -195,22 +210,10 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
         }
         // write non-default geometry
         if (!e->hasDefaultGeometry()) {
-            PositionVector geom = e->getGeometry();
-            if (useGeo) {
-                for (int i = 0; i < (int) geom.size(); i++) {
-                    gch.cartesian2geo(geom[i]);
-                }
-            }
-            if (geoAccuracy) {
-                edevice.setPrecision(gPrecisionGeo);
-            }
-            edevice.writeAttr(SUMO_ATTR_SHAPE, geom);
-            if (geoAccuracy) {
-                edevice.setPrecision();
-            }
+            writeShape(edevice, gch, e->getGeometry(), SUMO_ATTR_SHAPE, useGeo, geoAccuracy);
         }
         // write the spread type if not default ("right")
-        if (e->getLaneSpreadFunction() != LANESPREAD_RIGHT) {
+        if (e->getLaneSpreadFunction() != defaultSpread) {
             edevice.writeAttr(SUMO_ATTR_SPREADTYPE, toString(e->getLaneSpreadFunction()));
         }
         // write the length if it was specified
@@ -227,11 +230,17 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
         if (!e->hasLaneSpecificPermissions()) {
             writePermissions(edevice, e->getPermissions(0));
         }
+        if (!e->hasLaneSpecificStopOffsets() && e->getStopOffsets().size() != 0) {
+            NWWriter_SUMO::writeStopOffsets(edevice, e->getStopOffsets());
+        }
+        if (e->getDistance() != 0) {
+            edevice.writeAttr(SUMO_ATTR_DISTANCE, e->getDistance());
+        }
         if (e->needsLaneSpecificOutput()) {
-            for (int i = 0; i < (int)e->getLanes().size(); ++i) {
-                const NBEdge::Lane& lane = e->getLanes()[i];
+            int idx = 0;
+            for (const NBEdge::Lane& lane : e->getLanes()) {
                 edevice.openTag(SUMO_TAG_LANE);
-                edevice.writeAttr(SUMO_ATTR_INDEX, i);
+                edevice.writeAttr(SUMO_ATTR_INDEX, idx++);
                 // write allowed lanes
                 if (e->hasLaneSpecificPermissions()) {
                     writePermissions(edevice, lane.permissions);
@@ -250,21 +259,57 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
                 if (lane.accelRamp) {
                     edevice.writeAttr(SUMO_ATTR_ACCELERATION, lane.accelRamp);
                 }
+                if (lane.customShape.size() > 0) {
+                    writeShape(edevice, gch, lane.customShape, SUMO_ATTR_SHAPE, useGeo, geoAccuracy);
+                }
+                if (lane.type != "") {
+                    edevice.writeAttr(SUMO_ATTR_TYPE, lane.type);
+                }
+                if (lane.changeLeft != SVCAll && lane.changeLeft != SVC_UNSPECIFIED && lane.changeLeft != SVC_IGNORING) {
+                    edevice.writeAttr(SUMO_ATTR_CHANGE_LEFT, getVehicleClassNames(lane.changeLeft));
+                }
+                if (lane.changeRight != SVCAll && lane.changeRight != SVC_UNSPECIFIED && lane.changeRight != SVC_IGNORING) {
+                    edevice.writeAttr(SUMO_ATTR_CHANGE_RIGHT, getVehicleClassNames(lane.changeRight));
+                }
                 if (lane.oppositeID != "") {
                     edevice.openTag(SUMO_TAG_NEIGH);
                     edevice.writeAttr(SUMO_ATTR_LANE, lane.oppositeID);
                     edevice.closeTag();
                 }
+                lane.writeParams(edevice);
+                NWWriter_SUMO::writeStopOffsets(edevice, lane.stopOffsets);
                 edevice.closeTag();
             }
         }
+        e->writeParams(edevice);
         edevice.closeTag();
         // write this edge's connections to the connections-files
         const std::vector<NBEdge::Connection> connections = e->getConnections();
-        for (std::vector<NBEdge::Connection>::const_iterator c = connections.begin(); c != connections.end(); ++c) {
-            NWWriter_SUMO::writeConnection(cdevice, *e, *c, false, NWWriter_SUMO::PLAIN);
-        }
-        if (connections.size() > 0) {
+        if (connections.empty()) {
+            // if there are no connections and this appears to be customized, preserve the information
+            const int numOutgoing = (int)e->getToNode()->getOutgoingEdges().size();
+            if (numOutgoing > 0) {
+                const SVCPermissions inPerm = e->getPermissions();
+                SVCPermissions outPerm = 0;
+                for (auto out : e->getToNode()->getOutgoingEdges()) {
+                    outPerm |= out->getPermissions();
+                }
+                if ((inPerm & outPerm) != 0 && (inPerm & outPerm) != SVC_PEDESTRIAN) {
+                    cdevice.openTag(SUMO_TAG_CONNECTION);
+                    cdevice.writeAttr(SUMO_ATTR_FROM, e->getID());
+                    cdevice.closeTag();
+                    cdevice << "\n";
+                }
+            }
+        } else {
+            for (NBEdge::Connection c : connections) {
+                if (useGeo) {
+                    for (Position& p : c.customShape) {
+                        gch.cartesian2geo(p);
+                    }
+                }
+                NWWriter_SUMO::writeConnection(cdevice, *e, c, false, NWWriter_SUMO::PLAIN, geoAccuracy);
+            }
             cdevice << "\n";
         }
     }
@@ -280,25 +325,39 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
     }
     // write pedestrian crossings to the connections-file
     for (std::map<std::string, NBNode*>::const_iterator it_node = nc.begin(); it_node != nc.end(); ++it_node) {
-        const std::vector<NBNode::Crossing>& crossings = (*it_node).second->getCrossings();
-        for (std::vector<NBNode::Crossing>::const_iterator it = crossings.begin(); it != crossings.end(); it++) {
+        const std::vector<NBNode::Crossing*>& crossings = (*it_node).second->getCrossings();
+        for (auto c : crossings) {
             cdevice.openTag(SUMO_TAG_CROSSING);
             cdevice.writeAttr(SUMO_ATTR_NODE, (*it_node).second->getID());
-            cdevice.writeAttr(SUMO_ATTR_EDGES, (*it).edges);
-            cdevice.writeAttr(SUMO_ATTR_PRIORITY, (*it).priority);
-            if ((*it).width != NBNode::DEFAULT_CROSSING_WIDTH) {
-                cdevice.writeAttr(SUMO_ATTR_WIDTH, (*it).width);
+            cdevice.writeAttr(SUMO_ATTR_EDGES, c->edges);
+            cdevice.writeAttr(SUMO_ATTR_PRIORITY, c->priority);
+            if (c->customWidth != NBEdge::UNSPECIFIED_WIDTH) {
+                cdevice.writeAttr(SUMO_ATTR_WIDTH, c->customWidth);
+            }
+            if (c->customTLIndex != -1) {
+                cdevice.writeAttr(SUMO_ATTR_TLLINKINDEX, c->customTLIndex);
+            }
+            if (c->customTLIndex2 != -1) {
+                cdevice.writeAttr(SUMO_ATTR_TLLINKINDEX2, c->customTLIndex2);
+            }
+            if (c->customShape.size() != 0) {
+                writeShape(cdevice, gch, c->customShape, SUMO_ATTR_SHAPE, useGeo, geoAccuracy);
             }
             cdevice.closeTag();
         }
     }
-    // write customShapes to the connection-file
+    // write custom walkingarea shapes to the connections file
     for (std::map<std::string, NBNode*>::const_iterator it_node = nc.begin(); it_node != nc.end(); ++it_node) {
-        NBNode::CustomShapeMap customShapes = (*it_node).second->getCustomLaneShapes();
-        for (NBNode::CustomShapeMap::const_iterator it = customShapes.begin(); it != customShapes.end(); ++it) {
-            cdevice.openTag(SUMO_TAG_CUSTOMSHAPE);
-            cdevice.writeAttr(SUMO_ATTR_ID, (*it).first);
-            cdevice.writeAttr(SUMO_ATTR_SHAPE, (*it).second);
+        for (const auto& wacs : it_node->second->getWalkingAreaCustomShapes()) {
+            cdevice.openTag(SUMO_TAG_WALKINGAREA);
+            cdevice.writeAttr(SUMO_ATTR_NODE, it_node->first);
+            cdevice.writeAttr(SUMO_ATTR_EDGES, joinNamedToString(wacs.edges, " "));
+            if (wacs.shape.size() != 0) {
+                writeShape(cdevice, gch, wacs.shape, SUMO_ATTR_SHAPE, useGeo, geoAccuracy);
+            }
+            if (wacs.width != NBEdge::UNSPECIFIED_WIDTH) {
+                cdevice.writeAttr(SUMO_ATTR_WIDTH, wacs.width);
+            }
             cdevice.closeTag();
         }
     }
@@ -311,7 +370,7 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
 void
 NWWriter_XML::writeTrafficLights(const OptionsCont& oc, NBTrafficLightLogicCont& tc, NBEdgeCont& ec) {
     std::map<SumoXMLAttr, std::string> attrs;
-    attrs[SUMO_ATTR_VERSION] = NWFrame::MAJOR_VERSION;
+    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION, 1);
     OutputDevice& device = OutputDevice::getDevice(oc.getString("plain-output-prefix") + ".tll.xml");
     device.writeXMLHeader("tlLogics", "tllogic_file.xsd", attrs);
     NWWriter_SUMO::writeTrafficLights(device, tc);
@@ -334,7 +393,7 @@ NWWriter_XML::writeTrafficLights(const OptionsCont& oc, NBTrafficLightLogicCont&
 void
 NWWriter_XML::writeJoinedJunctions(const OptionsCont& oc, NBNodeCont& nc) {
     std::map<SumoXMLAttr, std::string> attrs;
-    attrs[SUMO_ATTR_VERSION] = NWFrame::MAJOR_VERSION;
+    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION, 1);
     OutputDevice& device = OutputDevice::getDevice(oc.getString("junctions.join-output"));
     device.writeXMLHeader("nodes", "nodes_file.xsd", attrs);
     const std::vector<std::set<std::string> >& clusters = nc.getJoinedClusters();
@@ -377,7 +436,48 @@ NWWriter_XML::writePTStops(const OptionsCont& oc, NBPTStopCont& sc) {
     }
     device.close();
 }
+void NWWriter_XML::writePTLines(const OptionsCont& oc, NBPTLineCont& lc, NBEdgeCont& ec) {
+    OutputDevice& device = OutputDevice::getDevice(oc.getString("ptline-output"));
+    device.writeXMLHeader("ptLines", "ptlines_file.xsd");
+    for (const auto& item : lc.getLines()) {
+        item.second->write(device, ec);
+    }
+    device.close();
+}
 
+void NWWriter_XML::writeParkingAreas(const OptionsCont& oc, NBParkingCont& pc, NBEdgeCont& ec) {
+    OutputDevice& device = OutputDevice::getDevice(oc.getString("parking-output"));
+    device.writeXMLHeader("additional", "additional_file.xsd");
+    for (NBParking& p : pc) {
+        p.write(device, ec);
+    }
+    device.close();
+}
+
+void
+NWWriter_XML::writeDistricts(const OptionsCont& oc, NBDistrictCont& dc) {
+    OutputDevice& device = OutputDevice::getDevice(oc.getString("taz-output"));
+    device.writeXMLHeader("additional", "additional_file.xsd");
+    for (std::map<std::string, NBDistrict*>::const_iterator i = dc.begin(); i != dc.end(); i++) {
+        NWWriter_SUMO::writeDistrict(device, *(*i).second);
+    }
+}
+
+void
+NWWriter_XML::writeShape(OutputDevice& out, const GeoConvHelper& gch, PositionVector shape, SumoXMLAttr attr, bool useGeo, bool geoAccuracy) {
+    if (useGeo) {
+        for (int i = 0; i < (int) shape.size(); i++) {
+            gch.cartesian2geo(shape[i]);
+        }
+    }
+    if (geoAccuracy) {
+        out.setPrecision(gPrecisionGeo);
+    }
+    out.writeAttr(attr, shape);
+    if (geoAccuracy) {
+        out.setPrecision();
+    }
+}
 
 /****************************************************************************/
 
