@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NWWriter_XML.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 11.05.2011
-/// @version $Id: NWWriter_XML.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Exporter writing networks using XML (native input) format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef NWWriter_XML_h
-#define NWWriter_XML_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -46,6 +36,9 @@ class NBEdgeCont;
 class NBNetBuilder;
 class NBNode;
 class NBNodeCont;
+class NBParkingCont;
+class NBPTStopCont;
+class NBPTLineCont;
 class NBTrafficLightLogicCont;
 class NBTypeCont;
 class OptionsCont;
@@ -87,6 +80,12 @@ private:
      */
     static void writeNodes(const OptionsCont& oc, NBNodeCont& nc);
 
+    /** @brief Writes the types file
+     * @param[in] oc The options to use
+     * @param[in] nc The type-container from which to read data
+     */
+    static void writeTypes(const OptionsCont& oc, NBTypeCont& tc);
+
     /** @brief Writes the edges and connections files
      * @param[in] oc The options to use
      * @param[in] nb The network build from which to read data
@@ -101,10 +100,18 @@ private:
      */
     static void writeTrafficLights(const OptionsCont& oc, NBTrafficLightLogicCont& tc, NBEdgeCont& ec);
 
+    /** @brief Writes the pt stops file
+     * @param[in] oc The options to use
+     * @param[in] nc The pt stop container from which to read data
+     */
+    static void writePTStops(const OptionsCont& oc, NBPTStopCont& ec);
+    static void writePTLines(const OptionsCont& cont, NBPTLineCont& lc, NBEdgeCont& ec);
+
+    /// @brief writes imported parking areas to file
+    static void writeParkingAreas(const OptionsCont& cont, NBParkingCont& pc, NBEdgeCont& ec);
+
+    /// @brief writes imported districts (TAZ) to file
+    static void writeDistricts(const OptionsCont& oc, NBDistrictCont& dc);
+
+    static void writeShape(OutputDevice& out, const GeoConvHelper& gch, PositionVector shape, SumoXMLAttr attr, bool useGeo, bool geoAccuracy);
 };
-
-
-#endif
-
-/****************************************************************************/
-
