@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    ODCell.h
 /// @author  Peter Mieth
 /// @author  Daniel Krajzewicz
 /// @author  Yun-Pang Floetteroed
 /// @date    Sept 2002
-/// @version $Id: ODCell.h 18534 2015-07-07 09:26:36Z behrisch $
 ///
 // A single O/D-matrix cell
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef ODCell_h
-#define ODCell_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <map>
@@ -57,7 +47,7 @@ class RORoute;
  */
 struct ODCell {
     /// @brief The number of vehicles
-    SUMOReal vehicleNumber;
+    double vehicleNumber;
 
     /// @brief The begin time this cell describes
     SUMOTime begin;
@@ -79,10 +69,10 @@ struct ODCell {
 
     /// @brief mapping of departure times to departing vehicles, if already fixed
     std::map<SUMOTime, std::vector<std::string> > departures;
+
+    /// @brief the origin "district" is an edge id
+    bool originIsEdge = false;
+
+    /// @brief the destination "district" is an edge id
+    bool destinationIsEdge = false;
 };
-
-
-#endif
-
-/****************************************************************************/
-
