@@ -1,42 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    ODDistrict.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: ODDistrict.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A district (origin/destination)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef ODDistrict_h
-#define ODDistrict_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
 #include <utility>
 #include <utils/common/Named.h>
 #include <utils/common/UtilExceptions.h>
-#include <utils/common/RandomDistributor.h>
+#include <utils/distribution/RandomDistributor.h>
 
 
 // ===========================================================================
@@ -75,7 +65,7 @@ public:
      * @param[in] id The id of the source
      * @param[in] weight The weight (probability to be chosen) of the source
      */
-    void addSource(const std::string& id, SUMOReal weight);
+    void addSource(const std::string& id, double weight);
 
 
     /** @brief Adds a sink connection
@@ -91,7 +81,7 @@ public:
      * @param[in] id The id of the sink
      * @param[in] weight The weight (probability to be chosen) of the sink
      */
-    void addSink(const std::string& id, SUMOReal weight);
+    void addSink(const std::string& id, double weight);
 
 
     /** @brief Returns the id of a source to use
@@ -120,14 +110,14 @@ public:
      *
      * @return The number of known sinks
      */
-    unsigned int sinkNumber() const;
+    int sinkNumber() const;
 
 
     /** @brief Returns the number of sources
      *
      * @return The number of known sources
      */
-    unsigned int sourceNumber() const;
+    int sourceNumber() const;
 
 
 private:
@@ -147,9 +137,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
