@@ -1,102 +1,95 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2006-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    ODMatrix.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Yun-Pang Floetteroed
 /// @date    05 Apr. 2006
-/// @version $Id: ODMatrix.cpp 20842 2016-06-02 07:53:40Z behrisch $
 ///
 // An O/D (origin/destination) matrix
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2006-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <algorithm>
 #include <list>
 #include <iterator>
 #include <utils/options/OptionsCont.h>
+#include <utils/common/FileHelpers.h>
 #include <utils/common/StdDefs.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/ToString.h>
 #include <utils/common/RandHelper.h>
 #include <utils/common/StringUtils.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/StringTokenizer.h>
 #include <utils/common/SUMOTime.h>
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/importio/LineReader.h>
 #include <utils/xml/SUMOSAXHandler.h>
 #include <utils/xml/XMLSubSys.h>
+#include <router/RORoute.h>
 #include "ODAmitranHandler.h"
 #include "ODMatrix.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
 ODMatrix::ODMatrix(const ODDistrictCont& dc)
-    : myDistricts(dc), myNumLoaded(0), myNumWritten(0), myNumDiscarded(0) {}
+    : myDistricts(dc), myNumLoaded(0), myNumWritten(0), myNumDiscarded(0), myBegin(-1), myEnd(-1) {}
 
 
 ODMatrix::~ODMatrix() {
-    for (std::vector<ODCell*>::iterator i = myContainer.begin(); i != myContainer.end(); ++i) {
-        delete *i;
+    for (ODCell* const cell : myContainer) {
+        for (RORoute* const r : cell->pathsVector) {
+            delete r;
+        }
+        delete cell;
     }
     myContainer.clear();
 }
 
 
 bool
-ODMatrix::add(SUMOReal vehicleNumber, SUMOTime begin,
+ODMatrix::add(double vehicleNumber, SUMOTime begin,
               SUMOTime end, const std::string& origin, const std::string& destination,
-              const std::string& vehicleType) {
+              const std::string& vehicleType, const bool originIsEdge, const bool destinationIsEdge) {
     myNumLoaded += vehicleNumber;
-    if (myDistricts.get(origin) == 0 && myDistricts.get(destination) == 0) {
+    if (!originIsEdge && !destinationIsEdge && myDistricts.get(origin) == nullptr && myDistricts.get(destination) == nullptr) {
         WRITE_WARNING("Missing origin '" + origin + "' and destination '" + destination + "' (" + toString(vehicleNumber) + " vehicles).");
+        myNumDiscarded += vehicleNumber;
         myMissingDistricts.insert(origin);
         myMissingDistricts.insert(destination);
         return false;
-    } else if (myDistricts.get(origin) == 0 && vehicleNumber > 0) {
+    } else if (!originIsEdge && myDistricts.get(origin) == 0 && vehicleNumber > 0) {
         WRITE_ERROR("Missing origin '" + origin + "' (" + toString(vehicleNumber) + " vehicles).");
         myNumDiscarded += vehicleNumber;
         myMissingDistricts.insert(origin);
         return false;
-    } else if (myDistricts.get(destination) == 0 && vehicleNumber > 0) {
+    } else if (!destinationIsEdge && myDistricts.get(destination) == 0 && vehicleNumber > 0) {
         WRITE_ERROR("Missing destination '" + destination + "' (" + toString(vehicleNumber) + " vehicles).");
         myNumDiscarded += vehicleNumber;
         myMissingDistricts.insert(destination);
         return false;
     }
-    if (myDistricts.get(origin)->sourceNumber() == 0) {
+    if (!originIsEdge && myDistricts.get(origin)->sourceNumber() == 0) {
         WRITE_ERROR("District '" + origin + "' has no source.");
         myNumDiscarded += vehicleNumber;
         return false;
-    } else if (myDistricts.get(destination)->sinkNumber() == 0) {
+    } else if (!destinationIsEdge && myDistricts.get(destination)->sinkNumber() == 0) {
         WRITE_ERROR("District '" + destination + "' has no sink.");
         myNumDiscarded += vehicleNumber;
         return false;
@@ -108,6 +101,8 @@ ODMatrix::add(SUMOReal vehicleNumber, SUMOTime begin,
     cell->destination = destination;
     cell->vehicleType = vehicleType;
     cell->vehicleNumber = vehicleNumber;
+    cell->originIsEdge = originIsEdge;
+    cell->destinationIsEdge = destinationIsEdge;
     myContainer.push_back(cell);
     return true;
 }
@@ -115,26 +110,26 @@ ODMatrix::add(SUMOReal vehicleNumber, SUMOTime begin,
 
 bool
 ODMatrix::add(const std::string& id, const SUMOTime depart,
-              const std::pair<const std::string, const std::string>& od,
-              const std::string& vehicleType) {
-    if (myMissingDistricts.count(od.first) > 0 || myMissingDistricts.count(od.second) > 0) {
+              const std::string& fromTaz, const std::string& toTaz,
+              const std::string& vehicleType, const bool originIsEdge, const bool destinationIsEdge) {
+    if (myMissingDistricts.count(fromTaz) > 0 || myMissingDistricts.count(toTaz) > 0) {
         myNumLoaded += 1.;
         myNumDiscarded += 1.;
         return false;
     }
     // we start looking from the end because there is a high probability that the input is sorted by time
-    std::vector<ODCell*>& odList = myShortCut[od];
-    ODCell* cell = 0;
+    std::vector<ODCell*>& odList = myShortCut[std::make_pair(fromTaz, toTaz)];
+    ODCell* cell = nullptr;
     for (std::vector<ODCell*>::const_reverse_iterator c = odList.rbegin(); c != odList.rend(); ++c) {
         if ((*c)->begin <= depart && (*c)->end > depart && (*c)->vehicleType == vehicleType) {
             cell = *c;
             break;
         }
     }
-    if (cell == 0) {
+    if (cell == nullptr) {
         const SUMOTime interval = string2time(OptionsCont::getOptions().getString("aggregation-interval"));
         const int intervalIdx = (int)(depart / interval);
-        if (add(1., intervalIdx * interval, (intervalIdx + 1) * interval, od.first, od.second, vehicleType)) {
+        if (add(1., intervalIdx * interval, (intervalIdx + 1) * interval, fromTaz, toTaz, vehicleType, originIsEdge, destinationIsEdge)) {
             cell = myContainer.back();
             odList.push_back(cell);
         } else {
@@ -149,27 +144,27 @@ ODMatrix::add(const std::string& id, const SUMOTime depart,
 }
 
 
-SUMOReal
+double
 ODMatrix::computeDeparts(ODCell* cell,
-                         size_t& vehName, std::vector<ODVehicle>& into,
+                         int& vehName, std::vector<ODVehicle>& into,
                          const bool uniform, const bool differSourceSink,
                          const std::string& prefix) {
     int vehicles2insert = (int) cell->vehicleNumber;
     // compute whether the fraction forces an additional vehicle insertion
-    if (RandHelper::rand() < cell->vehicleNumber - (SUMOReal)vehicles2insert) {
+    if (RandHelper::rand() < cell->vehicleNumber - (double)vehicles2insert) {
         vehicles2insert++;
     }
     if (vehicles2insert == 0) {
         return cell->vehicleNumber;
     }
 
-    const SUMOReal offset = (SUMOReal)(cell->end - cell->begin) / (SUMOReal) vehicles2insert / (SUMOReal) 2.;
+    const double offset = (double)(cell->end - cell->begin) / (double) vehicles2insert / (double) 2.;
     for (int i = 0; i < vehicles2insert; ++i) {
         ODVehicle veh;
         veh.id = prefix + toString(vehName++);
 
         if (uniform) {
-            veh.depart = (SUMOTime)(offset + cell->begin + ((SUMOReal)(cell->end - cell->begin) * (SUMOReal) i / (SUMOReal) vehicles2insert));
+            veh.depart = (SUMOTime)(offset + cell->begin + ((double)(cell->end - cell->begin) * (double) i / (double) vehicles2insert));
         } else {
             veh.depart = (SUMOTime)RandHelper::rand(cell->begin, cell->end);
         }
@@ -221,18 +216,27 @@ void
 ODMatrix::write(SUMOTime begin, const SUMOTime end,
                 OutputDevice& dev, const bool uniform,
                 const bool differSourceSink, const bool noVtype,
-                const std::string& prefix, const bool stepLog) {
+                const std::string& prefix, const bool stepLog,
+                bool pedestrians, bool persontrips,
+                const std::string& modes) {
     if (myContainer.size() == 0) {
         return;
     }
-    std::map<std::pair<std::string, std::string>, SUMOReal> fractionLeft;
-    size_t vehName = 0;
+    std::map<std::pair<std::string, std::string>, double> fractionLeft;
+    int vehName = 0;
     sortByBeginTime();
     // recheck begin time
     begin = MAX2(begin, myContainer.front()->begin);
     std::vector<ODCell*>::iterator next = myContainer.begin();
     std::vector<ODVehicle> vehicles;
     SUMOTime lastOut = -DELTA_T;
+
+    const OptionsCont& oc = OptionsCont::getOptions();
+    std::string personDepartPos = oc.isSet("departpos") ? oc.getString("departpos") : "random";
+    std::string personArrivalPos = oc.isSet("arrivalpos") ? oc.getString("arrivalpos") : "random";
+    SumoXMLAttr fromAttr = oc.getBool("junctions") ? SUMO_ATTR_FROMJUNCTION : SUMO_ATTR_FROM;
+    SumoXMLAttr toAttr = oc.getBool("junctions") ? SUMO_ATTR_TOJUNCTION : SUMO_ATTR_TO;
+
     // go through the time steps
     for (SUMOTime t = begin; t < end;) {
         if (stepLog && t - lastOut >= DELTA_T) {
@@ -249,9 +253,9 @@ ODMatrix::write(SUMOTime begin, const SUMOTime end,
                 fractionLeft[odID] = 0;
             }
             // get the new departures (into tmp)
-            const size_t oldSize = vehicles.size();
-            const SUMOReal fraction = computeDeparts(*next, vehName, vehicles, uniform, differSourceSink, prefix);
-            if (oldSize != vehicles.size()) {
+            const int oldSize = (int)vehicles.size();
+            const double fraction = computeDeparts(*next, vehName, vehicles, uniform, differSourceSink, prefix);
+            if (oldSize != (int)vehicles.size()) {
                 changed = true;
             }
             if (fraction != 0) {
@@ -262,13 +266,40 @@ ODMatrix::write(SUMOTime begin, const SUMOTime end,
         if (changed) {
             sort(vehicles.begin(), vehicles.end(), descending_departure_comperator());
         }
+
         for (std::vector<ODVehicle>::reverse_iterator i = vehicles.rbegin(); i != vehicles.rend() && (*i).depart == t; ++i) {
             if (t >= begin) {
                 myNumWritten++;
-                dev.openTag(SUMO_TAG_TRIP).writeAttr(SUMO_ATTR_ID, (*i).id).writeAttr(SUMO_ATTR_DEPART, time2string(t));
-                dev.writeAttr(SUMO_ATTR_FROM, (*i).from).writeAttr(SUMO_ATTR_TO, (*i).to);
-                writeDefaultAttrs(dev, noVtype, i->cell);
-                dev.closeTag();
+                if (pedestrians) {
+                    dev.openTag(SUMO_TAG_PERSON).writeAttr(SUMO_ATTR_ID, (*i).id).writeAttr(SUMO_ATTR_DEPART, time2string(t));
+                    dev.writeAttr(SUMO_ATTR_DEPARTPOS, personDepartPos);
+                    dev.openTag(SUMO_TAG_WALK);
+                    dev.writeAttr(fromAttr, (*i).from);
+                    dev.writeAttr(toAttr, (*i).to);
+                    dev.writeAttr(SUMO_ATTR_FROM_TAZ, (*i).cell->origin).writeAttr(SUMO_ATTR_TO_TAZ, (*i).cell->destination);
+                    dev.writeAttr(SUMO_ATTR_ARRIVALPOS, personArrivalPos);
+                    dev.closeTag();
+                    dev.closeTag();
+                } else if (persontrips) {
+                    dev.openTag(SUMO_TAG_PERSON).writeAttr(SUMO_ATTR_ID, (*i).id).writeAttr(SUMO_ATTR_DEPART, time2string(t));
+                    dev.writeAttr(SUMO_ATTR_DEPARTPOS, personDepartPos);
+                    dev.openTag(SUMO_TAG_PERSONTRIP);
+                    dev.writeAttr(fromAttr, (*i).from);
+                    dev.writeAttr(toAttr, (*i).to);
+                    dev.writeAttr(SUMO_ATTR_FROM_TAZ, (*i).cell->origin).writeAttr(SUMO_ATTR_TO_TAZ, (*i).cell->destination);
+                    dev.writeAttr(SUMO_ATTR_ARRIVALPOS, personArrivalPos);
+                    if (modes != "") {
+                        dev.writeAttr(SUMO_ATTR_MODES, modes);
+                    }
+                    dev.closeTag();
+                    dev.closeTag();
+                } else {
+                    dev.openTag(SUMO_TAG_TRIP).writeAttr(SUMO_ATTR_ID, (*i).id).writeAttr(SUMO_ATTR_DEPART, time2string(t));
+                    dev.writeAttr(fromAttr, (*i).from);
+                    dev.writeAttr(toAttr, (*i).to);
+                    writeDefaultAttrs(dev, noVtype, i->cell);
+                    dev.closeTag();
+                }
             }
         }
         while (vehicles.size() != 0 && vehicles.back().depart == t) {
@@ -290,21 +321,87 @@ ODMatrix::write(SUMOTime begin, const SUMOTime end,
 void
 ODMatrix::writeFlows(const SUMOTime begin, const SUMOTime end,
                      OutputDevice& dev, bool noVtype,
-                     const std::string& prefix) {
+                     const std::string& prefix,
+                     bool asProbability, bool pedestrians, bool persontrips,
+                     const std::string& modes) {
     if (myContainer.size() == 0) {
         return;
     }
-    size_t flowName = 0;
+    int flowName = 0;
     sortByBeginTime();
     // recheck begin time
     for (std::vector<ODCell*>::const_iterator i = myContainer.begin(); i != myContainer.end(); ++i) {
         const ODCell* const c = *i;
         if (c->end > begin && c->begin < end) {
-            dev.openTag(SUMO_TAG_FLOW).writeAttr(SUMO_ATTR_ID, prefix + toString(flowName++));
-            dev.writeAttr(SUMO_ATTR_BEGIN, time2string(c->begin));
-            dev.writeAttr(SUMO_ATTR_END, time2string(c->end)).writeAttr(SUMO_ATTR_NUMBER, int(c->vehicleNumber));
-            writeDefaultAttrs(dev, noVtype, *i);
-            dev.closeTag();
+            const double probability = asProbability ? float(c->vehicleNumber) / STEPS2TIME(c->end - c->begin) : 1;
+            if (probability <= 0) {
+                continue;
+            }
+            //Person flows
+            if (pedestrians) {
+                dev.openTag(SUMO_TAG_PERSONFLOW).writeAttr(SUMO_ATTR_ID, prefix + toString(flowName++));
+                dev.writeAttr(SUMO_ATTR_BEGIN, time2string(c->begin)).writeAttr(SUMO_ATTR_END, time2string(c->end));
+                if (!asProbability) {
+                    dev.writeAttr(SUMO_ATTR_NUMBER, int(c->vehicleNumber));
+                } else {
+                    if (probability > 1) {
+                        WRITE_WARNING("Flow density of " + toString(probability) + " vehicles per second, cannot be represented with a simple probability. Falling back to even spacing.");
+                        dev.writeAttr(SUMO_ATTR_NUMBER, int(c->vehicleNumber));
+                    } else {
+                        dev.setPrecision(6);
+                        dev.writeAttr(SUMO_ATTR_PROB, probability);
+                        dev.setPrecision();
+                    }
+                }
+                dev.openTag(SUMO_TAG_WALK);
+                dev.writeAttr(SUMO_ATTR_FROM_TAZ, c->origin).writeAttr(SUMO_ATTR_TO_TAZ, c->destination);
+                dev.writeAttr(SUMO_ATTR_ARRIVALPOS, "random");
+                dev.closeTag();
+                dev.closeTag();
+            } else if (persontrips) {
+                dev.openTag(SUMO_TAG_PERSONFLOW).writeAttr(SUMO_ATTR_ID, prefix + toString(flowName++));
+                dev.writeAttr(SUMO_ATTR_BEGIN, time2string(c->begin)).writeAttr(SUMO_ATTR_END, time2string(c->end));
+                if (!asProbability) {
+                    dev.writeAttr(SUMO_ATTR_NUMBER, int(c->vehicleNumber));
+                } else {
+                    if (probability > 1) {
+                        WRITE_WARNING("Flow density of " + toString(probability) + " vehicles per second, cannot be represented with a simple probability. Falling back to even spacing.");
+                        dev.writeAttr(SUMO_ATTR_NUMBER, int(c->vehicleNumber));
+                    } else {
+                        dev.setPrecision(6);
+                        dev.writeAttr(SUMO_ATTR_PROB, probability);
+                        dev.setPrecision();
+                    }
+                }
+                dev.openTag(SUMO_TAG_PERSONTRIP);
+                dev.writeAttr(SUMO_ATTR_FROM_TAZ, c->origin).writeAttr(SUMO_ATTR_TO_TAZ, c->destination);
+                dev.writeAttr(SUMO_ATTR_ARRIVALPOS, "random");
+                if (modes != "") {
+                    dev.writeAttr(SUMO_ATTR_MODES, modes);
+                }
+                dev.closeTag();
+                dev.closeTag();
+            } else {
+                // Normal flow output
+                dev.openTag(SUMO_TAG_FLOW).writeAttr(SUMO_ATTR_ID, prefix + toString(flowName++));
+                dev.writeAttr(SUMO_ATTR_BEGIN, time2string(c->begin));
+                dev.writeAttr(SUMO_ATTR_END, time2string(c->end));
+
+                if (!asProbability) {
+                    dev.writeAttr(SUMO_ATTR_NUMBER, int(c->vehicleNumber));
+                } else {
+                    if (probability > 1) {
+                        WRITE_WARNING("Flow density of " + toString(probability) + " vehicles per second, cannot be represented with a simple probability. Falling back to even spacing.");
+                        dev.writeAttr(SUMO_ATTR_NUMBER, int(c->vehicleNumber));
+                    } else {
+                        dev.setPrecision(6);
+                        dev.writeAttr(SUMO_ATTR_PROB, probability);
+                        dev.setPrecision();
+                    }
+                }
+                writeDefaultAttrs(dev, noVtype, *i);
+                dev.closeTag();
+            }
         }
     }
 }
@@ -312,14 +409,13 @@ ODMatrix::writeFlows(const SUMOTime begin, const SUMOTime end,
 
 std::string
 ODMatrix::getNextNonCommentLine(LineReader& lr) {
-    std::string line;
-    do {
-        line = lr.readLine();
+    while (lr.good() && lr.hasMore()) {
+        const std::string line = lr.readLine();
         if (line[0] != '*') {
             return StringUtils::prune(line);
         }
-    } while (lr.good() && lr.hasMore());
-    throw ProcessError();
+    }
+    throw ProcessError("End of file while reading " + lr.getFileName() + ".");
 }
 
 
@@ -330,7 +426,7 @@ ODMatrix::parseSingleTime(const std::string& time) {
     }
     std::string hours = time.substr(0, time.find('.'));
     std::string minutes = time.substr(time.find('.') + 1);
-    return TIME2STEPS(TplConvert::_2int(hours.c_str()) * 3600 + TplConvert::_2int(minutes.c_str()) * 60);
+    return TIME2STEPS(StringUtils::toInt(hours) * 3600 + StringUtils::toInt(minutes) * 60);
 }
 
 
@@ -339,12 +435,12 @@ ODMatrix::readTime(LineReader& lr) {
     std::string line = getNextNonCommentLine(lr);
     try {
         StringTokenizer st(line, StringTokenizer::WHITECHARS);
-        SUMOTime begin = parseSingleTime(st.next());
-        SUMOTime end = parseSingleTime(st.next());
-        if (begin >= end) {
-            throw ProcessError("Begin time is larger than end time.");
+        myBegin = parseSingleTime(st.next());
+        myEnd = parseSingleTime(st.next());
+        if (myBegin >= myEnd) {
+            throw ProcessError("Matrix begin time " + time2string(myBegin) + " is larger than end time " + time2string(myEnd) + ".");
         }
-        return std::make_pair(begin, end);
+        return std::make_pair(myBegin, myEnd);
     } catch (OutOfBoundsException&) {
         throw ProcessError("Broken period definition '" + line + "'.");
     } catch (NumberFormatException&) {
@@ -352,12 +448,13 @@ ODMatrix::readTime(LineReader& lr) {
     }
 }
 
-SUMOReal
-ODMatrix::readFactor(LineReader& lr, SUMOReal scale) {
+
+double
+ODMatrix::readFactor(LineReader& lr, double scale) {
     std::string line = getNextNonCommentLine(lr);
-    SUMOReal factor = -1;
+    double factor = -1;
     try {
-        factor = TplConvert::_2SUMOReal(line.c_str()) * scale;
+        factor = StringUtils::toDouble(line) * scale;
     } catch (NumberFormatException&) {
         throw ProcessError("Broken factor: '" + line + "'.");
     }
@@ -365,7 +462,7 @@ ODMatrix::readFactor(LineReader& lr, SUMOReal scale) {
 }
 
 void
-ODMatrix::readV(LineReader& lr, SUMOReal scale,
+ODMatrix::readV(LineReader& lr, double scale,
                 std::string vehType, bool matrixHasVehType) {
     PROGRESS_BEGIN_MESSAGE("Reading matrix '" + lr.getFileName() + "' stored as VMR");
     // parse first defs
@@ -383,27 +480,33 @@ ODMatrix::readV(LineReader& lr, SUMOReal scale,
     SUMOTime end = times.second;
 
     // factor
-    SUMOReal factor = readFactor(lr, scale);
+    double factor = readFactor(lr, scale);
 
     // districts
     line = getNextNonCommentLine(lr);
-    int districtNo = TplConvert::_2int(StringUtils::prune(line).c_str());
+    const int numDistricts = StringUtils::toInt(StringUtils::prune(line));
     // parse district names (normally ints)
     std::vector<std::string> names;
-    do {
+    while ((int)names.size() != numDistricts && lr.hasMore()) {
         line = getNextNonCommentLine(lr);
         StringTokenizer st2(line, StringTokenizer::WHITECHARS);
         while (st2.hasNext()) {
             names.push_back(st2.next());
         }
-    } while ((int) names.size() != districtNo);
+    }
+    if (!lr.hasMore()) {
+        throw ProcessError("Missing line with " + toString(numDistricts) + " district names.");
+    }
 
     // parse the cells
     for (std::vector<std::string>::iterator si = names.begin(); si != names.end(); ++si) {
         std::vector<std::string>::iterator di = names.begin();
-        //
         do {
-            line = getNextNonCommentLine(lr);
+            try {
+                line = getNextNonCommentLine(lr);
+            } catch (ProcessError&) {
+                throw ProcessError("Missing line for district " + (*si) + ".");
+            }
             if (line.length() == 0) {
                 continue;
             }
@@ -411,7 +514,7 @@ ODMatrix::readV(LineReader& lr, SUMOReal scale,
                 StringTokenizer st2(line, StringTokenizer::WHITECHARS);
                 while (st2.hasNext()) {
                     assert(di != names.end());
-                    SUMOReal vehNumber = TplConvert::_2SUMOReal(st2.next().c_str()) * factor;
+                    double vehNumber = StringUtils::toDouble(st2.next()) * factor;
                     if (vehNumber != 0) {
                         add(vehNumber, begin, end, *si, *di, vehType);
                     }
@@ -433,14 +536,14 @@ ODMatrix::readV(LineReader& lr, SUMOReal scale,
 
 
 void
-ODMatrix::readO(LineReader& lr, SUMOReal scale,
+ODMatrix::readO(LineReader& lr, double scale,
                 std::string vehType, bool matrixHasVehType) {
     PROGRESS_BEGIN_MESSAGE("Reading matrix '" + lr.getFileName() + "' stored as OR");
     // parse first defs
     std::string line;
     if (matrixHasVehType) {
         line = getNextNonCommentLine(lr);
-        int type = TplConvert::_2int(StringUtils::prune(line).c_str());
+        int type = StringUtils::toInt(StringUtils::prune(line));
         if (vehType == "") {
             vehType = toString(type);
         }
@@ -452,7 +555,7 @@ ODMatrix::readO(LineReader& lr, SUMOReal scale,
     SUMOTime end = times.second;
 
     // factor
-    SUMOReal factor = readFactor(lr, scale);
+    double factor = readFactor(lr, scale);
 
     // parse the cells
     while (lr.hasMore()) {
@@ -467,7 +570,7 @@ ODMatrix::readO(LineReader& lr, SUMOReal scale,
         try {
             std::string sourceD = st2.next();
             std::string destD = st2.next();
-            SUMOReal vehNumber = TplConvert::_2SUMOReal(st2.next().c_str()) * factor;
+            double vehNumber = StringUtils::toDouble(st2.next()) * factor;
             if (vehNumber != 0) {
                 add(vehNumber, begin, end, sourceD, destD, vehType);
             }
@@ -482,19 +585,19 @@ ODMatrix::readO(LineReader& lr, SUMOReal scale,
 
 
 
-SUMOReal
+double
 ODMatrix::getNumLoaded() const {
     return myNumLoaded;
 }
 
 
-SUMOReal
+double
 ODMatrix::getNumWritten() const {
     return myNumWritten;
 }
 
 
-SUMOReal
+double
 ODMatrix::getNumDiscarded() const {
     return myNumDiscarded;
 }
@@ -502,14 +605,15 @@ ODMatrix::getNumDiscarded() const {
 
 void
 ODMatrix::applyCurve(const Distribution_Points& ps, ODCell* cell, std::vector<ODCell*>& newCells) {
-    for (size_t i = 0; i < ps.getAreaNo(); ++i) {
+    const std::vector<double>& times = ps.getVals();
+    for (int i = 0; i < (int)times.size() - 1; ++i) {
         ODCell* ncell = new ODCell();
-        ncell->begin = TIME2STEPS(ps.getAreaBegin(i));
-        ncell->end = TIME2STEPS(ps.getAreaEnd(i));
+        ncell->begin = TIME2STEPS(times[i]);
+        ncell->end = TIME2STEPS(times[i + 1]);
         ncell->origin = cell->origin;
         ncell->destination = cell->destination;
         ncell->vehicleType = cell->vehicleType;
-        ncell->vehicleNumber = cell->vehicleNumber * ps.getAreaPerc(i);
+        ncell->vehicleNumber = cell->vehicleNumber * ps.getProbs()[i] / ps.getOverallProb();
         newCells.push_back(ncell);
     }
 }
@@ -593,31 +697,26 @@ ODMatrix::loadRoutes(OptionsCont& oc, SUMOSAXHandler& handler) {
 
 Distribution_Points
 ODMatrix::parseTimeLine(const std::vector<std::string>& def, bool timelineDayInHours) {
-    bool interpolating = !timelineDayInHours;
-    PositionVector points;
-    SUMOReal prob = 0;
+    Distribution_Points result("N/A");
     if (timelineDayInHours) {
         if (def.size() != 24) {
             throw ProcessError("Assuming 24 entries for a day timeline, but got " + toString(def.size()) + ".");
         }
         for (int chour = 0; chour < 24; ++chour) {
-            prob = TplConvert::_2SUMOReal(def[chour].c_str());
-            points.push_back(Position((SUMOReal)(chour * 3600), prob));
+            result.add(chour * 3600., StringUtils::toDouble(def[chour]));
         }
-        points.push_back(Position((SUMOReal)(24 * 3600), prob));
+        result.add(24 * 3600., 0.); // dummy value to finish the last interval
     } else {
-        size_t i = 0;
-        while (i < def.size()) {
-            StringTokenizer st2(def[i++], ":");
+        for (int i = 0; i < (int)def.size(); i++) {
+            StringTokenizer st2(def[i], ":");
             if (st2.size() != 2) {
-                throw ProcessError("Broken time line definition: missing a value in '" + def[i - 1] + "'.");
+                throw ProcessError("Broken time line definition: missing a value in '" + def[i] + "'.");
             }
-            int time = TplConvert::_2int(st2.next().c_str());
-            prob = TplConvert::_2SUMOReal(st2.next().c_str());
-            points.push_back(Position((SUMOReal) time, prob));
+            const double time = StringUtils::toDouble(st2.next());
+            result.add(time, StringUtils::toDouble(st2.next()));
         }
     }
-    return Distribution_Points("N/A", points, interpolating);
+    return result;
 }
 
 
