@@ -1,101 +1,67 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIOSGBuilder.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    19.01.2012
-/// @version $Id: GUIOSGBuilder.cpp 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // Builds OSG nodes from microsim objects
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #ifdef HAVE_OSG
 
-#ifdef _MSC_VER
-#pragma warning(push)
-#pragma warning(disable: 4127) // do not warn about constant conditional expression
-#endif
-#include <osg/Version>
-#include <osgViewer/ViewerEventHandlers>
-#include <osgGA/TrackballManipulator>
-#include <osgDB/ReadFile>
-#include <osgDB/WriteFile>
-#include <osg/ShapeDrawable>
-#include <osg/Node>
-#include <osg/Group>
-#include <osg/Geode>
-#include <osg/Geometry>
-#include <osg/Sequence>
-#include <osg/Texture2D>
-#include <osgViewer/Viewer>
-#include <osgUtil/Tessellator>
-#include <osg/PositionAttitudeTransform>
-#include <osg/ShadeModel>
-#include <osg/Light>
-#include <osg/LightSource>
-#ifdef _MSC_VER
-#pragma warning(pop)
-#endif
-
-#include <microsim/MSNet.h>
+#include <guisim/GUIEdge.h>
+#include <guisim/GUIJunctionWrapper.h>
+#include <guisim/GUILane.h>
+#include <guisim/GUINet.h>
 #include <microsim/MSEdge.h>
-#include <microsim/MSLane.h>
 #include <microsim/MSEdgeControl.h>
-#include <microsim/MSJunctionControl.h>
 #include <microsim/MSJunction.h>
+#include <microsim/MSJunctionControl.h>
+#include <microsim/MSLane.h>
+#include <microsim/MSNet.h>
 #include <microsim/MSVehicleType.h>
 #include <microsim/traffic_lights/MSTLLogicControl.h>
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
-#include <guisim/GUIJunctionWrapper.h>
-#include <guisim/GUINet.h>
-#include <guisim/GUIEdge.h>
-#include <guisim/GUILane.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/gui/windows/GUISUMOAbstractView.h>
-#include "GUIOSGBoundingBoxCalculator.h"
+
 #include "GUIOSGView.h"
 #include "GUIOSGBuilder.h"
 
+//#define DEBUG_TESSEL
 
 // ===========================================================================
 // static member variables
 // ===========================================================================
-std::map<std::string, osg::ref_ptr<osg::Node> > GUIOSGBuilder::myCars;
 
+std::map<std::string, osg::ref_ptr<osg::Node> > GUIOSGBuilder::myCars;
 
 // ===========================================================================
 // member method definitions
 // ===========================================================================
+
 osg::Group*
 GUIOSGBuilder::buildOSGScene(osg::Node* const tlg, osg::Node* const tly, osg::Node* const tlr, osg::Node* const tlu) {
     osgUtil::Tessellator tesselator;
     osg::Group* root = new osg::Group();
     GUINet* net = static_cast<GUINet*>(MSNet::getInstance());
     // build edges
-    const MSEdgeVector& edges = net->getEdgeControl().getEdges();
-    for (MSEdgeVector::const_iterator i = edges.begin(); i != edges.end(); ++i) {
-        if ((*i)->getPurpose() != MSEdge::EDGEFUNCTION_INTERNAL) {
-            buildOSGEdgeGeometry(**i, *root, tesselator);
+    for (const MSEdge* e : net->getEdgeControl().getEdges()) {
+        if (!e->isInternal()) {
+            buildOSGEdgeGeometry(*e, *root, tesselator);
         }
     }
     // build junctions
@@ -111,6 +77,9 @@ GUIOSGBuilder::buildOSGScene(osg::Node* const tlg, osg::Node* const tly, osg::No
         const MSLane* lastLane = 0;
         int idx = 0;
         for (MSTrafficLightLogic::LaneVectorVector::const_iterator j = lanes.begin(); j != lanes.end(); ++j, ++idx) {
+            if ((*j).size() == 0) {
+                continue;
+            }
             const MSLane* const lane = (*j)[0];
             const Position pos = lane->getShape().back();
             const double angle =  osg::DegreesToRadians(lane->getShape().rotationDegreeAtOffset(-1.) + 90.);
@@ -172,18 +141,27 @@ GUIOSGBuilder::buildOSGEdgeGeometry(const MSEdge& edge,
         osg::Geometry* geom = new osg::Geometry();
         geode->addDrawable(geom);
         addTo.addChild(geode);
-        osg::Vec3dArray* osg_coords = new osg::Vec3dArray((int)shape.size() * 2);
+        const int shapeSize = (int)(edge.isWalkingArea() ? shape.size() : shape.size() * 2);
+        const float zOffset = edge.isWalkingArea() || edge.isCrossing() ? 0.01f : 0.f;
+        osg::Vec3Array* osg_coords = new osg::Vec3Array(shapeSize);
         geom->setVertexArray(osg_coords);
-        PositionVector rshape = shape;
-        rshape.move2side(SUMO_const_halfLaneWidth);
-        int index = 0;
-        for (int k = 0; k < (int)rshape.size(); ++k, ++index) {
-            (*osg_coords)[index].set(rshape[k].x(), rshape[k].y(), rshape[k].z());
-        }
-        PositionVector lshape = shape;
-        lshape.move2side(-SUMO_const_halfLaneWidth);
-        for (int k = (int) lshape.size() - 1; k >= 0; --k, ++index) {
-            (*osg_coords)[index].set(lshape[k].x(), lshape[k].y(), lshape[k].z());
+        if (edge.isWalkingArea()) {
+            int index = 0;
+            for (int k = 0; k < (int)shape.size(); ++k, ++index) {
+                (*osg_coords)[index].set((float)shape[k].x(), (float)shape[k].y(), (float)shape[k].z() + zOffset);
+            }
+        } else {
+            PositionVector rshape = shape;
+            rshape.move2side(l->getWidth() / 2);
+            int index = 0;
+            for (int k = 0; k < (int)rshape.size(); ++k, ++index) {
+                (*osg_coords)[index].set((float)rshape[k].x(), (float)rshape[k].y(), (float)rshape[k].z() + zOffset);
+            }
+            PositionVector lshape = shape;
+            lshape.move2side(-l->getWidth() / 2);
+            for (int k = (int) lshape.size() - 1; k >= 0; --k, ++index) {
+                (*osg_coords)[index].set((float)lshape[k].x(), (float)lshape[k].y(), (float)lshape[k].z() + zOffset);
+            }
         }
         osg::Vec3Array* osg_normals = new osg::Vec3Array(1);
         (*osg_normals)[0] = osg::Vec3(0, 0, 1);
@@ -201,7 +179,7 @@ GUIOSGBuilder::buildOSGEdgeGeometry(const MSEdge& edge,
         geom->setColorArray(osg_colors);
         geom->setColorBinding(osg::Geometry::BIND_OVERALL);
 #endif
-        geom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::POLYGON, 0, (int)shape.size() * 2));
+        geom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::POLYGON, 0, shapeSize));
 
         osg::ref_ptr<osg::StateSet> ss = geode->getOrCreateStateSet();
         ss->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
@@ -209,6 +187,14 @@ GUIOSGBuilder::buildOSGEdgeGeometry(const MSEdge& edge,
 
         if (shape.size() > 2) {
             tessellator.retessellatePolygons(*geom);
+#ifdef DEBUG_TESSEL
+            std::cout << "l=" << l->getID() << " origPoints=" << shape.size() << " geomSize=" << geom->getVertexArray()->getNumElements() << " points=";
+            for (int i = 0; i < (int)geom->getVertexArray()->getNumElements(); i++) {
+                const osg::Vec3& p = (*((osg::Vec3Array*)geom->getVertexArray()))[i];
+                std::cout << p.x() << "," << p.y() << "," << p.z() << " ";
+            }
+            std::cout << "\n";
+#endif
         }
         static_cast<GUILane*>(l)->setGeometry(geom);
     }
@@ -224,12 +210,12 @@ GUIOSGBuilder::buildOSGJunctionGeometry(GUIJunctionWrapper& junction,
     osg::Geometry* geom = new osg::Geometry();
     geode->addDrawable(geom);
     addTo.addChild(geode);
-    osg::Vec3dArray* osg_coords = new osg::Vec3dArray((int)shape.size());
+    osg::Vec3Array* osg_coords = new osg::Vec3Array((int)shape.size());
     geom->setVertexArray(osg_coords);
     for (int k = 0; k < (int)shape.size(); ++k) {
-        (*osg_coords)[k].set(shape[k].x(), shape[k].y(), shape[k].z());
+        (*osg_coords)[k].set((float)shape[k].x(), (float)shape[k].y(), (float)shape[k].z());
     }
-    osg::Vec3dArray* osg_normals = new osg::Vec3dArray(1);
+    osg::Vec3Array* osg_normals = new osg::Vec3Array(1);
     (*osg_normals)[0] = osg::Vec3(0, 0, 1);
 #if OSG_MIN_VERSION_REQUIRED(3,2,0)
     geom->setNormalArray(osg_normals, osg::Array::BIND_PER_PRIMITIVE_SET);
@@ -261,7 +247,7 @@ GUIOSGBuilder::buildOSGJunctionGeometry(GUIJunctionWrapper& junction,
 void
 GUIOSGBuilder::buildDecal(const GUISUMOAbstractView::Decal& d, osg::Group& addTo) {
     osg::Node* pLoadedModel = osgDB::readNodeFile(d.filename);
-    if (pLoadedModel == 0) {
+    if (pLoadedModel == nullptr) {
         WRITE_ERROR("Could not load '" + d.filename + "'.");
         return;
     }
@@ -270,7 +256,7 @@ GUIOSGBuilder::buildDecal(const GUISUMOAbstractView::Decal& d, osg::Group& addTo
     pLoadedModel->getOrCreateStateSet()->setAttribute(sm);
     osg::PositionAttitudeTransform* base = new osg::PositionAttitudeTransform();
     base->addChild(pLoadedModel);
-    GUIOSGBoundingBoxCalculator bboxCalc;
+    osg::ComputeBoundsVisitor bboxCalc;
     pLoadedModel->accept(bboxCalc);
     const osg::BoundingBox& bbox = bboxCalc.getBoundingBox();
     WRITE_MESSAGE("Loaded decal '" + d.filename + "' with bounding box " + toString(Position(bbox.xMin(), bbox.yMin(), bbox.zMin())) + " " + toString(Position(bbox.xMax(), bbox.yMax(), bbox.zMax())) + ".");
@@ -292,10 +278,10 @@ GUIOSGBuilder::buildDecal(const GUISUMOAbstractView::Decal& d, osg::Group& addTo
 osg::PositionAttitudeTransform*
 GUIOSGBuilder::getTrafficLight(const GUISUMOAbstractView::Decal& d, osg::Node* tl, const osg::Vec4& color, const double size) {
     osg::PositionAttitudeTransform* ret = new osg::PositionAttitudeTransform();
-    if (tl != 0) {
+    if (tl != nullptr) {
         osg::PositionAttitudeTransform* base = new osg::PositionAttitudeTransform();
         base->addChild(tl);
-        GUIOSGBoundingBoxCalculator bboxCalc;
+        osg::ComputeBoundsVisitor bboxCalc;
         tl->accept(bboxCalc);
         const osg::BoundingBox& bbox = bboxCalc.getBoundingBox();
         double xScale = d.width > 0 ? d.width / (bbox.xMax() - bbox.xMin()) : 1.;
@@ -350,8 +336,8 @@ GUIOSGBuilder::buildMovable(const MSVehicleType& type) {
         }
     }
     osg::Node* carNode = myCars[osgFile];
-    if (carNode != 0) {
-        GUIOSGBoundingBoxCalculator bboxCalc;
+    if (carNode != nullptr) {
+        osg::ComputeBoundsVisitor bboxCalc;
         carNode->accept(bboxCalc);
         const osg::BoundingBox& bbox = bboxCalc.getBoundingBox();
         osg::PositionAttitudeTransform* base = new osg::PositionAttitudeTransform();
@@ -405,11 +391,10 @@ GUIOSGBuilder::buildMovable(const MSVehicleType& type) {
     ellipse->setPosition(center);
     ellipse->setScale(osg::Vec3d(type.getWidth() + enlarge, type.getLength() + enlarge, type.getHeight() + enlarge));
     m.pos->addChild(ellipse);
+    m.active = true;
     return m;
 }
 
 #endif
 
-
 /****************************************************************************/
-
