@@ -1,47 +1,35 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIOSGView.h
 /// @author  Daniel Krajzewicz
 /// @date    19.01.2012
-/// @version $Id: GUIOSGView.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // An OSG-based 3D view on the simulation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIOSGView_h
-#define GUIOSGView_h
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #ifdef HAVE_OSG
 
+#include "GUIOSGHeader.h"
+
 #include <string>
-#include <osgGA/TerrainManipulator>
-#include <osgViewer/Viewer>
-#include <osg/PositionAttitudeTransform>
-#include <osg/ShapeDrawable>
 #include <microsim/traffic_lights/MSTLLogicControl.h>
 #include <utils/geom/Boundary.h>
 #include <utils/geom/Position.h>
 #include <utils/common/RGBColor.h>
 #include <utils/geom/PositionVector.h>
-#include <utils/shapes/Polygon.h>
 #include <gui/GUISUMOViewParent.h>
 #include <utils/gui/windows/GUISUMOAbstractView.h>
 
@@ -49,17 +37,20 @@
 // ===========================================================================
 // class declarations
 // ===========================================================================
+
 class GUINet;
 class GUISUMOViewParent;
 class GUIVehicle;
 class GUILaneWrapper;
 class MSRoute;
+class MSTransportable;
+class MSVehicle;
+
 namespace osgGA {
 class CameraManipulator;
 class NodeTrackerManipulator;
 }
 
-
 // ===========================================================================
 // class definitions
 // ===========================================================================
@@ -86,11 +77,9 @@ public:
          */
         Command_TLSChange(const MSLink* const link, osg::Switch* switchNode);
 
-
         /// @brief Destructor
         virtual ~Command_TLSChange();
 
-
         /** @brief Executes the command
          *
          * Called when an active tls program switches.
@@ -99,7 +88,6 @@ public:
          */
         void execute();
 
-
     private:
         /// @brief The link to observe
         const MSLink* const myLink;
@@ -110,35 +98,34 @@ public:
         /// @brief The previous link state
         LinkState myLastState;
 
-
     private:
         /// @brief Invalidated copy constructor.
-        Command_TLSChange(const Command_TLSChange&);
+        Command_TLSChange(const Command_TLSChange&) = delete;
 
         /// @brief Invalidated assignment operator.
-        Command_TLSChange& operator=(const Command_TLSChange&);
-
+        Command_TLSChange& operator=(const Command_TLSChange&) = delete;
     };
 
-
+    /// @brief struct for OSG movable elements
     struct OSGMovable {
         osg::ref_ptr<osg::PositionAttitudeTransform> pos;
         osg::ref_ptr<osg::ShapeDrawable> geom;
         osg::ref_ptr<osg::Switch> lights;
+        bool active;
     };
 
-
-    /// constructor
+    /// @brief constructor
     GUIOSGView(FXComposite* p, GUIMainWindow& app,
                GUISUMOViewParent* parent, GUINet& net, FXGLVisual* glVis,
                FXGLCanvas* share);
 
+    /// @brief destructor
     virtual ~GUIOSGView();
 
-    /// builds the view toolbars
-    virtual void buildViewToolBars(GUIGlChildWindow&);
+    /// @brief builds the view toolbars
+    virtual void buildViewToolBars(GUIGlChildWindow*);
 
-    /// recenters the view
+    /// @brief recenters the view
     void recenterView();
 
     /** @brief centers to the chosen artifact
@@ -149,12 +136,13 @@ public:
      */
     void centerTo(GUIGlID id, bool applyZoom, double zoomDist = 20);
 
+    /// @brief show viewport editor
     void showViewportEditor();
 
-    /// applies the given viewport settings
-    void setViewportFromTo(const Position& lookFrom, const Position& lookAt);
+    /// @brief applies the given viewport settings
+    void setViewportFromToRot(const Position& lookFrom, const Position& lookAt, double rotation);
 
-    ///@brief copy the viewport to the given view
+    /// @brief copy the viewport to the given view
     void copyViewportTo(GUISUMOAbstractView* view);
 
     /** @brief Starts vehicle tracking
@@ -162,12 +150,10 @@ public:
      */
     void startTrack(int id);
 
-
     /** @brief Stops vehicle tracking
      */
     void stopTrack();
 
-
     /** @brief Returns the id of the tracked vehicle (-1 if none)
      * @return The glID of the vehicle to track
      */
@@ -181,7 +167,8 @@ public:
     /// @brief get the current simulation time
     SUMOTime getCurrentTimeStep() const;
 
-    void remove(GUIVehicle* veh);
+    void removeVeh(MSVehicle* veh);
+    void removeTransportable(MSTransportable* t);
 
     // callback
     long onConfigure(FXObject*, FXSelector, void*);
@@ -203,6 +190,7 @@ private:
     public:
         SUMOTerrainManipulator() {
             setAllowThrow(false);
+            setRotationMode(ELEVATION_AZIM_ROLL); // default is ELEVATION_AZIM and this prevents rotating the view around the z-axis
         }
         bool performMovementLeftMouseButton(const double eventTimeDelta, const double dx, const double dy) {
             return osgGA::TerrainManipulator::performMovementMiddleMouseButton(eventTimeDelta, dx, dy);
@@ -249,6 +237,7 @@ private:
     };
 
 protected:
+    GUIOSGView() {}
 
     osg::ref_ptr<FXOSGAdapter> myAdapter;
     osg::ref_ptr<osgViewer::Viewer> myViewer;
@@ -267,16 +256,6 @@ private:
     osg::ref_ptr<osg::Node> myYellowLight;
     osg::ref_ptr<osg::Node> myRedLight;
     osg::ref_ptr<osg::Node> myRedYellowLight;
-
-protected:
-    GUIOSGView() { }
-
 };
 
 #endif
-
-#endif
-
-/****************************************************************************/
-
-
