@@ -1,35 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PCLoaderDlrNavteq.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Christoph Sommer
 /// @author  Michael Behrisch
 /// @date    Thu, 02.11.2006
-/// @version $Id: PCLoaderDlrNavteq.cpp 20801 2016-05-28 05:31:30Z behrisch $
 ///
 // A reader of pois and polygons stored in DLR-Navteq (Elmar)-format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -39,7 +30,7 @@
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/StringUtils.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include <utils/common/StringTokenizer.h>
 #include <utils/common/FileHelpers.h>
@@ -55,10 +46,6 @@
 #include <utils/geom/Position.h>
 #include <utils/geom/GeoConvHelper.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -155,7 +142,7 @@ PCLoaderDlrNavteq::loadPOIFile(const std::string& file,
 
         // patch the values
         bool discard = oc.getBool("discard");
-        SUMOReal layer = oc.getFloat("layer");
+        double layer = oc.getFloat("layer");
         RGBColor color;
         if (tm.has(type)) {
             const PCTypeMap::TypeDef& def = tm.get(type);
@@ -170,7 +157,7 @@ PCLoaderDlrNavteq::loadPOIFile(const std::string& file,
             color = c;
         }
         if (!discard) {
-            PointOfInterest* poi = new PointOfInterest(name, type, color, pos, layer);
+            PointOfInterest* poi = new PointOfInterest(name, type, color, pos, false, "", 0, 0, layer);
             toFill.add(poi, OptionsCont::getOptions().isInStringVector("prune.keep-list", name));
         }
     }
@@ -208,14 +195,14 @@ PCLoaderDlrNavteq::loadPolyFile(const std::string& file,
         std::string type = values[2];
         std::string name = values[3];
         PositionVector vec;
-        size_t index = 4;
+        int index = 4;
         // now collect the positions
-        while (values.size() > index) {
+        while ((int)values.size() > index) {
             std::string xpos = values[index];
             std::string ypos = values[index + 1];
             index += 2;
-            SUMOReal x = TplConvert::_2SUMOReal(xpos.c_str());
-            SUMOReal y = TplConvert::_2SUMOReal(ypos.c_str());
+            double x = StringUtils::toDouble(xpos);
+            double y = StringUtils::toDouble(ypos);
             Position pos(x, y);
             if (!GeoConvHelper::getProcessing().x2cartesian(pos)) {
                 WRITE_WARNING("Unable to project coordinates for polygon '" + id + "'.");
@@ -241,7 +228,7 @@ PCLoaderDlrNavteq::loadPolyFile(const std::string& file,
         // patch the values
         bool fill = vec.front() == vec.back();
         bool discard = oc.getBool("discard");
-        SUMOReal layer = oc.getFloat("layer");
+        double layer = oc.getFloat("layer");
         RGBColor color;
         if (tm.has(type)) {
             const PCTypeMap::TypeDef& def = tm.get(type);
@@ -257,7 +244,7 @@ PCLoaderDlrNavteq::loadPolyFile(const std::string& file,
             color = c;
         }
         if (!discard) {
-            SUMO::Polygon* poly = new SUMO::Polygon(name, type, color, vec, fill, layer);
+            SUMOPolygon* poly = new SUMOPolygon(name, type, color, vec, false, fill, 1, layer);
             toFill.add(poly);
         }
         vec.clear();
@@ -265,8 +252,4 @@ PCLoaderDlrNavteq::loadPolyFile(const std::string& file,
 }
 
 
-
-
-
 /****************************************************************************/
-
