@@ -1,35 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PCLoaderVisum.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Christoph Sommer
 /// @author  Michael Behrisch
 /// @date    Thu, 02.11.2006
-/// @version $Id: PCLoaderVisum.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A reader of pois and polygons stored in VISUM-format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -38,7 +29,7 @@
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/StringUtils.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include <utils/common/FileHelpers.h>
 #include <utils/options/OptionsCont.h>
@@ -54,9 +45,53 @@
 #include <utils/geom/GeoConvHelper.h>
 #include <utils/importio/NamedColumnsParser.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
+StringBijection<PCLoaderVisum::VISUM_KEY>::Entry PCLoaderVisum::KEYS_DE[] = {
+    // duplicates NIImporter_VISUM::KEYS_DE due to lack of suitable common location
+    { "VSYS", VISUM_SYS },
+    { "STRECKENTYP", VISUM_LINKTYPE },
+    { "KNOTEN", VISUM_NODE },
+    { "BEZIRK", VISUM_DISTRICT },
+    { "PUNKT", VISUM_POINT },
+    { "STRECKE", VISUM_LINK },
+    { "V0IV", VISUM_V0 },
+    { "VSYSSET", VISUM_TYPES },
+    { "RANG", VISUM_RANK },
+    { "KAPIV", VISUM_CAPACITY },
+    { "XKOORD", VISUM_XCOORD },
+    { "YKOORD", VISUM_YCOORD },
+    { "ID", VISUM_ID },
+    { "CODE", VISUM_CODE },
+    { "VONKNOTNR", VISUM_FROMNODE },
+    { "NACHKNOTNR", VISUM_TONODE },
+    { "TYPNR", VISUM_TYPE },
+    { "TYP", VISUM_TYP },
+    { "ANBINDUNG", VISUM_DISTRICT_CONNECTION },
+    { "BEZNR", VISUM_SOURCE_DISTRICT },
+    { "KNOTNR",  VISUM_FROMNODENO },
+    { "RICHTUNG",  VISUM_DIRECTION },
+    { "FLAECHEID",  VISUM_SURFACEID },
+    { "TFLAECHEID",  VISUM_FACEID },
+    { "VONPUNKTID",  VISUM_FROMPOINTID },
+    { "NACHPUNKTID",  VISUM_TOPOINTID },
+    { "KANTE",  VISUM_EDGE },
+    { "ABBIEGER",  VISUM_TURN },
+    { "UEBERKNOTNR",  VISUM_VIANODENO },
+    { "ANZFAHRSTREIFEN",  VISUM_NUMLANES },
+    { "INDEX",  VISUM_INDEX },
+    { "STRECKENPOLY",  VISUM_LINKPOLY },
+    { "FLAECHENELEMENT",  VISUM_SURFACEITEM },
+    { "TEILFLAECHENELEMENT",  VISUM_FACEITEM },
+    { "KANTEID",  VISUM_EDGEID },
+    { "Q",  VISUM_ORIGIN },
+    { "Z",  VISUM_DESTINATION },
+    { "KATNR", VISUM_CATID },
+    { "ZWISCHENPUNKT", VISUM_EDGEITEM },
+    { "POIKATEGORIE", VISUM_POICATEGORY },
+    { "NR", VISUM_NO } // must be the last one
+};
+
+
+StringBijection<PCLoaderVisum::VISUM_KEY> PCLoaderVisum::KEYS(PCLoaderVisum::KEYS_DE, VISUM_NO);
 
 
 // ===========================================================================
@@ -68,6 +103,10 @@ PCLoaderVisum::loadIfSet(OptionsCont& oc, PCPolyContainer& toFill,
     if (!oc.isSet("visum-files")) {
         return;
     }
+    const std::string languageFile = oc.getString("visum.language-file");
+    if (languageFile != "") {
+        loadLanguage(languageFile);
+    }
     // parse file(s)
     std::vector<std::string> files = oc.getStringVector("visum-files");
     for (std::vector<std::string>::const_iterator file = files.begin(); file != files.end(); ++file) {
@@ -100,46 +139,46 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             what = "";
         }
         // read items
-        if (what == "$PUNKT") {
+        if (what == "$" + KEYS.getString(VISUM_POINT)) {
             lineParser.parseLine(line);
-            long long int id = TplConvert::_2long(lineParser.get("ID").c_str());
-            SUMOReal x = TplConvert::_2SUMOReal(lineParser.get("XKOORD").c_str());
-            SUMOReal y = TplConvert::_2SUMOReal(lineParser.get("YKOORD").c_str());
+            long long int id = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_ID)));
+            double x = StringUtils::toDouble(lineParser.get(KEYS.getString(VISUM_XCOORD)));
+            double y = StringUtils::toDouble(lineParser.get(KEYS.getString(VISUM_YCOORD)));
             Position pos(x, y);
             if (!geoConvHelper.x2cartesian(pos)) {
                 WRITE_WARNING("Unable to project coordinates for point '" + toString(id) + "'.");
             }
             punkte[id] = pos;
             continue;
-        } else if (what == "$KANTE") {
+        } else if (what == "$" + KEYS.getString(VISUM_EDGE)) {
             lineParser.parseLine(line);
-            long long int id = TplConvert::_2long(lineParser.get("ID").c_str());
-            long long int fromID = TplConvert::_2long(lineParser.get("VONPUNKTID").c_str());
-            long long int toID = TplConvert::_2long(lineParser.get("NACHPUNKTID").c_str());
+            long long int id = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_ID)));
+            long long int fromID = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_FROMPOINTID)));
+            long long int toID = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_TOPOINTID)));
             PositionVector vec;
             vec.push_back(punkte[fromID]);
             vec.push_back(punkte[toID]);
             kanten[id] = vec;
             continue;
-        } else if (what == "$ZWISCHENPUNKT") {
+        } else if (what == "$" + KEYS.getString(VISUM_EDGEITEM)) {
             lineParser.parseLine(line);
-            long long int id = TplConvert::_2long(lineParser.get("KANTEID").c_str());
-            int index = TplConvert::_2int(lineParser.get("INDEX").c_str());
-            SUMOReal x = TplConvert::_2SUMOReal(lineParser.get("XKOORD").c_str());
-            SUMOReal y = TplConvert::_2SUMOReal(lineParser.get("YKOORD").c_str());
+            long long int id = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_EDGEID)));
+            int index = StringUtils::toInt(lineParser.get(KEYS.getString(VISUM_INDEX)));
+            double x = StringUtils::toDouble(lineParser.get(KEYS.getString(VISUM_XCOORD)));
+            double y = StringUtils::toDouble(lineParser.get(KEYS.getString(VISUM_YCOORD)));
             Position pos(x, y);
             if (!geoConvHelper.x2cartesian(pos)) {
                 WRITE_WARNING("Unable to project coordinates for edge '" + toString(id) + "'.");
             }
             kanten[id].insert(kanten[id].begin() + index, pos);
             continue;
-        } else if (what == "$TEILFLAECHENELEMENT") {
+        } else if (what == "$" + KEYS.getString(VISUM_FACEITEM)) {
             lineParser.parseLine(line);
-            long long int id = TplConvert::_2long(lineParser.get("TFLAECHEID").c_str());
-            //int index = TplConvert::_2int(lineParser.get("INDEX").c_str());
+            long long int id = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_FACEID)));
+            //int index = StringUtils::toInt(lineParser.get("INDEX"));
             //index = 0; /// hmmmm - assume it's sorted...
-            long long int kid = TplConvert::_2long(lineParser.get("KANTEID").c_str());
-            int dir = TplConvert::_2int(lineParser.get("RICHTUNG").c_str());
+            long long int kid = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_EDGEID)));
+            int dir = StringUtils::toInt(lineParser.get(KEYS.getString(VISUM_DIRECTION)));
             if (teilflaechen.find(id) == teilflaechen.end()) {
                 teilflaechen[id] = PositionVector();
             }
@@ -153,26 +192,26 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
                 }
             }
             continue;
-        } else if (what == "$FLAECHENELEMENT") {
+        } else if (what == "$" + KEYS.getString(VISUM_SURFACEITEM)) {
             lineParser.parseLine(line);
-            long long int id = TplConvert::_2long(lineParser.get("FLAECHEID").c_str());
-            long long int tid = TplConvert::_2long(lineParser.get("TFLAECHEID").c_str());
+            long long int id = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_SURFACEID)));
+            long long int tid = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_FACEID)));
             flaechenelemente[id] = tid;
             continue;
         }
         // set if read
         if (line[0] == '$') {
             what = "";
-            if (line.find("$PUNKT") == 0) {
-                what = "$PUNKT";
-            } else if (line.find("$KANTE") == 0) {
-                what = "$KANTE";
-            } else if (line.find("$ZWISCHENPUNKT") == 0) {
-                what = "$ZWISCHENPUNKT";
-            } else if (line.find("$TEILFLAECHENELEMENT") == 0) {
-                what = "$TEILFLAECHENELEMENT";
-            } else if (line.find("$FLAECHENELEMENT") == 0) {
-                what = "$FLAECHENELEMENT";
+            if (line.find("$" + KEYS.getString(VISUM_POINT) + ":") == 0) {
+                what = "$" + KEYS.getString(VISUM_POINT);
+            } else if (line.find("$" + KEYS.getString(VISUM_EDGE) + ":") == 0) {
+                what = "$" + KEYS.getString(VISUM_EDGE);
+            } else if (line.find("$" + KEYS.getString(VISUM_EDGEITEM) + ":") == 0) {
+                what = "$" + KEYS.getString(VISUM_EDGEITEM);
+            } else if (line.find("$" + KEYS.getString(VISUM_FACEITEM) + ":") == 0) {
+                what = "$" + KEYS.getString(VISUM_FACEITEM);
+            } else if (line.find("$" + KEYS.getString(VISUM_SURFACEITEM) + ":") == 0) {
+                what = "$" + KEYS.getString(VISUM_SURFACEITEM);
             }
             if (what != "") {
                 lineParser.reinit(line.substr(what.length() + 1));
@@ -221,12 +260,12 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             // parse the poi
             // $POI:Nr;CATID;CODE;NAME;Kommentar;XKoord;YKoord;
             lineParser.parseLine(line);
-            long long int idL = TplConvert::_2long(lineParser.get("Nr").c_str());
+            long long int idL = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_NO)));
             std::string id = toString(idL);
-            std::string catid = lineParser.get("CATID");
+            std::string catid = lineParser.get(KEYS.getString(VISUM_CATID));
             // process read values
-            SUMOReal x = TplConvert::_2SUMOReal(lineParser.get("XKoord").c_str());
-            SUMOReal y = TplConvert::_2SUMOReal(lineParser.get("YKoord").c_str());
+            double x = StringUtils::toDouble(lineParser.get(KEYS.getString(VISUM_XCOORD)));
+            double y = StringUtils::toDouble(lineParser.get(KEYS.getString(VISUM_YCOORD)));
             Position pos(x, y);
             if (!geoConvHelper.x2cartesian(pos)) {
                 WRITE_WARNING("Unable to project coordinates for POI '" + id + "'.");
@@ -234,7 +273,7 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             std::string type = typemap[catid];
             // patch the values
             bool discard = oc.getBool("discard");
-            int layer = oc.getInt("layer");
+            double layer = oc.getFloat("layer");
             RGBColor color;
             if (tm.has(type)) {
                 const PCTypeMap::TypeDef& def = tm.get(type);
@@ -245,12 +284,16 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
                 layer = def.layer;
             } else {
                 id = oc.getString("prefix") + id;
-                type = oc.getString("type");
                 color = c;
             }
             if (!discard) {
-                PointOfInterest* poi = new PointOfInterest(id, type, color, pos, (SUMOReal)layer);
-                toFill.insert(id, poi, layer);
+                const std::string origId = id;
+                int index = 1;
+                while (toFill.getPOIs().get(id) != nullptr) {
+                    id = origId + "#" + toString(index++);
+                }
+                PointOfInterest* poi = new PointOfInterest(id, type, color, pos, false, "", 0, 0, layer);
+                toFill.add(poi);
             }
         }
 
@@ -262,7 +305,7 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             if (!first && lastID != id) {
                 // we have parsed a polygon completely
                 RGBColor color;
-                int layer = oc.getInt("layer");
+                double layer = oc.getFloat("layer");
                 bool discard = oc.getBool("discard");
                 if (tm.has(polyType)) {
                     const PCTypeMap::TypeDef& def = tm.get(polyType);
@@ -277,8 +320,13 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
                     color = c;
                 }
                 if (!discard) {
-                    Polygon* poly = new Polygon(id, type, color, vec, false, (SUMOReal)layer);
-                    toFill.insert(id, poly, 1);
+                    const std::string origId = id;
+                    int index = 1;
+                    while (toFill.getPolygons().get(id) != nullptr) {
+                        id = origId + "#" + toString(index++);
+                    }
+                    SUMOPolygon* poly = new SUMOPolygon(id, type, color, vec, false, false, 1, layer);
+                    toFill.add(poly);
                 }
                 vec.clear();
             }
@@ -288,7 +336,7 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             std::string index = st.next();
             std::string xpos = st.next();
             std::string ypos = st.next();
-            Position pos2D((SUMOReal) atof(xpos.c_str()), (SUMOReal) atof(ypos.c_str()));
+            Position pos2D((double) atof(xpos.c_str()), (double) atof(ypos.c_str()));
             if (!geoConvHelper.x2cartesian(pos2D)) {
                 WRITE_WARNING("Unable to project coordinates for polygon '" + id + "'.");
             }
@@ -299,15 +347,15 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
         if (parsingDistrictsDirectly) {
             //$BEZIRK:NR	CODE	NAME	TYPNR	XKOORD	YKOORD	FLAECHEID	BEZART	IVANTEIL_Q	IVANTEIL_Z	OEVANTEIL	METHODEANBANTEILE	ZWERT1	ZWERT2	ZWERT3	ISTINAUSWAHL	OBEZNR	NOM_COM	COD_COM
             lineParser.parseLine(line);
-            long long int idL = TplConvert::_2long(lineParser.get("NR").c_str());
+            long long int idL = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_NO)));
             std::string id = toString(idL);
-            long long int area = TplConvert::_2long(lineParser.get("FLAECHEID").c_str());
-            SUMOReal x = TplConvert::_2SUMOReal(lineParser.get("XKOORD").c_str());
-            SUMOReal y = TplConvert::_2SUMOReal(lineParser.get("YKOORD").c_str());
+            long long int area = StringUtils::toLong(lineParser.get(KEYS.getString(VISUM_SURFACEID)));
+            double x = StringUtils::toDouble(lineParser.get(KEYS.getString(VISUM_XCOORD)));
+            double y = StringUtils::toDouble(lineParser.get(KEYS.getString(VISUM_YCOORD)));
             // patch the values
             std::string type = "district";
             bool discard = oc.getBool("discard");
-            int layer = oc.getInt("layer");
+            double layer = oc.getFloat("layer");
             RGBColor color;
             if (tm.has(type)) {
                 const PCTypeMap::TypeDef& def = tm.get(type);
@@ -323,31 +371,41 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             }
             if (!discard) {
                 if (teilflaechen[flaechenelemente[area]].size() > 0) {
-                    Polygon* poly = new Polygon(id, type, color, teilflaechen[flaechenelemente[area]], false, (SUMOReal)layer);
-                    toFill.insert(id, poly, layer);
+                    const std::string origId = id;
+                    int index = 1;
+                    while (toFill.getPolygons().get(id) != nullptr) {
+                        id = origId + "#" + toString(index++);
+                    }
+                    SUMOPolygon* poly = new SUMOPolygon(id, type, color, teilflaechen[flaechenelemente[area]], false, false, 1, layer);
+                    toFill.add(poly);
                 } else {
                     Position pos(x, y);
                     if (!geoConvHelper.x2cartesian(pos)) {
                         WRITE_WARNING("Unable to project coordinates for POI '" + id + "'.");
                     }
-                    PointOfInterest* poi = new PointOfInterest(id, type, color, pos, (SUMOReal)layer);
-                    toFill.insert(id, poi, layer);
+                    const std::string origId = id;
+                    int index = 1;
+                    while (toFill.getPOIs().get(id) != nullptr) {
+                        id = origId + "#" + toString(index++);
+                    }
+                    PointOfInterest* poi = new PointOfInterest(id, type, color, pos, false, "", 0, 0, layer);
+                    toFill.add(poi);
                 }
             }
         }
 
 
-        if (line.find("$POIKATEGORIEDEF:") == 0 || line.find("$POIKATEGORIE:") == 0) {
+        if (line.find("$POIKATEGORIEDEF:") == 0 || line.find("$" + KEYS.getString(VISUM_POICATEGORY) + ":") == 0) {
             // ok, got categories, begin parsing from next line
             parsingCategories = true;
             lineParser.reinit(line.substr(line.find(":") + 1));
         }
-        if (line.find("$POI:") == 0) {
+        if ((line.find("$POI:") == 0) || line.find("$POIOFCAT") != std::string::npos) {
             // ok, got pois, begin parsing from next line
             parsingPOIs = true;
             lineParser.reinit(line.substr(line.find(":") + 1));
         }
-        if (line.find("$BEZIRK") == 0 && line.find("FLAECHEID") != std::string::npos) {
+        if (line.find("$" + KEYS.getString(VISUM_DISTRICT)) == 0 && line.find(KEYS.getString(VISUM_SURFACEID)) != std::string::npos) {
             // ok, have a district header, and it seems like districts would reference shapes...
             parsingDistrictsDirectly = true;
             lineParser.reinit(line.substr(line.find(":") + 1));
@@ -365,6 +423,27 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
 }
 
 
+void
+PCLoaderVisum::loadLanguage(const std::string& file) {
+    std::ifstream strm(file.c_str());
+    if (!strm.good()) {
+        throw ProcessError("Could not load VISUM language map from '" + file + "'.");
+    }
+    while (strm.good()) {
+        std::string keyDE;
+        std::string keyNew;
+        strm >> keyDE;
+        strm >> keyNew;
+        if (KEYS.hasString(keyDE)) {
+            VISUM_KEY key = KEYS.get(keyDE);
+            KEYS.remove(keyDE, key);
+            KEYS.insert(keyNew, key);
+        } else if (keyDE != "") {
+            // do not warn about network-related keys (NIImporter_VISUM)
+            //WRITE_WARNING("Unknown entry '" + keyDE + "' in VISUM language map");
+        }
+    }
+
+}
 
 /****************************************************************************/
-
