@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PCLoaderVisum.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Thu, 02.11.2006
-/// @version $Id: PCLoaderVisum.h 22608 2017-01-17 06:28:54Z behrisch $
 ///
 // A reader of pois and polygons stored in VISUM-format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef PCLoaderVisum_h
-#define PCLoaderVisum_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include "PCPolyContainer.h"
@@ -78,10 +68,60 @@ protected:
                      PCTypeMap& tm);
 
 
+    /**
+     * @enum VISUM keys
+     * @brief Numbers representing VISUM keywords
+     */
+    enum VISUM_KEY {
+        VISUM_SYS,
+        VISUM_LINKTYPE,
+        VISUM_NODE,
+        VISUM_DISTRICT,
+        VISUM_POINT,
+        VISUM_LINK,
+        VISUM_V0,
+        VISUM_TYPES,
+        VISUM_RANK,
+        VISUM_CAPACITY,
+        VISUM_XCOORD,
+        VISUM_YCOORD,
+        VISUM_FROMNODE,
+        VISUM_TONODE,
+        VISUM_TYPE,
+        VISUM_TYP,
+        VISUM_ID,
+        VISUM_CODE,
+        VISUM_DISTRICT_CONNECTION,
+        VISUM_SOURCE_DISTRICT,
+        VISUM_FROMNODENO,
+        VISUM_DIRECTION,
+        VISUM_SURFACEID,
+        VISUM_FACEID,
+        VISUM_FROMPOINTID,
+        VISUM_TOPOINTID,
+        VISUM_EDGE,
+        VISUM_VIANODENO,
+        VISUM_NUMLANES,
+        VISUM_TURN,
+        VISUM_INDEX,
+        VISUM_LINKPOLY,
+        VISUM_SURFACEITEM,
+        VISUM_FACEITEM,
+        VISUM_EDGEID,
+        VISUM_ORIGIN,
+        VISUM_DESTINATION,
+        // polyconvert keys added to avoid warnings
+        VISUM_CATID,
+        VISUM_EDGEITEM,
+        VISUM_POICATEGORY,
+        VISUM_NO // must be the last one
+    };
+
+    /// Strings for the keywords
+    static StringBijection<VISUM_KEY>::Entry KEYS_DE[];
+
+    /// @brief link directions
+    static StringBijection<VISUM_KEY> KEYS;
+
+    static void loadLanguage(const std::string& file);
 };
-
-
-#endif
-
-/****************************************************************************/
-
