@@ -1,35 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2005-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PCPolyContainer.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Melanie Knocke
 /// @date    Mon, 05 Dec 2005
-/// @version $Id: PCPolyContainer.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A storage for loaded polygons and pois
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2005-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <algorithm>
@@ -39,7 +30,7 @@
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/StringUtils.h>
 #include <utils/geom/GeoConvHelper.h>
-#include <utils/shapes/Polygon.h>
+#include <utils/shapes/SUMOPolygon.h>
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/xml/SUMOSAXAttributes.h>
 #include <utils/options/OptionsCont.h>
@@ -63,7 +54,7 @@ PCPolyContainer::~PCPolyContainer() {
 
 
 bool
-PCPolyContainer::add(SUMO::Polygon* poly, bool ignorePruning) {
+PCPolyContainer::add(SUMOPolygon* poly, bool ignorePruning) {
     // check whether the polygon lies within the wished area
     //  - if such an area was given
     if (myDoPrune && !ignorePruning) {
@@ -102,8 +93,8 @@ PCPolyContainer::add(PointOfInterest* poi, bool ignorePruning) {
 
 
 void
-PCPolyContainer::addLanePos(const std::string& poiID, const std::string& laneID, double lanePos) {
-    myLanePosPois[poiID] = std::make_pair(laneID, lanePos);
+PCPolyContainer::addLanePos(const std::string& poiID, const std::string& laneID, double lanePos, double lanePosLat) {
+    myLanePosPois[poiID] = LanePos(laneID, lanePos, lanePosLat);
 }
 
 
@@ -122,17 +113,17 @@ PCPolyContainer::save(const std::string& file, bool useGeo) {
         GeoConvHelper::writeLocation(out);
     }
     // write polygons
-    for (std::map<std::string, SUMO::Polygon*>::const_iterator i = myPolygons.getMyMap().begin(); i != myPolygons.getMyMap().end(); ++i) {
-        i->second->writeXML(out, useGeo);
+    for (auto i : myPolygons) {
+        i.second->writeXML(out, useGeo);
     }
     // write pois
     const double zOffset = OptionsCont::getOptions().getFloat("poi-layer-offset");
-    for (std::map<std::string, PointOfInterest*>::const_iterator i = myPOIs.getMyMap().begin(); i != myPOIs.getMyMap().end(); ++i) {
-        std::map<std::string, std::pair<std::string, double> >::const_iterator it = myLanePosPois.find(i->first);
+    for (auto i : myPOIs) {
+        std::map<std::string, LanePos>::const_iterator it = myLanePosPois.find(i.first);
         if (it == myLanePosPois.end()) {
-            i->second->writeXML(out, useGeo, zOffset);
+            i.second->writeXML(out, useGeo, zOffset);
         } else {
-            i->second->writeXML(out, useGeo, zOffset, it->second.first, it->second.second);
+            i.second->writeXML(out, useGeo, zOffset, it->second.laneID, it->second.pos, it->second.posLat);
         }
     }
     out.close();
@@ -141,13 +132,7 @@ PCPolyContainer::save(const std::string& file, bool useGeo) {
 
 void PCPolyContainer::writeDlrTDPHeader(OutputDevice& device, const OptionsCont& oc) {
     // XXX duplicate of NWWriter_DlrNavteq::writeHeader()
-    const std::string OUTPUT_VERSION = "6.5"; // XXX duplicate of NWWriter_DlrNavteq OUTPUT_VERSION
-    time_t rawtime;
-    time(&rawtime);
-    char buffer [80];
-    strftime(buffer, 80, "on %c", localtime(&rawtime));
-    device << "# Generated " << buffer << " by " << oc.getFullName() << "\n";
-    device << "# Format matches Extraction version: V" << OUTPUT_VERSION << " \n";
+    device << "# Format matches Extraction version: V6.5 \n";
     std::stringstream tmp;
     oc.writeConfiguration(tmp, true, false, false);
     tmp.seekg(std::ios_base::beg);
@@ -173,14 +158,14 @@ PCPolyContainer::saveDlrTDP(const std::string& prefix) {
     // write format specifier
     out << "# ID\tCITY\tTYPE\tNAME\tgeo_x\tgeo_y\n";
     int id = 0;
-    for (std::map<std::string, PointOfInterest*>::const_iterator i = myPOIs.getMyMap().begin(); i != myPOIs.getMyMap().end(); ++i) {
-        Position pos(*(i->second));
+    for (const auto& i : myPOIs) {
+        Position pos(*i.second);
         gch.cartesian2geo(pos);
         pos.mul(geoScale);
         out << id << "\t";
         out << "" << "\t";
-        out << i->second->getType() << "\t";
-        out << i->first << "\t";
+        out << i.second->getShapeType() << "\t";
+        out << i.first << "\t";
         out << pos.x() << "\t";
         out << pos.y() << "\t";
         id++;
@@ -193,15 +178,13 @@ PCPolyContainer::saveDlrTDP(const std::string& prefix) {
     // write format specifier
     out2 << "# ID\tCITY\tTYPE\tNAME\tgeo_x1\tgeo_y1\t[geo_x2 geo_y2 ...]\n";
     id = 0;
-    for (std::map<std::string, SUMO::Polygon*>::const_iterator i = myPolygons.getMyMap().begin(); i != myPolygons.getMyMap().end(); ++i) {
+    for (const auto& i : myPolygons) {
         out2 << id << "\t";
         out2 << "" << "\t";
-        out2 << i->second->getType() << "\t";
-        out2 << i->first << "\t";
+        out2 << i.second->getShapeType() << "\t";
+        out2 << i.first << "\t";
 
-        PositionVector shape(i->second->getShape());
-        for (int i = 0; i < (int) shape.size(); i++) {
-            Position pos = shape[i];
+        for (Position pos : i.second->getShape()) {
             gch.cartesian2geo(pos);
             pos.mul(geoScale);
             out2 << pos.x() << "\t";
@@ -219,6 +202,4 @@ PCPolyContainer::getEnumIDFor(const std::string& key) {
 }
 
 
-
 /****************************************************************************/
-
