@@ -1,45 +1,41 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2005-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PCPolyContainer.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 05 Dec 2005
-/// @version $Id: PCPolyContainer.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // A storage for loaded polygons and pois
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2005-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef PCPolyContainer_h
-#define PCPolyContainer_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
 #include <vector>
-#include <utils/shapes/Polygon.h>
-#include <utils/shapes/PointOfInterest.h>
 #include <utils/shapes/ShapeContainer.h>
-#include <utils/geom/Boundary.h>
-#include <utils/common/UtilExceptions.h>
+
+
+// ===========================================================================
+// class declarations
+// ===========================================================================
+class Boundary;
+class SUMOPolygon;
+class PointOfInterest;
+class OptionsCont;
+class OutputDevice;
 
 
 // ===========================================================================
@@ -77,7 +73,7 @@ public:
      * @param[in] ignorePruning Whether the polygon shall be kept, even though it would be pruned
      * @return Whether the polygon could be added
      */
-    bool add(SUMO::Polygon* poly, bool ignorePruning = false);
+    bool add(SUMOPolygon* poly, bool ignorePruning = false);
 
 
     /** @brief Adds a poi to the storage
@@ -96,7 +92,7 @@ public:
     bool add(PointOfInterest* poi, bool ignorePruning = false);
 
 
-    void addLanePos(const std::string& poiID, const std::string& laneID, double lanePos);
+    void addLanePos(const std::string& poiID, const std::string& laneID, double lanePos, double lanePosLat);
 
     /** @brief Saves the stored polygons and pois into the given file
      * @param[in] file The name of the file to write stored objects' definitions into
@@ -123,8 +119,18 @@ public:
 
 
 private:
+
+    struct LanePos {
+        LanePos() {}
+        LanePos(const std::string& _laneID, double _pos, double _posLat) :
+            laneID(_laneID), pos(_pos), posLat(_posLat) {}
+        std::string laneID;
+        double pos;
+        double posLat;
+    };
+
     /// @brief An id to pos map for lane pos specs
-    std::map<std::string, std::pair<std::string, double> > myLanePosPois;
+    std::map<std::string, LanePos> myLanePosPois;
 
     /// @brief An id to int map for proper enumeration
     std::map<std::string, int> myIDEnums;
@@ -149,9 +155,3 @@ private:
 
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
