@@ -1,44 +1,31 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2005-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PCTypeMap.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 05 Dec 2005
-/// @version $Id: PCTypeMap.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A storage for type mappings
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2005-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
 #include "utils/options/OptionsCont.h"
 #include "PCTypeMap.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -46,7 +33,7 @@
 PCTypeMap::PCTypeMap(const OptionsCont& oc) {
     myDefaultType.id = oc.getString("type");
     myDefaultType.color = RGBColor::parseColor(oc.getString("color"));
-    myDefaultType.layer = oc.getInt("layer");
+    myDefaultType.layer = oc.getFloat("layer");
     myDefaultType.discard = oc.getBool("discard");
     myDefaultType.allowFill = oc.getBool("fill");
     myDefaultType.prefix = oc.getString("prefix");
@@ -59,7 +46,8 @@ PCTypeMap::~PCTypeMap() {}
 bool
 PCTypeMap::add(const std::string& id, const std::string& newid,
                const std::string& color, const std::string& prefix,
-               int layer, bool discard, bool allowFill) {
+               double layer, double angle, const std::string& imgFile,
+               bool discard, bool allowFill) {
     if (has(id)) {
         return false;
     }
@@ -67,6 +55,8 @@ PCTypeMap::add(const std::string& id, const std::string& newid,
     td.id = newid;
     td.color = RGBColor::parseColor(color);
     td.layer = layer;
+    td.angle = angle;
+    td.imgFile = imgFile;
     td.discard = discard;
     td.allowFill = allowFill;
     td.prefix = prefix;
@@ -87,6 +77,4 @@ PCTypeMap::has(const std::string& id) {
 }
 
 
-
 /****************************************************************************/
-
