@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2005-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    polyconvert_main.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -6,31 +18,10 @@
 /// @author  Michael Behrisch
 /// @author  Melanie Knocke
 /// @date    Mon, 05 Dec 2005
-/// @version $Id: polyconvert_main.cpp 20801 2016-05-28 05:31:30Z behrisch $
 ///
 // Main for POLYCONVERT
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2005-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #ifdef HAVE_VERSION_H
 #include <version.h>
@@ -44,7 +35,7 @@
 #include <utils/common/StringTokenizer.h>
 #include <utils/common/SystemFrame.h>
 #include <utils/common/MsgHandler.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/importio/LineReader.h>
@@ -61,11 +52,7 @@
 #include <polyconvert/PCTypeMap.h>
 #include <polyconvert/PCTypeDefHandler.h>
 #include <polyconvert/PCNetProjectionLoader.h>
-#include "pc_typemap.h"
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
+#include <polyconvert/pc_typemap.h>
 
 
 // ===========================================================================
@@ -84,7 +71,6 @@ fillOptions() {
     oc.addOptionSubTopic("Pruning");
     oc.addOptionSubTopic("Processing");
     oc.addOptionSubTopic("Building Defaults");
-    SystemFrame::addReportOptions(oc); // fill this subtopic, too
 
 
     // register options
@@ -105,6 +91,9 @@ fillOptions() {
     oc.addSynonyme("visum-files", "visum");
     oc.addDescription("visum-files", "Input", "Reads polygons from FILE assuming it's a Visum-net");
 
+    oc.doRegister("visum.language-file", new Option_FileName());
+    oc.addDescription("visum.language-file", "Input", "Load language mappings from FILE");
+
     // xml import
     oc.doRegister("xml-files", new Option_FileName());
     oc.addSynonyme("xml-files", "xml");
@@ -118,6 +107,8 @@ fillOptions() {
     oc.addDescription("osm.keep-full-type", "Input", "The type will be made of the key-value - pair");
     oc.doRegister("osm.use-name", new Option_Bool(false));
     oc.addDescription("osm.use-name", "Input", "The id will be set from the given 'name' attribute");
+    oc.doRegister("osm.merge-relations", new Option_Float(-1));
+    oc.addDescription("osm.merge-relations", "Input", "If FLOAT >= 0, assemble one polygon from all ways of a relation if they all connect with gaps below FLOAT");
 
     // arcview import
     oc.doRegister("shapefile-prefixes", new Option_FileName());
@@ -130,16 +121,23 @@ fillOptions() {
     oc.addSynonyme("shapefile.guess-projection", "arcview.guess-projection", true);
     oc.addDescription("shapefile.guess-projection", "Input", "Guesses the shapefile's projection");
 
+    oc.doRegister("shapefile.traditional-axis-mapping", new Option_Bool(false));
+    oc.addDescription("shapefile.traditional-axis-mapping", "Input", "Use traditional axis order (lon, lat)");
+
     oc.doRegister("shapefile.id-column", new Option_String());
     oc.addSynonyme("shapefile.id-column", "shapefile.id-name", true);
     oc.addSynonyme("shapefile.id-column", "shape-files.id-name", true);
     oc.addDescription("shapefile.id-column", "Input", "Defines in which column the id can be found");
 
-    oc.doRegister("shapefile.use-running-id", new Option_Bool());
+    oc.doRegister("shapefile.type-columns", new Option_StringVector());
+    oc.addSynonyme("shapefile.type-columns", "shapefile.type-column");
+    oc.addDescription("shapefile.type-columns", "Input", "Defines which columns form the type id (comma separated list)");
+
+    oc.doRegister("shapefile.use-running-id", new Option_Bool(false));
     oc.addDescription("shapefile.use-running-id", "Input", "A running number will be used as id");
 
-    oc.doRegister("shapefile.add-param", new Option_Bool());
-    oc.addDescription("shapefile.add-param", "Input", "Extract all additonal columns as params");
+    oc.doRegister("shapefile.add-param", new Option_Bool(false));
+    oc.addDescription("shapefile.add-param", "Input", "Extract all additional columns as params");
 
     oc.doRegister("shapefile.fill", new Option_String());
     oc.addDescription("shapefile.fill", "Input", "[auto|true|false]. Forces the 'fill' status to the given value. Default 'auto' tries to determine it from the data type");
@@ -149,12 +147,17 @@ fillOptions() {
     oc.addSynonyme("type-file", "typemap", true);
     oc.addDescription("type-file", "Input", "Reads types from FILE");
 
+    // need to do this here to be able to check for network and route input options
+    SystemFrame::addReportOptions(oc);
 
     // output
-    oc.doRegister("output-file", 'o', new Option_FileName("polygons.xml"));
+    oc.doRegister("output-file", 'o', new Option_FileName());
     oc.addSynonyme("output-file", "output");
     oc.addDescription("output-file", "Output", "Write generated polygons/pois to FILE");
 
+    oc.doRegister("dlr-tdp-output", new Option_FileName());
+    oc.addDescription("dlr-tdp-output", "Output", "Write generated polygons/pois to a dlr-tdp file with the given prefix");
+
 
     // prunning options
     oc.doRegister("prune.in-net", new Option_Bool(false));
@@ -173,9 +176,9 @@ fillOptions() {
     oc.addSynonyme("prune.keep-list", "prune.ignore", true);
     oc.addDescription("prune.keep-list", "Pruning", "Items in STR will be kept though out of boundary");
 
-    oc.doRegister("prune.explicit", new Option_String(""));
+    oc.doRegister("prune.explicit", new Option_StringVector(StringVector({ "" })));
     oc.addSynonyme("prune.explicit", "remove");
-    oc.addDescription("prune.explicit", "Pruning", "Items with names in STR will be removed");
+    oc.addDescription("prune.explicit", "Pruning", "Items with names in STR[] will be removed");
 
 
     oc.doRegister("offset.x", new Option_Float(0));
@@ -186,6 +189,9 @@ fillOptions() {
     oc.addSynonyme("offset.y", "y-offset-to-apply", true);
     oc.addDescription("offset.y", "Processing", "Adds FLOAT to net y-positions");
 
+    oc.doRegister("offset.z", new Option_Float(0));
+    oc.addDescription("offset.z", "Processing", "Adds FLOAT to net z-positions");
+
     oc.doRegister("all-attributes", new Option_Bool(false));
     oc.addDescription("all-attributes", "Processing", "Imports all attributes as key/value pairs");
 
@@ -223,8 +229,8 @@ fillOptions() {
 int
 main(int argc, char** argv) {
     OptionsCont& oc = OptionsCont::getOptions();
-    oc.setApplicationDescription("Importer of polygons and POIs for the road traffic simulation SUMO.");
-    oc.setApplicationName("polyconvert", "SUMO polyconvert Version " VERSION_STRING);
+    oc.setApplicationDescription("Importer of polygons and POIs for the microscopic, multi-modal traffic simulation SUMO.");
+    oc.setApplicationName("polyconvert", "Eclipse SUMO polyconvert Version " VERSION_STRING);
     int ret = 0;
     try {
         // initialise subsystems
@@ -236,21 +242,24 @@ main(int argc, char** argv) {
             SystemFrame::close();
             return 0;
         }
-        XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"));
+        SystemFrame::checkOptions();
+        XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"), "never");
         MsgHandler::initOutputOptions();
         // build the projection
-        int shift = 0;
+        double scale = 1.0;
         if ((oc.isSet("dlr-navteq-poly-files") || oc.isSet("dlr-navteq-poi-files")) && oc.isDefault("proj.scale")) {
-            shift = 5;
+            scale = 1e-5;
         }
         if (!oc.isSet("net")) {
             // from the given options
-#ifdef HAVE_PROJ
-            unsigned numProjections = oc.getBool("simple-projection") + oc.getBool("proj.utm") + oc.getBool("proj.dhdn") + (oc.getString("proj").length() > 1);
+#ifdef PROJ_API_FILE
+            const int numProjections = oc.getBool("simple-projection") + oc.getBool("proj.utm") + oc.getBool("proj.dhdn") + (oc.getString("proj").length() > 1);
             if ((oc.isSet("osm-files") || oc.isSet("dlr-navteq-poly-files") || oc.isSet("dlr-navteq-poi-files")) && numProjections == 0) {
                 oc.set("proj.utm", "true");
             }
-            oc.set("proj.scale", toString(shift));
+            if (oc.isDefault("proj.scale")) {
+                oc.set("proj.scale", toString(scale, 5));
+            }
 #endif
             if (!GeoConvHelper::init(oc)) {
                 throw ProcessError("Could not build projection!");
@@ -258,7 +267,7 @@ main(int argc, char** argv) {
         } else {
             // from the supplied network
             // @todo warn about given options being ignored
-            PCNetProjectionLoader::load(oc.getString("net"), shift);
+            PCNetProjectionLoader::load(oc.getString("net"), scale);
         }
         Boundary pruningBoundary = GeoConvHelper::getFinal().getConvBoundary();
         // check whether the input shall be pruned
@@ -269,7 +278,7 @@ main(int argc, char** argv) {
             }
             bool ok = true;
             // !!! no proper error handling
-            Boundary offsets = GeomConvHelper::parseBoundaryReporting(oc.getString("prune.in-net.offsets"), "--prune.on-net.offsets", 0, ok);
+            Boundary offsets = GeomConvHelper::parseBoundaryReporting(oc.getString("prune.in-net.offsets"), "--prune.on-net.offsets", nullptr, ok);
             pruningBoundary = Boundary(
                                   pruningBoundary.xmin() + offsets.xmin(),
                                   pruningBoundary.ymin() + offsets.ymin(),
@@ -280,7 +289,7 @@ main(int argc, char** argv) {
         if (oc.isSet("prune.boundary")) {
             bool ok = true;
             // !!! no proper error handling
-            pruningBoundary = GeomConvHelper::parseBoundaryReporting(oc.getString("prune.boundary"), "--prune.boundary", 0, ok);
+            pruningBoundary = GeomConvHelper::parseBoundaryReporting(oc.getString("prune.boundary"), "--prune.boundary", nullptr, ok);
             prune = true;
         }
         if (oc.isSet("osm-files") && oc.isDefault("poi-layer-offset")) {
@@ -292,7 +301,7 @@ main(int argc, char** argv) {
         // read in the type defaults
         if (!oc.isSet("type-file")) {
             const char* sumoPath = std::getenv("SUMO_HOME");
-            if (sumoPath == 0) {
+            if (sumoPath == nullptr) {
                 WRITE_WARNING("Environment variable SUMO_HOME is not set, using built in type maps.");
             } else {
                 const std::string path = sumoPath + std::string("/data/typemap/");
@@ -339,7 +348,21 @@ main(int argc, char** argv) {
         if (MsgHandler::getErrorInstance()->wasInformed() && !oc.getBool("ignore-errors")) {
             throw ProcessError();
         }
-        toFill.save(oc.getString("output-file"), oc.getBool("proj.plain-geo"));
+        // output
+        if (!oc.isSet("output-file") && !oc.isSet("dlr-tdp-output")) {
+            std::string out = "polygons.xml";
+            if (oc.isSet("configuration-file")) {
+                out = FileHelpers::getConfigurationRelative(oc.getString("configuration-file"), out);
+            }
+            oc.setDefault("output-file", out);
+        }
+        if (oc.isSet("output-file")) {
+            toFill.save(oc.getString("output-file"), oc.getBool("proj.plain-geo"));
+        }
+        if (oc.isSet("dlr-tdp-output")) {
+            toFill.saveDlrTDP(oc.getString("dlr-tdp-output"));
+        }
+
     } catch (const ProcessError& e) {
         if (std::string(e.what()) != std::string("Process Error") && std::string(e.what()) != std::string("")) {
             WRITE_ERROR(e.what());
@@ -367,6 +390,4 @@ main(int argc, char** argv) {
 }
 
 
-
 /****************************************************************************/
-
