@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    ROEdge.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -7,33 +19,11 @@
 /// @author  Melanie Knocke
 /// @author  Yun-Pang Floetteroed
 /// @date    Sept 2002
-/// @version $Id: ROEdge.h 24122 2017-04-28 12:50:58Z namdre $
 ///
 // A basic edge for routing applications
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef ROEdge_h
-#define ROEdge_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -43,9 +33,11 @@
 #include <utils/common/StdDefs.h>
 #include <utils/common/ValueTimeLine.h>
 #include <utils/common/SUMOVehicleClass.h>
+#include <utils/common/RandHelper.h>
 #include <utils/emissions/PollutantsInterface.h>
+#include <utils/geom/Boundary.h>
 #ifdef HAVE_FOX
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
 #endif
 #include <utils/vehicle/SUMOVTypeParameter.h>
 #include "RONode.h"
@@ -60,6 +52,7 @@ class ROEdge;
 
 typedef std::vector<ROEdge*> ROEdgeVector;
 typedef std::vector<const ROEdge*> ConstROEdgeVector;
+typedef std::vector<std::pair<const ROEdge*, const ROEdge*> > ROConstEdgePairVector;
 
 
 // ===========================================================================
@@ -74,30 +67,8 @@ typedef std::vector<const ROEdge*> ConstROEdgeVector;
  *  the weights, it is needed to call "buildTimeLines" in order to initialise
  *  these time lines.
  */
-class ROEdge : public Named {
+class ROEdge : public Named, public Parameterised {
 public:
-    /**
-     * @enum EdgeFunc
-     * @brief Possible functions of edges
-     */
-    enum EdgeFunc {
-        /// @brief A normal edge
-        ET_NORMAL,
-        /// @brief An edge representing a whole district
-        ET_DISTRICT,
-        /// @brief An edge where vehicles are inserted at (no vehicle may come from back)
-        ET_SOURCE,
-        /// @brief An edge where vehicles disappear (no vehicle may leave this edge)
-        ET_SINK,
-        /// @brief An internal edge which models walking areas for pedestrians
-        ET_WALKINGAREA,
-        /// @brief An internal edge which models pedestrian crossings
-        ET_CROSSING,
-        /// @brief An internal edge which models vehicles driving across a junction. This is currently not used for routing
-        ET_INTERNAL
-    };
-
-
     /** @brief Constructor
      *
      * @param[in] id The id of the edge
@@ -132,14 +103,30 @@ public:
      * @param[in] s The edge to add
      * @todo What about vehicle-type aware connections?
      */
-    virtual void addSuccessor(ROEdge* s, std::string dir = "");
+    virtual void addSuccessor(ROEdge* s, ROEdge* via = nullptr, std::string dir = "");
 
 
     /** @brief Sets the function of the edge
-     * @param[in] func The new function for the edge
-     */
-    inline void setFunc(EdgeFunc func) {
-        myFunc = func;
+    * @param[in] func The new function for the edge
+    */
+    inline void setFunction(SumoXMLEdgeFunc func) {
+        myFunction = func;
+    }
+
+
+    /** @brief Sets whether the edge is a source
+    * @param[in] func The new source functionality for the edge
+    */
+    inline void setSource(const bool isSource = true) {
+        myAmSource = isSource;
+    }
+
+
+    /** @brief Sets whether the edge is a sink
+    * @param[in] func The new sink functionality for the edge
+    */
+    inline void setSink(const bool isSink = true) {
+        myAmSink = isSink;
     }
 
 
@@ -150,20 +137,35 @@ public:
         myRestrictions = restrictions;
     }
 
+    inline void setTimePenalty(double value) {
+        myTimePenalty = value;
+    }
 
     /// @brief return whether this edge is an internal edge
     inline bool isInternal() const {
-        return myFunc == ET_INTERNAL;
+        return myFunction == SumoXMLEdgeFunc::INTERNAL;
     }
 
     /// @brief return whether this edge is a pedestrian crossing
     inline bool isCrossing() const {
-        return myFunc == ET_CROSSING;
+        return myFunction == SumoXMLEdgeFunc::CROSSING;
     }
 
     /// @brief return whether this edge is walking area
     inline bool isWalkingArea() const {
-        return myFunc == ET_WALKINGAREA;
+        return myFunction == SumoXMLEdgeFunc::WALKINGAREA;
+    }
+
+    inline bool isTazConnector() const {
+        return myFunction == SumoXMLEdgeFunc::CONNECTOR;
+    }
+
+    void setOtherTazConnector(const ROEdge* edge) {
+        myOtherTazConnector = edge;
+    }
+
+    const ROEdge* getOtherTazConnector() const {
+        return myOtherTazConnector;
     }
 
     /** @brief Builds the internal representation of the travel time/effort
@@ -176,6 +178,8 @@ public:
      * @param[in] measure The name of the measure to use.
      */
     void buildTimeLines(const std::string& measure, const bool boundariesOverride);
+
+    void cacheParamRestrictions(const std::vector<std::string>& restrictionKeys);
     //@}
 
 
@@ -184,11 +188,19 @@ public:
     //@{
 
     /** @brief Returns the function of the edge
-     * @return This edge's basic function
-     * @see EdgeFunc
-     */
-    EdgeFunc getFunc() const {
-        return myFunc;
+    * @return This edge's basic function
+    * @see SumoXMLEdgeFunc
+    */
+    inline SumoXMLEdgeFunc getFunction() const {
+        return myFunction;
+    }
+
+
+    /** @brief Returns whether the edge acts as a sink
+    * @return whether the edge is a sink
+    */
+    inline bool isSink() const {
+        return myAmSink;
     }
 
 
@@ -216,13 +228,7 @@ public:
 
     /// @brief return a lower bound on shape.length() / myLength that is
     // sufficient for the astar air-distance heuristic
-    double getLengthGeometryFactor() const {
-        if (myFromJunction != 0 && myToJunction != 0) {
-            return MAX2(1.0, myFromJunction->getPosition().distanceTo(myToJunction->getPosition()) / myLength);
-        } else {
-            return 1.0;
-        }
-    }
+    double getLengthGeometryFactor() const;
 
     /** @brief Returns the lane's maximum speed, given a vehicle's speed limit adaptation
      * @param[in] The vehicle to return the adapted speed limit for
@@ -242,7 +248,7 @@ public:
     /** @brief Returns the number of lanes this edge has
      * @return This edge's number of lanes
      */
-    int getLaneNo() const {
+    int getNumLanes() const {
         return (int) myLanes.size();
     }
 
@@ -250,10 +256,10 @@ public:
     /** @brief returns the information whether this edge is directly connected to the given
      *
      * @param[in] e The edge which may be connected
-     * @param[in] vehicle The vehicle for which the connectivity is checked
+     * @param[in] vClass The vehicle class for which the connectivity is checked
      * @return Whether the given edge is a direct successor to this one
      */
-    bool isConnectedTo(const ROEdge* const e, const ROVehicle* const vehicle) const;
+    bool isConnectedTo(const ROEdge& e, const SUMOVehicleClass vClass) const;
 
 
     /** @brief Returns whether this edge prohibits the given vehicle to pass it
@@ -269,6 +275,21 @@ public:
         return myCombinedPermissions;
     }
 
+    /** @brief Returns whether this edge has restriction parameters forbidding the given vehicle to pass it
+     * @param[in] vehicle The vehicle for which the information has to be returned
+     * @return Whether the vehicle must not enter this edge
+     */
+    inline bool restricts(const ROVehicle* const vehicle) const {
+        const std::vector<double>& vTypeRestrictions = vehicle->getType()->paramRestrictions;
+        assert(vTypeRestrictions.size() == myParamRestrictions.size());
+        for (int i = 0; i < (int)vTypeRestrictions.size(); i++) {
+            if (vTypeRestrictions[i] > myParamRestrictions[i]) {
+                return true;
+            }
+        }
+        return false;
+    }
+
 
     /** @brief Returns whether this edge succeeding edges prohibit the given vehicle to pass them
      * @param[in] vehicle The vehicle for which the information has to be returned
@@ -310,18 +331,17 @@ public:
     int getNumSuccessors() const;
 
 
-    /** @brief Returns the following edges
-     */
-    const ROEdgeVector& getSuccessors() const {
-        return myFollowingEdges;
-    }
-
-
     /** @brief Returns the following edges, restricted by vClass
-     * @param[in] vClass The vClass for which to restrict the successors
-     * @return The eligible following edges
-     */
-    const ROEdgeVector& getSuccessors(SUMOVehicleClass vClass) const;
+    * @param[in] vClass The vClass for which to restrict the successors
+    * @return The eligible following edges
+    */
+    const ROEdgeVector& getSuccessors(SUMOVehicleClass vClass = SVC_IGNORING) const;
+
+    /** @brief Returns the following edges including vias, restricted by vClass
+    * @param[in] vClass The vClass for which to restrict the successors
+    * @return The eligible following edges
+    */
+    const ROConstEdgePairVector& getViaSuccessors(SUMOVehicleClass vClass = SVC_IGNORING) const;
 
 
     /** @brief Returns the number of edges connected to this edge
@@ -342,6 +362,11 @@ public:
         return myApproachingEdges;
     }
 
+    /// @brief if this edge is an internal edge, return its first normal predecessor, otherwise the edge itself
+    const ROEdge* getNormalBefore() const;
+
+    /// @brief if this edge is an internal edge, return its first normal successor, otherwise the edge itself
+    const ROEdge* getNormalAfter() const;
 
     /** @brief Returns the effort for this edge
      *
@@ -394,6 +419,24 @@ public:
         return edge->getTravelTime(veh, time);
     }
 
+    static inline double getTravelTimeStaticRandomized(const ROEdge* const edge, const ROVehicle* const veh, double time) {
+        return edge->getTravelTime(veh, time) * RandHelper::rand(1., gWeightsRandomFactor);
+    }
+
+    /// @brief Alias for getTravelTimeStatic (there is no routing device to provide aggregated travel times)
+    static inline double getTravelTimeAggregated(const ROEdge* const edge, const ROVehicle* const veh, double time) {
+        return edge->getTravelTime(veh, time);
+    }
+
+    /// @brief Return traveltime weighted by edge priority (scaled penalty for low-priority edges)
+    static inline double getTravelTimeStaticPriorityFactor(const ROEdge* const edge, const ROVehicle* const veh, double time) {
+        double result = edge->getTravelTime(veh, time);
+        // lower priority should result in higher effort (and the edge with
+        // minimum priority receives a factor of myPriorityFactor
+        const double relativeInversePrio = 1 - ((edge->getPriority() - myMinEdgePriority) / myEdgePriorityRange);
+        result *= 1 + relativeInversePrio * myPriorityFactor;
+        return result;
+    }
 
     /** @brief Returns a lower bound for the travel time on this edge without using any stored timeLine
      *
@@ -401,7 +444,13 @@ public:
      * @param[in] time The time for which the effort shall be returned [s]
      */
     inline double getMinimumTravelTime(const ROVehicle* const veh) const {
-        return myLength / MIN2(veh->getType()->maxSpeed, veh->getChosenSpeedFactor() * mySpeed);
+        if (isTazConnector()) {
+            return 0;
+        } else if (veh != 0) {
+            return myLength / MIN2(veh->getType()->maxSpeed, veh->getChosenSpeedFactor() * mySpeed);
+        } else {
+            return myLength / mySpeed;
+        }
     }
 
 
@@ -419,11 +468,17 @@ public:
 
 
     static double getNoiseEffort(const ROEdge* const edge, const ROVehicle* const veh, double time);
+
+    static double getStoredEffort(const ROEdge* const edge, const ROVehicle* const /*veh*/, double time) {
+        double ret = 0;
+        edge->getStoredEffort(time, ret);
+        return ret;
+    }
     //@}
 
 
     /// @brief optimistic distance heuristic for use in routing
-    double getDistanceTo(const ROEdge* other) const;
+    double getDistanceTo(const ROEdge* other, const bool doBoundaryEstimate = false) const;
 
 
     /** @brief Returns all ROEdges */
@@ -438,6 +493,9 @@ public:
         myInterpolate = interpolate;
     }
 
+    /// @brief return the coordinates of the center of the given stop
+    static const Position getStopPosition(const SUMOVehicleParameter::Stop& stop);
+
     /// @brief get edge priority (road class)
     int getPriority() const {
         return myPriority;
@@ -451,7 +509,6 @@ public:
         return myToJunction;
     }
 
-
     /** @brief Returns this edge's lanes
      *
      * @return This edge's lanes
@@ -459,6 +516,39 @@ public:
     const std::vector<ROLane*>& getLanes() const {
         return myLanes;
     }
+
+    /// @brief return opposite superposable/congruent edge, if it exist and 0 else
+    inline const ROEdge* getBidiEdge() const {
+        return myBidiEdge;
+    }
+
+    /// @brief set opposite superposable/congruent edge
+    inline void setBidiEdge(const ROEdge* bidiEdge) {
+        myBidiEdge = bidiEdge;
+    }
+
+    ReversedEdge<ROEdge, ROVehicle>* getReversedRoutingEdge() const {
+        if (myReversedRoutingEdge == nullptr) {
+            myReversedRoutingEdge = new ReversedEdge<ROEdge, ROVehicle>(this);
+        }
+        return myReversedRoutingEdge;
+    }
+
+    RailEdge<ROEdge, ROVehicle>* getRailwayRoutingEdge() const {
+        if (myRailwayRoutingEdge == nullptr) {
+            myRailwayRoutingEdge = new RailEdge<ROEdge, ROVehicle>(this);
+        }
+        return myRailwayRoutingEdge;
+    }
+
+    /// @brief whether effort data was loaded for this edge
+    bool hasStoredEffort() const {
+        return myUsingETimeLine;
+    }
+
+    /// @brief initialize priority factor range
+    static bool initPriorityFactor(double priorityFactor);
+
 protected:
     /** @brief Retrieves the stored effort
      *
@@ -487,7 +577,8 @@ protected:
     /// @brief The length of the edge
     double myLength;
 
-
+    /// @brief whether the edge is a source or a sink
+    bool myAmSink, myAmSource;
     /// @brief Container storing passing time varying over time for the edge
     mutable ValueTimeLine<double> myTravelTimes;
     /// @brief Information whether the time line shall be used instead of the length value
@@ -509,11 +600,13 @@ protected:
     /// @brief List of edges that may be approached from this edge
     ROEdgeVector myFollowingEdges;
 
+    ROConstEdgePairVector myFollowingViaEdges;
+
     /// @brief List of edges that approached this edge
     ROEdgeVector myApproachingEdges;
 
     /// @brief The function of the edge
-    EdgeFunc myFunc;
+    SumoXMLEdgeFunc myFunction;
 
     /// The vClass speed restrictions for this edge
     const std::map<SUMOVehicleClass, double>* myRestrictions;
@@ -524,12 +617,40 @@ protected:
     /// @brief The list of allowed vehicle classes combined across lanes
     SVCPermissions myCombinedPermissions;
 
+    /// @brief the other taz-connector if this edge isTazConnector, otherwise nullptr
+    const ROEdge* myOtherTazConnector;
+
+    /// @brief the bidirectional rail edge or nullpr
+    const ROEdge* myBidiEdge;
+
+    /// @brief The bounding rectangle of end nodes incoming or outgoing edges for taz connectors or of my own start and end node for normal edges
+    Boundary myBoundary;
+
+    /// @brief flat penalty when computing traveltime
+    double myTimePenalty;
+
+    /// @brief cached value of parameters which may restrict access
+    std::vector<double> myParamRestrictions;
+
     static ROEdgeVector myEdges;
 
+    /// @brief Coefficient for factoring edge priority into routing weight
+    static double myPriorityFactor;
+    /// @brief Minimum priority for all edges
+    static double myMinEdgePriority;
+    /// @brief the difference between maximum and minimum priority for all edges
+    static double myEdgePriorityRange;
 
     /// @brief The successors available for a given vClass
     mutable std::map<SUMOVehicleClass, ROEdgeVector> myClassesSuccessorMap;
 
+    /// @brief The successors with vias available for a given vClass
+    mutable std::map<SUMOVehicleClass, ROConstEdgePairVector> myClassesViaSuccessorMap;
+
+    /// @brief a reversed version for backward routing
+    mutable ReversedEdge<ROEdge, ROVehicle>* myReversedRoutingEdge = nullptr;
+    mutable RailEdge<ROEdge, ROVehicle>* myRailwayRoutingEdge = nullptr;
+
 #ifdef HAVE_FOX
     /// The mutex used to avoid concurrent updates of myClassesSuccessorMap
     mutable FXMutex myLock;
@@ -543,9 +664,3 @@ private:
     ROEdge& operator=(const ROEdge& src);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
