@@ -1,33 +1,25 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 /****************************************************************************/
 /// @file    ROFrame.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // Sets and checks options for routing
 /****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <ctime>
@@ -39,6 +31,7 @@
 #include <utils/common/RandHelper.h>
 #include <utils/common/ToString.h>
 #include <utils/common/SUMOTime.h>
+#include <utils/common/SystemFrame.h>
 #include "ROFrame.h"
 
 
@@ -81,6 +74,12 @@ ROFrame::fillOptions(OptionsCont& oc) {
     oc.addSynonyme("route-files", "trips", true);
     oc.addDescription("route-files", "Input", "Read sumo routes, alternatives, flows, and trips from FILE(s)");
 
+    oc.doRegister("phemlight-path", new Option_FileName(StringVector({ "./PHEMlight/" })));
+    oc.addDescription("phemlight-path", "Input", "Determines where to load PHEMlight definitions from.");
+
+    // need to do this here to be able to check for network and route input options
+    SystemFrame::addReportOptions(oc);
+
     // register the time settings
     oc.doRegister("begin", 'b', new Option_String("0", "TIME"));
     oc.addDescription("begin", "Time", "Defines the begin time; Previous trips will be discarded");
@@ -91,7 +90,7 @@ ROFrame::fillOptions(OptionsCont& oc) {
     // register the processing options
     oc.doRegister("ignore-errors", new Option_Bool(false));
     oc.addSynonyme("ignore-errors", "continue-on-unbuild", true);
-    oc.addDescription("ignore-errors", "Processing", "Continue if a route could not be build");
+    oc.addDescription("ignore-errors", "Report", "Continue if a route could not be build");
 
     oc.doRegister("unsorted-input", new Option_Bool(false));
     oc.addSynonyme("unsorted-input", "unsorted");
@@ -100,6 +99,9 @@ ROFrame::fillOptions(OptionsCont& oc) {
     oc.doRegister("route-steps", 's', new Option_String("200", "TIME"));
     oc.addDescription("route-steps", "Processing", "Load routes for the next number of seconds ahead");
 
+    oc.doRegister("no-internal-links", new Option_Bool(false));
+    oc.addDescription("no-internal-links", "Processing", "Disable (junction) internal links");
+
     oc.doRegister("randomize-flows", new Option_Bool(false));
     oc.addDescription("randomize-flows", "Processing", "generate random departure times for flow input");
 
@@ -122,15 +124,30 @@ ROFrame::fillOptions(OptionsCont& oc) {
     oc.addSynonyme("weights.interpolate", "interpolate", true);
     oc.addDescription("weights.interpolate", "Processing", "Interpolate edge weights at interval boundaries");
 
+    oc.doRegister("weights.minor-penalty", new Option_Float(1.5));
+    oc.addDescription("weights.minor-penalty", "Processing", "Apply the given time penalty when computing routing costs for minor-link internal lanes");
+
     oc.doRegister("with-taz", new Option_Bool(false));
     oc.addDescription("with-taz", "Processing", "Use origin and destination zones (districts) for in- and output");
 
+    oc.doRegister("junction-taz", new Option_Bool(false));
+    oc.addDescription("junction-taz", "Input", "Initialize a TAZ for every junction to use attributes toJunction and fromJunction");
+
+    oc.doRegister("mapmatch.distance", new Option_Float(100));
+    oc.addDescription("mapmatch.distance", "Processing", "Maximum distance when mapping input coordinates (fromXY etc.) to the road network");
+
+    oc.doRegister("mapmatch.junctions", new Option_Bool(false));
+    oc.addDescription("mapmatch.junctions", "Processing", "Match positions to junctions instead of edges");
+
     oc.doRegister("bulk-routing", new Option_Bool(false));
     oc.addDescription("bulk-routing", "Processing", "Aggregate routing queries with the same origin");
 
     oc.doRegister("routing-threads", new Option_Integer(0));
     oc.addDescription("routing-threads", "Processing", "The number of parallel execution threads used for routing");
 
+    oc.doRegister("restriction-params", new Option_StringVector());
+    oc.addDescription("restriction-params", "Processing", "Comma separated list of param keys to compare for additional restrictions");
+
     // register defaults options
     oc.doRegister("departlane", new Option_String());
     oc.addDescription("departlane", "Defaults", "Assigns a default depart lane");
@@ -185,6 +202,4 @@ ROFrame::checkOptions(OptionsCont& oc) {
 }
 
 
-
 /****************************************************************************/
-
