@@ -1,39 +1,30 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RONet.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: RONet.cpp 20811 2016-05-30 11:05:58Z behrisch $
 ///
 // The router's network representation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <algorithm>
-#include <utils/vehicle/RouteCostCalculator.h>
+#include <utils/router/RouteCostCalculator.h>
 #include <utils/vehicle/SUMOVTypeParameter.h>
-#include <utils/vehicle/SUMOAbstractRouter.h>
+#include <utils/router/SUMOAbstractRouter.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/ToString.h>
@@ -41,22 +32,20 @@
 #include <utils/common/SUMOVehicleClass.h>
 #include <utils/iodevices/OutputDevice.h>
 #include "ROEdge.h"
+#include "ROLane.h"
 #include "RONode.h"
 #include "ROPerson.h"
 #include "RORoute.h"
 #include "RORouteDef.h"
 #include "ROVehicle.h"
+#include "ROAbstractEdgeBuilder.h"
 #include "RONet.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static member definitions
 // ===========================================================================
-RONet* RONet::myInstance = 0;
+RONet* RONet::myInstance = nullptr;
 
 
 // ===========================================================================
@@ -64,73 +53,90 @@ RONet* RONet::myInstance = 0;
 // ===========================================================================
 RONet*
 RONet::getInstance(void) {
-    if (myInstance != 0) {
+    if (myInstance != nullptr) {
         return myInstance;
     }
     throw ProcessError("A network was not yet constructed.");
 }
 
 
-RONet::RONet()
-    : myVehicleTypes(), myDefaultVTypeMayBeDeleted(true), myDefaultPedTypeMayBeDeleted(true),
-      myRoutesOutput(0), myRouteAlternativesOutput(0), myTypesOutput(0),
-      myReadRouteNo(0), myDiscardedRouteNo(0), myWrittenRouteNo(0),
-      myHavePermissions(false),
-      myNumInternalEdges(0),
-      myErrorHandler(OptionsCont::getOptions().exists("ignore-errors")
-                     && OptionsCont::getOptions().getBool("ignore-errors") ? MsgHandler::getWarningInstance() : MsgHandler::getErrorInstance()) {
-    if (myInstance != 0) {
+RONet::RONet() :
+    myVehicleTypes(), myDefaultVTypeMayBeDeleted(true),
+    myDefaultPedTypeMayBeDeleted(true), myDefaultBikeTypeMayBeDeleted(true),
+    myHaveActiveFlows(true),
+    myRoutesOutput(nullptr), myRouteAlternativesOutput(nullptr), myTypesOutput(nullptr),
+    myReadRouteNo(0), myDiscardedRouteNo(0), myWrittenRouteNo(0),
+    myHavePermissions(false),
+    myNumInternalEdges(0),
+    myErrorHandler(OptionsCont::getOptions().exists("ignore-errors")
+                   && OptionsCont::getOptions().getBool("ignore-errors") ? MsgHandler::getWarningInstance() : MsgHandler::getErrorInstance()),
+    myKeepVTypeDist(OptionsCont::getOptions().exists("keep-vtype-distributions")
+                    && OptionsCont::getOptions().getBool("keep-vtype-distributions")),
+    myHasBidiEdges(false) {
+    if (myInstance != nullptr) {
         throw ProcessError("A network was already constructed.");
     }
-    SUMOVTypeParameter* type = new SUMOVTypeParameter(DEFAULT_VTYPE_ID, SVC_IGNORING);
+    SUMOVTypeParameter* type = new SUMOVTypeParameter(DEFAULT_VTYPE_ID, SVC_PASSENGER);
     type->onlyReferenced = true;
     myVehicleTypes.add(type->id, type);
+
     SUMOVTypeParameter* defPedType = new SUMOVTypeParameter(DEFAULT_PEDTYPE_ID, SVC_PEDESTRIAN);
     defPedType->onlyReferenced = true;
-    defPedType->setParameter |= VTYPEPARS_VEHICLECLASS_SET;
+    defPedType->parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
     myVehicleTypes.add(defPedType->id, defPedType);
+
+    SUMOVTypeParameter* defBikeType = new SUMOVTypeParameter(DEFAULT_BIKETYPE_ID, SVC_BICYCLE);
+    defBikeType->onlyReferenced = true;
+    defBikeType->parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
+    myVehicleTypes.add(defBikeType->id, defBikeType);
+
+    SUMOVTypeParameter* defTaxiType = new SUMOVTypeParameter(DEFAULT_TAXITYPE_ID, SVC_TAXI);
+    defTaxiType->onlyReferenced = true;
+    defTaxiType->parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
+    myVehicleTypes.add(defTaxiType->id, defTaxiType);
+
     myInstance = this;
 }
 
 
 RONet::~RONet() {
     for (RoutablesMap::iterator routables = myRoutables.begin(); routables != myRoutables.end(); ++routables) {
-        for (std::deque<RORoutable*>::iterator r = routables->second.begin(); r != routables->second.end(); ++r) {
-            const ROVehicle* const veh = dynamic_cast<const ROVehicle*>(*r);
+        for (RORoutable* const r : routables->second) {
+            const ROVehicle* const veh = dynamic_cast<const ROVehicle*>(r);
             // delete routes and the vehicle
-            if (veh != 0 && veh->getRouteDefinition()->getID()[0] == '!') {
-                if (!myRoutes.erase(veh->getRouteDefinition()->getID())) {
+            if (veh != nullptr && veh->getRouteDefinition()->getID()[0] == '!') {
+                if (!myRoutes.remove(veh->getRouteDefinition()->getID())) {
                     delete veh->getRouteDefinition();
                 }
             }
-            delete *r;
+            delete r;
         }
     }
-    for (std::map<std::string, SUMOVehicleParameter::Stop*>::iterator it = myBusStops.begin(); it != myBusStops.end(); ++it) {
-        delete it->second;
+    for (const RORoutable* const r : myPTVehicles) {
+        const ROVehicle* const veh = dynamic_cast<const ROVehicle*>(r);
+        // delete routes and the vehicle
+        if (veh != nullptr && veh->getRouteDefinition()->getID()[0] == '!') {
+            if (!myRoutes.remove(veh->getRouteDefinition()->getID())) {
+                delete veh->getRouteDefinition();
+            }
+        }
+        delete r;
     }
-    for (std::map<std::string, SUMOVehicleParameter::Stop*>::iterator it = myContainerStops.begin(); it != myContainerStops.end(); ++it) {
-        delete it->second;
-    }
-    myNodes.clear();
-    myEdges.clear();
-    myVehicleTypes.clear();
-    myRoutes.clear();
     myRoutables.clear();
 }
 
 
 void
-RONet::addRestriction(const std::string& id, const SUMOVehicleClass svc, const SUMOReal speed) {
+RONet::addRestriction(const std::string& id, const SUMOVehicleClass svc, const double speed) {
     myRestrictions[id][svc] = speed;
 }
 
 
-const std::map<SUMOVehicleClass, SUMOReal>*
+const std::map<SUMOVehicleClass, double>*
 RONet::getRestrictions(const std::string& id) const {
-    std::map<std::string, std::map<SUMOVehicleClass, SUMOReal> >::const_iterator i = myRestrictions.find(id);
+    std::map<std::string, std::map<SUMOVehicleClass, double> >::const_iterator i = myRestrictions.find(id);
     if (i == myRestrictions.end()) {
-        return 0;
+        return nullptr;
     }
     return &i->second;
 }
@@ -143,7 +149,7 @@ RONet::addEdge(ROEdge* edge) {
         delete edge;
         return false;
     }
-    if (edge->getFunc() == ROEdge::ET_INTERNAL) {
+    if (edge->isInternal()) {
         myNumInternalEdges += 1;
     }
     return true;
@@ -158,10 +164,12 @@ RONet::addDistrict(const std::string id, ROEdge* source, ROEdge* sink) {
         delete sink;
         return false;
     }
-    sink->setFunc(ROEdge::ET_DISTRICT);
+    sink->setFunction(SumoXMLEdgeFunc::CONNECTOR);
     addEdge(sink);
-    source->setFunc(ROEdge::ET_DISTRICT);
+    source->setFunction(SumoXMLEdgeFunc::CONNECTOR);
     addEdge(source);
+    sink->setOtherTazConnector(source);
+    source->setOtherTazConnector(sink);
     myDistricts[id] = std::make_pair(std::vector<std::string>(), std::vector<std::string>());
     return true;
 }
@@ -174,7 +182,7 @@ RONet::addDistrictEdge(const std::string tazID, const std::string edgeID, const
         return false;
     }
     ROEdge* edge = getEdge(edgeID);
-    if (edge == 0) {
+    if (edge == nullptr) {
         WRITE_ERROR("The edge '" + edgeID + "' for TAZ '" + tazID + "' is unknown.");
         return false;
     }
@@ -189,6 +197,51 @@ RONet::addDistrictEdge(const std::string tazID, const std::string edgeID, const
 }
 
 
+void
+RONet::addJunctionTaz(ROAbstractEdgeBuilder& eb) {
+    for (auto item : myNodes) {
+        const std::string tazID = item.first;
+        if (myDistricts.count(tazID) != 0) {
+            WRITE_WARNINGF("A TAZ with id '%' already exists. Not building junction TAZ.", tazID);
+            continue;
+        }
+        const std::string sourceID = tazID + "-source";
+        const std::string sinkID = tazID + "-sink";
+        // sink must be addd before source
+        ROEdge* sink = eb.buildEdge(sinkID, nullptr, nullptr, 0);
+        ROEdge* source = eb.buildEdge(sourceID, nullptr, nullptr, 0);
+        sink->setOtherTazConnector(source);
+        source->setOtherTazConnector(sink);
+        addDistrict(tazID, source, sink);
+        auto& district = myDistricts[tazID];
+        const RONode* junction = item.second;
+        for (const ROEdge* edge : junction->getIncoming()) {
+            if (!edge->isInternal()) {
+                const_cast<ROEdge*>(edge)->addSuccessor(sink);
+                district.second.push_back(edge->getID());
+            }
+        }
+        for (const ROEdge* edge : junction->getOutgoing()) {
+            if (!edge->isInternal()) {
+                source->addSuccessor(const_cast<ROEdge*>(edge));
+                district.first.push_back(edge->getID());
+            }
+        }
+    }
+}
+
+void
+RONet::setBidiEdges(const std::map<ROEdge*, std::string>& bidiMap) {
+    for (const auto& item : bidiMap) {
+        ROEdge* bidi = myEdges.get(item.second);
+        if (bidi == nullptr) {
+            WRITE_ERROR("The bidi edge '" + item.second + "' is not known.");
+        }
+        item.first->setBidiEdge(bidi);
+        myHasBidiEdges = true;
+    }
+}
+
 void
 RONet::addNode(RONode* node) {
     if (!myNodes.add(node->getID(), node)) {
@@ -199,24 +252,11 @@ RONet::addNode(RONode* node) {
 
 
 void
-RONet::addBusStop(const std::string& id, SUMOVehicleParameter::Stop* stop) {
-    std::map<std::string, SUMOVehicleParameter::Stop*>::const_iterator it = myBusStops.find(id);
-    if (it != myBusStops.end()) {
-        WRITE_ERROR("The bus stop '" + id + "' occurs at least twice.");
+RONet::addStoppingPlace(const std::string& id, const SumoXMLTag category, SUMOVehicleParameter::Stop* stop) {
+    if (!myStoppingPlaces[category == SUMO_TAG_TRAIN_STOP ? SUMO_TAG_BUS_STOP : category].add(id, stop)) {
+        WRITE_ERROR("The " + toString(category) + " '" + id + "' occurs at least twice.");
         delete stop;
     }
-    myBusStops[id] = stop;
-}
-
-
-void
-RONet::addContainerStop(const std::string& id, SUMOVehicleParameter::Stop* stop) {
-    std::map<std::string, SUMOVehicleParameter::Stop*>::const_iterator it = myContainerStops.find(id);
-    if (it != myContainerStops.end()) {
-        WRITE_ERROR("The container stop '" + id + "' occurs at least twice.");
-        delete stop;
-    }
-    myContainerStops[id] = stop;
 }
 
 
@@ -227,19 +267,41 @@ RONet::addRouteDef(RORouteDef* def) {
 
 
 void
-RONet::openOutput(const OptionsCont& options, const std::string altFilename) {
+RONet::openOutput(const OptionsCont& options) {
     if (options.isSet("output-file") && options.getString("output-file") != "") {
-        OutputDevice::createDeviceByOption("output-file", "routes", "routes_file.xsd");
-        myRoutesOutput = &OutputDevice::getDeviceByOption("output-file");
+        myRoutesOutput = &OutputDevice::getDevice(options.getString("output-file"));
+        myRoutesOutput->writeHeader<ROEdge>(SUMO_TAG_ROUTES);
+        myRoutesOutput->writeAttr("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance").writeAttr("xsi:noNamespaceSchemaLocation", "http://sumo.dlr.de/xsd/routes_file.xsd");
     }
-    if (altFilename != "") {
-        myRouteAlternativesOutput = &OutputDevice::getDevice(altFilename);
+    if (options.exists("alternatives-output") && options.isSet("alternatives-output")
+            && !(options.exists("write-trips") && options.getBool("write-trips"))) {
+        myRouteAlternativesOutput = &OutputDevice::getDevice(options.getString("alternatives-output"));
         myRouteAlternativesOutput->writeHeader<ROEdge>(SUMO_TAG_ROUTES);
         myRouteAlternativesOutput->writeAttr("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance").writeAttr("xsi:noNamespaceSchemaLocation", "http://sumo.dlr.de/xsd/routes_file.xsd");
     }
-    if (options.isSet("vtype-output") && options.getString("vtype-output") != "") {
-        OutputDevice::createDeviceByOption("vtype-output", "routes", "routes_file.xsd");
-        myTypesOutput = &OutputDevice::getDeviceByOption("vtype-output");
+    if (options.isSet("vtype-output")) {
+        myTypesOutput = &OutputDevice::getDevice(options.getString("vtype-output"));
+        myTypesOutput->writeHeader<ROEdge>(SUMO_TAG_ROUTES);
+        myTypesOutput->writeAttr("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance").writeAttr("xsi:noNamespaceSchemaLocation", "http://sumo.dlr.de/xsd/routes_file.xsd");
+    }
+}
+
+
+void
+RONet::writeIntermodal(const OptionsCont& options, ROIntermodalRouter& router) const {
+    if (options.exists("intermodal-network-output") && options.isSet("intermodal-network-output")) {
+        OutputDevice::createDeviceByOption("intermodal-network-output", "intermodal");
+        router.writeNetwork(OutputDevice::getDevice(options.getString("intermodal-network-output")));
+    }
+    if (options.exists("intermodal-weight-output") && options.isSet("intermodal-weight-output")) {
+        OutputDevice::createDeviceByOption("intermodal-weight-output", "weights", "meandata_file.xsd");
+        OutputDevice& dev = OutputDevice::getDeviceByOption("intermodal-weight-output");
+        dev.openTag(SUMO_TAG_INTERVAL);
+        dev.writeAttr(SUMO_ATTR_ID, "intermodalweights");
+        dev.writeAttr(SUMO_ATTR_BEGIN, 0);
+        dev.writeAttr(SUMO_ATTR_END, SUMOTime_MAX);
+        router.writeWeights(dev);
+        dev.closeTag();
     }
 }
 
@@ -247,15 +309,15 @@ RONet::openOutput(const OptionsCont& options, const std::string altFilename) {
 void
 RONet::cleanup() {
     // end writing
-    if (myRoutesOutput != 0) {
+    if (myRoutesOutput != nullptr) {
         myRoutesOutput->close();
     }
     // only if opened
-    if (myRouteAlternativesOutput != 0) {
+    if (myRouteAlternativesOutput != nullptr) {
         myRouteAlternativesOutput->close();
     }
     // only if opened
-    if (myTypesOutput != 0) {
+    if (myTypesOutput != nullptr) {
         myTypesOutput->close();
     }
     RouteCostCalculator<RORoute, ROEdge, ROVehicle>::cleanup();
@@ -278,7 +340,10 @@ RONet::getVehicleTypeSecure(const std::string& id) {
     if (id == DEFAULT_PEDTYPE_ID) {
         myDefaultPedTypeMayBeDeleted = false;
     }
-    if (type != 0) {
+    if (id == DEFAULT_BIKETYPE_ID) {
+        myDefaultBikeTypeMayBeDeleted = false;
+    }
+    if (type != nullptr) {
         return type;
     }
     VTypeDistDictType::iterator it2 = myVTypeDistDict.find(id);
@@ -347,8 +412,15 @@ bool
 RONet::addVehicle(const std::string& id, ROVehicle* veh) {
     if (myVehIDs.find(id) == myVehIDs.end()) {
         myVehIDs.insert(id);
+        if (veh->isPublicTransport()) {
+            if (!veh->isPartOfFlow()) {
+                myPTVehicles.push_back(veh);
+            }
+            if (OptionsCont::getOptions().exists("ptline-routing") && !OptionsCont::getOptions().getBool("ptline-routing")) {
+                return true;
+            }
+        }
         myRoutables[veh->getDepart()].push_back(veh);
-        myReadRouteNo++;
         return true;
     }
     WRITE_ERROR("Another vehicle with the id '" + id + "' exists.");
@@ -356,6 +428,12 @@ RONet::addVehicle(const std::string& id, ROVehicle* veh) {
 }
 
 
+bool
+RONet::knowsVehicle(const std::string& id) {
+    return myVehIDs.find(id) != myVehIDs.end();
+}
+
+
 bool
 RONet::addFlow(SUMOVehicleParameter* flow, const bool randomize) {
     if (randomize) {
@@ -390,14 +468,16 @@ RONet::addContainer(const SUMOTime depart, const std::string desc) {
 
 void
 RONet::checkFlows(SUMOTime time, MsgHandler* errorHandler) {
-    std::vector<std::string> toRemove;
-    for (NamedObjectCont<SUMOVehicleParameter*>::IDMap::const_iterator i = myFlows.getMyMap().begin(); i != myFlows.getMyMap().end(); ++i) {
-        SUMOVehicleParameter* pars = i->second;
+    myHaveActiveFlows = false;
+    for (const auto& i : myFlows) {
+        SUMOVehicleParameter* pars = i.second;
         if (pars->repetitionProbability > 0) {
+            if (pars->repetitionEnd > pars->depart) {
+                myHaveActiveFlows = true;
+            }
             const SUMOTime origDepart = pars->depart;
             while (pars->depart < time) {
                 if (pars->repetitionEnd <= pars->depart) {
-                    toRemove.push_back(i->first);
                     break;
                 }
                 // only call rand if all other conditions are met
@@ -413,9 +493,7 @@ RONet::checkFlows(SUMOTime time, MsgHandler* errorHandler) {
                     pars->repetitionsDone++;
                     // try to build the vehicle
                     SUMOVTypeParameter* type = getVehicleTypeSecure(pars->vtypeid);
-                    if (type == 0) {
-                        type = getVehicleTypeSecure(DEFAULT_VTYPE_ID);
-                    } else {
+                    if (!myKeepVTypeDist) {
                         // fix the type id in case we used a distribution
                         newPars->vtypeid = type->id;
                     }
@@ -429,6 +507,7 @@ RONet::checkFlows(SUMOTime time, MsgHandler* errorHandler) {
             }
         } else {
             while (pars->repetitionsDone < pars->repetitionNumber) {
+                myHaveActiveFlows = true;
                 SUMOTime depart = static_cast<SUMOTime>(pars->depart + pars->repetitionsDone * pars->repetitionOffset);
                 if (myDepartures.find(pars->id) != myDepartures.end()) {
                     depart = myDepartures[pars->id].back();
@@ -450,7 +529,7 @@ RONet::checkFlows(SUMOTime time, MsgHandler* errorHandler) {
                 pars->repetitionsDone++;
                 // try to build the vehicle
                 SUMOVTypeParameter* type = getVehicleTypeSecure(pars->vtypeid);
-                if (type == 0) {
+                if (type == nullptr) {
                     type = getVehicleTypeSecure(DEFAULT_VTYPE_ID);
                 } else {
                     // fix the type id in case we used a distribution
@@ -462,26 +541,19 @@ RONet::checkFlows(SUMOTime time, MsgHandler* errorHandler) {
                 addVehicle(newPars->id, veh);
                 delete newPars;
             }
-            if (pars->repetitionsDone == pars->repetitionNumber) {
-                toRemove.push_back(i->first);
-            }
         }
     }
-    for (std::vector<std::string>::const_iterator i = toRemove.begin(); i != toRemove.end(); ++i) {
-        myFlows.erase(*i);
-    }
 }
 
 
 void
 RONet::createBulkRouteRequests(const RORouterProvider& provider, const SUMOTime time, const bool removeLoops) {
-    std::map<const unsigned int, std::vector<RORoutable*> > bulkVehs;
+    std::map<const int, std::vector<RORoutable*> > bulkVehs;
     for (RoutablesMap::const_iterator i = myRoutables.begin(); i != myRoutables.end(); ++i) {
         if (i->first >= time) {
             break;
         }
-        for (std::deque<RORoutable*>::const_iterator r = i->second.begin(); r != i->second.end(); ++r) {
-            RORoutable* const routable = *r;
+        for (RORoutable* const routable : i->second) {
             const ROEdge* const depEdge = routable->getDepartEdge();
             bulkVehs[depEdge->getNumericalID()].push_back(routable);
             RORoutable* const first = bulkVehs[depEdge->getNumericalID()].front();
@@ -494,7 +566,7 @@ RONet::createBulkRouteRequests(const RORouterProvider& provider, const SUMOTime
         }
     }
     int workerIndex = 0;
-    for (std::map<const unsigned int, std::vector<RORoutable*> >::const_iterator i = bulkVehs.begin(); i != bulkVehs.end(); ++i) {
+    for (std::map<const int, std::vector<RORoutable*> >::const_iterator i = bulkVehs.begin(); i != bulkVehs.end(); ++i) {
 #ifdef HAVE_FOX
         if (myThreadPool.size() > 0) {
             RORoutable* const first = i->second.front();
@@ -513,9 +585,9 @@ RONet::createBulkRouteRequests(const RORouterProvider& provider, const SUMOTime
 #endif
         for (std::vector<RORoutable*>::const_iterator j = i->second.begin(); j != i->second.end(); ++j) {
             (*j)->computeRoute(provider, removeLoops, myErrorHandler);
-            provider.getVehicleRouter().setBulkMode(true);
+            provider.getVehicleRouter((*j)->getVClass()).setBulkMode(true);
         }
-        provider.getVehicleRouter().setBulkMode(false);
+        provider.getVehicleRouter(SVC_IGNORING).setBulkMode(false);
     }
 }
 
@@ -525,7 +597,9 @@ RONet::saveAndRemoveRoutesUntil(OptionsCont& options, const RORouterProvider& pr
                                 SUMOTime time) {
     MsgHandler* mh = (options.getBool("ignore-errors") ?
                       MsgHandler::getWarningInstance() : MsgHandler::getErrorInstance());
-    checkFlows(time, mh);
+    if (myHaveActiveFlows) {
+        checkFlows(time, mh);
+    }
     SUMOTime lastTime = -1;
     const bool removeLoops = options.getBool("remove-loops");
     const int maxNumThreads = options.getInt("routing-threads");
@@ -542,17 +616,23 @@ RONet::saveAndRemoveRoutesUntil(OptionsCont& options, const RORouterProvider& pr
                 if (i->first >= time) {
                     break;
                 }
-                for (std::deque<RORoutable*>::const_iterator r = i->second.begin(); r != i->second.end(); ++r) {
-                    RORoutable* const routable = *r;
+                for (RORoutable* const routable : i->second) {
 #ifdef HAVE_FOX
                     // add task
                     if (maxNumThreads > 0) {
-                        // add thread if necessary
                         const int numThreads = (int)myThreadPool.size();
-                        if (numThreads < maxNumThreads && myThreadPool.isFull()) {
+                        if (numThreads == 0) {
+                            // This is the very first routing. Since at least the CHRouter needs initialization
+                            // before it gets cloned, we do not do this in parallel
+                            routable->computeRoute(provider, removeLoops, myErrorHandler);
                             new WorkerThread(myThreadPool, provider);
+                        } else {
+                            // add thread if necessary
+                            if (numThreads < maxNumThreads && myThreadPool.isFull()) {
+                                new WorkerThread(myThreadPool, provider);
+                            }
+                            myThreadPool.add(new RoutingTask(routable, removeLoops, myErrorHandler));
                         }
-                        myThreadPool.add(new RoutingTask(routable, removeLoops, myErrorHandler));
                         continue;
                     }
 #endif
@@ -578,40 +658,39 @@ RONet::saveAndRemoveRoutesUntil(OptionsCont& options, const RORouterProvider& pr
         }
         const SUMOTime minTime = MIN2(routableTime, containerTime);
         if (routableTime == minTime) {
-            const RORoutable* const r = routables->second.front();
             // check whether to print the output
             if (lastTime != routableTime && lastTime != -1) {
                 // report writing progress
-                if (options.getInt("stats-period") >= 0 && ((int) routableTime % options.getInt("stats-period")) == 0) {
-                    WRITE_MESSAGE("Read: " + toString(myReadRouteNo) + ",  Discarded: " + toString(myDiscardedRouteNo) + ",  Written: " + toString(myWrittenRouteNo));
+                if (options.getInt("stats-period") >= 0 && ((int)routableTime % options.getInt("stats-period")) == 0) {
+                    WRITE_MESSAGE("Read: " + toString(myVehIDs.size()) + ",  Discarded: " + toString(myDiscardedRouteNo) + ",  Written: " + toString(myWrittenRouteNo));
                 }
             }
             lastTime = routableTime;
-
-            // ok, compute the route (try it)
-            if (r->getRoutingSuccess()) {
-                // write the route
-                r->write(*myRoutesOutput, myRouteAlternativesOutput, myTypesOutput, options);
-                myWrittenRouteNo++;
-            } else {
-                myDiscardedRouteNo++;
-            }
-            const ROVehicle* const veh = dynamic_cast<const ROVehicle*>(r);
-            // delete routes and the vehicle
-            if (veh != 0 && veh->getRouteDefinition()->getID()[0] == '!') {
-                if (!myRoutes.erase(veh->getRouteDefinition()->getID())) {
-                    delete veh->getRouteDefinition();
+            for (const RORoutable* const r : routables->second) {
+                // ok, check whether it has been routed
+                if (r->getRoutingSuccess()) {
+                    // write the route
+                    r->write(*myRoutesOutput, myRouteAlternativesOutput, myTypesOutput, options);
+                    myWrittenRouteNo++;
+                } else {
+                    myDiscardedRouteNo++;
+                }
+                // delete routes and the vehicle
+                if (!r->isPublicTransport() || r->isPartOfFlow()) {
+                    const ROVehicle* const veh = dynamic_cast<const ROVehicle*>(r);
+                    if (veh != nullptr && veh->getRouteDefinition()->getID()[0] == '!') {
+                        if (!myRoutes.remove(veh->getRouteDefinition()->getID())) {
+                            delete veh->getRouteDefinition();
+                        }
+                    }
+                    delete r;
                 }
             }
-            routables->second.pop_front();
-            if (routables->second.empty()) {
-                myRoutables.erase(routables);
-            }
-            delete r;
+            myRoutables.erase(routables);
         }
         if (containerTime == minTime) {
             myRoutesOutput->writePreformattedTag(container->second);
-            if (myRouteAlternativesOutput != 0) {
+            if (myRouteAlternativesOutput != nullptr) {
                 myRouteAlternativesOutput->writePreformattedTag(container->second);
             }
             myContainers.erase(container);
@@ -623,12 +702,12 @@ RONet::saveAndRemoveRoutesUntil(OptionsCont& options, const RORouterProvider& pr
 
 bool
 RONet::furtherStored() {
-    return myRoutables.size() > 0 || myFlows.size() > 0 || myContainers.size() > 0;
+    return myRoutables.size() > 0 || (myFlows.size() > 0 && myHaveActiveFlows) || myContainers.size() > 0;
 }
 
 
-size_t
-RONet::getEdgeNo() const {
+int
+RONet::getEdgeNumber() const {
     return myEdges.size();
 }
 
@@ -639,33 +718,43 @@ RONet::getInternalEdgeNumber() const {
 }
 
 
-const std::map<std::string, ROEdge*>&
-RONet::getEdgeMap() const {
-    return myEdges.getMyMap();
-}
-
-
 void
 RONet::adaptIntermodalRouter(ROIntermodalRouter& router) {
-    // add access to all public transport stops
-    for (std::map<std::string, SUMOVehicleParameter::Stop*>::const_iterator i = myInstance->myBusStops.begin(); i != myInstance->myBusStops.end(); ++i) {
-        router.addAccess(i->first, myInstance->getEdgeForLaneID(i->second->lane), i->second->endPos);
-        for (std::multimap<std::string, SUMOReal>::const_iterator a = i->second->accessPos.begin(); a != i->second->accessPos.end(); ++a) {
-            router.addAccess(i->first, myInstance->getEdgeForLaneID(a->first), a->second);
+    double taxiWait = STEPS2TIME(string2time(OptionsCont::getOptions().getString("persontrip.taxi.waiting-time")));
+    for (const auto& stopType : myInstance->myStoppingPlaces) {
+        // add access to all stopping places
+        const SumoXMLTag element = stopType.first;
+        for (const auto& stop : stopType.second) {
+            router.getNetwork()->addAccess(stop.first, myInstance->getEdgeForLaneID(stop.second->lane), (stop.second->startPos + stop.second->endPos) / 2., 0., element, false, taxiWait);
+            // add access to all public transport stops
+            if (element == SUMO_TAG_BUS_STOP) {
+                for (const auto& a : stop.second->accessPos) {
+                    router.getNetwork()->addAccess(stop.first, myInstance->getEdgeForLaneID(std::get<0>(a)), std::get<1>(a), std::get<2>(a), SUMO_TAG_BUS_STOP, true, taxiWait);
+                }
+            }
         }
     }
     // fill the public transport router with pre-parsed public transport lines
-    for (std::map<std::string, SUMOVehicleParameter*>::const_iterator i = myInstance->myFlows.getMyMap().begin(); i != myInstance->myFlows.getMyMap().end(); ++i) {
-        if (i->second->line != "") {
-            router.addSchedule(*i->second);
+    for (const auto& i : myInstance->myFlows) {
+        if (i.second->line != "") {
+            const RORouteDef* const route = myInstance->getRouteDef(i.second->routeid);
+            const std::vector<SUMOVehicleParameter::Stop>* addStops = nullptr;
+            if (route != nullptr && route->getFirstRoute() != nullptr) {
+                addStops = &route->getFirstRoute()->getStops();
+            }
+            router.getNetwork()->addSchedule(*i.second, addStops);
         }
     }
-    for (RoutablesMap::const_iterator i = myInstance->myRoutables.begin(); i != myInstance->myRoutables.end(); ++i) {
-        for (std::deque<RORoutable*>::const_iterator r = i->second.begin(); r != i->second.end(); ++r) {
-            const ROVehicle* const veh = dynamic_cast<ROVehicle*>(*r);
-            // add single vehicles with line attribute which are not part of a flow
-            if (veh != 0 && veh->getParameter().line != "" && veh->getParameter().repetitionNumber < 0) {
-                router.addSchedule(veh->getParameter());
+    for (const RORoutable* const veh : myInstance->myPTVehicles) {
+        // add single vehicles with line attribute which are not part of a flow
+        // no need to add route stops here, they have been added to the vehicle before
+        router.getNetwork()->addSchedule(veh->getParameter());
+    }
+    // add access to transfer from walking to taxi-use
+    if ((router.getCarWalkTransfer() & ROIntermodalRouter::Network::TAXI_PICKUP_ANYWHERE) != 0) {
+        for (const ROEdge* edge : ROEdge::getAllEdges()) {
+            if ((edge->getPermissions() & SVC_PEDESTRIAN) != 0 && (edge->getPermissions() & SVC_TAXI) != 0) {
+                router.getNetwork()->addCarAccess(edge, SVC_TAXI, taxiWait);
             }
         }
     }
@@ -683,6 +772,40 @@ RONet::setPermissionsFound() {
     myHavePermissions = true;
 }
 
+bool
+RONet::hasLoadedEffort() const {
+    for (const auto& item : myEdges) {
+        if (item.second->hasStoredEffort()) {
+            return true;
+        }
+    }
+    return false;
+}
+
+const std::string
+RONet::getStoppingPlaceName(const std::string& id) const {
+    for (const auto& mapItem : myStoppingPlaces) {
+        SUMOVehicleParameter::Stop* stop = mapItem.second.get(id);
+        if (stop != nullptr) {
+            // see RONetHandler::parseStoppingPlace
+            return stop->busstop;
+        }
+    }
+    return "";
+}
+
+const std::string
+RONet::getStoppingPlaceElement(const std::string& id) const {
+    for (const auto& mapItem : myStoppingPlaces) {
+        SUMOVehicleParameter::Stop* stop = mapItem.second.get(id);
+        if (stop != nullptr) {
+            // see RONetHandler::parseStoppingPlace
+            return stop->actType;
+        }
+    }
+    return toString(SUMO_TAG_BUS_STOP);
+}
+
 
 #ifdef HAVE_FOX
 // ---------------------------------------------------------------------------
@@ -696,4 +819,3 @@ RONet::RoutingTask::run(FXWorkerThread* context) {
 
 
 /****************************************************************************/
-
