@@ -1,45 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RONet.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @author  Yun-Pang Floetteroed
 /// @date    Sept 2002
-/// @version $Id: RONet.h 21131 2016-07-08 07:59:22Z behrisch $
 ///
 // The router's network representation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef RONet_h
-#define RONet_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <queue>
 #include <vector>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/NamedObjectCont.h>
-#include <utils/common/RandomDistributor.h>
-#include <utils/vehicle/PedestrianRouter.h>
-#include <utils/vehicle/SUMOAbstractRouter.h>
+#include <utils/distribution/RandomDistributor.h>
 #include <utils/vehicle/SUMOVehicleParameter.h>
 #include <utils/vehicle/SUMOVTypeParameter.h>
 #include "ROLane.h"
@@ -55,11 +42,10 @@
 // class declarations
 // ===========================================================================
 class ROEdge;
-class ROLane;
 class RONode;
 class ROPerson;
-class RORoutable;
 class ROVehicle;
+class ROAbstractEdgeBuilder;
 class OptionsCont;
 class OutputDevice;
 
@@ -76,7 +62,7 @@ class OutputDevice;
 class RONet {
 public:
 
-    typedef std::map<const SUMOTime, std::deque<RORoutable*> > RoutablesMap;
+    typedef std::map<const SUMOTime, std::vector<RORoutable*> > RoutablesMap;
 
     /// @brief Constructor
     RONet();
@@ -97,7 +83,7 @@ public:
      * @param[in] svc The vehicle class the restriction refers to
      * @param[in] speed The restricted speed
      */
-    void addRestriction(const std::string& id, const SUMOVehicleClass svc, const SUMOReal speed);
+    void addRestriction(const std::string& id, const SUMOVehicleClass svc, const double speed);
 
 
     /** @brief Returns the restrictions for an edge type
@@ -105,7 +91,7 @@ public:
      * @param[in] id The id of the type
      * @return The mapping of vehicle classes to maximum speeds
      */
-    const std::map<SUMOVehicleClass, SUMOReal>* getRestrictions(const std::string& id) const;
+    const std::map<SUMOVehicleClass, double>* getRestrictions(const std::string& id) const;
 
 
     /// @name Insertion and retrieval of graph parts
@@ -146,6 +132,12 @@ public:
      */
     bool addDistrictEdge(const std::string tazID, const std::string edgeID, const bool isSource);
 
+    /// @brief add a taz for every junction unless a taz with the same id already exists
+    void addJunctionTaz(ROAbstractEdgeBuilder& eb);
+
+    /// @brief add a taz for every junction unless a taz with the same id already exists
+    void setBidiEdges(const std::map<ROEdge*, std::string>& bidiMap);
+
     /** @brief Retrieves all TAZ (districts) from the network
      *
      * @return The map of all districts
@@ -199,54 +191,36 @@ public:
     }
 
 
-    /* @brief Adds a read bus stop to the network
+    /* @brief Adds a read stopping place (bus, train, container, parking)  to the network
      *
-     * If the bus stop is already known (another one with the same id exists),
+     * If the place is already known (another one with the same id and category exists),
      *  an error is generated and given to msg-error-handler. The stop
      *  is deleted in this case
      *
-     * @param[in] node The stop to add
+     * @param[in] id The name of the stop to add
+     * @param[in] category The type of stop
+     * @param[in] stop The detailed stop description
      */
-    void addBusStop(const std::string& id, SUMOVehicleParameter::Stop* stop);
+    void addStoppingPlace(const std::string& id, const SumoXMLTag category, SUMOVehicleParameter::Stop* stop);
 
-
-    /* @brief Adds a read container stop to the network
+    /** @brief Retrieves a stopping place from the network
      *
-     * If the container stop is already known (another one with the same id exists),
-     *  an error is generated and given to msg-error-handler. The stop
-     *  is deleted in this case
-     *
-     * @param[in] node The stop to add
-     */
-    void addContainerStop(const std::string& id, SUMOVehicleParameter::Stop* stop);
-
-
-    /** @brief Retrieves a bus stop from the network
-     *
-     * @param[in] name The name of the stop to retrieve
+     * @param[in] id The name of the stop to retrieve
+     * @param[in] category The type of stop
      * @return The named stop if known, otherwise 0
      */
-    const SUMOVehicleParameter::Stop* getBusStop(const std::string& id) const {
-        std::map<std::string, SUMOVehicleParameter::Stop*>::const_iterator it = myBusStops.find(id);
-        if (it == myBusStops.end()) {
-            return 0;
+    const SUMOVehicleParameter::Stop* getStoppingPlace(const std::string& id, const SumoXMLTag category) const {
+        if (myStoppingPlaces.count(category) > 0) {
+            return myStoppingPlaces.find(category)->second.get(id);
         }
-        return it->second;
+        return 0;
     }
 
+    /// @brief return the name for the given stopping place id
+    const std::string getStoppingPlaceName(const std::string& id) const;
 
-    /** @brief Retrieves a container stop from the network
-     *
-     * @param[in] name The name of the stop to retrieve
-     * @return The named stop if known, otherwise 0
-     */
-    const SUMOVehicleParameter::Stop* getContainerStop(const std::string& id) const {
-        std::map<std::string, SUMOVehicleParameter::Stop*>::const_iterator it = myContainerStops.find(id);
-        if (it == myContainerStops.end()) {
-            return 0;
-        }
-        return it->second;
-    }
+    /// @brief return the element name for the given stopping place id
+    const std::string getStoppingPlaceElement(const std::string& id) const;
     //@}
 
 
@@ -342,6 +316,8 @@ public:
      */
     virtual bool addVehicle(const std::string& id, ROVehicle* veh);
 
+    /// @brief returns whether a vehicle with the given id was already loaded
+    bool knowsVehicle(const std::string& id);
 
     /* @brief Adds a flow of vehicles to the network
      *
@@ -391,23 +367,26 @@ public:
 
 
     /// Returns the information whether further vehicles, persons or containers are stored
-    virtual bool furtherStored();
+    bool furtherStored();
     //@}
 
 
-
-
-
     /** @brief Opens the output for computed routes
      *
-     * If the second parameter is set, a second file for route alternatives
-     *  will be opened.
      * If one of the file outputs can not be build, an IOError is thrown.
      *
-     * @param[in] options The options to be asked for "output-file" and "vtype-output"
-     * @param[in] altFilename The name of the file for writing alternatives, "" means no alternatives
+     * @param[in] options The options to be asked for "output-file", "alternatives-output" and "vtype-output"
      */
-    void openOutput(const OptionsCont& options, const std::string altFilename = "");
+    void openOutput(const OptionsCont& options);
+
+
+    /** @brief Writes the intermodal network and weights if requested
+     *
+     * If one of the file outputs can not be build, an IOError is thrown.
+     *
+     * @param[in] options The options to be asked for "intermodal-network-output" and "intermodal-weight-output"
+     */
+    void writeIntermodal(const OptionsCont& options, ROIntermodalRouter& router) const;
 
 
     /** @brief closes the file output for computed routes and deletes associated threads if necessary */
@@ -415,12 +394,14 @@ public:
 
 
     /// Returns the total number of edges the network contains including internal edges
-    size_t getEdgeNo() const;
+    int getEdgeNumber() const;
 
     /// Returns the number of internal edges the network contains
     int getInternalEdgeNumber() const;
 
-    const std::map<std::string, ROEdge*>& getEdgeMap() const;
+    const NamedObjectCont<ROEdge*>& getEdgeMap() const {
+        return myEdges;
+    }
 
     static void adaptIntermodalRouter(ROIntermodalRouter& router);
 
@@ -428,6 +409,14 @@ public:
 
     void setPermissionsFound();
 
+    /// @brief return whether the network contains bidirectional rail edges
+    bool hasBidiEdges() const {
+        return myHasBidiEdges;
+    }
+
+    /// @brief whether efforts were loaded from file
+    bool hasLoadedEffort() const;
+
     OutputDevice* getRouteOutput(const bool alternative = false) {
         if (alternative) {
             return myRouteAlternativesOutput;
@@ -454,7 +443,7 @@ public:
     public:
         BulkmodeTask(const bool value) : myValue(value) {}
         void run(FXWorkerThread* context) {
-            static_cast<WorkerThread*>(context)->getVehicleRouter().setBulkMode(myValue);
+            static_cast<WorkerThread*>(context)->getVehicleRouter(SVC_IGNORING).setBulkMode(myValue);
         }
     private:
         const bool myValue;
@@ -486,11 +475,8 @@ private:
     /// @brief Known edges
     NamedObjectCont<ROEdge*> myEdges;
 
-    /// @brief Known bus stops
-    std::map<std::string, SUMOVehicleParameter::Stop*> myBusStops;
-
-    /// @brief Known container stops
-    std::map<std::string, SUMOVehicleParameter::Stop*> myContainerStops;
+    /// @brief Known bus / train / container stops and parking areas
+    std::map<SumoXMLTag, NamedObjectCont<SUMOVehicleParameter::Stop*> > myStoppingPlaces;
 
     /// @brief Known vehicle types
     NamedObjectCont<SUMOVTypeParameter*> myVehicleTypes;
@@ -500,12 +486,15 @@ private:
     /// @brief A distribution of vehicle types (probability->vehicle type)
     VTypeDistDictType myVTypeDistDict;
 
-    /// @brief Whether no vehicle type has been loaded yet
+    /// @brief Whether the default vehicle type was already used or can still be replaced
     bool myDefaultVTypeMayBeDeleted;
 
-    /// @brief Whether no pedestrian type has been loaded yet
+    /// @brief Whether the default pedestrian type was already used or can still be replaced
     bool myDefaultPedTypeMayBeDeleted;
 
+    /// @brief Whether the default bicycle type was already used or can still be replaced
+    bool myDefaultBikeTypeMayBeDeleted;
+
     /// @brief Known routes
     NamedObjectCont<RORouteDef*> myRoutes;
 
@@ -515,10 +504,16 @@ private:
     /// @brief Known flows
     NamedObjectCont<SUMOVehicleParameter*> myFlows;
 
+    /// @brief whether any flows are still active
+    bool myHaveActiveFlows;
+
     /// @brief Known containers
     typedef std::multimap<const SUMOTime, const std::string> ContainerMap;
     ContainerMap myContainers;
 
+    /// @brief vehicles to keep for public transport routing
+    std::vector<const RORoutable*> myPTVehicles;
+
     /// @brief Departure times for randomized flows
     std::map<std::string, std::vector<SUMOTime> > myDepartures;
 
@@ -535,19 +530,19 @@ private:
     OutputDevice* myTypesOutput;
 
     /// @brief The number of read routes
-    unsigned int myReadRouteNo;
+    int myReadRouteNo;
 
     /// @brief The number of discarded routes
-    unsigned int myDiscardedRouteNo;
+    int myDiscardedRouteNo;
 
     /// @brief The number of written routes
-    unsigned int myWrittenRouteNo;
+    int myWrittenRouteNo;
 
     /// @brief Whether the network contains edges which not all vehicles may pass
     bool myHavePermissions;
 
     /// @brief The vehicle class specific speed restrictions
-    std::map<std::string, std::map<SUMOVehicleClass, SUMOReal> > myRestrictions;
+    std::map<std::string, std::map<SUMOVehicleClass, double> > myRestrictions;
 
     /// @brief The number of internal edges in the dictionary
     int myNumInternalEdges;
@@ -555,6 +550,12 @@ private:
     /// @brief handler for ignorable error messages
     MsgHandler* myErrorHandler;
 
+    /// @brief whether to keep the the vtype distribution in output
+    const bool myKeepVTypeDist;
+
+    /// @brief whether the network contains bidirectional railway edges
+    bool myHasBidiEdges;
+
 #ifdef HAVE_FOX
 private:
     class RoutingTask : public FXWorkerThread::Task {
@@ -585,9 +586,3 @@ private:
     RONet& operator=(const RONet& src);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
