@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RONetHandler.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: RONetHandler.h 22608 2017-01-17 06:28:54Z behrisch $
 ///
 // The handler that parses a SUMO-network for its usage in a router
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef RONetHandler_h
-#define RONetHandler_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/xml/SUMOSAXHandler.h>
@@ -64,12 +54,16 @@ public:
      * @param[in] net The network instance to fill
      * @param[in] eb The abstract edge builder to use
      */
-    RONetHandler(RONet& net, ROAbstractEdgeBuilder& eb);
+    RONetHandler(RONet& net, ROAbstractEdgeBuilder& eb, const bool ignoreInternal, const double minorPenalty);
 
 
     /// @brief Destructor
     virtual ~RONetHandler();
 
+    /// @brief retrieve mapping of edges to bidi edges (must be resolved after loading network)
+    const std::map<ROEdge*, std::string>& getBidiMap() const {
+        return myBidiEdges;
+    }
 
 protected:
     /// @name inherited from GenericSAXHandler
@@ -97,6 +91,9 @@ protected:
     /// @name called from myStartElement
     //@{
 
+    /// @brief assign arbitrary vehicle parameters
+    void addParam(const SUMOSAXAttributes& attrs);
+
     /** @brief Parses and builds an edge
      *
      * Parses attributes from an "edge"-element (id, from/to-nodes, function, etc.).
@@ -182,11 +179,22 @@ protected:
 
     //@}
 
+    /// Parses network location description
+    void setLocation(const SUMOSAXAttributes& attrs);
 
 protected:
     /// @brief The net to store the information into
     RONet& myNet;
 
+    /// @brief the loaded network version
+    double myNetworkVersion;
+
+    /// @brief The object used to build of edges of the desired type
+    ROAbstractEdgeBuilder& myEdgeBuilder;
+
+    /// @brief whether to ignore junction internal edges
+    const bool myIgnoreInternal;
+
     /// @brief The name of the edge/node that is currently processed
     std::string myCurrentName;
 
@@ -199,16 +207,14 @@ protected:
     /// @brief The currently built stopping place
     SUMOVehicleParameter::Stop* myCurrentStoppingPlace;
 
-    /** @brief An indicator whether the next edge shall be read (internal edges are not read by now) */
-    bool myProcess;
+    /// @brief temporary data for checking node initialisation after network parsing is finished
+    std::set<std::string> myUnseenNodeIDs;
 
-    /// @brief The object used to build of edges of the desired type
-    ROAbstractEdgeBuilder& myEdgeBuilder;
-
-    /// @brief temporary data for building the junction graph after network parsing is finished
-    typedef std::map<std::string, std::pair<std::string, std::string> > JunctionGraph;
-    JunctionGraph myJunctionGraph;
+    /// @brief time penalty for passing a minor link
+    const double myMinorPenalty;
 
+    /// @brief temporary storage for bidi attributes (to be resolved after loading all edges)
+    std::map<ROEdge*, std::string> myBidiEdges;
 
 private:
     /// @brief Invalidated copy constructor
@@ -218,9 +224,3 @@ private:
     RONetHandler& operator=(const RONetHandler& src);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
