@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RONode.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: RONode.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Base class for nodes used by the router
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef RONode_h
-#define RONode_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -41,6 +31,8 @@
 // ===========================================================================
 class ROEdge;
 
+typedef std::vector<const ROEdge*> ConstROEdgeVector;
+
 // ===========================================================================
 // class definitions
 // ===========================================================================
@@ -69,16 +61,16 @@ public:
     /** @brief Returns the position of the node
      * @return This node's position
      */
-    const Position& getPosition() {
+    const Position& getPosition() const {
         return myPosition;
     }
 
 
-    inline const std::vector<const ROEdge*>& getIncoming() const {
+    inline const ConstROEdgeVector& getIncoming() const {
         return myIncoming;
     }
 
-    inline const std::vector<const ROEdge*>& getOutgoing() const {
+    inline const ConstROEdgeVector& getOutgoing() const {
         return myOutgoing;
     }
 
@@ -95,9 +87,9 @@ private:
     Position myPosition;
 
     /// @brief incoming edges
-    std::vector<const ROEdge*> myIncoming;
+    ConstROEdgeVector myIncoming;
     /// @brief outgoing edges
-    std::vector<const ROEdge*> myOutgoing;
+    ConstROEdgeVector myOutgoing;
 
 
 private:
@@ -108,9 +100,3 @@ private:
     RONode& operator=(const RONode& src);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
