@@ -1,41 +1,31 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RORouteDef.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: RORouteDef.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Base class for a vehicle's route definition
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef RORouteDef_h
-#define RORouteDef_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <iostream>
 #include <utils/common/Named.h>
-#include <utils/vehicle/SUMOAbstractRouter.h>
+#include <utils/router/SUMOAbstractRouter.h>
 #include "RORoute.h"
 
 
@@ -147,6 +137,13 @@ public:
     /** @brief Returns the sum of the probablities of the contained routes */
     double getOverallProb() const;
 
+
+    /// @brief whether this route shall be silently discarded
+    bool discardSilent() const {
+        return myDiscardSilent;
+    }
+
+
     static void setUsingJTRR() {
         myUsingJTRR = true;
     }
@@ -170,27 +167,16 @@ protected:
     const bool myTryRepair;
     const bool myMayBeDisconnected;
 
+    /// @brief Whether this route should be silently discarded
+    mutable bool myDiscardSilent;
+
     static bool myUsingJTRR;
 
-private:
-    /** Function-object for sorting routes from highest to lowest probability. */
-    struct ComparatorProbability {
-        bool operator()(const RORoute* const a, const RORoute* const b) {
-            return a->getProbability() > b->getProbability();
-        }
-    };
-
 private:
     /// @brief Invalidated copy constructor
-    RORouteDef(const RORouteDef& src);
+    RORouteDef(const RORouteDef& src) = delete;
 
     /// @brief Invalidated assignment operator
-    RORouteDef& operator=(const RORouteDef& src);
+    RORouteDef& operator=(const RORouteDef& src) = delete;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
