@@ -1,39 +1,30 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    emissionsDrivingCycle_main.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Wed, 21.08.2013
-/// @version $Id: emissionsDrivingCycle_main.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Main for an emissions calculator
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #ifdef HAVE_VERSION_H
 #include <version.h>
 #endif
 
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <iostream>
 #include <string>
 #include <ctime>
@@ -46,7 +37,7 @@
 #include <utils/common/ToString.h>
 #include <utils/xml/XMLSubSys.h>
 #include <utils/common/FileHelpers.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/StringTokenizer.h>
 #include <utils/common/StringUtils.h>
 #include <utils/emissions/PollutantsInterface.h>
@@ -54,10 +45,6 @@
 #include <utils/importio/LineReader.h>
 #include "TrajectoriesHandler.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // functions
@@ -72,8 +59,8 @@ main(int argc, char** argv) {
     // build options
     OptionsCont& oc = OptionsCont::getOptions();
     //  give some application descriptions
-    oc.setApplicationDescription("Computes emissions by driving a time line.");
-    oc.setApplicationName("emissionsDrivingCycle", "SUMO emissionsDrivingCycle Version " VERSION_STRING);
+    oc.setApplicationDescription("Computes emissions by driving a time line using SUMO's emission models.");
+    oc.setApplicationName("emissionsDrivingCycle", "Eclipse SUMO emissionsDrivingCycle Version " VERSION_STRING);
     //  add options
 
     SystemFrame::addConfigurationOptions(oc);
@@ -106,6 +93,9 @@ main(int argc, char** argv) {
     oc.doRegister("compute-a.forward", new Option_Bool(false));
     oc.addDescription("compute-a.forward", "Processing", "If set, the acceleration for time t is computed from v(t+1) - v(t) instead of v(t) - v(t-1). ");
 
+    oc.doRegister("compute-a.zero-correction", new Option_Bool(false));
+    oc.addDescription("compute-a.zero-correction", "Processing", "If set, the acceleration for time t is set to 0 if the speed is 0. ");
+
     oc.doRegister("skip-first", 's', new Option_Bool(false));
     oc.addDescription("skip-first", "Processing", "If set, the first line of the read file is skipped.");
 
@@ -131,7 +121,7 @@ main(int argc, char** argv) {
     oc.addDescription("sum-output", "Output", "Save the aggregated and normed emission values of each vehicle in CSV");
 
     oc.addOptionSubTopic("Emissions");
-    oc.doRegister("phemlight-path", new Option_FileName("./PHEMlight/"));
+    oc.doRegister("phemlight-path", new Option_FileName(StringVector({ "./PHEMlight/" })));
     oc.addDescription("phemlight-path", "Emissions", "Determines where to load PHEMlight definitions from.");
 
     SystemFrame::addReportOptions(oc);
@@ -146,7 +136,6 @@ main(int argc, char** argv) {
         XMLSubSys::init();
         OptionsIO::setArgs(argc, argv);
         OptionsIO::getOptions();
-        OptionsCont& oc = OptionsCont::getOptions();
         if (oc.processMetaOptions(argc < 2)) {
             SystemFrame::close();
             return 0;
@@ -159,18 +148,18 @@ main(int argc, char** argv) {
         if (!oc.isSet("output-file") && (oc.isSet("timeline-file") || !oc.isSet("emission-output"))) {
             throw ProcessError("The output file must be given.");
         }
-        std::ostream* out = 0;
+        std::ostream* out = nullptr;
         if (oc.isSet("output-file")) {
             out = new std::ofstream(oc.getString("output-file").c_str());
         }
         OutputDevice::createDeviceByOption("emission-output", "emission-export", "emission_file.xsd");
-        OutputDevice* xmlOut = 0;
+        OutputDevice* xmlOut = nullptr;
         if (oc.isSet("emission-output")) {
             xmlOut = &OutputDevice::getDeviceByOption("emission-output");
-        } else if (out == 0) {
+        } else if (out == nullptr) {
             out = &std::cout;
         }
-        std::ostream* sumOut = 0;
+        std::ostream* sumOut = nullptr;
         if (oc.isSet("sum-output")) {
             sumOut = new std::ofstream(oc.getString("sum-output").c_str());
             (*sumOut) << "Vehicle,Cycle,Time,Speed,Gradient,Acceleration,FC,FCel,CO2,NOx,CO,HC,PM" << std::endl;
@@ -178,15 +167,15 @@ main(int argc, char** argv) {
 
         const SUMOEmissionClass defaultClass = PollutantsInterface::getClassByName(oc.getString("emission-class"));
         const bool computeA = oc.getBool("compute-a") || oc.getBool("compute-a.forward");
-        TrajectoriesHandler handler(computeA, oc.getBool("compute-a.forward"), defaultClass, oc.getFloat("slope"), out, xmlOut);
+        TrajectoriesHandler handler(computeA, oc.getBool("compute-a.forward"), oc.getBool("compute-a.zero-correction"), defaultClass, oc.getFloat("slope"), out, xmlOut);
 
         if (oc.isSet("timeline-file")) {
             int skip = oc.getBool("skip-first") ? 1 : oc.getInt("timeline-file.skip");
             const bool inKMH = oc.getBool("kmh");
             const bool haveSlope = oc.getBool("have-slope");
-            SUMOReal l = 0;
-            SUMOReal totalA = 0;
-            SUMOReal totalS = 0;
+            double l = 0;
+            double totalA = 0;
+            double totalS = 0;
             int time = 0;
 
             LineReader lr(oc.getString("timeline-file"));
@@ -197,34 +186,39 @@ main(int argc, char** argv) {
                     continue;
                 }
                 StringTokenizer st(StringUtils::prune(line), oc.getString("timeline-file.separator"));
-                if (st.size() < 2) {
-                    throw ProcessError("Each line must at least include the time and the speed.");
-                }
-                try {
-                    const SUMOReal t = TplConvert::_2SUMOReal<char>(st.next().c_str());
-                    SUMOReal v = TplConvert::_2SUMOReal<char>(st.next().c_str());
-                    if (inKMH) {
-                        v /= 3.6;
+                if (st.hasNext()) {
+                    try {
+                        double t = StringUtils::toDouble(st.next());
+                        double v = 0;
+                        if (st.hasNext()) {
+                            v = StringUtils::toDouble(st.next());
+                        } else {
+                            v = t;
+                            t = time;
+                        }
+                        if (inKMH) {
+                            v /= 3.6;
+                        }
+                        double a = !computeA && st.hasNext() ? StringUtils::toDouble(st.next()) : TrajectoriesHandler::INVALID_VALUE;
+                        double s = haveSlope && st.hasNext() ? StringUtils::toDouble(st.next()) : TrajectoriesHandler::INVALID_VALUE;
+                        if (handler.writeEmissions(*out, "", defaultClass, t, v, a, s)) {
+                            l += v;
+                            totalA += a;
+                            totalS += s;
+                            time++;
+                        }
+                    } catch (EmptyData&) {
+                        throw ProcessError("Missing an entry in line '" + line + "'.");
+                    } catch (NumberFormatException&) {
+                        throw ProcessError("Not numeric entry in line '" + line + "'.");
                     }
-                    SUMOReal a = !computeA && st.hasNext() ? TplConvert::_2SUMOReal<char>(st.next().c_str()) : TrajectoriesHandler::INVALID_VALUE;
-                    SUMOReal s = haveSlope ? TplConvert::_2SUMOReal<char>(st.next().c_str()) : TrajectoriesHandler::INVALID_VALUE;
-                    if (handler.writeEmissions(*out, "", defaultClass, t, v, a, s)) {
-                        l += v;
-                        totalA += a;
-                        totalS += s;
-                        time++;
-                    }
-                } catch (EmptyData&) {
-                    throw ProcessError("Missing an entry in line '" + line + "'.");
-                } catch (NumberFormatException&) {
-                    throw ProcessError("Not numeric entry in line '" + line + "'.");
                 }
             }
             if (!quiet) {
                 std::cout << "sums" << std::endl
                           << "length:" << l << std::endl;
             }
-            if (sumOut != 0) {
+            if (sumOut != nullptr) {
                 (*sumOut) << oc.getString("emission-class") << "," << lr.getFileName() << "," << time << ","
                           << (l / time * 3.6) << "," << (totalS / time) << "," << (totalA / time) << ",";
                 handler.writeNormedSums(*sumOut, "", l);
@@ -236,6 +230,7 @@ main(int argc, char** argv) {
         if (!quiet) {
             handler.writeSums(std::cout, "");
         }
+        delete sumOut;
     } catch (InvalidArgument& e) {
         MsgHandler::getErrorInstance()->inform(e.what());
         MsgHandler::getErrorInstance()->inform("Quitting (on error).", false);
@@ -260,6 +255,4 @@ main(int argc, char** argv) {
 }
 
 
-
 /****************************************************************************/
-
