@@ -1,39 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    TraCIServerAPI_InductionLoop.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    07.05.2009
-/// @version $Id: TraCIServerAPI_InductionLoop.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // APIs for getting/setting induction loop values via TraCI
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef TraCIServerAPI_InductionLoop_h
-#define TraCIServerAPI_InductionLoop_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#ifndef NO_TRACI
-
-#include "TraCIException.h"
 #include "TraCIServer.h"
 #include <foreign/tcpip/storage.h>
 
@@ -56,20 +43,16 @@ public:
     static bool processGet(TraCIServer& server, tcpip::Storage& inputStorage,
                            tcpip::Storage& outputStorage);
 
-
-    /** @brief Returns the named inductive loop's position
-     * @param[in] id The id of the searched inductive loop
-     * @param[out] p The position, if the inductive loop is known
-     * @return Whether the inductive loop is known (and on road)
+    /** @brief Processes a set value command (Command 0xc0: Set Induction Loop Variable)
+     *
+     * @param[in] server The TraCI-server-instance which schedules this request
+     * @param[in] inputStorage The storage to read the command from
+     * @param[out] outputStorage The storage to write the result to
      */
-    static bool getPosition(const std::string& id, Position& p);
+    static bool processSet(TraCIServer& server, tcpip::Storage& inputStorage,
+                           tcpip::Storage& outputStorage);
 
 
-    /** @brief Returns a tree filled with inductive loop instances
-     * @return The rtree of inductive loops
-     */
-    static NamedRTree* getTree();
-
 
 private:
     /// @brief invalidated copy constructor
@@ -80,11 +63,3 @@ private:
 
 
 };
-
-
-#endif
-
-#endif
-
-/****************************************************************************/
-
