@@ -1,36 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    TraCIServerAPI_Person.h
 /// @author  Daniel Krajzewicz
 /// @date    26.05.2014
-/// @version $Id: TraCIServerAPI_Person.h 22715 2017-01-26 14:55:39Z namdre $
 ///
 // APIs for getting/setting person values via TraCI
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef TraCIServerAPI_Person_h
-#define TraCIServerAPI_Person_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
-
-#ifndef NO_TRACI
 
 #include <foreign/tcpip/storage.h>
 
@@ -70,16 +58,6 @@ public:
                            tcpip::Storage& outputStorage);
 
 
-    /** @brief Returns the named persons's position
-     * @param[in] id The id of the searched person
-     * @param[out] p The position, if the person is on the network
-     * @return Whether the person is known
-     */
-    static bool getPosition(const std::string& id, Position& p);
-
-private:
-    static MSVehicleType& getSingularType(MSTransportable* const t);
-
 private:
     /// @brief invalidated copy constructor
     TraCIServerAPI_Person(const TraCIServerAPI_Person& s);
@@ -89,10 +67,3 @@ private:
 
 
 };
-
-
-#endif
-
-#endif
-
-/****************************************************************************/
