@@ -1,39 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    TraCIServerAPI_Polygon.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    07.05.2009
-/// @version $Id: TraCIServerAPI_Polygon.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // APIs for getting/setting polygon values via TraCI
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef TraCIServerAPI_Polygon_h
-#define TraCIServerAPI_Polygon_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#ifndef NO_TRACI
-
-#include "TraCIException.h"
 #include "TraCIServer.h"
 #include <foreign/tcpip/storage.h>
 
@@ -67,29 +54,6 @@ public:
                            tcpip::Storage& outputStorage);
 
 
-    /** @brief Returns the named polygons's shape
-     *
-     * @param[in] id The id of the searched polygon
-     * @param[out] shape The shape, if the polygon is known
-     * @return Whether the polygon is known
-     */
-    static bool getShape(const std::string& id, PositionVector& shape);
-
-
-    /** @brief Returns a tree filled with polygon instances
-     * @return The rtree of polygons
-     */
-    static NamedRTree* getTree();
-
-
-private:
-    /** @brief Returns the named polygon
-     * @param[in] id The id of the searched polygon
-     * @return The named polygon, or 0 if it is not known
-     */
-    static Polygon* getPolygon(const std::string& id);
-
-
 private:
     /// @brief invalidated copy constructor
     TraCIServerAPI_Polygon(const TraCIServerAPI_Polygon& s);
@@ -99,11 +63,3 @@ private:
 
 
 };
-
-
-#endif
-
-#endif
-
-/****************************************************************************/
-
