@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    TraCIServerAPI_VehicleType.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
@@ -7,42 +19,18 @@
 /// @author  Robert Hilbrich
 /// @author  Gregor Laemmel
 /// @date    07.05.2009
-/// @version $Id: TraCIServerAPI_VehicleType.cpp 24108 2017-04-27 18:43:30Z behrisch $
 ///
 // APIs for getting/setting vehicle type values via TraCI
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
-
-#ifndef NO_TRACI
 
 #include <limits>
 #include <utils/emissions/PollutantsInterface.h>
 #include <microsim/MSNet.h>
 #include <microsim/MSVehicleType.h>
-#include <traci-server/lib/TraCI_VehicleType.h>
-#include "TraCIConstants.h"
+#include <libsumo/TraCIConstants.h>
+#include <libsumo/VehicleType.h>
 #include "TraCIServerAPI_VehicleType.h"
-#include "lib/TraCI.h"
 
 
 // ===========================================================================
@@ -51,168 +39,23 @@
 bool
 TraCIServerAPI_VehicleType::processGet(TraCIServer& server, tcpip::Storage& inputStorage,
                                        tcpip::Storage& outputStorage) {
-    // variable & id
-    int variable = inputStorage.readUnsignedByte();
-    std::string id = inputStorage.readString();
-    // check variable
-    if (variable != ID_LIST && variable != VAR_LENGTH && variable != VAR_MAXSPEED && variable != VAR_ACCEL
-            && variable != VAR_DECEL
-            && variable != VAR_TAU && variable != VAR_VEHICLECLASS && variable != VAR_EMISSIONCLASS
-            && variable != VAR_SHAPECLASS
-            && variable != VAR_SPEED_FACTOR && variable != VAR_SPEED_DEVIATION && variable != VAR_IMPERFECTION
-            && variable != VAR_MINGAP && variable != VAR_WIDTH && variable != VAR_COLOR && variable != ID_COUNT
-            && variable != VAR_HEIGHT
-            && variable != VAR_MINGAP_LAT
-            && variable != VAR_MAXSPEED_LAT
-            && variable != VAR_LATALIGNMENT
-            && variable != VAR_PARAMETER) {
-        return server.writeErrorStatusCmd(CMD_GET_VEHICLETYPE_VARIABLE,
-                                          "Get Vehicle Type Variable: unsupported variable " + toHex(variable, 2)
-                                          + " specified", outputStorage);
-    }
-    // begin response building
-    tcpip::Storage tempMsg;
-    //  response-code, variableID, objectID
-    tempMsg.writeUnsignedByte(RESPONSE_GET_VEHICLETYPE_VARIABLE);
-    tempMsg.writeUnsignedByte(variable);
-    tempMsg.writeString(id);
-    // process request
-    if (variable == ID_LIST) {
-        std::vector<std::string> ids = TraCI_VehicleType::getIDList();
-        tempMsg.writeUnsignedByte(TYPE_STRINGLIST);
-        tempMsg.writeStringList(ids);
-    } else if (variable == ID_COUNT) {
-        std::vector<std::string> ids = TraCI_VehicleType::getIDList();
-        tempMsg.writeUnsignedByte(TYPE_INTEGER);
-        tempMsg.writeInt((int) ids.size());
-    } else {
-        try {
-            switch (variable) {
-                case VAR_PARAMETER: {
-                    std::string paramName = "";
-                    if (!server.readTypeCheckingString(inputStorage, paramName)) {
-                        return server.writeErrorStatusCmd(CMD_GET_VEHICLETYPE_VARIABLE,
-                                                          "Retrieval of a parameter requires its name.", outputStorage);
-                    }
-                    tempMsg.writeUnsignedByte(TYPE_STRING);
-                    tempMsg.writeString(TraCI_VehicleType::getParameter(id, paramName));
-                }
-                break;
-                default:
-                    getVariable(variable, id, tempMsg);
-                    break;
-            }
-        } catch (TraCIException& e) {
-            return server.writeErrorStatusCmd(CMD_GET_VEHICLETYPE_VARIABLE, e.what(), outputStorage);
+    const int variable = inputStorage.readUnsignedByte();
+    const std::string id = inputStorage.readString();
+    server.initWrapper(libsumo::RESPONSE_GET_VEHICLETYPE_VARIABLE, variable, id);
+    try {
+        if (!libsumo::VehicleType::handleVariable(id, variable, &server, &inputStorage)) {
+            return server.writeErrorStatusCmd(libsumo::CMD_GET_VEHICLETYPE_VARIABLE,
+                                              "Get Vehicle Type Variable: unsupported variable " + toHex(variable, 2)
+                                              + " specified", outputStorage);
         }
+    } catch (libsumo::TraCIException& e) {
+        return server.writeErrorStatusCmd(libsumo::CMD_GET_VEHICLETYPE_VARIABLE, e.what(), outputStorage);
     }
-    server.writeStatusCmd(CMD_GET_VEHICLETYPE_VARIABLE, RTYPE_OK, "", outputStorage);
-    server.writeResponseWithLength(outputStorage, tempMsg);
+    server.writeStatusCmd(libsumo::CMD_GET_VEHICLETYPE_VARIABLE, libsumo::RTYPE_OK, "", outputStorage);
+    server.writeResponseWithLength(outputStorage, server.getWrapperStorage());
     return true;
 }
 
-bool
-TraCIServerAPI_VehicleType::getVariable(const int variable, const std::string& id, tcpip::Storage& tempMsg) {
-    switch (variable) {
-        case VAR_LENGTH: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getLength(id));
-        }
-        break;
-        case VAR_HEIGHT: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getHeight(id));
-        }
-        break;
-        case VAR_MINGAP: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getMinGap(id));
-        }
-        break;
-        case VAR_MAXSPEED: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getMaxSpeed(id));
-        }
-        break;
-        case VAR_ACCEL: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getAccel(id));
-        }
-        break;
-        case VAR_DECEL: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getDecel(id));
-        }
-        break;
-        case VAR_IMPERFECTION: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getImperfection(id));
-        }
-        break;
-        case VAR_TAU: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getTau(id));
-        }
-        break;
-        case VAR_SPEED_FACTOR: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getSpeedFactor(id));
-        }
-        break;
-        case VAR_SPEED_DEVIATION: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getSpeedDeviation(id));
-        }
-        break;
-        case VAR_VEHICLECLASS: {
-            tempMsg.writeUnsignedByte(TYPE_STRING);
-            tempMsg.writeString(TraCI_VehicleType::getVehicleClass(id));
-        }
-        break;
-        case VAR_EMISSIONCLASS: {
-            tempMsg.writeUnsignedByte(TYPE_STRING);
-            tempMsg.writeString(TraCI_VehicleType::getEmissionClass(id));
-        }
-        break;
-        case VAR_SHAPECLASS: {
-            tempMsg.writeUnsignedByte(TYPE_STRING);
-            tempMsg.writeString(TraCI_VehicleType::getShapeClass(id));
-        }
-        break;
-        case VAR_WIDTH: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getWidth(id));
-        }
-        break;
-        case VAR_COLOR: {
-            tempMsg.writeUnsignedByte(TYPE_COLOR);
-            const TraCIColor& col = TraCI_VehicleType::getColor(id);
-            tempMsg.writeUnsignedByte(col.r);
-            tempMsg.writeUnsignedByte(col.g);
-            tempMsg.writeUnsignedByte(col.b);
-            tempMsg.writeUnsignedByte(col.a);
-        }
-        break;
-        case VAR_MINGAP_LAT: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getMinGapLat(id));
-        }
-        break;
-        case VAR_MAXSPEED_LAT: {
-            tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-            tempMsg.writeDouble(TraCI_VehicleType::getMaxSpeedLat(id));
-        }
-        break;
-        case VAR_LATALIGNMENT: {
-            tempMsg.writeUnsignedByte(TYPE_STRING);
-            tempMsg.writeString(TraCI_VehicleType::getLateralAlignment(id));
-        }
-        break;
-        default:
-            break;
-    }
-    return true;
-}
 
 bool
 TraCIServerAPI_VehicleType::processSet(TraCIServer& server, tcpip::Storage& inputStorage,
@@ -220,38 +63,40 @@ TraCIServerAPI_VehicleType::processSet(TraCIServer& server, tcpip::Storage& inpu
     std::string warning = ""; // additional description for response
     // variable
     int variable = inputStorage.readUnsignedByte();
-    if (variable != VAR_LENGTH && variable != VAR_MAXSPEED && variable != VAR_VEHICLECLASS
-            && variable != VAR_SPEED_FACTOR && variable != VAR_SPEED_DEVIATION && variable != VAR_EMISSIONCLASS
-            && variable != VAR_WIDTH && variable != VAR_MINGAP && variable != VAR_SHAPECLASS
-            && variable != VAR_ACCEL && variable != VAR_DECEL && variable != VAR_IMPERFECTION
-            && variable != VAR_TAU && variable != VAR_COLOR
-            && variable != VAR_HEIGHT
-            && variable != VAR_MINGAP_LAT
-            && variable != VAR_MAXSPEED_LAT
-            && variable != VAR_LATALIGNMENT
-            && variable != VAR_PARAMETER
+    if (variable != libsumo::VAR_LENGTH && variable != libsumo::VAR_MAXSPEED && variable != libsumo::VAR_VEHICLECLASS
+            && variable != libsumo::VAR_SPEED_FACTOR && variable != libsumo::VAR_SPEED_DEVIATION && variable != libsumo::VAR_EMISSIONCLASS
+            && variable != libsumo::VAR_WIDTH && variable != libsumo::VAR_MINGAP && variable != libsumo::VAR_SHAPECLASS
+            && variable != libsumo::VAR_ACCEL && variable != libsumo::VAR_IMPERFECTION
+            && variable != libsumo::VAR_DECEL && variable != libsumo::VAR_EMERGENCY_DECEL && variable != libsumo::VAR_APPARENT_DECEL
+            && variable != libsumo::VAR_TAU && variable != libsumo::VAR_COLOR && variable != libsumo::VAR_ACTIONSTEPLENGTH
+            && variable != libsumo::VAR_HEIGHT
+            && variable != libsumo::VAR_MINGAP_LAT
+            && variable != libsumo::VAR_MAXSPEED_LAT
+            && variable != libsumo::VAR_LATALIGNMENT
+            && variable != libsumo::VAR_PARAMETER
+            && variable != libsumo::COPY
        ) {
-        return server.writeErrorStatusCmd(CMD_SET_VEHICLETYPE_VARIABLE,
+        return server.writeErrorStatusCmd(libsumo::CMD_SET_VEHICLETYPE_VARIABLE,
                                           "Change Vehicle Type State: unsupported variable " + toHex(variable, 2)
                                           + " specified", outputStorage);
     }
     // id
     std::string id = inputStorage.readString();
-//    MSVehicleType* v = TraCI_VehicleType::getVType(id);
+//    MSVehicleType* v = libsumo::VehicleType::getVType(id);
 //    if (v == 0) {
-//        return server.writeErrorStatusCmd(CMD_SET_VEHICLETYPE_VARIABLE, "Vehicle type '" + id + "' is not known",
+//        return server.writeErrorStatusCmd(libsumo::CMD_SET_VEHICLETYPE_VARIABLE, "Vehicle type '" + id + "' is not known",
 //                                          outputStorage);
 //    }
     // process
     try {
-        if (setVariable(CMD_SET_VEHICLETYPE_VARIABLE, variable, id, server, inputStorage, outputStorage)) {
-            server.writeStatusCmd(CMD_SET_VEHICLETYPE_VARIABLE, RTYPE_OK, warning, outputStorage);
+        if (setVariable(libsumo::CMD_SET_VEHICLETYPE_VARIABLE, variable, id, server, inputStorage, outputStorage)) {
+            server.writeStatusCmd(libsumo::CMD_SET_VEHICLETYPE_VARIABLE, libsumo::RTYPE_OK, warning, outputStorage);
             return true;
         }
     } catch (ProcessError& e) {
-        return server.writeErrorStatusCmd(CMD_SET_VEHICLETYPE_VARIABLE, e.what(), outputStorage);
-    } catch (TraCIException& e) {
-        return server.writeErrorStatusCmd(CMD_SET_VEHICLETYPE_VARIABLE, e.what(), outputStorage);
+        return server.writeErrorStatusCmd(libsumo::CMD_SET_VEHICLETYPE_VARIABLE, e.what(), outputStorage);
+    } catch (libsumo::TraCIException& e) {
+        return server.writeErrorStatusCmd(libsumo::CMD_SET_VEHICLETYPE_VARIABLE, e.what(), outputStorage);
     }
     return false;
 }
@@ -262,7 +107,7 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
                                         const std::string& id, TraCIServer& server,
                                         tcpip::Storage& inputStorage, tcpip::Storage& outputStorage) {
     switch (variable) {
-        case VAR_LENGTH: {
+        case libsumo::VAR_LENGTH: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting length requires a double.", outputStorage);
@@ -270,10 +115,10 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value <= 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid length.", outputStorage);
             }
-            TraCI_VehicleType::setLength(id, value);
+            libsumo::VehicleType::setLength(id, value);
         }
         break;
-        case VAR_HEIGHT: {
+        case libsumo::VAR_HEIGHT: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting height requires a double.", outputStorage);
@@ -281,10 +126,10 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value <= 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid height.", outputStorage);
             }
-            TraCI_VehicleType::setHeight(id, value);
+            libsumo::VehicleType::setHeight(id, value);
         }
         break;
-        case VAR_MAXSPEED: {
+        case libsumo::VAR_MAXSPEED: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting maximum speed requires a double.", outputStorage);
@@ -292,22 +137,22 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value <= 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid maximum speed.", outputStorage);
             }
-            TraCI_VehicleType::setMaxSpeed(id, value);
+            libsumo::VehicleType::setMaxSpeed(id, value);
         }
         break;
-        case VAR_VEHICLECLASS: {
+        case libsumo::VAR_VEHICLECLASS: {
             std::string vclass;
             if (!server.readTypeCheckingString(inputStorage, vclass)) {
                 return server.writeErrorStatusCmd(cmd, "Setting vehicle class requires a string.", outputStorage);
             }
             try {
-                TraCI_VehicleType::setVehicleClass(id, vclass);
-            } catch (InvalidArgument e) {
+                libsumo::VehicleType::setVehicleClass(id, vclass);
+            } catch (InvalidArgument&) {
                 return server.writeErrorStatusCmd(cmd, "Unknown vehicle class '" + vclass + "'.", outputStorage);
             }
         }
         break;
-        case VAR_SPEED_FACTOR: {
+        case libsumo::VAR_SPEED_FACTOR: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting speed factor requires a double.", outputStorage);
@@ -315,10 +160,10 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value <= 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid speed factor.", outputStorage);
             }
-            TraCI_VehicleType::setSpeedFactor(id, value);
+            libsumo::VehicleType::setSpeedFactor(id, value);
         }
         break;
-        case VAR_SPEED_DEVIATION: {
+        case libsumo::VAR_SPEED_DEVIATION: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting speed deviation requires a double.", outputStorage);
@@ -326,22 +171,22 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value < 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid speed deviation.", outputStorage);
             }
-            TraCI_VehicleType::setSpeedDeviation(id, value);
+            libsumo::VehicleType::setSpeedDeviation(id, value);
         }
         break;
-        case VAR_EMISSIONCLASS: {
+        case libsumo::VAR_EMISSIONCLASS: {
             std::string eclass;
             if (!server.readTypeCheckingString(inputStorage, eclass)) {
                 return server.writeErrorStatusCmd(cmd, "Setting emission class requires a string.", outputStorage);
             }
             try {
-                TraCI_VehicleType::setEmissionClass(id, eclass);
-            } catch (InvalidArgument e) {
-                return server.writeErrorStatusCmd(cmd, "Unknown emission class '" + eclass + "'.", outputStorage);
+                libsumo::VehicleType::setEmissionClass(id, eclass);
+            } catch (InvalidArgument& e) {
+                return server.writeErrorStatusCmd(cmd, e.what(), outputStorage);
             }
         }
         break;
-        case VAR_WIDTH: {
+        case libsumo::VAR_WIDTH: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting width requires a double.", outputStorage);
@@ -349,10 +194,10 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value <= 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid width.", outputStorage);
             }
-            TraCI_VehicleType::setWidth(id, value);
+            libsumo::VehicleType::setWidth(id, value);
         }
         break;
-        case VAR_MINGAP: {
+        case libsumo::VAR_MINGAP: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting minimum gap requires a double.", outputStorage);
@@ -360,10 +205,10 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value < 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid minimum gap.", outputStorage);
             }
-            TraCI_VehicleType::setMinGap(id, value);
+            libsumo::VehicleType::setMinGap(id, value);
         }
         break;
-        case VAR_MINGAP_LAT: {
+        case libsumo::VAR_MINGAP_LAT: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting minimum lateral gap requires a double.", outputStorage);
@@ -371,10 +216,10 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value < 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid minimum lateral gap.", outputStorage);
             }
-            TraCI_VehicleType::setMinGapLat(id, value);
+            libsumo::VehicleType::setMinGapLat(id, value);
         }
         break;
-        case VAR_MAXSPEED_LAT: {
+        case libsumo::VAR_MAXSPEED_LAT: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting maximum lateral speed requires a double.", outputStorage);
@@ -382,35 +227,35 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value < 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid maximum lateral speed.", outputStorage);
             }
-            TraCI_VehicleType::setMaxSpeedLat(id, value);
+            libsumo::VehicleType::setMaxSpeedLat(id, value);
         }
         break;
-        case VAR_LATALIGNMENT: {
+        case libsumo::VAR_LATALIGNMENT: {
             std::string latAlign;
             if (!server.readTypeCheckingString(inputStorage, latAlign)) {
                 return server.writeErrorStatusCmd(cmd, "Setting preferred lateral alignment requires a string.",
                                                   outputStorage);
             }
             if (SUMOXMLDefinitions::LateralAlignments.hasString(latAlign)) {
-                TraCI_VehicleType::setLateralAlignment(id, latAlign);
+                libsumo::VehicleType::setLateralAlignment(id, latAlign);
             } else {
                 return server.writeErrorStatusCmd(cmd, "Unknown lateral alignment " + latAlign + "'.", outputStorage);
             }
         }
         break;
-        case VAR_SHAPECLASS: {
+        case libsumo::VAR_SHAPECLASS: {
             std::string sclass;
             if (!server.readTypeCheckingString(inputStorage, sclass)) {
                 return server.writeErrorStatusCmd(cmd, "Setting vehicle shape requires a string.", outputStorage);
             }
             try {
-                TraCI_VehicleType::setShapeClass(id, sclass);
-            } catch (InvalidArgument e) {
-                return server.writeErrorStatusCmd(cmd, "Unknown vehicle shape " + sclass + "'.", outputStorage);
+                libsumo::VehicleType::setShapeClass(id, sclass);
+            } catch (InvalidArgument& e) {
+                return server.writeErrorStatusCmd(cmd, e.what(), outputStorage);
             }
         }
         break;
-        case VAR_ACCEL: {
+        case libsumo::VAR_ACCEL: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting acceleration requires a double.", outputStorage);
@@ -418,10 +263,10 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value < 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid acceleration.", outputStorage);
             }
-            TraCI_VehicleType::setAccel(id, value);
+            libsumo::VehicleType::setAccel(id, value);
         }
         break;
-        case VAR_DECEL: {
+        case libsumo::VAR_DECEL: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting deceleration requires a double.", outputStorage);
@@ -429,10 +274,44 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value < 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid deceleration.", outputStorage);
             }
-            TraCI_VehicleType::setDecel(id, value);
+            libsumo::VehicleType::setDecel(id, value);
         }
         break;
-        case VAR_IMPERFECTION: {
+        case libsumo::VAR_EMERGENCY_DECEL: {
+            double value = 0;
+            if (!server.readTypeCheckingDouble(inputStorage, value)) {
+                return server.writeErrorStatusCmd(cmd, "Setting deceleration requires a double.", outputStorage);
+            }
+            if (value < 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
+                return server.writeErrorStatusCmd(cmd, "Invalid deceleration.", outputStorage);
+            }
+            libsumo::VehicleType::setEmergencyDecel(id, value);
+        }
+        break;
+        case libsumo::VAR_APPARENT_DECEL: {
+            double value = 0;
+            if (!server.readTypeCheckingDouble(inputStorage, value)) {
+                return server.writeErrorStatusCmd(cmd, "Setting deceleration requires a double.", outputStorage);
+            }
+            if (value < 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
+                return server.writeErrorStatusCmd(cmd, "Invalid deceleration.", outputStorage);
+            }
+            libsumo::VehicleType::setApparentDecel(id, value);
+        }
+        break;
+        case libsumo::VAR_ACTIONSTEPLENGTH: {
+            double value = 0;
+            if (!server.readTypeCheckingDouble(inputStorage, value)) {
+                return server.writeErrorStatusCmd(libsumo::CMD_SET_VEHICLE_VARIABLE, "Setting action step length requires a double.", outputStorage);
+            }
+            if (fabs(value) == std::numeric_limits<double>::infinity()) {
+                return server.writeErrorStatusCmd(libsumo::CMD_SET_VEHICLE_VARIABLE, "Invalid action step length.", outputStorage);
+            }
+            bool resetActionOffset = value >= 0.0;
+            libsumo::VehicleType::setActionStepLength(id, fabs(value), resetActionOffset);
+        }
+        break;
+        case libsumo::VAR_IMPERFECTION: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting driver imperfection requires a double.", outputStorage);
@@ -440,10 +319,10 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value < 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid driver imperfection.", outputStorage);
             }
-            TraCI_VehicleType::setImperfection(id, value);
+            libsumo::VehicleType::setImperfection(id, value);
         }
         break;
-        case VAR_TAU: {
+        case libsumo::VAR_TAU: {
             double value = 0;
             if (!server.readTypeCheckingDouble(inputStorage, value)) {
                 return server.writeErrorStatusCmd(cmd, "Setting headway time requires a double.", outputStorage);
@@ -451,19 +330,28 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
             if (value < 0.0 || fabs(value) == std::numeric_limits<double>::infinity()) {
                 return server.writeErrorStatusCmd(cmd, "Invalid headway time.", outputStorage);
             }
-            TraCI_VehicleType::setTau(id, value);
+            libsumo::VehicleType::setTau(id, value);
         }
         break;
-        case VAR_COLOR: {
-            TraCIColor col;
+        case libsumo::VAR_COLOR: {
+            libsumo::TraCIColor col;
             if (!server.readTypeCheckingColor(inputStorage, col)) {
                 return server.writeErrorStatusCmd(cmd, "The color must be given using the according type.", outputStorage);
             }
-            TraCI_VehicleType::setColor(id, col);
+            libsumo::VehicleType::setColor(id, col);
         }
         break;
-        case VAR_PARAMETER: {
-            if (inputStorage.readUnsignedByte() != TYPE_COMPOUND) {
+        case libsumo::COPY: {
+            std::string newTypeID;
+            if (!server.readTypeCheckingString(inputStorage, newTypeID)) {
+                return server.writeErrorStatusCmd(cmd, "copying a vehicle type requires a string.",
+                                                  outputStorage);
+            }
+            libsumo::VehicleType::copy(id, newTypeID);
+        }
+        break;
+        case libsumo::VAR_PARAMETER: {
+            if (inputStorage.readUnsignedByte() != libsumo::TYPE_COMPOUND) {
                 return server.writeErrorStatusCmd(cmd, "A compound object is needed for setting a parameter.",
                                                   outputStorage);
             }
@@ -479,7 +367,7 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
                 return server.writeErrorStatusCmd(cmd, "The value of the parameter must be given as a string.",
                                                   outputStorage);
             }
-            TraCI_VehicleType::addParameter(id, name, value);
+            libsumo::VehicleType::setParameter(id, name, value);
         }
         break;
         default:
@@ -488,7 +376,5 @@ TraCIServerAPI_VehicleType::setVariable(const int cmd, const int variable,
     return true;
 }
 
-#endif
-
 
 /****************************************************************************/
