@@ -1,37 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    FileHelpers.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: FileHelpers.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Functions for an easier usage of files
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef FileHelpers_h
-#define FileHelpers_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
-
 #include <cassert>
 #include <fstream>
 #include <string>
@@ -49,7 +38,7 @@
 class FileHelpers {
 public:
     /// @name file access functions
-    //@{
+    /// @{
 
     /** @brief Checks whether the given file is readable
      *
@@ -57,12 +46,17 @@ public:
      * @return Whether the named file is readable
      */
     static bool isReadable(std::string path);
-    //@}
-
 
+    /** @brief Checks whether the given file is a directory
+     *
+     * @param[in] path The path to the file that shall be examined
+     * @return Whether the named file is a directory.
+     */
+    static bool isDirectory(std::string path);
+    /// @}
 
     /// @name file path evaluating functions
-    //@{
+    /// @{
 
     /** @brief Removes the file information from the given path
      *
@@ -71,6 +65,13 @@ public:
      */
     static std::string getFilePath(const std::string& path);
 
+    /** @brief Add an extension to the given file path
+     *
+     * @param[in] path The path to the file
+     * @param[in] extension new extension (with dot, example: '.xml')
+     * @return the new path with extension, the same path if it already has the extension, or a empty string if path is invalid
+     */
+    static std::string addExtension(const std::string& path, const std::string& extension);
 
     /** @brief Returns the second path as a relative path to the first file
      *
@@ -84,9 +85,7 @@ public:
      * @param[in] path The path to the references file (relativ to configuration path)
      * @return The file's position (relative to curent working directory)
      */
-    static std::string getConfigurationRelative(const std::string& configPath,
-            const std::string& path);
-
+    static std::string getConfigurationRelative(const std::string& configPath, const std::string& path);
 
     /** @brief Returns the information whether the given name represents a socket
      *
@@ -98,7 +97,6 @@ public:
      */
     static bool isSocket(const std::string& name);
 
-
     /** @brief Returns the information whether the given path is absolute
      *
      * A path is meant to be absolute, if
@@ -111,7 +109,6 @@ public:
      */
     static bool isAbsolute(const std::string& path);
 
-
     /** @brief Returns the path from a configuration so that it is accessable from the current working directory
      *
      * If the path is absolute, it is returned. Otherwise, the file's position
@@ -123,18 +120,15 @@ public:
      * @param[in] basePath The path the configuration file (including the config's file name)
      * @return The file's position
      */
-    static std::string checkForRelativity(const std::string& filename,
-                                          const std::string& basePath);
+    static std::string checkForRelativity(const std::string& filename, const std::string& basePath);
 
     /// @brief prepend the given prefix to the last path component of the given file path
     static std::string prependToLastPathComponent(const std::string& prefix, const std::string& path);
 
-    //@}
-
-
+    /// @}
 
     /// @name binary writing functions
-    //@{
+    /// @{
 
     /** @brief Writes an integer binary
      *
@@ -144,7 +138,6 @@ public:
      */
     static std::ostream& writeInt(std::ostream& strm, int value);
 
-
     /** @brief Writes a float binary
      *
      * This method behaves differently depending on the definition of double at compile time.
@@ -155,7 +148,6 @@ public:
      */
     static std::ostream& writeFloat(std::ostream& strm, double value);
 
-
     /** @brief Writes a byte binary
      *
      * @param[in, out] strm The stream to write into
@@ -164,7 +156,6 @@ public:
      */
     static std::ostream& writeByte(std::ostream& strm, unsigned char value);
 
-
     /** @brief Writes a string binary
      *
      * Writes the length of the string, first, using writeInt. Writes then the string's
@@ -177,7 +168,6 @@ public:
      */
     static std::ostream& writeString(std::ostream& strm, const std::string& value);
 
-
     /** @brief Writes a time description binary
      *
      * This method behaves differently depending on the definition of SUMOTime at compile time,
@@ -189,7 +179,6 @@ public:
      */
     static std::ostream& writeTime(std::ostream& strm, SUMOTime value);
 
-
     /** @brief Writes an edge vector binary
      *
      * @param[in, out] os The stream to write into
@@ -199,7 +188,6 @@ public:
     template <typename E>
     static std::ostream& writeEdgeVector(std::ostream& os, const std::vector<E>& edges);
 
-
     /** @brief Reads an edge vector binary
      *
      * @param[in] is The stream to read from
@@ -208,9 +196,7 @@ public:
      */
     template <typename E>
     static void readEdgeVector(std::istream& in, std::vector<const E*>& edges, const std::string& rid);
-    //@}
-
-
+    /// @}
 };
 
 
@@ -316,7 +302,3 @@ void FileHelpers::readEdgeVector(std::istream& in, std::vector<const E*>& edges,
         }
     }
 }
-#endif
-
-/****************************************************************************/
-
