@@ -1,49 +1,40 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RandHelper.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Tue, 29.05.2005
-/// @version $Id: RandHelper.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 //
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
+#include <ctime>
 #include <utils/options/OptionsCont.h>
 #include <utils/common/SysUtils.h>
 #include "RandHelper.h"
-#include <ctime>
-#include <cmath>
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
 
 
 // ===========================================================================
 // static member variables
 // ===========================================================================
-MTRand RandHelper::myRandomNumberGenerator;
+std::mt19937 RandHelper::myRandomNumberGenerator;
+#ifdef DEBUG_RANDCALLS
+std::map<std::mt19937*, int> RandHelper::myCallCount;
+std::map<std::mt19937*, int> RandHelper::myRngId;
+int RandHelper::myDebugIndex(7);
+#endif
 
 
 // ===========================================================================
@@ -64,25 +55,28 @@ RandHelper::insertRandOptions() {
     oc.addDescription("seed", "Random Number", "Initialises the random number generator with the given value");
 }
 
+
 void
-RandHelper::initRandGlobal(MTRand* which) {
-    OptionsCont& oc = OptionsCont::getOptions();
-    if (which == 0) {
+RandHelper::initRand(std::mt19937* which, const bool random, const int seed) {
+    if (which == nullptr) {
         which = &myRandomNumberGenerator;
     }
-    if (oc.getBool("random")) {
-#ifdef _MSC_VER
-        long s = myRandomNumberGenerator.hash(time(NULL), clock()) + SysUtils::getWindowsTicks();
-        unsigned int s2 = (unsigned int)(s & 0xffff) ^ (s >> 16);
-        which->seed(s2);
-#else
-        which->seed();
+#ifdef DEBUG_RANDCALLS
+    myRngId[which] = myRngId.size();
 #endif
+    if (random) {
+        which->seed((unsigned long)time(nullptr));
     } else {
-        which->seed(oc.getInt("seed"));
+        which->seed(seed);
     }
 }
 
 
-/****************************************************************************/
+void
+RandHelper::initRandGlobal(std::mt19937* which) {
+    OptionsCont& oc = OptionsCont::getOptions();
+    initRand(which, oc.getBool("random"), oc.getInt("seed"));
+}
 
+
+/****************************************************************************/
