@@ -1,37 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    SUMOTime.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Fri, 29.04.2005
-/// @version $Id: SUMOTime.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Variables, methods, and tools for internal time representation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef SUMOTime_h
-#define SUMOTime_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
-
 #include <limits>
 #include <string>
 #include "UtilExceptions.h"
@@ -63,20 +52,26 @@ extern SUMOTime DELTA_T;
 #define SPEED2ACCEL(x) ((x)/TS)
 
 #define STEPS2TIME(x) (static_cast<double>((x)/1000.))
-#define TIME2STEPS(x) (static_cast<SUMOTime>((x)*1000))
+// static cast to long long int truncates so we must pad away from 0 for correct rounding
+#define TIME2STEPS(x) (static_cast<SUMOTime>((x) * 1000. + ((x) >= 0 ? 0.5 : -0.5)))
 #define STEPFLOOR(x) (int(x/DELTA_T)*DELTA_T)
 #define STEPS2MS(x) (x)
 
+#define SIMSTEP MSNet::getInstance()->getCurrentTimeStep()
 #define SIMTIME STEPS2TIME(MSNet::getInstance()->getCurrentTimeStep())
 
 // ===========================================================================
 // method declarations
 // ===========================================================================
+
+/// @brief convert string to SUMOTime
 SUMOTime string2time(const std::string& r);
+
+/// @brief convert SUMOTime to string
 std::string time2string(SUMOTime t);
 
+/// @brief convert ms to string for log output
+std::string elapsedMs2string(long long int t);
 
-#endif
-
-/****************************************************************************/
-
+/// @brief check if given SUMOTime is multiple of the step length
+bool checkStepLengthMultiple(const SUMOTime t, const std::string& error = "", SUMOTime deltaT = DELTA_T);
