@@ -1,55 +1,44 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    SUMOVehicleClass.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
+/// @author  Laura Bieker-Walz
 /// @date    2006-01-24
-/// @version $Id: SUMOVehicleClass.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Definitions of SUMO vehicle classes and helper functions
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
 #include "SUMOVehicleClass.h"
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/StringTokenizer.h>
 #include <utils/iodevices/OutputDevice.h>
-
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
+#include <utils/xml/SUMOSAXAttributes.h>
 
 
 // ===========================================================================
 // static members
 // ===========================================================================
-StringBijection<SUMOVehicleClass>::Entry sumoVehicleClassStringInitializer[] = {
+
+static StringBijection<SUMOVehicleClass>::Entry sumoVehicleClassStringInitializer[] = {
     {"ignoring",          SVC_IGNORING},
     {"private",           SVC_PRIVATE},
     {"public_emergency",  SVC_EMERGENCY}, // !!! deprecated
@@ -75,7 +64,7 @@ StringBijection<SUMOVehicleClass>::Entry sumoVehicleClassStringInitializer[] = {
     {"rail_urban",        SVC_RAIL_URBAN},
     {"rail_slow",         SVC_RAIL}, // !!! deprecated
     {"rail",              SVC_RAIL},
-    {"rail_fast",         SVC_RAIL_ELECTRIC}, // !!! deprecated
+    {"rail_fast",         SVC_RAIL_FAST},
     {"rail_electric",     SVC_RAIL_ELECTRIC},
     {"motorcycle",        SVC_MOTORCYCLE},
     {"moped",             SVC_MOPED},
@@ -87,13 +76,15 @@ StringBijection<SUMOVehicleClass>::Entry sumoVehicleClassStringInitializer[] = {
     {"custom2",           SVC_CUSTOM2}
 };
 
+
 StringBijection<SUMOVehicleClass> SumoVehicleClassStrings(
     sumoVehicleClassStringInitializer, SVC_CUSTOM2, false);
 
+
 std::set<std::string> deprecatedVehicleClassesSeen;
 
 
-StringBijection<SUMOVehicleShape>::Entry sumoVehicleShapeStringInitializer[] = {
+static StringBijection<SUMOVehicleShape>::Entry sumoVehicleShapeStringInitializer[] = {
     {"pedestrian",            SVS_PEDESTRIAN},
     {"bicycle",               SVS_BICYCLE},
     {"moped",                 SVS_MOPED},
@@ -127,6 +118,9 @@ StringBijection<SUMOVehicleShape>::Entry sumoVehicleShapeStringInitializer[] = {
     {"ant",                   SVS_ANT},
     {"ship",                  SVS_SHIP},
     {"emergency",             SVS_EMERGENCY},
+    {"firebrigade",           SVS_FIREBRIGADE},
+    {"police",                SVS_POLICE},
+    {"rickshaw",              SVS_RICKSHAW },
     {"",                      SVS_UNKNOWN}
 };
 
@@ -134,42 +128,75 @@ StringBijection<SUMOVehicleShape>::Entry sumoVehicleShapeStringInitializer[] = {
 StringBijection<SUMOVehicleShape> SumoVehicleShapeStrings(
     sumoVehicleShapeStringInitializer, SVS_UNKNOWN, false);
 
+// ===========================================================================
+// static values used for cached
+// ===========================================================================
+
+static std::map<int, std::vector<std::string> > vehicleClassNamesListCached;
+static std::map<std::string, SVCPermissions> parseVehicleClassesCached;
+static std::map<SVCPermissions, std::string> getVehicleClassNamesCached;
+static std::string vehicleClassNameAll = "all";
 
 // ===========================================================================
 // additional constants
 // ===========================================================================
 
-const int SUMOVehicleClass_MAX = SVC_CUSTOM2;
-const SVCPermissions SVCAll = 2 * SUMOVehicleClass_MAX - 1; // all relevant bits set to 1
+const SUMOVehicleClass SUMOVehicleClass_MAX = SVC_CUSTOM2;
+
+const SVCPermissions SVCAll = 2 * (int)SUMOVehicleClass_MAX - 1; // all relevant bits set to 1
+
 const SVCPermissions SVC_UNSPECIFIED = -1;
 
+const std::string DEFAULT_VTYPE_ID("DEFAULT_VEHTYPE");
+
+const std::string DEFAULT_PEDTYPE_ID("DEFAULT_PEDTYPE");
+
+const std::string DEFAULT_BIKETYPE_ID("DEFAULT_BIKETYPE");
+
+const std::string DEFAULT_CONTAINERTYPE_ID("DEFAULT_CONTAINERTYPE");
+
+const std::string DEFAULT_TAXITYPE_ID("DEFAULT_TAXITYPE");
+
+const double DEFAULT_VEH_PROB(1.);
+
+const double DEFAULT_PEDESTRIAN_SPEED(5. / 3.6);
+
+const double DEFAULT_CONTAINER_TRANSHIP_SPEED(5. / 3.6);
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
 // ------------ Conversion of SUMOVehicleClass
 
-std::string
-getVehicleClassNames(SVCPermissions permissions) {
-    if (permissions == SVCAll) {
-        return "all";
+const std::string&
+getVehicleClassNames(SVCPermissions permissions, bool expand) {
+    if (permissions == SVCAll && !expand) {
+        return vehicleClassNameAll;
     }
-    return joinToString(getVehicleClassNamesList(permissions), ' ');
+    // check if previously was cached
+    if (getVehicleClassNamesCached.count(permissions) == 0) {
+        getVehicleClassNamesCached[permissions] = joinToString(getVehicleClassNamesList(permissions), ' ');
+    }
+    return getVehicleClassNamesCached.at(permissions);
 }
 
 
-std::vector<std::string>
+const std::vector<std::string>&
 getVehicleClassNamesList(SVCPermissions permissions) {
-    /// @todo cache values?
-    const std::vector<std::string> classNames = SumoVehicleClassStrings.getStrings();
-    std::vector<std::string> result;
-    for (std::vector<std::string>::const_iterator it = classNames.begin(); it != classNames.end(); it++) {
-        const int svc = (int)SumoVehicleClassStrings.get(*it);
-        if ((svc & permissions) == svc && svc != SVC_IGNORING) {
-            result.push_back(*it);
+    // first check if it's cached
+    if (vehicleClassNamesListCached.count(permissions) == 0) {
+        const std::vector<std::string> classNames = SumoVehicleClassStrings.getStrings();
+        std::vector<std::string> result;
+        for (std::vector<std::string>::const_iterator it = classNames.begin(); it != classNames.end(); it++) {
+            const int svc = (int)SumoVehicleClassStrings.get(*it);
+            if ((svc & permissions) == svc && svc != SVC_IGNORING) {
+                result.push_back(*it);
+            }
         }
+        // add it into vehicleClassNamesListCached
+        vehicleClassNamesListCached[permissions] = result;
     }
-    return result;
+    return vehicleClassNamesListCached.at(permissions);
 }
 
 
@@ -200,18 +227,27 @@ parseVehicleClasses(const std::string& allowedS) {
     if (allowedS == "all") {
         return SVCAll;
     }
-    SVCPermissions result = 0;
-    StringTokenizer sta(allowedS, " ");
-    while (sta.hasNext()) {
-        const std::string s = sta.next();
-        const SUMOVehicleClass vc = getVehicleClassID(s);
-        const std::string& realName = SumoVehicleClassStrings.getString(vc);
-        if (realName != s) {
-            deprecatedVehicleClassesSeen.insert(s);
+    // check  if allowedS was previously cached
+    if (parseVehicleClassesCached.count(allowedS) == 0) {
+        SVCPermissions result = 0;
+        StringTokenizer sta(allowedS, " ");
+        while (sta.hasNext()) {
+            const std::string s = sta.next();
+            if (!SumoVehicleClassStrings.hasString(s)) {
+                WRITE_ERROR("Unknown vehicle class '" + s + "' encountered.");
+            } else {
+                const SUMOVehicleClass vc = getVehicleClassID(s);
+                const std::string& realName = SumoVehicleClassStrings.getString(vc);
+                if (realName != s) {
+                    deprecatedVehicleClassesSeen.insert(s);
+                }
+                result |= vc;
+            }
         }
-        result |= vc;
+        // save parsed vehicle class cached
+        parseVehicleClassesCached[allowedS] = result;
     }
-    return result;
+    return parseVehicleClassesCached.at(allowedS);
 }
 
 
@@ -220,6 +256,10 @@ canParseVehicleClasses(const std::string& classes) {
     if (classes == "all") {
         return true;
     }
+    // check if was previously cached
+    if (parseVehicleClassesCached.count(classes) != 0) {
+        return true;
+    }
     StringTokenizer sta(classes, " ");
     while (sta.hasNext()) {
         if (!SumoVehicleClassStrings.hasString(sta.next())) {
@@ -230,7 +270,8 @@ canParseVehicleClasses(const std::string& classes) {
 }
 
 
-extern SVCPermissions parseVehicleClasses(const std::string& allowedS, const std::string& disallowedS) {
+SVCPermissions
+parseVehicleClasses(const std::string& allowedS, const std::string& disallowedS, double networkVersion) {
     if (allowedS.size() == 0 && disallowedS.size() == 0) {
         return SVCAll;
     } else if (allowedS.size() > 0 && disallowedS.size() > 0) {
@@ -239,14 +280,23 @@ extern SVCPermissions parseVehicleClasses(const std::string& allowedS, const std
     } else if (allowedS.size() > 0) {
         return parseVehicleClasses(allowedS);
     } else {
-        return SVCAll & ~parseVehicleClasses(disallowedS);
+        return invertPermissions(parseVehicleClasses(disallowedS) | (networkVersion < 1.3 ? SVC_RAIL_FAST : 0));
     }
 }
 
 
+SVCPermissions
+invertPermissions(SVCPermissions permissions) {
+    return SVCAll & ~permissions;
+}
+
+
 SVCPermissions
 parseVehicleClasses(const std::vector<std::string>& allowedS) {
     SVCPermissions result = 0;
+    if (std::find(allowedS.begin(), allowedS.end(), "all") != allowedS.end()) {
+        return SVCAll;
+    }
     for (std::vector<std::string>::const_iterator i = allowedS.begin(); i != allowedS.end(); ++i) {
         const SUMOVehicleClass vc = getVehicleClassID(*i);
         const std::string& realName = SumoVehicleClassStrings.getString(vc);
@@ -267,7 +317,7 @@ writePermissions(OutputDevice& into, SVCPermissions permissions) {
         into.writeAttr(SUMO_ATTR_DISALLOW, "all");
         return;
     } else {
-        size_t num_allowed = 0;
+        int num_allowed = 0;
         for (int mask = 1; mask <= SUMOVehicleClass_MAX; mask = mask << 1) {
             if ((mask & permissions) == mask) {
                 ++num_allowed;
@@ -302,6 +352,12 @@ getVehicleShapeID(const std::string& name) {
 }
 
 
+bool
+canParseVehicleShape(const std::string& shape) {
+    return SumoVehicleShapeStrings.hasString(shape);
+}
+
+
 std::string
 getVehicleShapeName(SUMOVehicleShape id) {
     return SumoVehicleShapeStrings.getString(id);
@@ -309,28 +365,102 @@ getVehicleShapeName(SUMOVehicleShape id) {
 
 
 bool isRailway(SVCPermissions permissions) {
-    return (permissions & (SVC_RAIL_ELECTRIC | SVC_RAIL | SVC_RAIL_URBAN | SVC_TRAM)) > 0 && (permissions & SVC_PASSENGER) == 0;
+    return (permissions & SVC_RAIL_CLASSES) > 0 && (permissions & SVC_PASSENGER) == 0;
+}
+
+bool isTram(SVCPermissions permissions) {
+    return (permissions & SVC_RAIL_CLASSES) == SVC_TRAM && (permissions & SVC_PASSENGER) == 0;
 }
 
 
-bool isWaterway(SVCPermissions permissions) {
+bool
+isWaterway(SVCPermissions permissions) {
     return permissions == SVC_SHIP;
 }
 
 
-bool isForbidden(SVCPermissions permissions) {
+bool
+isForbidden(SVCPermissions permissions) {
     return (permissions & SVCAll) == 0;
 }
 
 
-const std::string DEFAULT_VTYPE_ID("DEFAULT_VEHTYPE");
-const std::string DEFAULT_PEDTYPE_ID("DEFAULT_PEDTYPE");
+bool
+isSidewalk(SVCPermissions permissions) {
+    return (permissions & SVCAll) == SVC_PEDESTRIAN;
+}
 
-const SUMOReal DEFAULT_VEH_PROB(1.);
 
-const SUMOReal DEFAULT_PEDESTRIAN_SPEED(5. / 3.6);
+bool
+noVehicles(SVCPermissions permissions) {
+    return isForbidden(permissions) || isSidewalk(permissions);
+}
+
+
+std::map<SVCPermissions, double> parseStopOffsets(const SUMOSAXAttributes& attrs, bool& ok) {
+    const std::string vClasses = attrs.getOpt<std::string>(SUMO_ATTR_VCLASSES, nullptr, ok, "");
+    const std::string exceptions = attrs.getOpt<std::string>(SUMO_ATTR_EXCEPTIONS, nullptr, ok, "");
+    if (attrs.hasAttribute(SUMO_ATTR_VCLASSES) && attrs.hasAttribute(SUMO_ATTR_EXCEPTIONS)) {
+        WRITE_ERROR("Simultaneous specification of vClasses and exceptions is not allowed!");
+        ok = false;
+        return std::map<SVCPermissions, double>();
+    }
+    const double value = attrs.get<double>(SUMO_ATTR_VALUE, nullptr, ok);
+
+    int vClassBitset;
+    if (attrs.hasAttribute(SUMO_ATTR_VCLASSES)) {
+        vClassBitset = parseVehicleClasses(vClasses);
+    } else if (attrs.hasAttribute(SUMO_ATTR_EXCEPTIONS)) {
+        vClassBitset = ~parseVehicleClasses(exceptions);
+    } else {
+        // no vClasses specified, thus apply to all
+        vClassBitset = parseVehicleClasses("all");
+    }
+
+    std::map<SVCPermissions, double> offsets;
+    offsets[vClassBitset] = value;
+    return offsets;
+}
+
+
+double
+getDefaultVehicleLength(const SUMOVehicleClass vc) {
+    switch (vc) {
+        case SVC_PEDESTRIAN:
+            return 0.215;
+        case SVC_BICYCLE:
+            return 1.6;
+        case SVC_MOPED:
+            return 2.1;
+        case SVC_MOTORCYCLE:
+            return 2.2;
+        case SVC_TRUCK:
+            return 7.1;
+        case SVC_TRAILER:
+            return 16.5;
+        case SVC_BUS:
+            return 12.;
+        case SVC_COACH:
+            return 14.;
+        case SVC_TRAM:
+            return 22.;
+        case SVC_RAIL_URBAN:
+            return 36.5 * 3;
+        case SVC_RAIL:
+            return 67.5 * 2;
+        case SVC_RAIL_ELECTRIC:
+        case SVC_RAIL_FAST:
+            return 25. * 8;
+        case SVC_DELIVERY:
+        case SVC_EMERGENCY:
+            return 6.5;
+        case SVC_SHIP:
+            return 17;
+        default:
+            return 5; /*4.3*/
+    }
+}
+
 
-const SUMOReal DEFAULT_CONTAINER_TRANSHIP_SPEED(5. / 3.6);
 
 /****************************************************************************/
-
