@@ -1,41 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    SUMOVehicleClass.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
+/// @author  Laura Bieker
 /// @date    2006-01-24
-/// @version $Id: SUMOVehicleClass.h 18610 2015-08-06 14:15:04Z namdre $
 ///
 // Definitions of SUMO vehicle classes and helper functions
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef SUMOVehicleClass_h
-#define SUMOVehicleClass_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
-
 #include <string>
 #include <set>
 #include <limits>
+#include <utils/common/StdDefs.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/StringBijection.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
@@ -45,7 +36,7 @@
 // class declarations
 // ===========================================================================
 class OutputDevice;
-
+class SUMOSAXAttributes;
 
 // ===========================================================================
 // enum definitions
@@ -75,6 +66,8 @@ enum SUMOVehicleShape {
     SVS_PASSENGER_WAGON,
     /// @brief render as a van
     SVS_PASSENGER_VAN,
+    /// @brief automated car (with cruise controllers)
+    //SVS_PASSENGER_AUTOMATED,
     /// @brief render as a delivery vehicle
     SVS_DELIVERY,
     /// @brief render as a transport vehicle
@@ -102,7 +95,15 @@ enum SUMOVehicleShape {
     /// @brief render as a giant ant
     SVS_ANT,
     /// @brief render as a arbitrary ship
-    SVS_SHIP
+    SVS_SHIP,
+    /// @brief render as an emergency vehicle
+    SVS_EMERGENCY,
+    /// @brief render as a fire brigade
+    SVS_FIREBRIGADE,
+    /// @brief render as a police car
+    SVS_POLICE,
+    /// @brief render as a rickshaw
+    SVS_RICKSHAW
 };
 
 
@@ -149,65 +150,76 @@ enum SUMOVehicleClass {
 
     /// @name vehicle size
     //@{
+    /// @brief pedestrian
+    SVC_PEDESTRIAN = 1 << 5,
 
     /// @brief vehicle is a passenger car (a "normal" car)
-    SVC_PASSENGER = 1 << 5,
+    SVC_PASSENGER = 1 << 6,
     /// @brief vehicle is a HOV
-    SVC_HOV = 1 << 6,
+    SVC_HOV = 1 << 7,
     /// @brief vehicle is a taxi
-    SVC_TAXI = 1 << 7,
+    SVC_TAXI = 1 << 8,
     /// @brief vehicle is a bus
-    SVC_BUS = 1 << 8,
+    SVC_BUS = 1 << 9,
     /// @brief vehicle is a coach
-    SVC_COACH = 1 << 9,
+    SVC_COACH = 1 << 10,
     /// @brief vehicle is a small delivery vehicle
-    SVC_DELIVERY = 1 << 10,
+    SVC_DELIVERY = 1 << 11,
     /// @brief vehicle is a large transport vehicle
-    SVC_TRUCK = 1 << 11,
+    SVC_TRUCK = 1 << 12,
     /// @brief vehicle is a large transport vehicle
-    SVC_TRAILER = 1 << 12,
-    /// @brief vehicle is a light rail
-    SVC_TRAM = 1 << 13,
-    /// @brief vehicle is a city rail
-    SVC_RAIL_URBAN = 1 << 14,
-    /// @brief vehicle is a not electrified rail
-    SVC_RAIL = 1 << 15,
-    /// @brief vehicle is a (possibly fast moving) electric rail
-    SVC_RAIL_ELECTRIC = 1 << 16,
-
+    SVC_TRAILER = 1 << 13,
     /// @brief vehicle is a motorcycle
-    SVC_MOTORCYCLE = 1 << 17,
+    SVC_MOTORCYCLE = 1 << 14,
     /// @brief vehicle is a moped
-    SVC_MOPED = 1 << 18,
+    SVC_MOPED = 1 << 15,
     /// @brief vehicle is a bicycle
-    SVC_BICYCLE = 1 << 19,
-    /// @brief is a pedestrian
-    SVC_PEDESTRIAN = 1 << 20,
+    SVC_BICYCLE = 1 << 16,
     /// @brief is an electric vehicle
-    SVC_E_VEHICLE = 1 << 21,
+    SVC_E_VEHICLE = 1 << 17,
+
+    /// @brief vehicle is a light rail
+    SVC_TRAM = 1 << 18,
+    /// @brief vehicle is a city rail
+    SVC_RAIL_URBAN = 1 << 19,
+    /// @brief vehicle is a not electrified rail
+    SVC_RAIL = 1 << 20,
+    /// @brief rail vehicle that requires electrified tracks
+    SVC_RAIL_ELECTRIC = 1 << 21,
+    /// @brief vehicle that is allowed to drive on high-speed rail tracks
+    SVC_RAIL_FAST = 1 << 22,
+
     /// @brief is an arbitrary ship
-    SVC_SHIP = 1 << 22,
+    SVC_SHIP = 1 << 23,
+
     /// @brief is a user-defined type
-    SVC_CUSTOM1 = 1 << 23,
+    SVC_CUSTOM1 = 1 << 24,
     /// @brief is a user-defined type
-    SVC_CUSTOM2 = 1 << 24,
+    SVC_CUSTOM2 = 1 << 25,
     //@}
 
+    /// @brief classes which drive on tracks
+    SVC_RAIL_CLASSES = SVC_RAIL_ELECTRIC | SVC_RAIL_FAST | SVC_RAIL | SVC_RAIL_URBAN | SVC_TRAM,
+    /// @brief classes which drive on roads
+    SVC_ROAD_CLASSES = (SVC_PEDESTRIAN | SVC_PASSENGER | SVC_HOV | SVC_TAXI | SVC_BUS | SVC_COACH | SVC_DELIVERY
+                        | SVC_TRUCK | SVC_TRAILER | SVC_MOTORCYCLE | SVC_MOPED | SVC_BICYCLE | SVC_E_VEHICLE),
     /// @brief classes which (normally) do not drive on normal roads
-    SVC_NON_ROAD = SVC_TRAM | SVC_RAIL | SVC_RAIL_URBAN | SVC_RAIL_ELECTRIC | SVC_SHIP
+    SVC_NON_ROAD = SVC_RAIL_CLASSES | SVC_SHIP
 };
 
-extern const int SUMOVehicleClass_MAX;
+extern const SUMOVehicleClass SUMOVehicleClass_MAX;
 extern StringBijection<SUMOVehicleClass> SumoVehicleClassStrings;
 extern std::set<std::string> deprecatedVehicleClassesSeen;
 extern StringBijection<SUMOVehicleShape> SumoVehicleShapeStrings;
 
-/* @brief bitset where each bit declares whether a certain SVC may use this edge/lane
- */
+/// @brief bitset where each bit declares whether a certain SVC may use this edge/lane
 typedef int SVCPermissions;
-extern const SVCPermissions SVCAll; // everything allowed
-extern const SVCPermissions SVC_UNSPECIFIED; // permissions not specified
 
+/// @brief all VClasses are allowed
+extern const SVCPermissions SVCAll;
+
+/// @brief permissions not specified
+extern const SVCPermissions SVC_UNSPECIFIED;
 
 /**
  * @enum SUMOEmissionClass
@@ -226,17 +238,16 @@ typedef int SUMOEmissionClass;
 // ---------------------------------------------------------------------------
 /** @brief Returns the ids of the given classes, divided using a ' '
  * @param[in] the permissions to encode
+ * @param[in] expand whether 'all' should be used
  * @return The string representation of these classes
  */
-extern std::string getVehicleClassNames(SVCPermissions permissions);
-
+extern const std::string& getVehicleClassNames(SVCPermissions permissions, bool expand = false);
 
 /** @brief Returns the ids of the given classes, divided using a ' '
  * @param[in] the permissions to encode
  * @return The string representation of these classes as a vector
  */
-extern std::vector<std::string> getVehicleClassNamesList(SVCPermissions permissions);
-
+extern const std::vector<std::string>& getVehicleClassNamesList(SVCPermissions permissions);
 
 /** @brief Returns the class id of the abstract class given by its name
  * @param[in] name The name of the abstract vehicle class
@@ -260,17 +271,14 @@ extern int getVehicleClassCompoundID(const std::string& name);
  */
 extern SVCPermissions parseVehicleClasses(const std::string& allowedS);
 
-
-/** @brief Checks whether the given string contains only known vehicle classes
- */
+/// @brief Checks whether the given string contains only known vehicle classes
 extern bool canParseVehicleClasses(const std::string& classes);
 
 /** @brief Encodes the given vector of allowed and disallowed classes into a bitset
  * @param[in] allowedS Definition which classes are allowed
  * @param[in] disallowedS Definition which classes are not allowed
  */
-extern SVCPermissions parseVehicleClasses(const std::string& allowedS, const std::string& disallowedS);
-
+extern SVCPermissions parseVehicleClasses(const std::string& allowedS, const std::string& disallowedS, double networkVersion = NETWORK_VERSION);
 
 /** @brief Encodes the given vector of allowed classs into a bitset
  * Unlike the methods which parse a string it gives immediately a warning output on deprecated vehicle classes.
@@ -278,6 +286,8 @@ extern SVCPermissions parseVehicleClasses(const std::string& allowedS, const std
  */
 extern SVCPermissions parseVehicleClasses(const std::vector<std::string>& allowedS);
 
+/// @brief negate the given permissions and ensure that only relevant bits are set
+extern SVCPermissions invertPermissions(SVCPermissions permissions);
 
 /// @brief writes allowed disallowed attributes if needed;
 extern void writePermissions(OutputDevice& into, SVCPermissions permissions);
@@ -285,6 +295,9 @@ extern void writePermissions(OutputDevice& into, SVCPermissions permissions);
 /// @brief writes allowed disallowed attributes if needed;
 extern void writePreferences(OutputDevice& into, SVCPermissions preferred);
 
+/// @brief Extract stopOffsets from attributes of stopOffset element
+extern std::map<SVCPermissions, double> parseStopOffsets(const SUMOSAXAttributes& attrs, bool& ok);
+
 // ---------------------------------------------------------------------------
 // vehicle shape class
 // ---------------------------------------------------------------------------
@@ -294,13 +307,14 @@ extern void writePreferences(OutputDevice& into, SVCPermissions preferred);
  */
 extern std::string getVehicleShapeName(SUMOVehicleShape id);
 
-
 /** @brief Returns the class id of the shape class given by its name
  * @param[in] name The name of the shape class
  * @return The internal representation of this class
  */
 extern SUMOVehicleShape getVehicleShapeID(const std::string& name);
 
+/// @brief Checks whether the given string contains only known vehicle shape
+extern bool canParseVehicleShape(const std::string& shape);
 
 /** @brief Returns whether an edge with the given permission is a railway edge
  * @param[in] permissions The permissions of the edge
@@ -308,6 +322,12 @@ extern SUMOVehicleShape getVehicleShapeID(const std::string& name);
  */
 extern bool isRailway(SVCPermissions permissions);
 
+/** @brief Returns whether an edge with the given permission is a tram edge
+ * @param[in] permissions The permissions of the edge
+ * @return Whether the edge is a tram edge
+ */
+extern bool isTram(SVCPermissions permissions);
+
 /** @brief Returns whether an edge with the given permission is a waterway edge
  * @param[in] permissions The permissions of the edge
  * @return Whether the edge is a waterway edge
@@ -320,19 +340,36 @@ extern bool isWaterway(SVCPermissions permissions);
  */
 extern bool isForbidden(SVCPermissions permissions);
 
+/** @brief Returns whether an edge with the given permission is a sidewalk
+ * @param[in] permissions The permissions of the edge
+ * @return Whether the edge is a sidewalk
+ */
+extern bool isSidewalk(SVCPermissions permissions);
+
+/** @brief Returns whether an edge with the given permission forbids vehicles
+ * @param[in] permissions The permissions of the edge
+ * @return Whether the edge is forbidden for vehicles
+ */
+extern bool noVehicles(SVCPermissions permissions);
+
+/** @brief Returns the default vehicle length
+ * This put into a function so it can be used by NBVehicle
+ * @param[in] vc the vehicle class
+ * @return the default length in m
+ */
+extern double getDefaultVehicleLength(const SUMOVehicleClass vc = SVC_IGNORING);
+
 // ---------------------------------------------------------------------------
 // default vehicle type parameter
 // ---------------------------------------------------------------------------
 extern const std::string DEFAULT_VTYPE_ID;
 extern const std::string DEFAULT_PEDTYPE_ID;
+extern const std::string DEFAULT_BIKETYPE_ID;
+extern const std::string DEFAULT_CONTAINERTYPE_ID;
+extern const std::string DEFAULT_TAXITYPE_ID;
 
-extern const SUMOReal DEFAULT_VEH_PROB; // !!! does this belong here?
+extern const double DEFAULT_VEH_PROB; // !!! does this belong here?
 
-extern const SUMOReal DEFAULT_PEDESTRIAN_SPEED;
-
-extern const SUMOReal DEFAULT_CONTAINER_TRANSHIP_SPEED;
-
-#endif
-
-/****************************************************************************/
+extern const double DEFAULT_PEDESTRIAN_SPEED;
 
+extern const double DEFAULT_CONTAINER_TRANSHIP_SPEED;
