@@ -1,45 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    StringTokenizer.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    ?
-/// @version $Id: StringTokenizer.cpp 21217 2016-07-22 10:57:44Z behrisch $
 ///
 // A java-style StringTokenizer for c++ (stl)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
 #include <iostream> // !!! debug only
+
 #include "UtilExceptions.h"
 #include "StringTokenizer.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // variable definitions
@@ -47,29 +35,39 @@
 const int StringTokenizer::NEWLINE = -256;
 const int StringTokenizer::WHITECHARS = -257;
 const int StringTokenizer::SPACE = 32;
+const int StringTokenizer::TAB = 9;
 
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
-StringTokenizer::StringTokenizer(std::string tosplit)
-    : myTosplit(tosplit), myPos(0) {
+
+StringTokenizer::StringTokenizer() :
+    myPos(0) {
+}
+
+
+StringTokenizer::StringTokenizer(std::string tosplit) :
+    myTosplit(tosplit), myPos(0) {
     prepareWhitechar(tosplit);
 }
 
 
-StringTokenizer::StringTokenizer(std::string tosplit, std::string token, bool splitAtAllChars)
-    : myTosplit(tosplit), myPos(0) {
+StringTokenizer::StringTokenizer(std::string tosplit, std::string token, bool splitAtAllChars) :
+    myTosplit(tosplit), myPos(0) {
     prepare(tosplit, token, splitAtAllChars);
 }
 
 
-StringTokenizer::StringTokenizer(std::string tosplit, int special)
-    : myTosplit(tosplit), myPos(0) {
+StringTokenizer::StringTokenizer(std::string tosplit, int special) :
+    myTosplit(tosplit), myPos(0) {
     switch (special) {
         case NEWLINE:
             prepare(tosplit, "\r\n", true);
             break;
+        case TAB:
+            prepare(tosplit, "\t", true);
+            break;
         case WHITECHARS:
             prepareWhitechar(tosplit);
             break;
@@ -86,14 +84,17 @@ StringTokenizer::StringTokenizer(std::string tosplit, int special)
 
 StringTokenizer::~StringTokenizer() {}
 
+
 void StringTokenizer::reinit() {
     myPos = 0;
 }
 
+
 bool StringTokenizer::hasNext() {
-    return myPos != myStarts.size();
+    return myPos != (int)myStarts.size();
 }
 
+
 std::string StringTokenizer::next() {
     if (myPos >= (int)myStarts.size()) {
         throw OutOfBoundsException();
@@ -107,6 +108,7 @@ std::string StringTokenizer::next() {
     return myTosplit.substr(start, length);
 }
 
+
 std::string StringTokenizer::front() {
     if (myStarts.size() == 0) {
         throw OutOfBoundsException();
@@ -117,6 +119,7 @@ std::string StringTokenizer::front() {
     return myTosplit.substr(myStarts[0], myLengths[0]);
 }
 
+
 std::string StringTokenizer::get(int pos) const {
     if (pos >= (int)myStarts.size()) {
         throw OutOfBoundsException();
@@ -134,6 +137,7 @@ int StringTokenizer::size() const {
     return (int)myStarts.size();
 }
 
+
 void StringTokenizer::prepare(const std::string& tosplit, const std::string& token, bool splitAtAllChars) {
     int beg = 0;
     int len = (int)token.length();
@@ -153,13 +157,14 @@ void StringTokenizer::prepare(const std::string& tosplit, const std::string& tok
         myStarts.push_back(beg);
         myLengths.push_back((int)end - beg);
         beg = (int)end + len;
-        if (beg == tosplit.length()) {
+        if (beg == (int)tosplit.length()) {
             myStarts.push_back(beg - 1);
             myLengths.push_back(0);
         }
     }
 }
 
+
 void StringTokenizer::prepareWhitechar(const std::string& tosplit) {
     std::string::size_type len = tosplit.length();
     std::string::size_type beg = 0;
@@ -167,12 +172,12 @@ void StringTokenizer::prepareWhitechar(const std::string& tosplit) {
         beg++;
     }
     while (beg != std::string::npos && beg < len) {
-        int end = (int)beg;
+        std::string::size_type end = beg;
         while (end < len && tosplit[end] > SPACE) {
             end++;
         }
         myStarts.push_back((int)beg);
-        myLengths.push_back(end - (int)beg);
+        myLengths.push_back((int)end - (int)beg);
         beg = end;
         while (beg < len && tosplit[beg] <= SPACE) {
             beg++;
@@ -180,6 +185,7 @@ void StringTokenizer::prepareWhitechar(const std::string& tosplit) {
     }
 }
 
+
 std::vector<std::string>
 StringTokenizer::getVector() {
     std::vector<std::string> ret;
@@ -192,6 +198,4 @@ StringTokenizer::getVector() {
 }
 
 
-
 /****************************************************************************/
-
