@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    SystemFrame.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 23.06.2003
-/// @version $Id: SystemFrame.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A set of actions common to all applications
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "SystemFrame.h"
 #include <string>
@@ -52,7 +43,7 @@ SystemFrame::addConfigurationOptions(OptionsCont& oc) {
     oc.addDescription("configuration-file", "Configuration", "Loads the named config on startup");
     oc.addXMLDefault("configuration-file");
 
-    oc.doRegister("save-configuration", new Option_FileName());
+    oc.doRegister("save-configuration", 'C', new Option_FileName());
     oc.addSynonyme("save-config", "save-configuration");
     oc.addDescription("save-configuration", "Configuration", "Saves current configuration into FILE");
 
@@ -78,8 +69,8 @@ SystemFrame::addReportOptions(OptionsCont& oc) {
     oc.doRegister("print-options", new Option_Bool(false));
     oc.addDescription("print-options", "Report", "Prints option values before processing");
 
-    oc.doRegister("help", '?', new Option_Bool(false));
-    oc.addDescription("help", "Report", "Prints this screen");
+    oc.doRegister("help", '?', new Option_BoolExtended(false));
+    oc.addDescription("help", "Report", "Prints this screen or selected topics");
 
     oc.doRegister("version", 'V', new Option_Bool(false));
     oc.addDescription("version", "Report", "Prints the current version");
@@ -87,13 +78,23 @@ SystemFrame::addReportOptions(OptionsCont& oc) {
     oc.doRegister("xml-validation", 'X', new Option_String("auto"));
     oc.addDescription("xml-validation", "Report", "Set schema validation scheme of XML inputs (\"never\", \"auto\" or \"always\")");
 
-    oc.doRegister("xml-validation.net", new Option_String("never"));
-    oc.addDescription("xml-validation.net", "Report", "Set schema validation scheme of SUMO network inputs (\"never\", \"auto\" or \"always\")");
+    if (oc.exists("net-file")) {
+        oc.doRegister("xml-validation.net", new Option_String("never"));
+        oc.addDescription("xml-validation.net", "Report", "Set schema validation scheme of SUMO network inputs (\"never\", \"auto\" or \"always\")");
+    }
+
+    if (oc.exists("route-files")) {
+        oc.doRegister("xml-validation.routes", new Option_String("auto"));
+        oc.addDescription("xml-validation.routes", "Report", "Set schema validation scheme of SUMO route inputs (\"never\", \"auto\" or \"always\")");
+    }
 
     oc.doRegister("no-warnings", 'W', new Option_Bool(false));
     oc.addSynonyme("no-warnings", "suppress-warnings", true);
     oc.addDescription("no-warnings", "Report", "Disables output of warnings");
 
+    oc.doRegister("aggregate-warnings", new Option_Integer(-1));
+    oc.addDescription("aggregate-warnings", "Report", "Aggregate warnings of the same type whenever more than INT occur");
+
     oc.doRegister("log", 'l', new Option_FileName());
     oc.addSynonyme("log", "log-file");
     oc.addDescription("log", "Report", "Writes all messages to FILE (implies verbose)");
@@ -104,6 +105,9 @@ SystemFrame::addReportOptions(OptionsCont& oc) {
     oc.doRegister("error-log", new Option_FileName());
     oc.addDescription("error-log", "Report", "Writes all warnings and errors to FILE");
 
+    oc.doRegister("write-license", new Option_Bool(false));
+    oc.addDescription("write-license", "Output", "Include license info into every output file");
+
     oc.doRegister("output-prefix", new Option_String());
     oc.addDescription("output-prefix", "Output", "Prefix which is applied to all output files. The special string 'TIME' is replaced by the current time.");
 
@@ -112,6 +116,9 @@ SystemFrame::addReportOptions(OptionsCont& oc) {
 
     oc.doRegister("precision.geo", new Option_Integer(6));
     oc.addDescription("precision.geo", "Output", "Defines the number of digits after the comma for lon,lat output");
+
+    oc.doRegister("human-readable-time", 'H', new Option_Bool(false));
+    oc.addDescription("human-readable-time", "Output", "Write time values as hour:minute:second or day:hour:minute:second rather than seconds");
 }
 
 
@@ -120,12 +127,22 @@ SystemFrame::checkOptions() {
     OptionsCont& oc = OptionsCont::getOptions();
     gPrecision = oc.getInt("precision");
     gPrecisionGeo = oc.getInt("precision.geo");
+    gHumanReadableTime = oc.getBool("human-readable-time");
+    if (oc.exists("weights.random-factor")) {
+        gWeightsRandomFactor = oc.getFloat("weights.random-factor");
+    }
+    if (oc.exists("xml-validation.routes") && oc.isDefault("xml-validation.routes") && !oc.isDefault("xml-validation")) {
+        oc.set("xml-validation.routes", oc.getString("xml-validation"));
+    }
+    std::cout << std::setprecision(gPrecision);
     return true;
 }
 
 
 void
 SystemFrame::close() {
+    // flush aggregated warnings
+    MsgHandler::getWarningInstance()->clear();
     // close all output devices
     OutputDevice::closeAll();
     // close the xml-subsystem
@@ -138,4 +155,3 @@ SystemFrame::close() {
 
 
 /****************************************************************************/
-
