@@ -1,38 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    VectorHelper.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: VectorHelper.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
-// A simple vector of SUMOReals
+// A simple vector of doubles
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef VectorHelper_h
-#define VectorHelper_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
-
 #include <vector>
+#include <limits>
 #include <algorithm>
 #include <iostream>
 
@@ -95,8 +85,8 @@ public:
     }
 
     static T maxValue(const std::vector<T>& v) {
-        SUMOReal m = *(v.begin());
-        for (typename std::vector<T>::const_iterator j = v.begin() + 1; j != v.end(); j++) {
+        T m = -std::numeric_limits<T>::max();
+        for (typename std::vector<T>::const_iterator j = v.begin() ; j != v.end(); j++) {
             if ((*j) > m) {
                 m = *j;
             }
@@ -105,8 +95,8 @@ public:
     }
 
     static T minValue(const std::vector<T>& v) {
-        SUMOReal m = *(v.begin());
-        for (typename std::vector<T>::const_iterator j = v.begin() + 1; j != v.end(); j++) {
+        T m = std::numeric_limits<T>::max();
+        for (typename std::vector<T>::const_iterator j = v.begin(); j != v.end(); j++) {
             if ((*j) < m) {
                 m = *j;
             }
@@ -165,10 +155,3 @@ std::ostream& operator<<(std::ostream& os, const std::vector<T>& v) {
     }
     return os;
 }
-
-
-
-#endif
-
-/****************************************************************************/
-
