@@ -1,30 +1,23 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    DistributionCont.cpp
 /// @author  Daniel Krajzewicz
 /// @date    Sept 2002
-/// @version $Id: DistributionCont.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A container for distributions
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "DistributionCont.h"
 
@@ -62,12 +55,12 @@ DistributionCont::dictionary(const std::string& type,
                              const std::string& id) {
     TypedDistDict::iterator i = myDict.find(type);
     if (i == myDict.end()) {
-        return 0;
+        return nullptr;
     }
     DistDict& dict = (*i).second;
     DistDict::iterator j = dict.find(id);
     if (j == dict.end()) {
-        return 0;
+        return nullptr;
     }
     return (*j).second;
 }
@@ -78,7 +71,7 @@ DistributionCont::clear() {
     for (TypedDistDict::iterator i = myDict.begin(); i != myDict.end(); i++) {
         DistDict& dict = (*i).second;
         for (DistDict::iterator j = dict.begin(); j != dict.end(); j++) {
-            delete(*j).second;
+            delete (*j).second;
         }
     }
 }
