@@ -1,42 +1,29 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    HelpersHBEFA.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 10.05.2004
-/// @version $Id: HelpersHBEFA.cpp 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Helper methods for HBEFA-based emission computation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <utils/common/ToString.h>
 #include "HelpersHBEFA.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static definitions
@@ -144,15 +131,14 @@ HelpersHBEFA::myFunctionParameter[42][36] = {
 // ===========================================================================
 // method definitions
 // ===========================================================================
-HelpersHBEFA::HelpersHBEFA() : PollutantsInterface::Helper("HBEFA2") {
-    myEmissionClassStrings.insert("zero", PollutantsInterface::ZERO_EMISSIONS);
+HelpersHBEFA::HelpersHBEFA() : PollutantsInterface::Helper("HBEFA2", HBEFA_BASE, -1) {
     int clusterSizesH[] = {3, 6, 12};
     int clusterSizesP[] = {7, 14};
-    int index = 1;
+    int index = HBEFA_BASE;
     for (int i = 0; i < 3; i++) {
         for (int j = 0; j < clusterSizesH[i]; j++) {
             myEmissionClassStrings.insert("HDV_" + toString(clusterSizesH[i]) + "_" + toString(j + 1), index | PollutantsInterface::HEAVY_BIT);
-            myEmissionClassStrings.addAlias("hdv_" + toString(clusterSizesH[i]) + "_" + toString(j + 1), index);
+            myEmissionClassStrings.addAlias("hdv_" + toString(clusterSizesH[i]) + "_" + toString(j + 1), index | PollutantsInterface::HEAVY_BIT);
             index++;
         }
     }
@@ -166,7 +152,7 @@ HelpersHBEFA::HelpersHBEFA() : PollutantsInterface::Helper("HBEFA2") {
     for (int i = 0; i < 3; i++) {
         for (int j = 0; j < clusterSizesH[i]; j++) {
             myEmissionClassStrings.insert("HDV_A0_" + toString(clusterSizesH[i]) + "_" + toString(j + 1), index | PollutantsInterface::HEAVY_BIT);
-            myEmissionClassStrings.addAlias("hdv_a0_" + toString(clusterSizesH[i]) + "_" + toString(j + 1), index);
+            myEmissionClassStrings.addAlias("hdv_a0_" + toString(clusterSizesH[i]) + "_" + toString(j + 1), index | PollutantsInterface::HEAVY_BIT);
             index++;
         }
     }
@@ -178,6 +164,7 @@ HelpersHBEFA::HelpersHBEFA() : PollutantsInterface::Helper("HBEFA2") {
         }
     }
     myEmissionClassStrings.addAlias("unknown", myEmissionClassStrings.get("P_7_7"));
+    myEmissionClassStrings.addAlias("default", myEmissionClassStrings.get("P_7_7"));
 }
 
 
