@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    HelpersHarmonoise.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 10.05.2004
-/// @version $Id: HelpersHarmonoise.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Helper methods for Harmonoise-based noise emission computation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef HelpersHarmonoise_h
-#define HelpersHarmonoise_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <limits>
@@ -57,7 +47,7 @@ public:
      * @param[in] a The vehicle's current acceleration
      * @return The noise produced by the vehicle of the given class running with v and a
      */
-    static SUMOReal computeNoise(SUMOEmissionClass c, double v, double a);
+    static double computeNoise(SUMOEmissionClass c, double v, double a);
 
 
     /** @brief Computes the resulting noise
@@ -65,8 +55,8 @@ public:
      * @param[in] val The sum of converted vehicle noises ( pow(10., (<NOISE>/10.)) )
      * @return The resulting sum
      */
-    inline static SUMOReal sum(SUMOReal val) {
-        return SUMOReal(10. * log10(val));
+    inline static double sum(double val) {
+        return double(10. * log10(val));
     }
 
 
@@ -99,15 +89,9 @@ private:
     static double myT_B_C3_Parameter[27];
     /// @}
 
-
     /// @brief A-weighted correction for octave bands
-    static double myAOctaveBandCorrection[27];
-
+    static const double myAOctaveBandCorrection[27];
 
+    /// @brief A-weighted correction for surface
+    static const double mySurfaceCorrection[27];
 };
-
-
-#endif
-
-/****************************************************************************/
-
