@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2013-2020 German Aerospace Center (DLR) and others.
+// Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -37,7 +37,9 @@
 // ===========================================================================
 // method definitions
 // ===========================================================================
-HelpersPHEMlight::HelpersPHEMlight() : PollutantsInterface::Helper("PHEMlight"), myIndex(PHEMLIGHT_BASE) {
+HelpersPHEMlight::HelpersPHEMlight() :
+    PollutantsInterface::Helper("PHEMlight", PHEMLIGHT_BASE, -1),
+    myIndex(PHEMLIGHT_BASE) {
 }
 
 
