@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PHEMCEP.cpp
 /// @author  Nikolaus Furian
 /// @author  Daniel Krajzewicz
@@ -6,30 +18,10 @@
 /// @author  Michael Behrisch
 /// @author  Marek Heinrich
 /// @date    Thu, 13.06.2013
-/// @version $Id: PHEMCEP.cpp 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Helper class for PHEM Light, holds a specific CEP for a PHEM emission class
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cmath>
 #include <string>
@@ -38,45 +30,54 @@
 #include <utils/common/UtilExceptions.h>
 #include "PHEMCEP.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
-PHEMCEP::PHEMCEP(bool heavyVehicel, SUMOEmissionClass emissionClass, const std::string& emissionClassIdentifier,
+PHEMCEP::PHEMCEP(bool heavyVehicle, SUMOEmissionClass emissionClass, const std::string& emissionClassIdentifier,
                  double vehicleMass, double vehicleLoading, double vehicleMassRot,
-                 double crossArea, double cWValue,
+                 double crossArea, double cdValue,
                  double f0, double f1, double f2, double f3, double f4,
-                 double ratedPower, double pNormV0, double pNormP0, double pNormV1,
-                 double pNormP1, std::string vehicelFuelType,
+                 double ratedPower, double pNormV0, double pNormP0, double pNormV1, double pNormP1,
+                 double axleRatio, double engineIdlingSpeed, double engineRatedSpeed, double effectiveWheelDiameter,
+                 double idlingFC,
+                 const std::string& vehicleFuelType,
                  const std::vector< std::vector<double> >& matrixFC,
                  const std::vector<std::string>& headerLinePollutants,
                  const std::vector< std::vector<double> >& matrixPollutants,
-                 const std::vector< std::vector<double> > matrixSpeedRotational) {
+                 const std::vector< std::vector<double> >& matrixSpeedRotational,
+                 const std::vector< std::vector<double> >& normedDragTable,
+                 const std::vector<double>& idlingValuesPollutants) {
     _emissionClass = emissionClass;
     _resistanceF0 = f0;
     _resistanceF1 = f1;
     _resistanceF2 = f2;
     _resistanceF3 = f3;
     _resistanceF4 = f4;
-    _cwValue = cWValue;
+    _cdValue = cdValue;
     _crossSectionalArea = crossArea;
     _massVehicle = vehicleMass;
     _vehicleLoading = vehicleLoading;
     _massRot = vehicleMassRot;
     _ratedPower = ratedPower;
-    _vehicleFuelType = vehicelFuelType;
+    _vehicleFuelType = vehicleFuelType;
 
     _pNormV0 = pNormV0 / 3.6;
     _pNormP0 = pNormP0;
     _pNormV1 = pNormV1 / 3.6;
     _pNormP1 = pNormP1;
 
+    _axleRatio = axleRatio;
+    _engineIdlingSpeed = engineIdlingSpeed;
+    _engineRatedSpeed = engineRatedSpeed;
+    _effictiveWheelDiameter = effectiveWheelDiameter;
+
+    _heavyVehicle = heavyVehicle;
+    _idlingFC = idlingFC;
+
     std::vector<std::string> pollutantIdentifier;
     std::vector< std::vector<double> > pollutantMeasures;
+    std::vector<std::vector<double> > normalizedPollutantMeasures;
 
     // init pollutant identifiers
     for (int i = 0; i < (int)headerLinePollutants.size(); i++) {
@@ -90,41 +91,67 @@ PHEMCEP::PHEMCEP(bool heavyVehicel, SUMOEmissionClass emissionClass, const std::
     // initialize measures
     for (int i = 0; i < (int)headerLinePollutants.size(); i++) {
         pollutantMeasures.push_back(std::vector<double>());
+        normalizedPollutantMeasures.push_back(std::vector<double>());
     } // end for
 
     // looping through matrix and assigning values for speed rotational table
     _speedCurveRotational.clear();
     _speedPatternRotational.clear();
+    _gearTransmissionCurve.clear();
     for (int i = 0; i < (int)matrixSpeedRotational.size(); i++) {
-        if (matrixSpeedRotational[i].size() != 2) {
+        if (matrixSpeedRotational[i].size() != 3) {
             throw InvalidArgument("Error loading vehicle file for: " + emissionClassIdentifier);
         }
 
         _speedPatternRotational.push_back(matrixSpeedRotational[i][0] / 3.6);
         _speedCurveRotational.push_back(matrixSpeedRotational[i][1]);
+        _gearTransmissionCurve.push_back(matrixSpeedRotational[i][2]);
+    } // end for
+
+    // looping through matrix and assigning values for drag table
+    _nNormTable.clear();
+    _dragNormTable.clear();
+    for (int i = 0; i < (int) normedDragTable.size(); i++) {
+        if (normedDragTable[i].size() != 2) {
+            return;
+        }
+
+        _nNormTable.push_back(normedDragTable[i][0]);
+        _dragNormTable.push_back(normedDragTable[i][1]);
 
     } // end for
 
     // looping through matrix and assigning values for Fuel consumption
     _cepCurveFC.clear();
+    _powerPatternFC.clear();
+    _normalizedPowerPatternFC.clear();
+    _normedCepCurveFC.clear();
     for (int i = 0; i < (int)matrixFC.size(); i++) {
         if (matrixFC[i].size() != 2) {
             throw InvalidArgument("Error loading vehicle file for: " + emissionClassIdentifier);
         }
 
         _powerPatternFC.push_back(matrixFC[i][0] * _ratedPower);
-        _cepCurveFC.push_back(matrixFC[i][1]);
+        _normalizedPowerPatternFC.push_back(matrixFC[i][0]);
+        _cepCurveFC.push_back(matrixFC[i][1] * _ratedPower);
+        _normedCepCurveFC.push_back(matrixFC[i][1]);
 
     } // end for
 
+    _powerPatternPollutants.clear();
+    double pollutantMultiplyer = 1;
+
+    _drivingPower = _normalizingPower = CalcPower(NORMALIZING_SPEED, NORMALIZING_ACCELARATION, 0, vehicleLoading);
 
     // looping through matrix and assigning values for pollutants
-    double normalizingPower = 0;
 
-    if (heavyVehicel) {
-        normalizingPower = _ratedPower;
+    if (heavyVehicle) {
+        _normalizingPower = _ratedPower;
+        pollutantMultiplyer = _ratedPower;
+        _normalizingType = RatedPower;
     } else {
-        normalizingPower = PHEMCEP::CalcPower(NORMALIZING_SPEED, NORMALIZING_ACCELARATION, 0);
+        _normalizingPower = _drivingPower;
+        _normalizingType = DrivingPower;
     } // end if
 
     const int headerCount = (int)headerLinePollutants.size();
@@ -135,17 +162,23 @@ PHEMCEP::PHEMCEP(bool heavyVehicel, SUMOEmissionClass emissionClass, const std::
             }
 
             if (j == 0) {
-                _powerPatternPollutants.push_back(matrixPollutants[i][j] * normalizingPower);
+                _normailzedPowerPatternPollutants.push_back(matrixPollutants[i][j]);
+                _powerPatternPollutants.push_back(matrixPollutants[i][j] * _normalizingPower);
             } else {
-                pollutantMeasures[j - 1].push_back(matrixPollutants[i][j]);
+                pollutantMeasures[j - 1].push_back(matrixPollutants[i][j] * pollutantMultiplyer);
+                normalizedPollutantMeasures[j - 1].push_back(matrixPollutants[i][j]);
             } // end if
         } // end for
     } // end for
 
-    for (int i = 0; i < headerCount; i++) {
-        _cepCurvePollutants[pollutantIdentifier[i]] = pollutantMeasures[i];
+    for (int i = 0; i < (int) headerLinePollutants.size(); i++) {
+        _cepCurvePollutants.insert(pollutantIdentifier[i], pollutantMeasures[i]);
+        _normalizedCepCurvePollutants.insert(pollutantIdentifier[i], normalizedPollutantMeasures[i]);
+        _idlingValuesPollutants.insert(pollutantIdentifier[i], idlingValuesPollutants[i] * pollutantMultiplyer);
     } // end for
 
+    _idlingFC = idlingFC * _ratedPower;
+
 } // end of Cep
 
 
@@ -160,33 +193,42 @@ PHEMCEP::~PHEMCEP() {
 
 
 double
-PHEMCEP::CalcPower(double v, double a, double slope) const {
-    const double rotFactor = GetRotationalCoeffecient(v);
-    double power = (_massVehicle + _vehicleLoading) * GRAVITY_CONST * (_resistanceF0 + _resistanceF1 * v + _resistanceF4 * pow(v, 4)) * v;
-    power += (_crossSectionalArea * _cwValue * AIR_DENSITY_CONST / 2) * pow(v, 3);
-    power += (_massVehicle * rotFactor + _massRot + _vehicleLoading) * a * v;
-    power += (_massVehicle + _vehicleLoading) * slope * 0.01 * v;
-    return power / 950.;
-}
+PHEMCEP::GetEmission(const std::string& pollutant, double power, double speed, bool normalized) const {
+    std::vector<double> emissionCurve;
+    std::vector<double> powerPattern;
 
-
-double
-PHEMCEP::GetMaxAccel(double v, double a, double gradient) const {
-    UNUSED_PARAMETER(a);
-    const double pMaxForAcc = GetPMaxNorm(v) * _ratedPower - PHEMCEP::CalcPower(v, 0, gradient);
-    return (pMaxForAcc * 1000) / ((_massVehicle * GetRotationalCoeffecient(v) + _massRot + _vehicleLoading) * v);
-}
-
-
-double
-PHEMCEP::GetEmission(const std::string& pollutant, double power) const {
-    std::map<std::string, std::vector<double> >::const_iterator it = _cepCurvePollutants.find(pollutant);
-    if (pollutant != "FC" && it == _cepCurvePollutants.end()) {
-        throw InvalidArgument("Emission pollutant " + pollutant + " not found!");
+    if (!normalized && fabs(speed) <= ZERO_SPEED_ACCURACY) {
+        if (pollutant == "FC") {
+            return _idlingFC;
+        } else {
+            return _idlingValuesPollutants.get(pollutant);
+        }
     } // end if
 
-    const std::vector<double>& emissionCurve = (pollutant == "FC" ? _cepCurveFC : it->second);
-    const std::vector<double>& powerPattern = (pollutant == "FC" ? _powerPatternFC : _powerPatternPollutants);
+    if (pollutant == "FC") {
+        if (normalized) {
+            emissionCurve = _normedCepCurveFC;
+            powerPattern = _normalizedPowerPatternFC;
+        } else {
+            emissionCurve = _cepCurveFC;
+            powerPattern = _powerPatternFC;
+        }
+    } else {
+        if (!_cepCurvePollutants.hasString(pollutant)) {
+            throw InvalidArgument("Emission pollutant " + pollutant + " not found!");
+        }
+
+        if (normalized) {
+            emissionCurve = _normalizedCepCurvePollutants.get(pollutant);
+            powerPattern = _normailzedPowerPatternPollutants;
+        } else {
+            emissionCurve = _cepCurvePollutants.get(pollutant);
+            powerPattern = _powerPatternPollutants;
+        }
+
+    } // end if
+
+
 
     if (emissionCurve.size() == 0) {
         throw InvalidArgument("Empty emission curve for " + pollutant + " found!");
@@ -233,6 +275,45 @@ PHEMCEP::Interpolate(double px, double p1, double p2, double e1, double e2) cons
 } // end of Interpolate
 
 
+double PHEMCEP::GetDecelCoast(double speed, double acc, double gradient, double /* vehicleLoading */) const {
+    if (speed < SPEED_DCEL_MIN) {
+        return speed / SPEED_DCEL_MIN * GetDecelCoast(SPEED_DCEL_MIN, acc, gradient, _vehicleLoading); // !!!vehicleLoading
+    } // end if
+
+    double rotCoeff = GetRotationalCoeffecient(speed);
+
+    int upperIndex;
+    int lowerIndex;
+
+    double iGear = GetGearCoeffecient(speed);
+
+    double iTot = iGear * _axleRatio;
+
+    double n = (30 * speed * iTot) / ((_effictiveWheelDiameter / 2) * M_PI2);
+    double nNorm = (n - _engineIdlingSpeed) / (_engineRatedSpeed - _engineIdlingSpeed);
+
+    FindLowerUpperInPattern(lowerIndex, upperIndex, _nNormTable, nNorm);
+
+    double fMot = 0;
+
+    if (speed >= 10e-2) {
+        fMot = (-GetDragCoeffecient(nNorm) * _ratedPower * 1000 / speed) / 0.9;
+    } // end if
+
+    double fRoll = (_resistanceF0
+                    + _resistanceF1 * speed
+                    + pow(_resistanceF2 * speed, 2)
+                    + pow(_resistanceF3 * speed, 3)
+                    + pow(_resistanceF4 * speed, 4)) * (_massVehicle + _vehicleLoading) * GRAVITY_CONST; // !!!vehicleLoading
+
+    double fAir = _cdValue * _crossSectionalArea * 1.2 * 0.5 * pow(speed, 2);
+
+    double fGrad = (_massVehicle + _vehicleLoading) * GRAVITY_CONST * gradient / 100; // !!!vehicleLoading
+
+    return -(fMot + fRoll + fAir + fGrad) / ((_massVehicle + _vehicleLoading) * rotCoeff); // !!!vehicleLoading
+} // end of GetDecelCoast
+
+
 double
 PHEMCEP::GetRotationalCoeffecient(double speed) const {
     int upperIndex;
@@ -247,8 +328,33 @@ PHEMCEP::GetRotationalCoeffecient(double speed) const {
                                 _speedCurveRotational[upperIndex]);
 } // end of GetRotationalCoeffecient
 
-void
-PHEMCEP::FindLowerUpperInPattern(int& lowerIndex, int& upperIndex, std::vector<double> pattern, double value) const {
+double PHEMCEP::GetGearCoeffecient(double speed) const {
+    int upperIndex;
+    int lowerIndex;
+
+    FindLowerUpperInPattern(lowerIndex, upperIndex, _gearTransmissionCurve, speed);
+
+    return Interpolate(speed,
+                       _speedPatternRotational[lowerIndex],
+                       _speedPatternRotational[upperIndex],
+                       _gearTransmissionCurve[lowerIndex],
+                       _gearTransmissionCurve[upperIndex]);
+} // end of GetGearCoefficient
+
+double PHEMCEP::GetDragCoeffecient(double nNorm)  const {
+    int upperIndex;
+    int lowerIndex;
+
+    FindLowerUpperInPattern(lowerIndex, upperIndex, _dragNormTable, nNorm);
+
+    return Interpolate(nNorm,
+                       _nNormTable[lowerIndex],
+                       _nNormTable[upperIndex],
+                       _dragNormTable[lowerIndex],
+                       _dragNormTable[upperIndex]);
+} // end of GetGearCoefficient
+
+void PHEMCEP::FindLowerUpperInPattern(int& lowerIndex, int& upperIndex, const std::vector<double>& pattern, double value) const {
     if (value <= pattern.front()) {
         lowerIndex = 0;
         upperIndex = 0;
@@ -289,6 +395,27 @@ PHEMCEP::FindLowerUpperInPattern(int& lowerIndex, int& upperIndex, std::vector<d
 } // end of FindLowerUpperInPattern
 
 
+double
+PHEMCEP::CalcPower(double v, double a, double slope, double /* vehicleLoading */) const {
+    const double rotFactor = GetRotationalCoeffecient(v);
+    double power = (_massVehicle + _vehicleLoading) * GRAVITY_CONST * (_resistanceF0 + _resistanceF1 * v + _resistanceF4 * pow(v, 4)) * v;
+    power += (_crossSectionalArea * _cdValue * AIR_DENSITY_CONST / 2) * pow(v, 3);
+    power += (_massVehicle * rotFactor + _massRot + _vehicleLoading) * a * v;
+    power += (_massVehicle + _vehicleLoading) * slope * 0.01 * v;
+    return power / 950.;
+}
+
+
+double
+PHEMCEP::GetMaxAccel(double v, double a, double gradient, double /* vehicleLoading */) const {
+    UNUSED_PARAMETER(a);
+    double rotFactor = GetRotationalCoeffecient(v);
+    const double pMaxForAcc = GetPMaxNorm(v) * _ratedPower - PHEMCEP::CalcPower(v, 0, gradient, _vehicleLoading); // !!!vehicleLoading
+    return (pMaxForAcc * 1000) / ((_massVehicle * rotFactor + _massRot + _vehicleLoading) * v); // !!!vehicleLoading
+}
+
+
+
 double PHEMCEP::GetPMaxNorm(double speed) const {
     // Linear function between v0 and v1, constant elsewhere
     if (speed <= _pNormV0) {
@@ -300,4 +427,5 @@ double PHEMCEP::GetPMaxNorm(double speed) const {
     }
 } // end of GetPMaxNorm
 
+
 /****************************************************************************/
