@@ -1,44 +1,45 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PHEMCEP.h
 /// @author  Nikolaus Furian
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Marek Heinrich
 /// @date    Thu, 13.06.2013
-/// @version $$
 ///
 // Helper class for PHEM Light, holds a specific CEP for a PHEM emission class
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef PHEMCEP_h
-#define PHEMCEP_h
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <map>
 #include <vector>
 #include <utils/common/SUMOVehicleClass.h>
+#include <utils/common/StringBijection.h>
 #include "PHEMCEP.h"
 #include "PHEMConstants.h"
 
 
+
+// ===========================================================================
+// enumerations
+// ===========================================================================
+enum NormalizingType {
+    RatedPower,
+    DrivingPower
+};
+
+
 // ===========================================================================
 // class definitions
 // ===========================================================================
@@ -74,16 +75,18 @@ public:
      */
     PHEMCEP(bool heavyVehicel, SUMOEmissionClass emissionClass, const std::string& emissionClassIdentifier,
             double vehicleMass, double vehicleLoading, double vehicleMassRot,
-            double crossArea, double cWValue,
+            double crossArea, double cdValue,
             double f0, double f1, double f2, double f3, double f4,
-            double ratedPower, double pNormV0, double pNormP0, double pNormV1,
-            double pNormP1, std:: string vehicelFuelType,
+            double ratedPower, double pNormV0, double pNormP0, double pNormV1, double pNormP1,
+            double axleRatio, double engineIdlingSpeed, double engineRatedSpeed, double effectiveWheelDiameter,
+            double idlingFC,
+            const std::string& vehicleFuelType,
             const std::vector< std::vector<double> >& matrixFC,
             const std::vector<std::string>& headerLinePollutants,
             const std::vector< std::vector<double> >& matrixPollutants,
-            const std::vector< std::vector<double> > matrixSpeedRotational
-           );
-
+            const std::vector< std::vector<double> >& matrixSpeedRotational,
+            const std::vector< std::vector<double> >& normedDragTable,
+            const std::vector<double>& idlingValuesPollutants);
 
     /// @brief Destructor
     ~PHEMCEP();
@@ -91,30 +94,31 @@ public:
     /** @brief Returns the power of used for a vehicle at state v,a, slope and loading
          * @param[in] v The vehicle's average velocity
          * @param[in] a The vehicle's average acceleration
-         * @param[in] slope The road's slope at vehicle's position []
+         * @param[in] slope The road's slope at vehicle's position [deg]
     	 * @param{in] vehicleCep vehicles CEP data
          * @param{in] loading vehicle loading [kg]
          * @return The power demand for desired state [kW]
          */
-    double CalcPower(double v, double a, double slope) const;
+    double CalcPower(double v, double a, double slope, double vehicleLoading = 0) const;
 
 
     /**	 @brief Returns the maximum accelaration for a vehicle at state v,a, slope and loading
          * @param[in] v The vehicle's average velocity
          * @param[in] a The vehicle's average acceleration
-         * @param[in] slope The road's slope at vehicle's position []
+         * @param[in] slope The road's slope at vehicle's position [deg]
     	 * @param{in] vehicleCep vehicles CEP data
          * @param{in] loading vehicle loading [kg]
          * @return The maximum accelaration for desired state [kW]
          */
-    double GetMaxAccel(double v, double a, double gradient) const;
+    double GetMaxAccel(double v, double a, double gradient, double vehicleLoading = 0) const;
 
     /** @brief Returns a emission measure for power[kW] level
      * @param[in] pollutantIdentifier Desired pollutant, e.g. NOx
      * @param[in] power in [kW]
      * @return emission in [g/h]
      */
-    double GetEmission(const std::string& pollutantIdentifier, double power) const;
+    double GetEmission(const std::string& pollutantIdentifier, double power, double speed, bool normalized = false) const;
+    double GetDecelCoast(double speed, double acc, double gradient, double vehicleLoading) const;
 
 
     /** @brief Getter function to recieve vehicle data from CEP
@@ -170,7 +174,7 @@ public:
      * @todo: Why is it named "cdValue", here?
      */
     double GetCdValue() const {
-        return _cwValue;
+        return _cdValue;
     }
 
     /** @brief Getter function to recieve vehicle data from CEP
@@ -214,7 +218,7 @@ public:
     /** @brief Getter function to recieve vehicle data from CEP
      * @return fuel type of vehicle
      */
-    std::string GetVehicleFuelType() const {
+    const std::string& GetVehicleFuelType() const {
         return _vehicleFuelType;
     }
 
@@ -235,12 +239,14 @@ private:
      * @param[in] pattern to search
      * @param[in] value to search
      */
-    void FindLowerUpperInPattern(int& lowerIndex, int& upperIndex, std::vector<double> pattern, double value) const;
+    void FindLowerUpperInPattern(int& lowerIndex, int& upperIndex, const std::vector<double>& pattern, double value) const;
 
     /** @brief Calculates rotational index for speed
      * @param[in] speed desired speed
      */
     double GetRotationalCoeffecient(double speed) const;
+    double GetGearCoeffecient(double speed) const;
+    double GetDragCoeffecient(double nNorm) const;
 
     /** @brief Calculates maximum available rated power for speed
      * @param[in] speed desired speed
@@ -250,6 +256,7 @@ private:
 private:
     /// @brief PHEM emission class of vehicle
     SUMOEmissionClass _emissionClass;
+    NormalizingType _normalizingType;
     /// @brief Rolling resistance f0
     double _resistanceF0;
     /// @brief Rolling resistance f1
@@ -261,7 +268,7 @@ private:
     /// @brief Rolling resistance f4
     double _resistanceF4;
     /// @brief Cw value
-    double _cwValue;
+    double _cdValue;
     /// @brief crosssectional area of vehicle
     double _crossSectionalArea;
     /// @brief vehicle mass
@@ -280,27 +287,35 @@ private:
     double _pNormV1;
     /// @brief Step functions parameter for maximum rated power
     double _pNormP1;
-    /// @todo describe
+    double _axleRatio;
+    double _engineIdlingSpeed;
+    double _engineRatedSpeed;
+    double _effictiveWheelDiameter;
+    double _idlingFC;
+    std::string _vehicleFuelType;
     int _sizeOfPatternFC;
     /// @todo describe
     int _sizeOfPatternPollutants;
+    double _normalizingPower;
+    double _drivingPower;
+    bool _heavyVehicle;
+    std::vector<double> _speedPatternRotational;
     /// @todo describe
     std::vector<double> _powerPatternFC;
     /// @todo describe
     std::vector<double> _powerPatternPollutants;
+    std::vector<double> _normalizedPowerPatternFC;
+    std::vector<double> _normailzedPowerPatternPollutants;
     /// @todo describe
     std::vector<double> _cepCurveFC;
     /// @todo describe
-    std::map<std::string, std::vector<double> > _cepCurvePollutants;
-    /// @todo describe
-    std::vector<double> _speedPatternRotational;
-    /// @todo describe
+    std::vector<double> _normedCepCurveFC;
     std::vector<double> _speedCurveRotational;
-    /// @todo describe
-    std::string _vehicleFuelType;
+    std::vector<double> _gearTransmissionCurve;
+    std::vector<double> _nNormTable;
+    std::vector<double> _dragNormTable;
+    StringBijection< std::vector<double> > _cepCurvePollutants;
+    StringBijection<std::vector<double> > _normalizedCepCurvePollutants;
+    StringBijection<double> _idlingValuesPollutants;
 
 };
-
-#endif
-
-/****************************************************************************/
