@@ -1,34 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PHEMCEPHandler.cpp
 /// @author  Nikolaus Furian
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Marek Heinrich
 /// @date    Thu, 13.06.2013
-/// @version $$
 ///
 // Helper class for PHEM Light, holds CEP data for emission computation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cstdlib>
 #include <fstream>
@@ -50,7 +42,7 @@ PHEMCEPHandler::PHEMCEPHandler() {
 PHEMCEPHandler::~PHEMCEPHandler() {
     std::map<SUMOEmissionClass, PHEMCEP*>::iterator iter = _ceps.begin();
     while (iter != _ceps.end()) {
-        delete(iter->second);
+        delete (iter->second);
         iter++;
     } // end while
     _ceps.clear();
@@ -102,10 +94,10 @@ PHEMCEPHandler::Load(SUMOEmissionClass emissionClass, const std::string& emissio
     //std::string phemPath = oc.getString("phemlight-path") + "/";
     std::vector<std::string> phemPath;
     phemPath.push_back(oc.getString("phemlight-path") + "/");
-    if (getenv("PHEMLIGHT_PATH") != 0) {
+    if (getenv("PHEMLIGHT_PATH") != nullptr) {
         phemPath.push_back(std::string(getenv("PHEMLIGHT_PATH")) + "/");
     }
-    if (getenv("SUMO_HOME") != 0) {
+    if (getenv("SUMO_HOME") != nullptr) {
         phemPath.push_back(std::string(getenv("SUMO_HOME")) + "/data/emissions/PHEMlight/");
     }
     if (!ReadVehicleFile(phemPath, emissionClassIdentifier,
@@ -181,7 +173,7 @@ PHEMCEP*
 PHEMCEPHandler::GetCep(SUMOEmissionClass emissionClass) {
     // check if Cep has been loaded
     if (_ceps.find(emissionClass) == _ceps.end()) {
-        return 0;
+        return nullptr;
     } // end if
 
     return _ceps[emissionClass];
@@ -356,9 +348,8 @@ PHEMCEPHandler::ReadVehicleFile(const std::vector<std::string>& path, const std:
     } // end while
 
     while (std::getline(fileVehicle, line) && line.substr(0, 1) != commentPrefix) {
-        std::stringstream  lineStream(line);
-        std::string cell;
-        std::vector <double> vi;
+        std::stringstream lineStream(line);
+        std::vector<double> vi;
         while (std::getline(lineStream, cell, ',')) {
             double entry;
             std::istringstream(cell) >> entry;
@@ -374,8 +365,7 @@ PHEMCEPHandler::ReadVehicleFile(const std::vector<std::string>& path, const std:
         }
 
         std::stringstream  lineStream(line);
-        std::string cell;
-        std::vector <double> vi;
+        std::vector<double> vi;
         while (std::getline(lineStream, cell, ',')) {
             double entry;
             std::istringstream(cell) >> entry;
@@ -449,7 +439,6 @@ bool PHEMCEPHandler::ReadEmissionData(bool readFC, const std::vector<std::string
 
     while (std::getline(fileEmission, line)) {
         std::stringstream  lineStream(line);
-        std::string cell;
         std::vector <double> vi;
         while (std::getline(lineStream, cell, ',')) {
             double entry;
