@@ -1,36 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PHEMCEPHandler.h
 /// @author  Nikolaus Furian
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Marek Heinrich
 /// @date    Thu, 13.06.2013
-/// @version $$
 ///
-// Helper singelton class for PHEM Light, holds CEP data for emission computation
+// Helper singleton class for PHEM Light, holds CEP data for emission computation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef PHEMCEPHandler_h
-#define PHEMCEPHandler_h
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <utils/common/StringBijection.h>
@@ -72,7 +63,7 @@ public:
 
 private:
     /** @brief Helper method to read a vehicle file from file system
-    * @param[in] path The path to PHEMlight data files
+    * @param[in] path The possible paths to PHEMlight data files
     * @param[in] emissionClass desired PHEM emission class
     * @param[in] vehicleMass out variable for vehicle mass
     * @param[in] vehivleLoading out variable for vehicle loading
@@ -93,27 +84,48 @@ private:
     * @param[in] matrixRotFactor out variable for rotational factors over speed for more accurate power calculation
     * @return Indicator if reading was successul
     */
-    bool ReadVehicleFile(const std::string& path, const std::string& emissionClass,
-                         double& vehicleMass, double& vehicleLoading, double& vehicleMassRot,
-                         double& crossArea, double& cWValue,
-                         double& f0, double& f1, double& f2, double& f3, double& f4, double& ratedPower, std::string& vehicleMassType, std::string& vehicleFuelType,
-                         double& pNormV0, double& pNormP0, double& pNormV1, double& pNormP1, std::vector< std::vector<double> >& matrixRotFactor);
+    bool ReadVehicleFile(const std::vector<std::string>& path, const std::string& emissionClass,
+                         double& vehicleMass,
+                         double& vehicleLoading,
+                         double& vehicleMassRot,
+                         double& crossArea,
+                         double& cWValue,
+                         double& f0,
+                         double& f1,
+                         double& f2,
+                         double& f3,
+                         double& f4,
+                         double& axleRatio,
+                         double& ratedPower,
+                         double& engineIdlingSpeed,
+                         double& engineRatedSpeed,
+                         double& effectiveWheelDiameter,
+                         std::string& vehicleMassType,
+                         std::string& vehicleFuelType,
+                         double& pNormV0,
+                         double& pNormP0,
+                         double& pNormV1,
+                         double& pNormP1,
+                         std::vector< std::vector<double> >& matrixSpeedInertiaTable,
+                         std::vector< std::vector<double> >& normedDragTable);
+
 
 
     /** @brief Helper method to read a CEP file from file system
-     * @param[in] path The path to PHEMlight data files
+     * @param[in] path The possible paths to PHEMlight data files
      * @param[in] emissionClass desired PHEM emission class
      * @param[in] header vector of pollutant identifiers
      * @param[in] matrix matrix holding power pattern and CEP curves
      * @return Indicator if reading was successul
      */
-    bool ReadEmissionData(bool readFC, const std::string& path, const std::string& emissionClass,
-                          std::vector<std::string>& header, std::vector<std::vector<double> >& matrix);
+    bool ReadEmissionData(bool readFC, const std::vector<std::string>& path, const std::string& emissionClass,
+                          std::vector<std::string>& header, std::vector<std::vector<double> >& matrix, std::vector<double>& idlingValues);
 
 
 private:
     /// @brief bijection between PHEMEmissionClass and CEPs
     std::map<SUMOEmissionClass, PHEMCEP*> _ceps;
+//	StringBijection<PHEMEmissionClass> _stringRepsPhemEmissionClass;
 
 
 private:
@@ -126,4 +138,3 @@ private:
 
 };
 
-#endif
