@@ -1,39 +1,34 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2003-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    FXBaseObject.cpp
 /// @author  Mathew Robertson
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    2004-03-19
-/// @version $Id: FXBaseObject.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 //
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2003-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
 
 
 /* =========================================================================
  * included modules
  * ======================================================================= */
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
-#include <fxver.h>
-#include <xincs.h>
-#include <fxdefs.h>
-#include <fx.h>
+#define NOMINMAX
+#undef NOMINMAX
+#include "fxheader.h"
 /*
 #include <FXString.h>
 #include <FXHash.h>
@@ -61,34 +56,34 @@ FXIMPLEMENT(FXBaseObject, FXObject, FXBaseObjectMap, ARRAYNUMBER(FXBaseObjectMap
 
 // ctor
 FXBaseObject::FXBaseObject(FXObject* tgt, FXSelector sel) : FXObject() {
-    data = NULL;
+    data = nullptr;
     target = tgt;
     message = sel;
     flags = 0;
     app = FXApp::instance();
-    if (app == NULL) {
+    if (app == nullptr) {
         fxerror("%s: Cannot create object without FXApp object\n", getClassName());
     }
 }
 
 // ctor
 FXBaseObject::FXBaseObject(FXApp* a, FXObject* tgt, FXSelector sel) : FXObject() {
-    data = NULL;
+    data = nullptr;
     target = tgt;
     message = sel;
     flags = 0;
     app = a;
-    if (app == NULL) {
+    if (app == nullptr) {
         app = FXApp::instance();
     }
-    if (app == NULL) {
+    if (app == nullptr) {
         fxerror("%s: Cannot create object without FXApp object\n", getClassName());
     }
 }
 
 // free up all resources
 FXBaseObject::~FXBaseObject() {
-    if (data != NULL && data != (void*) - 1) {
+    if (data != nullptr && data != (void*) - 1) {
         fxerror("%s::~%s - user data is not NULL prior to destruction\n", getClassName(), getClassName());
     }
     app = (FXApp*) - 1;
@@ -151,9 +146,9 @@ long FXBaseObject::onCmdDisable(FXObject*, FXSelector, void*) {
 // handle update event
 long FXBaseObject::onUpdate(FXObject* sender, FXSelector, void*) {
     if (flags & FLAG_ENABLED) {
-        sender->handle(this, FXSEL(SEL_UPDATE, FXWindow::ID_ENABLE), NULL);
+        sender->handle(this, FXSEL(SEL_UPDATE, FXWindow::ID_ENABLE), nullptr);
     } else {
-        sender->handle(this, FXSEL(SEL_UPDATE, FXWindow::ID_DISABLE), NULL);
+        sender->handle(this, FXSEL(SEL_UPDATE, FXWindow::ID_DISABLE), nullptr);
     }
     return 1;
 }
