@@ -1,38 +1,29 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    FXSevenSegment.h
 /// @author  Mathew Robertson
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    2004-03-19
-/// @version $Id: FXSevenSegment.h 20482 2016-04-18 20:49:42Z behrisch $
 ///
 //
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
 
 
 #ifndef FXSEVENSEGMENT_H
 #define FXSEVENSEGMENT_H
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #ifndef FXFRAME_H
 
@@ -52,10 +43,10 @@ private:
     FXchar  value;      // The currently shown character
     FXColor fgcolor;    // The color of the LCD text
     FXColor bgcolor;    // The color of the LCD background
-    FXint   hsl;        // This is pixel length of a horizontal segment
-    FXint   vsl;        // This is pixel length of a vertical segment
-    FXint   st;         // This is segment thickness, in pixels
-    FXint   groove;     // Groove between segments
+    FXshort hsl;        // This is pixel length of a horizontal segment
+    FXshort vsl;        // This is pixel length of a vertical segment
+    FXshort st;         // This is segment thickness, in pixels
+    FXshort groove;     // Groove between segments
 
 private:
     void checkSize();
@@ -125,13 +116,13 @@ public:
     }
 
     /// get/set segment thickness
-    void setThickness(const FXint width);
+    void setThickness(const FXint w);
     FXint getThickness() const {
         return st;
     }
 
     /// get/set groove thickness
-    void setGroove(const FXint width);
+    void setGroove(const FXint w);
     FXint getGroove() const {
         return groove;
     }
