@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    FXSingleEventThread.cpp
 /// @author  unknown_author
 /// @author  Daniel Krajzewicz
@@ -6,35 +18,18 @@
 /// @author  Laura Bieker
 /// @author  Jakob Erdmann
 /// @date    2004-03-19
-/// @version $Id: FXSingleEventThread.cpp 17235 2014-11-03 10:53:02Z behrisch $
 ///
 //
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
 
 /* =========================================================================
  * included modules
  * ======================================================================= */
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <utils/common/StdDefs.h>
 #include "MFXInterThreadEventClient.h"
 #include "FXSingleEventThread.h"
-#include "fxexdefs.h"
 #ifndef WIN32
 #include <pthread.h>
 #include <stdlib.h>
@@ -42,10 +37,7 @@
 #else
 #include <process.h>
 #endif
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif
+#include <thread>
 
 #ifndef WIN32
 # define PIPE_READ 0
@@ -73,7 +65,7 @@ FXSingleEventThread::FXSingleEventThread(FXApp* a, MFXInterThreadEventClient* cl
     UNUSED_PARAMETER(res); // only used for assertion
     myApp->addInput(event[PIPE_READ], INPUT_READ, this, ID_THREAD_EVENT);
 #else
-    event = CreateEvent(NULL, FALSE, FALSE, NULL);
+    event = CreateEvent(nullptr, FALSE, FALSE, nullptr);
     FXASSERT(event != NULL);
     myApp->addInput(event, INPUT_READ, this, ID_THREAD_EVENT);
 #endif
@@ -97,7 +89,8 @@ void
 FXSingleEventThread::signal() {
 #ifndef WIN32
     FXuint seltype = SEL_THREAD;
-    ::write(event[PIPE_WRITE], &seltype, sizeof(seltype));
+    FXint res = ::write(event[PIPE_WRITE], &seltype, sizeof(seltype));
+    UNUSED_PARAMETER(res); // to make the compiler happy
 #else
     ::SetEvent(event);
 #endif
@@ -108,7 +101,8 @@ void
 FXSingleEventThread::signal(FXuint seltype) {
     UNUSED_PARAMETER(seltype);
 #ifndef WIN32
-    ::write(event[PIPE_WRITE], &seltype, sizeof(seltype));
+    FXint res = ::write(event[PIPE_WRITE], &seltype, sizeof(seltype));
+    UNUSED_PARAMETER(res); // to make the compiler happy
 #else
     ::SetEvent(event);
 #endif
@@ -119,19 +113,20 @@ long
 FXSingleEventThread::onThreadSignal(FXObject*, FXSelector, void*) {
 #ifndef WIN32
     FXuint seltype = SEL_THREAD;
-    ::read(event[PIPE_READ], &seltype, sizeof(seltype));
+    FXint res = ::read(event[PIPE_READ], &seltype, sizeof(seltype));
+    UNUSED_PARAMETER(res); // to make the compiler happy
 #else
     //FIXME need win32 support
 #endif
     FXSelector sel = FXSEL(SEL_THREAD, 0);
-    handle(this, sel, 0);
+    handle(this, sel, nullptr);
     return 0;
 }
 
 
 long
-FXSingleEventThread::onThreadEvent(FXObject*, FXSelector , void*) {
-    myClient->eventOccured();
+FXSingleEventThread::onThreadEvent(FXObject*, FXSelector, void*) {
+    myClient->eventOccurred();
     /*
     FXuint seltype1 = FXSELTYPE(SEL_THREAD);
     if(myTarget && myTarget->handle(this,FXSEL(seltype1,mySelector),NULL)) {
@@ -145,12 +140,7 @@ FXSingleEventThread::onThreadEvent(FXObject*, FXSelector , void*) {
 
 void
 FXSingleEventThread::sleep(long ms) {
-#ifdef WIN32
-    Sleep(ms);
-#else
-    long long us = ms * 1000;
-    usleep(us);
-#endif
+    std::this_thread::sleep_for(std::chrono::milliseconds(ms));
 }
 
 
