@@ -1,43 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MFXAddEditTypedTable.cpp
 /// @author  Daniel Krajzewicz
 /// @date    2004-07-02
-/// @version $Id: MFXAddEditTypedTable.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // missing_desc
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
-#include <fx.h>
-#include <fxkeys.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include "MFXAddEditTypedTable.h"
-#include <iostream>
-
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
 
 
 // Map
@@ -106,7 +90,7 @@ MFXAddEditTypedTable::editItem(FXTableItem* item,FXint how)
         {
             try {
                 myNumberEditor->setValue(
-                    TplConvert::_2SUMOReal(it->getText().text()));
+                    TplConvert::_2double(it->getText().text()));
             } catch (NumberFormatException &) {
             } catch (EmptyData &) {
             }
@@ -166,7 +150,7 @@ MFXAddEditTypedTable::editItem(FXTableItem* item,FXint how)
             myEnumEditor->hide();
             myEnumEditor->clearItems();
             if(myEnums.size()>myEditedCol) {
-                for(size_t i=0; i<myEnums[myEditedCol].size(); i++) {
+                for(int i=0; i<myEnums[myEditedCol].size(); i++) {
                     myEnumEditor->appendItem(myEnums[myEditedCol][i].c_str());
                 }
             }
@@ -200,22 +184,22 @@ MFXAddEditTypedTable::editItem(FXTableItem* item,FXint how)
 
 FXWindow*
 MFXAddEditTypedTable::getControlForItem(FXint r, FXint c) {
-    register FXTableItem* item = cells[r * ncols + c];
-    if (item == NULL) {
-        return 0;
+    FXTableItem* item = cells[r * ncols + c];
+    if (item == nullptr) {
+        return nullptr;
 //         cells[r * ncols + c] = item = createItem("", NULL, NULL);
 //         if (isItemSelected(r, c)) {
 //             item->setSelected(FALSE);
 //         }
     }
     delete editor;
-    editor = NULL;
+    editor = nullptr;
     switch (getCellType(c)) {
         case CT_UNDEFINED:
         case CT_STRING: {
-            register FXTextField* field;
-            register FXuint justify = 0;
-            field = new FXTextField(this, 1, NULL, 0, TEXTFIELD_ENTER_ONLY, 0, 0, 0, 0, getMarginLeft(), getMarginRight(), getMarginTop(), getMarginBottom());
+            FXTextField* field;
+            FXuint justify = 0;
+            field = new FXTextField(this, 1, nullptr, 0, TEXTFIELD_ENTER_ONLY, 0, 0, 0, 0, getMarginLeft(), getMarginRight(), getMarginTop(), getMarginBottom());
             // !!! if(state&LEFT) justify|=JUSTIFY_LEFT;
             // !!! if(state&RIGHT) justify|=JUSTIFY_RIGHT;
             // !!! if(state&TOP) justify|=JUSTIFY_TOP;
@@ -234,9 +218,9 @@ MFXAddEditTypedTable::getControlForItem(FXint r, FXint c) {
         case CT_REAL:
 //        return myNumberEditor;
         case CT_INT: {
-            register FXRealSpinDial* field;
-            //register FXuint justify=0;
-            field = new FXRealSpinDial(this, 1, NULL, 0, TEXTFIELD_ENTER_ONLY, 0, 0, 0, 0, getMarginLeft(), getMarginRight(), getMarginTop(), getMarginBottom());
+            FXRealSpinner* field;
+            //FXuint justify=0;
+            field = new FXRealSpinner(this, 1, nullptr, 0, TEXTFIELD_ENTER_ONLY, 0, 0, 0, 0, getMarginLeft(), getMarginRight(), getMarginTop(), getMarginBottom());
             // !!! if(state&LEFT) justify|=JUSTIFY_LEFT;
             // !!! if(state&RIGHT) justify|=JUSTIFY_RIGHT;
             // !!! if(state&TOP) justify|=JUSTIFY_TOP;
@@ -250,20 +234,21 @@ MFXAddEditTypedTable::getControlForItem(FXint r, FXint c) {
             field->setSelTextColor(getSelTextColor());
             NumberCellParams p = getNumberCellParams(c);
             if (p.format != "undefined") {
-                field->setFormatString((char*) p.format.c_str());
-                field->setIncrements(p.steps1, p.steps2, p.steps3);
+                //field->setFormatString((char*) p.format.c_str());
+                //field->setIncrements(p.steps1, p.steps2, p.steps3);
+                field->setIncrement(p.steps2);
                 field->setRange(p.min, p.max);
             }
             try {
                 if (getCellType(c) == CT_REAL) {
-                    field->setValue(TplConvert::_2SUMOReal(item->getText().text()));
+                    field->setValue(StringUtils::toDouble(item->getText().text()));
                 } else {
-                    field->setValue(TplConvert::_2int(item->getText().text()));
+                    field->setValue(StringUtils::toInt(item->getText().text()));
                 }
             } catch (NumberFormatException&) {
                 field->setValue(0);
             }
-            field->selectAll();
+            //field->selectAll();
             return field;
         }
         case CT_BOOL:
@@ -285,7 +270,7 @@ MFXAddEditTypedTable::cancelInput() {
         input.to.row = -1;
         input.fm.col = -1;
         input.to.col = -1;
-        editor = NULL;
+        editor = nullptr;
     }
 }
 
@@ -295,17 +280,11 @@ MFXAddEditTypedTable::acceptInput(FXbool notify) {
     bool set = false;
     FXTableRange tablerange = input;
     if (editor) {
-        //
-        //
-        FXRealSpinDial* dial = dynamic_cast<FXRealSpinDial*>(editor);
-        if (dial != 0) {
-            if (!dial->getDial().grabbed()) {
-                set = true;
-            } else {
-                setItemFromControl_NoRelease(input.fm.row, input.fm.col, editor);
-            }
+        FXRealSpinner* dial = dynamic_cast<FXRealSpinner*>(editor);
+        if (dial != nullptr) {
+            setItemFromControl_NoRelease(input.fm.row, input.fm.col, editor);
         }
-        if (dynamic_cast<FXTextField*>(editor) != 0) {
+        if (dynamic_cast<FXTextField*>(editor) != nullptr) {
             set = true;
         }
     }
@@ -323,9 +302,9 @@ MFXAddEditTypedTable::acceptInput(FXbool notify) {
 
 void
 MFXAddEditTypedTable::setItemFromControl(FXint r, FXint c, FXWindow* control) {
-    register FXTableItem* item = cells[r * ncols + c];
-    if (item == NULL) {
-        cells[r * ncols + c] = item = createItem("", NULL, NULL);
+    FXTableItem* item = cells[r * ncols + c];
+    if (item == nullptr) {
+        cells[r * ncols + c] = item = createItem("", nullptr, nullptr);
         if (isItemSelected(r, c)) {
             item->setSelected(FALSE);
         }
@@ -336,10 +315,10 @@ MFXAddEditTypedTable::setItemFromControl(FXint r, FXint c, FXWindow* control) {
             item->setFromControl(control);
             break;
         case CT_REAL:
-            item->setText(toString(static_cast<FXRealSpinDial*>(control)->getValue()).c_str());
+            item->setText(toString(static_cast<FXRealSpinner*>(control)->getValue()).c_str());
             break;
         case CT_INT:
-            item->setText(toString((int) static_cast<FXRealSpinDial*>(control)->getValue()).c_str());
+            item->setText(toString((int) static_cast<FXRealSpinner*>(control)->getValue()).c_str());
             break;
         case CT_BOOL:
 //        return myBoolEditor;
@@ -378,8 +357,8 @@ MFXAddEditTypedTable::setItemFromControl(FXint r, FXint c, FXWindow* control) {
 
 void
 MFXAddEditTypedTable::setItemFromControl_NoRelease(FXint r, FXint c, FXWindow* control) {
-    register FXTableItem* item = cells[r * ncols + c];
-    if (item == NULL) {
+    FXTableItem* item = cells[r * ncols + c];
+    if (item == nullptr) {
         return;
     }
     switch (getCellType(c)) {
@@ -388,10 +367,10 @@ MFXAddEditTypedTable::setItemFromControl_NoRelease(FXint r, FXint c, FXWindow* c
             item->setFromControl(control);
             break;
         case CT_REAL:
-            item->setText(toString(static_cast<FXRealSpinDial*>(control)->getValue()).c_str());
+            item->setText(toString(static_cast<FXRealSpinner*>(control)->getValue()).c_str());
             break;
         case CT_INT:
-            item->setText(toString((int) static_cast<FXRealSpinDial*>(control)->getValue()).c_str());
+            item->setText(toString((int) static_cast<FXRealSpinner*>(control)->getValue()).c_str());
             break;
         case CT_BOOL:
 //        return myBoolEditor;
@@ -476,7 +455,7 @@ MFXAddEditTypedTable::onLeftBtnPress(FXObject*, FXSelector, void* ptr) {
         }
 
         // Change current item
-        bool wasEdited = editor != 0;
+        bool wasEdited = editor != nullptr;
         setCurrentItem(tablepos.row, tablepos.col, TRUE);
         if (!wasEdited) {
 
@@ -515,21 +494,21 @@ MFXAddEditTypedTable::onLeftBtnPress(FXObject*, FXSelector, void* ptr) {
 
 // Clicked in list
 long
-MFXAddEditTypedTable::onClicked(FXObject*, FXSelector , void* ptr) {
+MFXAddEditTypedTable::onClicked(FXObject*, FXSelector, void* ptr) {
     if (editor) {
         delete editor;
         input.fm.row = -1;
         input.to.row = -1;
         input.fm.col = -1;
         input.to.col = -1;
-        editor = NULL;
+        editor = nullptr;
         current.row = -1;
         current.col = -1;
     }
     if (target && target->tryHandle(this, FXSEL(SEL_CLICKED, message), ptr)) {
         return 1;
     }
-    handle(this, FXSEL(SEL_COMMAND, ID_START_INPUT), NULL);
+    handle(this, FXSEL(SEL_COMMAND, ID_START_INPUT), nullptr);
     return 1;
 }
 
@@ -542,20 +521,20 @@ long MFXAddEditTypedTable::onDoubleClicked(FXObject*, FXSelector, void* ptr) {
         input.to.row = -1;
         input.fm.col = -1;
         input.to.col = -1;
-        editor = NULL;
+        editor = nullptr;
     } else {
         if (target && target->tryHandle(this, FXSEL(SEL_CLICKED, message), ptr)) {
             return 1;
         }
-        handle(this, FXSEL(SEL_COMMAND, ID_START_INPUT), NULL);
+        handle(this, FXSEL(SEL_COMMAND, ID_START_INPUT), nullptr);
     }
     return 1;
 }
 
 
 CellType
-MFXAddEditTypedTable::getCellType(size_t pos) const {
-    if (myCellTypes.size() <= pos) {
+MFXAddEditTypedTable::getCellType(int pos) const {
+    if ((int)myCellTypes.size() <= pos) {
         return CT_UNDEFINED;
     }
     return myCellTypes[pos];
@@ -563,20 +542,20 @@ MFXAddEditTypedTable::getCellType(size_t pos) const {
 
 
 void
-MFXAddEditTypedTable::setCellType(size_t pos, CellType t) {
-    while (myCellTypes.size() < pos + 1) {
+MFXAddEditTypedTable::setCellType(int pos, CellType t) {
+    while ((int)myCellTypes.size() < pos + 1) {
         myCellTypes.push_back(CT_UNDEFINED);
     }
     myCellTypes[pos] = t;
 }
 
 void
-MFXAddEditTypedTable::setNumberCellParams(size_t pos, double min, double max,
+MFXAddEditTypedTable::setNumberCellParams(int pos, double min, double max,
         double steps1,
         double steps2,
         double steps3,
         const std::string& format) {
-    while (myNumberCellParams.size() <= pos) {
+    while ((int)myNumberCellParams.size() <= pos) {
         NumberCellParams np;
         np.format = "undefined";
         myNumberCellParams.push_back(np);
@@ -594,8 +573,8 @@ MFXAddEditTypedTable::setNumberCellParams(size_t pos, double min, double max,
 
 
 MFXAddEditTypedTable::NumberCellParams
-MFXAddEditTypedTable::getNumberCellParams(size_t pos) const {
-    if (myNumberCellParams.size() <= pos) {
+MFXAddEditTypedTable::getNumberCellParams(int pos) const {
+    if ((int)myNumberCellParams.size() <= pos) {
         NumberCellParams np;
         np.format = "undefined";
         return np;
@@ -606,9 +585,9 @@ MFXAddEditTypedTable::getNumberCellParams(size_t pos) const {
 
 
 void
-MFXAddEditTypedTable::setEnums(size_t pos,
+MFXAddEditTypedTable::setEnums(int pos,
                                const std::vector<std::string>& params) {
-    while (myEnums.size() <= pos) {
+    while ((int)myEnums.size() <= pos) {
         myEnums.push_back(std::vector<std::string>());
     }
     myEnums[pos] = params;
@@ -616,9 +595,9 @@ MFXAddEditTypedTable::setEnums(size_t pos,
 
 
 void
-MFXAddEditTypedTable::addEnum(size_t pos,
+MFXAddEditTypedTable::addEnum(int pos,
                               const std::string& e) {
-    while (myEnums.size() <= pos) {
+    while ((int)myEnums.size() <= pos) {
         myEnums.push_back(std::vector<std::string>());
     }
     myEnums[pos].push_back(e);
@@ -626,11 +605,9 @@ MFXAddEditTypedTable::addEnum(size_t pos,
 
 
 const std::vector<std::string>&
-MFXAddEditTypedTable::getEnums(size_t pos) const {
+MFXAddEditTypedTable::getEnums(int pos) const {
     return myEnums[pos];
 }
 
 
-
 /****************************************************************************/
-
