@@ -1,41 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MFXEditableTable.cpp
 /// @author  Daniel Krajzewicz
 /// @date    2004-07-02
-/// @version $Id: MFXEditableTable.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // missing_desc
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2015 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
-#include <fx.h>
 #include <fxkeys.h>
 #include "MFXEditableTable.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 FXDEFMAP(MFXEditableTable) MFXEditableTableMap[] = {
     FXMAPFUNC(SEL_LEFTBUTTONPRESS, 0, MFXEditableTable::onLeftBtnPress),
     FXMAPFUNC(SEL_LEFTBUTTONRELEASE, 0, MFXEditableTable::onLeftBtnRelease),
@@ -115,7 +101,7 @@ MFXEditableTable::MFXEditableTable(FXComposite* p, FXObject* tgt,
                                    FXint x, FXint y, FXint w, FXint h,
                                    FXint pl, FXint pr, FXint pt, FXint pb)
     : FXTable(p, tgt, sel, opts, x, y, w, h, pl, pr, pt, pb),
-      myEditedItem(0) {
+      myEditedItem(nullptr) {
     myEditor =
         new FXTextField((FXComposite*)getParent(), 1, this,
                         ID_EDITOR, FRAME_NORMAL | LAYOUT_EXPLICIT);
@@ -136,7 +122,7 @@ MFXEditableTable::onLeftBtnPress(FXObject* sender, FXSelector sel, void* ptr) {
 
 
 long
-MFXEditableTable::onLeftBtnClicked(FXObject* , FXSelector , void* ptr) {
+MFXEditableTable::onLeftBtnClicked(FXObject*, FXSelector, void* ptr) {
     FXTablePos* e = (FXTablePos*)ptr;
     int r = e->row;
     int c = e->col;
@@ -170,7 +156,7 @@ MFXEditableTable::onLeftBtnRelease(FXObject* sender, FXSelector sel, void* ptr)
 
 void
 MFXEditableTable::editItem(FXTableItem* item, FXint how) {
-    if (item == 0) {
+    if (item == nullptr) {
         editEnd();
         return;
     }
@@ -218,11 +204,11 @@ MFXEditableTable::editEnd() {
         return;
     }
     FXTableItem* item = myEditedItem;
-    myEditedItem = NULL;
+    myEditedItem = nullptr;
     myEditor->hide();
     setFocus();
     FXString text = myEditor->getText();
-    myEditedItem = 0;
+    myEditedItem = nullptr;
     if (item->getText() == text) {
         return;
     }
@@ -244,9 +230,9 @@ MFXEditableTable::editEnd() {
 
 
 long
-MFXEditableTable::onEditEnd(FXObject*, FXSelector , void*) {
+MFXEditableTable::onEditEnd(FXObject*, FXSelector, void*) {
     delete myEditedItem;
-    myEditedItem = 0;
+    myEditedItem = nullptr;
     return 1;
 }
 
@@ -256,7 +242,7 @@ MFXEditableTable::editCancel() {
     if (!myEditedItem) {
         return;
     }
-    myEditedItem = 0;
+    myEditedItem = nullptr;
     myEditor->hide();
     setFocus();
     killSelection(true);
@@ -299,7 +285,7 @@ MFXEditableTable::onKeyRelease(FXObject* sender, FXSelector sel, void* ptr) {
 
 
 long
-MFXEditableTable::onEditFocusOut(FXObject*, FXSelector , void*) {
+MFXEditableTable::onEditFocusOut(FXObject*, FXSelector, void*) {
     editEnd();
     return 0;
 }
@@ -446,7 +432,7 @@ MFXEditableTable::onClipboardRequest(FXObject* o, FXSelector s, void* d) {
 
 
 long
-MFXEditableTable::onDoubleClicked(FXObject*, FXSelector , void*) {
+MFXEditableTable::onDoubleClicked(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -454,7 +440,7 @@ MFXEditableTable::onDoubleClicked(FXObject*, FXSelector , void*) {
 
 
 long
-MFXEditableTable::onTripleClicked(FXObject*, FXSelector , void*) {
+MFXEditableTable::onTripleClicked(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -497,7 +483,7 @@ MFXEditableTable::onCmdVertGrid(FXObject* o, FXSelector s, void* d) {
 
 
 long
-MFXEditableTable::onCmdDeleteColumn(FXObject*, FXSelector , void*) {
+MFXEditableTable::onCmdDeleteColumn(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -505,7 +491,7 @@ MFXEditableTable::onCmdDeleteColumn(FXObject*, FXSelector , void*) {
 
 
 long
-MFXEditableTable::onUpdDeleteColumn(FXObject*, FXSelector , void*) {
+MFXEditableTable::onUpdDeleteColumn(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -513,7 +499,7 @@ MFXEditableTable::onUpdDeleteColumn(FXObject*, FXSelector , void*) {
 
 
 long
-MFXEditableTable::onCmdDeleteRow(FXObject*, FXSelector , void*) {
+MFXEditableTable::onCmdDeleteRow(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -521,7 +507,7 @@ MFXEditableTable::onCmdDeleteRow(FXObject*, FXSelector , void*) {
 
 
 long
-MFXEditableTable::onUpdDeleteRow(FXObject*, FXSelector , void*) {
+MFXEditableTable::onUpdDeleteRow(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -529,7 +515,7 @@ MFXEditableTable::onUpdDeleteRow(FXObject*, FXSelector , void*) {
 
 
 long
-MFXEditableTable::onCmdInsertColumn(FXObject*, FXSelector , void*) {
+MFXEditableTable::onCmdInsertColumn(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -537,7 +523,7 @@ MFXEditableTable::onCmdInsertColumn(FXObject*, FXSelector , void*) {
 
 
 long
-MFXEditableTable::onCmdInsertRow(FXObject*, FXSelector , void*) {
+MFXEditableTable::onCmdInsertRow(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -624,7 +610,7 @@ MFXEditableTable::onCmdSelectRowIndex(FXObject* o, FXSelector s, void* d) {
 
 
 long
-MFXEditableTable::onCmdSelectColumnIndex(FXObject*, FXSelector , void*) {
+MFXEditableTable::onCmdSelectColumnIndex(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -632,7 +618,7 @@ MFXEditableTable::onCmdSelectColumnIndex(FXObject*, FXSelector , void*) {
 
 
 long
-MFXEditableTable::onCmdSelectColumn(FXObject*, FXSelector , void*) {
+MFXEditableTable::onCmdSelectColumn(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -647,7 +633,7 @@ MFXEditableTable::onCmdSelectRow(FXObject* o, FXSelector s, void* d) {
 
 
 long
-MFXEditableTable::onCmdSelectCell(FXObject*, FXSelector , void*) {
+MFXEditableTable::onCmdSelectCell(FXObject*, FXSelector, void*) {
 //    editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -655,7 +641,7 @@ MFXEditableTable::onCmdSelectCell(FXObject*, FXSelector , void*) {
 
 
 long
-MFXEditableTable::onCmdSelectAll(FXObject*, FXSelector , void*) {
+MFXEditableTable::onCmdSelectAll(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -663,7 +649,7 @@ MFXEditableTable::onCmdSelectAll(FXObject*, FXSelector , void*) {
 
 
 long
-MFXEditableTable::onCmdDeselectAll(FXObject*, FXSelector , void*) {
+MFXEditableTable::onCmdDeselectAll(FXObject*, FXSelector, void*) {
     editEnd();
     return 1;
 //    return FXTable::onUngrabbed(o, s, d);
@@ -697,6 +683,4 @@ MFXEditableTable::writeProtectCol(int col, bool val) {
 }
 
 
-
 /****************************************************************************/
-
