@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    Boundary.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: Boundary.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A class that stores the 2D geometrical boundary
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 #include <utility>
 
 #include "GeomHelper.h"
@@ -36,10 +27,6 @@
 #include "PositionVector.h"
 #include "Position.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -47,18 +34,30 @@
 Boundary::Boundary()
     : myXmin(10000000000.0), myXmax(-10000000000.0),
       myYmin(10000000000.0), myYmax(-10000000000.0),
+      myZmin(10000000000.0), myZmax(-10000000000.0),
       myWasInitialised(false) {}
 
 
-Boundary::Boundary(SUMOReal x1, SUMOReal y1, SUMOReal x2, SUMOReal y2)
+Boundary::Boundary(double x1, double y1, double x2, double y2)
     : myXmin(10000000000.0), myXmax(-10000000000.0),
       myYmin(10000000000.0), myYmax(-10000000000.0),
+      myZmin(10000000000.0), myZmax(-10000000000.0),
       myWasInitialised(false) {
     add(x1, y1);
     add(x2, y2);
 }
 
 
+Boundary::Boundary(double x1, double y1, double z1, double x2, double y2, double z2)
+    : myXmin(10000000000.0), myXmax(-10000000000.0),
+      myYmin(10000000000.0), myYmax(-10000000000.0),
+      myZmin(10000000000.0), myZmax(-10000000000.0),
+      myWasInitialised(false) {
+    add(x1, y1, z1);
+    add(x2, y2, z2);
+}
+
+
 Boundary::~Boundary() {}
 
 
@@ -68,22 +67,28 @@ Boundary::reset() {
     myXmax = -10000000000.0;
     myYmin = 10000000000.0;
     myYmax = -10000000000.0;
+    myZmin = 10000000000.0;
+    myZmax = -10000000000.0;
     myWasInitialised = false;
 }
 
 
 void
-Boundary::add(SUMOReal x, SUMOReal y) {
+Boundary::add(double x, double y, double z) {
     if (!myWasInitialised) {
         myYmin = y;
         myYmax = y;
         myXmin = x;
         myXmax = x;
+        myZmin = z;
+        myZmax = z;
     } else {
         myXmin = myXmin < x ? myXmin : x;
         myXmax = myXmax > x ? myXmax : x;
         myYmin = myYmin < y ? myYmin : y;
         myYmax = myYmax > y ? myYmax : y;
+        myZmin = myZmin < z ? myZmin : z;
+        myZmax = myZmax > z ? myZmax : z;
     }
     myWasInitialised = true;
 }
@@ -91,69 +96,88 @@ Boundary::add(SUMOReal x, SUMOReal y) {
 
 void
 Boundary::add(const Position& p) {
-    add(p.x(), p.y());
+    add(p.x(), p.y(), p.z());
 }
 
 
 void
 Boundary::add(const Boundary& p) {
-    add(p.xmin(), p.ymin());
-    add(p.xmax(), p.ymax());
+    add(p.xmin(), p.ymin(), p.zmin());
+    add(p.xmax(), p.ymax(), p.zmax());
 }
 
 
 Position
 Boundary::getCenter() const {
-    return Position((myXmin + myXmax) / (SUMOReal) 2.0, (myYmin + myYmax) / (SUMOReal) 2.0);
+    return Position((myXmin + myXmax) / (double) 2.0, (myYmin + myYmax) / (double) 2.0, (myZmin + myZmax) / (double) 2.0);
 }
 
 
-SUMOReal
+double
 Boundary::xmin() const {
     return myXmin;
 }
 
 
-SUMOReal
+double
 Boundary::xmax() const {
     return myXmax;
 }
 
 
-SUMOReal
+double
 Boundary::ymin() const {
     return myYmin;
 }
 
 
-SUMOReal
+double
 Boundary::ymax() const {
     return myYmax;
 }
 
 
-SUMOReal
+double
+Boundary::zmin() const {
+    return myZmin;
+}
+
+
+double
+Boundary::zmax() const {
+    return myZmax;
+}
+
+
+double
 Boundary::getWidth() const {
     return myXmax - myXmin;
 }
 
 
-SUMOReal
+double
 Boundary::getHeight() const {
     return myYmax - myYmin;
 }
 
 
-bool
-Boundary::around(const Position& p, SUMOReal offset) const {
-    return
-        (p.x() <= myXmax + offset && p.x() >= myXmin - offset) &&
-        (p.y() <= myYmax + offset && p.y() >= myYmin - offset);
+double
+Boundary::getZRange() const {
+    return myZmax - myZmin;
 }
 
 
 bool
-Boundary::overlapsWith(const AbstractPoly& p, SUMOReal offset) const {
+Boundary::around(const Position& p, double offset) const {
+    return
+        (p.x() <= myXmax + offset && p.x() >= myXmin - offset) &&
+        (p.y() <= myYmax + offset && p.y() >= myYmin - offset) &&
+        (p.z() <= myZmax + offset && p.z() >= myZmin - offset);
+}
+
+
+bool
+Boundary::overlapsWith(const AbstractPoly& p, double offset) const {
     if (
         // check whether one of my points lies within the given poly
         partialWithin(p, offset) ||
@@ -188,7 +212,81 @@ Boundary::crosses(const Position& p1, const Position& p2) const {
 
 
 bool
-Boundary::partialWithin(const AbstractPoly& poly, SUMOReal offset) const {
+Boundary::isInitialised() const {
+    return myWasInitialised;
+}
+
+
+double
+Boundary::distanceTo2D(const Position& p) const {
+    const double leftDist = myXmin - p.x();
+    const double rightDist = p.x() - myXmax;
+    const double bottomDist = myYmin - p.y();
+    const double topDist = p.y() - myYmax;
+    if (leftDist > 0.) {
+        if (bottomDist > 0.) {
+            return sqrt(leftDist * leftDist + bottomDist * bottomDist);
+        }
+        if (topDist > 0.) {
+            return sqrt(leftDist * leftDist + topDist * topDist);
+        }
+        return leftDist;
+    }
+    if (rightDist > 0.) {
+        if (bottomDist > 0.) {
+            return sqrt(rightDist * rightDist + bottomDist * bottomDist);
+        }
+        if (topDist > 0.) {
+            return sqrt(rightDist * rightDist + topDist * topDist);
+        }
+        return rightDist;
+    }
+    if (bottomDist > 0) {
+        return bottomDist;
+    }
+    if (topDist > 0) {
+        return topDist;
+    }
+    return 0.;
+}
+
+
+double
+Boundary::distanceTo2D(const Boundary& b) const {
+    const double leftDist = myXmin - b.myXmax;
+    const double rightDist = b.myXmin - myXmax;
+    const double bottomDist = myYmin - b.myYmax;
+    const double topDist = b.myYmin - myYmax;
+    if (leftDist > 0.) {
+        if (bottomDist > 0.) {
+            return sqrt(leftDist * leftDist + bottomDist * bottomDist);
+        }
+        if (topDist > 0.) {
+            return sqrt(leftDist * leftDist + topDist * topDist);
+        }
+        return leftDist;
+    }
+    if (rightDist > 0.) {
+        if (bottomDist > 0.) {
+            return sqrt(rightDist * rightDist + bottomDist * bottomDist);
+        }
+        if (topDist > 0.) {
+            return sqrt(rightDist * rightDist + topDist * topDist);
+        }
+        return rightDist;
+    }
+    if (bottomDist > 0) {
+        return bottomDist;
+    }
+    if (topDist > 0) {
+        return topDist;
+    }
+    return 0.;
+}
+
+
+bool
+Boundary::partialWithin(const AbstractPoly& poly, double offset) const {
     return
         poly.around(Position(myXmax, myYmax), offset) ||
         poly.around(Position(myXmin, myYmax), offset) ||
@@ -198,7 +296,7 @@ Boundary::partialWithin(const AbstractPoly& poly, SUMOReal offset) const {
 
 
 Boundary&
-Boundary::grow(SUMOReal by) {
+Boundary::grow(double by) {
     myXmax += by;
     myYmax += by;
     myXmin -= by;
@@ -207,14 +305,14 @@ Boundary::grow(SUMOReal by) {
 }
 
 void
-Boundary::growWidth(SUMOReal by) {
+Boundary::growWidth(double by) {
     myXmin -= by;
     myXmax += by;
 }
 
 
 void
-Boundary::growHeight(SUMOReal by) {
+Boundary::growHeight(double by) {
     myYmin -= by;
     myYmax += by;
 }
@@ -223,7 +321,7 @@ void
 Boundary::flipY() {
     myYmin *= -1.0;
     myYmax *= -1.0;
-    SUMOReal tmp = myYmin;
+    double tmp = myYmin;
     myYmin = myYmax;
     myYmax = tmp;
 }
@@ -237,8 +335,27 @@ operator<<(std::ostream& os, const Boundary& b) {
 }
 
 
+bool
+Boundary::operator==(const Boundary& b) const {
+    return (
+               myXmin == b.myXmin &&
+               myXmax == b.myXmax &&
+               myYmin == b.myYmin &&
+               myYmax == b.myYmax &&
+               myZmin == b.myZmin &&
+               myZmax == b.myZmax &&
+               myWasInitialised == b.myWasInitialised);
+}
+
+
+bool
+Boundary::operator!=(const Boundary& b) const {
+    return !(*this == b);
+}
+
+
 void
-Boundary::set(SUMOReal xmin, SUMOReal ymin, SUMOReal xmax, SUMOReal ymax) {
+Boundary::set(double xmin, double ymin, double xmax, double ymax) {
     myXmin = xmin;
     myYmin = ymin;
     myXmax = xmax;
@@ -247,14 +364,14 @@ Boundary::set(SUMOReal xmin, SUMOReal ymin, SUMOReal xmax, SUMOReal ymax) {
 
 
 void
-Boundary::moveby(SUMOReal x, SUMOReal y) {
+Boundary::moveby(double x, double y, double z) {
     myXmin += x;
     myYmin += y;
+    myZmin += z;
     myXmax += x;
     myYmax += y;
+    myZmax += z;
 }
 
 
-
 /****************************************************************************/
-
