@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    Bresenham.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: Bresenham.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A class to realise a uniform n:m - relationship using the
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef Bresenham_h
-#define Bresenham_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 
 // ===========================================================================
@@ -57,7 +47,7 @@ public:
         virtual ~BresenhamCallBack() { }
 
         /** called when a bresenham step has been computed */
-        virtual void execute(const unsigned int val1, const unsigned int val2) = 0;
+        virtual void execute(const int val1, const int val2) = 0;
     };
 
 public:
@@ -65,11 +55,5 @@ public:
         the higher number is increased by one for each step while the smaller
         is increased by smaller/higher.
         In each step, the callback is executed. */
-    static void compute(BresenhamCallBack* callBack, const unsigned int val1, const unsigned int val2);
+    static void compute(BresenhamCallBack* callBack, const int val1, const int val2);
 };
-
-
-#endif
-
-/****************************************************************************/
-
