@@ -1,41 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GeomConvHelper.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2003
-/// @version $Id: GeomConvHelper.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Some helping functions for geometry parsing
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <sstream>
 #include <utils/geom/PositionVector.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/StringTokenizer.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include "GeomConvHelper.h"
 
 
@@ -62,12 +53,12 @@ GeomConvHelper::parseShapeReporting(const std::string& shpdef, const std::string
             return PositionVector();
         }
         try {
-            double x = TplConvert::_2double(pos.next().c_str());
-            double y = TplConvert::_2double(pos.next().c_str());
+            double x = StringUtils::toDouble(pos.next());
+            double y = StringUtils::toDouble(pos.next());
             if (pos.size() == 2) {
                 shape.push_back(Position(x, y));
             } else {
-                double z = TplConvert::_2double(pos.next().c_str());
+                double z = StringUtils::toDouble(pos.next());
                 shape.push_back(Position(x, y, z));
             }
         } catch (NumberFormatException&) {
@@ -94,10 +85,10 @@ GeomConvHelper::parseBoundaryReporting(const std::string& def, const std::string
         return Boundary();
     }
     try {
-        double xmin = TplConvert::_2double(st.next().c_str());
-        double ymin = TplConvert::_2double(st.next().c_str());
-        double xmax = TplConvert::_2double(st.next().c_str());
-        double ymax = TplConvert::_2double(st.next().c_str());
+        double xmin = StringUtils::toDouble(st.next());
+        double ymin = StringUtils::toDouble(st.next());
+        double xmax = StringUtils::toDouble(st.next());
+        double ymax = StringUtils::toDouble(st.next());
         return Boundary(xmin, ymin, xmax, ymax);
     } catch (NumberFormatException&) {
         emitError(report, "Shape", objecttype, objectid, "not numeric entry");
@@ -117,11 +108,11 @@ GeomConvHelper::emitError(bool report, const std::string& what, const std::strin
     }
     std::ostringstream oss;
     oss << what << " of ";
-    if (objectid == 0) {
+    if (objectid == nullptr) {
         oss << "a(n) ";
     }
     oss << objecttype;
-    if (objectid != 0) {
+    if (objectid != nullptr) {
         oss << " '" << objectid << "'";
     }
     oss << " is broken: " << desc << ".";
@@ -129,6 +120,4 @@ GeomConvHelper::emitError(bool report, const std::string& what, const std::strin
 }
 
 
-
 /****************************************************************************/
-
