@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GeomHelper.h
 /// @author  Daniel Krajzewicz
 /// @author  Friedemann Wesner
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GeomHelper.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Some static methods performing geometrical operations
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GeomHelper_h
-#define GeomHelper_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <cmath>
 #include "Position.h"
@@ -44,6 +34,7 @@
 
 #define DEG2RAD(x) static_cast<double>((x) * M_PI / 180.)
 #define RAD2DEG(x) static_cast<double>((x) * 180. / M_PI)
+#define GRAVITY 9.80665
 
 
 // ===========================================================================
@@ -81,6 +72,10 @@ public:
         const Position& lineStart, const Position& lineEnd,
         const Position& p, bool perpendicular = true);
 
+    static double nearest_offset_on_line_to_point25D(
+        const Position& lineStart, const Position& lineEnd,
+        const Position& p, bool perpendicular = true);
+
     static Position crossPoint(const Boundary& b,
                                const PositionVector& v);
 
@@ -128,6 +123,13 @@ public:
      */
     static double naviDegree(const double angle);
 
+    /** Converts an angle from navigational degrees to mathematical radians.
+     * @see naviDegree
+     * @param[in] angle The angle in degree to convert
+     * @return the angle in radians
+     */
+    static double fromNaviDegree(const double angle);
+
     /** Converts an angle from mathematical radians where 0 is to the right and positive angles
      *  are counterclockwise to the legacy degrees used in sumo where 0 is down and positive means clockwise.
      *  If positive is true the result is in the range [0, 360), otherwise in the range [-180, 180).
@@ -136,9 +138,30 @@ public:
      */
     static double legacyDegree(const double angle, const bool positive = false);
 
+    /** Creates a circular polygon
+     * @param[in] radius Radius of the circle
+     * @param[in] center Position of the circle's center
+     * @param[in] nPoints Number of points of the circle (Polygon's shape will have noPoints+1 points), must be >=3
+     * @return the polygon approximating the circle
+     */
+    static PositionVector makeCircle(const double radius, const Position& center, unsigned int nPoints);
+
+    /** Creates a circular polygon
+     * @param[in] radius1 Inner radius of the ring
+     * @param[in] radius2 Outer radius of the ring
+     * @param[in] center Position of the circle's center
+     * @param[in] nPoints Number of points of the circle (Polygon's shape will have noPoints+1 points), must be >=3
+     * @return the polygon approximating the circle
+     */
+    static PositionVector makeRing(const double radius1, const double radius2, const Position& center, unsigned int nPoints);
+
+    /// @brief calculate lotSpace position
+    static const Position calculateLotSpacePosition(const PositionVector& shape, const int index,
+            const double spaceDim, const double angle, const double width, const double length);
+
+    /// @brief calculate lotSpace angle
+    static double calculateLotSpaceAngle(const PositionVector& shape, const int index, const double spaceDim, const double angle);
+
+    /// @brief calculate lotSpace slope
+    static double calculateLotSpaceSlope(const PositionVector& shape, const int index, const double spaceDim);
 };
-
-
-#endif
-
-/****************************************************************************/
