@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PositionVector.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: PositionVector.h 23242 2017-03-07 12:07:08Z namdre $
 ///
 // A list of positions
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef PositionVector_h
-#define PositionVector_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <limits>
@@ -77,6 +67,9 @@ public:
     /// @brief Destructor
     ~PositionVector();
 
+    /// @brief empty Vector
+    static const PositionVector EMPTY;
+
     /// @name methode for iterate over PositionVector
     /// @{
     /// @brief iterator
@@ -161,7 +154,6 @@ public:
     const Position& operator[](int index) const;
 
     /// @brief returns the position at the given index
-    /// @ToDo !!! exceptions?
     Position& operator[](int index);
 
     /// @brief Returns the position at the given length
@@ -217,7 +209,7 @@ public:
     bool partialWithin(const AbstractPoly& poly, double offset = 0) const;
 
     /// @brief Returns the two lists made when this list vector is splitted at the given point
-    std::pair<PositionVector, PositionVector> splitAt(double where) const;
+    std::pair<PositionVector, PositionVector> splitAt(double where, bool use2D = false) const;
 
     //// @brief Output operator
     friend std::ostream& operator<<(std::ostream& os, const PositionVector& geom);
@@ -228,18 +220,24 @@ public:
     //// @brief add a position
     void add(double xoff, double yoff, double zoff);
 
-    //// @brief add a position
+    //// @brief substract a position
     void add(const Position& offset);
 
+    //// @brief add a position
+    void sub(double xoff, double yoff, double zoff);
+
+    //// @brief substract a position
+    void sub(const Position& offset);
+
+    //// @brief adds a position without modifying the vector itself but returning the result
+    PositionVector added(const Position& offset) const;
+
     //// @brief mirror coordinates along the x-axis
     void mirrorX();
 
     //// @brief rotate all points around (0,0) in the plane by the given angle
     void rotate2D(double angle);
 
-    //// @brief get a convex Hull of position vector
-    PositionVector convexHull() const;
-
     //// @brief append the given vector to this one
     void append(const PositionVector& v, double sameThreshold = 2.0);
 
@@ -252,10 +250,10 @@ public:
     /// @brief get subpart of a position vector using index and a cout
     PositionVector getSubpartByIndex(int beginIndex, int count) const;
 
-    /// @brief short as polygon CV by angle
+    /// @brief sort as polygon CW by angle
     void sortAsPolyCWByAngle();
 
-    /// @brief shory by increasing X-Y Psitions
+    /// @brief sort by increasing X-Y Positions
     void sortByIncreasingXY();
 
     /// @brief extrapolate position vector
@@ -271,13 +269,20 @@ public:
     static Position sideOffset(const Position& beg, const Position& end, const double amount);
 
     /// @brief move position vector to side using certain ammount
-    void move2side(double amount);
+    void move2side(double amount, double maxExtension = 100);
 
-    /// @brief get angle  in certain position of position vector
+    /// @brief move position vector to side using a custom offset for each geometry point
+    void move2side(std::vector<double> amount, double maxExtension = 100);
+
+    /// @brief get angle in certain position of position vector
     double angleAt2D(int pos) const;
 
-    /// @brief inserts p between the two closest positions and returns the insertion index
-    int insertAtClosest(const Position& p);
+    /**@brief inserts p between the two closest positions
+     * @param p position to be inserted
+     * @param interpolateZ flag to enable/disable interpolation of Z Value between the two closest positions
+     * @return the insertion index
+     */
+    int insertAtClosest(const Position& p, bool interpolateZ);
 
     /// @brief removes the point closest to p and return the removal index
     int removeClosest(const Position& p);
@@ -285,6 +290,15 @@ public:
     /// @brief comparing operation
     bool operator==(const PositionVector& v2) const;
 
+    /// @brief comparing operation
+    bool operator!=(const PositionVector& v2) const;
+
+    /// @brief substracts two vectors (requires vectors of the same length)
+    PositionVector operator-(const PositionVector& v2) const;
+
+    /// @brief adds two vectors (requires vectors of the same length)
+    PositionVector operator+(const PositionVector& v2) const;
+
     /// @brief clase for CW Sorter
     class as_poly_cw_sorter {
     public:
@@ -315,6 +329,9 @@ public:
     /// @brief return the nearest offest to point 2D
     double nearest_offset_to_point2D(const Position& p, bool perpendicular = true) const;
 
+    /// @brief return the nearest offest to point 2D projected onto the 3D geometry
+    double nearest_offset_to_point25D(const Position& p, bool perpendicular = true) const;
+
     /** @brief return position p within the length-wise coordinate system
      * defined by this position vector. The x value is the same as that returned
      * by nearest_offset_to_point2D(p) and the y value is the perpendicular distance to this
@@ -324,9 +341,11 @@ public:
      */
     Position transformToVectorCoordinates(const Position& p, bool extend = false) const;
 
-    /// @brief index of the closest position to p
-    /// @note: may only be called for a non-empty vector
-    int indexOfClosest(const Position& p) const;
+    /* @brief index of the closest position to p
+     * @in twoD whether all positions should be projected onto the plan
+       @note: may only be called for a non-empty vector
+    */
+    int indexOfClosest(const Position& p, bool twoD = false) const;
 
     /// @brief distances of all my points to s and all of s points to myself
     /// @note if perpendicular is set to true, only the perpendicular distances are returned
@@ -335,15 +354,27 @@ public:
     /// @brief closest 2D-distance to point p (or -1 if perpendicular is true and the point is beyond this vector)
     double distance2D(const Position& p, bool perpendicular = false) const;
 
+    /// @brief insert in front a Position
+    void push_front(const Position& p);
+
+    /// @brief pop first Position
+    void pop_front();
+
     /// @brief insert in back a non double position
     void push_back_noDoublePos(const Position& p);
 
     /// @brief insert in front a non double position
     void push_front_noDoublePos(const Position& p);
 
+    /// @brief insert in front a non double position
+    void insert_noDoublePos(const std::vector<Position>::iterator& at, const Position& p);
+
     /// @brief check if PositionVector is closed
     bool isClosed() const;
 
+    /// @brief check if PositionVector is NAN
+    bool isNAN() const;
+
     /** @brief Removes positions if too near
      * @param[in] minDist The minimum accepted distance; default: POSITION_EPS
      * @param[in] assertLength Whether the result must at least contain two points (be a line); default: false, to ensure original behaviour
@@ -361,26 +392,34 @@ public:
      * @param[in] extend how long to extend this vector for finding an orthogonal
      * @param[in] front Whether to take the segment before or after the base point in case of ambiguity
      * @param[in] length the length of the orthogonal
+     * @param[in] deg the rotation angle relative to the shape direction
      */
-    PositionVector getOrthogonal(const Position& p, double extend, bool before, double length = 1.0) const;
-
+    PositionVector getOrthogonal(const Position& p, double extend, bool before, double length = 1.0, double deg = 90) const;
 
     /// @brief returned vector that is smoothed at the front (within dist)
     PositionVector smoothedZFront(double dist = std::numeric_limits<double>::max()) const;
 
+    /// @brief returned vector that varies z smoothly over its length
+    PositionVector interpolateZ(double zStart, double zEnd) const;
+
+    /**@brief resample shape (i.e. transform to segments, equal spacing)
+     * @param[in] maxLength lenght of every segment
+     * @param[in] adjustEnd enable or disable adjust end (i.e. result has the same original lenght, last segment could be short)
+     */
+    PositionVector resample(double maxLength, const bool adjustEnd) const;
+
     /// @brief return the offset at the given index
     double offsetAtIndex2D(int index) const;
 
-    /// @brief return the maximum grade of all segments as a fraction of zRange/length2D
-    double getMaxGrade() const;
+    /* @brief return the maximum grade of all segments as a fraction of zRange/length2D
+     * @param[out] maxJump The maximum vertical jump (with grade infinity)
+     */
+    double getMaxGrade(double& maxJump) const;
+
+    /// @brief return a bezier interpolation
+    PositionVector bezier(int numPoints);
 
 private:
     /// @brief return whether the line segments defined by Line p11,p12 and Line p21,p22 intersect
     static bool intersects(const Position& p11, const Position& p12, const Position& p21, const Position& p22, const double withinDist = 0., double* x = 0, double* y = 0, double* mu = 0);
 };
-
-
-#endif
-
-/****************************************************************************/
-
