@@ -1,45 +1,36 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GLObjectValuePassConnector.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Fri, 29.04.2005
-/// @version $Id: GLObjectValuePassConnector.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Class passing values from a GUIGlObject to another object
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GLObjectValuePassConnector_h
-#define GLObjectValuePassConnector_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <algorithm>
 #include <vector>
 #include <map>
+#include <functional>
+#include <utils/foxtools/fxheader.h>
 #include <utils/common/ValueSource.h>
 #include <utils/common/ValueRetriever.h>
 #include <utils/gui/globjects/GUIGlObject.h>
-#include <utils/foxtools/MFXMutex.h>
 
 
 // ===========================================================================
@@ -57,7 +48,7 @@ class GUIGlObject;
  *
  * A templated instance has some static member variables. They have to be defined
  *  in a cpp file. They may be found in GUINet.cpp. Two instances are used:
- * - one passing SUMOReal-values
+ * - one passing double-values
  * - one passing time tls phase definitions
  *
  * @see GUIGlObject
@@ -72,7 +63,7 @@ public:
      */
     GLObjectValuePassConnector(GUIGlObject& o, ValueSource<T>* source, ValueRetriever<T>* retriever)
         : myObject(o), mySource(source), myRetriever(retriever) { /*, myIsInvalid(false) */
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         myContainer.push_back(this);
     }
 
@@ -95,17 +86,19 @@ public:
     /** @brief Updates all instances (passes values)
      */
     static void updateAll() {
-        AbstractMutex::ScopedLocker locker(myLock);
-        std::for_each(myContainer.begin(), myContainer.end(), std::mem_fun(&GLObjectValuePassConnector<T>::passValue));
+        FXMutexLock locker(myLock);
+        for (GLObjectValuePassConnector<T>* const connector : myContainer) {
+            connector->passValue();
+        }
     }
 
 
     /** @brief Deletes all instances
      */
     static void clear() {
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         while (!myContainer.empty()) {
-            delete(*myContainer.begin());
+            delete (*myContainer.begin());
         }
         myContainer.clear();
     }
@@ -117,7 +110,7 @@ public:
      * @param[in] o The object which shall no longer be asked for values
      */
     static void removeObject(GUIGlObject& o) {
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         for (typename std::vector< GLObjectValuePassConnector<T>* >::iterator i = myContainer.begin(); i != myContainer.end();) {
             if ((*i)->myObject.getGlID() == o.getGlID()) {
                 i = myContainer.erase(i);
@@ -153,7 +146,7 @@ protected:
     ValueRetriever<T>* myRetriever;
 
     /// @brief The mutex used to avoid concurrent updates of the connectors container
-    static MFXMutex myLock;
+    static FXMutex myLock;
 
     /// @brief The container of items that shall be updated
     static std::vector< GLObjectValuePassConnector<T>* > myContainer;
@@ -173,10 +166,4 @@ private:
 template<typename T>
 std::vector< GLObjectValuePassConnector<T>* > GLObjectValuePassConnector<T>::myContainer;
 template<typename T>
-MFXMutex GLObjectValuePassConnector<T>::myLock;
-
-
-#endif
-
-/****************************************************************************/
-
+FXMutex GLObjectValuePassConnector<T>::myLock;
