@@ -1,40 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2003-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIMessageWindow.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Tue, 25 Nov 2003
-/// @version $Id: GUIMessageWindow.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A logging window for the gui
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2003-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cassert>
 #include <utils/common/MsgHandler.h>
 #include <utils/gui/globjects/GUIGlObjectStorage.h>
 #include <utils/gui/windows/GUIGlChildWindow.h>
 #include <utils/gui/windows/GUIMainWindow.h>
+#include <utils/gui/div/GUIGlobalSelection.h>
+#include <fxkeys.h>
 #include "GUIMessageWindow.h"
 
 
@@ -48,18 +41,19 @@ bool GUIMessageWindow::myLocateLinks = true;
 // method definitions
 // ===========================================================================
 GUIMessageWindow::GUIMessageWindow(FXComposite* parent) :
-    FXText(parent, 0, 0, 0, 0, 0, 0, 50),
-    myStyles(new FXHiliteStyle[7]),
-    myErrorRetriever(0),
-    myMessageRetriever(0),
-    myWarningRetriever(0) {
+    FXText(parent, nullptr, 0, 0, 0, 0, 0, 50),
+    myStyles(new FXHiliteStyle[8]),
+    myErrorRetriever(nullptr),
+    myMessageRetriever(nullptr),
+    myWarningRetriever(nullptr) {
     setStyled(true);
     setEditable(false);
-    const FXColor white = FXRGB(0xff, 0xff, 0xff);
-    const FXColor blue  = FXRGB(0x00, 0x00, 0x88);
-    const FXColor green = FXRGB(0x00, 0x88, 0x00);
-    const FXColor red   = FXRGB(0x88, 0x00, 0x00);
-    const FXColor yellow = FXRGB(0xe6, 0x98, 0x00);
+    const FXColor white   = FXRGB(0xff, 0xff, 0xff);
+    const FXColor blue    = FXRGB(0x00, 0x00, 0x88);
+    const FXColor green   = FXRGB(0x00, 0x88, 0x00);
+    const FXColor red     = FXRGB(0x88, 0x00, 0x00);
+    const FXColor yellow  = FXRGB(0xe6, 0x98, 0x00);
+    const FXColor fuchsia = FXRGB(0x88, 0x00, 0x88);
     // set separator style
     myStyles[0].normalForeColor = blue;
     myStyles[0].normalBackColor = white;
@@ -90,6 +84,11 @@ GUIMessageWindow::GUIMessageWindow(FXComposite* parent) :
     myStyles[3].hiliteForeColor = yellow;
     myStyles[6] = myStyles[3];
     myStyles[6].style = STYLE_UNDERLINE;
+    // set GLDebug text style
+    myStyles[7] = myStyles[0];
+    myStyles[7].normalForeColor = fuchsia;
+    myStyles[7].selectBackColor = fuchsia;
+    myStyles[7].hiliteForeColor = fuchsia;
     //
     setHiliteStyles(myStyles);
 }
@@ -108,17 +107,30 @@ GUIMessageWindow::getActiveStringObject(const FXString& text, const FXint pos, c
     const FXint idS = MAX2(text.rfind(" '", pos), text.rfind("='", pos));
     const FXint idE = text.find("'", pos);
     if (idS >= 0 && idE >= 0 && idS >= lineS && idE <= lineE) {
-        const FXint typeS = text.rfind(" ", idS - 1);
+        FXint typeS = text.rfind(" ", idS - 1);
         if (typeS >= 0) {
+            if (text.at(typeS + 1) == '(') {
+                typeS++;
+            }
             std::string type(text.mid(typeS + 1, idS - typeS - 1).lower().text());
             if (type == "tllogic") {
                 type = "tlLogic"; // see GUIGlObject.cpp
+            } else if (type == "busstop" || type == "trainstop") {
+                type = "busStop";
+            } else if (type == "containerstop") {
+                type = "containerStop";
+            } else if (type == "chargingstation") {
+                type = "chargingStation";
+            } else if (type == "overheadwiresegment") {
+                type = "overheadWireSegment";
+            } else if (type == "parkingarea") {
+                type = "parkingArea";
             }
             const std::string id(text.mid(idS + 2, idE - idS - 2).text());
             return GUIGlObjectStorage::gIDStorage.getObjectBlocking(type + ":" + id);
         }
     }
-    return 0;
+    return nullptr;
 }
 
 
@@ -131,12 +143,15 @@ GUIMessageWindow::setCursorPos(FXint pos, FXbool notify) {
         if (viewIDs.empty()) {
             return;
         }
-        GUIGlChildWindow* const child = dynamic_cast<GUIGlChildWindow*>(main->getViewByID(viewIDs[0]));
+        GUIGlChildWindow* const child = main->getViewByID(viewIDs[0]);
         const FXString text = getText();
         const GUIGlObject* const glObj = getActiveStringObject(text, pos, lineStart(pos), lineEnd(pos));
-        if (glObj != 0) {
+        if (glObj != nullptr) {
             child->setView(glObj->getGlID());
             GUIGlObjectStorage::gIDStorage.unblockObject(glObj->getGlID());
+            if (getApp()->getKeyState(KEY_Control_L)) {
+                gSelected.toggleSelection(glObj->getGlID());
+            }
         }
     }
 }
@@ -150,15 +165,23 @@ GUIMessageWindow::appendMsg(GUIEventType eType, const std::string& msg) {
     // build the styled message
     FXint style = 1;
     switch (eType) {
-        case EVENT_ERROR_OCCURED:
+        case EVENT_DEBUG_OCCURRED:
+            // color: blue
+            style = 0;
+            break;
+        case EVENT_GLDEBUG_OCCURRED:
+            // color: fuchsia
+            style = 7;
+            break;
+        case EVENT_ERROR_OCCURRED:
             // color: red
             style = 2;
             break;
-        case EVENT_WARNING_OCCURED:
+        case EVENT_WARNING_OCCURRED:
             // color: yellow
             style = 3;
             break;
-        case EVENT_MESSAGE_OCCURED:
+        case EVENT_MESSAGE_OCCURRED:
             // color: green
             style = 1;
             break;
@@ -170,7 +193,7 @@ GUIMessageWindow::appendMsg(GUIEventType eType, const std::string& msg) {
         FXint pos = text.find("'");
         while (pos >= 0) {
             const GUIGlObject* const glObj = getActiveStringObject(text, pos + 1, 0, text.length());
-            if (glObj != 0) {
+            if (glObj != nullptr) {
                 GUIGlObjectStorage::gIDStorage.unblockObject(glObj->getGlID());
                 FXString insText = text.left(pos + 1);
                 FXText::appendStyledText(insText, style + 1);
@@ -222,13 +245,17 @@ GUIMessageWindow::clear() {
 
 void
 GUIMessageWindow::registerMsgHandlers() {
-    if (myMessageRetriever == 0) {
+    if (myMessageRetriever == nullptr) {
         // initialize only if registration is requested
-        myMessageRetriever = new MsgOutputDevice(this, EVENT_MESSAGE_OCCURED);
-        myErrorRetriever = new MsgOutputDevice(this, EVENT_ERROR_OCCURED);
-        myWarningRetriever = new MsgOutputDevice(this, EVENT_WARNING_OCCURED);
+        myMessageRetriever = new MsgOutputDevice(this, EVENT_MESSAGE_OCCURRED);
+        myErrorRetriever = new MsgOutputDevice(this, EVENT_ERROR_OCCURRED);
+        myDebugRetriever = new MsgOutputDevice(this, EVENT_DEBUG_OCCURRED);
+        myGLDebugRetriever = new MsgOutputDevice(this, EVENT_GLDEBUG_OCCURRED);
+        myWarningRetriever = new MsgOutputDevice(this, EVENT_WARNING_OCCURRED);
     }
     MsgHandler::getMessageInstance()->addRetriever(myMessageRetriever);
+    MsgHandler::getDebugInstance()->addRetriever(myDebugRetriever);
+    MsgHandler::getGLDebugInstance()->addRetriever(myGLDebugRetriever);
     MsgHandler::getErrorInstance()->addRetriever(myErrorRetriever);
     MsgHandler::getWarningInstance()->addRetriever(myWarningRetriever);
 }
@@ -237,10 +264,11 @@ GUIMessageWindow::registerMsgHandlers() {
 void
 GUIMessageWindow::unregisterMsgHandlers() {
     MsgHandler::getMessageInstance()->removeRetriever(myMessageRetriever);
+    MsgHandler::getDebugInstance()->removeRetriever(myDebugRetriever);
+    MsgHandler::getGLDebugInstance()->removeRetriever(myGLDebugRetriever);
     MsgHandler::getErrorInstance()->removeRetriever(myErrorRetriever);
     MsgHandler::getWarningInstance()->removeRetriever(myWarningRetriever);
 }
 
 
 /****************************************************************************/
-
