@@ -1,38 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIParameterTableItem.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date
-/// @version $Id: GUIParameterTableItem.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // A single line in a parameter window
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIParameterTableItem_h
-#define GUIParameterTableItem_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
 #include <utils/common/ValueSource.h>
 #include <utils/common/ToString.h>
 #include <utils/gui/div/GUIParam_PopupMenu.h>
@@ -70,32 +60,18 @@ public:
     /// @name Methods to be implemented by derived classes
     /// @{
 
-    /** @brief Returns the information whether the value changes over simulation time
-     *
-     * @return Whether the value changes over simulation time
-     */
+    /// @brief Returns the information whether the value changes over simulation time
     virtual bool dynamic() const = 0;
 
-
-    /** @brief Forces an update of the value
-     */
+    /// @brief Forces an update of the value
     virtual void update() = 0;
 
-
-    /** @brief Returns a double-typed copy of the value-source
-     *
-     * @return A double-typed copy of the value-source
-     */
+    /// @brief Returns a double-typed copy of the value-source
     virtual ValueSource<double>* getdoubleSourceCopy() const = 0;
 
-
-    /** @brief Returns the name of the value
-     *
-     * @return The name of the value
-     */
+    /// @brief Returns the name of the value
     virtual const std::string& getName() const = 0;
     /// @}
-
 };
 
 
@@ -129,15 +105,13 @@ public:
      * @todo Consider using a reference to the table
      * @todo Check whether the name should be stored in GUIParameterTableItemInterface
      */
-    GUIParameterTableItem(FXTable* table, unsigned pos,
-                          const std::string& name, bool dynamic,
-                          ValueSource<T>* src)
-        : myAmDynamic(dynamic), myName(name), myTablePosition((FXint) pos), mySource(src),
-          myValue(src->getValue()), myTable(table) {
+    GUIParameterTableItem(FXTable* table, unsigned pos, const std::string& name,
+                          bool dynamic, ValueSource<T>* src) :
+        myAmDynamic(dynamic), myName(name), myTablePosition((FXint) pos), mySource(src),
+        myValue(src->getValue()), myTable(table) {
         init(dynamic, toString<T>(src->getValue()));
     }
 
-
     /** @brief Constructor for non-changing (static) values
      *
      * @param[in] table The table this item belongs to
@@ -149,41 +123,18 @@ public:
      * @todo Check whether the name should be stored in GUIParameterTableItemInterface
      * @todo Should never be dynamic!?
      */
-    GUIParameterTableItem(FXTable* table, unsigned pos,
-                          const std::string& name, bool dynamic,
-                          T value)
-        : myAmDynamic(dynamic), myName(name), myTablePosition((FXint) pos), mySource(0),
-          myValue(value), myTable(table) {
+    GUIParameterTableItem(FXTable* table, unsigned pos, const std::string& name,
+                          bool dynamic, T value) :
+        myAmDynamic(dynamic), myName(name), myTablePosition((FXint) pos), mySource(0),
+        myValue(value), myTable(table) {
         init(dynamic, toString<T>(value));
     }
 
-
-    /** @brief Constructor for string-typed, non-changing (static) values
-     *
-     * @param[in] table The table this item belongs to
-     * @param[in] pos The row of the table this item fills
-     * @param[in] name The name of the represented value
-     * @param[in] dynamic Information whether this value changes over time
-     * @param[in] value The value
-     * @todo Consider using a reference to the table
-     * @todo Check whether the name should be stored in GUIParameterTableItemInterface
-     * @todo Should never be dynamic!?
-     */
-    GUIParameterTableItem(FXTable* table, unsigned pos,
-                          const std::string& name, bool dynamic,
-                          std::string value)
-        : myAmDynamic(dynamic), myName(name), myTablePosition((FXint) pos), mySource(0),
-          myValue(0), myTable(table) {
-        init(dynamic, value);
-    }
-
-
     /// @brief Destructor
     ~GUIParameterTableItem() {
         delete mySource;
     }
 
-
     /** @brief Initialises the line
      *
      * Fills the line using the name, the current value, and the information
@@ -196,33 +147,27 @@ public:
         myTable->setItemText(myTablePosition, 0, myName.c_str());
         myTable->setItemText(myTablePosition, 1, value.c_str());
         if (dynamic) {
-            myTable->setItemIcon(myTablePosition, 2, GUIIconSubSys::getIcon(ICON_YES));
+            if (getdoubleSourceCopy() == nullptr) {
+                myTable->setItemIcon(myTablePosition, 2, GUIIconSubSys::getIcon(GUIIcon::YES));
+            } else {
+                myTable->setItemIcon(myTablePosition, 2, GUIIconSubSys::getIcon(GUIIcon::TRACKER));
+            }
         } else {
-            myTable->setItemIcon(myTablePosition, 2, GUIIconSubSys::getIcon(ICON_NO));
+            myTable->setItemIcon(myTablePosition, 2, GUIIconSubSys::getIcon(GUIIcon::NO));
         }
         myTable->setItemJustify(myTablePosition, 2, FXTableItem::CENTER_X | FXTableItem::CENTER_Y);
     }
 
-
-
-    /** @brief Returns the information whether this item may change
-     *
-     * @return Whether this item changes over time
-     */
+    /// @brief Returns the information whether this item may change over time
     bool dynamic() const {
         return myAmDynamic;
     }
 
-
-    /** @brief Returns the name of this value
-     *
-     * @return The name of this value
-     */
+    /// @brief Returns the name of this value
     const std::string& getName() const {
         return myName;
     }
 
-
     /** @brief Resets the value if it's dynamic
      *
      * If the value is dynamic, the current value is retrieved from the value
@@ -241,11 +186,7 @@ public:
         }
     }
 
-
-    /** @brief Returns a copy of the source if the value is dynamic
-     *
-     * @return A copy of the value source
-     */
+    /// @brief Returns a copy of the source if the value is dynamic
     ValueSource<T>* getSourceCopy() const {
         if (mySource == 0) {
             return 0;
@@ -253,11 +194,7 @@ public:
         return mySource->copy();
     }
 
-
-    /** @brief Returns a double-typed copy of the source if the value is dynamic
-     *
-     * @return A double-typed copy of the value source
-     */
+    /// @brief Returns a double-typed copy of the source if the value is dynamic
     ValueSource<double>* getdoubleSourceCopy() const {
         if (mySource == 0) {
             return 0;
@@ -265,7 +202,6 @@ public:
         return mySource->makedoubleReturningCopy();
     }
 
-
 private:
     /// @brief Information whether the value may change
     bool myAmDynamic;
@@ -276,7 +212,7 @@ private:
     /// @brief The position within the table
     FXint myTablePosition;
 
-    /** @brief The source to gain new values from; this source is==0 if the values are not dynamic */
+    /// @brief The source to gain new values from; this source is==0 if the values are not dynamic
     ValueSource<T>* mySource;
 
     /// @brief A backup of the value to avoid the redrawing when nothing has changed
@@ -284,11 +220,4 @@ private:
 
     /// @brief The table this entry belongs to
     FXTable* myTable;
-
 };
-
-
-#endif
-
-/****************************************************************************/
-
