@@ -1,45 +1,35 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIParameterTableWindow.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: GUIParameterTableWindow.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // The window that holds the table of an object's parameter
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIParameterTableWindow_h
-#define GUIParameterTableWindow_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
 #include <algorithm>
 #include <functional>
-#include <fx.h>
-#include <utils/foxtools/MFXMutex.h>
+#include <utils/foxtools/fxheader.h>
 #include <utils/common/ValueSource.h>
 #include <utils/common/SUMOTime.h>
+#include "GUIParameterTableItem.h"
 
 
 // ===========================================================================
@@ -48,6 +38,7 @@
 class GUIGlObject;
 class GUIMainWindow;
 class GUIParameterTableItemInterface;
+class Parameterised;
 
 
 // ===========================================================================
@@ -73,10 +64,8 @@ public:
      *
      * @param[in] app The application this window belongs to
      * @param[in] o The gl-object this table describes
-     * @param[in] noRows Number of rows to allocate
      */
-    GUIParameterTableWindow(GUIMainWindow& app,
-                            GUIGlObject& o, int noRows);
+    GUIParameterTableWindow(GUIMainWindow& app, GUIGlObject& o);
 
 
     /// @brief Destructor
@@ -91,7 +80,7 @@ public:
      *
      * @see GUIMainWindow::addChild
      */
-    void closeBuilding();
+    void closeBuilding(const Parameterised* p = 0);
 
 
 
@@ -105,31 +94,18 @@ public:
     /// @name Row adding functions
     /// @{
 
-    /** @brief Adds a row which obtains its value from an unsigned-ValueSource
+    /** @brief Adds a row which obtains its value from a ValueSource
      *
      * @param[in] name The name of the row entry
      * @param[in] dynamic Information whether the entry is dynamic
      * @param[in] src The value source to use
      */
-    void mkItem(const char* name, bool dynamic, ValueSource<unsigned>* src);
-
-    /** @brief Adds a row which obtains its value from an integer-ValueSource
-     *
-     * @param[in] name The name of the row entry
-     * @param[in] dynamic Information whether the entry is dynamic
-     * @param[in] src The value source to use
-     */
-    void mkItem(const char* name, bool dynamic, ValueSource<int>* src);
-
-
-    /** @brief Adds a row which obtains its value from an double-ValueSource
-     *
-     * @param[in] name The name of the row entry
-     * @param[in] dynamic Information whether the entry is dynamic
-     * @param[in] src The value source to use
-     */
-    void mkItem(const char* name, bool dynamic, ValueSource<double>* src);
-
+    template<class T>
+    void mkItem(const char* name, bool dynamic, ValueSource<T>* src) {
+        myTable->insertRows((int)myItems.size() + 1);
+        GUIParameterTableItemInterface* i = new GUIParameterTableItem<T>(myTable, myCurrentPos++, name, dynamic, src);
+        myItems.push_back(i);
+    }
 
     /** @brief Adds a row which shows a string-value
      *
@@ -201,6 +177,7 @@ public:
      */
     long onTableDeselected(FXObject*, FXSelector, void*);
 
+
     /** @brief Shows a popup
      *
      * Callback for right-mouse-button pressing event. Obtains the selected row
@@ -211,13 +188,18 @@ public:
      * @see GUIParam_PopupMenuInterface
      */
     long onRightButtonPress(FXObject*, FXSelector, void*);
+
+    /// @brief directly opens tracker when clicking on last column
+    long onLeftBtnPress(FXObject*, FXSelector, void*);
     /// @}
 
     /** @brief Updates all instances
      */
     static void updateAll() {
-        AbstractMutex::ScopedLocker locker(myGlobalContainerLock);
-        std::for_each(myContainer.begin(), myContainer.end(), std::mem_fun(&GUIParameterTableWindow::updateTable));
+        FXMutexLock locker(myGlobalContainerLock);
+        for (GUIParameterTableWindow* const window : myContainer) {
+            window->updateTable();
+        }
     }
 
 protected:
@@ -230,7 +212,7 @@ protected:
     void updateTable();
 
     /// @brief The mutex used to avoid concurrent updates of the instance container
-    static MFXMutex myGlobalContainerLock;
+    static FXMutex myGlobalContainerLock;
 
     /// @brief The container of items that shall be updated
     static std::vector<GUIParameterTableWindow*> myContainer;
@@ -248,20 +230,19 @@ private:
     /// @brief The list of table rows
     std::vector<GUIParameterTableItemInterface*> myItems;
 
+    /// @brief y-position for opening new tracker window
+    int myTrackerY;
+
     /// @brief The index of the next row to add - used while building
     unsigned myCurrentPos;
 
     /// @brief A lock assuring save updates in case of object deletion
-    mutable MFXMutex myLock;
+    mutable FXMutex myLock;
+
+    /// @brief returns the number of parameters if obj is Parameterised and 0 otherwise
+    static int numParams(const GUIGlObject* obj);
 
 protected:
-    /// FOX needs this
-    GUIParameterTableWindow() { }
+    FOX_CONSTRUCTOR(GUIParameterTableWindow)
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
