@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUISelectedStorage.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Jun 2004
-/// @version $Id: GUISelectedStorage.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Storage for "selected" objects
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <algorithm>
 #include <utils/gui/globjects/GUIGlObject.h>
@@ -38,17 +29,15 @@
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/common/ToString.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // member method definitions
 // ===========================================================================
+
 /* -------------------------------------------------------------------------
  * for GUISelectedStorage::SingleTypeSelections
  * ----------------------------------------------------------------------- */
+
 GUISelectedStorage::SingleTypeSelections::SingleTypeSelections() {}
 
 
@@ -90,11 +79,10 @@ GUISelectedStorage::SingleTypeSelections::getSelected() const {
     return mySelected;
 }
 
-
-
 /* -------------------------------------------------------------------------
  * for GUISelectedStorage
  * ----------------------------------------------------------------------- */
+
 GUISelectedStorage::GUISelectedStorage() {}
 
 
@@ -106,13 +94,19 @@ GUISelectedStorage::isSelected(GUIGlObjectType type, GUIGlID id) {
     switch (type) {
         case GLO_NETWORK:
             return false;
-        case GLO_ADDITIONAL:
-            return isSelected(GLO_TRIGGER, id) || isSelected(GLO_DETECTOR, id);
         default:
             return mySelections[type].isSelected(id);
     }
 }
 
+bool
+GUISelectedStorage::isSelected(const GUIGlObject* o) {
+    if (o == nullptr) {
+        return false;
+    } else {
+        return isSelected(o->getType(), o->getGlID());
+    }
+}
 
 void
 GUISelectedStorage::select(GUIGlID id, bool update) {
@@ -268,7 +262,7 @@ GUISelectedStorage::add2Update(UpdateTarget* updateTarget) {
 
 void
 GUISelectedStorage::remove2Update() {
-    myUpdateTarget = 0;
+    myUpdateTarget = nullptr;
 }
 
 
@@ -277,7 +271,7 @@ GUISelectedStorage::save(const std::string& filename, const std::set<GUIGlID>& i
     OutputDevice& dev = OutputDevice::getDevice(filename);
     for (std::set<GUIGlID>::const_iterator i = ids.begin(); i != ids.end(); ++i) {
         GUIGlObject* object = GUIGlObjectStorage::gIDStorage.getObjectBlocking(*i);
-        if (object != 0) {
+        if (object != nullptr) {
             std::string name = object->getFullName();
             dev << name << "\n";
             GUIGlObjectStorage::gIDStorage.unblockObject(*i);
@@ -285,4 +279,6 @@ GUISelectedStorage::save(const std::string& filename, const std::set<GUIGlID>& i
     }
     dev.close();
 }
+
+
 /****************************************************************************/
