@@ -1,42 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUISelectedStorage.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Jun 2004
-/// @version $Id: GUISelectedStorage.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Storage for "selected" objects
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUISelectedStorage_h
-#define GUISelectedStorage_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <set>
 #include <string>
 #include <map>
 #include <fstream>
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 
@@ -77,9 +67,14 @@ class OutputDevice;
 class GUISelectedStorage {
 
 public:
+    /// @class update target
     class UpdateTarget {
+
     public:
+        /// @brief virtual destructor
         virtual ~UpdateTarget() {};
+
+        /// @brief called when selection is updated
         virtual void selectionUpdated() = 0;
     };
 
@@ -87,11 +82,9 @@ public:
     /// @brief Constructor
     GUISelectedStorage();
 
-
     /// @brief Destructor
     ~GUISelectedStorage();
 
-
     /** @brief Returns the information whether the object with the given type and id is selected
      *
      * If the type is ==-1, it is determined, first. If it could not be obtained,
@@ -110,6 +103,7 @@ public:
      */
     bool isSelected(GUIGlObjectType type, GUIGlID id);
 
+    bool isSelected(const GUIGlObject* o);
 
     /** @brief Adds the object with the given id
      *
@@ -128,7 +122,6 @@ public:
      */
     void select(GUIGlID id, bool update = true);
 
-
     /** @brief Deselects the object with the given id
      *
      * The id of the object is removed from the sub-container that is
@@ -146,7 +139,6 @@ public:
      */
     void deselect(GUIGlID id);
 
-
     /** @brief Toggles selection of an object
      *
      * If the object can not be obtained a ProcessError is thrown.
@@ -163,15 +155,9 @@ public:
      */
     void toggleSelection(GUIGlID id);
 
-
-    /** @brief Returns the list of ids of all selected objects
-     *
-     *
-     * @return The set of all selected objects (ids)
-     */
+    /// @brief Returns the set of ids of all selected objects
     const std::set<GUIGlID>& getSelected() const;
 
-
     /**  @brief Returns the set of ids of all selected objects' of a certain type
      *
      * @param[in] type The type of the object
@@ -180,7 +166,6 @@ public:
      */
     const std::set<GUIGlID>& getSelected(GUIGlObjectType type);
 
-
     /** @brief Clears the list of selected objects
      *
      * Clears the global container and all sub-containers via SingleTypeSelections::clear.
@@ -189,16 +174,14 @@ public:
      */
     void clear();
 
-
     /** @brief Loads a selection list (optionally with restricted type)
      *
      * @param[in] filename The name of the file to load the list of selected objects from
      * @param[in] type The type of the objects to load if changed from default
-     * @return error messages if errors occured or the empty string
+     * @return error messages if errors occurred or the empty string
      */
     std::string load(const std::string& filename, GUIGlObjectType type = GLO_MAX);
 
-
     /** @brief Loads a selection list (optionally with restricted type) and
      * returns the ids of all active objects
      *
@@ -210,7 +193,6 @@ public:
      */
     std::set<GUIGlID> loadIDs(const std::string& filename, std::string& msgOut, GUIGlObjectType type = GLO_MAX, int maxErrors = 16);
 
-
     /** @brief Saves a selection list
      *
      * @param[in] type The type of the objects to save
@@ -224,23 +206,20 @@ public:
      */
     void save(const std::string& filename) const;
 
-
     /** @brief Adds a dialog to be updated
      * @param[in] updateTarget the callback for selection changes
      */
     void add2Update(UpdateTarget* updateTarget);
 
-
-    /** @brief Removes the dialog to be updated
-     */
+    /// @brief @brief Removes the dialog to be updated
     void remove2Update();
 
-
     /**
      * @class SingleTypeSelections
      * @brief A container for ids of selected objects of a certain type.
      */
     class SingleTypeSelections {
+
     public:
         /// @brief Constructor
         SingleTypeSelections();
@@ -248,36 +227,30 @@ public:
         /// @brief Destructor
         ~SingleTypeSelections();
 
-
         /** @brief Returns the information whether the object with the given id is qithin the selection
          * @param[in] id The id of the object
          * @return Whether the object is selected
          */
         bool isSelected(GUIGlID id);
 
-
         /** @brief Adds the object with the given id to the list of selected objects
          * @param[in] id The id of the object
          */
         void select(GUIGlID id);
 
-
         /** @brief Deselects the object with the given id from the list of selected objects
          * @param[in] id The id of the object
          */
         void deselect(GUIGlID id);
 
-
         /// @brief Clears the list of selected objects
         void clear();
 
-
         /** @brief Saves the list of selected objects to a file named as given
          * @param[in] filename The name of the file to save the list into
          */
         void save(const std::string& filename);
 
-
         /** @brief Returns the list of selected ids
          * @return A list containing the ids of all selected objects
          */
@@ -288,10 +261,13 @@ public:
         std::set<GUIGlID> mySelected;
 
     };
+
+    /// @brief set SingleTypeSelections as friend class
     friend class SingleTypeSelections;
 
 
 private:
+    /// @brief map with the selections
     std::map<GUIGlObjectType, SingleTypeSelections> mySelections;
 
     /// @brief List of selected objects
@@ -300,18 +276,9 @@ private:
     /// @brief The dialog to be updated
     UpdateTarget* myUpdateTarget;
 
-    /* @brief load items into the given set, optionally restricting to type
-     */
+    /// @brief load items into the given set, optionally restricting to type
     std::string load(GUIGlObjectType type, const std::string& filename, bool restrictType, std::set<GUIGlID>& into);
 
-
     /// @brief saves items from the given set
     static void save(const std::string& filename, const std::set<GUIGlID>& ids);
-
 };
-
-
-#endif
-
-/****************************************************************************/
-
