@@ -1,33 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIUserIO.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    2006-10-12
-/// @version $Id: GUIUserIO.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Some OS-dependant functions to ease cliboard manipulation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "GUIUserIO.h"
 
@@ -49,5 +40,14 @@ GUIUserIO::copyToClipboard(const FXApp& app, const std::string& text) {
     }
 }
 
-/****************************************************************************/
+std::string
+GUIUserIO::copyFromClipboard(const FXApp& app) {
+    FXString string;
+    if (app.getActiveWindow()->getDNDData(FROM_CLIPBOARD, FXWindow::utf8Type, string)) {
+        return string.text();
+    };
+    return string.text();
+}
 
+
+/****************************************************************************/
