@@ -1,38 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIUserIO.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Sept 2006
-/// @version $Id: GUIUserIO.h 22608 2017-01-17 06:28:54Z behrisch $
 ///
 // Some OS-dependant functions to ease cliboard manipulation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIUserIO_h
-#define GUIUserIO_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
 
 
 // ===========================================================================
@@ -53,20 +43,18 @@
 class GUIUserIO {
 public:
     /** @brief Copies the given text to clipboard
-     *
-     * This method is only implemented for MS Windows.
      *
      * @param[in] app The application to use
      * @param[in] text The text to copy
      */
     static void copyToClipboard(const FXApp& app, const std::string& text);
 
+    /** @brief Copies text from the clipboard
+     *
+     * @param[in] app The application to use
+     */
+    static std::string copyFromClipboard(const FXApp& app);
+
     static std::string clipped;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
