@@ -1,34 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIEvent_Message.h
 /// @author  Daniel Krajzewicz
 /// @date    Wed 18 Jun 2003
-/// @version $Id: GUIEvent_Message.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Event send when a message (message, warning, error) has to besubmitted
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIEvent_Message_h
-#define GUIEvent_Message_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include "GUIEvent.h"
 #include <string>
@@ -45,18 +35,29 @@
  */
 class GUIEvent_Message : public GUIEvent {
 public:
+    /// constructor
+    GUIEvent_Message(const std::string& msg)
+        : GUIEvent(EVENT_STATUS_OCCURRED), myMsg(msg) {
+    }
+
     /// constructor
     GUIEvent_Message(MsgHandler::MsgType type, const std::string& msg)
-        : GUIEvent(EVENT_MESSAGE_OCCURED), myMsg(msg) {
+        : GUIEvent(EVENT_MESSAGE_OCCURRED), myMsg(msg) {
         switch (type) {
-            case MsgHandler::MT_MESSAGE:
-                myType = EVENT_MESSAGE_OCCURED;
+            case MsgHandler::MsgType::MT_MESSAGE:
+                myType = EVENT_MESSAGE_OCCURRED;
                 break;
-            case MsgHandler::MT_WARNING:
-                myType = EVENT_WARNING_OCCURED;
+            case MsgHandler::MsgType::MT_WARNING:
+                myType = EVENT_WARNING_OCCURRED;
                 break;
-            case MsgHandler::MT_ERROR:
-                myType = EVENT_ERROR_OCCURED;
+            case MsgHandler::MsgType::MT_ERROR:
+                myType = EVENT_ERROR_OCCURRED;
+                break;
+            case MsgHandler::MsgType::MT_DEBUG:
+                myType = EVENT_DEBUG_OCCURRED;
+                break;
+            case MsgHandler::MsgType::MT_GLDEBUG:
+                myType = EVENT_GLDEBUG_OCCURRED;
                 break;
             default:
                 throw 1;
@@ -77,9 +78,3 @@ protected:
     std::string myMsg;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
