@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIPointOfInterest.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    June 2006
-/// @version $Id: GUIPointOfInterest.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // missing_desc
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIPointOfInterest_h
-#define GUIPointOfInterest_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/shapes/PointOfInterest.h>
@@ -56,17 +46,23 @@ public:
      * @param[in] id The name of the POI
      * @param[in] type The (abstract) type of the POI
      * @param[in] color The color of the POI
+     * @param[in] pos The position of the POI
+     * @param[in[ geo use GEO coordinates (lon/lat)
+     * @param[in] lane The Lane in which this POI is placed
+     * @param[in] posOverLane The position over Lane
+     * @param[in] posLat The position lateral over Lane
      * @param[in] layer The layer of the POI
      * @param[in] angle The rotation of the POI
      * @param[in] imgFile The raster image of the shape
-     * @param[in] pos The position of the POI
+     * @param[in] relativePath set image file as relative path
      * @param[in] width The width of the POI image
      * @param[in] height The height of the POI image
      */
     GUIPointOfInterest(const std::string& id, const std::string& type,
-                       const RGBColor& color, const Position& pos,
+                       const RGBColor& color, const Position& pos, bool geo,
+                       const std::string& lane, double posOverLane, double posLat,
                        double layer, double angle, const std::string& imgFile,
-                       double width, double height);
+                       bool relativePath, double width, double height);
 
     /// @brief Destructor
     virtual ~GUIPointOfInterest();
@@ -113,10 +109,12 @@ public:
     void drawGL(const GUIVisualizationSettings& s) const;
     //@}
 
+    /// @brief check if POI can be drawn
+    static bool checkDraw(const GUIVisualizationSettings& s, const GUIGlObject* o);
+
+    /// @brief set color
+    static void setColor(const GUIVisualizationSettings& s, const PointOfInterest* POI, const GUIGlObject* o, bool forceSelectionColor);
+
+    /// @brief draw inner POI (before pushName() )
+    static void drawInnerPOI(const GUIVisualizationSettings& s, const PointOfInterest* POI, const GUIGlObject* o, const bool disableSelectionColor, const double layer);
 };
-
-
-#endif
-
-/****************************************************************************/
-
