@@ -1,105 +1,51 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIPolygon.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    June 2006
-/// @version $Id: GUIPolygon.cpp 21182 2016-07-18 06:46:01Z behrisch $
 ///
 // The GUI-version of a polygon
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
-#include "GUIPolygon.h"
+#include <utils/geom/GeomHelper.h>
 #include <utils/gui/images/GUITexturesHelper.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <utils/gui/div/GUIParameterTableWindow.h>
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
 #include <utils/gui/settings/GUIVisualizationSettings.h>
+#include <utils/gui/div/GUIGlobalSelection.h>
 #include <utils/gui/div/GLHelper.h>
-#include <foreign/polyfonts/polyfonts.h>
+#include <utils/gui/div/GUIDesigns.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
+#include "GUIPolygon.h"
 
 //#define GUIPolygon_DEBUG_DRAW_VERTICES
 
 // ===========================================================================
-// method definitions
+// callbacks definitions
 // ===========================================================================
-GUIPolygon::GUIPolygon(const std::string& id, const std::string& type,
-                       const RGBColor& color, const PositionVector& shape, bool fill,
-                       SUMOReal layer, SUMOReal angle, const std::string& imgFile):
-    Polygon(id, type, color, shape, fill, layer, angle, imgFile),
-    GUIGlObject_AbstractAdd("poly", GLO_POLYGON, id),
-    myDisplayList(0),
-    myLineWidth(1) // m
-
-{}
-
-
-GUIPolygon::~GUIPolygon() {}
-
-
-
-GUIGLObjectPopupMenu*
-GUIPolygon::getPopUpMenu(GUIMainWindow& app,
-                         GUISUMOAbstractView& parent) {
-    GUIGLObjectPopupMenu* ret = new GUIGLObjectPopupMenu(app, parent, *this);
-    buildPopupHeader(ret, app, false);
-    FXString t(myType.c_str());
-    new FXMenuCommand(ret, "(" + t + ")", 0, 0, 0);
-    new FXMenuSeparator(ret);
-    buildCenterPopupEntry(ret);
-    buildNameCopyPopupEntry(ret);
-    buildSelectionPopupEntry(ret);
-    buildPositionCopyEntry(ret, false);
-    return ret;
-}
-
-
-GUIParameterTableWindow*
-GUIPolygon::getParameterWindow(GUIMainWindow&,
-                               GUISUMOAbstractView&) {
-    return 0;
-}
-
-
-Boundary
-GUIPolygon::getCenteringBoundary() const {
-    Boundary b;
-    b.add(myShape.getBoxBoundary());
-    b.grow(2);
-    return b;
-}
-
 
 void APIENTRY beginCallback(GLenum which) {
     glBegin(which);
 }
 
+
 void APIENTRY errorCallback(GLenum errorCode) {
     const GLubyte* estring;
 
@@ -108,14 +54,17 @@ void APIENTRY errorCallback(GLenum errorCode) {
     exit(0);
 }
 
+
 void APIENTRY endCallback(void) {
     glEnd();
 }
 
+
 void APIENTRY vertexCallback(GLvoid* vertex) {
     glVertex3dv((GLdouble*) vertex);
 }
 
+
 void APIENTRY combineCallback(GLdouble coords[3],
                               GLdouble* vertex_data[4],
                               GLfloat weight[4], GLdouble** dataOut) {
@@ -131,42 +80,210 @@ void APIENTRY combineCallback(GLdouble coords[3],
     *dataOut = vertex;
 }
 
+static const GLfloat INV_POLY_TEX_DIM = 1.0 / 256.0;
+static const GLfloat xPlane[] = {INV_POLY_TEX_DIM, 0.0, 0.0, 0.0};
+static const GLfloat yPlane[] = {0.0, INV_POLY_TEX_DIM, 0.0, 0.0};
+
+// ===========================================================================
+// method definitions
+// ===========================================================================
+
+GUIPolygon::GUIPolygon(const std::string& id, const std::string& type,
+                       const RGBColor& color, const PositionVector& shape, bool geo,
+                       bool fill, double lineWidth, double layer, double angle, const std::string& imgFile,
+                       bool relativePath):
+    SUMOPolygon(id, type, color, shape, geo, fill, lineWidth, layer, angle, imgFile, relativePath),
+    GUIGlObject_AbstractAdd(GLO_POLYGON, id),
+    myDisplayList(0),
+    myRotatedShape(nullptr) {
+    if (angle != 0.) {
+        setShape(shape);
+    }
+}
+
+
+GUIPolygon::~GUIPolygon() {
+    delete myRotatedShape;
+}
+
+
+GUIGLObjectPopupMenu*
+GUIPolygon::getPopUpMenu(GUIMainWindow& app,
+                         GUISUMOAbstractView& parent) {
+    GUIGLObjectPopupMenu* ret = new GUIGLObjectPopupMenu(app, parent, *this);
+    buildPopupHeader(ret, app, false);
+    GUIDesigns::buildFXMenuCommand(ret, "(" + getShapeType() + ")", nullptr, nullptr, 0);
+    new FXMenuSeparator(ret);
+    buildCenterPopupEntry(ret);
+    buildNameCopyPopupEntry(ret);
+    buildSelectionPopupEntry(ret);
+    buildShowParamsPopupEntry(ret, false);
+    buildPositionCopyEntry(ret, false);
+    return ret;
+}
+
+
+GUIParameterTableWindow*
+GUIPolygon::getParameterWindow(GUIMainWindow& app,
+                               GUISUMOAbstractView&) {
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(app, *this);
+    // add items
+    ret->mkItem("type", false, getShapeType());
+    ret->mkItem("layer", false, toString(getShapeLayer()));
+    ret->closeBuilding(this);
+    return ret;
+}
+
+
+Boundary
+GUIPolygon::getCenteringBoundary() const {
+    const PositionVector& shape = myRotatedShape != nullptr ? *myRotatedShape : myShape;
+    Boundary b;
+    b.add(shape.getBoxBoundary());
+    b.grow(2);
+    return b;
+}
 
-GLfloat INV_POLY_TEX_DIM = 1.0 / 256.0;
-GLfloat xPlane[] = {INV_POLY_TEX_DIM, 0.0, 0.0, 0.0};
-GLfloat yPlane[] = {0.0, INV_POLY_TEX_DIM, 0.0, 0.0};
 
 void
 GUIPolygon::drawGL(const GUIVisualizationSettings& s) const {
-    if (s.polySize.getExaggeration(s) == 0) {
-        return;
+    // first check if polygon can be drawn
+    if (checkDraw(s, this, this)) {
+        FXMutexLock locker(myLock);
+        //if (myDisplayList == 0 || (!getFill() && myLineWidth != s.polySize.getExaggeration(s))) {
+        //    storeTesselation(s.polySize.getExaggeration(s));
+        //}
+        // push name (needed for getGUIGlObjectsUnderCursor(...)
+        glPushName(getGlID());
+        // draw inner polygon
+        if (myRotatedShape) {
+            drawInnerPolygon(s, this, this, *myRotatedShape, getShapeLayer(), false);
+        } else {
+            drawInnerPolygon(s, this, this, myShape, getShapeLayer(), false);
+        }
+        // pop name
+        glPopName();
     }
-    Boundary boundary = myShape.getBoxBoundary();
+}
+
+
+void
+GUIPolygon::setShape(const PositionVector& shape) {
+    FXMutexLock locker(myLock);
+    SUMOPolygon::setShape(shape);
+    if (getShapeNaviDegree() != 0.) {
+        if (myRotatedShape == nullptr) {
+            myRotatedShape = new PositionVector();
+        }
+        const Position& centroid = myShape.getCentroid();
+        *myRotatedShape = myShape;
+        myRotatedShape->sub(centroid);
+        myRotatedShape->rotate2D(-DEG2RAD(getShapeNaviDegree()));
+        myRotatedShape->add(centroid);
+    } else {
+        delete myRotatedShape;
+        myRotatedShape = nullptr;
+    }
+    //storeTesselation(myLineWidth);
+}
+
+
+void
+GUIPolygon::performTesselation(const bool fill, const PositionVector& shape, const double lineWidth) {
+    if (fill) {
+        // draw the tesselated shape
+        double* points = new double[shape.size() * 3];
+        GLUtesselator* tobj = gluNewTess();
+        gluTessCallback(tobj, GLU_TESS_VERTEX, (GLvoid(APIENTRY*)()) &glVertex3dv);
+        gluTessCallback(tobj, GLU_TESS_BEGIN, (GLvoid(APIENTRY*)()) &beginCallback);
+        gluTessCallback(tobj, GLU_TESS_END, (GLvoid(APIENTRY*)()) &endCallback);
+        //gluTessCallback(tobj, GLU_TESS_ERROR, (GLvoid (APIENTRY*) ()) &errorCallback);
+        gluTessCallback(tobj, GLU_TESS_COMBINE, (GLvoid(APIENTRY*)()) &combineCallback);
+        gluTessProperty(tobj, GLU_TESS_WINDING_RULE, GLU_TESS_WINDING_ODD);
+        gluTessBeginPolygon(tobj, nullptr);
+        gluTessBeginContour(tobj);
+        for (int i = 0; i != (int)shape.size(); ++i) {
+            points[3 * i]  = shape[(int) i].x();
+            points[3 * i + 1]  = shape[(int) i].y();
+            points[3 * i + 2]  = 0;
+            gluTessVertex(tobj, points + 3 * i, points + 3 * i);
+        }
+        gluTessEndContour(tobj);
+
+        gluTessEndPolygon(tobj);
+        gluDeleteTess(tobj);
+        delete[] points;
+
+    } else {
+        GLHelper::drawLine(shape);
+        GLHelper::drawBoxLines(shape, lineWidth);
+    }
+}
+
+
+void
+GUIPolygon::storeTesselation(const bool fill, const PositionVector& shape, double lineWidth) const {
+    if (myDisplayList > 0) {
+        glDeleteLists(myDisplayList, 1);
+    }
+    myDisplayList = glGenLists(1);
+    if (myDisplayList == 0) {
+        throw ProcessError("GUIPolygon::storeTesselation() could not create display list");
+    }
+    glNewList(myDisplayList, GL_COMPILE);
+    performTesselation(fill, shape, lineWidth);
+    glEndList();
+}
+
+
+void
+GUIPolygon::setColor(const GUIVisualizationSettings& s, const SUMOPolygon* polygon, const GUIGlObject* o, bool disableSelectionColor) {
+    const GUIColorer& c = s.polyColorer;
+    const int active = c.getActive();
+    if (s.netedit && active != 1 && gSelected.isSelected(GLO_POLYGON, o->getGlID()) && disableSelectionColor) {
+        // override with special selection colors (unless the color scheme is based on selection)
+        GLHelper::setColor(RGBColor(0, 0, 204));
+    } else if (active == 0) {
+        GLHelper::setColor(polygon->getShapeColor());
+    } else if (active == 1) {
+        GLHelper::setColor(c.getScheme().getColor(gSelected.isSelected(GLO_POLYGON, o->getGlID())));
+    } else {
+        GLHelper::setColor(c.getScheme().getColor(0));
+    }
+}
+
+
+bool
+GUIPolygon::checkDraw(const GUIVisualizationSettings& s, const SUMOPolygon* polygon, const GUIGlObject* o) {
+    if (s.polySize.getExaggeration(s, o) == 0) {
+        return false;
+    }
+    Boundary boundary = polygon->getShape().getBoxBoundary();
     if (s.scale * MAX2(boundary.getWidth(), boundary.getHeight()) < s.polySize.minSize) {
-        return;
+        return false;
     }
-    if (getFill()) {
-        if (myShape.size() < 3) {
-            return;
+    if (polygon->getFill()) {
+        if (polygon->getShape().size() < 3) {
+            return false;
         }
     } else {
-        if (myShape.size() < 2) {
-            return;
+        if (polygon->getShape().size() < 2) {
+            return false;
         }
     }
-    AbstractMutex::ScopedLocker locker(myLock);
-    //if (myDisplayList == 0 || (!getFill() && myLineWidth != s.polySize.getExaggeration(s))) {
-    //    storeTesselation(s.polySize.getExaggeration(s));
-    //}
-    glPushName(getGlID());
-    glPushMatrix();
-    glTranslated(0, 0, getLayer());
-    glRotated(-getNaviDegree(), 0, 0, 1);
-    GLHelper::setColor(getColor());
+    return true;
+}
 
+
+void
+GUIPolygon::drawInnerPolygon(const GUIVisualizationSettings& s, const SUMOPolygon* polygon, const GUIGlObject* o,
+                             const PositionVector shape, double layer, bool disableSelectionColor) {
+    glPushMatrix();
+    glTranslated(0, 0, layer);
+    setColor(s, polygon, o, disableSelectionColor);
     int textureID = -1;
-    if (getFill()) {
-        const std::string& file = getImgFile();
+    if (polygon->getFill()) {
+        const std::string& file = polygon->getShapeImgFile();
         if (file != "") {
             textureID = GUITexturesHelper::getTextureID(file, true);
         }
@@ -196,7 +313,7 @@ GUIPolygon::drawGL(const GUIVisualizationSettings& s) const {
     }
     // recall tesselation
     //glCallList(myDisplayList);
-    performTesselation(myLineWidth * s.polySize.getExaggeration(s));
+    performTesselation(polygon->getFill(), shape, polygon->getLineWidth() * s.polySize.getExaggeration(s, o));
     // de-init generation of texture coordinates
     if (textureID >= 0) {
         glEnable(GL_DEPTH_TEST);
@@ -206,75 +323,16 @@ GUIPolygon::drawGL(const GUIVisualizationSettings& s) const {
         glDisable(GL_TEXTURE_GEN_T);
     }
 #ifdef GUIPolygon_DEBUG_DRAW_VERTICES
-    GLHelper::debugVertices(myShape, 80 / s.scale);
+    GLHelper::debugVertices(shape, 80 / s.scale);
 #endif
     glPopMatrix();
-    const Position namePos = myShape.getPolygonCenter();
-    drawName(namePos, s.scale, s.polyName);
+    const Position& namePos = shape.getPolygonCenter();
+    o->drawName(namePos, s.scale, s.polyName, s.angle);
     if (s.polyType.show) {
-        GLHelper::drawText(myType, namePos + Position(0, -0.6 * s.polyType.size / s.scale),
-                           GLO_MAX, s.polyType.size / s.scale, s.polyType.color);
+        const Position p = namePos + Position(0, -0.6 * s.polyType.size / s.scale);
+        GLHelper::drawTextSettings(s.polyType, polygon->getShapeType(), p, s.scale, s.angle);
     }
-    glPopName();
-}
-
-
-void
-GUIPolygon::setShape(const PositionVector& shape) {
-    AbstractMutex::ScopedLocker locker(myLock);
-    Polygon::setShape(shape);
-    //storeTesselation(myLineWidth);
-}
-
-
-void
-GUIPolygon::performTesselation(SUMOReal lineWidth) const {
-    if (getFill()) {
-        // draw the tesselated shape
-        double* points = new double[myShape.size() * 3];
-        GLUtesselator* tobj = gluNewTess();
-        gluTessCallback(tobj, GLU_TESS_VERTEX, (GLvoid(APIENTRY*)()) &glVertex3dv);
-        gluTessCallback(tobj, GLU_TESS_BEGIN, (GLvoid(APIENTRY*)()) &beginCallback);
-        gluTessCallback(tobj, GLU_TESS_END, (GLvoid(APIENTRY*)()) &endCallback);
-        //gluTessCallback(tobj, GLU_TESS_ERROR, (GLvoid (APIENTRY*) ()) &errorCallback);
-        gluTessCallback(tobj, GLU_TESS_COMBINE, (GLvoid(APIENTRY*)()) &combineCallback);
-        gluTessProperty(tobj, GLU_TESS_WINDING_RULE, GLU_TESS_WINDING_ODD);
-        gluTessBeginPolygon(tobj, NULL);
-        gluTessBeginContour(tobj);
-        for (int i = 0; i != myShape.size(); ++i) {
-            points[3 * i]  = myShape[(int) i].x();
-            points[3 * i + 1]  = myShape[(int) i].y();
-            points[3 * i + 2]  = 0;
-            gluTessVertex(tobj, points + 3 * i, points + 3 * i);
-        }
-        gluTessEndContour(tobj);
-
-        gluTessEndPolygon(tobj);
-        gluDeleteTess(tobj);
-        delete[] points;
-
-    } else {
-        GLHelper::drawLine(myShape);
-        GLHelper::drawBoxLines(myShape, lineWidth);
-    }
-    //std::cout << "OpenGL says: '" << gluErrorString(glGetError()) << "'\n";
-}
-
-
-void
-GUIPolygon::storeTesselation(SUMOReal lineWidth) const {
-    if (myDisplayList > 0) {
-        glDeleteLists(myDisplayList, 1);
-    }
-    myDisplayList = glGenLists(1);
-    if (myDisplayList == 0) {
-        throw ProcessError("GUIPolygon::storeTesselation() could not create display list");
-    }
-    glNewList(myDisplayList, GL_COMPILE);
-    performTesselation(lineWidth);
-    glEndList();
 }
 
 
 /****************************************************************************/
-
