@@ -1,44 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIPolygon.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    June 2006
-/// @version $Id: GUIPolygon.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The GUI-version of a polygon
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIPolygon_h
-#define GUIPolygon_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
-#include <utils/foxtools/MFXMutex.h>
-#include <utils/shapes/Polygon.h>
+#include <utils/shapes/SUMOPolygon.h>
 #include <utils/gui/globjects/GUIGlObject_AbstractAdd.h>
 #include <utils/gui/globjects/GLIncludes.h>
 #include <utils/gui/settings/GUIVisualizationSettings.h>
-using namespace SUMO;
+
 
 // ===========================================================================
 // class definitions
@@ -47,7 +36,8 @@ using namespace SUMO;
  * @class GUIPolygon
  * @brief The GUI-version of a polygon
  */
-class GUIPolygon : public Polygon, public GUIGlObject_AbstractAdd {
+class GUIPolygon : public SUMOPolygon, public GUIGlObject_AbstractAdd {
+
 public:
     /** @brief Constructor
      * @param[in] id The name of the polygon
@@ -56,18 +46,20 @@ public:
      * @param[in] layer The layer of the polygon
      * @param[in] angle The rotation of the polygon
      * @param[in] imgFile The raster image of the polygon
+     * @param[in] relativePath set image file as relative path
      * @param[in] shape The shape of the polygon
+     * @param[in] geo specifiy if shape was loaded as GEO
      * @param[in] fill Whether the polygon shall be filled
+     * @param[in] lineWidth Line width when drawing unfilled polygon
      */
     GUIPolygon(const std::string& id, const std::string& type,
-               const RGBColor& color, const PositionVector& shape, bool fill,
-               SUMOReal layer = 0, SUMOReal angle = 0, const std::string& imgFile = "");
+               const RGBColor& color, const PositionVector& shape, bool geo, bool fill, double lineWidth,
+               double layer = 0, double angle = 0, const std::string& imgFile = "", bool relativePath = false);
 
     /// @brief Destructor
     ~GUIPolygon();
 
 
-
     /// @name inherited from GUIGlObject
     //@{
 
@@ -112,31 +104,39 @@ public:
     /// @brief set a new shape and update the tesselation
     virtual void setShape(const PositionVector& shape);
 
-    /// @brief set a new shape and update the tesselation
-    void setLineWidth(SUMOReal lineWidth) {
-        myLineWidth = lineWidth;
+    /** @brief Sets a new angle in navigational degrees
+     * @param[in] layer The new angle to use
+     */
+    virtual void setShapeNaviDegree(const double angle) {
+        SUMOPolygon::setShapeNaviDegree(angle);
+        if (angle != 0.) {
+            setShape(myShape);
+        }
     }
 
+    /// @brief set color
+    static void setColor(const GUIVisualizationSettings& s, const SUMOPolygon* polygon, const GUIGlObject* o, bool disableSelectionColor);
+
+    /// @brief check if Polygon can be drawn
+    static bool checkDraw(const GUIVisualizationSettings& s, const SUMOPolygon* polygon, const GUIGlObject* o);
+
+    /// @brief draw inner Polygon (before pushName() )
+    static void drawInnerPolygon(const GUIVisualizationSettings& s, const SUMOPolygon* polygon, const GUIGlObject* o, const PositionVector shape, double layer, bool disableSelectionColor);
+
 private:
     /// The mutex used to avoid concurrent updates of the shape
-    mutable MFXMutex myLock;
+    mutable FXMutex myLock;
 
     /// @brief id of the display list for the cached tesselation
     mutable GLuint myDisplayList;
 
-    /// @brief the previous line width for deciding whether the display list must be refreshed
-    mutable SUMOReal myLineWidth;
+    /// @brief shape rotated on the centroid, if rotation is needed, nullptr otherwise
+    PositionVector* myRotatedShape;
 
     /// @brief store the drawing commands in a display list
-    void storeTesselation(SUMOReal lineWidth) const;
+    void storeTesselation(const bool fill, const PositionVector& shape, double lineWidth) const;
 
     // @brief perform the tesselation / drawing
-    void performTesselation(SUMOReal lineWidth) const;
+    static void performTesselation(const bool fill, const PositionVector& shape, const double lineWidth);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
