@@ -1,36 +1,28 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIIconSubSys.h
 /// @author  Daniel Krajzewicz
 /// @date    Jul 2003
-/// @version $Id: GUIIconSubSys.h 22929 2017-02-13 14:38:39Z behrisch $
 ///
 // A class to manage icons of SUMO
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIIconSubSys_h
-#define GUIIconSubSys_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
+#include <map>
+
 #include "GUIIcons.h"
 
 
@@ -47,7 +39,7 @@ public:
     /**@brief returns a icon previously defined in the enum GUIIcon
      * @param[in] which GUIIcon enum
      */
-    static FXIcon* getIcon(GUIIcon which);
+    static FXIcon* getIcon(const GUIIcon which);
 
     /// @brief close GUIIconSubSys
     static void close();
@@ -63,11 +55,5 @@ private:
     static GUIIconSubSys* myInstance;
 
     /// @brief vector with the icons
-    FXIcon* myIcons[ICON_MAX];
+    std::map<GUIIcon, FXIcon*> myIcons;
 };
-
-
-#endif
-
-/****************************************************************************/
-
