@@ -1,49 +1,36 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUITexturesHelper.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 08.03.2004
-/// @version $Id: GUITexturesHelper.cpp 21092 2016-07-02 18:16:29Z behrisch $
 ///
 // Global storage for textures; manages and draws them
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
-#include <fx.h>
-#include <fx3d.h>
+#include <utils/foxtools/fxheader.h>
 #include <utils/foxtools/MFXImageHelper.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <utils/gui/globjects/GLIncludes.h>
 #include <utils/gui/windows/GUIMainWindow.h>
+#include <utils/options/OptionsCont.h>
 #include <utils/common/MsgHandler.h>
 #include "GUITexturesHelper.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // definition of static variables
@@ -81,15 +68,14 @@ GUITexturesHelper::add(FXImage* i) {
 
 
 void
-GUITexturesHelper::drawTexturedBox(unsigned int which, SUMOReal size) {
+GUITexturesHelper::drawTexturedBox(int which, double size) {
     drawTexturedBox(which, size, size, -size, -size);
 }
 
 
 void
-GUITexturesHelper::drawTexturedBox(unsigned int which,
-                                   SUMOReal sizeX1, SUMOReal sizeY1,
-                                   SUMOReal sizeX2, SUMOReal sizeY2) {
+GUITexturesHelper::drawTexturedBox(int which, double sizeX1, double sizeY1, double sizeX2, double sizeY2) {
+    // first check that textures are allowed
     if (!myAllowTextures) {
         return;
     }
@@ -125,12 +111,15 @@ int
 GUITexturesHelper::getTextureID(const std::string& filename, const bool mirrorX) {
     if (myTextures.count(filename) == 0) {
         try {
+            // load image
             FXImage* i = MFXImageHelper::loadImage(GUIMainWindow::getInstance()->getApp(), filename);
             if (mirrorX) {
                 i->mirror(false, true);
             }
             MFXImageHelper::scalePower2(i, getMaxTextureSize());
+            // Create GL structure using texture
             GUIGlID id = add(i);
+            // delete texture after creating GL Structure
             delete i;
             myTextures[filename] = (int)id;
         } catch (InvalidArgument& e) {
@@ -147,4 +136,5 @@ GUITexturesHelper::clearTextures() {
     myTextures.clear();
 }
 
+
 /****************************************************************************/
