@@ -1,35 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUICompleteSchemeStorage.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    2006-01-09
-/// @version $Id: GUICompleteSchemeStorage.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Storage for available visualization settings
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "GUICompleteSchemeStorage.h"
 #include <utils/common/ToString.h>
@@ -116,23 +107,25 @@ GUICompleteSchemeStorage::getNumInitialSettings() const {
 
 
 void
-GUICompleteSchemeStorage::init(FXApp* app) {
+GUICompleteSchemeStorage::init(FXApp* app, bool netedit) {
     {
-        GUIVisualizationSettings vs;
+        GUIVisualizationSettings vs(netedit);
         vs.name = "standard";
         vs.laneShowBorders = true;
         gSchemeStorage.add(vs);
     }
     {
-        GUIVisualizationSettings vs;
+        GUIVisualizationSettings vs(netedit);
         vs.name = "faster standard";
         vs.laneShowBorders = false;
         vs.showLinkDecals = false;
         vs.showRails = false;
+        vs.showRails = false;
+        vs.showSublanes = false;
         gSchemeStorage.add(vs);
     }
     {
-        GUIVisualizationSettings vs;
+        GUIVisualizationSettings vs(netedit);
         vs.name = "real world";
         vs.vehicleQuality = 2;
         vs.backgroundColor = RGBColor(51, 128, 51, 255);
@@ -141,6 +134,31 @@ GUICompleteSchemeStorage::init(FXApp* app) {
         vs.vehicleSize.minSize = 0;
         vs.personQuality = 2;
         vs.containerQuality = 2;
+        vs.showSublanes = false;
+        gSchemeStorage.add(vs);
+    }
+    {
+        GUIVisualizationSettings vs(netedit);
+        vs.name = "rail";
+        vs.vehicleQuality = 2;
+        vs.showLaneDirection = true;
+        vs.spreadSuperposed = true;
+        vs.junctionSize.constantSize = true;
+        vs.junctionColorer.setSchemeByName(GUIVisualizationSettings::SCHEME_NAME_TYPE);
+        gSchemeStorage.add(vs);
+    }
+
+    if (!netedit) {
+        GUIVisualizationSettings vs(netedit);
+        vs.name = "selection";
+        vs.vehicleColorer.setSchemeByName(GUIVisualizationSettings::SCHEME_NAME_SELECTION);
+        vs.edgeColorer.setSchemeByName(GUIVisualizationSettings::SCHEME_NAME_SELECTION);
+        vs.laneColorer.setSchemeByName(GUIVisualizationSettings::SCHEME_NAME_SELECTION);
+        vs.junctionColorer.setSchemeByName(GUIVisualizationSettings::SCHEME_NAME_SELECTION);
+        vs.personColorer.setSchemeByName(GUIVisualizationSettings::SCHEME_NAME_SELECTION);
+        vs.containerColorer.setSchemeByName(GUIVisualizationSettings::SCHEME_NAME_SELECTION);
+        vs.poiColorer.setSchemeByName(GUIVisualizationSettings::SCHEME_NAME_SELECTION);
+        vs.polyColorer.setSchemeByName(GUIVisualizationSettings::SCHEME_NAME_SELECTION);
         gSchemeStorage.add(vs);
     }
     myNumInitialSettings = (int) mySortedSchemeNames.size();
@@ -150,7 +168,7 @@ GUICompleteSchemeStorage::init(FXApp* app) {
         std::string name = "visset#" + toString(i);
         std::string setting = app->reg().readStringEntry("VisualizationSettings", name.c_str(), "");
         if (setting != "") {
-            GUIVisualizationSettings vs;
+            GUIVisualizationSettings vs(netedit);
 
             vs.name = setting;
             app->reg().readStringEntry("VisualizationSettings", name.c_str(), "");
@@ -170,9 +188,9 @@ GUICompleteSchemeStorage::init(FXApp* app) {
             }
             if (content != "" && xmlSize == 0) {
                 try {
-                    GUISettingsHandler handler(content, false);
+                    GUISettingsHandler handler(content, false, netedit);
                     handler.addSettings();
-                } catch (ProcessError) { }
+                } catch (ProcessError&) { }
             }
         }
     }
@@ -186,8 +204,8 @@ GUICompleteSchemeStorage::writeSettings(FXApp* app) {
     const std::vector<std::string>& names = getNames();
     app->reg().writeIntEntry("VisualizationSettings", "settingNo", (FXint) names.size() - myNumInitialSettings);
     int gidx = 0;
-    for (std::vector<std::string>::const_iterator i = names.begin() + myNumInitialSettings; i != names.end(); ++i, ++gidx) {
-        const GUIVisualizationSettings& item = mySettings.find(*i)->second;
+    for (std::vector<std::string>::const_iterator it = names.begin() + myNumInitialSettings; it != names.end(); ++it, ++gidx) {
+        const GUIVisualizationSettings& item = mySettings.find(*it)->second;
         std::string sname = "visset#" + toString(gidx);
 
         app->reg().writeStringEntry("VisualizationSettings", sname.c_str(), item.name.c_str());
@@ -205,8 +223,9 @@ GUICompleteSchemeStorage::writeSettings(FXApp* app) {
 
 
 void
-GUICompleteSchemeStorage::saveViewport(const double x, const double y, const double z) {
+GUICompleteSchemeStorage::saveViewport(const double x, const double y, const double z, const double rot) {
     myLookFrom.set(x, y, z);
+    myRotation = rot;
 }
 
 
@@ -214,7 +233,7 @@ void
 GUICompleteSchemeStorage::setViewport(GUISUMOAbstractView* view) {
     if (myLookFrom.z() > 0) {
         // look straight down
-        view->setViewportFromTo(myLookFrom, Position(myLookFrom.x(), myLookFrom.y(), 0));
+        view->setViewportFromToRot(myLookFrom, Position(myLookFrom.x(), myLookFrom.y(), 0), myRotation);
     } else {
         view->recenterView();
     }
@@ -222,4 +241,3 @@ GUICompleteSchemeStorage::setViewport(GUISUMOAbstractView* view) {
 
 
 /****************************************************************************/
-
