@@ -1,41 +1,32 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIPropertyScheme.h
 /// @author  Michael Behrisch
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    Mon, 20.07.2009
-/// @version $Id: GUIPropertyScheme.h 21182 2016-07-18 06:46:01Z behrisch $
 ///
 //
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIPropertyScheme_h
-#define GUIPropertyScheme_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <cassert>
 #include <vector>
 #include <utils/common/RGBColor.h>
 #include <utils/iodevices/OutputDevice.h>
+#include <utils/gui/images/GUIIcons.h>
 
 
 // ===========================================================================
@@ -53,14 +44,18 @@ class GUIPropertyScheme {
 public:
     /// Constructor
     GUIPropertyScheme(const std::string& name, const T& baseColor,
-                      const std::string& colName = "", const bool isFixed = false, SUMOReal baseValue = 0) :
+                      const std::string& colName = "", const bool isFixed = false, double baseValue = 0,
+                      RGBColor bgColor = RGBColor::INVISIBLE,
+                      GUIIcon icon = GUIIcon::EMPTY) :
         myName(name), myIsInterpolated(!isFixed),
         myIsFixed(isFixed),
-        myAllowNegativeValues(false) {
+        myAllowNegativeValues(false),
+        myIcon(icon),
+        myBgColor(bgColor) {
         addColor(baseColor, baseValue, colName);
     }
 
-    void setThreshold(const int pos, const SUMOReal threshold) {
+    void setThreshold(const int pos, const double threshold) {
         myThresholds[pos] = threshold;
     }
 
@@ -80,9 +75,9 @@ public:
         return false;
     }
 
-    int addColor(const T& color, const SUMOReal threshold, const std::string& name = "") {
+    int addColor(const T& color, const double threshold, const std::string& name = "") {
         typename std::vector<T>::iterator colIt = myColors.begin();
-        std::vector<SUMOReal>::iterator threshIt = myThresholds.begin();
+        std::vector<double>::iterator threshIt = myThresholds.begin();
         std::vector<std::string>::iterator nameIt = myNames.begin();
         int pos = 0;
         while (threshIt != myThresholds.end() && (*threshIt) < threshold) {
@@ -98,7 +93,7 @@ public:
     }
 
     void removeColor(const int pos) {
-        assert(pos < myColors.size());
+        assert(pos < (int)myColors.size());
         myColors.erase(myColors.begin() + pos);
         myThresholds.erase(myThresholds.begin() + pos);
         myNames.erase(myNames.begin() + pos);
@@ -110,12 +105,12 @@ public:
         myNames.clear();
     }
 
-    const T getColor(const SUMOReal value) const {
+    const T getColor(const double value) const {
         if (myColors.size() == 1 || value < myThresholds.front()) {
             return myColors.front();
         }
         typename std::vector<T>::const_iterator colIt = myColors.begin() + 1;
-        std::vector<SUMOReal>::const_iterator threshIt = myThresholds.begin() + 1;
+        std::vector<double>::const_iterator threshIt = myThresholds.begin() + 1;
         while (threshIt != myThresholds.end() && (*threshIt) <= value) {
             ++threshIt;
             ++colIt;
@@ -126,11 +121,11 @@ public:
         if (!myIsInterpolated) {
             return *(colIt - 1);
         }
-        SUMOReal lowVal = *(threshIt - 1);
+        double lowVal = *(threshIt - 1);
         return interpolate(*(colIt - 1), *colIt, (value - lowVal) / ((*threshIt) - lowVal));
     }
 
-    void setInterpolated(const bool interpolate, SUMOReal interpolationStart = 0.f) {
+    void setInterpolated(const bool interpolate, double interpolationStart = 0.f) {
         myIsInterpolated = interpolate;
         if (interpolate) {
             myThresholds[0] = interpolationStart;
@@ -145,7 +140,7 @@ public:
         return myColors;
     }
 
-    const std::vector<SUMOReal>& getThresholds() const {
+    const std::vector<double>& getThresholds() const {
         return myThresholds;
     }
 
@@ -169,6 +164,14 @@ public:
         myAllowNegativeValues = value;
     }
 
+    GUIIcon getIcon() const {
+        return myIcon;
+    }
+
+    const RGBColor& getBackgroundColor() const {
+        return myBgColor;
+    }
+
     void save(OutputDevice& dev) const {
         const std::string tag = getTagName(myColors);
 
@@ -178,12 +181,12 @@ public:
             dev.writeAttr(SUMO_ATTR_INTERPOLATED, myIsInterpolated);
         }
         typename std::vector<T>::const_iterator colIt = myColors.begin();
-        std::vector<SUMOReal>::const_iterator threshIt = myThresholds.begin();
+        std::vector<double>::const_iterator threshIt = myThresholds.begin();
         std::vector<std::string>::const_iterator nameIt = myNames.begin();
         while (threshIt != myThresholds.end()) {
             dev.openTag(SUMO_TAG_ENTRY);
             dev.writeAttr(SUMO_ATTR_COLOR, *colIt);
-            if (!myIsFixed) {
+            if (!myIsFixed && (*threshIt) != std::numeric_limits<double>::max()) {
                 dev.writeAttr(SUMO_ATTR_THRESHOLD, *threshIt);
             }
             if ((*nameIt) != "") {
@@ -203,7 +206,7 @@ public:
 
 
     /// @brief specializations for GUIColorScheme
-    RGBColor interpolate(const RGBColor& min, const RGBColor& max, SUMOReal weight) const {
+    RGBColor interpolate(const RGBColor& min, const RGBColor& max, double weight) const {
         return RGBColor::interpolate(min, max, weight);
     }
 
@@ -213,11 +216,11 @@ public:
 
 
     /// @brief specializations for GUIScaleScheme
-    SUMOReal interpolate(const SUMOReal& min, const SUMOReal& max, SUMOReal weight) const {
+    double interpolate(const double& min, const double& max, double weight) const {
         return min + (max - min) * weight;
     }
 
-    std::string getTagName(std::vector<SUMOReal>) const {
+    std::string getTagName(std::vector<double>) const {
         return toString(SUMO_TAG_SCALINGSCHEME);
     }
 
@@ -225,17 +228,15 @@ public:
 private:
     std::string myName;
     std::vector<T> myColors;
-    std::vector<SUMOReal> myThresholds;
+    std::vector<double> myThresholds;
     bool myIsInterpolated;
     std::vector<std::string> myNames;
     bool myIsFixed;
     bool myAllowNegativeValues;
+    GUIIcon myIcon;
+    RGBColor myBgColor;
 
 };
 
 typedef GUIPropertyScheme<RGBColor> GUIColorScheme;
-typedef GUIPropertyScheme<SUMOReal> GUIScaleScheme;
-
-#endif
-
-/****************************************************************************/
+typedef GUIPropertyScheme<double> GUIScaleScheme;
