@@ -1,55 +1,44 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIParameterTracker.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIParameterTracker.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A window which displays the time line of one (or more) value(s)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <fstream>
-#include <foreign/polyfonts/polyfonts.h>
 #include <utils/foxtools/MFXUtils.h>
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/common/ToString.h>
 #include <utils/common/StringUtils.h>
 #include <utils/common/SUMOTime.h>
+#include <utils/gui/div/GLHelper.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <utils/gui/div/GUIIOGlobals.h>
+#include <utils/gui/div/GUIDesigns.h>
 #include <utils/gui/windows/GUIAppEnum.h>
 #include <utils/gui/windows/GUIMainWindow.h>
 #include <utils/gui/images/GUIIconSubSys.h>
+#include <foreign/fontstash/fontstash.h>
 #include "GUIParameterTracker.h"
 #include <utils/gui/globjects/GLIncludes.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // FOX callback mapping
@@ -72,25 +61,25 @@ FXIMPLEMENT(GUIParameterTracker, FXMainWindow, GUIParameterTrackerMap, ARRAYNUMB
 // ===========================================================================
 GUIParameterTracker::GUIParameterTracker(GUIMainWindow& app,
         const std::string& name)
-    : FXMainWindow(app.getApp(), "Tracker", NULL, NULL, DECOR_ALL, 20, 20, 300, 200),
+    : FXMainWindow(app.getApp(), "Tracker", nullptr, nullptr, DECOR_ALL, 20, 20, 300, 200),
       myApplication(&app) {
     buildToolBar();
-    app.addChild(this, true);
+    app.addChild(this);
     FXVerticalFrame* glcanvasFrame = new FXVerticalFrame(this, FRAME_SUNKEN | LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0);
     myPanel = new GUIParameterTrackerPanel(glcanvasFrame, *myApplication, *this);
     setTitle(name.c_str());
-    setIcon(GUIIconSubSys::getIcon(ICON_APP_TRACKER));
+    setIcon(GUIIconSubSys::getIcon(GUIIcon::APP_TRACKER));
 }
 
 
 GUIParameterTracker::~GUIParameterTracker() {
     myApplication->removeChild(this);
     for (std::vector<TrackerValueDesc*>::iterator i1 = myTracked.begin(); i1 != myTracked.end(); i1++) {
-        delete(*i1);
+        delete (*i1);
     }
     // deleted by GUINet
-    for (std::vector<GLObjectValuePassConnector<SUMOReal>*>::iterator i2 = myValuePassers.begin(); i2 != myValuePassers.end(); i2++) {
-        delete(*i2);
+    for (std::vector<GLObjectValuePassConnector<double>*>::iterator i2 = myValuePassers.begin(); i2 != myValuePassers.end(); i2++) {
+        delete (*i2);
     }
     delete myToolBarDrag;
     delete myToolBar;
@@ -106,17 +95,16 @@ GUIParameterTracker::create() {
 
 void
 GUIParameterTracker::buildToolBar() {
-    myToolBarDrag = new FXToolBarShell(this, FRAME_NORMAL);
+    myToolBarDrag = new FXToolBarShell(this, GUIDesignToolBar);
     myToolBar = new FXToolBar(this, myToolBarDrag, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | FRAME_RAISED);
-    new FXToolBarGrip(myToolBar, myToolBar, FXToolBar::ID_TOOLBARGRIP, TOOLBARGRIP_DOUBLE);
+    new FXToolBarGrip(myToolBar, myToolBar, FXToolBar::ID_TOOLBARGRIP, GUIDesignToolBarGrip);
     // save button
     new FXButton(myToolBar, "\t\tSave the data...",
-                 GUIIconSubSys::getIcon(ICON_SAVE), this, GUIParameterTracker::MID_SAVE,
-                 ICON_ABOVE_TEXT | BUTTON_TOOLBAR | FRAME_RAISED | LAYOUT_TOP | LAYOUT_LEFT);
+                 GUIIconSubSys::getIcon(GUIIcon::SAVE), this, GUIParameterTracker::MID_SAVE, GUIDesignButtonToolbar);
     // aggregation interval combo
     myAggregationInterval =
         new FXComboBox(myToolBar, 8, this, MID_AGGREGATIONINTERVAL,
-                       FRAME_SUNKEN | LAYOUT_LEFT | LAYOUT_TOP | COMBOBOX_STATIC);
+                       GUIDesignComboBoxStatic);
     myAggregationInterval->appendItem("1s");
     myAggregationInterval->appendItem("1min");
     myAggregationInterval->appendItem("5min");
@@ -128,25 +116,25 @@ GUIParameterTracker::buildToolBar() {
 
 
 void
-GUIParameterTracker::addTracked(GUIGlObject& o, ValueSource<SUMOReal>* src,
+GUIParameterTracker::addTracked(GUIGlObject& o, ValueSource<double>* src,
                                 TrackerValueDesc* newTracked) {
     myTracked.push_back(newTracked);
     // build connection (is automatically set into an execution map)
-    myValuePassers.push_back(new GLObjectValuePassConnector<SUMOReal>(o, src, newTracked));
+    myValuePassers.push_back(new GLObjectValuePassConnector<double>(o, src, newTracked));
 }
 
 
 long
-GUIParameterTracker::onConfigure(FXObject* sender, FXSelector sel, void* data) {
-    myPanel->onConfigure(sender, sel, data);
-    return FXMainWindow::onConfigure(sender, sel, data);
+GUIParameterTracker::onConfigure(FXObject* sender, FXSelector sel, void* ptr) {
+    myPanel->onConfigure(sender, sel, ptr);
+    return FXMainWindow::onConfigure(sender, sel, ptr);
 }
 
 
 long
-GUIParameterTracker::onPaint(FXObject* sender, FXSelector sel, void* data) {
-    myPanel->onPaint(sender, sel, data);
-    return FXMainWindow::onPaint(sender, sel, data);
+GUIParameterTracker::onPaint(FXObject* sender, FXSelector sel, void* ptr) {
+    myPanel->onPaint(sender, sel, ptr);
+    return FXMainWindow::onPaint(sender, sel, ptr);
 }
 
 
@@ -160,7 +148,7 @@ GUIParameterTracker::onSimStep(FXObject*, FXSelector, void*) {
 long
 GUIParameterTracker::onCmdChangeAggregation(FXObject*, FXSelector, void*) {
     int index = myAggregationInterval->getCurrentItem();
-    size_t aggInt = 0;
+    int aggInt = 0;
     switch (index) {
         case 0:
             aggInt = 1;
@@ -182,7 +170,6 @@ GUIParameterTracker::onCmdChangeAggregation(FXObject*, FXSelector, void*) {
             break;
         default:
             throw 1;
-            break;
     }
     for (std::vector<TrackerValueDesc*>::iterator i1 = myTracked.begin(); i1 != myTracked.end(); i1++) {
         (*i1)->setAggregationSpan(TIME2STEPS(aggInt));
@@ -193,7 +180,7 @@ GUIParameterTracker::onCmdChangeAggregation(FXObject*, FXSelector, void*) {
 
 long
 GUIParameterTracker::onCmdSave(FXObject*, FXSelector, void*) {
-    FXString file = MFXUtils::getFilename2Write(this, "Save Data", ".csv", GUIIconSubSys::getIcon(ICON_EMPTY), gCurrentFolder);
+    FXString file = MFXUtils::getFilename2Write(this, "Save Data", ".csv", GUIIconSubSys::getIcon(GUIIcon::EMPTY), gCurrentFolder);
     if (file == "") {
         return 1;
     }
@@ -211,17 +198,17 @@ GUIParameterTracker::onCmdSave(FXObject*, FXSelector, void*) {
         }
         dev << '\n';
         // count entries
-        size_t max = 0;
+        int max = 0;
         for (i = myTracked.begin(); i != myTracked.end(); ++i) {
             TrackerValueDesc* tvd = *i;
-            size_t sizei = tvd->getAggregatedValues().size();
+            int sizei = (int)tvd->getAggregatedValues().size();
             if (max < sizei) {
                 max = sizei;
             }
             tvd->unlockValues();
         }
         // write entries
-        for (unsigned int j = 0; j < max; j++) {
+        for (int j = 0; j < max; j++) {
             for (i = myTracked.begin(); i != myTracked.end(); ++i) {
                 if (i != myTracked.begin()) {
                     dev << ';';
@@ -257,7 +244,7 @@ FXIMPLEMENT(GUIParameterTracker::GUIParameterTrackerPanel, FXGLCanvas, GUIParame
 GUIParameterTracker::GUIParameterTrackerPanel::GUIParameterTrackerPanel(
     FXComposite* c, GUIMainWindow& app,
     GUIParameterTracker& parent)
-    : FXGLCanvas(c, app.getGLVisual(), app.getBuildGLCanvas(), (FXObject*) 0, (FXSelector) 0, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 300, 200),
+    : FXGLCanvas(c, app.getGLVisual(), app.getBuildGLCanvas(), (FXObject*) nullptr, (FXSelector) 0, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 300, 200),
       myParent(&parent), myApplication(&app) {}
 
 
@@ -266,19 +253,16 @@ GUIParameterTracker::GUIParameterTrackerPanel::~GUIParameterTrackerPanel() {}
 
 void
 GUIParameterTracker::GUIParameterTrackerPanel::drawValues() {
-    pfSetScale((SUMOReal) 0.1);
-    pfSetScaleXY((SUMOReal)(.1 * 300. / myWidthInPixels), (SUMOReal)(.1 * 300. / (SUMOReal) myHeightInPixels));
-    //
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
     glDisable(GL_TEXTURE_2D);
-    size_t run = 0;
+    int run = 0;
     for (std::vector<TrackerValueDesc*>::iterator i = myParent->myTracked.begin(); i != myParent->myTracked.end(); i++) {
         TrackerValueDesc* desc = *i;
         drawValue(*desc,
-                  (SUMOReal) myWidthInPixels / (SUMOReal) myParent->myTracked.size() * (SUMOReal) run);
+                  (double) myWidthInPixels / (double) myParent->myTracked.size() * (double) run);
         run++;
     }
 }
@@ -286,14 +270,17 @@ GUIParameterTracker::GUIParameterTrackerPanel::drawValues() {
 
 void
 GUIParameterTracker::GUIParameterTrackerPanel::drawValue(TrackerValueDesc& desc,
-        SUMOReal /*namePos*/) {
+        double /*namePos*/) {
+    const double fontWidth = 0.1 * 300. / myWidthInPixels;
+    const double fontHeight = 0.1 * 300. /  myHeightInPixels;
+    //
     // apply scaling
     glPushMatrix();
 
     // apply the positiopn offset of the display
     glScaled(0.8, 0.8, 1);
     // apply value range scaling
-    SUMOReal ys = (SUMOReal) 2.0 / (SUMOReal) desc.getRange();
+    double ys = (double) 2.0 / (double) desc.getRange();
     glScaled(1.0, ys, 1.0);
     glTranslated(-1.0, -desc.getYCenter(), 0);
 
@@ -313,14 +300,14 @@ GUIParameterTracker::GUIParameterTrackerPanel::drawValue(TrackerValueDesc& desc,
     glEnd();
     glColor4ub(red, green, blue, 77);
     for (int a = 1; a < 6; a++) {
-        SUMOReal ypos = (desc.getRange()) / (SUMOReal) 6.0 * (SUMOReal) a + desc.getMin();
+        const double yp = desc.getRange() / 6.0 * (double) a + desc.getMin();
         glBegin(GL_LINES);
-        glVertex2d(0, ypos);
-        glVertex2d(2.0, ypos);
+        glVertex2d(0, yp);
+        glVertex2d(2.0, yp);
         glEnd();
     }
-    const std::vector<SUMOReal>& values = desc.getAggregatedValues();
-    SUMOReal latest = 0;
+    const std::vector<double>& values = desc.getAggregatedValues();
+    double latest = 0;
     if (values.size() < 2) {
         glPopMatrix();
         desc.unlockValues();
@@ -328,15 +315,15 @@ GUIParameterTracker::GUIParameterTrackerPanel::drawValue(TrackerValueDesc& desc,
     } else {
         latest = values.back();
         // init values
-        SUMOReal xStep = (SUMOReal) 2.0 / (SUMOReal) values.size();
-        std::vector<SUMOReal>::const_iterator i = values.begin();
-        SUMOReal yp = (*i);
-        SUMOReal xp = 0;
+        const double xStep = 2.0 / (double) values.size();
+        std::vector<double>::const_iterator i = values.begin();
+        double yp = (*i);
+        double xp = 0;
         i++;
         glColor4ub(red, green, blue, 255);
         for (; i != values.end(); i++) {
-            SUMOReal yn = (*i);
-            SUMOReal xn = xp + xStep;
+            double yn = (*i);
+            double xn = xp + xStep;
             glBegin(GL_LINES);
             glVertex2d(xp, yp);
             glVertex2d(xn, yn);
@@ -354,55 +341,39 @@ GUIParameterTracker::GUIParameterTrackerPanel::drawValue(TrackerValueDesc& desc,
     // draw min time
     SUMOTime beginStep = desc.getRecordingBegin();
     std::string begStr = time2string(beginStep);
-    SUMOReal w = pfdkGetStringWidth(begStr.c_str());
-    glRotated(180, 1, 0, 0);
-    pfSetPosition(0, 0);
-    glTranslated(-0.8 - w / 2., 0.88, 0);
-    pfDrawString(begStr.c_str());
-    glTranslated(0.8 + w / 2., -0.88, 0);
-    glRotated(-180, 1, 0, 0);
+    double w = 50 / myWidthInPixels;
+    glTranslated(-0.8 - w / 2., -0.88, 0);
+    GLHelper::drawText(begStr, Position(0, 0), 1, fontHeight, RGBColor::BLACK, 0, FONS_ALIGN_LEFT | FONS_ALIGN_MIDDLE, fontWidth);
+    glTranslated(0.8 + w / 2., 0.88, 0);
 
     // draw max time
-    glRotated(180, 1, 0, 0);
-    pfSetPosition(0, 0);
-    glTranslated(0.75, 0.88, 0);
-    pfDrawString(time2string(beginStep + static_cast<SUMOTime>(values.size() * desc.getAggregationSpan())).c_str());
-    glTranslated(-0.75, -0.88, 0);
-    glRotated(-180, 1, 0, 0);
+    glTranslated(0.75, -0.88, 0);
+    GLHelper::drawText(time2string(beginStep + static_cast<SUMOTime>(values.size() * desc.getAggregationSpan())),
+                       Position(0, 0), 1, fontHeight, RGBColor::BLACK, 0, FONS_ALIGN_LEFT | FONS_ALIGN_MIDDLE, fontWidth);
+    glTranslated(-0.75, 0.88, 0);
 
     // draw min value
-    glRotated(180, 1, 0, 0);
-    pfSetPosition(0, 0);
-    glTranslated(-0.98, 0.82, 0);
-    pfDrawString(toString(desc.getMin()).c_str());
-    glTranslated(0.98, -0.82, 0);
-    glRotated(-180, 1, 0, 0);
+    glTranslated(-0.98, -0.82, 0);
+    GLHelper::drawText(toString(desc.getMin()), Position(0, 0), 1, fontHeight, RGBColor::BLACK, 0, FONS_ALIGN_LEFT | FONS_ALIGN_MIDDLE, fontWidth);
+    glTranslated(0.98, 0.82, 0);
 
     // draw max value
-    glRotated(180, 1, 0, 0);
-    pfSetPosition(0, 0);
-    glTranslated(-0.98, -0.78, 0);
-    pfDrawString(toString(desc.getMax()).c_str());
-    glTranslated(0.98, 0.78, 0);
-    glRotated(-180, 1, 0, 0);
-
-    // draw current value
-    glRotated(180, 1, 0, 0);
-    pfSetPosition(0, 0);
-    SUMOReal p = (SUMOReal) 0.8 -
-                 ((SUMOReal) 1.6 / (desc.getMax() - desc.getMin()) * (latest - desc.getMin()));
-    glTranslated(-0.98, p + .02, 0);
-    pfDrawString(toString(latest).c_str());
-    glTranslated(0.98, -(p + .02), 0);
-    glRotated(-180, 1, 0, 0);
+    glTranslated(-0.98, 0.78, 0);
+    GLHelper::drawText(toString(desc.getMax()), Position(0, 0), 1, fontHeight, RGBColor::BLACK, 0, FONS_ALIGN_LEFT | FONS_ALIGN_MIDDLE, fontWidth);
+    glTranslated(0.98, -0.78, 0);
 
     // draw name
-    glRotated(180, 1, 0, 0);
-    pfSetPosition(0, 0);
-    glTranslated(-0.98, -.92, 0);
-    pfDrawString(desc.getName().c_str());
-    glTranslated(0.98, .92, 0);
-    glRotated(-180, 1, 0, 0);
+    glTranslated(-0.98, .92, 0);
+    GLHelper::drawText(desc.getName(), Position(0, 0), 1, fontHeight, RGBColor::BLACK, 0, FONS_ALIGN_LEFT | FONS_ALIGN_MIDDLE, fontWidth);
+    glTranslated(0.98, -.92, 0);
+
+    // draw current value (with contrasting color)
+    double p = (double) 0.8 -
+               ((double) 1.6 / (desc.getMax() - desc.getMin()) * (latest - desc.getMin()));
+    glTranslated(-0.98, -(p + .02), 0);
+    GLHelper::drawText(toString(latest), Position(0, 0), 1, fontHeight, RGBColor::RED, 0, FONS_ALIGN_LEFT | FONS_ALIGN_MIDDLE, fontWidth);
+    glTranslated(0.98, p + .02, 0);
+
 }
 
 
@@ -461,6 +432,4 @@ GUIParameterTracker::GUIParameterTrackerPanel::onPaint(FXObject*,
 }
 
 
-
 /****************************************************************************/
-
