@@ -1,39 +1,29 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIParameterTracker.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIParameterTracker.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // A window which displays the time line of one (or more) value(s)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIParameterTracker_h
-#define GUIParameterTracker_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
 // fx3d includes windows.h so we need to guard against macro pollution
 #ifdef WIN32
 #define NOMINMAX
@@ -173,8 +163,7 @@ public:
         GUIMainWindow* myApplication;
 
     protected:
-        /// FOX needs this
-        GUIParameterTrackerPanel() { }
+        FOX_CONSTRUCTOR(GUIParameterTrackerPanel)
     };
 
 public:
@@ -212,13 +201,6 @@ protected:
     FXToolBar* myToolBar;
 
 protected:
-    /// Fox needs this
-    GUIParameterTracker() { }
+    FOX_CONSTRUCTOR(GUIParameterTracker)
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
