@@ -1,33 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    TrackerValueDesc.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: TrackerValueDesc.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Storage for a tracked value
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -35,20 +28,19 @@
 #include <utils/gui/globjects/GUIGlObject.h>
 #include "TrackerValueDesc.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
 TrackerValueDesc::TrackerValueDesc(const std::string& name,
                                    const RGBColor& col,
-                                   SUMOTime recordBegin)
+                                   SUMOTime recordBegin,
+                                   double aggregationSeconds)
     : myName(name), myActiveCol(col), myInactiveCol(col),
       myMin(0), myMax(0),
-      myAggregationInterval(TIME2STEPS(1) / DELTA_T), myInvalidValue(-1), myValidNo(0),
+      myAggregationInterval(MAX2(1, (int)(TIME2STEPS(aggregationSeconds) / DELTA_T))),
+      myInvalidValue(INVALID_DOUBLE),
+      myValidNo(0),
       myRecordingBegin(recordBegin), myTmpLastAggValue(0) {}
 
 
@@ -61,7 +53,7 @@ TrackerValueDesc::~TrackerValueDesc() {
 
 
 void
-TrackerValueDesc::addValue(SUMOReal value) {
+TrackerValueDesc::addValue(double value) {
     if (myValues.size() == 0) {
         myMin = value;
         myMax = value;
@@ -69,13 +61,13 @@ TrackerValueDesc::addValue(SUMOReal value) {
         myMin = value < myMin ? value : myMin;
         myMax = value > myMax ? value : myMax;
     }
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     myValues.push_back(value);
     if (value != myInvalidValue) {
         myTmpLastAggValue += value;
         myValidNo++;
     }
-    const SUMOReal avg = myValidNo == 0 ? static_cast<SUMOReal>(0) : myTmpLastAggValue / static_cast<SUMOReal>(myValidNo);
+    const double avg = myValidNo == 0 ? static_cast<double>(0) : myTmpLastAggValue / static_cast<double>(myValidNo);
     if (myAggregationInterval == 1 || myValues.size() % myAggregationInterval == 1) {
         myAggregatedValues.push_back(avg);
     } else {
@@ -88,25 +80,25 @@ TrackerValueDesc::addValue(SUMOReal value) {
 }
 
 
-SUMOReal
+double
 TrackerValueDesc::getRange() const {
     return myMax - myMin;
 }
 
 
-SUMOReal
+double
 TrackerValueDesc::getMin() const {
     return myMin;
 }
 
 
-SUMOReal
+double
 TrackerValueDesc::getMax() const {
     return myMax;
 }
 
 
-SUMOReal
+double
 TrackerValueDesc::getYCenter() const {
     return (myMin + myMax) / 2.0f;
 }
@@ -118,14 +110,14 @@ TrackerValueDesc::getColor() const {
 }
 
 
-const std::vector<SUMOReal>&
+const std::vector<double>&
 TrackerValueDesc::getValues() {
     myLock.lock();
     return myValues;
 }
 
 
-const std::vector<SUMOReal>&
+const std::vector<double>&
 TrackerValueDesc::getAggregatedValues() {
     myLock.lock();
     return myAggregatedValues;
@@ -145,13 +137,13 @@ TrackerValueDesc::unlockValues() {
 
 void
 TrackerValueDesc::setAggregationSpan(SUMOTime as) {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     if (myAggregationInterval != as / DELTA_T) {
-        myAggregationInterval = as / DELTA_T;
+        myAggregationInterval = (int)(as / DELTA_T);
         // ok, the aggregation has changed,
         //  let's recompute the list of aggregated values
         myAggregatedValues.clear();
-        std::vector<SUMOReal>::const_iterator i = myValues.begin();
+        std::vector<double>::const_iterator i = myValues.begin();
         while (i != myValues.end()) {
             myTmpLastAggValue = 0;
             myValidNo = 0;
@@ -164,7 +156,7 @@ TrackerValueDesc::setAggregationSpan(SUMOTime as) {
             if (myValidNo == 0) {
                 myAggregatedValues.push_back(0);
             } else {
-                myAggregatedValues.push_back(myTmpLastAggValue / static_cast<SUMOReal>(myValidNo));
+                myAggregatedValues.push_back(myTmpLastAggValue / static_cast<double>(myValidNo));
             }
         }
     }
@@ -183,6 +175,4 @@ TrackerValueDesc::getRecordingBegin() const {
 }
 
 
-
 /****************************************************************************/
-
