@@ -1,38 +1,29 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    TrackerValueDesc.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: TrackerValueDesc.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Representation of a timeline of floats with their names and moments
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef TrackerValueDesc_h
-#define TrackerValueDesc_h
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
 #include <string>
 #include <vector>
-#include <utils/foxtools/MFXMutex.h>
 #include <utils/common/RGBColor.h>
 #include <utils/common/SUMOTime.h>
 #include <utils/common/ValueRetriever.h>
@@ -52,7 +43,8 @@ class TrackerValueDesc : public ValueRetriever<double> {
 public:
     /// Constructor
     TrackerValueDesc(const std::string& name, const RGBColor& col,
-                     SUMOTime recordBegin);
+                     SUMOTime recordBegin,
+                     double aggregationSeconds);
 
     /// Destructor
     ~TrackerValueDesc();
@@ -121,7 +113,7 @@ private:
     double myMin, myMax;
 
     // Mutex to avoid parallel drawing and insertion of new items
-    MFXMutex myLock;
+    FXMutex myLock;
 
     /// The aggregation interval in simulation steps
     int myAggregationInterval;
@@ -139,9 +131,3 @@ private:
     double myTmpLastAggValue;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
