@@ -1,38 +1,30 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GUIDanielPerspectiveChanger.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIDanielPerspectiveChanger.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A class that allows to steer the visual output in dependence to
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <fxkeys.h>
 #include <utils/geom/Boundary.h>
 #include <utils/geom/Position.h>
+#include <utils/geom/GeomHelper.h>
 #include <utils/gui/settings/GUICompleteSchemeStorage.h>
 #include "GUIPerspectiveChanger.h"
 #include "GUIDanielPerspectiveChanger.h"
@@ -66,7 +58,7 @@ GUIDanielPerspectiveChanger::move(int xdiff, int ydiff) {
 
 void
 GUIDanielPerspectiveChanger::zoom(double factor) {
-    if (myCallback.getApp()->reg().readIntEntry("gui", "zoomAtCenter", 1)) {
+    if (myCallback.getApp()->reg().readIntEntry("gui", "zoomAtCenter", 0)) {
         myZoomBase = myViewPort.getCenter();
     }
     if (factor > 0) {
@@ -159,7 +151,7 @@ GUIDanielPerspectiveChanger::onLeftBtnPress(void* data) {
 
 bool
 GUIDanielPerspectiveChanger::onLeftBtnRelease(void* data) {
-    myMouseButtonState &= !MOUSEBTN_LEFT;
+    myMouseButtonState &= ~MOUSEBTN_LEFT;
     FXEvent* e = (FXEvent*) data;
     myMouseXPosition = e->win_x;
     myMouseYPosition = e->win_y;
@@ -181,8 +173,8 @@ GUIDanielPerspectiveChanger::onRightBtnPress(void* data) {
 
 bool
 GUIDanielPerspectiveChanger::onRightBtnRelease(void* data) {
-    myMouseButtonState &= !MOUSEBTN_RIGHT;
-    if (data != 0) {
+    myMouseButtonState &= ~MOUSEBTN_RIGHT;
+    if (data != nullptr) {
         FXEvent* e = (FXEvent*) data;
         myMouseXPosition = e->win_x;
         myMouseYPosition = e->win_y;
@@ -229,7 +221,13 @@ GUIDanielPerspectiveChanger::onMouseMove(void* data) {
     switch (myMouseButtonState) {
         case MOUSEBTN_LEFT:
             if (pastDelay) {
-                move(xdiff, ydiff);
+                if (myRotation != 0) {
+                    Position diffRot = Position(xdiff, ydiff).rotateAround2D(
+                                           DEG2RAD(myRotation), Position(0, 0));
+                    move((int)diffRot.x(), (int)diffRot.y());
+                } else {
+                    move(xdiff, ydiff);
+                }
                 if (moved) {
                     myMoveOnClick = true;
                 }
@@ -273,6 +271,11 @@ GUIDanielPerspectiveChanger::setViewportFrom(double xPos, double yPos, double zP
 }
 
 
+void
+GUIDanielPerspectiveChanger::setRotation(double rotation) {
+    myRotation = rotation;
+}
+
 void
 GUIDanielPerspectiveChanger::changeCanvasSizeLeft(int change) {
     myViewPort = Boundary(
@@ -295,9 +298,7 @@ GUIDanielPerspectiveChanger::onKeyPress(void* data) {
     double moveY = 0;
     double moveFactor = 1;
     bool pageVertical = true;
-    bool ctrl = false;
     if (e->state & CONTROLMASK) {
-        ctrl = true;
         zoomDiff /= 2;
         moveFactor /= 10;
     } else if (e->state & SHIFTMASK) {
@@ -353,13 +354,6 @@ GUIDanielPerspectiveChanger::onKeyPress(void* data) {
             myCallback.recenterView();
             myCallback.update();
             return 1;
-        case FX::KEY_v:
-            // from an architecture standpoint this isn't the best place to put
-            // this. But its simple
-            if (ctrl) {
-                myCallback.showViewschemeEditor();
-                return 1;
-            }
         default:
             return 0;
     }
