@@ -1,38 +1,28 @@
 /****************************************************************************/
-/// @brief @file    GUIPerspectiveChanger.h
-/// @brief @author  Daniel Krajzewicz
-/// @brief @author  Jakob Erdmann
-/// @brief @author  Michael Behrisch
-/// @brief @date    Sept 2002
-/// @brief @version $Id: GUIPerspectiveChanger.h 21131 2016-07-08 07:59:22Z behrisch $
-/// @brief
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
+/// @file    GUIPerspectiveChanger.h
+/// @author  Daniel Krajzewicz
+/// @author  Jakob Erdmann
+/// @author  Michael Behrisch
+/// @date    Sept 2002
+///
 // A virtual class that allows to steer the visual output in dependence to
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef GUIPerspectiveChanger_h
-#define GUIPerspectiveChanger_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <fx.h>
+#include <utils/foxtools/fxheader.h>
 #include <utils/geom/Boundary.h>
 #include <utils/geom/Position.h>
 #include "GUISUMOAbstractView.h"
@@ -104,27 +94,42 @@ public:
     //@}
 
     /// @brief Returns the rotation of the canvas stored in this changer
-    virtual SUMOReal getRotation() const = 0;
+    virtual double getRotation() const = 0;
 
     /// @brief Returns the x-offset of the field to show stored in this changer
-    virtual SUMOReal getXPos() const = 0;
+    virtual double getXPos() const = 0;
 
     /// @brief Returns the y-offset of the field to show stored in this changer
-    virtual SUMOReal getYPos() const = 0;
+    virtual double getYPos() const = 0;
 
     /// @brief Returns the zoom factor computed stored in this changer
-    virtual SUMOReal getZoom() const = 0;
+    virtual double getZoom() const = 0;
+
+    /// @brief Returns the camera height corresponding to the current zoom factor
+    virtual double getZPos() const = 0;
+
+    /// @brief Returns the camera height at which the given zoom level is reached
+    virtual double zoom2ZPos(double zoom) const = 0;
+
+    /// @brief Returns the zoom level that is achieved at a given camera height
+    virtual double zPos2Zoom(double zPos) const = 0;
 
     /// @brief Centers the view to the given position, setting it to a size that covers the radius. Used for: Centering of vehicles and junctions */
-    virtual void centerTo(const Position& pos, SUMOReal radius, bool applyZoom = true) = 0;
+    virtual void centerTo(const Position& pos, double radius, bool applyZoom = true) = 0;
 
     /// @brief Sets the viewport Used for: Adapting a new viewport
-    virtual void setViewport(SUMOReal zoom, SUMOReal xPos, SUMOReal yPos) = 0;
+    virtual void setViewport(double zoom, double xPos, double yPos) = 0;
 
-    /// @brief Returns the last mouse x-position an event occured at
+    /// @brief Alternative method for setting the viewport
+    virtual void setViewportFrom(double xPos, double yPos, double zPos) = 0;
+
+    /// @brief Sets the rotation
+    virtual void setRotation(double rotation) = 0;
+
+    /// @brief Returns the last mouse x-position an event occurred at
     FXint getMouseXPosition() const;
 
-    /// @brief Returns the last mouse y-position an event occured at
+    /// @brief Returns the last mouse y-position an event occurred at
     FXint getMouseYPosition() const;
 
     /* @brief Adapts the viewport so that a change in canvass size keeps most of the
@@ -133,7 +138,7 @@ public:
      *
      * @param[in] change The horizontal change in canvas size in pixels
      */
-    virtual void changeCanvassLeft(int change) = 0;
+    virtual void changeCanvasSizeLeft(int change) = 0;
 
     /// @brief get viewport
     Boundary getViewport(bool fixRatio = true);
@@ -164,9 +169,3 @@ private:
     /// @brief Invalidated assignment operator.
     GUIPerspectiveChanger& operator=(const GUIPerspectiveChanger&);
 };
-
-
-#endif
-
-/****************************************************************************/
-
