@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    LineReader.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    Fri, 19 Jul 2002
-/// @version $Id: LineReader.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Retrieves a file linewise and reports the lines to a handler.
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <fstream>
@@ -132,6 +123,7 @@ LineReader::readLine() {
         if (idx == 0) {
             myStrBuffer = myStrBuffer.substr(1);
             myRread++;
+            myLinesRead++;
             return "";
         }
         if (idx != std::string::npos) {
@@ -152,6 +144,7 @@ LineReader::readLine() {
                 toReport = myStrBuffer;
                 myRread += 1024;
                 if (toReport == "") {
+                    myLinesRead++;
                     return toReport;
                 }
             }
@@ -170,6 +163,7 @@ LineReader::readLine() {
     } else {
         toReport = "";
     }
+    myLinesRead++;
     return toReport;
 }
 
@@ -206,9 +200,19 @@ LineReader::reinit() {
     myStrm.seekg(0, std::ios::end);
     myAvailable = static_cast<int>(myStrm.tellg());
     myStrm.seekg(0, std::ios::beg);
+    if (myAvailable >= 3) {
+        // check for BOM
+        myStrm.read(myBuffer, 3);
+        if (myBuffer[0] == '\xef' && myBuffer[1] == '\xbb' && myBuffer[2] == '\xbf') {
+            myAvailable -= 3;
+        } else {
+            myStrm.seekg(0, std::ios::beg);
+        }
+    }
     myRead = 0;
     myRread = 0;
     myStrBuffer = "";
+    myLinesRead = 0;
 }
 
 
@@ -227,6 +231,4 @@ LineReader::good() const {
 }
 
 
-
 /****************************************************************************/
-
