@@ -1,33 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    NamedColumnsParser.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Fri, 19 Jul 2002
-/// @version $Id: NamedColumnsParser.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A parser to retrieve information from a table with known column
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <map>
 #include <string>
@@ -35,10 +26,6 @@
 #include <utils/common/StringUtils.h>
 #include "NamedColumnsParser.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -83,10 +70,10 @@ NamedColumnsParser::get(const std::string& name, bool prune) const {
             i = myDefinitionsMap.find(StringUtils::to_lower_case(name));
         }
         if (i == myDefinitionsMap.end()) {
-            throw UnknownElement(name);
+            throw UnknownElement("Element '" + name + "' is missing");
         }
     }
-    size_t pos = (*i).second;
+    int pos = (*i).second;
     if (myLineParser.size() <= pos) {
         throw OutOfBoundsException();
     }
@@ -107,14 +94,14 @@ NamedColumnsParser::know(const std::string& name) const {
     if (i == myDefinitionsMap.end()) {
         return false;
     }
-    size_t pos = (*i).second;
+    int pos = (*i).second;
     return myLineParser.size() > pos;
 }
 
 
 bool
 NamedColumnsParser::hasFullDefinition() const {
-    return myDefinitionsMap.size() == myLineParser.size();
+    return (int)myDefinitionsMap.size() == myLineParser.size();
 }
 
 
@@ -141,7 +128,7 @@ NamedColumnsParser::checkPrune(std::string& str, bool prune) const {
     if (!prune) {
         return;
     }
-    size_t idx = str.find_first_not_of(" ");
+    std::string::size_type idx = str.find_first_not_of(" ");
     if (idx != std::string::npos) {
         str = str.substr(idx);
     }
@@ -152,6 +139,4 @@ NamedColumnsParser::checkPrune(std::string& str, bool prune) const {
 }
 
 
-
 /****************************************************************************/
-
