@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    OutputDevice.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    2004
-/// @version $Id: OutputDevice.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Static storage of an output device and its base (abstract) implementation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <map>
 #include <fstream>
@@ -41,17 +32,13 @@
 #include "OutputDevice_CERR.h"
 #include "OutputDevice_Network.h"
 #include "PlainXMLFormatter.h"
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/FileHelpers.h>
 #include <utils/common/ToString.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/options/OptionsCont.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static member definitions
@@ -69,7 +56,7 @@ OutputDevice::getDevice(const std::string& name) {
         return *myOutputDevices[name];
     }
     // build the device
-    OutputDevice* dev = 0;
+    OutputDevice* dev = nullptr;
     // check whether the device shall print to stdout
     if (name == "stdout") {
         dev = OutputDevice_COUT::getDevice();
@@ -77,7 +64,7 @@ OutputDevice::getDevice(const std::string& name) {
         dev = OutputDevice_CERR::getDevice();
     } else if (FileHelpers::isSocket(name)) {
         try {
-            int port = TplConvert::_2int(name.substr(name.find(":") + 1).c_str());
+            int port = StringUtils::toInt(name.substr(name.find(":") + 1));
             dev = new OutputDevice_Network(name.substr(0, name.find(":")), port);
         } catch (NumberFormatException&) {
             throw IOError("Given port number '" + name.substr(name.find(":") + 1) + "' is not numeric.");
@@ -85,21 +72,22 @@ OutputDevice::getDevice(const std::string& name) {
             throw IOError("No port number given.");
         }
     } else {
-        const size_t len = name.length();
+        const int len = (int)name.length();
         std::string name2 = name;
         if (OptionsCont::getOptions().isSet("output-prefix") && name != "/dev/null") {
             std::string prefix = OptionsCont::getOptions().getString("output-prefix");
-            size_t metaTimeIndex = prefix.find("TIME");
+            const std::string::size_type metaTimeIndex = prefix.find("TIME");
             if (metaTimeIndex != std::string::npos) {
                 time_t rawtime;
                 char buffer [80];
                 time(&rawtime);
-                strftime(buffer, 80, "%F-%H-%M-%S", localtime(&rawtime));
+                struct tm* timeinfo = localtime(&rawtime);
+                strftime(buffer, 80, "%Y-%m-%d-%H-%M-%S", timeinfo);
                 prefix.replace(metaTimeIndex, 4, std::string(buffer));
             }
             name2 = FileHelpers::prependToLastPathComponent(prefix, name);
         }
-        dev = new OutputDevice_File(name2, len > 4 && name.substr(len - 4) == ".sbx");
+        dev = new OutputDevice_File(name2, len > 3 && name.substr(len - 3) == ".gz");
     }
     dev->setPrecision();
     dev->getOStream() << std::setiosflags(std::ios::fixed);
@@ -117,11 +105,7 @@ OutputDevice::createDeviceByOption(const std::string& optionName,
     }
     OutputDevice& dev = OutputDevice::getDevice(OptionsCont::getOptions().getString(optionName));
     if (rootElement != "") {
-        if (schemaFile != "") {
-            dev.writeXMLHeader(rootElement, "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/" + schemaFile + "\"");
-        } else {
-            dev.writeXMLHeader(rootElement);
-        }
+        dev.writeXMLHeader(rootElement, schemaFile);
     }
     return true;
 }
@@ -138,7 +122,7 @@ OutputDevice::getDeviceByOption(const std::string& optionName) {
 
 
 void
-OutputDevice::closeAll() {
+OutputDevice::closeAll(bool keepErrorRetrievers) {
     std::vector<OutputDevice*> errorDevices;
     std::vector<OutputDevice*> nonErrorDevices;
     for (std::map<std::string, OutputDevice*>::iterator i = myOutputDevices.begin(); i != myOutputDevices.end(); ++i) {
@@ -150,25 +134,28 @@ OutputDevice::closeAll() {
     }
     for (std::vector<OutputDevice*>::iterator i = nonErrorDevices.begin(); i != nonErrorDevices.end(); ++i) {
         try {
+            //std::cout << "  close '" << (*i)->getFilename() << "'\n";
             (*i)->close();
         } catch (const IOError& e) {
             WRITE_ERROR("Error on closing output devices.");
             WRITE_ERROR(e.what());
         }
     }
-    for (std::vector<OutputDevice*>::iterator i = errorDevices.begin(); i != errorDevices.end(); ++i) {
-        try {
-            (*i)->close();
-        } catch (const IOError& e) {
-            std::cerr << "Error on closing error output devices." << std::endl;
-            std::cerr << e.what() << std::endl;
+    if (!keepErrorRetrievers) {
+        for (std::vector<OutputDevice*>::iterator i = errorDevices.begin(); i != errorDevices.end(); ++i) {
+            try {
+                (*i)->close();
+            } catch (const IOError& e) {
+                std::cerr << "Error on closing error output devices." << std::endl;
+                std::cerr << e.what() << std::endl;
+            }
         }
     }
 }
 
 
 std::string
-OutputDevice::realString(const SUMOReal v, const int precision) {
+OutputDevice::realString(const double v, const int precision) {
     std::ostringstream oss;
     if (v == 0) {
         return "0";
@@ -176,7 +163,7 @@ OutputDevice::realString(const SUMOReal v, const int precision) {
     if (v < pow(10., -precision)) {
         oss.setf(std::ios::scientific, std::ios::floatfield);
     } else {
-        oss.setf(std::ios::fixed , std::ios::floatfield);    // use decimal format
+        oss.setf(std::ios::fixed, std::ios::floatfield);     // use decimal format
         oss.setf(std::ios::showpoint);    // print decimal point
         oss << std::setprecision(precision);
     }
@@ -188,13 +175,9 @@ OutputDevice::realString(const SUMOReal v, const int precision) {
 // ===========================================================================
 // member method definitions
 // ===========================================================================
-OutputDevice::OutputDevice(const bool binary, const unsigned int defaultIndentation)
-    : myAmBinary(binary) {
-    if (binary) {
-        myFormatter = new BinaryFormatter();
-    } else {
-        myFormatter = new PlainXMLFormatter(defaultIndentation);
-    }
+OutputDevice::OutputDevice(const int defaultIndentation, const std::string& filename) :
+    myFilename(filename) {
+    myFormatter = new PlainXMLFormatter(defaultIndentation);
 }
 
 
@@ -209,6 +192,11 @@ OutputDevice::ok() {
 }
 
 
+const std::string&
+OutputDevice::getFilename() {
+    return myFilename;
+}
+
 void
 OutputDevice::close() {
     while (closeTag()) {}
@@ -218,20 +206,26 @@ OutputDevice::close() {
             break;
         }
     }
+    MsgHandler::removeRetrieverFromAllInstances(this);
     delete this;
 }
 
 
 void
-OutputDevice::setPrecision(unsigned int precision) {
+OutputDevice::setPrecision(int precision) {
     getOStream() << std::setprecision(precision);
 }
 
 
 bool
 OutputDevice::writeXMLHeader(const std::string& rootElement,
-                             const std::string& attrs, const std::string& comment) {
-    return myFormatter->writeXMLHeader(getOStream(), rootElement, attrs, comment);
+                             const std::string& schemaFile,
+                             std::map<SumoXMLAttr, std::string> attrs) {
+    if (schemaFile != "") {
+        attrs[SUMO_ATTR_XMLNS] = "http://www.w3.org/2001/XMLSchema-instance";
+        attrs[SUMO_ATTR_SCHEMA_LOCATION] = "http://sumo.dlr.de/xsd/" + schemaFile;
+    }
+    return myFormatter->writeXMLHeader(getOStream(), rootElement, attrs);
 }
 
 
@@ -250,8 +244,8 @@ OutputDevice::openTag(const SumoXMLTag& xmlElement) {
 
 
 bool
-OutputDevice::closeTag() {
-    if (myFormatter->closeTag(getOStream())) {
+OutputDevice::closeTag(const std::string& comment) {
+    if (myFormatter->closeTag(getOStream(), comment)) {
         postWriteHook();
         return true;
     }
@@ -275,4 +269,3 @@ OutputDevice::inform(const std::string& msg, const char progress) {
 
 
 /****************************************************************************/
-
