@@ -1,44 +1,34 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    OutputDevice.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Mario Krumnow
 /// @date    2004
-/// @version $Id: OutputDevice.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Static storage of an output device and its base (abstract) implementation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef OutputDevice_h
-#define OutputDevice_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
+#include <cassert>
 #include <utils/common/ToString.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include "PlainXMLFormatter.h"
-#include "BinaryFormatter.h"
 
 
 // ===========================================================================
@@ -126,7 +116,7 @@ public:
 
     /**  Closes all registered devices
      */
-    static void closeAll();
+    static void closeAll(bool keepErrorRetrievers = false);
     /// @}
 
 
@@ -144,7 +134,7 @@ public:
     /// @{
 
     /// @brief Constructor
-    OutputDevice(const bool binary = false, const int defaultIndentation = 0);
+    OutputDevice(const int defaultIndentation = 0, const std::string& filename = "");
 
 
     /// @brief Destructor
@@ -156,6 +146,8 @@ public:
      */
     virtual bool ok();
 
+    /// @brief get filename or suitable description of this device
+    const std::string& getFilename();
 
     /** @brief Closes the device and removes it from the dictionary
      */
@@ -167,6 +159,11 @@ public:
      */
     void setPrecision(int precision = gPrecision);
 
+    /** @brief Returns the precison of the underlying stream
+     */
+    int getPrecision() {
+        return (int)getOStream().precision();
+    }
 
     /** @brief Writes an XML header with optional configuration
      *
@@ -186,9 +183,6 @@ public:
 
     template <typename E>
     bool writeHeader(const SumoXMLTag& rootElement) {
-        if (myAmBinary) {
-            return static_cast<BinaryFormatter*>(myFormatter)->writeHeader<E>(getOStream(), rootElement);
-        }
         return static_cast<PlainXMLFormatter*>(myFormatter)->writeHeader(getOStream(), rootElement);
     }
 
@@ -215,7 +209,7 @@ public:
     OutputDevice& openTag(const SumoXMLTag& xmlElement);
 
 
-    /** @brief Closes the most recently opened tag
+    /** @brief Closes the most recently opened tag and optionally adds a comment
      *
      * The topmost xml-element from the stack is written into the stream
      *  as a closing element. Depending on the formatter used
@@ -225,24 +219,14 @@ public:
      * @return Whether a further element existed in the stack and could be closed
      * @todo it is not verified that the topmost element was closed
      */
-    bool closeTag();
+    bool closeTag(const std::string& comment = "");
 
 
 
     /** @brief writes a line feed if applicable
      */
     void lf() {
-        if (!myAmBinary) {
-            getOStream() << "\n";
-        }
-    }
-
-
-    /** @brief Returns whether we have a binary output
-     * @return whether we have a binary output
-     */
-    bool isBinary() const {
-        return myAmBinary;
+        getOStream() << "\n";
     }
 
 
@@ -254,9 +238,21 @@ public:
      */
     template <typename T>
     OutputDevice& writeAttr(const SumoXMLAttr attr, const T& val) {
-        if (myAmBinary) {
-            BinaryFormatter::writeAttr(getOStream(), attr, val);
-        } else {
+        PlainXMLFormatter::writeAttr(getOStream(), attr, val);
+        return *this;
+    }
+
+    /** @brief writes a named attribute unless filtered
+     *
+     * @param[in] attr The attribute (name)
+     * @param[in] val The attribute value
+     * @param[in] attributeMask The filter that specifies whether the attribute shall be written
+     * @return The OutputDevice for further processing
+     */
+    template <typename T>
+    OutputDevice& writeOptionalAttr(const SumoXMLAttr attr, const T& val, long long int attributeMask) {
+        assert((int)attr <= 63);
+        if (attributeMask == 0 || attributeMask & ((long long int)1 << attr)) {
             PlainXMLFormatter::writeAttr(getOStream(), attr, val);
         }
         return *this;
@@ -271,15 +267,10 @@ public:
      */
     template <typename T>
     OutputDevice& writeAttr(const std::string& attr, const T& val) {
-        if (myAmBinary) {
-            BinaryFormatter::writeAttr(getOStream(), attr, val);
-        } else {
-            PlainXMLFormatter::writeAttr(getOStream(), attr, val);
-        }
+        PlainXMLFormatter::writeAttr(getOStream(), attr, val);
         return *this;
     }
 
-
     /** @brief writes a string attribute only if it is not the empty string and not the string "default"
      *
      * @param[in] attr The attribute (name)
@@ -304,6 +295,11 @@ public:
         return *this;
     }
 
+    /// @brief writes padding (ignored for binary output)
+    OutputDevice& writePadding(const std::string& val) {
+        myFormatter->writePadding(getOStream(), val);
+        return *this;
+    }
 
     /** @brief Retrieves a message to this device.
      *
@@ -324,6 +320,10 @@ public:
         return *this;
     }
 
+    void flush() {
+        getOStream().flush();
+    }
+
 protected:
     /// @brief Returns the associated ostream
     virtual std::ostream& getOStream() = 0;
@@ -345,7 +345,8 @@ private:
     /// @brief The formatter for XML
     OutputFormatter* myFormatter;
 
-    const bool myAmBinary;
+protected:
+    std::string myFilename;
 
 public:
     /// @brief Invalidated copy constructor.
@@ -357,9 +358,3 @@ private:
     OutputDevice& operator=(const OutputDevice&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
