@@ -1,46 +1,33 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    OutputDevice_CERR.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    2004
-/// @version $Id: OutputDevice_CERR.cpp 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // An output device that encapsulates cout
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include "OutputDevice_CERR.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // static member definitions
 // ===========================================================================
-OutputDevice* OutputDevice_CERR::myInstance = 0;
+OutputDevice* OutputDevice_CERR::myInstance = nullptr;
 
 
 // ===========================================================================
@@ -49,7 +36,7 @@ OutputDevice* OutputDevice_CERR::myInstance = 0;
 OutputDevice*
 OutputDevice_CERR::getDevice() {
     // check whether the device has already been aqcuired
-    if (myInstance == 0) {
+    if (myInstance == nullptr) {
         myInstance = new OutputDevice_CERR();
     }
     return myInstance;
@@ -59,11 +46,13 @@ OutputDevice_CERR::getDevice() {
 // ===========================================================================
 // method definitions
 // ===========================================================================
-OutputDevice_CERR::OutputDevice_CERR() {}
+OutputDevice_CERR::OutputDevice_CERR() {
+    myFilename = "CERR";
+}
 
 
 OutputDevice_CERR::~OutputDevice_CERR() {
-    myInstance = 0;
+    myInstance = nullptr;
 }
 
 
