@@ -1,37 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    OutputDevice_File.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    2004
-/// @version $Id: OutputDevice_File.h 22608 2017-01-17 06:28:54Z behrisch $
 ///
 // An output device that encapsulates an ofstream
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef OutputDevice_File_h
-#define OutputDevice_File_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
-#include <fstream>
+#include <iostream>
 #include "OutputDevice.h"
 
 
@@ -48,10 +38,11 @@
 class OutputDevice_File : public OutputDevice {
 public:
     /** @brief Constructor
-     * @param[in] strm The output stream to used
+     * @param[in] fullName The name of the output file to use
+     * @param[in] compressed whether to apply gzip compression
      * @exception IOError Should not be thrown by this implementation
      */
-    OutputDevice_File(const std::string& fullName, const bool binary);
+    OutputDevice_File(const std::string& fullName, const bool compressed = false);
 
 
     /// @brief Destructor
@@ -71,12 +62,6 @@ protected:
 
 private:
     /// The wrapped ofstream
-    std::ofstream* myFileStream;
+    std::ostream* myFileStream;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
