@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    OutputFormatter.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    2012
-/// @version $Id: OutputFormatter.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Abstract base class for output formatters
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef OutputFormatter_h
-#define OutputFormatter_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -70,13 +60,11 @@ public:
      * @param[in] into The output stream to use
      * @param[in] rootElement The root element to use
      * @param[in] attrs Additional attributes to save within the rootElement
-     * @param[in] comment Additional comment (saved in front the rootElement)
      * @todo Check which parameter is used herein
      * @todo Describe what is saved
      */
     virtual bool writeXMLHeader(std::ostream& into, const std::string& rootElement,
-                                const std::string& attrs = "",
-                                const std::string& comment = "") = 0;
+                                const std::map<SumoXMLAttr, std::string>& attrs) = 0;
 
 
     /** @brief Opens an XML tag
@@ -102,20 +90,16 @@ public:
     virtual void openTag(std::ostream& into, const SumoXMLTag& xmlElement) = 0;
 
 
-    /** @brief Closes the most recently opened tag
+    /** @brief Closes the most recently opened tag and optinally add a comment
      *
      * @param[in] into The output stream to use
      * @return Whether a further element existed in the stack and could be closed
      * @todo it is not verified that the topmost element was closed
      */
-    virtual bool closeTag(std::ostream& into) = 0;
+    virtual bool closeTag(std::ostream& into, const std::string& comment = "") = 0;
 
     virtual void writePreformattedTag(std::ostream& into, const std::string& val) = 0;
 
+    virtual void writePadding(std::ostream& into, const std::string& val) = 0;
+
 };
-
-
-#endif
-
-/****************************************************************************/
-
