@@ -1,33 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    PlainXMLFormatter.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    2012
-/// @version $Id: PlainXMLFormatter.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // Static storage of an output device and its base (abstract) implementation
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <utils/common/ToString.h>
 #include <utils/options/OptionsCont.h>
@@ -88,14 +79,14 @@ PlainXMLFormatter::openTag(std::ostream& into, const SumoXMLTag& xmlElement) {
 
 
 bool
-PlainXMLFormatter::closeTag(std::ostream& into) {
+PlainXMLFormatter::closeTag(std::ostream& into, const std::string& comment) {
     if (!myXMLStack.empty()) {
         if (myHavePendingOpener) {
-            into << "/>\n";
+            into << "/>" << comment << "\n";
             myHavePendingOpener = false;
         } else {
             const std::string indent(4 * (myXMLStack.size() + myDefaultIndentation - 1), ' ');
-            into << indent << "</" << myXMLStack.back() << ">\n";
+            into << indent << "</" << myXMLStack.back() << ">" << comment << "\n";
         }
         myXMLStack.pop_back();
         return true;
@@ -113,5 +104,10 @@ PlainXMLFormatter::writePreformattedTag(std::ostream& into, const std::string& v
     into << val;
 }
 
-/****************************************************************************/
+void
+PlainXMLFormatter::writePadding(std::ostream& into, const std::string& val) {
+    into << val;
+}
 
+
+/****************************************************************************/
