@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    Option.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: Option.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // Classes representing a single program option (with different types)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef Option_h
-#define Option_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -46,7 +36,11 @@
  * @brief Definition of a vector of ints
  */
 typedef std::vector<int> IntVector;
-
+/**
+ * @typedef StringVector
+ * @brief Definition of a vector of strings
+ */
+typedef std::vector<std::string> StringVector;
 
 /* -------------------------------------------------------------------------
  * Option
@@ -117,11 +111,12 @@ public:
 
     /** @brief Returns the stored string value
      *
-     * Option_String and Option_FileName return the stored string in this method's reimplementation.
+     * Option_String returns the stored string in this method's reimplementation.
+     *  Option_FileName's reimplementation is only to be used for single filename string-vector options.
      *  All other option classes do not override this method which throws an InvalidArgument-exception.
      *
-     * @return Returns the stored string if being an instance of Option_String
-     * @exception InvalidArgument If the class is not an instance of Option_String
+     * @return Returns the stored string if being an instance of Option_String or Option_FileName
+     * @exception InvalidArgument If the class is not an instance of Option_String or Option_FileName
      */
     virtual std::string getString() const;
 
@@ -147,6 +142,15 @@ public:
      */
     virtual const IntVector& getIntVector() const;
 
+    /** @brief Returns the stored string vector
+     *
+     * Option_StringVector returns the stored string vector in this method's reimplementation.
+     *  All other option classes do not override this method which throws an InvalidArgument-exception.
+     *
+     * @return Returns the stored string vector if being an instance of Option_StringVector
+     * @exception InvalidArgument If the class is not an instance of Option_StringVector
+     */
+    virtual const StringVector& getStringVector() const;
 
     /** @brief Stores the given value
      *
@@ -253,6 +257,18 @@ public:
     virtual const std::string& getTypeName() const;
 
 
+    /** @brief Create a new Option of the given type with given default value but make it unset
+     *  @note Was implemented to allow warning once if user didn't set the option, refs. #4567
+     *  @see MSDeviceSSM::insertOptions()
+     */
+    template<class OptionType, class ValueType>
+    static OptionType* makeUnsetWithDefault(ValueType def) {
+        OptionType* o = new OptionType(def);
+        o->unSet();
+        return o;
+    }
+
+
 protected:
     /** @brief Marks the information as set
      *
@@ -546,6 +562,65 @@ public:
      */
     bool getBool() const;
 
+    /** sets the given value (converts it to bool) */
+    virtual bool set(const std::string& v);
+
+
+    /** @brief Returns the string-representation of the value
+     *
+     * If myValue is true, "true" is returned, "false" otherwise.
+     *
+     * @see std::string Option::getValueString()
+     * @return The stored value encoded into a string
+     */
+    virtual std::string getValueString() const;
+
+
+    /** @brief Returns true, the information whether the option is a bool option
+     *
+     * Returns true.
+     *
+     * @see bool Option::isBool()
+     * @return true
+     */
+    bool isBool() const;
+
+
+protected:
+    /** the value, valid only when the base-classes "myAmSet"-member is true */
+    bool        myValue;
+
+};
+
+
+
+/* -------------------------------------------------------------------------
+ * Option_BoolExtended
+ * ----------------------------------------------------------------------- */
+class Option_BoolExtended : public Option_Bool {
+public:
+    /** @brief Constructor for an option that can be used without an argument
+     * like Option_BoolExtended but which also handles value strings
+     *
+     * Calls Option(true)
+     *
+     * @param[in] value This option's default value
+     */
+    Option_BoolExtended(bool value);
+
+
+    /** @brief Copy constructor */
+    Option_BoolExtended(const Option_BoolExtended& s);
+
+
+    /** @brief Destructor */
+    ~Option_BoolExtended();
+
+
+    /** @brief Assignment operator */
+    Option_BoolExtended& operator=(const Option_BoolExtended& s);
+
+
     /** sets the given value (converts it to bool) */
     bool set(const std::string& v);
 
@@ -560,69 +635,9 @@ public:
     std::string getValueString() const;
 
 
-    /** @brief Returns true, the information whether the option is a bool option
-     *
-     * Returns true.
-     *
-     * @see bool Option::isBool()
-     * @return true
-     */
-    bool isBool() const;
-
-
 private:
     /** the value, valid only when the base-classes "myAmSet"-member is true */
-    bool        myValue;
-
-};
-
-
-/* -------------------------------------------------------------------------
- * Option_FileName
- * ----------------------------------------------------------------------- */
-class Option_FileName : public Option_String {
-public:
-    /** @brief Constructor for an option with no default value
-     */
-    Option_FileName();
-
-
-    /** @brief Constructor for an option with a default value
-     *
-     * @param[in] value This option's default value
-     */
-    Option_FileName(const std::string& value);
-
-
-    /** @brief Copy constructor */
-    Option_FileName(const Option_String& s);
-
-
-    /** @brief Destructor */
-    virtual ~Option_FileName();
-
-    /** @brief Assignment operator */
-    Option_FileName& operator=(const Option_FileName& s);
-
-
-    /** @brief Returns true, the information whether this option is a file name
-     *
-     * Returns true.
-     *
-     * @return true
-     */
-    bool isFileName() const;
-
-
-    /** @brief Returns the string-representation of the value
-     *
-     * The value is URL-encoded using StringUtils::urlEncode and returned.
-     *
-     * @see std::string Option::getValueString()
-     * @return The stored value encoded into a string
-     */
-    std::string getValueString() const;
-
+    std::string myValueString;
 
 };
 
@@ -697,7 +712,117 @@ private:
 };
 
 
-#endif
+/* -------------------------------------------------------------------------
+ * Option_StringVector
+ * ----------------------------------------------------------------------- */
+class Option_StringVector : public Option {
+public:
+    /** @brief Constructor for an option with no default value
+     */
+    Option_StringVector();
 
-/****************************************************************************/
+    /** @brief Constructor for an option with a default value
+     *
+     * @param[in] value This option's default value
+     */
+    Option_StringVector(const StringVector& value);
 
+    /** @brief Copy constructor */
+    Option_StringVector(const Option_StringVector& s);
+
+    /** @brief Destructor */
+    virtual ~Option_StringVector();
+
+    /** @brief Assignment operator */
+    Option_StringVector& operator=(const Option_StringVector& s);
+
+    /** @brief Returns the stored string vector
+     * @see const StringVector &Option::getStringVector()
+     * @return Returns the stored string vector
+     */
+    const StringVector& getStringVector() const;
+
+    /** @brief Stores the given value after parsing it into a vector of strings
+     *
+     *  The value is converted into a vector of strings and stored in "myValue".
+     *  Then, "markSet" is called in order to know that a value has been set.
+     *
+     * The method returns whether the value could be set (the return value from
+     *  "markSet").
+     *
+     * If the string could not be converted into a vector of strings, an
+     * InvalidArgument is thrown.
+     *
+     * @see bool Option::set(std::string v)
+     * @return Whether the new value could be set
+     * @exception InvalidArgument If the value could not be converted into a
+     * vector of strings
+     */
+    bool set(const std::string& v);
+
+    /** @brief Returns the string-representation of the value
+     *
+     * The stored value is encoded into a string and returned.
+     *
+     * @see std::string Option::getValueString()
+     * @return The stored value encoded into a string
+     */
+    std::string getValueString() const;
+
+private:
+    /** the value, valid only when the base-classes "myAmSet"-member is true */
+    StringVector myValue;
+};
+
+
+/* -------------------------------------------------------------------------
+ * Option_FileName
+ * ----------------------------------------------------------------------- */
+class Option_FileName : public Option_StringVector {
+public:
+    /** @brief Constructor for an option with no default value
+     */
+    Option_FileName();
+
+    /** @brief Constructor for an option with a default value
+     *
+     * @param[in] value This option's default value
+     */
+    Option_FileName(const StringVector& value);
+
+    /** @brief Copy constructor */
+    Option_FileName(const Option_FileName& s);
+
+    /** @brief Destructor */
+    virtual ~Option_FileName();
+
+    /** @brief Assignment operator */
+    Option_FileName& operator=(const Option_FileName& s);
+
+    /** @brief Returns true, the information whether this option is a file name
+     *
+     * Returns true.
+     *
+     * @return true
+     */
+    bool isFileName() const;
+
+    /** @brief Legacy method that returns the stored filenames as a comma-separated string.
+     *
+     * @see std::string Option::getString()
+     * @see std::string StringVector::getValueString()
+     * @return Returns comma-separated string of the stored filenames
+     * @deprecated Legacy method used when Option_FileName was still derived from Option_String;
+     * not in line with code style of the Options sub-system.
+     */
+    std::string getString() const;
+
+    /** @brief Returns the string-representation of the value
+     *
+     * The value is URL-encoded using StringUtils::urlEncode and returned.
+     *
+     * @see std::string Option::getValueString()
+     * @return The stored value encoded into a string
+     */
+    std::string getValueString() const;
+};
