@@ -1,33 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    OptionsCont.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: OptionsCont.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A storage for options (typed value containers)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <map>
 #include <string>
@@ -68,8 +61,8 @@ OptionsCont::getOptions() {
 
 
 OptionsCont::OptionsCont()
-    : myAddresses(), myValues(), myDeprecatedSynonymes(), myHaveInformedAboutDeprecatedDivider(false) {
-    myCopyrightNotices.push_back("Copyright (C) 2001-2017 DLR and contributors; http://sumo.dlr.de");
+    : myAddresses(), myValues(), myDeprecatedSynonymes() {
+    myCopyrightNotices.push_back("Copyright (C) 2001-2021 German Aerospace Center (DLR) and others; https://sumo.dlr.de");
 }
 
 
@@ -81,7 +74,7 @@ OptionsCont::~OptionsCont() {
 void
 OptionsCont::doRegister(const std::string& name, Option* v) {
     assert(v != 0);
-    ItemAddressContType::iterator i = find(myAddresses.begin(), myAddresses.end(), v);
+    ItemAddressContType::iterator i = std::find(myAddresses.begin(), myAddresses.end(), v);
     if (i == myAddresses.end()) {
         myAddresses.push_back(v);
     }
@@ -205,6 +198,13 @@ OptionsCont::getSecure(const std::string& name) const {
 }
 
 
+std::string
+OptionsCont::getValueString(const std::string& name) const {
+    Option* o = getSecure(name);
+    return o->getValueString();
+}
+
+
 std::string
 OptionsCont::getString(const std::string& name) const {
     Option* o = getSecure(name);
@@ -239,6 +239,11 @@ OptionsCont::getIntVector(const std::string& name) const {
     return o->getIntVector();
 }
 
+const StringVector&
+OptionsCont::getStringVector(const std::string& name) const {
+    Option* o = getSecure(name);
+    return o->getStringVector();
+}
 
 bool
 OptionsCont::set(const std::string& name, const std::string& value) {
@@ -294,13 +299,19 @@ OptionsCont::getSynonymes(const std::string& name) const {
 }
 
 
+const std::string&
+OptionsCont::getDescription(const std::string& name) const {
+    return getSecure(name)->getDescription();
+}
+
+
 std::ostream&
 operator<<(std::ostream& os, const OptionsCont& oc) {
     std::vector<std::string> done;
     os << "Options set:" << std::endl;
     for (OptionsCont::KnownContType::const_iterator i = oc.myValues.begin();
             i != oc.myValues.end(); i++) {
-        std::vector<std::string>::iterator j = find(done.begin(), done.end(), (*i).first);
+        std::vector<std::string>::iterator j = std::find(done.begin(), done.end(), (*i).first);
         if (j == done.end()) {
             std::vector<std::string> synonymes = oc.getSynonymes((*i).first);
             if (synonymes.size() != 0) {
@@ -330,30 +341,23 @@ operator<<(std::ostream& os, const OptionsCont& oc) {
 
 void
 OptionsCont::relocateFiles(const std::string& configuration) const {
-    for (ItemAddressContType::const_iterator i = myAddresses.begin(); i != myAddresses.end(); i++) {
-        if ((*i)->isFileName() && (*i)->isSet()) {
-            StringTokenizer st((*i)->getString(), ";, ", true);
-            std::string conv;
-            while (st.hasNext()) {
-                std::string tmp = st.next();
-                // Test whether this is a whitespace string and disregard item if so.
-                // This may stem, e.g., from separating filenames by ', ' in the configuration file
-                if (tmp.find_first_not_of("\t ") == std::string::npos) {
-                    continue;
+    for (Option* const option : myAddresses) {
+        if (option->isFileName() && option->isSet()) {
+            StringVector fileList = StringVector(option->getStringVector());
+            for (std::string& f : fileList) {
+                f = FileHelpers::checkForRelativity(f, configuration);
+                try {
+                    f = StringUtils::urlDecode(f);
+                } catch (NumberFormatException& e) {
+                    WRITE_WARNING(toString(e.what()) + " when trying to decode filename '" + f + "'.");
                 }
-                if (!FileHelpers::isAbsolute(tmp)) {
-                    tmp = FileHelpers::getConfigurationRelative(configuration, tmp);
-                }
-                if (conv.length() != 0) {
-                    conv += ',';
-                }
-                conv += StringUtils::urlDecode(tmp);
             }
-            if (conv != (*i)->getString()) {
-                const bool hadDefault = (*i)->isDefault();
-                (*i)->set(conv);
+            const std::string conv = joinToString(fileList, ',');
+            if (conv != joinToString(option->getStringVector(), ',')) {
+                const bool hadDefault = option->isDefault();
+                option->set(conv);
                 if (hadDefault) {
-                    (*i)->resetDefault();
+                    option->resetDefault();
                 }
             }
         }
@@ -465,7 +469,7 @@ void
 OptionsCont::clear() {
     ItemAddressContType::iterator i;
     for (i = myAddresses.begin(); i != myAddresses.end(); i++) {
-        delete(*i);
+        delete (*i);
     }
     myAddresses.clear();
     myValues.clear();
@@ -544,7 +548,7 @@ OptionsCont::splitLines(std::ostream& os, std::string what,
             }
             if (splitPos != std::string::npos) {
                 os << what.substr(0, splitPos) << std::endl;
-                what = what.substr(splitPos);
+                what = what.substr(splitPos + 1);
                 for (int r = 0; r < nextOffset + 1; ++r) {
                     os << ' ';
                 }
@@ -572,11 +576,12 @@ OptionsCont::processMetaOptions(bool missingOptions) {
                     myCopyrightNotices.begin(); it != myCopyrightNotices.end(); ++it) {
             std::cout << " " << *it << std::endl;
         }
-        std::cout << " License GPLv3+: GNU GPL Version 3 or later <http://gnu.org/licenses/gpl.html>\n";
+        std::cout << " License EPL-2.0: Eclipse Public License Version 2 <https://eclipse.org/legal/epl-v20.html>\n";
         std::cout << " Use --help to get the list of options." << std::endl;
         return true;
     }
 
+    myWriteLicense = getBool("write-license");
     // check whether the help shall be printed
     if (getBool("help")) {
         std::cout << myFullName << std::endl;
@@ -596,16 +601,11 @@ OptionsCont::processMetaOptions(bool missingOptions) {
             std::cout << " " << *it << std::endl;
         }
         std::cout << "\n" << myFullName << " is part of SUMO.\n";
-        std::cout << "SUMO is free software: you can redistribute it and/or modify\n";
-        std::cout << "it under the terms of the GNU General Public License as published by\n";
-        std::cout << "the Free Software Foundation, either version 3 of the License, or\n";
-        std::cout << "(at your option) any later version.\n\n";
-        std::cout << "This program is distributed in the hope that it will be useful,\n";
-        std::cout << "but WITHOUT ANY WARRANTY; without even the implied warranty of\n";
-        std::cout << "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n";
-        std::cout << "GNU General Public License for more details.\n\n";
-        std::cout << "You should have received a copy of the GNU General Public License\n";
-        std::cout << "along with this program. If not, see http://www.gnu.org/licenses/gpl.html" << std::endl;
+        std::cout << "This program and the accompanying materials\n";
+        std::cout << "are made available under the terms of the Eclipse Public License v2.0\n";
+        std::cout << "which accompanies this distribution, and is available at\n";
+        std::cout << "http://www.eclipse.org/legal/epl-v20.html\n";
+        std::cout << "SPDX-License-Identifier: EPL-2.0" << std::endl;
         return true;
     }
     // check whether the settings shall be printed
@@ -649,14 +649,14 @@ OptionsCont::processMetaOptions(bool missingOptions) {
     }
     if (isSet("save-schema", false)) { // sumo-gui does not register these
         if (getString("save-schema") == "-" || getString("save-schema") == "stdout") {
-            writeSchema(std::cout, getBool("save-commented"));
+            writeSchema(std::cout);
             return true;
         }
         std::ofstream out(getString("save-schema").c_str());
         if (!out.good()) {
             throw ProcessError("Could not save schema to '" + getString("save-schema") + "'");
         } else {
-            writeSchema(out, getBool("save-commented"));
+            writeSchema(out);
             if (getBool("verbose")) {
                 WRITE_MESSAGE("Written schema to '" + getString("save-schema") + "'");
             }
@@ -672,14 +672,8 @@ OptionsCont::printHelp(std::ostream& os) {
     // print application description
     splitLines(os, myAppDescription, 0, 0);
     os << std::endl;
-    // print usage BNF
-    os << "Usage: " << myAppName << " [OPTION]*" << std::endl;
-    // print additional text if any
-    if (myAdditionalMessage.length() > 0) {
-        os << myAdditionalMessage << std::endl << ' ' << std::endl;
-    }
-    // print the options
-    // check their sizes first
+
+    // check option sizes first
     //  we want to know how large the largest not-too-large-entry will be
     int tooLarge = 40;
     int maxSize = 0;
@@ -690,9 +684,12 @@ OptionsCont::printHelp(std::ostream& os) {
             // name, two leading spaces and "--"
             int csize = (int)j->length() + 2 + 4;
             // abbreviation length ("-X, "->4chars) if any
-            std::vector<std::string> synonymes = getSynonymes(*j);
-            if (find_if(synonymes.begin(), synonymes.end(), abbreviation_finder()) != synonymes.end()) {
-                csize += 4;
+            const std::vector<std::string> synonymes = getSynonymes(*j);
+            for (std::vector<std::string>::const_iterator s = synonymes.begin(); s != synonymes.end(); ++s) {
+                if (s->length() == 1 && myDeprecatedSynonymes.count(*s) == 0) {
+                    csize += 4;
+                    break;
+                }
             }
             // the type name
             if (!o->isBool()) {
@@ -706,41 +703,33 @@ OptionsCont::printHelp(std::ostream& os) {
         }
     }
 
-    for (i = mySubTopics.begin(); i != mySubTopics.end(); ++i) {
-        os << *i << " Options:" << std::endl;
-        const std::vector<std::string>& entries = mySubTopicEntries[*i];
-        for (j = entries.begin(); j != entries.end(); ++j) {
-            // start length computation
-            int csize = (int)j->length() + 2;
-            Option* o = getSecure(*j);
-            os << "  ";
-            // write abbreviation if given
-            std::vector<std::string> synonymes = getSynonymes(*j);
-            std::vector<std::string>::iterator a = find_if(synonymes.begin(), synonymes.end(), abbreviation_finder());
-            if (a != synonymes.end()) {
-                os << '-' << (*a) << ", ";
-                csize += 4;
+    const std::string helpTopic = StringUtils::to_lower_case(getSecure("help")->getValueString());
+    if (helpTopic != "") {
+        bool foundTopic = false;
+        for (const std::string& topic : mySubTopics) {
+            if (StringUtils::to_lower_case(topic).find(helpTopic) != std::string::npos) {
+                foundTopic = true;
+                printHelpOnTopic(topic, tooLarge, maxSize, os);
             }
-            // write leading '-'/"--"
-            os << "--";
-            csize += 2;
-            // write the name
-            os << *j;
-            // write the type if not a bool option
-            if (!o->isBool()) {
-                os << ' ' << o->getTypeName();
-                csize += 1 + (int)o->getTypeName().length();
-            }
-            csize += 2;
-            // write the description formatting it
-            os << "  ";
-            for (int r = maxSize; r > csize; --r) {
-                os << ' ';
-            }
-            int offset = csize > tooLarge ? csize : maxSize;
-            splitLines(os, o->getDescription(), offset, maxSize);
         }
-        os << std::endl;
+        if (!foundTopic) {
+            // print topic list
+            os << "Help Topics:"  << std::endl;
+            for (std::string t : mySubTopics) {
+                os << "    " << t << std::endl;
+            }
+        }
+        return;
+    }
+    // print usage BNF
+    os << "Usage: " << myAppName << " [OPTION]*" << std::endl;
+    // print additional text if any
+    if (myAdditionalMessage.length() > 0) {
+        os << myAdditionalMessage << std::endl << ' ' << std::endl;
+    }
+    // print the options
+    for (i = mySubTopics.begin(); i != mySubTopics.end(); ++i) {
+        printHelpOnTopic(*i, tooLarge, maxSize, os);
     }
     os << std::endl;
     // print usage examples, calc size first
@@ -752,16 +741,56 @@ OptionsCont::printHelp(std::ostream& os) {
         }
     }
     os << std::endl;
-    os << "Report bugs at <http://sumo.dlr.de/trac/>." << std::endl;
+    os << "Report bugs at <https://github.com/eclipse/sumo/issues>." << std::endl;
     os << "Get in contact via <sumo@dlr.de>." << std::endl;
 }
 
+void
+OptionsCont::printHelpOnTopic(const std::string& topic, int tooLarge, int maxSize, std::ostream& os) {
+    os << topic << " Options:" << std::endl;
+    for (std::string entry : mySubTopicEntries[topic]) {
+        // start length computation
+        int csize = (int)entry.length() + 2;
+        Option* o = getSecure(entry);
+        os << "  ";
+        // write abbreviation if given
+        std::vector<std::string> synonymes = getSynonymes(entry);
+        for (std::vector<std::string>::const_iterator s = synonymes.begin(); s != synonymes.end(); ++s) {
+            if (s->length() == 1 && myDeprecatedSynonymes.count(*s) == 0) {
+                os << '-' << *s << ", ";
+                csize += 4;
+                break;
+            }
+        }
+        // write leading '-'/"--"
+        os << "--";
+        csize += 2;
+        // write the name
+        os << entry;
+        // write the type if not a bool option
+        if (!o->isBool()) {
+            os << ' ' << o->getTypeName();
+            csize += 1 + (int)o->getTypeName().length();
+        }
+        csize += 2;
+        // write the description formatting it
+        os << "  ";
+        for (int r = maxSize; r > csize; --r) {
+            os << ' ';
+        }
+        int offset = csize > tooLarge ? csize : maxSize;
+        splitLines(os, o->getDescription(), offset, maxSize);
+    }
+    os << std::endl;
+}
 
 void
 OptionsCont::writeConfiguration(std::ostream& os, const bool filled,
                                 const bool complete, const bool addComments,
-                                const bool maskDoubleHyphen) const {
-    os << "<?xml version=\"1.0\"" << SUMOSAXAttributes::ENCODING << "?>\n\n";
+                                const bool inComment) const {
+    if (!inComment) {
+        writeXMLHeader(os, false);
+    }
     os << "<configuration xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/";
     if (myAppName == "sumo-gui") {
         os << "sumo";
@@ -780,26 +809,29 @@ OptionsCont::writeConfiguration(std::ostream& os, const bool filled,
         std::transform(subtopic.begin(), subtopic.end(), subtopic.begin(), tolower);
         const std::vector<std::string>& entries = mySubTopicEntries.find(*i)->second;
         bool hadOne = false;
-        for (std::vector<std::string>::const_iterator j = entries.begin(); j != entries.end(); ++j) {
-            Option* o = getSecure(*j);
+        for (const std::string& name : entries) {
+            Option* o = getSecure(name);
             bool write = complete || (filled && !o->isDefault());
             if (!write) {
                 continue;
             }
+            if (name == "registry-viewport" && !complete) {
+                continue;
+            }
             if (!hadOne) {
                 os << "    <" << subtopic << ">" << std::endl;
             }
             // add the comment if wished
             if (addComments) {
-                os << "        <!-- " << StringUtils::escapeXML(o->getDescription(), maskDoubleHyphen) << " -->" << std::endl;
+                os << "        <!-- " << StringUtils::escapeXML(o->getDescription(), inComment) << " -->" << std::endl;
             }
             // write the option and the value (if given)
-            os << "        <" << *j << " value=\"";
+            os << "        <" << name << " value=\"";
             if (o->isSet() && (filled || o->isDefault())) {
-                os << StringUtils::escapeXML(o->getValueString(), maskDoubleHyphen);
+                os << StringUtils::escapeXML(o->getValueString(), inComment);
             }
             if (complete) {
-                std::vector<std::string> synonymes = getSynonymes(*j);
+                std::vector<std::string> synonymes = getSynonymes(name);
                 if (!synonymes.empty()) {
                     os << "\" synonymes=\"";
                     for (std::vector<std::string>::const_iterator s = synonymes.begin(); s != synonymes.end(); ++s) {
@@ -830,8 +862,8 @@ OptionsCont::writeConfiguration(std::ostream& os, const bool filled,
 
 
 void
-OptionsCont::writeSchema(std::ostream& os, bool /* addComments */) {
-    os << "<?xml version=\"1.0\"" << SUMOSAXAttributes::ENCODING << "?>\n\n";
+OptionsCont::writeSchema(std::ostream& os) {
+    writeXMLHeader(os, false);
     os << "<xsd:schema elementFormDefault=\"qualified\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n\n";
     os << "    <xsd:include schemaLocation=\"baseTypes.xsd\"/>\n";
     os << "    <xsd:element name=\"configuration\" type=\"configurationType\"/>\n\n";
@@ -844,7 +876,7 @@ OptionsCont::writeSchema(std::ostream& os, bool /* addComments */) {
         }
         std::replace(subtopic.begin(), subtopic.end(), ' ', '_');
         std::transform(subtopic.begin(), subtopic.end(), subtopic.begin(), tolower);
-        os << "            <xsd:element name=\"" << subtopic << "\" type=\"" << subtopic << "Type\" minOccurs=\"0\"/>\n";
+        os << "            <xsd:element name=\"" << subtopic << "\" type=\"" << subtopic << "TopicType\" minOccurs=\"0\"/>\n";
     }
     os << "        </xsd:all>\n";
     os << "    </xsd:complexType>\n\n";
@@ -855,7 +887,7 @@ OptionsCont::writeSchema(std::ostream& os, bool /* addComments */) {
         }
         std::replace(subtopic.begin(), subtopic.end(), ' ', '_');
         std::transform(subtopic.begin(), subtopic.end(), subtopic.begin(), tolower);
-        os << "    <xsd:complexType name=\"" << subtopic << "Type\">\n";
+        os << "    <xsd:complexType name=\"" << subtopic << "TopicType\">\n";
         os << "        <xsd:all>\n";
         const std::vector<std::string>& entries = mySubTopicEntries[*i];
         for (std::vector<std::string>::const_iterator j = entries.begin(); j != entries.end(); ++j) {
@@ -865,6 +897,9 @@ OptionsCont::writeSchema(std::ostream& os, bool /* addComments */) {
             if (type == "int[]") {
                 type = "intArray";
             }
+            if (type == "str[]") {
+                type = "strArray";
+            }
             os << "            <xsd:element name=\"" << *j << "\" type=\"" << type << "OptionType\" minOccurs=\"0\"/>\n";
         }
         os << "        </xsd:all>\n";
@@ -875,42 +910,34 @@ OptionsCont::writeSchema(std::ostream& os, bool /* addComments */) {
 
 
 void
-OptionsCont::writeXMLHeader(std::ostream& os) {
+OptionsCont::writeXMLHeader(std::ostream& os, const bool includeConfig) const {
     time_t rawtime;
     char buffer [80];
 
     os << "<?xml version=\"1.0\"" << SUMOSAXAttributes::ENCODING << "?>\n\n";
     time(&rawtime);
-    strftime(buffer, 80, "<!-- generated on %c by ", localtime(&rawtime));
+    strftime(buffer, 80, "<!-- generated on %F %T by ", localtime(&rawtime));
     os << buffer << myFullName << "\n";
-    writeConfiguration(os, true, false, false, true);
+    if (myWriteLicense) {
+        os << "This data file and the accompanying materials\n";
+        os << "are made available under the terms of the Eclipse Public License v2.0\n";
+        os << "which accompanies this distribution, and is available at\n";
+        os << "http://www.eclipse.org/legal/epl-v20.html\n";
+        os << "SPDX-License-Identifier: EPL-2.0\n";
+    }
+    if (includeConfig) {
+        writeConfiguration(os, true, false, false, true);
+    }
     os << "-->\n\n";
 }
 
 
-std::vector<std::string>
-OptionsCont::getStringVector(const std::string& name) const {
-    Option* o = getSecure(name);
-    std::string def = o->getString();
-    if (def.find(';') != std::string::npos && !myHaveInformedAboutDeprecatedDivider) {
-        WRITE_WARNING("Please note that using ';' as list separator is deprecated.\n From 1.0 onwards, only ',' will be accepted.");
-        myHaveInformedAboutDeprecatedDivider = true;
-    }
-    StringTokenizer st(def, ";,", true);
-    std::vector<std::string> ret = st.getVector();
-    for (std::vector<std::string>::iterator i = ret.begin(); i != ret.end(); ++i) {
-        (*i) = StringUtils::prune(*i);
-    }
-    return ret;
-}
-
-
 bool
 OptionsCont::isInStringVector(const std::string& optionName,
-                              const std::string& itemName) {
+                              const std::string& itemName) const {
     if (isSet(optionName)) {
         std::vector<std::string> values = getStringVector(optionName);
-        return find(values.begin(), values.end(), itemName) != values.end();
+        return std::find(values.begin(), values.end(), itemName) != values.end();
     }
     return false;
 }
