@@ -1,35 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    OptionsCont.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: OptionsCont.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // A storage for options (typed value containers)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef OptionsCont_h
-#define OptionsCont_h
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <map>
 #include <string>
@@ -93,8 +85,6 @@
  *  subtopic the option belongs to must be given to OptionsCont. This is
  *  done using addDescription(<OPTION_NAME>, <SUBTOPIC>, <DESCRIPTION>).
  * @see addDescription
- *
- * @see http://sumo.dlr.de/wiki/index.php/OptionsSubSystem
  */
 class OptionsCont {
 public:
@@ -173,6 +163,12 @@ public:
      */
     void printHelp(std::ostream& os);
 
+    /** @brief Prints help on the given topic
+     *
+     * @param[in] topic The topic name
+     * @param[in] os The stream to write the help into
+     */
+    void printHelpOnTopic(const std::string& topic, int tooLarge, int maxSize, std::ostream& os);
 
     /** @brief Writes the configuration
      *
@@ -183,11 +179,11 @@ public:
      * @param[in] filled Whether only set (and not default) options shall be written
      * @param[in] complete Whether all options shall be written
      * @param[in] addComments Whether comments (option descriptions) shall be written
-     * @param[in] maskDoubleHyphen Whether -- in input shall be converted to &#45;&#45; (semantically equivalent but allowed in XML comments)
+     * @param[in] inComment Whether -- in input shall be converted to &#45;&#45; (semantically equivalent but allowed in XML comments)
      */
     void writeConfiguration(std::ostream& os, const bool filled,
                             const bool complete, const bool addComments,
-                            const bool maskDoubleHyphen = false) const;
+                            const bool inComment = false) const;
 
 
     /** @brief Writes the xml schema for the configuration
@@ -196,9 +192,8 @@ public:
      *  allowing to validate the configuration against.
      *
      * @param[in] os The stream to write the schema into
-     * @param[in] addComments Whether comments (option descriptions) shall be written
      */
-    void writeSchema(std::ostream& os, bool addComments);
+    void writeSchema(std::ostream& os);
 
 
     /** @brief Writes a standard XML header, including the configuration
@@ -209,7 +204,7 @@ public:
      *
      * @param[in] os The stream to write the header into
      */
-    void writeXMLHeader(std::ostream& os);
+    void writeXMLHeader(std::ostream& os, const bool includeConfig = true) const;
     /// @}
 
 
@@ -401,6 +396,14 @@ public:
      **/
     std::vector<std::string> getSynonymes(const std::string& name) const;
 
+    /** @brief Returns the option description
+     *
+     * @param[in] name The name of the option to return the description of
+     * @return description
+     * @exception InvalidArgument If the named option does not exist
+     **/
+    const std::string& getDescription(const std::string& name) const;
+
 
     /** @brief Returns the information whether the named option may be set
      *
@@ -421,6 +424,17 @@ public:
 
     /// @name Methods for retrieving values from options
     /// @{
+    /** @brief Returns the string-value of the named option (all options)
+     *
+     * This method returns the string-value of an existing option.
+     * If the named option does not exist, an
+     *  InvalidArgument is thrown.
+     *
+     * @param[in] name The name of the option to return the string-value of
+     * @return The string-value of the named, existing option
+     * @exception InvalidArgument If the option does not exist
+     */
+    std::string getValueString(const std::string& name) const;
 
     /** @brief Returns the string-value of the named option (only for Option_String)
      *
@@ -486,25 +500,22 @@ public:
      */
     const IntVector& getIntVector(const std::string& name) const;
 
-
-    /** @brief Returns the list of string-vector-value of the named option (only for Option_String)
+    /** @brief Returns the list of string-value of the named option (only for Option_StringVector)
      *
-     * This method returns the string-vector-value of an existing string-option.
-     * If the named option does not exist or is not a string-option, an
+     * This method returns the string-vector-value of an existing string-vector-option.
+     * If the named option does not exist or is not a string-vector-option, an
      *  InvalidArgument is thrown.
      *
-     * The string-value is determined, first. If the legacy-divider ';' is found
-     *  within the string, a warning is generated. The retrieved string is splitted
-     *  at ',' and ';'.
+     * If the legacy-divider ';' is found within the string, a warning is generated.
+     * The retrieved string is only splitted at ','.
      *
      * @param[in] name The name of the option to return the string-vector-value of
-     * @return The string-vector-value of the named, existing string-option
-     * @exception InvalidArgument If the option does not exist or is not a string-option
+     * @return The string-vector-value of the named, existing string-vector-option
+     * @exception InvalidArgument If the option does not exist or is not a string-vector-option
      * @todo Is it possible to retrieve a const-reference of the string?
      * @see getString()
      */
-    std::vector<std::string> getStringVector(const std::string& name) const;
-
+    const StringVector& getStringVector(const std::string& name) const;
 
     /** @brief Returns the named option is a list of string values containing the specified item
      *
@@ -524,7 +535,7 @@ public:
      * @todo Try to optimize - at each call, the vector is rebuilt
      */
     bool isInStringVector(const std::string& optionName,
-                          const std::string& itemName);
+                          const std::string& itemName) const;
     /// @}
 
 
@@ -739,29 +750,8 @@ private:
     /// A map from deprecated options to a bool indicating whether we warned about deprecation
     mutable std::map<std::string, bool> myDeprecatedSynonymes;
 
-    /// Information whether a warning a deprecated divider
-    mutable bool myHaveInformedAboutDeprecatedDivider;
-
-
-private:
-    /**
-     * @class abbreviation_finder
-     * @brief A class to find abbreviated option names (length=1)
-     */
-    class abbreviation_finder {
-    public:
-        /** constructor */
-        explicit abbreviation_finder() { }
-
-        /** @brief The comparing function
-         *
-         * @param[in] The option name
-         * @return Whether the length of the option name is 1
-         */
-        bool operator()(const std::string& s) {
-            return s.length() == 1;
-        }
-    };
+    /// Information whether we should always include license information in file headers
+    bool myWriteLicense;
 
 
 private:
@@ -772,9 +762,3 @@ private:
     OptionsCont& operator=(const OptionsCont& s);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
