@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    OptionsParser.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: OptionsParser.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Parses the command line arguments
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <iostream>
 #include <cstring>
@@ -38,10 +29,6 @@
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/MsgHandler.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -56,7 +43,7 @@ OptionsParser::parse(int argc, char** argv) {
             if (i < argc - 1) {
                 add = check(argv[i], argv[i + 1], ok);
             } else {
-                add = check(argv[i], 0, ok);
+                add = check(argv[i], nullptr, ok);
             }
             i += add;
         } catch (ProcessError& e) {
@@ -82,12 +69,12 @@ OptionsParser::check(const char* arg1, const char* arg2, bool& ok) {
     // process not abbreviated switches
     if (!isAbbreviation(arg1)) {
         std::string tmp(arg1 + 2);
-        size_t idx1 = tmp.find('=');
+        const std::string::size_type idx1 = tmp.find('=');
         // check whether a parameter was submitted
         if (idx1 != std::string::npos) {
             ok &= oc.set(tmp.substr(0, idx1), tmp.substr(idx1 + 1));
         } else {
-            if (arg2 == 0 || (oc.isBool(convert(arg1 + 2)) && arg2[0] == '-')) {
+            if (arg2 == nullptr || (oc.isBool(convert(arg1 + 2)) && arg2[0] == '-')) {
                 ok &= oc.set(convert(arg1 + 2), "true");
             } else {
                 ok &= oc.set(convert(arg1 + 2), convert(arg2));
@@ -100,7 +87,7 @@ OptionsParser::check(const char* arg1, const char* arg2, bool& ok) {
     for (int i = 1; arg1[i] != 0; i++) {
         // set boolean switches
         if (oc.isBool(convert(arg1[i]))) {
-            if (arg2 == 0 || arg2[0] == '-' || arg1[i + 1] != 0) {
+            if (arg2 == nullptr || arg2[0] == '-' || arg1[i + 1] != 0) {
                 ok &= oc.set(convert(arg1[i]), "true");
             } else {
                 ok &= oc.set(convert(arg1[i]), convert(arg2));
@@ -110,7 +97,7 @@ OptionsParser::check(const char* arg1, const char* arg2, bool& ok) {
         } else {
             // check whether the parameter comes directly after the switch
             //  and process if so
-            if (arg2 == 0 || arg1[i + 1] != 0) {
+            if (arg2 == nullptr || arg1[i + 1] != 0) {
                 ok &= processNonBooleanSingleSwitch(oc, arg1 + i);
                 return 1;
                 // process parameter following after a space
@@ -179,6 +166,4 @@ OptionsParser::convert(char abbr) {
 }
 
 
-
 /****************************************************************************/
-
