@@ -1,35 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    Shape.h
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Oct 2012
-/// @version $Id: Shape.h 23150 2017-02-27 12:08:30Z behrisch $
 ///
 // A 2D- or 3D-Shape
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef Shape_h
-#define Shape_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <utils/common/Named.h>
@@ -45,6 +35,19 @@
  */
 class Shape : public Named {
 public:
+    /// @nane default shape's values
+    /// @{
+    static const std::string DEFAULT_TYPE;
+    static const double DEFAULT_LAYER;
+    static const double DEFAULT_LINEWIDTH;
+    static const double DEFAULT_LAYER_POI;
+    static const double DEFAULT_ANGLE;
+    static const std::string DEFAULT_IMG_FILE;
+    static const bool DEFAULT_RELATIVEPATH;
+    static const double DEFAULT_IMG_WIDTH;
+    static const double DEFAULT_IMG_HEIGHT;
+    /// @}
+
     /** @brief Constructor
      * @param[in] id The name of the shape
      * @param[in] type The (abstract) type of the shape
@@ -52,55 +55,59 @@ public:
      * @param[in] layer The layer of the shape
      * @param[in] angle The rotation of the shape in navigational degrees
      * @param[in] imgFile The raster image of the shape
+     * @param[in] relativePath set image file as relative path
      */
     Shape(const std::string& id, const std::string& type,
           const RGBColor& color, double layer,
-          double angle, const std::string& imgFile);
-
+          double angle, const std::string& imgFile, bool relativePath);
 
     /// @brief Destructor
     virtual ~Shape();
 
-
     /// @name Getter
     /// @{
 
     /** @brief Returns the (abstract) type of the Shape
      * @return The Shape's (abstract) type
      */
-    inline const std::string& getType() const {
+    inline const std::string& getShapeType() const {
         return myType;
     }
 
-
     /** @brief Returns the color of the Shape
      * @return The Shape's color
      */
-    inline const RGBColor& getColor() const {
+    inline const RGBColor& getShapeColor() const {
         return myColor;
     }
 
-
     /** @brief Returns the layer of the Shape
      * @return The Shape's layer
      */
-    inline double getLayer() const {
+    inline double getShapeLayer() const {
         return myLayer;
     }
 
     /** @brief Returns the angle of the Shape in navigational degrees
      * @return The Shape's rotation angle
      */
-    inline double getNaviDegree() const {
+    inline double getShapeNaviDegree() const {
         return myNaviDegreeAngle;
     }
 
     /** @brief Returns the imgFile of the Shape
      * @return The Shape's rotation imgFile
      */
-    inline const std::string& getImgFile() const {
+    inline const std::string& getShapeImgFile() const {
         return myImgFile;
     }
+
+    /** @brief Returns the relativePath of the Shape
+    * @return The Shape's relativePath
+    */
+    inline bool getShapeRelativePath() const {
+        return myRelativePath;
+    }
     /// @}
 
 
@@ -110,50 +117,54 @@ public:
     /** @brief Sets a new type
      * @param[in] type The new type to use
      */
-    inline void setType(const std::string& type) {
+    inline void setShapeType(const std::string& type) {
         myType = type;
     }
 
-
     /** @brief Sets a new color
      * @param[in] col The new color to use
      */
-    inline void setColor(const RGBColor& col) {
+    inline void setShapeColor(const RGBColor& col) {
         myColor = col;
     }
 
+    /** @brief Sets a new alpha value
+     * @param[in] alpha The new value to use
+     */
+    inline void setShapeAlpha(unsigned char alpha) {
+        myColor.setAlpha(alpha);
+    }
 
     /** @brief Sets a new layer
      * @param[in] layer The new layer to use
      */
-    inline void setLayer(const double layer) {
+    inline void setShapeLayer(const double layer) {
         myLayer = layer;
     }
 
-
     /** @brief Sets a new angle in navigational degrees
      * @param[in] layer The new angle to use
      */
-    inline void setNaviDegree(const double angle) {
+    virtual void setShapeNaviDegree(const double angle) {
         myNaviDegreeAngle = angle;
     }
 
     /** @brief Sets a new imgFile
      * @param[in] imgFile The new imgFile to use
      */
-    inline void setImgFile(const std::string& imgFile) {
+    inline void setShapeImgFile(const std::string& imgFile) {
         myImgFile = imgFile;
     }
+
+    /** @brief Sets a new relativePath value
+    * @param[in] relativePath The new relative path to set
+    */
+    inline void setShapeRelativePath(bool relativePath) {
+        myRelativePath = relativePath;
+    }
     /// @}
 
-    static const std::string DEFAULT_TYPE;
-    static const double DEFAULT_LAYER;
-    static const double DEFAULT_ANGLE;
-    static const std::string DEFAULT_IMG_FILE;
-    static const double DEFAULT_IMG_WIDTH;
-    static const double DEFAULT_IMG_HEIGHT;
-
-protected:
+private:
     /// @brief The type of the Shape
     std::string myType;
 
@@ -166,12 +177,9 @@ protected:
     /// @brief The angle of the Shape
     double myNaviDegreeAngle;
 
-    /// @brief The angle of the Shape
+    /// @brief The img file (include path)
     std::string myImgFile;
+
+    /// @brief Enable or disable save imgFile as relative path
+    bool myRelativePath;
 };
-
-
-#endif
-
-/****************************************************************************/
-
