@@ -1,40 +1,31 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    GenericSAXHandler.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: GenericSAXHandler.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A handler which converts occuring elements and attributes into enums
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cassert>
 #include "GenericSAXHandler.h"
-#include <utils/common/TplConvert.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/FileHelpers.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/ToString.h>
@@ -48,8 +39,8 @@
 GenericSAXHandler::GenericSAXHandler(
     StringBijection<int>::Entry* tags, int terminatorTag,
     StringBijection<int>::Entry* attrs, int terminatorAttr,
-    const std::string& file)
-    : myParentHandler(0), myParentIndicator(SUMO_TAG_NOTHING), myFileName(file) {
+    const std::string& file, const std::string& expectedRoot)
+    : myParentHandler(nullptr), myParentIndicator(SUMO_TAG_NOTHING), myFileName(file), myExpectedRoot(expectedRoot), myRootSeen(false) {
     int i = 0;
     while (tags[i].key != terminatorTag) {
         myTagMap.insert(TagMap::value_type(tags[i].str, tags[i].key));
@@ -57,9 +48,14 @@ GenericSAXHandler::GenericSAXHandler(
     }
     i = 0;
     while (attrs[i].key != terminatorAttr) {
-        assert(myPredefinedTags.find(attrs[i].key) == myPredefinedTags.end());
-        myPredefinedTags[attrs[i].key] = convert(attrs[i].str);
-        myPredefinedTagsMML[attrs[i].key] = attrs[i].str;
+        int key = attrs[i].key;
+        assert(key >= 0);
+        while (key >= (int)myPredefinedTags.size()) {
+            myPredefinedTags.push_back(nullptr);
+            myPredefinedTagsMML.push_back("");
+        }
+        myPredefinedTags[key] = convert(attrs[i].str);
+        myPredefinedTagsMML[key] = attrs[i].str;
         i++;
     }
 }
@@ -67,7 +63,7 @@ GenericSAXHandler::GenericSAXHandler(
 
 GenericSAXHandler::~GenericSAXHandler() {
     for (AttrMap::iterator i1 = myPredefinedTags.begin(); i1 != myPredefinedTags.end(); i1++) {
-        delete[](*i1).second;
+        delete[](*i1);
     }
 }
 
@@ -102,7 +98,11 @@ GenericSAXHandler::startElement(const XMLCh* const /*uri*/,
                                 const XMLCh* const /*localname*/,
                                 const XMLCh* const qname,
                                 const XERCES_CPP_NAMESPACE::Attributes& attrs) {
-    std::string name = TplConvert::_2str(qname);
+    std::string name = StringUtils::transcode(qname);
+    if (!myRootSeen && myExpectedRoot != "" && name != myExpectedRoot) {
+        WRITE_WARNING("Found root element '" + name + "' in file '" + getFileName() + "' (expected '" + myExpectedRoot + "').");
+    }
+    myRootSeen = true;
     int element = convertTag(name);
     myCharactersVector.clear();
     SUMOSAXAttributesImpl_Xerces na(attrs, myPredefinedTags, myPredefinedTagsMML, name);
@@ -122,7 +122,7 @@ void
 GenericSAXHandler::endElement(const XMLCh* const /*uri*/,
                               const XMLCh* const /*localname*/,
                               const XMLCh* const qname) {
-    std::string name = TplConvert::_2str(qname);
+    std::string name = StringUtils::transcode(qname);
     int element = convertTag(name);
     // collect characters
     if (myCharactersVector.size() != 0) {
@@ -153,7 +153,7 @@ GenericSAXHandler::endElement(const XMLCh* const /*uri*/,
         if (myParentHandler && myParentIndicator == element) {
             XMLSubSys::setHandler(*myParentHandler);
             myParentIndicator = SUMO_TAG_NOTHING;
-            myParentHandler = 0;
+            myParentHandler = nullptr;
         }
     }
 }
@@ -170,7 +170,7 @@ GenericSAXHandler::registerParent(const int tag, GenericSAXHandler* handler) {
 void
 GenericSAXHandler::characters(const XMLCh* const chars,
                               const XERCES3_SIZE_t length) {
-    myCharactersVector.push_back(TplConvert::_2str(chars, (int)length));
+    myCharactersVector.push_back(StringUtils::transcode(chars, (int)length));
 }
 
 
@@ -228,4 +228,3 @@ GenericSAXHandler::myEndElement(int) {}
 
 
 /****************************************************************************/
-
