@@ -1,36 +1,26 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2007-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    SUMOSAXAttributes.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Fri, 30 Mar 2007
-/// @version $Id: SUMOSAXAttributes.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // Encapsulated SAX-Attributes
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2007-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef SUMOSAXAttributes_h
-#define SUMOSAXAttributes_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -84,7 +74,7 @@ public:
      * @param[in] objectid The name of the parsed object; used for error message generation
      * @param[out] ok Whether the value could be read
      * @param[in] report Whether errors shall be written to msg handler's error instance
-     * @return The read value if given and correct; -1 if an error occured
+     * @return The read value if given and correct; -1 if an error occurred
      */
     template <typename T>
     T get(int attr, const char* objectid, bool& ok, bool report = true) const;
@@ -103,7 +93,7 @@ public:
      * @param[out] ok Whether the value could be read
      * @param[in] defaultValue The value to return if the attribute is not within the element
      * @param[in] report Whether errors shall be written to msg handler's error instance
-     * @return The read value if given and correct; the default value if the attribute does not exist;  -1 if an error occured
+     * @return The read value if given and correct; the default value if the attribute does not exist;  -1 if an error occurred
      */
     template <typename T>
     T getOpt(int attr, const char* objectid, bool& ok, T defaultValue, bool report = true) const;
@@ -116,14 +106,14 @@ public:
      *
      * Otherwise, "ok" is not changed.
      *
-     * In dependence to the used time representation, either get<int> or get<SUMOReal>
+     * In dependence to the used time representation, either get<int> or get<double>
      *  is used.
      *
      * @param[in] attr The id of the attribute to read
      * @param[in] objectid The name of the parsed object; used for error message generation
      * @param[out] ok Whether the value could be read
      * @param[in] report Whether errors shall be written to msg handler's error instance
-     * @return The read value if given and correct; -1 if an error occured
+     * @return The read value if given and correct; -1 if an error occurred
      */
     SUMOTime getSUMOTimeReporting(int attr, const char* objectid, bool& ok,
                                   bool report = true) const;
@@ -138,7 +128,7 @@ public:
      *
      * Otherwise, "ok" is not changed.
      *
-     * In dependence to the used time representation, either get<int> or get<SUMOReal>
+     * In dependence to the used time representation, either get<int> or get<double>
      *  is used.
      *
      * @param[in] attr The id of the attribute to read
@@ -146,7 +136,7 @@ public:
      * @param[out] ok Whether the value could be read
      * @param[in] defaultValue The value to return if the attribute is not within the element
      * @param[in] report Whether errors shall be written to msg handler's error instance
-     * @return The read value if given and correct; the default value if the attribute does not exist;  -1 if an error occured
+     * @return The read value if given and correct; the default value if the attribute does not exist;  -1 if an error occurred
      */
     SUMOTime getOptSUMOTimeReporting(int attr, const char* objectid, bool& ok,
                                      SUMOTime defaultValue, bool report = true) const;
@@ -222,7 +212,7 @@ public:
      * @exception EmptyData If the attribute is not known or the attribute value is an empty string
      * @exception NumberFormatException If the attribute value can not be parsed to an int
      */
-    virtual SUMOLong getLong(int id) const = 0;
+    virtual long long int getLong(int id) const = 0;
 
 
     /**
@@ -257,39 +247,39 @@ public:
 
 
     /**
-     * @brief Returns the SUMOReal-value of the named (by its enum-value) attribute
+     * @brief Returns the double-value of the named (by its enum-value) attribute
      *
      * Tries to retrieve the attribute from the the attribute list. The retrieved
-     *  attribute  (which may be 0) is then parsed using TplConvert<XMLCh>::_2SUMOReal.
-     *  If the attribute is empty or ==0, TplConvert<XMLCh>::_2SUMOReal throws an
+     *  attribute  (which may be 0) is then parsed using TplConvert<XMLCh>::_2double.
+     *  If the attribute is empty or ==0, TplConvert<XMLCh>::_2double throws an
      *  EmptyData-exception which is passed.
-     * If the value can not be parsed to a SUMOReal, TplConvert<XMLCh>::_2SUMOReal throws a
+     * If the value can not be parsed to a double, TplConvert<XMLCh>::_2double throws a
      *  NumberFormatException-exception which is passed.
      *
      * @param[in] id The id of the attribute to return the value of
      * @return The attribute's value as a float, if it could be read and parsed
      * @exception EmptyData If the attribute is not known or the attribute value is an empty string
-     * @exception NumberFormatException If the attribute value can not be parsed to an SUMOReal
+     * @exception NumberFormatException If the attribute value can not be parsed to an double
      */
-    virtual SUMOReal getFloat(int id) const = 0;
+    virtual double getFloat(int id) const = 0;
 
 
     /**
-     * @brief Returns the SUMOReal-value of the named attribute
+     * @brief Returns the double-value of the named attribute
      *
      * Tries to retrieve the attribute from the the attribute list. The retrieved
-     *  attribute  (which may be 0) is then parsed using TplConvert<XMLCh>::_2SUMOReal.
-     *  If the attribute is empty or ==0, TplConvert<XMLCh>::_2SUMOReal throws an
+     *  attribute  (which may be 0) is then parsed using TplConvert<XMLCh>::_2double.
+     *  If the attribute is empty or ==0, TplConvert<XMLCh>::_2double throws an
      *  EmptyData-exception which is passed.
-     * If the value can not be parsed to a SUMOReal, TplConvert<XMLCh>::_2SUMOReal throws a
+     * If the value can not be parsed to a double, TplConvert<XMLCh>::_2double throws a
      *  NumberFormatException-exception which is passed.
      *
      * @param[in] id The name of the attribute to return the value of
      * @return The attribute's value as a float, if it could be read and parsed
      * @exception EmptyData If the attribute is not known or the attribute value is an empty string
-     * @exception NumberFormatException If the attribute value can not be parsed to an SUMOReal
+     * @exception NumberFormatException If the attribute value can not be parsed to an double
      */
-    virtual SUMOReal getFloat(const std::string& id) const = 0;
+    virtual double getFloat(const std::string& id) const = 0;
 
 
     /**
@@ -322,6 +312,13 @@ public:
      */
     virtual SumoXMLNodeType getNodeType(bool& ok) const = 0;
 
+    /**
+     * @brief Returns the right-of-way method
+     */
+    virtual RightOfWay getRightOfWay(bool& ok) const = 0;
+
+    /// @brief returns fringe type
+    virtual FringeType getFringeType(bool& ok) const = 0;
 
     /**
      * @brief Returns the value of the named attribute
@@ -335,23 +332,42 @@ public:
     /** @brief Tries to read given attribute assuming it is a PositionVector
      *
      * @param[in] attr The id of the attribute to read
-     * @return The read value if given and not empty; empty position vector if an error occured
+     * @return The read value if given and not empty; empty position vector if an error occurred
      */
     virtual PositionVector getShape(int attr) const = 0;
 
     /** @brief Tries to read given attribute assuming it is a Boundary
      *
      * @param[in] attr The id of the attribute to read
-     * @return The read value if given and not empty; empty Boundary if an error occured
+     * @return The read value if given and not empty; empty Boundary if an error occurred
      */
     virtual Boundary getBoundary(int attr) const = 0;
 
     /** @brief Tries to read given attribute assuming it is a string vector
+     *
+     * The behavior is similar to Python's string.split(), so multiple consecutive
+     *  whitespace do not generate empty strings and leading and trailing whitespace is silently omitted.
      *
      * @param[in] attr The id of the attribute to read
-     * @return The read value if given and not empty; empty vector if an error occured
+     * @return The read value if given and not empty; empty vector if an error occurred
      */
-    virtual std::vector<std::string> getStringVector(int attr) const = 0;
+    const std::vector<std::string> getStringVector(int attr) const;
+
+    /// @brief convenience function to avoid the default argument and the template stuff at getOpt<>
+    const std::vector<std::string> getOptStringVector(int attr, const char* objectid, bool& ok, bool report = true) const;
+
+    /** @brief Tries to read given attribute assuming it is an int vector
+     *
+     * The behavior is similar to Python's string.split(), so multiple consecutive
+     *  whitespace do not generate empty strings and leading and trailing whitespace is silently omitted.
+     *
+     * @param[in] attr The id of the attribute to read
+     * @return The read value if given and not empty; empty vector if an error occurred
+     */
+    const std::vector<int> getIntVector(int attr) const;
+
+    /// @brief convenience function to avoid the default argument and the template stuff at getOpt<>
+    const std::vector<int> getOptIntVector(int attr, const char* objectid, bool& ok, bool report = true) const;
     //}
 
 
@@ -369,6 +385,10 @@ public:
      */
     virtual void serialize(std::ostream& os) const = 0;
 
+    /** @brief Retrieves all attribute names
+     */
+    virtual std::vector<std::string> getAttributeNames() const = 0;
+
 
     /// @brief return the objecttype to which these attributes belong
     const std::string& getObjectType() const {
@@ -378,42 +398,19 @@ public:
 
     friend std::ostream& operator<<(std::ostream& os, const SUMOSAXAttributes& src);
 
+    /// @brief return a new deep-copy attributes object
+    virtual SUMOSAXAttributes* clone() const = 0;
+
     /** @brief The encoding of parsed strings */
     static const std::string ENCODING;
 
 
-    /** @brief Splits the given string
-     *
-     * Spaces, ",", and ";" are assumed to be separator characters.
-     * Though, in the case a "," or a ";" occurs, a warning is generated (once).
-     *
-     * @param[in] def The string to split
-     * @param[out] into The vector to fill
-     */
-    static void parseStringVector(const std::string& def, std::vector<std::string>& into);
-
-
-    /** @brief Splits the given string, stores it in a set
-     *
-     * Spaces, ",", and ";" are assumed to be separator characters.
-     * Though, in the case a "," or a ";" occurs, a warning is generated (once).
-     *
-     * @param[in] def The string to split
-     * @param[out] into The set to fill
-     */
-    static void parseStringSet(const std::string& def, std::set<std::string>& into);
-
-
 protected:
     template <typename T> T getInternal(const int attr) const;
     void emitUngivenError(const std::string& attrname, const char* objectid) const;
     void emitEmptyError(const std::string& attrname, const char* objectid) const;
     void emitFormatError(const std::string& attrname, const std::string& type, const char* objectid) const;
 
-private:
-    /// @brief Information whether the usage of a deprecated divider was reported
-    static bool myHaveInformedAboutDeprecatedDivider;
-
 private:
     /// @brief Invalidated copy constructor.
     SUMOSAXAttributes(const SUMOSAXAttributes& src);
@@ -438,6 +435,56 @@ template<typename X> struct invalid_return {
     static const std::string type;
 };
 
+template<> struct invalid_return<bool> {
+    static const bool value;
+    static const std::string type;
+};
+
+template<> struct invalid_return<int> {
+    static const int value;
+    static const std::string type;
+};
+
+template<> struct invalid_return<long long int> {
+    static const long long int value;
+    static const std::string type;
+};
+
+template<> struct invalid_return<double> {
+    static const double value;
+    static const std::string type;
+};
+
+template<> struct invalid_return<std::string> {
+    static const std::string value;
+    static const std::string type;
+};
+
+template<> struct invalid_return<RGBColor> {
+    static const RGBColor value;
+    static const std::string type;
+};
+
+template<> struct invalid_return<PositionVector> {
+    static const PositionVector value;
+    static const std::string type;
+};
+
+template<> struct invalid_return<Boundary> {
+    static const Boundary value;
+    static const std::string type;
+};
+
+template<> struct invalid_return<std::vector<std::string> > {
+    static const std::vector<std::string> value;
+    static const std::string type;
+};
+
+template<> struct invalid_return<std::vector<int> > {
+    static const std::vector<int> value;
+    static const std::string type;
+};
+
 
 template <typename T>
 T SUMOSAXAttributes::get(int attr, const char* objectid,
@@ -485,9 +532,3 @@ T SUMOSAXAttributes::getOpt(int attr, const char* objectid,
     ok = false;
     return invalid_return<T>::value;
 }
-
-
-#endif
-
-/****************************************************************************/
-
