@@ -1,34 +1,25 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    SUMOSAXHandler.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: SUMOSAXHandler.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // SAX-handler base for SUMO-files
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <iostream>
@@ -36,21 +27,16 @@
 #include "SUMOXMLDefinitions.h"
 #include "SUMOSAXHandler.h"
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
-SUMOSAXHandler::SUMOSAXHandler(const std::string& file)
+SUMOSAXHandler::SUMOSAXHandler(const std::string& file, const std::string& expectedRoot)
     : GenericSAXHandler(SUMOXMLDefinitions::tags, SUMO_TAG_NOTHING,
-                        SUMOXMLDefinitions::attrs, SUMO_ATTR_NOTHING, file) {}
+                        SUMOXMLDefinitions::attrs, SUMO_ATTR_NOTHING, file, expectedRoot) {}
 
 
 SUMOSAXHandler::~SUMOSAXHandler() {}
 
 
 /****************************************************************************/
-
