@@ -1,47 +1,41 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    SUMOSAXReader.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Nov 2012
-/// @version $Id: SUMOSAXReader.h 17235 2014-11-03 10:53:02Z behrisch $
 ///
 // SAX-reader encapsulation containing binary reader
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2014 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef SUMOSAXReader_h
-#define SUMOSAXReader_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
+#include <string>
+#include <memory>
+#include <vector>
 #include <xercesc/sax2/SAX2XMLReader.hpp>
 #include <xercesc/sax/EntityResolver.hpp>
 #include <xercesc/sax/InputSource.hpp>
+#include "SUMOXMLDefinitions.h"
 
 
 // ===========================================================================
 // class declarations
 // ===========================================================================
 class GenericSAXHandler;
-class BinaryInputDevice;
+class IStreamInputSource;
 
 
 // ===========================================================================
@@ -62,7 +56,7 @@ public:
      *
      * @param[in] file The name of the processed file
      */
-    SUMOSAXReader(GenericSAXHandler& handler, const XERCES_CPP_NAMESPACE::SAX2XMLReader::ValSchemes validationScheme);
+    SUMOSAXReader(GenericSAXHandler& handler, const XERCES_CPP_NAMESPACE::SAX2XMLReader::ValSchemes validationScheme, XERCES_CPP_NAMESPACE::XMLGrammarPool* grammarPool);
 
     /// Destructor
     ~SUMOSAXReader();
@@ -88,6 +82,9 @@ private:
     class LocalSchemaResolver : public XERCES_CPP_NAMESPACE::EntityResolver {
     public:
         XERCES_CPP_NAMESPACE::InputSource* resolveEntity(const XMLCh* const publicId, const XMLCh* const systemId);
+        void setHandler(GenericSAXHandler& handler);
+    private:
+        GenericSAXHandler* myHandler;
     };
 
 private:
@@ -110,11 +107,19 @@ private:
     /// @brief Information whether built reader/parser shall validate XML-documents against schemata
     XERCES_CPP_NAMESPACE::SAX2XMLReader::ValSchemes myValidationScheme;
 
+    /// @brief Schema cache to be used for grammars which are not declared
+    XERCES_CPP_NAMESPACE::XMLGrammarPool* myGrammarPool;
+
     XERCES_CPP_NAMESPACE::XMLPScanToken myToken;
 
     XERCES_CPP_NAMESPACE::SAX2XMLReader* myXMLReader;
 
-    BinaryInputDevice* myBinaryInput;
+    std::unique_ptr<std::istream> myIStream;
+
+    std::unique_ptr<IStreamInputSource> myInputStream;
+
+    /// @brief The stack of begun xml elements
+    std::vector<SumoXMLTag> myXMLStack;
 
     LocalSchemaResolver mySchemaResolver;
 
@@ -126,8 +131,3 @@ private:
     const SUMOSAXReader& operator=(const SUMOSAXReader& s);
 
 };
-
-
-#endif
-
-/****************************************************************************/
