@@ -1,36 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    XMLSubSys.h
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 1 Jul 2002
-/// @version $Id: XMLSubSys.h 22608 2017-01-17 06:28:54Z behrisch $
 ///
 // Utility methods for initialising, closing and using the XML-subsystem
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef XMLSubSys_h
-#define XMLSubSys_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
+#include <string>
 #include <vector>
 #include <xercesc/sax2/SAX2XMLReader.hpp>
 
@@ -67,8 +58,6 @@ class SUMOSAXReader;
  * In addition to initialisation and shutdown, this module allows to build
  *  SAXReaders and/or running a given handler on a given file without
  *  dealing with the reader at all.
- *
- * @todo make schema checking optional
  */
 class XMLSubSys {
 public:
@@ -84,14 +73,14 @@ public:
 
 
     /**
-     * @brief Enables or disables validation.
-     *
-     * The setting is only valid for parsers created after the call. Existing parsers are not adapted.
-     *
-     * @param[in] validationScheme Whether validation of XML-documents against schemata shall be enabled
-     * @param[in] netValidationScheme Whether validation of SUMO networks against schemata shall be enabled
-     */
-    static void setValidation(const std::string& validationScheme, const std::string& netValidationScheme);
+    * @brief Enables or disables validation.
+    *
+    * The setting is only valid for parsers created after the call. Existing parsers are not adapted.
+    *
+    * @param[in] validationScheme Whether validation of XML-documents against schemata shall be enabled
+    * @param[in] netValidationScheme Whether validation of SUMO networks against schemata shall be enabled
+    */
+    static void setValidation(const std::string& validationScheme, const std::string& netValidationScheme, const std::string& routeValidationScheme);
 
 
     /**
@@ -110,10 +99,13 @@ public:
      *  to the reader as the current DefaultHandler and ErrorHandler.
      *
      * @param[in] handler The handler to assign to the built reader
+     * @param[in] isNet   whether a network gets loaded
+     * @param[in] isRoute whether routes get loaded
      * @return The built Xerces-SAX-reader, 0 if something failed
      * @see getSAXReader()
      */
-    static SUMOSAXReader* getSAXReader(SUMOSAXHandler& handler);
+    static SUMOSAXReader* getSAXReader(SUMOSAXHandler& handler,
+                                       const bool isNet = false, const bool isRoute = false);
 
 
     /**
@@ -142,10 +134,11 @@ public:
      * @param[in] handler The handler to assign to the built reader
      * @param[in] file    The file to run the parser at
      * @param[in] isNet   whether a network gets loaded
+     * @param[in] isRoute whether routes get loaded
      * @return true if the parsing was done without errors, false otherwise (error was printed)
      */
-    static bool runParser(GenericSAXHandler& handler,
-                          const std::string& file, const bool isNet = false);
+    static bool runParser(GenericSAXHandler& handler, const std::string& file,
+                          const bool isNet = false, const bool isRoute = false);
 
 
 private:
@@ -161,10 +154,10 @@ private:
     /// @brief Information whether built reader/parser shall validate SUMO networks against schemata
     static XERCES_CPP_NAMESPACE::SAX2XMLReader::ValSchemes myNetValidationScheme;
 
+    /// @brief Information whether built reader/parser shall validate SUMO routes against schemata
+    static XERCES_CPP_NAMESPACE::SAX2XMLReader::ValSchemes myRouteValidationScheme;
+
+    /// @brief Schema cache to be used for grammars which are not declared
+    static XERCES_CPP_NAMESPACE::XMLGrammarPool* myGrammarPool;
+
 };
-
-
-#endif
-
-/****************************************************************************/
-
