@@ -1,25 +1,26 @@
 #!/usr/bin/env python
-"""
-@file    setup.py
-@author  Michael Behrisch
-@author  Jakob Erdmann
-@date    2010-05-23
-@version $Id: setup.py 22608 2017-01-17 06:28:54Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2008-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    setup.py
+# @author  Michael Behrisch
+# @author  Jakob Erdmann
+# @date    2010-05-23
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2008-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 from __future__ import absolute_import
 
 from distutils.core import setup
-import py2exe
+# import is needed to make the module known
+import py2exe  # noqa
 import sys
 import shutil
 import os
@@ -38,19 +39,19 @@ sys.argv[1] = "py2exe"
 
 base = os.path.abspath(os.path.dirname(__file__))
 oldDir = os.getcwd()
+os.chdir(base)
+# add sumo configs and input files
+files = [d for d in os.listdir(".") if os.path.isdir(d)]
+for pattern in ['*.sumocfg', 'input_additional.add.xml', '*.gif']:
+    files += glob.glob(pattern)
+zipfName = inZip.replace("sumo-", "sumo-game-")
+subprocess.call(['git', 'archive', 'HEAD', '-o', zipfName] + files)
+# run py2exe
 tmpDir = tempfile.mkdtemp()
 os.chdir(tmpDir)
 os.mkdir("dist")
-
 setup(console=[os.path.join(base, 'runner.py')])
-
-for pattern in ['*.sumocfg', 'input_additional.add.xml', '*.gif']:
-    for f in glob.glob(os.path.join(base, pattern)):
-        shutil.copy2(f, "dist")
-for d in os.listdir(base):
-    path = os.path.join(base, d)
-    if os.path.isdir(path):
-        subprocess.call(['svn', 'export', path, os.path.join("dist", d)])
+# collect sumo binaries and dlls
 osgPlugins = None
 with zipfile.ZipFile(inZip) as binZip:
     for f in binZip.namelist():
@@ -71,13 +72,13 @@ if osgPlugins:
     os.rename(osgPlugins, os.path.basename(osgPlugins))
     for f in glob.glob(os.path.join(base, '..', '..', 'data', '3D', '*')):
         shutil.copy2(f, ".")
+    os.mkdir("bs3d")
     os.chdir("bs3d")
     subprocess.call([sevenZip, 'x', os.path.join(
         os.path.dirname(inZip), '..', '3D_Modell_Forschungskreuzung_BS.7z')])
     os.chdir("..")
-zipf = zipfile.ZipFile(
-    inZip.replace("sumo-", "sumo-game-"), 'w', zipfile.ZIP_DEFLATED)
-
+# package the zip
+zipf = zipfile.ZipFile(zipfName, 'a', zipfile.ZIP_DEFLATED)
 root_len = len(os.path.abspath("."))
 for root, dirs, files in os.walk("."):
     archive_root = os.path.abspath(root)[root_len:]
