@@ -1,23 +1,25 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-"""
-@file    tls_vissimXML2SUMOnet_update.py
-@author  Lukas Grohmann <Lukas.Grohmann@ait.ac.at>
-@author  Gerald Richter <Gerald.Richter@ait.ac.at>
-@date    Jun 11 2015
-@version $Id: tls_vissimXML2SUMOnet_update.py 22608 2017-01-17 06:28:54Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2009-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    tls_vissimXML2SUMOnet_update.py
+# @author  Lukas Grohmann <Lukas.Grohmann@ait.ac.at>
+# @author  Gerald Richter <Gerald.Richter@ait.ac.at>
+# @date    Jun 11 2015
+
+"""
 Converts a VISSIM-tls-description into a SUMO-tls-description and writes
 the appended information to a copy of the given sumo.net file
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2009-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 # TODO: usage doc.ref
 
@@ -53,8 +55,8 @@ def get_conn_verb_rel(conn_tab, from_to_tab):
     for conn in conn_tab:
         if ':' not in conn['from']:
             link = from_to_tab[
-                (from_to_tab['f_link'] == conn['from'].split("[")[0])
-                & (from_to_tab['t_link'] == conn['to'].split("[")[0])]
+                (from_to_tab['f_link'] == conn['from'].split("[")[0]) &
+                (from_to_tab['t_link'] == conn['to'].split("[")[0])]
             if len(link) > 0:
                 if link["_link"][0] in conn_link_d:
                     conn_link_d[link["_link"][0]].append(conn['via'])
@@ -269,7 +271,7 @@ def get_sg_connection_data(
                     link = signal['link']
                     lane = str(int(signal['lane']) - 1)
                     # tls on normal edge or verbinder?
-                    if is_verbinder_d[link] == False:
+                    if is_verbinder_d[link] is False:
                         if link in edge_list:
                             connection = conn_tab[
                                 (conn_tab["from"] == link) & (
@@ -286,8 +288,8 @@ def get_sg_connection_data(
                                     check = False
                             print("".join([link, "[", str(split_len), "]"]))
                             connection = conn_tab[(conn_tab["from"] == "".join(
-                                [link, "[", str(split_len), "]"]))
-                                & (conn_tab["fromLane"] == lane)][0]
+                                [link, "[", str(split_len), "]"])) &
+                                (conn_tab["fromLane"] == lane)][0]
                     else:
                         connection = conn_tab[
                             (conn_tab["via"] ==
