@@ -1,11 +1,22 @@
 #!/usr/bin/env python
-"""
-@file    vissim_parseBusStops.py
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@date    2009-05-27
-@version $Id: vissim_parseBusStops.py 17235 2014-11-03 10:53:02Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2009-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    vissim_parseBusStops.py
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @date    2009-05-27
+
+"""
 
 Parses bus stops and bus routes given in the Vissim file (first parameter).
 
@@ -14,16 +25,10 @@ The read routes are saved as <OUTPUT_PREFIX>_stops.add.xml
 
 (Starting?) edges of the route may be renamed by setting them within "edgemap"
  variable (see below).
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2009-2014 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
+from __future__ import absolute_import
+from __future__ import print_function
+import sys
 
 edgemap = {}
 edgemap["203"] = "203[0]"
@@ -32,60 +37,57 @@ edgemap["77"] = "77[0]"
 edgemap["203"] = "203[0]"
 
 
-
-
-import sys
-from random import *
-
 def getName(vals, beg):
     name = vals[beg]
-    while name.count('"')!=2:
+    while name.count('"') != 2:
         beg = beg + 1
         name = name + " " + vals[beg]
     return name.replace('"', '')
-        
+
 
 def parseBusStop(bs):
     vals = bs.split()
     id = int(vals[1])
     i = vals.index("NAME")
-    name = getName(vals, i+1)
+    name = getName(vals, i + 1)
     i = vals.index("STRECKE", i)
-    strecke = vals[i+1]
+    strecke = vals[i + 1]
     i = vals.index("SPUR", i)
-    spur = int(vals[i+1]) - 1
+    spur = int(vals[i + 1]) - 1
     i = vals.index("BEI", i)
-    von = float(vals[i+1])
+    von = float(vals[i + 1])
     i = vals.index("LAENGE", i)
-    bis = von + float(vals[i+1])
+    bis = von + float(vals[i + 1])
     return (id, name, strecke, spur, von, bis)
 
+
 def parseBusRoute(br, stops):
     vals = br.split()
     id = vals[1]
     i = vals.index("NAME")
-    name = getName(vals, i+1)
+    name = getName(vals, i + 1)
     i = vals.index("EINFAHRT", i)
-    startKante = vals[i+2]
+    startKante = vals[i + 2]
     i = vals.index("ZIEL", i)
-    ziel = vals[i+2]
+    ziel = vals[i + 2]
     zeiten = []
     endI = vals.index("HALTESTELLE", i)
     i = vals.index("STARTZEITEN", i)
     i = i + 1
-    while i>0 and i<endI:
+    while i > 0 and i < endI:
         zeiten.append(int(float(vals[i])))
         i = i + 5
     stops = []
-    while i>0 and i<len(vals):
+    while i > 0 and i < len(vals):
         try:
             i = vals.index("HALTESTELLE", i)
             i = i + 1
             stops.insert(0, int(vals[i]))
-        except:
+        except Exception:
             i = len(vals) + 1
     return (id, name, startKante, ziel, zeiten, stops)
 
+
 def sorter(idx):
     def t(i, j):
         if i[idx] < j[idx]:
@@ -97,10 +99,10 @@ def sorter(idx):
 
 
 if len(sys.argv) < 3:
-    print "Usage: " + sys.argv[0] + " <VISSIM_NETWORK> <OUTPUT_PREFIX>"
+    print("Usage: " + sys.argv[0] + " <VISSIM_NETWORK> <OUTPUT_PREFIX>")
     sys.exit()
 
-print "Parsing Vissim input..."
+print("Parsing Vissim input...")
 fd = open(sys.argv[1])
 haveStop = False
 haveRoute = False
@@ -109,24 +111,24 @@ stopsL = []
 routesL = []
 for line in fd:
     # process bus stops ("HALTESTELLE")
-    if line.find("HALTESTELLE")==0:
+    if line.find("HALTESTELLE") == 0:
         if haveStop:
             stopsL.append(" ".join(currentItem.split()))
         haveStop = True
         currentItem = ""
-    elif line[0]!=' ':
+    elif line[0] != ' ':
         if haveStop:
             stopsL.append(" ".join(currentItem.split()))
         haveStop = False
     if haveStop:
         currentItem = currentItem + line
     # process bus routes ("LINIE")
-    if line.find(" LINIE")==0:
+    if line.find(" LINIE") == 0:
         if haveRoute:
             routesL.append(" ".join(currentItem.split()))
         haveRoute = True
         currentItem = ""
-    elif len(line)>2 and line[0]!=' ' and line[1]!=' ':
+    elif len(line) > 2 and line[0] != ' ' and line[1] != ' ':
         if haveRoute:
             routesL.append(" ".join(currentItem.split()))
         haveRoute = False
@@ -157,31 +159,33 @@ for br in routesL:
             edges[i] = edgemap[edges[i]]
     for t in zeiten:
         id = str(pid) + "_" + str(t)
-        emissions.append( ( int(t), id, edges, stops ) )
+        emissions.append((int(t), id, edges, stops))
 
 # sort emissions
-print "Sorting routes..."
+print("Sorting routes...")
 emissions.sort(sorter(0))
 # write routes
-print "Writing bus routes..."
+print("Writing bus routes...")
 fdo = open(sys.argv[2] + "_busses.rou.xml", "w")
 fdo.write("<routes>\n")
 for emission in emissions:
-    if len(emission[2])<2:
-        continue;
-    fdo.write('    <vehicle id="' + emission[1] + '" depart="' + str(emission[0]) + '" type="bus" color="0,1,0"><route>' + " ".join(emission[2]) + '</route>\n');
+    if len(emission[2]) < 2:
+        continue
+    fdo.write('    <vehicle id="' + emission[1] + '" depart="' + str(emission[
+              0]) + '" type="bus" color="0,1,0"><route>' + " ".join(emission[2]) + '</route>\n')
     for s in emission[3]:
-        fdo.write('        <stop bus_stop="' + str(s) + '_0" duration="20"/>\n')
-    fdo.write('    </vehicle>\n');
-fdo.write("</routes>\n")    
+        fdo.write(
+            '        <stop bus_stop="' + str(s) + '_0" duration="20"/>\n')
+    fdo.write('    </vehicle>\n')
+fdo.write("</routes>\n")
 
 # process bus stops
-print "Writing bus stops..."
+print("Writing bus stops...")
 fdo = open(sys.argv[2] + "_stops.add.xml", "w")
 fdo.write("<add>\n")
 for bs in stopsL:
     (id, name, strecke, spur, von, bis) = parseBusStop(bs)
-    fdo.write('    <busStop id="' + str(id) + '" lane="' + strecke + "_" + str(spur) + '" from="' + str(von) + '" to="' + str(bis) + '" lines="--"/>\n')
-fdo.write("</add>\n")    
+    fdo.write('    <busStop id="' + str(id) + '" lane="' + strecke + "_" +
+              str(spur) + '" from="' + str(von) + '" to="' + str(bis) + '" lines="--"/>\n')
+fdo.write("</add>\n")
 fdo.close()
-
