@@ -1,48 +1,39 @@
 #!/usr/bin/env python
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2009-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+
+# @file    visum_convertTurnPercentages.py
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @date    2009-06-12
+
 """
-@file    visum_convertTurnPercentages.py
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@date    2009-06-12
-@version $Id: visum_convertTurnPercentages.py 22608 2017-01-17 06:28:54Z behrisch $
-
-
 Converts VISUM turning percentages into
  turning percentages JTRROUTER can read.
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2009-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from __future__ import print_function
 
 
 import os
-import string
 import sys
-import StringIO
-from xml.sax import saxutils, make_parser, handler
-
-sys.path.append(
-    os.path.join(os.path.abspath(os.path.dirname(sys.argv[0])), "../../lib"))
-import sumonet
-
+sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..'))
+import sumolib  # noqa
 
 if len(sys.argv) < 4:
     print("Usage: " + sys.argv[0] + " <SUMO_NET> <VISUM_TURNINGS> <OUTPUT>")
     sys.exit()
 print("Reading net...")
-parser = make_parser()
-net = sumonet.NetReader()
-parser.setContentHandler(net)
-parser.parse(sys.argv[1])
-net = net.getNet()
+net = sumolib.net.readNet(sys.argv[1])
 
 # initialise edges and nodes map
 emap = {}
@@ -64,6 +55,7 @@ found = 0
 foundN = 0
 missing = 0
 missingN = 0
+attributes = []
 fd = open(sys.argv[2])
 for line in fd:
     if line.find("$") == 0 or line.find("*") == 0 or line.find(";") < 0:
