@@ -1,22 +1,24 @@
 #!/usr/bin/env python
-"""
-@file    testUtil.py
-@author  Michael Behrisch
-@date    2010-10-26
-@version $Id: testUtil.py 20433 2016-04-13 08:00:14Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    testUtil.py
+# @author  Michael Behrisch
+# @date    2010-10-26
+
+"""
 This library wraps useful functions for the complex tests
 and automatic GUI control
 from AutoPy (http://github.com/msanders/autopy).
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2010-2016 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 
@@ -28,7 +30,7 @@ _bindir = os.path.abspath(os.path.join(_libdir, '..', '..', 'bin'))
 try:
     import autopy
 
-    #PLAY = autopy.bitmap.Bitmap.from_string()
+    # PLAY = autopy.bitmap.Bitmap.from_string()
     PLAY = autopy.bitmap.Bitmap.open(os.path.join(_libdir, "play.png"))
     STOP = autopy.bitmap.Bitmap.open(os.path.join(_libdir, "stop.png"))
 
@@ -53,6 +55,7 @@ def checkBinary(name, bindir=_bindir):
         return name
     return binary
 
+
 if __name__ == "__main__":
     findAndClick(PLAY)
     time.sleep(10)
