@@ -1,23 +1,25 @@
 #!/usr/bin/env python
-"""
-@file    createRoundaboutConnections.py
-@author  Laura Bieker
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@date    2007-02-21
-@version $Id: createRoundaboutConnections.py 22608 2017-01-17 06:28:54Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    createRoundaboutConnections.py
+# @author  Laura Bieker
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @date    2007-02-21
+
+"""
 This script reads in the network given as
  first parameter and generates additional connections for roundabouts.
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2010-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from __future__ import print_function
@@ -26,7 +28,7 @@ from __future__ import print_function
 import os
 import sys
 sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
-import sumolib.net
+import sumolib.net  # noqa
 
 
 def writeConnections(net):
@@ -39,7 +41,7 @@ def writeConnections(net):
             for edgeOut in edgesOut:
                 outNumber = edgeOut.getLaneNumber()
                 for edgeIn in edgesIn:
-                    if not edgeOut in edgeIn._outgoing:
+                    if edgeOut not in edgeIn._outgoing:
                         continue
                     inNumber = edgeIn.getLaneNumber()
                     for x in range(inNumber):
