@@ -1,38 +1,37 @@
 #!/usr/bin/env python
-"""
-@file    visum_mapDistricts.py
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@date    2007-10-25
-@version $Id: visum_mapDistricts.py 22608 2017-01-17 06:28:54Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2008-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    visum_mapDistricts.py
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @date    2007-10-25
+
+"""
 
 This script reads a network and a dump file and
  draws the network, coloring it by the values
  found within the dump-file.
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2008-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from __future__ import print_function
 
 import os
-import string
 import sys
 import math
 from optparse import OptionParser
-from matplotlib.collections import LineCollection
 
 sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
-import sumolib.net
-import netshiftadaptor
+import sumolib.net  # noqa
 
 
 def computeDistance(n1, n2):
@@ -77,7 +76,7 @@ if options.verbose:
 net2 = sumolib.net.readNet(options.net2)
 
 # reproject the visum net onto the navteq net
-adaptor = netshiftadaptor.NetShiftAdaptor(
+adaptor = sumolib.net.netshiftadaptor.NetShiftAdaptor(
     net1, net2, options.nodes1.split(","), options.nodes2.split(","))
 adaptor.reproject(options.verbose)
 
@@ -174,8 +173,8 @@ nnn = {}
 for n1 in nodes1:
     if n1._id.find('-', 1) < 0:
         continue
-#	if n1._id.find("38208387")<0:
-#		continue
+#   if n1._id.find("38208387")<0:
+#       continue
     un1 = None
     for e in n1._outgoing:
         un1 = e._to
@@ -431,7 +430,8 @@ for d in nmap1to2:
                 for i in range(0, outgoingLaneNo):
                     net1.addLane(e2, 18, 100.)
                     fdd.write('    <connection from="' + dn1i._incoming[
-                              0]._id + '" to="' + "i" + d + "#" + n2._id + '" lane="' + str(runningInLaneNumber) + ':' + str(i) + '"/>\n')
+                              0]._id + '" to="' + "i" + d + "#" + n2._id + '" lane="' + str(runningInLaneNumber) +
+                              ':' + str(i) + '"/>\n')
                     runningInLaneNumber = runningInLaneNumber + 1
                 if outgoingLaneNo == 0:
                     net1.addLane(e2, 18, 100.)
@@ -496,6 +496,7 @@ def writeEdges(net):
     fd.write("</edges>\n")
     fd.close()
 
+
 fdd.write("</connections>\n")
 writeNodes(net1)
 writeEdges(net1)
