@@ -1,34 +1,40 @@
 #!/usr/bin/env python
-"""
-@file    xmledges_applyOffset.py
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@date    2009-08-01
-@version $Id: xmledges_applyOffset.py 17235 2014-11-03 10:53:02Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2009-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    xmledges_applyOffset.py
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @date    2009-08-01
+
+"""
 Applies a given offset to edges given in an xml-edge-file.
 The results are written into <XMLEDGES>.mod.xml.
 Call: xmledges_applyOffset.py <XMLEDGES> <X-OFFSET> <Y-OFFSET>
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2009-2014 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
+from __future__ import absolute_import
+from __future__ import print_function
 
-import os, string, sys, StringIO
+import sys
 from xml.sax import saxutils, make_parser, handler
 
+
 class XMLNodesReader(handler.ContentHandler):
+
     def __init__(self, outFileName, xoff, yoff):
         self._out = open(outFileName, 'w')
         self._xoff = xoff
         self._yoff = yoff
-        
+
     def endDocument(self):
         self._out.close()
 
@@ -39,9 +45,11 @@ class XMLNodesReader(handler.ContentHandler):
                 shape = value.split(" ")
                 nshape = []
                 for s in shape:
-                    (x,y) = s.split(",")
-                    nshape.append(str(float(x)+self._xoff)+","+str(float(y)+self._yoff))
-                self._out.write(' %s="%s"' % (key, saxutils.escape(" ".join(nshape))))
+                    (x, y) = s.split(",")
+                    nshape.append(
+                        str(float(x) + self._xoff) + "," + str(float(y) + self._yoff))
+                self._out.write(' %s="%s"' %
+                                (key, saxutils.escape(" ".join(nshape))))
             else:
                 self._out.write(' %s="%s"' % (key, saxutils.escape(value)))
         self._out.write('>')
@@ -51,19 +59,19 @@ class XMLNodesReader(handler.ContentHandler):
 
     def characters(self, content):
         self._out.write(saxutils.escape(content))
-                    
+
     def ignorableWhitespace(self, content):
         self._out.write(content)
-        
+
     def processingInstruction(self, target, data):
         self._out.write('<?%s %s?>' % (target, data))
 
 
-            
 if len(sys.argv) < 4:
-    print "Usage: " + sys.argv[0] + " <XMLEDGES> <X-OFFSET> <Y-OFFSET>"
+    print("Usage: " + sys.argv[0] + " <XMLEDGES> <X-OFFSET> <Y-OFFSET>")
     sys.exit()
 parser = make_parser()
-reader = XMLNodesReader(sys.argv[1]+".mod.xml", float(sys.argv[2]), float(sys.argv[3]))
+reader = XMLNodesReader(
+    sys.argv[1] + ".mod.xml", float(sys.argv[2]), float(sys.argv[3]))
 parser.setContentHandler(reader)
 parser.parse(sys.argv[1])
