@@ -1,28 +1,27 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-"""
-@file    vehLanes.py
-@author  Jakob Erdmann
-@date    2012-11-20
-@version $Id: vehLanes.py 22608 2017-01-17 06:28:54Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2008-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-Compute a vehroute-like output for lanes based on a sumo netstate-dump
+# @file    vehLanes.py
+# @author  Jakob Erdmann
+# @date    2012-11-20
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2008-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 from __future__ import absolute_import
 import os
 import sys
 from collections import defaultdict
 sys.path.append(os.path.join(os.path.dirname(sys.argv[0]), '..'))
-from sumolib.output import parse
+from sumolib.output import parse  # noqa
 
 
 def trackLanes(netstate, out):
@@ -45,7 +44,7 @@ def trackLanes(netstate, out):
                             if lane.vehicle is not None:
                                 for vehicle in lane.vehicle:
                                     seen.add(vehicle.id)
-                                    if not vehicle.id in running or laneTimes[vehicle.id][-1][1] != lane.id:
+                                    if vehicle.id not in running or laneTimes[vehicle.id][-1][1] != lane.id:
                                         laneTimes[vehicle.id].append(
                                             (timestep.time, lane.id))
                                         running.add(vehicle.id)
@@ -53,7 +52,7 @@ def trackLanes(netstate, out):
                                             laneChanges[vehicle.id] += 1
                                         lastEdge[vehicle.id] = edge.id
             for veh_id in running:
-                if not veh_id in seen:
+                if veh_id not in seen:
                     arrivals[veh_id] = timestep.time
             running = running - set(arrivals.keys())
 
@@ -65,6 +64,7 @@ def trackLanes(netstate, out):
                 laneChanges[veh_id]))
         f.write("</vehLanes>\n")
 
+
 if __name__ == "__main__":
     if len(sys.argv) != 3:
         sys.exit("call %s <netstate-dump> <output-file>" % sys.argv[0])
