@@ -1,30 +1,30 @@
 #!/usr/bin/env python
-"""
-@file    route2alts.py
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@date    11.09.2009
-@version $Id: route2alts.py 22608 2017-01-17 06:28:54Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2008-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    route2alts.py
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @date    11.09.2009
+
+"""
 Counts possible routes for all depart/arrival edges.
 Builds route alternatives assigning the so determined probabilities to use a route.
 
 Please note that the cost of the route is not computed!
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2008-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from __future__ import print_function
 import sys
-import optparse
-import array
 from xml.sax import make_parser, handler
 
 
