@@ -1,34 +1,35 @@
 #!/usr/bin/env python
-"""
-@file    sort_routes.py
-@author  Jakob Erdmann
-@author  Michael Behrisch
-@author  Pieter Loof
-@date    2011-07-14
-@version $Id: sort_routes.py 23248 2017-03-07 13:48:57Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2007-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-This script sorts the vehicles in the given route file by their depart time
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2007-2017 DLR (http://www.dlr.de/) and contributors
+# @file    sort_routes.py
+# @author  Jakob Erdmann
+# @author  Michael Behrisch
+# @author  Pieter Loof
+# @date    2011-07-14
 
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 from __future__ import absolute_import
 from __future__ import print_function
-import sys
 import os
-import re
+import sys
 from xml.dom import pulldom
 from xml.sax import handler
 from xml.sax import make_parser
-from xml.sax import handler
 from optparse import OptionParser
 
-DEPART_ATTRS = {'vehicle': 'depart', 'flow': 'begin', 'person': 'depart'}
+sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
+import sumolib  # noqa
+
+DEPART_ATTRS = {'vehicle': 'depart', 'trip': 'depart', 'flow': 'begin', 'person': 'depart'}
 
 
 def get_options(args=None):
@@ -47,7 +48,7 @@ def get_options(args=None):
 
 
 def sort_departs(routefilename, outfile):
-    routes_doc = pulldom.parse(sys.argv[1])
+    routes_doc = pulldom.parse(routefilename)
     vehicles = []
     root = None
     for event, parsenode in routes_doc:
@@ -59,7 +60,13 @@ def sort_departs(routefilename, outfile):
             routes_doc.expandNode(parsenode)
             departAttr = DEPART_ATTRS.get(parsenode.localName)
             if departAttr is not None:
-                start = float(parsenode.getAttribute(departAttr))
+                startString = parsenode.getAttribute(departAttr)
+                if ':' in startString:
+                    start = sumolib.miscutils.parseTime(startString)
+                elif startString == "triggered":
+                    start = -1  # before everything else
+                else:
+                    start = float(startString)
                 vehicles.append(
                     (start, parsenode.toprettyxml(indent="", newl="")))
             else:
@@ -151,8 +158,8 @@ def main(args=None):
         outfile = open(options.outfile, 'w')
         # copy header
         for line in open(options.routefile):
-            if (line.find('<routes') == 0
-                    or line.find('<additional') == 0):
+            if (line.find('<routes') == 0 or
+                    line.find('<additional') == 0):
                 break
             else:
                 outfile.write(line)
