@@ -1,51 +1,49 @@
 # -*- coding: utf-8 -*-
-"""
-@file    __init__.py
-@author  Daniel Krajzewicz
-@author  Jakob Erdmann
-@author  Michael Behrisch
-@date    2011-06-23
-@version $Id: __init__.py 20482 2016-04-18 20:49:42Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2011-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-Python interface to SUMO especially for parsing xml input and output files.
+# @file    __init__.py
+# @author  Daniel Krajzewicz
+# @author  Jakob Erdmann
+# @author  Michael Behrisch
+# @date    2011-06-23
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 from __future__ import absolute_import
 import os
 import sys
 import subprocess
-import datetime
+import gzip
+import io
+try:
+    from urllib.request import urlopen
+except ImportError:
+    from urllib import urlopen
 from xml.sax import parseString, handler
 from optparse import OptionParser, OptionGroup, Option
 
-try:
-    from . import visualization
-except ImportError as e:
-    class VisDummy:
-
-        def __getattr__(self, name):
-            raise e
-    visualization = VisDummy()
-from . import files, net, output, sensors, shapes
-from . import color, geomhelper, miscutils, options, route
+from . import files, net, output, sensors, shapes, statistics, fpdiff  # noqa
+from . import color, geomhelper, miscutils, options, route, version  # noqa
+from .xml import writeHeader as writeXMLHeader  # noqa
+# the visualization submodule is not imported to avoid an explicit matplotlib dependency
 
 
 class ConfigurationReader(handler.ContentHandler):
 
     """Reads a configuration template, storing the options in an OptionParser"""
 
-    def __init__(self, optParse, groups, options):
+    def __init__(self, optParse, groups, configoptions):
         self._opts = optParse
         self._groups = groups
-        self._options = options
+        self._options = configoptions
         self._group = self._opts
 
     def startElement(self, name, attrs):
@@ -79,22 +77,22 @@ class ConfigurationReader(handler.ContentHandler):
             self._group = self._opts
 
 
-def pullOptions(executable, optParse, groups=None, options=None):
-    output = subprocess.Popen(
+def pullOptions(executable, optParse, groups=None, configoptions=None):
+    optoutput = subprocess.Popen(
         [executable, "--save-template", "-"], stdout=subprocess.PIPE).communicate()[0]
-    parseString(output, ConfigurationReader(optParse, groups, options))
+    parseString(optoutput, ConfigurationReader(optParse, groups, configoptions))
 
 
-def saveConfiguration(executable, options, filename):
-    options.save_configuration = filename
-    call(executable, options)
+def saveConfiguration(executable, configoptions, filename):
+    configoptions.save_configuration = filename
+    call(executable, configoptions)
 
 
-def call(executable, options):
+def call(executable, args):
     optParser = OptionParser()
     pullOptions(executable, optParser)
     cmd = [executable]
-    for option, value in options.__dict__.iteritems():
+    for option, value in args.__dict__.iteritems():
         o = "--" + option.replace("_", "-")
         opt = optParser.get_option(o)
         if opt is not None and value is not None and opt.default != value:
@@ -131,10 +129,15 @@ def checkBinary(name, bindir=None):
         binary = join(env.get("SUMO_HOME"), "bin", name)
         if exeExists(binary):
             return binary
-    binary = os.path.abspath(
-        join(os.path.dirname(__file__), '..', '..', 'bin', name))
-    if exeExists(binary):
-        return binary
+    if bindir is None:
+        binary = os.path.abspath(join(os.path.dirname(__file__), '..', '..', 'bin', name))
+        if exeExists(binary):
+            return binary
+    if name[-1] != "D" and name[-5:] != "D.exe":
+        binaryD = (name[:-4] if name[-4:] == ".exe" else name) + "D"
+        found = checkBinary(binaryD, bindir)
+        if found != binaryD:
+            return found
     return name
 
 
@@ -168,7 +171,7 @@ class _Running:
                 if self.warn:
                     try:
                         int(id)
-                    except:
+                    except ValueError:
                         sys.stderr.write(
                             'Warning: ID "%s" is not an integer.\n' % id)
                         self.warn = False
@@ -192,8 +195,8 @@ class TeeFile:
 
     """A helper class which allows simultaneous writes to several files"""
 
-    def __init__(self, *files):
-        self.files = files
+    def __init__(self, *outputfiles):
+        self.files = outputfiles
 
     def write(self, txt):
         """Writes the text to all files"""
@@ -204,7 +207,7 @@ class TeeFile:
         """flushes all file contents to disc"""
         for fp in self.files:
             fp.flush()
-            if type(fp) is int or hasattr(fp, "fileno"):
+            if isinstance(fp, int) or hasattr(fp, "fileno"):
                 try:
                     os.fsync(fp)
                 except OSError:
@@ -222,10 +225,12 @@ def _laneID2edgeID(laneID):
     return laneID[:laneID.rfind("_")]
 
 
-def writeXMLHeader(outf, script):
-    outf.write("""<?xml version="1.0"?>
-<!-- generated on %s by %s
-  options: %s
--->
-""" % (datetime.datetime.now(), script,
-       (' '.join(sys.argv[1:]).replace('--', '<doubleminus>'))))
+def open(fileOrURL, tryGZip=True, mode="rb"):
+    try:
+        if fileOrURL.startswith("http"):
+            return io.BytesIO(urlopen(fileOrURL).read())
+        if tryGZip:
+            return gzip.open(fileOrURL)
+    finally:
+        pass
+    return io.open(fileOrURL, mode=mode)
