@@ -1,21 +1,23 @@
-"""
-@file    gpx.py
-@author  Jakob Erdmann
-@author  Laura Bieker
-@date    2014-02-13
-@version $Id: gpx.py 20165 2016-03-09 09:19:13Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2014-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    gpx.py
+# @author  Jakob Erdmann
+# @author  Laura Bieker
+# @date    2014-02-13
+
+"""
 This module includes functions for converting SUMO's fcd-output into
 GPX format (http://en.wikipedia.org/wiki/GPS_eXchange_Format)
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2014 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from collections import defaultdict
