@@ -1,22 +1,24 @@
-"""
-@file    phem.py
-@author  Daniel Krajzewicz
-@author  Jakob Erdmann
-@author  Michael Behrisch
-@date    2013-01-15
-@version $Id: phem.py 23247 2017-03-07 13:46:58Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    phem.py
+# @author  Daniel Krajzewicz
+# @author  Jakob Erdmann
+# @author  Michael Behrisch
+# @date    2013-01-15
+
+"""
 This module includes functions for converting SUMO's fcd-output into
 data files read by PHEM.
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2013-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import print_function
 from __future__ import absolute_import
@@ -47,7 +49,7 @@ def fcd2dri(inpFCD, outSTRM, ignored):
     The following may be a matter of changes:
     - the engine torque is not given
     """
-    #print >> outSTRM, "v1\n<t>,<v>,<grad>,<n>\n[s],[km/h],[%],[1/min]\n"
+    # print >> outSTRM, "v1\n<t>,<v>,<grad>,<n>\n[s],[km/h],[%],[1/min]\n"
     print("v1\n<t>,<v>,<grad>\n[s],[km/h],[%]", file=outSTRM)
     for q in inpFCD:
         if q.vehicle:
@@ -92,8 +94,7 @@ def fcd2fzp(inpFCD, outSTRM, further):
     """
     sIDm = further["phemStreetMap"]
     if outSTRM is not None:
-        print(
-            "t,WeltX,WeltY,Veh. No,v,Gradient,veh.Typ-Id,Str-Id", file=outSTRM)
+        print("t,WeltX,WeltY,Veh. No,v,Gradient,veh.Typ-Id,Str-Id", file=outSTRM)
     vIDm = sumolib._Running(further["orig-ids"], True)
     vtIDm = sumolib._Running()
     vtIDm.g("PKW")
@@ -125,4 +126,4 @@ def vehicleTypes2flt(outSTRM, vtIDm):
     """
     for q in sorted(vtIDm._m):
         print("%s,%s,%s" %
-              (vtIDm.g(q), "<VEHDIR>\PC\PC_%s.GEN" % q, 1.), file=outSTRM)
+              (vtIDm.g(q), r"<VEHDIR>\PC\PC_%s.GEN" % q, 1.), file=outSTRM)
