@@ -1,27 +1,29 @@
-"""
-@file    rilsa1_out_tls.py
-@author  Daniel Krajzewicz
-@date    2014-09-01
-@version $Id: rilsa1_out_tls.py 22608 2017-01-17 06:28:54Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2012-2017 DLR (http://www.dlr.de/) and contributors
+# @file    rilsa1_out_tls.py
+# @author  Daniel Krajzewicz
+# @date    2014-09-01
 
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 
 from __future__ import absolute_import
 
 
-from . import *
+from . import fileNeedsRebuild, Scenario
 import os
 import shutil
+import subprocess
 import sumolib.net.generator.demand as demandGenerator
-from sumolib.net.generator.network import *
+import sumolib
 
 
 flowsRiLSA1 = [
@@ -66,8 +68,7 @@ class Scenario_RiLSA1OutTLS(Scenario):
         # network
         if fileNeedsRebuild(os.path.join(self.THIS_DIR, self.NET_FILE), "netconvert"):
             netconvert = sumolib.checkBinary("netconvert")
-            retCode = subprocess.call(
-                [netconvert, "-c", os.path.join(self.THIS_DIR, "build.netc.cfg")])
+            subprocess.call([netconvert, "-c", os.path.join(self.THIS_DIR, "build.netc.cfg")])
         # build the demand model (streams)
         if withDefaultDemand:
             self.demand = demandGenerator.Demand()
@@ -82,7 +83,10 @@ class Scenario_RiLSA1OutTLS(Scenario):
                     lkwNprob = prob - lkwEprob
 
                     self.demand.addStream(demandGenerator.Stream(f[0] + "__" + rel[0], 0, 3600, rel[1], f[0], rel[0],
-                                                                 {"passenger": pkwEprob, "COLOMBO_undetectable_passenger": pkwNprob, "hdv": lkwEprob, "COLOMBO_undetectable_hdv": lkwNprob}))
+                                                                 {"passenger": pkwEprob,
+                                                                  "COLOMBO_undetectable_passenger": pkwNprob,
+                                                                  "hdv": lkwEprob,
+                                                                  "COLOMBO_undetectable_hdv": lkwNprob}))
             if fileNeedsRebuild(self.fullPath("routes.rou.xml"), "duarouter"):
                 self.demand.build(
                     0, 3600, self.fullPath(self.NET_FILE), self.fullPath("routes.rou.xml"))
@@ -91,4 +95,4 @@ class Scenario_RiLSA1OutTLS(Scenario):
         self.netName = self.sandboxPath(self.NET_FILE)
         shutil.copy(
             self.fullPath(self.NET_FILE), self.sandboxPath(self.NET_FILE))
-        #shutil.copy(self.fullPath(self.TLS_FILE), self.sandboxPath(self.TLS_FILE))
+        # shutil.copy(self.fullPath(self.TLS_FILE), self.sandboxPath(self.TLS_FILE))
