@@ -1,54 +1,39 @@
 #!/usr/bin/env python
-"""
-A script for converting SUMO's fcd-output into files readable by PHEM and communication simulators.
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-@file    traceExporter.py
-@author  Daniel Krajzewicz
-@author  Jakob Erdmann
-@author  Michael Behrisch
-@date    2013-01-15
-@version $Id: traceExporter.py 18756 2015-08-31 19:16:33Z behrisch $
+# @file    traceExporter.py
+# @author  Daniel Krajzewicz
+# @author  Jakob Erdmann
+# @author  Michael Behrisch
+# @author  Evamarie Wiessner
+# @author  Mirko Barthauer
+# @date    2013-01-15
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2013-2015 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 
 from __future__ import print_function
+from __future__ import absolute_import
 import os
 import sys
+import gzip
 import random
 import datetime
 sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..', 'tools'))
 sys.path.append(os.path.join(os.environ.get(
     "SUMO_HOME", os.path.join(os.path.dirname(__file__), '..', '..')), 'tools'))
 
-import sumolib.net
-import sumolib.output.convert.phem as phem
-import sumolib.output.convert.omnet as omnet
-import sumolib.output.convert.shawn as shawn
-import sumolib.output.convert.ns2 as ns2
-import sumolib.output.convert.gpsdat as gpsdat
-import sumolib.output.convert.gpx as gpx
-import sumolib.output.convert.poi as poi
-
-
-class FCDVehicleEntry:
-
-    def __init__(self, id, x, y, z, speed, typev, lane, slope):
-        self.id = id
-        self.x = x
-        self.y = y
-        self.z = z
-        self.speed = speed
-        self.type = typev
-        self.lane = lane
-        self.slope = slope
+from sumolib.miscutils import getSocketStream  # noqa
+import sumolib.net  # noqa
+from sumolib.output.convert import phem, omnet, shawn, ns2, gpsdat, kml, gpx, poi, ipg, fcdfilter, keplerjson, trj  # noqa
 
 
 class FCDTimeEntry:
@@ -66,10 +51,12 @@ def disturb_gps(x, y, deviation):
     return x, y
 
 
-def _getOutputStream(name):
+def _getOutputStream(name, binary=False):
     if not name:
         return None
-    return open(name, "w")
+    if name.endswith(".gz"):
+        return gzip.open(name, "wt")
+    return open(name, "wb" if binary else "w")
 
 
 def _closeOutputStream(strm):
@@ -77,21 +64,41 @@ def _closeOutputStream(strm):
         strm.close()
 
 
-def procFCDStream(fcdstream, options):
-    pt = -1  # "prior" time step
-    lt = -1  # "last" time step
-    ft = -1  # "first" time step
-    lastExported = -1
-    chosen = {}
+def makeEntries(movables, chosen, options):
     if options.boundary:
         xmin, ymin, xmax, ymax = [float(e)
                                   for e in options.boundary.split(",")]
-    for i, q in enumerate(fcdstream):
+    result = []
+    negative = False
+    for v in movables:
+        if v.id not in chosen:
+            chosen[v.id] = random.random() < options.penetration
+        if chosen[v.id]:
+            v.x, v.y = disturb_gps(float(v.x), float(v.y), options.blur)
+            if v.x < 0 or v.y < 0:
+                if options.shift:
+                    v.x = round(v.x + float(options.shift), 2)
+                    v.y = round(v.y + float(options.shift), 2)
+                else:
+                    negative = True
+            if not v.z:
+                v.z = 0
+            if not options.boundary or (v.x >= xmin and v.x <= xmax and v.y >= ymin and v.y <= ymax):
+                if v.lane:
+                    v.edge = sumolib._laneID2edgeID(v.lane)
+                result.append(v)
+    return result, negative
+
+
+def procFCDStream(fcdstream, options):
+    pt = -1  # "prior" time step
+    lt = -1  # "last" time step
+    lastExported = -1
+    chosen = {}
+    hasWarning = False
+    for q in fcdstream:
         pt = lt
-        lt = float(q.time.encode("latin1"))
-        if ft < 0:
-            # this is the first step contained in the simulation
-            ft = lt  # save it for later purposes
+        lt = sumolib.miscutils.parseTime(q.time)
         if options.begin and options.begin > lt:
             continue  # do not export steps before a set begin
         if options.end and options.end <= lt:
@@ -101,33 +108,37 @@ def procFCDStream(fcdstream, options):
         lastExported = lt
         e = FCDTimeEntry(lt)
         if q.vehicle:
-            for v in q.vehicle:
-                if v.id not in chosen:
-                    chosen[v.id] = random.random() < options.penetration
-                if chosen[v.id]:
-                    x, y = disturb_gps(float(v.x), float(v.y), options.blur)
-                    if v.z:
-                        z = v.z
-                    else:
-                        z = 0
-                    if not options.boundary or (x >= xmin and x <= xmax and y >= ymin and y <= ymax):
-                        e.vehicle.append(
-                            FCDVehicleEntry(v.id, x, y, z, v.speed, v.type, v.lane, v.slope))
+            result, warning = makeEntries(q.vehicle, chosen, options)
+            e.vehicle += result
+            hasWarning = hasWarning or warning
+        if options.persons and q.person:
+            result, warning = makeEntries(q.person, chosen, options)
+            e.vehicle += result
+            hasWarning = hasWarning or warning
         yield e
     t = lt - pt + lt
+    if hasWarning:
+        print("One or more coordinates are negative, some applications might need strictly positive values. " +
+              "To avoid this use the option --shift.")
     yield FCDTimeEntry(t)
 
 
 def runMethod(inputFile, outputFile, writer, options, further={}):
     further["app"] = os.path.split(__file__)[1]
     further["orig-ids"] = options.orig_ids
+    further["ignore-gaps"] = options.ignore_gaps
     if options.base >= 0:
         further["base-date"] = datetime.datetime.fromtimestamp(options.base)
     else:
         further["base-date"] = datetime.datetime.now().replace(hour=0,
                                                                minute=0, second=0, microsecond=0)
-    o = _getOutputStream(outputFile)
-    fcdStream = sumolib.output.parse(inputFile, "timestep")
+    binaryOutput = True if options.trj else False
+    o = _getOutputStream(outputFile, binary=binaryOutput)
+    if inputFile.isdigit():
+        inp = getSocketStream(int(inputFile))
+    else:
+        inp = inputFile
+    fcdStream = sumolib.output.parse(inp, "timestep")
     ret = writer(procFCDStream(fcdStream, options), o, further)
     _closeOutputStream(o)
     return ret
@@ -136,7 +147,7 @@ def runMethod(inputFile, outputFile, writer, options, further={}):
 def main(args=None):
     """The main function; parses options and converts..."""
     # ---------- build and read options ----------
-    USAGE = """Usage %prog -i <FCDDATA> [options] 
+    USAGE = """Usage %prog -i <FCDDATA> [options]
 Converts the given fcd file (generated by sumo --fcd-output) into the selected
 output format. Optionally the output can be sampled, filtered and distorted.
 """
@@ -144,7 +155,8 @@ output format. Optionally the output can be sampled, filtered and distorted.
     from optparse import OptionParser
     optParser = OptionParser(usage=USAGE)
     optParser.add_option("-i", "--fcd-input", dest="fcd", metavar="FILE",
-                         help="Defines the FCD-output file to use as input")
+                         help="Defines the FCD-output file to use as input " +
+                              "(numeric value is interpreted as port to listen on)")
     optParser.add_option("-n", "--net-input", dest="net", metavar="FILE",
                          help="Defines the network file to use as input")
     optParser.add_option("-p", "--penetration", type="float", dest="penetration",
@@ -165,6 +177,10 @@ output format. Optionally the output can be sampled, filtered and distorted.
         "--base-date", dest="base", default=-1, type="int", help="Defines the base date")
     optParser.add_option("--orig-ids", dest="orig_ids", default=False, action="store_true",
                          help="Write original vehicle IDs instead of a running index")
+    optParser.add_option("--ignore-gaps", dest="ignore_gaps", default=False, action="store_true",
+                         help="Ignore steps where a vehicle is not in the network")
+    optParser.add_option("--persons", default=False, action="store_true",
+                         help="Include person data")
     # PHEM
     optParser.add_option("--dri-output", dest="dri", metavar="FILE",
                          help="Defines the name of the PHEM .dri-file to generate")
@@ -190,13 +206,41 @@ output format. Optionally the output can be sampled, filtered and distorted.
     # GPSDAT
     optParser.add_option("--gpsdat-output", dest="gpsdat", metavar="FILE",
                          help="Defines the name of the gpsdat file to generate")
-
+    # KML
+    optParser.add_option("--kml-output", dest="kml", metavar="FILE",
+                         help="Defines the name of the kml file to generate")
     # GPX
     optParser.add_option("--gpx-output", dest="gpx", metavar="FILE",
                          help="Defines the name of the gpx file to generate")
     # POI
     optParser.add_option("--poi-output", dest="poi", metavar="FILE",
                          help="Defines the name of the poi file to generate")
+    # FCD
+    optParser.add_option("--fcd-filter", dest="fcdfilter", metavar="FILE",
+                         help="Defines the name of the filter definition file")
+    # kepler.gl JSON
+    optParser.add_option("--keplerjson-output", dest="keplerjson", metavar="FILE",
+                         help="Defines the name of the kelper.gl JSON file to generate")
+    optParser.add_option("--fcd-filter-comment", dest="fcdcomment",
+                         help="Extra comments to include in fcd file")
+    optParser.add_option("--fcd-filter-type", dest="fcdtype",
+                         help="vehicle type to include in fcd file")
+    optParser.add_option("--shift", dest="shift",
+                         help="shift coordinates to postive values only")
+    # IPG
+    optParser.add_option("--ipg-output", dest="ipg", metavar="FILE",
+                         help="Defines the name of the ipg trace file to generate")
+
+    # TRJ
+    optParser.add_option("--trj-output", dest="trj", metavar="FILE",
+                         help="Defines the name of the trj file to generate")
+    optParser.add_option("--trj-veh-width", dest="trjvehwidth", default=1.7,
+                         type="float", help="Defines the assumed vehicle width")
+    optParser.add_option("--trj-vehicle-length", dest="trjvehlength", default=4.8,
+                         type="float", help="Defines the assumed vehicle length")
+    optParser.add_option("--timestep", dest="timestep", default=1.0,
+                         type="float", help="Used time step duration")
+
     # parse
     if len(args) == 1:
         sys.exit(USAGE.replace('%prog', os.path.basename(__file__)))
@@ -221,34 +265,57 @@ output format. Optionally the output can be sampled, filtered and distorted.
     if options.omnet and not options.fcd:
         print("A fcd-output from SUMO must be given using the --fcd-input.")
         return 1
+    # trj
+    if options.trj and not options.fcd:
+        print("A fcd-output from SUMO must be given using the --fcd-input.")
+        return 1
+    if options.trj and not options.net:
+        print("A SUMO network must be given using the --net-input option.")
+        return 1
     # ----- check needed values
 
-    ## ----- OMNET
+    # ----- OMNET
     if options.omnet:
         runMethod(options.fcd, options.omnet, omnet.fcd2omnet, options)
-    ## ----- OMNET
+    # ----- OMNET
 
-    ## ----- Shawn
+    # ----- Shawn
     if options.shawn:
         runMethod(options.fcd, options.shawn, shawn.fcd2shawn, options)
-    ## ----- Shawn
+    # ----- Shawn
 
-    ## ----- GPSDAT
+    # ----- GPSDAT
     if options.gpsdat:
         runMethod(options.fcd, options.gpsdat, gpsdat.fcd2gpsdat, options)
-    ## ----- GPSDAT
+    # ----- GPSDAT
 
-    ## ----- GPX
+    # ----- KML
+    if options.kml:
+        runMethod(options.fcd, options.kml, kml.fcd2kml, options)
+    # ----- KML
+
+    # ----- GPX
     if options.gpx:
         runMethod(options.fcd, options.gpx, gpx.fcd2gpx, options)
-    ## ----- GPX
+    # ----- GPX
 
-    ## ----- GPX
+    # ----- POI
     if options.poi:
         runMethod(options.fcd, options.poi, poi.fcd2poi, options)
-    ## ----- GPX
+    # ----- POI
 
-    ## ----- ns2
+    # ----- FCD
+    if options.fcdfilter:
+        runMethod(options.fcd, None, fcdfilter.fcdfilter, options,
+                  {"filter": options.fcdfilter, "comment": options.fcdcomment, "type": options.fcdtype})
+    # ----- FCD
+
+    # ----- kepler.gl JSON
+    if options.keplerjson:
+        runMethod(options.fcd, options.keplerjson, keplerjson.fcd2keplerjson, options)
+    # ----- kepler.gl JSON
+
+    # ----- ns2
     if options.ns2mobility or options.ns2config or options.ns2activity:
         vIDm, vehInfo, begin, end, area = runMethod(
             options.fcd, options.ns2mobility, ns2.fcd2ns2mobility, options)
@@ -261,9 +328,9 @@ output format. Optionally the output can be sampled, filtered and distorted.
         ns2.writeNS2config(
             o, vehInfo, options.ns2activity, options.ns2mobility, begin, end, area)
         _closeOutputStream(o)
-    ## ----- ns2
+    # ----- ns2
 
-    ## ----- PHEM
+    # ----- PHEM
     # .dri
     if options.dri:
         runMethod(options.fcd, options.dri, phem.fcd2dri, options)
@@ -286,7 +353,22 @@ output format. Optionally the output can be sampled, filtered and distorted.
         o = _getOutputStream(options.flt)
         phem.vehicleTypes2flt(o, vtIDm)
         _closeOutputStream(o)
-    ## ----- PHEM
+    # ----- PHEM
+
+    # ----- IPG
+    if options.ipg:
+        runMethod(options.fcd, options.ipg, ipg.fcd2ipg, options)
+    # ----- IPG
+
+    # ----- TRJ
+    if options.trj:
+        if not net:
+            net = sumolib.net.readNet(options.net)
+        runMethod(options.fcd, options.trj, trj.fcd2trj, options,
+                  {"length": options.trjvehlength, "width": options.trjvehwidth,
+                   "bbox": net.getBBoxXY(), "timestep": options.timestep})
+    # ----- TRJ
+
     return 0
 
 
