@@ -1,54 +1,25 @@
 # -*- coding: utf-8 -*-
-"""
-@file    edge.py
-@author  Michael Behrisch
-@author  Jakob Erdmann
-@date    2011-03-17
-@version $Id: _edge.py 21105 2016-07-04 12:48:27Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2011-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-Python implementation of the TraCI interface.
+# @file    _edge.py
+# @author  Michael Behrisch
+# @author  Jakob Erdmann
+# @date    2011-03-17
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 from __future__ import absolute_import
-import struct
 from . import constants as tc
 from .domain import Domain
-from .storage import Storage
-
-
-def _TIME2STEPS(time):
-    """Conversion from (float) time in seconds to milliseconds as int"""
-    return int(time * 1000)
-
-
-_RETURN_VALUE_FUNC = {tc.VAR_EDGE_TRAVELTIME:       Storage.readDouble,
-                      tc.VAR_WAITING_TIME:          Storage.readDouble,
-                      tc.VAR_EDGE_EFFORT:           Storage.readDouble,
-                      tc.VAR_CO2EMISSION:           Storage.readDouble,
-                      tc.VAR_COEMISSION:            Storage.readDouble,
-                      tc.VAR_HCEMISSION:            Storage.readDouble,
-                      tc.VAR_PMXEMISSION:           Storage.readDouble,
-                      tc.VAR_NOXEMISSION:           Storage.readDouble,
-                      tc.VAR_FUELCONSUMPTION:       Storage.readDouble,
-                      tc.VAR_NOISEEMISSION:         Storage.readDouble,
-                      tc.VAR_ELECTRICITYCONSUMPTION: Storage.readDouble,
-                      tc.LAST_STEP_MEAN_SPEED:      Storage.readDouble,
-                      tc.LAST_STEP_OCCUPANCY:       Storage.readDouble,
-                      tc.LAST_STEP_LENGTH:          Storage.readDouble,
-                      tc.VAR_CURRENT_TRAVELTIME:    Storage.readDouble,
-                      tc.LAST_STEP_VEHICLE_NUMBER:  Storage.readInt,
-                      tc.LAST_STEP_VEHICLE_HALTING_NUMBER: Storage.readInt,
-                      tc.LAST_STEP_VEHICLE_ID_LIST: Storage.readStringList,
-                      tc.LAST_STEP_PERSON_ID_LIST: Storage.readStringList,
-                      }
+from .exceptions import TraCIException
 
 
 class EdgeDomain(Domain):
@@ -57,23 +28,18 @@ class EdgeDomain(Domain):
         Domain.__init__(self, "edge", tc.CMD_GET_EDGE_VARIABLE, tc.CMD_SET_EDGE_VARIABLE,
                         tc.CMD_SUBSCRIBE_EDGE_VARIABLE, tc.RESPONSE_SUBSCRIBE_EDGE_VARIABLE,
                         tc.CMD_SUBSCRIBE_EDGE_CONTEXT, tc.RESPONSE_SUBSCRIBE_EDGE_CONTEXT,
-                        _RETURN_VALUE_FUNC)
+                        subscriptionDefault=(tc.LAST_STEP_VEHICLE_NUMBER,))
 
     def getAdaptedTraveltime(self, edgeID, time):
         """getAdaptedTraveltime(string, double) -> double
 
-        Returns the travel time value (in s) used for (re-)routing 
+        Returns the travel time value (in s) used for (re-)routing
         which is valid on the edge at the given time.
         """
-        self._connection._beginMessage(tc.CMD_GET_EDGE_VARIABLE, tc.VAR_EDGE_TRAVELTIME,
-                                       edgeID, 1 + 4)
-        self._connection._string += struct.pack(
-            "!Bi", tc.TYPE_INTEGER, _TIME2STEPS(time))
-        return self._connection._checkResult(tc.CMD_GET_EDGE_VARIABLE,
-                                             tc.VAR_EDGE_TRAVELTIME, edgeID).readDouble()
+        return self._getUniversal(tc.VAR_EDGE_TRAVELTIME, edgeID, "d", time)
 
     def getWaitingTime(self, edgeID):
-        """getWaitingTime() -> double 
+        """getWaitingTime(string) -> double
         Returns the sum of the waiting time of all vehicles currently on
         that edge (see traci.vehicle.getWaitingTime).
         """
@@ -82,15 +48,10 @@ class EdgeDomain(Domain):
     def getEffort(self, edgeID, time):
         """getEffort(string, double) -> double
 
-        Returns the effort value used for (re-)routing 
+        Returns the effort value used for (re-)routing
         which is valid on the edge at the given time.
         """
-        self._connection._beginMessage(tc.CMD_GET_EDGE_VARIABLE, tc.VAR_EDGE_EFFORT,
-                                       edgeID, 1 + 4)
-        self._connection._string += struct.pack(
-            "!Bi", tc.TYPE_INTEGER, _TIME2STEPS(time))
-        return self._connection._checkResult(tc.CMD_GET_EDGE_VARIABLE,
-                                             tc.VAR_EDGE_EFFORT, edgeID).readDouble()
+        return self._getUniversal(tc.VAR_EDGE_EFFORT, edgeID, "d", time)
 
     def getCO2Emission(self, edgeID):
         """getCO2Emission(string) -> double
@@ -158,7 +119,7 @@ class EdgeDomain(Domain):
     def getLastStepOccupancy(self, edgeID):
         """getLastStepOccupancy(string) -> double
 
-        Returns the occupancy in % for the last time step on the given edge.
+        Returns the net occupancy (excluding inter-vehicle gaps) in % for the last time step on the given edge.
         """
         return self._getUniversal(tc.LAST_STEP_OCCUPANCY, edgeID)
 
@@ -169,6 +130,20 @@ class EdgeDomain(Domain):
         """
         return self._getUniversal(tc.LAST_STEP_LENGTH, edgeID)
 
+    def getLaneNumber(self, edgeID):
+        """getLaneNumber(string) -> int
+
+        Returns the number of lanes of this edge
+        """
+        return self._getUniversal(tc.VAR_LANE_INDEX, edgeID)
+
+    def getStreetName(self, edgeID):
+        """getStreetName(string) -> string
+
+        Returns the street name of this edge
+        """
+        return self._getUniversal(tc.VAR_NAME, edgeID)
+
     def getTraveltime(self, edgeID):
         """getTraveltime(string) -> double
 
@@ -205,35 +180,63 @@ class EdgeDomain(Domain):
         """
         return self._getUniversal(tc.LAST_STEP_PERSON_ID_LIST, edgeID)
 
-    def adaptTraveltime(self, edgeID, time):
-        """adaptTraveltime(string, double) -> None
+    def getPendingVehicles(self, edgeID):
+        """getPendingVehicles(string) -> list(string)
+        Returns a list of all vehicle ids waiting for insertion on this edge (with depart delay)
+        """
+        return self._getUniversal(tc.VAR_PENDING_VEHICLES, edgeID)
+
+    def adaptTraveltime(self, edgeID, time, begin=None, end=None):
+        """adaptTraveltime(string, double, double, double) -> None
 
         Adapt the travel time value (in s) used for (re-)routing for the given edge.
-        """
-        self._connection._beginMessage(
-            tc.CMD_SET_EDGE_VARIABLE, tc.VAR_EDGE_TRAVELTIME, edgeID, 1 + 4 + 1 + 8)
-        self._connection._string += struct.pack("!BiBd",
-                                                tc.TYPE_COMPOUND, 1, tc.TYPE_DOUBLE, time)
-        self._connection._sendExact()
 
-    def setEffort(self, edgeID, effort):
-        """setEffort(string, double) -> None
+        When setting begin time and end time (in seconds), the changes only
+        apply to that time range. Otherwise they apply all the time
+        """
+        if begin is None and end is None:
+            self._setCmd(tc.VAR_EDGE_TRAVELTIME, edgeID, "td", 1, time)
+        elif begin is not None and end is not None:
+            self._setCmd(tc.VAR_EDGE_TRAVELTIME, edgeID, "tddd", 3, begin, end, time)
+        else:
+            raise TraCIException("Both, begin time and end time must be specified")
+
+    def setEffort(self, edgeID, effort, begin=None, end=None):
+        """setEffort(string, double, double, double) -> None
 
         Adapt the effort value used for (re-)routing for the given edge.
+
+        When setting begin time and end time (in seconds), the changes only
+        apply to that time range. Otherwise they apply all the time.
         """
-        self._connection._beginMessage(
-            tc.CMD_SET_EDGE_VARIABLE, tc.VAR_EDGE_EFFORT, edgeID, 1 + 4 + 1 + 8)
-        self._connection._string += struct.pack("!BiBd",
-                                                tc.TYPE_COMPOUND, 1, tc.TYPE_DOUBLE, effort)
-        self._connection._sendExact()
+        if begin is None and end is None:
+            self._setCmd(tc.VAR_EDGE_EFFORT, edgeID, "td", 1, effort)
+        elif begin is not None and end is not None:
+            self._setCmd(tc.VAR_EDGE_EFFORT, edgeID, "tddd", 3, begin, end, effort)
+        else:
+            raise TraCIException("Both, begin time and end time must be specified")
+
+    def setAllowed(self, edgeID, allowedClasses):
+        """setAllowed(string, list) -> None
+
+        Sets a list of allowed vehicle classes. Setting an empty list means all vehicles are allowed.
+        """
+        if isinstance(allowedClasses, str):
+            allowedClasses = [allowedClasses]
+        self._setCmd(tc.LANE_ALLOWED, edgeID, "l", allowedClasses)
+
+    def setDisallowed(self, edgeID, disallowedClasses):
+        """setDisallowed(string, list) -> None
+
+        Sets a list of disallowed vehicle classes.
+        """
+        if isinstance(disallowedClasses, str):
+            disallowedClasses = [disallowedClasses]
+        self._setCmd(tc.LANE_DISALLOWED, edgeID, "l", disallowedClasses)
 
     def setMaxSpeed(self, edgeID, speed):
         """setMaxSpeed(string, double) -> None
 
         Set a new maximum speed (in m/s) for all lanes of the edge.
         """
-        self._connection._sendDoubleCmd(
-            tc.CMD_SET_EDGE_VARIABLE, tc.VAR_MAXSPEED, edgeID, speed)
-
-
-EdgeDomain()
+        self._setCmd(tc.VAR_MAXSPEED, edgeID, "d", speed)
