@@ -1,28 +1,25 @@
 # -*- coding: utf-8 -*-
-"""
-@file    lane.py
-@author  Michael Behrisch
-@author  Daniel Krajzewicz
-@author  Laura Bieker
-@author  Jakob Erdmann
-@date    2011-03-17
-@version $Id: _lane.py 21105 2016-07-04 12:48:27Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2011-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-Python implementation of the TraCI interface.
+# @file    _lane.py
+# @author  Michael Behrisch
+# @author  Daniel Krajzewicz
+# @author  Laura Bieker
+# @author  Jakob Erdmann
+# @date    2011-03-17
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 from __future__ import absolute_import
-import struct
 from .domain import Domain
-from .storage import Storage
 from . import constants as tc
 
 
@@ -30,17 +27,17 @@ def _readLinks(result):
     result.read("!Bi")  # Type Compound, Length
     nbLinks = result.readInt()
     links = []
-    for i in range(nbLinks):
+    for _ in range(nbLinks):
         result.read("!B")                           # Type String
         approachedLane = result.readString()
         result.read("!B")                           # Type String
         approachedInternal = result.readString()
         result.read("!B")                           # Type Byte
-        hasPrio = bool(result.read("!B"))
+        hasPrio = bool(result.read("!B")[0])
         result.read("!B")                           # Type Byte
-        isOpen = bool(result.read("!B"))
+        isOpen = bool(result.read("!B")[0])
         result.read("!B")                           # Type Byte
-        hasFoe = bool(result.read("!B"))
+        hasFoe = bool(result.read("!B")[0])
         result.read("!B")                           # Type String
         state = result.readString()
         result.read("!B")                           # Type String
@@ -52,31 +49,7 @@ def _readLinks(result):
     return links
 
 
-_RETURN_VALUE_FUNC = {tc.VAR_LENGTH:                Storage.readDouble,
-                      tc.VAR_MAXSPEED:              Storage.readDouble,
-                      tc.VAR_WIDTH:                 Storage.readDouble,
-                      tc.LANE_ALLOWED:              Storage.readStringList,
-                      tc.LANE_DISALLOWED:           Storage.readStringList,
-                      tc.LANE_LINK_NUMBER: lambda result: result.read("!B")[0],
-                      tc.LANE_LINKS:                _readLinks,
-                      tc.VAR_SHAPE:                 Storage.readShape,
-                      tc.LANE_EDGE_ID:              Storage.readString,
-                      tc.VAR_CO2EMISSION:           Storage.readDouble,
-                      tc.VAR_COEMISSION:            Storage.readDouble,
-                      tc.VAR_HCEMISSION:            Storage.readDouble,
-                      tc.VAR_PMXEMISSION:           Storage.readDouble,
-                      tc.VAR_NOXEMISSION:           Storage.readDouble,
-                      tc.VAR_FUELCONSUMPTION:       Storage.readDouble,
-                      tc.VAR_NOISEEMISSION:         Storage.readDouble,
-                      tc.VAR_ELECTRICITYCONSUMPTION: Storage.readDouble,
-                      tc.LAST_STEP_MEAN_SPEED:      Storage.readDouble,
-                      tc.LAST_STEP_OCCUPANCY:       Storage.readDouble,
-                      tc.LAST_STEP_LENGTH:          Storage.readDouble,
-                      tc.VAR_WAITING_TIME:          Storage.readDouble,
-                      tc.VAR_CURRENT_TRAVELTIME:    Storage.readDouble,
-                      tc.LAST_STEP_VEHICLE_NUMBER:  Storage.readInt,
-                      tc.LAST_STEP_VEHICLE_HALTING_NUMBER: Storage.readInt,
-                      tc.LAST_STEP_VEHICLE_ID_LIST: Storage.readStringList}
+_RETURN_VALUE_FUNC = {tc.LANE_LINKS: _readLinks}
 
 
 class LaneDomain(Domain):
@@ -85,7 +58,7 @@ class LaneDomain(Domain):
         Domain.__init__(self, "lane", tc.CMD_GET_LANE_VARIABLE, tc.CMD_SET_LANE_VARIABLE,
                         tc.CMD_SUBSCRIBE_LANE_VARIABLE, tc.RESPONSE_SUBSCRIBE_LANE_VARIABLE,
                         tc.CMD_SUBSCRIBE_LANE_CONTEXT, tc.RESPONSE_SUBSCRIBE_LANE_CONTEXT,
-                        _RETURN_VALUE_FUNC)
+                        _RETURN_VALUE_FUNC, subscriptionDefault=(tc.LAST_STEP_VEHICLE_NUMBER,))
 
     def getLength(self, laneID):
         """getLength(string) -> double
@@ -129,13 +102,23 @@ class LaneDomain(Domain):
         """
         return self._getUniversal(tc.LANE_LINK_NUMBER, laneID)
 
-    def getLinks(self, laneID, extended=False):
+    def getLinks(self, laneID, extended=True):
         """getLinks(string) -> list((string, bool, bool, bool))
         A list containing id of successor lane together with priority, open and foe
         for each link.
         if extended=True, each result tuple contains
-        (string approachedLane, bool hasPrio, bool isOpen, bool hasFoe, 
+        (string approachedLane, bool hasPrio, bool isOpen, bool hasFoe,
         string approachedInternal, string state, string direction, float length)
+
+        isOpen: whether a vehicle driving at the speed limit (minimum auf
+                incoming and outgoing lane) could safely pass the junction with
+                regard to approaching foes if it were to enter it in this step
+                (false for red traffic light).
+                Foe vehicles that are already on the junction are ignored!
+        hasPrio: whether the link is the main road at a priority junction or
+                 currently has green light ('G')
+        hasFoe: whether any foe vehicles are approaching the junction or on the
+                junction that would interfere with passing it in the current time step
         """
         complete_data = self._getUniversal(tc.LANE_LINKS, laneID)
         if extended:
@@ -271,6 +254,24 @@ class LaneDomain(Domain):
         """
         return self._getUniversal(tc.LAST_STEP_VEHICLE_ID_LIST, laneID)
 
+    def getFoes(self, laneID, toLaneID):
+        """getFoes(string, string) -> list(string)
+        Returns the ids of incoming lanes that have right of way over the connection from laneID to toLaneID
+        """
+        return self._getUniversal(tc.VAR_FOES, laneID, "s", toLaneID)
+
+    def getInternalFoes(self, laneID):
+        """getFoes(string) -> list(string)
+        Returns the ids of internal lanes that are in conflict with the given internal lane id
+        """
+        return self.getFoes(laneID, "")
+
+    def getPendingVehicles(self, laneID):
+        """getPendingVehicles(string) -> list(string)
+        Returns a list of all vehicle ids waiting for insertion on this lane (with depart delay)
+        """
+        return self._getUniversal(tc.VAR_PENDING_VEHICLES, laneID)
+
     def setAllowed(self, laneID, allowedClasses):
         """setAllowed(string, list) -> None
 
@@ -278,10 +279,7 @@ class LaneDomain(Domain):
         """
         if isinstance(allowedClasses, str):
             allowedClasses = [allowedClasses]
-        self._connection._beginMessage(tc.CMD_SET_LANE_VARIABLE, tc.LANE_ALLOWED, laneID,
-                                       1 + 4 + sum(map(len, allowedClasses)) + 4 * len(allowedClasses))
-        self._connection._packStringList(allowedClasses)
-        self._connection._sendExact()
+        self._setCmd(tc.LANE_ALLOWED, laneID, "l", allowedClasses)
 
     def setDisallowed(self, laneID, disallowedClasses):
         """setDisallowed(string, list) -> None
@@ -290,26 +288,18 @@ class LaneDomain(Domain):
         """
         if isinstance(disallowedClasses, str):
             disallowedClasses = [disallowedClasses]
-        self._connection._beginMessage(tc.CMD_SET_LANE_VARIABLE, tc.LANE_DISALLOWED, laneID,
-                                       1 + 4 + sum(map(len, disallowedClasses)) + 4 * len(disallowedClasses))
-        self._connection._packStringList(disallowedClasses)
-        self._connection._sendExact()
+        self._setCmd(tc.LANE_DISALLOWED, laneID, "l", disallowedClasses)
 
     def setMaxSpeed(self, laneID, speed):
         """setMaxSpeed(string, double) -> None
 
         Sets a new maximum allowed speed on the lane in m/s.
         """
-        self._connection._sendDoubleCmd(
-            tc.CMD_SET_LANE_VARIABLE, tc.VAR_MAXSPEED, laneID, speed)
+        self._setCmd(tc.VAR_MAXSPEED, laneID, "d", speed)
 
     def setLength(self, laneID, length):
         """setLength(string, double) -> None
 
         Sets the length of the lane in m.
         """
-        self._connection._sendDoubleCmd(
-            tc.CMD_SET_LANE_VARIABLE, tc.VAR_LENGTH, laneID, length)
-
-
-LaneDomain()
+        self._setCmd(tc.VAR_LENGTH, laneID, "d", length)
