@@ -1,40 +1,38 @@
 #!/usr/bin/env python
-"""
-@file    mpl_dump_onNet.py
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@author  Jakob Erdmann
-@date    2007-10-25
-@version $Id: mpl_dump_onNet.py 20482 2016-04-18 20:49:42Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2007-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    mpl_dump_onNet.py
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @author  Jakob Erdmann
+# @date    2007-10-25
+
+"""
 
 This script reads a network and a dump file and
  draws the network, coloring it by the values
  found within the dump-file.
 
 matplotlib has to be installed for this purpose
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2007-2016 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from __future__ import print_function
 
 from matplotlib import rcParams
-from pylab import *
-import os
-import string
-import sys
-import StringIO
+from pylab import arange, figure, plot, savefig, show, xlim, xticks, ylim, yticks
 import math
 from optparse import OptionParser
-from xml.sax import saxutils, make_parser, handler
+from xml.sax import make_parser, handler
 
 
 def toHex(val):
@@ -175,7 +173,6 @@ class NetReader(handler.ContentHandler):
                 ys.append(float(self._node2y[self._edge2to[edge]]))
             else:
                 shape = self._edge2shape[edge].split(" ")
-                l = []
                 for s in shape:
                     p = s.split(",")
                     xs.append(float(p[0]))
@@ -219,9 +216,9 @@ class NetReader(handler.ContentHandler):
             rcParams['backend'] = 'Agg'
         # set figure size
         if options.size and not options.show:
-            f = figure(figsize=(options.size.split(",")))
+            figure(figsize=(options.size.split(",")))
         else:
-            f = figure()
+            figure()
         for edge in edge2plotLines:
             plot(edge2plotLines[edge][0], edge2plotLines[edge][
                  1], color=edge2plotColors[edge], linewidth=edge2plotWidth[edge])
@@ -387,9 +384,11 @@ class WeightsReader(handler.ContentHandler):
         else:
             for i in weights._intervalBegins:
                 self.valueDependantNorm(self._unaggEdge2value1[
-                                        i], self._minValue1, self._maxValue1, False, percSpeed and self._value1 == "speed")
+                                        i], self._minValue1, self._maxValue1, False, percSpeed and
+                                        self._value1 == "speed")
                 self.valueDependantNorm(self._unaggEdge2value2[
-                                        i], self._minValue2, self._maxValue2, tendency, percSpeed and self._value2 == "speed")
+                                        i], self._minValue2, self._maxValue2, tendency, percSpeed and
+                                        self._value2 == "speed")
 
 
 # initialise
@@ -425,11 +424,11 @@ optParser.add_option("--color-map", dest="colormap",
 # axes/legend
 optParser.add_option("--xticks", dest="xticks", type="string", default="",
                      help="defines ticks on x-axis")
-optParser.add_option("--yticks", dest="yticks", type="string",  default="",
+optParser.add_option("--yticks", dest="yticks", type="string", default="",
                      help="defines ticks on y-axis")
-optParser.add_option("--xlim", dest="xlim", type="string",  default="",
+optParser.add_option("--xlim", dest="xlim", type="string", default="",
                      help="defines x-axis range")
-optParser.add_option("--ylim", dest="ylim", type="string",  default="",
+optParser.add_option("--ylim", dest="ylim", type="string", default="",
                      help="defines y-axis range")
 # output
 optParser.add_option("--size", dest="size", type="string", default="",
