@@ -1,38 +1,35 @@
 #!/usr/bin/env python
-"""
-@file    mpl_dump_timeline.py
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@date    2007-10-25
-@version $Id: mpl_dump_timeline.py 23178 2017-03-02 08:27:45Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2008-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    mpl_dump_timeline.py
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @date    2007-10-25
+
+"""
 
 This script reads the value of designated edges from
  a set of given dump-files and displays them.
 
 matplotlib has to be installed for this purpose
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2008-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from __future__ import print_function
 
 from matplotlib import rcParams
-from pylab import *
-import os
-import string
-import sys
-import StringIO
-import math
+from pylab import arange, figure, plot, savefig, show, xlim, xticks, ylim, yticks
 from optparse import OptionParser
-from xml.sax import saxutils, make_parser, handler
+from xml.sax import make_parser, handler
 
 
 def updateMinMax(min, max, value):
@@ -63,6 +60,7 @@ class WeightsReader(handler.ContentHandler):
             if id in self._edges:
                 self._edge2value[self._time][id] = float(attrs[self._value])
 
+
 # initialise
 optParser = OptionParser()
 optParser.add_option("-v", "--verbose", action="store_true", dest="verbose",
@@ -143,7 +141,7 @@ for dump in weights:
         yss.append(ys)
         cs.append(colors[index])
         index = index + 1
-     # plot
+    # plot
 print("data x-range: " + str(xmin) + " - " + str(xmax))
 print("data y-range: " + str(ymin) + " - " + str(ymax))
 for i in range(0, len(cs)):
