@@ -1,38 +1,35 @@
 #!/usr/bin/env python
-"""
-@file    mpl_dump_twoAgainst.py
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@date    2007-10-25
-@version $Id: mpl_dump_twoAgainst.py 20433 2016-04-13 08:00:14Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2008-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    mpl_dump_twoAgainst.py
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @date    2007-10-25
+
+"""
 
 This script reads two dump files and plots one of the values
  stored therein as an x-/y- plot.
 
 matplotlib has to be installed for this purpose
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2008-2016 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from __future__ import print_function
 
 from matplotlib import rcParams
-from pylab import *
-import os
-import string
-import sys
-import StringIO
-import math
+from pylab import arange, figure, iterable, plot, savefig, show, xlim, xticks, ylim, yticks
 from optparse import OptionParser
-from xml.sax import saxutils, make_parser, handler
+from xml.sax import make_parser, handler
 
 
 def toHex(val):
@@ -48,9 +45,9 @@ def toColor(val):
 
 
 def updateMinMax(min, max, value):
-    if min == None or min > value:
+    if min is None or min > value:
         min = value
-    if max == None or max < value:
+    if max is None or max < value:
         max = value
     return (min, max)
 
@@ -76,6 +73,7 @@ class WeightsReader(handler.ContentHandler):
             self._edge2value[self._time][self._id] = self._edge2value[
                 self._time][self._id] + float(attrs[self._value])
 
+
 # initialise
 optParser = OptionParser()
 optParser.add_option("-v", "--verbose", action="store_true", dest="verbose",
@@ -101,11 +99,11 @@ optParser.add_option("-C", "--time-coloring", action="store_true", dest="time_co
 # axes/legend
 optParser.add_option("--xticks", dest="xticks", type="string", default="",
                      help="defines ticks on x-axis")
-optParser.add_option("--yticks", dest="yticks", type="string",  default="",
+optParser.add_option("--yticks", dest="yticks", type="string", default="",
                      help="defines ticks on y-axis")
-optParser.add_option("--xlim", dest="xlim", type="string",  default="",
+optParser.add_option("--xlim", dest="xlim", type="string", default="",
                      help="defines x-axis range")
-optParser.add_option("--ylim", dest="ylim", type="string",  default="",
+optParser.add_option("--ylim", dest="ylim", type="string", default="",
                      help="defines y-axis range")
 # parse options
 (options, args) = optParser.parse_args()
@@ -195,7 +193,7 @@ else:
                         min, max, weights1._edge2value[t][edge])
                     (min, max) = updateMinMax(
                         min, max, weights2._edge2value[t][edge])
-     # plot
+    # plot
 print("data range: " + str(min) + " - " + str(max))
 if options.verbose:
     print("Plotting...")
