@@ -1,38 +1,37 @@
 #!/usr/bin/env python
-"""
-@file    plot_csv_bars.py
-@author  Daniel Krajzewicz
-@date    2014-01-27
-@version $Id: plot_csv_bars.py 23178 2017-03-02 08:27:45Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2008-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    plot_csv_bars.py
+# @author  Daniel Krajzewicz
+# @date    2014-01-27
+
+"""
 
 This script plots name / value pairs from a given .csv file (';'-separated).
 The values are plotted as bars.
 matplotlib (http://matplotlib.org/) has to be installed for this purpose
 
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2008-2016 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from __future__ import print_function
 
 import os
-import subprocess
 import sys
-import random
 
 sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
-import sumolib
-from sumolib.visualization import helpers
-
-import matplotlib.pyplot as plt
+import sumolib  # noqa
+from sumolib.visualization import helpers  # noqa
+import matplotlib.pyplot as plt  # noqa
 
 
 def main(args=None):
@@ -127,5 +126,6 @@ def main(args=None):
         plt.xticks(ts, labels)
     helpers.closeFigure(fig, ax, options, False)
 
+
 if __name__ == "__main__":
     sys.exit(main(sys.argv))
