@@ -1,39 +1,38 @@
 #!/usr/bin/env python
-"""
-@file    plot_csv_pie.py
-@author  Daniel Krajzewicz
-@author  Laura Bieker
-@date    2014-01-15
-@version $Id: plot_csv_pie.py 23178 2017-03-02 08:27:45Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2014-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    plot_csv_pie.py
+# @author  Daniel Krajzewicz
+# @author  Laura Bieker
+# @date    2014-01-15
+
+"""
 
 This script plots name / value pairs from a given .csv file (';'-separated).
 The values are plotted as a pie diagram.
 matplotlib (http://matplotlib.org/) has to be installed for this purpose
 
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2014-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from __future__ import print_function
 
 import os
-import subprocess
 import sys
-import random
 
 sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
-import sumolib
-from sumolib.visualization import helpers
-
-import matplotlib.pyplot as plt
+import sumolib  # noqa
+from sumolib.visualization import helpers  # noqa
+import matplotlib.pyplot as plt  # noqa
 
 
 def main(args=None):
@@ -89,12 +88,17 @@ def main(args=None):
         labels = None
     shadow = options.shadow
     if options.percentage:
-        autopct = lambda p: '{:.1f}%'.format(p)
+        def autopct(p):
+            return '{:.1f}%'.format(p)
+        # autopct = lambda p: '{:.1f}%'.format(p)
     else:
-        autopct = lambda p: '{:.0f}'.format(p * total / 100)
+        def autopct(p):
+            return '{:.0f}'.format(p * total / 100)
+        # autopct = lambda p: '{:.0f}'.format(p * total / 100)
     patches, texts, autotexts = plt.pie(
         vals, labels=labels, autopct=autopct, colors=colors, shadow=shadow, startangle=options.startangle)
     helpers.closeFigure(fig, ax, options)
 
+
 if __name__ == "__main__":
     sys.exit(main(sys.argv))
