@@ -1,34 +1,30 @@
 #!/usr/bin/env python
-"""
-@file    plot_net_selection.py
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@date    2014-02-19
-@version $Id: plot_net_selection.py 23178 2017-03-02 08:27:45Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2008-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2008-2017 DLR (http://www.dlr.de/) and contributors
+# @file    plot_net_selection.py
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @date    2014-02-19
 
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 from __future__ import absolute_import
 from __future__ import print_function
 
 import os
-import subprocess
 import sys
-import random
-from xml.sax.handler import ContentHandler
 
 sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
-import sumolib
-from sumolib.visualization import helpers
-
-import matplotlib.pyplot as plt
+import sumolib  # noqa
+from sumolib.visualization import helpers  # noqa
 
 
 def main(args=None):
