@@ -1,37 +1,35 @@
 #!/usr/bin/env python
-"""
-@file    plot_tripinfo_distributions.py
-@author  Daniel Krajzewicz
-@author  Laura Bieker
-@date    2013-11-11
-@version $Id: plot_tripinfo_distributions.py 23178 2017-03-02 08:27:45Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
+# @file    plot_tripinfo_distributions.py
+# @author  Daniel Krajzewicz
+# @author  Laura Bieker
+# @date    2013-11-11
+
+"""
 This script plots measures from the tripinfo output, classified into bins
 matplotlib (http://matplotlib.org/) has to be installed for this purpose
-
-
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2013-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
 """
 from __future__ import absolute_import
 from __future__ import print_function
 
 import os
-import subprocess
 import sys
-import random
 
-sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
-import sumolib
-from sumolib.visualization import helpers
-
-import matplotlib.pyplot as plt
+sys.path.append(os.path.join(os.environ['SUMO_HOME'], 'tools'))
+import sumolib  # noqa
+from sumolib.visualization import helpers  # noqa
+import matplotlib.pyplot as plt  # noqa
 
 
 def main(args=None):
@@ -57,7 +55,7 @@ def main(args=None):
     helpers.addInteractionOptions(optParser)
     helpers.addPlotOptions(optParser)
     # parse
-    options, remaining_args = optParser.parse_args(args=args)
+    options, _ = optParser.parse_args(args=args)
 
     if options.tripinfos is None:
         print("Error: at least one tripinfo file must be given")
@@ -101,11 +99,11 @@ def main(args=None):
     fig, ax = helpers.openFigure(options)
     for i, f in enumerate(files):
         c = helpers.getColor(options, i, len(files))
-        l = helpers.getLabel(f, i, options)
-        plt.bar(center, hists[f], width=width, label=l, color=c)
-        for j in range(0, options.bins):
-            center[j] = center[j] + width
+        plt.bar(center, hists[f], width=width, label=helpers.getLabel(f, i, options), color=c)
+        for j in range(options.bins):
+            center[j] += width
     helpers.closeFigure(fig, ax, options)
 
+
 if __name__ == "__main__":
     sys.exit(main(sys.argv))
