@@ -1,22 +1,21 @@
 #!/usr/bin/env python
-"""
-@file    addSchema.py
-@author  Daniel Krajzewicz
-@author  Michael Behrisch
-@date    2010
-@version $Id: addSchema.py 20433 2016-04-13 08:00:14Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-Inserts XML schema references.
+# @file    addSchema.py
+# @author  Daniel Krajzewicz
+# @author  Michael Behrisch
+# @date    2010
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2010-2016 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 from __future__ import absolute_import
 from __future__ import print_function
 
@@ -37,7 +36,7 @@ proc = {
 srcRoot = os.path.join(os.path.dirname(__file__), "..", "..")
 
 for root, dirs, files in os.walk(srcRoot):
-    for pattern, repTo in proc.iteritems():
+    for pattern, repTo in proc.items():
         for name in glob.glob(os.path.join(root, pattern)):
             repFrom = repTo[:repTo.find(' ')]
             print("Patching '%s'" % name)
