@@ -1,34 +1,33 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-"""
-@file    csv2xml.py
-@author  Jakob Erdmann
-@author  Michael Behrisch
-@author  Laura Bieker
-@date    2013-12-08
-@version $Id: csv2xml.py 23948 2017-04-14 17:50:54Z namdre $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2013-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-Convert csv files to selected xml input files for SUMO
+# @file    csv2xml.py
+# @author  Jakob Erdmann
+# @author  Michael Behrisch
+# @author  Laura Bieker
+# @date    2013-12-08
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2013-2017 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 
 from __future__ import print_function
 from __future__ import absolute_import
 import os
 import sys
-PY3 = sys.version_info > (3,)
 import csv
 import contextlib
+import io
 
-from collections import defaultdict, OrderedDict
+from collections import OrderedDict
 from optparse import OptionParser
 
 import xsd
@@ -53,8 +52,7 @@ def get_options():
         optParser.print_help()
         sys.exit()
     if not options.xsd and not options.type:
-        print(
-            "either a schema or a type needs to be specified", file=sys.stderr)
+        print("either a schema or a type needs to be specified", file=sys.stderr)
         sys.exit()
     options.source = args[0]
     if not options.output:
@@ -64,8 +62,8 @@ def get_options():
 
 def row2xml(row, tag, close="/>\n", depth=1):
     attrString = ' '.join(['%s="%s"' % (a[len(tag) + 1:], v)
-                           for a, v in row.items() if v != "" and a.startswith(tag)])
-    return ('%s<%s %s%s' % ((depth * '    '), tag, attrString, close))
+                           for a, v in row.items() if v != "" and a.startswith(tag + "_")])
+    return (u'%s<%s %s%s' % ((depth * '    '), tag, attrString, close))
 
 
 def row2vehicle_and_route(row, tag):
@@ -73,23 +71,23 @@ def row2vehicle_and_route(row, tag):
         return row2xml(row, tag)
     else:
         edges = row.get("route_edges", "MISSING_VALUE")
-        return ('    <%s %s>\n        <route edges="%s"/>\n    </%s>\n' % (
-            tag,
-            ' '.join(['%s="%s"' % (a[len(tag) + 1:], v)
-                      for a, v in row.items() if v != "" and a != "route_edges"]),
-            edges, tag))
+        attrs = ' '.join(['%s="%s"' % (a[len(tag) + 1:], v)
+                          for a, v in sorted(row.items()) if v != "" and a != "route_edges"])
+        return (u'    <%s %s>\n        <route edges="%s"/>\n    </%s>\n' % (tag, attrs, edges, tag))
 
 
 def write_xml(toptag, tag, options, printer=row2xml):
-    with open(options.output, 'w') as outputf:
-        outputf.write('<%s>\n' % toptag)
-        if (options.source.isdigit()):
-            inputf = getSocketStream(int(options.source))
+    with io.open(options.output, 'w', encoding="utf8") as outputf:
+        outputf.write(u'<%s>\n' % toptag)
+        if options.source.isdigit():
+            inputf = xml2csv.getSocketStream(int(options.source))
         else:
-            inputf = open(options.source)
-        for row in csv.DictReader(inputf, delimiter=options.delimiter):
-            outputf.write(printer(row, tag))
-        outputf.write('</%s>\n' % toptag)
+            inputf = io.open(options.source, encoding="utf8")
+        reader = csv.DictReader(inputf, delimiter=options.delimiter)
+        for row in reader:
+            orderedRow = OrderedDict([(key, row[key]) for key in reader.fieldnames])
+            outputf.write(printer(orderedRow, tag))
+        outputf.write(u'</%s>\n' % toptag)
 
 
 def checkAttributes(out, old, new, ele, tagStack, depth):
@@ -97,22 +95,19 @@ def checkAttributes(out, old, new, ele, tagStack, depth):
         name = "%s_%s" % (ele.name, attr.name)
         if new.get(name, "") != "":
             if depth > 0:
-                out.write(str.encode(">\n"))
-            if(PY3):
-                out.write(str.encode(row2xml(new, ele.name, "", depth)))
-            else:
-                out.write(row2xml(new, ele.name, "", depth))
+                out.write(u">\n")
+            out.write(row2xml(new, ele.name, "", depth))
             return True
     return False
 
 
 def checkChanges(out, old, new, currEle, tagStack, depth):
-    #print(depth, currEle.name, tagStack)
+    # print(depth, currEle.name, tagStack)
     if depth >= len(tagStack):
         for ele in currEle.children:
-            #print(depth, "try", ele.name)
+            # print(depth, "try", ele.name)
             if ele.name not in tagStack and checkAttributes(out, old, new, ele, tagStack, depth):
-                #print(depth, "adding", ele.name, ele.children)
+                # print(depth, "adding", ele.name, ele.children)
                 tagStack.append(ele.name)
                 if ele.children:
                     checkChanges(out, old, new, ele, tagStack, depth + 1)
@@ -128,22 +123,14 @@ def checkChanges(out, old, new, currEle, tagStack, depth):
                     changed = True
                 if new.get(name, "") != "":
                     present = True
-            #print(depth, "seeing", ele.name, changed, tagStack)
+            # print(depth, "seeing", ele.name, changed, tagStack)
             if changed:
-                out.write(str.encode("/>\n"))
+                out.write(u"/>\n")
                 del tagStack[-1]
                 while len(tagStack) > depth:
-                    if(PY3):
-                        out.write(str.encode("%s</%s>\n" %
-                                             ((len(tagStack) - 1) * '    ', tagStack[-1])))
-                    else:
-                        out.write("%s</%s>\n" %
-                                  ((len(tagStack) - 1) * '    ', tagStack[-1]))
+                    out.write(u"%s</%s>\n" % ((len(tagStack) - 1) * '    ', tagStack[-1]))
                     del tagStack[-1]
-                if(PY3):
-                    out.write(str.encode(row2xml(new, ele.name, "", depth)))
-                else:
-                    out.write(row2xml(new, ele.name, "", depth))
+                out.write(row2xml(new, ele.name, "", depth))
                 tagStack.append(ele.name)
                 changed = False
             if present and ele.children:
@@ -157,14 +144,11 @@ def writeHierarchicalXml(struct, options):
         if options.source.isdigit():
             inputf = xml2csv.getSocketStream(int(options.source))
         else:
-            inputf = open(options.source)
+            inputf = io.open(options.source, encoding="utf8")
         lastRow = OrderedDict()
         tagStack = [struct.root.name]
         if options.skip_root:
-            if(PY3):
-                outputf.write(str.encode('<%s' % struct.root.name))
-            else:
-                outputf.write('<%s' % struct.root.name)
+            outputf.write(u'<%s' % struct.root.name)
         fields = None
         enums = {}
         first = True
@@ -172,7 +156,7 @@ def writeHierarchicalXml(struct, options):
             if not fields:
                 fields = raw
                 for f in fields:
-                    if not '_' in f:
+                    if '_' not in f:
                         continue
                     enum = struct.getEnumerationByAttr(*f.split('_', 1))
                     if enum:
@@ -184,18 +168,13 @@ def writeHierarchicalXml(struct, options):
                         entry = enums[field][int(entry)]
                     row[field] = entry
                 if first and not options.skip_root:
-                    checkAttributes(
-                        outputf, lastRow, row, struct.root, tagStack, 0)
+                    checkAttributes(outputf, lastRow, row, struct.root, tagStack, 0)
                     first = False
                 checkChanges(outputf, lastRow, row, struct.root, tagStack, 1)
                 lastRow = row
-        outputf.write(str.encode("/>\n"))
+        outputf.write(u"/>\n")
         for idx in range(len(tagStack) - 2, -1, -1):
-            if(PY3):
-                outputf.write(
-                    str.encode("%s</%s>\n" % (idx * '    ', tagStack[idx])))
-            else:
-                outputf.write("%s</%s>\n" % (idx * '    ', tagStack[idx]))
+            outputf.write(u"%s</%s>\n" % (idx * '    ', tagStack[idx]))
 
 
 def main():
