@@ -1,34 +1,40 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-"""
-@file    protobuf2xml.py
-@author  Michael Behrisch
-@author  Laura Bieker
-@author  Jakob Erdmann
-@date    2014-01-23
-@version $Id: protobuf2xml.py 17235 2014-11-03 10:53:02Z behrisch $
+# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+# Copyright (C) 2014-2021 German Aerospace Center (DLR) and others.
+# This program and the accompanying materials are made available under the
+# terms of the Eclipse Public License 2.0 which is available at
+# https://www.eclipse.org/legal/epl-2.0/
+# This Source Code may also be made available under the following Secondary
+# Licenses when the conditions for such availability set forth in the Eclipse
+# Public License 2.0 are satisfied: GNU General Public License, version 2
+# or later which is available at
+# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 
-Convert protobuf files to selected xml input files for SUMO
+# @file    protobuf2xml.py
+# @author  Michael Behrisch
+# @author  Laura Bieker
+# @author  Jakob Erdmann
+# @date    2014-01-23
 
-SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2014-2014 DLR (http://www.dlr.de/) and contributors
-
-This file is part of SUMO.
-SUMO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-"""
 
 from __future__ import print_function
-import os, sys, struct, contextlib
+from __future__ import absolute_import
+import os
+import sys
+import struct
+import contextlib
 
 from optparse import OptionParser
 import google.protobuf.descriptor
-import xsd, xml2csv, xml2protobuf
+import xml2csv
+import xml2protobuf
+
 
 def get_options():
-    optParser = OptionParser(usage=os.path.basename(sys.argv[0])+" [<options>] <input_file_or_port>")
+    optParser = OptionParser(
+        usage=os.path.basename(sys.argv[0]) + " [<options>] <input_file_or_port>")
     optParser.add_option("-p", "--protodir", default=".",
                          help="where to put and read .proto files")
     optParser.add_option("-x", "--xsd", help="xsd schema to use")
@@ -45,12 +51,13 @@ def get_options():
         options.output = os.path.splitext(options.source)[0] + ".xml"
     return options
 
+
 def read_n(inputf, n):
     """ Read exactly n bytes from the input.
         Raise RuntimeError if the stream ended before
         n bytes were read.
     """
-    buf = ''
+    buf = b''
     while n > 0:
         data = inputf.read(n)
         if data == '':
@@ -59,8 +66,9 @@ def read_n(inputf, n):
         n -= len(data)
     return buf
 
+
 def msg2xml(desc, cont, out, depth=1):
-    out.write(">\n%s<%s" % (depth*'    ', desc.name))
+    out.write(u">\n%s<%s" % (depth * '    ', desc.name))
     haveChildren = False
 #    print(depth, cont)
     for attr, value in cont.ListFields():
@@ -68,21 +76,22 @@ def msg2xml(desc, cont, out, depth=1):
             if attr.label == google.protobuf.descriptor.FieldDescriptor.LABEL_REPEATED:
                 haveChildren = True
                 for item in value:
-                    msg2xml(attr, item, out, depth+1)
+                    msg2xml(attr, item, out, depth + 1)
         else:
             if attr.type == google.protobuf.descriptor.FieldDescriptor.TYPE_ENUM:
                 value = attr.enum_type.values_by_number[value].name
                 if value[0] == "_" and value[1].isdigit():
                     value = value[1:]
-            out.write(' %s="%s"' % (attr.name, value))
+            out.write(u' %s="%s"' % (attr.name, value))
     if haveChildren:
-        out.write(">\n%s</%s" % (depth*'    ', desc.name))
+        out.write(u">\n%s</%s" % (depth * '    ', desc.name))
     else:
-        out.write("/")
+        out.write(u"/")
+
 
 def writeXml(root, module, options):
     with contextlib.closing(xml2csv.getOutStream(options.output)) as outputf:
-        outputf.write('<?xml version="1.0" encoding="UTF-8"?>\n\n<%s' % root)
+        outputf.write(u'<?xml version="1.0" encoding="UTF-8"?>\n\n<%s' % root)
         if options.source.isdigit():
             inp = xml2csv.getSocketStream(int(options.source))
         else:
@@ -101,9 +110,9 @@ def writeXml(root, module, options):
                             for item in value:
                                 msg2xml(attr, item, outputf)
                     elif first:
-                        outputf.write(' %s="%s"' % (attr.name, value))
+                        outputf.write(u' %s="%s"' % (attr.name, value))
                 first = False
-        outputf.write(">\n</%s>\n" % root)
+        outputf.write(u">\n</%s>\n" % root)
 
 
 def main():
