@@ -1,23 +1,23 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSEventControlTest.cpp
 /// @author  Matthias Heppner
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    2009-11-23
-/// @version $Id: MSEventControlTest.cpp 24012 2017-04-22 09:02:16Z behrisch $
 ///
-// Tests the class MSEventControl  
-/****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
+// Tests the class MSEventControl
 /****************************************************************************/
 
 #include <gtest/gtest.h>
@@ -28,12 +28,11 @@
 /* Test the method 'execute'. Tests if the execute method from the Command Class is called.*/
 
 TEST(MSEventControl, test_method_execute) {
-    
+
     MSEventControl eventControl;
-    CommandMock *mock = new CommandMock(); 
-    eventControl.setCurrentTimeStep(4);
+    CommandMock* mock = new CommandMock();
     eventControl.addEvent(mock);
-    
+
     EXPECT_FALSE(mock->isExecuteCalled());
     eventControl.execute(5);
     EXPECT_TRUE(mock->isExecuteCalled());
