@@ -1,21 +1,21 @@
 /****************************************************************************/
-/// @file    OutputDeviceMock.cpp
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
+/// @file    OutputDeviceMock.h
 /// @author  Matthias Heppner
 /// @author  Michael Behrisch
 /// @date    2009-11-23
-/// @version $Id: OutputDeviceMock.h 22608 2017-01-17 06:28:54Z behrisch $
 ///
-// 
-/****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
 //
 /****************************************************************************/
 
@@ -26,11 +26,7 @@
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <fstream>
 #include <utils/iodevices/OutputDevice.h>
@@ -41,29 +37,29 @@
 // ===========================================================================
 /**
  * @class OutputDeviceMock
- * Mock Implementation for Unit Tests 
+ * Mock Implementation for Unit Tests
  *
  */
 class OutputDeviceMock : public OutputDevice {
 public:
-    /** @brief Constructor    
+    /** @brief Constructor
      */
-	OutputDeviceMock() throw(IOError){}
+    OutputDeviceMock() {}
 
     /// @brief Destructor
-	~OutputDeviceMock() throw() {}
+    ~OutputDeviceMock()  {}
 
 
-    /** @brief Returns the current content as a string   
+    /** @brief Returns the current content as a string
      */
-    std::string getString() throw() {
+    std::string getString()  {
         return myStream.str();
     }
 
-protected:    
-    /** @brief Returns the associated ostream    
+protected:
+    /** @brief Returns the associated ostream
      */
-    std::ostream &getOStream() throw() {
+    std::ostream& getOStream()  {
         return myStream;
     }
 
